// DashboardOrganizerSettings.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.DashboardOrganizerSettings");


/*
	DashboardOrganizerSettings
	Manages retrieval, parsing, persisting a string representing the order in which dashboards, scoreboards and saved searches should be displayed on the Portal tab bar.
	It also identifies which dashboards/saved searches are currently hidden yet available for display
	
	I've avoided using names in the preference string (dashboard names, saved search names) for two reasons:
		1. Dashboards can be renamed by the user.  If the pref string were to identify a dashboard's order by using the name, and
			the dashboard were renamed, then the pref value would be stale and we wouldn't be able to find a match  between the preference
			identifier and the exposed dashboard.
		2. Saved searches unfortunately allow any characters in their names, e.g., []{}!@#$%^&*()_+?></\, which makes it difficult
			to create a *simple* preference string that has separators between items - no character can be safely used as a separator.
			Instead, I use the savedSearchID returned by REST.
	
	Preference string details:
	
	Here's an example preference string; I'll describe it below.
	The preference string (Portal Dashboard Display Order) is comprised of identifiers separated by forward slashes "/".
		
				     1             2         3			     4           5             6
			2015.25-ff89fa3ef0b3/1003/2015.43-dce9fa3b03fe/HIDDEN/2015.65-ecdfbcb03fa/1004
			
	1. Visible Dashboard
	2. Saved Search
	3. Visible dashboard
	4. Separator between visible and hidden items
	5. Hidden dashboard
	6. Hidden Saved Search

	*IMPORTANT NOTE ABOUT PRE-POPULATING THE PREF STRING*
	Note that Bulk User Attribute Assignment in Process Center/Server Admin can be used to populate the Portal Dashboard Display Order preference.
	The only part of the string example shown above that needs to be pre-populated is the dashboard ID or saved search ID.  In other words,
	for dashboards the part following "-" doesn't need to be pre-populated.  An administrator can easily find the ID of a dashboard
	via the REST exposed items service (as shown below) and create a pref string like:   2015.25/2015.66/1023/HIDDEN/2016.45
	The Dashboard Organizer will automatically modify the pref string to include the second segment.
	
	*Dashboard identifiers*
		A dashboard identifier is comprised of two attributes returned by the REST exposed items service:
			1. The ID attribute for the dashboard
				For example:       "ID":"2015.25".
			2. '-'
			3. The last segment of the "itemID" attribute for the dashboard
				for example:     "itemID":"2066.4fc070d9-4b5c-4905-bee4-dce9f6f903fe".  The last segment is used as the second part of the identifier
								 in this example   dce9f6f903fe
				Result:   2015.25-dce9f6f903fe
		    This second segment is needed so we can match up items even when the following occurs:
			   1. If a user installs a new snapshot of an app ON PROCESS SERVER then we won't find the *previous* newest snapshot without the 2nd segment.
			   2. If a user sets a new snapshot to be the default version ON BOTH PC AND PS then we won't find the *previous* default version without the 2nd segment.
			

	*Saved Search identifiers*
		A saved search identifier is the item's savedSearchID.
	
*/


dojo.declare("com.ibm.bpm.social.util.common.DashboardOrganizerSettings", null, {


  // copy constructor
  constructor: function constructor_DashboardOrganizerSettings(obj){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.defaultProfileString = "[]";
	this.dashboardOrderObj=[];
    if (obj) {
      dojo.safeMixin(this, obj);
      this.createProfileString();
    }
    else {
      this._currentProfileString = this.defaultProfileString;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * Cache for the last calculated profileString
   * 
   */
  _currentProfileString: "",

  /**
   * Indicate if we found the DashboardOrganizerSettings valid
   */
  isValid: true,
  MAX_PREF_STRING_LENGTH : 999,


  /**
   * Valid parameters in the user attributes string
   */
  profileParameters: {
    HIDDEN: "HIDDEN",
	DASHSEP: "/"
  },

  /**
   * default value for the start page, calculated in constructor
   */
  defaultProfileString: "[]",

  /**
   * Reset the class to it default values
   * 
   * @type {void}
   */
  reset: function reset_DashboardOrganizerSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.defaultProfileString = "[]";
	this.dashboardOrderObj=[];
    // update the current profile string
    this.createProfileString();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Return the cached profile string
   * 
   * @type {void}
   */
  getCurrentProfileString: function getCurrentProfileString_DashboardOrganizerSettings(){
    return this._currentProfileString;
  },

  /**
   * Take the string and parse it into a new DashboardOrganizerSettings object<br>
   * Note: This function does not change the class until the doMixin is set to true
   * 
   * @param {String} a string - dashboard page order from the user profile
   * @param {Boolean} if true and the string is found valid, copy the settings into the current object
   * @returns {com.ibm.bpm.social.util.common.DashboardOrganizerSettings}
   */
  parseFromString: function parseFromString_DashboardOrganizerSettings(profileString, doMixin){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var newDashboardOrganizerSettings = com.ibm.bpm.social.util.common.DashboardOrganizerSettings();

	var valid=true;
    var dashboardObj=[];
	profileString = unescape(profileString);

	com.ibm.bpm.social.util.common.Log.log(2, "about to parse dashboard order preference string, value is: [" + profileString+"]");
	dashboardObj=profileString.split(this.profileParameters.DASHSEP);
	
	if (valid) {
		if (dojo.isArray(dashboardObj)) {
			dojo.forEach(dashboardObj, dojo.hitch(this,function(item){
				if (item){
					var baseIDSeg="", itemIDSeg="";
					var arr = item.split("-");
					if (arr && dojo.isArray(arr)) {
						baseIDSeg = arr[0];   // identifier for dashboard on Process Center, and saved search. 
						if (arr.length==2) {  // identifier for dashboard on Process Server will have this second part; if this is a saved search, this segment isn't needed.
							itemIDSeg=arr[1];
						}
						var dashItem={};
						dashItem["baseIDSeg"]=baseIDSeg;
						dashItem["itemIDSeg"]=itemIDSeg;
						newDashboardOrganizerSettings.dashboardOrderObj.push(dashItem);
					} else {
						console.log("invalid item: "+item);
					}
				}
			}));
			
		} else {
			valid=false;
		}
	}
		
	
    newDashboardOrganizerSettings.isValid = valid;

    if (valid && doMixin) {
      // update the profile string presentation
      newDashboardOrganizerSettings.createProfileString();
      dojo.safeMixin(this, newDashboardOrganizerSettings);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return newDashboardOrganizerSettings;
  },
	  
  /**
   * Take all relevant settings and create a string that can be written into the users profile
   * 
   * @type {void}
   */
  createProfileString: function createProfileString_DashboardOrganizerSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var result="";

	
	dojo.forEach(this.dashboardOrderObj, dojo.hitch(this, function(item){
	    // Max length is 1000, due to database constraints;
		var curStr = item.baseIDSeg;
		if (item.itemIDSeg && item.itemIDSeg.length>0) {
			// the environment is process server; we'll add a '-' between identifier segments
			curStr = curStr+"-"+item.itemIDSeg;
		}
		curStr=curStr+this.profileParameters.DASHSEP;
		if ((this.MAX_PREF_STRING_LENGTH - result.length) >= curStr.length) {
			result+=curStr;
		}
	 }));

    // cache the string for quick access
    this._currentProfileString = result;

    com.ibm.bpm.social.util.common.Log.logExit(2, "Calculated Start page string is: " + result);
    return result;
  }
});
