// Log.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

/**
 * These functions provide a logging facility that may determine the calling function
 * 
 * Usage:
 * 
 * To log/trace a string/message:
 * <li> com.ibm.bpm.social.util.common.Log.log(1,"x is larger ...")
 * 
 * To trace the entry of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logEntry(1)
 * 
 * To trace the exit of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logExit(1)
 * 
 * To log objects:
 * <li> com.ibm.bpm.social.util.common.Log.logObject(integer pLogLevel, Object pObject, [String pTitle]);
 * 
 * Note: Third parameter is optional for example: com.ibm.bpm.social.util.common.Log.logObject(2, domNode,
 * "This is node hugo:");
 * 
 * 
 * Using the logError function, you may log an error to the console, and let the Log class determine the call
 * stack automatically
 * 
 * 
 * Output of the log may be adjusted/filtered using
 * <ul>
 * <li> com.ibm.bpm.social.util.common.Log.logLevel
 * <li> com.ibm.bpm.social.util.common.Log.includeClasses
 * <li> com.ibm.bpm.social.util.common.Log.excludeClasses
 * <li> com.ibm.bpm.social.util.common.Log.logArguments
 * 
 * 
 * Note about auto-resolving class names <br>
 * -------------------------------------
 * 
 * If you see the message "Couldn't resolve class name, might be within a constructor or invoked by
 * dojo.hitch" you should use the optional parameters cname while invoking the log functions. Example:
 * 
 * com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksContainer");
 * 
 * 
 * Cases where you need this: - while a class is constructed (logging within constructor, postCreate) - while
 * logging in functions that were called using apply, call, dojo.hitch
 * 
 */


dojo.provide("com.ibm.bpm.social.util.common.Log");
dojo.require("dojo.string");


/**
 * Logging level to provide some granularity/filtering Values:
 * <ul>
 * <li> 0 - No logging
 * <li> 1 - Log function entries and exits
 * <li> 2 - Additional data
 * <li> 3,.. - Much more data
 * </ul>
 */
// set 0 to just to initialize, will be set later according to the various settings, 
// see com.ibm.bpm.social.util.common.Log.determinLogContent()
com.ibm.bpm.social.util.common.Log.logLevel = 0;

/**
 * You may include only a subset in the output using includeClasses or excludeClasses
 * <p> - classes may be specified as regExp string
 * <p> - example using includeClasses : includeClasses = ["Selector", "com.ibm.bpm.social.util.common"]
 * <p> - to include all, just use ["*"]
 * <p> - exclude overrules include
 * <p> - to unset exclude or include use [] <p
 */

com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
com.ibm.bpm.social.util.common.Log.excludeClasses = [];


/**
 * Using this flag you can enable/disable the logEntry function to print out the arguments of the function
 */
com.ibm.bpm.social.util.common.Log.logArguments = true;


/**
 * Use this flag to also get info about the time when the log was written
 */
com.ibm.bpm.social.util.common.Log.logTime = false;

/**
 * Use this flag to also log also the date when the log was written
 */

com.ibm.bpm.social.util.common.Log.logDate = false;

/**
 * Use this flag to indent on each function call (when logEntry is called) and un-indent when leaving the
 * function (when logExit is called)
 * <p>
 * This works only if browser supports the console.group functionality
 */
/*@ignore@*/
com.ibm.bpm.social.util.common.Log.indentFunctions = (typeof console.group == "function" && !dojo.isIE);
/*@end@*/

com.ibm.bpm.social.util.common.Log.logEntriesAndExits = false;

/**
 * Determine the details of what and how to log
 * <ul>
 * <li> logLevel : (0 - 10) how much will be logged
 * <li> indentFunctions : (true|false) shall we track function calls ?
 * <li> logTime: (true|false) prefix log statements with a timestamp
 * <li> logDate: (true|false) prefix log statements with a timestamp including date
 * <li> logArguments : (true|false) print out the arguments of the function
 * <li> includeClasses : comma-separated string of class names to be included in the log
 * <li> excludeClasses : comma-separated string of class names to be excluded from the log
 * </ul>
 * 
 * @type {void}
 */
com.ibm.bpm.social.util.common.Log.determinLogContent =
  function(){

    if (com.ibm.bpm.social.debug.isLogging) { // allowClientSideLogging under the Mashups_ConfigService == true
      // logging to the Portal log console requested, provide as much details as possible
      com.ibm.bpm.social.util.common.Log.logLevel = 10;
      com.ibm.bpm.social.util.common.Log.logArguments = true;
      com.ibm.bpm.social.util.common.Log.logTime = true;
      com.ibm.bpm.social.util.common.Log.logDate = true;
      com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
      com.ibm.bpm.social.util.common.Log.excludeClasses = [];
      com.ibm.bpm.social.util.common.Log.indentFunctions = false;
    }
    else if ((typeof dojoConfig != "undefined") && dojoConfig.isDebug || (typeof djConfig != "undefined") &&
      djConfig.isDebug) {
      // typical developers setup, just use defaults
      com.ibm.bpm.social.util.common.Log.logLevel = 2;
      com.ibm.bpm.social.util.common.Log.logEntriesAndExits = true;
    }

    // check for query string settings 
    var paramString = window.location.search.replace("?", "");
    var paramKey = "";
    var paramValue = "";
    var value = "";
    var params = paramString.split("&");

    for ( var i = 0; i < params.length; i++) {
      paramKey = params[i].split("=")[0];
      paramValue = params[i].split("=")[1];

      if (paramKey && paramValue) {

        // logLevel
        if (paramKey == "logLevel") {
          value = Number(paramValue);
          if (!isNaN(value)) {
            com.ibm.bpm.social.util.common.Log.logLevel = value;
          }
        }
        // logArgument
        else if (paramKey == "logArguments") {
          com.ibm.bpm.social.util.common.Log.logArguments = (paramValue == "true");
        }
        // logDate
        else if (paramKey == "logDate") {
          com.ibm.bpm.social.util.common.Log.logTime = (paramValue == "true");
        }
        // logTime
        else if (paramKey == "logTime") {
          com.ibm.bpm.social.util.common.Log.logDate = (paramValue == "true");
        }
        // indentFunctions
        else if (paramKey == "indentFunctions") {
          com.ibm.bpm.social.util.common.Log.indentFunctions = (paramValue == "true");
        }
        // includeClasses
        else if (paramKey == "includeClasses") {
          com.ibm.bpm.social.util.common.Log.includeClasses = paramValue.split(",");
        }
        // excludeClasses
        else if (paramKey == "excludeClasses") {
          com.ibm.bpm.social.util.common.Log.excludeClasses = paramValue.split(",");
        }
        else if (paramKey == "logEntriesAndExits") {
          com.ibm.bpm.social.util.common.Log.logEntriesAndExits = (paramValue == "true");
        }
      }
    }


    // disable logging at all if no one requested it
    if (!com.ibm.bpm.social.debug.isLogging && !dojoConfig.isDebug && !djConfig.isDebug &&
      com.ibm.bpm.social.util.common.Log.logLevel === 0) {
      var emptyFunction = function(){};
      com.ibm.bpm.social.util.common.Log.log = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logEntry = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logExit = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logError = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logObject = emptyFunction;
    }

    return;
  };


/**
 * Write a message to the console
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.log =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
      if (!pMessage) {
        pMessage = "";
      }

      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      com.ibm.bpm.social.util.common.Log.consoleLog(scope + " " + pMessage);
    }
  };

/**
 * Trace the entry of a method/function <br>
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */

com.ibm.bpm.social.util.common.Log.logEntry =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope =
        com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName, com.ibm.bpm.social.util.common.Log.logArguments);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroup(scope + " entry " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " entry " + pMessage);
      }
    }
  };


/**
 * Trace the exit of a method/function
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.logExit =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroupEnd(scope + " exit " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " exit " + pMessage);
      }
    }
  };


/**
 * Helper to determine if the message should be traced
 * 
 * @param {String} cname the class name, may be null
 * @param {String} fktName the function of the class, may be null
 * @param {String} includeArgs indicate if the args should be traced
 * @return boolean
 */
com.ibm.bpm.social.util.common.Log._shouldTrace =
  function(cname, fktName, includeArgs){

    var originFunction = arguments.callee.caller.caller;

    if ((cname == null || typeof (cname) == "undefined") || (fktName == null || typeof (fktName) == "undefined")) {
      // Note: the following cannot extracted to a utility function, since the arguments object will change
      var cls = null;

      if (originFunction._containingClass) {
        cname = originFunction._containingClass;
      }

      // works not for constructors
      if (originFunction.ctor && originFunction.ctor.prototype) {
        cls = originFunction.ctor.prototype.declaredClass;
      }
      if (cls) {
        var packages = cls.split(".");
        cname = packages[packages.length - 1];
      }
      else if (originFunction.name && (originFunction.name.lastIndexOf("_") > 0)) {
        var l = originFunction.name.split("_");
        // fktName = l.slice(0,l.length-1).join(_);
        cname = l[l.length - 1];
      }

      cname = cname || "<undefined>";

      if (originFunction.nom) {
        fktName = originFunction.nom;
      }
      else if (originFunction.name) {
        fktName = originFunction.name;
        fktName = fktName.replace("_" + cname, "");
      }

      fktName = fktName || "<undefined>";

      if (cname == "<undefined>" && fktName != "<undefined>" && typeof originFunction.toString == "function") {
        // in IE this is the only known way to get the class name
        // returns the whole function
        var originFunctionToString = originFunction.toString();
        // look if the function name is contained (if it was declared as function methodeName_className() it is)
        var iFktName = originFunctionToString.indexOf(fktName);
        if (iFktName != -1) {
          // find the first ( - that's directly after the class name
          var iFirstOpeningBracket = originFunctionToString.indexOf("(");
          // check that the function name does not come after the the first (, would mean it is contained anywhere in the function body
          if (iFirstOpeningBracket != -1 && iFirstOpeningBracket > iFktName) {
            // extract the class name
            cname = originFunctionToString.substring(iFktName + fktName.length + 1, iFirstOpeningBracket);
          }
        }
      }

      if (cname == "<undefined>") {
        com.ibm.bpm.social.util.common.Log
          .consoleWarn("Couldn't resolve classname, might be within a constructor or invoked by dojo.hitch. See Log.js to learn how to fix.");
      }
      else {
        originFunction._containingClass = cname;
      }
    }

    var include = com.ibm.bpm.social.util.common.Log.includeClasses;
    var exclude = com.ibm.bpm.social.util.common.Log.excludeClasses;
    var rc = cname + "." + fktName + "(";
    if (includeArgs === true && originFunction.arguments) {
      rc += com.ibm.bpm.social.util.common.Log._argumentsToString(originFunction.arguments);
    }
    rc += ")";


    if (include.length === 0 && exclude.length === 0) {
      return null;
    }

    for ( var i = 0; i < exclude.length; i++) {
      if (cname.search(exclude[i]) > -1) {
        return null;
      }
    }

    if (include[0] == "*") {
      return rc;
    }

    for (i = 0; i < include.length; i++) {
      if (cname.search(include[i]) > -1) {
        return rc;
      }
    }

    return false;
  };

/**
 * converts an array of arguments to a string
 * 
 * @param {String} args the arguments
 * @return {String} the formatted arguments
 * 
 */
com.ibm.bpm.social.util.common.Log._argumentsToString = function(/*Array*/args, /*int*/counter){
  var stringArgs = [];
  counter = counter || 0;
  if (args) {
    for ( var i = 0; i < args.length; i++) {
      if (args[i] === null) {
        stringArgs.push("null");
      }
      else if (typeof args[i] == "undefined") {
        stringArgs.push("undefined");
      }
      else if (typeof args[i] == "function") {
        stringArgs.push("[function]");
      }
      else if (typeof args[i] == "string" || typeof args[i] == "number" || typeof args[i] == "boolean") {
        // dojo.toJson automatically replaces line breaks with \n in strings and converts numbers and booleans to a string
        stringArgs.push(dojo.toJson(args[i]));
      }
      else if (typeof args[i] == "object") {
        if (dojo.isArrayLike(args[i])) {
          // to prevent recursion as arrays might reference themselve or contain cross references we have an counter with an exit condition
          if (counter < 3) {
            stringArgs.push("[" + com.ibm.bpm.social.util.common.Log._argumentsToString(args[i], counter + 1) + "]");
          }
          else {
            stringArgs.push("[array]");
          }
        }
        else if (typeof args[i].toString == "function") {
          stringArgs.push(args[i].toString());
        }
        else {
          stringArgs.push("[object]");
        }
      }
      else {
        stringArgs.push("[unknown]");
      }
    }
  }
  return stringArgs.join(", ");
};

/**
 * Log an error to the console, and let the Log class determine the call stack
 * 
 * @param {Number} pLogLevel
 * @param {String} pMethod Method or Message
 * @param {Object} xcp optional exception object
 */

com.ibm.bpm.social.util.common.Log.logError = function(/*integer*/pLogLevel, /*String*/pMethod, /*Object*/xcp){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length < 1) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logError"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logError"]));
    }

    var pMessage = pMethod ? pMethod : "";

    if (xcp) {
      pMessage = pMessage + " " + xcp.name + ":" + xcp.message;
      if (xcp.stack) {
        pMessage = pMessage + "\nStack:";
        var stack = xcp.stack.split("\n");
        var out = "";
        for ( var i = 0; i < stack.length; i++) {
          out = stack[i];
          if (out.length > 200) {
            // this is a eval'ed file instead of a function, which we don't wan to see in the console
            var file1 = out.match(/(\w*\.js)/) ? out.match(/(\w*\.js)/)[0] : "";
            var file2 = out.match(/@http.*$/) ? out.match(/@http.*$/)[0] : "";

            out = (file1 ? file1 : "");
            out += (file2 ? ":" + file2 : "");
            if (!out) {
              out = "undefined";
            }
          }
          pMessage = pMessage + "\n" + out;
        }
      }
    }
    com.ibm.bpm.social.util.common.Log.consoleError(pMessage);
  }
};

com.ibm.bpm.social.util.common.Log.logObject = function(/* integer */pLogLevel, /* Object */pObject, /* String */pTitle){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length != 2 && arguments.length != 3) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logObject"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logObject"]));
    }

    if (pTitle) {
      com.ibm.bpm.social.util.common.Log.consoleDebug(pTitle);
    }
    /*@ignore@*/
    console.dir(pObject);
    /*@end@*/
  }
};

com.ibm.bpm.social.util.common.Log._getTS = function(){
  var d = new Date();
  var rc = "";
  var millis = d.getMilliseconds();
  millis = (millis.length == 2 ? "0" + millis : millis);
  if (com.ibm.bpm.social.util.common.Log.logDate) {
    rc = d.getFullYear() + "-" + ((d.getMonth() < 9) ? "0" : "") + (d.getMonth() + 1) + "-";
    rc = rc + ((d.getDate() < 9) ? "0" : "") + d.getDate() + " ";
  }
  rc = rc + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds() + "." + d.getMilliseconds() + " ";
  return rc;
};

// wrapper which might be used to handle browser issues

com.ibm.bpm.social.util.common.Log.consoleLog = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroup = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.group) {
    console.group(message);
  }
  else {
    console.log(message);
  }
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroupEnd = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.groupEnd) {
    console.groupEnd();
  }
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleDebug = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.debug(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleError = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.error(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleWarn = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.warn(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleInfo = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.info(message);
  /*@end@*/
};


/*
 * Do not translate
 */
com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR =
  "Invalid number of parameters for com.ibm.bpm.social.util.common.Log.${0}";
com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a number";
com.ibm.bpm.social.util.common.Log.PARM_NO_STRING_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a string";
com.ibm.bpm.social.util.common.Log.PARM_NO_OBJECT_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a object";

// determine the log content to be displayed
com.ibm.bpm.social.util.common.Log.determinLogContent();