// QueryRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.QueryRestHandler");

dojo.require("dojo.string");

dojo.require("com.ibm.bpm.social.util.common.RestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.declare("com.ibm.bpm.social.util.common.QueryRestHandler", com.ibm.bpm.social.util.common.RestHandler, {

  constructor: function constructor_QueryRestHandler(params){},

  /**
   * Returns all query tables for the given entity for the given domain.
   * 
   * @param {String} type
   * @param {String} requiredContent
   * @param {function} load
   * @param {function} error
   * @param {String} domain federation only
   * @param {String} process application name
   * @returns {void} nothing
   * @throws error if no domain name is specified for a federated backend
   */
  getAllQueryTables: function getAllQueryTables_QueryRestHandler(type, requiredContent, load, error, domain, appName){
    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(type);

    // Call API
    if (apiType) {
      var url = apiType;
      var parameters = "";
      // AB: For Escalations , Workbasktest and BusinessCategories still the old API style is necessary 
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        url += "/queryTables";
      }
      else {
        url += "/queries";
      }

      if (this.isApiTypeFederated()) {
        if (domain) {
          parameters += "&domain=" + domain;
        }
        else {
          this.errorNoDomain();
        }
        parameters += "&all=true";
      }

      if (requiredContent) {
        parameters += "&content=" + requiredContent;
      }
      // TODO AB: for federated and WLE API we can not use these parameters right now. 
      // they would not return query tables as these values are not correct for WLE
      if (this.isApiTypeWps()) {
        if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE ||
          type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE) {
          // We only support templates with authorizationType=NONE, otherwise
          // only the BPC admin would see the templates.
          parameters += "&authorizationType=NONE";
        }
        else {
          // For instance QT'S only query tables requiring instance-based authorization
          // are returned, because we heavily make use of WORKITEM_REASON.
          parameters += "&authorizationType=INSTANCE_BASED";
        }
        // Get composite query tables only.
        parameters += "&kind=COMPOSITE";
      }
      // User parameters are not supported.
      parameters += "&userParametersAllowed=false";

      if (appName) {
        parameters += "&processAppName=" + encodeURIComponent(appName);
      }

      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      url = this.getRequestUrl(url);
      var requestLoad = load;
      var expirationPeriod = 3600000;
      var data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
      var requestError = error;
      var myself = this;
      // if we have data in the cache for this url
      if (data) {
        // Handling 200 requests
        if (data.data && typeof load == "function") {
          // emulate 2.argument
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else if (data.ioArgs && typeof error == "function") {
          // handle error requests 401 unauthorized
          // emulate 2.argument
          error.apply(null, [{},
                             data.ioArgs]);
          return;
        }
      }
      else {
        if (typeof load == "function") {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
        if (typeof error == "function") {
          requestError = function(data, ioArgs){
            // do not cache other errors as they might stay for a server error.
            // But unauthorized is an error related to user rights and not server
            // problems.
            if (ioArgs.xhr.status == 401) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                ioArgs: ioArgs
              }, expirationPeriod);
            }
            return error.apply(this, [data,
                                      ioArgs]);
          };
        }
      }
      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: true,
        load: requestLoad,
        error: requestError
      }, true);
    }
    else {
      // TODO: trace
      /*@pass@*/
    }
  },

  /**
   * Returns all attributes for a query table for the given entity.
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {boolean} includeTaskData flag whether to include task data into a processes query
   * @param {function} load
   * @param {function} error
   * @param {boolean} preventCache
   * @param {String} domain federation only
   * @param {String} systemID federation only
   * @returns {void} nothing
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getAttributesForQueryTable: function getAttributesForQueryTable_QueryRestHandler(type, queryTable, processAppName,
    includeTaskData, load, error, preventCache, domain, systemID){

    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(type);

    // Call API
    if (apiType) {
      var url = apiType;
      // AB: For Escalations , Workbasktest and BusinessCategories still the old API style is necessary
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        url += "/queryTable/";
      }
      else {
        url += "/query/";
      }
      url += encodeURIComponent(queryTable) + "/attributes";

      var parmString = "";

      if (processAppName) {
        parmString += "&processAppName=" + encodeURIComponent(processAppName);
      }

      if (typeof includeTaskData == "boolean") {
        parmString += "&includeTaskData=" + includeTaskData;
      }

      if (this.isApiTypeFederated()) {
        if (systemID) {
          parmString += "&systemID=" + systemID;
        }
        else if (domain) {
          parmString += "&domain=" + domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parmString += "&all=true";
      }

      if (parmString.length > 0) {
        url = url + "?" + parmString.substr(1);
      }

      url = this.getRequestUrl(url);

      var requestLoad = load;
      if (typeof load == "function") {
        // Handling for 200 (ok) is implemented, do not cache an 404 result,
        // because a 404 indicates a real error. One of the next calls for
        // this template should bring the real template.
        var expirationPeriod = 3600000;
        var data = null;
        // If the preventCache is not provided, do try cache
        // otherwise take  preventCache parameter
        if (typeof preventCache == "undefined" || !preventCache) {
          data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
        }
        if (data) {
          // emulate 2.argument ??
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
      }

      if (preventCache === undefined || preventCache === null) {
        // ASZ@696877: allow browser to cache attributes.
        preventCache = false;
      }

      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: true,
        load: requestLoad,
        error: error,
        preventCache: preventCache

      }, true);
    }
    else {
      // TODO: trace
    }
  },

  /**
   * Returns all entities in a query table.
   * 
   * Use the object format for the parameters
   * 
   * @param {object}
   * 
   * 
   * deprecated way of passing the parameters please use the object style
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {String} selectedAttributes
   * @param {String} interactionFilter
   * @param {String} queryFilter
   * @param {String} sortAttributes
   * @param {String} offset
   * @param {String} size
   * @param {Function} load
   * @param {Function} error
   * @param {boolean} async
   * @param {String} domain federation only
   * @param {String} systemID federation only, mutually exclusive with domain contain the systemID property!
   * @returns {dojo.Deferred} an object which allows to cancel the dojo.xhr request
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getQueryTableContent: function getQueryTableContent_QueryRestHandler(type, queryTable, processAppName, selectedAttributes,
    interactionFilter, queryFilter, sortAttributes, offset, size, load, error, async, domain, systemID){
    var param = {};
    if (type.queryTable) {
      param = type;
    }
    else {
      param.type = type;
      param.queryTable = queryTable;
      param.processAppName = processAppName;
      param.selectedAttributes = selectedAttributes;
      param.interactionFilter = interactionFilter;
      param.queryFilter = queryFilter;
      param.sortAttributes = sortAttributes;
      param.offset = offset;
      param.size = size;
      param.load = load;
      param.error = error;
      param.async = async;
      param.domain = domain;
      param.systemID = systemID;
    }
    return this._getQueryTableContent(param);
  },


  _getQueryTableContent: function getQueryTableContent_QueryRestHandler(params){

    var deferred = null;

    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(params.type);

    // Call API
    if (apiType) {
      var url = apiType + "/query";
      var parameters = "";
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        parameters += "&queryTableName=" + params.queryTable;
      }
      else {
        url += "/" + encodeURIComponent(params.queryTable);
      }

      // include the task data into process queries.
      if (params.includeTaskData) {
        parameters += "&includeTaskData=true";
      }
      if (params.processAppName) {
        parameters += "&processAppName=" + encodeURIComponent(params.processAppName);
      }

      if (this.isApiTypeFederated()) {
        if (params.systemID) {
          parameters += "&systemID=" + params.systemID;
        }
        else if (params.domain) {
          parameters += "&domain=" + params.domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parameters += "&all=true";
      }

      if (params.searchFilter) {
        parameters += "&searchFilter=" + window.encodeURIComponent(params.searchFilter);
      }
      if (params.selectedAttributes) {
        parameters += "&selectedAttributes=" + encodeURIComponent(params.selectedAttributes);
      }
      if (params.interactionFilter) {
        parameters += "&interactionFilter=" + window.encodeURIComponent(params.interactionFilter);
      }
      if (params.queryFilter) {
        parameters += "&queryFilter=" + window.encodeURIComponent(params.queryFilter);
      }
      if (params.offset) {
        parameters += "&offset=" + params.offset;
      }
      if (params.size) {
        parameters += "&size=" + params.size;
      }
      else {
        // add threshold
        parameters += "&size=" + this.threshold;
      }
      if (params.sortAttributes) {
        parameters += "&sortAttributes=" + encodeURIComponent(params.sortAttributes);
      }
      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      if (params.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE ||
        params.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
        url = url + "&admin=true";
      }

      var myself = this;
      deferred = this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: !params.async,
        load: function(responseObject, ioArgs){
          if (myself.isApiTypeFederated() && params.systemID && responseObject.items && responseObject.items.length > 0) {
            // add systemID to items
            // TODO SA remove this warning - just to see if this happens
            com.ibm.bpm.social.util.common.Log.consoleWarn("Adding systemID to query result!!!");
            for ( var i = 0; i < responseObject.items.length; i++) {
              responseObject.items[i].systemID = params.systemID;
            }
          }
          params.load(responseObject, ioArgs);
        },
        error: params.error
      });
    }
    else {
      // TODO: trace
    }
    return deferred;
  },

  /**
   * 
   * Returns the number of all entities in a query table.
   * 
   * @param {Object} param
   * 
   * the simple params are deprecated use the object structure
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {String} interactionFilter
   * @param {String} queryFilter
   * @param {Function} load
   * @param {Function} error
   * @param {boolean} async
   * @param {String} domain federation only
   * @param {String} systemID federation only
   * @returns {void} nothing
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getQueryTableCount: function getQueryTableCount_QueryRestHandler(type, queryTable, processAppName, interactionFilter,
    queryFilter, load, error, async, domain, systemID){
    var params = {};
    // new format
    if (type.queryTable) {
      params = type;
    }
    else {
      // old style
      params.type = type;
      params.queryTable = queryTable;
      params.processAppName = processAppName;
      params.interactionFilter = interactionFilter;
      params.queryFilter = queryFilter;
      params.load = load;
      params.error = error;
      params.async = async;
      params.domain = domain;
      params.systemID = systemID;
    }
    this._getQueryTableCount(params);
  },

  _getQueryTableCount: function _getQueryTableCount_QueryRestHandler(params){
    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(params.type);

    // Call API
    if (apiType) {
      var url = apiType;
      var parameters = "";
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        url += "/queryCount";
        parameters += "&queryTableName=" + params.queryTable;
      }
      else {
        url += "/query/" + params.queryTable + "/count";
      }

      if (params.searchFilter) {
        parameters += "&searchFilter=" + window.encodeURIComponent(params.searchFilter);
      }
      if (params.processAppName) {
        parameters += "&processAppName=" + encodeURIComponent(params.processAppName);
      }

      if (this.isApiTypeFederated()) {
        if (params.systemID) {
          parameters += "&systemID=" + params.systemID;
        }
        else if (params.domain) {
          parameters += "&domain=" + params.domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parameters += "&all=true";
      }

      if (params.interactionFilter) {
        parameters += "&interactionFilter=" + window.encodeURIComponent(params.interactionFilter);
      }
      if (params.queryFilter) {
        parameters += "&queryFilter=" + window.encodeURIComponent(params.queryFilter);
      }
      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      // Add parameter "admin=true" if we have business categories.
      if (params.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE ||
        params.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
        url = url + "&admin=true";
      }

      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: !params.async,
        load: params.load,
        error: params.error
      });
    }
    else {
      // TODO: trace
    }
  },

  _getQueryTableAPI: function _getQueryTableAPI_QueryRestHandler(type){
    var apiType = null;
    if (type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
      apiType = "/tasks";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE) {
      apiType = "/taskTemplates";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE) {
      apiType = "/processes";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE) {
      apiType = "/processTemplates";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.ESCALATION_TYPE) {
      apiType = "/escalations";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
      apiType = "/workBaskets"; //FIXME: change to correct base URI of work basket REST service
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE) {
      apiType = "/businessCategories";
    }

    return apiType;
  },

  statics: {
    _instance: null,

    _createInstance: function(params){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.util.common.QueryRestHandler();
        // TODO handle params, e.g. 
//        this._instance.setIWidget(iWidget);
//        this._instance.setUrl("prefix", this.getServiceURLRoot());
//        this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//        this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
      }
      else {
        // programming error
        var error = new Error("RestHandler instance has already been initialized!");
        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
        throw error;
      }
      return this._instance;
    },

    _getInstance: function(){
      if (this._instance === null) {
        // TODO for now return default handler
        this._instance = new com.ibm.bpm.social.util.common.QueryRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.util.common.QueryRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.QueryRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.QueryRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.QueryRestHandler.prototype.statics._getInstance();
};
