// TaskRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.TaskRestHandler");

dojo.require("dojo.string");

dojo.require("com.ibm.bpm.social.util.common.RestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.notifications");

dojo.declare("com.ibm.bpm.social.util.common.TaskRestHandler", com.ibm.bpm.social.util.common.RestHandler,
  {

    constructor: function constructor_TaskRestHandler(params){},

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getTask: function getTask_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/task";
      url += "/" + id;
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        // load: load,
        load: function(task, ioArgs){
          task = myself.addSystemProperties(task, systemID);
          load(task, ioArgs);
        },
        error: error
      });
    },

    /**
     * @param {String[]} idList
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * 
     */
    getTasksInBulkMode: function getTasksInBulkMode_TaskRestHandler(idList, load, error, async, parts){
      // TODO: re-enable federated mode once the REST Api supports a federated version of this

      // URL may look like this: /task?action=getdetails&taskIDs=2,3,5,7,11
      //           or like this: /task?action=getdetails&taskIDs=2,3,5,7,11&parts=data

      if (!idList || !idList.length || idList.length === 0) {
        // defensive - do nothing
        return;
      }

      var url = "/task";
      url += "?action=getdetails&taskIDs=";
      // we have at least one id
      url += idList[0];
      for ( var k = 1; k < idList.length; k++) {
        url += "," + idList[k];
      }

      if (parts && parts != "all") {
        url += "&parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },

    /**
     * @param taskID
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param systemID
     * @param domain
     * @returns {void} nothing
     */
    getActionsForTask: function getActionsForTask_TaskRestHandler(taskID, actionsFilter, load, error, async, systemID,
      domain){
      return this.getActions([{
        tkiid: taskID,
        systemID: systemID
      }], "tkiid", actionsFilter, load, error, async, domain);
    },

    /**
     * @param {Array} items
     * @param {String} idKey e.g TKIID or tkiid or id
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param {String} domain, optional for federated backend and more than one item
     * @returns {dojo.Deferred}
     */
    getActions: function getActions_TaskRestHandler(items, idKey, actionsFilter, load, error, async, domain){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (actionsFilter != null) {
        com.ibm.bpm.social.util.common.Log.logObject(3, actionsFilter, "actionsFilter");
      }

      var deferred = null;

      var self = this;
      var idString = "";
      dojo.forEach(items, function(item){
        if (idString !== "") {
          idString += ",";
        }
        var id = item[idKey];
        if (id !== null) {
          idString += self.prefixID(item[idKey], domain, item.systemID);
        }
      });

      var url = "/task/actions?";
      var newLoad = load;
      if (this.isApiTypeFederated()) {
        if (domain) {
          url += "domain=" + domain + "&";
        }
        else if (items.length === 1 && items[0].systemID) {
          url += "systemID=" + items[0].systemID + "&";
          // insert the system id, in federated case the system id is not returned if no domain is specified
          newLoad = function(data, ioArgs){
            if (data && data.items) {
              for ( var i = 0; i < data.items.length; i++) {
                data.items[i].systemID = items[0].systemID;
              }
            }
            load(data, ioArgs);
          };

        }
        else {
          this.errorNoDomainOrSystemID();
        }
      }
      url += "taskIDs=" + encodeURIComponent(idString);

      if (actionsFilter) {
        var actionsFilterString = "";
        for ( var i in actionsFilter) {
          if (actionsFilterString != "") {
            actionsFilterString += ",";
          }
          actionsFilterString += actionsFilter[i];
        }
        url += "&actions=" + actionsFilterString;
      }

      deferred = this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });

      com.ibm.bpm.social.util.common.Log.logExit(2);
      return deferred;
    },

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getProcessActions: function getProcessActions_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/process";
      url += "/" + id + "/actions";
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },


    /**
     * @param id
     * @param load
     * @param error
     * @param priority e.g. Highest/10, High/20, Normal/30, Low/40 or Lowest/50
     * @param dueDate in ISO-8601 format (e.g. 2011-01-05T03:15:30.000-00:00)
     * @param systemID
     * @returns {void} nothing
     */
    updateTaskProperties: function updateTaskProperties_RestHandler(id, load, error, priority, dueDate, systemID){
      var url = "/task";
      url += "/" + id + "?action=update";

      if (priority) {
        url += "&priority=" + priority;
      }

      if (dueDate) {
        url += "&dueDate=" + dueDate;
      }

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Updates all tasks
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param {Object} content <br>
//       *        WPS API {"updatedTaskIDs": [..., ...], "updatedPropertyValues": {...}} <br>
//       *        Federated API {"tasks": [{taskID: ..., systemID: ...}, ...], "updatedPropertyValues": {...}}
//       * @param {String} domain federation only
//       * @returns {void} nothing
//       */
//      updateTasksData: function updateTasksData_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=update";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    claimTask: function claimTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=claim";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json"
      });
    },

    /**
     * Completes a simple inline task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param property The property for the json result.
     * @param result The value of the result property in the json result.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleInlineTask: function approveSimpleApprovalTask_TaskRestHandler(taskId, claimTask, property, result,
      comment, loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call            
      var data = {};
      data[property] = result;
      if (comment != "") {
        data["comment"] = comment;
      }
      var url = "/task/" + taskId + "?action=complete&params=" + window.encodeURIComponent(dojo.toJson(data));
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    /**
     * Completes a simple completion task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleCompletionTask: function completeSimpleCompletionTask_TaskRestHandler(taskId, claimTask, comment,
      loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call                   
      var url = "/task/" + taskId + "?action=complete";
      if (comment != "") {
        var data = {
          "comment": comment
        };
        url = url + "&params=" + window.encodeURIComponent(dojo.toJson(data));
      }
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    publishOnInstanceStream: function _publishOnInstanceStream_TaskRow(piid, comment, load, error, systemID){
      if (piid != null) {
        var url = "/process/" + piid + "?action=comment&comment=";
        url = url + window.encodeURIComponent(comment);
        url = this.addSystemID(url, systemID);
        this.performRequest("post", {
          url: url,
          sync: true,
          load: load,
          error: error,
          handleAs: "text"
        });
      }
    },

    // TODO remove or enable
//      /**
//       * Claims all tasks in content.
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param content {"taskIDs": [...]}
//       * @param {String} domain optional, needed for bulk calls to a federated system
//       * @returns {void} nothing
//       */
//      claimTasks: function claimTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=claim";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    cancelTask: function cancelTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=cancel";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Cancels all tasks in content = {"taskIDs": [...]}.
//       */
//      cancelTasks: function cancelTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=cancel";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    /**
     * 
     * @param tktid
     * @param tkiid
     * @param type
     * @param load
     * @param error
     * @param async
     * @param systemID
     * @returns {void} nothing
     */
    getClientSetting: function getClientSetting_TaskRestHandler(tktid, tkiid, type, load, error, async, systemID){
      //
      // Determine the system of the task
      //
      var wps = false;
      if (this.getSystems().isApiTypeFederated() && this.getSystems().isSystemWps(systemID)) {
        wps = true;
      }
      else if (this.getSystems().isApiTypeWps()) {
        wps = true;
      }
      //
      // Calculate the key for the cache.
      //
      var key = null;
      if (wps && tktid) {
        // WPS has client setting which are based on task templates
        // Therefore use template id if available (Take care of Ad hoc task which do not have a template id)      
        key = "/taskTemplate/" + tktid + "/clientSettings/" + type;
        key = this.getRequestUrl(key);
      }
      //
      // Set url for the call.
      //
      var url = null;
      if (tkiid) {
        // Call as specific as possible due to authorization reasons.
        // Because a user that is only OWNER or POTENTIAL_OWNER of a task can only retrieve the client settings 
        // of a task using the task resource and the tkiid 
        url = "/task/" + tkiid + "/clientSettings/" + type;
      }
      else {
        url = "/taskTemplate/" + tktid + "/clientSettings/" + type;
      }
      url = this.addSystemID(url, systemID);

      var requestLoad = load;
      if (key !== null && typeof load == "function") {
        // Handling for 200 (ok) is implemented, do not cache an 404 result,
        // because a 404 indicates a real error. One of the next calls for
        // this template should bring the real template.
        var expirationPeriod = 3600000;
        var data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(key);
        if (data) {
          // emulate 2.argument ??
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(key, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
      }
      this.performRequest("get", {
        url: url,
        sync: !async,
        load: requestLoad,
        error: error
      });
    },

    /**
     * 
     * @param id
     * @param load
     * @param error
     * @param async
     * @returns {void} nothing
     */
    getTaskTemplateDetails: function getTaskTemplateDetails_RestHandler(tktid, load, error, async, systemID){
      var url = "/taskTemplate";
      url += "/" + tktid;

      url = this.addSystemID(url, systemID);

      this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });
    },

    assignTaskBack: function assignTaskBack_RestHandler(tktid, load, error, systemID){
      var url = "/task";
      url += "/" + tktid + "?action=assign&back=true";

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToUser: function assignTaskToUser_RestHandler(id, load, error, user, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toUser=" + window.encodeURIComponent(user);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToGroup: function assignTaskToGroup_RestHandler(id, load, error, group, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toGroup=" + window.encodeURIComponent(group);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    inviteUserToCollaborate: function inviteUserToCollaborate_RestHandler(id, load, error, userid, invitationMessage,
      systemID){
      var url = "/task/" + id;

      url = this.addSystemID(url, systemID);

      this.performRequest("post", {
        url: url,
        content: {
          action: 'invite',
          user: userid,
          message: invitationMessage
        },
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "Accept": "application/json"
        },
        sync: true,
        load: load,
        error: error
      });
    },

    statics: {
      _instance: null,

      _createInstance: function(params){
        if (this._instance === null) {
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
          com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
          // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
        }
        else {
          // programming error
          var error = new Error("RestHandler instance has already been initialized!");
          com.ibm.bpm.social.util.common.Log.consoleError(error.message);
          throw error;
        }
        return this._instance;
      },

      _getInstance: function(){
        if (this._instance === null) {
          // TODO for now return default handler
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
        }
        return this._instance;
      }
    }
  });

com.ibm.bpm.social.util.common.TaskRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.TaskRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._getInstance();
};
