/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.UserChooser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

/**
 * Dojo widget that allows the selection of one element from a list of valid ones or also the typing of a new element.
 * <p>
 *
 * The valid elements are provided in the constructor of this class, using the mandatory argument "ucModel".<br>
 * The ucModel should be an object that follows the contract/interface exemplified in the class
 * com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel.
 * <p>
 * Features:
 * <ol>
 *  <li>Embeddable UserChooser or ready to use UserChooserDialog</li>
 *  <li>Configurable model</li>
 *  <li>Configurable initially displayed suggestions</li>
 *  <li>Configurable delay between searchs</li>
 *  <li>Configurable minimum number of letters before starting search</li>
 *  <li>Configurable maximum number of results displayed at a time</li>
 *  <li>Allows to list users and/or groups</li>
 *  <li></li>
 * </ol>
 *
 * Arguments:
 * <ol>
 *  <li>ucModel (Mandatory): the source for the data.</li>
 *  <li>dropDownMaxSize (Optional): The maximum number of elements to display in the dropdown.</li>
 *  <li>usePointerStyle (Optional): Default is false.  If true, the ui will be styled as a pointer.
 *        If false, the ui will be styled as a combo box.</li>
 *  <li>searchDelay (Optional): The time to wait since a key is pressed to when a search is fired. In milliseconds.</li>
 *  <li>searchMinInputLength (Optional): The minimum length on the input to start a search.
 *      Values smaller than this in length will not perform any search.</li>
 *  <li>onChangeHandler (Optional): Handler function to be notified when a selection is done (or undone). 
 *                                  This handler should check
 *                                  the value of the passed in
 *                                  'choice' parameter.  If
 *                                  choice != null, the change
 *                                  is valid.</li>
 *  <li>onEnterHandler (Optional): Handler function for ENTER keystroke.</li>
 *  <li>onEscapeHandler (Optional): Handler function for ESCAPE keystroke.</li>
 * </ol>
 *
 */
dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.UserChooser", [dijit._Widget, dijit._Templated],
{

  templatePath: dojo.moduleUrl("com.ibm.bpm.social.util.ui.userchooser","templates/UserChooser.html"),
  
  /**
   * Arguments and default values. See description in the header of this class.
   */
  ucModel: null,
  assignTaskidFilter: null,
  collabTaskidFilter: null,
  namesonly: true,
  dropDownMaxSize: (dojo.isSafari && (navigator.userAgent.indexOf('iPad') != -1))? 5:10,
  usePointerStyle: false,
  multiSelect: false,
  showProgressLabel: true,
  searchDelay: 800,
  searchMinInputLength: 1,
  onChangeHandler: null,
  onEnterHandler: null,
  onEscapeHandler: null,
  topic: "com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred",
  preloadInitialSuggestions: false,

  /**
   * Indicates if this instance is inside a USerChooserDialog.
   */
  isInDialog: false,

  /**
   * Private attributes.
   */
  _progressDiv: null,
  _inputWidth: 16,
  _eventConnections: null,
  _multiSelectConnections: null,
  _highlighter: null,
  _chosenItem: null,
  _chosenItems: null,
  _buttonNumber: 1,

  //base text direction user preference
  _btd: "",
  _bodyDir: "",

  /**
   * Indicates if in the middle of a search.
   */
  _performingSearch: false,

  /**
   * The result of the last query.
   */
  _lastQueryItems: [],

  /**
   * The value searched for in the previous attempt. Used to avoid performing the same search twice.
   */
  _lastSearchValue: "",

  _hasFocus: true,
  
  _justOpenedDropdown: true,// Defect 69068

  /**
   * Id of the last scheduled timer for a search.
   */
  _pendingId: null,
  _concurrentSearches: 0,
  _nextSearchId: 0,

  widgetsInTemplate: true,

  constructor: function()
  {
  },
  
  postCreate: function()
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this.inherited("postCreate", arguments);        
    this.setAssignTaskidFilter(this.assignTaskidFilter);
    this.setCollabTaskidFilter(this.collabTaskidFilter);
    if (this.multiSelect) {
        this.usePointerStyle = false;
    }
    //this._debugArguments();
    this._initLocalization();
    this._initUiElements();
    this._setHandlers();
    this._btd = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
    this._bodyDir = document.body.dir;
    if (this._bodyDir) {
        this._bodyDir == this._bodyDir.toLowerCase();
    } else {
        this._bodyDir == "ltr";
    }
    if (dojo.isIE == 8) {
    } else {
        var self = this;
        dojo.subscribe(this.topic, self._handleRestError);
    }
 
    if (this.preloadInitialSuggestions && this.ucModel.getInitialSuggestions) {
    	var model = this.ucModel;
    	var size = this.dropDownMaxSize + this._getMaxSizeFactor();
    	var af = this.assignTaskidFilter;
    	var cf = this.collabTaskidFilter;
        var no = this.namesonly;
    	window.setTimeout(function(){model.getInitialSuggestions(size,af,cf,no);},100);
    }
  },


  _debugArguments: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "dropDownMaxSize=" + this.dropDownMaxSize,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchDelay=" + this.searchDelay,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchMinInputLength=" + this.searchMinInputLength,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "assignTaskidFilter=" + this.assignTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "collabTaskidFilter=" + this.collabTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "namesonly=" + this.namesonly,"UserChooser");
  },


  _initLocalization: function()
  {
    dojo.requireLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
  },
  
  _getFormForElement: function(e) {
  		var form = e.parentNode;
  		while (form && (!form.nodeName || (form.nodeName.toLowerCase() != 'form'))) {
  			form = form.parentNode;
  		}
  		if (!form) {
  			form = document.body;
  		}
  		return form;
	},
  
  _initUiElements: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    if (!this.multiSelect) {
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="uchInput";
        this.chooserInput.autocomplete="off";
        dojo.attr(this.chooserInput,"aria-required","true");
        dojo.place(this.chooserInput, this.inputDiv, "first");
        this._progressDiv = document.createElement("div");
        dojo.place(this._progressDiv,this.inputDiv,"last");
    } else {
        var div = document.createElement("div");             
        div.className = "multiselect-container";
        var ul = document.createElement("ul");
        ul.className = "multiselect-choices";
        ul.id = "userchooserchosenitems";
        div.appendChild(ul);
        this.inputDiv.appendChild(div);
        var li = document.createElement("li");
        li.className = "multiselect-search";
        ul.appendChild(li);
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="multiselect-input";
        this.chooserInput.autocomplete="off";
        li.appendChild(this.chooserInput);
        this._progressDiv = document.createElement("div");
        li.appendChild(this._progressDiv);
    }
    this.noUsersFoundDiv.innerHTML = this._nlsMessages.noUsersFound;
    this.noUsersFoundDiv.id = "noUsersFound_"+this.id;
    this.chooserInput.focus();
    

    if (this.usePointerStyle) {
        dojo.addClass(this.dropdownDiv,"pointer");
    } else {
        dojo.addClass(this.dropdownDiv,"dropdown");
    }

       
    if (!this.chooserInput.id) {
        var labelCounter = 0; 
        while (dojo.byId("bpsace_userChooserLabel_" + labelCounter) != null) {
            labelCounter = labelCounter + 1;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "labelCounter: "+labelCounter,"UserChooser");
        this.chooserInput.id = "bpsace_userChooserLabel_" + labelCounter;        
    }
    
    // RTC 44850 -- for A11y
    this.chooserInput.title = this._nlsMessages.typeInTextboxLabel;

    
    // Init highlighter
    this._highlighter = new com.ibm.bpm.social.util.ui.userchooser.Highlighter( this.resultsDiv, this._lastQueryItems );

		// Init shimmer.
    dojo.require("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect");
    this._shimmerUtil = new com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect("UCIFramePool_", this.isInDialog);
    
    // Set a unique id for some divs.
    if (!this.dropdownDiv.id)
    {
      this.dropdownDiv.id = this._generateUid();
    }
    if (!this.resultsDiv.id)
    {
      this.resultsDiv.id = this._generateUid();
    }    
    
    // Set into initial state.
    this._chooseItem(-1);
    this._hideResults();
    this._exitSearchingMode();
  },

  //removes duplicates from array 
  _unique: function(  /*{cn,userID,fullName,userName}[]*/ origArray, /*boolean*/ twoArrays ) {
		var newArray = [];
        var x,y,prop,entry,found,equal;		
		for (x=0; x < origArray.length; x++) {
		    found = false;
			entry = origArray[x];
            if (twoArrays) {            
    			for (y = 0; y < newArray.length; y++) {
    				equal = true;
    				for (prop in entry) {
    					if (newArray[y][prop] !== undefined) {
                            com.ibm.bpm.social.util.common.Log.log(2, "1["+prop+"]: "+entry[prop],"UserChooser");
                            com.ibm.bpm.social.util.common.Log.log(2, "2["+prop+"]: "+newArray[y][prop],"UserChooser");
    						if (entry[prop] != newArray[y][prop]) {
    							equal = false;
    							break;
    						}
    					} else {
    						equal = false;
    						break;
    					}
    				}//end for prop
    				if (equal) {
    					for (prop in newArray[y]) {
    						if (entry[prop] === undefined) {
    							equal = false;
    							break;
    						}
    					}
    				}
    			    if (equal) {
    				    found = true;
    				    break;
    				}
    			}//end for y
            }//end twoArrays
			if (!found && this.multiSelect) {//if multiSelect, remove items already selected
               if (this._chosenItems && this._chosenItems.length) {
                   for (y in this._chosenItems) {
                       equal = true;
                       for (prop in entry) {
                           if (this._chosenItems[y][prop] !== undefined) {
                               com.ibm.bpm.social.util.common.Log.log(2, "3["+prop+"]: "+entry[prop],"UserChooser");
                               com.ibm.bpm.social.util.common.Log.log(2, "4["+prop+"]: "+this._chosenItems[y][prop],"UserChooser");
                               if (entry[prop] != this._chosenItems[y][prop]) {
                                   com.ibm.bpm.social.util.common.Log.log(2, "5["+prop+"] not equal","UserChooser");
                                   equal = false;
                                   break;
                               }
                           } else {
                               com.ibm.bpm.social.util.common.Log.log(2, "6["+prop+"] undefined","UserChooser");
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           if (entry["cn"] === undefined && (entry["userID"] === undefined || entry["fullName"] === undefined)) {//need these 2 fields to determine a unique user
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           found = true;
                       }
                   }
               }
            }//end !found && multiSelect            
            if (!found) {
			   newArray.push(entry);
			}
		}
	 return newArray;
  },

  _requestSearch: function()
  {
    var searchValue = this._getSearchValue();
    com.ibm.bpm.social.util.common.Log.log(2,  "_requestSearch(): [" + searchValue + "]","UserChooser");
    if ( searchValue.length < this.searchMinInputLength ) {    
      com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. Input should be at least " + this.searchMinInputLength + " characters.","UserChooser");
      this._clearPendingId();
      this._hideResults();
    } else if (this._isInvalidWildcard(searchValue, true)) {
        com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. InvalidWildcard.","UserChooser");
        this._hideResults();
        // Unchoose previous selection
        this._chooseItem(-1);
    } else {
      // Enqueue search.
      this._requestDelayedSearch();
    }

    // Unchoose previous selection
    this._chooseItem(-1);
  },

  _isInvalidWildcard: function(searchString,showMessage) {
       //check for wildcard character in the middle of the search string
       var wildcard = searchString.indexOf('*');
       while (wildcard != -1) {
           if ((wildcard > 0) && (wildcard < (searchString.length-1))) {
               // we do not support wildcards in the middle of the string
               //var dlg = new com.ibm.bspace.common.util.widget.ErrorPopup();
               //dlg.setMessage(this._nlsMessages.invalidWildcardMessage, null, null, null);
               //dlg.destroyOnHide = true;
               //dlg.show();
               if (showMessage) {
                   dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
                   dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");
                   com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(this._nlsMessages.invalidWildcardMessage);             
               }
               return (true);
           } else {
               wildcard = searchString.indexOf('*',wildcard+1);
           }
       }
       return (false);
  },

 /**
  * Returns search value trimmed of spaces.
  * */
  _getSearchValue: function()
  {
    var searchString = this.chooserInput.value;
    // 76246 convert the % character to * so that we are only using one wildcard char
    var wildcard = searchString.indexOf('%');
    while (wildcard != -1) {
    	searchString = searchString.substring(0,wildcard)+'*'+searchString.substring(wildcard+1);
        wildcard = searchString.indexOf('%',wildcard+1);
    }
    return searchString;
    //72937 return dojo.trim( this.chooserInput.value );
  },

   /**
   *Returns search value trimmed of leading and ending wild cards.
  *  */

  _getValueNoWildCards: function(value)
  {
      if (value.length > 2 && value.charAt(0) == '"' && value.charAt(value.length-1) == '"') {
          value = value.substring(1, value.length-1);
          com.ibm.bpm.social.util.common.Log.log(2, '_getValueNoWildCards quotes found: '+value,"UserChooser");
      }
      if (value.length > 1) {
          if (value.substr(0,1) == '*') {
              value = value.substr(1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards beg * found: "+value,"UserChooser");
          }
          if (value.length > 1 && value.indexOf('*',value.length-1) > -1) {
              value = value.substring(0,value.length-1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards end * found: "+value,"UserChooser");
          }
      }
    return value;    
  },

   /**
    * Returns the id of the input field generated for the user chooser
    */
   getInputId: function()
   {
      return this.chooserInput.id;
   },
   
   /**
    * Sets the aria-labelled by field for the generated input field for the user chooser to the value in the input string
    * which should be the id field of the corresponding label tag
    */
   setAriaLabelledby: function( /*String*/ labelId ) {
   
   	if (this.chooserInput) {
   		
   		dojo.attr(this.chooserInput, "aria-labelledby", labelId);
   	}
   },

  /**
   * Handles a delay before calling _performSearch(). Cancels all pending requests before scheduling a new one.
   */
  _requestDelayedSearch: function()
  {

    // Cancel pending request if any.
    this._clearPendingId();

    // If different from previous search, do a search.
    var searchValue = this._getSearchValue();
    if (searchValue == this._lastSearchValue)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Ignoring search request. Search value is the same as last request: '" + searchValue + "'.","UserChooser");
    }
    else
    {
      if (searchValue == "")
      {
        this._processEmptyState();
      }
      else
      {
        var self = this;
        this._pendingId = window.setTimeout(function() 
                          {
                            self._lastSearchValue = searchValue;
                            self._performSearch(searchValue);
                          }, 
                          this.searchDelay);
      }
    } // else
  },

  
  _enterSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOn";
  },


  _exitSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOff";
  },


  /**
   * Executes a search against VMM.
   */
  _performSearch: function( /*String*/ searchString )
  {
    var searchId = this._nextSearchId++;
    this._concurrentSearches++;
    com.ibm.bpm.social.util.common.Log.log(2, "START    search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches,"UserChooser");

    this._enterSearchingMode();

	//new code to check for mashups wild card setting
	com.ibm.bpm.social.util.common.Log.log(2, "get mashups service","UserChooser");
	var useWildCards = true;
	try {
		var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		if (cs == null)
		   com.ibm.bpm.social.util.common.Log.log(2, "could not find mashups service", "UserChooser");
		else {
		   var autoWildCard = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.AUTO_WILDCARD_DISABLED);
		   com.ibm.bpm.social.util.common.Log.log(2, "autowildcard value =" + autoWildCard,"UserChooser");
		   if (autoWildCard == true) 
		   {
			   useWildCards = false;
		   }
		}
	} catch (e2) {
		// if we can't get mashups, then assume wildcards are ok
        com.ibm.bpm.social.util.common.Log.log(2, "error getting mashups service: "+e2 ,"UserChooser");
	}


	//end code to check for mashups wild card setting
    // Add wildcards
	var searchStringToUse;
	var equalSearchString;
	if (useWildCards == true) 
	{
       searchStringToUse = "*";
       if (searchString && searchString != "") {
    	   searchStringToUse = searchString;
		   if (searchString.length > 2 && searchString.charAt(0) == '"' && searchString.charAt(searchString.length-1) == '"') {
			   searchStringToUse = searchString.substring(1, searchString.length-1);
		   }
		   else {
			   if (searchString.charAt(0) != '*') {
				   searchStringToUse = '*' + searchStringToUse;
			   }
			   if (searchString.charAt(searchString.length-1) != '*') {
				   searchStringToUse = searchStringToUse + '*';
			   }
		   }
       }
	   equalSearchString = searchString;
	}
	else 
	{
	   searchStringToUse = searchString;
	}

	com.ibm.bpm.social.util.common.Log.log(2, "Search string is " + searchStringToUse,"UserChooser");

	var modelItems = [];
    // Request 1 more element than can be displayed, so to identify if there are more elements than we are displaying.
	if (useWildCards == true) {
		//78062 var modelItems1 = this.ucModel.query( equalSearchString, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		var modelItems2 = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		
		/* 78062 if (modelItems1.length > 0) {
		   if (modelItems2.length > 0) {
		      modelItems = this._unique(modelItems1.concat(modelItems2),true);
		   } else {
		      modelItems = this._unique(modelItems1,false);
		   }
		} else */
        if (modelItems2.length > 0) {
			modelItems = this._unique(modelItems2,false);
		}
	} else {
		modelItems = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
        modelItems = this._unique(modelItems,false);
	}

    //var modelItems = [];

    // Are we displaying all the items?
    var allItemsDisplayed = true;
    if ( modelItems.length >= (this.dropDownMaxSize + this._getMaxSizeFactor()) )
    {
      // There are more elements than the ones to be displayed. Set a flag indicating this.
      allItemsDisplayed = false;

      // Remove last unwanted element.
      modelItems.length = this.dropDownMaxSize;
    }
    else if ( modelItems.length <= this.dropDownMaxSize )
    {
      // We are ready to display all query elements. allItemsDisplayed is true by default.
    }
    else if ( modelItems.length <= 0 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Got " + modelItems.length + " results." ,"UserChooser");
      // No matches or ucModel (or VMM in the case of VMMModel) is down.
    }
    else
    {
      // Unexpected situation.
      // TODO use NLS. Use common dialogs.
      com.ibm.bpm.social.util.common.Log.log(2,  "*** ucModel is returning " + modelItems.length + " when the maximum allowed is " + this.dropDownMaxSize ,"UserChooser");
      modelItems.length = this.dropDownMaxSize;
    }

    // This search is finishing in sequence? If not, discard.
    var outOfSequenceSearch = (searchId < this._nextSearchId-1);
    if (outOfSequenceSearch)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Out of sequence search: " + searchId + ". Discarding results.","UserChooser");
    }
    else
    {
      // Populate dropdown with search result.
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
      this._exitSearchingMode();
    }
    
    this._concurrentSearches--;
    if (modelItems.length == 0) {
        dojo.removeClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"true","aria-describedby":this.noUsersFoundDiv.id});

    } else {
        dojo.addClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    com.ibm.bpm.social.util.common.Log.log(2,  "FINISHED search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches + " : " + modelItems.length + " results.","UserChooser");
  },

  _getMaxSizeFactor: function() {
    var num = 1;
    if (this.multiSelect && this._chosenItems) {
        num = this._chosenItems.length + 1;
    }
    com.ibm.bpm.social.util.common.Log.log(2, "_getMaxSizeFactor: "+num,"UserChooser");
    return num;
  },

  _setModelItems: function( /*{cn,userID,fullName,userName}[]*/ modelItems )
  {
    this._lastQueryItems = modelItems;

    // Update highlighter model.
    this._highlighter.updateModel( modelItems );
    this._highlighter.reset();
  },


  _setModelItemsAndUpdateView: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*boolean*/ allItemsDisplayed )
  {
    this._setModelItems( modelItems );
    this._populateDropdown( modelItems, allItemsDisplayed );

    // The component can loose focus before the search is finished, so only show the dropdown if still 
    // with focus.
    if (this._hasFocus)
    {
      this._showResults();
    }
    this._highlighter.reset();

    // For BPM, we are not using the cn or userID fields for searching, so skip this for now
    // If the typed text is a value (cn or userID) in the model, then choose it.
    //this._tryAutomaticChoosing(modelItems);
  },


  /**
   * If the typed text is a value (cn or userID) in the model, then choose it.
   */  
  _tryAutomaticChoosing: function(/*{cn,userID,fullName,userName}[]*/ modelItems)
  {
    var searchValue = this._getValueNoWildCards(this._getSearchValue());
    
    // If there are items, try to choose automatically.
    if (modelItems.length > 0)
    {
      var index = this._searchModelForField( modelItems, searchValue, "cn" );
      com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (cn) on '" + searchValue + "': " + index ,"UserChooser");
      if (index != -1)
      {
        com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
        this._chooseItem( index );
      }
      else
      {
        // Search for id
        var index = this._searchModelForField( modelItems, searchValue, "userID" );
        com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (userID) on '" + searchValue + "': " + index ,"UserChooser");
        if (index != -1)
        {
          com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
          this._chooseItem( index );
        }
      }
    }
    else
    {
      // Set to true to enable free form choice.
      var freeFormChoiceEnabled = false;
      if (freeFormChoiceEnabled)
      {
        // If no values are returned from query (no actuall matches or ucModel down), allow free-form entry.
        com.ibm.bpm.social.util.common.Log.log(2, "Free-form choice made: " + searchValue,"UserChooser");
        this._chooseItemValue( {cn:"", userID:"", fullName: searchValue} );
      }
    }
  },
  

  /**
   * Searchs modelItems for an element with the same value in the searchField as specified.
   * The search is case insensitive.
   * @return The index of the found element or -1 if none.
   */
  _searchModelForField: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*String*/ searchValue, /*String*/ searchField )
  {
    var index = -1;
    if (searchValue && searchValue!="" && searchField && searchField!="")
    {
      var searchValueLC = searchValue.toLowerCase();
      for( var i=0; i<modelItems.length; i++ )
      {
        var currentValue = modelItems[i][searchField];
        if (currentValue && currentValue.toLowerCase && currentValue.toLowerCase() == searchValueLC)
        {
          index = i;
          break;
        }
      }
    }
    return index;
  },

  /**
   * Show the results if there is at least one item. Otherwise ignore.
   */
  _showResults: function()
  {
  	 
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    if ( this._lastQueryItems.length > 0 && this.resultsDiv.childNodes.length > 0 )
    {
      var optimizeShimmer;
      if ( dojo.hasClass(this.dropdownDiv,"hidden") )
      {
        dojo.removeClass(this.dropdownDiv,"hidden");

        // If just opened the dropdown, initialize highlight.
        this._highlighter.reset();
        this._justOpenedDropdown = true;// Defect 69068
        
        // Do not optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Not visible yet. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = false;
      }
      else
      {
        // Already visible and possible changed size. Optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Already visible. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = true;
      }
      this._shimmerUtil.reapplyShimmersOn(this.resultsDiv, optimizeShimmer);
      // Defect 69068 User Chooser in Modify Task/Collaboration prefills the selection with first user from result
      // comment out the next line to prevent prefilling
      // this._chooseItem( this._highlighter.index, dojo.keys.DOWN_ARROW );//allow JAWS to speak this selection
    }
    else
    {
      this._hideResults();
    }
  },


  _hideResults: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    dojo.addClass(this.dropdownDiv,"hidden");
    this._shimmerUtil.removeShimmer(this.resultsDiv);
    this._lastSearchValue = null;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
  },


  /**
   * Generates a UID.
   */
  _generateUid: function() /*string*/
  {
    var newId =  "bspace.userchooser." + new Date().getTime();
    if (newId == this._lastGeneratedUid)
    {
      newId += ".x";
    }
    
    this._lastGeneratedUid = newId;
    return newId;
  },
  
  
  _resultsAreVisible: function()
  {
    return dojo.hasClass(this.dropdownDiv,"hidden") == false;
  },


  /**
   * Populates the dropdown with elements created from the specified items.
   */
  _populateDropdown: function( /*{cn,userID,fullName,userName}[]*/ items, /*boolean*/ allItemsDisplayed )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    // Disconnect previous
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = [];

    // Clean previous values.
    this._removeAllChildren(this.resultsDiv);

    // Add new values
    for( var i=0; i<items.length; i++ )
    {
      var qItem = items[i];

      // Calculate display string.
      var identifier = (qItem.fullName && qItem.fullName!="") 
      						? qItem.fullName 
      						: null;
      //format for BPMModel
      //  "userID":d2.userID,
      //  "fullName":d2.fullName,
      //  "userName":d2.userName} 
      //  var dispHtml = qItem.cn + (identifier ? ("&rlm; [ " + identifier + " ]") : "");
      var dispHtml = identifier;
      var rlm = "";
      if (document.body.dir == "rtl") {
          //rlm = "&rlm;";//after applying BTD, this doesn't seem to anything
      }
      if (identifier && qItem.userName && qItem.userName!="") {
          dispHtml = this._createDisplayName(identifier, qItem.userName );//identifier + (rlm+" [ " + qItem.userName + " ]") ;
      }
/* to test with userID      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + (rlm+" [ " + qItem.userID + " ]") ;
      }
*/
/*BSpace      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + ("&rlm; [ " + qItem.userID + " ]") ;
      }
*/
      //format for other models
      var inputName = "";
      if (!dispHtml) {
          inputName = qItem.cn + (identifier ? rlm: "");
          dispHtml = this._createDisplayName(qItem.cn, identifier);//qItem.cn + (identifier ? (rlm+" [ " + identifier + " ]") : "");
      }
      this._setInputWidth(inputName);
      com.ibm.bpm.social.util.common.Log.log(2, "dispHtml: "+dispHtml,"UserChooser");

      // Bold matching substring
      if (this._lastSearchValue && this._lastSearchValue.length > 0)
      {
        dispHtml = this._emphasize( dispHtml, this._getValueNoWildCards(this._lastSearchValue) );
      }

      // Add element with display string.
      var item = this._addChild( this.resultsDiv, "div", dispHtml);
      item.className = "uchDropdownItem";
      
      
      // Hook event handlers.
      this._dropdownEventConnections.push(dojo.connect( item, "onmouseover", this, this._menuItemMouseOver ));
      this._dropdownEventConnections.push(dojo.connect( item, "onmousedown", this, this._menuItemMouseDown ));
    }
    
    // If needed, add a last DIV with a special message indicating that not all items are being displayed.
    if ( this.showProgressLabel && !allItemsDisplayed )
    {
      var msg = dojo.string.substitute( this._nlsMessages.progressLabel, {1:this.dropDownMaxSize} );
      var item = this._addChild( this.resultsDiv, "div", msg );
      item.className = "uchDropdownMessage";
    }
  },

  _createDisplayName: function(fullname, username) {
       if (this._btd == "default") {
           return fullname + (username ? (" [ " + username + " ]") : "");
       }
       var btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(fullname);
       if (this._btd == "ctx") {//must apply UCC to fullname and username separately
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(username);
               username = this._applyUCC(username, btdDir);
           }
           return fullname + (username ? (this._getMarker()+" [ " + username + " ]") : "");
       } else {
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               username = this._applyUCC(username, btdDir);
           }
           return this._getUCCWrapper() + fullname + (username ? this._getMarker()+" [ " + username + " ]"+com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF : "");
       }
       return fullname + (username ? (" [ " + username + " ]") : "");
  },

  _applyUCC: function(text, btdDir) { 
       var controlChar = (btdDir == "ltr") ? 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

       //wrap the text in the control characters
       text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;
       return text;       
  },

  _getMarker: function() {
    var marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
    if (this._bodyDir == "rtl") {
        marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;
    }    
    return marker;
  },


  _getUCCWrapper: function() {
       if (this._bodyDir == "ltr") {
           return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE;
       } 
       return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;
  },

  _setInputWidth: function(str) {       
       var len = str.length;
       if (len && len > this._inputWidth) {
           this._inputWidth = len;
           dojo.style(this.chooserInput,"minWidth",(len/2.15)+"em");
       }
  },

  /**
   * Emphasizes the matching text in the source text.
   */
  _emphasize: function( /*String*/ source, /*String*/ toMatch )
  {
    // Homogeinize strings to do case-insensitive matching.
    var sourceLC = source.toLowerCase();
    toMatch = toMatch.toLowerCase();
    var replaced = new String();
    var toMatchLength = toMatch.length;
    var idx = 0;
    while( idx < source.length )
    {
      var startIdx = idx;
      // Search on the lowercase version.
      idx = sourceLC.indexOf( toMatch, idx );
      if ( idx == -1 )
      {
        // Add previous text from the original version.
        var preFoundStr = source.substring( startIdx );
        replaced += preFoundStr;
        break;
      }
      else
      {
        // Add previous text
        var preFoundStr = source.substring(startIdx, idx);
        replaced += preFoundStr;

        // Add new replacement. Use text from original str, to match case.
        var foundStr = source.substr(idx, toMatch.length);
        replaced += "<b>" + foundStr + "</b>";

        idx += toMatchLength;
      }
    }

    com.ibm.bpm.social.util.common.Log.log(2, "_replaced('" + source + "', '" + toMatch + ")='" + replaced + "'","UserChooser");
    return replaced;
  },


  _processEmptyState: function()
  {
    if ( this._getSearchValue() == "" || this._isInvalidWildcard(this._getSearchValue()) )
    {
      // Show initial suggestions
      var modelItems = (this.ucModel.getInitialSuggestions) ? this.ucModel.getInitialSuggestions(this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly) : [];
      
      
      // Are we displaying all the items?
      var allItemsDisplayed = true;
      if ( modelItems.length > this.dropDownMaxSize )
      {
        // There are more elements than the ones to be displayed. Set a flag indicating this.
        allItemsDisplayed = false;
  
        // Remove last unwanted element.
        modelItems.length = this.dropDownMaxSize;
      }
      
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
    }
  },


  /**
   * Sets new handlers. The previous handlers will be disconnected.
   */
  _setHandlers: function()
  {
    // Disconnect previous
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = [];
    
    // Hook event handlers.
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeydown", this, this._onKeyDown ));
	this._eventConnections.push(dojo.connect( this.chooserInput, "onkeypress", this, this._onKeyPress ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeyup", this, this._onKeyUp ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onfocus", this, this._onUCFocus ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onblur", this, this._onUCBlur ));
  },



  /**
   * Special treatment to identify the correct DIV in the menu.
   *
   * Because of construction, is necessary to avoid the selection of an inner span in the DIV (this happens with the <B> tags inside the
   * DIV when bolding a part of the string).
   * If a SPAN is identified, is assumed that the parent is the div.
   * Note: this only makes sense according to the construction of the dropdown.
   * If that implementation is changed or the events associated with any other DIV here, this will not work anymore.
   *
   * TODO Find a better solution. This works on FF3 but not on IE7?
   */
  _getSmartTarget: function( /*Event*/ e )
  {
    var smartTarget = null;
    var target = e.target;
    if ( target )
    {
      if ( dojo.isIE )
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - IE" ,"UserChooser");
        smartTarget = target;
        while (smartTarget && (smartTarget.nodeName.toLowerCase() != "div")) {
        	smartTarget = smartTarget.parentNode;
        }
      }
      else
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - not IE" ,"UserChooser");
        if(target.tagName == "B") {
          target = target.parentNode;
        }
        if ( target instanceof HTMLDivElement)
        {
          smartTarget = target;
        }
        else if (((HTMLSpanElement && target instanceof HTMLSpanElement) || target.tagName == "SPAN") && target.parentNode instanceof HTMLDivElement)
        {
          smartTarget = target.parentNode;
        }
        else
        {
          // Ignore
        }
      }
    }

    return smartTarget;
  },


  _menuItemMouseOver: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_menuItemMouseOver()" ,"UserChooser");
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._highlighter.moveSelectionToItem( smartTarget );
    }
  },


  _menuItemMouseDown: function( /*Event*/ e )
  {
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._hideResults();
      this._chooseItem( this._highlighter._getIndexForDivItem( smartTarget ) );
    }
  },

  _onKeyPress: function( /*Event*/ e) {
    com.ibm.bpm.social.util.common.Log.log(2, "keyCode="+e.keyCode+" charCode="+e.charCode,"UserChooser");
    if((e.charCode>0)) {
      setTimeout(dojo.hitch(this, "_requestSearch"), 10);
    }
  },

  _onKeyUp: function( /*Event*/ e )
  {
    if (e.keyCode == dojo.keys.BACKSPACE || e.keyCode == dojo.keys.CLEAR || e.keyCode == dojo.keys.DELETE) {
      this._requestSearch();
    }
  },


  _onKeyDown: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_onKeyDown(): " + e.keyCode ,"UserChooser");
    var key = e.keyCode;
    //var key = (window.event) ? event.keyCode : e.keyCode;

    if (key == dojo.keys.DOWN_ARROW)
    {
      this._showResults();
      //First time dropdown is opened, use the down arrow key to select the first item
      //instead of moving to the next item
      if ( !this._justOpenedDropdown ) {// Defect 69068
      if ( this._highlighter.moveSelectionForward() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
      } else {
          this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
          this._justOpenedDropdown = false;// Defect 69068
      }
    }
    else if (key == dojo.keys.UP_ARROW)
    {
      this._showResults();
      if ( this._highlighter.moveSelectionBack() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
    }
    else if (key == dojo.keys.ENTER)
    {
      if ( this._resultsAreVisible() )
      {
        // Choose the currently selected item.
        this._hideResults();
        this._chooseItem( this._highlighter.index, key );
      }
      else
      {
        this._onEnterHandler();
      }
      dojo.stopEvent(e);
    }
    else if (key == dojo.keys.TAB)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
      }
    }
    else if (key == dojo.keys.ESCAPE)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
        com.ibm.bpm.social.util.common.Log.log(2, "escape detected, unchoose item","UserChooser");
        this._clearPendingId();
        this._chooseItem(-1);//make unchoose item the default behavior for escape
        this._exitSearchingMode();
      }
      else
      {
        this._onEscapeHandler();
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _clearPendingId: function() {
     if (this._pendingId)
     {
        com.ibm.bpm.social.util.common.Log.log(2, "Canceling timer: '" + this._pendingId + "'.","UserChooser");
        window.clearTimeout(this._pendingId);
        this._pendingId = null;
     }
  },

  /**
   * Chooses the specified item as the user choice. The value in this._lastQueryItems in that index will be returned when getChoice
   * is called.
   * If itemIndex is -1, will unchoose any previous selection.
   */
  _chooseItem: function( /*Number*/ itemIndex, /*KeyCode*/ keyCode )
  {
    var item = (itemIndex in this._lastQueryItems) ? this._lastQueryItems[itemIndex] : null;
    this._chooseItemValue(item, keyCode);
  },


  /**
   * Chooses the specified item as the user choice. The specified item will be returned when getChoice is called.
   * If item is null, will unchoose any previous selection.
   */
  _chooseItemValue: function( /*{cn,userID,fullName,userName}*/ item, /*KeyCode*/ keyCode )
  {
    var unchoose = true;
    if (item)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Item chosen: " + item + ":" + dojo.toJson(item),"UserChooser");
      this._chosenItem = item;
      
      // Replace input string with match.
      if (item.cn && item.cn != "")
      {
        this.chooserInput.value = item.cn;
      } else if (item.fullName && item.fullName != "") {
    	this.chooserInput.value = item.fullName;
      }

      if (!keyCode || (keyCode && keyCode === dojo.keys.ENTER)) {
          this._onChangeHandler();
      }
      unchoose = false;
      
      // Forget last searched value.
      this._lastSearchValue = null;
    }

    if ( unchoose )
    {
      this._chosenItem = null;
      this._onChangeHandler();
    }
    //com.ibm.bpm.social.util.ui.TextField.setIncorrectEntry(this.chooserInput,unchoose);
  },

  setAssignTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.collabTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter found collabTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting collabTaskidFilter to null.","UserChooser");
         this.collabTaskidFilter = null;
     }
     this.assignTaskidFilter = taskid;
  },

  setCollabTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.assignTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter found assignTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting assignTaskidFilter to null.","UserChooser");
         this.assignTaskidFilter = null;
     }
     this.collabTaskidFilter = taskid;
  },

  _onChangeHandler: function() {  
    if (this.onChangeHandler) {
      if (this.multiSelect) {
           this._onChangeHandlerMultiselect();
          this.onChangeHandler(this._chosenItems);
      } else {      
          this.onChangeHandler(this._chosenItem);
      }
    }  
  },

  _onChangeHandlerMultiselect: function() {
   if (this._chosenItem != null) {
       if (!this._chosenItems) {
           this._chosenItems = new Array();
           if (!this._multiSelectConnections) {
               this._multiSelectConnections = new Array();
           }
       }
       var li = document.createElement("li");
       li.className = "choice";
       var span = document.createElement("span");
       var displayName =this._chosenItem.fullName;
       if (displayName == undefined) {
           displayName = this._chosenItem.userName;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.cn;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.userID;
       }
       span.innerHTML = displayName;
       li.appendChild(span);
       var button = document.createElement("button");
       button.className = "multiselect-delete-button";
       var idStr = this._getNextButtonId();
       button.id = idStr;
       span.id = idStr+"_span";
       dojo.attr(button, "aria-label", this._nlsMessages.deleteButtonLabel);
       dojo.attr(button, "aria-describedby", idStr+"_span");
       this._multiSelectConnections.push(dojo.connect( button, "onclick", this, this._multiSelectDeleteChoice ));
       var hcSpan = document.createElement("span");//a11y high contrast
       hcSpan.innerHTML = "X";
       hcSpan.className = "multi-select-delete-button-hc";
       button.appendChild(hcSpan);
       li.appendChild(button);
       dojo.place(li,"userchooserchosenitems",this._chosenItems.length);
       this._chosenItem = 
               {"cn":this._chosenItem.cn,
               "userID":this._chosenItem.userID,
               "fullName":this._chosenItem.fullName,
               "userName":this._chosenItem.userName,
               "buttonId": idStr};
       this._chosenItems.push(this._chosenItem);
       this._chosenItem = null;
       this.chooserInput.value = "";
       this._resetInputWidth();
   }
  },

  _resetInputWidth: function(){
       this._inputWidth = 16;
       dojo.style(this.chooserInput,"minWidth",this._inputWidth+"em");
  },

  _multiSelectDeleteChoice: function(e) {
    com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice id: "+e.target.id,"UserChooser");
    dojo.disconnect(e.target);
    //userId and fullName
    var node = document.getElementById(e.target.id);
    if (node) {//button anchor node
      node = node.parentNode;//parent li node
      if (node) {      
          node.parentNode.removeChild(node);
          for (var i in this._chosenItems) {
              var item = this._chosenItems[i];
              if (item.buttonId == e.target.id) {
                  this._chosenItems.splice(i,1);
                  //com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice deleted index: "+i+" _chosenItems.length: "+this._chosenItems.length,"UserChooser");
                 if (this._chosenItems.length == 0) {
                      this._chosenItems = null;
                  }
                  break;
              }
          }
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _getNextButtonId: function() {
       var num = this._buttonNumber++;
       //com.ibm.bpm.social.util.common.Log.log(2, "_getNextButtonId num: "+num,"UserChooser");
       return "chosenitem_"+num;
  },

  _onEscapeHandler: function()
  {
    if (this.onEscapeHandler)
    {
      this.onEscapeHandler();
    } 
  },


  _onEnterHandler: function()
  {
    if (this.onEnterHandler)
    {
      this.onEnterHandler();
    }
  },


  _onUCFocus: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = true;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    if ( this._getSearchValue() == "" )
    {
      this._processEmptyState();
    }
    else if (this._isInvalidWildcard(this._getSearchValue())) {
      this._processEmptyState();
    }
    else
    {
      if (!this._chosenItem) {
    	  this._requestSearch();
      }
    }
  },


  /**
   * Cannot use _onBlur name to avoid collision with Dojo core.
   */
  _onUCBlur: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = false;
    this._hideResults();
    this._exitSearchingMode();
    if (this._chosenItem == null) {//if nothing selected, clear input field
        this.chooserInput.value = "";
    } else {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    dojo.addClass(this.noUsersFoundDiv,"hidden");    

    // Defect 69068 - don't autoselect - may not be what the user wanted
    // If only one item in the last query, choose it.
    //if ( this._lastQueryItems.length == 1 )
    //{      
      //this._chooseItem( 0 );
    //}
  },
  
  _handleRestError: function() {
       com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
       this.chooserInput.value = "";
       this._exitSearchingMode();
       this._clearPendingId();
	   com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },

  /**
   * @return The currently choosen value or null if none.
   */
  getChoice: function() /*{cn,userID,fullName,userName}*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "getChoice(): " + dojo.toJson(this._chosenItem),"UserChooser");
    return this._chosenItem;
  },


  _addHtmlSelectOptions: function( /*DOMSelect*/ domSelect, /*String[]*/ options )
  {
    for( var i=0; i<options.length; i++ )
    {
      var option = document.createElement("option");
      option.value = options[i];
      option.innerHTML = options[i];
      domSelect.appendChild(option);
    }
  },


  _removeAllChildren: function( /*DOMSelect*/ domSelect )
  {
    while( domSelect.hasChildNodes() )
    {
      domSelect.removeChild( domSelect.lastChild );
    }
  },


  _addChild: function( parent, /*String*/ elementName, /*String?*/ innerHTML )
  {
    var newElem = document.createElement(elementName);
    parent.appendChild(newElem);

    if ( innerHTML )
    {
      newElem.innerHTML = innerHTML;
    }
    return newElem;
  },


  destroy: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    // Destroy pending event handlers.
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = null;
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = null;
    dojo.unsubscribe(this.topic);
    this.inherited("destroy", arguments);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },


  _disconnectEvents: function(/*handler[]*/ handlers)
  {
    if (handlers && handlers.length > 0)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Disconnecting " + handlers.length + " event handlers.","UserChooser");
      while(handlers.length)
      {
        dojo.disconnect(handlers.pop());
      }
    }
  },
  
  
  _log: function( message )
  {
    //console.log( message );
  }
}
);


dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.Highlighter", null,
{
  index: null,
  item: null,
  _resultsDiv: null,
  _model: null,

  constructor: function( resultsDiv, model )
  {
    this.index = null;
    this.item = null;
    this._resultsDiv = resultsDiv;
    this._model = model;
  },


  updateModel: function( model )
  {
    this._model = model;
  },


  reset: function()
  {
    // Unselect previous item.
    this.unselectCurrent();

    // If the model has elements, select first one. Otherwise, no one.
    if ( this._model && 0 < this._model.length )
    {
      this.index = 0;
      this.selectItemIndex( this.index );
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  unselectCurrent: function()
  {
    if (this.item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "_unselectCurrent(): " + this.item.innerHTML ,"UserChooser");
      this.item.className = "uchDropdownItem";
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  /**
   * Selects the specified div, remembers the last selection and calculate the index.
   * It is expected that item is in this.resultsDiv.childNodes.
   */
  selectItem: function( /*HTMLDiv*/ item )
  {
    if (item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "selectItem(): "  + item.innerHTML,"UserChooser");
      item.className = "uchDropdownItemSelected";
      this.item = item;
      this.index = this._getIndexForDivItem( item );
    }
  },


  selectItemIndex: function( /*Number*/ itemIndex )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    var divItems = this._resultsDiv.childNodes;
    if ( itemIndex != null && itemIndex < divItems.length )
    {
      this.item = divItems[itemIndex];
      if ( this.item )
      {
        this.item.className = "uchDropdownItemSelected";
        this.index = itemIndex;
      }
      else
      {
        this.index = null;
        com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex ,"UserChooser");
      }
    }
    else
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex + ", divItems.length=" + divItems.length ,"UserChooser");
    }
  },


  moveSelectionToItemIndex: function( /*Number*/ itemIndex )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItemIndex(): " + itemIndex ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItemIndex( itemIndex );
  },


  moveSelectionToItem: function( /*HTMLDiv*/ item )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItem(): " + item.innerHTML ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItem( item );
  },


  _getIndexForDivItem: function( /*HTMLDiv*/ item )
  {
    var index = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf( this._resultsDiv.childNodes, item );
    if ( index == -1 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found: "  + item.innerHTML ,"UserChooser");
      com.ibm.bpm.social.util.common.Log.log(2,  "**: "  + dojo.toJson(item) ,"UserChooser");
    }

    return index;
  },


  moveSelectionForward: function()
  {
    var moved = false;
    if (this.index != null && this.index < this._model.length-1)
    {
      this.index++;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },


  moveSelectionBack: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "moveSelectionBack","UserChooser");
    var moved = false;
    if (this.index != null && 0<this.index)
    {
      this.index--;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },

  
  debug: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2,  "index:" + this.index + ", item:" + this.item.innerHTML ,"UserChooser");
  },


  _log: function( message )
  {
    //console.log( "Highlighter: " + message );
  }
}
);
