/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.ProcessList");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("com.ibm.bpm.social.widgets.SmartList");

/* ProcessList - A dojo widget responsible for displaying a list of processes. 
	Originally created for the FollowedProcessList and LaunchableProcessList
	widgets. Each item in a process list has a clickable icon and title. 
	SmartList was later created as a base class to allow for more generic, non-
	process lists (such as TaggedProcessist, which  isn't a ProcessList 
	proper).
	
	Updated: 1/9/2012
	Experts: Ben Myers @ Raleigh, Scott Johnson @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.ProcessList",
	[com.ibm.bpm.social.widgets.SmartList, dijit._Templated],
	{
	
	templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets", "templates/ProcessList.html"),
	constructor: function() {
		//dojo event connectors are added to an array so they can be disconnected
		this.processListEventConnectors = [];
		
		//used with _restoreFocusedItem and related event handlers
		this.currFocusIndex = null;
		this.currFocusIsIcon = false;
	},
	
	postCreate: function() {
		//localization
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.processListEventConnectors = [];
		
		this.inherited(arguments);
	},
	
	/* addProcessItem: Adds an item to the ProcessList with title as its text. 
		When the item is clicked any params it was created with will be 
		exposed, so the event can be better handled. */
	addProcessItem: function(title, params) {
		if(typeof(params)=="undefined" ) {
			params = null;
		}
		return this.addItem(params, {'title':title});
	},
	
	/* _createItem: Handles creating each process item's HTML elements. Params 
		is provided when addProcessItem is called. */
	_createItem: function(index, li, params, createParams) {
		var title = "<i>Unknown</i>";
		if(createParams!=null && typeof(createParams)!='undefined') {
			title = createParams['title'];
		}
		
		//create the HTML elements
		var icon = dojo.create("div", {'class':"process-icon "+params['iconClass'], 'tabIndex':"0"}, li);
		dijit.setWaiRole(icon, "button");
		var text = dojo.create("a", {'class':"process-text", 'innerHTML':title}, li);
		dijit.setWaiRole(text, "link");
		
		//add dojo event connectors
		this.processListEventConnectors.push(dojo.connect(icon, 'keypress', this, "_processIconKeyPressEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onclick', this, "_processIconClickEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onfocus', this, "_processIconFocusEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onblur', this, "_processIconBlurEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onmouseenter', this, "_processIconMouseoverEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onmouseleave', this, "_processIconMouseoutEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'keypress', this, "_processTextKeyPressEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onclick', this, "_processTextClickEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onfocus', this, "_processTextFocusEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onblur', this, "_processTextBlurEvent"));
	},
	
	/* Below are event handlers that do little other than handle basic 
		interactions with the ProcessList and usually end up calling 
		_onProcessIconClicked or _onProcessTextClicked. */
	_processIconMouseoverEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['li']!=null) {
			dojo.addClass(dojo.query(".process-icon", info['li'])[0], "hover");
		}
	},
	_processIconMouseoutEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['li']!=null) {
			dojo.removeClass(dojo.query(".process-icon", info['li'])[0], "hover");
		}
	},
	_processIconClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['index']!=null && info['li']!=null) {
			this._onProcessIconClicked(info['index'], info['li'], info['params']);
		}
	},
	_processIconKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt);
			if(info['index']!=null && info['li']!=null) {
				this._onProcessIconClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_processTextClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['index']!=null && info['li']!=null) {
			this._onProcessTextClicked(info['index'], info['li'], info['params']);
		}
	},
	_processTextKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt);
			if(info['index']!=null && info['li']!=null) {
				this._onProcessTextClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	
	/* Below are event handlers that allow the ProcessList to remember what 
		element was focused even after the list is torn apart by clearList. 
		This is espcially useful for keyboard-only users. */
	_processIconFocusEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		this.currFocusIndex = info['index'];
		this.currFocusIsIcon = true;
	},
	_processTextFocusEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		this.currFocusIndex = info['index'];
		this.currFocusIsIcon = false;
	},
	_processIconBlurEvent: function(evt) {
		this.currFocusIndex = null;
	},
	_processTextBlurEvent: function(evt) {
		this.currFocusIndex = null;
	},
	
	/* _restoreFocusedItem: Since DOM elements may be deleted and recreated 
		when the ProcessList is refreshed, it is necessary to restore focus 
		to the last element the user interacted with to support 
		keyboard-only users. Subclasses of ProcessList should call this method 
		anytime the process list is rebuilt. */
	_restoreFocusedItem: function() {
		var item = this.getItem(this.currFocusIndex);
		if(item!=null && item['li']!=null) {
			var focusTarget = null;
			if(this.currFocusIsIcon) {
				focusTarget = dojo.query(".process-icon", item['li'])[0];
			}
			else {
				focusTarget = dojo.query(".process-text", item['li'])[0];
			}
			if(focusTarget!=null) {
				focusTarget.focus();
			}
		}
	},
	
	/* _onProcessIconClicked: Called whenever an item's icon is clicked. The 
		params argument is provided when the item is created and should hold 
		whatever information is required to properly handle the event. */
	_onProcessIconClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onProcessIconClicked: Called whenever an item's text is clicked. The 
		params argument is provided when the item is created and should hold 
		whatever information is required to properly handle the event. */
	_onProcessTextClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemMouseOver: Called whenver an item is moused over. Overridden to 
		set the process icon's class to an appropriate css sprite class. */
	_onItemMouseOver: function(index, li, params) {
		var icon = dojo.query(".process-icon", li)[0];
		dojo.removeClass(icon, params['iconClass']);
		dojo.addClass(icon, params['iconClassHighlight']);
	},
	
	/* _onItemMouseOut: Called whenver an item is moused out. Overridden to 
		set the process icon's class to an appropriate css sprite class. */
	_onItemMouseOut: function(index, li, params) {
		var icon = dojo.query(".process-icon", li)[0];
		dojo.removeClass(icon, params['iconClassHighlight']);
		dojo.addClass(icon, params['iconClass']);
	},
	
	/* clearList: Make sure to clear extra event connectors. */
	clearList: function() {
		this.inherited(arguments);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.processListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
	},
	
	destroy: function(finalize) {
		this.inherited(arguments);
	}
	
});


