/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.SmartList");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

/* SmartList - A dojo widget responsible for making life easier when handling 
	lists of items that need to be refreshed. Used most notably in the social 
	portal sidebar for LaunchableProceeList, FollowedProcessList, and 
	TaggedProcessList. Functions to override in subclasses (optional): 
	_createItem, _onItemAdded, _onItemClicked, and refresh.
	
	Updated: 1/9/2012
	Experts: Ben Myers @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.SmartList",
	[dijit._Widget, dijit._Templated],
	{
	
	templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets", "templates/SmartList.html"),
	constructor: function() {
		//items added to the SmartList are indexed using currIndex
		this.currIndex = -1; 
		//items can be assigned parameters that are exposed during events
		this.itemParams = [];
		//event connectors are added to an array so they can be disconnected
		this.smartListEventConnectors = [];
		//number of items in the list - used by the numItems function
		this.itemCount = 0;
		
		//timer variables - see startRefreshTimer and stopRefreshTimer
		this.timer = null;
		this.refreshRate = 0; //in SECONDS
		
		//whether a message should be displayed when the list is empty
		this.showEmptyMessage = true;
	},
	
	postCreate: function() {
		//localization
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.smartListEventConnectors = [];
		this.itemParams = [];
		this.itemCount = 0;
		
		//refresh the list to start
		this.refresh();
		
		//start refreshing the list (if refreshRate was passed to the widget)
		if(this.refreshRate!=null && this.refreshRate>0) {
			this.startRefreshTimer(this.refreshRate);
		}
		
		//hide the empty message if showEmptyMessage is false (even if empty)
		if(!this.showEmptyMessage) {
			dojo.style(this.emptyMessageParagraph, "display", "none");
		}
	},
	
	/* addItem: Adds an item to the SmartList. When the item is clicked any 
		params it was created with will be exposed, so the event can be better 
		handled. createParams will be provided to the _createItem method so 
		the actual HTML can be properly created. Essentially, if you have data 
		about the item, add the data to the params unless it is only used to 
		set up the item's HTML - in that case add it to createParams. */
	addItem: function(params, createParams, addToTop) {
		if(typeof(params)=='undefined') {
			params = null;
		}
		if(typeof(createParams)=='undefined') {
			createParams = null;
		}
		if(typeof(addToTop)=='undefined' || addToTop==null) {
			addToTop = false;
		}
		
		//since we're adding an item, hide the empty list message
		if(this.itemCount <= 0) {
			dojo.style(this.emptyMessageParagraph, "display", "none");
			dojo.empty(this.listElement);
		}
		
		//create the HTML element and add it to the DOM
		this.currIndex++;
		var li = dojo.create("li", {'smartListIndex':this.currIndex});
		if(addToTop) {
			dojo.place(li, this.listElement, 'first');
		}
		else {
			dojo.place(li, this.listElement);
		}
		
		//add dojo event connectors
		this.smartListEventConnectors.push(dojo.connect(li, 'onmouseenter', this, "_itemMouseoverEvent"));
		this.smartListEventConnectors.push(dojo.connect(li, 'onmouseleave', this, "_itemMouseoutEvent"));
		this.smartListEventConnectors.push(dojo.connect(li, 'onclick',      this, "_itemClickEvent"));
		
		//increment the item count
		this.itemCount++;
		//store the params for retrieval when the user interacts with the item
		this.itemParams[this.currIndex] = params;
		//call the _creatItem method, to be used as a hook for subclasses
		this._createItem(this.currIndex, li, params, createParams);
		//call the _onItemAdded method, to be used as a hook for subclasses
		this._onItemAdded(this.currIndex, li, params);
		//return the index so the user can retrieve the item if necessary
		return this.currIndex;
	},
	
	/* removeItem: Removes an item from the SmartList. Consider using 
		clearList and rebuilding items if the majority of the list needs to be 
		removed. Note the index is not guaranteed to be the visual index of 
		the item (index=0 could be anywhere) since items can be added to the 
		top or the bottom. */
	removeItem: function(index) {
		//get the item and make sure it exists
		var item = this.getItem(index);
		if(item==null) {
			return;
		}
		
		//remove the element from the DOM
		if(typeof(item.li.parentNode)!='undefined' && item.li.parentNode!=null) {
			item.li.parentNode.removeChild(item.li);
		}
		
		//remove the recorded params of the item
		this.itemParams[item.index] = null;
		
		//decrement the item count
		this.itemCount--;
		
		//if the list should display an empty message, now might be a good time
		if(this.itemCount==0) {
			if(this.showEmptyMessage) {
				dojo.style(this.emptyMessageParagraph, "display", "");
			}
			dojo.place(dojo.create("li", {'innerHTML':"&nbsp;", 'style':"display:none"}), this.listElement);
		}
		
		this._onItemRemoved(item.index, item.li, item.params);
	},
	
	/* numItems: Returns the number of items in the list. */
	numItems: function() {
		return this.itemCount;
	},
	
	/* _createItem: Called whenever addItem is successful. Used as a hook for 
		subclasses. This function should be overriden to create the actual 
		HTML elements necessary to properly display the item. */
	_createItem: function(index, li, params, createParams) {}, //OVERRIDE ME!
	
	/* _onItemAdded: Called whenever addItem is successful. Used as a hook for 
		subclasses. This function should be overriden when something needs to 
		happen whenever an item is added (e.g. incrementing a count). Called 
		after _createItem. Note that although this function is similar to 
		_createItem, it should be used for different purposes (_createItem 
		should be used for setting up the HTML and relevant events only). */
	_onItemAdded: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemAdded: Called whenever removeItem is successful. Used as a hook 
		for subclasses. */
	_onItemRemoved: function(index, li, params) {}, //OVERRIDE ME!
	
	/* getItem: Returns an item given an index. Note the SmartList is
		0-indexed and there is no guarantee the visual order of the list is 
		exactly the same as the indexed order. It is guaranteed, however, that 
		each item can be uniquely identified by its index. Returns a JSON 
		object with the item's index, li, and params if the item was found.
		Returns null otherwise. */
	getItem: function(index) {
		if(typeof(index)=='undefined' || index==null) {
			return null;
		}
		
		//search for the HTML element
		var li = dojo.query('li[smartListIndex="'+index+'"]', this.listElement);
		if(typeof(li)=='undefined' || li==null || typeof(li.length)=='undefined' || li.length==null || li.length<=0) {
			return null;
		}
		li = li[0];
		
		//grab the params
		var params = this.itemParams[index];
		if(typeof(params)=='undefined') {
			params = null;
		}
		
		//return the item's index, HTML element, and params
		return {'index':index, 'li':li, 'params':params};
	},
	
	/* getItems: Returns an array of all items as JSON objects, each 
		containing an item's index, parent li element, and parameters that 
		were given when the item was initially created. */
	getItems: function() {
		var itemArr = [];
		dojo.query("li[smartListIndex]", this.listElement).forEach(function(li) {
			var index = parseInt(dojo.attr(li, "smartListIndex"));
			var params = this.itemParams[index];
			if(typeof(params)=='undefined') {
				params = null;
			}
			itemArr.push({'index':index, 'li':li, 'params':params});
		}, this);
		return itemArr;
	},
	
	/* _extractItemInfoFromEvent: A helper method called whenever an event 
		fires for an item. This will extract the pertintent item information 
		before the appropriate handler is called. */
	_extractItemInfoFromEvent: function(evt) {
		if(evt==null) {
			return {'index':null, 'li':null, 'params':null};
		}
		var target = evt.target;
		var targetIndex = null;
		var targetParams = null;
		
		//since a child element within the item may have triggered the event, 
		// it is necessary to search the event's target's parents for the 
		// actual item element
		for(var i=0; i<=10 && target!=null && dojo.attr(target, "smartListIndex")==null; i++) {
			//note any infinite loop issues are avoided by only looping a 
			// maximum of 10 times - no error has ever been encountered but 
			// there's no point in one little widget crashing a whole site
			if(i==10) {
				return null;
			}
			target = target.parentNode;
		}
		if(target!=null) {
			targetIndex = dojo.attr(target, "smartListIndex");
			
			//the params given to the item when it was created
			targetParams = this.itemParams[targetIndex];
			if(typeof(targetParams)=='undefined') {
				targetParams = null;
			}
		}
		return {'index':targetIndex, 'li':target, 'params':targetParams};
	},
	
	/* Below are event handlers that do little other than handle basic 
		interactions with the SmartList. */
	_itemMouseoverEvent: function(evt) {
		//add the "hover" class, since li:hover is technically improper css
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null && item['li']!=null) {
			dojo.addClass(item['li'], "hover");
			this._onItemMouseOver(item.index, item.li, item.params);
		}
	},
	_itemMouseoutEvent: function(evt) {
		//remove the "hover" class, since li:hover is technically improper css
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null && item['li']!=null) {
			dojo.removeClass(item['li'], "hover");
			this._onItemMouseOut(item.index, item.li, item.params);
		}
	},
	_itemClickEvent: function(evt) {
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null) {
			this._onItemClicked(item['index'], item['li'], item['params']);
		}
	},
	
	/* _onItemClicked: Called whenever an item is clicked. The params argument 
		is provided when the item is created and should hold whatever 
		information is required to properly handle the event. */
	_onItemClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemClicked: Called whenever an item is moused over. */
	_onItemMouseOver: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemClicked: Called whenever an item is moused out. */
	_onItemMouseOut: function(index, li, params) {}, //OVERRIDE ME!
	
	/* clearList: Removes all items from the SmartList. This is also where 
		dojo event connectors are cleared. */
	clearList: function() {
		//remove all the HTML elements
		dojo.empty(this.listElement);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.smartListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		//reset variables
		this.smartListEventConnectors = [];
		this.itemParams = [];
		this.currIndex = -1;
		this.itemCount = 0;
		
		//if the list should display an empty message, now is a good time
		if(this.showEmptyMessage) {
			dojo.style(this.emptyMessageParagraph, "display", "");
			dojo.place(dojo.create("li", {'innerHTML':"&nbsp;", 'style':"display:none"}), this.listElement);
		}
	},
	
	/* setEmptyMessage: Sets the message that appears when the list is empty. 
		Note that nothing will be displayed if showEmptyMessage is set to 
		false or if the list contains items. */
	setEmptyMessage: function(msg) {
		if(typeof(msg)=='undefined' || msg==null) {
			dojo.attr(this.emptyMessageParagraph, "innerHTML", "");
		}
		else {
			dojo.attr(this.emptyMessageParagraph, "innerHTML", msg);
		}
	},
	
	/* startRefreshTimer: Starts a timer that fires every so often, depending 
		on the SmartList's refreshRate. Typically used to refresh the data in 
		the SmartList. Timer variables and their lifecycles are handled by the 
		SmartList. Note that refreshRate is in SECONDS. */
	startRefreshTimer: function(refreshRate) {
		if(typeof(refreshRate)!="undefined") {
			this.refreshRate = Math.max(0, Number(refreshRate));
			if(this.refreshRate>=1000) {
				console.log("A SmartList has been assigned a large refresh rate ("+this.refreshRate+" seconds)."
					+" Please ensure all refresh rates are in seconds and not milliseconds.");
			}
		}
		if(this.refreshRate!=null && this.refreshRate>0) {
			if(this.timer) {
				clearTimeout(this.timer);
			}
			this.timer = setTimeout(dojo.hitch(this, "_handleRefresh"), 1000*Number(this.refreshRate));
		}
	},
	
	/* stopRefreshTimer: Prevents the timer created through startRefreshTimer 
		from firing any more events. Returns true if the timer was running and 
		stopped successfully. Returns false otherwise.*/
	stopRefreshTimer: function() {
		if(this.timer) {
			clearTimeout(this.timer);
			return true;
		}
		return false;
	},
	
	/* _handleRefresh: Makes sure the timer continues refreshing the SmartList 
		if it has been started. refresh will be called by this method. */
	_handleRefresh: function() {
		this.refresh();
		
		//if the timer was running when it was stopped...
		if(this.stopRefreshTimer()) {
			//restart the timer with the same refresh rate
			this.startRefreshTimer();
		}
	},
	
	/* refresh: Refreshes the list with new data. */
	refresh: function() {}, //OVERRIDE ME!
	
	destroy: function(finalize) {
		//stop the timer
		this.stopRefreshTimer();
		this.timer = null;
		
		//destroy the item elements
		this.clearList();
		
		this.inherited(arguments);
	}
	
});


