/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2011. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.task.form.Renderer");


/**
 * A Renderer is responsible for rendering a form within the Task Information Widget based on the
 * clientSettings of a task or a task template. The result is displayed in the form tab of the task within the
 * Task Information Widget.<br />
 * The purpose of the form depends on the mode in which the task has been opened in the Task Information
 * Widget: <br />
 * <ul>
 * <li>"init" - the form is used to collect the input message for creating or creating and starting a task.</li>
 * <li>"edit" - the form is used to collect the output message while editing a task or completing a task.</li>
 * <li>"view" - the form is used to show the content of the input and output message.</li>
 * </ul>
 * In order to use a Renderer, the Renderer need to be associated with a clientType or defined as a default
 * Renderer using Extension Service. <br />
 * An instance of this module is instanciated for each task form which this Renderer is configured for. <br />
 * <br />
 * The following methods of the Renderer are called by the Task Information Widget and need to be implemented
 * by any Dojo module derived from Renderer:
 * <ul>
 * <li>load()</li>
 * <li>unload()</li>
 * <li>resize(dim)</li>
 * <li>show()</li>
 * <li>hide()</li>
 * <li>isSaveSupported()</li>
 * <li>save() but only if isSaveSupported() returns true</li>
 * <li>enableActions()</li>
 * <li>disableActions()</li>
 * <li>isValid()</li>
 * </ul>
 * <br />
 * The following methods are used to call back to the Task Information Widget. I.e. the Task Information
 * Widget connects to these methods in order get updates about the form and its state.:
 * <ul>
 * <li>onLoaded(tkiid)</li>
 * <li>onSubmitted(tkiid, nextTkiid?)</li>
 * <li>onSaved(tkiid)</li>
 * <li>onCanceled(tkiid)</li>
 * <li>onErrorOccurred(message, description, details, action)</li>
 * <li>onLoadNewTask(followOnTkiid) but only if follow-on task need to ne opened</li>
 * <li>onDirty()</li>
 * <li>onReset()</li>
 * <li>onValid()</li>
 * <li>onInValid()</li>
 * </ul>
 */
dojo.declare("com.ibm.bpm.social.widgets.task.form.Renderer", null, {

  /**
   * Provides the context for the form rendering. <br />
   * <br />
   * 
   * 
   * The properties of the parameter object: <br />
   * <ul>
   * <li>iContext - The iContext object that allows to interact with the environment. </li>
   * <li>htmServiceUrl - The service url for the HTM Rest API</li>
   * <li>bfmServiceUrl - The service url for the BFM Rest API</li>
   * <li>clientType - The client type the renderer was registered for. Might be null for a default renderer.</li>
   * <li>clientSettings - The client settings for the client type. Might be null for a default renderer.</li>
   * <li>task - The task object if available. </li>
   * <li>taskTemplate - The task template object. In case no task is yet available.</li>
   * <li>mode - The mode of operation. </li>
   * </ul>
   * <br />
   * The clientType and its clientSettings provide information about the configuration of the form. <br />
   * The task is not available if the form has been opened to create or create and start a task for a task
   * template. The task template object is provided in case no task is yet available. <br />
   * <br />
   * Possible values for mode are:
   * <ul>
   * <li>"init" - for creating and starting a task.</li>
   * <li>"edit" - for working on a task.</li>
   * <li>"view" - for readonly rendering of the task form.</li>
   * </ul>
   * <br />
   * <br />
   * 
   * @param {object} parameters The context parameter.
   */
  constructor: function(parameters){},

  // ****************** Lifecycle Methods Begin ******************** 

  /**
   * The Task Information Widget calls load as first method after the form renderer has been instantiated. The
   * form is consider to be clean and valid at the beginning. Subsequently the resize function is called with
   * the dimensions. The dimensions allow to establish a block with a fixed size that can be scrolled. (For
   * example, one might not want that the whole form including its buttons can be scrolled.) <br />
   * <br />
   * 
   * @param {DOMElement} element A reference to the element where the form should appear.
   * @type {void}
   */
  load: function(element /* DOM element */){},

  /**
   * Task Information Widget calls unload if the Renderer needs to be destroyed.
   * 
   * This method should also be used to release any resources.
   * 
   * @type {void}
   */
  unload: function(){},

  /**
   * The Task Information Widget calls resize if the widget need to be resized. For example, this allows to
   * implement a block with an absolute height or width which can be scrolled. Resize will be called at least
   * once after the invocation of the load method.
   * 
   * @param {object} dim The dimensions {width:int, height: int}
   * @type {void}
   */
  resize: function(dim /*{width:1,height:1} */){},

  /**
   * The Task Information Widget calls show if the form should become visible.
   * 
   * @type {void}
   */
  show: function(){},

  /**
   * The Task Information Widget calls hide if the form should become hidden.
   * 
   * @type {void}
   */
  hide: function(){},

  // ****************** Lifecycle Methods End ******************** 

  // ****************** Actions Begin *************************** 

  /**
   * The Task Information Widget calls isSaveSupported in order to decide whether it need to render a Save
   * button.
   * 
   * If save is supported, the Task Information Widget provides a save button which triggers the save method
   * of this Renderer if possible. For example, a user might only be entitled to see a task but not entitled
   * to save any changes in which case the save button of the Task Information Widget would be disabled.
   * 
   * @returns {boolean} true if supported.
   */
  isSaveSupported: function(){
    return true; // Default is true
  },

  /**
   * The Task Information Widget calls save if the form content need to be saved.
   * 
   * The method will only be called if isSaveSupported returned true. In case of success, onSaved should be
   * called otherwise onErrorOccurred.
   * 
   * @type {void}
   */
  save: function(){},

  /**
   * The Task Information Widget calls enableActions if the task can be started in case of "init" mode or if
   * the task can be completed in case of "edit" mode.
   * 
   * <br />
   * Basically this method is called if no reasons for disabling the main action do not exist any more. In
   * additino to that the Renderer need to take into account additional pieces of information before enabling
   * actions. For example, the Renderer should only enable actions related to the form content if the provided
   * input is valid.
   * 
   * 
   * @type {void}
   */
  enableActions: function(){},

  /**
   * The Task Information Widget calls disableActions if the task cannot be started in case of "init" mode or
   * if the task cannot be completed in case of "edit" mode.
   * 
   * This might be the case if the task is suspended or waiting for subtasks. Basically this method is called
   * by the Task Information Widget as callback, if any reasons for disabling the main action exist.
   * 
   * @type {void}
   */
  disableActions: function(){},

  // ****************** Actions End *************************** 

  // ****************** Notification Begin ******************** 


  /**
   * onLoaded has to be called if the form has been completely loaded. Added with version 8.0.
   * 
   * This method allows to asynchronously report that the form has been completely loaded.
   * 
   * @param {string} tkiid of the task.
   * @type {void}
   */
  onLoaded: function(tkiid /* string */){},

  /**
   * onSubmitted has to be called if the main action was successful.
   * 
   * These are the scenarios: <br />
   * <br />
   * <ul>
   * <li>Mode is "init" and create and start task has run successfully. The tkiid of the started task is
   * provided.</li>
   * <li> Mode is "edit" and complete task has run successfully. The tkiid of the completed task is provided.</li>
   * </ul>
   * <br />
   * 
   * If a follow-on task should be opened in the Task Information Widget, the onLoadNewTask(followOnTkiid) can
   * be called. Where followOnTkiid is the id of the next task.
   * 
   * In case the next task is shown within the renderer, the tkiid of that task can be reported using the
   * optional nextTkiid parameter. The tab where the form is shown need not be opened again in contrast to a
   * follow-on task. Instead only the details about the task need to be updated.
   * 
   * @param {string} tkiid of the task.
   * @param {string] nextTkiid of the task.
   * @type {void}
   */
  onSubmitted: function(tkiid /* string */, nextTkiid /* string */){},

  /**
   * onSave has to be called called if the task was either created or saved.
   * 
   * These are the scenarios: <br />
   * <br />
   * <ul>
   * <li>Mode is "init" and create task has run successfully. The tkiid of the created task is provided.</li>
   * <li>Mode is "edit" and save task has run successfully. The tkiid of the task is provided.</li>
   * </ul>
   * 
   * @param {string} tkiid The task instance if of the task.
   * @type {void}
   */
  onSaved: function(tkiid /* object */){},

  /**
   * onCanceled has to be called if the user interaction is canceled.
   * 
   * This is the following scenario: <br />
   * <br />
   * <ul>
   * <li>If user canceled his or her work on the form and tab needs to be closed.</li>
   * </ul>
   * 
   * @param {string} tkiid The task instance if of the task.
   * @type {void}
   */
  onCanceled: function(tkiid /* object */){},

  /**
   * onErrorOccurred has to be called if an error occurred.
   * 
   * The Task Information Widget will shown an error popup. Prior to calling this method any controls should
   * be disabled. The parameter allows to specify the information shown in the error popup.
   * 
   * @param {string} message The message to be shown.
   * @param {string} description The description to be shown.
   * @param {string} details The details to be shown.
   * @param {string} action The actions to be shown.
   * @type {void}
   */
  onErrorOccurred: function(message, description, details, action){},

  // ****************** Notification End ********************

  // ****************** Pageflow Handling Begin ******************** 

  /**
   * onLoadNewTask has to be called if a new task should be opened in the Task Information Widget.
   * 
   * This method is particularly helpful if a follow-on task need to be displayed in the Task Information
   * Widget.
   * 
   * @param {string} followOnTkiid The task instance if of the new task.
   * @type {void}
   */
  onLoadNewTask: function(followOnTkiid /* object */){},

  // ****************** Pageflow Handling End ******************** 


  // ****************** Validation and dirty Handling Begin ******************** 

  /**
   * onDirty has to be called if the form has been modified and unsaved work would be lost if the tab is
   * closed.
   * 
   * For dirty forms, closing the tab triggers a confirmation dialog by the Task Information Widget.
   * 
   * @type {void}
   */
  onDirty: function(){},

  /**
   * onReset has to be called if the form does not contain any unsaved information any more.
   * 
   * Basically the form can now be closed without confirmation dialog.
   * 
   * @type {void}
   */
  onReset: function(){},

  /**
   * onValid has to be called if the form is valid.
   * 
   * The method need to be called at the point in time when the form gets valid.
   * 
   * @type {void}
   */
  onValid: function(){},

  /**
   * onInvalid has to be called if the form is invalid.
   * 
   * The method need to be called at the point in the time when the form gets invalid. <br />
   * The load method makes the assumption that the form is initially valid. Therefore onInvalid also need to
   * be called after load() if the form is initially invalid.
   * 
   * @type {void}
   */
  onInvalid: function(){},

  /**
   * Returns whether the form is valid.
   * 
   * @returns boolean true if valid.
   */
  isValid: function(){
    return true;
  }

// ****************** Validation and dirty Handling End ******************** 


});


/**
 * Constant for the "init" mode of the Task Information Widget. In the "init" mode, only the input message is
 * usually rendered. The input message needs to be entered for creating and starting a task instance.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT = "init";

/**
 * Constant for the "edit" mode of the Task Information Widget. In the "edit" mode, the input and the output
 * message are usually rendered. Only the output message needs to be entered or modified for saving and
 * completing the task instance.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT = "edit";

/**
 * Constant for the "view" mode of the Task Information Widget. In the "view" mode, the input and the output
 * message are usually rendered. Neither message needs to be modified.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_VIEW = "view";
