// TasksListRefreshHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.Log");

/**
 * Handles refresh processing if "TASK_RESOURCE_ASSIGNED" or "TASK_FIELD_CHANGED" task list push events are
 * received.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler", null, {

  _disabled: false,

  _refreshCallback: null,

  _setModelCallback: null,

  _updateModelCallback: null,

  _parentId: null,

  _deltaRefresh: false,

  _subscribedTaskListUpdated: null,

  _refreshing: false,

  _autoRefresh: false,

  _waitingForQueryResult: false,

  _suspendedFull: false,

  _suspendedFullIds: null,

  _queryResultQueued: false,

  _refreshRequestQueued: false,

  _fullRefreshRequestQueued: false,

  _moreRefreshRequestsQueued: false,

  _refreshWaitLevel: 0,

  _refreshTimer: null,

  _delayedTasks: null,

  _delayedTaskIds: null,

  _handledMessageTimes: null, // serialize messages - {tkiid: timeOfOccurrence}

  _fullScreenDialogWithoutRefreshOpened: false,

  _fullRefreshOnResume: false,

  /**
   * @param refreshCallback call back method for executing a full refresh
   * @param setModelCallback call back method to display query result
   * @param updateModelCallback call back method for delta updates of the list
   * @param parentId the id of the caller for logging
   * @param deltaRefresh list supports delta updates
   */
  constructor: function constructor_TasksListRefreshHandler(refreshCallback, setModelCallback, updateModelCallback,
    parentId, deltaRefresh){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListRefreshHandler");

    this._refreshCallback = refreshCallback;
    this._setModelCallback = setModelCallback;
    this._updateModelCallback = updateModelCallback;
    this._parentId = parentId;
    this._deltaRefresh = deltaRefresh;
    this._suspendedFullIds = new Array();
    this._delayedTasks = new Array();
    this._delayedTaskIds = new Array();
    this._handledMessageTimes = {
      latest: null,
      ids: new Array()
    };

    com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListRefreshHandler");
    return;
  },

  /**
   * 
   * @type {void}
   */
  destroy: function destroy_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._unsubscribeTaskListUpdated();
    this._clearRefreshTimer();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Disables the refresh handling while a task list is not visible. This is resumed on the next list refresh.
   */
  disable: function disable_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(2, this._parentId);
    this._disabled = true;
    this._unsubscribeTaskListUpdated();
    this._clearRefreshTimer();
  },

  /**
   * Refresh of the list is started.
   * <p>
   * Clears the timer and resets all refresh properties.
   */
  refreshStarted: function refreshStarted_TasksListRefreshHandler(autoRefresh){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh started, autoRefresh=" + autoRefresh);
    // initialize refresh handling
    this._disabled = false;
    this._refreshing = true;
    this._autoRefresh = autoRefresh;
    if (!this._deltaRefresh) {
      // do not listen any longer
      this._unsubscribeTaskListUpdated();
    }
    // clear running timers and parameters
    this._clearRefreshTimer();
    this._waitingForQueryResult = false;
    this._suspendedFull = false;
    this._suspendedFullIds = new Array();
    this._queryResultQueued = false;
    this._refreshRequestQueued = false;
    this._fullRefreshRequestQueued = false;
    this._moreRefreshRequestsQueued = false;
    this._delayedTasks = new Array();
    this._delayedTaskIds = new Array();
    this._fullScreenDialogWithoutRefreshOpened = false;
    this._fullRefreshOnResume = false;
    this._refreshWaitLevel = 0;
  },

  /**
   * Listen for notifications while waiting for the query result.
   */
  waitingForQueryResult: function waitingForQueryResult_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Query submitted");
    // listen while waiting for query result
    this._subscribeTaskListUpdated();
    this._waitingForQueryResult = true;
  },

  /**
   * The query is finished. Result is not displayed if refresh is suspended.
   * 
   * @param success {Boolean} the query finished successful
   */
  queryFinished: function queryFinished_TasksListRefreshHandler(success){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Query finished");
    if (!this._disabled) {
      this._waitingForQueryResult = false;
      if (this._suspendedFull) {
        // delay display of query result
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Delay display of query result");
        this._queryResultQueued = true;
      }
      else if ((this._refreshRequestQueued || this._fullRefreshRequestQueued) && this._autoRefresh) {
        // another auto refresh request is queued, ignore this query result
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Ignore query result");
        this.refreshFinished(this._autoRefresh);
      }
      else if (success) {
        // no refresh request queued or refresh was not an auto refresh, prepare list from query result (remove "Loading" message)
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Display query result");
        this._setModelCallback(this._autoRefresh);
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Refresh of the list is finished.
   * <p>
   * Starts listening for notifications and sets the timer if a request has been queued while waiting for the
   * query result.
   */
  refreshFinished: function refreshFinished_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh finished");
    if (!this._disabled) {
      this._refreshing = false;
      // make sure we are listening after refresh
      this._subscribeTaskListUpdated();
      // TODO do we really want to refresh one second after last refresh completed???
      if (this._refreshRequestQueued) {
        // start delayed refresh processing
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start delayed refresh processing after refresh");
        this._prepareRefreshAfterNotification();
      }
      if (this._fullRefreshRequestQueued) {
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start full refresh processing after refresh");
        this.doFullRefresh();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Suspends full refresh handling for lists supporting delta refresh.
   * <p>
   * Should be called if the view should not be disturbed by full refresh processing, e.g. if a dialog is
   * opened or tasks are expanded. Refresh is delayed until resumeFullRefresh is called.
   * <p>
   * Suspend requests are stacked and automatic refresh is only resumed if all suspend requests are resumed or
   * if a refresh is triggered.
   * <p>
   * If the dialog is "Process Diagram" or "Audit History" a full refresh will be triggered on resume if any
   * notifications are received in the meantime
   * 
   * @param dialogId a unique id of the dialog which has been opened
   */
  suspendFullRefresh: function suspendFullRefresh_TasksListRefreshHandler(dialogId){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspend full refresh for " + dialogId);
    if (!this._disabled) {
      if (this._isFullScreenDialogWithoutRefresh(dialogId)) {
        // do a full refresh on return from the dialog
        this._fullScreenDialogWithoutRefreshOpened = true;
      }
      this._suspendedFullIds.push(dialogId);
      if (!this._suspendedFull && (!this._refreshing || this._waitingForQueryResult)) {
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspend full refresh");
        this._suspendedFull = true;
        // delay refresh until refresh handling is resumed
        this._clearRefreshTimer();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Resumes full refresh handling, triggers an immediate refresh if a request was queued or shows the result
   * of a refresh which was received in the meantime.
   * 
   * @param dialogId a unique id of the dialog which has been opened
   */
  resumeFullRefresh: function resumeFullRefresh_TasksListRefreshHandler(dialogId){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Resume full refresh for " + dialogId);
    if (!this._disabled) {
      if (this._suspendedFull) {
        var index = dojo.indexOf(this._suspendedFullIds, dialogId);
        if (index >= 0) {
          this._suspendedFullIds.splice(index, 1);
        }
        if (this._suspendedFullIds.length === 0) {
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Resume full refresh");
          this._suspendedFull = false;
          if (this._queryResultQueued &&
            ((!this._refreshRequestQueued && !this._fullRefreshRequestQueued) || !this._autoRefresh)) {
            // no additional request or manual refresh - update list with queued query result
            com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do setModel after suspend");
            this._setModelCallback(this._autoRefresh);
          }
          else if (!this._refreshing) {
            if ((this._refreshRequestQueued && this._refreshTimer === null) || this._fullRefreshRequestQueued ||
              this._fullRefreshOnResume) {
              // do immediate refresh after suspend if request was queued and no refresh is in progress
              com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do refresh after suspend");
              this._updateTaskList();
            }
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspended dialogIds: " + this._suspendedFullIds);
        }
      }
      else {
        com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
      }
    }
  },

  /**
   * Checks if the opened dialog is "Process Diagram" or "Audit History"
   * 
   * @param dialogId
   * @returns {boolean} true if the dialog is a full screen dialog which does not trigger a refresh on return
   *          to the list
   */
  _isFullScreenDialogWithoutRefresh: function __isFullScreenDialogWithoutRefresh_TasksListRefreshHandler(dialogId){
    if (dialogId.indexOf("ProcessDiagramDialog") !== -1 || dialogId.indexOf("AuditHistoryDialog") !== -1) {
      com.ibm.bpm.social.util.common.Log.log(2, "Dialog does not trigger refresh on return: " + dialogId);
      return true;
    }
    return false;
  },

  /**
   * Trigger an immediate full refresh.
   * <p>
   * If refresh or full refresh is suspended delay until refresh is resumed
   * 
   */
  doFullRefresh: function doFullRefresh_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (!this._disabled) {
      this._fullRefreshRequestQueued = true;
      if (!this._refreshing && !this._suspendedFull) {
        // clear timer
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Clear timer");
        this._clearRefreshTimer();
        this._updateTaskList();
      }
      else if (!this._deltaRefresh) {
        // refresh request queued while refreshing or suspended, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Starts listening for task list update events
   */
  _subscribeTaskListUpdated: function _subscribeTaskListUpdated_TasksListRefreshHandler(){
    // task list refresh if task list push event received for task assigned to me or my groups 
    if (this._subscribedTaskListUpdated === null) {
      this._subscribedTaskListUpdated =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED, dojo.hitch(this, this._handleTaskListMessage));
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Subscribed");
    }
  },

  /**
   * Stops listening for task list update events
   */
  _unsubscribeTaskListUpdated: function _unsubscribeTaskListUpdated_TasksListRefreshHandler(){
    if (this._subscribedTaskListUpdated !== null) {
      dojo.unsubscribe(this._subscribedTaskListUpdated);
      this._subscribedTaskListUpdated = null;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Unsubscribed");
    }
  },

  /**
   * Triggers a delayed refresh of the list.<br>
   * The initial wait time is 1 second (wait level 1).<br>
   * If within this second another notification is received the refresh is delayed for another 2 seconds (wait
   * level 2).<br>
   * If within these 2 seconds another notification is received the refresh is delayed for another 4 seconds
   * (wait level 3).
   * <p>
   * The maximum delay is 7 seconds.
   * <p>
   * If the list is actually refreshing, setting the timer is delayed until the actual refresh is finished.
   * 
   * @param data notification data
   * @returns {void} nothing
   */
  _prepareRefreshAfterNotification: function _prepareRefreshAfterNotification_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Prepare, waitLevel=" + this._refreshWaitLevel +
      ", refreshing=" + this._refreshing);
    if (this._refreshTimer === null) {
      this._refreshRequestQueued = true;
      this._refreshWaitLevel = 1;
      if (!this._refreshing) {
        // start timer
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start timer");
        this._setRefreshTimeout();
      }
      else if (!this._deltaRefresh) {
        // refresh request queued while refreshing or suspended, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Additional request");
      this._moreRefreshRequestsQueued = true;
      if (!this._deltaRefresh) {
        // additional refresh request queued, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
  },

  /**
   * Sets the timer for refresh
   */
  _setRefreshTimeout: function _setRefreshTimeout_TasksListRefreshHandler(){
    if (this._refreshTimer === null) {
      // wait level 1: 1 second, 2: 2 seconds, 3: 4 seconds -> max of 7 seconds
      var waitTime = Math.pow(2, (this._refreshWaitLevel - 1)) * 1000;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Prepare " + waitTime);
      this._refreshTimer = setTimeout(dojo.hitch(this, this._refreshAfterNotification), waitTime);
    }
    else {
      // Code logic problem!
      com.ibm.bpm.social.util.common.Log.log(1, this._parentId + " - Refresh timer is already set!!!");
    }
  },

  /**
   * This method is called when the refresh timer has finished. If additional notifications have been received
   * another timer is started for delayed refresh, if the maximum waits are not exceeded.
   */
  _refreshAfterNotification: function _refreshAfterNotification_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._refreshTimer = null;
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh?");
    if (this._moreRefreshRequestsQueued && this._refreshWaitLevel < 3) {
      // extend wait
      this._refreshWaitLevel++;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Extend wait, waitLevel=" + this._refreshWaitLevel);
      this._setRefreshTimeout();
      if (this._refreshWaitLevel < 3) {
        // listen for more requests
        this._moreRefreshRequestsQueued = false;
        this._subscribeTaskListUpdated();
      }
    }
    else {
      // no other requests or wait limit is reached
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do refresh");
      this._updateTaskList();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Handles the task list update push messages
   * 
   * @param message the push message for task list updates
   * @returns {_handleTaskListMessage_TasksListRefreshHandler} {void} nothing
   */
  _handleTaskListMessage: function _handleTaskListMessage_TasksListRefreshHandler(message){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message: channel=" + message.channel);
    var data = message.data;
    var handleTask = false;
    var newTaskAssignment = false;
    if (data && data.task) {
      com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message data: kind=" + data.kind +
        ", timeOfOccurrence=" + data.timeOfOccurrence);
      com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message task data: tkiid=" + data.task.tkiid +
        ", assignedTo=" + data.task.assignedTo + ", state=" + data.task.state + ", priority=" + data.task.priority +
        ", dueTime=" + data.task.dueTime + ", atRiskTime=" + data.task.atRiskTime + ", completionTime=" +
        data.task.completionTime);
      switch (data.kind) {
        case "TASK_RESOURCE_ASSIGNED" :
          var groupMessage = message.channel && message.channel.indexOf("/bpm/process_portal/task_list/group/") === 0;
          // check for delayed processing
          if (data.task.assignedToType) {
            if (data.task.assignedToType === "group") {
              if (groupMessage) {
                // ignore additional user message
                newTaskAssignment = true;
                handleTask = true;
              }
              else {
                com.ibm.bpm.social.util.common.Log.log(3, "Additional user message is ignored for task " + data.task.tkiid);
              }

            }
            else if (data.task.assignedToType === "user") {
              var userData = com.ibm.bpm.social.util.common.Util.getCurrentUser();
              var userId = userData.userName;
              if (data.task.assignedTo === userId) {
                if (!groupMessage) {
                  // ignore additional group message
                  newTaskAssignment = true;
                  handleTask = true;
                }
                else {
                  com.ibm.bpm.social.util.common.Log.log(3, "Additional group message is ignored for task " +
                    data.task.tkiid);
                }
              }
              else {
                // assigned to another user, needs to be removed from list
                handleTask = true;
              }
            }
          }
          break;
        case "TASK_FIELD_CHANGED" :
          handleTask = true;
          break;
        default:
          com.ibm.bpm.social.util.common.Log.log(3, "Invalid message kind: " * data.kind);
          break;
      }
      if (this._fullScreenDialogWithoutRefreshOpened) {
        // do not handle update, trigger a full refresh on dialog close
        com.ibm.bpm.social.util.common.Log.log(2, "Full refresh will be triggered after resume!");
        this._fullRefreshOnResume = true;
      }
      else if (this._deltaRefresh) {
        if (handleTask) {
          if (this._serializeMessage(data)) {
            if (newTaskAssignment || dojo.indexOf(this._delayedTaskIds, data.task.tkiid) !== -1 || this._refreshing) {
              // delayed delta refresh for 
              // - tasks which are assigned and may need to be added to the list 
              // - tasks which are already part of delayed delta refresh
              //    ensures that state change for inline completion is not handled before task assignment (which adds task to list)
              // - all tasks while the task list is refreshing
              com.ibm.bpm.social.util.common.Log.log(2, "Delayed delta refresh for task " + data.task.tkiid);
              this._addDelayedTask(data.task);
              this._prepareRefreshAfterNotification();
            }
            else {
              // immediate delta refresh
              com.ibm.bpm.social.util.common.Log.log(2, "Immediate delta refresh for task " + data.task.tkiid);
              this._updateModelCallback([data.task], true);
            }
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(2, "Redundant message is ignored for task " + data.task.tkiid);
        }
      }
      else {
        // delayed full refresh
        com.ibm.bpm.social.util.common.Log.log(2, "Delayed full refresh");
        this._prepareRefreshAfterNotification();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "No task data!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Messages need to be serialized because they may not come in the order they have been submitted. Older
   * messages are ignored.
   * 
   * @param data message data
   * @returns {Boolean} true if the message needs to be handled, false if the message must be ignored
   */
  _serializeMessage: function _serializeMessage_TasksListRefreshHandler(data){
    var handle = true;
    var timeOfOccurrence = this._handledMessageTimes[data.task.tkiid];
    var handledTime = dojo.date.stamp.fromISOString(timeOfOccurrence);
    var newTime = dojo.date.stamp.fromISOString(data.timeOfOccurrence);
    com.ibm.bpm.social.util.common.Log.log(2, data.task.tkiid + " - timeOfOccurrence " + data.timeOfOccurrence +
      ", already handled " + timeOfOccurrence);
    if (timeOfOccurrence !== undefined) {
      handle = dojo.date.compare(newTime, handledTime) >= 0;
    }
    else {
      this._handledMessageTimes.ids.push(data.task.tkiid);
    }
    if (handle) {
      // save/replace time for handled message
      this._handledMessageTimes[data.task.tkiid] = data.timeOfOccurrence;
      if (this._handledMessageTimes.latest !== null) {
        var lastHandledTime = dojo.date.stamp.fromISOString(this._handledMessageTimes.latest);
        if (dojo.date.compare(newTime, lastHandledTime) > 0) {
          this._handledMessageTimes.latest = data.timeOfOccurrence;
        }
      }
      else {
        this._handledMessageTimes.latest = data.timeOfOccurrence;
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Newer message already handled for task " + data.task.tkiid);
    }
    return handle;
  },

  /**
   * Add or replace the task data for a task message which is handled delayed
   * 
   * @param task
   * @returns {void} nothing
   */
  _addDelayedTask: function _addDelayedTask_TasksListRefreshHandler(task){
    if (this._delayedTasks === null) {
      this._delayedTasks = new Array();
      this._delayedTaskIds = new Array();
    }
    var index = dojo.indexOf(this._delayedTaskIds, task.tkiid);
    if (index !== -1) {
      // replace task with latest version
      this._delayedTasks[index] = task;
      com.ibm.bpm.social.util.common.Log.log(2, "Replaced delayed delta refresh data for task " + task.tkiid);
    }
    else {
      // add task
      this._delayedTasks.push(task);
      this._delayedTaskIds.push(task.tkiid);
      com.ibm.bpm.social.util.common.Log.log(2, "Added delayed delta refresh data for task " + task.tkiid);
    }
  },

  /**
   * Update the task list with full refresh or delta refresh with task messages which have been delayed.
   * 
   * @returns {void} nothing
   */
  _updateTaskList: function _updateTaskList_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (!this._disabled) {
      if (!this._deltaRefresh || this._fullRefreshRequestQueued || this._fullRefreshOnResume) {
        if (this._suspendedFull) {
          // delay full refresh until it is resumed
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Full refresh is suspended by " +
            this._suspendedFullIds);
        }
        else {
          // do full refresh
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do full refresh");
          this._refreshCallback(true);
        }
      }
      else if (this._deltaRefresh && this._delayedTasks.length > 0) {
        // delayed adding of new tasks

        // TODO if search string specified delayed query to check if tasks need to be added!

        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Update model for task(s): " + this._delayedTaskIds);
        this._updateModelCallback(this._delayedTasks);
        this._delayedTasks = new Array();
        this._delayedTaskIds = new Array();
        this._cleanupHandledMessageTimes();
        this._refreshRequestQueued = false;
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Remove all remembered message times which are older than 1 minute
   */
  _cleanupHandledMessageTimes: function _cleanupHandledMessageTimes_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "Number of last handled message times: " +
      this._handledMessageTimes.ids.length);
    var latest = dojo.date.stamp.fromISOString(this._handledMessageTimes.latest);
    var toOld = dojo.date.add(latest, "minute", -1);
    com.ibm.bpm.social.util.common.Log.log(3, "Removing messages older than " + toOld);
    for ( var i = 0; i < this._handledMessageTimes.ids.length; i++) {
      var tkiid = this._handledMessageTimes.ids[i];
      var lastOccurence = dojo.date.stamp.fromISOString(this._handledMessageTimes[tkiid]);
      if (dojo.date.compare(toOld, lastOccurence) >= 0) {
        this._handledMessageTimes[tkiid] = undefined;
        this._handledMessageTimes.ids.splice(i, 1);
        com.ibm.bpm.social.util.common.Log.log(3, "Removed last handled message time for " + tkiid);
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "Number of last handled message times: " +
      this._handledMessageTimes.ids.length);
  },

  /**
   * Clears the timer if it is running
   */
  _clearRefreshTimer: function _clearRefreshTimer_TasksListRefreshHandler(){
    if (this._refreshTimer !== null) {
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Clear timer");
      clearTimeout(this._refreshTimer);
      this._refreshTimer = null;
    }
  }

});
