// ErrorHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT


// **************************************
//
// Version: 1.34 11/10/27 02:28:39
//
// **************************************
dojo.provide("com.ibm.bpm.social.widgets.task.util.ErrorHandler");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.requireLocalization("com.ibm.bpm.social.widgets.task.util", "ErrorMessage");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.ErrorHandler",
    null,
    {

      /*
       * Error Handling

      For errors recognized during processing of a REST request, an appropriate HTTP status code is returned to the calling client (see the individual operations for details). The following HTTP status codes are returned by IBM Business Process Manager REST methods:

        * Successful completion of the request:
              o "200 OK" - successful completion
              o "201 Created" - successful completion, new resource created
              o "204 No Content" - successful completion, no content available
        * Expected error situations - additional error information is provided depending on the error type:
              o "400 Bad Request" - parameters are not valid or they are missing
              o "401 Unauthorized" - caller is not authorized for this request
              o "403 Forbidden" - caller is not allowed to complete this request
              o "404 Not Found" - resource does not exist
              o "406 Not Acceptable" - unsupported content type or content encoding requested
              o "409 Conflict" - conflict exists with the current state of the resource
              o "415 Unsupported Media Type" - unsupported content type or content encoding of the request
        * Unexpected error:
              o "500 Internal Server Error" - severe problem - programmer's details provided
              o "501 Not Implemented" - used for certain federated requests
              o "503 Service Unavailable" - federated requests could not be delivered to individual federation targets
              o "504 Gateway Timeout" - federated response has partial content due to missing individual responses
       */
      _genericStatusCodeMap: {
        "400": {
          key: "KEY_REST_BAD_REQUEST"
        },
        "401": {
          key: "KEY_REST_UNAUTHORIZED"
        },
        "403": {
          key: "KEY_REST_FORBIDDEN"
        },
        "404": {
          key: "KEY_REST_NOT_FOUND"
        },
        "406": {
          key: "KEY_REST_NOT_ACCEPTABLE"
        },
        "409": {
          key: "KEY_REST_CONFLICT"
        },
        "415": {
          key: "KEY_REST_UNSUPPORTED_MEDIA_TYPE"
        },
        "500": {
          key: "KEY_REST_INTERNAL_SERVER_ERROR"
        },
        "501": {
          key: "KEY_REST_NOT_IMPLEMENTED"
        },
        "503": {
          key: "KEY_REST_SERVICE_UNAVAILABLE"
        },
        "504": {
          key: "KEY_REST_GATEWAY_TIMEOUT"
        }
      },

      _errorCodeMapBC: {
        "CWTKB0001E": {
          key: "KEY_BC_NOT_FOUND"
        },
        "CWTKB0002E": {
          key: "KEY_BC_AUTHORIZATION_READ"
        },
        "CWTKB0003E": {
          key: "KEY_BC_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_BC_PARAMETER_TO_LONG_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKB0005E": {
          key: "KEY_BC_DELETE_WITH_TASKS"
        },
        "CWTKA0013E": {
          key: "KEY_BC_PARAMETER_NOT_SET_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKA0151E": {
          key: "KEY_BC_NOT_ENABLED"
        }
      },

      _errorCodeMapWB: {
        "CWTKW0001E": {
          key: "KEY_WB_NOT_FOUND"
        },
        "CWTKW0002E": {
          key: "KEY_WB_AUTHORIZATION_READ"
        },
        "CWTKW0003E": {
          key: "KEY_WB_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_WB_PARAMETER_TO_LONG_WORKBASKET"
        },
        "CWTKW0006E": {
          key: "KEY_DELETE_WORKBASKET_CONTAINS_TASKS"
        },
        "CWTKW0007E": {
          key: "KEY_DELETE_WORKBASKET_HAS_DISTRIBUTION_SOURCES"
        },
        "CWTKA0013E": {
          key: "KEY_WB_PARAMETER_NOT_SET_WORKBASKET"
        },
        // "CWTKA0149E": {    // error handled in WBInfoWidget
        "CWTKA0152E": {
          key: "KEY_WB_NOT_ENABLED"
        }
      },

      constructor: function constructor_ErrorHandler(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social.widgets.task.util", "ErrorMessage");
      },

      /**
       * Displays a generic error message with details.
       * 
       * @see #_genericStatusCodeMap for handled status codes
       * 
       * @param data
       * @param ioArgs {void} nothing
       * @param {function} callback to connect to close on error dialog
       */
      showGenericError: function showGenericError_ErrorHandler(data, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var clientMessage = this._nlsResources["KEY_InternalError"];
        if (ioArgs && ioArgs.xhr) {
          var statusCode = ioArgs.xhr.status;
          var mapEntry = this._genericStatusCodeMap[statusCode];
          if (mapEntry && mapEntry !== null) {
            var key = mapEntry.key;
            if (key !== null) {
              clientMessage = this._nlsResources[key];
            }
          }
        }
        this._displayError(clientMessage, this._getDetailsMessage(data, ioArgs), null, ioArgs, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message mapped using the errorCodeMap and the statusCodeMap.
       * 
       * The errorCodeMap has precedence over the statusCodeMap. If no parameters are passed with the error
       * code the parameters are taken out of the rest call (errorMessage.errorMessageParameters) and used for
       * the specified message in business space.
       * 
       * Otherwise a generic message will be shown.
       * 
       * @param data
       * @param ioArgs
       * @param statusCodeMap e.g. {404: {key: "KEY_MissingForm", parameters: {1: htmlURL}, hideDetails:
       *        true}}
       * @param errorCodeMap e.g. {CWTKB0001E: {key: "KEY_Task_API_BusinessCategoryDoesNotExist", parameters:
       *        {1: name}, hideDetails: true}}, <br>
       *        in addition to support older code simple mapping like {CWTKB0001E:
       *        "KEY_Task_API_BusinessCategoryDoesNotExist"} is also supported
       * 
       * @return {void} nothing
       * 
       */
      showError: function showError_ErrorHandler(data, ioArgs, statusCodeMap, errorCodeMap, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if ((ioArgs && ioArgs.xhr) || errorCodeMap) {
          var statusCode = ioArgs.xhr.status;
          var mappedMessageProperties =
            this._getMappedMessageProperties(statusCode, statusCodeMap, data.responseText, errorCodeMap);
          this.showErrorMessage(data, ioArgs, mappedMessageProperties.key, mappedMessageProperties.parameters,
            mappedMessageProperties.hideDetails, null, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message with the error details.
       * 
       * @param data
       * @param ioArgs
       * @param key the message key
       * @param parameters parameters for the message, if no parameters specified, the parameters are
       *        retrieved from the original message
       * @param hideDetails don't show details
       * @param message default message if no message is found for key or no details
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showErrorMessage: function showErrorMessage_ErrorHandler(data, ioArgs, key, parameters, hideDetails, message, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (key && key !== null) {
          var mappedMessage = this._getMappedMessage(key, parameters);
          var detailsMessage = this._getDetailsMessage(data, ioArgs, hideDetails);

          if (mappedMessage === null) {
            if (message) {
              mappedMessage = message;
            }
            else {
              // no message at all: fall back to HTTP error message
              if (ioArgs && ioArgs.xhr && ioArgs.xhr.status) {
                var mapEntry = this._genericStatusCodeMap[ioArgs.xhr.status];
                if (mapEntry && mapEntry !== null) {
                  var newKey = mapEntry.key;
                  if (newKey) {
                    mappedMessage = this._getMappedMessage(newKey, null);
                  }
                }
              }
            }
          }
          else if (detailsMessage === null) {
            detailsMessage = message;
          }

          this._displayError(mappedMessage, detailsMessage, null, ioArgs, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * @param statusCode
       * @param statusCodeMap
       * @param response data.responseText or responseObject.failedOperations[i] for Bulk APIs
       * @param errorCodeMap
       * @return {key: "key", parameters: {parameters...}, hideDetails: true|false, fromErrorCode: true|false}
       */
      _getMappedMessageProperties: function _getMappedMessageProperties_ErrorHandler(statusCode, statusCodeMap, response,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var key = null;
        var parameters = null;
        var hideDetails = false;
        var fromErrorCode = false;
        if (errorCodeMap && response && response !== null) {
          try {
            var responseObject = this.getResponseObject(response);
            if (responseObject.errorNumber && typeof responseObject.errorNumber == "string" &&
              errorCodeMap[responseObject.errorNumber] !== null) {
              var mappedEntry = errorCodeMap[responseObject.errorNumber];
              if (mappedEntry) {
                if (typeof mappedEntry == "string") {
                  // simple error code map
                  key = mappedEntry;
                }
                else {
                  key = mappedEntry.key;
                  hideDetails = mappedEntry.hideDetails;
                  parameters = mappedEntry.parameters;
                }
                if (!parameters || parameters === null) {
                  if (responseObject.errorMessageParameters) {
                    parameters = {};
                    for ( var i = 0; i < responseObject.errorMessageParameters.length; i++) {
                      parameters[(i + 1).toString()] = responseObject.errorMessageParameters[i];
                    }
                  }
                }
                fromErrorCode = true;
              }
            }
          }
          catch (err) {
            // continue with the state map
          }
        }
        if (!fromErrorCode && statusCode !== undefined && statusCode !== null) {
          var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
          if (mapEntry === null) {
            mapEntry = this._mapStatusCode(statusCode, this._genericStatusCodeMap);
          }
          if (mapEntry !== null) {
            key = mapEntry.key;
            parameters = mapEntry.parameters;
            if (key) {
              hideDetails = mapEntry.hideDetails;
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return {
          key: key,
          parameters: parameters,
          hideDetails: hideDetails,
          fromErrorCode: fromErrorCode
        };
      },

      _getMessageForStatusCode: function _getMessageForStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
        if (mapEntry && mapEntry !== null) {
          var key = mapEntry.key;
          if (key !== null) {
            result = this._nlsResources[key];
          }
        }
        return result;
      },

      _mapStatusCode: function _mapStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        if (statusCode !== undefined && statusCode !== null) {
          if (statusCodeMap && statusCodeMap !== null && statusCodeMap[statusCode]) {
            result = statusCodeMap[statusCode];
          }
        }
        return result;
      },

      _getMappedMessage: function _getMappedMessage_ErrorHandler(key, parameters){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "key=" + key);
        var mappedMessage = null;

        var i;
        if (key !== null && this._nlsResources[key]) {
          mappedMessage = this._nlsResources[key];
          if (mappedMessage !== null && parameters !== null && parameters !== undefined) {
            // if the parameter is an empty object, the parameters are false
            var parameterOk = false;
            // check if the parameters include valid values
            for (i in parameters) {
              var value = parameters[i];
              if (value === null) {
                parameterOk = false;
                break;
              } else {
                parameterOk = true;
              }
            }
            if (parameterOk) {
              com.ibm.bpm.social.util.common.Log.logEntry(2," parameters=" + parameters);
              mappedMessage = dojo.string.substitute(mappedMessage, parameters);
            }
          }
        }
        // In case parameter where unknown
        if (/UNKOWN/.test(mappedMessage)) {
          mappedMessage = mappedMessage.replace(/'UNKOWN'/g, "");
          mappedMessage = mappedMessage.replace(/UNKOWN/g, "");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2, mappedMessage);
        return mappedMessage;
      },

      _getDetailsMessage: function _getDetailsMessage_ErrorHandler(data, ioArgs, hideDetails){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var detailsMessage = null;
        if (data && !data.responseText && data.message !== undefined && data.name !== undefined) {
          // this is a JavaScriptError instead of a back-end error
          detailsMessage = data.name + ":" + data.message + " (JavaScript Error)<br>" + "at " + data.stack;
        }
        else if (ioArgs && ioArgs.xhr) {
          if (ioArgs && !hideDetails) {
            var details = this.getResponseObject(ioArgs.xhr.responseText);
            if (details) {
              if (details.errorMessage) {
                detailsMessage = details.errorMessage;
              }
              else if (dojo.isString(details)) {
                detailsMessage = details;
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return detailsMessage;
      },

      _displayError: function _displayError_ErrorHandler(message, details, messageDescription, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // add federation result errors
        if (ioArgs && ioArgs.xhr && ioArgs.xhr.responseText) {
          var responseObject = this.getResponseObject(ioArgs.xhr.responseText);
          if (responseObject.federationResult) {
            var errors = this._getFederationResultErrors(responseObject.federationResult);
            if (errors !== null) {
              // TODO SA: should we show them using BspaceGeneralHelper.handleException?
              // TODO SA: add ioArgs.url?
              messageDescription = this._getMessageDetailsList(errors);
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.log(2, "Message: " + message);
        com.ibm.bpm.social.util.common.Log.log(2, "Details: " + details);

        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, callback);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Shows the bulk API failure in an error dialog if only one item has been selected for bulk processing.
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperation the response for the failing operation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Array} federationResults optional, the complete federation result
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIError: function showBulkAPIError_ErrorHandler(failedOperation, statusCodeMap, errorCodeMap,
        federationResults, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var response = null;
        var statusCode = null;
        var statusCodes = statusCodeMap;
        if (failedOperation) {
          response = failedOperation;
          statusCode = failedOperation.derivedHttpStatusCode;
        }
        else if (federationResults & federationResults.length > 0) {
          // in federation the error may be returned as federation result, if the backend has a problem
          response = federationResults[0];
          statusCode = response.statusCode;
          // use generic mapping for status codes because federation result is related to errors with backend systems
          statusCodes = this._genericStatusCodeMap;
        }
        var mappedMessageProperties = this._getMappedMessageProperties(statusCode, statusCodes, response, errorCodeMap);
        this.showErrorMessage(null, null, mappedMessageProperties.key, mappedMessageProperties.parameters,
          mappedMessageProperties.hideDetails, response.errorMessage, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the bulk API failures in an error dialog for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperations the failedOperations as returned in the responseObject of the bulk
       *        API
       * @param {String} messageKey the general message key for the error
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Object} idToNameMap maps ids to names for replacement of ids in error messages
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {String} nameKey the message key for the entity name
       * @param {Array} federationResults optional, the complete federation result
       * @param {Object} failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIErrors: function showBulkAPIErrors_ErrorHandler(failedOperations, messageKey, statusCodeMap, errorCodeMap,
        idToNameMap, idKey, nameKey, federationResults, failedNamesForSystemIDs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        var errors = new Array();

        // get federation errors
        if (failedNamesForSystemIDs && federationResults && federationResults.length > 0) {
          errors = errors.concat(this._getFederationResultErrors(federationResults, failedNamesForSystemIDs));
        }

        // get errors for failed operations
        errors = errors.concat(this._getBulkAPIErrors(failedOperations, idToNameMap, idKey, statusCodeMap, errorCodeMap));

        if (errors !== null && errors.length > 0) {
          // show list of errors
          this._displayError(this._nlsResources[messageKey], null, null, this._getMessageDetailsList(errors, nameKey,
            failedNamesForSystemIDs), null, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the errors in the federation result as a popup
       * 
       * @param {Array} federationResults the complete federation result
       * @param {String} messageKey the message key for the error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @param {function} callback to connect to close on error dialog
       * @returns {boolean} whether there were errors shown
       */
      showFederationResultErrors: function showFederationResultErrors_ErrorHandler(federationResults, messageKey, ignore401,
        callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1, "messageKey=" + messageKey);
        var errorsShown = false;
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults, null, ignore401);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            var message = this._nlsResources[messageKey];
            var messageDescription = this._getMessageDetailsList(errors);
            this._displayError(message, null, messageDescription, null, callback);
            errorsShown = true;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
        return errorsShown;
      },

      /**
       * Shows the errors in the federation result as global exceptions in BSpace
       * 
       * uses BSpace exception handler to show error on top of space
       * 
       * @param federationResults the complete federation result
       * @param iWidget
       * @param method
       * @returns {void} nothing
       */
      handleFederationResultErrors: function handleFederationResultErrors_ErrorHandler(federationResults, iWidget, method,
        systemsSending401Error){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            for ( var i = 0; i < errors.length; i++) {
              var error = errors[i];
              var message = "";

              // message for status code
              if (error.statusCode !== undefined) {
                var statusMessage = this._getMessageForStatusCode(error.statusCode, this._genericStatusCodeMap);
                if (statusMessage) {
                  message += statusMessage;
                }
                else {
                  message += dojo.string.substitute(this._nlsResources.Key_HTTPStatusCode, [error.statusCode]);
                }
              }

              // system ID
              if (error.systemID) {
                message += " " + dojo.string.substitute(this._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
              }

              // error number, and not part of error message
              if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
                // WPS error message contains error number, WLE message not
                message += " " + error.number + ":";
              }

              // error message
              if (error.message) {
                message += " " + error.message;
              }

              com.ibm.bpm.social.util.common.Log.logError(1, "com.ibm.bpm.social.widgets.task.util.ErrorHandler" + "." +
                method, new Error(message));

              // Do not show "CWTBF0005E: The 'queryFilter' parameter is invalid.",
              // issued if a filter attribute is not supported by one of the backends

              // Do not show "CWTBF0020E: Request not propagated by the federation layer because a previous systems metadata request indicated that the system does not support '...' resources.
              // issued if a target system does not support that REST resource (e. g. WLE does not support escalations)

              // Do not show "CWTBF0021E: Request not propagated by the federation layer because a previous query metadata request indicated that the system does not support this resource.
              // issued if a target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0002E: The following query name is unknown: '...'.
              // issued if a WLE target system does not know a query, can happen because we use all=true

              // Do not show "CWWBQ0001E: The following query table name is unknown: '...'.
              // issued if a WPS target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0530E": The following process application name is unknown: '...'
              // issued if a WLE target system does not know a process application, can happen because we use all=true

              // Do not show "CWTKA0162E": The following process application name is unknown: '...'
              // issued if a WPS target system does not know a process application, can happen because we use all=true

              var handleException =
                !error.number ||
                  (error.number != "CWTBF0005E" && error.number != "CWTBF0020E" && error.number != "CWTBF0021E" &&
                    error.number != "CWTBG0002E" && error.number != "CWTBG0530E" && error.number != "CWWBQ0001E" && error.number != "CWTKA0162E");

              // for the systems that had returned a 401 in the initial systems call, ignore 401 errors now because that was already covered
              // But do not generally exclude all 401 errors - could be a different reason for those systems that had not returned a 401 ealier
              if (handleException && systemsSending401Error && error.statusCode == 401 &&
                (dojo.indexOf(systemsSending401Error, error.systemID) != -1)) {
                // overwrite the handleException flag
                handleException = false;
              }

              // Special handling for ICM 404 errors:
              // For version 7.5.1.0, the ICM rest api could not provide details for the 404 error cases.
              // However, the only relevant case that produce 404 errors from ICM systems in federated mode is
              // that a processAppName was specified that does not exist on the ICM system.
              // We must gracefully handle that case and ignore those errors, botp 719802
              // Currently, it is ok to ignore ALL 404 errors from ICM that enter this method
              // (only entered in federation mode); 404 in the /systems call are handles separately.  
              // NEEDSWORK: As soon as the ICM rest api provides details, then rather do a positive list
              // of error ICM details that should be ignored. (Not NLS specific, please).
              if (handleException && error.statusCode == 404 && error.systemID) {
                if (iWidget && iWidget._systems && iWidget._systems.getSystemType) {
                  if (iWidget._systems.getSystemType(error.systemID) === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
                    // overwrite the handleException flag
                    handleException = false;
                  }
                }
              }

              if (handleException && iWidget) {
                iWidget.handleException("com.ibm.bpm.social.widgets.task.util.ErrorHandler", method, message, null);
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * 
       * @param errors [{name: "name or array of names", systemID: "systemID", statusCode: "statusCode",
       *        number: "Number", message: "Message", details: "Details"}]
       * @param nameKey optional, only for bulk API errors, NLS key of text for error.name, e.g. "Task"
       * @returns {Object} a div containing the message details list
       */
      _getMessageDetailsList: function _getMessageDetailsList_ErrorHandler(errors, nameKey){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var messageDetails = null;

        if (errors !== null) {

          //***********************************
          // message details list
          // systemID, name and reason/details for each failing entry
          // TODO add hostName for system???

          var messageDetailsListDiv = document.createElement("div");
          var detailsDiv = document.createElement("div");
          dojo.addClass(detailsDiv, "errorList");
          messageDetailsListDiv.appendChild(detailsDiv);

          var myself = this;
          dojo.forEach(errors, function(error){

            var ulItem = document.createElement("ul");
            detailsDiv.appendChild(ulItem);

            //
            // item name, may constist of one or more names and optional a systemID
            //
            var itemName = "";
            // name
            if (error.name) {
              if (!dojo.isArray(error.name)) {
                itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name]);
              }
              else {
                for ( var i = 0; i < error.name.length; i++) {
                  if (itemName.length > 0) {
                    itemName += "<br/>";
                  }
                  itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name[i]]);
                }
              }
            }

            // system ID
            if (error.systemID) {
              if (itemName.length > 0) {
                itemName += "<br/>";
              }
              itemName += dojo.string.substitute(myself._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
            }

            if (itemName.length > 0) {
              var liItem = document.createElement("li");
              dojo.addClass(liItem, "errorListItem");
              liItem.innerHTML = itemName;
              ulItem.appendChild(liItem);
            }

            //
            // message, may consist of several parts
            //
            var ulMessage = document.createElement("ul");
            dojo.addClass(ulMessage, "errorListMessage");
            ulItem.appendChild(ulMessage);

            // message for status code
            if (error.statusCode !== undefined) {
              var statusMessage = myself._getMessageForStatusCode(error.statusCode, myself._genericStatusCodeMap);
              if (statusMessage && statusMessage !== null) {
                var liStatus = document.createElement("li");
                liStatus.innerHTML = statusMessage;
                ulMessage.appendChild(liStatus);
              }
            }

            // error number, and not part of error message
            if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
              var liNumber = document.createElement("li");
              liNumber.innerHTML = error.number;
              ulMessage.appendChild(liNumber);
            }

            // error message
            if (error.message) {
              var liMessage = document.createElement("li");
              liMessage.innerHTML = error.message;
              ulMessage.appendChild(liMessage);
            }

            //
            // message details
            //
            if (error.details) {
              var ulMessageDetails = document.createElement("ul");
              dojo.addClass(ulMessageDetails, "errorListMessageDetails");
              ulMessage.appendChild(ulMessageDetails);

              var liDetails = document.createElement("li");
              liDetails.innerHTML = error.details;
              ulMessageDetails.appendChild(liDetails);
            }
          });

          messageDetails = messageDetailsListDiv.innerHTML;

        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return messageDetails;
      },

      /**
       * Retrieves the bulk API failures for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param failedOperations the failedOperations as returned in the responseObject of the bulk API
       * @param idToNameMap maps ids to (display) names
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @returns {Array} an array of objects containing error information <br>
       *          [{name: "name", systemID: "systemID", message: "Message", details: "Details"}]
       */
      _getBulkAPIErrors: function _getBulkAPIErrors_ErrorHandler(failedOperations, idToNameMap, idKey, statusCodeMap,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var errors = new Array();

        if (failedOperations && failedOperations.length > 0) {
          var self = this;

          dojo.forEach(failedOperations, function(failedOperation){
            var error = {};

            if (failedOperation.systemID) {
              error.systemID = failedOperation.systemID;
            }
            var name = idToNameMap[failedOperation[idKey]];
            if (name !== null) {
              error.name = name;
            }

            var mappedMessageProperties =
              self._getMappedMessageProperties(failedOperation.derivedHttpStatusCode, statusCodeMap, failedOperation,
                errorCodeMap);
            var mappedMessage = self._getMappedMessage(mappedMessageProperties.key, mappedMessageProperties.parameters);
            var errorMessage = null;
            if (mappedMessage === null) {
              mappedMessage = failedOperation.errorMessage;
            }
            else if (!mappedMessageProperties.fromErrorCode && !mappedMessageProperties.hideDetails) {
              // if not mapped from error code add detailed error message
              errorMessage = failedOperation.errorMessage;
            }
            error.message = mappedMessage;

            if (errorMessage && errorMessage !== null) {
              error.details = errorMessage;
            }
            errors.push(error);
          });

        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      _getFederationResultErrors: function _getFederationResultErrors_ErrorHandler(federationResults,
        failedNamesForSystemIDs, ignore401){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errors = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errors = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (ignore401 && ignore401 === true && federationResult.statusCode === "401") {
                com.ibm.bpm.social.util.common.Log.log(2,
                  "HTTP error 401 skipped, will not be displayed as an error message");
              }
              else {
                // error returned from a backend system
                var error = {};
                error.statusCode = federationResult.statusCode;
                error.systemID = federationResult.systemID;
                error.number = federationResult.errorNumber;
                error.message = federationResult.errorMessage;
                if (!error.message) {
                  // TODO WLE: for 404 we get only statusCode and body - is this a temporary bug?
                  if (federationResult.body) {
                    error.message = federationResult.body;
                  }
                }
                if (failedNamesForSystemIDs && failedNamesForSystemIDs[error.systemID]) {
                  error.name = failedNamesForSystemIDs[error.systemID];
                }
                errors.push(error);
              }
            }
          });
          if (errors.length === 0) {
            // all responses have been ok
            errors = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      getSystemsSending401Error: function getSystemsSending401Error_ErrorHandler(federationResults){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errorSystems = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errorSystems = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (federationResult.statusCode === "401") {
                errorSystems.push(federationResult.systemID);
              }
            }
          });
          if (errorSystems.length === 0) {
            // all responses have been ok or other than 401
            errorSystems = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errorSystems, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errorSystems;
      },

      getResponseObject: function getResponseObject(response){
        var responseObject = response;
        if (dojo.isString(response)) {
          try {
            responseObject = dojo.fromJson(response);
            // There may be an extra layer ".Data"; if this does exist, use it
            // but do it defensive after some extra checks
            if (responseObject.Data && responseObject.Data.errorNumber && !responseObject.errorNumber) {
              // all checks passed, skip the extra layer
              responseObject = responseObject.Data;
            }
          }
          catch (err) {
            // keep it just as string
          }
        }
        return responseObject;
      },

      getErrorCodeMapBC: function getErrorCodeMapBC_ErrorHandler(){
        return this._errorCodeMapBC;
      },

      getErrorCodeMapWB: function getErrorCodeMapWB_ErrorHandler(){
        return this._errorCodeMapWB;
      },

      showErrorDialog: function showErrorDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(mappedMessage, null /* details */, callback);
      },

      showWarningDialog: function showWarningDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(mappedMessage, null /* details */, callback);
      },

      destroy: function(){
        // TODO temp error dialog
        if (this.popup && this.popup.destroy) {
          this.popup.destroy();
          this.popup = null;
        }
        this.inherited(arguments);
      },

      statics: {
        _instance: null,
        _getInstance: function(){
          if (this._instance === null) {
            this._instance = new com.ibm.bpm.social.widgets.task.util.ErrorHandler();
          }
          return this._instance;
        }
      }
    });

com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.util.ErrorHandler.prototype.statics._getInstance();
};
