// Queue.js
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
// END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.Queue");

dojo.require("com.ibm.bpm.social.widgets.task.util.PropertyFormatter");
dojo.require("com.ibm.bpm.social.util.common.Log");


/**
 * Wraps the config object queue as described in the Config document and provides some helper functions
 * 
 * @author ASZ
 * 
 */

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.Queue",
    null,
    {
      /**
       * Identifier of the task queue as provided by the REST API. In domain setups, the name will get
       * suffixed with "-<i>domainName</i>", e.g. IBM.ALLTASKS-defaultDomain
       * 
       * @type {String}
       */
      name: "",

      /**
       * Display name of task queue provided by the REST API.
       * 
       * @type {String}
       */
      displayName: "",

      /**
       * Description as provided by the REST API.
       * 
       * @type {String}
       */
      description: "",

      /**
       * Display name of task queue provided by the REST API deprecated, use displayName.
       * 
       * @type {String}
       */
      title: "",

      /**
       * An Object, if the Queue has an error, Null otherwise
       * 
       * @type {Object}
       */
      exception: null,


      /**
       * JSON object which defines the item representation, e.g. which properties are shown.
       * 
       * <code>
       *   item = [ 
       *     titleKey: "Property key used for title (configurable,persisted)" 
       *     properties: "Array of properties for this query table"
       *   ]
       * </code>
       * 
       * @type {Object}
       */
      item: null,

      /**
       * Maps the different variables in a whereClause to the right values. That can be place holders for
       * query table attribute names or date values like today.
       * 
       * Is an Array of strings, with name-value pair where
       * <li>Name is the replacement variable of the whereClause
       * <li> value is the attribute name in the query table
       * 
       * @type {String[]}
       */
      queuePropertiesMap: null,

      /**
       * ID/name of the domain the query table belongs to, contains the value as retrieved from the REST API.
       * 
       * @type {String}
       */
      domain: null,

      /**
       * Process application name the query table belongs to, contains the value as retrieved from the REST
       * API.
       * 
       * @type {String}
       */
      processAppName: null,

      /**
       * Display name of the domain the query table belongs to.
       * 
       * @type {String}
       */
      domainDisplayName: null,


      /**
       * Type of domain the query table belongs to.
       * 
       * One of
       * <ol>
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED
       * </ol>
       * 
       * 
       * @type {String}
       */
      domainType: null,

      /**
       * deprecated, use <link>domainType</link>
       * 
       * @type {String}
       */
      type: null,


      /**
       * The used navigation type of the list as selected by the user. One of
       * <ul>
       * <li>pagings
       * <li>scrolling
       * </ul>
       * In single system setups, this just contains the type as set for Widget. Depending on the domain
       * setup, if the domain is a multi system domain, this is always set to <i>scrolling</i>.
       * 
       * @type {String}
       */
      navigationType: null,


      /**
       * An Domain object as returned by Systems.getDomain
       * 
       * @type {Object}
       */
      _domainInstance: null,

      /**
       * Flag to remember whether there are multiple domains available in federation
       * 
       * @type {boolean}
       */
      _federationHasMultipleDomains: false,

      /**
       * Flag if Attributes for this queue have been already loaded/set via setAttributes()
       * 
       * @type {boolean}
       */
      attributesLoaded: false,

      /**
       * create a Queue object from scratch or an unnamed object
       * 
       * @constructor
       * @param {Object} aQueue an object to copy/init attributes from
       * @param {Object} systems a reference to a util.Systems instance (optional)
       * @type {void}
       */
      constructor: function constructor_Queue(aQueue, systems){
        com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Queue", "constructor");

        if (aQueue) {
          dojo.safeMixin(this, aQueue);

          //
          // mappings for deprecated attributes:
          //
          if (!aQueue.displayName) {
            this.displayName = aQueue.title;
          }
          else {
            this.title = aQueue.displayName;
          }
          if (!aQueue.domainType) {
            this.domainType = aQueue.type;
          }
          else {
            this.type = aQueue.domainType;
          }

          //
          // ensure unique name:
          // 
          if (aQueue.name) {
            this.setName(aQueue.name);
          }

          if (aQueue.displayName) {
            this.setDisplayName(aQueue.displayName);
          }

          if (aQueue.domainDisplayName) {
            this.setDomainDisplayName(aQueue.domainDisplayName);
          }

          if (aQueue.description) {
            this.setDescription(aQueue.description);
          }

          if (aQueue.processAppName) {
            this.setProcessAppName(aQueue.processAppName);
          }

          if (aQueue.icmSystems) {
            this._icmSystems = aQueue.icmSystems;
          }

          if (systems) {
            this._systems = systems;

            // retrieve the domain object from the Systems class.
            if (aQueue.domain) {
              this.setDomain(this._systems.getDomain(this.domain));
            }

            if (systems.isApiTypeFederated() && systems.getDomainNames().length > 1) {
              this._federationHasMultipleDomains = true;
            }
          }

          com.ibm.bpm.social.util.common.Log.logExit(3, "", "Queue", "constructor");
          return;
        }
      },


      /**
       * Set Description for this queue
       * 
       * @type {void}
       */
      setDescription: function setDescription_Queue(desc){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // need to prevent code injection
        this.description = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(desc);
        com.ibm.bpm.social.util.common.Log.logExit(2, "set to " + this.description);
        return;
      },

      /**
       * Return the description for this queue
       * 
       * @type {String}
       */
      getDescription: function getDescription_Queue(){
        return this.description;
      },

      /**
       * Set ProcessAppName for this queue
       * 
       * @type {void}
       */
      setProcessAppName: function setProcessAppName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.processAppName = name;

        // process app name has changed, we need to update the queue name:
        this.setName(this.name);

        com.ibm.bpm.social.util.common.Log.logExit(2, "set to " + this.processAppName);
        return;
      },

      /**
       * Return the ProcessAppName for this queue
       * 
       * @type {String}
       */
      getProcessAppName: function getProcessAppName_Queue(){
        return this.processAppName;
      },

      /**
       * Return the Reference to the systems object
       * 
       * @type {Object}
       */
      getSystems: function getSystems_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning " + (this._systems ? this._systems : "null"));
        return this._systems;
      },
      /**
       * Set the Reference to the systems object
       * 
       * @type {Object}
       */
      setSystems: function setSystems_Queue(systems){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._systems = systems;
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Set the navigation type this queue shall be used with. Can be one of
       * <ol>
       * <li>com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_PAGING = "paging"
       * <li>com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_SCROLLING = "scrolling"
       * </ol>
       * 
       * @param {String} navType the type
       * @type {void}
       */
      setNavigationType: function setNavigationType_Queue(navType){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!navType ||
          (navType != com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_SCROLLING && navType != com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_PAGING)) {
          var error = new Error("Queue.setNavigationType: got invalid type " + navType);
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }
        this.navigationType = navType;
        com.ibm.bpm.social.util.common.Log.logExit(2, "set nav type to" + navType);
        return;
      },

      /**
       * Set the type of the domain this queue relates to.
       * 
       * @param {String} type
       * @type {void}
       */
      setDomainType: function setDomainType_Queue(type){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.domainType = type;
        this.type = type;
        com.ibm.bpm.social.util.common.Log.logExit(2, "type set to " + type);
        return;
      },

      /**
       * Get the domain type which this queue relates to.
       * 
       * @returns {String} the domain in the requested format String or Object
       */
      getDomainType: function getDomainType_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var result = this.type ? this.type : this.domainType;
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning domain type " + result);
        return result;
      },


      /**
       * Return the domain which this queue relates to.
       * 
       * @param {Boolean} asObject If true, return the domain object as returned by Systems.getDomain. Else
       *        return the domain name.
       * @returns {String} the domain in the requested format String or Object
       */
      getDomain: function getDomain_Queue(asObject){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var result = (asObject ? this._domain : this.domain);
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning domain as " + (typeof result));
        return result;
      },

      /**
       * Get the domain name as string
       * 
       * @type {String}
       */
      getDomainName: function getDomainName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var rc = this.getDomain(false);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return rc;
      },


      /**
       * Set the the related domain of the queue.
       * 
       * @param {Object} domain Either a domain object as returned by Systems.getDomain or the name of the
       *        domain
       * @param {Boolean} noCheck do
       * @type {void}
       */
      setDomain: function setDomain_Queue(domain){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (typeof domain == "string") {
          this.domain = domain;
          this.setDomainDisplayName(domain); // might be overwritten later
          // ensure that we recurse only once
          if (this._systems && !this._recursion) {
            this._recursion = true;
            var domainNames = this._systems.getDomainNames();
            if (dojo.indexOf(domainNames, domain) > -1) {
              this.setDomain(this._systems.getDomain(domain));
            }
          }
          this._recursion = false;
        }
        else {
          this.setDomainDisplayName(domain.displayName);
          this._domain = domain;
          this.domain = domain.name;
          this.setDomainType(this._systems.getDomainType(domain.displayName));
        }

        // domain has changed, we need to update the queue name:
        this.setName(this.name);


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Set the display name for the domain Attention, is owerwritten when using setDomains
       * 
       * @param {String} name name of the queue
       * @type {void}
       */
      setDomainDisplayName: function setDomainDisplayName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!name) {
          var error = new Error("Queue.setDomainDisplayName: got null or empty string as name!");
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }

        this.domainDisplayName = name;

        com.ibm.bpm.social.util.common.Log.logExit(2, "set name to " + this.domainDisplayName);
        return;
      },


      /**
       * Get the display name of the domain.
       * 
       * @return {String} the name
       */
      getDomainDisplayName: function getDomainDisplayName_Queue(){
        return this.domainDisplayName;
      },
      /**
       * Set the display Name of the Queue
       * 
       * @param {String} name name of the queue
       * @type {void}
       */
      setDisplayName: function setDisplayName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        this.displayName = name;

        // deprecated
        this.title = this.displayName;

        com.ibm.bpm.social.util.common.Log.logExit(3, "set name to " + this.displayName);
        return;
      },


      /**
       * Get the display name of the queue.
       * 
       * @return {String} the name
       */
      getDisplayName: function getDisplayName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        var name;
        if (this.displayName) {
          name = this.displayName;
        }
        else if (this.title) {
          name = this.title;
          this.displayName = name;
        }
        else {
          // this.name may contain domain and/or process app name
          name = this.name.split("$#")[0];
          this.displayName = name;
        }

        com.ibm.bpm.social.util.common.Log.logExit(3, "returning: " + name);
        return name;
      },


      /**
       * Set the name of the queue. In a domain setup, the domain is appended automatically to get a unique
       * name.
       * 
       * @param {String} name name of the queue
       * @return {String} the unique name
       */
      setName: function setName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        if (!name) {
          var error = new Error("Queue.setName: got null or empty string as name!");
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }

        this.name = name.split("$#")[0]; // this string is in both, domain and appname separator

        if (this.domain) {
          this.name = this.name + com.ibm.bpm.social.widgets.task.util.Queue.DOMAIN_SEPARATOR + this.domain;
        }

        if (this.processAppName) {
          this.name = this.name + com.ibm.bpm.social.widgets.task.util.Queue.APPNAME_SEPARATOR + this.processAppName;
        }

        com.ibm.bpm.social.util.common.Log.logExit(3, "set name to " + name);
        return this.name;
      },

      /**
       * Get the name(id) of the queue.
       * 
       * @return {String} the name
       */
      getName: function getName_Queue(){
        return this.name;
      },

      /**
       * Get the plain name of the queue.
       * 
       * This is either just the "name" attribute, or in a domain setup, the name of the queue without the
       * domain added.
       * 
       * 
       * @type {void}
       */
      getPlainQueueName: function getPlainQueueName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var pName = this.name.split("$#")[0];
        com.ibm.bpm.social.util.common.Log.logExit(2, "plain name is " + pName);
        return pName;
      },

      /**
       * Get the fully qualified display Name of a queue<br>
       * If domain_prefix is null or there is only one domain used in federation, just return the queue title
       * <br>
       * Returned format: <b>&lt;prefix&gt; - &lt;suffix&gt;</b>
       * 
       * @returns {String} the domain prefixed name, e.g. "Domain X - QT1" or just the Queue displayName
       */

      getQualifiedDisplayName: function getQualifiedDisplayName_Queue(noAppName){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        var domain = this.domainTitle ? this.domainTitle : this.domain;
        var displayName = this.getDisplayName();
        if (domain && this._federationHasMultipleDomains) {
          displayName = domain + " - " + displayName;
        }
        if (this.processAppName && !noAppName) {
          displayName = displayName + " (" + this.processAppName + ")";
        }
        com.ibm.bpm.social.util.common.Log.logExit(3, "returning: " + displayName);
        return displayName;
      },

      /**
       * Create a div with an exclamation mark indication that this queue has an error
       * 
       * @param {String} message the message to be included
       * 
       * @type {String}
       */
      getExceptionHtml: function getExceptionHtml_Queue(message){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var htmlName =
          '\
          <table id="hugo" style="white-space: nowrap;border-spacing: 0;border-collapse: collapsed" title="_title_">\
            <tbody>\
              <tr>\
                <td>\
                  <div class="smallWarning"></div>\
                </td>\
                <td>&nbsp;_queueName_</td>\
              </tr>\
            </tbody>\
          </table>';

        htmlName =
          htmlName.replace("_queueName_", com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this
            .getQualifiedDisplayName()));
        htmlName = htmlName.replace("_title_", message);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return htmlName;
      },


      /**
       * Helper function to determine base on type or content if the given attribute property is associated
       * with a label
       * 
       * @param {Object} property
       * @param {String} type
       * @returns {Boolean} true if also a label exists
       */
      _hasLabelAssociated: function _hasLabelAssociated_Queue(property, type){
        if (property.content == "TASK_DESC.DESCRIPTION" ||
          type == com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG) {
          return false;
        }
        else {
          return true;
        }
      },

      /**
       * Helper function to determine base on type or content if the given attribute is inline
       * 
       * @param {Object} property
       * @param {String} type
       * @returns {Boolean} true if also a label exists
       */
      _isInlineValue: function _isInlineValue_Queue(property, type){
        if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG ||
          property.content == "TASK.IS_WAIT_FOR_SUB_TK" || property.content == "TASK.IS_ESCALATED") {
          return true;
        }
        else {
          return false;
        }
      },

      /**
       * get the attributes for the queue
       * 
       * @returns {object} return the list of attributes
       */
      getAttributes: function getAttributes_Queue(){
        if (this.attributesLoaded !== true) {
          this._setAttributes();
        }
        return this.item;
      },

      /**
       * Set the attributes describing the queue
       * 
       * @param {Object} attributes JSON object as return from REST call /tasks/query/{queryName}/attributes
       * @type {void}
       */
      _setAttributes: function _setAttributes_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var priority = null; // TODO: ASZ@MoveToSocial : was EntityList.config.priorities;
        // var pushFilterTypeMapping = null; // TODO: ASZ@MoveToSocial : was EntityList.config.pushFilterTypeMapping;
        var property;
        var attributes = null;
        if (this.attributesLoaded !== true) {
          if (this.queueHandler) {
            attributes = this.queueHandler.getAttributes(this);
          }
        }
        else {
          return;
        }

        this.setDisplayName(attributes.displayName);

        // Reset the changes from previous runs.
        this.item = {
          properties: [],
          sortProperties: []
        };

        var sortHash = {};

        // store sort order
        if (attributes.sortAttributes) {
          for ( var i = 0; i < attributes.sortAttributes.length; i++) {
            var sort = {
              name: attributes.sortAttributes[i].name,
              sortDir: attributes.sortAttributes[i].sortOrder,
              sourceAttribute: attributes.sortAttributes[i].sourceAttribute,
              isSort: true
            };
            this.item.sortProperties.push(sort);
            // keep track for later mapping 
            sortHash[sort.sourceAttribute] = sort;
          }
        }


        for (i = 0; i < attributes.items.length; i++) {
          var propertyMetaData = attributes.items[i];

          var displayed = true;
          if (propertyMetaData.name == "KIND" || propertyMetaData.name == "STATE" ||
            propertyMetaData.name == "CONTAINMENT_CTX_ID" || propertyMetaData.name == "PI_DISPLAY_NAME" ||
            propertyMetaData.name == "PI_STATE" || propertyMetaData.name == "PT_NAME") {
            displayed = false;
          }

          var type = com.ibm.bpm.social.widgets.task.util.Constants.mapPropertyType(propertyMetaData, priority);
          property = {
            description: propertyMetaData.description,
            property: propertyMetaData.name,
            title: com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(propertyMetaData.displayName),
            type: type,
            labelDisplayed: this._hasLabelAssociated(propertyMetaData, type),
            inline: this._isInlineValue(propertyMetaData, type),
            content: propertyMetaData.content,
            displayed: displayed,
            isSortable: false,
            isFilterable: false,
            sortDir: "ASC",
            sourceAttribute: propertyMetaData.sourceAttribute,
            isSort: false
          };

          this.item.properties.push(property);

          // Set the sortable flag for the current property.
          if (typeof propertyMetaData.isSortable == "boolean" && !propertyMetaData.isSortable) {
            property.isSortable = false;
          }
          else {
            property.isSortable = true;
          }

          // do not map PI_DISPLAY_NAME because PI_NAME has the same sourceAttribute -> displayed has to be true
          if (displayed && sortHash[property.sourceAttribute]) {
            // if in the sortHash, saved search defines sort for this property
            // map sortname to property name  
            sortHash[property.sourceAttribute].property = property.property;
          }

          // Set the filterable flag for the current property. 
          if (typeof propertyMetaData.isFilterable == "boolean" && !propertyMetaData.isFilterable) {
            property.isFilterable = false;
          }
          else {
            property.isFilterable = true;
          }
        }


        // TODO ASZ@Move2Social : Do we need this? 
        // set the mapping of the required source attribute to the respective queue attribute name
        //          this.queuePropertiesMap =
        //            myself._createReqiredPropertiesMap(attributes.items,
        //              myself.config[com.ibm.bpm.social.widgets.task.util.Constants.LIST_PROPERTIES]);
        //          };

        // flag that we have retrieved attributes, may be used to prevent a reload via REST API
        this.attributesLoaded = true;

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      }


    });

/**
 * Character used to separate QT name and domain. ("$#$" in not allowed in QT names and saved searches.)
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Queue.DOMAIN_SEPARATOR = "$#$";
com.ibm.bpm.social.widgets.task.util.Queue.APPNAME_SEPARATOR = "$##$";