// Systems.js
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
// END COPYRIGHT 


//**************************************
//
// Version: 1.42 11/10/14 16:06:11
//
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.Systems");

dojo.require("com.ibm.bpm.social.util.common.Log");

/**
 * Wraps the result of the <i>/systems</i> REST API call <br>
 * TODO add more getters
 * 
 * @author admard
 * 
 */

/*
 * "systems":
      [
         {  "systemID": {"type": "string", 
               "description": "ID of the system that is described by this object."
            },
            "systemType": {"type": "string",
               "description": "Type of the system that is described by this object.",
               "enum":
               [
                  "WLE",
                  "WPS",
                  "ICM"
               ]
            },
            "version": {"type": "string",
               "description": "Version of the system that is described by this object.",
            },
            "groupWorkItemsEnabled": {"type": "boolean",
               "description": "States whether group workitems can be used."
            },
            "substitutionEnabled": {"type": "boolean",
               "description": "States whether substitution is enabled."
            },
            "substitutionManagementRestrictedToAdministrators": {"type": "boolean",
               "description": "States whether substitution management is restricted to administrators."
            },
            "taskHistoryEnabled": {"type": "boolean",
               "description": "States whether the collection of task history events is enabled."
            },
            "workBasketsEnabled": {"type": "boolean",
               "description": "States whether work baskets are enabled."
            },
            "businessCategoriesEnabled": {"type": "boolean",
               "description": "States whether business categories are enabled."
            }
         }
      ],
   "domains": 
      [
         {  "name": {"type": "string",
               "description": "Name of the domain."
            },
            "displayName": {"type": "string",
               "description": "Display name of the domain."
            },
            "systems": {"type": ["string"],
               "description": "List of IDs for the systems in this domain."
            }
         }
      ], "optional": true
 */

dojo.declare("com.ibm.bpm.social.widgets.task.util.Systems", null, {

  /**
   * The api type of the REST service endpoint.
   * <ul>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   * <li>TODO temporarily "old" if no system</li>
   * </ul>
   * 
   * @type {String}
   */
  _apiType: null,

  /**
   * Federated endpoint: maps system ids to a system object as returned by the <i>/systems</i> API for all
   * systems
   * 
   * @type {Map}
   */
  _systems: null,

  /**
   * Federated endpoint: an array of all system ids that are available to the federated endpoint systems
   * 
   * @type {Map}
   */
  _systemIDs: null,

  /**
   * Configuration information for single system endpoint
   * 
   * @type {Object}
   */
  _system: null,

  /**
   * Federated endpoint: maps domain names to a domain object as returned by the <i>/systems</i> API for all
   * domains
   * 
   * @type {Map}
   */
  _domains: null,

  /**
   * List of all available domains
   * 
   * @type {String[]}
   */
  _domainNames: null,

  _systemsSending401Error: null,

  /**
   * @constructor
   * @param {Object} params param.systems a systems object as returned by the <i>/systems</i> API</li>
   * @throws Error if the expected parameters are not specified
   */
  constructor: function constructor_Systems(params){

    this._systemsSending401Error = null;
    if (params && params.systemsSending401Error) {
      this._systemsSending401Error = params.systemsSending401Error;
    }

    this._systemIDs = [];

    if (params && params.systems) {

      var systems = params.systems.systems;
      var domains = params.systems.domains;

      if (systems) {
        if (systems.length > 0) {
          // domains
          if (domains && domains.length > 0) {
            // save map of domains
            this._domainNames = [];
            if (domains.length > 0) {
              this._domains = {};
              for (i = 0; i < domains.length; i++) {
                var domain = domains[i];
                this._domains[domain.name] = domain;
                this._domainNames.push(domain.name);
                // TODO: SA REST API workaround - domains.systems may contain null values
                dojo.forEach(domain.systems, function(system, index){
                  if (system === null) {
                    domain.systems.splice(index, 1);
                  }
                });
              }
            }

            // federated endpoint
            this._apiType = com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;

            // save systems
            if (systems.length) {
              this._systems = {};
            }
            for ( var i = 0; i < systems.length; i++) {
              if (systems[i].systemID) {
                this._systems[systems[i].systemID] = systems[i];
                this._systemIDs.push(systems[i].systemID);
              }
            }
          }
          else {
            // single system endpoint
            this._apiType = systems[0].systemType;
            // save system
            this._system = systems[0];
            this._systemIDs.push(systems[0].systemID);
          }
        }
        else {
          // log if no system information was returned
          // the error popup with the federation result should show errors
          com.ibm.bpm.social.util.common.Log.log(1, "Warning!!! No system meta information available! systems=" +
            params.systems);
        }
      }
      else {
        // TODO temp old API - no systems object
        this._apiType = "old";
        this._system = {
          version: "7.0.0.0",
          systemType: com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
          systemID: "oldWPS"
        };
      }
      com.ibm.bpm.social.util.common.Log.log(1, "Systems.constructor: " + this);
    }
    else {
      // ASZ: throw prevents cloning via dojo.clone  
      // throw new Error("Systems.constructor: The parameter 'param.systems' is required.");
      com.ibm.bpm.social.util.common.Log.log(1,
        "Warning!!! The parameter 'param.systems' should be set (Ignore if appears during dojo.clone)");
    }
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_Systems(){
    return this._apiType;
  },

  /**
   * Returns whether the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if and only if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * Returns whether the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if and only if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns whether the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if and only if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns whether the REST service endpoint is an ICM system
   * 
   * @returns {boolean} true if and only if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns the system properties of the REST service endpoint for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system for a federated REST service endpoint
   * @returns {Object} the system properties of a system as returned by the <i>/systems</i> API
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystem: function getSystem_Systems(systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (this._systems[systemID]) {
          return this._systems[systemID];
        }
        else {
          //throw new Error("Systems.getSystem: No system found for systemID=" + systemID);
          com.ibm.bpm.social.util.common.Log.log(1, "Systems.getSystem: No system found for systemID=" + systemID);
          return null;
        }
      }
      else {
        // programming error
        throw new Error("Systems.getSystem: A systemID is required for a federated system!");
      }
    }
    else {
      return this._system;
    }
  },

  /**
   * Returns the systems object
   * 
   * @returns {String[]} an array of all systems ids in all domains
   */
  getSystemIDs: function getSystemIDs_Systems(){
    return this._systemIDs;
  },

  /**
   * Returns the system type for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType for the specified system id or the systemType of the single system if no
   *          systemID was specified
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          </ul>
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystemType: function getSystemType_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType;
    }
    return null;
  },

  /**
   * Returns the estimated system type for the specified system id.
   * 
   * @param {String} systemID mandatory, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType derived from the specified system id
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          <li>null</li>
   *          </ul>
   */
  deriveSystemTypeFromSystemId: function deriveSystemTypeFromSystemId_Systems(systemID){
    if (systemID && systemID.length > 0) {
      // WPS systemID looks like this: "_PK:90230132.462033e2.cdc167f6.795e0002"
      if (systemID.indexOf("_PK:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
      }
      // ICM systemID looks like this: "_ICM:fmtc4235:P8ConnPt1:ICMPersonal"
      if (systemID.indexOf("_ICM:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      // WLE systemID looks like this: "3b1ce0ca-f622-44e4-9ec5-5288ca724856"
      // Since only these 3 types exist and 2 of them can be recognized by a prefix substring,
      //  we can conclude the catch-all case as WLE
      return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    // garbage-in - garbage-out case: return null if param 1 was null or the empty string
    return null;
  },

  /**
   * Returns whether the system is a WLE system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wle system
   */
  isSystemWle: function isSystemWle_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    return false;
  },

  /**
   * Returns whether the system is a WPS system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wps system
   */
  isSystemWps: function isSystemWps_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
    }
    return false;
  },

  /**
   * Returns whether the system is an ICM system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if icm system
   */
  isSystemIcm: function isSystemIcm_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
    }
    return false;
  },

  /**
   * Returns the setting for search enabled on the system
   * 
   * @param systemID
   * @returns {boolean} true if search is enabled on system
   */
  isSystemSearchable: function isSystemSearchable_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system) {
      return system.taskSearchEnabled === true;
    }
    return false;

  },


  /**
   * @returns {String} the system id if we have a single system backend
   * @throws Error if the backend is not a single system
   */
  getSystemID: function getSystemID_Systems(){
    if (!this.isApiTypeFederated()) {
      return this._system.systemID;
    }
    else {
      // programming error
      throw new Error("Systems.getSystemID: Method is not supported for Federation!");
    }
  },

  /**
   * Returns the map of all domains for a federated REST service endpoint
   * 
   * @returns {Map} the map of all domains, null if it is not a federated REST service endpoint.
   *          <dl>
   *          <dt>key</dt>
   *          <dd>domainName</dd>
   *          <dt>value</dt>
   *          <dd>Object of domain properties of the domain as returned by the <i>/systems</i> API</dd>
   *          </dl>
   * 
   */
  getDomains: function getDomains_Systems(){
    return this._domains;
  },

  /**
   * Returns a list of all available domain names
   * 
   * @returns {String[]} the list of all domains, null if it is not a federated REST service endpoint.
   */
  getDomainNames: function getDomainNames_Systems(){
    return this._domainNames;
  },


  /**
   * Returns the domain properties of the REST service endpoint for the specified domain name.
   * 
   * @param {String} domainName the name of a backend domain for a federated REST service endpoint
   * @returns {Object} the domain properties of the domain as returned by the <i>/systems</i> API
   * @throws Error if no domain name is specified.
   */
  getDomain: function getDomain_Systems(domainName){
    if (domainName) {
      if (this._domains[domainName]) {
        return this._domains[domainName];
      }
      else {
        // throw new Error("Systems.getDomain: No domain found for domainName=" + domainName);
        com.ibm.bpm.social.util.common.Log.log(1, "Systems.getDomain: No domain found for domainName=" + domainName);
        return null;
      }
    }
    else {
      // programming error
      throw new Error("Systems.getDomain: A domainName is required!");
    }
  },

  /**
   * Returns if the domain contains only WPS systems
   * 
   * @returns {boolean} true if the domain contains only WPS systems
   * @throws Error if no domain name is specified.
   */
  isDomainWpsOnly: function isDomainWpsOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
  },

  /**
   * Returns if the domain contains only WLE systems
   * 
   * @returns {boolean} true if the domain contains only WLE systems
   * @throws Error if no domain name is specified.
   */
  isDomainWleOnly: function isDomainWleOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
  },

  /**
   * Returns if the domain contains only ICM systems
   * 
   * @returns {boolean} true if the domain contains only ICM systems
   * @throws Error if no domain name is specified.
   */
  isDomainIcmOnly: function isDomainIcmOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
  },

  /**
   * Are the systems in the domain wps only, icm only, wle only, or mixed:=more that one (any two or three)
   * 
   * @param domainName
   * @returns {String} the systemType if only one type, or mixed otherwise
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED</li>
   *          </ul>
   * @throws Error if no domain name is specified.
   */
  getDomainType: function getDomainType_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._domainType === undefined) {
      var wps = false;
      var wle = false;
      var icm = false;
      for ( var s = 0; s < domain.systems.length; s++) {
        var systemType = this.getSystemType(domain.systems[s]);
        wps = (wps || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS);
        wle = (wle || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE);
        icm = (icm || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM);
      }
      if (wps && !wle && !icm) {
        // wps only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
      }
      else if (wle && !wps && !icm) {
        // wle only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
      }
      else if (icm && !wps && !wle) {
        // icm only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
      }
      else {
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED;
      }
    }
    return domain._domainType;
  },
  /**
   * Check for the domain, if it has a WPS system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWpsSystem: function getWpsSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WPS system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWleSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WLE system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getIcmSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not an ICM system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Returns systems for domain name
   * 
   * @param domainName
   * @returns {object} systemIds
   * @throws Error if no domain name is specified.
   */
  getSystems: function getSystems_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systems = domain.systems;
    return systems;
  },

  /**
   * Return true if a domain of kind WLE_ONLY or WPS_ONLY or ICM_ONLY has multiple systems in it.
   * 
   * @param {String} domainName
   * 
   * @type {Boolean}
   */
  isMultiSystemDomain: function isMultiSystemDomain_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "domainName");
    var domain = this.getDomain(domainName);
    var domainType = this.getDomainType(domainName);
    var isMulti = true;
    // if we erroneously got a MIXED domain, then is must be a multiSys domain 
    if (domainType != com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED) {
      isMulti = (domain.systems.length > 1);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "isMultiSystemDomain==" + (isMulti ? "true" : "false"));
    return isMulti;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS, federated with WPS et al., or federated WPS only
   */
  isWps: function isWps_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWpsSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE, federated with WLE et al., or federated WLE only
   */
  isWle: function isWle_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWleSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM, federated with ICM et al., or federated ICM only
   */
  isIcm: function isIcm_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getIcmSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS or federated WPS only
   */
  isWpsOnly: function isWpsOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE or federated WLE only
   */
  isWleOnly: function isWleOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM or federated ICM only
   */
  isIcmOnly: function isIcmOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },


  /**
   * Is there at leaset one icm systems in the domain?
   * 
   * @param domainName
   * @returns {boolean} tru if domain includes ICM system, else false
   * @throws Error if no domain name is specified.
   */
  hasIcmSystem: function hasIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._hasIcm === undefined) {
      var icm = false;
      var s = 0;
      while (s < domain.systems.length && !icm) {
        var systemType = this.getSystemType(domain.systems[s]);
        s++;
        icm = systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      domain._hasIcm = icm;
    }
    return domain._hasIcm;
  },

  /**
   * @param domainName optional
   * @return {String} backendType com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM
   */
  _getBackendType: function _getBackendType_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null;
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      backendType = this.getDomainType(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, backendType);
    return backendType;
  }


});

////////////////////////////
// constants for systems
////////////////////////////

/**
 * API type WPS
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * API type WLE
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * API type ICM
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM = "SYSTEM_TYPE_ICM";

/**
 * API type WPS federated
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED = "federated";

/**
 * System type WPS tbd. if we need HTM/BFM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * System type WLE
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * System type ICM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM = "SYSTEM_TYPE_ICM";


/**
 * Domain type for WPS only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS = "WPS";

/**
 * Domain type for WLE only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE = "WLE";

/**
 * Domain type for ICM only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM = "ICM";

/**
 * Domain type for mixed domains (any two of {WLE,WPS,ICM} or all three)
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED = "MIXED";


/**
 * Define for the default domain name
 */
com.ibm.bpm.social.widgets.task.util.Systems.DEFAULT_DOMAIN_NAME = "default";


/**
 * Regular expression for a WLE TASK ID.
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
// integer number
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING = "^([0-9]+)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING);

/**
 * Regular expression for a WLE TEMPLATE ID.
 * 
 * If this changes, then update as well REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// UUID 8-4-4-4-12 (32 hexadecimal digit in 5 groups)
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING =
  "^([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12})$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING);

/**
 * Regular expression for a WPS ID
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below and
 * REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// tkiid or tktid or piid or ptid
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING);

/**
 * Regular expression for an ICM TKIID as received by Hong Dong Zhao (12. July 2011 12:12)
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING =
  "^([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING);


/**
 * Now specify the merged patterns, combined with OR (either pattern will be accepted)
 * 
 * We could have done as well the 3 two-out-of-three combinations, to e.g. reject an ICM id when no ICM is
 * involved. But the newly added ICM pattern is quite restrictive. And this pattern is only used for a first
 * syntactical check - semantical checks will follow anyway. So the damage by letting 3 types pass when only
 * two types of systems are involved is rather small.
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+)|([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING);

com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING);