// TabContainer.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.util.TabContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.widgets.task.util.TabConfiguration");
dojo.require("com.ibm.bpm.social.util.common.Log");

/**
 * based on LightWeightTabContainer.js Version: 1.22 11/10/05 16:22:49
 * 
 * this class implements an lightweight tab container
 * 
 * The content of the tabs is created only when the tab is selected. After creating the content of a tab and
 * selecting a new one the old content is hidden, but not deleted. After selecting the tab again the content
 * has not to be created again.
 * 
 * For the creation of the content of the tab a context a function (loadFunction) has to be set inside the
 * TabConfiguration-object.
 * 
 * Furthermore a function has to be provided inside the class passed as context that checks if the tab has to
 * be reloaded or not. If the tab has to be reloaded - all other tabs will also be reloaded. This function has
 * to be called as reloadAllTabs and returns a boolean value.
 * 
 * The lightweight tab container has also a function for resizing the tab content.
 * 
 * For getting know if a tab has been changed the function showTab is provided. This function is only provided
 * as point for connecting.
 * 
 * Furthermore the functionality for showing an error on a tab is provided. Therefore the function markError
 * has to be called. The error-symbol is only shown if the content of the tab is currently not shown. For
 * removing the error-symbol the function removeError has to be called.
 * 
 * using this class - you have first to call the function configuration and pass the parent node where the
 * lightweight tab container has to be added and an array of TabConfiguration-objects for all tabs - than you
 * have to call the function createTabs
 */
dojo.declare("com.ibm.bpm.social.widgets.task.util.TabContainer", [dijit._Widget,
                                                                   dijit._Templated], {

  //parent node where the nodes of the lightweight tab container will be added
  parentNode: null,
  //array containing the TabConfiguration-objects for all tabs
  tabInformations: null,
  //list of all tab nodes
  tabList: null,

  //needed, when an new tab is selected
  lastTab: null,

  rtl: false,

  tabHeaderDiv: null,
  paramTabHeaderDiv: false,

  tabContent: null,
  paramTabContentDiv: false,

  // template of the lightweight tab container    
  // To ensure that the css definitions from LightWeightTab.css have precedence (e.g. TaskEditor.css)
  // the class "lightWeightTabContainer" has been added to tabContainer for more specific css rules.
  templateString: //'<div class="lightWeightTabContainer bspace-style bspace-text5" dojoAttachPoint="tabContainer">\
  '<div class="lightWeightTabContainer" dojoAttachPoint="tabContainer">\
      </div>',

  constructor: function constructor_TabContainer(params){
    var loc = dojo.locale;
    if ((loc.indexOf("iw") === 0) || (loc.indexOf("he") === 0) || (loc.indexOf("ar") === 0)) {
      this.rtl = true;
    }

    if (params) {
      if (params.tabHeaderDiv) {
        this.tabHeaderDiv = params.tabHeaderDiv;
        this.paramTabHeaderDiv = true;
      }

      if (params.tabContent) {
        this.tabContent = params.tabContentDiv;
        this.paramTabContentDiv = true;
      }
    }

  },

  postCreate: function(){
    /* create tab header */
    if (!this.paramTabHeaderDiv) {
      this.tabHeaderDiv = document.createElement("div");
      dojo.addClass(this.tabHeaderDiv, "lightWeightTabContainer");
      this.tabContainer.appendChild(this.tabHeaderDiv);
    }
    this.tabHeader = document.createElement("div");
    dojo.addClass(this.tabHeader, "tabHeader");
    this.tabHeaderDiv.appendChild(this.tabHeader);

    /* create tab content */
    if (!this.paramTabContentDiv) {
      this.tabContent = document.createElement("div");
      dojo.addClass(this.tabContent, "tabContent");
    }
    this.tabContainer.appendChild(this.tabContent);
  },

  /**
   * this function is needed to set the configuration for the lightweight tab container
   */
  setTabConfiguration: function setTabConfiguration_TabContainer(tabInformations){
    this.parentNode = this.tabContainer.parentNode;
    this.tabInformations = tabInformations;

    //array containing all connects to disconnect them
    this.allCreatedConnects = new Array();

    //array containing all widgets which are not part of the dom tree
    this.allWidgets = new Array();
  },

  /**
   * This function displays the active tab and hides all others. It also set the style for the active tab and
   * remove it from the former active tab. It is called after selecting a tab.
   */
  selectTab: function selectTab_TabContainer(tabInformation, tab){

    //check if same tab is selected
    if (tabInformation.tabName === this.lastTab.tabName) {
      return;
    }

    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0) {
      //remove class from last active tab
      dojo.removeClass(activeTab[0], "activeLightWeightTab");
      dijit.setWaiState(activeTab[0], "selected", "false");
      activeTab[0].firstChild.lastChild.tabIndex = -1;
    }

    //mark current tab as active
    dojo.addClass(tab, "activeLightWeightTab");
    dijit.setWaiState(tab, "selected", "true");
    tab.firstChild.lastChild.tabIndex = 0;

    //hide the content of the former active tab
    this.hideTabContent();
    //show the content of the new active tab (this could include the creation of the content)
    this.showTabContent(tabInformation);

    //set new tab to last tab
    this.lastTab = tabInformation;
  },

  /**
   * This function is called if a key is pressed. if the key is the right or left arrow the tab is changed.
   */
  _changeTabByKey: function _changeTabByKey_TabContainer(event, tab, sortedTabInformations, tabInformation){
    var keys = dojo.keys;

    switch (event.keyCode) {
      case (!this.rtl && keys.RIGHT_ARROW) || (this.rtl && keys.LEFT_ARROW) :
        var nextSibling = tab.nextSibling;

        //check if a next tab exists
        if (nextSibling) {
          // There is a separator "|" between tabs!
          var nextTab = nextSibling.nextSibling;
          nextTab.firstChild.lastChild.focus();

          //call the function for handling a change of the tab
          this.selectTab(sortedTabInformations[tabInformation.position + 1], nextTab);
        }
        break;
      case (!this.rtl && keys.LEFT_ARROW) || (this.rtl && keys.RIGHT_ARROW) :
        var previousSibling = tab.previousSibling;

        //check if previous tab exists
        if (previousSibling) {
          // There is a separator "|" between tabs!
          var previousTab = previousSibling.previousSibling;
          previousTab.firstChild.lastChild.focus();

          //call the function for handling a change of the tab
          this.selectTab(sortedTabInformations[tabInformation.position - 1], previousTab);
        }
        break;
      default:
        break;
    }
  },

  /**
   * this function creates the tab container and the content of the first tab
   */
  createTabs: function createTabs_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.createTabs entry");

    var myself = this;

    //creating a node for surrounding the tabs
    myself.tabList = document.createElement("div");
    dojo.addClass(myself.tabList, "lightWeightTabList");
    dojo.addClass(myself.tabList, "lightWeightTabLink");

    dijit.setWaiRole(myself.tabList, "tablist");

    //used for set the position of the tab inside the TabConfiguration-object
    var position = 1;

    //this variable is used for marking the tab for which the content is shown as the active one
    var activeTab = null;

    //used for navigating between tabs with arrows
    var sortedTabInformations = {};

    //iterate through all tabs and create it
    dojo.forEach(myself.tabInformations, function(tabInformation, index){

      //check if the tab should be active - if not it will not be created
      if (tabInformation.isActive) {
        //id of the node - consisting of the widget id and the index of the tab
        var id = tabInformation.widgetID + "_" + tabInformation.id;

        //set position in tabInformation - is used for navigate with arrows through tabs
        tabInformation.setPosition(position);
        sortedTabInformations[position] = tabInformation;
        position = position + 1;

        // create new tab
        //first create a div element
        tabInformation.tab = document.createElement("div");
        tabInformation.tab.id = id;
        tabInformation.tab.title = tabInformation.tooltip;
        tabInformation.tab.style.display = "inline-block";
        dojo.addClass(tabInformation.tab, "lightWeightTab");
        dijit.setWaiRole(tabInformation.tab, "tab");

        //create a link containing the name of the tab
        var link = document.createElement("a");
        link.href = "javascript:{}";
        link.innerHTML = tabInformation.tabName;
        //wrap the created link
        var linkWrapper = document.createElement("div");
        linkWrapper.style.display = "inline-block";
        dojo.addClass(tabInformation.tab, "lightWeightTabLinkDiv");
        linkWrapper.appendChild(link);
        tabInformation.tab.appendChild(linkWrapper);
        tabInformation.tab.focusNode = link;

        // Add a11y stuff (high contrast) for the error symbol				
        var errorText = document.createElement("span");
        errorText.innerHTML = "!";
        errorText.style.display = "none";
        dojo.addClass(errorText, "a11yErrorText");

        tabInformation.tab.appendChild(errorText);

        //add event to tab - containing the function that should process in case
        //of selecting a tab
        myself.allCreatedConnects.push(dojo.connect(link, "onclick", dojo.hitch(myself, "selectTab", tabInformation,
          tabInformation.tab)));

        //set tab index for changing the order when using tab key
        link.tabIndex = -1;

        // Mark the first tab as open
        //content for the tab is created later
        if (index === 0 || tabInformation.contentIsShownAfterInit) {
          myself.lastTab = tabInformation;
          activeTab = tabInformation.tab;
        }

        // Create separator between tabs
        if (index > 0) {
          var sep = document.createElement("span");
          dojo.addClass(sep, "lightWeightTabSeparator");
          sep.innerHTML = "|";
          myself.tabList.appendChild(sep);
        }

        // Navigate through tabs by using LEFT and RIGHT arrow keys.
        myself.allCreatedConnects.push(dojo.connect(tabInformation.tab, "onkeydown", function(event){
          myself._changeTabByKey(event, tabInformation.tab, sortedTabInformations, tabInformation);
        }));

        //add the tab to the div containing all tabs
        myself.tabList.appendChild(tabInformation.tab);

      }
    });

    //adds all tabs to the node containing the tabs
    myself.tabHeader.appendChild(myself.tabList);

    //adds the tab container to the defined parent node
    myself.parentNode.appendChild(myself.tabContainer);

    //mark the tab as active
    dojo.addClass(activeTab, "activeLightWeightTab");
    activeTab.firstChild.lastChild.tabIndex = 0;
    dijit.setWaiState(activeTab, "selected", "true");

    //creates the content of the first tab - that is shown after creating the tab container
    myself.showTabContent(myself.lastTab);

    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.createTabs exit");
  },

  /**
   * this function is responsible for showing the content of a tab - for example after changing a tab
   */
  showTabContent: function showTabContent_TabContainer(tabInformation){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var context = tabInformation.context;
    var loadFunction = tabInformation.loadFunction;
    var id = "tab_" + tabInformation.id + "_" + loadFunction;
    var contentNode = null;

    //check if all tabs has to be reloaded
    var reloadTabs = dojo.hitch(context, "reloadAllTabs")();
    var allContentNodes = this.tabContent.childNodes;

    //if all tabs has to be reloaded first the old content of the tabs has to be destroyed
    if (reloadTabs) {
      //implementation this way because for each does not work correct here
      //for destroying the content iterate through all nodes of the tabContent area and destroy them
      if (allContentNodes.length > 0) {
        var node = allContentNodes[0];
        var nextNode = null;

        while (node) {
          nextNode = node.nextSibling;

          //destroy all widgets and elements of the content of the tab
          this.destroyTabContent(node);

          //destroy the div that has not been destroyed yet
          dojo._destroyElement(node);

          //read the next node
          node = nextNode;
        }
      }

    }
    else {
      //read the content of the selected tab if it has been already generated
      //therefore read all child-nodes of the content area, iterate through the nodes 
      //and check if the set class name is equivalent to the id
      dojo.forEach(allContentNodes, function(node){
        if (node.id === id) {
          contentNode = node;
        }
      });
    }

    //if the content of the selected tab has not been generated
    if (!contentNode) {

      //create the div element containing the content
      var divTab = document.createElement("div");

      //add the id to the element - so it can be identified later
      divTab.id = id;
      dojo.addClass(divTab, "tabContent");
      dijit.setWaiRole(divTab, "tabpanel");

      //append the div to the content area
      this.tabContent.appendChild(divTab);

      //if the function that has to be called when creating the content of the tab is set 
      //call it
      if (loadFunction) {
        //the div is passed for adding the content of the tab inside it
        dojo.hitch(context, loadFunction)(divTab, tabInformation);
      }
      else {
        //log an message, because the content for the tab could not be generated
        com.ibm.bpm.social.util.common.Log.log(2, "For the current tab no load function is passed");
      }
      contentNode = divTab;
    }

    //display the content of the selected tab
    contentNode.style.display = "block";
    //call the function so that inside another context it can be recognized, that the tab has been changed
    this.showTab(tabInformation);
    //this field is used to check if in the content area - the content of a tab is shown or not
    this._isVisible = true;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  //function for connecting to recognize if a tab is changed - this is necessary, because in case
  //of that the content has already been loaded only the content is shown and the load function has
  //not been called
  //in some cases after a tab change some functions has to be called for example to change the size
  //of the content area
  showTab: function showTab_TabContainer(tabInformation){}, // TODO tabInformation???

  /**
   * this function is used to hide the content area
   */
  hideTabContent: function hideTabContent_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.hideTabContent entry");

    //check if the content of an area is visible
    //if nothing is visible - no area has to be hidden
    if (this._isVisible) {

      //iterate through all child nodes of the content area and hide them
      var allContentNodes = this.tabContent.childNodes;

      dojo.forEach(allContentNodes, function(node){
        node.style.display = "none";
      });
      this._isVisible = false;
    }
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.hideTabContent exit");
  },

  /**
   * this function is used to show the content of the first tab of the lightweight tab container
   */
  showFirstTab: function showFirstTab_TabContainer(){
    var tabInformation = this.tabInformations[0];
    var tab = tabInformation.tab;

    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0) {
      //remove class from last active tab
      dojo.removeClass(activeTab[0], "activeLightWeightTab");
      activeTab[0].firstChild.lastChild.tabIndex = -1;
    }

    //mark current tab as active
    dojo.addClass(tab, "activeLightWeightTab");
    tab.firstChild.lastChild.tabIndex = 0;

    //hide the content that is shown at the moment
    this.hideTabContent();
    //show the content of the first tab
    this.showTabContent(tabInformation);

    //set new tab to last tab
    this.lastTab = tabInformation;
  },

  /**
   * this function is used to trigger a reload of the content of the tabs even no other tab has been selected
   * 
   * this function could be used after an update if the data of the content should contain the latest data
   */
  reloadTabs: function reloadTabs_TabContainer(){
    //the content could only be updated, if the content of a tab is shown
    if (this.lastTab) {
      this.showTabContent(this.lastTab);
    }
  },

  /**
   * this function is used to destroy the content of a tab - for example for reloading the content
   */
  destroyTabContent: function destroyTabContent_TabContainer(div){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroyTabContent entry");

    //read all widgets of the passed node
    var allWidgets = div.getElementsByTagName('*');
    var length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    if (div.hasChildNodes()) {
      //iterate through all child nodes of the passed div to remove them
      while (div.childNodes.length > 0) {
        div.removeChild(div.firstChild);
      }
    }
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroyTabContent exit");
  },

  /**
   * this function is used to mark an error at a tab - if the content of the tab is not shown a error-symbol
   * is shown behind the tab
   */
  markError: function markError_TabContainer(tabName){
    var i;
    var tabInformation;
    var id;

    //iterate through the tabConfiguration-objects
    for (i = 0; i < this.tabInformations.length; i++) {
      tabInformation = this.tabInformations[i];

      //check if the current tab should be marked as error
      if (tabInformation.tabName === tabName) {

        //read the node that should to marked as error
        id = "#" + tabInformation.widgetID + "_" + tabInformation.id;
        var nodes = dojo.query(id, this.tabHeader);

        //add the class for showing the error-symbol to the node
        if (nodes.length > 0) {
          dojo.addClass(nodes[0], "lightWeigthTabShowError");
        }
        return;
      }
    }
  },

  /**
   * this function is used to remove a error-symbol from a tab
   */
  removeError: function removeError_TabContainer(tabName){
    var i;
    var tabInformation;
    var id;

    //iterate through the tabConfiguration-objects
    for (i = 0; i < this.tabInformations.length; i++) {
      tabInformation = this.tabInformations[i];

      //check if from the current tab error-symbol should be removed
      if (tabInformation.tabName === tabName) {

        //read the node where the error-symbol should remove
        id = "#" + tabInformation.widgetID + "_" + tabInformation.id;
        var nodes = dojo.query(id, this.tabHeader);
        if (nodes.length > 0) {
          //remove the class for showing the error-symbol to the node
          dojo.removeClass(nodes[0], "lightWeigthTabShowError");
        }
        return;
      }
    }
  },

  /**
   * this function resize the lightweight tab container and returns an object containing the height and width
   * of the content pane
   */
  resizeContentPaneDimension: function resizeContentPaneDimension_TabContainer(height){
    com.ibm.bpm.social.util.common.Log.log(1,
      "com.ibm.bpm.social.widgets.task.util.TabContainer.resizeContentPaneDimension entry");

    //change the height of the lightweight tab container
    this.domNode.style.height = height + "px";

    var width;

    //calculate the height of the content area of the lightweight tab container
    height = height - 25;
    if (!this.paramTabHeaderDiv) {
      if (dojo.isIE < 8) {
        height = height - this.tabHeaderDiv.offsetHeight;
      }
      else {
        height = height - this.tabHeaderDiv.offsetHeight - this.tabHeader.offsetHeight;
      }
    }
    if (height > 0) {
      //set the height of the content area
      this.tabContent.style.height = (height) + "px";

      //subtract the padding of the content area for resizing the content inside
      height = this.tabContent.offsetHeight;
      height = height - 24;
      width = this.tabContent.offsetWidth;
      width = width - 24;
    }
    else {

      //calculate the result even if the content area is not resized
      height = this.tabContent.offsetHeight;
      height = height - 24;
      width = this.tabContent.offsetWidth;
      width = width - 24;
    }
    var dimension = {
      h: height,
      w: width
    };
    com.ibm.bpm.social.util.common.Log.log(1,
      "com.ibm.bpm.social.widgets.task.util.TabContainer.resizeContentPaneDimension exit");
    return dimension;
  },

  /**
   * this function is used to destroy all widgets of the lightweight tab container
   */
  destroy: function destroy_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroy entry");

    //disconnect the saved connects
    var i;
    for (i in this.allCreatedConnects) {
      dojo.disconnect(this.allCreatedConnects[i]);
    }
    this.allCreatedConnects = new Array();

    //destroy all widgets that are not part of the dom node
    for (i in this.allWidgets) {
      if (this.allWidgets[i].destroyRecursive) {
        this.allWidgets[i].destroyRecursive();
      }
      else {
        this.allWidgets[i].destroy();
      }
      delete this.allWidgets[i];
    }
    this.allWidgets = new Array();

    //read all elements of the lightweight tab content
    var allWidgets = this.tabContent.getElementsByTagName('*');
    var length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    if (this.tabContainer.hasChildNodes()) {
      // remove tab header div
      if (!this.paramTabHeaderDiv) {
        this.tabContainer.removeChild(this.tabHeaderDiv);
      }
      // remove tab content div
      if (!this.paramTabContentDiv) {
        this.tabContainer.removeChild(this.tabContent);
      }
    }

    //read all elements of the lightweight tab header
    allWidgets = this.tabHeaderDiv.getElementsByTagName('*');
    length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    this.inherited("destroy", arguments);

    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroy exit");
  },

  focusActiveTab: function focusActiveTab_TabContainer(){
    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0 && activeTab[0].focusNode) {
      // focus header node
      activeTab[0].focusNode.focus();
    }

  }

});