/*
	Licensed Materials - Property of IBM
	5725-C96
	Copyright IBM Corp. 2008, 2011, 2012 All Rights Reserved.
	US Government Users Restricted Rights - Use, duplication or
	disclosure restricted by GSA ADP Schedule Contract with
	IBM Corp.
*/
dojo.provide("com.ibm.bpm.social_base");
if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");


dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						var widget = hash[widgetId];
						if(widget){	// may be null on page w/multiple dojo's loaded
							outAry.push(widget);
						}
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body.contentEditable == 'true' || (body.firstChild && body.firstChild.contentEditable == 'true');
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};
		function radioName(node) {
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE <= 9 && child.scopeName !== "HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(dojo.attr(child, "checked") && rn) {
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		function rs(node) {
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}
		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dojo.Stateful"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.Stateful"] = true;
dojo.provide("dojo.Stateful");


dojo.declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			dojo.mixin(this, mixin);
		}
	},
	
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3
		
		return this[name];
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this;
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.
		
		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
                        propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							try{
								propertyCallbacks[i].call(self, name, oldValue, value);
							}catch(e){
								console.error(e);
							}
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(dojo.indexOf(propertyCallbacks, callback), 1);
			}
		};
	}
	
});

}

if(!dojo._hasResource["dijit._WidgetBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._WidgetBase"] = true;
dojo.provide("dijit._WidgetBase");




(function(){

dojo.declare("dijit._WidgetBase", dojo.Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// mix in our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo._mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(this._getSetterAttributes(), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	_getSetterAttributes: function(){
		// summary:
		//		Returns list of attributes with custom setters for this widget
		var ctor = this.constructor;
		if(!ctor._setterAttrs){
			var r = (ctor._setterAttrs = []),
				attrs,
				proto = ctor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
		}
		return ctor._setterAttrs;	// String[]
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || dojo.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.replaceClass(mapNode, value, this["class"]);
		this._set("class", value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.replaceClass(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			this._set(name, value);
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handles = [dojo._connect(obj, event, this, method)];
		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var handle = dojo.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	},

	defer: function(fcn, delay){
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(dojo.hitch(this, 
			function(){ 
				timer = null;
				if(!this._destroyed){ 
					dojo.hitch(this, fcn)(); 
	}
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

})();

}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");

dojo.getObject("window", true, dojo);

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view using minimal movement, if it is not already.
	
	// Don't rely on node.scrollIntoView working just because the function is there since
	// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
	// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
	// and when there's a fixed position scrollable element

	node = dojo.byId(node);
	var body, doc = node.ownerDocument || dojo.doc;

	// has() functions but without has() support
	if(!("rtl_adjust_position_for_verticalScrollBar" in dojo.window)){
		body = dojo.body();
		var	scrollable = dojo.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = dojo.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last");
		dojo.window.rtl_adjust_position_for_verticalScrollBar = dojo.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
	}

	if(!("position_fixed_support" in dojo.window)){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		body = dojo.body();
		var	outer = dojo.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = dojo.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last");
		dojo.window.position_fixed_support = dojo.position(inner).x != dojo.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
	}

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		body = doc.body || doc.getElementsByTagName("body")[0];
		var	html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE,
			isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if(node == body || node == html){ return; }
		if(!(dojo.isMozilla || isIE || isWK || dojo.isOpera) && ("scrollIntoView" in node)){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
			rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
			scrollRoot = (isWK || backCompat) ? body : html,
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return (isIE <= 6 || (isIE == 7 && backCompat))
					? false
					: (dojo.window.position_fixed_support && (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do
		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el),
				rtl = dojo.getComputedStyle(el).direction.toLowerCase() == "rtl";
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE || isIE >= 9){ elPos.x = 0; } // older IE can have values > 0
				if(elPos.y < 0 || !isIE || isIE >= 9){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					if(rtl && dojo.window.rtl_adjust_position_for_verticalScrollBar){
						elPos.x += scrollBarSize;
					}
					elPos.w = clientSize;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				t = nodePos.y - elPos.y, // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			var s, old;
			if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
				s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				old = el.scrollLeft;
				el.scrollLeft += s;
				s = el.scrollLeft - old;
				nodePos.x -= s;
			}
			if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
				s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
				old = el.scrollTop;
				el.scrollTop += s;
				s = el.scrollTop - old;
				nodePos.y -= s;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");




// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
					if(sel.rangeCount){
						bm.mark = sel.getRangeAt(0).cloneRange();
					}
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					targetWindow.document.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.body.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.body.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget.set("focused", false);
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				widget.set("focused", true);
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");


dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
};

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary:
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");




dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, choices, layoutNode, /*Object*/ aroundNodeCoords){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//		It also passes in the available size for the popup, which is useful for tooltips to
	//		tell them that their width is limited to a certain amount.   layoutNode() may return a value expressing
	//		how much the popup had to be modified to fit into the available space.   This is used to determine
	//		what the best placement is.
	// aroundNodeCoords: Object
	//		Size of aroundNode, ex: {w: 200, h: 50}

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;
		var overflow = 0;

		// calculate amount of space available given specified position of node
		var spaceAvailable = {
			w: corner.charAt(1) == 'L' ? (view.l + view.w) - pos.x : pos.x - view.l,
			h: corner.charAt(1) == 'T' ? (view.t + view.h) - pos.y : pos.y - view.t
		};

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltip's size changes based on position, due to triangle)
		if(layoutNode){
			var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
			overflow = typeof res == "undefined" ? 0 : res;
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY;

		overflow += (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow,
				spaceAvailable: spaceAvailable
			};
		}
		
		return !overflow;
	});

	// In case the best position is not the last one we checked, need to call
	// layoutNode() again.
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
	}

	// And then position the node.   Do this last, after the layoutNode() above
	// has sized the node, due to browser quirks when the viewport is scrolled
	// (specifically that a Tooltip will shrink to fit as though the window was
	// scrolled to the left).
	//
	// In RTL mode, set style.right rather than style.left so in the common case,
	// window resizes move the popup along with the aroundNode.
	var l = dojo._isBodyLtr(),
		s = node.style;
	s.top = best.y + "px";
	s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
	s[l ? "right" : "left"] = "auto";       // needed for FF or else tooltip goes to far left

	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var aroundNodePos = dojo.position(aroundNode, true);

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode, {w: width, h: height});
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below-alt":
				leftToRight = !leftToRight;
				// fall through
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above-alt":
				leftToRight = !leftToRight;
				// fall through
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");



dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 1000,

	_idGen: 1,

	_createWrapper: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups.
		//		Puts widget inside a wrapper DIV (if not already in one),
		//		and returns pointer to that wrapper DIV.

		var node = widget.domNode || widget,
			wrapper = widget.declaredClass ? widget._popupWrapper :
				node.parentNode && dojo.hasClass(node.parentNode, "dijitPopup") ? node.parentNode : null;

		if(!wrapper){
			// Create wrapper <div> for when this widget [in the future] will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{ display: "none"},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
		
		return wrapper;	// DOMNode
	},

	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Moves the popup widget off-screen.
		//		Do not use this method to hide popups when not in use, because
		//		that will create an accessibility issue: the offscreen popup is
		//		still in the tabbing order.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, {
			visibility: "hidden",
			top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
			display: ""
		});
	},

	hide: function(/*dijit._Widget*/ widget){
		// summary:
		//		Hide this popup widget (until it is ready to be shown).
		//		Initialization for widgets that will be used as popups
		//
		// 		Also puts widget inside a wrapper DIV (if not already in one)
		//
		//		If popup widget needs to layout it should
		//		do so when it is made visible, and popup._onShow() is called.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, "display", "none");
	},
		
	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

		// If we are opening a new popup that isn't a child of a currently opened popup, then
		// close currently opened popup(s).   This should happen automatically when the old popups
		// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
		while(stack.length && (!args.parent || !dojo.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
			dijit.popup.close(stack[stack.length-1].widget);
		}

		// Get pointer to popup wrapper, and create wrapper if it doesn't exist
		var wrapper = this._createWrapper(widget);


		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.display = "";
		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget?*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented.
		//		If no popup is specified, closes all popups.

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while((popup && dojo.some(stack, function(elem){return elem.widget == popup;})) ||
			(!popup && stack.length)){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.hide(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE < 9){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/*DomNode*/ node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = (this.iframe = dijit._frames.pop());
		node.appendChild(iframe);
		if(dojo.isIE<7 || dojo.isQuirks){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		Resize the iframe so it's the same size as node.
		//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
		if(this.iframe){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");



dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");


(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_quirks: d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = d.isQuirks;
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");



// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");


dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE < 9){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");


dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
		// summary:
		//		Determines if an element has a particular role.
		// returns:
		//		True if elem has the specific role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has a role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the role for an element (which should be a wai role).
		// returns:
		//		The role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.

			dojo.attr(elem, "role", role);
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");





////////////////// DEFERRED CONNECTS ///////////////////

// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

////////////////// ONDIJITCLICK SUPPORT ///////////////////

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

dojo.declare("dijit._Widget", dijit._WidgetBase, {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- deferred connections
	//				A call like dojo.connect(myWidget, "onMouseMove", func)
	//				will essentially do a dojo.connect(myWidget.domNode, "onMouseMove", func)
	//			- ondijitclick
	//				Support new dojoAttachEvent="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)
	

	////////////////// DEFERRED CONNECTS ///////////////////

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		this.inherited(arguments);

		if(this.domNode){
			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	////////////////// FOCUS RELATED ///////////////////
	// _onFocus() and _onBlur() are called by the focus manager

	// focused: [readonly] Boolean
	//		This widget or a widget it contains has focus, or is "active" because
	//		it was recently clicked.
	focused: false,

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	////////////////// ONDIJITCLICK SUPPORT ///////////////////

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = this.inherited(arguments, [obj, event == "ondijitclick" ? "onclick" : event, method]);

		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(d.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							
							// Stop event to prevent scrolling on space key in IE.
							// But don't do this for _HasDropDown because it surpresses the onkeypress
							// event needed to open the drop down when the user presses the SPACE key.
							if(!("openDropDown" in this && obj == this._buttonNode)){
								e.preventDefault();
							}
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target == dijit._lastKeyDownNode &&	// === breaks greasemonkey
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
		}

		return handles;		// _Widget.Handle
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");



new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".	 interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }").
					//	Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var dummyClass = {}, instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	// TODO: remove this in 2.0, when we stop caching parameters.
	d.connect(d, "extend", function(){
		instanceClasses = {};
	});

	function getProtoInfo(cls, params){
		// cls: A prototype
		//		The prototype of the class to check props on
		// params: Object
		//		The parameters object to mix found parameters onto.
		for(var name in cls){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
			params[name] = val2type(cls[name]);
		}
		return params;
	}

	function getClassInfo(/*String*/ className, /*Boolean*/ skipParamsLookup){
		// summary:
		//		Maps a widget name string like "dijit.form.Button" to the widget constructor itself,
		//		and a list of that widget's parameters and their types
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{
		//				cls: dijit.Button,
		//				params: { label: "string", disabled: "boolean"}
		//			}

		var c = instanceClasses[className];
		if(!c){
			// get pointer to widget class
			var cls = d.getObject(className), params = null;
			if(!cls){ return null; }		// class not defined [yet]
			if(!skipParamsLookup){ // from fastpath, we don't need to lookup the attrs on the proto because they are explicit
				params = getProtoInfo(cls.prototype, {})
			}
			c = { cls: cls, params: params };
			
		}else if(!skipParamsLookup && !c.params){
			// if we're calling getClassInfo and have a cls proto, but no params info, scan that cls for params now
			// and update the pointer in instanceClasses[className]. This happens when a widget appears in another
			// widget's template which still uses dojoType, but an instance of the widget appears prior with a data-dojo-type,
			// skipping this lookup the first time.
			c.params = getProtoInfo(c.cls.prototype, {});
		}
		
		return c;
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.

		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var attrName = (args.scope || d._scopeName) + "Type",	// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.
			var node, type, clsInfo, clazz, scripts, fastpath;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				fastpath = obj.fastpath;
				clsInfo = obj.clsInfo || (type && getClassInfo(type, fastpath));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes. no fastpath/data-dojo-type support here.
				node = obj;
				type = attrName in mixin ? mixin[attrName] : node.getAttribute(attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] :
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};
				
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				d._mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				d._mixin(params, obj.inherited);
			}
			
			// mix things found in data-dojo-props into the params
			if(fastpath){
				var extra = node.getAttribute(attrData + "props");
				if(extra && extra.length){
					try{
						extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
						d._mixin(params, extra);
					}catch(e){
						// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
						throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
					}
				}

				// For the benefit of _Templated, check if node has data-dojo-attach-point/data-dojo-attach-event
				// and mix those in as though they were parameters
				var attachPoint = node.getAttribute(attrData + "attach-point");
				if(attachPoint){
					params.dojoAttachPoint = attachPoint;
				}
				var attachEvent = node.getAttribute(attrData + "attach-event");
				if(attachEvent){
					params.dojoAttachEvent = attachEvent;
				}
				dojo.mixin(params, mixin);
			}else{
				// FIXME: we need something like "deprecateOnce()" to throw dojo.deprecation for something.
				// remove this logic in 2.0
				// read parameters (ie, attributes) specified on DOMNode

				var attributes = node.attributes;

				// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
				for(var name in clsInfo.params){
					var item = name in mixin ? { value:mixin[name], specified:true } : attributes.getNamedItem(name);
					if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
					var value = item.value;
					// Deal with IE quirks for 'class' and 'style'
					switch(name){
					case "class":
						value = "className" in mixin ? mixin.className : node.className;
						break;
					case "style":
						value = "style" in mixin ? mixin.style : (node.style && node.style.cssText); // FIXME: Opera?
					}
					var _type = clsInfo.params[name];
					if(typeof value == "string"){
						params[name] = str2obj(value, _type);
					}else{
						params[name] = value;
					}
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
				var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script, attrData);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			// FIXME: in 2.0, drop jsId support. use data-dojo-id instead
			var jsname = (node.getAttribute(attrData + "id") || node.getAttribute("jsId"));
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(rootNode, args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantitate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a
		//		`rootNode` member, that is used.
		//
		// args: Object
		//		a kwArgs object passed along to instantiate()
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._Templated`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dojo.byId(root) : dojo.body();
		args = args || {};

		var attrName = (args.scope || d._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode,			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				// TODO: what if this is a widget and dir/lang are declared in data-dojo-props?
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = (!parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser) || (args && args.template);

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					// FIXME: desupport dojoType in 2.0. use data-dojo-type instead
					var type, html5 = recurse && child.getAttribute(attrData + "type");
					if(html5){
						type = html5;
					}else{
						// fallback to backward compatible mode, using dojoType. remove in 2.0
						type = recurse && child.getAttribute(attrName);
					}
					
					var fastpath = html5 == type;

					if(type){
						// if dojoType/data-dojo-type specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							fastpath: fastpath,
							clsInfo: getClassInfo(type, fastpath), // note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Ignore bogus entries in inherited hash like {dir: ""}
		var inherited = {};
		if(args && args.inherited){
			for(var key in args.inherited){
				if(args.inherited[key]){ inherited[key] = args.inherited[key]; }
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root,
			inherited: inherited
		}, list);

		// go build the object instances
		var mixin = args && args.template ? {template: true} : null;
		return this.instantiate(list, mixin, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){
		if(dojo.config.parseOnLoad){
			dojo.parser.parse();
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.getObject("dijit.wai.onload") === dojo._loaders[0]){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");


/*=====
dojo.cache = {
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary:
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with dojoAttachEvent=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to attributeMap
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process dojoAttachPoint
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");;
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this._attachEvents.push(this.connect(baseNode, event, thisFunc));
						}
					}
				}

				// waiRole, waiState
				// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directicly
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			dojo.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];
			
			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.baseTextDirection.BidiTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.baseTextDirection.BidiTextBox"] = true;
/*
 * IBM Confidential
 *
 * IBM Rational Focal Point
 *
 * Copyright IBM Corporation 2010. All Rights Reserved.
 */

// Input widget that addresses dynamic Bidi complex expressions
// (e.g. URIs, emails).
//
// @author bidi_bdl_lkemmel
//

//------------------------------------------------------------------------------
// BidiTextBox class and commonly used bidi utils
//------------------------------------------------------------------------------

//==============================================================================
// CONSTANTS
//==============================================================================

dojo.provide("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox = {};

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF = navigator.userAgent.toLowerCase().indexOf("firefox") >= 0;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE = !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF && navigator.appName.toLowerCase().indexOf("microsoft") >= 0;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants = {};

// Direction constants
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR = 1;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR = 2;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR = 3;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM = "\u200e";
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM = "\u200f";

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE = '\u202A';
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE = '\u202B';

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF = '\u202C';

// Regex for a left-to-right character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR = new RegExp("[a-zA-Z" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + "]");

// Keep these distinct, if we would need to support some language specific feature
// Regex for a Hebrew character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR = new RegExp("[\\u05be\\u05c0\\u05c3\\u05c6\\u05d0-\\u05f4"
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + "\\ufb1d\\ufb1f-\\ufb28\\ufb2a-\\ufb4f]");

// Regex for an Arabic character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR = new RegExp("[\\u061b\\u0621-\\u064a\\u066d-\\u066f\\u0671-\\u06d5"
        + "\\u06dd\\u06e5\\u06e6\\u06ee\\u06ef\\u06fa-\\u070d\\u0710\\u0712-\\u072f"
        + "\\u074d-\\u07a5\\u07b1\\ufb50-\\ufd3d\\ufd50-\\ufdfc\\ufe70-\\ufefc]");

// Regex for control character (only address those we utilize)
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR = new RegExp("[" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE + "]", "m");

// Delimiting strings to create Bidi segments
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;

//==============================================================================
// com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory namespace
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory = {

//==============================================================================
// Extends the given element with the properties of BidiTextBox.
//==============================================================================
    subclass: function(id, direction, delimiters) {
        this.subclassElement(document.getElementById(id), direction, delimiters);
    },

    subclassElement: function(element, direction, delimiters) {
        if (element && "BidiTextBox" != element.subclass) {
            element = new com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox(element, direction, delimiters);
        }
    }
};

//==============================================================================
// BidiTextBox class
//==============================================================================

//==============================================================================
// BidiTextBox constructor
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox = function(element, direction, delimiters) {
    for (var property in com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype) {
        try {
            if (!element[property]) {
                element[property] = this[property];
            }
        } catch(e) {
            return element;
        }
    }
    element.delimiters = delimiters;

    if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR) {
        element.dir = "";
        element.isContextualDir = true;
        element.init();
    } else {
        if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR) {
            element.dir = "rtl";
        } else if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR) {
            element.dir = "ltr";
        }
        if (delimiters) {
            element.init();
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.push(element);
        }
    }
    return element;
}

// Static members
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.subclass = "BidiTextBox";

//==============================================================================
// Init BidiTextBox object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.init = function() {
    this.connectListeners();

    var value = this.normalizeContent(false);
    this.formatContent(value, true);
};

if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {

//==============================================================================
// Connect event listeners for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        // this.attachEvent("onfocus", this.onFocusIn);
        // this.attachEvent("onkeyup", this.onKeyUp);
        // this.attachEvent("oncut", this.onCut);
        // this.attachEvent("onpaste", this.onPaste);
        // this.attachEvent("ondrop", this.onPaste);
        // window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        // if (this.delimiters) {
            // this.attachEvent("onkeydown", this.onKeyDown);
            // this.attachEvent("oncopy", this.onCopy);
            // this.attachEvent("onmouseup", this.onMouseUp);
            // this.attachEvent("oncontextmenu", this.onContextMenu);
            // this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        // }
		
		this.attachEvent("onfocus", dojo.hitch(this, this.onFocusIn));
        this.attachEvent("onkeyup", dojo.hitch(this, this.onKeyUp));
        this.attachEvent("oncut", dojo.hitch(this, this.onCut));
        this.attachEvent("onpaste", dojo.hitch(this, this.onPaste));
        this.attachEvent("ondrop", dojo.hitch(this, this.onPaste));
        window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        if (this.delimiters) {
            this.attachEvent("onkeydown", dojo.hitch(this, this.onKeyDown));
            this.attachEvent("oncopy", dojo.hitch(this, this.onCopy));
            this.attachEvent("onmouseup", dojo.hitch(this, this.onMouseUp));
            this.attachEvent("oncontextmenu", dojo.hitch(this, this.onContextMenu));
            this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        }
    };

//==============================================================================
// Copy event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();

        var range = document.selection.createRange();
        if (range && range.text) {
            window.clipboardData.setData('Text', range.text.replace(/[\u200e\u200f]/g, ''));
            event = event || window.event;
            event.cancelBubble = true;
            event.returnValue = false;
            return false;
        }
    };

//==============================================================================
// Cut event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCut = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onCopy(event);

        var range = document.selection.createRange();
        if (range) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.hideText();
            range.text = "";
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };

//==============================================================================
// Paste event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onPaste = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.update(false);"
                + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();", 0);
    };

//==============================================================================
// Mouseup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onMouseUp = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    };

//==============================================================================
// Contextmenu event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onContextMenu = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem;
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = setInterval(
                "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem.update(false)", 1000);
    };

//==============================================================================
// Keydown event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        if (!event.ctrlKey && !event.altKey) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();
        }
    };
//==============================================================================
// Keyup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyUp = function(event) {
        if (this.subclass != "BidiTextBox") {
            if(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == null || 
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == undefined) {
        	var inputs = document.getElementsByTagName("input");
        	for(var i=0; i<inputs.length; i++) {
        	  var input = inputs[i];
        	    if(input.name == "reportName") {
        	        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = input;
        	        break;
    		     }
        	}
            }
            return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onKeyUp(event);
        }
        if (!event) {
            event = window.event;
        }
        if (!event.altKey) {
            if (!event.ctrlKey) {
                // 'Home', left arrow and right arrow key code values
                NAVIGATION_KEYS = [36, 37, 39];
                i = NAVIGATION_KEYS.length;
                while (i-- > 0) {
                    if (event.keyCode == NAVIGATION_KEYS[i]) {
                        this.getSafePosition();
                        return;
                    }
                }
            }
            this.update(false);
        }
    };

//==============================================================================
// Validates caret position after a key or mouse event occurs.
// The caret will not be placed between a Bidi formatting code and the
// associated character. To accomplish this, the caret may be moved 1 position
// forward or backwards, depending on navigation direction.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSafePosition = function() {
        if (!this.selection) {
            return;
        }
        var value = this.getValue();
        if (!value) {
            return;
        }
        var startMove, endMove;
        var prevSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (!this.selection) {
            return;
        }
        startMove = this.selection[0] - prevSelection[0];
        endMove = this.selection[1] - prevSelection[1];
    
        // First do the end. We may need to move both end and start (when selection
        // is collapsed), so have the start be updated also.
        if (startMove) {
            if (!this.isMovingOverControl(value, this.selection[0], startMove)) {
                startMove = 0;
            } else {
                startMove = startMove > 0 ? 1 : -1;
                do {
                    this.selection[0] = Math.max(0, this.selection[0] + startMove);
                } while (this.isMovingOverControl(value, this.selection[0], startMove));
                startMove = this.selection[0] - prevSelection[0];
            }
        }
        if (endMove && this.isMovingOverControl(value, this.selection[1], endMove)) {
                endMove = endMove > 0 ? 1 : -1;
            var textLen = value.length;
            do {
                this.selection[1] = Math.min(textLen, this.selection[1] + endMove);
            } while (this.isMovingOverControl(value, this.selection[1], endMove));
    
            if (this.selection[0] > this.selection[1]) {
                this.selection[0] = this.selection[1];
            }
            this.setSelection();
    
        } else if (startMove) {
            if (this.selection[0] > this.selection[1]) {
                this.selection[1] = this.selection[0];
                this.setSelection();
            } else {
                this.moveSelectionStart(startMove);
            }
        }
    };

//==============================================================================
// Checks if the caret is moving over a control character.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.isMovingOverControl = function(value, position, direction) {
        if (direction < 0) {
            return position - 1 >= 0
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position));
        }
        if (direction > 0) {
            return position + 1 <= this.getValue().length
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position - 1));
        }
        return false;
    };

//==============================================================================
// Moves selection start point, leaving the end intact.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.moveSelectionStart = function(value) {
        if (this.selection) {
            var range = document.selection.createRange();
            if (range) {
                range.moveStart('character', value);
                range.select();
            }
        }
    };

} else {

//==============================================================================
// Connect event listeners for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        this.addEventListener("focus", this.onFocusIn, false);
        this.addEventListener("change", this.update, true);
        this.addEventListener("input", this.update, true);
        this.addEventListener("drop", this.update, false);
		
		//If a div element
		if(this.localName == "div") {
			this.addEventListener("keyup", this.update, true);
		}
		
        window.addEventListener("unload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload, true);
        if (this.delimiters) {
            this.addEventListener("keydown", this.onKeyDown, true);

            this.setAttribute("oncopy", ';');
            if (typeof this["oncopy"] == 'function') {
                this.removeAttribute("oncopy");
                this.addEventListener("copy", this.onCopy, true);
            }
            this.setAttribute("oncut", ';');
            if (typeof this["oncut"] == 'function') {
                this.removeAttribute("oncut");
                this.addEventListener("cut", this.onCopy, true);
            }
            window.addEventListener("submit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit, false);
        }
    };

//==============================================================================
// Copy and cut event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        this.getSelection();
        if (this.selection && this.selectionStart != this.selectionEnd) {
            this.normalizeContent(true);

            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };
    
//==============================================================================
// Keydown event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        // older Firefox of version < 3
        if (event.ctrlKey && !event.shiftKey && !event.altKey
                && (event.keyCode == event.DOM_VK_C && typeof this["oncopy"] != 'function'
                || event.keyCode == event.DOM_VK_X && typeof this["oncut"] != 'function')) {
            this.onCopy(event);
        } else {
            this.getSelection();
        }
    };
}

//==============================================================================
// Sets global Bidi element to this object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onFocusIn = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == this
        || document.activeElement && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == document.activeElement) {
        return;
    }
    if (this.subclass != "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
        var target = event.srcElement || event.target;
        if (target && target.subclass == "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = target;
        }
    } else {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    }
};

//==============================================================================
// Inserts controls characters after delimiters and adjusts locally saved
// selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.formatContent = function(value, applyValue) {
    if (!value) {
        value = this.getValue();
    }
    if (this.isContextualDir) {
        this.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(0, value);
    }
    if (!this.delimiters) {
        return value;
    }
    if (this.dir == "ltr" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(value)
        || this.dir == "rtl" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(value)) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var CONTROL_STR = this.dir == "rtl" ? com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING
        : com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING;
    CONTROL_STR_LEN = CONTROL_STR.length;

    var newStr = new String(value);
    var i, j, c;
    var start = 0, end = -1;
    var afterStrong = false;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length - 1; i++, j++) {
        c = value.charAt(i);
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
            afterStrong = this.dir == "ltr";
        } else if (c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR)) {
            afterStrong = this.dir == "rtl";
        } else if (afterStrong && this.delimiters.indexOf(c) >= 0) {
            newStr = newStr.substr(0, j + 1) + CONTROL_STR + newStr.substr(j + 1);
            afterStrong = false;
            if (this.selection) {
                if (this.selection[0] == i) {
                    ++start;
                } else if (this.selection[0] > i) {
                    start += CONTROL_STR_LEN;
                }
                if (this.selection[1] == i) {
                    ++end;
                } else if (this.selection[1] > i) {
                    end += CONTROL_STR_LEN;
                }
            }
            j += CONTROL_STR_LEN;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.setValue(newStr, true);
    }
    this.unhideText();
    return newStr;
};

//==============================================================================
// Gets and caches the current native selection.
// Returns integer array containing the selection start and end points.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSelection = function() {
    if (typeof this.selectionStart == "number") {
        this.selection = [this.selectionStart, this.selectionEnd];
    } else if (document.selection) {
        try {
            var start = 0;
            var range = document.selection.createRange().duplicate();
            var range2 = range.duplicate();
            var rangeLength = range.text.length;
            
            if (this.isMultiLine) {
                range2.moveToElementText(this);
            } else {
                range2.expand('textedit');
            }
            // Return value of range#moveStart doesn't seem reliable, so
            // also limit number of iterations.
            // Otherwise we can fall into infinite loop.
            var n = range2.text.length;
            while (start < n && range.compareEndPoints('StartToStart', range2) > 0) {
                if (range.text.charAt(0).match(/[\r\n]/)) {
                    if (!range.moveStart("word", -1)) {
                        break;
                    }
                } else if (!range.moveStart('character', -1)) {
                    break;
                }
                ++start;
            }
            this.selection = [start, start + rangeLength];
        } catch (e) {
            this.selection = null;
        }
    }
};

//==============================================================================
// Gets text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getValue = function() {
	return this.value || this.text || this.textContent || this.innerHTML || "";
};

//==============================================================================
// Form submit event handler.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.handleSubmit = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    }
    this.resetSelection();
    this.normalizeContent(true);
};

//==============================================================================
// Sets text color to the background color making that way the text invisible.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.hideText = function() {
    if (typeof this.foreground == 'undefined') {
        this.foreground = this.style.color;
    }
    this.style.color = this.style.backgroundColor || "white";
};

//==============================================================================
// Removes controls characters and adjusts locally saved selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.normalizeContent = function(applyValue) {
    var value = this.getValue();
    if (!value || !this.delimiters) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var newStr = new String(value);
    var i, j;
    var start = 0, end = -1;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length; i++) {
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(i))) {
            newStr = newStr.substr(0, j) + newStr.substr(j + 1);
            if (this.selection) {
                if (this.selection[0] > i) {
                    --start;
                }
                if (this.selection[1] > i) {
                    --end;
                }
            }
        } else {
            ++j;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.hideText();
        this.setValue(newStr, true);
    }
    return newStr;
};

//==============================================================================
// Resets our private selection object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.resetSelection = function() {
    this.selection = null;
};

//==============================================================================
// Sets the native selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setSelection = function() {
    if (!this.selection || this.readOnly) {
        return;
    }
    if (this.setSelectionRange) {
        if (this.selectionStart != this.selection[0] || this.selectionEnd != this.selection[1]) {
            this.focus();
            this.setSelectionRange(this.selection[0], this.selection[1]);
        }
    } else if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        var newSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (newSelection[0] != this.selection[0] || newSelection[1] != this.selection[1]) {
            var range = this.createTextRange() || document.body.createTextRange();
            if (range) {
                if (this.isMultiLine) {
                    range.moveToElementText(item);
                } else {
                    range.expand('textedit');
                }
                range.collapse(true);
                range.moveEnd('character', newSelection[1]);
                range.moveStart('character', newSelection[0]);
                range.select();
            }
        }
    }
};

//==============================================================================
// Sets the text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setValue = function(newValue, setSelection) {
    if (!newValue) {
        newValue = this.newValue;
    }
    if (newValue != this.getValue()) {
        if (this.text) {
            this.text = newValue;
        } else if (this.value) {
            this.value = newValue;
        }
        if (setSelection) {
            this.setSelection();
        }
    }
};

//==============================================================================
// Resets element's text color.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.unhideText = function() {
    this.style.color = this.foreground;
};

//==============================================================================
// Updates the text value though normalizing and subsequent formatting.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.update = function(delay) {
        // On load FF fires focus on body element, and as a result
        // BidiTextBox.gCurrentItem can be uninitialized.
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    this.getSelection();
	
    var origValue = this.getValue();
    this.newValue = this.formatContent(this.normalizeContent(false), false);
	
    if (this.newValue != origValue) {
        this.hideText();
        if (delay == true) {
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.setValue(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.unhideText();"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        } else {
            this.setValue(this.newValue, true);
            this.unhideText();
            this.resetSelection();
        }
    }
};

//==============================================================================
// Handle submit events in this window.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit = function(event) {
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.length; i-- > 0; ) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes[i].handleSubmit(event);
    }
}

//==============================================================================
// Handle window unload.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload = function(event) {
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
}

//==============================================================================
// Stops item update invocation.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate = function() {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval) {
        clearInterval(BidiTextBox.gInterval);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
    }
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR.test(text)
            || com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains LTR characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar = function(c) {
    return c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR);
}

//==============================================================================
// Returns direction resolved to LTR or RTL.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection = function(dir, value) {
    if (dir) {
        return dir;
    }
    if (value) {
        for (var i = 0, n = value.length; i < n; i++) {
            var c = value.charAt(i);
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
                return "rtl";
            }
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(c)) {
                return "ltr";
            }
        }
    }
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2);
}

//==============================================================================
// Extends text controls with Bidi properties and applies base text direction
// for both dynamic and static elements.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleBidi = function() {
    //var visibility = document.body.style.visibility;
    //document.body.style.visibility = 'hidden';
    var nodes = document.getElementsByTagName('input');
    for (i = nodes.length; i-- > 0;) {
        if (/^text/.test(nodes[i].type)) {
            if (/bidi_url/.test(nodes[i].className)) {
                if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
                    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(nodes[i],
                        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR, ".:@\\/");
                }
            } else {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
            }
        }
    }
    nodes = document.getElementsByTagName('textarea');
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
    }
    nodes = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements();
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(nodes[i]);
    }
    //document.body.style.visibility = visibility;
}

//==============================================================================
// Applies base text direction for dynamic element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection = function(node) {
    if (node && !node.dir) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir = getBidiDir(0);
        if (bidiTextDir) {
            node.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir;
        } else if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
        }
    }
}

//==============================================================================
// Applies base text direction for static element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection = function(node) {
    if (!node || /bidi_url/.test(node.className)) {
        return;
    }
    if (node.nodeType == '3' && /[^\s]+/.test(node.nodeValue)) {
        var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(0), node.nodeValue);
        if (dir == 'ltr') {
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'rtl') {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "ltrRightAlign");
            } else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM);
            }
        } else {
        	if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'ltr') {
        		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "rtlLeftAlign");
        	} else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM);
            }
        }
    } else {
        for (var i = node.childNodes.length; i-- > 0;) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(node.childNodes[i]);
        }
    }
}

// Code added to handle the case of English then Arabic then ( in LTR
// Or Arabic then English then ( in RTL
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses = function(node, markup) {
	node.nodeValue = node.nodeValue.replace(/\(/g, markup + '('); 
}

//==============================================================================
// Applies base text direction for a text node.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection = function(node, className) {
    if (node) {
        if (node.nodeValue) {
            node.nodeValue = node.nodeValue.replace(/[\u00a0]$/gm, "");
        }
        node = node.parentNode;
        if (node && node.tagName && node.tagName.toUpperCase() != 'SCRIPT') {
            // When there is no block-level element parented to this node,
            // use immediate parent anyway
            var parent = node;
            while (node && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle(node, 'display') == 'inline') {
                node = node.parentNode; 
            }
            if (node) {
                parent = node;
            }            
            if (!/ltrRightAlign|rtlLeftAlign/.test(parent.className)) {
                parent.className += (parent.className ? " " : "") + className;
            }
        }
    }
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle = function(node, style) {
    var gcs = "getComputedStyle" in window;
    return (gcs ? window.getComputedStyle(node, null) : node.currentStyle).getPropertyValue(style); 
}


com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES = {
	bidiText: 'bidiText'
 };
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES = ['TR', 'TD', 'a', 'h1'];

//==============================================================================
// Retreives document static elements enabled for bidi.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements = function() {
    var bidiNodes = [];
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES.length; i-- > 0;) {
        var nodes = document.getElementsByTagName(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES[i]);
        for (var j = nodes.length; j-- > 0;) {
            var classNames = nodes[j].className.split(' ');
            for (var k = classNames.length; k-- > 0;) {
                if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES[classNames[k]]) {
                    bidiNodes.push(nodes[j]);
                    break;
                }
            }
        }
    }
    return bidiNodes;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir = function(which) {
        
	// which = 0, to get the text direction explicitly defined by the user (LTR or RTL or default)
	// If user setting is contextual, we return nothing... And the JS will check for buffer content
	// If the text direction can not be determined from the buffer content
	// The JS will call which=2, to get the default direction for user language preference
	// 
	// As we'll always use contextual mode, we always return an empty string when which=0
	if (which == 0) return '';

			
	// which = 2, to return the default direction for user language preference
	// Currently we do not support RTL languages, so this will always return "ltr"
	if (which == 2) return 'ltr';
}

/**
 * ============================================================================================================================================
 * BELOW ARE ALL EDITS CONTRIBUTED AFTER THE PORT OF THIS FILE TO EVEREST
 * This includes pretty much all of the changes which we had to make to get the file working in 
 * our environment, as well as some small snippets which help consolidate actions.
 * ============================================================================================================================================
 */ 

/**
 * This method accepts text as a parameter and determines the proper BTD depending on content
 * and user setting
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD = function(text) {
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	
	if(btdpref != "ctx") {
		return btdpref;
	} else {
		if(typeof text != "undefined" && text != "") {
			var calcbtd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection("", text);
			if(calcbtd == "") {
				//something went wrong
				return "default";
			} else {
				return calcbtd;
			}
		} else {
			//should pass in a valid string, but if no string, obviously doesn't matter.
			return "default";
		}
	}
}

/**
 * This method takes in a node, and sets the appropriate BTD value to the "dir" attribute, depending
 * on the user preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD = function(node) {

	if(typeof node == "undefined") {
		return;
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(node.innerHTML);
	if(btdpref != "default") {
		node.setAttribute("dir", btdpref);
        if(btdpref == "rtl") {
            var text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(node.innerHTML);
			if (node.innerText) {
				node.innerText = text;
			} else {
				node.innerHTML = text;
			}
        }
	}
}

/**
 * This method takes in text, and returns the appropriate BTD
 * value for the "dir" attribute, depending on the user
 * preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD = function(text) {

    var dir = "";
	if(text == "") {
		return dir;
	}	
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
	if(btdpref != "default") {
		dir = btdpref;
	}
    return dir;
}

/**
 * This method abstracts the process of setting an input elements BTD based on the user preference
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD = function(node) {
	
	if(typeof node == "undefined" || node == null) {
		return;
	}
	
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	if(btdpref == "ctx") {
		//contextual requires bidiTextBox to be used or auto if WebKit browsers
		if(!com.ibm.bpm.social.util.common.Util.isChrome) {
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
		} else {
			//Webkit supports the "auto" dir attribute
			node.dir = "auto";
		}
	} else if (btdpref != "default") {
		//either ltr or rtl
		node.dir = btdpref;
	}
}

/**
 * This method inputs text and adds Unicode Control Characters
 * (UCC) based on the user preference.  Returns the text with
 * UCC added.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText = function(text) {
    var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
    if(dir != "default") {
        //set the appropriate control character
        var controlChar = (dir == "ltr") ? 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

        //wrap the html in the control characters
        text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;

        if (dir == "rtl") {
            text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(text);
        }
    }
    return text;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis = function(text) {
    var newTxt = text;
            //fixParenthesis, fix colon
    newTxt = newTxt.replace(/\(/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + '('); 
    newTxt = newTxt.replace(/\:/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + ':'); 

    return newTxt;
}


}

if(!dojo._hasResource["com.ibm.bpm.social.Aether"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.Aether"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.Aether");






dojo.declare(
	"com.ibm.bpm.social.Aether",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	
    templateString:"<div class='social-portal-notification-instance' dojoAttachEvent='onmouseover:_doMouseOver, onmouseout:_doMouseOut, onclick:_doClick'>\n\t<div class = 'social-portal-notification-window' dojoAttachPoint ='window'>\n\t\t<div class='social-portal-notification-close' dojoAttachPoint='closeIcon' dojoAttachEvent='onmouseover:_doIconMouseOver,onmouseout:_doIconMouseOut,onclick:_doIconClick'>&times;</div>\n\t\t<table>\n\t\t\t<tr>\n\t\t\t\t<td class='social-portal-notification-icon-align'>\n\t\t\t\t\t<div class='social-portal-notification-icon' dojoAttachPoint='icon'></div>\n\t\t\t\t</td>\n\t\t\t\t<td>\n\t\t\t\t\t<div class='social-portal-notification-message' dojoAttachPoint='message'>${message}</div>\n\t\t\t\t</td>\n\t\t\t</tr>\n\t\t</table>\n\t\t\n\t</div>\n</div>\n",
	
	containerDivId : "aether",
	
    message: null,
	
	img_src: dojo.moduleUrl("dojo", "resources/blank.gif"),

	eventParam: null,

    timeout: 5000,
	
    timer: null,
		
	_container: null,
	
	positionOptions: ['top-left', 'top-center', 'top-right', 'bottom-left', 'bottom-center', 'bottom-right'],
	
	position: null,
	
	width: null,
	
	fade: true,
	
	displayCloseIcon: false,
	
	constructor: function(params){
		
		dojo.mixin(this, params);
		
		if(!this.position) this.position = 'center';
		
		this.createContainer();
			
	},
	
	createContainer: function(){
		if(dojo.byId(this.containerDivId)){
			this.container = dojo.byId(this.containerDivId);
			this.container.className = 'social-portal-notification '+this.position;
		} else {
			this.container = dojo.create('div', {
				'id': this.containerDivId,
				'class': 'social-portal-notification '+this.position
			}, dojo.body());
		}
	},
	
	createIconImage:function(){
		var image = document.createElement('img');
		image.src = this.img_src;
		dojo.addClass(image, "bpm-social-generic-system-avatar");
		this.icon.appendChild(image);
		
		if(this.eventParam){
			try{
				if(this.eventParam.process.instance.comment.author.id){
					dojo.addClass(image, "bpm-social-default-avatar-medium");
					dojo.removeClass(image, "bpm-social-generic-system-avatar");
					image.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this.eventParam.process.instance.comment.author.id);
				}
			} catch(e){}

		}

	},
	
    postCreate: function() {
		dojo.style(this.domNode, 'opacity', 0);
		dojo.place(this.domNode, this.container);
		dojo.anim(this.domNode, {opacity: 1}, 250);
		
		if(this.width){
			dojo.style(this.domNode, 'width', this.width + 'px');
		}

		if(this.displayCloseIcon===false){
			dojo.style(this.closeIcon, 'display', 'none');
		}
		
		if(this.img_src)	{
			this.createIconImage();
		}
		
		if(this.fade===true){
			this.setTimer();
		}
	},
	
	setTimer: function() {
		this.timer = setTimeout(dojo.hitch(this, 'close'), this.timeout);
	},
	
	_doMouseOver: function(e){
		clearInterval(this.timer);
		dojo.addClass(this.domNode, 'hover');
	}, 
	
	_doMouseOut:function(e){
		dojo.removeClass(this.domNode, 'hover');
		this.setTimer();
	},
	
	_doClick: function(e){
		/*
		if(this.displayCloseIcon===false){
			//Only close the notification on a message click if the close icon is not displayed
			this.close();
		}
		*/
	},

	_doIconMouseOver: function(e){
		dojo.addClass(this.closeIcon, 'hover');		
	},
	
	_doIconMouseOut: function(e){
		dojo.removeClass(this.closeIcon, 'hover');
	},
	
	_doIconClick: function(e){
		this.close();
	},
	
	fireEvent:function(){
		//	parent defined
		console.debug("This function needs to be implemented by the parent.");
	},

	_handleActionClick:function(event){		
		this.fireEvent(event.target.id,this.eventParam);
	},

	close: function(){
		dojo.anim(this.domNode,{opacity: 0}, 500, null, dojo.hitch(this, 'remove'));
	},
	
	remove: function(){
		dojo.anim(this.domNode,{height: 0, margin: 0}, 250, null, dojo.partial(dojo.destroy, this.domNode));
		//TODO Look at destroying container if last notification
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.Log"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.Log"] = true;
// Log.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

/**
 * These functions provide a logging facility that may determine the calling function
 * 
 * Usage:
 * 
 * To log/trace a string/message:
 * <li> com.ibm.bpm.social.util.common.Log.log(1,"x is larger ...")
 * 
 * To trace the entry of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logEntry(1)
 * 
 * To trace the exit of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logExit(1)
 * 
 * To log objects:
 * <li> com.ibm.bpm.social.util.common.Log.logObject(integer pLogLevel, Object pObject, [String pTitle]);
 * 
 * Note: Third parameter is optional for example: com.ibm.bpm.social.util.common.Log.logObject(2, domNode,
 * "This is node hugo:");
 * 
 * 
 * Using the logError function, you may log an error to the console, and let the Log class determine the call
 * stack automatically
 * 
 * 
 * Output of the log may be adjusted/filtered using
 * <ul>
 * <li> com.ibm.bpm.social.util.common.Log.logLevel
 * <li> com.ibm.bpm.social.util.common.Log.includeClasses
 * <li> com.ibm.bpm.social.util.common.Log.excludeClasses
 * <li> com.ibm.bpm.social.util.common.Log.logArguments
 * 
 * 
 * Note about auto-resolving class names <br>
 * -------------------------------------
 * 
 * If you see the message "Couldn't resolve class name, might be within a constructor or invoked by
 * dojo.hitch" you should use the optional parameters cname while invoking the log functions. Example:
 * 
 * com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksContainer");
 * 
 * 
 * Cases where you need this: - while a class is constructed (logging within constructor, postCreate) - while
 * logging in functions that were called using apply, call, dojo.hitch
 * 
 */


dojo.provide("com.ibm.bpm.social.util.common.Log");



/**
 * Logging level to provide some granularity/filtering Values:
 * <ul>
 * <li> 0 - No logging
 * <li> 1 - Log function entries and exits
 * <li> 2 - Additional data
 * <li> 3,.. - Much more data
 * </ul>
 */
// set 0 to just to initialize, will be set later according to the various settings, 
// see com.ibm.bpm.social.util.common.Log.determinLogContent()
com.ibm.bpm.social.util.common.Log.logLevel = 0;

/**
 * You may include only a subset in the output using includeClasses or excludeClasses
 * <p> - classes may be specified as regExp string
 * <p> - example using includeClasses : includeClasses = ["Selector", "com.ibm.bpm.social.util.common"]
 * <p> - to include all, just use ["*"]
 * <p> - exclude overrules include
 * <p> - to unset exclude or include use [] <p
 */

com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
com.ibm.bpm.social.util.common.Log.excludeClasses = [];


/**
 * Using this flag you can enable/disable the logEntry function to print out the arguments of the function
 */
com.ibm.bpm.social.util.common.Log.logArguments = true;


/**
 * Use this flag to also get info about the time when the log was written
 */
com.ibm.bpm.social.util.common.Log.logTime = false;

/**
 * Use this flag to also log also the date when the log was written
 */

com.ibm.bpm.social.util.common.Log.logDate = false;

/**
 * Use this flag to indent on each function call (when logEntry is called) and un-indent when leaving the
 * function (when logExit is called)
 * <p>
 * This works only if browser supports the console.group functionality
 */
/*@ignore@*/
com.ibm.bpm.social.util.common.Log.indentFunctions = (typeof console.group == "function" && !dojo.isIE);
/*@end@*/

com.ibm.bpm.social.util.common.Log.logEntriesAndExits = false;

/**
 * Determine the details of what and how to log
 * <ul>
 * <li> logLevel : (0 - 10) how much will be logged
 * <li> indentFunctions : (true|false) shall we track function calls ?
 * <li> logTime: (true|false) prefix log statements with a timestamp
 * <li> logDate: (true|false) prefix log statements with a timestamp including date
 * <li> logArguments : (true|false) print out the arguments of the function
 * <li> includeClasses : comma-separated string of class names to be included in the log
 * <li> excludeClasses : comma-separated string of class names to be excluded from the log
 * </ul>
 * 
 * @type {void}
 */
com.ibm.bpm.social.util.common.Log.determinLogContent =
  function(){

    if (com.ibm.bpm.social.debug.isLogging) { // allowClientSideLogging under the Mashups_ConfigService == true
      // logging to the Portal log console requested, provide as much details as possible
      com.ibm.bpm.social.util.common.Log.logLevel = 10;
      com.ibm.bpm.social.util.common.Log.logArguments = true;
      com.ibm.bpm.social.util.common.Log.logTime = true;
      com.ibm.bpm.social.util.common.Log.logDate = true;
      com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
      com.ibm.bpm.social.util.common.Log.excludeClasses = [];
      com.ibm.bpm.social.util.common.Log.indentFunctions = false;
    }
    else if ((typeof dojoConfig != "undefined") && dojoConfig.isDebug || (typeof djConfig != "undefined") &&
      djConfig.isDebug) {
      // typical developers setup, just use defaults
      com.ibm.bpm.social.util.common.Log.logLevel = 2;
      com.ibm.bpm.social.util.common.Log.logEntriesAndExits = true;
    }

    // check for query string settings 
    var paramString = window.location.search.replace("?", "");
    var paramKey = "";
    var paramValue = "";
    var value = "";
    var params = paramString.split("&");

    for ( var i = 0; i < params.length; i++) {
      paramKey = params[i].split("=")[0];
      paramValue = params[i].split("=")[1];

      if (paramKey && paramValue) {

        // logLevel
        if (paramKey == "logLevel") {
          value = Number(paramValue);
          if (!isNaN(value)) {
            com.ibm.bpm.social.util.common.Log.logLevel = value;
          }
        }
        // logArgument
        else if (paramKey == "logArguments") {
          com.ibm.bpm.social.util.common.Log.logArguments = (paramValue == "true");
        }
        // logDate
        else if (paramKey == "logDate") {
          com.ibm.bpm.social.util.common.Log.logTime = (paramValue == "true");
        }
        // logTime
        else if (paramKey == "logTime") {
          com.ibm.bpm.social.util.common.Log.logDate = (paramValue == "true");
        }
        // indentFunctions
        else if (paramKey == "indentFunctions") {
          com.ibm.bpm.social.util.common.Log.indentFunctions = (paramValue == "true");
        }
        // includeClasses
        else if (paramKey == "includeClasses") {
          com.ibm.bpm.social.util.common.Log.includeClasses = paramValue.split(",");
        }
        // excludeClasses
        else if (paramKey == "excludeClasses") {
          com.ibm.bpm.social.util.common.Log.excludeClasses = paramValue.split(",");
        }
        else if (paramKey == "logEntriesAndExits") {
          com.ibm.bpm.social.util.common.Log.logEntriesAndExits = (paramValue == "true");
        }
      }
    }


    // disable logging at all if no one requested it
    if (!com.ibm.bpm.social.debug.isLogging && !dojoConfig.isDebug && !djConfig.isDebug &&
      com.ibm.bpm.social.util.common.Log.logLevel === 0) {
      var emptyFunction = function(){};
      com.ibm.bpm.social.util.common.Log.log = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logEntry = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logExit = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logError = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logObject = emptyFunction;
    }

    return;
  };


/**
 * Write a message to the console
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.log =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
      if (!pMessage) {
        pMessage = "";
      }

      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      com.ibm.bpm.social.util.common.Log.consoleLog(scope + " " + pMessage);
    }
  };

/**
 * Trace the entry of a method/function <br>
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */

com.ibm.bpm.social.util.common.Log.logEntry =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope =
        com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName, com.ibm.bpm.social.util.common.Log.logArguments);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroup(scope + " entry " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " entry " + pMessage);
      }
    }
  };


/**
 * Trace the exit of a method/function
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.logExit =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroupEnd(scope + " exit " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " exit " + pMessage);
      }
    }
  };


/**
 * Helper to determine if the message should be traced
 * 
 * @param {String} cname the class name, may be null
 * @param {String} fktName the function of the class, may be null
 * @param {String} includeArgs indicate if the args should be traced
 * @return boolean
 */
com.ibm.bpm.social.util.common.Log._shouldTrace =
  function(cname, fktName, includeArgs){

    var originFunction = arguments.callee.caller.caller;

    if ((cname == null || typeof (cname) == "undefined") || (fktName == null || typeof (fktName) == "undefined")) {
      // Note: the following cannot extracted to a utility function, since the arguments object will change
      var cls = null;

      if (originFunction._containingClass) {
        cname = originFunction._containingClass;
      }

      // works not for constructors
      if (originFunction.ctor && originFunction.ctor.prototype) {
        cls = originFunction.ctor.prototype.declaredClass;
      }
      if (cls) {
        var packages = cls.split(".");
        cname = packages[packages.length - 1];
      }
      else if (originFunction.name && (originFunction.name.lastIndexOf("_") > 0)) {
        var l = originFunction.name.split("_");
        // fktName = l.slice(0,l.length-1).join(_);
        cname = l[l.length - 1];
      }

      cname = cname || "<undefined>";

      if (originFunction.nom) {
        fktName = originFunction.nom;
      }
      else if (originFunction.name) {
        fktName = originFunction.name;
        fktName = fktName.replace("_" + cname, "");
      }

      fktName = fktName || "<undefined>";

      if (cname == "<undefined>" && fktName != "<undefined>" && typeof originFunction.toString == "function") {
        // in IE this is the only known way to get the class name
        // returns the whole function
        var originFunctionToString = originFunction.toString();
        // look if the function name is contained (if it was declared as function methodeName_className() it is)
        var iFktName = originFunctionToString.indexOf(fktName);
        if (iFktName != -1) {
          // find the first ( - that's directly after the class name
          var iFirstOpeningBracket = originFunctionToString.indexOf("(");
          // check that the function name does not come after the the first (, would mean it is contained anywhere in the function body
          if (iFirstOpeningBracket != -1 && iFirstOpeningBracket > iFktName) {
            // extract the class name
            cname = originFunctionToString.substring(iFktName + fktName.length + 1, iFirstOpeningBracket);
          }
        }
      }

      if (cname == "<undefined>") {
        com.ibm.bpm.social.util.common.Log
          .consoleWarn("Couldn't resolve classname, might be within a constructor or invoked by dojo.hitch. See Log.js to learn how to fix.");
      }
      else {
        originFunction._containingClass = cname;
      }
    }

    var include = com.ibm.bpm.social.util.common.Log.includeClasses;
    var exclude = com.ibm.bpm.social.util.common.Log.excludeClasses;
    var rc = cname + "." + fktName + "(";
    if (includeArgs === true && originFunction.arguments) {
      rc += com.ibm.bpm.social.util.common.Log._argumentsToString(originFunction.arguments);
    }
    rc += ")";


    if (include.length === 0 && exclude.length === 0) {
      return null;
    }

    for ( var i = 0; i < exclude.length; i++) {
      if (cname.search(exclude[i]) > -1) {
        return null;
      }
    }

    if (include[0] == "*") {
      return rc;
    }

    for (i = 0; i < include.length; i++) {
      if (cname.search(include[i]) > -1) {
        return rc;
      }
    }

    return false;
  };

/**
 * converts an array of arguments to a string
 * 
 * @param {String} args the arguments
 * @return {String} the formatted arguments
 * 
 */
com.ibm.bpm.social.util.common.Log._argumentsToString = function(/*Array*/args, /*int*/counter){
  var stringArgs = [];
  counter = counter || 0;
  if (args) {
    for ( var i = 0; i < args.length; i++) {
      if (args[i] === null) {
        stringArgs.push("null");
      }
      else if (typeof args[i] == "undefined") {
        stringArgs.push("undefined");
      }
      else if (typeof args[i] == "function") {
        stringArgs.push("[function]");
      }
      else if (typeof args[i] == "string" || typeof args[i] == "number" || typeof args[i] == "boolean") {
        // dojo.toJson automatically replaces line breaks with \n in strings and converts numbers and booleans to a string
        stringArgs.push(dojo.toJson(args[i]));
      }
      else if (typeof args[i] == "object") {
        if (dojo.isArrayLike(args[i])) {
          // to prevent recursion as arrays might reference themselve or contain cross references we have an counter with an exit condition
          if (counter < 3) {
            stringArgs.push("[" + com.ibm.bpm.social.util.common.Log._argumentsToString(args[i], counter + 1) + "]");
          }
          else {
            stringArgs.push("[array]");
          }
        }
        else if (typeof args[i].toString == "function") {
          stringArgs.push(args[i].toString());
        }
        else {
          stringArgs.push("[object]");
        }
      }
      else {
        stringArgs.push("[unknown]");
      }
    }
  }
  return stringArgs.join(", ");
};

/**
 * Log an error to the console, and let the Log class determine the call stack
 * 
 * @param {Number} pLogLevel
 * @param {String} pMethod Method or Message
 * @param {Object} xcp optional exception object
 */

com.ibm.bpm.social.util.common.Log.logError = function(/*integer*/pLogLevel, /*String*/pMethod, /*Object*/xcp){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length < 1) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logError"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logError"]));
    }

    var pMessage = pMethod ? pMethod : "";

    if (xcp) {
      pMessage = pMessage + " " + xcp.name + ":" + xcp.message;
      if (xcp.stack) {
        pMessage = pMessage + "\nStack:";
        var stack = xcp.stack.split("\n");
        var out = "";
        for ( var i = 0; i < stack.length; i++) {
          out = stack[i];
          if (out.length > 200) {
            // this is a eval'ed file instead of a function, which we don't wan to see in the console
            var file1 = out.match(/(\w*\.js)/) ? out.match(/(\w*\.js)/)[0] : "";
            var file2 = out.match(/@http.*$/) ? out.match(/@http.*$/)[0] : "";

            out = (file1 ? file1 : "");
            out += (file2 ? ":" + file2 : "");
            if (!out) {
              out = "undefined";
            }
          }
          pMessage = pMessage + "\n" + out;
        }
      }
    }
    com.ibm.bpm.social.util.common.Log.consoleError(pMessage);
  }
};

com.ibm.bpm.social.util.common.Log.logObject = function(/* integer */pLogLevel, /* Object */pObject, /* String */pTitle){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length != 2 && arguments.length != 3) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logObject"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logObject"]));
    }

    if (pTitle) {
      com.ibm.bpm.social.util.common.Log.consoleDebug(pTitle);
    }
    /*@ignore@*/
    console.dir(pObject);
    /*@end@*/
  }
};

com.ibm.bpm.social.util.common.Log._getTS = function(){
  var d = new Date();
  var rc = "";
  var millis = d.getMilliseconds();
  millis = (millis.length == 2 ? "0" + millis : millis);
  if (com.ibm.bpm.social.util.common.Log.logDate) {
    rc = d.getFullYear() + "-" + ((d.getMonth() < 9) ? "0" : "") + (d.getMonth() + 1) + "-";
    rc = rc + ((d.getDate() < 9) ? "0" : "") + d.getDate() + " ";
  }
  rc = rc + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds() + "." + d.getMilliseconds() + " ";
  return rc;
};

// wrapper which might be used to handle browser issues

com.ibm.bpm.social.util.common.Log.consoleLog = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroup = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.group) {
    console.group(message);
  }
  else {
    console.log(message);
  }
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroupEnd = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.groupEnd) {
    console.groupEnd();
  }
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleDebug = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.debug(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleError = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.error(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleWarn = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.warn(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleInfo = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.info(message);
  /*@end@*/
};


/*
 * Do not translate
 */
com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR =
  "Invalid number of parameters for com.ibm.bpm.social.util.common.Log.${0}";
com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a number";
com.ibm.bpm.social.util.common.Log.PARM_NO_STRING_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a string";
com.ibm.bpm.social.util.common.Log.PARM_NO_OBJECT_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a object";

// determine the log content to be displayed
com.ibm.bpm.social.util.common.Log.determinLogContent();

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation"] = true;
// SecureInnerHtmlPreparation.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation");



dojo
  .declare(
    "com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation",
    null,
    {

      /*static*/_cache: {},

      originalString: null,
      parsed: false,
      parsedString: null,

      constructor: function(originalString){
        com.ibm.bpm.social.util.common.Log.logEntry(2, null, "SecureInnerHtmlPreparation", "constructor");
        this.originalString = originalString;
        com.ibm.bpm.social.util.common.Log.logExit(2, null, "SecureInnerHtmlPreparation", "constructor");
      },

      /**
       * creates a secure inner HTML
       * 
       * @returns the secure inner HTML
       */
      getSecureString: function getSecureString_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.originalString === null) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning null.");
          return null;
        }
        else if (this._cache[this.originalString]) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning cached value: " + this._cache[this.originalString]);
          return this._cache[this.originalString];
        }
        else if (!this.parsed) {
          this._parse();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "Returning: " + this.parsedString);
        return this.parsedString;
      },

      _parse: function _parse_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.parsedString = "";

        // first step: tokenize the original string
        var tokens = [];

        var currentToken = null;
        var position = 0;

        while (position < this.originalString.length) {
          switch (this.originalString.charAt(position)) {
            case '<' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LT,
                "text": "<"
              };
              break;
            case '>' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_GT,
                "text": ">"
              };
              break;
            case '/' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SLASH,
                "text": "/"
              };
              break;
            case '\n' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case '\r' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK) {
                  tokens.push(currentToken);
                }
                else {
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case ' ' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_SPACE) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += " ";
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SPACE,
                "text": " "
              };
              break;
            default:
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_TEXT) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += this.originalString.charAt(position);
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_TEXT,
                "text": this.originalString.charAt(position)
              };
              break;
          }

          position++;
        }
        if (currentToken !== null) {
          tokens.push(currentToken);
        }

        // Generate output
        this.parsedString = "";
        var containsHTML = false;
        var openTags = [];
        var tagName;

        for ( var i = 0; i < tokens.length; i++) {
          var token = tokens[i];
          var nextToken1 = i + 1 < tokens.length ? tokens[i + 1] : null;
          var nextToken2 = i + 2 < tokens.length ? tokens[i + 2] : null;
          var nextToken3 = i + 3 < tokens.length ? tokens[i + 3] : null;

          if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT) {
            // is there an allowed tag starting
            if (nextToken1 &&
              nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_TEXT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken1.text.toLowerCase()) > -1 &&
              nextToken2 &&
              (nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SPACE ||
                nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SLASH || nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_GT) &&
              this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2) > -1) {

              tagName = nextToken1.text.toLowerCase();

              var nextSlash = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_SLASH, i + 2);
              var nextGt = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2);

              if (nextSlash == -1 || nextGt < nextSlash) {
                containsHTML = true;
                this.parsedString += "<" + tagName + ">";
                i = nextGt;

                if (dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS, tagName) == -1) {
                  openTags.push(tagName);
                }
              }
              else if (nextSlash + 1 == nextGt) {
                containsHTML = true;
                this.parsedString += "<" + tagName + "/>";
                i = nextGt;
              }
              else {
                this.parsedString += "&lt;";
              }
            }
            // is there an allowed tag that is open ending
            else if (nextToken1 && nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_SLASH && nextToken2 &&
              nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_TEXT && nextToken3 &&
              nextToken3.type == com.ibm.bpm.social.util.common.TOKEN_GT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken2.text.toLowerCase()) > -1 &&
              dojo.indexOf(openTags, nextToken2.text.toLowerCase()) > -1) {

              tagName = nextToken2.text.toLowerCase();

              // ensure to close all open tags that were opened after the corresponding opening tag to create valid HTML
              var closingTagName;
              do {
                closingTagName = openTags.pop();

                this.parsedString += "</" + closingTagName + ">";
              }
              while (closingTagName != tagName);

              i += 3;
            }
            else {
              this.parsedString += "&lt;";
            }
          }
          else if (token.type == com.ibm.bpm.social.util.common.TOKEN_GT) {
            this.parsedString += "&gt;";
          }
          else {
            this.parsedString += token.text;
          } // end if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT)
        } // end for ( var i = 0; i < tokens.length; i++)

        // close all opened tags that were not closed to create valid HTML
        while (openTags.length > 0) {
          tagName = openTags.pop();
          this.parsedString += "</" + tagName + ">";
        }

        // if there is no HTML at all replace \n with <br>
        if (!containsHTML) {
          this.parsedString = this.parsedString.replace(com.ibm.bpm.social.util.common.REGEXP_LINEBREAK, "<br>");
        }

        this.parsed = true;

        this._cache[this.originalString] = this.parsedString;

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _getNextTokenPosition: function(/*Token[]*/tokens, /*int*/type, /*int*/offset){
        for ( var i = offset; i < tokens.length; i++) {
          if (tokens[i].type == type) {
            return i;
          }
        }

        return -1;
      }
    });

com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS = ["abbr",
                                                   "acronym",
                                                   "b",
                                                   "big",
                                                   "br",
                                                   "cite",
                                                   "code",
                                                   "dfn",
                                                   "em",
                                                   "h1",
                                                   "h2",
                                                   "h3",
                                                   "h4",
                                                   "h5",
                                                   "h6",
                                                   "hr",
                                                   "i",
                                                   "kbd",
                                                   "li",
                                                   "ol",
                                                   "p",
                                                   "s",
                                                   "samp",
                                                   "small",
                                                   "strike",
                                                   "strong",
                                                   "sub",
                                                   "sup",
                                                   "tt",
                                                   "u",
                                                   "ul",
                                                   "var"];

com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS = ["br",
                                                          "hr"];

com.ibm.bpm.social.util.common.REGEXP_LINEBREAK = new RegExp("\n", "g");

com.ibm.bpm.social.util.common.TOKEN_GT = 0; // >
com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK = 1; // \n
com.ibm.bpm.social.util.common.TOKEN_LT = 2; // <
com.ibm.bpm.social.util.common.TOKEN_SLASH = 3; // /
com.ibm.bpm.social.util.common.TOKEN_SPACE = 4;
com.ibm.bpm.social.util.common.TOKEN_TEXT = 5;

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ErrorDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ErrorDialog"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");






dojo.declare("com.ibm.bpm.social.widgets.dialogs.ErrorDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString: "<div><div class='bpm-social-dialog-box'>"
    + "<div dojoAttachPoint='message' class='bpm-social-dialog-message'></div>"
    + "<div dojoAttachPoint='detailsTitle' style='display:inline'></div>"
    + "<div dojoAttachPoint='detailsContent' class='bpm-social-dialog-details-content'></div>" + "</div>"
    + "<div dojoAttachPoint='buttonContainer' class='bspace-dialog-buttonContainer'></div>"
    + "<div style='clear: both'></div>" + "</div>",

  _message: "",
  _label: null,
  _details: null,

  /**
   * Creates a popup which will show a message.
   * 
   * @param message The message to be shown
   * @param details The details to be shown
   * @param label The label of the confirmation button
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    com.ibm.bpm.social.util.common.Log.logEntry();
    if (typeof params.message === 'string') {
      this._message = params.message;
    }
    if (typeof params.label === 'string') {
      this._label = params.label;
    }
    if (typeof params.details === 'string') {
      this._details = params.details;
    }
    com.ibm.bpm.social.util.common.Log.logExit();
  },

  postCreate: function postCreate_ErrorDialog(){
    this.message.innerHTML = this._message;
    var self = this;
    if (this._label != null) {
      this.addButton(this.id + "_confirmationButton", this._label, function(){
        self.hide();
      });
    }

    // If details exist, create a title pane for it.
    if (this._details) {
      var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

      var detailsTitlePic = document.createElement("div");
      detailsTitlePic.innerHTML = "+";
      dojo.addClass(detailsTitlePic, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitlePic);

      var detailsTitleLabel = document.createElement("div");
      detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
      dojo.addClass(detailsTitleLabel, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitleLabel);

      this.detailsTitle.tabIndex = "0";

      dojo.style(this.detailsContent, "display", "none");
      this.detailsContent.innerHTML = this._details;

      var detailsHidden = true;
      this.connect(this.detailsTitle, "ondijitclick", function(e){
        if (detailsHidden) {
          dojo.style(this.detailsContent, "display", "block");
          detailsHidden = false;
          detailsTitlePic.innerHTML = "-";
          detailsTitleLabel.innerHTML = bundle["dialog_details_hide"];
        }
        else {
          dojo.style(this.detailsContent, "display", "none");
          detailsHidden = true;
          detailsTitlePic.innerHTML = "+";
          detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
        }
      });
    }
    else {
      dojo.style(this.detailsTitle, "display", "none");
      dojo.style(this.detailsContent, "display", "none");
    }

    this.connect(this.popup, 'onHide', "_finish");
    this.inherited("postCreate", arguments);
  },

  _finish: function _finish_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    this.onClose();
    // Basically this is also a dijit._Widget with a meaningful destroy method.#
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
    com.ibm.bpm.social.util.common.Log.logEntry();
  },

  /**
   * Event handling hook.
   */
  onClose: function onClose_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    com.ibm.bpm.social.util.common.Log.logExit();
  }

});

/**
 * Shows an info popup. For example in case an ad hoc event has been started but no immediate task is
 * available.
 * 
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional} If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["info_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-info"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };


/**
 * Creates a modal warning popup with the provided message. <br />
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["warning_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-warning"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };

/**
 * Creates a modal error popup with the provided message. <br />
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["error_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-error"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };

}

if(!dojo._hasResource["dojox.date.hebrew.numerals"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.numerals"] = true;
dojo.provide("dojox.date.hebrew.numerals");

//Conversion from "Hindi" numerals to Hebrew numerals and vice versa

(function(){

	var DIG="אבגדהוזחט";
	var	TEN="יכלמנסעפצ";
	var	HUN="קרשת";

	var transformChars = function(str, nogrsh){
		str = str.replace("יה", "טו").replace("יו", "טז");

		if(!nogrsh){
			var len = str.length;
			if(len > 1){
				str = str.substr(0, len - 1) + '"' + str.charAt(len - 1);
			}else{
				str += "\u05F3"; // 05F3:geresh
			}
		}
		return str; // String
	};
	 
	var parseStrToNumber = function(str){
		var num = 0;
		dojo.forEach(str, function(ch){
			var i;
			if((i = DIG.indexOf(ch)) != -1){
				num += ++i;
			}else if((i = TEN.indexOf(ch)) != -1){
				num += 10 * ++i;
			}else if((i = HUN.indexOf(ch)) != -1){
				num += 100 * ++i;
			}
		});
		return num; //Number
	};
	 
	var convertNumberToStr = function(num){
		var str  = "", n = 4, j = 9;
  		while(num){
			if(num >= n*100){
				str += HUN.charAt(n-1);
				num -= n*100;
				continue;
			}else if(n > 1){
				n--;
				continue;
			}else if(num >= j*10){
				str += TEN.charAt(j-1);
				num -= j*10;
			}else if(j > 1){
				j--;
				continue;
			}else if(num > 0){
				str += DIG.charAt(num-1);
				num = 0;
			}
		}
		return str; //String
	};

	dojox.date.hebrew.numerals.getYearHebrewLetters = function(/*Number */ year){
		// summary: converts the year from an integer to Hebrew numerals.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getYearHebrewLetters(date1.getFullYear());
		
		var rem = year % 1000;
		//FIXME: tests include dates outside this range and seem to pass.
		//	    	if((year - rem) / 1000 != 5){ throw new Error("Hebrew year "+year+" is not in range 5001-5999");}
		return transformChars(convertNumberToStr(rem)); // String
	};
	
	dojox.date.hebrew.numerals.parseYearHebrewLetters  = function(/*String hebrew year*/ year){
		// summary: converts the year written in Hebrew numerals to an integer
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |        	date.setFullYear(dojox.date.hebrew.numerals.parseYearHebrewLetters('\u05ea\u05e9\u05e1\u05f4\u05d7'));

		return parseStrToNumber(year) + 5000; // int
	};
	
	dojox.date.hebrew.numerals.getDayHebrewLetters =  function(day, /*boolean?*/ nogrsh){
		// summary: converts an integer to a String representing the number in Hebrew numerals.   Can be formatted with or without geresh &#x05f3;
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getDayHebrewLetters(date1.getDay());

		return transformChars(convertNumberToStr(day), nogrsh); // String
	};
	
	dojox.date.hebrew.numerals.parseDayHebrewLetters =  function(/*String hebrew*/ day){
		// summary: converts the string containing a Hebrew numeral to an integer
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		date1.setDate(dojox.date.hebrew.numerals.parseDayHebrewLetters('\u05d0')); // ALEPH
		return parseStrToNumber(day); // int
	};

	dojox.date.hebrew.numerals.getMonthHebrewLetters =  function(/*int*/month){
		// summary: converts an integer representing a  month to a String written in Hebrew numerals
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getMonthHebrewLetters(date1.getMonth());

		return transformChars(convertNumberToStr(month+1)); // String
	};

	dojox.date.hebrew.numerals.parseMonthHebrewLetters = function(/*String*/monthStr){
		// summary: converts a Hebrew numeral string representing
		//	a month to an integer.  The returned value
		//	is indexed in the month name array.  To use it for
		//	setMonth, do correction for leap year
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |            var number = dojox.date.hebrew.numerals.parseMonthHebrewLetters("\u05ea\u05de\u05d5\u05d6"); // Tammuz
		// |		date.setMonth(number);
			
		//month number from 0 to 12
		var monnum = dojox.date.hebrew.numerals.parseDayHebrewLetters(monthStr) - 1;

		if(monnum == -1 || monnum > 12){
			throw new Error("The month name is incorrect , month = " + monnum);
		}
		return monnum;
	};
})();

}

if(!dojo._hasResource["dojox.date.hebrew.Date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.Date"] = true;
dojo.provide("dojox.date.hebrew.Date");



dojo.declare("dojox.date.hebrew.Date", null, {
	// summary: A Date-like object which implements the Hebrew calendar
	//
	// description:
	//	A Date-like object which implements the Hebrew Calendar.  Because this object
	//	implements many of the same methods as the native JavaScript Date object, which
	//	implements the Gregorian calendar, it can often be used its place.  Note that
	//	this object does not extend Date or use its prototype.
	//
	// example:
	// |	
	// |
	// |	var date = new dojox.date.hebrew.Date();
	// |	console.log(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());

	// Hebrew date calculations are performed in terms of days, hours, and
	// "parts" (or halakim), which are 1/1080 of an hour, or 3 1/3 seconds.
	//_HOUR_PARTS: 1080,
	//_DAY_PARTS: 24*1080,
   
	// An approximate value for the length of a lunar month.
	// It is used to calculate the approximate year and month of a given
	// absolute date.
	//_MONTH_FRACT: 12*1080 + 793,
	//_MONTH_PARTS: 29*24*1080 + 12*1080 + 793,
	    
	// The time of the new moon (in parts) on 1 Tishri, year 1 (the epoch)
	// counting from noon on the day before.  BAHARAD is an abbreviation of
	// Bet (Monday), Hey (5 hours from sunset), Resh-Daled (204).
	//_BAHARAD: 11*1080 + 204,

	// The Julian day of the Gregorian epoch, that is, January 1, 1 on the
	// Gregorian calendar.
	//_JAN_1_1_JULIAN_DAY: 1721426,

	/**
	* The lengths of the Hebrew months.  This is complicated, because there
	* are three different types of years, or six if you count leap years.
	* Due to the rules for postponing the start of the year to avoid having
	* certain holidays fall on the sabbath, the year can end up being three
	* different lengths, called "deficient", "normal", and "complete".
	*/
	
	//"Absolute" indexes of  months:  Tishri -  0, Heshvan - 1, Kislev - 2, Tevet - 3, Shevat -  4, Adar I (leap years only) - 5, Adar - 6, Nisan - 7, Iyar - 8, Sivan - 9,   Tammuz-10, Av - 11, Elul - 12.

	_MONTH_LENGTH:  [
		// Deficient  Normal     Complete
		[   30,	    30,	    30	],		 //Tishri    0
		[   29,	    29,	    30	],		 //Heshvan   1
		[   29,	    30,	    30	],		 //Kislev   2
		[   29,	    29,	    29	],		 //Tevet  3
		[   30,	    30,	    30	],		 //Shevat  4
		[   30,	    30,	    30	],		 //Adar I (leap years only)  5
		[   29,	    29,	    29	],		 //Adar  6
		[   30,	    30,	    30	],		 //Nisan  7
		[   29,	    29,	    29	],		 //Iyar  8
		[   30,	    30,	    30	],		 //Sivan  9
		[   29,	    29,	    29	],		 //Tammuz  10
		[   30,	    30,	    30	],		 //Av  11
		[   29,	    29,	    29	]		 //Elul  12
	],

	/**
	* The cumulative # of days to the end of each month in a non-leap year
	* Although this can be calculated from the MONTH_LENGTH table,
	* keeping it around separately makes some calculations a lot faster
	*/
	_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  147,	   148,	   149  ],		// (Adar I placeholder)
		[  176,	   177,	   178  ],		// Adar
		[  206,	   207,	   208  ],		// Nisan
		[  235,	   236,	   237  ],		// Iyar
		[  265,	   266,	   267  ],		// Sivan
		[  294,	   295,	   296  ],		// Tammuz
		[  324,	   325,	   326  ],		// Av
		[  353,	   354,	   355  ]		// Elul
	],

	/**
	* The cumulative # of days to the end of each month in a leap year
	*/
	_LEAP_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  177,	   178,	   179  ],		// Adar I
		[  206,	   207,	   208  ],		// Adar II
		[  236,	   237,	   238  ],		// Nisan
		[  265,	   266,	   267  ],		// Iyar
		[  295,	   296,	   297  ],		// Sivan
		[  324,	   325,	   326  ],		// Tammuz
		[  354,	   355,	   356  ],		// Av
		[  383,	   384,	   385  ]		// Elul
	],
	
	_GREGORIAN_MONTH_COUNT:  [
		//len len2   st  st2
		[  31,  31,   0,   0 ], // Jan
		[  28,  29,  31,  31 ], // Feb
		[  31,  31,  59,  60 ], // Mar
		[  30,  30,  90,  91 ], // Apr
		[  31,  31, 120, 121 ], // May
		[  30,  30, 151, 152 ], // Jun
		[  31,  31, 181, 182 ], // Jul
		[  31,  31, 212, 213 ], // Aug
		[  30,  30, 243, 244 ], // Sep
		[  31,  31, 273, 274 ], // Oct
		[  30,  30, 304, 305 ], // Nov
		[  31,  31, 334, 335 ] // Dec
		// len  length of month
		// len2 length of month in a leap year
		// st   days in year before start of month
		// st2  days in year before month in leap year
	],

    _date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,

 	constructor: function(){
		// summary: initialize the date object value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		var date2 = new dojox.date.hebrew.Date(date1);
		// |
		// |		var date3 = new dojox.date.hebrew.Date(5768,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._year = this._month = this._date = this._hours = this._minutes = this._seconds = this._milliseconds = NaN;
			}else{  // this is hebrew.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >= 3){
			// YYYY, MM, DD arguments passed, month is from 0-12,  "absolute" index of month
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			
			if(this._month > 12){
				console.warn("the month is incorrect , set 0  " + this._month + "   " + this._year );
				this._month = 0;
			}
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
  
		this._setDay();
	},
	
	getDate: function(){
		// summary: returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return this._date; // int
	},

	getDateLocalized: function(/*String?*/locale){
		// summary: returns the date value as hebrew numerals for the Hebrew locale,
		//		a number for all others.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return (locale || dojo.locale).match(/^he(?:-.+)?$/) ?
			dojox.date.hebrew.numerals.getDayHebrewLetters(this._date) : this.getDate();
	},

	getMonth: function(){
		// summary: returns the month value (0 - 12)
		//
		// description: the result is the index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		// For non leap years, for months after Shevat, the actual position of
		// the month in the year (used for short format) is less than
		// the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getMonth()+1);
		// |		>> 7

		return this._month;
	},

	getFullYear: function(){
		// summary: returns the Year value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getFullYear());
		// |		>> 5769
		return this._year;
	},
			
	getHours: function(){
 		//summary: returns the hour value
		return this._hours;
	},
		
	getMinutes: function(){
		//summary: returns the minutes value

		return this._minutes;
	},

	getSeconds: function(){
		//summary: returns the seconds value
		return this._seconds;
	},

	getMilliseconds: function(){
		//summary: returns the milliseconds value

		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: sets the date number for a given month
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		date1.setDate(2);

		date = +date;
		var mdays;
		if(date>0){
			while (date > (mdays = this.getDaysInHebrewMonth(this._month, this._year))){
				date -= mdays;
				this._month++;
				if(this._month >= 13){this._year++; this._month -= 13;}
			}
		}else{
			while(date<=0){
				mdays = this.getDaysInHebrewMonth((this._month-1)>=0 ? (this._month-1) : 12, ((this._month-1)>=0)? this._year : this._year-1);
				this._month--;
				if(this._month < 0){this._year--; this._month += 13;}
				date += mdays;
			}
		}
		this._date = date;
		this._setDay();
		return this;
	},
	

	setFullYear: function(/*number*/year, /*number?*/month, /*number?*/ date){
		// summary: set the year
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setFullYear(5768);
		// |		date1.setFullYear(5768, 1, 1);
		
		this._year = year = +year;
		if(!this.isLeapYear(year) && this._month==5){  //incorrect month number for non leap year
			this._month++;
		}
		
		if(month !== undefined){this.setMonth(month);}
		if(date !== undefined){this.setDate(date);}
		
		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setMonth: function(/*number*/month){
		// summary: sets the month.  You should use "absolute" index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		//  For non leap years, for months after Shevat, the actual position of
		//	the month in the year (used for short format) is less than
		//	the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setMonth(0); //first month

		month = +month; // coerce to a Number
		if(!this.isLeapYear(this._year) && month == 5){month++;}
	
		if(month>=0){
			while(month >12){
				this._year++;
				month -= 13;
				if (!this.isLeapYear(this._year) && month >= 5){month++;}
			}
		}else{
			while(month<0){
				this._year--;
				month += (!this.isLeapYear(this._year)  &&  month < -7) ? 12 : 13;
			}
		}
		
		this._month = month;

		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setHours: function(){
		//	summary: sets the hour
		//
		//	description: Sets the hour and optionally minutes, seconds, milliseconds also.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setHours(12, 30, 0, 0);

		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours += +arguments[0];
		}

		if(hours_arg_no >= 2){
			this._minutes += +arguments[1];
		}

		if(hours_arg_no >= 3){
			this._seconds += +arguments[2];
		}

		if(hours_arg_no == 4){
			this._milliseconds += +arguments[3];
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInHebrewMonth(this._month, this._year);
			if(this._date > mdays)
			{
				this._month++;
				if(!this.isLeapYear(this._year) && this._month==5){ this._month++; }
				if(this._month >= 13){this._year++; this._month -= 13;}
				this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
		this._setDay();
		return this;
	},

	setMinutes: function(/*Number*/minutes){
		//summary: sets the minutes (0-59)
		minutes = +minutes;
		this._minutes = minutes % 60;
		this.setHours(parseInt(minutes / 60));
		this._setDay();
		return this;
	},

	setSeconds: function(/*Number*/seconds){
		//summary: sets the seconds (0-59)

		seconds = +seconds;
		this._seconds = seconds % 60;
		this.setMinutes(parseInt(seconds / 60));
		this._setDay();
		return this;
	},

	setMilliseconds: function(/*Number*/milliseconds){
		//summary: sets the milliseconds

		milliseconds = +milliseconds;
		this._milliseconds = milliseconds % 1000;
		this.setSeconds(parseInt(milliseconds / 1000));
		this._setDay();
		return this;
	},

	_setDay: function(){
		var day = this._startOfYear(this._year);
		if(this._month != 0){
			day += (this.isLeapYear(this._year) ? this._LEAP_MONTH_START : this._MONTH_START)[this._month || 0][this._yearType(this._year)];
		}
		day += this._date - 1;
		this._day = (day+1) % 7;
	},

	toString: function(){
		// summary: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format
		//
		// description: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format (all numeric)
		//	For user presentation, use dojox.date.hebrew.locale.format which will present in the appropriate language
		//  and format.  toString() language- and culturally-specific conventions to keep this module free of
		//	dependencies on dojox.date.locale and dojo.cldr.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		console.log(date1.toString());
		// |		>>> "1, 6, 5769 0:0:0"
		return this._date + ", " + this._month + ", " + this._year + "  " + this._hours + ":" + this._minutes + ":" + this._seconds; // String
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInHebrewMonth: function(/*Number*/month, /*Number*/ year){
		// summary: returns the number of days in the given month and year

		// Aside from the leap month, these two months can vary: 1=HESHVAN, 2=KISLEV
		// The rest are a fixed length
		var yearType = (month == 1 || month == 2) ? this._yearType(year) : 0;
		return (!this.isLeapYear(this._year) && month == 5) ? 0 : this._MONTH_LENGTH[month][yearType];
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearType: function(/*Number*/year){
		var yearLength = this._handleGetYearLength(Number(year));
		if(yearLength > 380){
			yearLength -= 30;        // Subtract length of leap month.
		}

		var yearType = yearLength - 353;
		if (yearType < 0 || yearType > 2){
			throw new Error("Illegal year length " + yearLength + " in year " + year);
		}
		return yearType;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_handleGetYearLength: function(/*number*/eyear){
		return this._startOfYear(eyear+1) - this._startOfYear(eyear);
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_startOfYear: function(/*number*/year){
		var months = Math.floor((235 * year - 234) / 19),	// # of months before year
			frac = months * (12*1080 + 793) + 11*1080 + 204/*BAHARAD*/,	// Fractional part of day #
			day  = months * 29 + Math.floor(frac / (24*1080));	// Whole # part of calculation
		frac %= 24*1080;	// Time of day

		var wd = day % 7;	// Day of week (0 == Monday)

		if(wd == 2 || wd == 4 || wd == 6){
			// If the 1st is on Sun, Wed, or Fri, postpone to the next day
			day += 1;
			wd = day % 7;
		}
		if(wd == 1 && frac > 15 * 1080 + 204 && !this.isLeapYear(year)){
			// If the new moon falls after 3:11:20am (15h204p from the previous noon)
			// on a Tuesday and it is not a leap year, postpone by 2 days.
			// This prevents 356-day years.
			day += 2;
		}else if(wd == 0 && frac > 21 * 1080 + 589 && this.isLeapYear(year-1)){
			// If the new moon falls after 9:32:43 1/3am (21h589p from yesterday noon)
			// on a Monday and *last* year was a leap year, postpone by 1 day.
			// Prevents 382-day years.
			day += 1;
		}

		return day;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	isLeapYear: function(/*Number*/year){
	//	summary:
	//		Determines if the year (argument) is a leap year
	//	description: The Leap year contains additional month adar sheni
	//
		//return (year * 12 + 17) % 19 >= 12;
		var x = (year*12 + 17) % 19;
		return x >= ((x < 0) ? -7 : 12);
	},

	
	fromGregorian: function(/*Date*/gdate){
		// summary: This function sets this Date to the Hebrew Date corresponding to the Gregorian Date
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateHebrew.fromGregorian(dateGregorian);

		var result = (!isNaN(gdate)) ? this._computeHebrewFields(gdate) : NaN;
		this._year = (!isNaN(gdate)) ? result[0] : NaN;
		this._month = (!isNaN(gdate))? result[1] : NaN;
		this._date = (!isNaN(gdate)) ? result[2] : NaN;
		this._hours = gdate.getHours();
		this._milliseconds = gdate.getMilliseconds();
		this._minutes = gdate.getMinutes();
		this._seconds = gdate.getSeconds();
		if (!isNaN(gdate)) this._setDay();
		return this;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar.handleComputeFields from ICU4J v3.6.1 at http://www.icu-project.org/
	_computeHebrewFields: function(/*Date*/gdate){
		var julianDay = this._getJulianDayFromGregorianDate(gdate),
			d = julianDay - 347997,
			m = Math.floor((d * 24*1080) / (29*24*1080 + 12*1080 + 793)),       // Months (approx)
			year = Math.floor((19 * m + 234) / 235) + 1,  // Years (approx)
			ys  = this._startOfYear(year),                 // 1st day of year
			dayOfYear = (d - ys);
		// Because of the postponement rules, it's possible to guess wrong.  Fix it.
		while(dayOfYear < 1){
			year--;
			ys  = this._startOfYear(year);
			dayOfYear = d - ys;
		}

		// Now figure out which month we're in, and the date within that month

		var typeofYear = this._yearType(year),
			monthStart = this.isLeapYear(year) ? this._LEAP_MONTH_START : this._MONTH_START,
			month = 0;

		while(dayOfYear > monthStart[month][typeofYear]){
			month++;
		}
		month--;
		var dayOfMonth = dayOfYear - monthStart[month][typeofYear];
		return [year, month, dayOfMonth];
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianFields from ICU4J v3.6.1 at http://www.icu-project.org/
	toGregorian: function(){
		// summary: returns the equivalent Grogorian date value as a native Date object
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date(5768,11,20);
		// |		var dateGregorian = dateHebrew.toGregorian();
		
		var hYear = this._year || 0,
			hMonth = this._month || 0,
			hDate = this._date || 0,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		var julianDay =  (hDate + day + 347997),
		// The Gregorian epoch day is zero for Monday January 1, year 1.
			gregorianEpochDay = julianDay - 1721426;

		// Here we convert from the day number to the multiple radix
		// representation.  We use 400-year, 100-year, and 4-year cycles.
		// For example, the 4-year cycle has 4 years + 1 leap day; giving
		// 1461 == 365*4 + 1 days.
		var rem = [];
		var n400 = this._floorDivide(gregorianEpochDay , 146097, rem), // 400-year cycle length
			n100 = this._floorDivide(rem[0] , 36524, rem), // 100-year cycle length
			n4 = this._floorDivide(rem[0] , 1461, rem), // 4-year cycle length
			n1 = this._floorDivide(rem[0] , 365, rem),
			year = 400*n400 + 100*n100 + 4*n4 + n1,
			dayOfYear = rem[0]; // zero-based day of year

		if(n100 == 4 || n1 == 4){
			dayOfYear = 365; // Dec 31 at end of 4- or 400-yr cycle
		}else{
			++year;
		}

		var isLeap = !(year%4) && // equiv. to (year%4 == 0)
				(year%100 || !(year%400)),
			correction = 0,
			march1 = isLeap ? 60 : 59; // zero-based DOY for March 1
		if(dayOfYear >= march1){ correction = isLeap ? 1 : 2; }
		var month = Math.floor((12 * (dayOfYear + correction) + 6) / 367); // zero-based month
		var dayOfMonth = dayOfYear -
				this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2] + 1; // one-based DOM

		return new Date(year, month, dayOfMonth, this._hours, this._minutes, this._seconds, this._milliseconds); // Date
	},
	_floorDivide: function(numerator, denominator, remainder){
		if(numerator >= 0){
			remainder[0] = (numerator % denominator);
			return Math.floor(numerator / denominator);
		}
		var quotient = Math.floor(numerator / denominator);
		remainder[0] = numerator - (quotient * denominator);
		return quotient;
	},

	getDay: function(){
		// summary: returns weekday value (0 - 6)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDay());

		var hYear = this._year,
			hMonth = this._month,
			hDate = this._date,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		day += hDate - 1;
		return (day+1) % 7;
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianMonthStart from ICU4J v3.6.1 at http://www.icu-project.org/
	_getJulianDayFromGregorianDate: function(gdate){
		//summary: returns the Julian day of a Gregorian date

		var year = gdate.getFullYear(),
			month = gdate.getMonth(),
			d = gdate.getDate(),
			isLeap = !(year%4) && (year%100 || !(year%400)), //TODO: dup
			y = year - 1;
		// This computation is actually ... + (_JAN_1_1_JULIAN_DAY - 3) + 2.
		// Add 2 because Gregorian calendar starts 2 days after Julian
		// calendar.
		var julianDay = 365*y + Math.floor(y/4) - Math.floor(y/100) +
			Math.floor(y/400) + 1721426 - 1;
		// At this point julianDay indicates the day BEFORE the first day
		// of January 1, <eyear> of the Gregorian calendar.
		if(month > 0) {
			julianDay += this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2];
		}
		
		julianDay += d;
		return julianDay;
	}
});

dojox.date.hebrew.Date.prototype.valueOf = function(){
	return this.toGregorian().valueOf();
};

}

if(!dojo._hasResource["dojox.date.hebrew"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew"] = true;
dojo.provide("dojox.date.hebrew");


 // for compare
	
// Utility methods to do arithmetic calculations with hebrew.Dates

	// added for compat to date
dojox.date.hebrew.getDaysInMonth = function(/*hebrew.Date*/month){
	return month.getDaysInHebrewMonth(month.getMonth(), month.getFullYear());
};

//TODO: define hebrew.isLeapYear?  Or should it be invalid, since it has different meaning?

dojox.date.hebrew.compare = function(/*hebrew.Date*/dateheb1, /*hebrew.Date*/dateheb2, /*String?*/portion){
	//	summary:
	//		Compare two hebrew date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		hebrew.Date object
	//	date2:
	//		hebrew.Date object.  If not specified, the current hebrew.Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	if(dateheb1 instanceof dojox.date.hebrew.Date){
		dateheb1 = dateheb1.toGregorian();
	}
	if(dateheb2 instanceof dojox.date.hebrew.Date){
		dateheb2 = dateheb2.toGregorian();
	}
	
	return dojo.date.compare.apply(null, arguments);
};


dojox.date.hebrew.add = function(/*dojox.date.hebrew.Date*/date, /*String*/interval, /*int*/amount){
	//	based on and similar to dojo.date.add
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: hebrew.Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var newHebrDate = new dojox.date.hebrew.Date(date);

	switch(interval){
		case "day":
			newHebrDate.setDate(date.getDate() + amount);
			break;
		case "weekday":
			var day = date.getDay();
			var remdays = 0;
			if(amount < 0 && day == 6){ day = 5; remdays = -1; }
			
			if((day + amount) < 5 && (day + amount) >= 0){ //in the same week
				 newHebrDate.setDate(date.getDate() + amount + remdays);
			}else{
				var add = (amount > 0) ? 5 : -1;
				var adddays = (amount > 0) ? 2 : -2 ; /*first weekend */
				if  (amount > 0 && (day == 5 || day == 6)) { remdays =  4 - day; day = 4;}
				var newamount  =  day + amount - add;
				var weeks = parseInt(newamount / 5);
				var newday = newamount%5;
				newHebrDate.setDate(date.getDate() - day+ adddays + weeks * 7 + remdays + newday + add);
			}
			break;
		case "year":
			newHebrDate.setFullYear(date.getFullYear() + amount);
			break;
		case "week":
			amount *= 7;
			newHebrDate.setDate(date.getDate() + amount);
			break;
		case "month":
			var month = date.getMonth();
			var add = month + amount;
			if(!date.isLeapYear(date.getFullYear())){
				if(month < 5 && add >= 5){ add++;}
				else if (month > 5 && add <= 5){ add--;}
			}
			newHebrDate.setMonth(add);
			break;
		case "hour":
			newHebrDate.setHours(date.getHours() + amount);
			break;
		case "minute":
			newHebrDate.setMinutes(date.getMinutes() + amount);
			break;
		case "second":
			newHebrDate.setSeconds(date.getSeconds() + amount);
			break;
		case "millisecond":
			newHebrDate.setMilliseconds(date.getMilliseconds() + amount);
			break;
	}

	return newHebrDate; // dojox.date.hebrew.Date
};

dojox.date.hebrew.difference = function(/*dojox.date.hebrew.Date*/date1, /*dojox.date.hebrew.Date?*/date2, /*String?*/interval){
	//	based on and similar to dojo.date.difference
	//	summary:
	//        date1 - date2
	//	 date2 is hebrew.Date object.  If not specified, the current hebrew.Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond",  "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new dojox.date.hebrew.Date();
	interval = interval || "day";
	var yearDiff = date1.getFullYear() - date2.getFullYear();
	var delta = 1; // Integer return value
	switch(interval){
		case "weekday":
			var days = Math.round(dojox.date.hebrew.difference(date1, date2, "day"));
			var weeks = parseInt(dojox.date.hebrew.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date2.getDay();
				var bDay = date1.getDay();
	
				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new dojox.date.hebrew.Date(date2);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();
	
				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = -1;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = -1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = 0;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 1;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = 2;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			var startdate =  (date1.toGregorian() > date2.toGregorian()) ? date1 : date2; // more
			var enddate = (date1.toGregorian() > date2.toGregorian()) ? date2 : date1;
			
			var month1 = startdate.getMonth();
			var month2 = enddate.getMonth();
			
			if(yearDiff == 0){
				delta = ( !date1.isLeapYear(date1.getFullYear())  && startdate.getMonth() > 5 && enddate.getMonth() <=5) ? (startdate.getMonth() - enddate.getMonth() - 1) :
						(startdate.getMonth() - enddate.getMonth() );
			}else{
				delta = (!enddate.isLeapYear(enddate.getFullYear()) &&  month2 < 6) ? (13-month2-1) : (13-month2);
				delta +=  (!startdate.isLeapYear(startdate.getFullYear()) &&  month1 > 5) ? (month1 -1): month1;
				var i = enddate.getFullYear()  + 1;
				var e = startdate.getFullYear();
				for (i;   i < e;  i++){
					delta += enddate.isLeapYear(i) ? 13 : 12;
				}
			}
			if(date1.toGregorian() < date2.toGregorian()){
				delta = -delta;
			}
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojox.date.hebrew.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date1.toGregorian().getTime()- date2.toGregorian().getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojox.date.hebrew.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.locale"] = true;
dojo.provide("dojox.date.hebrew.locale");









//Load the bundles containing localization information for
// names and formats


(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, locale, fullYear,  pattern){

		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad;
			var c = match.charAt(0);
			var l = match.length;
			var widthList = ["abbr", "wide", "narrow"];
			
			switch(c){
				case 'y':
					if(locale.match(/^he(?:-.+)?$/)){
						s = dojox.date.hebrew.numerals.getYearHebrewLetters(dateObject.getFullYear());
					}else{
						s = String(dateObject.getFullYear());
					}
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						if(!dateObject.isLeapYear(dateObject.getFullYear()) && m>5){m--;}
						if(locale.match(/^he(?:-.+)?$/)){
							s = dojox.date.hebrew.numerals.getMonthHebrewLetters(m);
						}else{
							s = m+1; pad = true;
						}
					}else{
						var monthNames = dojox.date.hebrew.locale.getNames('months',widthList[l-3], 'format', locale, dateObject);
						s = monthNames[m];
					}
					break;
				case 'd':
					if(locale.match(/^he(?:-.+)?$/)){
						s =  dateObject.getDateLocalized(locale);
					}else{
						s = dateObject.getDate(); pad = true;
					}
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'z':
					s = "";
					break;
				default:
					throw new Error("dojox.date.hebrew.locale.formatPattern: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}
	
dojox.date.hebrew.locale.format = function(/*hebrew.Date*/dateObject, /*object?*/options){
	// based on and similar to dojo.date.locale.format
	//summary:
	//		Format a Date object as a String, using  settings.
	//
	// description:
	//		Create a string from a hebrew.Date object using a known pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Default formatting lengths is 'short'
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojox.date.hebrew.locale._getHebrewBundle(locale);
	var str = [];

	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle, locale, options.fullYear);
	if(options.selector == "year"){
		var year = dateObject.getFullYear();
		return locale.match(/^he(?:-.+)?$/) ?
			dojox.date.hebrew.numerals.getYearHebrewLetters(year) : year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time

	return result; // String
};

dojox.date.hebrew.locale.regexp = function(/*object?*/options){
	//	based on and similar to dojo.date.locale.regexp
	// summary:
	//		Builds the regular needed to parse a hebrew.Date

	return dojox.date.hebrew.locale._parseInfo(options).regexp; // String
};

dojox.date.hebrew.locale._parseInfo = function(/*oblect?*/options){
/* based on and similar to dojo.date.locale._parseInfo */

	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojox.date.hebrew.locale._getHebrewBundle(locale);

	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];

	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = (timePattern === undefined) ? datePattern : datePattern + ' ' + timePattern; //hebrew resource file does not contain time patterns - a bug?
	}

	var tokens = [];
	
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};




dojox.date.hebrew.locale.parse= function(/*String*/value, /*object?*/options){
		// based on and similar to dojo.date.locale.parse
		// summary: This function parse string date value according to options
		// example:
		// |		var dateHebrew = dojox.date.hebrew.locale.parse('11/10/5740', {datePattern:'dd/MM/yy', selector:'date'});
		// |		in Hebrew locale string for parsing contains Hebrew Numerals
		// |
		// |  options = {datePattern:'dd MMMM yy', selector:'date'};
		// |
		// |   y - year
		// |   M, MM  - short month
		// |  MMM, MMMM - long month
		// |  d - date
		// |  a - am, pm
		// |   E, EE, EEE, EEEE  - week day
		// |
		// |    h, H, k, K, m, s, S,  -  time format
		
	value =  value.replace(/[\u200E\u200F\u202A-\u202E]/g, ""); //remove special chars

	if(!options){options={};}
	var info = dojox.date.hebrew.locale._parseInfo(options);
	
	var tokens = info.tokens, bundle = info.bundle;
	var re = new RegExp("^" + info.regexp + "$");
	
	var match = re.exec(value);

	var locale = dojo.i18n.normalizeLocale(options.locale);

	if(!match){ return null; } // null
	
	var date, date1;
	
	//var result = [1970,0,1,0,0,0,0]; //
	var result = [5730,3,23,0,0,0,0];  // hebrew date for [1970,0,1,0,0,0,0] used in gregorian locale
	var amPm = "";
	var mLength = 0;
	var widthList = ["abbr", "wide", "narrow"];
	var valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(locale.match(/^he(?:-.+)?$/)){
					result[0] = dojox.date.hebrew.numerals.parseYearHebrewLetters(v);
				}else{
					result[0] = Number(v);
				}
				break;
			case 'M':
				//if  it is short format, month is one letter or two letter with "geresh"
				if(l>2){
					//we do not know here if the year is leap or not
					var months = dojox.date.hebrew.locale.getNames('months', widthList[l-3], 'format', locale, new dojox.date.hebrew.Date(5769, 1, 1)),
						leapmonths = dojox.date.hebrew.locale.getNames('months', widthList[l-3], 'format', locale, new dojox.date.hebrew.Date(5768, 1, 1));
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
						leapmonths = dojo.map(leapmonths, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
					}
					var monthName = v;
					v = dojo.indexOf(months, monthName);
					if(v == -1){
						v = dojo.indexOf(leapmonths, monthName);
						if(v == -1){
							//console.debug("dojox.date.hebrew.locale.parse: Could not parse month name:  second   " + v +"'.");
							return false;
						}
					}
					mLength = l;
				}else{
					if(locale.match(/^he(?:-.+)?$/)){
						v = dojox.date.hebrew.numerals.parseMonthHebrewLetters(v);
					}else{
						v--;
					}
				}
				result[1] = Number(v);
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				if(locale.match(/^he(?:-.+)?$/)){
					result[2] = dojox.date.hebrew.numerals.parseDayHebrewLetters(v);
				}else{
					result[2] = Number(v);
				}
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = Number(v);
				break;
			case 'm': //minutes
				result[4] = Number(v);
				break;
			case 's': //seconds
				result[5] = Number(v);
				break;
			case 'S': //milliseconds
				result[6] = Number(v);
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}
	var dateObject = new dojox.date.hebrew.Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // hebrew.Date
	//for non leap year, the index of the short month start from adar should be increased by 1
	if(mLength < 3 && result[1] >= 5 && !dateObject.isLeapYear(dateObject.getFullYear())){
		dateObject.setMonth(result[1]+1);
	}
	return dateObject; // hebrew.Date
};


function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g);
	var literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE  (tokens, bundle, options, pattern){
		// based on and similar to dojo.date.locale._buildDateTimeRE
		//
	
	pattern = dojo.regexp.escapeString(pattern);
	var locale = dojo.i18n.normalizeLocale(options.locale);
	
	return pattern.replace(/([a-z])\1*/ig, function(match){

			// Build a simple regexp.  Avoid captures, which would ruin the tokens list
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var p2 = '', p3 = '';
			if(options.strict){
				if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
				if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
			}else{
				p2 = '0?'; p3 = '0{0,2}';
			}
			switch(c){
				case 'y':
					s = '\\S+';
					break;
				case 'M':
					if(locale.match('^he(?:-.+)?$')){
						s = (l>2) ? '\\S+ ?\\S+' : '\\S{1,4}';
					}else{
						s = (l>2) ?  '\\S+ ?\\S+' : p2+'[1-9]|1[0-2]';
					}
					break;
				case 'd':
					if(locale.match('^he(?:-.+)?$')){
						s = '\\S[\'\"\'\u05F3]{1,2}\\S?';
					}else{
						s = '[12]\\d|'+p2+'[1-9]|30';
					}
					break;
				case 'E':
					if(locale.match('^he(?:-.+)?$')){
						s = (l>3) ? '\\S+ ?\\S+' : '\\S';
					}else{
						s = '\\S+';
					}
					break;
				case 'h': //hour (1-12)
					s = p2+'[1-9]|1[0-2]';
					break;
				case 'k': //hour (0-11)
					s = p2+'\\d|1[01]';
					break;
				case 'H': //hour (0-23)
					s = p2+'\\d|1\\d|2[0-3]';
					break;
				case 'K': //hour (1-24)
					s = p2+'[1-9]|1\\d|2[0-4]';
					break;
				case 'm':
				case 's':
					s = p2+'\\d|[0-5]\\d';
					break;
				case 'S':
					s = '\\d{'+l+'}';
					break;
				case 'a':
					var am = options.am || bundle['dayPeriods-format-wide-am'],
						pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					if(options.strict){
						s = am + '|' + pm;
					}else{
						s = am + '|' + pm;
						if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
						if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					}
					break;
				default:
					s = ".*";
			}
			if(tokens){ tokens.push(match); }
			return "(" + s + ")"; // add capture
		}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE. */
}
})();



(function(){
var _customFormats = [];
dojox.date.hebrew.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojox.date.hebrew.locale._getHebrewBundle = function(/*String*/locale){
	var hebrew = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		hebrew = dojo.mixin(hebrew, bundle);
	}, this);
	return hebrew; /*Object*/
};
})();

dojox.date.hebrew.locale.addCustomFormats("dojo.cldr","hebrew");

dojox.date.hebrew.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale, /*dojox.date.hebrew.Date?*/date){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// use:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names
	// date:
	//	required for item=months to determine leap month name
	//
	// using  var monthNames = dojox.date.hebrew.locale.getNames('months', 'wide', 'format', 'he', new dojox.date.hebrew.Date(5768, 2, 12));

	var label,
		lookup = dojox.date.hebrew.locale._getHebrewBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';
	
	// return by copy so changes won't be made accidentally to the in-memory model
	var result = (label || lookup[props.join('-')]).concat();

	if(item == "months"){
		if(date.isLeapYear(date.getFullYear())){
			// Adar I (6th position in the array) will be used.
			// Substitute the leap month Adar II for the regular Adar (7th position)
			props.push("leap");
			result[6] = lookup[props.join('-')];
		}else{
			// Remove Adar I but leave an empty position in the array
			delete result[5];
		}
	}

	return result; /*Array*/
};

}

if(!dojo._hasResource["dojox.date.islamic.Date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic.Date"] = true;
dojo.provide("dojox.date.islamic.Date");




dojo.declare("dojox.date.islamic.Date", null, {
	// summary: The component defines the Islamic (Hijri) Calendar Object
	//
	// description:
	//	This module is similar to the Date() object provided by JavaScript
	//
	// example:
	// |	
	// |
	// |	var date = new dojox.date.islamic.Date();
	// |	document.writeln(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());


	_date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,
	_GREGORIAN_EPOCH : 1721425.5,
	_ISLAMIC_EPOCH : 1948439.5,

	constructor: function(){
		// summary: This is the constructor
		// description:
		//	This function initialize the date object values
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		var date2 = new dojox.date.islamic.Date("12\2\1429");
		// |
		// |		var date3 = new dojox.date.islamic.Date(date2);
		// |
		// |		var date4 = new dojox.date.islamic.Date(1429,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._date = new Date(""); //TODO: should this be NaN?  _date is not a Date object
			}else{  // this is Islamic.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >=3){
			// YYYY MM DD arguments passed, month is from 0-12
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
	},

	getDate:function(){
		// summary: This function returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDate);
		return this._date;
	},
	
	getMonth:function(){
		// summary: This function return the month value ( 0 - 11 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getMonth()+1);

		return this._month;
	},

	getFullYear:function(){
		// summary: This function return the Year value
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getFullYear());

		return this._year;
	},
		
	getDay:function(){
		// summary: This function return Week Day value ( 0 - 6 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDay());

		return this.toGregorian().getDay();
	},
		
	getHours:function(){
		//summary: returns the Hour value
		return this._hours;
	},
	
	getMinutes:function(){
		//summary: returns the Minuites value
		return this._minutes;
	},

	getSeconds:function(){
		//summary: returns the seconde value
		return this._seconds;
	},

	getMilliseconds:function(){
		//summary: returns the Milliseconds value
		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: This function sets the Date
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setDate(2);

		date = parseInt(date);

		if(date > 0 && date <= this.getDaysInIslamicMonth(this._month, this._year)){
			this._date = date;
		}else{
			var mdays;
			if(date>0){
				for(mdays = this.getDaysInIslamicMonth(this._month, this._year);
					date > mdays;
						date -= mdays,mdays =this.getDaysInIslamicMonth(this._month, this._year)){
					this._month++;
					if(this._month >= 12){this._year++; this._month -= 12;}
				}

				this._date = date;
			}else{
				for(mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ?(this._month-1) :11 ,((this._month-1)>=0)? this._year: this._year-1);
						date <= 0;
							mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ? (this._month-1) :11,((this._month-1)>=0)? this._year: this._year-1)){
					this._month--;
					if(this._month < 0){this._year--; this._month += 12;}

					date+=mdays;
				}
				this._date = date;
			}
		}
		return this;
	},

	setFullYear:function(/*number*/year){
		// summary: This function set Year
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setYear(1429);

		this._year = +year;
	},

	setMonth: function(/*number*/month) {
		// summary: This function set Month
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setMonth(2);

		this._year += Math.floor(month / 12);
		if(month > 0){
			this._month = Math.floor(month % 12);
		}else{
			this._month = Math.floor(((month % 12) + 12) % 12);
		}
	},

	setHours:function(){
		//summary: set the Hours
		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours = parseInt(arguments[0]);
		}

		if(hours_arg_no >= 2){
			this._minutes = parseInt(arguments[1]);
		}

		if(hours_arg_no >= 3){
			this._seconds = parseInt(arguments[2]);
		}

		if(hours_arg_no == 4){
			this._milliseconds = parseInt(arguments[3]);
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInIslamicMonth(this._month, this._year);
			if(this._date > mdays){
					this._month ++;
					if(this._month >= 12){this._year++; this._month -= 12;}
					this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
	},

	setMinutes:function(/*number*/minutes){
		//summary: set the Minutes

		while(minutes >= 60){
			this._hours++;
			if(this._hours >= 24){
				this._date++;
				this._hours -= 24;
				var mdays = this.getDaysInIslamicMonth(this._month, this._year);
				if(this._date > mdays){
						this._month ++;
						if(this._month >= 12){this._year++; this._month -= 12;}
						this._date -= mdays;
				}
			}
			minutes -= 60;
		}
		this._minutes = minutes;
	},
		
		
	setSeconds:function(/*number*/seconds){
		//summary: set Seconds
		while(seconds >= 60){
			this._minutes++;
			if(this._minutes >= 60){
				this._hours++;
				this._minutes -= 60;
				if(this._hours >= 24){
					this._date++;
					this._hours -= 24;
					var mdays = this.getDaysInIslamicMonth(this._month, this._year);
					if(this._date > mdays){
						this._month ++;
						if(this._month >= 12){this._year++; this._month -= 12;}
						this._date -= mdays;
					}
				}
			}
			seconds -= 60;
		}
		this._seconds = seconds;
	},
		
	setMilliseconds:function(/*number*/milliseconds){
		//summary: set the Millisconds
		while(milliseconds >= 1000){
			this.setSeconds++;
			if(this.setSeconds >= 60){
				this._minutes++;
				this.setSeconds -= 60;
				if(this._minutes >= 60){
					this._hours++;
					this._minutes -= 60;
					if(this._hours >= 24){
						this._date++;
						this._hours -= 24;
						var mdays = this.getDaysInIslamicMonth(this._month, this._year);
				if(this._date > mdays){
					this._month ++;
					if(this._month >= 12){this._year++; this._month -= 12;}
					this._date -= mdays;
					}
				}
			}
		}
			milliseconds -= 1000;
		}
		this._milliseconds = milliseconds;
	},
		
		
	toString:function(){
		// summary: This returns a string representation of the date in "DDDD MMMM DD YYYY HH:MM:SS" format
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		document.writeln(date1.toString());

		//FIXME: TZ/DST issues?
		var x = new Date();
		x.setHours(this._hours);
		x.setMinutes(this._minutes);
		x.setSeconds(this._seconds);
		x.setMilliseconds(this._milliseconds);
		return this._month+" "+ this._date + " " + this._year + " " + x.toTimeString();
	},
		
		
	toGregorian:function(){
		// summary: This returns the equevalent Grogorian date value in Date object
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date(1429,11,20);
		// |		var dateGregorian = dateIslamic.toGregorian();

		var hYear = this._year;
		var hMonth = this._month;
		var hDate = this._date;
		var julianDay = hDate + Math.ceil(29.5 * hMonth) + (hYear - 1) * 354
						+ Math.floor((3 + (11 * hYear)) / 30) + this._ISLAMIC_EPOCH - 1;

		var wjd = Math.floor(julianDay - 0.5) + 0.5,
			depoch = wjd - this._GREGORIAN_EPOCH,
			quadricent = Math.floor(depoch / 146097),
			dqc = this._mod(depoch, 146097),
			cent = Math.floor(dqc / 36524),
			dcent = this._mod(dqc, 36524),
			quad = Math.floor(dcent / 1461),
			dquad = this._mod(dcent, 1461),
			yindex = Math.floor(dquad / 365),
			year = (quadricent * 400) + (cent * 100) + (quad * 4) + yindex;
		if(!(cent == 4 || yindex == 4)){
			year++;
		}
		
		var gYearStart = this._GREGORIAN_EPOCH + (365 * (year - 1)) + Math.floor((year - 1) / 4)
						- ( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400);
						
		var yearday = wjd - gYearStart;
		
		var tjd = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1)) + Math.floor((year - 1) / 4)
				-( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400) + Math.floor( (739 / 12)
				+ ( (dojo.date.isLeapYear(new Date(year,3,1)) ? -1 : -2)) + 1);
			
		var leapadj = ((wjd < tjd ) ? 0 : (dojo.date.isLeapYear(new Date(year,3,1)) ? 1 : 2));
					
		var month = Math.floor((((yearday + leapadj) * 12) + 373) / 367);
		var tjd2 = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1))
					+ Math.floor((year - 1) / 4) - (Math.floor((year - 1) / 100))
					+ Math.floor((year - 1) / 400) + Math.floor((((367 * month) - 362) / 12)
					+ ((month <= 2) ? 0 : (dojo.date.isLeapYear(new Date(year,month,1)) ? -1 : -2)) + 1);
					
		var day = (wjd - tjd2) + 1;

		var gdate = new Date(year, (month - 1), day, this._hours, this._minutes, this._seconds, this._milliseconds);

		return gdate;
	},

	//TODO: would it make more sense to make this a constructor option? or a static?
	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	fromGregorian:function(/*Date*/gdate){
		// summary: This function returns the equivalent Islamic Date value for the Gregorian Date
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateIslamic.fromGregorian(dateGregorian);

		var date = new Date(gdate);
		var gYear = date.getFullYear(),
			gMonth = date.getMonth(),
			gDay = date.getDate();
		
		var julianDay = (this._GREGORIAN_EPOCH - 1) + (365 * (gYear - 1)) + Math.floor((gYear - 1) / 4)
					+ (-Math.floor((gYear - 1) / 100)) + Math.floor((gYear - 1) / 400)
					+ Math.floor((((367 * (gMonth+1)) - 362) / 12)
					+ (((gMonth+1) <= 2) ? 0 : (dojo.date.isLeapYear(date) ? -1 : -2)) + gDay);
		julianDay = Math.floor(julianDay) + 0.5;

		var days = julianDay - this._ISLAMIC_EPOCH;
		var hYear  = Math.floor( (30 * days + 10646) / 10631.0 );
		var hMonth = Math.ceil((days - 29 - this._yearStart(hYear)) / 29.5 );
		hMonth = Math.min(hMonth, 11);
		var hDay = Math.ceil(days - this._monthStart(hYear, hMonth)) + 1;

		this._date = hDay;
		this._month = hMonth;
		this._year = hYear;
		this._hours = date.getHours();
		this._minutes = date.getMinutes();
		this._seconds = date.getSeconds();
		this._milliseconds = date.getMilliseconds();
		this._day = date.getDay();
		return this;
	},
	
	valueOf:function(){
		// summary: This function returns The stored time value in milliseconds
		// since midnight, January 1, 1970 UTC

		return this.toGregorian().valueOf();
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearStart:function(/*Number*/year){
		//summary: return start of Islamic year
		return (year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_monthStart:function(/*Number*/year, /*Number*/month){
		//summary: return the start of Islamic Month
		return Math.ceil(29.5*month) +
			(year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_civilLeapYear:function(/*Number*/year){
		//summary: return Boolean value if Islamic leap year
		return (14 + 11 * year) % 30 < 11;
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInIslamicMonth:function(/*Number*/month, /*Number*/ year){
		//summary: returns the number of days in the given Islamic Month
		var length = 0;
		length = 29 + ((month+1) % 2);
		if(month == 11 && this._civilLeapYear(year)){
			length++;
		}
		return length;
	},

	_mod:function(a, b){
		return a - (b * Math.floor(a / b));
	}
});

//TODOC
dojox.date.islamic.Date.getDaysInIslamicMonth = function(/*dojox.date.islamic.Date*/month){
	return new dojox.date.islamic.Date().getDaysInIslamicMonth(month.getMonth(),month.getFullYear()); // dojox.date.islamic.Date
};

}

if(!dojo._hasResource["dojox.date.islamic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic"] = true;
dojo.provide("dojox.date.islamic");


 // for compare
	
// Utility methods to do arithmetic calculations with islamic.Dates

	// added for compat to date
dojox.date.islamic.getDaysInMonth = function(/*islamic.Date*/month){
	return month.getDaysInIslamicMonth(month.getMonth(), month.getFullYear());
};

//TODO: define islamic.isLeapYear?  Or should it be invalid, since it has different meaning?

dojox.date.islamic.compare = function(/*islamic.Date*/date1, /*islamic.Date*/date2, /*String?*/portion){
	//	summary:
	//		Compare two islamic date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		islamic.Date object
	//	date2:
	//		islamic.Date object.  If not specified, the current islamic.Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	if(date1 instanceof dojox.date.islamic.Date){
		date1 = date1.toGregorian();
	}
	if(date2 instanceof dojox.date.islamic.Date){
		date2 = date2.toGregorian();
	}
	
	return dojo.date.compare.apply(null, arguments);
};

dojox.date.islamic.add = function(/*dojox.date.islamic.Date*/date, /*String*/interval, /*int*/amount){
	//	based on and similar to dojo.date.add
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: islamic.Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var newIslamDate = new dojox.date.islamic.Date(date);

	switch(interval){
		case "day":
			newIslamDate.setDate(date.getDate() + amount);
			break;
		case "weekday":
			var day = date.getDay();
			if(((day + amount) < 5) && ((day + amount) > 0)){
				 newIslamDate.setDate(date.getDate() + amount);
			}else{
				var adddays = 0, /*weekend */
					remdays = 0;
				if(day == 5){//friday
					day = 4;
					remdays = (amount > 0) ?  -1 : 1;
				}else if(day == 6){ //shabat
					day = 4;
					remdays = (amount > 0) ? -2 : 2;
				}
				var add = (amount > 0) ? (5 - day - 1) : -day
				var amountdif = amount - add;
				var div = parseInt(amountdif / 5);
				if(amountdif % 5 != 0){
					adddays = (amount > 0)  ? 2 : -2;
				}
				adddays = adddays + div * 7 + amountdif % 5 + add;
				newIslamDate.setDate(date.getDate() + adddays +  remdays);
			}
			break;
		case "year":
			newIslamDate.setFullYear(date.getFullYear() + amount);
			break;
		case "week":
			amount *= 7;
			newIslamDate.setDate(date.getDate() + amount);
			break;
		case "month":
			var month = date.getMonth();
			newIslamDate.setMonth(month + amount);
			break;
		case "hour":
			newIslamDate.setHours(date.getHours() + amount);
			break;
		case "minute":
			newIslamDate.setMinutes(date.getMinutes() + amount);
			break;
		case "second":
			newIslamDate.setSeconds(date.getSeconds() + amount);
			break;
		case "millisecond":
			newIslamDate.setMilliseconds(date.getMilliseconds() + amount);
			break;
	}

	return newIslamDate; // dojox.date.islamic.Date
};

dojox.date.islamic.difference = function(/*dojox.date.islamic.Date*/date1, /*dojox.date.islamic.Date?*/date2, /*String?*/interval){
	//	based on and similar to dojo.date.difference
	//	summary:
	//        date1 - date2
	//	 date2 is islamic.Date object.  If not specified, the current islamic.Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond",  "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new dojox.date.islamic.Date();
	interval = interval || "day";
	var yearDiff = date1.getFullYear() - date2.getFullYear();
	var delta = 1; // Integer return value
	switch(interval){
		case "weekday":
			var days = Math.round(dojox.date.islamic.difference(date1, date2, "day"));
			var weeks = parseInt(dojox.date.islamic.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date2.getDay();
				var bDay = date1.getDay();
	
				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new dojox.date.islamic.Date(date2);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();
	
				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = -1;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = -1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = 0;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 1;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = 2;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			var startdate =  (date1.toGregorian() > date2.toGregorian()) ? date1 : date2; // more
			var enddate = (date1.toGregorian() > date2.toGregorian()) ? date2 : date1;
			
			var month1 = startdate.getMonth();
			var month2 = enddate.getMonth();
			
			if (yearDiff == 0){
				delta = startdate.getMonth() - enddate.getMonth() ;
			}else{
				delta = 12-month2;
				delta +=  month1;
				var i = enddate.getFullYear()+1;
				var e = startdate.getFullYear();
				for (i;   i < e;  i++){
					delta += 12;
				}
			}
			if (date1.toGregorian() < date2.toGregorian()){
				delta = -delta;
			}
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojox.date.islamic.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date1.toGregorian().getTime()- date2.toGregorian().getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojox.date.islamic.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic.locale"] = true;
dojo.provide("dojox.date.islamic.locale");









(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, locale, fullYear,  pattern){

		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad;
			var c = match.charAt(0);
			var l = match.length;
			var widthList = ["abbr", "wide", "narrow"];
			
			switch(c){
				case 'G':
					s = bundle["eraAbbr"][0];
					break;
				case 'y':
					s = String(dateObject.getFullYear());
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = ["months", "format", widthList[l-3]].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'd':
					s = dateObject.getDate(true); pad = true;
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.getTimezoneName(dateObject.toGregorian());
					if(s){ break; }
					l = 4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dateObject.toGregorian().getTimezoneOffset();
					var tz = [
						(offset <= 0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset) / 60), 2),
						dojo.string.pad(Math.abs(offset) % 60, 2)
					];
					if(l == 4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
				default:
					throw new Error("dojox.date.islamic.locale.formatPattern: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}
	
// based on and similar to dojo.date.locale.format
dojox.date.islamic.locale.format = function(/*islamic.Date*/dateObject, /*Object?*/options){
	// summary:
	//		Format a Date object as a String, using  settings.
	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojox.date.islamic.locale._getIslamicBundle(locale);
	var str = [];

	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle, locale, options.fullYear);
	if(options.selector == "year"){
		var year = dateObject.getFullYear();
		return year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time

	return result; // String
};

dojox.date.islamic.locale.regexp = function(/*object?*/options){
	//	based on and similar to dojo.date.locale.regexp
	// summary:
	//		Builds the regular needed to parse a islamic.Date
	return dojox.date.islamic.locale._parseInfo(options).regexp; // String
};

dojox.date.islamic.locale._parseInfo = function(/*oblect?*/options){
/* based on and similar to dojo.date.locale._parseInfo */

	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojox.date.islamic.locale._getIslamicBundle(locale);
	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];

	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = (typeof (timePattern) == "undefined") ? datePattern : datePattern + ' ' + timePattern;
	}

	var tokens = [];
	
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};




dojox.date.islamic.locale.parse= function(/*String*/value, /*Object?*/options){
	// based on and similar to dojo.date.locale.parse
	// summary: This function parse string date value according to options
	
	value =  value.replace(/[\u200E\u200F\u202A\u202E]/g, ""); //remove bidi non-printing chars

	if(!options){ options={}; }
	var info = dojox.date.islamic.locale._parseInfo(options);

	var tokens = info.tokens, bundle = info.bundle;
	var regexp = info.regexp.replace(/[\u200E\u200F\u202A\u202E]/g, ""); //remove bidi non-printing chars from the pattern
	var re = new RegExp("^" + regexp + "$");

	var match = re.exec(value);

	var locale = dojo.i18n.normalizeLocale(options.locale);

	if(!match){ return null; } // null
	
	var date, date1;
	
	var result = [1389,0,1,0,0,0,0];  //FIXME: islamic date for [1970,0,1,0,0,0,0] used in gregorian locale
	var amPm = "";
	var mLength = 0;
	var widthList = ["abbr", "wide", "narrow"];
	var valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				result[0] = Number(v);
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
						return false;
					}
					mLength = l;
				}else{
					v--;
				}
				result[1] = Number(v);
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
					result[2] =  Number(v);
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = Number(v);
				break;
			case 'm': //minutes
				result[4] = Number(v);
				break;
			case 's': //seconds
				result[5] = Number(v);
				break;
			case 'S': //milliseconds
				result[6] = Number(v);
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}
	var dateObject = new dojox.date.islamic.Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]);
	return dateObject;
};


function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g);
	var literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE  (tokens, bundle, options, pattern){
		// based on and similar to dojo.date.locale._buildDateTimeRE
		//
	
	pattern = dojo.regexp.escapeString(pattern);
	var locale = dojo.i18n.normalizeLocale(options.locale);
	
	return pattern.replace(/([a-z])\1*/ig, function(match){

			// Build a simple regexp.  Avoid captures, which would ruin the tokens list
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var p2 = '', p3 = '';
			if(options.strict){
				if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
				if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
			}else{
				p2 = '0?'; p3 = '0{0,2}';
			}
			switch(c){
				case 'y':
					s = '\\d+';
					break;
				case 'M':
					s = (l>2) ?  '\\S+ ?\\S+' : p2+'[1-9]|1[0-2]';
					break;
				case 'd':
					s = '[12]\\d|'+p2+'[1-9]|3[01]';
					break;
				case 'E':
					s = '\\S+';
					break;
				case 'h': //hour (1-12)
					s = p2+'[1-9]|1[0-2]';
					break;
				case 'k': //hour (0-11)
					s = p2+'\\d|1[01]';
					break;
				case 'H': //hour (0-23)
					s = p2+'\\d|1\\d|2[0-3]';
					break;
				case 'K': //hour (1-24)
					s = p2+'[1-9]|1\\d|2[0-4]';
					break;
				case 'm':
				case 's':
					s = p2+'\\d|[0-5]\\d';
					break;
				case 'S':
					s = '\\d{'+l+'}';
					break;
				case 'a':
					var am = options.am || bundle['dayPeriods-format-wide-am'],
						pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					if(options.strict){
						s = am + '|' + pm;
					}else{
						s = am + '|' + pm;
						if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
						if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					}
					break;
				default:
					s = ".*";
			}
			if(tokens){ tokens.push(match); }
			return "(" + s + ")"; // add capture
		}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE. */
}
})();



(function(){
var _customFormats = [];
dojox.date.islamic.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	_customFormats.push({pkg:packageName,name:bundleName});
};

dojox.date.islamic.locale._getIslamicBundle = function(/*String*/locale){
	var islamic = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		islamic = dojo.mixin(islamic, bundle);
	}, this);
	return islamic; /*Object*/
};
})();

dojox.date.islamic.locale.addCustomFormats("dojo.cldr","islamic");

dojox.date.islamic.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale, /*islamic Date Object?*/date){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	var label;
	var lookup = dojox.date.islamic.locale._getIslamicBundle(locale);
	var props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';
	
	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};


dojox.date.islamic.locale.weekDays = dojox.date.islamic.locale.getNames('days', 'wide', 'format');

dojox.date.islamic.locale.months = dojox.date.islamic.locale.getNames('months', 'wide', 'format');

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.Util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.Util"] = true;
/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.util.common.Util");

















/**
 * creates a secure string which can be set using DOMElement.innerHTML
 *
 * @param innerHTML the original inner HTML string
 * @returns {String} the secure inner HTML string
 */
com.ibm.bpm.social.util.common.Util.createSecureInnerHtml = function(/*String*/ innerHTML) {
  if(typeof innerHTML == "string") {
    var secureInnerHtmlPreparation = new com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation(innerHTML);
    return secureInnerHtmlPreparation.getSecureString();
  } else {
    return innerHTML;
  }
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS.sssZ
 */
com.ibm.bpm.social.util.common.Util.getDateString = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getFullYear() + "-";
  if (d.getMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getMonth() + 1) + '-';
  if (d.getDate() < 10) {
    result = result + '0';
  }
  result = result + d.getDate() + "T";
  if (d.getHours() < 10) {
    result = result + '0';
  }
  result = result + d.getHours() + ":";
  if (d.getMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getMinutes() + ":";
  if (d.getSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getSeconds() + ".";
  if (d.getMilliseconds() < 100) {
    result = result + '0';
  }
  if (d.getMilliseconds() < 10) {
    result = result + '0';
  }
  result = result + d.getMilliseconds();
  var offset = d.getTimezoneOffset();
  var modifier = "";
  if (offset < 0) {
    modifier = "+";
  } else {
	modifier = "-";
  }
  var tzh = Math.floor(Math.abs(offset) / 60);
  var tzm = offset - (tzh * 60);
  if (tzh < 10) {
    tzh = '0' + tzh;
  }
  result = result + modifier + tzh + ":";
  if (tzm < 10) {
    result = result + '0';
  }
  result = result + tzm;
  return result;
};

com.ibm.bpm.social.util.common.Util.fixDateTimeZone = function(str){
	var i = str.indexOf('Z');
	if (i != -1) {
		str = str.substring(0,i);
		if (i <20) {
			str = str + '.000';
		}
		str = str + '+0000';
	}
	return str;
};

com.ibm.bpm.social.util.common.Util.formatDate = function(str){
	var date,d1,t1;
	
	str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
	date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
	
    d1 = com.ibm.bpm.social.util.common.Util.formatDateObject(date, {formatLength: "long", selector: "date"});
	t1 = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});
	
	return (d1+" "+t1);
};

com.ibm.bpm.social.util.common.Util.formatDateObject = function(d, options){
  var ret = "";
  var calPref = com.ibm.bpm.social.util.common.Util.getCalendarUserPreference();
  if (calPref == "hebrew") {
      var dateObj = new dojox.date.hebrew.Date(d);
      ret = dojox.date.hebrew.locale.format(dateObj, options);
  } else if (calPref == "islamic") {
      var dateObj = new dojox.date.islamic.Date(d);
      ret = dojox.date.islamic.locale.format(dateObj, options);
  } else {
      ret = dojo.date.locale.format(d, options);
  }
  return ret;
};

/*
 *  @param Date string
 *  @returns String in long date format, no time
 */
com.ibm.bpm.social.util.common.Util.formatDateLong = function(str){
  var date, d1;

  str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
  date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);

  d1 = dojo.date.locale.format(date, {
    formatLength: "long",
    selector: "date"
  });
  return d1;
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS
 */
com.ibm.bpm.social.util.common.Util.getTimestampUTC = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getUTCFullYear() + "-";
  if (d.getUTCMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getUTCMonth() + 1) + '-';
  if (d.getUTCDate() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCDate() + "T";
  if (d.getUTCHours() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCHours() + ":";
  if (d.getUTCMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCMinutes() + ":";
  if (d.getUTCSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCSeconds();
  // skip milliseconds and skip TIMEZONE identifier
  return result;
};

com.ibm.bpm.social.util.common.Util.getDateFromString = function(str){

	//Normalize "2011-12-19T10:36:56.060-05:00" to "2011-12-19T10:36:56.060-0500"
	if(str.lastIndexOf(":") > 23){
		var substr1 = str.substr(23,6);
		var substr2 = substr1.replace(":", "");
		str = str.replace(substr1, substr2);
	}
	
	var yr = parseInt(str.substr(0, 4));
	var tmp = str.substr(5, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var mnth = parseInt(tmp);
	tmp = str.substr(8, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var dy = parseInt(tmp);
	tmp = str.substr(11, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var hr = parseInt(tmp);
	tmp = str.substr(14, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var min = parseInt(tmp);
	tmp = str.substr(17, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var sec = parseInt(tmp);
	tmp = str.substr(20, 3);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var msec = parseInt(tmp);
	var result = new Date(yr, mnth - 1, dy, hr, min, sec, msec);
	tmp = str.substr(23, 3);
	var isNegative = (tmp.charAt(0) == '-');
	tmp = tmp.substr(1);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzh = parseInt(tmp);
	tmp = str.substr(26, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzm = parseInt(tmp);
	if (isNegative) {
		tzh = -tzh;
		tzm = -tzm;
	}
	var offset = result.getTimezoneOffset();
	offset = offset + (tzh * 60) + tzm;
	result = new Date(result.getTime() - (offset * 60000));
	return result;
};

com.ibm.bpm.social.util.common.Util.formatServerDateString = function(str){
  var dateStr;
  var d;

  if (dojo.isIE) {
    d = dojo.date.stamp.fromISOString(str);
  }
  else {
    d = new Date(str);
  }

  dateStr = d.toDateString() + " " + d.toLocaleTimeString();
  dateStr = dateStr.substring(4, dateStr.length);
  dateStr = dateStr.substring(0, 6) + "," + dateStr.substring(6, dateStr.length);

  return dateStr;
};

com.ibm.bpm.social.util.common.Util.portalPIIMessages = null;

com.ibm.bpm.social.util.common.Util.restConfigurationException = function(error, xhrArgs){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	if(com.ibm.bpm.social.util.common.Util.portalPIIMessages==null){
		//NOTE:  Locale 'en' is hard-coded here as the required message is only available in the English bundle for 8.0
		dojo.xhrGet({
			"url": ppConfig.portalContextRootJS+"/resources/PortalPIIMessages_en.json",
			"sync": true,
			"handleAs": "json",
			"load": function restConfigurationException_xhrGetLoad(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util.portalPIIMessages = data;
				com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
			},
			"error": function(error, xhrArgs){
				com.ibm.bpm.social.util.common.Log.logError(2, "PortalPIIMessages_en.json could not be retrieved.");
			}
		});
	} else {
		com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
	}
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.openRestConfigurationException = function(){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"]);
	
	var details = "<p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"];
	details += "</p><p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"];
	details += "</p>";
	
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"], details);
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.userData = {};

com.ibm.bpm.social.util.common.Util.getCurrentUser = function(){
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentUser) {
    com.ibm.bpm.social.util.common.Util.currentUser = {
      userName: ""
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/current?parts=none",
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentUser = data.data;
        com.ibm.bpm.social.util.common.Util.userData[data.data.userName] = data.data;
      },
      error: function(error, xhrArgs){
    	  com.ibm.bpm.social.util.common.Util.restConfigurationException(error, xhrArgs);
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentUser;
};

com.ibm.bpm.social.util.common.Util.getUser = function(username, ignoreCacheOnDirty){

  if(typeof(ignoreCacheOnDirty)=="undefined"){
	  ignoreCacheOnDirty = false;
  }
	
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (username == null) {
    return com.ibm.bpm.social.util.common.Util.getCurrentUser();
  }

  if (!com.ibm.bpm.social.util.common.Util.userData[username] ||
		  ((typeof(com.ibm.bpm.social.util.common.Util.userData[username].dirty)=="undefined"||
			com.ibm.bpm.social.util.common.Util.userData[username].dirty) && ignoreCacheOnDirty) ) {
    com.ibm.bpm.social.util.common.Util.userData[username] = {
      userName: username
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/" + encodeURIComponent(username) + "?parts=none&preventCache=" + new Date().getTime(),
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.userData[username] = data.data;
        com.ibm.bpm.social.util.common.Util.userData[username].dirty = false;
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.userData[username];
};

com.ibm.bpm.social.util.common.Util.primeUserCache = function(id, name, url){
	if(typeof(com.ibm.bpm.social.util.common.Util.userData[id])=="undefined"){
		var newUser = {
				"userName": id,
				"fullName": name,
				"userPreferences": {
					"Image": url
				},
				"dirty": true
		};
		com.ibm.bpm.social.util.common.Util.userData[id] = newUser;
	}
};

com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};

/**
 * Returns the URL for the user's avatar.
 * @param username the username (or user id) of the user you want to see the avatar for.
 * @param ignoreCache true if a new URL should be returned since the last time this method was called.
 */
com.ibm.bpm.social.util.common.Util.getUserAvatarURL = function(username, ignoreCache){
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL entering with username:{"+username+"}");
	if(typeof(ignoreCache)=="undefined"){
		ignoreCache = false;
	}
	
	if (com.ibm.bpm.social.util.common.Util.userAvatarURLs == null) {
		com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};
	}
	
	if (username == null) {
		return null;
	}

   	if (ignoreCache || !com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]) {
	   	com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = window.endpointURLObject.bpmRestJS +"/v1/avatar/image/" + encodeURIComponent(username) + "?preventCache=" + new Date().getTime();
		if(typeof(connectionsBridge)!="undefined" 
			&& connectionsBridge.bootstrap==true 
			&& connectionsBridge.isAvatarServiceAvailable()){ //Is Connections available and configured?
			
			var user = com.ibm.bpm.social.util.common.Util.getUser(username);
			if(typeof(user["emailAddress"])!="undefined" 
				&& user["emailAddress"]!=null 
				&& user["emailAddress"].length > 0){ //Does the user have a defined email address?
				
				var avatar = connectionsBridge.getAvatarUrl(user["emailAddress"]);
				if(avatar!=null){
					com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = avatar;
				}
			}
		}		
	}
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL returning {"+com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]+"}");
	return com.ibm.bpm.social.util.common.Util.userAvatarURLs[username];
};

com.ibm.bpm.social.util.common.Util._integrations = null;

com.ibm.bpm.social.util.common.Util.getSametimeURL = function(){
  var ints = com.ibm.bpm.social.util.common.Util._getIntegrations()
  return ints["SAMETIME_SERVER"];
};

com.ibm.bpm.social.util.common.Util.getConnectionsURL = function(){
	var ints = com.ibm.bpm.social.util.common.Util._getIntegrations();
	return ints["CONNECTIONS_SERVER"];
};

com.ibm.bpm.social.util.common.Util._getIntegrations = function(){
	if(!com.ibm.bpm.social.util.common.Util._integrations){
		dojo.xhrGet({
			//url: "/rest/bpm/wle/v1/system/env/variable?bpdShortname=TWP&name=SAMETIME_SERVER",
			url: window.endpointURLObject.bpmRestJS +"/v1/social/system/integrations",
			handleAs: "json",
			sync: true,
			headers: {
				"Content-Type": "application/json",
				"Accept": "application/json"
			},
			load: function(data, xhrArgs){
				if (data.data && data.data.collabSystemsList) {
					com.ibm.bpm.social.util.common.Util._integrations = {};
					
					for(var x = 0; x < data.data.collabSystemsList.length; x++){
						com.ibm.bpm.social.util.common.Util._integrations[data.data.collabSystemsList[x].name] = data.data.collabSystemsList[x].value;
					}
					
				} else {
					com.ibm.bpm.social.util.common.Util_integrations = {};
				}
			},
			error: function(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util._integrations = {};
			}
		});
	}
	return com.ibm.bpm.social.util.common.Util._integrations;
}


com.ibm.bpm.social.util.common.Util.getParameter = function(param){
  var result = null;
  if (window.location.search) {
    var search = window.location.search;
    if (search.indexOf("?") === 0) {
      search = search.substring(1); //Remove leading '?' from the query string
    }
    var list = search.split("&");
    for ( var i = 0; i < list.length; i++) {
      var key = "";
      var val = "";
      var j = list[i].indexOf('=');
      if (j == -1) {
        key = list[i];
      }
      else {
        key = list[i].substring(0, j);
        val = list[i].substring(j + 1);
      }
      if (key == param) {
        result = val;
        i = list.length;
      }
    }
  }
  return result;
};

com.ibm.bpm.social.util.common.Util.currentInstanceData = {};

com.ibm.bpm.social.util.common.Util.getInstanceData = function(paramObj){

  var iid = null;
  var params = "none";
  var skipCache = false;
  var updateTaskData = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["diagram"]) {
      p.push("diagram");
    }
    if (paramObj["header"]) {
      p.push("header");
    }
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["executionTree"]) {
      p.push("executionTree");
    }
    if (paramObj["instanceId"]) {
      iid = paramObj["instanceId"];
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
    if (paramObj["updateTaskData"]){
    	updateTaskData = true;
    }

    if (p.length > 0) {
      params = p.join("%7C");
    }
  }

  if(iid==null) return {}; //iid = com.ibm.bpm.social.util.common.Util.getInstanceId();

  if(com.ibm.bpm.social.util.common.Util.currentInstanceData == null) {
	  com.ibm.bpm.social.util.common.Util.currentInstanceData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = {
    		'piid': iid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/process/";
    url += iid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = data.data;

        if(updateTaskData==true){
        	for(var i = 0; i < data.data.tasks.length; i++){
        		var t = data.data.tasks[i];
        		com.ibm.bpm.social.util.common.Util.currentTaskData[t.tkiid] = t;
        	}
        }

      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentInstanceData[iid];
};

com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};

com.ibm.bpm.social.util.common.Util.getTaskTemplateData = function(paramObj) {

  var tktid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["taskTemplateId"]) {
      tktid = paramObj["taskTemplateId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
      skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskTemplateData == null) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = {
        'tktid': tktid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/taskTemplate/";
    url += tktid;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid];
};

com.ibm.bpm.social.util.common.Util.currentTaskData = {};

com.ibm.bpm.social.util.common.Util.getTaskData = function(paramObj){

  var tid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["taskId"]) {
      tid = paramObj["taskId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskData == null) {
	  com.ibm.bpm.social.util.common.Util.currentTaskData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskData[tid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = {
    		'tkiid': tid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/task/";
    url += tid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskData[tid];
};

com.ibm.bpm.social.util.common.Util.isCommentSupported = function(event){
  if (event.object && event.object.objectType) {
    if (event.object.objectType == 'ibm.bpm.comment') {
      return true;
    }
    if (event.object.objectType.indexOf('ibm.bpm.helprequest') == 0) {
      return true;
    }
  }
  return false;
};

com.ibm.bpm.social.util.common.Util.isIPad = navigator.userAgent.indexOf("iPad") > -1;

com.ibm.bpm.social.util.common.Util.isWebkit = navigator.userAgent.indexOf("WebKit") > -1;

/*
 * Simple method which directs the browser to the login page using the current hostname and port
 */
com.ibm.bpm.social.util.common.Util.redirectToLoginPage = function() {
	var location = window.location;
	var loginPage = 'login.jsp';
	// if the user has a custom logout page, we need to redirect to this
	// page instead of the login.jsp so that they can properly log out of
	// all resources
	if (portalConfigLogoutExitPage && (portalConfigLogoutExitPage != 'login.jsp')) {
		loginPage = 'customLogoutRedirect.jsp';
	}
	var redirUrl = window.endpointURLObject.processPortalJS +"/" +loginPage;
	window.location = redirUrl;
};

/*
 * Method to get the Base Text Direction User preference.  Needs to be used to properly align
 * necessary enabled fields with Base Text Direction.
 *
 * @return BTD 'ltr' || 'rtl' || 'ctx'
 */
com.ibm.bpm.social.util.common.Util.getBTDUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var baseTextDirPref = "ctx";
	
	if(typeof userProfileObj != "undefined") {
		baseTextDirPref = userProfileObj.userPreferences["Base Text Direction"];
		
		if(baseTextDirPref == "generalPreference.fields.baseTextDir.contextual" || baseTextDirPref == "") {
			baseTextDirPref = "ctx";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.ltr") {
			baseTextDirPref = "ltr";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.rtl") {
			baseTextDirPref = "rtl";
		} else {
			//unexpected, default to "ctx"
			baseTextDirPref = "ctx";
		}
	}
	
	return baseTextDirPref;
};

/*
 * Method to get the Calendar Type User preference.  Needs to be used to display
 * dates according to calendar types Gregorian, Hebrew or Islamic.
 *
 * @return calendarType 'gregorian' || 'islamic' || 'hebrew'
 */

com.ibm.bpm.social.util.common.Util.getCalendarUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var calPref = "gregorian";	
	if(typeof userProfileObj != "undefined") {
		var pref = userProfileObj.userPreferences["Calendar Type"];
        if (pref) {
            if (pref == "generalPreference.fields.calendarType.islamic") {
                calPref = "islamic";
            } else if (pref == "generalPreference.fields.calendarType.hebrew") {
                calPref = "hebrew";
            }
        }
    }
    return calPref;
};

/*
 * Method to get the hash portion of a URL
 */

com.ibm.bpm.social.util.common.Util.getCurrentHash = function(checkCookie) {
	var h = window.location.hash;
   	if (h && (h.indexOf('#dashboards%25') == 0)) {
   		h = '#' + decodeURI(h.substring(1));
   		window.location.hash = h;
   	} 
	if (checkCookie && !h) {
		h = dojo.cookie("com_ibm_bpm_process_portal_hash");
	}
	if (h && (h.charAt(0) == '#')) {
		h = h.substring(1);
	}
	if (h) {
		try {
			h = decodeURIComponent(h);
		} catch(e) {
			// if we cannot decode the text, assmume it is already decoded (FireFox)
		}
	}
	if (h) {
		while (h.length && (h.charAt(h.length-1) == ' ')) {
			h = h.substring(0,h.length-1);
		}
	}
	return h;
}

/*
 * Method to remove a parameter from a URL
 */
com.ibm.bpm.social.util.common.Util.removeParameter = function(url,param,skip) {
	var skipped = 0;
	var start = url.indexOf('?'+param+'=');
	if (start == -1) {
		start = url.indexOf('&'+param+'=');
	}
	while (start != -1) {
		var end = url.indexOf('&',start+1);
		var next = start;
		if (skip && (skip > skipped)) {
			skipped++;
			next++;
		}  else {
			if (end == -1) {
				url = url.substring(0,start);
			} else {
				url = url.substring(0,start+1)+url.substring(end+1);
			}
		}
		start = url.indexOf('?'+param+'=',next);
		if (start == -1) {
			start = url.indexOf('&'+param+'=',next);
		}
	}
	return url;
}

/*
 * Method to set the hash portion of a URL
 */
com.ibm.bpm.social.util.common.Util.setCurrentHash = function(h) {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	for (var i=portalHashHistory.length-1; i>-1; i--) {
		if (portalHashHistory[i] == h) {
			portalHashHistory.splice(i,1);
		}
	}
	if (h) {
		portalHashHistory.push(h);
	}
	if (portalHashHistory.length > 10) {
		portalHashHistory.splice(0,portalHashHistory.length-10);
	}
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS + '/' + h; 
	if (window.history && window.history.replaceState) {
		window.history.replaceState({},'',window.ppConfig.portalContextRootJS+'/'+h);
	} else {
		com.ibm.bpm.social.util.common.Util.lastHash = h;
		window.location.hash = '#'+encodeURIComponent(h);
	}
}

/*
 * Current window url information
 */
if (com.ibm.bpm.social.util.common.Util.urlData == undefined) {
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS;
}

/*
 * Show current window url information in dialog
 */
com.ibm.bpm.social.util.common.Util.showProcessPortalUrl = function(event) {
	
  	var messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
	var msg = '<div id="showLinkDialogText">'+messages.showLinkDialog+'</div><br><br><div style="word-wrap:break-word" tabIndex="0" aria-labelledBy="showLinkDialogText" role="textbox" dir="ltr">'+
			  com.ibm.bpm.social.util.common.Util.urlData+
			  '</div>';
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(msg);
}

/*
 * Creates an object to publish with the topic com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB (Topics.js);
 * This topic is published in many places and the required elements in the object were hard to keep track of
 */
com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB = function(itemParams) {
	var newObj={};
	newObj.title="";
	newObj.baseID="";
	newObj.itemID="";
	newObj.isTip=false;
	newObj.isSnapshot=false;
	newObj.snapshotName="";
	newObj.isWorkTab=false;
	if (itemParams) {
		if (itemParams.title!=undefined) {
			newObj.title=itemParams.title;
		}
		if (itemParams.baseID!=undefined) {
			newObj.baseID=itemParams.baseID;
		}
		if (itemParams.itemID!=undefined) {
			newObj.itemID=itemParams.itemID;
		}
		if (itemParams.isTip!=undefined) {
			newObj.isTip=itemParams.isTip;
		}
		if (itemParams.isSnapshot!=undefined) {
			newObj.isSnapshot=itemParams.isSnapshot;
		}
		if (itemParams.snapshotName!=undefined) {
			newObj.snapshotName=itemParams.snapshotName;
		}
		if (itemParams.isWorkTab!=undefined) {
			newObj.isWorkTab=itemParams.isWorkTab;
		}
	}
	return newObj;
}
 
/*
 * Method to go to return to the previous hash location
 */
com.ibm.bpm.social.util.common.Util.gotoPreviousHash = function() {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	if (portalHashHistory.length > 0) {
		portalHashHistory.pop();
	}	
	if (portalHashHistory.length > 0) {
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[portalHashHistory.pop()]);
	} else {
		window.location.hash = '#';
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[""]);
	}
}
 
 /*
 * Method to get query parameters from a url
 */
 com.ibm.bpm.social.util.common.Util.getQueryParamters = function(url) {
 	var urlParams = new Object(); 
 	if (url && (url.indexOf('?') != -1)) {
		var urlParamString = url.substring(url.indexOf('?')+1);
		var urlParamList = urlParamString.split('&');
		for (var p1=0; p1<urlParamList.length; p1++) {
			var p2 = urlParamList[p1].indexOf('=');
			if (p2 > -1) {
				urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
			}
		}
	}
	return urlParams;
}

com.ibm.bpm.social.util.common.Util.escapeChannelSegment = function(channelSegment) {
  return channelSegment.replace(/~/g, "~007e").replace(/\//g, "~002f");
};
 

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SimpleMarkupUtils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SimpleMarkupUtils"] = true;
// SimpleMarkupUtils.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT
dojo.provide("com.ibm.bpm.social.util.common.SimpleMarkupUtils");

com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags = ["span","b"];

com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup = function(markup){
	var result = '';
	if(markup !== undefined && markup !== null) {
		var temp = '';
		var quote = '';
		var tag = false;
		var i = 0;
		while (i < markup.length) {
			var c = markup.charAt(i);
			switch (c) {
			case '<':
				if (quote) {
					temp = temp + c;
				} else {
					result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(temp);
					temp = c;
					tag = true;
				}
				break;
			case '>':
				temp = temp + c;
				if (tag && !quote) {
					result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._checkTag(temp);
					temp = '';
					tag = false;
				}
				break;
			case '\'':
			case '\"':
				temp = temp + c;
				if (quote && (quote == c)) {
					quote = '';
				} else if (tag && !quote) {
					quote = c;
				}
				break;
			case '\\':
				temp = temp + c;
				i++;
				if (i < markup.length) {
					c = markup.charAt(i);
					temp = temp + c;
				}
				break;
			default:
				temp = temp + c;
			break;
			}
			i++;
		}
		if (temp) {
			result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(temp); 
		}
	}
	return result;
};

com.ibm.bpm.social.util.common.SimpleMarkupUtils._checkTag = function(tag) {
	var supported = false;
	var lc = tag.toLowerCase();
	for (i=0; i<com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags.length; i++) {
		var test = com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags[i];
		if (lc.indexOf('<'+test+' ') == 0) {
			supported = true;
			var tmp = document.createElement('div');
			tmp.innerHTML = tag + '</'+test+'>';
			tag = ('<'+test+' ');
			if (tmp.firstChild.className) {
				tag = tag + 'class="'+tmp.firstChild.className+'" ' + ' ';
			}
			if (tmp.firstChild.objid) {
				tag = tag + 'objid="'+tmp.firstChild.objid+'" ' + ' ';
			}
			tag = tag + '>';			
			delete tmp;
		}
		else if (lc.indexOf('<'+test+'>') == 0) {
			supported = true;
		}
		else if (lc.indexOf('<'+test+'/>') == 0) {
			supported = true;
		}
		else if (lc.indexOf('</'+test+' ') == 0) {
			supported = true;
		}
		else if (lc.indexOf('</'+test+'>') == 0) {
			supported = true;
		}
	}
	if (!supported) {
		tag = com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(tag);
	}	
	return tag;
}

com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize = function(markup) {
	var div = document.createElement('div');
	var textNode = document.createTextNode(markup);
	div.appendChild(textNode);
	return div.innerHTML;
}

}

if(!dojo._hasResource['org.cometd']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['org.cometd'] = true;
/*
 * Copyright (c) 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

if (typeof dojo !== 'undefined')
{
    dojo.provide('org.cometd');
}
else
{
    // Namespaces for the cometd implementation
    this.org = this.org || {};
    org.cometd = {};
}

org.cometd.JSON = {};
org.cometd.JSON.toJSON = org.cometd.JSON.fromJSON = function(object)
{
    throw 'Abstract';
};

org.cometd.Utils = {};

org.cometd.Utils.isString = function(value)
{
    if (value === undefined || value === null)
    {
        return false;
    }
    return typeof value === 'string' ||  value instanceof String;
};

org.cometd.Utils.isArray = function(value)
{
    if (value === undefined || value === null)
    {
        return false;
    }
    return value instanceof Array;
};

/**
 * Returns whether the given element is contained into the given array.
 * @param element the element to check presence for
 * @param array the array to check for the element presence
 * @return the index of the element, if present, or a negative index if the element is not present
 */
org.cometd.Utils.inArray = function(element, array)
{
    for (var i = 0; i < array.length; ++i)
    {
        if (element === array[i])
        {
            return i;
        }
    }
    return -1;
};

org.cometd.Utils.setTimeout = function(cometd, funktion, delay)
{
    return window.setTimeout(function()
    {
        try
        {
            funktion();
        }
        catch (x)
        {
            cometd._debug('Exception invoking timed function', funktion, x);
        }
    }, delay);
};

org.cometd.Utils.clearTimeout = function(timeoutHandle)
{
    window.clearTimeout(timeoutHandle);
};

/**
 * A registry for transports used by the Cometd object.
 */
org.cometd.TransportRegistry = function()
{
    var _types = [];
    var _transports = {};

    this.getTransportTypes = function()
    {
        return _types.slice(0);
    };

    this.findTransportTypes = function(version, crossDomain, url)
    {
        var result = [];
        for (var i = 0; i < _types.length; ++i)
        {
            var type = _types[i];
            if (_transports[type].accept(version, crossDomain, url) === true)
            {
                result.push(type);
            }
        }
        return result;
    };

    this.negotiateTransport = function(types, version, crossDomain, url)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            var type = _types[i];
            for (var j = 0; j < types.length; ++j)
            {
                if (type === types[j])
                {
                    var transport = _transports[type];
                    if (transport.accept(version, crossDomain, url) === true)
                    {
                        return transport;
                    }
                }
            }
        }
        return null;
    };

    this.add = function(type, transport, index)
    {
        var existing = false;
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                existing = true;
                break;
            }
        }

        if (!existing)
        {
            if (typeof index !== 'number')
            {
                _types.push(type);
            }
            else
            {
                _types.splice(index, 0, type);
            }
            _transports[type] = transport;
        }

        return !existing;
    };

    this.find = function(type)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                return _transports[type];
            }
        }
        return null;
    };

    this.remove = function(type)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                _types.splice(i, 1);
                var transport = _transports[type];
                delete _transports[type];
                return transport;
            }
        }
        return null;
    };

    this.clear = function()
    {
        _types = [];
        _transports = {};
    };

    this.reset = function()
    {
        for (var i = 0; i < _types.length; ++i)
        {
            _transports[_types[i]].reset();
        }
    };
};

/**
 * Base object with the common functionality for transports.
 */
org.cometd.Transport = function()
{
    var _type;
    var _cometd;

    /**
     * Function invoked just after a transport has been successfully registered.
     * @param type the type of transport (for example 'long-polling')
     * @param cometd the cometd object this transport has been registered to
     * @see #unregistered()
     */
    this.registered = function(type, cometd)
    {
        _type = type;
        _cometd = cometd;
    };

    /**
     * Function invoked just after a transport has been successfully unregistered.
     * @see #registered(type, cometd)
     */
    this.unregistered = function()
    {
        _type = null;
        _cometd = null;
    };

    this._debug = function()
    {
        _cometd._debug.apply(_cometd, arguments);
    };

    this._mixin = function()
    {
        return _cometd._mixin.apply(_cometd, arguments);
    };

    this.getConfiguration = function()
    {
        return _cometd.getConfiguration();
    };

    this.getAdvice = function()
    {
        return _cometd.getAdvice();
    };

    this.setTimeout = function(funktion, delay)
    {
        return org.cometd.Utils.setTimeout(_cometd, funktion, delay);
    };

    this.clearTimeout = function(handle)
    {
        org.cometd.Utils.clearTimeout(handle);
    };

    /**
     * Converts the given response into an array of bayeux messages
     * @param response the response to convert
     * @return an array of bayeux messages obtained by converting the response
     */
    this.convertToMessages = function (response)
    {
        if (org.cometd.Utils.isString(response))
        {
            try
            {
                return org.cometd.JSON.fromJSON(response);
            }
            catch(x)
            {
                this._debug('Could not convert to JSON the following string', '"' + response + '"');
                throw x;
            }
        }
        if (org.cometd.Utils.isArray(response))
        {
            return response;
        }
        if (response === undefined || response === null)
        {
            return [];
        }
        if (response instanceof Object)
        {
            return [response];
        }
        throw 'Conversion Error ' + response + ', typeof ' + (typeof response);
    };

    /**
     * Returns whether this transport can work for the given version and cross domain communication case.
     * @param version a string indicating the transport version
     * @param crossDomain a boolean indicating whether the communication is cross domain
     * @return true if this transport can work for the given version and cross domain communication case,
     * false otherwise
     */
    this.accept = function(version, crossDomain, url)
    {
        throw 'Abstract';
    };

    /**
     * Returns the type of this transport.
     * @see #registered(type, cometd)
     */
    this.getType = function()
    {
        return _type;
    };

    this.send = function(envelope, metaConnect)
    {
        throw 'Abstract';
    };

    this.reset = function()
    {
        this._debug('Transport', _type, 'reset');
    };

    this.abort = function()
    {
        this._debug('Transport', _type, 'aborted');
    };

    this.toString = function()
    {
        return this.getType();
    };
};

org.cometd.Transport.derive = function(baseObject)
{
    function F() {}
    F.prototype = baseObject;
    return new F();
};

/**
 * Base object with the common functionality for transports based on requests.
 * The key responsibility is to allow at most 2 outstanding requests to the server,
 * to avoid that requests are sent behind a long poll.
 * To achieve this, we have one reserved request for the long poll, and all other
 * requests are serialized one after the other.
 */
org.cometd.RequestTransport = function()
{
    var _super = new org.cometd.Transport();
    var _self = org.cometd.Transport.derive(_super);
    var _requestIds = 0;
    var _metaConnectRequest = null;
    var _requests = [];
    var _envelopes = [];

    function _coalesceEnvelopes(envelope)
    {
        while (_envelopes.length > 0)
        {
            var envelopeAndRequest = _envelopes[0];
            var newEnvelope = envelopeAndRequest[0];
            var newRequest = envelopeAndRequest[1];
            if (newEnvelope.url === envelope.url &&
                    newEnvelope.sync === envelope.sync)
            {
                _envelopes.shift();
                envelope.messages = envelope.messages.concat(newEnvelope.messages);
                this._debug('Coalesced', newEnvelope.messages.length, 'messages from request', newRequest.id);
                continue;
            }
            break;
        }
    }

    function _transportSend(envelope, request)
    {
        this.transportSend(envelope, request);
        request.expired = false;

        if (!envelope.sync)
        {
            var maxDelay = this.getConfiguration().maxNetworkDelay;
            var delay = maxDelay;
            if (request.metaConnect === true)
            {
                delay += this.getAdvice().timeout;
            }

            this._debug('Transport', this.getType(), 'waiting at most', delay, 'ms for the response, maxNetworkDelay', maxDelay);

            var self = this;
            request.timeout = this.setTimeout(function()
            {
                request.expired = true;
                if (request.xhr)
                {
                    request.xhr.abort();
                }
                var errorMessage = 'Request ' + request.id + ' of transport ' + self.getType() + ' exceeded ' + delay + ' ms max network delay';
                self._debug(errorMessage);
                self.complete(request, false, request.metaConnect);
                envelope.onFailure(request.xhr, envelope.messages, 'timeout', errorMessage);
            }, delay);
        }
    }

    function _queueSend(envelope)
    {
        var requestId = ++_requestIds;
        var request = {
            id: requestId,
            metaConnect: false
        };

        // Consider the metaConnect requests which should always be present
        if (_requests.length < this.getConfiguration().maxConnections - 1)
        {
            _requests.push(request);
            _transportSend.call(this, envelope, request);
        }
        else
        {
            this._debug('Transport', this.getType(), 'queueing request', requestId, 'envelope', envelope);
            _envelopes.push([envelope, request]);
        }
    }

    function _metaConnectComplete(request)
    {
        var requestId = request.id;
        this._debug('Transport', this.getType(), 'metaConnect complete, request', requestId);
        if (_metaConnectRequest !== null && _metaConnectRequest.id !== requestId)
        {
            throw 'Longpoll request mismatch, completing request ' + requestId;
        }

        // Reset metaConnect request
        _metaConnectRequest = null;
    }

    function _complete(request, success)
    {
        var index = org.cometd.Utils.inArray(request, _requests);
        // The index can be negative if the request has been aborted
        if (index >= 0)
        {
            _requests.splice(index, 1);
        }

        if (_envelopes.length > 0)
        {
            var envelopeAndRequest = _envelopes.shift();
            var nextEnvelope = envelopeAndRequest[0];
            var nextRequest = envelopeAndRequest[1];
            this._debug('Transport dequeued request', nextRequest.id);
            if (success)
            {
                if (this.getConfiguration().autoBatch)
                {
                    _coalesceEnvelopes.call(this, nextEnvelope);
                }
                _queueSend.call(this, nextEnvelope);
                this._debug('Transport completed request', request.id, nextEnvelope);
            }
            else
            {
                // Keep the semantic of calling response callbacks asynchronously after the request
                var self = this;
                this.setTimeout(function()
                {
                    self.complete(nextRequest, false, nextRequest.metaConnect);
                    nextEnvelope.onFailure(nextRequest.xhr, nextEnvelope.messages, 'error', 'Previous request failed');
                }, 0);
            }
        }
    }

    _self.complete = function(request, success, metaConnect)
    {
        if (metaConnect)
        {
            _metaConnectComplete.call(this, request);
        }
        else
        {
            _complete.call(this, request, success);
        }
    };

    /**
     * Performs the actual send depending on the transport type details.
     * @param envelope the envelope to send
     * @param request the request information
     */
    _self.transportSend = function(envelope, request)
    {
        throw 'Abstract';
    };

    _self.transportSuccess = function(envelope, request, responses)
    {
        if (!request.expired)
        {
            this.clearTimeout(request.timeout);
            this.complete(request, true, request.metaConnect);
            if (responses && responses.length > 0)
            {
                envelope.onSuccess(responses);
            }
            else
            {
                envelope.onFailure(request.xhr, envelope.messages, 'Empty HTTP response');
            }
        }
    };

    _self.transportFailure = function(envelope, request, reason, exception)
    {
        if (!request.expired)
        {
            this.clearTimeout(request.timeout);
            this.complete(request, false, request.metaConnect);
            envelope.onFailure(request.xhr, envelope.messages, reason, exception);
        }
    };

    function _metaConnectSend(envelope)
    {
        if (_metaConnectRequest !== null)
        {
            throw 'Concurrent metaConnect requests not allowed, request id=' + _metaConnectRequest.id + ' not yet completed';
        }

        var requestId = ++_requestIds;
        this._debug('Transport', this.getType(), 'metaConnect send, request', requestId, 'envelope', envelope);
        var request = {
            id: requestId,
            metaConnect: true
        };
        _transportSend.call(this, envelope, request);
        _metaConnectRequest = request;
    }

    _self.send = function(envelope, metaConnect)
    {
        if (metaConnect)
        {
            _metaConnectSend.call(this, envelope);
        }
        else
        {
            _queueSend.call(this, envelope);
        }
    };

    _self.abort = function()
    {
        _super.abort();
        for (var i = 0; i < _requests.length; ++i)
        {
            var request = _requests[i];
            this._debug('Aborting request', request);
            if (request.xhr)
            {
                request.xhr.abort();
            }
        }
        if (_metaConnectRequest)
        {
            this._debug('Aborting metaConnect request', _metaConnectRequest);
            if (_metaConnectRequest.xhr)
            {
                _metaConnectRequest.xhr.abort();
            }
        }
        this.reset();
    };

    _self.reset = function()
    {
        _super.reset();
        _metaConnectRequest = null;
        _requests = [];
        _envelopes = [];
    };

    return _self;
};

org.cometd.LongPollingTransport = function()
{
    var _super = new org.cometd.RequestTransport();
    var _self = org.cometd.Transport.derive(_super);
    // By default, support cross domain
    var _supportsCrossDomain = true;

    _self.accept = function(version, crossDomain, url)
    {
        return _supportsCrossDomain || !crossDomain;
    };

    _self.xhrSend = function(packet)
    {
        throw 'Abstract';
    };

    _self.transportSend = function(envelope, request)
    {
        this._debug('Transport', this.getType(), 'sending request', request.id, 'envelope', envelope);

        var self = this;
        try
        {
            var sameStack = true;
            request.xhr = this.xhrSend({
                transport: this,
                url: envelope.url,
                sync: envelope.sync,
                headers: this.getConfiguration().requestHeaders,
                body: org.cometd.JSON.toJSON(envelope.messages),
                onSuccess: function(response)
                {
                    self._debug('Transport', self.getType(), 'received response', response);
                    var success = false;
                    try
                    {
                        var received = self.convertToMessages(response);
                        if (received.length === 0)
                        {
                            _supportsCrossDomain = false;
                            self.transportFailure(envelope, request, 'no response', null);
                        }
                        else
                        {
                            success = true;
                            self.transportSuccess(envelope, request, received);
                        }
                    }
                    catch(x)
                    {
                        self._debug(x);
                        if (!success)
                        {
                            _supportsCrossDomain = false;
                            self.transportFailure(envelope, request, 'bad response', x);
                        }
                    }
                },
                onError: function(reason, exception)
                {
                    _supportsCrossDomain = false;
                    if (sameStack)
                    {
                        // Keep the semantic of calling response callbacks asynchronously after the request
                        self.setTimeout(function()
                        {
                            self.transportFailure(envelope, request, reason, exception);
                        }, 0);
                    }
                    else
                    {
                        self.transportFailure(envelope, request, reason, exception);
                    }
                }
            });
            sameStack = false;
        }
        catch (x)
        {
            _supportsCrossDomain = false;
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                self.transportFailure(envelope, request, 'error', x);
            }, 0);
        }
    };

    _self.reset = function()
    {
        _super.reset();
        _supportsCrossDomain = true;
    };

    return _self;
};

org.cometd.CallbackPollingTransport = function()
{
    var _super = new org.cometd.RequestTransport();
    var _self = org.cometd.Transport.derive(_super);
    var _maxLength = 2000;

    _self.accept = function(version, crossDomain, url)
    {
        return true;
    };

    _self.jsonpSend = function(packet)
    {
        throw 'Abstract';
    };

    _self.transportSend = function(envelope, request)
    {
        var self = this;

        // Microsoft Internet Explorer has a 2083 URL max length
        // We must ensure that we stay within that length
        var start = 0;
        var length = envelope.messages.length;
        var lengths = [];
        while (length > 0)
        {
            // Encode the messages because all brackets, quotes, commas, colons, etc
            // present in the JSON will be URL encoded, taking many more characters
            var json = org.cometd.JSON.toJSON(envelope.messages.slice(start, start + length));
            var urlLength = envelope.url.length + encodeURI(json).length;

            // Let's stay on the safe side and use 2000 instead of 2083
            // also because we did not count few characters among which
            // the parameter name 'message' and the parameter 'jsonp',
            // which sum up to about 50 chars
            if (urlLength > _maxLength)
            {
                if (length === 1)
                {
                    var x = 'Bayeux message too big (' + urlLength + ' bytes, max is ' + _maxLength + ') ' +
                            'for transport ' + this.getType();
                    // Keep the semantic of calling response callbacks asynchronously after the request
                    this.setTimeout(function()
                    {
                        self.transportFailure(envelope, request, 'error', x);
                    }, 0);
                    return;
                }

                --length;
                continue;
            }

            lengths.push(length);
            start += length;
            length = envelope.messages.length - start;
        }

        // Here we are sure that the messages can be sent within the URL limit

        var envelopeToSend = envelope;
        if (lengths.length > 1)
        {
            var begin = 0;
            var end = lengths[0];
            this._debug('Transport', this.getType(), 'split', envelope.messages.length, 'messages into', lengths.join(' + '));
            envelopeToSend = this._mixin(false, {}, envelope);
            envelopeToSend.messages = envelope.messages.slice(begin, end);
            envelopeToSend.onSuccess = envelope.onSuccess;
            envelopeToSend.onFailure = envelope.onFailure;

            for (var i = 1; i < lengths.length; ++i)
            {
                var nextEnvelope = this._mixin(false, {}, envelope);
                begin = end;
                end += lengths[i];
                nextEnvelope.messages = envelope.messages.slice(begin, end);
                nextEnvelope.onSuccess = envelope.onSuccess;
                nextEnvelope.onFailure = envelope.onFailure;
                this.send(nextEnvelope, request.metaConnect);
            }
        }

        this._debug('Transport', this.getType(), 'sending request', request.id, 'envelope', envelopeToSend);

        try
        {
            var sameStack = true;
            this.jsonpSend({
                transport: this,
                url: envelopeToSend.url,
                sync: envelopeToSend.sync,
                headers: this.getConfiguration().requestHeaders,
                body: org.cometd.JSON.toJSON(envelopeToSend.messages),
                onSuccess: function(responses)
                {
                    var success = false;
                    try
                    {
                        var received = self.convertToMessages(responses);
                        if (received.length === 0)
                        {
                            self.transportFailure(envelopeToSend, request, 'no response');
                        }
                        else
                        {
                            success=true;
                            self.transportSuccess(envelopeToSend, request, received);
                        }
                    }
                    catch (x)
                    {
                        self._debug(x);
                        if (!success)
                        {
                            self.transportFailure(envelopeToSend, request, 'bad response', x);
                        }
                    }
                },
                onError: function(reason, exception)
                {
                    if (sameStack)
                    {
                        // Keep the semantic of calling response callbacks asynchronously after the request
                        self.setTimeout(function()
                        {
                            self.transportFailure(envelopeToSend, request, reason, exception);
                        }, 0);
                    }
                    else
                    {
                        self.transportFailure(envelopeToSend, request, reason, exception);
                    }
                }
            });
            sameStack = false;
        }
        catch (xx)
        {
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                self.transportFailure(envelopeToSend, request, 'error', xx);
            }, 0);
        }
    };

    return _self;
};

org.cometd.WebSocketTransport = function()
{
    var _super = new org.cometd.Transport();
    var _self = org.cometd.Transport.derive(_super);
    var _cometd;
    // By default, support WebSocket
    var _supportsWebSocket = true;
    // Whether we were able to establish a WebSocket connection
    var _webSocketSupported = false;
    // Envelopes that have been sent
    var _envelopes = {};
    // Timeouts for messages that have been sent
    var _timeouts = {};
    var _webSocket = null;
    var _opened = false;
    var _connected = false;
    var _successCallback;

    function _websocketConnect()
    {
        // Mangle the URL, changing the scheme from 'http' to 'ws'
        var url = _cometd.getURL().replace(/^http/, 'ws');
        this._debug('Transport', this.getType(), 'connecting to URL', url);

        var webSocket = new org.cometd.WebSocket(url);
        var self = this;
        webSocket.onopen = function()
        {
            self._debug('WebSocket opened', webSocket);
            if (webSocket !== _webSocket)
            {
                // It's possible that the onopen callback is invoked
                // with a delay so that we have already reconnected
                self._debug('Ignoring open event, WebSocket', _webSocket);
                return;
            }
            self.onOpen();
        };
        webSocket.onclose = function(event)
        {
            var code = event ? event.code : 1000;
            var reason = event ? event.reason : undefined;
            self._debug('WebSocket closed', code, '/', reason, webSocket);
            if (webSocket !== _webSocket)
            {
                // The onclose callback may be invoked when the server sends
                // the close message reply, but after we have already reconnected
                self._debug('Ignoring close event, WebSocket', _webSocket);
                return;
            }
            self.onClose(code, reason);
        };
        webSocket.onerror = function()
        {
            webSocket.onclose({ code: 1002 });
        };
        webSocket.onmessage = function(message)
        {
            self._debug('WebSocket message', message, webSocket);
            if (webSocket !== _webSocket)
            {
                self._debug('Ignoring message event, WebSocket', _webSocket);
                return;
            }
            self.onMessage(message);
        };

        _webSocket = webSocket;
        this._debug('Transport', this.getType(), 'configured callbacks on', webSocket);
    }

    function _webSocketSend(envelope, metaConnect)
    {
        var json = org.cometd.JSON.toJSON(envelope.messages);

        _webSocket.send(json);
        this._debug('Transport', this.getType(), 'sent', envelope, 'metaConnect =', metaConnect);

        // Manage the timeout waiting for the response
        var maxDelay = this.getConfiguration().maxNetworkDelay;
        var delay = maxDelay;
        if (metaConnect)
        {
            delay += this.getAdvice().timeout;
            _connected = true;
        }

        var messageIds = [];
        for (var i = 0; i < envelope.messages.length; ++i)
        {
            var message = envelope.messages[i];
            if (message.id)
            {
                messageIds.push(message.id);
                var self = this;
                _timeouts[message.id] = this.setTimeout(function()
                {
                    if (_webSocket)
                    {
                        _webSocket.close(1000, 'Timeout');
                    }
                }, delay);
            }
        }

        this._debug('Transport', this.getType(), 'waiting at most', delay, 'ms for messages', messageIds, 'maxNetworkDelay', maxDelay, ', timeouts:', _timeouts);
    }

    function _send(envelope, metaConnect)
    {
        try
        {
            if (_webSocket === null)
            {
                _websocketConnect.call(this);
            }
            // We may have a non-null _webSocket, but not be open yet so
            // to avoid out of order deliveries, we check if we are open
            else if (_opened)
            {
                _webSocketSend.call(this, envelope, metaConnect);
            }
        }
        catch (x)
        {
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                envelope.onFailure(_webSocket, envelope.messages, 'error', x);
            }, 0);
        }
    }

    _self.onOpen = function()
    {
        this._debug('Transport', this.getType(), 'opened', _webSocket);
        _opened = true;
        _webSocketSupported = true;

        this._debug('Sending pending messages', _envelopes);
        for (var key in _envelopes)
        {
            var element = _envelopes[key];
            var envelope = element[0];
            var metaConnect = element[1];
            // Store the success callback, which is independent from the envelope,
            // so that it can be used to notify arrival of messages.
            _successCallback = envelope.onSuccess;
            _webSocketSend.call(this, envelope, metaConnect);
        }
    };

    _self.onMessage = function(wsMessage)
    {
        this._debug('Transport', this.getType(), 'received websocket message', wsMessage, _webSocket);

        var close = false;
        var messages = this.convertToMessages(wsMessage.data);
        var messageIds = [];
        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];

            // Detect if the message is a response to a request we made.
            // If it's a meta message, for sure it's a response;
            // otherwise it's a publish message and publish responses lack the data field
            if (/^\/meta\//.test(message.channel) || message.data === undefined)
            {
                if (message.id)
                {
                    messageIds.push(message.id);

                    var timeout = _timeouts[message.id];
                    if (timeout)
                    {
                        this.clearTimeout(timeout);
                        delete _timeouts[message.id];
                        this._debug('Transport', this.getType(), 'removed timeout for message', message.id, ', timeouts', _timeouts);
                    }
                }
            }

            if ('/meta/connect' === message.channel)
            {
                _connected = false;
            }
            if ('/meta/disconnect' === message.channel && !_connected)
            {
                close = true;
            }
        }

        // Remove the envelope corresponding to the messages
        var removed = false;
        for (var j = 0; j < messageIds.length; ++j)
        {
            var id = messageIds[j];
            for (var key in _envelopes)
            {
                var ids = key.split(',');
                var index = org.cometd.Utils.inArray(id, ids);
                if (index >= 0)
                {
                    removed = true;
                    ids.splice(index, 1);
                    var envelope = _envelopes[key][0];
                    var metaConnect = _envelopes[key][1];
                    delete _envelopes[key];
                    if (ids.length > 0)
                    {
                        _envelopes[ids.join(',')] = [envelope, metaConnect];
                    }
                    break;
                }
            }
        }
        if (removed)
        {
            this._debug('Transport', this.getType(), 'removed envelope, envelopes', _envelopes);
        }

        _successCallback.call(this, messages);

        if (close)
        {
            _webSocket.close(1000, 'Disconnect');
        }
    };

    _self.onClose = function(code, reason)
    {
        this._debug('Transport', this.getType(), 'closed', code, reason, _webSocket);

        // Remember if we were able to connect
        // This close event could be due to server shutdown, and if it restarts we want to try websocket again
        _supportsWebSocket = _webSocketSupported;

        for (var id in _timeouts)
        {
            this.clearTimeout(_timeouts[id]);
        }
        _timeouts = {};

        for (var key in _envelopes)
        {
            var envelope = _envelopes[key][0];
            var metaConnect = _envelopes[key][1];
            if (metaConnect)
            {
                _connected = false;
            }
            envelope.onFailure(_webSocket, envelope.messages, 'closed ' + code + '/' + reason);
        }
        _envelopes = {};

        if (_webSocket !== null && _opened)
        {
            _webSocket.close(1000, 'Close');
        }
        _opened = false;
        _webSocket = null;
    };

    _self.registered = function(type, cometd)
    {
        _super.registered(type, cometd);
        _cometd = cometd;
    };

    _self.accept = function(version, crossDomain, url)
    {
        // Using !! to return a boolean (and not the WebSocket object)
        return _supportsWebSocket && !!org.cometd.WebSocket && _cometd.websocketEnabled !== false;
    };

    _self.send = function(envelope, metaConnect)
    {
        this._debug('Transport', this.getType(), 'sending', envelope, 'metaConnect =', metaConnect);

        // Store the envelope in any case; if the websocket cannot be opened, we fail it in close()
        var messageIds = [];
        for (var i = 0; i < envelope.messages.length; ++i)
        {
            var message = envelope.messages[i];
            if (message.id)
            {
                messageIds.push(message.id);
            }
        }
        _envelopes[messageIds.join(',')] = [envelope, metaConnect];
        this._debug('Transport', this.getType(), 'stored envelope, envelopes', _envelopes);

        _send.call(this, envelope, metaConnect);
    };

    _self.reset = function()
    {
        _super.reset();
        if (_webSocket !== null && _opened)
        {
            _webSocket.close(1000, 'Reset');
        }
        _supportsWebSocket = true;
        _webSocketSupported = false;
        _timeouts = {};
        _envelopes = {};
        _webSocket = null;
        _opened = false;
        _successCallback = null;
    };

    return _self;
};

/**
 * The constructor for a Cometd object, identified by an optional name.
 * The default name is the string 'default'.
 * In the rare case a page needs more than one Bayeux conversation,
 * a new instance can be created via:
 * <pre>
 * var bayeuxUrl2 = ...;
 *
 * // Dojo style
 * var cometd2 = new dojox.Cometd('another_optional_name');
 *
 * // jQuery style
 * var cometd2 = new $.Cometd('another_optional_name');
 *
 * cometd2.init({url: bayeuxUrl2});
 * </pre>
 * @param name the optional name of this cometd object
 */
// IMPLEMENTATION NOTES:
// Be very careful in not changing the function order and pass this file every time through JSLint (http://jslint.com)
// The only implied globals must be "dojo", "org" and "window", and check that there are no "unused" warnings
// Failing to pass JSLint may result in shrinkers/minifiers to create an unusable file.
org.cometd.Cometd = function(name)
{
    var _cometd = this;
    var _name = name || 'default';
    var _crossDomain = false;
    var _transports = new org.cometd.TransportRegistry();
    var _transport;
    var _status = 'disconnected';
    var _messageId = 0;
    var _clientId = null;
    var _batch = 0;
    var _messageQueue = [];
    var _internalBatch = false;
    var _listeners = {};
    var _backoff = 0;
    var _scheduledSend = null;
    var _extensions = [];
    var _advice = {};
    var _handshakeProps;
    var _reestablish = false;
    var _connected = false;
    var _config = {
        maxConnections: 2,
        backoffIncrement: 1000,
        maxBackoff: 60000,
        logLevel: 'info',
        reverseIncomingExtensions: true,
        maxNetworkDelay: 10000,
        requestHeaders: {},
        appendMessageTypeToURL: true,
        autoBatch: false,
        advice: {
            timeout: 60000,
            interval: 0,
            reconnect: 'retry'
        }
    };

    /**
     * Mixes in the given objects into the target object by copying the properties.
     * @param deep if the copy must be deep
     * @param target the target object
     * @param objects the objects whose properties are copied into the target
     */
    this._mixin = function(deep, target, objects)
    {
        var result = target || {};

        // Skip first 2 parameters (deep and target), and loop over the others
        for (var i = 2; i < arguments.length; ++i)
        {
            var object = arguments[i];

            if (object === undefined || object === null)
            {
                continue;
            }

            for (var propName in object)
            {
                var prop = object[propName];
                var targ = result[propName];

                // Avoid infinite loops
                if (prop === target)
                {
                    continue;
                }
                // Do not mixin undefined values
                if (prop === undefined)
                {
                    continue;
                }

                if (deep && typeof prop === 'object' && prop !== null)
                {
                    if (prop instanceof Array)
                    {
                        result[propName] = this._mixin(deep, targ instanceof Array ? targ : [], prop);
                    }
                    else
                    {
                        var source = typeof targ === 'object' && !(targ instanceof Array) ? targ : {};
                        result[propName] = this._mixin(deep, source, prop);
                    }
                }
                else
                {
                    result[propName] = prop;
                }
            }
        }

        return result;
    };

    function _isString(value)
    {
        return org.cometd.Utils.isString(value);
    }

    function _isFunction(value)
    {
        if (value === undefined || value === null)
        {
            return false;
        }
        return typeof value === 'function';
    }

    function _log(level, args)
    {
        if (window.console)
        {
            var logger = window.console[level];
            if (_isFunction(logger))
            {
                logger.apply(window.console, args);
            }
        }
    }

    this._warn = function()
    {
        _log('warn', arguments);
    };

    this._info = function()
    {
        if (_config.logLevel !== 'warn')
        {
            _log('info', arguments);
        }
    };

    this._debug = function()
    {
        if (_config.logLevel === 'debug')
        {
            _log('debug', arguments);
        }
    };

    /**
     * Returns whether the given hostAndPort is cross domain.
     * The default implementation checks against window.location.host
     * but this function can be overridden to make it work in non-browser
     * environments.
     *
     * @param hostAndPort the host and port in format host:port
     * @return whether the given hostAndPort is cross domain
     */
    this._isCrossDomain = function(hostAndPort)
    {
        return hostAndPort && hostAndPort !== window.location.host;
    };

    function _configure(configuration)
    {
        _cometd._debug('Configuring cometd object with', configuration);
        // Support old style param, where only the Bayeux server URL was passed
        if (_isString(configuration))
        {
            configuration = { url: configuration };
        }
        if (!configuration)
        {
            configuration = {};
        }

        _config = _cometd._mixin(false, _config, configuration);

        if (!_config.url)
        {
            throw 'Missing required configuration parameter \'url\' specifying the Bayeux server URL';
        }

        // Check if we're cross domain
        // [1] = protocol://, [2] = host:port, [3] = host, [4] = IPv6_host, [5] = IPv4_host, [6] = :port, [7] = port, [8] = uri, [9] = rest
        var urlParts = /(^https?:\/\/)?(((\[[^\]]+\])|([^:\/\?#]+))(:(\d+))?)?([^\?#]*)(.*)?/.exec(_config.url);
        var hostAndPort = urlParts[2];
        var uri = urlParts[8];
        var afterURI = urlParts[9];
        _crossDomain = _cometd._isCrossDomain(hostAndPort);

        // Check if appending extra path is supported
        if (_config.appendMessageTypeToURL)
        {
            if (afterURI !== undefined && afterURI.length > 0)
            {
                _cometd._info('Appending message type to URI ' + uri + afterURI + ' is not supported, disabling \'appendMessageTypeToURL\' configuration');
                _config.appendMessageTypeToURL = false;
            }
            else
            {
                var uriSegments = uri.split('/');
                var lastSegmentIndex = uriSegments.length - 1;
                if (uri.match(/\/$/))
                {
                    lastSegmentIndex -= 1;
                }
                if (uriSegments[lastSegmentIndex].indexOf('.') >= 0)
                {
                    // Very likely the CometD servlet's URL pattern is mapped to an extension, such as *.cometd
                    // It will be difficult to add the extra path in this case
                    _cometd._info('Appending message type to URI ' + uri + ' is not supported, disabling \'appendMessageTypeToURL\' configuration');
                    _config.appendMessageTypeToURL = false;
                }
            }
        }
    }

    function _clearSubscriptions()
    {
        for (var channel in _listeners)
        {
            var subscriptions = _listeners[channel];
            for (var i = 0; i < subscriptions.length; ++i)
            {
                var subscription = subscriptions[i];
                if (subscription && !subscription.listener)
                {
                    delete subscriptions[i];
                    _cometd._debug('Removed subscription', subscription, 'for channel', channel);
                }
            }
        }
    }

    function _setStatus(newStatus)
    {
        if (_status !== newStatus)
        {
            _cometd._debug('Status', _status, '->', newStatus);
            _status = newStatus;
        }
    }

    function _isDisconnected()
    {
        return _status === 'disconnecting' || _status === 'disconnected';
    }

    function _nextMessageId()
    {
        return ++_messageId;
    }

    function _applyExtension(scope, callback, name, message, outgoing)
    {
        try
        {
            return callback.call(scope, message);
        }
        catch (x)
        {
            _cometd._debug('Exception during execution of extension', name, x);
            var exceptionCallback = _cometd.onExtensionException;
            if (_isFunction(exceptionCallback))
            {
                _cometd._debug('Invoking extension exception callback', name, x);
                try
                {
                    exceptionCallback.call(_cometd, x, name, outgoing, message);
                }
                catch(xx)
                {
                    _cometd._info('Exception during execution of exception callback in extension', name, xx);
                }
            }
            return message;
        }
    }

    function _applyIncomingExtensions(message)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            if (message === undefined || message === null)
            {
                break;
            }

            var index = _config.reverseIncomingExtensions ? _extensions.length - 1 - i : i;
            var extension = _extensions[index];
            var callback = extension.extension.incoming;
            if (_isFunction(callback))
            {
                var result = _applyExtension(extension.extension, callback, extension.name, message, false);
                message = result === undefined ? message : result;
            }
        }
        return message;
    }

    function _applyOutgoingExtensions(message)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            if (message === undefined || message === null)
            {
                break;
            }

            var extension = _extensions[i];
            var callback = extension.extension.outgoing;
            if (_isFunction(callback))
            {
                var result = _applyExtension(extension.extension, callback, extension.name, message, true);
                message = result === undefined ? message : result;
            }
        }
        return message;
    }

    function _notify(channel, message)
    {
        var subscriptions = _listeners[channel];
        if (subscriptions && subscriptions.length > 0)
        {
            for (var i = 0; i < subscriptions.length; ++i)
            {
                var subscription = subscriptions[i];
                // Subscriptions may come and go, so the array may have 'holes'
                if (subscription)
                {
                    try
                    {
                        subscription.callback.call(subscription.scope, message);
                    }
                    catch (x)
                    {
                        _cometd._debug('Exception during notification', subscription, message, x);
                        var listenerCallback = _cometd.onListenerException;
                        if (_isFunction(listenerCallback))
                        {
                            _cometd._debug('Invoking listener exception callback', subscription, x);
                            try
                            {
                                listenerCallback.call(_cometd, x, subscription.handle, subscription.listener, message);
                            }
                            catch (xx)
                            {
                                _cometd._info('Exception during execution of listener callback', subscription, xx);
                            }
                        }
                    }
                }
            }
        }
    }

    function _notifyListeners(channel, message)
    {
        // Notify direct listeners
        _notify(channel, message);

        // Notify the globbing listeners
        var channelParts = channel.split('/');
        var last = channelParts.length - 1;
        for (var i = last; i > 0; --i)
        {
            var channelPart = channelParts.slice(0, i).join('/') + '/*';
            // We don't want to notify /foo/* if the channel is /foo/bar/baz,
            // so we stop at the first non recursive globbing
            if (i === last)
            {
                _notify(channelPart, message);
            }
            // Add the recursive globber and notify
            channelPart += '*';
            _notify(channelPart, message);
        }
    }

    function _cancelDelayedSend()
    {
        if (_scheduledSend !== null)
        {
            org.cometd.Utils.clearTimeout(_scheduledSend);
        }
        _scheduledSend = null;
    }

    function _delayedSend(operation)
    {
        _cancelDelayedSend();
        var delay = _advice.interval + _backoff;
        _cometd._debug('Function scheduled in', delay, 'ms, interval =', _advice.interval, 'backoff =', _backoff, operation);
        _scheduledSend = org.cometd.Utils.setTimeout(_cometd, operation, delay);
    }

    // Needed to break cyclic dependencies between function definitions
    var _handleMessages;
    var _handleFailure;

    /**
     * Delivers the messages to the CometD server
     * @param messages the array of messages to send
     * @param longpoll true if this send is a long poll
     */
    function _send(sync, messages, longpoll, extraPath)
    {
        // We must be sure that the messages have a clientId.
        // This is not guaranteed since the handshake may take time to return
        // (and hence the clientId is not known yet) and the application
        // may create other messages.
        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];
            message.id = '' + _nextMessageId();
            if (_clientId)
            {
                message.clientId = _clientId;
            }
            message = _applyOutgoingExtensions(message);
            if (message !== undefined && message !== null)
            {
                messages[i] = message;
            }
            else
            {
                messages.splice(i--, 1);
            }
        }
        if (messages.length === 0)
        {
            return;
        }

        var url = _config.url;
        if (_config.appendMessageTypeToURL)
        {
            // If url does not end with '/', then append it
            if (!url.match(/\/$/))
            {
                url = url + '/';
            }
            if (extraPath)
            {
                url = url + extraPath;
            }
        }

        var envelope = {
            url: url,
            sync: sync,
            messages: messages,
            onSuccess: function(rcvdMessages)
            {
                try
                {
                    _handleMessages.call(_cometd, rcvdMessages);
                }
                catch (x)
                {
                    _cometd._debug('Exception during handling of messages', x);
                }
            },
            onFailure: function(conduit, messages, reason, exception)
            {
                try
                {
                    _handleFailure.call(_cometd, conduit, messages, reason, exception);
                }
                catch (x)
                {
                    _cometd._debug('Exception during handling of failure', x);
                }
            }
        };
        _cometd._debug('Send', envelope);
        _transport.send(envelope, longpoll);
    }

    function _queueSend(message)
    {
        if (_batch > 0 || _internalBatch === true)
        {
            _messageQueue.push(message);
        }
        else
        {
            _send(false, [message], false);
        }
    }

    /**
     * Sends a complete bayeux message.
     * This method is exposed as a public so that extensions may use it
     * to send bayeux message directly, for example in case of re-sending
     * messages that have already been sent but that for some reason must
     * be resent.
     */
    this.send = _queueSend;

    function _resetBackoff()
    {
        _backoff = 0;
    }

    function _increaseBackoff()
    {
        if (_backoff < _config.maxBackoff)
        {
            _backoff += _config.backoffIncrement;
        }
    }

    /**
     * Starts a the batch of messages to be sent in a single request.
     * @see #_endBatch(sendMessages)
     */
    function _startBatch()
    {
        ++_batch;
    }

    function _flushBatch()
    {
        var messages = _messageQueue;
        _messageQueue = [];
        if (messages.length > 0)
        {
            _send(false, messages, false);
        }
    }

    /**
     * Ends the batch of messages to be sent in a single request,
     * optionally sending messages present in the message queue depending
     * on the given argument.
     * @see #_startBatch()
     */
    function _endBatch()
    {
        --_batch;
        if (_batch < 0)
        {
            throw 'Calls to startBatch() and endBatch() are not paired';
        }

        if (_batch === 0 && !_isDisconnected() && !_internalBatch)
        {
            _flushBatch();
        }
    }

    /**
     * Sends the connect message
     */
    function _connect()
    {
        if (!_isDisconnected())
        {
            var message = {
                channel: '/meta/connect',
                connectionType: _transport.getType()
            };

            // In case of reload or temporary loss of connection
            // we want the next successful connect to return immediately
            // instead of being held by the server, so that connect listeners
            // can be notified that the connection has been re-established
            if (!_connected)
            {
                message.advice = { timeout: 0 };
            }

            _setStatus('connecting');
            _cometd._debug('Connect sent', message);
            _send(false, [message], true, 'connect');
            _setStatus('connected');
        }
    }

    function _delayedConnect()
    {
        _setStatus('connecting');
        _delayedSend(function()
        {
            _connect();
        });
    }

    function _updateAdvice(newAdvice)
    {
        if (newAdvice)
        {
            _advice = _cometd._mixin(false, {}, _config.advice, newAdvice);
            _cometd._debug('New advice', _advice);
        }
    }

    function _disconnect(abort)
    {
        _cancelDelayedSend();
        if (abort)
        {
            _transport.abort();
        }
        _clientId = null;
        _setStatus('disconnected');
        _batch = 0;
        _resetBackoff();

        // Fail any existing queued message
        if (_messageQueue.length > 0)
        {
            _handleFailure.call(_cometd, undefined, _messageQueue, 'error', 'Disconnected');
            _messageQueue = [];
        }
    }

    /**
     * Sends the initial handshake message
     */
    function _handshake(handshakeProps)
    {
        _clientId = null;

        _clearSubscriptions();

        // Reset the transports if we're not retrying the handshake
        if (_isDisconnected())
        {
            _transports.reset();
            _updateAdvice(_config.advice);
        }
        else
        {
            // We are retrying the handshake, either because another handshake failed
            // and we're backing off, or because the server timed us out and asks us to
            // re-handshake: in both cases, make sure that if the handshake succeeds
            // the next action is a connect.
            _updateAdvice(_cometd._mixin(false, _advice, {reconnect: 'retry'}));
        }

        _batch = 0;

        // Mark the start of an internal batch.
        // This is needed because handshake and connect are async.
        // It may happen that the application calls init() then subscribe()
        // and the subscribe message is sent before the connect message, if
        // the subscribe message is not held until the connect message is sent.
        // So here we start a batch to hold temporarily any message until
        // the connection is fully established.
        _internalBatch = true;

        // Save the properties provided by the user, so that
        // we can reuse them during automatic re-handshake
        _handshakeProps = handshakeProps;

        var version = '1.0';

        // Figure out the transports to send to the server
        var transportTypes = _transports.findTransportTypes(version, _crossDomain, _config.url);

        var bayeuxMessage = {
            version: version,
            minimumVersion: '0.9',
            channel: '/meta/handshake',
            supportedConnectionTypes: transportTypes,
            advice: {
                timeout: _advice.timeout,
                interval: _advice.interval
            }
        };
        // Do not allow the user to mess with the required properties,
        // so merge first the user properties and *then* the bayeux message
        var message = _cometd._mixin(false, {}, _handshakeProps, bayeuxMessage);

        // Pick up the first available transport as initial transport
        // since we don't know if the server supports it
        _transport = _transports.negotiateTransport(transportTypes, version, _crossDomain, _config.url);
        _cometd._debug('Initial transport is', _transport.getType());

        // We started a batch to hold the application messages,
        // so here we must bypass it and send immediately.
        _setStatus('handshaking');
        _cometd._debug('Handshake sent', message);
        _send(false, [message], false, 'handshake');
    }

    function _delayedHandshake()
    {
        _setStatus('handshaking');

        // We will call _handshake() which will reset _clientId, but we want to avoid
        // that between the end of this method and the call to _handshake() someone may
        // call publish() (or other methods that call _queueSend()).
        _internalBatch = true;

        _delayedSend(function()
        {
            _handshake(_handshakeProps);
        });
    }

    function _failHandshake(message)
    {
        _notifyListeners('/meta/handshake', message);
        _notifyListeners('/meta/unsuccessful', message);

        // Only try again if we haven't been disconnected and
        // the advice permits us to retry the handshake
        var retry = !_isDisconnected() && _advice.reconnect !== 'none';
        if (retry)
        {
            _increaseBackoff();
            _delayedHandshake();
        }
        else
        {
            _disconnect(false);
        }
    }

    function _handshakeResponse(message)
    {
        if (message.successful)
        {
            // Save clientId, figure out transport, then follow the advice to connect
            _clientId = message.clientId;

            var newTransport = _transports.negotiateTransport(message.supportedConnectionTypes, message.version, _crossDomain, _config.url);
            if (newTransport === null)
            {
                throw 'Could not negotiate transport with server; client ' +
                      _transports.findTransportTypes(message.version, _crossDomain, _config.url) +
                      ', server ' + message.supportedConnectionTypes;
            }
            else if (_transport !== newTransport)
            {
                _cometd._debug('Transport', _transport, '->', newTransport);
                _transport = newTransport;
            }

            // End the internal batch and allow held messages from the application
            // to go to the server (see _handshake() where we start the internal batch).
            _internalBatch = false;
            _flushBatch();

            // Here the new transport is in place, as well as the clientId, so
            // the listeners can perform a publish() if they want.
            // Notify the listeners before the connect below.
            message.reestablish = _reestablish;
            _reestablish = true;
            _notifyListeners('/meta/handshake', message);

            var action = _isDisconnected() ? 'none' : _advice.reconnect;
            switch (action)
            {
                case 'retry':
                    _resetBackoff();
                    _delayedConnect();
                    break;
                case 'none':
                    _disconnect(false);
                    break;
                default:
                    throw 'Unrecognized advice action ' + action;
            }
        }
        else
        {
            _failHandshake(message);
        }
    }

    function _handshakeFailure(xhr, message)
    {
        _failHandshake({
            successful: false,
            failure: true,
            channel: '/meta/handshake',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'retry',
                interval: _backoff
            }
        });
    }

    function _failConnect(message)
    {
        // Notify the listeners after the status change but before the next action
        _notifyListeners('/meta/connect', message);
        _notifyListeners('/meta/unsuccessful', message);

        // This may happen when the server crashed, the current clientId
        // will be invalid, and the server will ask to handshake again
        // Listeners can call disconnect(), so check the state after they run
        var action = _isDisconnected() ? 'none' : _advice.reconnect;
        switch (action)
        {
            case 'retry':
                _delayedConnect();
                _increaseBackoff();
                break;
            case 'handshake':
                // The current transport may be failed (e.g. network disconnection)
                // Reset the transports so the new handshake picks up the right one
                _transports.reset();
                _resetBackoff();
                _delayedHandshake();
                break;
            case 'none':
                _disconnect(false);
                break;
            default:
                throw 'Unrecognized advice action' + action;
        }
    }

    function _connectResponse(message)
    {
        _connected = message.successful;

        if (_connected)
        {
            _notifyListeners('/meta/connect', message);

            // Normally, the advice will say "reconnect: 'retry', interval: 0"
            // and the server will hold the request, so when a response returns
            // we immediately call the server again (long polling)
            // Listeners can call disconnect(), so check the state after they run
            var action = _isDisconnected() ? 'none' : _advice.reconnect;
            switch (action)
            {
                case 'retry':
                    _resetBackoff();
                    _delayedConnect();
                    break;
                case 'none':
                    _disconnect(false);
                    break;
                default:
                    throw 'Unrecognized advice action ' + action;
            }
        }
        else
        {
            _failConnect(message);
        }
    }

    function _connectFailure(xhr, message)
    {
        _connected = false;
        _failConnect({
            successful: false,
            failure: true,
            channel: '/meta/connect',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'retry',
                interval: _backoff
            }
        });
    }

    function _failDisconnect(message)
    {
        _disconnect(true);
        _notifyListeners('/meta/disconnect', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _disconnectResponse(message)
    {
        if (message.successful)
        {
            _disconnect(false);
            _notifyListeners('/meta/disconnect', message);
        }
        else
        {
            _failDisconnect(message);
        }
    }

    function _disconnectFailure(xhr, message)
    {
        _failDisconnect({
            successful: false,
            failure: true,
            channel: '/meta/disconnect',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failSubscribe(message)
    {
        _notifyListeners('/meta/subscribe', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _subscribeResponse(message)
    {
        if (message.successful)
        {
            _notifyListeners('/meta/subscribe', message);
        }
        else
        {
            _failSubscribe(message);
        }
    }

    function _subscribeFailure(xhr, message)
    {
        _failSubscribe({
            successful: false,
            failure: true,
            channel: '/meta/subscribe',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failUnsubscribe(message)
    {
        _notifyListeners('/meta/unsubscribe', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _unsubscribeResponse(message)
    {
        if (message.successful)
        {
            _notifyListeners('/meta/unsubscribe', message);
        }
        else
        {
            _failUnsubscribe(message);
        }
    }

    function _unsubscribeFailure(xhr, message)
    {
        _failUnsubscribe({
            successful: false,
            failure: true,
            channel: '/meta/unsubscribe',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failMessage(message)
    {
        _notifyListeners('/meta/publish', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _messageResponse(message)
    {
        if (message.successful === undefined)
        {
            if (message.data)
            {
                // It is a plain message, and not a bayeux meta message
                _notifyListeners(message.channel, message);
            }
            else
            {
                _cometd._debug('Unknown message', message);
            }
        }
        else
        {
            if (message.successful)
            {
                _notifyListeners('/meta/publish', message);
            }
            else
            {
                _failMessage(message);
            }
        }
    }

    function _messageFailure(xhr, message)
    {
        _failMessage({
            successful: false,
            failure: true,
            channel: message.channel,
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _receive(message)
    {
        message = _applyIncomingExtensions(message);
        if (message === undefined || message === null)
        {
            return;
        }

        _updateAdvice(message.advice);

        var channel = message.channel;
        switch (channel)
        {
            case '/meta/handshake':
                _handshakeResponse(message);
                break;
            case '/meta/connect':
                _connectResponse(message);
                break;
            case '/meta/disconnect':
                _disconnectResponse(message);
                break;
            case '/meta/subscribe':
                _subscribeResponse(message);
                break;
            case '/meta/unsubscribe':
                _unsubscribeResponse(message);
                break;
            default:
                _messageResponse(message);
                break;
        }
    }

    /**
     * Receives a message.
     * This method is exposed as a public so that extensions may inject
     * messages simulating that they had been received.
     */
    this.receive = _receive;

    _handleMessages = function(rcvdMessages)
    {
        _cometd._debug('Received', rcvdMessages);

        for (var i = 0; i < rcvdMessages.length; ++i)
        {
            var message = rcvdMessages[i];
            _receive(message);
        }
    };

    _handleFailure = function(conduit, messages, reason, exception)
    {
        _cometd._debug('handleFailure', conduit, messages, reason, exception);

        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];
            var channel = message.channel;
            switch (channel)
            {
                case '/meta/handshake':
                    _handshakeFailure(conduit, message);
                    break;
                case '/meta/connect':
                    _connectFailure(conduit, message);
                    break;
                case '/meta/disconnect':
                    _disconnectFailure(conduit, message);
                    break;
                case '/meta/subscribe':
                    _subscribeFailure(conduit, message);
                    break;
                case '/meta/unsubscribe':
                    _unsubscribeFailure(conduit, message);
                    break;
                default:
                    _messageFailure(conduit, message);
                    break;
            }
        }
    };

    function _hasSubscriptions(channel)
    {
        var subscriptions = _listeners[channel];
        if (subscriptions)
        {
            for (var i = 0; i < subscriptions.length; ++i)
            {
                if (subscriptions[i])
                {
                    return true;
                }
            }
        }
        return false;
    }

    function _resolveScopedCallback(scope, callback)
    {
        var delegate = {
            scope: scope,
            method: callback
        };
        if (_isFunction(scope))
        {
            delegate.scope = undefined;
            delegate.method = scope;
        }
        else
        {
            if (_isString(callback))
            {
                if (!scope)
                {
                    throw 'Invalid scope ' + scope;
                }
                delegate.method = scope[callback];
                if (!_isFunction(delegate.method))
                {
                    throw 'Invalid callback ' + callback + ' for scope ' + scope;
                }
            }
            else if (!_isFunction(callback))
            {
                throw 'Invalid callback ' + callback;
            }
        }
        return delegate;
    }

    function _addListener(channel, scope, callback, isListener)
    {
        // The data structure is a map<channel, subscription[]>, where each subscription
        // holds the callback to be called and its scope.

        var delegate = _resolveScopedCallback(scope, callback);
        _cometd._debug('Adding listener on', channel, 'with scope', delegate.scope, 'and callback', delegate.method);

        var subscription = {
            channel: channel,
            scope: delegate.scope,
            callback: delegate.method,
            listener: isListener
        };

        var subscriptions = _listeners[channel];
        if (!subscriptions)
        {
            subscriptions = [];
            _listeners[channel] = subscriptions;
        }

        // Pushing onto an array appends at the end and returns the id associated with the element increased by 1.
        // Note that if:
        // a.push('a'); var hb=a.push('b'); delete a[hb-1]; var hc=a.push('c');
        // then:
        // hc==3, a.join()=='a',,'c', a.length==3
        var subscriptionID = subscriptions.push(subscription) - 1;
        subscription.id = subscriptionID;
        subscription.handle = [channel, subscriptionID];

        _cometd._debug('Added listener', subscription, 'for channel', channel, 'having id =', subscriptionID);

        // The subscription to allow removal of the listener is made of the channel and the index
        return subscription.handle;
    }

    function _removeListener(subscription)
    {
        var subscriptions = _listeners[subscription[0]];
        if (subscriptions)
        {
            delete subscriptions[subscription[1]];
            _cometd._debug('Removed listener', subscription);
        }
    }

    //
    // PUBLIC API
    //

    /**
     * Registers the given transport under the given transport type.
     * The optional index parameter specifies the "priority" at which the
     * transport is registered (where 0 is the max priority).
     * If a transport with the same type is already registered, this function
     * does nothing and returns false.
     * @param type the transport type
     * @param transport the transport object
     * @param index the index at which this transport is to be registered
     * @return true if the transport has been registered, false otherwise
     * @see #unregisterTransport(type)
     */
    this.registerTransport = function(type, transport, index)
    {
        var result = _transports.add(type, transport, index);
        if (result)
        {
            this._debug('Registered transport', type);

            if (_isFunction(transport.registered))
            {
                transport.registered(type, this);
            }
        }
        return result;
    };

    /**
     * @return an array of all registered transport types
     */
    this.getTransportTypes = function()
    {
        return _transports.getTransportTypes();
    };

    /**
     * Unregisters the transport with the given transport type.
     * @param type the transport type to unregister
     * @return the transport that has been unregistered,
     * or null if no transport was previously registered under the given transport type
     */
    this.unregisterTransport = function(type)
    {
        var transport = _transports.remove(type);
        if (transport !== null)
        {
            this._debug('Unregistered transport', type);

            if (_isFunction(transport.unregistered))
            {
                transport.unregistered();
            }
        }
        return transport;
    };

    this.unregisterTransports = function()
    {
        _transports.clear();
    };

    this.findTransport = function(name)
    {
        return _transports.find(name);
    };

    /**
     * Configures the initial Bayeux communication with the Bayeux server.
     * Configuration is passed via an object that must contain a mandatory field <code>url</code>
     * of type string containing the URL of the Bayeux server.
     * @param configuration the configuration object
     */
    this.configure = function(configuration)
    {
        _configure.call(this, configuration);
    };

    /**
     * Configures and establishes the Bayeux communication with the Bayeux server
     * via a handshake and a subsequent connect.
     * @param configuration the configuration object
     * @param handshakeProps an object to be merged with the handshake message
     * @see #configure(configuration)
     * @see #handshake(handshakeProps)
     */
    this.init = function(configuration, handshakeProps)
    {
        this.configure(configuration);
        this.handshake(handshakeProps);
    };

    /**
     * Establishes the Bayeux communication with the Bayeux server
     * via a handshake and a subsequent connect.
     * @param handshakeProps an object to be merged with the handshake message
     */
    this.handshake = function(handshakeProps)
    {
        _setStatus('disconnected');
        _reestablish = false;
        _handshake(handshakeProps);
    };

    /**
     * Disconnects from the Bayeux server.
     * It is possible to suggest to attempt a synchronous disconnect, but this feature
     * may only be available in certain transports (for example, long-polling may support
     * it, callback-polling certainly does not).
     * @param sync whether attempt to perform a synchronous disconnect
     * @param disconnectProps an object to be merged with the disconnect message
     */
    this.disconnect = function(sync, disconnectProps)
    {
        if (_isDisconnected())
        {
            return;
        }

        if (disconnectProps === undefined)
        {
            if (typeof sync !== 'boolean')
            {
                disconnectProps = sync;
                sync = false;
            }
        }

        var bayeuxMessage = {
            channel: '/meta/disconnect'
        };
        var message = this._mixin(false, {}, disconnectProps, bayeuxMessage);
        _setStatus('disconnecting');
        _send(sync === true, [message], false, 'disconnect');
    };

    /**
     * Marks the start of a batch of application messages to be sent to the server
     * in a single request, obtaining a single response containing (possibly) many
     * application reply messages.
     * Messages are held in a queue and not sent until {@link #endBatch()} is called.
     * If startBatch() is called multiple times, then an equal number of endBatch()
     * calls must be made to close and send the batch of messages.
     * @see #endBatch()
     */
    this.startBatch = function()
    {
        _startBatch();
    };

    /**
     * Marks the end of a batch of application messages to be sent to the server
     * in a single request.
     * @see #startBatch()
     */
    this.endBatch = function()
    {
        _endBatch();
    };

    /**
     * Executes the given callback in the given scope, surrounded by a {@link #startBatch()}
     * and {@link #endBatch()} calls.
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to be executed within {@link #startBatch()} and {@link #endBatch()} calls
     */
    this.batch = function(scope, callback)
    {
        var delegate = _resolveScopedCallback(scope, callback);
        this.startBatch();
        try
        {
            delegate.method.call(delegate.scope);
            this.endBatch();
        }
        catch (x)
        {
            this._debug('Exception during execution of batch', x);
            this.endBatch();
            throw x;
        }
    };

    /**
     * Adds a listener for bayeux messages, performing the given callback in the given scope
     * when a message for the given channel arrives.
     * @param channel the channel the listener is interested to
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to call when a message is sent to the channel
     * @returns the subscription handle to be passed to {@link #removeListener(object)}
     * @see #removeListener(subscription)
     */
    this.addListener = function(channel, scope, callback)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }

        return _addListener(channel, scope, callback, true);
    };

    /**
     * Removes the subscription obtained with a call to {@link #addListener(string, object, function)}.
     * @param subscription the subscription to unsubscribe.
     * @see #addListener(channel, scope, callback)
     */
    this.removeListener = function(subscription)
    {
        if (!org.cometd.Utils.isArray(subscription))
        {
            throw 'Invalid argument: expected subscription, not ' + subscription;
        }

        _removeListener(subscription);
    };

    /**
     * Removes all listeners registered with {@link #addListener(channel, scope, callback)} or
     * {@link #subscribe(channel, scope, callback)}.
     */
    this.clearListeners = function()
    {
        _listeners = {};
    };

    /**
     * Subscribes to the given channel, performing the given callback in the given scope
     * when a message for the channel arrives.
     * @param channel the channel to subscribe to
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to call when a message is sent to the channel
     * @param subscribeProps an object to be merged with the subscribe message
     * @return the subscription handle to be passed to {@link #unsubscribe(object)}
     */
    this.subscribe = function(channel, scope, callback, subscribeProps)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        // Normalize arguments
        if (_isFunction(scope))
        {
            subscribeProps = callback;
            callback = scope;
            scope = undefined;
        }

        // Only send the message to the server if this client has not yet subscribed to the channel
        var send = !_hasSubscriptions(channel);

        var subscription = _addListener(channel, scope, callback, false);

        if (send)
        {
            // Send the subscription message after the subscription registration to avoid
            // races where the server would send a message to the subscribers, but here
            // on the client the subscription has not been added yet to the data structures
            var bayeuxMessage = {
                channel: '/meta/subscribe',
                subscription: channel
            };
            var message = this._mixin(false, {}, subscribeProps, bayeuxMessage);
            _queueSend(message);
        }

        return subscription;
    };

    /**
     * Unsubscribes the subscription obtained with a call to {@link #subscribe(string, object, function)}.
     * @param subscription the subscription to unsubscribe.
     */
    this.unsubscribe = function(subscription, unsubscribeProps)
    {
        if (arguments.length < 1)
        {
            throw 'Illegal arguments number: required 1, got ' + arguments.length;
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        // Remove the local listener before sending the message
        // This ensures that if the server fails, this client does not get notifications
        this.removeListener(subscription);

        var channel = subscription[0];
        // Only send the message to the server if this client unsubscribes the last subscription
        if (!_hasSubscriptions(channel))
        {
            var bayeuxMessage = {
                channel: '/meta/unsubscribe',
                subscription: channel
            };
            var message = this._mixin(false, {}, unsubscribeProps, bayeuxMessage);
            _queueSend(message);
        }
    };

    /**
     * Removes all subscriptions added via {@link #subscribe(channel, scope, callback, subscribeProps)},
     * but does not remove the listeners added via {@link addListener(channel, scope, callback)}.
     */
    this.clearSubscriptions = function()
    {
        _clearSubscriptions();
    };

    /**
     * Publishes a message on the given channel, containing the given content.
     * @param channel the channel to publish the message to
     * @param content the content of the message
     * @param publishProps an object to be merged with the publish message
     */
    this.publish = function(channel, content, publishProps)
    {
        if (arguments.length < 1)
        {
            throw 'Illegal arguments number: required 1, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        var bayeuxMessage = {
            channel: channel,
            data: content
        };
        var message = this._mixin(false, {}, publishProps, bayeuxMessage);
        _queueSend(message);
    };

    /**
     * Returns a string representing the status of the bayeux communication with the Bayeux server.
     */
    this.getStatus = function()
    {
        return _status;
    };

    /**
     * Returns whether this instance has been disconnected.
     */
    this.isDisconnected = _isDisconnected;

    /**
     * Sets the backoff period used to increase the backoff time when retrying an unsuccessful or failed message.
     * Default value is 1 second, which means if there is a persistent failure the retries will happen
     * after 1 second, then after 2 seconds, then after 3 seconds, etc. So for example with 15 seconds of
     * elapsed time, there will be 5 retries (at 1, 3, 6, 10 and 15 seconds elapsed).
     * @param period the backoff period to set
     * @see #getBackoffIncrement()
     */
    this.setBackoffIncrement = function(period)
    {
        _config.backoffIncrement = period;
    };

    /**
     * Returns the backoff period used to increase the backoff time when retrying an unsuccessful or failed message.
     * @see #setBackoffIncrement(period)
     */
    this.getBackoffIncrement = function()
    {
        return _config.backoffIncrement;
    };

    /**
     * Returns the backoff period to wait before retrying an unsuccessful or failed message.
     */
    this.getBackoffPeriod = function()
    {
        return _backoff;
    };

    /**
     * Sets the log level for console logging.
     * Valid values are the strings 'error', 'warn', 'info' and 'debug', from
     * less verbose to more verbose.
     * @param level the log level string
     */
    this.setLogLevel = function(level)
    {
        _config.logLevel = level;
    };

    /**
     * Registers an extension whose callbacks are called for every incoming message
     * (that comes from the server to this client implementation) and for every
     * outgoing message (that originates from this client implementation for the
     * server).
     * The format of the extension object is the following:
     * <pre>
     * {
     *     incoming: function(message) { ... },
     *     outgoing: function(message) { ... }
     * }
     * </pre>
     * Both properties are optional, but if they are present they will be called
     * respectively for each incoming message and for each outgoing message.
     * @param name the name of the extension
     * @param extension the extension to register
     * @return true if the extension was registered, false otherwise
     * @see #unregisterExtension(name)
     */
    this.registerExtension = function(name, extension)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(name))
        {
            throw 'Illegal argument type: extension name must be a string';
        }

        var existing = false;
        for (var i = 0; i < _extensions.length; ++i)
        {
            var existingExtension = _extensions[i];
            if (existingExtension.name === name)
            {
                existing = true;
                break;
            }
        }
        if (!existing)
        {
            _extensions.push({
                name: name,
                extension: extension
            });
            this._debug('Registered extension', name);

            // Callback for extensions
            if (_isFunction(extension.registered))
            {
                extension.registered(name, this);
            }

            return true;
        }
        else
        {
            this._info('Could not register extension with name', name, 'since another extension with the same name already exists');
            return false;
        }
    };

    /**
     * Unregister an extension previously registered with
     * {@link #registerExtension(name, extension)}.
     * @param name the name of the extension to unregister.
     * @return true if the extension was unregistered, false otherwise
     */
    this.unregisterExtension = function(name)
    {
        if (!_isString(name))
        {
            throw 'Illegal argument type: extension name must be a string';
        }

        var unregistered = false;
        for (var i = 0; i < _extensions.length; ++i)
        {
            var extension = _extensions[i];
            if (extension.name === name)
            {
                _extensions.splice(i, 1);
                unregistered = true;
                this._debug('Unregistered extension', name);

                // Callback for extensions
                var ext = extension.extension;
                if (_isFunction(ext.unregistered))
                {
                    ext.unregistered();
                }

                break;
            }
        }
        return unregistered;
    };

    /**
     * Find the extension registered with the given name.
     * @param name the name of the extension to find
     * @return the extension found or null if no extension with the given name has been registered
     */
    this.getExtension = function(name)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            var extension = _extensions[i];
            if (extension.name === name)
            {
                return extension.extension;
            }
        }
        return null;
    };

    /**
     * Returns the name assigned to this Cometd object, or the string 'default'
     * if no name has been explicitly passed as parameter to the constructor.
     */
    this.getName = function()
    {
        return _name;
    };

    /**
     * Returns the clientId assigned by the Bayeux server during handshake.
     */
    this.getClientId = function()
    {
        return _clientId;
    };

    /**
     * Returns the URL of the Bayeux server.
     */
    this.getURL = function()
    {
        return _config.url;
    };

    this.getTransport = function()
    {
        return _transport;
    };

    this.getConfiguration = function()
    {
        return this._mixin(true, {}, _config);
    };

    this.getAdvice = function()
    {
        return this._mixin(true, {}, _advice);
    };

    // WebSocket handling for Firefox, which deploys WebSocket
    // under the name of MozWebSocket in Firefox 6, 7, 8 and 9
    org.cometd.WebSocket = window.WebSocket;
    if (!org.cometd.WebSocket)
    {
        org.cometd.WebSocket = window.MozWebSocket;
    }
};


}

if(!dojo._hasResource["dojo.io.script"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.io.script"] = true;
dojo.provide("dojo.io.script");

dojo.getObject("io", true, dojo);

/*=====
dojo.declare("dojo.io.script.__ioArgs", dojo.__IoArgs, {
	constructor: function(){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply to this
		//		type as well, EXCEPT "handleAs". It is not applicable to
		//		dojo.io.script.get() calls, since it is implied by the usage of
		//		"jsonp" (response will be a JSONP call returning JSON)
		//		or the response is pure JavaScript defined in
		//		the body of the script that was attached.
		//	callbackParamName: String
		//		Deprecated as of Dojo 1.4 in favor of "jsonp", but still supported for
		// 		legacy code. See notes for jsonp property.
		//	jsonp: String
		//		The URL parameter name that indicates the JSONP callback string.
		//		For instance, when using Yahoo JSONP calls it is normally,
		//		jsonp: "callback". For AOL JSONP calls it is normally
		//		jsonp: "c".
		//	checkString: String
		//		A string of JavaScript that when evaluated like so:
		//		"typeof(" + checkString + ") != 'undefined'"
		//		being true means that the script fetched has been loaded.
		//		Do not use this if doing a JSONP type of call (use callbackParamName instead).
		//	frameDoc: Document
		//		The Document object for a child iframe. If this is passed in, the script
		//		will be attached to that document. This can be helpful in some comet long-polling
		//		scenarios with Firefox and Opera.
		this.callbackParamName = callbackParamName;
		this.jsonp = jsonp;
		this.checkString = checkString;
		this.frameDoc = frameDoc;
	}
});
=====*/
(function(){
	var loadEvent = dojo.isIE ? "onreadystatechange" : "load",
		readyRegExp = /complete|loaded/;

	dojo.io.script = {
		get: function(/*dojo.io.script.__ioArgs*/args){
			//	summary:
			//		sends a get request using a dynamically created script tag.
			var dfd = this._makeScriptDeferred(args);
			var ioArgs = dfd.ioArgs;
			dojo._ioAddQueryToUrl(ioArgs);
	
			dojo._ioNotifyStart(dfd);

			if(this._canAttach(ioArgs)){
				var node = this.attach(ioArgs.id, ioArgs.url, args.frameDoc);

				//If not a jsonp callback or a polling checkString case, bind
				//to load event on the script tag.
				if(!ioArgs.jsonp && !ioArgs.args.checkString){
					var handle = dojo.connect(node, loadEvent, function(evt){
						if(evt.type == "load" || readyRegExp.test(node.readyState)){
							dojo.disconnect(handle);
							ioArgs.scriptLoaded = evt;
						}
					});
				}
			}

			dojo._ioWatch(dfd, this._validCheck, this._ioCheck, this._resHandle);
			return dfd;
		},
	
		attach: function(/*String*/id, /*String*/url, /*Document?*/frameDocument){
			//	summary:
			//		creates a new <script> tag pointing to the specified URL and
			//		adds it to the document.
			//	description:
			//		Attaches the script element to the DOM.  Use this method if you
			//		just want to attach a script to the DOM and do not care when or
			//		if it loads.
			var doc = (frameDocument || dojo.doc);
			var element = doc.createElement("script");
			element.type = "text/javascript";
			element.src = url;
			element.id = id;
			element.charset = "utf-8";
			return doc.getElementsByTagName("head")[0].appendChild(element);
		},
	
		remove: function(/*String*/id, /*Document?*/frameDocument){
			//summary: removes the script element with the given id, from the given frameDocument.
			//If no frameDocument is passed, the current document is used.
			dojo.destroy(dojo.byId(id, frameDocument));
			
			//Remove the jsonp callback on dojo.io.script, if it exists.
			if(this["jsonp_" + id]){
				delete this["jsonp_" + id];
			}
		},
	
		_makeScriptDeferred: function(/*Object*/args){
			//summary:
			//		sets up a Deferred object for an IO request.
			var dfd = dojo._ioSetArgs(args, this._deferredCancel, this._deferredOk, this._deferredError);
	
			var ioArgs = dfd.ioArgs;
			ioArgs.id = dojo._scopeName + "IoScript" + (this._counter++);
			ioArgs.canDelete = false;
	
			//Special setup for jsonp case
			ioArgs.jsonp = args.callbackParamName || args.jsonp;
			if(ioArgs.jsonp){
				//Add the jsonp parameter.
				ioArgs.query = ioArgs.query || "";
				if(ioArgs.query.length > 0){
					ioArgs.query += "&";
				}
				ioArgs.query += ioArgs.jsonp
					+ "="
					+ (args.frameDoc ? "parent." : "")
					+ dojo._scopeName + ".io.script.jsonp_" + ioArgs.id + "._jsonpCallback";
	
				ioArgs.frameDoc = args.frameDoc;
	
				//Setup the Deferred to have the jsonp callback.
				ioArgs.canDelete = true;
				dfd._jsonpCallback = this._jsonpCallback;
				this["jsonp_" + ioArgs.id] = dfd;
			}
			return dfd; // dojo.Deferred
		},
		
		_deferredCancel: function(/*Deferred*/dfd){
			//summary: canceller function for dojo._ioSetArgs call.
	
			//DO NOT use "this" and expect it to be dojo.io.script.
			dfd.canceled = true;
			if(dfd.ioArgs.canDelete){
				dojo.io.script._addDeadScript(dfd.ioArgs);
			}
		},
	
		_deferredOk: function(/*Deferred*/dfd){
			//summary: okHandler function for dojo._ioSetArgs call.
	
			//DO NOT use "this" and expect it to be dojo.io.script.
			var ioArgs = dfd.ioArgs;
	
			//Add script to list of things that can be removed.
			if(ioArgs.canDelete){
				dojo.io.script._addDeadScript(ioArgs);
			}
	
			//Favor JSONP responses, script load events then lastly ioArgs.
			//The ioArgs are goofy, but cannot return the dfd since that stops
			//the callback chain in Deferred. The return value is not that important
			//in that case, probably a checkString case.
			return ioArgs.json || ioArgs.scriptLoaded || ioArgs;
		},
	
		_deferredError: function(/*Error*/error, /*Deferred*/dfd){
			//summary: errHandler function for dojo._ioSetArgs call.
	
			if(dfd.ioArgs.canDelete){
				//DO NOT use "this" and expect it to be dojo.io.script.
				if(error.dojoType == "timeout"){
					//For timeouts, remove the script element immediately to
					//avoid a response from it coming back later and causing trouble.
					dojo.io.script.remove(dfd.ioArgs.id, dfd.ioArgs.frameDoc);
				}else{
					dojo.io.script._addDeadScript(dfd.ioArgs);
				}
			}
			console.log("dojo.io.script error", error);
			return error;
		},
	
		_deadScripts: [],
		_counter: 1,
	
		_addDeadScript: function(/*Object*/ioArgs){
			//summary: sets up an entry in the deadScripts array.
			dojo.io.script._deadScripts.push({id: ioArgs.id, frameDoc: ioArgs.frameDoc});
			//Being extra paranoid about leaks:
			ioArgs.frameDoc = null;
		},
	
		_validCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if dfd is still valid.
	
			//Do script cleanup here. We wait for one inflight pass
			//to make sure we don't get any weird things by trying to remove a script
			//tag that is part of the call chain (IE 6 has been known to
			//crash in that case).
			var _self = dojo.io.script;
			var deadScripts = _self._deadScripts;
			if(deadScripts && deadScripts.length > 0){
				for(var i = 0; i < deadScripts.length; i++){
					//Remove the script tag
					_self.remove(deadScripts[i].id, deadScripts[i].frameDoc);
					deadScripts[i].frameDoc = null;
				}
				dojo.io.script._deadScripts = [];
			}
	
			return true;
		},
	
		_ioCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if IO finished.
			var ioArgs = dfd.ioArgs;
			//Check for finished jsonp
			if(ioArgs.json || (ioArgs.scriptLoaded && !ioArgs.args.checkString)){
				return true;
			}
	
			//Check for finished "checkString" case.
			var checkString = ioArgs.args.checkString;
			if(checkString && eval("typeof(" + checkString + ") != 'undefined'")){
				return true;
			}
	
			return false;
		},
	
		_resHandle: function(/*Deferred*/dfd){
			//summary: inflight function to handle a completed response.
			if(dojo.io.script._ioCheck(dfd)){
				dfd.callback(dfd);
			}else{
				//This path should never happen since the only way we can get
				//to _resHandle is if _ioCheck is true.
				dfd.errback(new Error("inconceivable dojo.io.script._resHandle error"));
			}
		},
	
		_canAttach: function(/*Object*/ioArgs){
			//summary: A method that can be overridden by other modules
			//to control when the script attachment occurs.
			return true;
		},
		
		_jsonpCallback: function(/*JSON Object*/json){
			//summary:
			//		generic handler for jsonp callback. A pointer to this function
			//		is used for all jsonp callbacks.  NOTE: the "this" in this
			//		function will be the Deferred object that represents the script
			//		request.
			this.ioArgs.json = json;
		}
	};
})();

}

if(!dojo._hasResource['dojox.cometd']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.cometd'] = true;
/*
 * Copyright (c) 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

dojo.provide('dojox.cometd');
dojo.registerModulePath('org','../org');



// Remap cometd JSON functions to dojo JSON functions
org.cometd.JSON.toJSON = dojo.toJson;
org.cometd.JSON.fromJSON = dojo.fromJson;

dojox.Cometd = function(name)
{
    var cometd = new org.cometd.Cometd(name);

    function LongPollingTransport()
    {
        var _super = new org.cometd.LongPollingTransport();
        var that = org.cometd.Transport.derive(_super);

        that.xhrSend = function(packet)
        {
            var deferred = dojo.rawXhrPost({
                url: packet.url,
                sync: packet.sync === true,
                contentType: 'application/json;charset=UTF-8',
                headers: packet.headers,
                postData: packet.body,
                withCredentials: true,
                handleAs: 'json',
                load: packet.onSuccess,
                error: function(error)
                {
                    packet.onError(error.message, deferred ? deferred.ioArgs.error : error);
                }
            });
            return deferred.ioArgs.xhr;
        };

        return that;
    }

    function CallbackPollingTransport()
    {
        var _super = new org.cometd.CallbackPollingTransport();
        var that = org.cometd.Transport.derive(_super);

        that.jsonpSend = function(packet)
        {
            var deferred = dojo.io.script.get({
                url: packet.url,
                sync: packet.sync === true,
                callbackParamName: 'jsonp',
                content: {
                    // In callback-polling, the content must be sent via the 'message' parameter
                    message: packet.body
                },
                load: packet.onSuccess,
                error: function(error)
                {
                    packet.onError(error.message, deferred ? deferred.ioArgs.error : error);
                }
            });
            return undefined;
        };

        return that;
    }

    // Registration order is important
    if (org.cometd.WebSocket)
    {
        cometd.registerTransport('websocket', new org.cometd.WebSocketTransport());
    }
    cometd.registerTransport('long-polling', new LongPollingTransport());
    cometd.registerTransport('callback-polling', new CallbackPollingTransport());

    return cometd;
};

// The default cometd instance
dojox.cometd = new dojox.Cometd();

// Create a compatibility API for dojox.cometd instance with
// the original API.

dojox.cometd._init = dojox.cometd.init;

dojox.cometd._unsubscribe = dojox.cometd.unsubscribe;

dojox.cometd.unsubscribe = function(channelOrToken, objOrFunc, funcName)
{
    if (typeof channelOrToken === 'string')
    {
        throw "Deprecated function unsubscribe(string). Use unsubscribe(object) passing as argument the return value of subscribe()";
    }

    dojox.cometd._unsubscribe(channelOrToken);
};

dojox.cometd._metaHandshakeEvent = function(event)
{
    event.action = "handshake";
    dojo.publish("/cometd/meta", [event]);
};

dojox.cometd._metaConnectEvent = function(event)
{
    event.action = "connect";
    dojo.publish("/cometd/meta", [event]);
};

dojox.cometd.addListener('/meta/handshake', dojox.cometd, dojox.cometd._metaHandshakeEvent);
dojox.cometd.addListener('/meta/connect', dojox.cometd, dojox.cometd._metaConnectEvent);

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.Topics"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.Topics"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.Topics");


com.ibm.bpm.social.portal.Topics.VIEW_READY = "com.ibm.bpm.social.portal.view.viewReady";

com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE = "com.ibm.bpm.social.portal.showProcessInstance";

com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE = "com.ibm.bpm.social.portal.showLegacyProcessInstance";

com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION = "com.ibm.bpm.social.portal.showTaskCompletion";

com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST = "com.ibm.bpm.social.portal.showTasks";

com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD = "com.ibm.bpm.social.portal.showDashboard";

com.ibm.bpm.social.portal.Topics.DASHBOARD_LOADED = "com.ibm.bpm.social.portal.dashboardLoaded";

com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT = "com.ibm.bpm.social.portal.view.maximizeContent";

com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT = "com.ibm.bpm.social.portal.view.restoreContent";

com.ibm.bpm.social.portal.Topics.REFRESH_FOLLOW_LIST = "com.ibm.bpm.social.portal.refreshFollowList";

com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST = "com.ibm.bpm.social.portal.refreshMentionsList";

com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED = "com.ibm.bpm.socialPortal.UserProfileChangedEvent"; //sic

com.ibm.bpm.social.portal.Topics.PROCESS_ACTIONS_AVAILABLE = "com.ibm.bpm.social.portal.processActionsAvailable";

com.ibm.bpm.social.portal.Topics.COACH_CANCELED = "com.ibm.bpm.social.portal.coachCanceled";

com.ibm.bpm.social.portal.Topics.CLOSE_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.closeProcessDiagram";

com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.refreshProcessDiagram";

com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED = "com.ibm.bpm.social.portal.instanceDueDateUpdated";

com.ibm.bpm.social.portal.Topics.TASK_UPDATED = "com.ibm.bpm.social.portal.taskUpdated";

com.ibm.bpm.social.portal.Topics.TASK_CLAIMED = "com.ibm.bpm.social.portal.taskClaimed";

com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.openProcessDiagram";

com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED = "com.ibm.bpm.social.portal.taskListUpdated";

com.ibm.bpm.social.portal.Topics.SHOW_DIALOG = "com.ibm.bpm.social.portal.showDialog";

com.ibm.bpm.social.portal.Topics.HIDE_DIALOG = "com.ibm.bpm.social.portal.hideDialog";

com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED = "com.ibm.bpm.social.portal.sidebarWidgetChanged";

com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED = "com.ibm.bpm.social.portal.connectionsLoaded";

com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS = "com.ibm.bpm.social.portal.updateDashboardTabs";

//See Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB() for the structure of the object to be published with this topic

com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB = "com.ibm.bpm.social.portal.selectDashboardTab";

com.ibm.bpm.social.portal.Topics.NAVIGATION = "com.ibm.bpm.social.portal.navigation";

com.ibm.bpm.social.portal.Topics.PREVENT_NAVIGATION = "com.ibm.bpm.social.portal.preventNavigation";

com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION = "com.ibm.bpm.social.portal.allowNavigation";

com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD = "com.ibm.bpm.social.portal.forceNewDashboard";



}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");



/*=====
dojo.__cookieProps = function(){
	//	expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes. << FIXME: 0 seems to disappear right away? FF3.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
	this.expires = expires;
	this.path = path;
	this.domain = domain;
	this.secure = secure;
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary:
	//		Get or set a cookie.
	//	description:
	// 		If one argument is passed, returns the value of the cookie
	// 		For two or more arguments, acts as a setter.
	//	name:
	//		Name of the cookie
	//	value:
	//		Value for the cookie
	//	props:
	//		Properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null, {expires: -1});
	var c = document.cookie;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + dojo.regexp.escapeString(name) + "=([^;]*)"));
		return matches ? decodeURIComponent(matches[1]) : undefined; // String or undefined
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
};

dojo.cookie.isSupported = function(){
	//	summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

}

if(!dojo._hasResource["com.ibm.bpm.social.notifications"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.notifications"] = true;
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.notifications");

var bpm_social_messages = {};

var connectedToBayeuxServer = false;
var logoutOngoing = false;
var currentUser;
window["BPMCometDConnectionEstablished"] = false;





function connectToBayeuxServer(){
 if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) {// If Safari on iPad don't use CometD but allow if Chrome on iPad
    return
  }

  var FUNC_NAME = "connectToBayeuxServer";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, "connecting to  Bayeux server", "notifications", FUNC_NAME);
  window["collaborationContainerDiv"] = dojo.byId("processPortalCollaborationBanner");

  
  
  
  

  
  bpm_social_messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

  // configure bayeux server url and other client options
  var configObject = {
    url: window.endpointURLObject.processPortalNotificationJS +"/cometd"
  };

  //workaround for defect 78650
  if (com.ibm.bpm.social.util.common.Util.isIPad) {
    configObject.maxNetworkDelay = 1000000;
  }
  
  dojox.cometd.configure(configObject);

  // disable the websocket transport
  dojox.cometd.websocketEnabled = false;

  if (typeof OnlineTrackingController === "function") {
    // Controller for online tracking
    // Trigger onConnectionBroken 1 minute after connection is lost.
    new OnlineTrackingController({
      cometd: dojox.cometd,
      userFunction: function(){
        var user = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (user != null) {
          return user["userName"];
        }
        else {
          return false;
        }
      },
      logFunction: function(message, cName, fName){
        com.ibm.bpm.social.util.common.Log.log(2, message, cName ? cName : "", fName ? fName : "");
      },
      timeout: 30000,
      update: true
    /* make sure new status is requested upon handshake not taking into account the timeout */});
  }

  // add a listener to the /meta/connect channel to inform us of connection events
  dojox.cometd.addListener("/meta/connect", handleOnMetaConnectMessage);
  dojox.cometd.addListener("/meta/disconnect", handleOnMetadisconnectMessage);

  // connect to bayuex server
  dojox.cometd.handshake();

  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function disconnectFromBayeuxServer() {
  logoutOngoing = true;
  if (typeof OnlineTrackingController === "function" && typeof OnlineTrackingController.instance === "object") {
    OnlineTrackingController.instance.logout();
  }
  dojox.cometd.disconnect();  
};

if (window.disconnectFromBayeuxServer == undefined) {
	window.disconnectFromBayeuxServer = disconnectFromBayeuxServer;
}

function handleOnMetadisconnectMessage(message){
  var FUNC_NAME = "handleOnMetadisconnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (logoutOngoing) {
    com.ibm.bpm.social.util.common.Log.log(2, "logout ongoing => no reconnection attempt", "notifications", FUNC_NAME);
  } else {
    // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
    // com.ibm.bpm.social.util.common.Log.log(2,"Meta Connect disconnect Message: "+ message, "notifications", FUNC_NAME);
    window["cometDForCollab"] = null;
    connectedToBayeuxServer = false;

    
    if (typeof (dojo.cookie("com_ibm_bpm_bspace_logout")) == "undefined" &&
      typeof (dojo.cookie("com_ibm_bpm_portal_unload")) == "undefined") {
      // if we're not logging out, reconnect to bayuex server
      dojox.cometd.handshake();
    }
  }   
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};
function handleOnMetaConnectMessage(message){
  var FUNC_NAME = "handleOnMetaConnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
  // com.ibm.bpm.social.util.common.Log.log(2, "Meta Connect Message: " + message, "notifications", FUNC_NAME);
  var wasPreviouslyConnected = connectedToBayeuxServer;
  connectedToBayeuxServer = (message.successful === true);
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage connectedToBayeuxServer:"+connectedToBayeuxServer, "notifications", "handleOnMetadisconnectMessage");
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage wasPreviouslyConnected:"+wasPreviouslyConnected, "notifications", "handleOnMetadisconnectMessage");
  if (!wasPreviouslyConnected && connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Success", "notifications", FUNC_NAME);

    handleConnectionSucceeded(message);
  }
  else if (wasPreviouslyConnected && !connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Broken", "notifications", FUNC_NAME);
    handleConnectionBroken(message);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleConnectionSucceeded(message){
  var FUNC_NAME = "handleConnectionSucceeded";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = true;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, setting windows variable cometd", "notifications",
    "handleConnectionSucceeded");
  window["cometDForCollab"] = dojox.cometd;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, calling subscribe to Channels", "notifications",
    "handleConnectionSucceeded");
  subscribeToChannels();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function getUserInfo(){
  var FUNC_NAME = "getUserInfo";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var userData = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return userData;
};

function handleConnectionBroken(message){
  var FUNC_NAME = "handleConnectionBroken";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = false;
  window["cometDForCollab"] = null;

  //Do a check here to see if the reason behind the connection break is a 401 Response.  If that is the case,
  //then redirect to the login page.
  var xhr = message.xhr;
  if (typeof xhr != "undefined") {
    var status = xhr.status;
    if (typeof status != "undefined" && status == 401) {
      //Perform REDIRECT to login page
      com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
    }
  }
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleFailedRestConnection(message){
  var FUNC_NAME = "handleFailedRestConnection";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToChannels(){
  var FUNC_NAME = "subscribeToChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //console.group("subscribeToChannels");
  if (!currentUser) {
    var userData = getUserInfo();
    currentUser = userData;
  }
  //com.ibm.bpm.social.util.common.Log.log(2, "User Info: " + userData, "notifications", "subscribeToChannels");
  //com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), "notifications", "subscribeToChannels");
  dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), handleNotificationMessage);
  var membershipsAll = userData.memberships;
  
  //remove duplicates
  var memberships = [];
  dojo.forEach(membershipsAll, function(item, i) {
    if(dojo.indexOf(memberships, item)  == -1) {
      memberships.push(item);
    }
  });
  
  if (memberships) {
    //batch the subscriptions so that we are not making multiple HTTP requests for each group subscription
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        // com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), "notifications", "subscribeToChannels");
        dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), handleNotificationMessage);
      }
    });
  }

  //
  // subscribe to the task list channels in order for automatic refresh of the task list
  subscribeToTaskListChannels(currentUser.userName, memberships);

  console.groupEnd();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToTaskListChannels(userName, memberships){
  var FUNC_NAME = "subscribeToTaskListChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // task list push events for user
  dojox.cometd.subscribe('/bpm/process_portal/task_list/user/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName), handleTaskNotificationMessage);

  // task list push events for groups
  if (memberships) {
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        dojox.cometd.subscribe('/bpm/process_portal/task_list/group/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(memberships[i]), handleTaskNotificationMessage);
      }
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    switch (data.kind) {
      case "TASK_RESOURCE_ASSIGNED" :
        handle_TASK_RESOURCE_ASSIGNED(data);
        break;
      case "TASK_COLLABORATION_INVITE" :
        handle_TASK_COLLABORATION_INVITE(data);
        break;
      case "PROCESS_COMMENT_TAGGED" :
        handle_PROCESS_COMMENT_TAGGED(data);
        break;
      default:
        break;
    }
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleTaskNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleTaskNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED, [incomingMessage]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_RESOURCE_ASSIGNED(data){
  var FUNC_NAME = "handle_TASK_RESOURCE_ASSIGNED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (!data.task.resource || !data.task.id) {
    return;
  }

  // check if task notification should be ignored, e.g. claim for inline completion
  if (!com.ibm.bpm.social.notifications._isIgnoreAssignedNotification(data.task.id)) {
    var shouldDisplayPopup = true;
    var notificationPopUpText = "";

    switch (data.task.resource.type) {
      case "ROLE" :
        //notificationMessage = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask,{'groupname': data.task.resource.displayName});
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var displayName =
          com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.resource.displayName);
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask, {
          'groupname': displayName,
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To My Group'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      case "USER" :
        //notificationMessage = bpm_social_messages.userReceivedNewTask;
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.userReceivedNewTask, {
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To Me'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      default:
        break;
    }

    if (shouldDisplayPopup) {
      //notificationPopUpText = "<div>${0} <a id='"+link_id+"' href='#' dojoattachevent='onclick: _handleActionClick'>${1}</a></div>";
      var aetherParams = {};
      aetherParams.message = notificationPopUpText;
      aetherParams.containerDivId = "messageNotificationDiv";
      aetherParams.position = "top-right";
      aetherParams.fade = true;
      aetherParams.parent = this;
      aetherParams.displayCloseIcon = true;
      aetherParams.eventParam = data;
      var message = new com.ibm.bpm.social.Aether(aetherParams);
      message.fireEvent = dojo.hitch(this, fireEvent);
    }
  }
  else {
    com.ibm.bpm.social.util.common.Log.log(2, "ignoring TASK_RESOURCE_ASSIGNED for " + data.task.id, "notifications",
      FUNC_NAME);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_COLLABORATION_INVITE(data){
  var FUNC_NAME = "handle_TASK_COLLABORATION_INVITE";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.resource.id);
  var task_link_id = getUniqueID("launch_task_action");
  var process_link_id = getUniqueID("launch_process_action");
  //apply BTD to the notifications message
  var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
  var actor = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.resource.displayName);
  var processName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
  var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.collaboration.message);

  var notificationPopUpText = dojo.string.substitute(bpm_social_messages.collaborationInviteMessage, {
    'taskName': taskName,
    'taskLinkID': task_link_id,
    'processLinkID': process_link_id,
    'actor': actor,
    'processName': processName,
    'messageBody': com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.task.collaboration.message)
  });
  var aetherParams = {};
  aetherParams.message = notificationPopUpText;
  aetherParams.containerDivId = "messageNotificationDiv";
  aetherParams.position = "top-right";
  aetherParams.fade = true;
  aetherParams.parent = this;
  aetherParams.displayCloseIcon = true;
  if (invitorInfo && invitorInfo.image) {
    aetherParams.img_src = invitorInfo.image;
  }
  aetherParams.eventParam = data;
  var message = new com.ibm.bpm.social.Aether(aetherParams);
  message.fireEvent = dojo.hitch(this, fireEvent);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_PROCESS_COMMENT_TAGGED(data){
  var FUNC_NAME = "handle_PROCESS_COMMENT_TAGGED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);
  dojo.publish("/com/ibm/bpm/social/updateStream/" + data.process.instance.id, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.process.instance.comment.author.id);
  com.ibm.bpm.social.util.common.Log.log(2, invitorInfo, "notifications", "handle_PROCESS_COMMENT_TAGGED");

  var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  if (usr == null || usr.userPreferences == null || usr.userPreferences['Portal Notification Post Mention'] != "false") {
    var link_id = getUniqueID("launch_process_action");
    var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
    var actor =
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.comment.author.name);
    var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.process.instance.comment.text));

    // Check if a user is mentioned in the text,
    // but make sure to not use the mentions text for other user then the logged in user.
    var owner= com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var messageText = bpm_social_messages.UserMentionedMessage;
    if (data.process.instance.comment.text && (data.process.instance.comment.text.indexOf("\"bpm-user-tag\"") < 0 ||
          data.process.instance.comment.text.indexOf("objId=\""+owner.userName+"\"") < 0)) {
      messageText = bpm_social_messages.UserCommentedMessage;
    }


    var notificationPopUpText = dojo.string.substitute(messageText, {
      'taskName': taskName,
      'linkID': link_id,
      'actor': actor,
      'messageBody': body
    });
    var aetherParams = {};
    aetherParams.message = notificationPopUpText;
    aetherParams.containerDivId = "messageNotificationDiv";
    aetherParams.position = "top-right";
    aetherParams.fade = true;
    aetherParams.parent = this;
    aetherParams.displayCloseIcon = true;
    if (invitorInfo && invitorInfo.image) {
      aetherParams.img_src = invitorInfo.image;
    }
    aetherParams.eventParam = data;
    var message = new com.ibm.bpm.social.Aether(aetherParams);
    message.fireEvent = dojo.hitch(this, fireEvent);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function fireEvent(link_id, param){//custom override
  var FUNC_NAME = "fireEvent";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var eventType = link_id.split("::")[0];

  switch (eventType) {
    case "launch_task_action" :
      checkCollaborationSessionAvailablity(param.task.id);
      break;
    case "launch_process_action" :
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        "instanceId": param.process.instance.id
      }]);
      break;
    default:
      break;
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

/* checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
function checkCollaborationSessionAvailablity(taskID){
  var FUNC_NAME = "checkCollaborationSessionAvailablity";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // Response will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  if (taskID) {
    /* /bpm/notification/roster/task/ */
    dojo.xhrGet({
      //url: this.getProxyUrl()+"/bpm/notification/roster/task/"+taskID,
      url: window.endpointURLObject.processPortalNotificationJS +"/roster/task/" + taskID,
      handleAs: "json",
      preventCache: true,
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: dojo.hitch(this, function(data, xhrArgs){
        checkNumberOfParticipants(data, xhrArgs, taskID);
      }),
      error: dojo.hitch(this, this.handleFailedRestConnection)
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


/* checkNumberOfParticipants: See if there aren't too many participants in this task already. */
function checkNumberOfParticipants(data, xhrArgs, taskID){
  var FUNC_NAME = "checkNumberOfParticipants";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // data will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  // or this:
  // {"message":"NO_SESSION_FOR_TASK"}
  if (!data) {
    return;
  }
  // If there's no collab session, then data won't contain an element called 'participants',
  // and in that case we go ahead and publish the show task topic.
  var doPublishShowTaskTopic = false;
  if (data.participants) {
    var participants = data.participants;
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var currentUser;
    if (userProfileObj) {
      currentUser = userProfileObj.userName;
    }
    if (participants && (participants.length < 2 || _currentUserIsAlreadyInSession(currentUser, participants))) {
      // publish the show task topic
      doPublishShowTaskTopic = true;
    }
    else {
      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(bpm_social_messages.collaborationSessionFull, function(){
        return;
      });
    }
  }
  else {
    doPublishShowTaskTopic = true;
  }
  if (doPublishShowTaskTopic) {

    //Defect 133537:  We should use the navigation API here instead of the show task completion topic, as this request
    //Might happen while on a dashboard page.  In which case, the proper pre requisite steps are not taken and the 
    //details page is not shown correctly.

    // publish the show task topic
    // dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
    //   "taskId": taskID
    // }]);

    dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION, 
      ['/dashboards/TWP/BPM_WORK?tw.local.view=taskcompletion&tw.local.taskid=' + taskID]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


function _currentUserIsAlreadyInSession(currentUser, participants){
  var FUNC_NAME = "_currentUserIsAlreadyInSession";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (currentUser && participants) {
    for ( var i in participants) {
      if (participants[i].userName == currentUser) {
        com.ibm.bpm.social.util.common.Log.logExit(1, "returning true", "notifications", FUNC_NAME);
        return true;
      }
    }

  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "returning false", "notifications", FUNC_NAME);
  return false;
};

function getUniqueID(prefix){
  var FUNC_NAME = "getUniqueID";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var currentTime = new Date();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return prefix + "::" + currentTime.getTime();
};


/** ********************************COLLABORATION UTILS END************************************************** */

com.ibm.bpm.social.notifications._ignoreIncomingAssigned = new Array();

/**
 * Prepare ignore of TASK_RESOURCE_ASSIGNED notification for a task id, e.g. for inline completion
 *
 * @param {String} taskID
 * @returns {void} nothing
 */
com.ibm.bpm.social.notifications.setIgnoreAssignedNotification = function(taskID){
  com.ibm.bpm.social.util.common.Log.log(2, taskID, "notifications", "setIgnoreAssignedNotification");
  com.ibm.bpm.social.notifications._ignoreIncomingAssigned.push(taskID);
};

/**
 * @param taskID
 * @returns {Boolean} true if TASK_RESOURCE_ASSIGNED notification for a task id should be ignored
 */
com.ibm.bpm.social.notifications._isIgnoreAssignedNotification = function(taskID){
  var ignore = false;
  var index = dojo.indexOf(com.ibm.bpm.social.notifications._ignoreIncomingAssigned, taskID);
  if (index >= 0) {
    com.ibm.bpm.social.notifications._ignoreIncomingAssigned.splice(index, 1);
    ignore = true;
  }
  com.ibm.bpm.social.util.common.Log.log(2, taskID + " - " + ignore, "notifications", "_isIgnoreAssignedNotification");
  return ignore;
};

com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX = "/bpm/ProcessPortal/notifications";

com.ibm.bpm.social.notifications.getUserNotificationChannelName = function(userName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName);
}

com.ibm.bpm.social.notifications.getGroupNotificationChannelName = function(groupName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(groupName);
}

/** ********************************************************************************** */

dojo.addOnLoad(connectToBayeuxServer);

var unload = function(event){
  if(window.exitCollaboration) {
    window.exitCollaboration(event);
  }

  if (!window.logoutInProgress) {
    dojox.cometd.disconnect();
  }
};
dojo.addOnUnload(unload);

}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");


dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	//
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into
	//		these animations from the constructor (for example, `node`,
	//		`showDuration`, `hideDuration`).
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.fx.wipeIn,
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually.
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary: Toggle the node to showing
		// delay: Integer?
		//		Ammount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		// delay: Integer?
		//		Ammount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");



/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo,
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = d.byId(args.node), s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo.Animation
	};

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()
		
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo.Animation
	};

	dojo.fx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = d.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	};

})();

}

if(!dojo._hasResource["dijit._Contained"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Contained"] = true;
dojo.provide("dijit._Contained");


dojo.declare("dijit._Contained",
		null,
		{
			// summary:
			//		Mixin for widgets that are children of a container widget
			//
			// example:
			// | 	// make a basic custom widget that knows about it's parents
			// |	dojo.declare("my.customClass",[dijit._Widget,dijit._Contained],{});

			getParent: function(){
				// summary:
				//		Returns the parent widget of this widget, assuming the parent
				//		specifies isContainer
				var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
				return parent && parent.isContainer ? parent : null;
			},

			_getSibling: function(/*String*/ which){
				// summary:
				//      Returns next or previous sibling
				// which:
				//      Either "next" or "previous"
				// tags:
				//      private
				var node = this.domNode;
				do{
					node = node[which+"Sibling"];
				}while(node && node.nodeType != 1);
				return node && dijit.byNode(node);	// dijit._Widget
			},

			getPreviousSibling: function(){
				// summary:
				//		Returns null if this is the first child of the parent,
				//		otherwise returns the next element sibling to the "left".

				return this._getSibling("previous"); // dijit._Widget
			},

			getNextSibling: function(){
				// summary:
				//		Returns null if this is the last child of the parent,
				//		otherwise returns the next element sibling to the "right".

				return this._getSibling("next"); // dijit._Widget
			},

			getIndexInParent: function(){
				// summary:
				//		Returns the index of this widget within its container parent.
				//		It returns -1 if the parent does not exist, or if the parent
				//		is not a dijit._Container

				var p = this.getParent();
				if(!p || !p.getIndexOfChild){
					return -1; // int
				}
				return p.getIndexOfChild(this); // int
			}
		}
	);

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");


dojo.declare("dijit._Container",
	null,
	{
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		// isContainer: [protected] Boolean
		//		Indicates that this widget acts as a "parent" to the descendant widgets.
		//		When the parent is started it will call startup() on the child widgets.
		//		See also `isLayoutContainer`.
		isContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget or int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//      Destroys all the widgets inside this.containerNode,
			//      but not this widget itself
			dojo.forEach(this.getChildren(), function(child){ child.destroyRecursive(preserveDom); });
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node && dijit.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return dojo.indexOf(this.getChildren(), child);	// int
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Startup all children of this widget
			dojo.forEach(this.getChildren(), function(child){ child.startup(); });

			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");





dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent()
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when my size changes so that I can re-layout.
				// For browsers where I can't directly monitor when my size changes,
				// monitor when the viewport changes size, which *may* indicate a size change for me.
				this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments to resize.
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				dojo.marginBox(node, changeSize);

				// set offset of the node
				if(changeSize.t){ node.style.top = changeSize.t + "px"; }
				if(changeSize.l){ node.style.left = changeSize.l + "px"; }
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = dojo.mixin(dojo.marginBox(node), mb);	// just use dojo.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling dojo.contentBox() since that may fail if size was recently set)
			var cs = dojo.getComputedStyle(node);
			var me = dojo._getMarginExtents(node, cs);
			var be = dojo._getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = dojo._getPadExtents(node, cs);
			this._contentBox = {
				l: dojo._toPixelValue(node, cs.paddingLeft),
				t: dojo._toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			dojo.addClass(child.domNode, cls);
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			dojo.removeClass(child.domNode, cls);
			
			this.inherited(arguments);
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return its content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me = dojo._getMarginExtents(node, cs);
	var pb = dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			dojo.mixin(widget, newSize);
		}else{
			// otherwise, call marginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			dojo.mixin(widget, dojo.marginBox(widget.domNode));
			dojo.mixin(widget, dim);
		}
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array of Widgets or at least objects containing:
		//			* domNode: pointer to DOM node to position
		//			* region or layoutAlign: position to place DOM node
		//			* resize(): (optional) method to set size of node
		//			* id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = dojo.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit.layout._ContentPaneResizeMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._ContentPaneResizeMixin"] = true;
dojo.provide("dijit.layout._ContentPaneResizeMixin");




dojo.declare("dijit.layout._ContentPaneResizeMixin", null, {
	// summary:
	//		Resize() functionality of ContentPane.   If there's a single layout widget
	//		child then it will call resize() with the same dimensions as the ContentPane.
	//		Otherwise just calls resize on each child.
	//
	//		Also implements basic startup() functionality, where starting the parent
	//		will start the children

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// isContainer: [protected] Boolean
	//		Indicates that this widget acts as a "parent" to the descendant widgets.
	//		When the parent is started it will call startup() on the child widgets.
	//		See also `isLayoutContainer`.
	isContainer: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	_startChildren: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		dojo.forEach(this.getChildren(), function(child){
			child.startup();
			child._started = true;
		});
	},

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		if(this._started){ return; }

		var parent = dijit._Contained.prototype.getParent.call(this);
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		this.inherited(arguments);

		this._startChildren();

		if(this._isShown()){
			this._onShow();
		}

		if(!this._childOfLayoutWidget){
			// If my parent isn't a layout container, since my style *may be* width=height=100%
			// or something similar (either set directly or via a CSS class),
			// monitor when my size changes so that I can re-layout.
			// For browsers where I can't directly monitor when my size changes,
			// monitor when the viewport changes size, which *may* indicate a size change for me.
			this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
				// Using function(){} closure to ensure no arguments to resize.
				this._needLayout = !this._childOfLayoutWidget;
				this.resize();
			});
		}
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propagate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = dojo.query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return dojo.hasAttr(node, "data-dojo-type") || dojo.hasAttr(node, "dojoType") || dojo.hasAttr(node, "widgetId");
			}),
			candidateWidgets = dojo.filter(childWidgetNodes.map(dijit.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		dojo.toggleClass(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download (see [20099]).
		// However, don't load href for closed TitlePanes.
		if(!this._wasShown && this.open !== false){
			this._onShow();
		}

		this._resizeCalled = true;

		this._scheduleLayout(changeSize, resultSize);
	},

	_scheduleLayout: function(changeSize, resultSize){
		// summary:
		//		Resize myself, and call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layout(changeSize, resultSize);
		}else{
			this._needLayout = true;
			this._changeSize = changeSize;
			this._resultSize = resultSize;
		}
	},

	_layout: function(changeSize, resultSize){
		// summary:
		//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
		//		Also, since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		//
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from set('content', ...))... but deferred until
		//		the ContentPane is visible

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			dojo.marginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = dojo.mixin(dojo.marginBox(cn), mb); // just use dojo.marginBox() to fill in missing values
			}
			this._contentBox = dijit.layout.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = dojo.contentBox(cn);
		}

		this._layoutChildren();

		delete this._needLayout;
	},
	
	_layoutChildren: function(){
		// Call _checkIfSingleChild() again in case app has manually mucked w/the content
		// of the ContentPane (rather than changing it through the set("content", ...) API.
		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || dojo.contentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			dojo.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			var node = this.domNode, parent = this.domNode.parentNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !dojo.hasClass(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does layout/resize of child widget(s)

		if(this._needLayout){
			// If a layout has been scheduled for when we become visible, do it now
			this._layout(this._changeSize, this._resultSize);
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	}
});

}

if(!dojo._hasResource["dojo.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.html"] = true;
dojo.provide("dojo.html");


dojo.getObject("html", true, dojo);

// the parser might be needed..
(function(){ // private scope, sort of a namespace

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0,
		d = dojo;
	
	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = dojo.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
		
		// always empty
		d.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = d._toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && d.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					d.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				d.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	dojo.declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",
			
			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,
			
			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: dojo._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.   Only obeyed if parseContent is true.
			startup: true,
			
			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..
 
				// the original params are mixed directly into the instance "this"
				dojo.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dojo.byId( this.node || node );
	
				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node) {
				    // can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
	
					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},
			
			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					dojo.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},
	
			onBegin: function(){
				// summary
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;
	
				if(dojo.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}
  
					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();
				
				this.content = cont;
				return this.node; /* DomNode */
			},
	
			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},
	
			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.node;
				delete this.content;
			},
  
			onContentError: function(err){
				return "Error occured setting content: " + err;
			},
			
			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					dojo.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					this.parseResults = dojo.parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},
  
			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params:
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//		dojo.html.set(node, "some string");
			//		dojo.html.set(node, contentNode, {options});
			//		dojo.html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(dojo.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};
})();

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");







dojo.declare(
	"dijit.layout.ContentPane", [dijit._Widget, dijit.layout._ContentPaneResizeMixin],
{
	// summary:
	//		A widget containing an HTML fragment, specified inline
	//		or by uri.  Fragment may include widgets.
	//
	// description:
	//		This widget embeds a document fragment in the page, specified
	//		either by uri, javascript generated markup or DOM reference.
	//		Any widgets within this content are instantiated and managed,
	//		but laid out according to the HTML structure.  Unlike IFRAME,
	//		ContentPane embeds a document fragment as would be found
	//		inside the BODY tag of a full HTML document.  It should not
	//		contain the HTML, HEAD, or BODY tags.
	//		For more advanced functionality with scripts and
	//		stylesheets, see dojox.layout.ContentPane.  This widget may be
	//		used stand alone or as a base class for other widgets.
	//		ContentPane is useful as a child of other layout containers
	//		such as BorderContainer or TabContainer, but note that those
	//		widgets can contain any widget as a child.
	//
	// example:
	//		Some quick samples:
	//		To change the innerHTML: cp.set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList: cp.set('content', dojo.query('div [class=selected]', userSelection))
	//
	//		To do an ajax update: cp.set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

/*=====
	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to set("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",
=====*/

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// parserScope: String
	//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
	//		will search for data-dojo-type (or dojoType).  For backwards compatibility
	//		reasons defaults to dojo._scopeName (which is "dojo" except when
	//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
	parserScope: dojo._scopeName,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via set('content', ...) / set('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div dojoType="dijit.layout.ContentPane" href="./bar" ioArgs="{timeout: 500}">
	ioArgs: {},

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior set('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href
	//		or content is loaded.
	onLoadDeferred: null,

	// Override _Widget's attributeMap because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		title: []
	}),

	// Flag to parser that I'll parse my contents, so it shouldn't.
	stopParser: true,

	// template: [private] Boolean
	//		Flag from the parser that this ContentPane is inside a template
	//		so the contents are pre-parsed.
	// (TODO: this declaration can be commented out in 2.0)
	template: false,

	create: function(params, srcNodeRef){
		// Convert a srcNodeRef argument into a content parameter, so that the original contents are
		// processed in the same way as contents set via set("content", ...), calling the parser etc.
		// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
		if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
			var df = dojo.doc.createDocumentFragment();
			srcNodeRef = dojo.byId(srcNodeRef)
			while(srcNodeRef.firstChild){
				df.appendChild(srcNodeRef.firstChild);
			}
			params = dojo.delegate(params, {content: df});
		}
		this.inherited(arguments, [params, srcNodeRef]);
	},

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
		// For subclasses of ContentPane that do have a template, does nothing.
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}

		// remove the title attribute so it doesn't show up when hovering
		// over a node  (TODO: remove in 2.0, no longer needed after #11490)
		this.domNode.title = "";

		if(!dojo.attr(this.domNode,"role")){
			dijit.setWaiRole(this.domNode, "group");
		}
	},

	_startChildren: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		this.inherited(arguments);

		// And this catches stuff like dojo.dnd.Source
		if(this._contentSetter){
			dojo.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && dojo.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so set("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));

		this._set("href", href);

		// _setHrefAttr() is called during creation and by the user, after creation.
		// Assuming preload == false, only in the second case do we actually load the URL;
		// otherwise it's done in startup(), and only if this widget is shown.
		if(this.preload || (this._created && this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// dojo.Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make set("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this._set("href", "");

		// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		if(this._created){
			// For back-compat reasons, call onLoad() for set('content', ...)
			// calls but not for content specified in srcNodeRef (ie: <div dojoType=ContentPane>...</div>)
			// or as initialization parameter (ie: new ContentPane({content: ...})
			this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));
		}

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

		return this.onLoadDeferred; 	// dojo.Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make get("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		this.inherited(arguments);

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				return this.refresh();	// If child has an href, promise that fires when the load is complete
			}
		}
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));
		this._load();
		return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this._set("isLoaded", true);
		try{
			this.onLoadDeferred.callback(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this._set("isLoaded", false);
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive();
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			dojo.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == dojo.body()){
					widget.destroyRecursive();
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		dojo.html._emptyNode(this.containerNode);

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// dojo.html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof dojo.html._ContentSetter)){
			setter = this._contentSetter = new dojo.html._ContentSetter({
				node: this.containerNode,
				_onError: dojo.hitch(this, this._onError),
				onContentError: dojo.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		};

		var setterParams = dojo.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: this.parseOnLoad,
			parserScope: this.parserScope,
			startup: false,
			dir: this.dir,
			lang: this.lang
		}, this._contentSetterParams || {});

		setter.set( (dojo.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(!isFakeContent){
			if(this._started){
				// Startup each top level child widget (and they will start their children, recursively)
				this._startChildren();
	
				// Call resize() on each of my child layout widgets,
				// or resize() on my single child layout widget...
				// either now (if I'm currently visible) or when I become visible
				this._scheduleLayout();
			}

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// override on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(data){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

}

if(!dojo._hasResource["dijit._CssStateMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._CssStateMixin"] = true;
dojo.provide("dijit._CssStateMixin");


dojo.declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	// hovering: [readonly] Boolean
	//		True if cursor is over this widget
	hovering: false,
	
	// active: [readonly] Boolean
	//		True if mouse was pressed while over this widget, and hasn't been released yet
	active: false,

	_applyAttributes: function(){
		// This code would typically be in postCreate(), but putting in _applyAttributes() for
		// performance: so the class changes happen before DOM is inserted into the document.
		// Change back to postCreate() in 2.0.  See #11635.

		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		dojo.forEach(["onmouseenter", "onmouseleave", "onmousedown"], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);
		
		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		dojo.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active"], function(attr){
			this.watch(attr, dojo.hitch(this, "_setStateClass"));
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets hovering and active properties depending on mouse state,
		//	which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._set("hovering", false);
					this._set("active", false);
					break;

				case "mousedown" :
					this._set("active", true);
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						this._mouseDown = false;
						this._set("active", false);
						this.disconnect(mouseUpConnector);
					});
					break;
			}
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this._focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		dojo.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			dojo.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because dojo.toggleClass() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = dojo.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			dojo.toggleClass(node, clazz+"Hover", hovering && !active && !disabled);
			dojo.toggleClass(node, clazz+"Active", active && !disabled);
			dojo.toggleClass(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn("onmousedown", function(){
			active = true;
			setClass();
		});
		cn("onmouseup", function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.watch("disabled", setClass);
		this.watch("readOnly", setClass);
	}
});

}

if(!dojo._hasResource["dijit.TitlePane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TitlePane"] = true;
dojo.provide("dijit.TitlePane");






dojo.declare(
	"dijit.TitlePane",
	[dijit.layout.ContentPane, dijit._Templated, dijit._CssStateMixin],
{
	// summary:
	//		A pane with a title on top, that can be expanded or collapsed.
	//
	// description:
	//		An accessible container with a title Heading, and a content
	//		section that slides open and closed. TitlePane is an extension to
	//		`dijit.layout.ContentPane`, providing all the useful content-control aspects from it.
	//
	// example:
	// | 	// load a TitlePane from remote file:
	// |	var foo = new dijit.TitlePane({ href: "foobar.html", title:"Title" });
	// |	foo.startup();
	//
	// example:
	// |	<!-- markup href example: -->
	// |	<div dojoType="dijit.TitlePane" href="foobar.html" title="Title"></div>
	//
	// example:
	// |	<!-- markup with inline data -->
	// | 	<div dojoType="dijit.TitlePane" title="Title">
	// |		<p>I am content</p>
	// |	</div>

	// title: String
	//		Title of the pane
	title: "",

	// open: Boolean
	//		Whether pane is opened or closed.
	open: true,

	// toggleable: Boolean
	//		Whether pane can be opened or closed by clicking the title bar.
	toggleable: true,

	// tabIndex: String
	//		Tabindex setting for the title (so users can tab to the title then
	//		use space/enter to open/close the title pane)
	tabIndex: "0",

	// duration: Integer
	//		Time in milliseconds to fade in/fade out
	duration: dijit.defaultDuration,

	// baseClass: [protected] String
	//		The root className to be placed on this widget's domNode.
	baseClass: "dijitTitlePane",

	templateString: dojo.cache("dijit", "templates/TitlePane.html", "<div>\n\t<div dojoAttachEvent=\"onclick:_onTitleClick, onkeypress:_onTitleKey\"\n\t\t\tclass=\"dijitTitlePaneTitle\" dojoAttachPoint=\"titleBarNode\">\n\t\t<div class=\"dijitTitlePaneTitleFocus\" dojoAttachPoint=\"focusNode\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" dojoAttachPoint=\"arrowNode\" class=\"dijitArrowNode\" role=\"presentation\"\n\t\t\t/><span dojoAttachPoint=\"arrowNodeInner\" class=\"dijitArrowNodeInner\"></span\n\t\t\t><span dojoAttachPoint=\"titleNode\" class=\"dijitTitlePaneTextNode\"></span>\n\t\t</div>\n\t</div>\n\t<div class=\"dijitTitlePaneContentOuter\" dojoAttachPoint=\"hideNode\" role=\"presentation\">\n\t\t<div class=\"dijitReset\" dojoAttachPoint=\"wipeNode\" role=\"presentation\">\n\t\t\t<div class=\"dijitTitlePaneContentInner\" dojoAttachPoint=\"containerNode\" role=\"region\" id=\"${id}_pane\">\n\t\t\t\t<!-- nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.  Put padding on inner div. -->\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n"),

	attributeMap: dojo.delegate(dijit.layout.ContentPane.prototype.attributeMap, {
		title: { node: "titleNode", type: "innerHTML" },
		tooltip: {node: "focusNode", type: "attribute", attribute: "title"},	// focusNode spans the entire width, titleNode doesn't
		id:""
	}),

	buildRendering: function(){
		this.inherited(arguments);
		dojo.setSelectable(this.titleNode, false);
	},

	postCreate: function(){
		this.inherited(arguments);
		
		// Hover and focus effect on title bar, except for non-toggleable TitlePanes
		// This should really be controlled from _setToggleableAttr() but _CssStateMixin
		// doesn't provide a way to disconnect a previous _trackMouseState() call
		if(this.toggleable){
			this._trackMouseState(this.titleBarNode, "dijitTitlePaneTitle");
		}

		// setup open/close animations
		var hideNode = this.hideNode, wipeNode = this.wipeNode;
		this._wipeIn = dojo.fx.wipeIn({
			node: this.wipeNode,
			duration: this.duration,
			beforeBegin: function(){
				hideNode.style.display="";
			}
		});
		this._wipeOut = dojo.fx.wipeOut({
			node: this.wipeNode,
			duration: this.duration,
			onEnd: function(){
				hideNode.style.display="none";
			}
		});
	},

	_setOpenAttr: function(/*Boolean*/ open, /*Boolean*/ animate){
		// summary:
		//		Hook to make set("open", boolean) control the open/closed state of the pane.
		// open: Boolean
		//		True if you want to open the pane, false if you want to close it.

		dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
			if(animation && animation.status() == "playing"){
				animation.stop();
			}
		});

		if(animate){
			var anim = this[open ? "_wipeIn" : "_wipeOut"];
			anim.play();
		}else{
			this.hideNode.style.display = this.wipeNode.style.display = open ? "" : "none";
		}

		// load content (if this is the first time we are opening the TitlePane
		// and content is specified as an href, or href was set when hidden)
		if(this._started){
			if(open){
				this._onShow();
			}else{
				this.onHide();
			}
		}

		this.arrowNodeInner.innerHTML = open ? "-" : "+";

		dijit.setWaiState(this.containerNode,"hidden", open ? "false" : "true");
		dijit.setWaiState(this.focusNode, "pressed", open ? "true" : "false");

		this._set("open", open);

		this._setCss();
	},

	_setToggleableAttr: function(/*Boolean*/ canToggle){
		// summary:
		//		Hook to make set("toggleable", boolean) work.
		// canToggle: Boolean
		//		True to allow user to open/close pane by clicking title bar.

		dijit.setWaiRole(this.focusNode, canToggle ? "button" : "heading");
		if(canToggle){
			// TODO: if canToggle is switched from true to false shouldn't we remove this setting?
			dijit.setWaiState(this.focusNode, "controls", this.id+"_pane");
			dojo.attr(this.focusNode, "tabIndex", this.tabIndex);
		}else{
			dojo.removeAttr(this.focusNode, "tabIndex");
		}

		this._set("toggleable", canToggle);

		this._setCss();
	},

	_setContentAttr: function(/*String|DomNode|Nodelist*/ content){
		// summary:
		//		Hook to make set("content", ...) work.
		// 		Typically called when an href is loaded.  Our job is to make the animation smooth.

		if(!this.open || !this._wipeOut || this._wipeOut.status() == "playing"){
			// we are currently *closing* the pane (or the pane is closed), so just let that continue
			this.inherited(arguments);
		}else{
			if(this._wipeIn && this._wipeIn.status() == "playing"){
				this._wipeIn.stop();
			}

			// freeze container at current height so that adding new content doesn't make it jump
			dojo.marginBox(this.wipeNode, { h: dojo.marginBox(this.wipeNode).h });

			// add the new content (erasing the old content, if any)
			this.inherited(arguments);

			// call _wipeIn.play() to animate from current height to new height
			if(this._wipeIn){
				this._wipeIn.play();
			}else{
				this.hideNode.style.display = "";
			}
		}
	},

	toggle: function(){
		// summary:
		//		Switches between opened and closed state
		// tags:
		//		private

		this._setOpenAttr(!this.open, true);
	},

	_setCss: function(){
		// summary:
		//		Set the open/close css state for the TitlePane
		// tags:
		//		private

		var node = this.titleBarNode || this.focusNode;
		var oldCls = this._titleBarClass;
		this._titleBarClass = "dijit" + (this.toggleable ? "" : "Fixed") + (this.open ? "Open" : "Closed");
		dojo.replaceClass(node, this._titleBarClass, oldCls || "");

		this.arrowNodeInner.innerHTML = this.open ? "-" : "+";
	},

	_onTitleKey: function(/*Event*/ e){
		// summary:
		//		Handler for when user hits a key
		// tags:
		//		private

		if(e.charOrCode == dojo.keys.ENTER || e.charOrCode == ' '){
			if(this.toggleable){
				this.toggle();
			}
			dojo.stopEvent(e);
		}else if(e.charOrCode == dojo.keys.DOWN_ARROW && this.open){
			this.containerNode.focus();
			e.preventDefault();
	 	}
	},

	_onTitleClick: function(){
		// summary:
		//		Handler when user clicks the title bar
		// tags:
		//		private
		if(this.toggleable){
			this.toggle();
		}
	},

	setTitle: function(/*String*/ title){
		// summary:
		//		Deprecated.  Use set('title', ...) instead.
		// tags:
		//		deprecated
		dojo.deprecated("dijit.TitlePane.setTitle() is deprecated.  Use set('title', ...) instead.", "", "2.0");
		this.set("title", title);
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.Category"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.Category"] = true;
// Category.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.list.Category");





dojo.declare("com.ibm.bpm.social.widgets.task.list.Category", dijit.TitlePane, {

  _displayed: false,
  _moreAvailable: null,

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_Category(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    if (!this.id) {
      // TODO check if one of the valid categories
      this.id = com.ibm.bpm.social.widgets.task.list.Category.OVERDUE;
    }
    this._moreAvailable = false;
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * postCreate
   * 
   */
  postCreate: function postCreate_Category(){
    this.inherited(arguments);
    dojo.addClass(this.domNode, "bpm-task-list-category");
    dijit.setWaiRole(this.domNode, "group");
    dijit.setWaiRole(this.containerNode, "presentation");

    // when the Category is hidden at the beginning, RPT complains about missing label:
    // Elements having a WAI-ARIA group role must have a label specified with aria-label or aria-labelledby
    if (this.id) {
      var nlsKey = "task_category_" + this.id + "_collapsed";
      dijit.setWaiState(this.domNode, "label", this._nlsResources[nlsKey]);
    }


  },

  _setTitle: function setTitle_Category(){
    var count = "0";
    // currently, "_collapsed" and "_more_available" suffix do exclude each other
    // (so only 3 versions of the message yet). Could be four in the future though. 
    var moreAvailableKeySuffix = ""; // by default, no suffix
    var sectionCollapsedSuffix = ""; // by default, no suffix
    if (this.count) {
      count = this.count;
      if (this.open) {
        // this category is expanded
        if (this._moreAvailable === true) {
          moreAvailableKeySuffix = "_more_available";
        }
      }
      else {
        // collapsed
        sectionCollapsedSuffix = "_collapsed";
      }
    }
    var title = "";
    if (this.id) {
      var nlsKey = "task_category_" + this.id + sectionCollapsedSuffix + moreAvailableKeySuffix;
      title = dojo.string.substitute(this._nlsResources[nlsKey], {
        1: count
      });
    }
    this.set("title", title);
    dijit.setWaiState(this.domNode, "label", title);
  },

  setCount: function setCount_Category(count, moreAvailable){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "count " + count + ", moreAvailable " + moreAvailable);
    if (moreAvailable !== undefined) {
      this._moreAvailable = moreAvailable;
    }
    if (count) {
      this.count = count;
    }
    if (this.count) {
      this._setTitle();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  getCount: function getCount_Category(){
    if (this.count && this.count > 0) {
      if (this._moreAvailable) {
        return this.count - 1;
      }
      else {
        return this.count;
      }
    }
    else {
      return 0;
    }
  },

  isIncomplete: function isIncomplete_Category(){
    if (this._moreAvailable) {
      return true;
    }
    else {
      return false;
    }
  },

  isDisplayed: function isDisplayed_Category(){
    return this._displayed;
  },

  isExpanded: function isExpanded_Category(){
    if (this.open) {
      return true;
    }
    else {
      return false;
    }
  },

  toggle: function toggle_Category(silent){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    if (silent !== true) {
      if (this.open) {
        this.onExpanded(this.count);
      }
      else {
        this.onCollapsed(this.count);
      }
    }
    com.ibm.bpm.social.util.common.Log.logEntry(2);
  },

  onCollapsed: function onCollapsed_Category(count){
    // event
    dojo.addClass(this.titleBarNode, "bpm-task-list-category-collapsed");
    this._setTitle();
  },

  onExpanded: function onExpanded_Category(count){
    // event
    dojo.removeClass(this.titleBarNode, "bpm-task-list-category-collapsed");
    this._setTitle();
  },

  show: function show_Category(){
    this._displayed = true;
    dojo.style(this.domNode, "display", "block");
  },

  hide: function hide_Category(){
    this._displayed = false;
    dojo.style(this.domNode, "display", "none");
  }

});

/*
 *  @param String categoryId
 *  @returns Date
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate = function(categoryId){
  var resultDate = null;
  if (categoryId) {
    var now = new Date();
    var endOfToday = new Date(now);
    endOfToday.setHours(23, 59, 59, 0);
    var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

    // getting the end of this week needs some more computation: depends on current weekday and locale
    // Note: there are no start and end due dates for the "atRisk" category - this is indicated by an extra flag 
    var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
    // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
    firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
    var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
    var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);

    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      resultDate = null;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      resultDate = now;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      resultDate = endOfToday;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      resultDate = endOfTomorrow;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      resultDate = endOfThisWeek;
    }
  }
  return resultDate;
};

/*
 *  @param String categoryId
 *  @returns Date
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate = function(categoryId){
  var resultDate = null;
  if (categoryId) {
    var now = new Date();
    var endOfToday = new Date(now);
    endOfToday.setHours(23, 59, 59, 0);
    var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

    // getting the end of this week needs some more computation: depends on current weekday and locale
    // Note: there are no start and end due dates for the "atRisk" category - this is indicated by an extra flag 
    var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
    // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
    firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
    var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
    var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);

    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      resultDate = now;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      resultDate = endOfToday;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      resultDate = endOfTomorrow;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      resultDate = endOfThisWeek;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      resultDate = null; // null, not empty string
    }
  }
  return resultDate;
};


/*
 *  This method makes categories comparable, so we can use < or > in order to find out
 *  whether an item is in a lower (later) or in a higher category.
 *  This is used, for example, to decide whether we have trailing collapsed categories (which should be suppressed).
 *  As numeric values we simply use multiples of 100, so if later on some category is introduced in the middle, we can give it
 *  a new value without having to shift the other values (e.g. give a new category a value of 350).
 *  
 *  @param String categoryId
 *  @returns int   some number that reflects the sort order, not necessarily adjacent
 *                 or -1 if the category id is not a known id 
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal = function(categoryId){
  var result = -1;
  if (categoryId) {
    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      result = 100;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
      result = 200;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      result = 300;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      result = 400;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      result = 500;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      result = 600;
    }
  }
  return result;
};


com.ibm.bpm.social.widgets.task.list.Category.OVERDUE = "overdue";
com.ibm.bpm.social.widgets.task.list.Category.AT_RISK = "atRisk";
com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY = "dueToday";
com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW = "dueTomorrow";
com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK = "dueThisWeek";
com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER = "dueLater";

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.config.FilterConfig"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.config.FilterConfig"] = true;
// FilterConfig.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.config.FilterConfig");



/**
 * Filter Config: Settings about which items and attributes to display or filter out and
 */
dojo.declare("com.ibm.bpm.social.widgets.task.config.FilterConfig", null, {
  id: null,
  titleKey: null,
  interactionFilter: null,
  queryFilter: null,
  categories: null,
  displayedProperties: null,

  /**
   * a list of sort properties as expected by the entity query as properties in queue.item.properties
   * {property: "PROPERTY_KEY", isSort: true/false, sortDir: "ASC/DESC"}
   */
  sortProperties: null,

  constructor: function constructor_FilterConfig(params){
    if (params) {
      dojo.safeMixin(this, params);
    }
  }
});

com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS = "customViews";
com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS = "completedTasks";
com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS = "openTasks";


com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigOpenTasks = function(){

  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,
    titleKey: "task-list-filter-open",
    interactionFilter: "ASSESS_AND_WORK_ON",
    queryFilter: null,
    categories: [com.ibm.bpm.social.widgets.task.list.Category.OVERDUE,
                 com.ibm.bpm.social.widgets.task.list.Category.AT_RISK,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER],
    /* Caution: if sort properties are changed the sort algorithm for delta refresh may need to be changed!!! */
    sortProperties: [{
                       property: "IS_AT_RISK",
                       isSort: true,
                       sortDir: "DESC"
                     },
                     {
                       property: "DUE",
                       isSort: true,
                       sortDir: "ASC"
                     },
                     {
                       property: "TKIID",
                       isSort: true,
                       sortDir: "ASC"
                     }],

    selectedProperties: ["TAD_DISPLAY_NAME",
                         "PRIORITY",
                         "DUE",
                         "PI_DISPLAY_NAME",
                         "ASSIGNED_TO_ROLE_DISPLAY_NAME",
                         "NAME",
                         "PI_PIID",
                         "PI_NAME",
                         "OWNER",
                         "STATE",
                         "IS_AT_RISK",
                         "TKIID"],

    displayedProperties: ["TAD_DISPLAY_NAME",
                          "PRIORITY",
                          "DUE",
                          "PI_DISPLAY_NAME",
                          "ASSIGNED_TO_ROLE_DISPLAY_NAME"]
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCompletedTasks = function(){
  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS,
    titleKey: "task-list-filter-completed",
    interactionFilter: "CHECK_COMPLETED",
    queryFilter: null, // TODO
    categories: null,
    /* Caution: if sort properties are changed the sort algorithm for delta refresh may need to be changed!!! */
    sortProperties: [{
      property: "COMPLETED",
      isSort: true,
      sortDir: "DESC"
    }],
    selectedProperties: ["TAD_DISPLAY_NAME",
                         "PI_DISPLAY_NAME",
                         "COMPLETED",
                         "NAME",
                         "PI_PIID",
                         "PI_NAME",
                         "OWNER",
                         "STATE",
                         "TKIID"],

    displayedProperties: ["TAD_DISPLAY_NAME",
                          "PI_DISPLAY_NAME",
                          "COMPLETED"]
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews = function(){
  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS,
    titleKey: "",
    interactionFilter: null,
    queryFilter: null,
    categories: null,
    sortProperties: [],
    displayedProperties: []
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById = function(id){
  if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews();
  }
  else if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCompletedTasks();
  }
  else if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigOpenTasks();
  }
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING =
// maps query columns to details properties
[{
   column: "TKIID",
   property: "tkiid"
 },
 {
   column: "TASK.TKIID",
   property: "tkiid"
 },
 {
   column: "NAME",
   property: "name"
 },
 {
   column: "TAD_DISPLAY_NAME",
   property: "displayName"
 },
 {
   column: "PRIORITY",
   property: "priority"
 },
 {
   column: "IS_AT_RISK",
   property: "isAtRisk"
 },
 {
   column: "DUE",
   property: "dueTime"
 },
 {
   column: "COMPLETED",
   property: "completionTime"
 },
 {
   column: "OWNER",
   property: "owner"
 },
 {
   column: "STATE",
   property: "state"
 },
 {
   column: "PI_PIID",
   property: "piid"
 },
 {
   column: "PI_NAME",
   property: "processInstanceName"
 },
 {
   column: "PI_DISPLAY_NAME",
   property: "processInstanceName"
 }];

/**
 * 
 * @param items an array of task details as returned from a push event
 * @returns an aray of task details as returned from a query
 */
com.ibm.bpm.social.widgets.task.config.FilterConfig.mapTasks = function(items){
  var mappedItems = new Array();
  if (items) {
    for ( var i = 0; i < items.length; i++) {
      var item = items[i];
      if (item) {
        var mappedItem = {};
        for ( var m = 0; m < com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING.length; m++) {
          var property = com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING[m].property;
          if (item[property] !== undefined) {
            var column = com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING[m].column;
            mappedItem[column] = item[property];
          }
        }
        if (item.assignedToType == "user" && item.assignedTo) {
          mappedItem.OWNER = item.assignedTo;
        }
        else if (item.assignedToType == "group") {
          if (item.assignedToDisplayName) {
            mappedItem.ASSIGNED_TO_ROLE_DISPLAY_NAME = item.assignedToDisplayName;
          }
          if (item.assignedTo) {
            mappedItem.ASSIGNED_TO_ROLE = item.assignedTo;
          }
        }
        mappedItems.push(mappedItem);
      }
    }
  }
  return mappedItems;
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.config.ViewConfig"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.config.ViewConfig"] = true;
// ViewConfig.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.config.ViewConfig");


/**
 * View Config: Settings about the view content of a WorkTab sub tab
 */
dojo.declare("com.ibm.bpm.social.widgets.task.config.ViewConfig", null, {

  id: null,

  allowCollapseExpand: true, // or false

  viewType: "list", // or table

  queueName: "IBM.DEFAULTALLTASKSLIST_75",

  titleKey: null,

  selectedAttributes: null,

  tooltipKey: null,

  constructor: function constructor_ViewConfig(obj){
    dojo.safeMixin(this, obj);
  }

});

com.ibm.bpm.social.widgets.task.config.ViewConfig.getMyTasksConfig = function(){

  var viewConfig = new com.ibm.bpm.social.widgets.task.config.ViewConfig({
    id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS,
    viewType: "list",
    allowCollapseExpand: true,
    queueName: "IBM.DEFAULTALLTASKSLIST_75",
    titleKey: "task_myTasks",
    tooltipKey: "task_myTasks_tooltip",
    selectedAttributes: null
  });


  // TODO activate if ASSIGNED_TO_ROLE is available
  // "TKIID,TAD_DISPLAY_NAME,PRIORITY,DUE,PI_DISPLAY_NAME,OWNER,ASSIGNED_TO_ROLE_DISPLAY_NAME,STATE");

  return viewConfig;
};

com.ibm.bpm.social.widgets.task.config.ViewConfig.getCustomViewsConfig = function(){

  var viewConfig = new com.ibm.bpm.social.widgets.task.config.ViewConfig({
    id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES,
    viewType: "table",
    allowCollapseExpand: false,
    queueName: null,
    titleKey: "task_customViews",
    tooltipKey: "task_customViews_tooltip",
    selectedAttributes: null
  });
  return viewConfig;
};

com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS = "myTasks";
com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES = "SavedSearches";

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.PageSettings"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.PageSettings"] = true;
// PageSettings.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.PageSettings");






dojo.declare("com.ibm.bpm.social.util.common.PageSettings", null, {


  // copy constructor
  constructor: function constructor_PageSettings(obj){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.tab = this.tabPrefixes.WORK;
    this.defaultPageString = this.tabPrefixes.WORK + "/" + this.profileFilterIds["OPEN"];

    if (obj) {
      dojo.safeMixin(this, obj);
      this.createProfileString();
    }
    else {
      this._currentProfileString = this.defaultPageString;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * the selected/active tab, default /tasks
   */

  tab: "",


  /**
   * URL of the dashboard
   */
  dashboardURL: "",

  /**
   * filterConfigId: empty or one of
   * 
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS
   * 
   */
  filterConfigId: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,

  /**
   * searchString: empty or plain text representation of the search string as entered by the user
   */
  searchFilterString: "",

  /**
   * savedSearchName: name of the used search if page is saved searches
   */
  savedSearchName: "",


  /**
   * sortAttributes: Array of attributes the list is sorted
   */
  sortAttributes: [],

  /**
   * Cache for the last calculated profileString
   * 
   */
  _currentProfileString: "",

  /**
   * Indicate if we found the PageSettings valid
   */
  isValid: true,


  /**
   * Valid parameters in the user attributes string
   */
  profileParameters: {
    QUERY: "query",
    SEARCHFILTER: "searchFilter",
    SORTATTRIBUTES: "sortAttributes",
    SORTORDER: "sortOrder",
    DASHBOARD: "dashboard"

  },

  /**
   * URL-Representation of filters
   */
  profileFilterIds: {
    OPEN: "open",
    COMPLETED: "completed",
    QUERIES: "queries"
  },


  /**
   * Map describing which string in the user attribute maps to which filter <br>
   * !!! Needs to be kept aligned with profileFilterIds !!!
   * 
   */
  filterIdMapping: {
    "open": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,
    "completed": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS,
    "queries": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS
  },


  /**
   * Available tabs
   */
  tabPrefixes: {
    WORK: "/tasks",
    DASHBOARDS: "/dashboards"
  },


  /**
   * default value for the start page, calculated in constructor
   */
  defaultPageString: "",

  /**
   * Reset the class to it default values
   * 
   * @type {void}
   */
  reset: function reset_PageSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.savedSearchName = "";
    this.searchFilterString = "";
    this.filterConfigId = com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS;
    this.sortAttributes = [];
    this.dashboardURL = "";

    // update the current profile string
    this.createProfileString();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Return the cached profile string
   * 
   * @type {void}
   */
  getCurrentProfileString: function getCurrentProfileString_PageSettings(){
    return this._currentProfileString;
  },

  /**
   * Take the string an parse it into a new PageSettings object<br>
   * Note: This function does not change the until the doMixin is set to true
   * 
   * @param {String} a string like from the user profile
   * @param {Boolean} if true and the string is found valid, copy the settings into the current object
   * @returns {com.ibm.bpm.social.util.common.PageSettings}
   */
  parseFromString: function parseFromString_PageSettings(profileString, doMixin){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var params = null;
    var param = "";
    var paramKey = "";
    var paramValue = "";
    var parts = null;
    var prefixPart = "";
    var desiredFilter = "";
    var sortAttributes = [];
    var sortOrders = [];
    var sortOrder = "";
    var valid = false;

    var newPageSettings = com.ibm.bpm.social.util.common.PageSettings();


    // Prefix check, string must start with anything from this.tabPrefixes
    for ( var i in this.tabPrefixes) {
      if (profileString.indexOf(this.tabPrefixes[i]) === 0) {
        valid = true;
        newPageSettings.tab = this.tabPrefixes[i];
        com.ibm.bpm.social.util.common.Log.log(3, "found valid prefix " + newPageSettings.tab);
        break;
      }
    }

    if (valid) {
      // check if there are more parts
      parts = profileString.split("?");
      prefixPart = parts[0] ? parts[0] : "";
      if (parts.length > 1) {
        params = parts[1].split("&");
      }
      else {
        params = null;
      }


      // if we got add. parameters appended with "?", read the params
      if (params) {
        for (i = 0; i < params.length; i++) {
          param = params[i].split("=");
          if (param.length == 2 && param[0] && param[1]) {
            paramKey = param[0];
            paramValue = param[1];

            if (paramKey == this.profileParameters.QUERY) {
              newPageSettings.savedSearchName = window.decodeURIComponent(paramValue);
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&lt;/g,'<');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&gt;/g,'>');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&quot;/g,'"');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&amp;/g,'&');
              
            }
            else if (paramKey == this.profileParameters.SEARCHFILTER) {
              newPageSettings.searchFilterString = window.decodeURIComponent(paramValue);
            }
            else if (paramKey == this.profileParameters.SORTATTRIBUTES) {
              sortAttributes = paramValue.split(",");
            }
            else if (paramKey == this.profileParameters.SORTORDER) {
              sortOrders = paramValue.split(",");
            }
            else if (paramKey == this.profileParameters.DASHBOARD) {
              newPageSettings.dashboardURL = window.decodeURIComponent(paramValue);
            }
          }
        }

        // set the sortAttributes array from the strings
        newPageSettings.sortAttributes = [];
        for (i = 0; i < sortAttributes.length; i++) {
          if (sortAttributes[i]) {
            sortOrder = sortOrders[i];
            if (sortOrders[i] != "ASC" && sortOrders[i] != "DESC") {
              sortOrder = "ASC";
            }
            newPageSettings.sortAttributes.push({
              property: sortAttributes[i],
              sortDir: sortOrder
            });
          }
        }
      }


      // Prefix processing for the Work tab
      if (newPageSettings.tab == this.tabPrefixes.WORK) {
        // read filter config
        desiredFilter = prefixPart.substr(prefixPart.lastIndexOf("/") + 1); // "/tasks/open"  -> "open"
        newPageSettings.filterConfigId = this.filterIdMapping[desiredFilter];
        if (!newPageSettings.filterConfigId) {
          newPageSettings.filterConfigId = this.filterIdMapping[this.profileFilterIds.OPEN];
          com.ibm.bpm.social.util.common.Log.log(2, "Invalid filter from profile, using default filter: " +
            newPageSettings.filterConfigId);
        }

        // sanity check: no queries filter w/o savedSearchName
        if (desiredFilter == this.profileFilterIds.QUERIES && !newPageSettings.savedSearchName) {
          com.ibm.bpm.social.util.common.Log.log(2,
            "Invalid name for saved search from profile, switching to default MyTasks/Open Tasks");
          valid = false;
        }
      }


      // Prefix processing for the Dashboards tab
      else if (newPageSettings.tab == this.tabPrefixes.DASHBOARDS) {
        // Check if we got a valid URL
        if (newPageSettings.dashboardURL === "") {
          com.ibm.bpm.social.util.common.Log.log(2, "missing dashboard URL, switching to default MyTasks/Open Tasks");
          valid = false;
        }
      }
    }

    newPageSettings.isValid = valid;

    if (valid && doMixin) {
      // update the profile string presentation
      newPageSettings.createProfileString();
      dojo.safeMixin(this, newPageSettings);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return newPageSettings;
  },

  /**
   * Take all relevant settings and create a string that can be written into the users profile
   * 
   * @type {void}
   */
  createProfileString: function createProfileString_PageSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var filterPart = "";
    var params = [];
    var result;

	if (this.dashboardURL != "") {
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'isWorkTab');
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'isScoreboard');
		// ensure that the processApp, serviceName and snapshot are not duplicated
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'processApp',1);
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'serviceName',1);
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'snapshot',1);
	}

    if (this.tab == this.tabPrefixes.DASHBOARDS) {
      result = this.tabPrefixes.DASHBOARDS;
      params.push(this.profileParameters.DASHBOARD + "=" + window.encodeURIComponent(this.dashboardURL));
    }
    else {

      // saved searches
      if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {

        // saved search name
        filterPart = this.profileFilterIds.QUERIES;
        params.push(this.profileParameters.QUERY + "=" + window.encodeURIComponent(this.savedSearchName));

        // sort attributes
        // ASZ: Tempoariliy disabled to be consistent in reading and writing, because reading from profile doesn't work correctly for sort.
//      if (this.sortAttributes && this.sortAttributes.length > 0) {
//        var sortAttributes = this.profileParameters.SORTATTRIBUTES + "=";
//        var sortOrder = "&" + this.profileParameters.SORTORDER + "=";
//        for ( var i = 0; i < this.sortAttributes.length; i++) {
//          if (i > 0) {
//            sortAttributes += ",";
//            sortOrder += ",";
//          }
//          sortAttributes += this.sortAttributes[i].property;
//          sortOrder += this.sortAttributes[i].sortDir;
//        }
//        params.push(sortAttributes + sortOrder);
//      }
      }
      // my task view
      else {
        if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
          filterPart = this.profileFilterIds.OPEN;
        }
        else if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
          filterPart = this.profileFilterIds.COMPLETED;
        }
      }

      if (this.searchFilterString) {
        params.push(this.profileParameters.SEARCHFILTER + "=" + window.encodeURIComponent(this.searchFilterString));
      }

      // compose String
      result = this.tabPrefixes.WORK + "/" + filterPart;
    }

    if (params.length > 0) {
      result += "?";
      while (params.length > 0) {
        result += params.shift();
        if (params.length > 0) {
          result += "&";
        }
      }
    }

    // cache the string for quick access
    this._currentProfileString = result;

    com.ibm.bpm.social.util.common.Log.logExit(2, "Calculated Start page string is: " + result);
    return result;
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.DashboardController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.DashboardController"] = true;
// DashboardController.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.portal.DashboardController");







dojo.declare("com.ibm.bpm.social.portal.DashboardController", [dijit.layout.ContentPane], {

  dashboardURL: "", // the currently selected dashboard's URL

  dashboardTitle: "", // the title set for the dashboard's URL

  pageHandler: null,

  _currentPageSettings: null,

  _jspUrl: "/jsp/socialPortal/dashboard.jsp",

  _subscribedChannels: null,

  _lastDashboardParams: null,

  /**
   * postCreate
   * 
   * @type {void}
   */
  postCreate: function postCreate_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);


    this.loadingMessage = ""; // overwrite the message from ContentPane, since we show our own one

    this.pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();
    this._currentPageSettings = new com.ibm.bpm.social.util.common.PageSettings(); // just a default, will be populated later 
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

    // create the start page icon/button
    this.startPageButton = this.pageHandler.createStartPageIcon();
    dojo.addClass(this.startPageButton, "startPageBookmark-dashboards");
    this.connect(this.startPageButton, "onclick", this._onStartPageButtonClick);
    this.connect(this.startPageButton, "onkeypress", function(e){
      if (e.keyCode == dojo.keys.ENTER) {
        this._onStartPageButtonClick(e);
      }
    });

    this._subscribedChannels = [];
    this._lastDashboardParams = new Object();

    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, dojo.hitch(this,
      "_onDashboardSelected")));
    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.DASHBOARD_LOADED, dojo.hitch(this,
      "_dashboardLoaded")));
    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD, dojo.hitch(this,
      "_forceNewDashboard")));

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Cleanup our resources
   * 
   * @type {void}
   */
  destroy: function destroy_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._lastDashboardParams = null;
    for ( var i = 0; i < this._subscribedChannels.length; i++) {
      if (this._subscribedChannels[i]) {
        dojo.unsubscribe(this._subscribedChannels[i]);
      }
    }

    this.inherited(arguments);

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

    /**
    * Event handler subscribed on topic com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD
    * 
    * @type {void}
    */
    _onDashboardSelected: function _onDashboardSelected_DashboardController(msg) {
    /**
    * The restartDashboardCookie keeps track of whether restartable dashboards are 
    * at the "restart" page.
    * 
    * The cookie is of the form:
    * {
    *   lastDashboardURL: string -> The last dashboard URL that was run (used in dashboard_restart.jsp)
    *   dashboards: array -> List of dashboards that have been accessed
    *       {
    *           dashboardURL: string -> The URL of this dashboard
    *           atRestartDashboard: boolean -> true if dashboard is at "restart" page
    *       }
    * }
    * Cookie expires in one day, and is deleted when the user logs out
    */
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (msg.runURL) {
            // remove the zForceNew param, since the URL is stored w/o that param
            var runURL = msg.runURL.replace("&zForceNew=true", "");
            var cookie = dojo.cookie("restartDashboardCookie");
            
            var url = runURL;
            if (url == 'RESTART_DASHBOARD') {
            	url = this.dashboardURL;
            	if (cookie) {
            		var cookieValues = dojo.fromJson(cookie);
            		url = cookieValues.lastDashboardURL;
            	}
            }
            var urlParamString = url.substring(url.indexOf('?')+1);
            var urlParamList = urlParamString.split('&');
            var urlParams = new Object();
            for (var p1=0; p1<urlParamList.length; p1++) {
            	var p2 = urlParamList[p1].indexOf('=');
            	if (p2 > -1) {
            		urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
            	}
            }
            if ((urlParams.processApp == undefined) && (runURL.indexOf('viewScoreboard.do') != -1)) {
            	// special case for scoreboard
            	urlParams.processApp = 'TWP';
            	urlParams.serviceName = 'Scoreboard';
            } 	
            
			var newHash = 'dashboards/'+urlParams.processApp+'/'+urlParams.serviceName;
			if (urlParams.snapshot) {
				newHash = newHash + '/'+urlParams.snapshot;
			}
			var newHashParams = "";
			for (var paramName in urlParams) {
				if ((paramName != 'processApp') && (paramName != 'serviceName') && (paramName != 'snapshot')) {
					if (newHashParams) {
						newHashParams = newHashParams + '&';
					}
					newHashParams = newHashParams + encodeURIComponent(paramName) + '=' + encodeURIComponent(urlParams[paramName]);
				}
			}
			var msgParamString = "";
			if (msg.params) {
				for (var msgParamName in msg.params) {
					if (msgParamString) {
						msgParamString = msgParamString + '&';
					}
					msgParamString = msgParamString + encodeURIComponent(msgParamName) + '=' + encodeURIComponent(msg.params[msgParamName]);
				}
			}
			if (newHashParams) {
				newHash = newHash + '?' + newHashParams;
				if (msgParamString) {
					newHash = newHash + '&' + msgParamString;
				}
			} else {
				if (msgParamString) {
					newHash = newHash + '?' + msgParamString;
				}
			}
			newHash = com.ibm.bpm.social.util.common.Util.removeParameter(newHash,'isWorkTab');
			newHash = com.ibm.bpm.social.util.common.Util.removeParameter(newHash,'zResumable');
			com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
			
			var tmpParamString = undefined;
			if (msgParamString) {
				tmpParamString =  com.ibm.bpm.social.util.common.Util.removeParameter('?'+msgParamString,'isWorkTab');
			}
			
			// if no snapshot specified, determine which snapshot to use
			if (!urlParams.snapshot) {
				urlParams.snapshot = "tip";
				var item = this.pageHandler._findDashboardFromURL(url);
				if (item) {
					var runParams = this.pageHandler._parseQueryString(item.params.runURL);
					if (runParams.snapshot) {
						urlParams.snapshot = runParams.snapshot;
					}
				}
			} 
			if (this._lastDashboardParams[urlParams.processApp] == undefined) {
				this._lastDashboardParams[urlParams.processApp] = new Object();
			}
			if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName] == undefined) {
				this._lastDashboardParams[urlParams.processApp][urlParams.serviceName] = new Object();
			}
			if (tmpParamString) {
				if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] != tmpParamString) {
					this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] = tmpParamString;
			        msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
				}
				msg.runURL = msg.runURL + '&' + msgParamString;
			} else {
				if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot]) {
					this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] = '';
					msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
				}
			}
            
			if (dojo.indexOf(window.zResumable,urlParams.processApp) == -1) {
				msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
			}

            if (runURL !== "RESTART_DASHBOARD") {
                var cookieValues;
                if (cookie) {
                    cookieValues = dojo.fromJson(cookie);
                    if(this._isUrlInCookie(cookieValues, runURL) === false) {
                        // Add the current dashboard to the cookie if it is not already included                        
                        cookieValues.dashboards.push({
                            atRestartDashboard: false,
                            dashboardURL: runURL
                        });
                    }
                } else {
                    // There is no cookie, so create one
                    var dashboardsArray = [{
                        atRestartDashboard: false,
                        dashboardURL: runURL
                    }];
                    cookieValues = { dashboards: dashboardsArray };
                }

                this.dashboardURL = msg.runURL;
                this.dashboardTitle = (msg.display ? msg.display : "dashboard");
                for (var i = 0; i < cookieValues.dashboards.length; i++) {
                    var cur = cookieValues.dashboards[i];
                    if (cur.dashboardURL === runURL && cur.atRestartDashboard === true) {
                        var newUrl = runURL + "&zForceNew=true";
                        cookieValues.dashboards[i].atRestartDashboard = false;
                        this.dashboardURL = newUrl;
                    }
                }

                // Lastly, update the lastDashboardURL to be the current URL
                cookieValues.lastDashboardURL = runURL;
                dojo.cookie("restartDashboardCookie", dojo.toJson(cookieValues), {expires: 1, path: '/'});
            } 
            else { // msg.runURL === "RESTART_DASHBOARD"
                if (cookie) {
                    var cookieValues = dojo.fromJson(cookie);
                    this.dashboardURL = cookieValues.lastDashboardURL + '&zForceNew=true';
                    for (var i = 0; i < cookieValues.dashboards.length; i++) {
                        var cur = cookieValues.dashboards[i];
                        if (cur.dashboardURL === cookieValues.lastDashboardURL) {
                            cur.atRestartDashboard = false;
                        }
                    }

                    dojo.cookie("restartDashboardCookie", dojo.toJson(cookieValues), {expires:1, path:'/'});
                } else {
                    this.dashboardURL += '&zForceNew=true';
                }
            }

            var content = ppConfig.portalContextRootJS + this._jspUrl;
            content += '?url=' + escape(this.dashboardURL) + '&title=' + this.dashboardTitle;

            // remove the startPageButton from the document, else IE will purge the inner span
            if (this.startPageButton.parentNode) {
                this.startPageButton.parentNode.removeChild(this.startPageButton);
            }

			// if this is the work tab, load it
			if (msg.params && msg.params.isWorkTab && msg.params.isWorkTab==true) {
				this.pageHandler.selectWorkTab();
			} else {
				// now load the dashboard by setting the ContentPane href attrib
				this.set("href", content);
				this.pageHandler.setSelectedTab('dashboardOrganizerTab');
			}
        }
        else {
            com.ibm.bpm.social.util.common.Log.logError(1, "missing parameter: runURL");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);

        return;
    },

	_forceNewDashboard: function _forceNewDashboard_DashboardController(processApp,serviceName,snapshot) {
		if (this._lastDashboardParams[processApp] == undefined) {
			this._lastDashboardParams[processApp] = new Object();
		}
		if (this._lastDashboardParams[processApp][serviceName] == undefined) {
			this._lastDashboardParams[processApp][serviceName] = new Object();
		}
		if (!snapshot) {
			snapshot = "tip";
			var item = this.pageHandler._findDashboardFromParams({
				processApp: processApp,
				serviceName: serviceName
			});
			if (item) {
				var runParams = this.pageHandler._parseQueryString(item.params.runURL);
				if (runParams.snapshot) {
					snapshot = runParams.snapshot;
				}
			}
		} 
		this._lastDashboardParams[processApp][serviceName][snapshot] = 'FORCE_NEW_DASHBOARD=true';
	},
	
    /**
    * Determine whether there is already an entry in the cookie for the 
    * current URL
    * 
    * Returns boolean
    */
    _isUrlInCookie: function _urlInCookie_DashboardController(cookieValues, compareURL) {
        var dashboardURLArray = cookieValues.dashboards;
        for (var i = 0; i < dashboardURLArray.length; i++) {
            if (dashboardURLArray[i].dashboardURL === compareURL) return true;
        }
        return false;
    },

  /**
   * 
   * @type {void}
   */
  getPageSettings: function getPageSettings_DashboardController(){
    com.ibm.bpm.social.util.common.Log.log(2);
    return this._currentPageSettings;
  },

  /**
   * Return page settings for the current dashboard
   * 
   * @type {void}
   */
  _createPageSettings: function _createPageSettings_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);


    // cut of the host:port part of teh dashboards URL  http(s)://host:port/portal/jsp/andSoOn
    var urlPart = this.dashboardURL.split('/'); // == ["http(s):", "", "host:port", "portal", "jsp", "andSoOn"]
    var path = this.dashboardURL;
    if (urlPart[0]) {
    	urlPart.splice(0, 3);
    	path = "/" + urlPart.join("/");
	}

    var pageSettings = new com.ibm.bpm.social.util.common.PageSettings({
      tab: this._currentPageSettings.tabPrefixes.DASHBOARDS,
      dashboardURL: path
    });

    this._currentPageSettings = pageSettings;

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return pageSettings;
  },

  /**
   * Overwrite of ContentPane. This is called when the jsp is loaded (however, the iframe may take longer)
   * 
   * @type {void}
   */
  onDownloadEnd: function onDownloadEnd_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    var startPageButtonDiv = dojo.byId("startPageButtonDashboards");
    startPageButtonDiv.innerHTML = this._nlsResources['task-list-loading'];
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * Update the start page icon after the content of the iframe is loaded
   * 
   * @type {void}
   */
  _dashboardLoaded: function _dashboardLoaded_DashboardController(dashboardInfo){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "url:" + dashboardInfo.url + ", title:" + dashboardInfo.title);

    var startPageButtonDiv = dojo.byId("startPageButtonDashboards");
    // this hides the loading message and shows the start page icon
    startPageButtonDiv.innerHTML = "";
    startPageButtonDiv.appendChild(this.startPageButton);
    this.pageHandler.checkAndUpdateStartPageIcon(this._createPageSettings(), this.startPageButton);
	// when a dashboard is loaded or reloaded, make sure navigation is enabled    
    dojo.publish(com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION,[]);
    this.startPageButton.focus();//a11y
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Internal event handler for the start page button
   * 
   * @type {void}
   */
  _onStartPageButtonClick: function _onStartPageButtonClick_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var currentPageSettings = this._createPageSettings();
    this.pageHandler.setStartPage(currentPageSettings, this.startPageButton);

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.DashboardOrganizerSettings"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.DashboardOrganizerSettings"] = true;
// DashboardOrganizerSettings.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.DashboardOrganizerSettings");


/*
	DashboardOrganizerSettings
	Manages retrieval, parsing, persisting a string representing the order in which dashboards, scoreboards and saved searches should be displayed on the Portal tab bar.
	It also identifies which dashboards/saved searches are currently hidden yet available for display
	
	I've avoided using names in the preference string (dashboard names, saved search names) for two reasons:
		1. Dashboards can be renamed by the user.  If the pref string were to identify a dashboard's order by using the name, and
			the dashboard were renamed, then the pref value would be stale and we wouldn't be able to find a match  between the preference
			identifier and the exposed dashboard.
		2. Saved searches unfortunately allow any characters in their names, e.g., []{}!@#$%^&*()_+?></\, which makes it difficult
			to create a *simple* preference string that has separators between items - no character can be safely used as a separator.
			Instead, I use the savedSearchID returned by REST.
	
	Preference string details:
	
	Here's an example preference string; I'll describe it below.
	The preference string (Portal Dashboard Display Order) is comprised of identifiers separated by forward slashes "/".
		
				     1             2         3			     4           5             6
			2015.25-ff89fa3ef0b3/1003/2015.43-dce9fa3b03fe/HIDDEN/2015.65-ecdfbcb03fa/1004
			
	1. Visible Dashboard
	2. Saved Search
	3. Visible dashboard
	4. Separator between visible and hidden items
	5. Hidden dashboard
	6. Hidden Saved Search

	*IMPORTANT NOTE ABOUT PRE-POPULATING THE PREF STRING*
	Note that Bulk User Attribute Assignment in Process Center/Server Admin can be used to populate the Portal Dashboard Display Order preference.
	The only part of the string example shown above that needs to be pre-populated is the dashboard ID or saved search ID.  In other words,
	for dashboards the part following "-" doesn't need to be pre-populated.  An administrator can easily find the ID of a dashboard
	via the REST exposed items service (as shown below) and create a pref string like:   2015.25/2015.66/1023/HIDDEN/2016.45
	The Dashboard Organizer will automatically modify the pref string to include the second segment.
	
	*Dashboard identifiers*
		A dashboard identifier is comprised of two attributes returned by the REST exposed items service:
			1. The ID attribute for the dashboard
				For example:       "ID":"2015.25".
			2. '-'
			3. The last segment of the "itemID" attribute for the dashboard
				for example:     "itemID":"2066.4fc070d9-4b5c-4905-bee4-dce9f6f903fe".  The last segment is used as the second part of the identifier
								 in this example   dce9f6f903fe
				Result:   2015.25-dce9f6f903fe
		    This second segment is needed so we can match up items even when the following occurs:
			   1. If a user installs a new snapshot of an app ON PROCESS SERVER then we won't find the *previous* newest snapshot without the 2nd segment.
			   2. If a user sets a new snapshot to be the default version ON BOTH PC AND PS then we won't find the *previous* default version without the 2nd segment.
			

	*Saved Search identifiers*
		A saved search identifier is the item's savedSearchID.
	
*/


dojo.declare("com.ibm.bpm.social.util.common.DashboardOrganizerSettings", null, {


  // copy constructor
  constructor: function constructor_DashboardOrganizerSettings(obj){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.defaultProfileString = "[]";
	this.dashboardOrderObj=[];
    if (obj) {
      dojo.safeMixin(this, obj);
      this.createProfileString();
    }
    else {
      this._currentProfileString = this.defaultProfileString;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * Cache for the last calculated profileString
   * 
   */
  _currentProfileString: "",

  /**
   * Indicate if we found the DashboardOrganizerSettings valid
   */
  isValid: true,
  MAX_PREF_STRING_LENGTH : 999,


  /**
   * Valid parameters in the user attributes string
   */
  profileParameters: {
    HIDDEN: "HIDDEN",
	DASHSEP: "/"
  },

  /**
   * default value for the start page, calculated in constructor
   */
  defaultProfileString: "[]",

  /**
   * Reset the class to it default values
   * 
   * @type {void}
   */
  reset: function reset_DashboardOrganizerSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.defaultProfileString = "[]";
	this.dashboardOrderObj=[];
    // update the current profile string
    this.createProfileString();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Return the cached profile string
   * 
   * @type {void}
   */
  getCurrentProfileString: function getCurrentProfileString_DashboardOrganizerSettings(){
    return this._currentProfileString;
  },

  /**
   * Take the string and parse it into a new DashboardOrganizerSettings object<br>
   * Note: This function does not change the class until the doMixin is set to true
   * 
   * @param {String} a string - dashboard page order from the user profile
   * @param {Boolean} if true and the string is found valid, copy the settings into the current object
   * @returns {com.ibm.bpm.social.util.common.DashboardOrganizerSettings}
   */
  parseFromString: function parseFromString_DashboardOrganizerSettings(profileString, doMixin){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var newDashboardOrganizerSettings = com.ibm.bpm.social.util.common.DashboardOrganizerSettings();

	var valid=true;
    var dashboardObj=[];
	profileString = unescape(profileString);

	com.ibm.bpm.social.util.common.Log.log(2, "about to parse dashboard order preference string, value is: [" + profileString+"]");
	dashboardObj=profileString.split(this.profileParameters.DASHSEP);
	
	if (valid) {
		if (dojo.isArray(dashboardObj)) {
			dojo.forEach(dashboardObj, dojo.hitch(this,function(item){
				if (item){
					var baseIDSeg="", itemIDSeg="";
					var arr = item.split("-");
					if (arr && dojo.isArray(arr)) {
						baseIDSeg = arr[0];   // identifier for dashboard on Process Center, and saved search. 
						if (arr.length==2) {  // identifier for dashboard on Process Server will have this second part; if this is a saved search, this segment isn't needed.
							itemIDSeg=arr[1];
						}
						var dashItem={};
						dashItem["baseIDSeg"]=baseIDSeg;
						dashItem["itemIDSeg"]=itemIDSeg;
						newDashboardOrganizerSettings.dashboardOrderObj.push(dashItem);
					} else {
						console.log("invalid item: "+item);
					}
				}
			}));
			
		} else {
			valid=false;
		}
	}
		
	
    newDashboardOrganizerSettings.isValid = valid;

    if (valid && doMixin) {
      // update the profile string presentation
      newDashboardOrganizerSettings.createProfileString();
      dojo.safeMixin(this, newDashboardOrganizerSettings);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return newDashboardOrganizerSettings;
  },
	  
  /**
   * Take all relevant settings and create a string that can be written into the users profile
   * 
   * @type {void}
   */
  createProfileString: function createProfileString_DashboardOrganizerSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var result="";

	
	dojo.forEach(this.dashboardOrderObj, dojo.hitch(this, function(item){
	    // Max length is 1000, due to database constraints;
		var curStr = item.baseIDSeg;
		if (item.itemIDSeg && item.itemIDSeg.length>0) {
			// the environment is process server; we'll add a '-' between identifier segments
			curStr = curStr+"-"+item.itemIDSeg;
		}
		curStr=curStr+this.profileParameters.DASHSEP;
		if ((this.MAX_PREF_STRING_LENGTH - result.length) >= curStr.length) {
			result+=curStr;
		}
	 }));

    // cache the string for quick access
    this._currentProfileString = result;

    com.ibm.bpm.social.util.common.Log.logExit(2, "Calculated Start page string is: " + result);
    return result;
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DataCache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DataCache"] = true;
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

//**************************************
//
// Version: 1.9 11/04/13 13:31:35
//              
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.DataCache");



dojo.declare("com.ibm.bpm.social.widgets.task.util.DataCache", null, {
  _data: {},
  _expirationDate: {},
  _usageCount: {},

  constructor: function(){},

  startCleanupProcess: function(period){
    var myself = this;
    window.setInterval(function(){
      myself._removeOutdatedEntries();
    }, period);
  },

  /**
   * Stores the data and the current date under the key.
   * 
   * @param {Object} key
   * @param {Object} data
   */
  set: function set_DataCache(key /* string */, data /* object */, expirationPeriod /* number (milliseconds) */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    this._data[key] = data;
    this._usageCount[key] = 0;
    if (expirationPeriod > 0) {
      this._expirationDate[key] = (new Date()).getTime() + expirationPeriod;
    }
    else {
      // default expiration to keep cache from increasing indefinitely
      // 1 hour = 3600000 milli seconds
      this._expirationDate[key] = (new Date()).getTime() + 3600000;
    }
    com.ibm.bpm.social.util.common.Log.logExit(3, "data:" + data);
  },

  /**
   * If there has been data cached for the url and it has not been expired, serve it.
   * 
   * @param {Object} key
   */
  get: function get_DataCache(key /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    var data = this._data[key];
    if (data) {
      var expirationDate = this._expirationDate[key];
      var now = (new Date()).getTime();
      if (expirationDate > now) {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> data available");
        this._usageCount[key]++;
        return data;
      }
      else {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> d=> data outdated");
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.logExit(3, "=> data not available");
    }
    return null;
  },

  _removeOutdatedEntries: function _removeOutdatedEntries_DataCache(){
    com.ibm.bpm.social.util.common.Log.logEntry(3);
    var now = (new Date()).getTime();
    for ( var key in this._expirationDate) {
      // check also for usage ?
      if (this._expirationDate[key] < now) {
        com.ibm.bpm.social.util.common.Log.log(2, "key:" + key);
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(3);
  }

});

com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE = new com.ibm.bpm.social.widgets.task.util.DataCache();
com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.startCleanupProcess(60000);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Systems"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Systems"] = true;
// Systems.js
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
// END COPYRIGHT 


//**************************************
//
// Version: 1.42 11/10/14 16:06:11
//
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.Systems");



/**
 * Wraps the result of the <i>/systems</i> REST API call <br>
 * TODO add more getters
 * 
 * @author admard
 * 
 */

/*
 * "systems":
      [
         {  "systemID": {"type": "string", 
               "description": "ID of the system that is described by this object."
            },
            "systemType": {"type": "string",
               "description": "Type of the system that is described by this object.",
               "enum":
               [
                  "WLE",
                  "WPS",
                  "ICM"
               ]
            },
            "version": {"type": "string",
               "description": "Version of the system that is described by this object.",
            },
            "groupWorkItemsEnabled": {"type": "boolean",
               "description": "States whether group workitems can be used."
            },
            "substitutionEnabled": {"type": "boolean",
               "description": "States whether substitution is enabled."
            },
            "substitutionManagementRestrictedToAdministrators": {"type": "boolean",
               "description": "States whether substitution management is restricted to administrators."
            },
            "taskHistoryEnabled": {"type": "boolean",
               "description": "States whether the collection of task history events is enabled."
            },
            "workBasketsEnabled": {"type": "boolean",
               "description": "States whether work baskets are enabled."
            },
            "businessCategoriesEnabled": {"type": "boolean",
               "description": "States whether business categories are enabled."
            }
         }
      ],
   "domains": 
      [
         {  "name": {"type": "string",
               "description": "Name of the domain."
            },
            "displayName": {"type": "string",
               "description": "Display name of the domain."
            },
            "systems": {"type": ["string"],
               "description": "List of IDs for the systems in this domain."
            }
         }
      ], "optional": true
 */

dojo.declare("com.ibm.bpm.social.widgets.task.util.Systems", null, {

  /**
   * The api type of the REST service endpoint.
   * <ul>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   * <li>TODO temporarily "old" if no system</li>
   * </ul>
   * 
   * @type {String}
   */
  _apiType: null,

  /**
   * Federated endpoint: maps system ids to a system object as returned by the <i>/systems</i> API for all
   * systems
   * 
   * @type {Map}
   */
  _systems: null,

  /**
   * Federated endpoint: an array of all system ids that are available to the federated endpoint systems
   * 
   * @type {Map}
   */
  _systemIDs: null,

  /**
   * Configuration information for single system endpoint
   * 
   * @type {Object}
   */
  _system: null,

  /**
   * Federated endpoint: maps domain names to a domain object as returned by the <i>/systems</i> API for all
   * domains
   * 
   * @type {Map}
   */
  _domains: null,

  /**
   * List of all available domains
   * 
   * @type {String[]}
   */
  _domainNames: null,

  _systemsSending401Error: null,

  /**
   * @constructor
   * @param {Object} params param.systems a systems object as returned by the <i>/systems</i> API</li>
   * @throws Error if the expected parameters are not specified
   */
  constructor: function constructor_Systems(params){

    this._systemsSending401Error = null;
    if (params && params.systemsSending401Error) {
      this._systemsSending401Error = params.systemsSending401Error;
    }

    this._systemIDs = [];

    if (params && params.systems) {

      var systems = params.systems.systems;
      var domains = params.systems.domains;

      if (systems) {
        if (systems.length > 0) {
          // domains
          if (domains && domains.length > 0) {
            // save map of domains
            this._domainNames = [];
            if (domains.length > 0) {
              this._domains = {};
              for (i = 0; i < domains.length; i++) {
                var domain = domains[i];
                this._domains[domain.name] = domain;
                this._domainNames.push(domain.name);
                // TODO: SA REST API workaround - domains.systems may contain null values
                dojo.forEach(domain.systems, function(system, index){
                  if (system === null) {
                    domain.systems.splice(index, 1);
                  }
                });
              }
            }

            // federated endpoint
            this._apiType = com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;

            // save systems
            if (systems.length) {
              this._systems = {};
            }
            for ( var i = 0; i < systems.length; i++) {
              if (systems[i].systemID) {
                this._systems[systems[i].systemID] = systems[i];
                this._systemIDs.push(systems[i].systemID);
              }
            }
          }
          else {
            // single system endpoint
            this._apiType = systems[0].systemType;
            // save system
            this._system = systems[0];
            this._systemIDs.push(systems[0].systemID);
          }
        }
        else {
          // log if no system information was returned
          // the error popup with the federation result should show errors
          com.ibm.bpm.social.util.common.Log.log(1, "Warning!!! No system meta information available! systems=" +
            params.systems);
        }
      }
      else {
        // TODO temp old API - no systems object
        this._apiType = "old";
        this._system = {
          version: "7.0.0.0",
          systemType: com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
          systemID: "oldWPS"
        };
      }
      com.ibm.bpm.social.util.common.Log.log(1, "Systems.constructor: " + this);
    }
    else {
      // ASZ: throw prevents cloning via dojo.clone  
      // throw new Error("Systems.constructor: The parameter 'param.systems' is required.");
      com.ibm.bpm.social.util.common.Log.log(1,
        "Warning!!! The parameter 'param.systems' should be set (Ignore if appears during dojo.clone)");
    }
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_Systems(){
    return this._apiType;
  },

  /**
   * Returns whether the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if and only if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * Returns whether the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if and only if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns whether the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if and only if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns whether the REST service endpoint is an ICM system
   * 
   * @returns {boolean} true if and only if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns the system properties of the REST service endpoint for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system for a federated REST service endpoint
   * @returns {Object} the system properties of a system as returned by the <i>/systems</i> API
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystem: function getSystem_Systems(systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (this._systems[systemID]) {
          return this._systems[systemID];
        }
        else {
          //throw new Error("Systems.getSystem: No system found for systemID=" + systemID);
          com.ibm.bpm.social.util.common.Log.log(1, "Systems.getSystem: No system found for systemID=" + systemID);
          return null;
        }
      }
      else {
        // programming error
        throw new Error("Systems.getSystem: A systemID is required for a federated system!");
      }
    }
    else {
      return this._system;
    }
  },

  /**
   * Returns the systems object
   * 
   * @returns {String[]} an array of all systems ids in all domains
   */
  getSystemIDs: function getSystemIDs_Systems(){
    return this._systemIDs;
  },

  /**
   * Returns the system type for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType for the specified system id or the systemType of the single system if no
   *          systemID was specified
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          </ul>
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystemType: function getSystemType_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType;
    }
    return null;
  },

  /**
   * Returns the estimated system type for the specified system id.
   * 
   * @param {String} systemID mandatory, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType derived from the specified system id
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          <li>null</li>
   *          </ul>
   */
  deriveSystemTypeFromSystemId: function deriveSystemTypeFromSystemId_Systems(systemID){
    if (systemID && systemID.length > 0) {
      // WPS systemID looks like this: "_PK:90230132.462033e2.cdc167f6.795e0002"
      if (systemID.indexOf("_PK:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
      }
      // ICM systemID looks like this: "_ICM:fmtc4235:P8ConnPt1:ICMPersonal"
      if (systemID.indexOf("_ICM:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      // WLE systemID looks like this: "3b1ce0ca-f622-44e4-9ec5-5288ca724856"
      // Since only these 3 types exist and 2 of them can be recognized by a prefix substring,
      //  we can conclude the catch-all case as WLE
      return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    // garbage-in - garbage-out case: return null if param 1 was null or the empty string
    return null;
  },

  /**
   * Returns whether the system is a WLE system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wle system
   */
  isSystemWle: function isSystemWle_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    return false;
  },

  /**
   * Returns whether the system is a WPS system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wps system
   */
  isSystemWps: function isSystemWps_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
    }
    return false;
  },

  /**
   * Returns whether the system is an ICM system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if icm system
   */
  isSystemIcm: function isSystemIcm_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
    }
    return false;
  },

  /**
   * Returns the setting for search enabled on the system
   * 
   * @param systemID
   * @returns {boolean} true if search is enabled on system
   */
  isSystemSearchable: function isSystemSearchable_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system) {
      return system.taskSearchEnabled === true;
    }
    return false;

  },


  /**
   * @returns {String} the system id if we have a single system backend
   * @throws Error if the backend is not a single system
   */
  getSystemID: function getSystemID_Systems(){
    if (!this.isApiTypeFederated()) {
      return this._system.systemID;
    }
    else {
      // programming error
      throw new Error("Systems.getSystemID: Method is not supported for Federation!");
    }
  },

  /**
   * Returns the map of all domains for a federated REST service endpoint
   * 
   * @returns {Map} the map of all domains, null if it is not a federated REST service endpoint.
   *          <dl>
   *          <dt>key</dt>
   *          <dd>domainName</dd>
   *          <dt>value</dt>
   *          <dd>Object of domain properties of the domain as returned by the <i>/systems</i> API</dd>
   *          </dl>
   * 
   */
  getDomains: function getDomains_Systems(){
    return this._domains;
  },

  /**
   * Returns a list of all available domain names
   * 
   * @returns {String[]} the list of all domains, null if it is not a federated REST service endpoint.
   */
  getDomainNames: function getDomainNames_Systems(){
    return this._domainNames;
  },


  /**
   * Returns the domain properties of the REST service endpoint for the specified domain name.
   * 
   * @param {String} domainName the name of a backend domain for a federated REST service endpoint
   * @returns {Object} the domain properties of the domain as returned by the <i>/systems</i> API
   * @throws Error if no domain name is specified.
   */
  getDomain: function getDomain_Systems(domainName){
    if (domainName) {
      if (this._domains[domainName]) {
        return this._domains[domainName];
      }
      else {
        // throw new Error("Systems.getDomain: No domain found for domainName=" + domainName);
        com.ibm.bpm.social.util.common.Log.log(1, "Systems.getDomain: No domain found for domainName=" + domainName);
        return null;
      }
    }
    else {
      // programming error
      throw new Error("Systems.getDomain: A domainName is required!");
    }
  },

  /**
   * Returns if the domain contains only WPS systems
   * 
   * @returns {boolean} true if the domain contains only WPS systems
   * @throws Error if no domain name is specified.
   */
  isDomainWpsOnly: function isDomainWpsOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
  },

  /**
   * Returns if the domain contains only WLE systems
   * 
   * @returns {boolean} true if the domain contains only WLE systems
   * @throws Error if no domain name is specified.
   */
  isDomainWleOnly: function isDomainWleOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
  },

  /**
   * Returns if the domain contains only ICM systems
   * 
   * @returns {boolean} true if the domain contains only ICM systems
   * @throws Error if no domain name is specified.
   */
  isDomainIcmOnly: function isDomainIcmOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
  },

  /**
   * Are the systems in the domain wps only, icm only, wle only, or mixed:=more that one (any two or three)
   * 
   * @param domainName
   * @returns {String} the systemType if only one type, or mixed otherwise
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED</li>
   *          </ul>
   * @throws Error if no domain name is specified.
   */
  getDomainType: function getDomainType_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._domainType === undefined) {
      var wps = false;
      var wle = false;
      var icm = false;
      for ( var s = 0; s < domain.systems.length; s++) {
        var systemType = this.getSystemType(domain.systems[s]);
        wps = (wps || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS);
        wle = (wle || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE);
        icm = (icm || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM);
      }
      if (wps && !wle && !icm) {
        // wps only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
      }
      else if (wle && !wps && !icm) {
        // wle only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
      }
      else if (icm && !wps && !wle) {
        // icm only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
      }
      else {
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED;
      }
    }
    return domain._domainType;
  },
  /**
   * Check for the domain, if it has a WPS system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWpsSystem: function getWpsSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WPS system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWleSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WLE system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getIcmSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not an ICM system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Returns systems for domain name
   * 
   * @param domainName
   * @returns {object} systemIds
   * @throws Error if no domain name is specified.
   */
  getSystems: function getSystems_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systems = domain.systems;
    return systems;
  },

  /**
   * Return true if a domain of kind WLE_ONLY or WPS_ONLY or ICM_ONLY has multiple systems in it.
   * 
   * @param {String} domainName
   * 
   * @type {Boolean}
   */
  isMultiSystemDomain: function isMultiSystemDomain_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "domainName");
    var domain = this.getDomain(domainName);
    var domainType = this.getDomainType(domainName);
    var isMulti = true;
    // if we erroneously got a MIXED domain, then is must be a multiSys domain 
    if (domainType != com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED) {
      isMulti = (domain.systems.length > 1);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "isMultiSystemDomain==" + (isMulti ? "true" : "false"));
    return isMulti;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS, federated with WPS et al., or federated WPS only
   */
  isWps: function isWps_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWpsSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE, federated with WLE et al., or federated WLE only
   */
  isWle: function isWle_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWleSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM, federated with ICM et al., or federated ICM only
   */
  isIcm: function isIcm_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getIcmSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS or federated WPS only
   */
  isWpsOnly: function isWpsOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE or federated WLE only
   */
  isWleOnly: function isWleOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM or federated ICM only
   */
  isIcmOnly: function isIcmOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },


  /**
   * Is there at leaset one icm systems in the domain?
   * 
   * @param domainName
   * @returns {boolean} tru if domain includes ICM system, else false
   * @throws Error if no domain name is specified.
   */
  hasIcmSystem: function hasIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._hasIcm === undefined) {
      var icm = false;
      var s = 0;
      while (s < domain.systems.length && !icm) {
        var systemType = this.getSystemType(domain.systems[s]);
        s++;
        icm = systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      domain._hasIcm = icm;
    }
    return domain._hasIcm;
  },

  /**
   * @param domainName optional
   * @return {String} backendType com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM
   */
  _getBackendType: function _getBackendType_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null;
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      backendType = this.getDomainType(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, backendType);
    return backendType;
  }


});

////////////////////////////
// constants for systems
////////////////////////////

/**
 * API type WPS
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * API type WLE
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * API type ICM
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM = "SYSTEM_TYPE_ICM";

/**
 * API type WPS federated
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED = "federated";

/**
 * System type WPS tbd. if we need HTM/BFM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * System type WLE
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * System type ICM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM = "SYSTEM_TYPE_ICM";


/**
 * Domain type for WPS only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS = "WPS";

/**
 * Domain type for WLE only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE = "WLE";

/**
 * Domain type for ICM only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM = "ICM";

/**
 * Domain type for mixed domains (any two of {WLE,WPS,ICM} or all three)
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED = "MIXED";


/**
 * Define for the default domain name
 */
com.ibm.bpm.social.widgets.task.util.Systems.DEFAULT_DOMAIN_NAME = "default";


/**
 * Regular expression for a WLE TASK ID.
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
// integer number
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING = "^([0-9]+)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING);

/**
 * Regular expression for a WLE TEMPLATE ID.
 * 
 * If this changes, then update as well REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// UUID 8-4-4-4-12 (32 hexadecimal digit in 5 groups)
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING =
  "^([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12})$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING);

/**
 * Regular expression for a WPS ID
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below and
 * REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// tkiid or tktid or piid or ptid
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING);

/**
 * Regular expression for an ICM TKIID as received by Hong Dong Zhao (12. July 2011 12:12)
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING =
  "^([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING);


/**
 * Now specify the merged patterns, combined with OR (either pattern will be accepted)
 * 
 * We could have done as well the 3 two-out-of-three combinations, to e.g. reject an ICM id when no ICM is
 * involved. But the newly added ICM pattern is quite restrictive. And this pattern is only used for a first
 * syntactical check - semantical checks will follow anyway. So the damage by letting 3 types pass when only
 * two types of systems are involved is rather small.
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+)|([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING);

com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ErrorHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ErrorHandler"] = true;
// ErrorHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT


// **************************************
//
// Version: 1.34 11/10/27 02:28:39
//
// **************************************
dojo.provide("com.ibm.bpm.social.widgets.task.util.ErrorHandler");








dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.ErrorHandler",
    null,
    {

      /*
       * Error Handling

      For errors recognized during processing of a REST request, an appropriate HTTP status code is returned to the calling client (see the individual operations for details). The following HTTP status codes are returned by IBM Business Process Manager REST methods:

        * Successful completion of the request:
              o "200 OK" - successful completion
              o "201 Created" - successful completion, new resource created
              o "204 No Content" - successful completion, no content available
        * Expected error situations - additional error information is provided depending on the error type:
              o "400 Bad Request" - parameters are not valid or they are missing
              o "401 Unauthorized" - caller is not authorized for this request
              o "403 Forbidden" - caller is not allowed to complete this request
              o "404 Not Found" - resource does not exist
              o "406 Not Acceptable" - unsupported content type or content encoding requested
              o "409 Conflict" - conflict exists with the current state of the resource
              o "415 Unsupported Media Type" - unsupported content type or content encoding of the request
        * Unexpected error:
              o "500 Internal Server Error" - severe problem - programmer's details provided
              o "501 Not Implemented" - used for certain federated requests
              o "503 Service Unavailable" - federated requests could not be delivered to individual federation targets
              o "504 Gateway Timeout" - federated response has partial content due to missing individual responses
       */
      _genericStatusCodeMap: {
        "400": {
          key: "KEY_REST_BAD_REQUEST"
        },
        "401": {
          key: "KEY_REST_UNAUTHORIZED"
        },
        "403": {
          key: "KEY_REST_FORBIDDEN"
        },
        "404": {
          key: "KEY_REST_NOT_FOUND"
        },
        "406": {
          key: "KEY_REST_NOT_ACCEPTABLE"
        },
        "409": {
          key: "KEY_REST_CONFLICT"
        },
        "415": {
          key: "KEY_REST_UNSUPPORTED_MEDIA_TYPE"
        },
        "500": {
          key: "KEY_REST_INTERNAL_SERVER_ERROR"
        },
        "501": {
          key: "KEY_REST_NOT_IMPLEMENTED"
        },
        "503": {
          key: "KEY_REST_SERVICE_UNAVAILABLE"
        },
        "504": {
          key: "KEY_REST_GATEWAY_TIMEOUT"
        }
      },

      _errorCodeMapBC: {
        "CWTKB0001E": {
          key: "KEY_BC_NOT_FOUND"
        },
        "CWTKB0002E": {
          key: "KEY_BC_AUTHORIZATION_READ"
        },
        "CWTKB0003E": {
          key: "KEY_BC_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_BC_PARAMETER_TO_LONG_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKB0005E": {
          key: "KEY_BC_DELETE_WITH_TASKS"
        },
        "CWTKA0013E": {
          key: "KEY_BC_PARAMETER_NOT_SET_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKA0151E": {
          key: "KEY_BC_NOT_ENABLED"
        }
      },

      _errorCodeMapWB: {
        "CWTKW0001E": {
          key: "KEY_WB_NOT_FOUND"
        },
        "CWTKW0002E": {
          key: "KEY_WB_AUTHORIZATION_READ"
        },
        "CWTKW0003E": {
          key: "KEY_WB_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_WB_PARAMETER_TO_LONG_WORKBASKET"
        },
        "CWTKW0006E": {
          key: "KEY_DELETE_WORKBASKET_CONTAINS_TASKS"
        },
        "CWTKW0007E": {
          key: "KEY_DELETE_WORKBASKET_HAS_DISTRIBUTION_SOURCES"
        },
        "CWTKA0013E": {
          key: "KEY_WB_PARAMETER_NOT_SET_WORKBASKET"
        },
        // "CWTKA0149E": {    // error handled in WBInfoWidget
        "CWTKA0152E": {
          key: "KEY_WB_NOT_ENABLED"
        }
      },

      constructor: function constructor_ErrorHandler(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social.widgets.task.util", "ErrorMessage");
      },

      /**
       * Displays a generic error message with details.
       * 
       * @see #_genericStatusCodeMap for handled status codes
       * 
       * @param data
       * @param ioArgs {void} nothing
       * @param {function} callback to connect to close on error dialog
       */
      showGenericError: function showGenericError_ErrorHandler(data, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var clientMessage = this._nlsResources["KEY_InternalError"];
        if (ioArgs && ioArgs.xhr) {
          var statusCode = ioArgs.xhr.status;
          var mapEntry = this._genericStatusCodeMap[statusCode];
          if (mapEntry && mapEntry !== null) {
            var key = mapEntry.key;
            if (key !== null) {
              clientMessage = this._nlsResources[key];
            }
          }
        }
        this._displayError(clientMessage, this._getDetailsMessage(data, ioArgs), null, ioArgs, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message mapped using the errorCodeMap and the statusCodeMap.
       * 
       * The errorCodeMap has precedence over the statusCodeMap. If no parameters are passed with the error
       * code the parameters are taken out of the rest call (errorMessage.errorMessageParameters) and used for
       * the specified message in business space.
       * 
       * Otherwise a generic message will be shown.
       * 
       * @param data
       * @param ioArgs
       * @param statusCodeMap e.g. {404: {key: "KEY_MissingForm", parameters: {1: htmlURL}, hideDetails:
       *        true}}
       * @param errorCodeMap e.g. {CWTKB0001E: {key: "KEY_Task_API_BusinessCategoryDoesNotExist", parameters:
       *        {1: name}, hideDetails: true}}, <br>
       *        in addition to support older code simple mapping like {CWTKB0001E:
       *        "KEY_Task_API_BusinessCategoryDoesNotExist"} is also supported
       * 
       * @return {void} nothing
       * 
       */
      showError: function showError_ErrorHandler(data, ioArgs, statusCodeMap, errorCodeMap, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if ((ioArgs && ioArgs.xhr) || errorCodeMap) {
          var statusCode = ioArgs.xhr.status;
          var mappedMessageProperties =
            this._getMappedMessageProperties(statusCode, statusCodeMap, data.responseText, errorCodeMap);
          this.showErrorMessage(data, ioArgs, mappedMessageProperties.key, mappedMessageProperties.parameters,
            mappedMessageProperties.hideDetails, null, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message with the error details.
       * 
       * @param data
       * @param ioArgs
       * @param key the message key
       * @param parameters parameters for the message, if no parameters specified, the parameters are
       *        retrieved from the original message
       * @param hideDetails don't show details
       * @param message default message if no message is found for key or no details
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showErrorMessage: function showErrorMessage_ErrorHandler(data, ioArgs, key, parameters, hideDetails, message, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (key && key !== null) {
          var mappedMessage = this._getMappedMessage(key, parameters);
          var detailsMessage = this._getDetailsMessage(data, ioArgs, hideDetails);

          if (mappedMessage === null) {
            if (message) {
              mappedMessage = message;
            }
            else {
              // no message at all: fall back to HTTP error message
              if (ioArgs && ioArgs.xhr && ioArgs.xhr.status) {
                var mapEntry = this._genericStatusCodeMap[ioArgs.xhr.status];
                if (mapEntry && mapEntry !== null) {
                  var newKey = mapEntry.key;
                  if (newKey) {
                    mappedMessage = this._getMappedMessage(newKey, null);
                  }
                }
              }
            }
          }
          else if (detailsMessage === null) {
            detailsMessage = message;
          }

          this._displayError(mappedMessage, detailsMessage, null, ioArgs, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * @param statusCode
       * @param statusCodeMap
       * @param response data.responseText or responseObject.failedOperations[i] for Bulk APIs
       * @param errorCodeMap
       * @return {key: "key", parameters: {parameters...}, hideDetails: true|false, fromErrorCode: true|false}
       */
      _getMappedMessageProperties: function _getMappedMessageProperties_ErrorHandler(statusCode, statusCodeMap, response,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var key = null;
        var parameters = null;
        var hideDetails = false;
        var fromErrorCode = false;
        if (errorCodeMap && response && response !== null) {
          try {
            var responseObject = this.getResponseObject(response);
            if (responseObject.errorNumber && typeof responseObject.errorNumber == "string" &&
              errorCodeMap[responseObject.errorNumber] !== null) {
              var mappedEntry = errorCodeMap[responseObject.errorNumber];
              if (mappedEntry) {
                if (typeof mappedEntry == "string") {
                  // simple error code map
                  key = mappedEntry;
                }
                else {
                  key = mappedEntry.key;
                  hideDetails = mappedEntry.hideDetails;
                  parameters = mappedEntry.parameters;
                }
                if (!parameters || parameters === null) {
                  if (responseObject.errorMessageParameters) {
                    parameters = {};
                    for ( var i = 0; i < responseObject.errorMessageParameters.length; i++) {
                      parameters[(i + 1).toString()] = responseObject.errorMessageParameters[i];
                    }
                  }
                }
                fromErrorCode = true;
              }
            }
          }
          catch (err) {
            // continue with the state map
          }
        }
        if (!fromErrorCode && statusCode !== undefined && statusCode !== null) {
          var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
          if (mapEntry === null) {
            mapEntry = this._mapStatusCode(statusCode, this._genericStatusCodeMap);
          }
          if (mapEntry !== null) {
            key = mapEntry.key;
            parameters = mapEntry.parameters;
            if (key) {
              hideDetails = mapEntry.hideDetails;
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return {
          key: key,
          parameters: parameters,
          hideDetails: hideDetails,
          fromErrorCode: fromErrorCode
        };
      },

      _getMessageForStatusCode: function _getMessageForStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
        if (mapEntry && mapEntry !== null) {
          var key = mapEntry.key;
          if (key !== null) {
            result = this._nlsResources[key];
          }
        }
        return result;
      },

      _mapStatusCode: function _mapStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        if (statusCode !== undefined && statusCode !== null) {
          if (statusCodeMap && statusCodeMap !== null && statusCodeMap[statusCode]) {
            result = statusCodeMap[statusCode];
          }
        }
        return result;
      },

      _getMappedMessage: function _getMappedMessage_ErrorHandler(key, parameters){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "key=" + key);
        var mappedMessage = null;

        var i;
        if (key !== null && this._nlsResources[key]) {
          mappedMessage = this._nlsResources[key];
          if (mappedMessage !== null && parameters !== null && parameters !== undefined) {
            // if the parameter is an empty object, the parameters are false
            var parameterOk = false;
            // check if the parameters include valid values
            for (i in parameters) {
              var value = parameters[i];
              if (value === null) {
                parameterOk = false;
                break;
              } else {
                parameterOk = true;
              }
            }
            if (parameterOk) {
              com.ibm.bpm.social.util.common.Log.logEntry(2," parameters=" + parameters);
              mappedMessage = dojo.string.substitute(mappedMessage, parameters);
            }
          }
        }
        // In case parameter where unknown
        if (/UNKOWN/.test(mappedMessage)) {
          mappedMessage = mappedMessage.replace(/'UNKOWN'/g, "");
          mappedMessage = mappedMessage.replace(/UNKOWN/g, "");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2, mappedMessage);
        return mappedMessage;
      },

      _getDetailsMessage: function _getDetailsMessage_ErrorHandler(data, ioArgs, hideDetails){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var detailsMessage = null;
        if (data && !data.responseText && data.message !== undefined && data.name !== undefined) {
          // this is a JavaScriptError instead of a back-end error
          detailsMessage = data.name + ":" + data.message + " (JavaScript Error)<br>" + "at " + data.stack;
        }
        else if (ioArgs && ioArgs.xhr) {
          if (ioArgs && !hideDetails) {
            var details = this.getResponseObject(ioArgs.xhr.responseText);
            if (details) {
              if (details.errorMessage) {
                detailsMessage = details.errorMessage;
              }
              else if (dojo.isString(details)) {
                detailsMessage = details;
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return detailsMessage;
      },

      _displayError: function _displayError_ErrorHandler(message, details, messageDescription, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // add federation result errors
        if (ioArgs && ioArgs.xhr && ioArgs.xhr.responseText) {
          var responseObject = this.getResponseObject(ioArgs.xhr.responseText);
          if (responseObject.federationResult) {
            var errors = this._getFederationResultErrors(responseObject.federationResult);
            if (errors !== null) {
              // TODO SA: should we show them using BspaceGeneralHelper.handleException?
              // TODO SA: add ioArgs.url?
              messageDescription = this._getMessageDetailsList(errors);
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.log(2, "Message: " + message);
        com.ibm.bpm.social.util.common.Log.log(2, "Details: " + details);

        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, callback);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Shows the bulk API failure in an error dialog if only one item has been selected for bulk processing.
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperation the response for the failing operation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Array} federationResults optional, the complete federation result
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIError: function showBulkAPIError_ErrorHandler(failedOperation, statusCodeMap, errorCodeMap,
        federationResults, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var response = null;
        var statusCode = null;
        var statusCodes = statusCodeMap;
        if (failedOperation) {
          response = failedOperation;
          statusCode = failedOperation.derivedHttpStatusCode;
        }
        else if (federationResults & federationResults.length > 0) {
          // in federation the error may be returned as federation result, if the backend has a problem
          response = federationResults[0];
          statusCode = response.statusCode;
          // use generic mapping for status codes because federation result is related to errors with backend systems
          statusCodes = this._genericStatusCodeMap;
        }
        var mappedMessageProperties = this._getMappedMessageProperties(statusCode, statusCodes, response, errorCodeMap);
        this.showErrorMessage(null, null, mappedMessageProperties.key, mappedMessageProperties.parameters,
          mappedMessageProperties.hideDetails, response.errorMessage, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the bulk API failures in an error dialog for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperations the failedOperations as returned in the responseObject of the bulk
       *        API
       * @param {String} messageKey the general message key for the error
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Object} idToNameMap maps ids to names for replacement of ids in error messages
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {String} nameKey the message key for the entity name
       * @param {Array} federationResults optional, the complete federation result
       * @param {Object} failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIErrors: function showBulkAPIErrors_ErrorHandler(failedOperations, messageKey, statusCodeMap, errorCodeMap,
        idToNameMap, idKey, nameKey, federationResults, failedNamesForSystemIDs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        var errors = new Array();

        // get federation errors
        if (failedNamesForSystemIDs && federationResults && federationResults.length > 0) {
          errors = errors.concat(this._getFederationResultErrors(federationResults, failedNamesForSystemIDs));
        }

        // get errors for failed operations
        errors = errors.concat(this._getBulkAPIErrors(failedOperations, idToNameMap, idKey, statusCodeMap, errorCodeMap));

        if (errors !== null && errors.length > 0) {
          // show list of errors
          this._displayError(this._nlsResources[messageKey], null, null, this._getMessageDetailsList(errors, nameKey,
            failedNamesForSystemIDs), null, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the errors in the federation result as a popup
       * 
       * @param {Array} federationResults the complete federation result
       * @param {String} messageKey the message key for the error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @param {function} callback to connect to close on error dialog
       * @returns {boolean} whether there were errors shown
       */
      showFederationResultErrors: function showFederationResultErrors_ErrorHandler(federationResults, messageKey, ignore401,
        callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1, "messageKey=" + messageKey);
        var errorsShown = false;
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults, null, ignore401);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            var message = this._nlsResources[messageKey];
            var messageDescription = this._getMessageDetailsList(errors);
            this._displayError(message, null, messageDescription, null, callback);
            errorsShown = true;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
        return errorsShown;
      },

      /**
       * Shows the errors in the federation result as global exceptions in BSpace
       * 
       * uses BSpace exception handler to show error on top of space
       * 
       * @param federationResults the complete federation result
       * @param iWidget
       * @param method
       * @returns {void} nothing
       */
      handleFederationResultErrors: function handleFederationResultErrors_ErrorHandler(federationResults, iWidget, method,
        systemsSending401Error){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            for ( var i = 0; i < errors.length; i++) {
              var error = errors[i];
              var message = "";

              // message for status code
              if (error.statusCode !== undefined) {
                var statusMessage = this._getMessageForStatusCode(error.statusCode, this._genericStatusCodeMap);
                if (statusMessage) {
                  message += statusMessage;
                }
                else {
                  message += dojo.string.substitute(this._nlsResources.Key_HTTPStatusCode, [error.statusCode]);
                }
              }

              // system ID
              if (error.systemID) {
                message += " " + dojo.string.substitute(this._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
              }

              // error number, and not part of error message
              if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
                // WPS error message contains error number, WLE message not
                message += " " + error.number + ":";
              }

              // error message
              if (error.message) {
                message += " " + error.message;
              }

              com.ibm.bpm.social.util.common.Log.logError(1, "com.ibm.bpm.social.widgets.task.util.ErrorHandler" + "." +
                method, new Error(message));

              // Do not show "CWTBF0005E: The 'queryFilter' parameter is invalid.",
              // issued if a filter attribute is not supported by one of the backends

              // Do not show "CWTBF0020E: Request not propagated by the federation layer because a previous systems metadata request indicated that the system does not support '...' resources.
              // issued if a target system does not support that REST resource (e. g. WLE does not support escalations)

              // Do not show "CWTBF0021E: Request not propagated by the federation layer because a previous query metadata request indicated that the system does not support this resource.
              // issued if a target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0002E: The following query name is unknown: '...'.
              // issued if a WLE target system does not know a query, can happen because we use all=true

              // Do not show "CWWBQ0001E: The following query table name is unknown: '...'.
              // issued if a WPS target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0530E": The following process application name is unknown: '...'
              // issued if a WLE target system does not know a process application, can happen because we use all=true

              // Do not show "CWTKA0162E": The following process application name is unknown: '...'
              // issued if a WPS target system does not know a process application, can happen because we use all=true

              var handleException =
                !error.number ||
                  (error.number != "CWTBF0005E" && error.number != "CWTBF0020E" && error.number != "CWTBF0021E" &&
                    error.number != "CWTBG0002E" && error.number != "CWTBG0530E" && error.number != "CWWBQ0001E" && error.number != "CWTKA0162E");

              // for the systems that had returned a 401 in the initial systems call, ignore 401 errors now because that was already covered
              // But do not generally exclude all 401 errors - could be a different reason for those systems that had not returned a 401 ealier
              if (handleException && systemsSending401Error && error.statusCode == 401 &&
                (dojo.indexOf(systemsSending401Error, error.systemID) != -1)) {
                // overwrite the handleException flag
                handleException = false;
              }

              // Special handling for ICM 404 errors:
              // For version 7.5.1.0, the ICM rest api could not provide details for the 404 error cases.
              // However, the only relevant case that produce 404 errors from ICM systems in federated mode is
              // that a processAppName was specified that does not exist on the ICM system.
              // We must gracefully handle that case and ignore those errors, botp 719802
              // Currently, it is ok to ignore ALL 404 errors from ICM that enter this method
              // (only entered in federation mode); 404 in the /systems call are handles separately.  
              // NEEDSWORK: As soon as the ICM rest api provides details, then rather do a positive list
              // of error ICM details that should be ignored. (Not NLS specific, please).
              if (handleException && error.statusCode == 404 && error.systemID) {
                if (iWidget && iWidget._systems && iWidget._systems.getSystemType) {
                  if (iWidget._systems.getSystemType(error.systemID) === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
                    // overwrite the handleException flag
                    handleException = false;
                  }
                }
              }

              if (handleException && iWidget) {
                iWidget.handleException("com.ibm.bpm.social.widgets.task.util.ErrorHandler", method, message, null);
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * 
       * @param errors [{name: "name or array of names", systemID: "systemID", statusCode: "statusCode",
       *        number: "Number", message: "Message", details: "Details"}]
       * @param nameKey optional, only for bulk API errors, NLS key of text for error.name, e.g. "Task"
       * @returns {Object} a div containing the message details list
       */
      _getMessageDetailsList: function _getMessageDetailsList_ErrorHandler(errors, nameKey){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var messageDetails = null;

        if (errors !== null) {

          //***********************************
          // message details list
          // systemID, name and reason/details for each failing entry
          // TODO add hostName for system???

          var messageDetailsListDiv = document.createElement("div");
          var detailsDiv = document.createElement("div");
          dojo.addClass(detailsDiv, "errorList");
          messageDetailsListDiv.appendChild(detailsDiv);

          var myself = this;
          dojo.forEach(errors, function(error){

            var ulItem = document.createElement("ul");
            detailsDiv.appendChild(ulItem);

            //
            // item name, may constist of one or more names and optional a systemID
            //
            var itemName = "";
            // name
            if (error.name) {
              if (!dojo.isArray(error.name)) {
                itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name]);
              }
              else {
                for ( var i = 0; i < error.name.length; i++) {
                  if (itemName.length > 0) {
                    itemName += "<br/>";
                  }
                  itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name[i]]);
                }
              }
            }

            // system ID
            if (error.systemID) {
              if (itemName.length > 0) {
                itemName += "<br/>";
              }
              itemName += dojo.string.substitute(myself._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
            }

            if (itemName.length > 0) {
              var liItem = document.createElement("li");
              dojo.addClass(liItem, "errorListItem");
              liItem.innerHTML = itemName;
              ulItem.appendChild(liItem);
            }

            //
            // message, may consist of several parts
            //
            var ulMessage = document.createElement("ul");
            dojo.addClass(ulMessage, "errorListMessage");
            ulItem.appendChild(ulMessage);

            // message for status code
            if (error.statusCode !== undefined) {
              var statusMessage = myself._getMessageForStatusCode(error.statusCode, myself._genericStatusCodeMap);
              if (statusMessage && statusMessage !== null) {
                var liStatus = document.createElement("li");
                liStatus.innerHTML = statusMessage;
                ulMessage.appendChild(liStatus);
              }
            }

            // error number, and not part of error message
            if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
              var liNumber = document.createElement("li");
              liNumber.innerHTML = error.number;
              ulMessage.appendChild(liNumber);
            }

            // error message
            if (error.message) {
              var liMessage = document.createElement("li");
              liMessage.innerHTML = error.message;
              ulMessage.appendChild(liMessage);
            }

            //
            // message details
            //
            if (error.details) {
              var ulMessageDetails = document.createElement("ul");
              dojo.addClass(ulMessageDetails, "errorListMessageDetails");
              ulMessage.appendChild(ulMessageDetails);

              var liDetails = document.createElement("li");
              liDetails.innerHTML = error.details;
              ulMessageDetails.appendChild(liDetails);
            }
          });

          messageDetails = messageDetailsListDiv.innerHTML;

        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return messageDetails;
      },

      /**
       * Retrieves the bulk API failures for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param failedOperations the failedOperations as returned in the responseObject of the bulk API
       * @param idToNameMap maps ids to (display) names
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @returns {Array} an array of objects containing error information <br>
       *          [{name: "name", systemID: "systemID", message: "Message", details: "Details"}]
       */
      _getBulkAPIErrors: function _getBulkAPIErrors_ErrorHandler(failedOperations, idToNameMap, idKey, statusCodeMap,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var errors = new Array();

        if (failedOperations && failedOperations.length > 0) {
          var self = this;

          dojo.forEach(failedOperations, function(failedOperation){
            var error = {};

            if (failedOperation.systemID) {
              error.systemID = failedOperation.systemID;
            }
            var name = idToNameMap[failedOperation[idKey]];
            if (name !== null) {
              error.name = name;
            }

            var mappedMessageProperties =
              self._getMappedMessageProperties(failedOperation.derivedHttpStatusCode, statusCodeMap, failedOperation,
                errorCodeMap);
            var mappedMessage = self._getMappedMessage(mappedMessageProperties.key, mappedMessageProperties.parameters);
            var errorMessage = null;
            if (mappedMessage === null) {
              mappedMessage = failedOperation.errorMessage;
            }
            else if (!mappedMessageProperties.fromErrorCode && !mappedMessageProperties.hideDetails) {
              // if not mapped from error code add detailed error message
              errorMessage = failedOperation.errorMessage;
            }
            error.message = mappedMessage;

            if (errorMessage && errorMessage !== null) {
              error.details = errorMessage;
            }
            errors.push(error);
          });

        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      _getFederationResultErrors: function _getFederationResultErrors_ErrorHandler(federationResults,
        failedNamesForSystemIDs, ignore401){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errors = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errors = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (ignore401 && ignore401 === true && federationResult.statusCode === "401") {
                com.ibm.bpm.social.util.common.Log.log(2,
                  "HTTP error 401 skipped, will not be displayed as an error message");
              }
              else {
                // error returned from a backend system
                var error = {};
                error.statusCode = federationResult.statusCode;
                error.systemID = federationResult.systemID;
                error.number = federationResult.errorNumber;
                error.message = federationResult.errorMessage;
                if (!error.message) {
                  // TODO WLE: for 404 we get only statusCode and body - is this a temporary bug?
                  if (federationResult.body) {
                    error.message = federationResult.body;
                  }
                }
                if (failedNamesForSystemIDs && failedNamesForSystemIDs[error.systemID]) {
                  error.name = failedNamesForSystemIDs[error.systemID];
                }
                errors.push(error);
              }
            }
          });
          if (errors.length === 0) {
            // all responses have been ok
            errors = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      getSystemsSending401Error: function getSystemsSending401Error_ErrorHandler(federationResults){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errorSystems = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errorSystems = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (federationResult.statusCode === "401") {
                errorSystems.push(federationResult.systemID);
              }
            }
          });
          if (errorSystems.length === 0) {
            // all responses have been ok or other than 401
            errorSystems = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errorSystems, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errorSystems;
      },

      getResponseObject: function getResponseObject(response){
        var responseObject = response;
        if (dojo.isString(response)) {
          try {
            responseObject = dojo.fromJson(response);
            // There may be an extra layer ".Data"; if this does exist, use it
            // but do it defensive after some extra checks
            if (responseObject.Data && responseObject.Data.errorNumber && !responseObject.errorNumber) {
              // all checks passed, skip the extra layer
              responseObject = responseObject.Data;
            }
          }
          catch (err) {
            // keep it just as string
          }
        }
        return responseObject;
      },

      getErrorCodeMapBC: function getErrorCodeMapBC_ErrorHandler(){
        return this._errorCodeMapBC;
      },

      getErrorCodeMapWB: function getErrorCodeMapWB_ErrorHandler(){
        return this._errorCodeMapWB;
      },

      showErrorDialog: function showErrorDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(mappedMessage, null /* details */, callback);
      },

      showWarningDialog: function showWarningDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(mappedMessage, null /* details */, callback);
      },

      destroy: function(){
        // TODO temp error dialog
        if (this.popup && this.popup.destroy) {
          this.popup.destroy();
          this.popup = null;
        }
        this.inherited(arguments);
      },

      statics: {
        _instance: null,
        _getInstance: function(){
          if (this._instance === null) {
            this._instance = new com.ibm.bpm.social.widgets.task.util.ErrorHandler();
          }
          return this._instance;
        }
      }
    });

com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.util.ErrorHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.RestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.RestHandler"] = true;
// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.RestHandler");










dojo.declare("com.ibm.bpm.social.util.common.RestHandler", null, {
  prefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the bfm calls
  bfmPrefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the vmm calls
  vmmPrefix: "",

  // root for form generator service
  generatorServicePrefix: "",

  _errorHandler: null,

  error: null,

  version: "v1",

  threshold: 100,

  // 30 seconds timeout to account for some slowness and also to be consistent with Business Space/Mashups settings
  defaultTimeout: 30000,

  _apiType: null,

  _iWidget: null,

  FEDERATION_MODE: "federationMode=true",

  _systems_refresh_required: false,

  constructor: function constructor_RestHandler(params){
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
    // IV09357:
    // Check whether a timoeout has been set using the mashup maker configuration:
    //
    // In order to overwrite the timeout for the Human Workflow Widget, open the Administration Console.
    // Select "Mashups_ConfigService" under Resources -> Resource Environment -> Resource Environment Providers.
    // Then select "Custom properties". Create a new property "com.ibm.bpm.social.widgets.task.requestTimeout" and set its value to the wanted timeout in milliseconds. 
    // In addition to that select the com.ibm.mashups.properties.client and add ",com.ibm.bpm.social.widgets.task.requestTimeout" to 
    // the value of this property (It is a comma separated list of values therefore do not forget the comma).
    //
// TODO disabled for now
//    if (typeof ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"] !== "undefined") {
//      var timeout = ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"];
//      if (typeof timeout === "string") {
//        timeout = parseInt(timeout);
//      }
//      com.ibm.bpm.social.util.common.Log.log(2, "The default timeout for the Human Workflow Widgets is set to " + timeout +
//        " ms");
//      this.defaultTimeout = timeout;
//    }
  },

  getTimeout: function(){
    return this.defaultTimeout;
  },

  /**
   * required for handling of federation results
   */
  setIWidget: function setIWidget_RestHandler(iWidget){
    this._iWidget = iWidget;
  },

  getVmmPrefix: function getVmmPrefix_RestHandler(){
    return this.vmmPrefix;
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_RestHandler(){
    if (this._apiType === null) {
      var systems = this.getSystems();
      if (systems) {
        this._apiType = systems.getApiType();
      }
    }
    return this._apiType;
  },

  /**
   * Returns if the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns if the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns if the REST service endpoint is a ICM system
   * 
   * @returns {boolean} true if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns if the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  performRequest: function performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var deferred = null;

    if (request) {

      if (request.load && this.isApiTypeFederated() && !isBulk) {
        // show federation result errors as exception in BSpace message area
        // for bulk request the error is shown in the error popup
        var load = request.load;
        this._apiCaller = null;
        if (arguments && arguments.callee && arguments.callee.caller && arguments.callee.caller.name) {
          this._apiCaller = arguments.callee.caller.name;
        }
        var systems = this.getSystems();
        var _systemsSending401Error = null;
        if (systems && systems._systemsSending401Error) {
          _systemsSending401Error = systems._systemsSending401Error;
        }
        var myself = this;
        request.load =
          function(data, ioArgs){
            if (data && data.federationResult && data.federationResult.length > 0) {
              if (!myself._iWidget) {
                // federation result cannot be handled with iWidget.handleException!
                com.ibm.bpm.social.util.common.Log.logError(1,
                  "com.ibm.bpm.social.util.common.RestHandler.performRequest",
                  new Error("Property '_iWidget' is required to handle federation results!!!"));
              }
              myself._errorHandler.handleFederationResultErrors(data.federationResult, myself._iWidget, myself._apiCaller,
                _systemsSending401Error);
            }
            // continue with result processing
            load(data, ioArgs);
          };
      }
    }
    deferred = this._performRequest(kind, request, isURLComplete, isBulk);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return deferred;
  },

  /**
   * Note: This function should not call any function which requires system information, because it is also
   * used by the getSystems call. Put code requiring system information in performRequest above.
   * 
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  _performRequest: function _performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var deferred = null;

    request.headers = request.headers || {};

    if (!isURLComplete) {
      request.url = this.getRequestUrl(request.url);
    }
    // make sure the handleAs parameter is set. 
    // Also it is important to set the right request header for json, otherwise 
    // JAXRS will set the header to XML which is not correct for json.
    if (!request.handleAs) {
      request.handleAs = "json";
      request.headers.Accept = "application/json";
    }
    else if (request.handleAs == "json") {
      request.headers.Accept = "application/json";
    }

    if (!request.timeout) {
      request.timeout = this.defaultTimeout;
    }
    if (kind == "get" && (request.preventCache === undefined || request.preventCache === null)) {
      request.preventCache = true;
    }

    if (this.FEDERATION_MODE) {
    if (request.url.indexOf("?") > -1) {
      request.url += "&" + this.FEDERATION_MODE;
    }
    else {
      request.url += "?" + this.FEDERATION_MODE;
      }
    }

	//Set up a listener for 401 responses -- set default action to redirect to login page
	//If an error for some other reason, allow specified handler to take over
	var _error = request.error;
	var myself = this;
	request.error = function(data, ioArgs) {
		var req = ioArgs.xhr;
		if(typeof req != "undefined") {
			var reqStatus = req.status;
			if(typeof reqStatus != undefined) {
				if(reqStatus == 401) {
					//double check to make sure its an actual authentication timeout-
					//check for the authenticate header
					var respHeaders = req.getAllResponseHeaders();
					if(respHeaders.indexOf("WWW-Authenticate") >= 0) {
						//Perform REDIRECT
						com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
					}
				}
			}
		}
		if(_error) {
			_error(data, ioArgs);
		} else {
			//Generic error handler if none is specified in original request
			myself._errorHandler.showGenericError(data, ioArgs);
		}
	};
	
    // if (!request.error) {
      // // default error handler
      // var myself = this;
      // request.error = function(data, ioArgs){
        // myself._errorHandler.showGenericError(data, ioArgs);
      // };
    // }

    if (kind == "get" && request.url.length > com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH) {
      // do a POST request instead of a GET
      var iParams = request.url.indexOf("?") + 1;
      if (iParams > 0) {
        com.ibm.bpm.social.util.common.Log.log(3, "Do a POST instead of a get because url is > " +
          com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH);
        com.ibm.bpm.social.util.common.Log.log(3, "url: " + request.url);
        kind = "post";
        request.contentType = "application/x-www-form-urlencoded; charset=UTF-8";
        request.headers["X-Method-Override"] = "GET";
        request.postData = request.url.substring(iParams);
        request.url = request.url.substring(0, iParams - 1);
      }
    }

    // overwrite the browser locale with the users profile language
    var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var locale = (userProfile && userProfile.userPreferences) ? userProfile.userPreferences['Locale'] : null;

    if (locale) {
      request.headers["Accept-Language"] = locale;
    }

    // Defect 116603 - SiteMinder may redirect the request to an HTML
    // login form when the current user authentication times out.  We
    // detect this case by checking for exceptions during json parsing.
    // If the response cannot be parsed as json we check the response
    // content type to see if "text/html" content was returned.  If so,
    // we assume it is a login form and redirect to our normal logout
    // processing.
    if (!dojo.contentHandlers['jsonOriginal']) {
		dojo.contentHandlers['jsonOriginal'] = dojo.contentHandlers['json'];
		dojo.contentHandlers['json'] = function(xhr) {
			var ret = null;
			try {
				ret = dojo.contentHandlers['jsonOriginal'](xhr);
			} catch (e) {
				contentType = xhr.getResponseHeader("Content-Type");
				if  (contentType && (contentType.indexOf('text/html') != -1)) {
					com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
				} else {
					throw e;
				}
			}
			return ret;
		}
	}
	
    if (kind == "get") {
      // GET request do not need a contentType because they do not have an entity body.
      deferred = dojo.xhrGet(request);
    }
    else if (kind == "rawPost") {
      deferred = dojo.rawXhrPost(request);
    }
    else if (kind == "post") {
      deferred = dojo.xhrPost(request);
    }
    else if (kind == "rawPut") {
      ///////////////
      deferred = dojo.rawXhrPut(request);
    }
    else if (kind == "put") {
      ///////////////
      deferred = dojo.xhrPut(request);
    }
    else if (kind == "delete") {
      deferred = dojo.xhrDelete(request);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);

    return deferred;
  },

  /**
   * @param {String} url the url without prefix
   * @return {String} the complete url for the request
   */
  getRequestUrl: function getRequestUrl_RestHandler(url){
    return this.prefix + this.version + url;
  },

  setUrl: function setUrl_RestHandler(/*String*/urlName, /*String*/url){
    if (urlName) {
      this[urlName] = url;
      if ((url !== "") && url.slice(-1) != "/") {
        this[urlName] += "/";
      }
    }
    com.ibm.bpm.social.util.common.Log.log(3, "urlName=" + this[urlName]);
  },

  /**
   * Prefixes the ID with the systemID, if a domain and a systemID is specified.
   * 
   * @param {String} id
   * @param {String} domain optional, needed for a federated system
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the id, prefixed with the system id, if domain and a systemID are specified
   * @throws error if domain, but no system ID is specified for a federated backend
   */
  prefixID: function prefixID_RestHandler(id, domain, systemID){
    var result = id;
    if (domain) {
      if (systemID) {
        result = systemID + '$' + id;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return result;
  },


  /**
   * Federation only - Adds the systemID to the url
   * 
   * @param {String} url
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the url including the specified systemID
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemID: function addSystemID_RestHandler(url, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (url.indexOf("?") > 0) {
          url += "&";
        }
        else {
          url += "?";
        }
        url += "systemID=" + systemID;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return url;
  },

  /**
   * Adds a processAppName to the url (future use - not used yet)
   * 
   * @param {String} url
   * @param {String} processAppName optional, needed for an ICM system
   * @returns {String} the url including the specified processAppName
   * @returns the identical url if processAppName is null or undefined
   */
  _addProcessAppName: function _addProcessAppName_RestHandler(url, processAppName){
    if (processAppName) {
      if (url.indexOf("?") > 0) {
        url += "&";
      }
      else {
        url += "?";
      }
      url += "processAppName=" + processAppName;
    }
    return url;
  },

  /**
   * Federation only - Adds the properties systemID and systemType to the retrieved item
   * 
   * @param {Object} item a task, task template, process, process template, activity object
   * @param {String} systemID
   * @returns {Object} the updated item
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemProperties: function addSystemProperties_RestHandler(item, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        item.systemID = systemID;
        item.systemType = this.getSystems().getSystemType(item.systemID);
      }
      else {
        this.errorNoSystemID();
      }
    }
    return item;
  },

  /**
   * @returns {Object} a {@link com.ibm.bpm.social.widgets.task.util.Systems} object
   * @throws error if no configuration information retrieved
   */
  getSystems: function getSystems_RestHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var url = this.getRequestUrl("/systems");

    var systems = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
    if (systems === null) {
      var myself = this;
      var error = function(data, ioArgs){
        if (ioArgs.xhr.status == 404) {
          //myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorEndpointConfiguration");
          com.ibm.bpm.social.util.common.Util.restConfigurationException(data, ioArgs);
        }
        else {
          myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorSystemsMetadata");
        }
      };
      var errors = false;
      var i;
      var load = function(data, ioArgs){
        com.ibm.bpm.social.util.common.Log.logObject(3, data, "data");
        if (data && data.federationResult && data.federationResult.length > 0) {
          // graceful handling of 401 in the systems call: ignore but memorize
          var _systemsSending401Error = myself._errorHandler.getSystemsSending401Error(data.federationResult);
          // ... and then make sure the 401 errors do not raise an error message:
          errors = myself._errorHandler.showFederationResultErrors(data.federationResult, "KEY_ErrorSystemsMetadata", true);
        }

        systems = new com.ibm.bpm.social.widgets.task.util.Systems({
          systems: data,
          systemsSending401Error: _systemsSending401Error
        });
      };

      this._performRequest("get", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json",
        // allow browser cache to prevent double calls during initialization
        preventCache: this._systems_refresh_required
      }, true);

      if (systems !== null) {
        com.ibm.bpm.social.util.common.Log.log(1, "systems=" + systems);
        // cache the data
        var cache = 3600000; // 1 hour
        if (errors) {
          // we had errors so cache only for 5 minutes
          cache = 300000;
          this._systems_refresh_required = false;
        }
        this._systems_refresh_required = true;
        com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, systems, cache);
      }
      else {
        var exc = new Error("RestHandler.getSystems: No configuration information retrieved!");
        com.ibm.bpm.social.util.common.Log.consoleError(exc.message);
        throw exc;
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systems;
  },

  errorNoDomain: function errorNoDomain_RestHandler(){
    var error = new Error("No domain name specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoSystemID: function errorNoSystemID_RestHandler(){
    var error = new Error("No system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoDomainOrSystemID: function errorNoDomainOrSystemID_RestHandler(){
    var error = new Error("No domain name or system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  _errorNotSupportedForFederation: function _errorNotSupportedForFederation_RestHandler(url){
    var error = new Error("This call is not supported for federation: " + url);
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    //throw error;
  }
});


/**
 * Maximum length for a GET request, if exceeded POST will be used
 * <p>
 * some browser have a length limit for the url, e.g. for IE this is 2083
 */
com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH = 2000;

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SystemRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SystemRestHandler"] = true;
// SystemRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.SystemRestHandler");






dojo.declare("com.ibm.bpm.social.util.common.SystemRestHandler", com.ibm.bpm.social.util.common.RestHandler, {

  constructor: function constructor_OtherRestHandler(params){},
  
  /**
   *
   * @param load
   * @param error
   * @param async
   * @param systemID
   * @returns {void} nothing
   */
  getExposedItems: function getExposedItems_OtherRestHandler(load, error, async, systemID) {
	var url = "/exposed";
	
	url = this.addSystemID(url, systemID);
	
	this.performRequest("get", {
		url: url,
		sync: !async,
		load: load,
		error: error
	});
  },
  
  
  statics: {
    _instance: null,

    _createInstance: function _createInstance_OtherRestHandler(params){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.util.common.SystemRestHandler();
        com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
        // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
      }
      else {
        // programming error
        var error = new Error("RestHandler instance has already been initialized!");
        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
        throw error;
      }
      return this._instance;
    },

    _getInstance: function _getInstance_SystemRestHandler(){
      if (this._instance === null) {
        // TODO for now return default handler
        this._instance = new com.ibm.bpm.social.util.common.SystemRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.util.common.SystemRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.SystemRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.SystemRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.SystemRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.common._RestWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.common._RestWidget"] = true;
dojo.provide("com.ibm.bpm.social.widgets.common._RestWidget");



dojo.declare("com.ibm.bpm.social.widgets.common._RestWidget", dijit._Widget, {

	proxyUrl: "",
	
	getProxyUrl: function(){
		var url = this.proxyUrl;
		if(url!=""){
			if(url.lastIndexOf("/")==(url.length-1)){
				url = url.substring(0, url.length-1);
			}
		}
		//TODO Should this be guaranteed to return something in the format '/prefix',
		//or is eliminating a trailing slash good enough?
		return url;
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.TaskRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.TaskRestHandler"] = true;
// TaskRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.TaskRestHandler");







dojo.declare("com.ibm.bpm.social.util.common.TaskRestHandler", com.ibm.bpm.social.util.common.RestHandler,
  {

    constructor: function constructor_TaskRestHandler(params){},

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getTask: function getTask_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/task";
      url += "/" + id;
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        // load: load,
        load: function(task, ioArgs){
          task = myself.addSystemProperties(task, systemID);
          load(task, ioArgs);
        },
        error: error
      });
    },

    /**
     * @param {String[]} idList
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * 
     */
    getTasksInBulkMode: function getTasksInBulkMode_TaskRestHandler(idList, load, error, async, parts){
      // TODO: re-enable federated mode once the REST Api supports a federated version of this

      // URL may look like this: /task?action=getdetails&taskIDs=2,3,5,7,11
      //           or like this: /task?action=getdetails&taskIDs=2,3,5,7,11&parts=data

      if (!idList || !idList.length || idList.length === 0) {
        // defensive - do nothing
        return;
      }

      var url = "/task";
      url += "?action=getdetails&taskIDs=";
      // we have at least one id
      url += idList[0];
      for ( var k = 1; k < idList.length; k++) {
        url += "," + idList[k];
      }

      if (parts && parts != "all") {
        url += "&parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },

    /**
     * @param taskID
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param systemID
     * @param domain
     * @returns {void} nothing
     */
    getActionsForTask: function getActionsForTask_TaskRestHandler(taskID, actionsFilter, load, error, async, systemID,
      domain){
      return this.getActions([{
        tkiid: taskID,
        systemID: systemID
      }], "tkiid", actionsFilter, load, error, async, domain);
    },

    /**
     * @param {Array} items
     * @param {String} idKey e.g TKIID or tkiid or id
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param {String} domain, optional for federated backend and more than one item
     * @returns {dojo.Deferred}
     */
    getActions: function getActions_TaskRestHandler(items, idKey, actionsFilter, load, error, async, domain){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (actionsFilter != null) {
        com.ibm.bpm.social.util.common.Log.logObject(3, actionsFilter, "actionsFilter");
      }

      var deferred = null;

      var self = this;
      var idString = "";
      dojo.forEach(items, function(item){
        if (idString !== "") {
          idString += ",";
        }
        var id = item[idKey];
        if (id !== null) {
          idString += self.prefixID(item[idKey], domain, item.systemID);
        }
      });

      var url = "/task/actions?";
      var newLoad = load;
      if (this.isApiTypeFederated()) {
        if (domain) {
          url += "domain=" + domain + "&";
        }
        else if (items.length === 1 && items[0].systemID) {
          url += "systemID=" + items[0].systemID + "&";
          // insert the system id, in federated case the system id is not returned if no domain is specified
          newLoad = function(data, ioArgs){
            if (data && data.items) {
              for ( var i = 0; i < data.items.length; i++) {
                data.items[i].systemID = items[0].systemID;
              }
            }
            load(data, ioArgs);
          };

        }
        else {
          this.errorNoDomainOrSystemID();
        }
      }
      url += "taskIDs=" + encodeURIComponent(idString);

      if (actionsFilter) {
        var actionsFilterString = "";
        for ( var i in actionsFilter) {
          if (actionsFilterString != "") {
            actionsFilterString += ",";
          }
          actionsFilterString += actionsFilter[i];
        }
        url += "&actions=" + actionsFilterString;
      }

      deferred = this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });

      com.ibm.bpm.social.util.common.Log.logExit(2);
      return deferred;
    },

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getProcessActions: function getProcessActions_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/process";
      url += "/" + id + "/actions";
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },


    /**
     * @param id
     * @param load
     * @param error
     * @param priority e.g. Highest/10, High/20, Normal/30, Low/40 or Lowest/50
     * @param dueDate in ISO-8601 format (e.g. 2011-01-05T03:15:30.000-00:00)
     * @param systemID
     * @returns {void} nothing
     */
    updateTaskProperties: function updateTaskProperties_RestHandler(id, load, error, priority, dueDate, systemID){
      var url = "/task";
      url += "/" + id + "?action=update";

      if (priority) {
        url += "&priority=" + priority;
      }

      if (dueDate) {
        url += "&dueDate=" + dueDate;
      }

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Updates all tasks
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param {Object} content <br>
//       *        WPS API {"updatedTaskIDs": [..., ...], "updatedPropertyValues": {...}} <br>
//       *        Federated API {"tasks": [{taskID: ..., systemID: ...}, ...], "updatedPropertyValues": {...}}
//       * @param {String} domain federation only
//       * @returns {void} nothing
//       */
//      updateTasksData: function updateTasksData_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=update";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    claimTask: function claimTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=claim";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json"
      });
    },

    /**
     * Completes a simple inline task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param property The property for the json result.
     * @param result The value of the result property in the json result.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleInlineTask: function approveSimpleApprovalTask_TaskRestHandler(taskId, claimTask, property, result,
      comment, loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call            
      var data = {};
      data[property] = result;
      if (comment != "") {
        data["comment"] = comment;
      }
      var url = "/task/" + taskId + "?action=complete&params=" + window.encodeURIComponent(dojo.toJson(data));
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    /**
     * Completes a simple completion task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleCompletionTask: function completeSimpleCompletionTask_TaskRestHandler(taskId, claimTask, comment,
      loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call                   
      var url = "/task/" + taskId + "?action=complete";
      if (comment != "") {
        var data = {
          "comment": comment
        };
        url = url + "&params=" + window.encodeURIComponent(dojo.toJson(data));
      }
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    publishOnInstanceStream: function _publishOnInstanceStream_TaskRow(piid, comment, load, error, systemID){
      if (piid != null) {
        var url = "/process/" + piid + "?action=comment&comment=";
        url = url + window.encodeURIComponent(comment);
        url = this.addSystemID(url, systemID);
        this.performRequest("post", {
          url: url,
          sync: true,
          load: load,
          error: error,
          handleAs: "text"
        });
      }
    },

    // TODO remove or enable
//      /**
//       * Claims all tasks in content.
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param content {"taskIDs": [...]}
//       * @param {String} domain optional, needed for bulk calls to a federated system
//       * @returns {void} nothing
//       */
//      claimTasks: function claimTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=claim";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    cancelTask: function cancelTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=cancel";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Cancels all tasks in content = {"taskIDs": [...]}.
//       */
//      cancelTasks: function cancelTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=cancel";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    /**
     * 
     * @param tktid
     * @param tkiid
     * @param type
     * @param load
     * @param error
     * @param async
     * @param systemID
     * @returns {void} nothing
     */
    getClientSetting: function getClientSetting_TaskRestHandler(tktid, tkiid, type, load, error, async, systemID){
      //
      // Determine the system of the task
      //
      var wps = false;
      if (this.getSystems().isApiTypeFederated() && this.getSystems().isSystemWps(systemID)) {
        wps = true;
      }
      else if (this.getSystems().isApiTypeWps()) {
        wps = true;
      }
      //
      // Calculate the key for the cache.
      //
      var key = null;
      if (wps && tktid) {
        // WPS has client setting which are based on task templates
        // Therefore use template id if available (Take care of Ad hoc task which do not have a template id)      
        key = "/taskTemplate/" + tktid + "/clientSettings/" + type;
        key = this.getRequestUrl(key);
      }
      //
      // Set url for the call.
      //
      var url = null;
      if (tkiid) {
        // Call as specific as possible due to authorization reasons.
        // Because a user that is only OWNER or POTENTIAL_OWNER of a task can only retrieve the client settings 
        // of a task using the task resource and the tkiid 
        url = "/task/" + tkiid + "/clientSettings/" + type;
      }
      else {
        url = "/taskTemplate/" + tktid + "/clientSettings/" + type;
      }
      url = this.addSystemID(url, systemID);

      var requestLoad = load;
      if (key !== null && typeof load == "function") {
        // Handling for 200 (ok) is implemented, do not cache an 404 result,
        // because a 404 indicates a real error. One of the next calls for
        // this template should bring the real template.
        var expirationPeriod = 3600000;
        var data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(key);
        if (data) {
          // emulate 2.argument ??
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(key, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
      }
      this.performRequest("get", {
        url: url,
        sync: !async,
        load: requestLoad,
        error: error
      });
    },

    /**
     * 
     * @param id
     * @param load
     * @param error
     * @param async
     * @returns {void} nothing
     */
    getTaskTemplateDetails: function getTaskTemplateDetails_RestHandler(tktid, load, error, async, systemID){
      var url = "/taskTemplate";
      url += "/" + tktid;

      url = this.addSystemID(url, systemID);

      this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });
    },

    assignTaskBack: function assignTaskBack_RestHandler(tktid, load, error, systemID){
      var url = "/task";
      url += "/" + tktid + "?action=assign&back=true";

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToUser: function assignTaskToUser_RestHandler(id, load, error, user, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toUser=" + window.encodeURIComponent(user);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToGroup: function assignTaskToGroup_RestHandler(id, load, error, group, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toGroup=" + window.encodeURIComponent(group);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    inviteUserToCollaborate: function inviteUserToCollaborate_RestHandler(id, load, error, userid, invitationMessage,
      systemID){
      var url = "/task/" + id;

      url = this.addSystemID(url, systemID);

      this.performRequest("post", {
        url: url,
        content: {
          action: 'invite',
          user: userid,
          message: invitationMessage
        },
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "Accept": "application/json"
        },
        sync: true,
        load: load,
        error: error
      });
    },

    statics: {
      _instance: null,

      _createInstance: function(params){
        if (this._instance === null) {
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
          com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
          // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
        }
        else {
          // programming error
          var error = new Error("RestHandler instance has already been initialized!");
          com.ibm.bpm.social.util.common.Log.consoleError(error.message);
          throw error;
        }
        return this._instance;
      },

      _getInstance: function(){
        if (this._instance === null) {
          // TODO for now return default handler
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
        }
        return this._instance;
      }
    }
  });

com.ibm.bpm.social.util.common.TaskRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.TaskRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.ExposedItemsList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.ExposedItemsList"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.ExposedItemsList");








/* ExposedItemList 
*/
dojo.declare(
    "com.ibm.bpm.social.widgets.ExposedItemsList",
    [com.ibm.bpm.social.widgets.common._RestWidget],
    {
      constructor: function(){
        this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		this.hasTips = false;  // this will be set during processing. false means this is process server;  true means this is process center
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.trackSnapshotsOnIPad=false;
		this.scoreboardMessageKeyMapping= {
			"scoreboard.MyPerf.name": "pmmyperformance",
			"scoreboard.MyTeamPerf.name": "pmmyteamperformance",
			"scoreboard.ProcPerf.name": "pmprocessperformance",
			"scoreboard.SLAView.name": "pmslaoverview",
			"Ad Hoc Reports": "adhocreports"
		  };
		this.workDashIdentifier="BPM_WORK";
      },

      getData: function(exposedItemsList/*response from exposed items REST query: data.data.exposedItemsList*/, typeArg/*array of strings*/){
        var exposedItems = exposedItemsList;

        var items = [];
        var snapshotItems = {};

		// This function supports the Portal Launch List and the Dashboard Organizer (also called the Tab Organizer).
		// It collects all instances of an item (e.g. a dashboard) and groups them together, with a 'top level' item and 'snapshots' beneath it.
		// Note that a top level items is defined this way, using a dashboard as an example:
		// 1.  If a default version of the PD in which the dashboard is found is set as the default (in Process Admin) then
		//        that instance of the dashboard is the top level item.
		//  2. If there is no default version but there is a tip (Process Center only) then that is the top level item.
		//  3. If there is no default or tip then the snapshot with the most recent create date is the top level item.
		// The other instances of the dashboard are grouped at a single level beneath the top item. On Process Center it is
		//   possible that the tip is beneath the top level item if a different version of the app is set as the default.
		// This function also disambiguates the displayed name of a top level item.  This is necessary when, for example,
		//    > 1 Process Apps have a dashboard with the same name. 
		//
        // For iPad: display only the top level item -- do not show any submenu **NOTE**:  if this.trackSnapshotsOnIPad then we'll ignore this restriction.

        var isIPad = false;
        if (dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
          isIPad = true;
        }
        var duplicateItemCount = {};
        var tipOrDefault = {};
		var toolkitItems = {};
        var filteredItems = [];
		// If there are no tips at all present, we assume that the environment is Process Server, which involves a slightly
		//    different piece of logic when associating snapshots later on.
		
		// Loop over the items and retain those that match the requested item type(s)
        dojo.forEach( exposedItems, dojo.hitch( this, function(item){
		  /* select items based on typeArg*/
		  var itemMatchesType = false;
		  if (item.type) {
			  if (dojo.indexOf(typeArg,"scoreboard")>=0 && item["type"] == "scoreboard") {
				itemMatchesType = true;
				item.itemType = "scoreboardOrDashboard";
				item.preciseItemType = "scoreboard";
			  }
			  else if (dojo.indexOf(typeArg,"dashboard")>=0  && (item["type"] == "service" && item["subtype"] == "dashboard")) {
				itemMatchesType = true;
				item.itemType = "scoreboardOrDashboard";
				item.preciseItemType = "dashboard";
			  }
			  else if (dojo.indexOf(typeArg,"queue")>=0 && item["type"] == "queue") {
				itemMatchesType = true;
				item.itemType = "queue";
				item.preciseItemType = "queue";
			  }
			  else if (dojo.indexOf(typeArg,"url")>=0  && (item["type"] == "service" && item["subtype"] == "url")) {
				itemMatchesType = true;
				item.itemType = "url";
				item.preciseItemType = "url";
			  }
			  else if (dojo.indexOf(typeArg,"processOrService")>=0) {
				  // keep two types of data: processes and startable services; for processes, keep only those for which there is a 'startURL'.
				  var startURLExists = ((typeof item.startURL != 'undefined') && item.startURL);
				  if (item.type == "process" && startURLExists) {
						item.itemType = "exposedprocess";
						item.preciseItemType = "exposedprocess";
						itemMatchesType = true;
				  } else if (item.type == "service" && item.subtype && item.subtype == "startable_service") {
						item.itemType = "tasktemplate";
						item.preciseItemType = "tasktemplate";
						itemMatchesType = true;
				  }
			  } 
		  }
		  if (itemMatchesType) {  // process only items of requested type
			
			var baseID = item.ID;
			item["baseID"] = baseID;  // add ID to item under attr 'baseID' because 'ID' is too common
			// save it!
			filteredItems.push(item);
		  }
		}));  // end dojo.forEach

		
		// Loop over the items and track dashboards and scoreboards that are defined in toolkits
        dojo.forEach( filteredItems, dojo.hitch( this, function(item){
			var itemID = item.itemID;
			if (typeof (toolkitItems[itemID]) == 'undefined') {
				// initialize...
				toolkitItems[itemID] = {
					'definedInAToolkit':false,
					'usedInProcessApp':false
				};
			}
			if (item.itemType == "scoreboardOrDashboard") {
				// Track dashboards and scoreboards that are defined in toolkits.
				//   The rules for a dashboard or scoreboard defined in a toolkit are:
				//   1. It must be used in a process app in order for it to be displayed
				//   2. It can be displayed only once even if it is used in many process apps
				// if item.topLevelToolkitAcronym is defined then this dashboard is defined in a toolkit
				if (item.topLevelToolkitAcronym) {
					toolkitItems[itemID].definedInAToolkit = true;
				} else {
					// It is not defined in a toolkit but it is used in a process app
					toolkitItems[itemID].usedInProcessApp = true;
				}
			}
		}));  // end dojo.forEach
		
		
		// Remove toolkit dashboards or scoreboards that don't meet the rules:
		//   1. It must be used in a process app in order for it to be displayed
		//   2. It can be displayed only once even if it is used in many process apps
        var tempItemsArray = [];
        dojo.forEach(filteredItems, dojo.hitch(this, function(item){
			var itemID = item.itemID;
			var keepItem = true;
			// If this dashboard or scoreboard was defined in a toolkit and is used in one or more process apps, then
			//   keep only the toolkit item.  If it is defined in a toolkit but not used in
			//   any process app then don't keep it.	
			if (item.itemType == "scoreboardOrDashboard") {
				if (toolkitItems[itemID].definedInAToolkit == true) {
					if (toolkitItems[itemID].usedInProcessApp == true) {
						if (!item.topLevelToolkitAcronym) {
							// not the toolkit item
							keepItem = false;
						}
						
					} else {
						// not used in a process app, so don't keep it
						keepItem = false;
					}
				}
			}
			// Be sure to keep items that aren't dashboards or scoreboards!  That's why keepItem is true by default.
			if (keepItem) {
				tempItemsArray.push(item);
			}
        }));
		
		// clone tempItemsArray into filteredItems for more processing
		filteredItems = dojo.clone(tempItemsArray);
		
		
		// We will track this information:
		// 1. duplicate items by title
		// 2. duplicate items by itemID
		// 3. whether or not a tip exists for an app
		// 4. latest or default Version for an app
        dojo.forEach( filteredItems, dojo.hitch( this, function(item){
			var itemID = item.itemID;
			var snapName = (item.tip ? this.messages["tipDisplayName"] : item.snapshotName);
			var baseID = item.ID;
			var snapshotCreateDate = item.snapshotCreatedOn;
			var title = item.display;
			
			// Use 'title' field if it exists
			if (typeof item.title != "undefined" && item.title) {
			  title = item.title;
			}
			
			// Track items by itemID, so we can keep track of tips and snapshots
			if (typeof (duplicateItemCount[itemID]) == 'undefined') {
			  duplicateItemCount[itemID] = {'count': 0};  // initialize duplicate count to 0 since this item is seen for the first time.
			}
			
			// On Process Center the tip and the latest snapshot have the same itemID and snapshot name
			if (typeof (duplicateItemCount[itemID][snapName]) == 'undefined') {
			  duplicateItemCount[itemID].count++;
			}
			
			// Also track items by display name, so we can know if we need to disambiguate the display name;
			// DO NOT disambiguate saved search names.  Don't add saved search names to duplicateItemCount[title].
			if (item.itemType != "queue") {
				if (typeof (duplicateItemCount[title]) == 'undefined') {
				  duplicateItemCount[title] = {'itemsArray': [item]}; //  this is the first time this 'title' has been seen; we may see more so keep an array of them
				}
				else {
				  // the title was seen before; add current item to array of items with this title
					duplicateItemCount[title].itemsArray.push(item);
				}
			}
			
			// track tips, default Versions and most recent Versions for this itemID.
			if (typeof (tipOrDefault[itemID]) == 'undefined') {
				// initialize...
				tipOrDefault[itemID] = {
					'tipExists':false,
					'baseID':"",
					'tipBaseID': "",
					'defaultBaseID': "",
					'snapshotCreateDate': ""
				};
			}
			if (toolkitItems[itemID].definedInAToolkit == false || (toolkitItems[itemID].definedInAToolkit == true && item.topLevelToolkitAcronym)){
				if (item.tip) {
					tipOrDefault[itemID].tipExists = true;
					tipOrDefault[itemID].tipBaseID = baseID;
					tipOrDefault[itemID].baseID = "";
					tipOrDefault[itemID].snapshotCreateDate = snapshotCreateDate;				
					this.hasTips = true;
					//  We need to remove the &snapshot query param from the runURL of a Tip.
					if (item["runURL"]) {
						item["runURL"] = this.removeSnapshotParamFromRunURL(item["runURL"]);
					}	
				} else {
					if (true==item.isDefault) {
						tipOrDefault[itemID].defaultBaseID = baseID;		
						// Set the baseID (latest version) to ""
						tipOrDefault[itemID].baseID = "";
					} else if ((tipOrDefault[itemID].defaultBaseID=="") && (tipOrDefault[itemID].tipExists==false) && tipOrDefault[itemID].snapshotCreateDate < snapshotCreateDate) {
						// Note that if the default or tip was already found, we don't want to track latest version, hence the checks in this if stmt
						tipOrDefault[itemID].baseID = baseID;
						tipOrDefault[itemID].snapshotCreateDate = snapshotCreateDate;
					}
				}
			}

		}));  // end dojo.forEach
		
				
		// So far we've tracked this information:
		// 1. duplicate items by title
		// 2. duplicate items by itemID
		// 3. whether or not a tip exists for the app
		// 4. latest or default Version
		// 5. toolkit information
		//
		// We need to do these things:
        // 1. For each tip (or default Version) create an array of objects containing snapshots (if any).
        // 		The top-level list will contain either tips or default Versions (on Process Center) 
		//           or default or latest Version (on Process Server);
		//           snapshots will be in an array that is the child of the top-level item.
		//      On Process Center -- if there is a snapshot that is identified as the default then we will not store the tip
		// 2. Disambiguate names.  
		// 3. For a dashboard that is defined in a toolkit:  Show it only if it is used in a Process App. Also,
		//     show such a dashboard only once regardless of how many Process Apps it is used in.
        dojo.forEach(filteredItems, dojo.hitch(this, function(item){
          var itemID = item.itemID;
          var title = item.display;
		  // use 'title' field if it exists
		  if (typeof item.title != "undefined" && item.title) {
			  title = item.title;
		  }
		  // originalTitle is the non-nls name of the item.  originalTitle is used in other places 
		  //   to identify specific dashboards, like the Work and Process Performance dashboards.
		  var originalTitle=item.display;
          var snapName = (item.tip ? this.messages["tipDisplayName"] : item.snapshotName);
          var appId = item.processAppID;
          var baseID = item.baseID;
          var snapshotCreateDate = item.snapshotCreatedOn;
		  var isTip_defaultSnapshotDoesNotExist= (item.tip && ""==tipOrDefault[itemID].defaultBaseID);
		  var isTip_defaultSnapshotExists= (item.tip && ""!=tipOrDefault[itemID].defaultBaseID);
		  var isDefault = item.isDefault;
		  
		  // Determine if the item is a snapshot.
		  var thisIsASnapshot = true;
		  if (duplicateItemCount[itemID].count > 1) {  // there can't be snapshots unless there are >1 items
			  if (isTip_defaultSnapshotDoesNotExist) {
				  thisIsASnapshot = false;
			  } else if (isTip_defaultSnapshotExists) {  // just putting this case here for clarity
				  thisIsASnapshot = true;		  
			  } else if (isDefault) {  // just putting this case here for clarity
				  thisIsASnapshot = false;		  
			  } else if (baseID==tipOrDefault[itemID].defaultBaseID) {  // this is the default Version
				  thisIsASnapshot = false;
			  } else if (baseID==tipOrDefault[itemID].baseID) {  // this is the latest Version
				  thisIsASnapshot = false;
			  }
		  }  else {
			  thisIsASnapshot = false;
		  }
		  //  If this is the Work dashboard (title is "BPM_WORK") then we want to keep only 1 item even
		  //     if we see numerous snapshots.  We will keep the one that has been identified as the
		  //     top-level item.   In addition, if the runURL has a snapshot request param, we need to
		  //     remove it.  This is because the Work dashboard will be recognized in Portal only *without* snapshot info.
		  var continueProcessing=!(this.workDashIdentifier==originalTitle && thisIsASnapshot);
		  if (continueProcessing) {
			  if (this.workDashIdentifier==originalTitle) {
				  if (item["runURL"]) {
					  item["runURL"] = this.removeSnapshotParamFromRunURL(item["runURL"]);
				  }
			  }
		  
			  //decide whether any additional identifying information should be displayed (disambiguation)
			  if (duplicateItemCount[title] && duplicateItemCount[title].itemsArray.length > 1 && (!thisIsASnapshot)) {
				//    Loop over items with the same title. If the items are in the same Process App/Toolkit but in a different track (also called branch or workspace)
				//    AND they're both tips, then add the branch name
				var useBranchName = false;
				var getTaskTemplate = false;
				for ( var x = 0; x < duplicateItemCount[title].itemsArray.length; x++) {
				  var arrayItem = duplicateItemCount[title].itemsArray[x];
				  if (baseID != arrayItem.baseID) { // don't compare an item to itself!
					if (itemID == arrayItem.itemID && appId == arrayItem.processAppID && item.tip == true && arrayItem.tip == true) {
					  useBranchName = true;
					}
					else if (itemID != arrayItem.itemID || appId != arrayItem.processAppID) {
					  getTaskTemplate = true;
					}
				  }
				}
				// Don't disambiguate saved search names
				if (item.itemType!="queue" ) {
					if (useBranchName) {
					  // disambiguate using branchName (if branchName doesn't exist - i.e., isn't even in the object -  or if it is null, use snapshotName)
					  var branchName =
						(typeof item.branchName != "undefined" && item.branchName) ? item.branchName : item.snapshotName;
					  title = title + " (" + branchName + ")";
					}
					else if (getTaskTemplate) {
					  var name1 = "";
					  // this might be a process app or a toolkit
					  name1 = (item.topLevelToolkitName) ? item.topLevelToolkitName: item.processAppName ;
					  title = title + " (" + name1 + ")";
					}
				}
			  }
			  //  For scoreboards - see if the title is actually a key in a message bundle:
			  //  NOTE that we did not want to disambiguate the NLS title of a scoreboard --
			  //     if a scoreboard and a item of another type, such as dashboard, have the same name their
			  //      types will be indicated by icons in displayed lists, not by disambiguated names.
			  if (item["type"] == "scoreboard") {
				  title = this.getNLSDisplayName(title);
				  // replace the title in the object
				  item.title=title;
			  }
			  // create object for the list
			  var runURL = (item["runURL"] ? item["runURL"] : null);  // used for dashboards
			  var startURL = (item["startURL"] ? item["startURL"] : null);  // used for exposed processes
			  var itemObj =
				this.createItemObject(title, itemID, item.itemType, item.preciseItemType, baseID, appId, item.snapshotID, item.tip, thisIsASnapshot,
					snapshotCreateDate, snapName, item.branchID, runURL, startURL, originalTitle, item.name, item.itemReference);
					
			  if (!isIPad || this.trackSnapshotsOnIPad==true) {
				if (thisIsASnapshot == true) {
				  // store this snapshot object - it will be added to the parent tip object later.
				  if (!(itemID in snapshotItems)) {  // create array and add item to it
					var snapArray = [];
					snapArray.push(itemObj);
					snapshotItems[itemID] = snapArray;
				  }
				  else {  // add item to existing array
					snapshotItems[itemID].push(itemObj);
				  }
				}
				else {  // this is not a snapshot
				  items.push(itemObj);
				}
			  } else { // iPad case
				if (thisIsASnapshot == false) {
				  items.push(itemObj);
				}
			  }
		  }
        }));

        //sort items alphabetically
        items.sort(function(a, b){
          var ret = 0;
          if (a.title > b.title) {
            ret = 1;
          }
          else if (a.title < b.title) {
            ret = -1;
          }
          return ret;
        });

        //sort array of snapshots from newest to oldest
        for ( var sItem in snapshotItems) {
          var snapItems = snapshotItems[sItem];
          snapItems.sort(function(a, b){
            var ret = 0;
            if (a.params.snapshotCreateDate < b.params.snapshotCreateDate) {
              ret = 1;
            }
            else if (a.params.snapshotCreateDate > b.params.snapshotCreateDate) {
              ret = -1;
            }
            return ret;
          });
        }

        // for each list item, associate snapshots, if any
        for ( var i = 0; i < items.length; i++) {
          var itemID = items[i]["itemID"];
		  var filteredSnapItems = [];
          var snapItems = snapshotItems[itemID];
          if (snapItems) {
			// Double check the branchID to make sure snapshot items are on same
			//     track, otherwise remove from the snapshot list.
			//     This check is only valid if the environment is Process Center.
			if(this.hasTips) {
				for(var j = 0; j < snapItems.length; j++) {
					var curItem = snapItems[j];
					if(typeof curItem.params != "undefined" && 
						typeof curItem.params.branchID != "undefined" &&
						typeof items[i]["params"] != "undefined" &&
						typeof items[i]["params"]["branchID"] != "undefined") {
						if(curItem.params.branchID == items[i]["params"]["branchID"]) {
							//if the IDs match, they should be on the same
							//branch, otherwise not
							filteredSnapItems.push(curItem);
						}
					} else {
						//by default, just add it
						filteredSnapItems.push(curItem);
					}	
				}
			} else {  // Process Server case:
				// Make sure the itemReference is the same, otherwise remove from the snapshot list.
				//     This check is only valid if the environment is Process Server.
				for(var j = 0; j < snapItems.length; j++) {
					var curItem = snapItems[j];
					if(typeof curItem.params != "undefined" && 
						typeof curItem.params.itemReference != "undefined" &&
						typeof items[i]["params"] != "undefined" &&
						typeof items[i]["params"]["itemReference"] != "undefined") {
						if(curItem.params.itemReference == items[i]["params"]["itemReference"]) {
							//if the IDs match, they should be on the same
							//branch, otherwise not
							filteredSnapItems.push(curItem);
						}
					} else {
						//by default, just add it
						filteredSnapItems.push(curItem);
					}	
				}
			}
			
			if(filteredSnapItems.length > 0) {
				// Now that we have associated a snapshot with a top-level item, we need
				// to change the snapshot 'title' to match the possibly disambiguated title
				// of the top-level item
				for (var x=0; x<filteredSnapItems.length; x++) {
					filteredSnapItems[x].title=items[i].title;
					filteredSnapItems[x].params.title=items[i].title;
				}
				// Add snapshot items:
				items[i].params["snapshotItems"] = filteredSnapItems;
			}
          }
        }

		return items;

      },

	  removeSnapshotParamFromRunURL: function (runURLArg) {
		var retRunURL=runURLArg;

		// remove snapshot info if present; this isn't fancy uri parsing, just text location and removal
		var inx = retRunURL.indexOf("&snapshot=");
		if (inx>0) {
			var parts1 = retRunURL.split("?");
			if (parts1.length==2) {
				var parts2 = parts1[1].split("&");
				var newRunURL = parts1[0]+"?";
				var count = 0;
				if (parts2.length>1) {
					for (var i=0; i<parts2.length; i++) {
						if (parts2[i].indexOf("snapshot=") <0) {
							if (count > 0) {
								newRunURL = newRunURL+"&";										
							}
							count++;
							newRunURL = newRunURL+parts2[i];
						}
					}
				}
				// was anything done?
				if (count>1) {
					retRunURL = newRunURL;
				}
			}
		}		
		return retRunURL;
	  },
	  
      createItemObject: function(title, itemID, itemType, preciseItemType, baseID,  appId, snapshotID, tip, isSnapshot, snapshotCreateDate,
        snapName, branchID, runURL, startURL, originalTitle, name, itemReference){
        var itemObj = {
          'title': title,
          'itemID': itemID,
          'params': {
            'type': itemType,
			'preciseItemType': preciseItemType,
            'baseID': baseID,
			'title' : title, // yes it is stored again
            'itemID': itemID, // yes it is stored again
            'processID': appId,
            'snapshotID': snapshotID,
            'isTip': tip,
            'isSnapshot': isSnapshot,
            'snapshotCreateDate': snapshotCreateDate,
            'snapshotName': snapName,
			'branchID': branchID,
			'runURL' : runURL,
			'startURL' : startURL,
			'originalTitle': originalTitle,
			'name': name,
			'itemReference': itemReference
          }
        };
        return itemObj;
      },

	  getNLSDisplayName: function(displayName){
		var msgKey = this.scoreboardMessageKeyMapping[displayName];
		if (msgKey && this.messages[msgKey]) {
		  displayName = this.messages[msgKey];
		}
		return displayName;
	  },

	  isProcessServer: function() {
		// Returns true if the current env is Process Server
		// *** The return value is reliable *if* there were >0 exposed items found in the getData() function.
		// If there were 0 exposed items found the the value is unreliable but is also irrelevant -- 
		//    the returned value only matters when there are exposed items.
		return (this.hasTips==false) ? true:false;
	  },
	  
      destroy: function(finalize){
		this.inherited(arguments);
      }

    });

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.SmartList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.SmartList"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.SmartList");




/* SmartList - A dojo widget responsible for making life easier when handling 
	lists of items that need to be refreshed. Used most notably in the social 
	portal sidebar for LaunchableProceeList, FollowedProcessList, and 
	TaggedProcessList. Functions to override in subclasses (optional): 
	_createItem, _onItemAdded, _onItemClicked, and refresh.
	
	Updated: 1/9/2012
	Experts: Ben Myers @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.SmartList",
	[dijit._Widget, dijit._Templated],
	{
	
	templateString: "",
	templateString:"<div class=\"bpm-social-smartlist\">\n\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t<ul dojoAttachPoint=\"listElement\"><li style=\"display:none\">&nbsp;</li></ul>\n</div>\n",
	constructor: function() {
		//items added to the SmartList are indexed using currIndex
		this.currIndex = -1; 
		//items can be assigned parameters that are exposed during events
		this.itemParams = [];
		//event connectors are added to an array so they can be disconnected
		this.smartListEventConnectors = [];
		//number of items in the list - used by the numItems function
		this.itemCount = 0;
		
		//timer variables - see startRefreshTimer and stopRefreshTimer
		this.timer = null;
		this.refreshRate = 0; //in SECONDS
		
		//whether a message should be displayed when the list is empty
		this.showEmptyMessage = true;
	},
	
	postCreate: function() {
		//localization
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.smartListEventConnectors = [];
		this.itemParams = [];
		this.itemCount = 0;
		
		//refresh the list to start
		this.refresh();
		
		//start refreshing the list (if refreshRate was passed to the widget)
		if(this.refreshRate!=null && this.refreshRate>0) {
			this.startRefreshTimer(this.refreshRate);
		}
		
		//hide the empty message if showEmptyMessage is false (even if empty)
		if(!this.showEmptyMessage) {
			dojo.style(this.emptyMessageParagraph, "display", "none");
		}
	},
	
	/* addItem: Adds an item to the SmartList. When the item is clicked any 
		params it was created with will be exposed, so the event can be better 
		handled. createParams will be provided to the _createItem method so 
		the actual HTML can be properly created. Essentially, if you have data 
		about the item, add the data to the params unless it is only used to 
		set up the item's HTML - in that case add it to createParams. */
	addItem: function(params, createParams, addToTop) {
		if(typeof(params)=='undefined') {
			params = null;
		}
		if(typeof(createParams)=='undefined') {
			createParams = null;
		}
		if(typeof(addToTop)=='undefined' || addToTop==null) {
			addToTop = false;
		}
		
		//since we're adding an item, hide the empty list message
		if(this.itemCount <= 0) {
			dojo.style(this.emptyMessageParagraph, "display", "none");
			dojo.empty(this.listElement);
		}
		
		//create the HTML element and add it to the DOM
		this.currIndex++;
		var li = dojo.create("li", {'smartListIndex':this.currIndex});
		if(addToTop) {
			dojo.place(li, this.listElement, 'first');
		}
		else {
			dojo.place(li, this.listElement);
		}
		
		//add dojo event connectors
		this.smartListEventConnectors.push(dojo.connect(li, 'onmouseenter', this, "_itemMouseoverEvent"));
		this.smartListEventConnectors.push(dojo.connect(li, 'onmouseleave', this, "_itemMouseoutEvent"));
		this.smartListEventConnectors.push(dojo.connect(li, 'onclick',      this, "_itemClickEvent"));
		
		//increment the item count
		this.itemCount++;
		//store the params for retrieval when the user interacts with the item
		this.itemParams[this.currIndex] = params;
		//call the _creatItem method, to be used as a hook for subclasses
		this._createItem(this.currIndex, li, params, createParams);
		//call the _onItemAdded method, to be used as a hook for subclasses
		this._onItemAdded(this.currIndex, li, params);
		//return the index so the user can retrieve the item if necessary
		return this.currIndex;
	},
	
	/* removeItem: Removes an item from the SmartList. Consider using 
		clearList and rebuilding items if the majority of the list needs to be 
		removed. Note the index is not guaranteed to be the visual index of 
		the item (index=0 could be anywhere) since items can be added to the 
		top or the bottom. */
	removeItem: function(index) {
		//get the item and make sure it exists
		var item = this.getItem(index);
		if(item==null) {
			return;
		}
		
		//remove the element from the DOM
		if(typeof(item.li.parentNode)!='undefined' && item.li.parentNode!=null) {
			item.li.parentNode.removeChild(item.li);
		}
		
		//remove the recorded params of the item
		this.itemParams[item.index] = null;
		
		//decrement the item count
		this.itemCount--;
		
		//if the list should display an empty message, now might be a good time
		if(this.itemCount==0) {
			if(this.showEmptyMessage) {
				dojo.style(this.emptyMessageParagraph, "display", "");
			}
			dojo.place(dojo.create("li", {'innerHTML':"&nbsp;", 'style':"display:none"}), this.listElement);
		}
		
		this._onItemRemoved(item.index, item.li, item.params);
	},
	
	/* numItems: Returns the number of items in the list. */
	numItems: function() {
		return this.itemCount;
	},
	
	/* _createItem: Called whenever addItem is successful. Used as a hook for 
		subclasses. This function should be overriden to create the actual 
		HTML elements necessary to properly display the item. */
	_createItem: function(index, li, params, createParams) {}, //OVERRIDE ME!
	
	/* _onItemAdded: Called whenever addItem is successful. Used as a hook for 
		subclasses. This function should be overriden when something needs to 
		happen whenever an item is added (e.g. incrementing a count). Called 
		after _createItem. Note that although this function is similar to 
		_createItem, it should be used for different purposes (_createItem 
		should be used for setting up the HTML and relevant events only). */
	_onItemAdded: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemAdded: Called whenever removeItem is successful. Used as a hook 
		for subclasses. */
	_onItemRemoved: function(index, li, params) {}, //OVERRIDE ME!
	
	/* getItem: Returns an item given an index. Note the SmartList is
		0-indexed and there is no guarantee the visual order of the list is 
		exactly the same as the indexed order. It is guaranteed, however, that 
		each item can be uniquely identified by its index. Returns a JSON 
		object with the item's index, li, and params if the item was found.
		Returns null otherwise. */
	getItem: function(index) {
		if(typeof(index)=='undefined' || index==null) {
			return null;
		}
		
		//search for the HTML element
		var li = dojo.query('li[smartListIndex="'+index+'"]', this.listElement);
		if(typeof(li)=='undefined' || li==null || typeof(li.length)=='undefined' || li.length==null || li.length<=0) {
			return null;
		}
		li = li[0];
		
		//grab the params
		var params = this.itemParams[index];
		if(typeof(params)=='undefined') {
			params = null;
		}
		
		//return the item's index, HTML element, and params
		return {'index':index, 'li':li, 'params':params};
	},
	
	/* getItems: Returns an array of all items as JSON objects, each 
		containing an item's index, parent li element, and parameters that 
		were given when the item was initially created. */
	getItems: function() {
		var itemArr = [];
		dojo.query("li[smartListIndex]", this.listElement).forEach(function(li) {
			var index = parseInt(dojo.attr(li, "smartListIndex"));
			var params = this.itemParams[index];
			if(typeof(params)=='undefined') {
				params = null;
			}
			itemArr.push({'index':index, 'li':li, 'params':params});
		}, this);
		return itemArr;
	},
	
	/* _extractItemInfoFromEvent: A helper method called whenever an event 
		fires for an item. This will extract the pertintent item information 
		before the appropriate handler is called. */
	_extractItemInfoFromEvent: function(evt) {
		if(evt==null) {
			return {'index':null, 'li':null, 'params':null};
		}
		var target = evt.target;
		var targetIndex = null;
		var targetParams = null;
		
		//since a child element within the item may have triggered the event, 
		// it is necessary to search the event's target's parents for the 
		// actual item element
		for(var i=0; i<=10 && target!=null && dojo.attr(target, "smartListIndex")==null; i++) {
			//note any infinite loop issues are avoided by only looping a 
			// maximum of 10 times - no error has ever been encountered but 
			// there's no point in one little widget crashing a whole site
			if(i==10) {
				return null;
			}
			target = target.parentNode;
		}
		if(target!=null) {
			targetIndex = dojo.attr(target, "smartListIndex");
			
			//the params given to the item when it was created
			targetParams = this.itemParams[targetIndex];
			if(typeof(targetParams)=='undefined') {
				targetParams = null;
			}
		}
		return {'index':targetIndex, 'li':target, 'params':targetParams};
	},
	
	/* Below are event handlers that do little other than handle basic 
		interactions with the SmartList. */
	_itemMouseoverEvent: function(evt) {
		//add the "hover" class, since li:hover is technically improper css
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null && item['li']!=null) {
			dojo.addClass(item['li'], "hover");
			this._onItemMouseOver(item.index, item.li, item.params);
		}
	},
	_itemMouseoutEvent: function(evt) {
		//remove the "hover" class, since li:hover is technically improper css
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null && item['li']!=null) {
			dojo.removeClass(item['li'], "hover");
			this._onItemMouseOut(item.index, item.li, item.params);
		}
	},
	_itemClickEvent: function(evt) {
		var item = this._extractItemInfoFromEvent(evt);
		if(item!=null) {
			this._onItemClicked(item['index'], item['li'], item['params']);
		}
	},
	
	/* _onItemClicked: Called whenever an item is clicked. The params argument 
		is provided when the item is created and should hold whatever 
		information is required to properly handle the event. */
	_onItemClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemClicked: Called whenever an item is moused over. */
	_onItemMouseOver: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemClicked: Called whenever an item is moused out. */
	_onItemMouseOut: function(index, li, params) {}, //OVERRIDE ME!
	
	/* clearList: Removes all items from the SmartList. This is also where 
		dojo event connectors are cleared. */
	clearList: function() {
		//remove all the HTML elements
		dojo.empty(this.listElement);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.smartListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		//reset variables
		this.smartListEventConnectors = [];
		this.itemParams = [];
		this.currIndex = -1;
		this.itemCount = 0;
		
		//if the list should display an empty message, now is a good time
		if(this.showEmptyMessage) {
			dojo.style(this.emptyMessageParagraph, "display", "");
			dojo.place(dojo.create("li", {'innerHTML':"&nbsp;", 'style':"display:none"}), this.listElement);
		}
	},
	
	/* setEmptyMessage: Sets the message that appears when the list is empty. 
		Note that nothing will be displayed if showEmptyMessage is set to 
		false or if the list contains items. */
	setEmptyMessage: function(msg) {
		if(typeof(msg)=='undefined' || msg==null) {
			dojo.attr(this.emptyMessageParagraph, "innerHTML", "");
		}
		else {
			dojo.attr(this.emptyMessageParagraph, "innerHTML", msg);
		}
	},
	
	/* startRefreshTimer: Starts a timer that fires every so often, depending 
		on the SmartList's refreshRate. Typically used to refresh the data in 
		the SmartList. Timer variables and their lifecycles are handled by the 
		SmartList. Note that refreshRate is in SECONDS. */
	startRefreshTimer: function(refreshRate) {
		if(typeof(refreshRate)!="undefined") {
			this.refreshRate = Math.max(0, Number(refreshRate));
			if(this.refreshRate>=1000) {
				console.log("A SmartList has been assigned a large refresh rate ("+this.refreshRate+" seconds)."
					+" Please ensure all refresh rates are in seconds and not milliseconds.");
			}
		}
		if(this.refreshRate!=null && this.refreshRate>0) {
			if(this.timer) {
				clearTimeout(this.timer);
			}
			this.timer = setTimeout(dojo.hitch(this, "_handleRefresh"), 1000*Number(this.refreshRate));
		}
	},
	
	/* stopRefreshTimer: Prevents the timer created through startRefreshTimer 
		from firing any more events. Returns true if the timer was running and 
		stopped successfully. Returns false otherwise.*/
	stopRefreshTimer: function() {
		if(this.timer) {
			clearTimeout(this.timer);
			return true;
		}
		return false;
	},
	
	/* _handleRefresh: Makes sure the timer continues refreshing the SmartList 
		if it has been started. refresh will be called by this method. */
	_handleRefresh: function() {
		this.refresh();
		
		//if the timer was running when it was stopped...
		if(this.stopRefreshTimer()) {
			//restart the timer with the same refresh rate
			this.startRefreshTimer();
		}
	},
	
	/* refresh: Refreshes the list with new data. */
	refresh: function() {}, //OVERRIDE ME!
	
	destroy: function(finalize) {
		//stop the timer
		this.stopRefreshTimer();
		this.timer = null;
		
		//destroy the item elements
		this.clearList();
		
		this.inherited(arguments);
	}
	
});



}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.getObject("dnd", true, dojo);

dojo.dnd.getCopyKeyState = dojo.isCopyKey;

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.dnd.Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Container"] = true;
dojo.provide("dojo.dnd.Container");




/*
	Container states:
		""		- normal state
		"Over"	- mouse over a container
	Container item states:
		""		- normal state
		"Over"	- mouse over a container item
*/

/*=====
dojo.declare("dojo.dnd.__ContainerArgs", [], {
	creator: function(){
		// summary:
		//		a creator function, which takes a data item, and returns an object like that:
		//		{node: newNode, data: usedData, type: arrayOfStrings}
	},

	// skipForm: Boolean
	//		don't start the drag operation, if clicked on form elements
	skipForm: false,

	// dropParent: Node||String
	//		node or node's id to use as the parent node for dropped items
	//		(must be underneath the 'node' parameter in the DOM)
	dropParent: null,

	// _skipStartup: Boolean
	//		skip startup(), which collects children, for deferred initialization
	//		(this is used in the markup mode)
	_skipStartup: false
});

dojo.dnd.Item = function(){
	// summary:
	//		Represents (one of) the source node(s) being dragged.
	//		Contains (at least) the "type" and "data" attributes.
	// type: String[]
	//		Type(s) of this item, by default this is ["text"]
	// data: Object
	//		Logical representation of the object being dragged.
	//		If the drag object's type is "text" then data is a String,
	//		if it's another type then data could be a different Object,
	//		perhaps a name/value hash.
	
	this.type = type;
	this.data = data;
}
=====*/

dojo.declare("dojo.dnd.Container", null, {
	// summary:
	//		a Container object, which knows when mouse hovers over it,
	//		and over which element it hovers
	
	// object attributes (for markup)
	skipForm: false,
	
	/*=====
	// current: DomNode
	//		The DOM node the mouse is currently hovered over
	current: null,
	
	// map: Hash<String, dojo.dnd.Item>
	//		Map from an item's id (which is also the DOMNode's id) to
	//		the dojo.dnd.Item itself.
	map: {},
	=====*/
	
	constructor: function(node, params){
		// summary:
		//		a constructor of the Container
		// node: Node
		//		node or node's id to build the container on
		// params: dojo.dnd.__ContainerArgs
		//		a dictionary of parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.creator = params.creator || null;
		this.skipForm = params.skipForm;
		this.parent = params.dropParent && dojo.byId(params.dropParent);
		
		// class-specific variables
		this.map = {};
		this.current = null;

		// states
		this.containerState = "";
		dojo.addClass(this.node, "dojoDndContainer");
		
		// mark up children
		if(!(params && params._skipStartup)){
			this.startup();
		}

		// set up events
		this.events = [
			dojo.connect(this.node, "onmouseover", this, "onMouseOver"),
			dojo.connect(this.node, "onmouseout",  this, "onMouseOut"),
			// cancel text selection and text dragging
			dojo.connect(this.node, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.node, "onselectstart", this, "onSelectStart")
		];
	},
	
	// object attributes (for markup)
	creator: function(){
		// summary:
		//		creator function, dummy at the moment
	},
	
	// abstract access to the map
	getItem: function(/*String*/ key){
		// summary:
		//		returns a data item by its key (id)
		return this.map[key];	// dojo.dnd.Item
	},
	setItem: function(/*String*/ key, /*dojo.dnd.Item*/ data){
		// summary:
		//		associates a data item with its key (id)
		this.map[key] = data;
	},
	delItem: function(/*String*/ key){
		// summary:
		//		removes a data item from the map by its key (id)
		delete this.map[key];
	},
	forInItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over a data map skipping members that
		//		are present in the empty object (IE and/or 3rd-party libraries).
		o = o || dojo.global;
		var m = this.map, e = dojo.dnd._empty;
		for(var i in m){
			if(i in e){ continue; }
			f.call(o, m[i], i, this);
		}
		return o;	// Object
	},
	clearItems: function(){
		// summary:
		//		removes all data items from the map
		this.map = {};
	},
	
	// methods
	getAllNodes: function(){
		// summary:
		//		returns a list (an array) of all valid child nodes
		return dojo.query("> .dojoDndItem", this.parent);	// NodeList
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		var map = {};
		this.getAllNodes().forEach(function(node){
			if(node.id){
				var item = this.getItem(node.id);
				if(item){
					map[node.id] = item;
					return;
				}
			}else{
				node.id = dojo.dnd.getUniqueId();
			}
			var type = node.getAttribute("dndType"),
				data = node.getAttribute("dndData");
			map[node.id] = {
				data: data || node.innerHTML,
				type: type ? type.split(/\s*,\s*/) : ["text"]
			};
		}, this);
		this.map = map;
		return this;	// self
	},
	insertNodes: function(data, before, anchor){
		// summary:
		//		inserts an array of new nodes before/after an anchor node
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		if(!this.parent.firstChild){
			anchor = null;
		}else if(before){
			if(!anchor){
				anchor = this.parent.firstChild;
			}
		}else{
			if(anchor){
				anchor = anchor.nextSibling;
			}
		}
		if(anchor){
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.insertBefore(t.node, anchor);
			}
		}else{
			for(var i = 0; i < data.length; ++i){
				var t = this._normalizedCreator(data[i]);
				this.setItem(t.node.id, {data: t.data, type: t.type});
				this.parent.appendChild(t.node);
			}
		}
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares this object to be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.clearItems();
		this.node = this.parent = this.current = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Container(node, params);
	},
	startup: function(){
		// summary:
		//		collects valid child items and populate the map
		
		// set up the real parent node
		if(!this.parent){
			// use the standard algorithm, if not assigned
			this.parent = this.node;
			if(this.parent.tagName.toLowerCase() == "table"){
				var c = this.parent.getElementsByTagName("tbody");
				if(c && c.length){ this.parent = c[0]; }
			}
		}
		this.defaultCreator = dojo.dnd._defaultCreator(this.parent);

		// process specially marked children
		this.sync();
	},

	// mouse events
	onMouseOver: function(e){
		// summary:
		//		event processor for onmouseover
		// e: Event
		//		mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ this._addItemClass(n, "Over"); }
		this.current = n;
	},
	onMouseOut: function(e){
		// summary:
		//		event processor for onmouseout
		// e: Event
		//		mouse event
		for(var n = e.relatedTarget; n;){
			if(n == this.node){ return; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(this.current){
			this._removeItemClass(this.current, "Over");
			this.current = null;
		}
		this._changeState("Container", "");
		this.onOutEvent();
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skipForm || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
	},
	_changeState: function(type, newState){
		// summary:
		//		changes a named state to new state value
		// type: String
		//		a name of the state to change
		// newState: String
		//		new state
		var prefix = "dojoDnd" + type;
		var state  = type.toLowerCase() + "State";
		//dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
		dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
		this[state] = newState;
	},
	_addItemClass: function(node, type){
		// summary:
		//		adds a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.addClass(node, "dojoDndItem" + type);
	},
	_removeItemClass: function(node, type){
		// summary:
		//		removes a class with prefix "dojoDndItem"
		// node: Node
		//		a node
		// type: String
		//		a variable suffix for a class name
		dojo.removeClass(node, "dojoDndItem" + type);
	},
	_getChildByEvent: function(e){
		// summary:
		//		gets a child, which is under the mouse at the moment, or null
		// e: Event
		//		a mouse event
		var node = e.target;
		if(node){
			for(var parent = node.parentNode; parent; node = parent, parent = node.parentNode){
				if(parent == this.parent && dojo.hasClass(node, "dojoDndItem")){ return node; }
			}
		}
		return null;
	},
	_normalizedCreator: function(/*dojo.dnd.Item*/ item, /*String*/ hint){
		// summary:
		//		adds all necessary data to the output of the user-supplied creator function
		var t = (this.creator || this.defaultCreator).call(this, item, hint);
		if(!dojo.isArray(t.type)){ t.type = ["text"]; }
		if(!t.node.id){ t.node.id = dojo.dnd.getUniqueId(); }
		dojo.addClass(t.node, "dojoDndItem");
		return t;
	}
});

dojo.dnd._createNode = function(tag){
	// summary:
	//		returns a function, which creates an element of given tag
	//		(SPAN by default) and sets its innerHTML to given text
	// tag: String
	//		a tag name or empty for SPAN
	if(!tag){ return dojo.dnd._createSpan; }
	return function(text){	// Function
		return dojo.create(tag, {innerHTML: text});	// Node
	};
};

dojo.dnd._createTrTd = function(text){
	// summary:
	//		creates a TR/TD structure with given text as an innerHTML of TD
	// text: String
	//		a text for TD
	var tr = dojo.create("tr");
	dojo.create("td", {innerHTML: text}, tr);
	return tr;	// Node
};

dojo.dnd._createSpan = function(text){
	// summary:
	//		creates a SPAN element with given text as its innerHTML
	// text: String
	//		a text for SPAN
	return dojo.create("span", {innerHTML: text});	// Node
};

// dojo.dnd._defaultCreatorNodes: Object
//		a dictionary that maps container tag names to child tag names
dojo.dnd._defaultCreatorNodes = {ul: "li", ol: "li", div: "div", p: "div"};

dojo.dnd._defaultCreator = function(node){
	// summary:
	//		takes a parent node, and returns an appropriate creator function
	// node: Node
	//		a container node
	var tag = node.tagName.toLowerCase();
	var c = tag == "tbody" || tag == "thead" ? dojo.dnd._createTrTd :
			dojo.dnd._createNode(dojo.dnd._defaultCreatorNodes[tag]);
	return function(item, hint){	// Function
		var isObj = item && dojo.isObject(item), data, type, n;
		if(isObj && item.tagName && item.nodeType && item.getAttribute){
			// process a DOM node
			data = item.getAttribute("dndData") || item.innerHTML;
			type = item.getAttribute("dndType");
			type = type ? type.split(/\s*,\s*/) : ["text"];
			n = item;	// this node is going to be moved rather than copied
		}else{
			// process a DnD item object or a string
			data = (isObj && item.data) ? item.data : item;
			type = (isObj && item.type) ? item.type : ["text"];
			n = (hint == "avatar" ? dojo.dnd._createSpan : c)(String(data));
		}
		if(!n.id){
			n.id = dojo.dnd.getUniqueId();
		}
		return {node: n, data: data, type: type};
	};
};

}

if(!dojo._hasResource["dojo.dnd.Selector"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Selector"] = true;
dojo.provide("dojo.dnd.Selector");




/*
	Container item states:
		""			- an item is not selected
		"Selected"	- an item is selected
		"Anchor"	- an item is selected, and is an anchor for a "shift" selection
*/

/*=====
dojo.declare("dojo.dnd.__SelectorArgs", [dojo.dnd.__ContainerArgs], {
	//	singular: Boolean
	//		allows selection of only one element, if true
	singular: false,

	//	autoSync: Boolean
	//		autosynchronizes the source with its list of DnD nodes,
	autoSync: false
});
=====*/

dojo.declare("dojo.dnd.Selector", dojo.dnd.Container, {
	// summary:
	//		a Selector object, which knows how to select its children
	
	/*=====
	// selection: Set<String>
	//		The set of id's that are currently selected, such that this.selection[id] == 1
	//		if the node w/that id is selected.  Can iterate over selected node's id's like:
	//	|		for(var id in this.selection)
	selection: {},
	=====*/

	constructor: function(node, params){
		// summary:
		//		constructor of the Selector
		// node: Node||String
		//		node or node's id to build the selector on
		// params: dojo.dnd.__SelectorArgs?
		//		a dictionary of parameters
		if(!params){ params = {}; }
		this.singular = params.singular;
		this.autoSync = params.autoSync;
		// class-specific variables
		this.selection = {};
		this.anchor = null;
		this.simpleSelection = false;
		// set up events
		this.events.push(
			dojo.connect(this.node, "onmousedown", this, "onMouseDown"),
			dojo.connect(this.node, "onmouseup",   this, "onMouseUp"));
	},
	
	// object attributes (for markup)
	singular: false,	// is singular property
	
	// methods
	getSelectedNodes: function(){
		// summary:
		//		returns a list (an array) of selected nodes
		var t = new dojo.NodeList();
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			t.push(dojo.byId(i));
		}
		return t;	// NodeList
	},
	selectNone: function(){
		// summary:
		//		unselects all items
		return this._removeSelection()._removeAnchor();	// self
	},
	selectAll: function(){
		// summary:
		//		selects all items
		this.forInItems(function(data, id){
			this._addItemClass(dojo.byId(id), "Selected");
			this.selection[id] = 1;
		}, this);
		return this._removeAnchor();	// self
	},
	deleteSelectedNodes: function(){
		// summary:
		//		deletes all selected items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var n = dojo.byId(i);
			this.delItem(i);
			dojo.destroy(n);
		}
		this.anchor = null;
		this.selection = {};
		return this;	// self
	},
	forInSelectedItems: function(/*Function*/ f, /*Object?*/ o){
		// summary:
		//		iterates over selected items;
		//		see `dojo.dnd.Container.forInItems()` for details
		o = o || dojo.global;
		var s = this.selection, e = dojo.dnd._empty;
		for(var i in s){
			if(i in e){ continue; }
			f.call(o, this.getItem(i), i, this);
		}
	},
	sync: function(){
		// summary:
		//		sync up the node list with the data map
		
		dojo.dnd.Selector.superclass.sync.call(this);
		
		// fix the anchor
		if(this.anchor){
			if(!this.getItem(this.anchor.id)){
				this.anchor = null;
			}
		}
		
		// fix the selection
		var t = [], e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			if(!this.getItem(i)){
				t.push(i);
			}
		}
		dojo.forEach(t, function(i){
			delete this.selection[i];
		}, this);
		
		return this;	// self
	},
	insertNodes: function(addSelected, data, before, anchor){
		// summary:
		//		inserts new data items (see `dojo.dnd.Container.insertNodes()` method for details)
		// addSelected: Boolean
		//		all new nodes will be added to selected items, if true, no selection change otherwise
		// data: Array
		//		a list of data items, which should be processed by the creator function
		// before: Boolean
		//		insert before the anchor, if true, and after the anchor otherwise
		// anchor: Node
		//		the anchor node to be used as a point of insertion
		var oldCreator = this._normalizedCreator;
		this._normalizedCreator = function(item, hint){
			var t = oldCreator.call(this, item, hint);
			if(addSelected){
				if(!this.anchor){
					this.anchor = t.node;
					this._removeItemClass(t.node, "Selected");
					this._addItemClass(this.anchor, "Anchor");
				}else if(this.anchor != t.node){
					this._removeItemClass(t.node, "Anchor");
					this._addItemClass(t.node, "Selected");
				}
				this.selection[t.node.id] = 1;
			}else{
				this._removeItemClass(t.node, "Selected");
				this._removeItemClass(t.node, "Anchor");
			}
			return t;
		};
		dojo.dnd.Selector.superclass.insertNodes.call(this, data, before, anchor);
		this._normalizedCreator = oldCreator;
		return this;	// self
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Selector.superclass.destroy.call(this);
		this.selection = this.anchor = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Selector(node, params);
	},

	// mouse events
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(this.autoSync){ this.sync(); }
		if(!this.current){ return; }
		if(!this.singular && !dojo.isCopyKey(e) && !e.shiftKey && (this.current.id in this.selection)){
			this.simpleSelection = true;
			if(e.button === dojo.mouseButtons.LEFT){
				// accept the left button and stop the event
				// for IE we don't stop event when multiple buttons are pressed
				dojo.stopEvent(e);
			}
			return;
		}
		if(!this.singular && e.shiftKey){
			if(!dojo.isCopyKey(e)){
				this._removeSelection();
			}
			var c = this.getAllNodes();
			if(c.length){
				if(!this.anchor){
					this.anchor = c[0];
					this._addItemClass(this.anchor, "Anchor");
				}
				this.selection[this.anchor.id] = 1;
				if(this.anchor != this.current){
					var i = 0;
					for(; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
					}
					for(++i; i < c.length; ++i){
						var node = c[i];
						if(node == this.anchor || node == this.current){ break; }
						this._addItemClass(node, "Selected");
						this.selection[node.id] = 1;
					}
					this._addItemClass(this.current, "Selected");
					this.selection[this.current.id] = 1;
				}
			}
		}else{
			if(this.singular){
				if(this.anchor == this.current){
					if(dojo.isCopyKey(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");
					this.selection[this.current.id] = 1;
				}
			}else{
				if(dojo.isCopyKey(e)){
					if(this.anchor == this.current){
						delete this.selection[this.anchor.id];
						this._removeAnchor();
					}else{
						if(this.current.id in this.selection){
							this._removeItemClass(this.current, "Selected");
							delete this.selection[this.current.id];
						}else{
							if(this.anchor){
								this._removeItemClass(this.anchor, "Anchor");
								this._addItemClass(this.anchor, "Selected");
							}
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[this.current.id] = 1;
						}
					}
				}else{
					if(!(this.current.id in this.selection)){
						this.selectNone();
						this.anchor = this.current;
						this._addItemClass(this.current, "Anchor");
						this.selection[this.current.id] = 1;
					}
				}
			}
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(!this.simpleSelection){ return; }
		this.simpleSelection = false;
		this.selectNone();
		if(this.current){
			this.anchor = this.current;
			this._addItemClass(this.anchor, "Anchor");
			this.selection[this.current.id] = 1;
		}
	},
	onMouseMove: function(e){
		// summary
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		this.simpleSelection = false;
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.disconnect(this.onmousemoveEvent);
		delete this.onmousemoveEvent;
	},
	_removeSelection: function(){
		// summary:
		//		unselects all items
		var e = dojo.dnd._empty;
		for(var i in this.selection){
			if(i in e){ continue; }
			var node = dojo.byId(i);
			if(node){ this._removeItemClass(node, "Selected"); }
		}
		this.selection = {};
		return this;	// self
	},
	_removeAnchor: function(){
		if(this.anchor){
			this._removeItemClass(this.anchor, "Anchor");
			this.anchor = null;
		}
		return this;	// self
	}
});

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");


dojo.getObject("dnd", true, dojo);

dojo.dnd.getViewport = dojo.window.getBox;

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.window.getBox(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo.position(n, true);
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2),
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - t.x, ry = e.pageY - t.y, dx = 0, dy = 0;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += dojo.body().scrollLeft;
					ry += dojo.body().scrollTop;
				}
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
				}
				//console.log("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Avatar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Avatar"] = true;
dojo.provide("dojo.dnd.Avatar");



dojo.declare("dojo.dnd.Avatar", null, {
	// summary:
	//		Object that represents transferred DnD items visually
	// manager: Object
	//		a DnD manager object

	constructor: function(manager){
		this.manager = manager;
		this.construct();
	},

	// methods
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need
		this.isA11y = dojo.hasClass(dojo.body(),"dijit_a11y");
		var a = dojo.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = dojo.create("tbody", null, a),
			tr = dojo.create("tr", null, b),
			td = dojo.create("td", null, tr),
			icon = this.isA11y ? dojo.create("span", {
						id : "a11yIcon",
						innerHTML : this.manager.copy ? '+' : "<"
					}, td) : null,
			span = dojo.create("span", {
				innerHTML: source.generateText ? this._generateText() : ""
			}, td),
			k = Math.min(5, this.manager.nodes.length), i = 0;
		// we have to set the opacity on IE only after the node is live
		dojo.attr(tr, {
			"class": "dojoDndAvatarHeader",
			style: {opacity: 0.9}
		});
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = dojo.create("table"),
						tbody = dojo.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = dojo.create("tr", null, b);
			td = dojo.create("td", null, tr);
			td.appendChild(node);
			dojo.attr(tr, {
				"class": "dojoDndAvatarItem",
				style: {opacity: (9 - i) / 10}
			});
		}
		this.node = a;
	},
	destroy: function(){
		// summary:
		//		destructor for the avatar; called to remove all references so it can be garbage-collected
		dojo.destroy(this.node);
		this.node = false;
	},
	update: function(){
		// summary:
		//		updates the avatar to reflect the current DnD state
		dojo[(this.manager.canDropFlag ? "add" : "remove") + "Class"](this.node, "dojoDndAvatarCanDrop");
		if (this.isA11y){
			var icon = dojo.byId("a11yIcon");
			var text = '+';   // assume canDrop && copy
			if (this.manager.canDropFlag && !this.manager.copy) {
				text = '< '; // canDrop && move
			}else if (!this.manager.canDropFlag && !this.manager.copy) {
				text = "o"; //!canDrop && move
			}else if(!this.manager.canDropFlag){
				text = 'x';  // !canDrop && copy
			}
			icon.innerHTML=text;
		}
		// replace text
		dojo.query(("tr.dojoDndAvatarHeader td span" +(this.isA11y ? " span" : "")), this.node).forEach(
			function(node){
				node.innerHTML = this._generateText();
			}, this);
	},
	_generateText: function(){
		// summary: generates a proper text to reflect copying or moving of items
		return this.manager.nodes.length.toString();
	}
});

}

if(!dojo._hasResource["dojo.dnd.Manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Manager"] = true;
dojo.provide("dojo.dnd.Manager");





dojo.declare("dojo.dnd.Manager", null, {
	// summary:
	//		the manager of DnD operations (usually a singleton)
	constructor: function(){
		this.avatar  = null;
		this.source = null;
		this.nodes = [];
		this.copy  = true;
		this.target = null;
		this.canDropFlag = false;
		this.events = [];
	},

	// avatar's offset from the mouse
	OFFSET_X: 16,
	OFFSET_Y: 16,
	
	// methods
	overSource: function(source){
		// summary:
		//		called when a source detected a mouse-over condition
		// source: Object
		//		the reporter
		if(this.avatar){
			this.target = (source && source.targetState != "Disabled") ? source : null;
			this.canDropFlag = Boolean(this.target);
			this.avatar.update();
		}
		dojo.publish("/dnd/source/over", [source]);
	},
	outSource: function(source){
		// summary:
		//		called when a source detected a mouse-out condition
		// source: Object
		//		the reporter
		if(this.avatar){
			if(this.target == source){
				this.target = null;
				this.canDropFlag = false;
				this.avatar.update();
				dojo.publish("/dnd/source/over", [null]);
			}
		}else{
			dojo.publish("/dnd/source/over", [null]);
		}
	},
	startDrag: function(source, nodes, copy){
		// summary:
		//		called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		this.source = source;
		this.nodes  = nodes;
		this.copy   = Boolean(copy); // normalizing to true boolean
		this.avatar = this.makeAvatar();
		dojo.body().appendChild(this.avatar.node);
		dojo.publish("/dnd/start", [source, nodes, this.copy]);
		this.events = [
			dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove"),
			dojo.connect(dojo.doc, "onmouseup",   this, "onMouseUp"),
			dojo.connect(dojo.doc, "onkeydown",   this, "onKeyDown"),
			dojo.connect(dojo.doc, "onkeyup",     this, "onKeyUp"),
			// cancel text selection and text dragging
			dojo.connect(dojo.doc, "ondragstart",   dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent)
		];
		var c = "dojoDnd" + (copy ? "Copy" : "Move");
		dojo.addClass(dojo.body(), c);
	},
	canDrop: function(flag){
		// summary:
		//		called to notify if the current target can accept items
		var canDropFlag = Boolean(this.target && flag);
		if(this.canDropFlag != canDropFlag){
			this.canDropFlag = canDropFlag;
			this.avatar.update();
		}
	},
	stopDrag: function(){
		// summary:
		//		stop the DnD in progress
		dojo.removeClass(dojo.body(), ["dojoDndCopy", "dojoDndMove"]);
		dojo.forEach(this.events, dojo.disconnect);
		this.events = [];
		this.avatar.destroy();
		this.avatar = null;
		this.source = this.target = null;
		this.nodes = [];
	},
	makeAvatar: function(){
		// summary:
		//		makes the avatar; it is separate to be overwritten dynamically, if needed
		return new dojo.dnd.Avatar(this);
	},
	updateAvatar: function(){
		// summary:
		//		updates the avatar; it is separate to be overwritten dynamically, if needed
		this.avatar.update();
	},
	
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		var a = this.avatar;
		if(a){
			dojo.dnd.autoScrollNodes(e);
			//dojo.dnd.autoScroll(e);
			var s = a.node.style;
			s.left = (e.pageX + this.OFFSET_X) + "px";
			s.top  = (e.pageY + this.OFFSET_Y) + "px";
			var copy = Boolean(this.source.copyState(dojo.isCopyKey(e)));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.avatar){
			if(this.target && this.canDropFlag){
				var copy = Boolean(this.source.copyState(dojo.isCopyKey(e))),
				params = [this.source, this.nodes, copy, this.target, e];
				dojo.publish("/dnd/drop/before", params);
				dojo.publish("/dnd/drop", params);
			}else{
				dojo.publish("/dnd/cancel");
			}
			this.stopDrag();
		}
	},
	
	// keyboard event processors
	onKeyDown: function(e){
		// summary:
		//		event processor for onkeydown:
		//		watching for CTRL for copy/move status, watching for ESCAPE to cancel the drag
		// e: Event
		//		keyboard event
		if(this.avatar){
			switch(e.keyCode){
				case dojo.keys.CTRL:
					var copy = Boolean(this.source.copyState(true));
					if(this.copy != copy){
						this._setCopyStatus(copy);
					}
					break;
				case dojo.keys.ESCAPE:
					dojo.publish("/dnd/cancel");
					this.stopDrag();
					break;
			}
		}
	},
	onKeyUp: function(e){
		// summary:
		//		event processor for onkeyup, watching for CTRL for copy/move status
		// e: Event
		//		keyboard event
		if(this.avatar && e.keyCode == dojo.keys.CTRL){
			var copy = Boolean(this.source.copyState(false));
			if(this.copy != copy){
				this._setCopyStatus(copy);
			}
		}
	},
	
	// utilities
	_setCopyStatus: function(copy){
		// summary:
		//		changes the copy status
		// copy: Boolean
		//		the copy status
		this.copy = copy;
		this.source._markDndStatus(this.copy);
		this.updateAvatar();
		dojo.replaceClass(dojo.body(),
			"dojoDnd" + (this.copy ? "Copy" : "Move"),
			"dojoDnd" + (this.copy ? "Move" : "Copy"));
	}
});

// dojo.dnd._manager:
//		The manager singleton variable. Can be overwritten if needed.
dojo.dnd._manager = null;

dojo.dnd.manager = function(){
	// summary:
	//		Returns the current DnD manager.  Creates one if it is not created yet.
	if(!dojo.dnd._manager){
		dojo.dnd._manager = new dojo.dnd.Manager();
	}
	return dojo.dnd._manager;	// Object
};

}

if(!dojo._hasResource["dojo.dnd.Source"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Source"] = true;
dojo.provide("dojo.dnd.Source");




/*
	Container property:
		"Horizontal"- if this is the horizontal container
	Source states:
		""			- normal state
		"Moved"		- this source is being moved
		"Copied"	- this source is being copied
	Target states:
		""			- normal state
		"Disabled"	- the target cannot accept an avatar
	Target anchor state:
		""			- item is not selected
		"Before"	- insert point is before the anchor
		"After"		- insert point is after the anchor
*/

/*=====
dojo.dnd.__SourceArgs = function(){
	//	summary:
	//		a dict of parameters for DnD Source configuration. Note that any
	//		property on Source elements may be configured, but this is the
	//		short-list
	//	isSource: Boolean?
	//		can be used as a DnD source. Defaults to true.
	//	accept: Array?
	//		list of accepted types (text strings) for a target; defaults to
	//		["text"]
	//	autoSync: Boolean
	//		if true refreshes the node list on every operation; false by default
	//	copyOnly: Boolean?
	//		copy items, if true, use a state of Ctrl key otherwise,
	//		see selfCopy and selfAccept for more details
	//	delay: Number
	//		the move delay in pixels before detecting a drag; 0 by default
	//	horizontal: Boolean?
	//		a horizontal container, if true, vertical otherwise or when omitted
	//	selfCopy: Boolean?
	//		copy items by default when dropping on itself,
	//		false by default, works only if copyOnly is true
	//	selfAccept: Boolean?
	//		accept its own items when copyOnly is true,
	//		true by default, works only if copyOnly is true
	//	withHandles: Boolean?
	//		allows dragging only by handles, false by default
	//  generateText: Boolean?
	//		generate text node for drag and drop, true by default
	this.isSource = isSource;
	this.accept = accept;
	this.autoSync = autoSync;
	this.copyOnly = copyOnly;
	this.delay = delay;
	this.horizontal = horizontal;
	this.selfCopy = selfCopy;
	this.selfAccept = selfAccept;
	this.withHandles = withHandles;
	this.generateText = true;
}
=====*/

dojo.declare("dojo.dnd.Source", dojo.dnd.Selector, {
	// summary:
	//		a Source object, which can be used as a DnD source, or a DnD target
	
	// object attributes (for markup)
	isSource: true,
	horizontal: false,
	copyOnly: false,
	selfCopy: false,
	selfAccept: true,
	skipForm: false,
	withHandles: false,
	autoSync: false,
	delay: 0, // pixels
	accept: ["text"],
	generateText: true,
	
	constructor: function(/*DOMNode|String*/node, /*dojo.dnd.__SourceArgs?*/params){
		// summary:
		//		a constructor of the Source
		// node:
		//		node or node's id to build the source on
		// params:
		//		any property of this class may be configured via the params
		//		object which is mixed-in to the `dojo.dnd.Source` instance
		dojo.mixin(this, dojo.mixin({}, params));
		var type = this.accept;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}
		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
		this._lastX = 0;
		this._lastY = 0;
		// states
		this.sourceState  = "";
		if(this.isSource){
			dojo.addClass(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			dojo.addClass(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			dojo.addClass(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			dojo.subscribe("/dnd/source/over", this, "onDndSourceOver"),
			dojo.subscribe("/dnd/start",  this, "onDndStart"),
			dojo.subscribe("/dnd/drop",   this, "onDndDrop"),
			dojo.subscribe("/dnd/cancel", this, "onDndCancel")
		];
	},
	
	// methods
	checkAcceptance: function(source, nodes){
		// summary:
		//		checks if the target can accept nodes from this source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		if(this == source){
			return !this.copyOnly || this.selfAccept;
		}
		for(var i = 0; i < nodes.length; ++i){
			var type = source.getItem(nodes[i].id).type;
			// type instanceof Array
			var flag = false;
			for(var j = 0; j < type.length; ++j){
				if(type[j] in this.accept){
					flag = true;
					break;
				}
			}
			if(!flag){
				return false;	// Boolean
			}
		}
		return true;	// Boolean
	},
	copyState: function(keyPressed, self){
		// summary:
		//		Returns true if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean
		//		the "copy" key was pressed
		// self: Boolean?
		//		optional flag that means that we are about to drop on itself
		
		if(keyPressed){ return true; }
		if(arguments.length < 2){
			self = this == dojo.dnd.manager().target;
		}
		if(self){
			if(this.copyOnly){
				return this.selfCopy;
			}
		}else{
			return this.copyOnly;
		}
		return false;	// Boolean
	},
	destroy: function(){
		// summary:
		//		prepares the object to be garbage-collected
		dojo.dnd.Source.superclass.destroy.call(this);
		dojo.forEach(this.topics, dojo.unsubscribe);
		this.targetAnchor = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Source(node, params);
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = dojo.dnd.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					before = (e.pageX - this.targetBox.x) < (this.targetBox.w / 2);
				}else{
					before = (e.pageY - this.targetBox.y) < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if(!this.mouseDown && this._legalMouseDown(e) && (!this.skipForm || !dojo.dnd.isFormElement(e))){
			this.mouseDown = true;
			this._lastX = e.pageX;
			this._lastY = e.pageY;
			dojo.dnd.Source.superclass.onMouseDown.call(this, e);
		}
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			dojo.dnd.Source.superclass.onMouseUp.call(this, e);
		}
	},
	
	// topic event processors
	onDndSourceOver: function(source){
		// summary:
		//		topic event processor for /dnd/source/over, called when detected a current source
		// source: Object
		//		the source which has the mouse over it
		if(this != source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = dojo.dnd.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary:
		//		topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		if(this.autoSync){ this.sync(); }
		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.accept && this.checkAcceptance(source, nodes);
		this._changeState("Target", accepted ? "" : "Disabled");
		if(this == source){
			dojo.dnd.manager().overSource(this);
		}
		this.isDragging = true;
	},
	onDndDrop: function(source, nodes, copy, target){
		// summary:
		//		topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		// target: Object
		//		the target which accepts items
		if(this == target){
			// this one is for us => move nodes!
			this.onDrop(source, nodes, copy);
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary:
		//		topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},
	
	// local events
	onDrop: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		if(this != source){
			this.onDropExternal(source, nodes, copy);
		}else{
			this.onDropInternal(nodes, copy);
		}
	},
	onDropExternal: function(source, nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from an external source
		// source: Object
		//		the source which provides items
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		var oldCreator = this._normalizedCreator;
		// transferring nodes from the source to the target
		if(this.creator){
			// use defined creator
			this._normalizedCreator = function(node, hint){
				return oldCreator.call(this, source.getItem(node.id).data, hint);
			};
		}else{
			// we have no creator defined => move/clone nodes
			if(copy){
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}else{
				// move nodes
				this._normalizedCreator = function(node, hint){
					var t = source.getItem(node.id);
					source.delItem(node.id);
					return {node: node, data: t.data, type: t.type};
				};
			}
		}
		this.selectNone();
		if(!copy && !this.creator){
			source.selectNone();
		}
		this.insertNodes(true, nodes, this.before, this.current);
		if(!copy && this.creator){
			source.deleteSelectedNodes();
		}
		this._normalizedCreator = oldCreator;
	},
	onDropInternal: function(nodes, copy){
		// summary:
		//		called only on the current target, when drop is performed
		//		from the same target/source
		// nodes: Array
		//		the list of transferred items
		// copy: Boolean
		//		copy items, if true, move items otherwise
		
		var oldCreator = this._normalizedCreator;
		// transferring nodes within the single source
		if(this.current && this.current.id in this.selection){
			// do nothing
			return;
		}
		if(copy){
			if(this.creator){
				// create new copies of data items
				this._normalizedCreator = function(node, hint){
					return oldCreator.call(this, this.getItem(node.id).data, hint);
				};
			}else{
				// clone nodes
				this._normalizedCreator = function(node, hint){
					var t = this.getItem(node.id);
					var n = node.cloneNode(true);
					n.id = dojo.dnd.getUniqueId();
					return {node: n, data: t.data, type: t.type};
				};
			}
		}else{
			// move nodes
			if(!this.current){
				// do nothing
				return;
			}
			this._normalizedCreator = function(node, hint){
				var t = this.getItem(node.id);
				return {node: node, data: t.data, type: t.type};
			};
		}
		this._removeSelection();
		this.insertNodes(true, nodes, this.before, this.current);
		this._normalizedCreator = oldCreator;
	},
	onDraggingOver: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged over this target, and it is not disabled
	},
	onDraggingOut: function(){
		// summary:
		//		called during the active DnD operation, when items
		//		are dragged away from this target, and it is not disabled
	},
	
	// utilities
	onOverEvent: function(){
		// summary:
		//		this function is called once, when mouse is over our container
		dojo.dnd.Source.superclass.onOverEvent.call(this);
		dojo.dnd.manager().overSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOver();
		}
	},
	onOutEvent: function(){
		// summary:
		//		this function is called once, when mouse is out of our container
		dojo.dnd.Source.superclass.onOutEvent.call(this);
		dojo.dnd.manager().outSource(this);
		if(this.isDragging && this.targetState != "Disabled"){
			this.onDraggingOut();
		}
	},
	_markTargetAnchor: function(before){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary:
		//		removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary:
		//		changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	},
	_legalMouseDown: function(e){
		// summary:
		//		checks if user clicked on "approved" items
		// e: Event
		//		mouse event
		
		// accept only the left mouse button
		if(!dojo.mouseButtons.isLeft(e)){ return false; }
		
		if(!this.withHandles){ return true; }
		
		// check for handles
		for(var node = e.target; node && node !== this.node; node = node.parentNode){
			if(dojo.hasClass(node, "dojoDndHandle")){ return true; }
			if(dojo.hasClass(node, "dojoDndItem") || dojo.hasClass(node, "dojoDndIgnore")){ break; }
		}
		return false;	// Boolean
	}
});

dojo.declare("dojo.dnd.Target", dojo.dnd.Source, {
	// summary: a Target object, which can be used as a DnD target
	
	constructor: function(node, params){
		// summary:
		//		a constructor of the Target --- see the `dojo.dnd.Source.constructor` for details
		this.isSource = false;
		dojo.removeClass(this.node, "dojoDndSource");
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.Target(node, params);
	}
});

dojo.declare("dojo.dnd.AutoSource", dojo.dnd.Source, {
	// summary:
	//		a source that syncs its DnD nodes by default
	
	constructor: function(node, params){
		// summary:
		//		constructor of the AutoSource --- see the Source constructor for details
		this.autoSync = true;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dojo.dnd.AutoSource(node, params);
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.DashboardOrganizer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.DashboardOrganizer"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.DashboardOrganizer");











	// DashboardOrganizer is a dropdown list that contains the names of all dashboards, saved searches and scoreboards
	// that are exposed to the current user.  The user can drag and drop the dashbard names to reorder the way the dashboards will be displayed on the tab bar.
	// A separator visually distinguishes the displayed dashboards (above the separator) from the hidden dashboards (below).

	// The user can drag dashboard names both within a section and across sections (from hidden to visible and vice versa).  
	// The results are saved on a per-user basis as a user preference - the preference is saved when the dropdown is closed *if* changes were made.
	// When the dropdown is closed and there were changes made then the tabs on the Portal tab bar are reordered to reflect the order selected by the user.
	

	
dojo.declare(
	"com.ibm.bpm.social.portal.DashboardOrganizer",
	[com.ibm.bpm.social.widgets.common._RestWidget, com.ibm.bpm.social.widgets.SmartList, dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "DashboardOrganizer",
	//MAX_DISPLAYED_DASHBOARDS: 3,
	LOG_LEVEL: 1,
	DASHB0ARD_SEPARATOR_ID: "dashboardSeparatorItem",
	dashboardOrganizerEventConnectors : [],
    widgetsToCleanUp : [],
	visibleTabsText: "",
	dragToReorderText: "",
	openOnCreate: false,
	dashboardOrganizerSnapshotIconDiv_id : "_id_dashboardOrganizerSnapshotIconDiv_",
	dashboardOrganizerSnapshotIcon_id : "_id_dashboardOrganizerSnapshotIcon_",
	dashboardOrganizerSnapshotIcon_avatar_id : "_id_dashboardOrganizerSnapshotIcon_avatar_",
	snapshotContainer_id : "_id_snapshotContainer_",
	snapshotElement_innerDiv_id : "_id_snapshotElement_innerDiv_",
	snapshotElementDate_id : "_id_snapshotElementDate_",
	snapshotElement_id : "_id_snapshotElement_",
	processIcon_id : "_id_processIcon_",
	innerLi_id : "_id_innerLi_",
	currentlySelectedTab : null,
	prefsDirty  : false,
	isProcessServer : false,
	keyboardMoveOperationActive: false,
	originalLocationOfItemBeingMoved: {},  // this variable should be used only if keyboardMoveOperationActive==true, otherwise its value is unpredictable.
											// also, when I use dojo.move() I intentionally don't use "first" and "last" but always use "before" and "after".
	
	showEmptyMessage: false,  // turn off display of message by parent class Smartlist
	
	templateString:"<div dojoAttachPoint=\"containerDiv\" style=\"display:none;\" class=\"bpm-social-dashboardOrganizer\">\n\t<div class=\"topArrow\" dojoAttachPoint=\"topArrow\" ></div>\n\t<!-- headers; inside ul so it's easier to style them like the separator header in the actual list, below -->\n\t<div dojoAttachPoint=\"scrollingDiv\" style=\"height:300px; margin-top:8px; overflow-y:auto;\">\n\t\t<ul class=\"containerTopHeaders\">\n\t\t\t<li>\n\t\t\t\t<div role=\"heading\" dojoAttachPoint=\"topHeadersDiv\">\n\t\t\t\t\t<div  id=\"visibleTabsHeader_id\" class=\"visibleTabsHeader\" dojoAttachPoint=\"displayedDashboardsHeader\">${visibleTabsText}</div>\n\t\t\t\t\t<div  id=\"dragToReorderHeader_id\" class=\"dragToReorderHeader\"  dojoAttachPoint=\"displayedDashboardsHeader_2\">${dragToReorderText}</div>\n\t\t\t\t\t<div class=\"dashboardSeparatorVisible\" dojoAttachPoint=\"dashboardSeparator\"></div>\n\t\t\t\t</div>\n\t\t\t</li>\n\t\t</ul>\n\t\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t\t<ul dojoAttachPoint=\"listElement\" class=\"dashboardItem-list container\"><li style=\"display:none\">&nbsp;</li></ul>\n\t</div>\n</div>\n",
	constructor: function constructor_DashboardOrganizer(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.visibleTabsText = this.messages["dashboardOrganizer-visibleTabs"];
		this.dragToReorderText = this.messages["dashboardOrganizer-dragToReorder"];
		this.saveDojoDndManager = dojo.dnd._manager;
		this.localDndManager = new com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndManager();
	},
	
    postCreate: function postCreate_DashboardOrganizer() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		// Read dashboard order prefs from user profile
		this._readFromUserProfile();

		this.systemRestHandler = com.ibm.bpm.social.util.common.SystemRestHandler.getInstance();
		// get exposed items
		this._getExposedItems();
		this.dashboardOrganizerSubscriptions=[];
		this.dashboardOrganizerSubscriptions.push(dojo.subscribe("dashboardOrganizer_hideSnapshotContainer", dojo.hitch(this, function(data) {
			this._closeSnapshotItems();
		})));
		if (!this.dojoConnections) {
			this.dojoConnections=[];
		}
		// make the top labels tabbable
		dojo.attr(this.topHeadersDiv, 'tabindex', 0);

		if (this.openOnCreate) {
			this.show();
		} else {
			this.hide(false);  // false=don't save dashboard order setting.
		}

		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_getSavedSearches: function _getSavedSearches_DashboardOrganizer(exposedItemsArray){
		var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var taskQueuesList = taskQueues.getCustomQueues();
		// create object in the same structure as exposed items from the exposed items REST sevice 
		for (var t=0; t<taskQueuesList.length; t++) {
			var tq = taskQueuesList[t];
			var item = new Object();
			item["type"]="queue";
			item["subtype"]="task";
			item["name"]=tq.name;
			item["itemID"]="TQ."+tq.name;
			item["ID"]=tq.savedSearchID;
			item["display"]=tq.displayName;
			item["snapshotID"]="";
			item["tip"]=false;  // important: must be false or Exposed Items List will mistakenly believe the env is Process Server
			exposedItemsArray.push(item);
		}	
        var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
        var processQueuesList = processQueues.getCustomQueues();
		// create object in the same structure as exposed items from the exposed items REST sevice 
		for (var p=0; p<processQueuesList.length; p++) {
			var pq = processQueuesList[p];
			var item = new Object();
			item["type"]="queue";
			item["subtype"]="process";
			item["name"]=pq.name;
			item["itemID"]="PQ."+pq.name;
			item["ID"]=pq.savedSearchID;
			item["display"]=pq.displayName;
			item["snapshotID"]="";
			item["tip"]=false;  // important: must be false or Exposed Items List will mistakenly believe the env is Process Server
			exposedItemsArray.push(item);
		}	
	},
	
	_getExposedItems: function _getExposedItems_DashboardOrganizer(){
		var FUNC_NAME = "_getExposedItems";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		// add saved search information to our list of exposed items
		this._getSavedSearches(exposedItemsArray);
		// this.orderedDashboardItems is filtered for the exposed dashboards and 
		//   the data has been manipulated (disambiguated names, snapshot grouping); it contains the dashboards in user preference order.  It is used
		//   to create the drag and drop node.
		if (!this.orderedDashboardItems) {
		    this.orderedDashboardItems = [];
	   		var itemListWidget = new com.ibm.bpm.social.widgets.ExposedItemsList({trackSnapshotsOnIPad:true});
			// itemListWidget.getData() takes all exposed items and returns only dashboard, scoreboard and 'queue' items, 
			//    with disambiguated names and snapshot groupings.
			// com.ibm.bpm.social.portal.dashboardItems is global - it is also used in PageHandler.
			com.ibm.bpm.social.portal.dashboardItems = itemListWidget.getData(exposedItemsArray,["dashboard","scoreboard","queue"]);
			
			// we'll need to know if this is Process Server
			this.isProcessServer = itemListWidget.isProcessServer();
			// sort the exposed items based on the user prefs
			var sortedDashboards=this._sortExposedItemsUsingUserPrefs(com.ibm.bpm.social.portal.dashboardItems, this._storeddashboardOrganizerSettings.dashboardOrderObj);
			// refresh  com.ibm.bpm.social.portal.dashboardItems
			com.ibm.bpm.social.portal.dashboardItems = sortedDashboards;
			
			//add new DOM elements using the given data
			dojo.forEach(sortedDashboards, dojo.hitch(this, function(item){
			  var newItem={};
			  if (item.baseID == "HIDDEN") {  // the HIDDEN separator will have been created in _sortExposedItemsUsingUserPrefs()
				// add menu separator
				newItem["name"]="";
				newItem["type"]=[];
				newItem["type"].push("separator");
				newItem["type"].push("uniqueSeparatorID");
				newItem["baseID"]="uniqueSeparatorID";
				newItem["params"]={"dndListItemType":"separator"};
			  } else {
				  newItem["name"]=item.title;  // this name has been disambiguated
				  newItem["originalTitle"]=item.params.originalTitle;  // this is the name of the dashboard directly from REST; used to compare with user prefs if necessary
				  newItem["type"]=[];
				  newItem["type"].push("dashboardItem");
				  newItem["baseID"]=item.params.baseID;
				  item.params["dndListItemType"]="dashboardItem";
				  newItem["params"]=item.params;
				  
				  // Identify what icon to show in the list.  These are the icon types:
				  // 1. Work - this is an out-of-the-box dashboard
				  // 2. Team Performance - this is an out-of-the-box dashboard
				  // 3. Process Performance - this is an out-of-the-box dashboard
				  // 4. Adhoc Reports- this is an out-of-the-box dashboard
				  // 5. Custom Dashboard (any dashboard other than the 3 above)
				  // 6. Scoreboards 
				  // 7. Saved Search (also known as 'queue')
				  
				  // First deal with the out-of-the-box dashboards; for now we can identify them only by name
				  if (newItem.params.preciseItemType && newItem.params.preciseItemType=="dashboard") {
					if (newItem["originalTitle"] == "Team Performance") {
						newItem.params.iconNamePrefix = "Header_team_perf";
					} else if (newItem["originalTitle"] == "Process Performance") {
						newItem.params.iconNamePrefix = "Header_process_perf";
					} else if (newItem["originalTitle"] == "BPM_WORK") {
						newItem.params.iconNamePrefix = "Header_work";
						newItem.params.isWorkTab=true;
						// the page handler will need to know the work tab title and other data.
						com.ibm.bpm.social.portal.PageHandler.getInstance().setWorkTabInfo(item);
					} else if (newItem["originalTitle"] == "Ad-Hoc Reports") {
						newItem.params.iconNamePrefix = "Header_adhoc_report";
					} else {
						newItem.params.iconNamePrefix = "Header_custom_dashboard";
					}
				  } else if (newItem.params.preciseItemType && newItem.params.preciseItemType=="scoreboard") {
						newItem.params.iconNamePrefix = "Header_scoreboard";					
				  } else if (newItem.params.preciseItemType && newItem.params.preciseItemType=="queue") {
						newItem.params.iconNamePrefix = "Header_saved_search";					
				  }
			  }
			  this.orderedDashboardItems.push(newItem);
			}));
			 
			// this.dashboardsList is the dnd object
			this.dashboardsList = this._makeDndNode(this.listElement, this.orderedDashboardItems, false);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_sortExposedItemsUsingUserPrefs: function _mergeExposedItemsAndUserPrefs_DashboardOrganizer(exposedDashboards, userPrefs){
		// Will return an array containing exposed items in the order they appear in user prefs;  a separator will separate the displayed and hidden dashboards.
		//    Find match for pref item within the exposed items:
		//         A. Compare userPrefs.baseIDSeg with the baseID in exposed dashboard or saved search item (on Process Center this is the only comparison needed).
		//             Note that if this is a saved search item then baseID is the item's savedSearchID.
		//         B. If there is no match for the baseIDSeg then we need to do another check if the env is Process Server:
		//            C. Compare the last segment of itemID in exposed dashboard item with userPrefs.itemIDSeg 
		//           	 If the itemIDs match, we accept the exposed dash as a match for the pref
		//    After loop, see if there are any more dashboards in exposed items that didn't get moved in first loop; move these into
		//    the return array at the top of the hidden group.
		//  
		
		retArray=[];
		hiddenSeparatorObject={"baseID":"HIDDEN", "title":"HIDDEN", "params":{"originalTitle":"HIDDEN"}};
		var hiddenIndex=-1;
		if (userPrefs && userPrefs.length>0 && exposedDashboards && exposedDashboards.length>0){
			dojo.forEach( userPrefs, dojo.hitch( this, function(prefItem){
				// try to find match in exposedDashboards
				var itemIndex=-1;
				for (var i=0; i<exposedDashboards.length; i++) {
					if (exposedDashboards[i].params && exposedDashboards[i].params.baseID) { 
						var baseID = exposedDashboards[i].params.baseID;
						if (prefItem.baseIDSeg == baseID) {
							itemIndex=i;
							break;
						} else {
							//   1. If a user installs a new snapshot of an app ON PROCESS SERVER then we won't find the *previous* newest snapshot.
							//   2. If a user sets a new snapshot to be the default version ON BOTH PC AND PS then we won't find the *previous* default version.
							//   So, we compare the pref item's itemID segment with the exposed dashboard's itemID segment,
							//      and if they match then we use it.
							//   *This check is never done for saved searches*
							if (exposedDashboards[i].params.itemID && exposedDashboards[i].params.itemID.length>0) {
								var itemIDArray = exposedDashboards[i].params.itemID.split('-');
								if (prefItem.itemIDSeg == itemIDArray[itemIDArray.length-1]) {
									itemIndex=i;
									break;
								}										
							}
						}
					}
				}
				// if match, move exposed item into retArray; 
				if (itemIndex>=0) {
					retArray.push(exposedDashboards[itemIndex]);
					exposedDashboards.splice(itemIndex,1);

				} else if (prefItem.baseIDSeg && prefItem.baseIDSeg == "HIDDEN") {
					retArray.push(hiddenSeparatorObject);	
					hiddenIndex=retArray.length-1;
				}
			}));
		}
		if (exposedDashboards && exposedDashboards.length>0) {
			// there are exposedDashboards that need to be moved to retArray
			// if there's no separator, just add dashboards to top of retArray.  Separator will be added below.
			for (var i=exposedDashboards.length-1; i>=0; i--){
				retArray.splice(hiddenIndex+1,0,exposedDashboards[i]);
			}
		}
		// if there's no separator add it at top
		if (hiddenIndex==-1) {
			retArray.splice(0,0,hiddenSeparatorObject);
			hiddenIndex=0;
		}
		// if there were no user prefs then put the 'default' dashboards above the separator.  These
		//   are Work, Team Performance and Process Performance
		
		if (!userPrefs || (userPrefs && userPrefs.length==0)) {
			var tArray = [];
			for (var i=0; i<retArray.length; i++){
				var tItem = retArray[i];

				if (tItem.params.originalTitle == "BPM_WORK") {
					tArray[0]=retArray.splice(i,1)[0];
					i--;
				}
				if (tItem.params.originalTitle == "Team Performance") {
						tArray[1]=retArray.splice(i,1)[0];
					i--;
				}
				if (tItem.params.originalTitle == "Process Performance") {
						tArray[2]=retArray.splice(i,1)[0];
					i--;
				}
			}
			
			for (var i=tArray.length-1; i>=0; i--) {
				if (tArray[i] && tArray[i].params.originalTitle) {
					retArray.splice(0,0,tArray[i]);
				}
			}
		}
		return retArray;
	},
	
	// creates a dojo/dnd/Source
	_makeDndNode: function _makeDndNode_DashboardOrganizer(node, data, selfAccept){
		// create the Source
		var dndObj = com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndSource(node, {	// CustomDndSource code is at the bottom of this file
			accept: [ "dashboardItem"],
			selfAccept: selfAccept === undefined ? true : selfAccept,
			copyOnly: false,
			creator: dojo.hitch(this, "_makeListItem")
		});

		// insert new nodes to the Source; the first argument indicates that they will not be highlighted (selected)
		// when inserted
		dndObj.insertNodes(false, data);

		return dndObj;
	},

	// this is called by the dnd framework  - it creates an item for the source it is attached to (see _makeDndNode)
	_makeListItem: function _makeListItem_DashboardOrganizer(item, hint){
		var type="";
		var node=null;
		if (hint != "avatar") {
			// addItem is in the parent class SmartList
			var params={};
			params['title']=item.name;
			params['iconClass'] = item.params.iconNamePrefix;
			
			var index = this.addItem(item.params, params);
			var li = dojo.query('li[smartListIndex="'+index+'"]', this.listElement);
			if(typeof(li)=='undefined' || li==null || typeof(li.length)=='undefined' || li.length==null || li.length<=0) {
				return null;
			}
			node = li[0];
		} else {
			// create avatar body
			// See if this item is the 'currently selected' item in the list; if it is then the text will need
			//   to be in bold
			var currentSelItem=false;
			if (item.params && item.params.baseID && this.currentlySelectedTab && (this.currentlySelectedTab == item.params.baseID)) {
				currentSelItem=true;
			}
			node=this._createAvatar(item.name, item.params, currentSelItem);
		}
		// set the returned type of the item
		type = item.type;
		return { node: node, data: item, type: type };
	},
	
	_createAvatar: function _createAvatar_DashboardOrganizer(itemName, params, currentSelItem) {
		var div = dojo.create("div", { 
				"class": "dojoDndItemOver ",
				"id": "dashboardOrganizerAvatar"
			});
		var selClas=" ";
		if (currentSelItem) {
			selClas = " listItemSelected ";
		}
        var innerDiv = dojo.create("div", {
          'class': "bpm-social-dashboardOrganizer-avatar "+selClas
        }, div);
		if (document.body.dir == 'rtl') {
		  this._createAvatarTextDiv(innerDiv, itemName, params, currentSelItem);
		  this._createAvatarIconDiv(innerDiv, itemName, params, currentSelItem);
		}
		else {				
		  this._createAvatarIconDiv(innerDiv, itemName, params, currentSelItem);
		  this._createAvatarTextDiv(innerDiv, itemName, params, currentSelItem);
		}

		return div;
	},

	_createAvatarIconDiv: function _createAvatarIconDiv_DashboardOrganizer(innerDiv, itemName, params, currentSelItem) {
        var icon = dojo.create("div", {
          'class': "bpm-social-dashboardOrganizer-avatar-process-icon " + params['iconNamePrefix'],
		  'id': this.processIcon_id+params.baseID,
          //'tabIndex': "0",  we don't want to tab here because it gets too busy
          //'aria-label': createParams['title'],  taken care of in text div
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, innerDiv);
	},
	_createAvatarTextDiv: function _createAvatarTextDiv_DashboardOrganizer(innerDiv, itemName, params, currentSelItem) {
		var textClas="bpm-social-dashboardOrganizer-avatar-text";
		if (currentSelItem) {
			textClas="bpm-social-dashboardOrganizer-avatar-text-selected";
		}
        var text = dojo.create("a", {
          'class': textClas,
          'aria-label': itemName
        }, innerDiv);
        var text2 = dojo.create("div", {
        }, text);
        text2.appendChild(document.createTextNode(itemName));	
        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
	},

      /* _createItem: Handles creating each item's HTML elements. params is provided when addItem is called in _makeListItem(). 
	      This function is called from addItem() in the parent class SmartList */
      _createItem: function _createItem_DashboardOrganizer(index, li, params, createParams){
		if (params && params.dndListItemType) {
			if (params.dndListItemType=="dashboardItem") {
				var innerLi = dojo.create("li", {
				  'class': "organizerInnerLi",
				  'id': this.innerLi_id+params.baseID
				}, li);
				if (document.body.dir == 'rtl') {
				  this._createItemNameElement(index, li, innerLi, params, createParams);
				  this._createItemIconElement(index, li, innerLi, params, createParams);
				}
				else {				
				  this._createItemIconElement(index, li, innerLi, params, createParams);
				  this._createItemNameElement(index, li, innerLi, params, createParams);
				}
			} else if (params.dndListItemType=="separator") {
				this._createSeparatorElement(index, li, params, createParams);
			}
		}
      },

      _createItemNameElement: function _createItemNameElement_DashboardOrganizer(index, li, innerLi, params, createParams){
        var title = "<i>Unknown</i>";
        if (createParams != null && typeof (createParams) != 'undefined') {
          title = createParams['title'];
        }
		if (dojo.isIE) {
			dojo.addClass(li, 'organizerOuterLi');
		}

        var text = dojo.create("a", {
          'class': "dashboardItem-text",
          //'aria-label': title //is reading text in attached div
        }, innerLi);
		text.tabIndex = 0;
        dijit.setWaiRole(text, "link");
        var text2 = dojo.create("div", {
          'class': "dashboardItem-title-div",
        }, text);
        //text2.tabIndex = 0;		
        text2.appendChild(document.createTextNode(title));			
        

        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
		
        // If there are snapshots for this item, add sections for them.
        if (params.snapshotItems && params.snapshotItems.length > 0) {
			  var menuIconDiv = dojo.create("div", {
				'class': "snapshotsContainerIconDiv",
				'border': "none",
				'id': this.dashboardOrganizerSnapshotIconDiv_id + params.baseID
			  }, innerLi);//attach to parent of text for proper aria role hierarchy
			  var menuIcon = dojo.create("img", {
				'class': "bpm-social-blue-twistie-down",
				'border': "none",
				'tabIndex': "0",
				'alt': "",
				'aria-label': this.messages.twistieJawsExpandText,
				'id': this.dashboardOrganizerSnapshotIcon_id + params.baseID
			  }, menuIconDiv);
			  var menuIconAltText = dojo.create("span", {
				'class': "hi-contrast-icon dropdown-text",
				'innerHTML': "v",
				'id': "_id_alttext_" + params.baseID

			  }, menuIconDiv);
			  menuIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
			  dijit.setWaiRole(menuIcon, "button");
			  
			  var snapshotsContainer = dojo.create("div", {
				'class': "snapshotsContainerHide",
				'id': this.snapshotContainer_id + params.baseID
			  }, li);
			  
			  var connectFunc = dojo.connect(menuIcon, 'onclick', this, function(evt) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);
			  
			  connectFunc = dojo.connect(menuIconDiv, 'onclick', this, function(evt) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);
			  
			  connectFunc = dojo.connect(menuIcon, 'keypress', this, function(evt){
				if (evt.keyCode == dojo.keys.ENTER || evt.keyCode==dojo.keys.RIGHT_ARROW || evt.keyCode==dojo.keys.LEFT_ARROW || evt.charCode==dojo.keys.SPACE ) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
				}
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);

			  for ( var i = 0; i < params.snapshotItems.length; i++) {
				this._addSnapshotElement(snapshotsContainer, params.snapshotItems[i].params["snapshotName"], params.snapshotItems[i], (i==0)/*first item*/);
			  }
        }
        this.dashboardOrganizerEventConnectors.push(dojo.connect(text, 'keypress', this, "_handleTextKeyPressEvent"));
        this.dashboardOrganizerEventConnectors.push(dojo.connect(text, 'onclick', this, "_handleTextClickEvent"));
      },
	  
      _createItemIconElement: function _createItemIconElement_DashboardOrganizer(index, li, innerLi, params, createParams){
        var icon = dojo.create("div", {
          'class': "process-icon " + createParams['iconClass'],
		  'id': this.processIcon_id+params.baseID,
          //'tabIndex': "0",  we don't want to tab here because it gets too busy
          //'aria-label': createParams['title'],  taken care of in text div
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, innerLi);
        //dijit.setWaiRole(icon, "button");
        this.dashboardOrganizerEventConnectors.push(dojo.connect(icon, 'keypress', this, "_handleTextKeyPressEvent"));
        this.dashboardOrganizerEventConnectors.push(dojo.connect(icon, 'onclick', this, "_handleTextClickEvent"));
      },
		
	 _hideShowSnapshotsContainer: function _hideShowSnapshotsContainer_DashboardOrganizer(evt, menuIcon, container, onlyHide) {
		// This function toggles the state of snapshot containers, hiding if open or opening if hidden.
		// param 'onlyHide' means hide open snapshot containers but don't open hidden ones.
		if (evt) {
			dojo.stopEvent(evt);
		}
		if (menuIcon && container) {
			if (dojo.hasClass(container, "snapshotsContainerHide") && !onlyHide) {
				dojo.removeClass(container, "snapshotsContainerHide");
				dojo.addClass(container, "snapshotsContainerShow");
				dojo.addClass(menuIcon, "twistie-selected");
				dojo.attr(menuIcon, 'aria-label', this.messages.twistieJawsCollapseText);
                if (container.firstChild) container.firstChild.focus();
			} else if (dojo.hasClass(container, "snapshotsContainerShow")) {
				dojo.removeClass(container, "snapshotsContainerShow");
				dojo.addClass(container, "snapshotsContainerHide");
				dojo.attr(menuIcon, 'aria-label', this.messages.twistieJawsExpandText);
				if (dojo.hasClass(menuIcon, "twistie-selected")) {
					dojo.removeClass(menuIcon, "twistie-selected");
				}
			}
		}
	 },
      _createSeparatorElement: function _createSeparatorElement_DashboardOrganizer(index, li, params, createParams){
		dojo.attr(li, "id", this.DASHB0ARD_SEPARATOR_ID);
		dojo.addClass(li, "indicatorClassForSeparator");
        var textDiv = dojo.create("div", {'class':"hiddenTabsHeader"}, li);      
        textDiv.appendChild(document.createTextNode(this.messages["dashboardOrganizer-hiddenTabs"]));			
        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(textDiv);
        var sepLine = dojo.create("div", {'class': "dashboardSeparator"}, li);      
		dojo.attr(textDiv, "tabindex", 0);
		dojo.attr(textDiv, "aria-label", this.messages["dashboardOrganizer-hiddenTabs"]);
		
	},

      _addSnapshotElement: function _addSnapshotElement_DashboardOrganizer(snapshotsContainer, display, params, firstItem){
        // params is the snapshot information
        if (display && params) {
			//var btd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(display);
			var innerDiv=dojo.create("div", {
				'id':this.snapshotElement_innerDiv_id+params.params.baseID,
				'tabindex':0,
                'role': "link"                
			}, snapshotsContainer);
			if (firstItem) {
				dojo.addClass(innerDiv, "firstSnapshotItem");
			}
			// keep an array of snapshot information for easy lookup
			// when a menu item is selected
			if (!this.snapshotElements) {
				this.snapshotElements = {};
			}			
			this.snapshotElements[this.snapshotElement_innerDiv_id+params.params.baseID] = {
				"display": display,
				"params": params
			};
			var connectFunc = dojo.connect(innerDiv, 'onclick', this, function(evt) {
				if (evt) {
					this._dashboardSnapshotItemClick(evt);
				}
			});
			this.dashboardOrganizerEventConnectors.push(connectFunc);
			connectFunc = dojo.connect(innerDiv, 'keypress', this, function(evt) {
				if (evt) {
					this._dashboardSnapshotKeypress(evt);
				}
			});
			this.dashboardOrganizerEventConnectors.push(connectFunc);

			var innerDiv2=dojo.create("div", {
				'class':"dashboardItem-snapshot-inner-div"
			}, innerDiv);
			var text = dojo.create("a", {
			  'id':this.snapshotElement_id+params.params.baseID,
			  'aria-label': display,
			  'class': "dashboardItem-snapshot-main-text"
			}, innerDiv2);
	        var text3 = dojo.create("div", {
        	  'class': "dashboardItem-snapshot-title-div",
        	}, text);
        	text3.appendChild(document.createTextNode(display));			
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text3);
			dijit.setWaiRole(text, "link");
			var innerDiv3=dojo.create("div", {
				'class':"dashboardItem-snapshot-inner-div-date"
			}, innerDiv);
			
			str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(params.params.snapshotCreateDate);
	        date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
			var snapDate = com.ibm.bpm.social.util.common.Util.formatDateObject(date, {formatLength: "short", selector: "date"});		
			
			var text2 = dojo.create("a", {
			  'innerHTML': this.messages["snapshotLastUpdated"]+snapDate,
			  'id':this.snapshotElementDate_id+params.params.baseID,
			  //'aria-label': display,
			  'class': "dashboardItem-snapshot-date-text"
			}, innerDiv3);
			dijit.setWaiRole(text2, "link");
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
        }
      },


      /**
       * Read the user profile setting "Portal Dashboard Display Order" and parse it into the members
       * 
       * @returns {com.ibm.bpm.social.util.common.DashboardOrganizerSettings}
       */
      _readFromUserProfile: function _readFromUserProfile_DashboardOrganizer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        var dashboardOrganizerSettings = new com.ibm.bpm.social.util.common.DashboardOrganizerSettings();
        var userProfileString;
        if (userProfile && userProfile.userPreferences &&
						userProfile.userPreferences[com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME]) {
		  
          userProfileString = userProfile.userPreferences[com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME];
		  if (!userProfileString) {
			userProfileString="";
		  }
          com.ibm.bpm.social.util.common.Log.log(2, "dashboard order preference string is: [" + userProfileString+"]");

          var newdashboardOrganizerSettings = dashboardOrganizerSettings.parseFromString(userProfileString, true);
          if (!newdashboardOrganizerSettings.isValid) {
            // String is invalid
            // the original dashboardOrganizerSettings object is unmodified and has default values
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is invalid, using default string: " +
              dashboardOrganizerSettings.getCurrentProfileString());
          }
          else {
            // the string is valid and the values are mixed into the original DashboardOrganizerSettings object
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is valid, using now string: " +
              dashboardOrganizerSettings.getCurrentProfileString());
          }
        }
        else {
          // No setting
          com.ibm.bpm.social.util.common.Log.log(2, "user attribute not set, using default string: " +
            dashboardOrganizerSettings.getCurrentProfileString());
        }


        if (com.ibm.bpm.social.util.common.Log.logLevel > 1) {
          com.ibm.bpm.social.util.common.Log.logObject(2, {
            DashboardOrderObj: dashboardOrganizerSettings.dashboardOrderObj,
			_currentProfileString: dashboardOrganizerSettings._currentProfileString
          }, "Got following settings from user profile: ");
        }

        // update the _storeddashboardOrganizerSettings to the newest settings, but use a separate copy
        this._storeddashboardOrganizerSettings = new com.ibm.bpm.social.util.common.DashboardOrganizerSettings(dashboardOrganizerSettings);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return dashboardOrganizerSettings;
      },


      /**
       * write the default start page settings into the user profile
       * 
       * @param {com.ibm.bpm.social.util.common.DashboardOrganizerSettings} DashboardOrganizerSettings if given, the settings to be
       *        written to the profile
       * 
       * @type {void}
       */
     _writeDashboardOrderToUserProfile: function _writeDashboardOrderToUserProfile_DashboardOrganizer(dashboardOrganizerSettings){
		com.ibm.bpm.social.util.common.Log.logEntry(2, "saving settings!");
        if (dashboardOrganizerSettings) {
          com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME,
            dashboardOrganizerSettings.getCurrentProfileString());
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
		this.prefsDirty = false;
        return;
      },
	  
	  
	  
     _dashboardSnapshotItemClick: function _dashboardSnapshotItemClick_DashboardOrganizer(evt){
		var itemData = this.snapshotElements[evt.currentTarget.id];
		this._onDashboardItemSelected(itemData.params.params);
     },

     _dashboardSnapshotKeypress: function _dashboardSnapshotKeypress_DashboardOrganizer(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
			var itemData = this.snapshotElements[evt.currentTarget.id];
			this._onDashboardItemSelected(itemData.params.params);
		}
     },

      /* _onDashboardTextClicked: Called when the text for an item is clicked. 
      	When clicked, display the dashboard. Same functionality 
      	as _onDashboardiconClicked. */
     _onDashboardTextClicked: function _onDashboardTextClicked_DashboardOrganizer(index, li, params){
		this._onDashboardItemSelected(params);
     },
	 
      /* _onDashboardTextSpaceBar: Called when the text for an item is spacebar'd.
      	This indicates that the item is being selected for dragging, or that the move operation should be completed.
		*/
     _onDashboardTextSpaceBar: function _onDashboardTextSpaceBar_DashboardOrganizer(index, li, params){
		var allItems=this._getOrderedItems(this.dashboardsList);
		// do nothing if this is the only item in the list
		this.originalLocationOfItemBeingMoved={};
		if (allItems && allItems.length>1) {
			if (this.keyboardMoveOperationActive==true) {
				// complete the operation
				this.keyboardMoveOperationActive=false;
				// close snapshot containers and untwist twistie if needed
				this._closeSnapshotItems();

				if (dojo.hasClass(li, "keyboardSelected")) {
					dojo.removeClass(li, "keyboardSelected");
				}
			} else {
				// We need to make it appear with the draggable style. 
				this.keyboardMoveOperationActive=true;
				this._closeSnapshotItems();
				if (!dojo.hasClass(li, "keyboardSelected")) {
					dojo.addClass(li, "keyboardSelected");
				}
				// Save the location of this item, so it can be replaced if move operation is cancelled
				var allItems=this._getOrderedItems(this.dashboardsList);
				// find where this item is in the list
				if (allItems && allItems.length>1) {
					for (var i=0; i<allItems.length; i++) {
						if (li.id && allItems[i].dojoUniqueId && (li.id == allItems[i].dojoUniqueId)) {
							var beforeOrAfter="";
							var toItem=null;
							if (i==allItems.length-1) {  // if it is the last item in the list, 
								beforeOrAfter="after";
								toItem=allItems[i-1].dojoUniqueId;
							} else {
								beforeOrAfter="before";
								toItem=allItems[i+1].dojoUniqueId;
							}
							this.originalLocationOfItemBeingMoved={"beforeOrAfter":beforeOrAfter, "toItem":toItem} ;
							break;
						}
					}
				}
			}
		}
     },
	 
      /* _onDashboardArrowMove: Called when an item is to be moved
      	*/
     _onDashboardArrowMove: function _onDashboardArrowMove_DashboardOrganizer(index, li, params, keyCode){
		if (this.keyboardMoveOperationActive==true) {
				var allItems=this._getOrderedItems(this.dashboardsList);
				// find where this item is in the list
				if (allItems && allItems.length>1) {
					var thisItemIndex=-1;
					for (var i=0; i<allItems.length; i++) {
						if (li.id && allItems[i].dojoUniqueId && (li.id == allItems[i].dojoUniqueId)) {
							thisItemIndex=i;
							break;
						}
					}
					if (thisItemIndex>=0) {
						var moveAccomplished=false;
						// which direction?
						if (keyCode==dojo.keys.UP_ARROW) {
							if (thisItemIndex>0) {
								var toItem=dojo.byId(allItems[thisItemIndex-1].dojoUniqueId);
								dojo.place(li, toItem, "before");
								moveAccomplished=true;
							}
						} else if (keyCode==dojo.keys.DOWN_ARROW) {
							if (thisItemIndex<allItems.length) {
								var toItem=dojo.byId(allItems[thisItemIndex+1].dojoUniqueId);
								dojo.place(li, toItem, "after");
								moveAccomplished=true;
							}
						}
						if (moveAccomplished) {
							this._focusOnTextElement(li);
						}
					}
				}
				/* var list = dojo.byId("list"),
					three = dojo.byId("three");
			 
				dojo.place(three, list, "first"); */
			
		}
     },
	 
      /* _onDashboardEscapeKey: 
	     If a move operation is not active, close the dropdown and save the dashboard order;
		 If a move operation is active, cancel the operation.
      	*/
     _onDashboardEscapeKey: function _onDashboardEscapeKey_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			// cancel the operation
			if (dojo.hasClass(li, "keyboardSelected")) {
				dojo.removeClass(li, "keyboardSelected");
			}
			this._resetDashboardOrder(index, li, params);
			this.keyboardMoveOperationActive=false;
			this.originalLocationOfItemBeingMoved= {};
		} else {
			this.hide(true);
		}
     },
	 
      /* _resetDashboardOrder: 
		 If a move operation is active, reset the order of dashboards to where it was before the move operation started.
      	*/
     _resetDashboardOrder: function _resetDashboardOrder_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			var allItems=this._getOrderedItems(this.dashboardsList);
			// find where this item is in the list
			if (allItems && allItems.length>1) { // this is just a sanity check
				var target=dojo.byId(this.originalLocationOfItemBeingMoved.toItem);
				if (target) {
					dojo.place(li, target, this.originalLocationOfItemBeingMoved.beforeOrAfter);
					this._focusOnTextElement(li);
				}
			}
		}
	},

	_focusOnTextElement: function _focusOnTextElement_DashboardOrganizer(li) {
		var a = dojo.query(".dashboardItem-text",li);
		var node=null;
		var doNotClose = this.doNotClose;
		if (a.length > 0) {
			this.doNotClose = true;
			node = a[0];
		} else {
			a = dojo.query(".dashboardItem-text-selected",li);
			if (a.length > 0) {
				this.doNotClose = true;
				node = a[0];
			}
		} 
		if (node!=null) {
			node.focus();
			this.doNotClose = doNotClose;
		}
	},
	
      /* _onDashboardTabKey: If a move operation is in progress, this
	     cancels it;
      	*/
     _onDashboardTabKey: function _onDashboardTabKey_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			this._onDashboardEscapeKey(index, li, params);
		}
     },

      _handleTextKeyPressEvent: function _handleTextKeyPressEvent_DashboardOrganizer(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user pressed on img, don't handle that here; it's handled in _hideShowSnapshotsContainer()
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
            if (info['index'] != null && info['li'] != null) {
			  if (evt.keyCode == dojo.keys.ENTER) {
				  // the item has been selected to be shown on the tab bar
				  this._onDashboardTextClicked(info['index'], info['li'], info['params']);
			  } else if (evt.charCode == dojo.keys.SPACE) {
				   // the item has been chosen to be moved to another location; if pressed while move
				   // operation is in progress then space completes the operation
				  this._onDashboardTextSpaceBar(info['index'], info['li'], info['params']);
			  } else if (evt.keyCode == dojo.keys.UP_ARROW || evt.keyCode == dojo.keys.DOWN_ARROW) {
				  // if a move operation is in progress, these keys will move the item up or down
				  this._onDashboardArrowMove(info['index'], info['li'], info['params'], evt.keyCode);
			  } else if (evt.keyCode == dojo.keys.ESCAPE) {
				  // if a move operation is in progress, escape will cancel it
				  this._onDashboardEscapeKey(info['index'], info['li'], info['params']);
			  } else if (evt.keyCode == dojo.keys.TAB) {
				  // if a move operation is in progress, tab will cancel it
				  this._onDashboardTabKey(info['index'], info['li'], info['params']);
			  }
		  }
        }
      },

      _handleTextClickEvent: function _handleTextClickEvent_DashboardOrganizer(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user clicked on img, don't handle that here
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
          if (info['index'] != null && info['li'] != null) {
            this._onDashboardTextClicked(info['index'], info['li'], info['params']);
          }
        }
      },

	 _onDashboardItemSelected: function _onDashboardItemSelected_DashboardOrganizer(params){
	    if (!com.ibm.bpm.social.portal.PageHandler.getInstance().isNavigationAllowed()) {
			this.hide(true);
	    	return;
	    }
		if (params.type == 'queue') {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
				dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
						savedSearchName: params.name
					}]);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
						dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
							savedSearchName: params.name
						}]);
					}
				}));
   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
		    }
		} else {
			var newparams={isWorkTab: params.isWorkTab?true:false};
			var pubObj={
				runURL: params.runURL,
				display: params.title,
				params: newparams
			};
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [pubObj]);
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(params)]);
		this.hide(true);
	},

      _cleanUpConnects: function _cleanUpConnects_DashboardOrganizer(){
        //disconnect all dojo event connectors
        if (this.dashboardOrganizerEventConnectors) {
          dojo.forEach(this.dashboardOrganizerEventConnectors, function(connector){
            dojo.disconnect(connector);
          });
        }
        this.dashboardOrganizerEventConnectors = [];
		for(var i=0; i<this.dashboardOrganizerSubscriptions.length; i++) {
			dojo.unsubscribe(this.dashboardOrganizerSubscriptions[i]);
		}
		this.dashboardOrganizerSubscriptions=[];
      },

      _cleanUpMenusAndWidgets: function _cleanUpMenusAndWidgets_DashboardOrganizer(){
        if (this.widgetsToCleanUp && this.widgetsToCleanUp.length > 0) {
          for ( var i = 0; i < this.widgetsToCleanUp.length; i++) {
            if (this.widgetsToCleanUp[i].destroyRecursive) {
              this.widgetsToCleanUp[i].destroyRecursive();
            }
            else if (this.widgetsToCleanUp[i].destroy) {
              this.widgetsToCleanUp[i].destroy();
            }
          }
        }
        this.widgetsToCleanUp = [];
        if (this.snapshotElements) {
          delete this.snapshotElements;
        }
        this._cleanUpConnects();
      },
	
	_onXhrErrors: function _onXhrErrors_DashboardOrganizer(data, xhrArgs){
		console.debug(["DashboardOrganizer experienced a data retrieval error.",
		   data,
		   xhrArgs]);
	},
	
	_updateDashboardOrder: function _updateDashboardOrder_DashboardOrganizer(displayedDashboards, hiddenDashboards){
		// displayed dashboards:
		var newObj=[];
		newObj = this._getDataForPrefs(displayedDashboards, newObj);
		newObj.push({"baseIDSeg":"HIDDEN","itemIDSeg":"HIDDEN"});
		this._getDataForPrefs(hiddenDashboards, newObj);
 
		// update the profile setting;  then compare the current with the new and persist the new only if it's different
		var saveCurrentStr = this._storeddashboardOrganizerSettings._currentProfileString;
		this._storeddashboardOrganizerSettings.dashboardOrderObj=newObj;
		this._storeddashboardOrganizerSettings.createProfileString();
		if (saveCurrentStr != this._storeddashboardOrganizerSettings._currentProfileString) {
			this.prefsDirty = true;
		}
	},
	_getDataForPrefs: function _getDataForPrefs_DashboardOrganizer(dataArray, newObj) {
		dojo.forEach(dataArray, dojo.hitch(this, function(item){
			var baseID = item.data.params.baseID;
			var dashItem={};
			if (baseID) {
				dashItem["baseIDSeg"]=baseID;
				dashItem["itemIDSeg"]="";
				if (item.data.params.type!="queue" && item.data.params.itemID && item.data.params.itemID.length>0 ) {
					var itemIDArray = item.data.params.itemID.split('-');
					if (itemIDArray && dojo.isArray(itemIDArray) && itemIDArray.length>0 ) {
						dashItem["itemIDSeg"]=itemIDArray[itemIDArray.length-1];
					}
				}
				newObj.push(dashItem);
			 } else {
				console.log("invalid exposed dashboard item, cannot save preference");
			 }
		 }));
		 return newObj;
	},
	
	show: function show_DashboardOrganizer(){
		if (this.dashboardsList) {
		  dojo.style(this.containerDiv, "display", "block");
		}
		
		// Add vertical scrolling if needed, and adjust widths as needed
		var processPortalFooter = dojo.byId("processPortalFooter");
		var footerHeight=0;
		if (processPortalFooter && processPortalFooter.offsetHeight) {
			footerHeight = processPortalFooter.offsetHeight;
		}
		var bannerArea = dojo.query('.processPortalBanner')[0];
		var bannerHeight=0;
		if (bannerArea && bannerArea.offsetHeight) {
			bannerHeight = bannerArea.offsetHeight;
		}
		var vs = dojo.window.getBox();
		var availableHeight=0;
		var totalH=0;
		var minHeight=375;
		// vs.h is height of the viewport
		if (vs && vs.h > 0) {
			// how may items are in this list?
			var itemCount = this._getNumberOfListItems();
			var y = itemCount*39 + 200;
			// take the greater of computed value and minHeight
			y = Math.max(y, minHeight);
			// take the smaller of viewport height and the computed height
			totalH = Math.min(vs.h, y);
			// subtract some constants to derive availableHeight;  further adjustments may be made below
			var allowedSpaceAboveBanner = 30;
			availableHeight = totalH - (footerHeight + bannerHeight + allowedSpaceAboveBanner);
			dojo.style(this.scrollingDiv,"height",availableHeight+"px");
		}
		
		dojo.style(this.scrollingDiv,"overflowY", "auto");
		var hasVerticalScrollbar= this.scrollingDiv.scrollHeight>this.scrollingDiv.clientHeight;
		
		// when there's no vertical scrollbar, we may need to add some height anyway - this is 
		//   because when a drag operation starts space is added above or below the drop target and
		//   that added space may cause scrollbars to appear.  If we add extra height then scrollbar won't
		//   appear.  
		if (!hasVerticalScrollbar) {
			var listElemOffsetHeight = this.listElement.offsetHeight;
			var scrollingDivOffsetHeight = this.scrollingDiv.offsetHeight;
			if (scrollingDivOffsetHeight > listElemOffsetHeight) {
				var y = scrollingDivOffsetHeight - listElemOffsetHeight;
				if (y < 99) {
					// here if we need to add some height...
					var z = 99-y;
					availableHeight+=z;
					// ... but add more height ONLY if it won't cause the dropdown to cover the footer
					if (totalH-availableHeight > 99) {
						dojo.style(this.scrollingDiv,"height",availableHeight+"px");					
					}
				} else if (y > 99) {
					// conversely there may be too much height
					var z = y - 99;;
					availableHeight-=z;
					dojo.style(this.scrollingDiv,"height",availableHeight+"px");					
				}
			}
		}
		
		// adjust when there's not enough space on left or right for the dropdown to be completely visible
		if (this.isLeftToRight()) {
			var tabIcon = dojo.query('.processPortalDashboardOrganizerTabIcon')[0];
			if (tabIcon.offsetLeft < 175) {
				// not enough space, so adjust
				dojo.style(this.containerDiv, "marginLeft", "-90px");
				dojo.style(this.topArrow, "marginLeft", "61px");
			} else { 
				dojo.style(this.containerDiv, "marginLeft", "");
				dojo.style(this.topArrow, "marginLeft", "");
			}
		} else {
			var banner = dojo.query('.processPortalBanner')[0];
			var tabIcon = dojo.query('.processPortalDashboardOrganizerTabIcon')[0];
			var x = 0;
			if (banner.offsetWidth > tabIcon.offsetLeft) {
				x = banner.offsetWidth - tabIcon.offsetLeft;
			}
			var leftVal = (tabIcon.offsetLeft - (this.containerDiv.offsetWidth/2));
			if (x<175) {
				// not enough space, so adjust; take scrollbar into account
				leftVal=leftVal-90;
				if (hasVerticalScrollbar) {
					leftVal=leftVal-30;
					dojo.style(this.topArrow, "marginRight", "46px");
				} else {
					dojo.style(this.topArrow, "marginRight", "59px");
				}
			} else {
				// take scrollbar into account in this case too
				if (hasVerticalScrollbar) {
					dojo.style(this.topArrow, "marginRight", "165px");
				} else {
					dojo.style(this.topArrow, "marginRight", "");
				}
			}
			if (!hasVerticalScrollbar) {
				leftVal=leftVal-16;
			}
			dojo.style(this.containerDiv, "marginLeft", "0px");
			dojo.style(this.containerDiv, "left", leftVal+"px");
		}
		// end adjustments
		
		// save the dnd manager so it can be replaced upon hide
		dojo.dnd._manager = this.localDndManager;
		
		this.dropdownOpen = true;
		
		// If there is a selected tab, highlight it.  Set focus to headers.
		var textNodeToHighlight = null;
		var liNodeToHighlight = null;
		if (this.currentlySelectedTab!=null) {
			// get all the items in the organizer.
			// if match found, find the node with the proper classname and highlight it.
			var map = this.dashboardsList.map;
			for (var elem in map) {
				if (map[elem].data && map[elem].data.baseID && map[elem].data.baseID==this.currentlySelectedTab) {
					textNodeToHighlight=dojo.byId(elem);
					liNodeToHighlight=dojo.byId(this.innerLi_id+map[elem].data.baseID);
				}
			}			
		}
		var node = null;
		if (textNodeToHighlight!=null) {
			node=textNodeToHighlight;
		} else {
			node = this.listElement.firstChild;
			while (node && dojo.hasClass(node,"indicatorClassForSeparator")) {
				node = node.nextSibling;
			}
		}
		var modClass=false;
		if (node && textNodeToHighlight) {
			var a = dojo.query(".dashboardItem-text",node);
			if (a.length > 0) {
				this.doNotClose = true;
				node = a[0];
				modClass=true;
			} else {
				node = null;
			} 
		}
		// This fixes a weird behavior that occurs when webkit is the useragent when there the temp tab has a snapshot name in it:  when the
		//   organizer is shown then the entire tab div moves upward.  Tweaking the height fixes it.
		if (dojo.isWebKit) {
			var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
			if (orgDiv) {
				if (dojo.hasClass(orgDiv,"hasSnapshot")) {
					dojo.style(orgDiv, "height", "1px");
				}
			}
		}
		window.setTimeout(dojo.hitch(this,function() {	
			if (!node) {
				node = this.listElement;
				this.listElement.tabIndex = 0;
			} else {
				this.listElement.tabIndex = -1;
			}

			this.topHeadersDiv.focus();
			
			//node.focus(); 
			if (modClass==true) {
				dojo.removeClass(node, "dashboardItem-text");
				dojo.addClass(node, "dashboardItem-text-selected");
                if (node.nextSibling) {
                    dojo.addClass(node.nextSibling, "dashboardItem-text-selected");//blue-twistie
                }
				if (liNodeToHighlight!=null) {
					dojo.addClass(liNodeToHighlight,"listItemSelected");
				}
			}
			this.doNotClose = false;
		}),500);
		dojo.query(".dashboardItem-title-div",this.containerDiv).forEach(function(node){
			if (node.scrollWidth > node.offsetWidth) {
				node.title = node.textContent;
				if (dojo.isIE < 9)
					node.title = node.innerText;
			}
		});
		
	},
	
	onBlur: function() {
		if (!this.doNotClose) {
			window.setTimeout(dojo.hitch(this,function() {
				if ((this.dropdownOpen == true) && !this.doNotClose) {
					this.hide(true);
				}
			}),250);
		}
	},

	_getNumberOfListItems: function _getNumberOfListItems_DashboardOrganizer () {
		var map = this.dashboardsList.map;
		var elementCount = 0;
		for (var elem in map) {
			if (map[elem].data) {
				elementCount++;
			}
		}			
		return elementCount;
	},
	
	hide: function hide_DashboardOrganizer(savePrefs){
		if (this.dashboardsList) {
		  dojo.style(this.containerDiv, "display", "none");
		}
		// close snapshot containers and untwist twistie if needed
		this._closeSnapshotItems();
		
		// remove selected style on text if needed
		this._removeSelectedStyleOnText();
		
		// remove selected style on list item if needed
		this._removeSelectedStyleOnLi();
		
		// remove hover state on item if needed (if keyboard spacebar was used to select an item for drag)
		this._removeHoverStateOnItem();
		
		this.dropdownOpen = false;

		dojo.dnd._manager = this.saveDojoDndManager;

		this.keyboardMoveOperationActive = false;
		this.originalLocationOfItemBeingMoved= {};
		// defect 134873: hide and show tab in order to remove visual artifact on IE
		if (dojo.isIE) {
			var tabBarWidget = dijit.byId("processPortalTabBar");
			if (tabBarWidget && tabBarWidget.organizerTab) {
				dojo.style(tabBarWidget.organizerTab,"display","none");
				dojo.style(tabBarWidget.organizerTab,"display","inline");
			}
		}
		
		if (savePrefs===true) {
			this._updateDashboardOrder(this.getDisplayedDashboards(), this.getHiddenDashboards());
			if (this.prefsDirty) {
				this._writeDashboardOrderToUserProfile(this._storeddashboardOrganizerSettings);
				dojo.publish(com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS,[]);
			}
		}
	},

	_closeSnapshotItems: function _closeSnapshotItems_DashboardOrganizer() {
		// get list of all items so we can close snapshot containers and untwist twistie if needed
		dojo.forEach(this.getItems(), dojo.hitch(this, function(item){
			if (item.params && item.params.snapshotItems && item.params.snapshotItems.length>0) {
				var twistie=dojo.byId(this.dashboardOrganizerSnapshotIcon_id + item.params.baseID);
				var container=dojo.byId(this.snapshotContainer_id + item.params.baseID);
				this._hideShowSnapshotsContainer(null, twistie, container, true);	 // last param, true, means hide open snapshot containers
			}
		}));
	},

	_removeSelectedStyleOnText: function _removeSelectedStyleOnText_DashboardOrganizer() {
		var a = dojo.query(".dashboardItem-text-selected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "dashboardItem-text-selected");
			dojo.addClass(a[0], "dashboardItem-text");
		}
	},

	_removeSelectedStyleOnLi: function _removeSelectedStyleOnLi_DashboardOrganizer() {
		var a = dojo.query(".listItemSelected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "listItemSelected");
		}
	},
	
	_removeHoverStateOnItem: function _removeHoverStateOnItem_DashboardOrganizer() {
		var a = dojo.query(".keyboardSelected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "keyboardSelected");
		}
	},

    // return a list of the displayed dashboards in the correct sorted order
	getDisplayedDashboards: function getDisplayedDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("displayed");
		}
		return null;
	},
  
    // return a list of the hidden dashboards in the correct sorted order (order doesn't really matter here)
	getHiddenDashboards: function getHiddenDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("hidden");
		}
		return null;
	},
  
    // return a list of all dashboards
	getAllDashboards: function getAllDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("all");
		}
		return null;
	},
  
    // find a specific dashboard, including within snapshots
	getSpecificDashboard: function getSpecificDashboard_DashboardOrganizer(baseID) {
		var dashboardItem=null;
		if (this.dashboardsList) {
			var allDashboards = this._getListOfDashboards("all");
			if (allDashboards.length > 0) {
				var tItem = null;
				for (var i=0; i<allDashboards.length; i++) {
					tItem = allDashboards[i];
					if (tItem.data.params.baseID == baseID) {
						dashboardItem = tItem;
					} else {
						// check snapshots, if there are any
						if (tItem.data.params.snapshotItems && tItem.data.params.snapshotItems.length > 0) {
							for (var x=0; x < tItem.data.params.snapshotItems.length; x++) {
								var tItem2 = {};
								// the attribute "data" is expected by the caller of this function
								tItem2["data"] = tItem.data.params.snapshotItems[x];
								if (tItem2["data"].params.baseID == baseID) {
									dashboardItem = tItem2;
								}
							}
						}
					}
				}
			}
		}
		return dashboardItem;
	},
  
    // find a specific dashboard by itemID, NOT including within snapshots
	getSpecificDashboardByItemID: function getSpecificDashboardByItemID_DashboardOrganizer(itemID) {
		var dashboardItem=null;
		if (this.dashboardsList) {
			var allDashboards = this._getListOfDashboards("all");
			if (allDashboards.length > 0) {
				var tItem = null;
				for (var i=0; i<allDashboards.length; i++) {
					tItem = allDashboards[i];
					if (tItem.data.params.itemID == itemID) {
						dashboardItem = tItem;
					}
				}
			}
		}
		return dashboardItem;
	},
	
	_getListOfDashboards: function _getListOfDashboards_DashboardOrganizer(type) {
		var returnItems=[];
		var separatorFound=false;
		dojo.forEach( this._getOrderedItems(this.dashboardsList), dojo.hitch( this, 
			function(item){	
				if (!separatorFound) {
					separatorFound=dojo.indexOf(item.type,"separator")>=0?true:false;
				}
				if (dojo.indexOf(item.type,"dashboardItem")>=0 && type=="displayed" && !separatorFound) {
					returnItems.push(item);
				} else if (dojo.indexOf(item.type,"dashboardItem")>=0 &&  type=="hidden" && separatorFound) {
					returnItems.push(item);
				} else if (dojo.indexOf(item.type,"dashboardItem")>=0 &&  type=="all") {
					returnItems.push(item);
				}
        }));
		return returnItems;
	},
  
    // return the items in a Source object in sorted order
	_getOrderedItems: function _getOrderedItems_DashboardOrganizer(container){
	  var items=[];
	  container.getAllNodes().forEach(function(node){
		var id = node.id;
		var tempItem=container.getItem(id);
		tempItem["dojoUniqueId"]=id;
		items.push(tempItem);
	  });
	  return items;
	},
	
	setCurrentlySelectedTab: function setCurrentlySelectedTab_DashboardOrganizer(baseID){
		this.currentlySelectedTab=baseID;
	},

	setCurrentlySelectedTabByItemID: function setCurrentlySelectedTabByItemID_DashboardOrganizer(itemID){
		// find the top-level item (not snapshot) with this itemID, and get its baseID
		var item = this.getSpecificDashboardByItemID(itemID);
		if (item) {
			this.currentlySelectedTab=item.data.baseID;
		}
	},
	
	// Convenience function; the returned value is what was retrieved from the ExposedItemsList in the _getExposedItems() above.
	// Returns true if the current env is Process Server
	// *** The return value is reliable *if* there were >0 exposed items found in the ExposedItemsList.getData() function.
	// If there were 0 exposed items found the the value is unreliable but is also irrelevant -- 
	//    the returned value only matters when there are exposed items.
	
	getIsProcessServer: function isProcessServer_DashboardOrganizer() {
		return this.isProcessServer;
	},

	destroy: function destroy_DashboardOrganizer(){
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if (this.dashboardsList) {
		  if (this.dashboardsList.destroyRecursive) {
			this.dashboardsList.destroyRecursive();
		  }
		  else if (this.dashboardsList.destroy) {
			this.dashboardsList.destroy();
		  }
		}
		if (this.dojoConnections) {
			for (var i=0; i <this.dojoConnections.length; i++) {
				this.dojoConnections[i].destroy();
			}
			this.dojoConnections=null;
		}
		this._cleanUpMenusAndWidgets();

		if (this.saveDojoDndManager) {
			dojo.dnd._manager = this.saveDojoDndManager;
			if (this.localDndManager && this.localDndManager.destroy) {
				this.localDndManager.destroy();
			}
			this.localDndManager=null;
		}
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});


////////////////////////////////////////
// GLOBALS
////////////////////////////////////////

com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME = "Portal Dashboard Display Order";



////////////////////////////////////////
// Extend dojo.dnd.Source in order to override checkAcceptance
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndSource", dojo.dnd.Source, {
    checkAcceptance : function(source, nodes) {
		// Don't allow the separator to be dragged.
		var attr = dojo.attr(nodes[0],"id");
		if (attr && attr==="dashboardSeparatorItem") {
			return false;
		}
		// since checkAcceptance is called when a drag operation starts, we can
		//   publish a message telling the Organizer to close the snapshot containers of 
		//   all items, as needed.
		dojo.publish("dashboardOrganizer_hideSnapshotContainer",[]);		
		/* // Get the ID that the Organizer will use to identify the snapshot container.
		if (nodes[0].id) {
			// source stores the baseID that we need
			if (source && source.map && source.map[nodes[0].id]) {
				var map = source.map[nodes[0].id];
				if (map.data && map.data.params && map.data.params.snapshotItems && map.data.params.snapshotItems.length>0) {
					// this is the ID we need:
					dojo.publish("dashboardOrganizer_hideSnapshotContainer",[{"baseID":map.data.params.baseID }]);
				}
			}
		} */
		
       return this.inherited(arguments);
    },
	onDndStart: function(source, nodes, copy){
		this.savedFocusNode = dijit.getFocus().node;
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		dijit.focus(organizer.domNode);
		this.inherited(arguments);
	},
	onDndCancel: function(){
		this.inherited(arguments);
		if (this.savedFocusNode) {
			dijit.focus(this.savedFocusNode);
			this.savedFocusNode = null;
		}
	},    
    onDropInternal: function(nodes, copy){
    	var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
    	organizer.doNotClose = true;
    	this.inherited(arguments);
   		window.setTimeout(function() {
   			organizer.doNotClose = false;
   		},250);
    },
	onMouseDown: function(e){
		// ON IE, need to add the dojoDndMove class *before* the drag starts.  Remove it in onMouseUp().
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if (dojo.isIE) {
			dojo.addClass(dojo.body(), "dojoDndMove");
		}
		this.inherited(arguments);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if (dojo.isIE) {
			dojo.removeClass(dojo.body(), "dojoDndMove");
		}
		this.inherited(arguments);
	}
});

////////////////////////////////////////
// Extend dojo.dnd.Manager in order to override avatar creation
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndManager", dojo.dnd.Manager, {
	// avatar's offset from the mouse
	OFFSET_X: 4,
	OFFSET_Y: 4,
    makeAvatar : function(source, nodes) {
		return new com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndAvatar(this);
    }
});
////////////////////////////////////////
// Extend dojo.dnd.Avatar in order to override avatar
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndAvatar", dojo.dnd.Avatar, {
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need
		this.isA11y = dojo.hasClass(dojo.body(),"dijit_a11y");
		var a = dojo.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = dojo.create("tbody", null, a),
			k = Math.min(5, this.manager.nodes.length), i = 0;
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = dojo.create("table"),
						tbody = dojo.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = dojo.create("tr", null, b);
			td = dojo.create("td", null, tr);
			if (this.isA11y) {
				icon = dojo.create("span", {id : "a11yIcon", innerHTML : "<"}, td);
				icon.style.display = 'none';
			}
			td.appendChild(node);
		}
		this.node = a;
	}
});

}

if(!dojo._hasResource["dijit._KeyNavContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._KeyNavContainer"] = true;
dojo.provide("dijit._KeyNavContainer");



dojo.declare("dijit._KeyNavContainer",
	dijit._Container,
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().
		//		It provides normalized keyboard and focusing code for Container
		//		widgets.
/*=====
		// focusedChild: [protected] Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		// tabIndex: Integer
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*dojo.keys[]*/ prevKeyCodes, /*dojo.keys[]*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: dojo.keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: dojo.keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			var keyCodes = (this._keyNavCodes = {});
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev; });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next; });
			keyCodes[dojo.keys.HOME] = dojo.hitch(this, "focusFirstChild");
			keyCodes[dojo.keys.END] = dojo.hitch(this, "focusLastChild");
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			this.connect(this.domNode, "onfocus", "_onContainerFocus");
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			// tags:
			//		protected
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._startupChild(widget);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected
			var child = this._getFirstFocusableChild();
			if(child){ // edge case: Menu could be empty or hidden
				this.focusChild(child);
			}
		},

		focusLastChild: function(){
			// summary:
			//		Focus the last focusable child in the container.
			// tags:
			//		protected
			var child = this._getLastFocusableChild();
			if(child){ // edge case: Menu could be empty or hidden
				this.focusChild(child);
			}
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			this.focusChild(child);
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			this.focusChild(child, true);
		},

		focusChild: function(/*dijit._Widget*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected
			
			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);
			}
			widget.set("tabIndex", this.tabIndex);	// for IE focus outline to appear, must set tabIndex before focs
			widget.focus(last ? "end" : "start");
			this._set("focusedChild", widget);
		},

		_startupChild: function(/*dijit._Widget*/ widget){
			// summary:
			//		Setup for each child widget
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will
			//		leave the container rather than visiting each child.
			// tags:
			//		private
			
			widget.set("tabIndex", "-1");
			
			this.connect(widget, "_onFocus", function(){
				// Set valid tabIndex so tabbing away from widget goes to right place, see #10272
				widget.set("tabIndex", this.tabIndex);
			});
			this.connect(widget, "_onBlur", function(){
				widget.set("tabIndex", "-1");
			});
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container gets focus
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively)

			// focus bubbles on Firefox,
			// so just make sure that focus has really gone to the container
			if(evt.target !== this.domNode){ return; }

			this.focusFirstChild();

			// and then set the container's tabIndex to -1,
			// (don't remove as that breaks Safari 4)
			// so that tab or shift-tab will go to the fields after/before
			// the container, rather than the container itself
			dojo.attr(this.domNode, "tabIndex", "-1");
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)
			if(this.tabIndex){
				dojo.attr(this.domNode, "tabIndex", this.tabIndex);
			}
			this.inherited(arguments);
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then
			//		it's handled here.
			// tags:
			//		private
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.charOrCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*dijit._Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			// tags:
			//		protected
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused
			return this._getNextFocusableChild(null, 1);	// dijit._Widget
		},

		_getLastFocusableChild: function(){
			// summary:
			//		Returns last child that can be focused
			return this._getNextFocusableChild(null, -1);	// dijit._Widget
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable child, compared
			//		to "child"
			// child: Widget
			//		The current widget
			// dir: Integer
			//		* 1 = after
			//		* -1 = before
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;	// dijit._Widget
				}
				child = this._getSiblingOfChild(child, dir);
			}
			// no focusable child found
			return null;	// dijit._Widget
		}
	}
);

}

if(!dojo._hasResource["dijit.MenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuItem"] = true;
dojo.provide("dijit.MenuItem");






dojo.declare("dijit.MenuItem",
		[dijit._Widget, dijit._Templated, dijit._Contained, dijit._CssStateMixin],
		{
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: dojo.cache("dijit", "templates/MenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" role=\"menuitem\" tabIndex=\"-1\"\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">\n\t\t<div dojoAttachPoint=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</div>\n\t</td>\n</tr>\n"),

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			label: { node: "containerNode", type: "innerHTML" },
			iconClass: { node: "iconNode", type: "class" }
		}),

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text
		label: '',

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "",

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination.
		//		Note that although Menu can display accelerator keys there
		//		is no infrastructure to actually catch and execute these
		//		accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this.set('label', source.innerHTML);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			var label = this.id+"_text";
			dojo.attr(this.containerNode, "id", label);
			if(this.accelKeyNode){
				dojo.attr(this.accelKeyNode, "id", this.id + "_accel");
				label += " " + this.id + "_accel";
			}
			dijit.setWaiState(this.domNode, "labelledby", label);
			dojo.setSelectable(this.domNode, false);
		},

		_onHover: function(){
			// summary:
			//		Handler when mouse is moved onto menu item
			// tags:
			//		protected
			this.getParent().onItemHover(this);
		},

		_onUnhover: function(){
			// summary:
			//		Handler when mouse is moved off of menu item,
			//		possibly to a child menu, or maybe to a sibling
			//		menuitem or somewhere else entirely.
			// tags:
			//		protected

			// if we are unhovering the currently selected item
			// then unselect it
			this.getParent().onItemUnhover(this);

			// When menu is hidden (collapsed) due to clicking a MenuItem and having it execute,
			// FF and IE don't generate an onmouseout event for the MenuItem.
			// So, help out _CssStateMixin in this case.
			this._set("hovering", false);
		},

		_onClick: function(evt){
			// summary:
			//		Internal handler for click events on MenuItem.
			// tags:
			//		private
			this.getParent().onItemClick(this, evt);
			dojo.stopEvent(evt);
		},

		onClick: function(/*Event*/ evt){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(dojo.isIE == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				dijit.focus(this.focusNode);
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected
			this._setSelected(true);
			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			/***
			 * TODO: remove this method and calls to it, when _onBlur() is working for MenuItem.
			 * Currently _onBlur() gets called when focus is moved from the MenuItem to a child menu.
			 * That's not supposed to happen, but the problem is:
			 * In order to allow dijit.popup's getTopPopup() to work,a sub menu's popupParent
			 * points to the parent Menu, bypassing the parent MenuItem... thus the
			 * MenuItem is not in the chain of active widgets and gets a premature call to
			 * _onBlur()
			 */

			dojo.toggleClass(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},
		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.

			dijit.setWaiState(this.focusNode, 'disabled', value ? 'true' : 'false');
			this._set("disabled", value);
		},
		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.

			this.accelKeyNode.style.display=value?"":"none";
			this.accelKeyNode.innerHTML=value;
			//have to use colSpan to make it work in IE
			dojo.attr(this.containerNode,'colSpan',value?"1":"2");
			
			this._set("accelKey", value);
		}
	});

}

if(!dojo._hasResource["dijit.PopupMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.PopupMenuItem"] = true;
dojo.provide("dijit.PopupMenuItem");



dojo.declare("dijit.PopupMenuItem",
		dijit.MenuItem,
		{
		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRefinnerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div dojoType="dijit.PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = dojo.query("*", this.srcNodeRef);
				dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  move it to dojo.doc.body.
			if(!this.popup){
				var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
				this.popup = dijit.byNode(node);
			}
			dojo.body().appendChild(this.popup.domNode);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				dojo.style(this.arrowWrapper, "visibility", "");
			}
			dijit.setWaiState(this.focusNode, "haspopup", "true");
		},

		destroyDescendants: function(){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive();
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});

}

if(!dojo._hasResource["dijit.CheckedMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.CheckedMenuItem"] = true;
dojo.provide("dijit.CheckedMenuItem");



dojo.declare("dijit.CheckedMenuItem",
		dijit.MenuItem,
		{
		// summary:
		//		A checkbox-like menu item for toggling on and off

		templateString: dojo.cache("dijit", "templates/CheckedMenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" role=\"menuitemcheckbox\" tabIndex=\"-1\"\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" role=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode,labelNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" role=\"presentation\">&nbsp;</td>\n</tr>\n"),

		// checked: Boolean
		//		Our checked state
		checked: false,
		_setCheckedAttr: function(/*Boolean*/ checked){
			// summary:
			//		Hook so attr('checked', bool) works.
			//		Sets the class and state for the check box.
			dojo.toggleClass(this.domNode, "dijitCheckedMenuItemChecked", checked);
			dijit.setWaiState(this.domNode, "checked", checked);
			this._set("checked", checked);
		},

		onChange: function(/*Boolean*/ checked){
			// summary:
			//		User defined function to handle check/uncheck events
			// tags:
			//		callback
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Clicking this item just toggles its state
			// tags:
			//		private
			if(!this.disabled){
				this.set("checked", !this.checked);
				this.onChange(this.checked);
			}
			this.inherited(arguments);
		}
	});

}

if(!dojo._hasResource["dijit.MenuSeparator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuSeparator"] = true;
dojo.provide("dijit.MenuSeparator");





dojo.declare("dijit.MenuSeparator",
		[dijit._Widget, dijit._Templated, dijit._Contained],
		{
		// summary:
		//		A line between two menu items

		templateString: dojo.cache("dijit", "templates/MenuSeparator.html", "<tr class=\"dijitMenuSeparator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>\n"),

		buildRendering: function(){
			this.inherited(arguments);
			dojo.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});

}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");










// "dijit/MenuItem", "dijit/PopupMenuItem", "dijit/CheckedMenuItem", "dijit/MenuSeparator" for Back-compat (TODO: remove in 2.0)

dojo.declare("dijit._MenuBase",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	// summary:
	//		Base class for Menu and MenuBar

	// parentMenu: [readonly] Widget
	//		pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//		number of milliseconds before hovering (without clicking) causes the popup to automatically open.
	popupDelay: 500,

	startup: function(){
		if(this._started){ return; }

		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();

		this.inherited(arguments);
	},

	onExecute: function(){
		// summary:
		//		Attach point for notification about when a menu item has been executed.
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them, because they are about to execute the onClick handler.   In
		//		general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary:
		//		Attach point for notification about when the user cancels the current menu
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them.  In general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	_moveToPopup: function(/*Event*/ evt){
		// summary:
		//		This handles the right arrow key (left arrow key on RTL systems),
		//		which will either open a submenu, or move to the next item in the
		//		ancestor MenuBar
		// tags:
		//		private

		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}else{
			var topMenu = this._getTopMenu();
			if(topMenu && topMenu._isMenuBar){
				topMenu.focusNext();
			}
		}
	},

	_onPopupHover: function(/*Event*/ evt){
		// summary:
		//		This handler is called when the mouse moves over the popup.
		// tags:
		//		private

		// if the mouse hovers over a menu popup that is in pending-close state,
		// then stop the close operation.
		// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)
		if(this.currentPopup && this.currentPopup._pendingClose_timer){
			var parentMenu = this.currentPopup.parentMenu;
			// highlight the parent menu item pointing to this popup
			if(parentMenu.focusedChild){
				parentMenu.focusedChild._setSelected(false);
			}
			parentMenu.focusedChild = this.currentPopup.from_item;
			parentMenu.focusedChild._setSelected(true);
			// cancel the pending close
			this._stopPendingCloseTimer(this.currentPopup);
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		// summary:
		//		Called when cursor is over a MenuItem.
		// tags:
		//		protected

		// Don't do anything unless user has "activated" the menu by:
		//		1) clicking it
		//		2) opening it from a parent menu (which automatically focuses it)
		if(this.isActive){
			this.focusChild(item);
			if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
				this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
			}
		}
		// if the user is mixing mouse and keyboard navigation,
		// then the menu may not be active but a menu item has focus,
		// but it's not the item that the mouse just hovered over.
		// To avoid both keyboard and mouse selections, use the latest.
		if(this.focusedChild){
			this.focusChild(item);
		}
		this._hoveredChild = item;
	},

	_onChildBlur: function(item){
		// summary:
		//		Called when a child MenuItem becomes inactive because focus
		//		has been removed from the MenuItem *and* it's descendant menus.
		// tags:
		//		private
		this._stopPopupTimer();
		item._setSelected(false);
		// Close all popups that are open and descendants of this menu
		var itemPopup = item.popup;
		if(itemPopup){
			this._stopPendingCloseTimer(itemPopup);
			itemPopup._pendingClose_timer = setTimeout(function(){
				itemPopup._pendingClose_timer = null;
				if(itemPopup.parentMenu){
					itemPopup.parentMenu.currentPopup = null;
				}
				dijit.popup.close(itemPopup); // this calls onClose
			}, this.popupDelay);
		}
	},

	onItemUnhover: function(/*MenuItem*/ item){
		// summary:
		//		Callback fires when mouse exits a MenuItem
		// tags:
		//		protected

		if(this.isActive){
			this._stopPopupTimer();
		}
		if(this._hoveredChild == item){ this._hoveredChild = null; }
	},

	_stopPopupTimer: function(){
		// summary:
		//		Cancels the popup timer because the user has stop hovering
		//		on the MenuItem, etc.
		// tags:
		//		private
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_stopPendingCloseTimer: function(/*dijit._Widget*/ popup){
		// summary:
		//		Cancels the pending-close timer because the close has been preempted
		// tags:
		//		private
		if(popup._pendingClose_timer){
			clearTimeout(popup._pendingClose_timer);
			popup._pendingClose_timer = null;
		}
	},

	_stopFocusTimer: function(){
		// summary:
		//		Cancels the pending-focus timer because the menu was closed before focus occured
		// tags:
		//		private
		if(this._focus_timer){
			clearTimeout(this._focus_timer);
			this._focus_timer = null;
		}
	},

	_getTopMenu: function(){
		// summary:
		//		Returns the top menu in this chain of Menus
		// tags:
		//		private
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item.
		// tags:
		//		private

		// this can't be done in _onFocus since the _onFocus events occurs asynchronously
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
			this._markActive();
		}

		this.focusChild(item);

		if(item.disabled){ return false; }

		if(item.popup){
			this._openPopup();
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick(evt);
		}
	},

	_openPopup: function(){
		// summary:
		//		Open the popup to the side of/underneath the current menu item
		// tags:
		//		protected

		this._stopPopupTimer();
		var from_item = this.focusedChild;
		if(!from_item){ return; } // the focused child lost focus since the timer was started
		var popup = from_item.popup;
		if(popup.isShowingNow){ return; }
		if(this.currentPopup){
			this._stopPendingCloseTimer(this.currentPopup);
			dijit.popup.close(this.currentPopup);
		}
		popup.parentMenu = this;
		popup.from_item = from_item; // helps finding the parent item that should be focused for this popup
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.domNode,
			orient: this._orient || (this.isLeftToRight() ?
									{'TR': 'TL', 'TL': 'TR', 'BR': 'BL', 'BL': 'BR'} :
									{'TL': 'TR', 'TR': 'TL', 'BL': 'BR', 'BR': 'BL'}),
			onCancel: function(){ // called when the child menu is canceled
				// set isActive=false (_closeChild vs _cleanUp) so that subsequent hovering will NOT open child menus
				// which seems aligned with the UX of most applications (e.g. notepad, wordpad, paint shop pro)
				self.focusChild(from_item);	// put focus back on my node
				self._cleanUp();			// close the submenu (be sure this is done _after_ focus is moved)
				from_item._setSelected(true); // oops, _cleanUp() deselected the item
				self.focusedChild = from_item;	// and unset focusedChild
			},
			onExecute: dojo.hitch(this, "_cleanUp")
		});

		this.currentPopup = popup;
		// detect mouseovers to handle lazy mouse movements that temporarily focus other menu items
		popup.connect(popup.domNode, "onmouseenter", dojo.hitch(self, "_onPopupHover")); // cleaned up when the popped-up widget is destroyed on close

		if(popup.focus){
			// If user is opening the popup via keyboard (right arrow, or down arrow for MenuBar),
			// if the cursor happens to collide with the popup, it will generate an onmouseover event
			// even though the mouse wasn't moved.   Use a setTimeout() to call popup.focus so that
			// our focus() call overrides the onmouseover event, rather than vice-versa.  (#8742)
			popup._focus_timer = setTimeout(dojo.hitch(popup, function(){
				this._focus_timer = null;
				this.focus();
			}), 0);
		}
	},

	_markActive: function(){
		// summary:
		//              Mark this menu's state as active.
		//		Called when this Menu gets focus from:
		//			1) clicking it (mouse or via space/arrow key)
		//			2) being opened by a parent menu.
		//		This is not called just from mouse hover.
		//		Focusing a menu via TAB does NOT automatically set isActive
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar
		//		menus (similar to TAB navigation) but the menu is not active
		//		(ie no dropdown) until an item is clicked.
		this.isActive = true;
		dojo.replaceClass(this.domNode, "dijitMenuActive", "dijitMenuPassive");
	},

	onOpen: function(/*Event*/ e){
		// summary:
		//		Callback when this menu is opened.
		//		This is called by the popup manager as notification that the menu
		//		was opened.
		// tags:
		//		private

		this.isShowingNow = true;
		this._markActive();
	},

	_markInactive: function(){
		// summary:
		//		Mark this menu's state as inactive.
		this.isActive = false; // don't do this in _onBlur since the state is pending-close until we get here
		dojo.replaceClass(this.domNode, "dijitMenuPassive", "dijitMenuActive");
	},

	onClose: function(){
		// summary:
		//		Callback when this menu is closed.
		//		This is called by the popup manager as notification that the menu
		//		was closed.
		// tags:
		//		private

		this._stopFocusTimer();
		this._markInactive();
		this.isShowingNow = false;
		this.parentMenu = null;
	},

	_closeChild: function(){
		// summary:
		//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
		// tags:
		//		private
		this._stopPopupTimer();

		var fromItem = this.focusedChild && this.focusedChild.from_item;

		if(this.currentPopup){
			// If focus is on my child menu then move focus to me,
			// because IE doesn't like it when you display:none a node with focus
			if(dijit._curFocus && dojo.isDescendant(dijit._curFocus, this.currentPopup.domNode)){
				this.focusedChild.focusNode.focus();
			}
			// Close all popups that are open and descendants of this menu
			dijit.popup.close(this.currentPopup);
			this.currentPopup = null;
		}

		if(this.focusedChild){ // unhighlight the focused item
			this.focusedChild._setSelected(false);
			this.focusedChild._onUnhover();
			this.focusedChild = null;
		}
	},

	_onItemFocus: function(/*MenuItem*/ item){
		// summary:
		//		Called when child of this Menu gets focus from:
		//			1) clicking it
		//			2) tabbing into it
		//			3) being opened by a parent menu.
		//		This is not called just from mouse hover.
		if(this._hoveredChild && this._hoveredChild != item){
			this._hoveredChild._onUnhover(); // any previous mouse movement is trumped by focus selection
		}
	},

	_onBlur: function(){
		// summary:
		//		Called when focus is moved away from this Menu and it's submenus.
		// tags:
		//		protected
		this._cleanUp();
		this.inherited(arguments);
	},

	_cleanUp: function(){
		// summary:
		//		Called when the user is done with this menu.  Closes hierarchy of menus.
		// tags:
		//		private

		this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
			this._markInactive();
		}
	}
});

dojo.declare("dijit.Menu",
	dijit._MenuBase,
	{
	// summary
	//		A context menu you can assign to multiple elements

	// TODO: most of the code in here is just for context menu (right-click menu)
	// support.  In retrospect that should have been a separate class (dijit.ContextMenu).
	// Split them for 2.0

	constructor: function(){
		this._bindings = [];
	},

	templateString: dojo.cache("dijit", "templates/Menu.html", "<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" role=\"menu\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"onkeypress:_onKeyPress\" cellspacing=\"0\">\n\t<tbody class=\"dijitReset\" dojoAttachPoint=\"containerNode\"></tbody>\n</table>\n"),

	baseClass: "dijitMenu",

	// targetNodeIds: [const] String[]
	//		Array of dom node ids of nodes to attach to.
	//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: [const] Boolean
	//		If true, right clicking anywhere on the window will cause this context menu to open.
	//		If false, must specify targetNodeIds.
	contextMenuForWindow: false,

	// leftClickToOpen: [const] Boolean
	//		If true, menu will open on left click instead of right click, similiar to a file menu.
	leftClickToOpen: false,

	// refocus: Boolean
	// 		When this menu closes, re-focus the element which had focus before it was opened.
	refocus: true,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			// TODO: should have _setTargetNodeIds() method to handle initialization and a possible
			// later set('targetNodeIds', ...) call.   There's also a problem that targetNodeIds[]
			// gets stale after calls to bindDomNode()/unBindDomNode() as it still is just the original list (see #9610)
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		var k = dojo.keys, l = this.isLeftToRight();
		this._openSubMenuKey = l ? k.RIGHT_ARROW : k.LEFT_ARROW;
		this._closeSubMenuKey = l ? k.LEFT_ARROW : k.RIGHT_ARROW;
		this.connectKeyNavHandlers([k.UP_ARROW], [k.DOWN_ARROW]);
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handle keyboard based menu navigation.
		// tags:
		//		protected

		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.charOrCode){
			case this._openSubMenuKey:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case this._closeSubMenuKey:
				if(this.parentMenu){
					if(this.parentMenu._isMenuBar){
						this.parentMenu.focusPrev();
					}else{
						this.onCancel(false);
					}
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns the window reference of the passed iframe
		// tags:
		//		private
		var win = dojo.window.get(this._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			this._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && dojo.doc.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns a reference to the document object inside iframe_el
		// tags:
		//		protected
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && dojo.doc.frames[iframe_el.name] && dojo.doc.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary:
		//		Attach menu to given node
		node = dojo.byId(node);

		var cn;	// Connect node

		// Support context menus on iframes.   Rather than binding to the iframe itself we need
		// to bind to the <body> node inside the iframe.
		if(node.tagName.toLowerCase() == "iframe"){
			var iframe = node,
				win = this._iframeContentWindow(iframe);
			cn = dojo.withGlobal(win, dojo.body);
		}else{
			
			// To capture these events at the top level, attach to <html>, not <body>.
			// Otherwise right-click context menu just doesn't work.
			cn = (node == dojo.body() ? dojo.doc.documentElement : node);
		}


		// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
		var binding = {
			node: node,
			iframe: iframe
		};

		// Save info about binding in _bindings[], and make node itself record index(+1) into
		// _bindings[] array.   Prefix w/_dijitMenu to avoid setting an attribute that may
		// start with a number, which fails on FF/safari.
		dojo.attr(node, "_dijitMenu" + this.id, this._bindings.push(binding));

		// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
		// loading yet, in which case we need to wait for the onload event first, and then connect
		// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
		// we need to monitor keyboard events in addition to the oncontextmenu event.
		var doConnects = dojo.hitch(this, function(cn){
			return [
				// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
				// rather than shift-F10?
				dojo.connect(cn, this.leftClickToOpen ? "onclick" : "oncontextmenu", this, function(evt){
					// Schedule context menu to be opened unless it's already been scheduled from onkeydown handler
					dojo.stopEvent(evt);
					this._scheduleOpen(evt.target, iframe, {x: evt.pageX, y: evt.pageY});
				}),
				dojo.connect(cn, "onkeydown", this, function(evt){
					if(evt.shiftKey && evt.keyCode == dojo.keys.F10){
						dojo.stopEvent(evt);
						this._scheduleOpen(evt.target, iframe);	// no coords - open near target node
					}
				})
			];
		});
		binding.connects = cn ? doConnects(cn) : [];

		if(iframe){
			// Setup handler to [re]bind to the iframe when the contents are initially loaded,
			// and every time the contents change.
			// Need to do this b/c we are actually binding to the iframe's <body> node.
			// Note: can't use dojo.connect(), see #9609.

			binding.onloadHandler = dojo.hitch(this, function(){
				// want to remove old connections, but IE throws exceptions when trying to
				// access the <body> node because it's already gone, or at least in a state of limbo

				var win = this._iframeContentWindow(iframe);
					cn = dojo.withGlobal(win, dojo.body);
				binding.connects = doConnects(cn);
			});
			if(iframe.addEventListener){
				iframe.addEventListener("load", binding.onloadHandler, false);
			}else{
				iframe.attachEvent("onload", binding.onloadHandler);
			}
		}
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary:
		//		Detach menu from given node

		var node;
		try{
			node = dojo.byId(nodeName);
		}catch(e){
			// On IE the dojo.byId() call will get an exception if the attach point was
			// the <body> node of an <iframe> that has since been reloaded (and thus the
			// <body> node is in a limbo state of destruction.
			return;
		}

		// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
		var attrName = "_dijitMenu" + this.id;
		if(node && dojo.hasAttr(node, attrName)){
			var bid = dojo.attr(node, attrName)-1, b = this._bindings[bid];
			dojo.forEach(b.connects, dojo.disconnect);

			// Remove listener for iframe onload events
			var iframe = b.iframe;
			if(iframe){
				if(iframe.removeEventListener){
					iframe.removeEventListener("load", b.onloadHandler, false);
				}else{
					iframe.detachEvent("onload", b.onloadHandler);
				}
			}

			dojo.removeAttr(node, attrName);
			delete this._bindings[bid];
		}
	},

	_scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
		// summary:
		//		Set timer to display myself.  Using a timer rather than displaying immediately solves
		//		two problems:
		//
		//		1. IE: without the delay, focus work in "open" causes the system
		//		context menu to appear in spite of stopEvent.
		//
		//		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
		//		even after a dojo.stopEvent(e).  (Shift-F10 on windows doesn't generate the
		//		oncontextmenu event.)

		if(!this._openTimer){
			this._openTimer = setTimeout(dojo.hitch(this, function(){
				delete this._openTimer;
				this._openMyself({
					target: target,
					iframe: iframe,
					coords: coords
				});
			}), 1);
		}
	},

	_openMyself: function(args){
		// summary:
		//		Internal function for opening myself when the user does a right-click or something similar.
		// args:
		//		This is an Object containing:
		//		* target:
		//			The node that is being clicked
		//		* iframe:
		//			If an <iframe> is being clicked, iframe points to that iframe
		//		* coords:
		//			Put menu at specified x/y position in viewport, or if iframe is
		//			specified, then relative to iframe.
		//
		//		_openMyself() formerly took the event object, and since various code references
		//		evt.target (after connecting to _openMyself()), using an Object for parameters
		//		(so that old code still works).

		var target = args.target,
			iframe = args.iframe,
			coords = args.coords;

		// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
		// then near the node the menu is assigned to.
		if(coords){
			if(iframe){
				// Specified coordinates are on <body> node of an <iframe>, convert to match main document
				var od = target.ownerDocument,
					ifc = dojo.position(iframe, true),
					win = this._iframeContentWindow(iframe),
					scroll = dojo.withGlobal(win, "_docScroll", dojo);
	
				var cs = dojo.getComputedStyle(iframe),
					tp = dojo._toPixelValue,
					left = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingLeft)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderLeftWidth) : 0),
					top = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingTop)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderTopWidth) : 0);

				coords.x += ifc.x + left - scroll.x;
				coords.y += ifc.y + top - scroll.y;
			}
		}else{
			coords = dojo.position(target, true);
			coords.x += 10;
			coords.y += 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			if(self.refocus){
				dijit.focus(savedFocus);
			}
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: coords.x,
			y: coords.y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			this.inherited('_onBlur', arguments);
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};
	},

	uninitialize: function(){
 		dojo.forEach(this._bindings, function(b){ if(b){ this.unBindDomNode(b.node); } }, this);
 		this.inherited(arguments);
	}
}
);

}

if(!dojo._hasResource["dijit._HasDropDown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._HasDropDown"] = true;
dojo.provide("dijit._HasDropDown");



dojo.declare("dijit._HasDropDown",
	null,
	{
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the popupActive class on.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subwidget
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon

			if(this.disabled || this.readOnly){ return; }

			// Prevent default to stop things like text selection, but don't stop propogation, so that:
			//		1. TimeTextBox etc. can focusthe <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires
			e.preventDefault();

			this._docHandler = this.connect(dojo.doc, "onmouseup", "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		dropDown node.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser <select> nodes:
			//			1. mouse down on the select node (probably on the arrow)
			//			2. move mouse to a menu item while holding down the mouse button
			//			3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = dojo.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(dojo.hasClass(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = dijit.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened && dropDown.focus && dropDown.autoFocus !== false){
				// Focus the dropdown widget - do it on a delay so that we
				// don't steal our own focus.
				window.setTimeout(dojo.hitch(dropDown, "focus"), 1);
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// the drop down was already opened on mousedown/keydown; just need to call stopEvent()
			if(this._stopClickEvents){
				dojo.stopEvent(e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			dojo.addClass(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keypress events

			this.inherited(arguments);

			this.connect(this._buttonNode, "onmousedown", "_onDropDownMouseDown");
			this.connect(this._buttonNode, "onclick", "_onDropDownClick");
			this.connect(this.focusNode, "onkeypress", "_onKey");
			this.connect(this.focusNode, "onkeyup", "_onKeyUp");
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }

			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					dojo.stopEvent(e);
					return;
				}
			}
			if(d && this._opened && e.charOrCode == dojo.keys.ESCAPE){
				this.closeDropDown();
				dojo.stopEvent(e);
			}else if(!this._opened &&
					(e.charOrCode == dojo.keys.DOWN_ARROW ||
						( (e.charOrCode == dojo.keys.ENTER || e.charOrCode == " ") &&
						  //ignore enter and space if the event is for a text input
						  ((target.tagName || "").toLowerCase() !== 'input' ||
						     (target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				dojo.stopEvent(e);
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					setTimeout(dojo.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Don't focus on button if the user has explicitly focused on something else (happens
			// when user clicks another control causing the current popup to close)..
			// But if focus is inside of the drop down then reset focus to me, because IE doesn't like
			// it when you display:none a node with focus.
			var focusMe = dijit._curFocus && this.dropDown && dojo.isDescendant(dijit._curFocus, this.dropDown.domNode);

			this.closeDropDown(focusMe);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns whether or not the dropdown is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/* Function */ loadCallback){
			// summary:
			//		Loads the data for the dropdown, and at some point, calls
			//		the given callback.   This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// tags:
			//		protected

			loadCallback();
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			if(!this._opened){
				// If we aren't loaded, load it first so there isn't a flicker
				if(!this.isLoaded()){
					this.loadDropDown(dojo.hitch(this, "openDropDown"));
					return;
				}else{
					this.openDropDown();
				}
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit.popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit.popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				this._preparedNode = true;
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				dojo.style(ddNode, myStyle);
				
				// Figure out maximum height allowed (if there is a height restriction)
				var maxHeight = this.maxHeight;
				if(maxHeight == -1){
					// limit height to space available in viewport either above or below my domNode
					// (whichever side has more room)
					var viewport = dojo.window.getBox(),
						position = dojo.position(aroundNode, false);
					maxHeight = Math.floor(Math.max(position.y, viewport.h - (position.y + position.h)));
				}

				// Attach dropDown to DOM and make make visibility:hidden rather than display:none
				// so we call startup() and also get the size
				if(dropDown.startup && !dropDown._started){
					dropDown.startup();
				}

				dijit.popup.moveOffScreen(dropDown);
				// Get size of drop down, and determine if vertical scroll bar needed
				var mb = dojo._getMarginSize(ddNode);
				var overHeight = (maxHeight && mb.h > maxHeight);
				dojo.style(ddNode, {
					overflowX: "hidden",
					overflowY: overHeight ? "auto" : "hidden"
				});
				if(overHeight){
					mb.h = maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = aroundNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, aroundNode.offsetWidth);
				}else{
					delete mb.w;
				}
				
				// And finally, resize the dropdown to calculated height and width
				if(dojo.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					dojo.marginBox(ddNode, mb);
				}
			}

			var retVal = dijit.popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: dijit.getPopupAroundAlignment((this.dropDownPosition && this.dropDownPosition.length) ? this.dropDownPosition : ["below"],this.isLeftToRight()),
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					dojo.attr(self._popupStateNode, "popupActive", false);
					dojo.removeClass(self._popupStateNode, "dijitHasDropDownOpen");
					self._opened = false;
				}
			});
			dojo.attr(this._popupStateNode, "popupActive", "true");
			dojo.addClass(self._popupStateNode, "dijitHasDropDownOpen");
			this._opened=true;

			// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._opened){
				if(focus){ this.focus(); }
				dijit.popup.close(this.dropDown);
				this._opened = false;
			}
		}

	}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.DashboardDropdown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.DashboardDropdown"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.DashboardDropdown");










dojo.declare("com.ibm.bpm.social.portal.DashboardDropdown", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                             dijit._Widget,
                                                             dijit._Templated], {

  templateString: "<div class=\"dashboard-dropdown\"/>",
  topic: "com.ibm.bpm.social.portal.DashboardDropdown.menuItemSelected",//only used inside this widget
  dropdownMenu: null,
  dropdownOpen: false,

  scoreboardMessageKeyMapping: {
    "scoreboard.MyPerf.name": "pmmyperformance",
    "scoreboard.MyTeamPerf.name": "pmmyteamperformance",
    "scoreboard.ProcPerf.name": "pmprocessperformance",
    "scoreboard.SLAView.name": "pmslaoverview",
    "Ad Hoc Reports": "adhocreports"
  },

  postCreate: function(){
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this.systemRestHandler = com.ibm.bpm.social.util.common.SystemRestHandler.getInstance();
    com.ibm.bpm.social.portal.DashboardDropdown.id = this.domNode.id;
    this._getExposedItems();
    var dashboardMenu = this._setMenuItems(com.ibm.bpm.social.portal.exposedItems["data"]);
    var node = dojo.byId("processPortalDashboardTabDropDownId");
    this.dropdownMenu = new com.ibm.bpm.social.portal.DropdownMenu({
      dropDown: dashboardMenu,
      dropDownContainer: this.domNode,
      _buttonNode: node,
      _popupStateNode: node,
      _aroundNode: node
    });
    this.dropdownMenu.startup();
    dojo.subscribe(this.topic, dojo.hitch(this, this.menuItemSelected));
    dojo.subscribe(this.topic, dojo.hitch(this, this.hide));
    this.dropdownMenu.loadDropDown();
    this.adjustDropdownStyle();

    this.connect(this.dropdownMenu.dropDownNode, "onblur", "hide");
    schuetza = this.dropdownMenu;
  },

  adjustDropdownStyle: function(){
    var node = dojo.byId("processPortalDashboardTabDropDownId_dropdown");
    if (node) {
      dojo.addClass(node, "processPortalDropDownMenu");
      dojo.addClass(node, "tundra");
    }
    // Move menu over to align with the left side of the gray column hover
    var nodes = dojo.query(".dijitMenuPopup");
    if (nodes && nodes[0]) {
      if (nodes[0]) {
        var attr = dojo.attr(nodes[0], "dijitpopupparent");
        if (attr && attr.indexOf("com_ibm_bpm_social_portal_DropdownMenu_") > -1) {
          if (document.body.dir != "rtl") {
            attr = dojo.style(nodes[0], "left");
            if (attr) {
              var num = parseInt(attr);
              if (num > 30) {
                num = num - 30;
                dojo.style(nodes[0], "left", num + "px");
                dojo.style(nodes[0], "top", "45px");
              }
            }
          }
          else {
            attr = dojo.style(nodes[0], "right");
            if (attr) {
              var num = parseInt(attr);
              if (num > 30) {
                num = num - 30;
                dojo.style(nodes[0], "right", num + "px");
                dojo.style(nodes[0], "left", "auto");
                dojo.style(nodes[0], "top", "45px");
              }
            }
          }
        }
      }
    }
  },

  _getExposedItems: function(){
    if (!com.ibm.bpm.social.portal.exposedItems) {
      com.ibm.bpm.social.portal.exposedItems = null;
      // dojo.xhrGet({
      // url: this.getProxyUrl()+"/rest/bpm/wle/v1/exposed",
      // handleAs: "json",
      // preventCache: true,
      // sync: true,
      // headers: {
      // "Content-Type": "application/json",
      // "Accept": "application/json"
      // },
      // load: function(data, xhrArgs){
      // com.ibm.bpm.social.portal.exposedItems = data;
      // }
      // });

      this.systemRestHandler.getExposedItems(function(data, xhrArgs){
        com.ibm.bpm.social.portal.exposedItems = data;
      }, dojo.hitch(this, this._onXhrErrors), false);
    }
  },

  _setMenuItems: function(data){
  
	//TEMPORARY
	//data["exposedItemsList"] = dojo.filter(data["exposedItemsList"], function(item){
	//	return item.display.indexOf("Process Performance")<0 && item.display.indexOf("Team Performance")<0; //TODO NLS-THIS
	//});
	//END TEMPORARY
	
    var dashboardMenu = new dijit.Menu();
    if (!com.ibm.bpm.social.portal.menuItems) {
      com.ibm.bpm.social.portal.menuItems = [];
      // 1. Add scoreboards
      //this._processMenuItems(data, dashboardMenu, "scoreboard");
      // 2. Add separator
      //dashboardMenu.addChild(new dijit.MenuSeparator());
      // 3. Add Dashboards
      this._processMenuItems(data, dashboardMenu, "dashboard");
      // 4. Add Ad Hoc Reports  - Changed to a newer dashboard, as it can be re-exposed by users at runtime
      //var display = "Ad Hoc Reports";
      //var runURL = window.location.protocol + "//" + window.location.host + "/portal/jsp/viewGeneratedReports.do";
      //this._addMenuItem(dashboardMenu, display, runURL);
    }
    return dashboardMenu;
  },

  _processMenuItems: function(data, dashboardMenu, typeArg){
    // If there is no tip but there are more than one snapshots then show the most recent snapshot.
    // If there is a tip then show it.
    // Disambiguate display names as needed.
    var items = [];
    var tipExistsForApp = {};
    var numDuplicateProcesses = {};
    var filteredItems = [];
    if(!data["exposedItemsList"]) {
      return;
    }
    // This loop collects data about the items. The data will enable us to decide which items to display and
    //   what the display name of an item should be.
    for ( var i = 0; i < data["exposedItemsList"].length; i++) {
      var item = data["exposedItemsList"][i];
      var itemMatchesType = false;
      if (typeArg == "scoreboard" && (item["type"] == "scoreboard")) {
        itemMatchesType = true;
      }
      else if (typeArg == "dashboard" && (item["type"] == "service" && item["subtype"] == "dashboard")) {
        itemMatchesType = true;
      }
      if (itemMatchesType) {
        filteredItems.push(item);
        //organize the items so we can tell if there are duplcates
        var itemID = item.itemID;
        var baseID = item.ID;
        // if the snapshotCreatedOn field doesn't exist, use ID
        var snapshotCreateDate = (item.snapshotCreatedOn) ? item.snapshotCreatedOn : baseID;

        if (typeof (tipExistsForApp[itemID]) == 'undefined') {
          tipExistsForApp[itemID] = {
            'tipExists': false,
            'baseID': baseID,
            'snapshotCreateDate': snapshotCreateDate
          };
        }
        if (item.tip) {
          tipExistsForApp[itemID].tipExists = true;
        }
        else {
          // save the most recent snapshot - if there is no tip, we will use the item with the highest ID as the tip
          if (tipExistsForApp[itemID].snapshotCreateDate < snapshotCreateDate) {
            tipExistsForApp[itemID].baseID = baseID;
            tipExistsForApp[itemID].snapshotCreateDate = snapshotCreateDate;
          }
        }
      }
    }
    // This loop determines if an item should be displayed -- it is displayed if it is a tip or if it is
    //   the most recent snapshot.  It also keeps track of duplicate names.
    for ( var i = 0; i < filteredItems.length; i++) {
      var item = filteredItems[i];
      var itemID = item.itemID;
      var baseID = item.ID;
      if ((tipExistsForApp[itemID].tipExists == true && item.tip == true) || tipExistsForApp[itemID].tipExists == false &&
        tipExistsForApp[itemID].baseID == baseID) {
        items.push(item);

        var title = item.display;
        // for scoreboards, use 'title' field if it exists
        if (item["type"] == "scoreboard" && (typeof item.title != "undefined" && item.title)) {
          title = item.title;
        }

        // track items by display name, so we can know if we need to disambiguate the display name
        //  we need to use itemID + snapshotID in order to distinguish between items
        var itemUniqueID = itemID + item.snapshotID;
        item["itemUniqueID"] = itemUniqueID;
        if (typeof (numDuplicateProcesses[title]) == 'undefined') {
          numDuplicateProcesses[title] = {
            'itemsArray': [item]
          }; // save the entire item
        }
        else {
          var idFound = false;
          for ( var j = 0; j < numDuplicateProcesses[title].itemsArray.length; j++) {
            if (itemUniqueID == numDuplicateProcesses[title].itemsArray[j].itemUniqueID) {
              idFound = true;
              break;
            }
          }

          if (!idFound) {
            numDuplicateProcesses[title].itemsArray.push(item);
          }
        }
      }
    }

    if (items && items.length > 0) {
      // This loop disambiguates names if necessary and adds items to the menu list.
      for ( var i = 0; i < items.length; i++) {
        var currentItem = items[i];
        var title = currentItem.display;
        // for scoreboards, use 'title' field if it exists
        if (item["type"] == "scoreboard" && (typeof currentItem.title != "undefined" && currentItem.title)) {
          title = currentItem.title;
        }
        var baseID = currentItem.ID;
        var itemID = currentItem.itemID;
        var appId = currentItem.processAppID;
        var itemUniqueID = currentItem["itemUniqueID"];
        //decide whether any additional identifying information should be displayed
        if (numDuplicateProcesses[title].itemsArray.length > 1) {
          //    Loop over items with the same title. If the items are in the same Process App/Toolkit but in a different track (also called branch or workspace)
          //    then add the branch name
          var useBranchName = false;
          var getTaskTemplate = false;
          for ( var x = 0; x < numDuplicateProcesses[title].itemsArray.length; x++) {
            // don't compare an item to itself!
            var arrayItem = numDuplicateProcesses[title].itemsArray[x];
            if (itemUniqueID != arrayItem.itemUniqueID) {
              if (itemID == arrayItem.itemID && appId == arrayItem.processAppID && currentItem.tip == true &&
                arrayItem.tip == true) {
                useBranchName = true;
              }
              else if (itemID != arrayItem.itemID && appId != arrayItem.processAppID) {
                getTaskTemplate = true;
              }
            }
          }
          if (useBranchName) {
            // disambiguate using branchName (if branchName doesn't exist - i.e., isn't even in the object -  or if it is null, use snapshotName)
            //  (we don't need to call REST for this information)
            var branchName =
              (typeof currentItem.branchName != "undefined" && currentItem.branchName) ? currentItem.branchName
                : currentItem.snapshotName;
            title = title + " (" + branchName + ")";
          }
          else if (getTaskTemplate) {
            // need to get more information for disambiguation
            this.taskTemplData = "";
			var myself=this;
			var async=false;
            this.taskRestHandler.getTaskTemplateDetails(baseID, function(data, xhrArgs){
              myself.taskTemplData = data;
            }, dojo.hitch(this, this._onXhrErrors), async);

            // this might be a process app or a toolkit
            var name1 = (myself.taskTemplData.processAppName) ? myself.taskTemplData.processAppName : myself.taskTemplData.topLevelToolkitName;
            title = title + " (" + name1 + ")";
          }
        }
        this._addMenuItem(dashboardMenu, title, currentItem["runURL"]);
      }
    }
  },

  _addMenuItem: function(dashboardMenu, display, runURL){
    if (display && runURL) {
      display = this._getDisplayName(display);
      var item = new dijit.MenuItem({
        label: display,
        onClick: this._menuItemClick,
        onKeyDown: this._menuKeyDown
      });
      dashboardMenu.addChild(item);
      com.ibm.bpm.social.portal.menuItems.push({
        "display": display,
        "id": item.id,
        "runURL": runURL
      });
    }
  },

  _onXhrErrors: function(data, xhrArgs){
    console.debug(["DashboardDropdown experienced an error.",
                   data,
                   xhrArgs]);
  },

  _getDisplayName: function(displayName){
    var msgKey = this.scoreboardMessageKeyMapping[displayName];
    if (msgKey && this.messages[msgKey]) {
      displayName = this.messages[msgKey];
    }
    return displayName;
  },

  _menuItemClick: function(evt){
    dojo.publish("com.ibm.bpm.social.portal.DashboardDropdown.menuItemSelected", [{
      display: evt.target.innerHTML,
      itemID: evt.currentTarget.id
    }]);
  },

  _menuKeyDown: function(evt){
    var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
    if (keypressed == dojo.keys.ENTER || keypressed == dojo.keys.SPACE) {
      var target = evt.target;
      if (dojo.isIE) {
        target = evt.target.childNodes[1];
      }
      else {
        target = evt.target.childNodes[3];
      }
      dojo.publish("com.ibm.bpm.social.portal.DashboardDropdown.menuItemSelected", [{
        display: target.innerHTML,
        itemID: evt.currentTarget.id
      }]);
      return;
    }
  },

  menuItemSelected: function(msg){
    for ( var i in com.ibm.bpm.social.portal.menuItems) {
      var currentItem = com.ibm.bpm.social.portal.menuItems[i];
      if (msg.itemID == currentItem.id) {
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: currentItem.runURL,
          display: currentItem.display
        }]);
        break;
      }
    }
  },

  show: function(){
    if (this.dropdownMenu) {
      this.dropdownMenu.loadDropDown();
    }
    this.adjustDropdownStyle();
    this.dropdownOpen = true;
  },

  hide: function(){
    if (this.dropdownMenu) {
      this.dropdownMenu.closeDropDown(false);
    }
    this.dropdownOpen = false;
  },

  destroy: function(){
    if (this.dropdownMenu) {
      if (this.dropdownMenu.destroyRecursive) {
        this.dropdownMenu.destroyRecursive();
      }
      else if (this.dropdownMenu.destroy) {
        this.dropdownMenu.destroy();
      }
    }
    dojo.unsubscribe(this.topic);
    this.inherited(arguments);
  }

});

dojo.provide("com.ibm.bpm.social.portal.DropdownMenu");



dojo.declare("com.ibm.bpm.social.portal.DropdownMenu", [dijit._Container,
                                                        dijit._HasDropDown,
                                                        dijit._Widget], {
  // summary:
  //		A menu drop down
  //
  // example:
  // |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
  // |		<div dojotype="dijit.Menu">...</div>
  // |	</button>
  //
  // example:
  // |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
  // |	dojo.body().appendChild(button1);
  //


  templateString: "",

  startup: function(){
    if (this._started) {
      return;
    }
    // the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
    // make it invisible, and store a reference to pass to the popup code.
    if (!this.dropDown && this.dropDownContainer) {
      var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
      this.dropDown = dijit.byNode(dropDownNode);
      delete this.dropDownContainer;
    }
    if (this.dropDown) {
      dijit.popup.hide(this.dropDown);
    }


    this.inherited(arguments);
  },

  isLoaded: function(){
    // Returns whether or not we are loaded - if our dropdown has an href,
    // then we want to check that.
    var dropDown = this.dropDown;
    return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
  },

  loadDropDown: function(){
    // Loads our dropdown
    var dropDown = this.dropDown;
    if (!dropDown) {
      return;
    }
    if (!this.isLoaded()) {
      var handler = dojo.connect(dropDown, "onLoad", this, function(){
        dojo.disconnect(handler);
        this.openDropDown();
        dropDown.focus();
      });
      dropDown.refresh();
    }
    else {
      this.openDropDown();
      dropDown.focus();
    }
  },

  isFocusable: function(){
    // Overridden so that focus is handled by the _HasDropDown mixin, not by
    // the _FormWidget mixin.
    return this.inherited(arguments) && !this._mouseDown;
  },

  focus: function(/*String*/position){
    // summary:
    //		Focuses this widget to according to position, if specified,
    //		otherwise on arrow node
    // position:
    //		"start" or "end"
    if (!this.disabled) {
      dijit.focus(position == "start" ? this.titleNode : this._popupStateNode);
    }
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.ProcessPortalTab"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.ProcessPortalTab"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.ProcessPortalTab");


dojo.declare(
	"com.ibm.bpm.social.portal.ProcessPortalTab",
	dijit._Widget , 
	{
	
	CLASS_NAME: "ProcessPortalTab",
	
	LOG_LEVEL: 1,
	params: null,
	savedParams: null, // if this tab is used to display a snapshot, savedParams will store the original params 
	savedTextScrollWidth: null, 
	isWorkTab: false,
	workTabBaseID: null,
	workTabTitle: null,
	isProcessServer: false,
	ffscrollwidth: null,

    postCreate: function postCreate_ProcessPortalTab() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if (this.params && this.params.isWorkTab && this.params.isWorkTab==true) {
			this.isWorkTab=true;
			this.workTabBaseID=this.params.baseID;
			this.workTabTitle=this.params.title;
			this._createWorkTab();
		} else {
			this._createTab();
		}
		
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_createTab: function _createTab_ProcessPortalTab() {
		dojo.addClass(this.domNode,'processPortalTab');
		dojo.addClass(this.domNode,'dashboardTab');
		var icon = document.createElement('div');
		icon.className = this.params.iconNamePrefix+'_processPortalDashboardTabIcon';  // the icon is specific to the dashboard/scoreboard/saved-search type
		dojo.attr(icon, "id", this.params.baseID+"_iconID"); // so we can find this icon later
		this.domNode.appendChild(icon);
		var txt = document.createElement('div');
		txt.className = 'processPortalWorkTabText';
		this.domNode.appendChild(txt);
		var a = document.createElement('a');
		a.href = 'javascript:void(0);';
		txt.appendChild(a);
		this.connect(a,'onclick','_handleTabClick');
        var text2 = dojo.create("div", {
          	'class': "processPortalDashboardTabTextDiv",
        }, a);
        text2.appendChild(document.createTextNode(this.params.title));			        
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);				
        //a11y: fix no visual focus indicator by default or by css only in IE; chrome needs it also for white outline
        this.connect(a,'onfocus','_handleTabFocus');
        this.connect(a,'onblur','_handleTabBlur');
	},
	
	_createWorkTab: function _createWorkTab_ProcessPortalTab() {
		dojo.attr(this.domNode, 'id', "processPortalWorkTab");
		dojo.addClass(this.domNode,'processPortalTab');
		dojo.addClass(this.domNode,'dashboardTab');//??????????//
		dojo.attr(this.domNode,'data-page',"workPageDiv");
		dojo.attr(this.domNode,'dojoAttachPoint',"workTab");
		var icon = document.createElement('div');
		icon.className = 'processPortalWorkTabIcon';
		dojo.attr(icon, "id", this.params.baseID+"_iconID"); // so we can find this icon later
		this.domNode.appendChild(icon);
		var txt = document.createElement('div');
		txt.className = 'processPortalWorkTabText';
		this.domNode.appendChild(txt);
		var a = document.createElement('a');
		a.id  = 'processPortalWorkLink';
		a.href = 'javascript:void(0);';
		txt.appendChild(a);
		this.connect(a,'onclick','_handleTabClick');
        var text2 = dojo.create("div", {
          	'class': "processPortalDashboardTabTextDiv",
        }, a);
        text2.appendChild(document.createTextNode(this.params.title));			        
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);		
		this.domNode.title = this.domNode.textContent;
		if (dojo.isIE < 9)
			this.domNode.title = this.domNode.innerText;		
        this.connect(a,'onfocus','_handleTabFocus');//a11y
        this.connect(a,'onblur','_handleTabBlur');//a11y
	},
	
	getCurrentWidth: function getCurrentWidth_ProcessPortalTab() {
		var w = 13 + this.domNode.offsetWidth;
		return w;
	},

	getCurrentTextWidth: function getCurrentTextWidth_ProcessPortalTab() {
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		if (div) {
			return div.offsetWidth;
		}
	},

	getSnapshotTextWidth: function getSnapshotTextWidth_ProcessPortalTab() {
		var snapshotTextDiv = dojo.query(".processPortalDashboardTabTextDivSnapshot",this.domNode)[0];
		var offsetWidth=0;
		if (snapshotTextDiv) {
			offsetWidth = snapshotTextDiv.offsetWidth;
		}
		return offsetWidth;
	},

	setCurrentTextWidth: function setCurrentTextWidth_ProcessPortalTab(w) {
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		var snapWidth = this.getSnapshotTextWidth();
		if (div) {
			div.title = div.textContent;
			
			if (dojo.isIE < 9)
				div.title = div.innerText;
			
			if (w) {
				if (div.scrollWidth < w) {
					div.style.width = div.scrollWidth + 'px';
				} else {
					div.style.width = w + 'px';
				}
			} else {
				if (snapWidth > div.scrollWidth) {
					div.style.width = snapWidth + 'px';
				} else {
					//Fix for defect 133534, for ff on mac, need to used the scrollWidth+1 to avoid truncation
					if (dojo.isFF && navigator.userAgent.toLowerCase().indexOf("mac os")>=0) {
						if (div.style.width == "" || div.style.width == null || typeof(div.style.width) === undefined) {							
							div.style.width = (div.scrollWidth+1) + 'px';
							if (this.ffscrollwidth == null) {
								this.ffscrollwidth = div.scrollWidth;
							}
						} else {
							if (this.ffscrollwidth != null) {
								div.style.width = this.ffscrollwidth + 'px';
							} else {
								div.style.width = div.scrollWidth + 'px';
							}
						}					
					} else {
						div.style.width = div.scrollWidth + 'px';
					}
				}
			}
		}
	},

	hideIcon: function hideIcon_ProcessPortalTab() {
		var icon=null;
		icon = dojo.byId(this.params.baseID+"_iconID");
		if (icon!=null) {
			icon.style.display = 'none';
		}
	},

	showIcon: function showIcon_ProcessPortalTab() {
		var icon=null;
		icon = dojo.byId(this.params.baseID+"_iconID");
		if (icon!=null) {
			icon.style.display = 'block';
		}
	},

	_handleViewReady: function _handleViewReady_ProcessPortalTab() {
		if (this.viewReadySub) {
			dojo.unsubscribe(this.viewReadySub);
			this.viewReadySub = null;
			this._handleTabClick();
		}
	},
	
	_handleTabClick: function _handleTabClick_ProcessPortalTab(e) {
		if (e) {
			dojo.stopEvent(e);
		}
	    if (!com.ibm.bpm.social.portal.PageHandler.getInstance().isNavigationAllowed()) {
	    	return;
	    }
		if (this.params.type == 'queue') {
		   if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
		    com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
					savedSearchName: this.params.name
				}]);
		    } else {
			this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,this,'_handleViewReady');
   		      com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
		    }
		} else if (this.params.isWorkTab && this.params.isWorkTab==true) {
			com.ibm.bpm.social.portal.PageHandler.getInstance().selectWorkTab();		
		} else {
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
				runURL: this.params.runURL,
				display: this.params.title
			}]);
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this.params)]);
	},
	
	saveOriginalParams: function saveOriginalParams_ProcessPortalTab(params) {
		this.savedParams = params;
	},
	
	setNewParams: function setNewParams_ProcessPortalTab(newParams) {
		// Called when this tab will be used to display a snapshot; save the original tab params
		this.saveOriginalParams(this.params);
		this.params = newParams;
		// save the width of the tab text so it can be restored when the tab no longer displays the snapshot name
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		if (div) {
				this.savedTextScrollWidth = div.scrollWidth;
		}
		// if this is a snapshot, add the snapshot name under the dashboard name
		if (this.params.isSnapshot!=undefined && this.params.isSnapshot==true) {
			var textNode = dojo.query(".processPortalDashboardTabTextDiv", this.domNode);
			if (textNode[0]) {
				var subText = dojo.create("div", {
					'class': "processPortalDashboardTabTextDivSnapshot",
				}, textNode[0]);
				subText.appendChild(document.createTextNode(this.params.snapshotName));	
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(subText);	
				var parentWidth = textNode[0].scrollWidth;
				var widthToUse = Math.max(parentWidth, subText.scrollWidth);
				dojo.style(textNode[0], "width", "inherit");
				subText.style.width = 'inherit';
			}
		}
		
	},
	
	restoreOriginalParams: function restoreOriginalParams_ProcessPortalTab() {
		var textNode=null;
		if (this.savedTextScrollWidth!=null) {
			textNode = dojo.query(".processPortalDashboardTabTextDiv", this.domNode);
			if (textNode[0]) {
				textNode[0].style.width = this.savedTextScrollWidth + 'px';
			}
		}
		textNode = dojo.query(".processPortalDashboardTabTextDivSnapshot", this.domNode);
		if (textNode[0]) {
			dojo.destroy(textNode[0]);
		}
		this.params = this.savedParams;
		this.savedParams=null;
		this.savedTextScrollWidth=null;
	},

    _handleTabFocus: function _handleTabFocus_ProcessPortalTab(evt) {
         var node = evt.target.firstChild;
         if (node) { dojo.addClass(node,"focus"); }
    },

    _handleTabBlur: function _handleTabBlur_ProcessPortalTab(evt) {
         var node = evt.target.firstChild;
         if (node) { dojo.removeClass(node,"focus"); }
    },

	destroy: function destroy_ProcessPortalTab() {
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
	
});	

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.ProcessPortalTabBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.ProcessPortalTabBar"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.ProcessPortalTabBar");






dojo.declare(
	"com.ibm.bpm.social.portal.ProcessPortalTabBar",
	[dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "ProcessPortalTabBar",
	
	LOG_LEVEL: 1,
	
	tabList: null,
	tempTabParams: null,
	snapshotParams : null,
	snapshotTab : null,
	organizeTabsText: "",
	textZoom: 0,
	textZoomTimer: 0,
	isProcessServer: false,
	mumContextRoot: window.endpointURLObject.mumJS,
	
	
	templateString:"\t<div id=\"processPortalNavigation\" class=\"processPortalNavigation\">\n\t\t\t<div id=\"dashboardOrganizerTab\" class=\"processPortalTab\" data-page=\"dashboardPageDiv\" data-widget=\"dashboardPage\" dojoAttachPoint=\"organizerTab\">\n\t\t\t\t<div class=\"processPortalDashboardOrganizerTabIcon\" title=\"${organizeTabsText}\">\n                    <a onclick=\"return com.ibm.bpm.social.portal.PageHandler.getInstance().onDashboardOrganizerClick(event);\" aria-label=\"${organizeTabsText}\"  href=\"javascript:void(0);\">\n                    <img src=\"${mumContextRoot}/mycontenthandler/mm/dav/filestore/public/themes/processPortalTheme/img/blank.gif\" alt=\"\" id=\"processPortalDashboardTabDropDownImg\" width=\"22\" height=\"22\"/>\n                    <span class=\"processPortalDashboardOrganizerTabIconHighContrast\">&#8853;</span> \n                    </a>\n                </div>\n\t\t\t\t<div class=\"processPortalDashboardTabText\" id=\"processPortalDashboardOrganizerId\">\n                    <a id=\"processPortalDashboardOrganizerLink\" onclick=\"return com.ibm.bpm.social.portal.PageHandler.getInstance().onDashboardOrganizerClick(event);\" href=\"javascript:void(0);\" dojoAttachPoint=\"organizerTabText\" style=\"display:none\">\n                    \t<div class=\"processPortalDashboardTabTextDiv\" dojoAttachPoint=\"organizerTabTextDiv\"></div>\n                    </a>\n                </div>\n            </div>\n\t\t\t<div class=\"preloadHandCursors\"><img src=\"${mumContextRoot}/mycontenthandler/mm/dav/filestore/public/themes/processPortalTheme/img/closedhand.cur\" alt=\"\" />\n\t\t\t\t\t\t\t\t\t\t\t<img src=\"${mumContextRoot}/mycontenthandler/mm/dav/filestore/public/themes/processPortalTheme/img/openhand.cur\" alt=\"\" /></div>\n\t\t\t\n\t</div>\n",
	constructor: function constructor_ProcessPortalTabBar(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.organizeTabsText = this.messages["dashboardOrganizer-OrganizeTabs"];
	},	
    postCreate: function postCreate_ProcessPortalTabBar() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.tabList = new Array();
		this.updateDashboardTabs();
		this.processPortalTabBarSubscriptions=[];
		this.processPortalTabBarSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS,dojo.hitch(this,function(msg) {
        		this.updateDashboardTabs(msg);
        	})));
		this.processPortalTabBarSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB,dojo.hitch(this,function(msg) {
        		this.selectDashboardTab(msg);
        	})));
		this.connect(window,'onresize','_checkTabWidths');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.textZoomTimer = window.setTimeout(dojo.hitch(this,'_checkForTextZoomChange'),500);
	}, 
	
	updateDashboardTabs: function updateDashboardTabs_ProcessPortalTabBar() {
		var selectedParams = this._removeDashboardTabs();
		var pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();
		var dd = pageHandler.getDisplayedDashboards();
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		// Set variable that indicates if the environment is Process Server.  Note that if there are 0 exposed dashboards
		//  then the value of isProcessServer is both irrelevant and unreliable.
		this.isProcessServer = organizer.getIsProcessServer();
		if (dd) {
			for (var i=0; i<dd.length; i++) {
				var tab = new com.ibm.bpm.social.portal.ProcessPortalTab({params: dd[i].data.params, isProcessServer:this.isProcessServer});
				this.organizerTab.parentNode.insertBefore(tab.domNode,this.organizerTab);
				this.tabList.push(tab);
				// Was this tab showing a snapshot?  If so, show it again
				var snapWasShowing=false;
				if (this.snapshotTab != null && this.snapshotTab.savedParams!=null && this.snapshotTab.savedParams.baseID == tab.params.baseID) {
					snapWasShowing=true;
				}
				if (selectedParams && ((selectedParams.baseID == tab.params.baseID) || 
				   (snapWasShowing && selectedParams.baseID == this.snapshotTab.params.baseID))) {
					dojo.addClass(tab.domNode,"processPortalSelectedTab");
					// tell organizer what the selected tab is
					organizer.setCurrentlySelectedTab(tab.params.baseID);
					if (snapWasShowing && selectedParams.baseID == this.snapshotTab.params.baseID) {
						tab.setNewParams(this.snapshotTab.params);
					}
					selectedParams = null;
				}
				// two cases for removing temp tab
				if (this.tempTabParams) {
					if (this.tempTabParams.baseID == dd[i].data.params.baseID) {
						this._removeTemporaryTab();
						dojo.addClass(tab.domNode,"processPortalSelectedTab");
					}  else if (this.tempTabParams.itemID == dd[i].data.params.itemID) {
						var dashItem=organizer.getSpecificDashboard(this.tempTabParams.baseID);
						if (dashItem!=null) {
							tab.setNewParams(dashItem.data.params);
							this.snapshotParams = dashItem.data.params;
							this.snapshotTab = tab;
							this._removeTemporaryTab();
							dojo.addClass(tab.domNode,"processPortalSelectedTab");
						}
					}
				}
				
				
			}
		}
		if (selectedParams) {
			this._createTemporaryTab(selectedParams);
			dojo.addClass(this.organizerTab,"processPortalSelectedTab");
		}
		// if the organizer tab is the only tab, change its class so it has a bigger margin on left (or right in rtl)
		if (!dd || dd&&dd.length==0) {
			if (dojo.hasClass(this.organizerTab,"processPortalTab")){
				dojo.removeClass(this.organizerTab,"processPortalTab");
				dojo.addClass(this.organizerTab,"processPortalOrganizerTab");
			}
		} else {
			if (dojo.hasClass(this.organizerTab,"processPortalOrganizerTab")){
				dojo.removeClass(this.organizerTab,"processPortalOrganizerTab");
				dojo.addClass(this.organizerTab,"processPortalTab");
			}
		}
		this._checkTabWidths();
	},
	
	_checkForTextZoomChange: function _checkForTextZoomChange_ProcessPortalTabBar() {
		if (this.textZoomTimer) {
			var userName = dojo.byId('processPortalUserName');
			if (userName.offsetHeight != this.textZoom) {
				this._checkTabWidths();
				this.textZoom = userName.offsetHeight;
			}
			this.textZoomTimer = window.setTimeout(dojo.hitch(this,'_checkForTextZoomChange'),500);
		}
	},
	
	_checkTabWidths: function _checkTabWidths_ProcessPortalTabBar() {
		var availableWidth = this.domNode.offsetWidth - 34;
	    var organizerDropSpace = 175 - this.organizerTab.offsetWidth;
		var userDiv = dojo.byId('processPortalUser');
		if (userDiv) {
			availableWidth = availableWidth - userDiv.offsetWidth;
			organizerDropSpace = organizerDropSpace - userDiv.offsetWidth;
			var userName = dojo.byId('processPortalUserName');
			if (userName.offsetWidth == 0) {
				window.setTimeout(dojo.hitch(this,'_checkTabWidths'),250);
			}
		}
		if (organizerDropSpace > 0) {
			availableWidth = availableWidth - organizerDropSpace;
		}
		if (availableWidth > 0) {
			var w = 0;
			for (var i=0; i<this.tabList.length; i++) {
				this.tabList[i].showIcon();
				this.tabList[i].domNode.style.display = 'block';
				this.tabList[i].setCurrentTextWidth();
				w = w + this.tabList[i].getCurrentWidth();
			}
			if (dojo.isFF && navigator.userAgent.toLowerCase().indexOf("mac os")>=0) {
			    this.organizerTabTextDiv.style.width = (this.organizerTabTextDiv.scrollWidth+1) + 'px';
			} else {
				this.organizerTabTextDiv.style.width = this.organizerTabTextDiv.scrollWidth + 'px';
			}		
			w = w + 13 + this.organizerTab.offsetWidth;
			if (w > availableWidth) {
				w = 0;
				var maxTxtWidth = 0;
				for (var i=0; i<this.tabList.length; i++) {
					this.tabList[i].hideIcon();
					var txtWidth = this.tabList[i].getCurrentTextWidth();
					if (txtWidth > maxTxtWidth) {
						maxTxtWidth = txtWidth;
					}
					w = w + this.tabList[i].getCurrentWidth();
				}
				if (this.organizerTabTextDiv.offsetWidth > maxTxtWidth) {
					maxTxtWidth = this.organizerTabTextDiv.offsetWidth;
				}
				w = w + 13 + this.organizerTab.offsetWidth;
				while ((w > availableWidth) && (maxTxtWidth > 25)) {
					maxTxtWidth = maxTxtWidth - 5;
					w = 0;
					for (var i=0; i<this.tabList.length; i++) {
						this.tabList[i].setCurrentTextWidth(maxTxtWidth);
						w = w + this.tabList[i].getCurrentWidth();
					}
					if (this.organizerTabTextDiv.offsetWidth > maxTxtWidth) {
						this.organizerTabTextDiv.style.width = maxTxtWidth + 'px';
					}
					w = w + 13 + this.organizerTab.offsetWidth;
				}
				if (w > availableWidth) {
					var i=this.tabList.length; 
					while((i > 0) && (w > availableWidth)) {
						i--;
						var tab = this.tabList[i];
						if (!dojo.hasClass(tab.domNode,'processPortalSelectedTab')) {
							w = w - tab.getCurrentWidth();
							tab.domNode.style.display = 'none';
						}
					}
				}
			}
			// is there a snapshot version displayed?
			var snapshotTextDiv = dojo.query(".processPortalDashboardTabTextDivSnapshot");
			if (snapshotTextDiv && snapshotTextDiv.length>0) {
				var offsetWidth = snapshotTextDiv[0].offsetWidth;
				var pNode = snapshotTextDiv[0].parentNode;
				if (pNode && offsetWidth > pNode.offsetWidth) {
					pNode.style.width = offsetWidth + 'px';
				}
				dojo.style(snapshotTextDiv, "width", "inherit");
			}
		} else {
			// no width, so hide all tabs
			for (var i=0; i<this.tabList.length; i++) {
				this.tabList[i].domNode.style.display = 'none';
			}
		}
		// always hide the drop down when changing the tabs
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		if (organizer.dropdownOpen) {
			organizer.hide(true);
		}
	},
	
	_removeDashboardTabs: function removeDashboardTabs_ProcessPortalTabBar() {
		var selectedParams = null;
		while (this.tabList.length > 0) {
			var tab = this.tabList.pop();
			if (dojo.hasClass(tab.domNode,"processPortalSelectedTab")) {
				selectedParams = tab.params;
			}
			tab.destroy();
		}
		return selectedParams;
	},
	
	_createTemporaryTab: function createTemporaryTab_ProcessPortalTabBar(params) {
		// See Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB() for the structure of the params argument
		this._removeTemporaryTab();
		this.tempTabParams = params;
		this.organizerTabTextDiv.innerHTML = ""; 
		dojo.style(this.organizerTabTextDiv, "fontWeight", "bold");
		if (params.title) {
			this.organizerTabTextDiv.appendChild(document.createTextNode(params.title));			
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.organizerTabTextDiv);	
			var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
			if (orgDiv) {
				dojo.removeClass(orgDiv,"hasSnapshot");
			}
			dojo.removeClass(this.organizerTab,"hasSnapshot");
			if (params.isSnapshot!=undefined && params.isSnapshot==true) {
				var subText = dojo.create("div", {
					'class': "processPortalDashboardTabTextDivSnapshot",
				}, this.organizerTabTextDiv);
				var snapText = document.createTextNode(params.snapshotName);
				subText.appendChild(snapText);	
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(subText);
				if (orgDiv) {
					dojo.addClass(orgDiv,"hasSnapshot");
				}
			}
			this.organizerTabTextDiv.title = this.organizerTabTextDiv.textContent;
			if (dojo.isIE < 9)
				this.organizerTabTextDiv.title = this.organizerTabTextDiv.innerText;
		}
		
		this.organizerTabText.style.display = 'inline';
		dojo.addClass(this.organizerTab,"processPortalTemporaryTab");
		// tell the organizer the id of the selected tab
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		// tell the organizer the id of the selected tab; 
		organizer.setCurrentlySelectedTabByItemID(params.itemID);
	},

	_removeTemporaryTab: function _removeTemporaryTab_ProcessPortalTabBar() {
		dojo.removeClass(this.organizerTab,"processPortalTemporaryTab");
		dojo.removeClass(this.organizerTab,"processPortalSelectedTab");
		var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
		if (orgDiv) {
			dojo.removeClass(orgDiv,"hasSnapshot");
		}
		this.organizerTabText.style.display = 'none';
		this.organizerTabTextDiv.innerHTML = "";
		this.organizerTabTextDiv.style.width = "0";
		this.tempTabParams = null;
	},

	_resetSnapshotTab: function _resetSnapshotTab_ProcessPortalTabBar(baseID) {
		// find the tab to reset
		var tab=null;
		for (var i=0; i<this.tabList.length; i++) {
			if (this.tabList[i].params.baseID == baseID) {
				tab = this.tabList[i];
			}
		}		
		if (tab != null) {
			tab.restoreOriginalParams();
			this.snapshotParams = null;
			this.snapshotTab = null;
		}
	},
	
	selectDashboardTab: function selectDashboardTab_ProcessPortalTabBar(params) {
		// This is called when the topic com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB is published.
		// The structure of the params argument is defined in Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB()
		if (this.tempTabParams) {
			if ((params == undefined) || (this.tempTabParams.baseID != params.baseID)) {
				this._removeTemporaryTab();
			}
		}
		// If a tab is currently showing a snapshot, and the selected tab is not that tab, then reset the tab to its original state
		if (this.snapshotParams) {
			if ((params == undefined) || (this.snapshotParams.baseID != params.baseID)) {
				this._resetSnapshotTab(this.snapshotParams.baseID);
			}
		}
		dojo.query(".processPortalSelectedTab",this.domNode).forEach(function(node) {
			dojo.removeClass(node,"processPortalSelectedTab");
		});
		if (params) {
			var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
			var tab = null;
			for (var i=0; i<this.tabList.length; i++) {
				if (params.baseID && this.tabList[i].params.baseID == params.baseID) {
					tab = this.tabList[i];
				}
			}
			if (tab == null) {
				// See if a different version of this item is already shown in a tab; if so, display it there instead
				//   of creating a temp tab
				var matchIndex=-1;
				for (var i=0; i<this.tabList.length; i++) {
					if (this.tabList[i].params.itemID == params.itemID) {
						matchIndex=i;
						break;
					}
				}
				if (matchIndex>-1) {
					var dashItem=organizer.getSpecificDashboard(params.baseID);
					if (dashItem!=null) {
						tab = this.tabList[matchIndex];
						tab.setNewParams(dashItem.data.params);
						this.snapshotParams = dashItem.data.params;
						this.snapshotTab = tab;
					}
				} else {
					this._createTemporaryTab(params);
					dojo.addClass(this.organizerTab,"processPortalSelectedTab");
				}
			}

			if (tab!=null) {
				// tell the organizer the id of the selected tab; if tab is currently showing a snapshot, use the baseID of the original tab
				var baseID = tab.savedParams!=null ? tab.savedParams.baseID : tab.params.baseID;
				organizer.setCurrentlySelectedTab(baseID);
				dojo.addClass(tab.domNode,"processPortalSelectedTab");
			}
		}
		this._checkTabWidths();
	},
	
	destroy: function destroy_ProcessPortalTabBar() {
		/**
		 * This is also called from theme.html::customLogoutFunction
		 */
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.textZoomTimer = 0;
		if (this.processPortalTabBarSubscriptions && this.processPortalTabBarSubscriptions.length) {
			for(var i=0; i<this.processPortalTabBarSubscriptions.length; i++) {
				dojo.unsubscribe(this.processPortalTabBarSubscriptions[i]);
			}
			this.processPortalTabBarSubscriptions=[];
		}
		this._removeDashboardTabs();
		this._removeTemporaryTab();
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SocialRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SocialRestHandler"] = true;
// SocialRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.util.common.SocialRestHandler");






dojo.declare("com.ibm.bpm.social.util.common.SocialRestHandler", com.ibm.bpm.social.util.common.RestHandler, {
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getCurrentUserAvatar: function getCurrentUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteUserAvatar: function deleteUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getMentions: function getMentions_SocialRestHandler(pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instances/mentions?page="+pagenum+"&pagesize="+pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param tkid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteMentions: function deleteMentions_SocialRestHandler(tkid, load, error, async, systemID) {
		var url = "/social/task/"+tkid+"/mentions";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteAllMentions: function deleteAllMentions_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/task/mentions/all";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getFollowedInstances: function getFollowedInstances_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/instances/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	isFollowingInstance: function isFollowingInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	followInstance: function followInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	unfollowInstance: function unfollowInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertsForTask: function getExpertsForTask_SocialRestHandler(tkiid, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/experts";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param user
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertUserData: function getExpertUserData_SocialRestHandler(tkiid, user, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/userdata?username=" + encodeURIComponent(user);
		
		var url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param comment
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	postTaskStreamComment: function postTaskStreamComment_SocialRestHandler(tkiid, comment, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/comment";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			postData: comment,
			contentType: "application/json",  //Need to override the default set for posts here.
			sync: !async
		});	
	},
	
	/**
	 * 
	 * @param piid
	 * @param since
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	updateStreamEvents: function updateStreamEvents_SocialRestHandler(piid, since, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		
		if(since) {
			url += "since=" + since;
		}
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getStreamEvents: function getStreamEvents_SocialRestHandler(piid, pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		url += "page=" + pagenum + "&pagesize=" + pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	statics: {
		_instance: null,

		_createInstance: function(params){
		  if (this._instance === null) {
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
			// TODO handle params, e.g. 
	//        this._instance.setIWidget(iWidget);
	//        this._instance.setUrl("prefix", this.getServiceURLRoot());
	//        this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
	//        this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
		  }
		  else {
			// programming error
			var error = new Error("RestHandler instance has already been initialized!");
			com.ibm.bpm.social.util.common.Log.consoleError(error.message);
			throw error;
		  }
		  return this._instance;
		},

		_getInstance: function(){
		  if (this._instance === null) {
			// TODO for now return default handler
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
	//        // programming error
	//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
	//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
	//        throw error;
		  }
		  return this._instance;
		}
	}
});

com.ibm.bpm.social.util.common.SocialRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.SocialRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["dojo.io.iframe"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.io.iframe"] = true;
dojo.provide("dojo.io.iframe");

dojo.getObject("io", true, dojo);

/*=====
dojo.declare("dojo.io.iframe.__ioArgs", dojo.__IoArgs, {
	constructor: function(){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply
		//		to this type. The following additional properties are allowed
		//		for dojo.io.iframe.send():
		//	method: String?
		//		The HTTP method to use. "GET" or "POST" are the only supported
		//		values.  It will try to read the value from the form node's
		//		method, then try this argument. If neither one exists, then it
		//		defaults to POST.
		//	handleAs: String?
		//		Specifies what format the result data should be given to the
		//		load/handle callback. Valid values are: text, html, xml, json,
		//		javascript. IMPORTANT: For all values EXCEPT html and xml, The
		//		server response should be an HTML file with a textarea element.
		//		The response data should be inside the textarea element. Using an
		//		HTML document the only reliable, cross-browser way this
		//		transport can know when the response has loaded. For the html
		//		handleAs value, just return a normal HTML document.  NOTE: xml
		//		is now supported with this transport (as of 1.1+); a known issue
		//		is if the XML document in question is malformed, Internet Explorer
		//		will throw an uncatchable error.
		//	content: Object?
		//		If "form" is one of the other args properties, then the content
		//		object properties become hidden form form elements. For
		//		instance, a content object of {name1 : "value1"} is converted
		//		to a hidden form element with a name of "name1" and a value of
		//		"value1". If there is not a "form" property, then the content
		//		object is converted into a name=value&name=value string, by
		//		using dojo.objectToQuery().
		this.method = method;
		this.handleAs = handleAs;
		this.content = content;
	}
});
=====*/

dojo.io.iframe = {
	// summary:
	//		Sends an Ajax I/O call using and Iframe (for instance, to upload files)
	
	create: function(/*String*/fname, /*String*/onloadstr, /*String?*/uri){
		//	summary:
		//		Creates a hidden iframe in the page. Used mostly for IO
		//		transports.  You do not need to call this to start a
		//		dojo.io.iframe request. Just call send().
		//	fname: String
		//		The name of the iframe. Used for the name attribute on the
		//		iframe.
		//	onloadstr: String
		//		A string of JavaScript that will be executed when the content
		//		in the iframe loads.
		//	uri: String
		//		The value of the src attribute on the iframe element. If a
		//		value is not given, then dojo/resources/blank.html will be
		//		used.
		if(window[fname]){ return window[fname]; }
		if(window.frames[fname]){ return window.frames[fname]; }
		var cframe = null;
		var turi = uri;
		if(!turi){
			if(dojo.config["useXDomain"] && !dojo.config["dojoBlankHtmlUrl"]){
				console.warn("dojo.io.iframe.create: When using cross-domain Dojo builds,"
					+ " please save dojo/resources/blank.html to your domain and set djConfig.dojoBlankHtmlUrl"
					+ " to the path on your domain to blank.html");
			}
			turi = (dojo.config["dojoBlankHtmlUrl"]||dojo.moduleUrl("dojo", "resources/blank.html"));
		}
		var cframe = dojo.place(
			'<iframe id="'+fname+'" name="'+fname+'" src="'+turi+'" onload="'+onloadstr+
			'" style="position: absolute; left: 1px; top: 1px; height: 1px; width: 1px; visibility: hidden">',
		dojo.body());

		window[fname] = cframe;

		return cframe;
	},

	setSrc: function(/*DOMNode*/iframe, /*String*/src, /*Boolean*/replace){
		//summary:
		//		Sets the URL that is loaded in an IFrame. The replace parameter
		//		indicates whether location.replace() should be used when
		//		changing the location of the iframe.
		try{
			if(!replace){
				if(dojo.isWebKit){
					iframe.location = src;
				}else{
					frames[iframe.name].location = src;
				}
			}else{
				// Fun with DOM 0 incompatibilities!
				var idoc;
				if(dojo.isIE || dojo.isWebKit){
					idoc = iframe.contentWindow.document;
				}else{ //  if(d.isMozilla){
					idoc = iframe.contentWindow;
				}
	
				//For Safari (at least 2.0.3) and Opera, if the iframe
				//has just been created but it doesn't have content
				//yet, then iframe.document may be null. In that case,
				//use iframe.location and return.
				if(!idoc){
					iframe.location = src;
					return;
				}else{
					idoc.location.replace(src);
				}
			}
		}catch(e){
			console.log("dojo.io.iframe.setSrc: ", e);
		}
	},

	doc: function(/*DOMNode*/iframeNode){
		//summary: Returns the document object associated with the iframe DOM Node argument.
		var doc = iframeNode.contentDocument || // W3
			(
				(
					(iframeNode.name) && (iframeNode.document) &&
					(dojo.doc.getElementsByTagName("iframe")[iframeNode.name].contentWindow) &&
					(dojo.doc.getElementsByTagName("iframe")[iframeNode.name].contentWindow.document)
				)
			) ||  // IE
			(
				(iframeNode.name)&&(dojo.doc.frames[iframeNode.name])&&
				(dojo.doc.frames[iframeNode.name].document)
			) || null;
		return doc;
	},

	send: function(/*dojo.io.iframe.__ioArgs*/args){
		//summary:
		//		Function that sends the request to the server.
		//		This transport can only process one send() request at a time, so if send() is called
		//multiple times, it will queue up the calls and only process one at a time.
		if(!this["_frame"]){
			this._frame = this.create(this._iframeName, dojo._scopeName + ".io.iframe._iframeOnload();");
		}

		//Set up the deferred.
		var dfd = dojo._ioSetArgs(
			args,
			function(/*Deferred*/dfd){
				//summary: canceller function for dojo._ioSetArgs call.
				dfd.canceled = true;
				dfd.ioArgs._callNext();
			},
			function(/*Deferred*/dfd){
				//summary: okHandler function for dojo._ioSetArgs call.
				var value = null;
				try{
					var ioArgs = dfd.ioArgs;
					var dii = dojo.io.iframe;
					var ifd = dii.doc(dii._frame);
					var handleAs = ioArgs.handleAs;

					//Assign correct value based on handleAs value.
					value = ifd; //html
					if(handleAs != "html"){
						if(handleAs == "xml"){
							//	FF, Saf 3+ and Opera all seem to be fine with ifd being xml.  We have to
							//	do it manually for IE6-8.  Refs #6334.
							if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){
								dojo.query("a", dii._frame.contentWindow.document.documentElement).orphan();
								var xmlText=(dii._frame.contentWindow.document).documentElement.innerText;
								xmlText=xmlText.replace(/>\s+</g, "><");
								xmlText=dojo.trim(xmlText);
								//Reusing some code in base dojo for handling XML content.  Simpler and keeps
								//Core from duplicating the effort needed to locate the XML Parser on IE.
								var fauxXhr = { responseText: xmlText };
								value = dojo._contentHandlers["xml"](fauxXhr); // DOMDocument
							}
						}else{
							value = ifd.getElementsByTagName("textarea")[0].value; //text
							if(handleAs == "json"){
								value = dojo.fromJson(value); //json
							}else if(handleAs == "javascript"){
								value = dojo.eval(value); //javascript
							}
						}
					}
				}catch(e){
					value = e;
				}finally{
					ioArgs._callNext();
				}
				return value;
			},
			function(/*Error*/error, /*Deferred*/dfd){
				//summary: errHandler function for dojo._ioSetArgs call.
				dfd.ioArgs._hasError = true;
				dfd.ioArgs._callNext();
				return error;
			}
		);

		//Set up a function that will fire the next iframe request. Make sure it only
		//happens once per deferred.
		dfd.ioArgs._callNext = function(){
			if(!this["_calledNext"]){
				this._calledNext = true;
				dojo.io.iframe._currentDfd = null;
				dojo.io.iframe._fireNextRequest();
			}
		};

		this._dfdQueue.push(dfd);
		this._fireNextRequest();
		
		//Add it the IO watch queue, to get things like timeout support.
		dojo._ioWatch(
			dfd,
			function(/*Deferred*/dfd){
				//validCheck
				return !dfd.ioArgs["_hasError"];
			},
			function(dfd){
				//ioCheck
				return (!!dfd.ioArgs["_finished"]);
			},
			function(dfd){
				//resHandle
				if(dfd.ioArgs._finished){
					dfd.callback(dfd);
				}else{
					dfd.errback(new Error("Invalid dojo.io.iframe request state"));
				}
			}
		);

		return dfd;
	},

	_currentDfd: null,
	_dfdQueue: [],
	_iframeName: dojo._scopeName + "IoIframe",

	_fireNextRequest: function(){
		//summary: Internal method used to fire the next request in the bind queue.
		try{
			if((this._currentDfd)||(this._dfdQueue.length == 0)){ return; }
			//Find next deferred, skip the canceled ones.
			do{
				var dfd = this._currentDfd = this._dfdQueue.shift();
			} while(dfd && dfd.canceled && this._dfdQueue.length);

			//If no more dfds, cancel.
			if(!dfd || dfd.canceled){
				this._currentDfd =  null;
				return;
			}

			var ioArgs = dfd.ioArgs;
			var args = ioArgs.args;

			ioArgs._contentToClean = [];
			var fn = dojo.byId(args["form"]);
			var content = args["content"] || {};
			if(fn){
				if(content){
					// if we have things in content, we need to add them to the form
					// before submission
					var pHandler = function(name, value) {
						dojo.create("input", {type: "hidden", name: name, value: value}, fn);
						ioArgs._contentToClean.push(name);
					};
					for(var x in content){
						var val = content[x];
						if(dojo.isArray(val) && val.length > 1){
							var i;
							for (i = 0; i < val.length; i++) {
								pHandler(x,val[i]);
							}
						}else{
							if(!fn[x]){
								pHandler(x,val);
							}else{
								fn[x].value = val;
							}
						}
					}
				}
				//IE requires going through getAttributeNode instead of just getAttribute in some form cases,
				//so use it for all.  See #2844
				var actnNode = fn.getAttributeNode("action");
				var mthdNode = fn.getAttributeNode("method");
				var trgtNode = fn.getAttributeNode("target");
				if(args["url"]){
					ioArgs._originalAction = actnNode ? actnNode.value : null;
					if(actnNode){
						actnNode.value = args.url;
					}else{
						fn.setAttribute("action",args.url);
					}
				}
				if(!mthdNode || !mthdNode.value){
					if(mthdNode){
						mthdNode.value= (args["method"]) ? args["method"] : "post";
					}else{
						fn.setAttribute("method", (args["method"]) ? args["method"] : "post");
					}
				}
				ioArgs._originalTarget = trgtNode ? trgtNode.value: null;
				if(trgtNode){
					trgtNode.value = this._iframeName;
				}else{
					fn.setAttribute("target", this._iframeName);
				}
				fn.target = this._iframeName;
				dojo._ioNotifyStart(dfd);
				fn.submit();
			}else{
				// otherwise we post a GET string by changing URL location for the
				// iframe
				var tmpUrl = args.url + (args.url.indexOf("?") > -1 ? "&" : "?") + ioArgs.query;
				dojo._ioNotifyStart(dfd);
				this.setSrc(this._frame, tmpUrl, true);
			}
		}catch(e){
			dfd.errback(e);
		}
	},

	_iframeOnload: function(){
		var dfd = this._currentDfd;
		if(!dfd){
			this._fireNextRequest();
			return;
		}

		var ioArgs = dfd.ioArgs;
		var args = ioArgs.args;
		var fNode = dojo.byId(args.form);
	
		if(fNode){
			// remove all the hidden content inputs
			var toClean = ioArgs._contentToClean;
			for(var i = 0; i < toClean.length; i++) {
				var key = toClean[i];
				//Need to cycle over all nodes since we may have added
				//an array value which means that more than one node could
				//have the same .name value.
				for(var j = 0; j < fNode.childNodes.length; j++){
					var chNode = fNode.childNodes[j];
					if(chNode.name == key){
						dojo.destroy(chNode);
						break;
					}
				}
			}

			// restore original action + target
			if(ioArgs["_originalAction"]){
				fNode.setAttribute("action", ioArgs._originalAction);
			}
			if(ioArgs["_originalTarget"]){
				fNode.setAttribute("target", ioArgs._originalTarget);
				fNode.target = ioArgs._originalTarget;
			}
		}

		ioArgs._finished = true;
	}
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.AvatarUploadDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.AvatarUploadDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.AvatarUploadDialog");



dojo.registerModulePath("com.ibm.socialPortal.templates", window.endpointURLObject.processPortal +"/com/ibm/bpm/social/widgets/dialogs/templates");

dojo.declare("com.ibm.bpm.social.widgets.dialogs.AvatarUploadDialog", com.ibm.bspace.common.util.ui.Dialog, {
	templateString: dojo.cache("com.ibm.socialPortal.templates", "avatarUpload.html", "<div>\t\n\t<label for=\"uploadAvatarFile\" dojoAttachPoint=\"avatarUploadLabel\"></label>\n\t<form dojoAttachPoint=\"avatarForm\" id=\"avatarForm\" method=\"POST\" action=\"\" enctype=\"multipart/form-data\" accept-charset=\"UTF-8\" onsubmit=\"return false;\"> \n\t\t<input id=\"uploadAvatarFile\" type=\"file\" name=\"uploadfile\">\n\t</form>\n\t<div dojoAttachPoint=\"buttonContainer\" class=\"bspace-dialog-buttonContainer\">\n\t</div>\t\n</div>\n"),
	dialogParams: { style: ""},
	error: null,
	
	postMixInProperties: function(){
		
		this.avatarUpdateMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.inherited(arguments);
	},
	
	postCreate: function() {
		var self = this;
		this.addButton("cancelButton", this.avatarUpdateMessages.cancel, "onCancel", self);
		this.addButton("saveButton", this.avatarUpdateMessages.ok, "onSave", self);
		this.avatarForm.action = window.endpointURLObject.bpmRestJS + "/v1/avatar/current";		
		this.avatarUploadLabel.innerHTML = this.avatarUpdateMessages.updateAvatarMessage;

		this.inherited(arguments);
	},

	onCancel: function() {
		this.hide();
	},	
	
	onSave: function() {
		var allowedExtenstions = [ ".jpg", ".jpeg", ".gif", ".png" ];
		var validFileType = false;
		var upload = dojo.byId("uploadAvatarFile");
		if (upload) {
			var fileName = upload.value.toLowerCase();
			for (var i=0; i<allowedExtenstions.length; i++) {
				if (fileName.indexOf(allowedExtenstions[i]) != -1) {
					validFileType = true;
					break;
				}
			}
			
			if (!validFileType && upload != "") {
				this.error = this.avatarUpdateMessages.invalidAvatarFileType;
				this.hide();
				dojo.publish("com.ibm.bpm.avatarUpload");
				return false;
			}
		}

		this.hide();

		
		var self = this;
		var td = dojo.io.iframe.send({
			url: window.endpointURLObject.bpmRestJS +"/v1/avatar/current",
			form: "avatarForm",
			method: "post",
			timeoutSeconds: 90,
			preventCache: true,			
			handleAs: "text",
			headers: {
				"Content-Encoding" : "UTF-8",
				"Content-Type": "text/html"
			},			
			load: function(data, ioArgs){
				var response = dojo.fromJson(data);
				if (response.status = "200") {
					self.error = null;
					var currUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile().userName;
					com.ibm.bpm.social.util.common.Util.getUserAvatarURL(currUser,true);
				} else {
					self.error = self.avatarUpdateMessages.uploadError;
				}
				dojo.publish("com.ibm.bpm.avatarUpload");
			},							
			error: function (res,ioArgs) {
				self.error = self.avatarUpdateMessages.uploadError;
				dojo.publish("com.ibm.bpm.avatarUpload");
			}						
		});
	},
	
	show: function() {
		var fileInputNode = dojo.byId("uploadAvatarFile");
		if (fileInputNode) {
			fileInputNode.value = "";
		}
		this.error = null;
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.UserProfileDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.UserProfileDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.UserProfileDialog");








dojo.registerModulePath("com.ibm.socialPortal.templates", window.endpointURLObject.processPortalJS +"/com/ibm/bpm/social/widgets/dialogs/templates");

dojo.declare("com.ibm.bpm.social.widgets.dialogs.UserProfileDialog", com.ibm.bspace.common.util.ui.Dialog, {
	templateString: dojo.cache("com.ibm.socialPortal.templates", "userProfile.html", "<div class=\"processPortalUserProfile\">\n\t<div class=\"userProfileSection\" tabindex=\"0\" aria-label=\"User Profile Section\">\n\t\t<div class=\"userAvatarContainer\">\n\t\t\t<div dojoAttachPoint=\"userAvatarBorder\" class=\"userAvatarBorder\">\n\t\t\t\t<div dojoAttachPoint=\"userAvatar\" class=\"userAvatar\">\n\t\t\t\t\t<div dojoAttachPoint=\"userAvatarError\" class=\"userAvatarError\">\n\t\t\t\t\t</div>\n\t\t\t\t</div>\n\t\t\t</div>\n\t\t\t<div class=\"userAvatarOptions\">\n\t\t\t\t<div class=\"userProfileChangeAvatar\">\n\t\t\t\t\t<button dojoAttachPoint=\"changeAvatarLabel\" class=\"userAvatarLink\" dojoAttachEvent=\"onclick:changeAvatar, onkeypress:changeAvatarByKeypress\" tabindex=\"0\" role=\"button\"></button>\n\t\t\t\t</div>\n\t\t\t\t<div dojoAttachPoint=\"avatarDivider\" class=\"userProfileAvatarOptionsDivider\"><span>|</span></div>\n\t\t\t\t<div class=\"nofloat userProfileAvatarOptionsNewLineMarker\" style=\"display: none\"></div>\n\t\t\t\t<div class=\"userProfileRemoveAvatar\">\n\t\t\t\t\t<button dojoAttachPoint=\"removeAvatarLabel\" class=\"userAvatarLink\" dojoAttachEvent=\"onclick:removeAvatar, onkeypress:removeAvatarByKeypress\" tabindex=\"0\" role=\"button\" aria-label=\"Remove Avatar Image\"></button>\n\t\t\t\t</div>\n\t\t\t\t<div class=\"nofloat\"></div>\n\t\t\t</div>\n\t\t</div>\n\t\t<div class=\"userProfileInfo\">\n\t\t\t<div dojoAttachPoint=\"userName\" class=\"userName\"></div>\n\t\t\t<div style=\"clear: both;\">\n\t\t\t\t<div id=\"userTitle\" dojoAttachPoint=\"userTitle\" class=\"userTitle\" dojoAttachEvent=\"onclick:_editField, onkeypress:_editFieldByKeypress, onmouseover:_editFieldMouseover, mouseout:_editFieldMouseout\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userTitleHint\" dojoAttachPoint=\"userTitleHint\" class=\"userTitleHint\" dojoAttachEvent=\"onclick:_editField, onkeypress:_editFieldByKeypress\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userTitleEditIcon\" dojoAttachPoint=\"userTitleEditIcon\"></div>\n\t\t\t</div>\n\t\t\t<div id=\"userTitleEdit\" class=\"userEdit\" style=\"display: none\">\n\t\t\t\t<input dojoAttachPoint=\"userTitleEditField\" id=\"userTitleEditField\" type=\"text\" dojoAttachEvent=\"onblur:_saveEditField\" />\n\t\t\t</div>\n\t\t\t<div style=\"clear: both;\">\n\t\t\t\t<div id=\"userPhoneNumber\" dojoAttachPoint=\"userPhoneNumber\" class=\"userPhoneNumber\" dojoAttachEvent=\"onclick:_editField, onkeypress:_editFieldByKeypress, onmouseover:_editFieldMouseover, mouseout:_editFieldMouseout\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userPhoneNumberHint\" dojoAttachPoint=\"userPhoneNumberHint\" class=\"userHint\" dojoAttachEvent=\"onclick:_editField, onkeypress:_editFieldByKeypress\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userPhoneNumberEditIcon\" dojoAttachPoint=\"userPhoneNumberEditIcon\"></div>\n\t\t\t</div>\n\t\t\t<div id=\"userPhoneNumberEdit\" class=\"userEdit\" style=\"display: none\">\n\t\t\t\t<input id=\"userPhoneNumberEditField\" type=\"text\" dojoAttachEvent=\"onblur:_saveEditField\" />\n\t\t\t</div>\t\t\n\t\t\t<div style=\"clear: both;\">\n\t\t\t\t<div id=\"userEmail\" dojoAttachPoint=\"userEmail\" class=\"userEmail userEmailComponent\" dojoAttachEvent=\"onclick:_editField, onkeypress:_editFieldByKeypress, onmouseover:_editFieldMouseover, mouseout:_editFieldMouseout\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userEmailHint\" dojoAttachPoint=\"userEmailHint\" class=\"userHint userEmailComponent\" dojoAttachEvent=\"onclick:_editField,onkeypress:_editFieldByKeypress\" tabindex=\"0\"></div>\n\t\t\t\t<div id=\"userEmailEditIcon\" dojoAttachPoint=\"userEmailEditIcon\"></div>\n\t\t\t</div>\n\t\t\t<div id=\"userEmailEdit\" class=\"userEdit\" style=\"display: none\">\n\t\t\t\t<input id=\"userEmailEditField\" dojoAttachPoint=\"userEmailEditField\" type=\"text\" dojoAttachEvent=\"onblur:_saveEditField\" />\n\t\t\t</div>\n            <div id=\"userEmailInvalidMessage\" dojoAttachPoint=\"userEmailInvalidMessage\" class=\"userProfileErrorMessage\" style=\"display: none\"></div>\n\t\t</div>\n\t\t<div class=\"nofloat\"></div>\n\t</div>\n\t<div class=\"userProfileSection\"  tabindex=\"0\" style=\"clear: both\">\n\t\t<div dojoAttachPoint=\"portalPreferencesLabel\" class=\"userProfileSectionTitle\"></div>\n\t\t<div class=\"userOption\">\n\t\t<label dojoAttachPoint=\"languageDropdownLabel\" class=\"bspace-labelText\" for=\"languageDropdown\"></label>\n\t\t<select id=\"languageDropdown\" dojoAttachPoint=\"languageDropdown\" style=\"width: 100px\">\n\t\t</select>\n\t\t</div>\n\t\t<div class=\"userOption\">\n\t\t<label dojoAttachPoint=\"baseTextDirDropdownLabel\" class=\"bspace-labelText\" for=\"baseTextDirDropdown\"></label>\n\t\t<select id=\"baseTextDirDropdown\" dojoAttachPoint=\"baseTextDirDropdown\" style=\"width: auto\">\n\t\t</select>\n\t\t</div>\n\t\t<div class=\"userOption\">\n\t\t<label dojoAttachPoint=\"calendarDropdownLabel\" class=\"bspace-labelText\" for=\"calendarTypeDropdown\"></label>\n\t\t<select id=\"calendarTypeDropdown\" dojoAttachPoint=\"calendarTypeDropdown\" style=\"width: 100px\">\n\t\t</select>\n\t\t</div>\n\t\t<div class=\"userOption\">\n\t\t<input id=\"showConfirmationCheckbox\" dojoAttachPoint=\"showConfirmationCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"showConfirmationLabel\" class=\"bspace-labelText\" for=\"showConfirmationCheckbox\"></label>\n\t\t</div>\t\n\t\t<div class=\"userOption\">\n\t\t<input id=\"sendEmailCheckbox\" dojoAttachPoint=\"sendEmailCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"sendEmailLabel\" class=\"bspace-labelText\" for=\"sendEmailCheckbox\"></label>\n\t\t</div>\n\t\t<div dojoAttachPoint=\"sendConnectionsNotificationOption\" class=\"userOption\">\n\t\t<input id=\"sendConnectionsNotificationCheckbox\" dojoAttachPoint=\"sendConnectionsNotificationCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"sendConnectionsNotificationLabel\" class=\"bspace-labelText\" for=\"sendConnectionsNotificationCheckbox\"></label>\n\t\t<span tabindex=\"0\" dojoAttachPoint=\"sendConnectionsNotificationError\"></span>\n\t\t</div>\n\t\t<div class=\"userOption\">\n\t\t<input id=\"autoFollowCheckbox\" dojoAttachPoint=\"autoFollowCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"autoFollowLabel\" class=\"bspace-labelText\" for=\"autoFollowCheckbox\"></label>\n\t\t</div>\n\t</div>\n\t<div class=\"userProfileSection\" tabindex=\"0\">\n\t\t<div dojoAttachPoint=\"notificationPreferencesLabel\" class=\"userProfileSectionTitle\"></div>\n\t\t<div class=\"userOption\">\t\t\n\t\t<input id=\"assignToMeCheckbox\" dojoAttachPoint=\"assignToMeCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"assignToMeLabel\" class=\"bspace-labelText\" for=\"assignToMeCheckbox\"></label>\n\t\t</div>\n\t\t<div class=\"userOption\">\n\t\t<input id=\"assignToGroupCheckbox\" dojoAttachPoint=\"assignToGroupCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"assignToGroupLabel\" class=\"bspace-labelText\" for=\"assignToGroupCheckbox\"></label>\n\t\t</div>\n\t\t<div class=\"userOption\">\t\t\n\t\t<input id=\"mentionedCheckbox\" dojoAttachPoint=\"mentionedCheckbox\" type=\"checkbox\" />\n\t\t<label dojoAttachPoint=\"mentionedLabel\" class=\"bspace-labelText\" for=\"mentionedCheckbox\"></label>\t\n\t\t</div>\n\t</div>\n\t<div dojoAttachPoint=\"buttonContainer\" class=\"bspace-dialog-buttonContainer\">\n\t</div>\n</div>\n"),
	avatarURL: null,
	avatarLoaded: false,
	dialogParams: { style: ""},
	connects: null,
	
	postMixInProperties: function(){
		
		this.userProfileMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.inherited(arguments);
	},	
	postCreate: function() {
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		var self = this;
		this.addButton("updateButton", this.userProfileMessages.uploadAvatar, "onUpdate", self);
		
		this.changeAvatarLabel.innerHTML = this.userProfileMessages.changeAvatar;
		this.changeAvatarLabel.title = this.userProfileMessages.changeAvatarImage;
		this.removeAvatarLabel.innerHTML = this.userProfileMessages.removeAvatar;
		this.languageDropdownLabel.innerHTML = this.userProfileMessages.language;
        this.baseTextDirDropdownLabel.innerHTML = this.userProfileMessages.baseTextDir;
		this.calendarDropdownLabel.innerHTML = this.userProfileMessages.calendarType;

		// Portal Preferences section
		this.portalPreferencesLabel.innerHTML = this.userProfileMessages.portalPreferences;
		this.showConfirmationLabel.innerHTML = this.userProfileMessages.showConfirmation;		
		this.sendEmailLabel.innerHTML = this.userProfileMessages.sendEmail;
		this.sendConnectionsNotificationLabel.innerHTML = this.userProfileMessages.sendConnectionsNotification;
        this.userEmailInvalidMessage.innerHTML = this.userProfileMessages.invalidEmailAddress;
		this.autoFollowLabel.innerHTML = this.userProfileMessages.autoFollow;
		
		// Notification section
		this.notificationPreferencesLabel.innerHTML = this.userProfileMessages.notificationPreferences;
		this.assignToMeLabel.innerHTML = this.userProfileMessages.assignToMe;
		this.assignToGroupLabel.innerHTML = this.userProfileMessages.assignToGroup;
		this.mentionedLabel.innerHTML = this.userProfileMessages.mentioned;
		
		dojo.subscribe("com.ibm.bpm.avatarUpload", this, "fileUploaded");

		this.inherited(arguments);
		//Apply BTD to the userTitleEditField
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.userTitleEditField);
		
		//initialize variables to control whether the Send Connections Notificiations checkbox is disabled
		this._hasValidEmail = false;
		this._isConnectionsConfigured = false;
		this._checkConnectionsCheckboxConditions();
		
		// Add listeners to make the pen icon appear
		var penClass = "userProfileEditIcon";
		if(!this.connects) {
			this.connects = new Array();
		}

		var aConnect;
		aConnect = dojo.connect(this.userTitle, 'onmouseenter', this, function() { dojo.addClass(this.userTitleEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userTitle, 'onmouseleave', this, function() { dojo.removeClass(this.userTitleEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userTitleHint, 'onmouseenter', this, function() { dojo.addClass(this.userTitleEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userTitleHint, 'onmouseleave', this, function() { dojo.removeClass(this.userTitleEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userPhoneNumber, 'onmouseenter', this, function() { dojo.addClass(this.userPhoneNumberEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userPhoneNumber, 'onmouseleave', this, function() { dojo.removeClass(this.userPhoneNumberEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userPhoneNumberHint, 'onmouseenter', this, function() { dojo.addClass(this.userPhoneNumberEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userPhoneNumberHint, 'onmouseleave', this, function() { dojo.removeClass(this.userPhoneNumberEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userEmail, 'onmouseenter', this, function() { dojo.addClass(this.userEmailEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userEmail, 'onmouseleave', this, function() { dojo.removeClass(this.userEmailEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userEmailHint, 'onmouseenter', this, function() { dojo.addClass(this.userEmailEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.userEmailHint, 'onmouseleave', this, function() { dojo.removeClass(this.userEmailEditIcon, penClass); });
		this.connects.push(aConnect);

		aConnect = dojo.connect(this.languageDropdown, 'onchange', this, this._handleUserPrefLanguageChange);
		this.connects.push(aConnect);
		
		// Only do if IE is less than 10 as the text field will have a blue background (probably some IE bug) that no longer occurs in 10
		if (dojo.isIE && dojo.isIE <10) dojo.style(this.changeAvatarLabel, "color", "#ffffff");
	},
	
    changeAvatarByKeypress: function(e) {
         if (e.charOrCode == dojo.keys.ENTER) {
             this.changeAvatar();
         }
    },

	// Called when the Change link is clicked to trigger the file selection dialog
	changeAvatar: function() {
		
		if (!this.changeAvatarDialog) {
			this.changeAvatarDialog = new com.ibm.bpm.social.widgets.dialogs.AvatarUploadDialog();
		}
		this.changeAvatarDialog.show();
	},
	
	fileUploaded: function() {
		if (this.changeAvatarDialog && this.changeAvatarDialog.error) {
			
			this._displayAvatarError(this.changeAvatarDialog.error);
		} else {
            this._clearAvatarError();        
			this.loadAvatar();
		}
	},

    removeAvatarByKeypress: function(e) {
         if (e.charOrCode == dojo.keys.ENTER) {
             this.removeAvatar();
         }
    },

	removeAvatar: function() {
		this._clearAvatarError();
				
		if (this.avatarLoaded) {
			var self = this;

			this.socialRestHandler.deleteUserAvatar(function(data) {
					// Remove the imgage tag containing the avatar image
					var avatarImageNode = dojo.byId("avatarImage");
					if (avatarImageNode) {
						avatarImageNode.parentNode.removeChild(avatarImageNode);
					}
					this.avatarLoaded = false;
					self.toggleRemoveButton(false);
					self.toggleDefaultAvatar(true);
					
					var currUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile().userName;
					com.ibm.bpm.social.util.common.Util.getUserAvatarURL(currUser,true);
					dojo.publish( com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, [{"username": currUser}]);
					var scoreboardFrameVar = document.getElementById("scoreboardFrame");
					if (scoreboardFrameVar) {
						scoreboardFrameVar.contentWindow.postMessage('{"name": "USER_PROFILE_CHANGED", "username": "'+currUser+'"}','*');
					}
					
				}, function(error, ioargs) {
					console.log("REST call DELETING avatar image caused ERROR '" + error + "'");
				}, true);
		}
	},
	
	toggleRemoveButton: function(shouldShow) {
		if(!shouldShow) {
			this.removeAvatarLabel.style.visibility = "hidden";
			this.avatarDivider.style.visibility = "hidden";
		}
		else {
			this.removeAvatarLabel.style.visibility = "visible";
			this.avatarDivider.style.visibility = "visible";
		}
	},
	
	toggleDefaultAvatar: function(shouldShow) {
		if(!shouldShow) {
			dojo.addClass(this.userAvatar, "userAvatarDisabled");
		}
		else {
			dojo.removeClass(this.userAvatar, "userAvatarDisabled");
		}
	},
	
	loadAvatar: function() {
		var self = this;

		//REST call needed to determine if it's the default or not.
		var load = function(data){
			if(typeof(connectionsBridge)=="undefined" || !connectionsBridge.isAvatarCached(com.ibm.bpm.socialportal.GlobalAssets._userProfile.emailAddress)){
				dojo.style(self.changeAvatarLabel, "visibility", "visible");
				self.toggleRemoveButton(data.data.isDefault != "true");
				self.toggleDefaultAvatar(data.data.isDefault == "true");
			}
		};
		
		var error = function(error, ioargs){
			switch(ioargs.xhr.status){
				// In the case of a 404, assume no avatar is defined for this user and
				// don't treat as an error since the default avatar will be displayed.
				case 404:
					self.avatarLoaded = true;
					break;
				default:
					console.log("REST call loading avatar image caused ERROR '" + error + "'");
			}
		};
		
		this.socialRestHandler.getCurrentUserAvatar(load, error, true);
		
		//Everything else can be determined by URL automatically
		var imageElement = dojo.byId("avatarImage");
		if (!imageElement) {
			imageElement = document.createElement("img");
			imageElement.id = "avatarImage";
			self.userAvatar.insertBefore(imageElement, self.userAvatarError);
		}
		var currUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile().userName;
		var avatar = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(currUser);
		
		dojo.attr(imageElement, "src", avatar);

		// Added for accessibility 
		dojo.attr(imageElement, "tabindex", "0");
		var imageElementAriaLabel = this.userProfileMessages.avatar_image_label_default;
		if(this.removeAvatarLabel.style.visibility == "visible") {
			imageElementAriaLabel = this.userProfileMessages.avatar_image_label_custom;	
		}
		dojo.attr(imageElement, "alt", imageElementAriaLabel);
		
		if(typeof(connectionsBridge)!="undefined" && connectionsBridge.isAvatarCached(com.ibm.bpm.socialportal.GlobalAssets._userProfile.emailAddress)){
			dojo.style(this.changeAvatarLabel, "visibility", "hidden");
			dojo.style(this.avatarDivider, "visibility", "hidden");
			dojo.style(this.removeAvatarLabel, "visibility", "hidden");
		} 
		
		// Note: no error if  userAvatarImage is null, no avatar is defined for this user. 
		self.avatarLoaded = true;

		dojo.publish( com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, [{"username": currUser}]);
		var scoreboardFrameVar = document.getElementById("scoreboardFrame");
		if (scoreboardFrameVar) {
			scoreboardFrameVar.contentWindow.postMessage('{"name": "USER_PROFILE_CHANGED", "username": "'+currUser+'"}','*');
		}
	},
	
	_displayAvatarError: function(errorMsg) {
		this.userAvatarError.innerHTML = errorMsg;
		var avatarImage = dojo.query("#avatarImage")[0];
		if(avatarImage) {
			avatarImage.style.display = "none"
		}
		this.userAvatarError.style.display = "";
	},
	
	_clearAvatarError: function() {
		this.userAvatarError.innerHTML = "";
		this.userAvatarError.style.display = "none";
		var avatarImage = dojo.query("#avatarImage")[0];
		if(avatarImage) {
			avatarImage.style.display = ""
		}
	},

	_clearEmailEditFieldError: function(email) {
        if (email == "") {
            this.userEmailInvalidMessage.style.display = "none";
            this._setHasValidEmail(false);
        } else {
            this._validateEmail(email);
        }	
	},

    _editFieldByKeypress: function(e) {
         if (e.charOrCode == dojo.keys.ENTER) {
             this._editField(e);
         }
    },

	_editField: function(e) {
		var targetId = e.currentTarget.id;
		if (targetId.indexOf("Hint") != -1) {
			dojo.byId(targetId).style.display = "none";
			targetId = targetId.replace("Hint", "");
		}	
	
		// Hide the edit icon if it is visibile
		dojo.byId(targetId + "EditIcon").style.display = "none";

		// Hide the displayed text and reveal the edit field
		var displayField = dojo.byId(targetId);
		var editField = dojo.byId(targetId + "EditField");
		editField.value = displayField.innerHTML;
		displayField.style.display = "none"
		dojo.byId(targetId + "Edit").style.display = "";
		editField.style.display = "";
		dijit.focus(editField);
	},
	_saveEditField: function(e) {
		var targetId = e.currentTarget.id;
	
		// Hide the edit field and reveal the text
		var editField = dojo.byId(targetId);
		
		editField.parentNode.style.display = "none";
		var displayFieldId = targetId.replace("EditField", "");
		var displayField = dojo.byId(displayFieldId);
		displayField.innerHTML = editField.value;
		
		var hintId = targetId.replace("EditField", "Hint");
		if (targetId == "userTitleEditField") {
			if (editField.value == "") {
				this.userTitleHint.style.display = "";
				dojo.byId(targetId.replace("EditField", "EditIcon")).style.display = "";
				displayField.style.display = "none";
			} else {
				this.userTitleHint.style.display = "none";
				displayField.style.display = "";
			}
			//Apply BTD to the userTitleEditField
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.userTitle);
		} else if (targetId == "userPhoneNumberEditField") {
			if (editField.value == "") {
				this.userPhoneNumberHint.style.display = "";
				dojo.byId(targetId.replace("EditField", "EditIcon")).style.display = "";
				displayField.style.display = "none";
			} else {
				this.userPhoneNumberHint.style.display = "none";
				displayField.style.display = "";
			}		
		} else if (targetId == "userEmailEditField") {
			if (editField.value == "") {
				this.userEmailHint.style.display = "";
				dojo.byId(targetId.replace("EditField", "EditIcon")).style.display = "";
				displayField.style.display = "none";
                dojo.byId("userEmailInvalidMessage").style.display = "none";
				this._setHasValidEmail(false);
			} else {
				this.userEmailHint.style.display = "none";
				displayField.style.display = "";
                this._validateEmail(editField.value);
			}
		} else {
			displayField.style.display = "";
		}
		editField.style.display = "none";
	},

    _validateEmail: function(email) {
         var emailRegEx = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i;
         if (email.search(emailRegEx) == -1) {
             this.userEmailInvalidMessage.style.display = "";
			this._setHasValidEmail(false);
         } else {
             this.userEmailInvalidMessage.style.display = "none";
			this._setHasValidEmail(true);
         }
    },

	_editFieldMouseover: function(e) {
		var editField = dojo.byId(e.currentTarget.id + "EditIcon");
		if (editField) {
			editField.style.display = "";
		}
	},
	_editFieldMouseout: function(e) {
		var editField = dojo.byId(e.currentTarget.id + "EditIcon");
		if (editField) {
			editField.style.display = "none";
		}		
	},

	_handleUserPrefLanguageChange: function() {
		// if the user profile language has changed to a RTL language,
		// make sure that the BTD Dropdown is shown
		var newOption = this.languageDropdown.options[this.languageDropdown.selectedIndex].value;
		if (newOption === 'ar' || newOption === 'he') {
			this._showBTDDropdown();
		} else {
			this._hideBTDDropdown();
		}
	},

	_setHasValidEmail: function(hasValidEmail) {
		this._hasValidEmail = hasValidEmail;
		this._checkConnectionsCheckboxConditions();
	},

	_setIsConnectionsConfigured: function(connectionsConfigured) {
		this._isConnectionsConfigured = connectionsConfigured;
		this._checkConnectionsCheckboxConditions();
	},

	_checkConnectionsCheckboxConditions: function() {
		if(!this._isConnectionsConfigured) {
			this._disableConnectionsCheckbox(this.userProfileMessages.sendConnectionsNotificationNoServer);
		}
		else {
			if(!this._hasValidEmail) {
				this._disableConnectionsCheckbox(this.userProfileMessages.sendConnectionsNotificationNoEmail);
			}
			else {
				this._enableConnectionsCheckbox();
			}
		}
	},

	_enableConnectionsCheckbox: function() {
		com.ibm.bspace.common.util.ui.CheckBox.setDisabled(this.sendConnectionsNotificationCheckbox, false);
		dojo.style(this.sendConnectionsNotificationError, "display", "none");
		this.sendConnectionsNotificationError.innerHTML = "";
	},

	_disableConnectionsCheckbox: function(reason) {
		com.ibm.bspace.common.util.ui.CheckBox.setDisabled(this.sendConnectionsNotificationCheckbox, true);
		dojo.style(this.sendConnectionsNotificationError, "display", "block");
		dojo.style(this.sendConnectionsNotificationError, "color", "#333"); //sorry for this!
		dojo.style(this.sendConnectionsNotificationError, "fontStyle", "italic");
		dojo.style(this.sendConnectionsNotificationError, "marginLeft", "2em");
		dojo.style(this.sendConnectionsNotificationError, "marginRight", "2em");
		this.sendConnectionsNotificationError.innerHTML = "" + reason;
	},
	
	/**
	 * If the logged in user isn't a member of tw_admins hide the checkbox for connection notifications (as it'll always be disabled for them).
	 * I might possibly be able to change the _disableConnectionsCheckbox to hide it, but maybe we'd still want to see it in certain cases
	 */
	_hideConnectionsNotification:function(memberships) {
		
		// Search for the admins groups, if we're a member of the tw_admins no need to continue
		for (var i=0; i<memberships.length;i++) {
			if ("tw_admins" == memberships[i])
				return;
		}
		
		// If the connections are configured no need to continue.
		if (this._isConnectionsConfigured)
			return;
		
		
		// Added a dojoAttachPoint to the option. I originally tried to hide the pieces inside, however it still left a one line whitespace.
		// Doing it on the option to tighten the view up
		// The connection isn't configured and the user isn't an admin. Hide the checkbox to send notifications
		dojo.style(this.sendConnectionsNotificationOption, "visibility", "hidden");
		dojo.style(this.sendConnectionsNotificationOption, "display", "none");


	},


	onUpdate: function() {
		// Gather the properties for the user from the dialog
		// For title, phone number, and address replace the hint with the empty string
		var userProfileFromDialog = {
			'Title': ((this.userTitle.innerHTML == this.userProfileMessages.jobTitle) ? "" : this.userTitle.innerHTML),
			'Phone Number': ((this.userPhoneNumber.innerHTML == this.userProfileMessages.phoneNumber) ? "" : this.userPhoneNumber.innerHTML),
			'Task Email Address': ((this.userEmail.innerHTML == this.userProfileMessages.emailAddress) ? "" : this.userEmail.innerHTML),
			'Locale': this.languageDropdown.options[this.languageDropdown.selectedIndex].value,
            'Base Text Direction': this.baseTextDirDropdown.options[this.baseTextDirDropdown.selectedIndex].value,
			'Calendar Type': this.calendarTypeDropdown.options[this.calendarTypeDropdown.selectedIndex].value,
			'Alert On Assign And Run': "" + this.showConfirmationCheckbox.checked,
			'Task Notification': "" + this.sendEmailCheckbox.checked,
			'Show Unfollow Messages': "" + this.autoFollowCheckbox.checked,
			'Portal Notification New Task To Me': "" + this.assignToMeCheckbox.checked,
			'Send Connections Notification On New Task': "" + this.sendConnectionsNotificationCheckbox.checked,
			'Portal Notification New Task To My Group': "" + this.assignToGroupCheckbox.checked,
			'Portal Notification Post Mention': "" + this.mentionedCheckbox.checked
		};	
		
		var localeChanged = false;
		for (var property in userProfileFromDialog) {
			if (this.originalUserProfile[property] != userProfileFromDialog[property]) {
				console.log("Updating user profile property '" + property + "' from '" + this.originalUserProfile[property] + "' to '" + userProfileFromDialog[property] + "'");
				com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(property, userProfileFromDialog[property]);
				
				if (property == "Locale") {
					localeChanged = true;
				}
			}
		}
			
		this.hide();
		
		if (localeChanged) {
			// Reset the Base Text Direction to Contextual, so they aren't stuck in RTL if changing to a LTR language
			// Unless the User has changed their locale to a RTL language, in which case we want to save the new BTD preference
			if (userProfileFromDialog["Locale"] != 'ar' && userProfileFromDialog["Locale"] != 'he') {
				com.ibm.bpm.socialportal.GlobalAssets.setUserPreference('Base Text Direction', 'generalPreference.fields.baseTextDir.contextual');
			}

			
			com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this.userProfileMessages.localeChangeWarning, function(){
				return;
			});
		}
	},

	_showBTDDropdown: function() {
		// shows the base text direction dropdown for RTL languages
		if (dojo.isIE) {
			dojo.style(this.baseTextDirDropdown, "display", "inline");			
			dojo.style(this.baseTextDirDropdownLabel, "display", "inline");
		} else {
			dojo.attr(this.baseTextDirDropdown, "hidden", false);
			dojo.attr(this.baseTextDirDropdownLabel, "hidden", false);
		}
	},

	_hideBTDDropdown: function() {
		// hides the base text direction dropdown for LTR languages
		if (dojo.isIE) {
			dojo.style(this.baseTextDirDropdown, "display", "none");			
			dojo.style(this.baseTextDirDropdownLabel, "display", "none");
		} else {
			dojo.attr(this.baseTextDirDropdown, "hidden", true);
			dojo.attr(this.baseTextDirDropdownLabel, "hidden", true);
		}			

		// change the styling so it doesn't look funny when these dropdowns are hidden
		dojo.attr(this.baseTextDirDropdown.parentNode, "style", "padding-top:0;padding-bottom:0");
	},
	
	refresh: function() {
		this._clearAvatarError();
	
		// Load the user profile from the GlobalAssets
		var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		this.originalUserProfile = userProfileObj.userPreferences;
		// Set locale to the browser locale if blank so the locale change warning won't get displayed the first time you update.
		if (this.originalUserProfile['Locale'] == "") {
			this.originalUserProfile['Locale'] = dojo.locale;
		}
		
		// Load the user profile from the GlobalAssets
		this.userName.innerHTML = userProfileObj['fullName'];
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.userName);
		var title = userProfileObj.userPreferences['Title'];
		this.userTitle.innerHTML = title;
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.userTitle);
		this.userTitleHint.innerHTML = this.userProfileMessages.jobTitle;
		if (!title || title == "") {
			this.userTitleHint.style.display = "";
			this.userTitle.style.display = "none";
		} else {
			this.userTitleHint.style.display = "none";
			this.userTitle.style.display = "";
		}
		
		var phoneNumber = userProfileObj.userPreferences['Phone Number'];
		this.userPhoneNumber.innerHTML = phoneNumber;
		this.userPhoneNumberHint.innerHTML = this.userProfileMessages.phoneNumber;
		if (!phoneNumber || phoneNumber == "") {
			this.userPhoneNumberHint.style.display = "";
			this.userPhoneNumber.style.display = "none";
		} else {
			this.userPhoneNumberHint.style.display = "none";
			this.userPhoneNumber.style.display = "";
		}
		
		var email = userProfileObj.userPreferences['Task Email Address'];
		this.userEmail.innerHTML = email;
		this.userEmailHint.innerHTML = this.userProfileMessages.emailAddress;
		if (!email || email == "") {
			this.userEmailHint.style.display = "";
			this.userEmail.style.display = "none";
		} else {
			this.userEmailHint.style.display = "none";
			this.userEmail.style.display = "";
		}        
        this._clearEmailEditFieldError(email);

		var self = this;
		var supportedLocales = [];
		var languageStrings = [];
		var xhrArgs = {
			url: window.endpointURLObject.processPortalJS +"/jsp/socialPortal/getLocales.jsp?availableLocales=" + window.bspaceSupportedLocales + "&userLocale=" + userProfileObj.userPreferences['Locale'],
			handleAs: "json",
			headers: {
				"Accept": "application/json"
			},
			load: function(data){
				for (var p in data) {
					supportedLocales.push(p);
					languageStrings[p] = data[p];
				}
        if (self.languageDropdown.length == 0) {
    		for (var i=0; i<supportedLocales.length; i++) {
    			var localeString = languageStrings[supportedLocales[i]];
    			if (!localeString) {
    				localeString = supportedLocales[i];
    			}
    			self.languageDropdown.add(new Option(localeString, supportedLocales[i]));
    			if (userProfileObj.userPreferences['Locale'] == '') {
    				// If no locale is provided, default to the locale set in the browser
    				if (supportedLocales[i] == dojo.locale) {
    					self.languageDropdown.selectedIndex = i;
    				}
    			} else if (supportedLocales[i] == userProfileObj.userPreferences['Locale']) {
    				self.languageDropdown.selectedIndex = i;
    			}
    		}
        }
			},
			 error: function(error, ioargs){
				// In case of errors, return just the hardcoded list
				supportedLocales = window.bspaceSupportedLocales.replace(/he/,"iw").split(",");
				languageStrings = this.userProfileMessages;
			}
		};

		// Call the asynchronous xhrGet
		var deferred = dojo.xhrGet(xhrArgs);
        /*if (this.languageDropdown.length == 0) {
    		for (var i=0; i<supportedLocales.length; i++) {
    			var localeString = languageStrings[supportedLocales[i]];
    			if (!localeString) {
    				localeString = supportedLocales[i];
    			}
    			this.languageDropdown.add(new Option(localeString, supportedLocales[i]));
    			if (userProfileObj.userPreferences['Locale'] == '') {
    				// If no locale is provided, default to the locale set in the browser
    				if (supportedLocales[i] == dojo.locale) {
    					this.languageDropdown.selectedIndex = i;
    				}
    			} else if (supportedLocales[i] == userProfileObj.userPreferences['Locale']) {
    				this.languageDropdown.selectedIndex = i;
    			}
    		}
        }*/
		
		// Enable the Send Connections Notifications On New Task checkbox if connections is enabled (provided a valid e-mail address)
		var connURL = com.ibm.bpm.social.util.common.Util.getConnectionsURL();
		this._setIsConnectionsConfigured(typeof(connURL) != "undefined" && connURL != null && connURL.indexOf("connectionUserId=") > -1);
		
		// Load base text direction dropdown values and set selected index
		this.baseTextDirDropdown.selectedIndex = 0;
		var baseTextDirIndex = 0;
		this.baseTextDirDropdown.options.length = 0;
		for (var options in com.ibm.bpm.baseTextDir) {
			this.baseTextDirDropdown.add(new Option(com.ibm.bpm.baseTextDir[options], options));
			if (userProfileObj.userPreferences['Base Text Direction'] == '') {
				// If no base text direction is provided, default to Default
				if (options == "generalPreference.fields.baseTextDir.default") {
					this.baseTextDirDropdown.selectedIndex = baseTextDirIndex;
				}
			} else if (options == userProfileObj.userPreferences['Base Text Direction']) {
				this.baseTextDirDropdown.selectedIndex = baseTextDirIndex;
			}
			baseTextDirIndex++;
		}

		// Hide the BTD dropdown if not a RTL language
		var locale = userProfileObj.userPreferences['Locale'];
		if (locale === '') {
			locale = dojo.locale;
		}

		var browserLocale;
		if (dojo.isIE) {
			browserLocale = navigator.browserLanguage;
		} else {
			browserLocale = navigator.language;
		}

		if (locale.substring(0, 2) != 'ar' && locale != 'he' && browserLocale.substring(0, 2) != 'ar' && browserLocale != 'he') { 
			this._hideBTDDropdown();						
		} else {
			this._showBTDDropdown();					
		}


        // Load calendar dropdown values and set selected index
        this.calendarTypeDropdown.selectedIndex = 0;
        var calendarIndex = 0;
        this.calendarTypeDropdown.options.length = 0;
        for (var options in com.ibm.bpm.calendarTypes) {
            this.calendarTypeDropdown.add(new Option(com.ibm.bpm.calendarTypes[options], options));
            if (userProfileObj.userPreferences['Calendar Type'] == '') {
                // If no calendar type is provided, default to Gregorian
                if (options == "generalPreference.fields.calendarType.gregorian") {
                    this.calendarTypeDropdown.selectedIndex = calendarIndex;
                }
            } else if (options == userProfileObj.userPreferences['Calendar Type']) {
                this.calendarTypeDropdown.selectedIndex = calendarIndex;
            }
            calendarIndex++;
        }

		// Portal Preferences section
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.showConfirmationCheckbox, (userProfileObj.userPreferences['Alert On Assign And Run'] == "true")?true:false);
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.sendEmailCheckbox, (userProfileObj.userPreferences['Task Notification'] == "true")?true:false);
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.sendConnectionsNotificationCheckbox, (userProfileObj.userPreferences['Send Connections Notification On New Task'] == "true")?true:false);
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.autoFollowCheckbox, (userProfileObj.userPreferences['Show Unfollow Messages'] == "true")?true:false);
		
		// Notification section
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.assignToMeCheckbox, (userProfileObj.userPreferences['Portal Notification New Task To Me'] == "true")?true:false);
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.assignToGroupCheckbox, (userProfileObj.userPreferences['Portal Notification New Task To My Group'] == "true")?true:false);
		com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.mentionedCheckbox, (userProfileObj.userPreferences['Portal Notification Post Mention'] == "true")?true:false);
		
		// The parameters to pass to xhrGet, the url, how to handle it, and the callbacks.
		if (!this.avatarLoaded) {
			this.loadAvatar();
		}
		
		this._hideConnectionsNotification(userProfileObj.memberships);
			
		return userProfileObj;
	},
	show: function() {
		this.avatarLoaded = false;	// Always reload the avatar when showing the dialog
		this.refresh();
		this.inherited(arguments);
		
		// Adjust spacing of Change and Remove links underneith the avatar if the strings are wider than the avatar image
		var avatarWidth = 105;
		var avatarOptionsContainer = dojo.query(".userAvatarOptions");
		if(avatarOptionsContainer && avatarOptionsContainer.length > 0) {
			//Hide if on Ipad
			if (dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
				avatarOptionsContainer[0].style.display = "none";
			} else {
				var avatarLinksNode = avatarOptionsContainer[0];
				/* Grab the options:
				 * [0] = userProfileChangeAvatar
				 * [1] = userProfileAvatarOptionsDivider
				 * [2] = userProfileAvatarOptionsNewLineMarker
				 * [3] = userProfileRemoveAvatar
				 * [4] = the nofloat float fix div
				 */
				var avatarOptions = dojo.query(".userAvatarOptions > *");
				
				//calculate how much extra space is left under the avatar with the options
				var changeWidth = dojo.style(avatarOptions[0], "width");
				var dividerWidth = dojo.style(avatarOptions[1], "width");
				var removeWidth = dojo.style(avatarOptions[3], "width");
				var totalWidth = changeWidth + dividerWidth + removeWidth;
				var extraSpace = avatarWidth - totalWidth;
				
				/* Determine if there is enough space to keep it all on one line and
				 * add the necessary padding to the divider, otherwise move remove to second 
				 * line and hide the divider if visible
				 */
				if(extraSpace > 6) {
					var dividerPadding = extraSpace / 2;
					dojo.style(avatarOptions[1], "padding", "0px " + dividerPadding + "px 0px " + dividerPadding + "px"); 
				} else {
					dojo.style(avatarOptions[2], "display", "block");
					if(dojo.style(avatarOptions[1], "visibility") == "visible") {
						dojo.style(avatarOptions[1], "display", "none");
					}	
				}
				
			}
		}
	},

	destroy: function() {
		//clean up the connects
		if(this.connects) {
			for(var i=0; i < this.connects.length; i++) {
				dojo.disconnect(this.connects[i]);
				this.connects[i] = null;
			}
			this.connects = null;
		}
		
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.UserDropdown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.UserDropdown"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.UserDropdown");









dojo.declare(
	"com.ibm.bpm.social.portal.UserDropdown",
	[dijit._Widget, dijit._Templated],
	{
		templateString: "<div class=\"user-dropdown processPortalDropdownMenu\"/>",
		dropdownMenu: null,
		dropdownOpen: false,
		blurBlock: true,
		userMenu: null,
		connects: null,
		
		postCreate: function() {
			//Sort out the localization
			
			this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
			com.ibm.bpm.social.portal.UserDropdown.id = this.domNode.id;
			
			//Create the menu
			this.userMenu = this._createMenu();
			/*TODO: Create the proper dom setup for this to work*/
			var node = dojo.byId("processPortalUserDropdownId");
			this.dropdownMenu = new com.ibm.bpm.social.portal.DropdownMenu(
				{dropDown: this.userMenu,
				 dropDownContainer: this.domNode,
				 _buttonNode: node,
				 _popupStateNode: node,
				 _aroundNode: node});
			this.dropdownMenu.startup();
			this.dropdownMenu.loadDropDown();
			this.userProfileDialog = null;
			
			//Setup Event Handling  We want the menu to close everytime focus is lost, but
			//the blur event gets processed before a click to the username link which opens
			//the menu. So in order to achieve the clicking behavior desired by design, a
			//state must be managed determining whether the mouse is over the open link 
			//when the blur event is fired.
			var self = this;
			
			if(!this.connects) {
				this.connects = new Array();
			}

			this.connects.push(dojo.query("#processPortalUserDropdownId > *").connect('mouseover',
			function(event) {
				self._blockBlur(true, self);
				}));
			
			this.connects.push(dojo.query("#processPortalUserDropdownId > *").connect('mouseout',
			function(event) {
				self._blockBlur(false, self);
				}));
			
			this.connects.push(dojo.connect(this.userMenu, 'onBlur', function(event) { 
				self._blurActions(self);
			}));
		},
		
		adjustMenuStyle: function() {
			var node = dojo.byId("processPortalUserDropdownId_dropdown");
			if(node) {
				dojo.addClass(node, "processPortalDropDownMenu");
			}
			
			dojo.style(this.userMenu.domNode, "padding-left", "15px");
			dojo.style(this.userMenu.domNode, "padding-right", "15px");
			
			var items = this.userMenu.getChildren();
			if(items) {
				dojo.forEach(items, function(item, index) {
					item.iconNode.style.display = "none";
				});
			}
		},
		
		_createMenu: function() {
			var menu = new dijit.Menu();
			var self = this;
			
			var userProfile = new  dijit.MenuItem({
				label: this.messages.userMenuProfile,
				onClick: function(event) {
					self._showUserProfile();
					self.dropdownOpen = false;
				},
				onKeyDown: this._menuKeyDown
			});
			menu.addChild(userProfile);
			
			var logout = new  dijit.MenuItem({
				label: this.messages.userMenuLogout,
				onClick: dojo.hitch(this,this._logoutUser),
				onKeyDown: this._menuKeyDown
			});
			menu.addChild(logout);
			
			return menu;
		},
	
		_menuKeyDown: function(evt) {
			var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
			if (keypressed == dojo.keys.ENTER || keypressed == dojo.keys.SPACE) {
				var target = evt.target;
				if(dojo.isIE) {
					target = evt.target.childNodes[1];
				} else {
					target = evt.target.childNodes[3];
				}
				
				var action = target.innerHTML;
				
				if (action == this.messages.userMenuProfile) {
					this._showUserProfile();
				} else if (action == this.messages.userMenuLogout) {
					this._logoutUser();
				} else {
					//shouldn't hit, something is messed up if it does.
				}
			}
		},
		
		_logoutUser: function() {
			try {				
				if (typeof disconnectFromBayeuxServer === "function") {
				  disconnectFromBayeuxServer();
				}
			} finally {
				// Otherwise the cometd calls are lost because the authorization is removed before
				// the cometd calls above are triggered.
				setTimeout(com.ibm.bpm.logout,500);				
			}			
		},
		
		toggle: function() {
			if(this.dropdownOpen) {
				this.hide();
			} else {
				this.show();
			}
		},
		
		show: function() {
			if (this.dropdownMenu) {
				this.dropdownMenu.loadDropDown();
				this.dropdownOpen = true;
			}
			this.adjustMenuStyle();
		},
		
		hide: function() {
			if (this.dropdownMenu) {
				this.dropdownMenu.closeDropDown(false);
				this.dropdownOpen = false;
			}
		},
		
		destroy: function() {
			if (this.dropdownMenu) {
				if(this.dropdownMenu.destroyRecursive) {
					this.dropdownMenu.destroyRecursive();
				} else if (this.dropdownMenu.destroy) {
					this.dropdownMenu.destroy();
				}
			}
			if(this.userProfileDialog) {
				this.userProfileDialog.destroy();
			}

			if(this.connects) {
				for(var i=0; i < this.connects.length; i++) {
					dojo.disconnect(this.connects[i]);
					this.connects[i] = null;
				}
				this.connects = null;
			}
			this.inherited(arguments);
		},
		
		_showUserProfile: function() {
			if (!this.userProfileDialog) {
				this.userProfileDialog = new com.ibm.bpm.social.widgets.dialogs.UserProfileDialog();
			}
			this.userProfileDialog.show();
		},
		
		_blockBlur: function(value, self) {
			self.blurBlock = value;
		},
		
		_blurActions: function(self) {
			if(self.dropdownOpen && !self.blurBlock) {
				self.toggle();
			}
		}
	}
);
		
			
			
		

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.PortalSidebar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.PortalSidebar"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.PortalSidebar");






dojo.declare(
	"com.ibm.bpm.social.portal.PortalSidebar",
	[dijit.layout._LayoutWidget, dijit._Templated],
	{
	
    templateString: "",
	CLASS_NAME: "PortalSidebar",
	LOG_LEVEL: 2,
  
	templateString:"<div class=\"bpm-social-portalsidebar\" dojoAttachPoint=\"portalSidebarWidget\">\n\t<ul class=\"tab-list\" dojoAttachPoint=\"tabList\"></ul>\n\t<div class=\"widget-area\" dojoAttachPoint=\"containerNode\"></div>\n</div>\n",
	constructor: function(){
		var FUNC_NAME = "constructor";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.tabs = null;
		this.selectedTabIndex = -1;		
		this.instantiatedDeclaratively = false;
		this.isDisplayingIcons = false;
		this.canDisplayIcons = true;
		this.showWidgetTopic = "defaultShowWidgetTopic";
		this.portalSidebarSubscriptions = [];
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},            	
	
    postCreate: function() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
				
		this.tabs = [];
		
		dijit.setWaiRole(this.tabList, "tablist");
		
		var childWidgets = dojo.query("div[tabName]", this.containerNode);
		if(childWidgets.length>0) {
			this.instantiatedDeclaratively = true;
			for(var i=0; i<childWidgets.length; i++) {
				var title = childWidgets[i].getAttribute("tabName");
				if(title=="") {
					title = "Tab "+(i+1);
				}
				this._createTab(title);
			}
		}
		
		this.portalSidebarSubscriptions.push(dojo.subscribe(this.showWidgetTopic, dojo.hitch(this, function(data) {
			window.setTimeout(dojo.hitch(this, '_fixSidebarTabWidth'), 150);
		})));
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    },
	
	addTab: function(title, widget, iconClass, iconCharacter, addSymbol) {
		var FUNC_NAME = "addTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.addChild(widget);
		dijit.setWaiRole(widget, "tabpanel");
		dojo.style(widget.domNode, "display", "none");
		var tab = this._createTab(title, widget, iconClass, iconCharacter, addSymbol);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		return tab;
	},
	
	setTabTag: function(index, tag, removeOnSelect, className) {
		if(typeof(removeOnSelect)=='undefined' || removeOnSelect==null) {
			removeOnSelect = true;
		}
		if(removeOnSelect && index==this.selectedTabIndex) {
			tag = null;
		}
		if(-1<index && index<this.tabs.length) {
			var tagEle = dojo.query(".tab-tag", this.tabs[index])[0];
			if(typeof(tagEle)!='undefined' && tagEle!=null) {
				dojo.removeClass(tagEle);
				dojo.addClass(tagEle, "tab-tag");
				if(typeof(tag)=='undefined' || tag==null || tag=="") {
					dojo.attr(tagEle, "innerHTML", "");
					dojo.style(tagEle, "display", "none");
					dojo.removeAttr(tagEle, "auto-remove");
				}
				else {
					if(typeof(className)!='undefined' && className!=null && className!="") {
						dojo.addClass(tagEle, className);
					}
					dojo.attr(tagEle, "auto-remove", ""+removeOnSelect);
					dojo.attr(tagEle, "innerHTML", tag);
					dojo.style(tagEle, "display", "");
				}
			}
		}
	},
	
	removeTabTag: function(index) {
		this.setTabTag(index, null);
	},
	
	_createTab: function(title, widget, iconClass, iconCharacter, addSymbol) {
		var FUNC_NAME = "_createTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(addSymbol) == 'undefined') {
			addSymbol = false;
		}
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Adding '"+title+"' tab to PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
		var li = dojo.create("li", {'tabIndex':"0",
			'tabnumber': this.tabs.length,
			'aria-controls':widget.id,
			'aria-selected':"false",
			'aria-label':title+". Unselected",
			'actual-title':title});
		
		// Defect 135222
		// Symbol to come before the text in rtl. I'll add it here as opposed to the resource bundle as I'm assuming the @ causes
		// an issues above in JAWS (added via 71295)
		if(addSymbol) {
            var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			if (userProfileObj.userPreferences['Locale'] == "he") { //Defect 136929 Hebrew only, not Bidi           
				title = title+' @';
			} else {            
				title = '@'+title;
            }
		}
		dijit.setWaiRole(li, "tab");
		var anchor = dojo.create("a", {'tabIndex':"-1"}, li);
		if(this.tabs.length==0){
			if(document.body.dir == 'rtl') { // if bidi
				dojo.addClass(li, "rightmost-tab");
				dojo.create("span", {'class':"tab-cap-right bpm-social-sidebar-nav-deselected-right"}, anchor);
			}
			else {
				dojo.addClass(li, "leftmost-tab");
				dojo.create("span", {'class':"tab-cap-left bpm-social-sidebar-nav-deselected-left"}, anchor);
			}
		}
		else {
			var prevTab = this.tabs[this.tabs.length-1];
			if(document.body.dir == 'rtl') { // if bidi
				dojo.removeClass(prevTab, "leftmost-tab");
				dojo.destroy(dojo.query("a .tab-cap-left", prevTab)[0]);
				dojo.create("span", {'class':"tab-divider bpm-social-sidebar-nav-divider"}, dojo.query("a", prevTab)[0]);
			}
			else {
				dojo.removeClass(prevTab, "rightmost-tab");
				dojo.destroy(dojo.query("a .tab-cap-right", prevTab)[0]);
				dojo.create("span", {'class':"tab-divider bpm-social-sidebar-nav-divider"}, dojo.query("a", prevTab)[0]);
			}
		}
		if(document.body.dir == 'rtl') { // if bidi
			dojo.addClass(li, "leftmost-tab");
			dojo.create("span", {'class':"tab-cap-left bpm-social-sidebar-nav-deselected-left"}, anchor);
		}
		else {
			dojo.addClass(li, "rightmost-tab");
			dojo.create("span", {'class':"tab-cap-right bpm-social-sidebar-nav-deselected-right"}, anchor);
		}
		if(typeof(iconClass) != "undefined") {
			if(this.isDisplayingIcons) {
				dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':""}, anchor);
				dojo.create("span", {'class':"tab-icon "+iconClass, 'style':"display: inline-block;", 'title':title}, anchor);
				dojo.create("span", {'class':"hi-contrast-icon", 'title':title, 'innerHTML': ""+iconCharacter}, anchor);
			}
			else {
				dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':title}, anchor);
				dojo.create("span", {'class':"tab-icon "+iconClass, 'style':"display: none;", 'title':title}, anchor);
				dojo.create("span", {'class':"hi-contrast-icon", 'style':"display: none;", 'title':title, 'innerHTML': ""+iconCharacter}, anchor);
			}
		}
		else {
			dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':title}, anchor);
			this.canDisplayIcons = false;
		}
		dojo.create("span", {'class':"tab-tag", 'style':"display: none;"}, anchor);
		com.ibm.bpm.social.util.common.Log.log(5, "Adding connectors to '"+title+"' tab of PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
		dojo.connect(anchor,'onclick',this,"_tabClickEvent");
		dojo.connect(li,'onkeypress',this,"_tabKeyPressEvent");
		if(document.body.dir == 'rtl') { // if bidi
			dojo.place(li, this.tabList, "first");
		}
		else {
			dojo.place(li, this.tabList);
		}
		
		this.tabs.push(li);


		//Round tab widths to 1 decimal point to ensure correct width calculation on chrome. Defect 124870
		var preferredWidth = (100/this.tabs.length).toFixed(1) +"%";

		com.ibm.bpm.social.util.common.Log.log(5, "Preferred width of '"+title+"' tab of PortalSidebar = "+preferredWidth, this.CLASS_NAME, FUNC_NAME);
		for(var i=0; i<this.tabs.length; i++) {
			com.ibm.bpm.social.util.common.Log.log(5, "Setting tab "+i+" of PortalSidebar to have width = "+preferredWidth, this.CLASS_NAME, FUNC_NAME);
			dojo.style(this.tabs[i], "width", preferredWidth);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		return this.tabs.length-1;
	},
	
	_tabClickEvent: function(evt) {
		var index = evt.target.parentNode.parentNode.getAttribute("tabnumber");
		this.selectTab(index);
	},
	
	_tabKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var index = evt.target.getAttribute("tabnumber");
			this.selectTab(index);
		}
	},
	
	selectTab: function(index) {
		var FUNC_NAME = "selectTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(-1<index && index<this.tabs.length) {
			var tag = dojo.query(".tab-tag", this.tabs[index])[0];
			if(dojo.attr(tag, "auto-remove")=="true") {
				this.removeTabTag(index);
			}
		}
		if(index!=this.selectedTabIndex) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Selecting tab "+index+" of PortalSidebar", this.CLASS_NAME, FUNC_NAME);
			var widgets = this.getChildren();
			if(-1<this.selectedTabIndex && this.selectedTabIndex<this.tabs.length){
				var tab = this.tabs[this.selectedTabIndex];
				var leftCap = dojo.query(".tab-cap-left", tab);
				if(leftCap.length > 0) {
					dojo.removeClass(leftCap[0], "bpm-social-sidebar-nav-selected-left");
					dojo.addClass(leftCap[0], "bpm-social-sidebar-nav-deselected-left");
				}
				var rightCap = dojo.query(".tab-cap-right", tab);
				if(rightCap.length > 0) {
					dojo.removeClass(rightCap[0], "bpm-social-sidebar-nav-selected-right");
					dojo.addClass(rightCap[0], "bpm-social-sidebar-nav-deselected-right");
				}
				var tabText = dojo.query(".tab-text", tab);
				if(tabText.length > 0) {
					dojo.removeClass(tabText[0], "bpm-social-sidebar-nav-selected-center");
					dojo.addClass(tabText[0], "bpm-social-sidebar-nav-deselected-center");
				}
				dojo.removeClass(tab, "selected");
				if(this.selectedTabIndex<widgets.length){
					dojo.style(widgets[this.selectedTabIndex].domNode, "display", "none");
				}
				dojo.attr(tab, 'aria-selected', 'false');
				dojo.attr(tab, 'aria-label', dojo.attr(tab, 'actual-title')+". Unselected");
			}
			if(-1<index && index<this.tabs.length){
				var tab = this.tabs[index];
				var leftCap = dojo.query(".tab-cap-left", tab);
				if(leftCap.length > 0) {
					dojo.removeClass(leftCap[0], "bpm-social-sidebar-nav-deselected-left");
					dojo.addClass(leftCap[0], "bpm-social-sidebar-nav-selected-left");
				}
				var rightCap = dojo.query(".tab-cap-right", tab);
				if(rightCap.length > 0) {
					dojo.removeClass(rightCap[0], "bpm-social-sidebar-nav-deselected-right");
					dojo.addClass(rightCap[0], "bpm-social-sidebar-nav-selected-right");
				}
				var tabText = dojo.query(".tab-text", tab);
				if(tabText.length > 0) {
					dojo.removeClass(tabText[0], "bpm-social-sidebar-nav-deselected-center");
					dojo.addClass(tabText[0], "bpm-social-sidebar-nav-selected-center");
				}
				dojo.addClass(tab, "selected");
				if(index<widgets.length) {
					dojo.style(widgets[index].domNode, "display", "block");
				}
				dojo.attr(tab, 'aria-selected', 'true');
				dojo.attr(tab, 'aria-label', dojo.attr(tab, 'actual-title')+". Selected");
			}
			this.selectedTabIndex = index;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED, [{}]);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	layout: function() {},
	
	startup: function(selectedTabIndex) {
		var FUNC_NAME = "startup";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		var i;
		
		if(this.instantiatedDeclaratively) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Instantiating PortalSidebar declaratively! (It was created via HTML.)", this.CLASS_NAME, FUNC_NAME);
			var widgets = this.getChildren();
			for(i=0; i<widgets.length; i++) {
				dojo.style(widgets[i].domNode, "display", "none");
			}
		}
		if (selectedTabIndex) {
			this.selectTab(selectedTabIndex);
		} else {
			this.selectTab(0);
		}
		this._fixSidebarHeight();
		window.setTimeout(dojo.hitch(this, '_fixSidebarTabWidth'), 150);
		this.connect(window, 'onresize', '_fixSidebarHeight');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_fixSidebarHeight: function() {
        try {
    		var y = dojo.coords(this.domNode).y;
    		if (y > 0) {
    			var h = document.documentElement.clientHeight;
    			this.domNode.style.height = (h - y - 50) + 'px';
    			var yWidgets = dojo.coords(this.containerNode).y;
    			this.containerNode.style.height = Math.max(50, h - yWidgets - 40) + 'px';
                com.ibm.bpm.social.util.common.Log.log(2, "y: "+y+", h: "+h+", yWidgets: "+yWidgets, "PortalSideBar");
    		} else {
                com.ibm.bpm.social.util.common.Log.log(2, "y: "+y, "PortalSideBar");
    			//window.setTimeout(dojo.hitch(this,'_fixSidebarHeight'), 250);//defect 99422
    		}
    	} catch (error) {
    		// throwing error on log out
            com.ibm.bpm.social.util.common.Log.log(2, "caught error: "+error, "PortalSideBar");
    	}
	},
	
	_fixSidebarTabWidth: function() {
		// if the domNode height is not set, try again
		if (!this.domNode.style.height) {
			this._fixSidebarHeight();
		}
		for(var i=0; i<this.tabs.length; i++) {
			var tabTitle = dojo.query(".tab-text", this.tabs[i])[0];
			var titleText = dojo.attr(tabTitle, 'innerHTML');
			dojo.attr(tabTitle, 'innerHTML', titleText+"&nbsp;");
			dojo.style(tabTitle, 'fontWeight', 'bold');
			if(tabTitle != null) {
				if(tabTitle.scrollWidth <= 0 || tabTitle.clientWidth <= 0) {
					dojo.attr(tabTitle, 'innerHTML', titleText);
					dojo.style(tabTitle, 'fontWeight', '');
					return;
				}
				if(tabTitle.scrollWidth > tabTitle.clientWidth) {
					dojo.attr(tabTitle, 'innerHTML', titleText);
					dojo.style(tabTitle, 'fontWeight', '');
					this.displayAlternateIcons();
					break;
				}
				dojo.attr(tabTitle, 'innerHTML', titleText);
				dojo.style(tabTitle, 'fontWeight', '');
			}
		}
	},
	
	displayAlternateIcons: function() {
		var FUNC_NAME = "displayAlternateIcons";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(this.canDisplayIcons) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Displaying alternate icons for PortalSidebar tabs due to inadequate width.", this.CLASS_NAME, FUNC_NAME);
			this.isDisplayingIcons = true;
			for(var i=0; i<this.tabs.length; i++) {
				dojo.query(".tab-text", this.tabs[i])[0].innerHTML = "";
				if(dojo.hasClass(dojo.query("body")[0], "dijit_a11y")) {
					dojo.style(dojo.query(".hi-contrast-icon", this.tabs[i])[0], "display", "inline-block");
				}
				else {
					dojo.style(dojo.query(".tab-icon", this.tabs[i])[0], "display", "inline-block");
				}
			}
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
    
	destroy: function(finalize) {
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		for(var i=0; i<this.portalSidebarSubscriptions.length; i++) {
			dojo.unsubscribe(this.portalSidebarSubscriptions[i]);
		}
		this.inherited('destroy',arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
});



}

if(!dojo._hasResource["com.ibm.bpm.social.portal.PageHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.PageHandler"] = true;
// PageHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.portal.PageHandler");















dojo
  .declare(
    "com.ibm.bpm.social.portal.PageHandler",
    null,
    {

      /**
       * PageHandler : a class handling setting and getting the default start page for the work tab
       * 
       * <p>
       * Mapping of pages to string:
       * 
       * <table width="100%" >
       * 
       * <tr>
       * <td><b> Page </b></td>
       * <td><b> Stored string </b></td>
       * </tr>
       * <tr>
       * <td> My Tasks </td>
       * <td> /tasks </td>
       * </tr>
       * <tr>
       * <td> Dashboards tab </td>
       * <td> /dashboards?dashboard=PATH_TO_DASHBOARD </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Open Tasks </td>
       * <td> /tasks/open?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Completed Tasks </td>
       * <td> /tasks/completed?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> </td>
       * <td> </td>
       * </tr>
       * <tr>
       * <td> Saved Search </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i> </td>
       * </tr>
       * <tr>
       * <td> Saved Search with search string </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i>&amp;searchFilter=<i>URL_encoded_search_string</i>
       * </td>
       * </tr>
       * <tr>
       * <td> Sorted Saved Search with search string </td>
       * <td> /tasks/queries<b>?query</b>=<i>URL_encoded_name_of_saved_search</i><b>&amp;searchFilter</b>=<i>URL_encoded_search_string</i><b>&amp;sortAttributes</b>=<i>sortAttr1,sortAttr2,..</i><b>&amp;sortOrder</b>=<i>order1,order2,...</i>
       * </td>
       * </tr>
       * </table>
       * 
       */


      _storedStartPageSettings: null,
      _currentPageSettings: null,

      _startPageButton: null, // DOM node of the icon/button we are maintaining

      // DOM connections tracker
      _connects: [],

      _activePageWidget: null,
      _workPageWidget: null,
      _dashboardPageWidget: null,
      _hashsub: null,
      _navsub1: null,
      _navsub2: null,
      _navsub3: null,
	  _workDashboardItem: null,
	  _pageHander: null,
	  _preventNavigation: null,


      /**
       * Constructor
       * 
       * @type {void}
       */
      constructor: function constructor_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Return a copy of PageSettings of this instance. <br>
       * Since we return a copy, we can protect our PageSettings object.
       * 
       * @type {void}
       */
      getStoredPageSettings: function getStoredPageSettings_PageHandler(){
        com.ibm.bpm.social.util.common.Log.log(3);
        return new com.ibm.bpm.social.util.common.PageSettings(this._storedStartPageSettings);
      },


      /**
       * Cleanup handler <br>
       * Note: since this is not a widget, needs to called manually
       * 
       * @type {void}
       */
      destroy: function destroy_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

		if (this._hashsub) {
			dojo.unsubscribe(this._hashsub);
			this._hashsub = null;
		}
		if (this._navsub1) {
			dojo.unsubscribe(this._navsub1);
			this._navsub1 = null;
		}
		if (this._navsub2) {
			dojo.unsubscribe(this._navsub2);
			this._navsub2 = null;
		}
		if (this._navsub3) {
			dojo.unsubscribe(this._navsub3);
			this._navsub3 = null;
		}
        while (this._connects.length > 0) {
          var c = this._connects.pop();
          try {
            dojo.disconnect(c);
          }
          catch (e) {
            // we are not really interested in errors while cleanup  
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Read the users profile setting "Portal default page" and parse it into the members
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings}
       */
      readFromUserProfile: function readFromUserProfile_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        var startPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        var userProfileString;

        if (userProfile && userProfile.userPreferences &&
          userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME]) {
          userProfileString = userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME];
          com.ibm.bpm.social.util.common.Log.log(2, "userProfile string is " + userProfileString);

          var newStartPageSettings = startPageSettings.parseFromString(userProfileString, true);
          if (!newStartPageSettings.isValid) {
            // String is invalid
            // the original startPageSettings object is unmodified and has default values
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is invalid, using default string: " +
              startPageSettings.getCurrentProfileString());
          }
          else {
            // the string is valid and the values are mixed into the original PageSettings object
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is valid, using now string: " +
              startPageSettings.getCurrentProfileString());
          }
        }
        else {
          // if the start page is not set, show My Tasks
          com.ibm.bpm.social.util.common.Log.log(2, "user attribute not set, using default string: " +
            startPageSettings.getCurrentProfileString());
        }


        if (com.ibm.bpm.social.util.common.Log.logLevel > 1) {
          com.ibm.bpm.social.util.common.Log.logObject(2, {
            Tab: startPageSettings.tab,
            Filter: startPageSettings.filterConfigId,
            Saved_Search: startPageSettings.savedSearchName,
            Search_Filter: startPageSettings.searchFilterString,
            Sort_Attributes: startPageSettings.sortAttributes,
            dashboardURL: startPageSettings.dashboardURL
          }, "Got following settings from user profile: ");
        }

        // update the _storedStartPageSettings to the newest settings, but use a separate copy
        this._storedStartPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageSettings;
      },


      /**
       * write the default start page settings into the user profile
       * 
       * @param {com.ibm.bpm.social.util.common.PageSettings} PageSettings if given, the settings to be
       *        written to the profile
       * 
       * @type {void}
       */
      _writeStartPageToUserProfile: function _writeStartPageToUserProfile_PageHandler(startPageSettings){
        com.ibm.bpm.social.util.common.Log.logEntry(2, startPageSettings.getCurrentProfileString);
        if (startPageSettings) {
          com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME,
            startPageSettings.getCurrentProfileString());
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * 
       * @type {void}
       */
      setStartPage: function setStartPage_PageHandler(startPageSettings, startPageButton){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var newPageSettings = startPageSettings;

        if (startPageButton.isSelected === true) {
          // we are already on the start page -> User wants to reset the start page to the default start page
          newPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        }

        this._writeStartPageToUserProfile(newPageSettings);
        this._storedStartPageSettings = newPageSettings;

        this._currentPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);
        this.checkAndUpdateStartPageIcon(this._currentPageSettings, startPageButton);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Compare the new content of the start page against the stored one and update the page icon if the
       * content changed
       * 
       * @type {void}
       */
      checkAndUpdateStartPageIcon: function checkAndUpdateStartPageIcon_PageHandler(startPageSettings, startPageButton /* domNode */){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler");

        if (startPageSettings.getCurrentProfileString() == this._storedStartPageSettings.getCurrentProfileString()) {
          // set the check mark icon
          dojo.removeClass(startPageButton, "startPageBookmark-unselected");
          dojo.addClass(startPageButton, "startPageBookmark-selected");
          startPageButton.isSelected = true;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "&#x2713;"; // a11y check mark sign
          if (this._storedStartPageSettings.getCurrentProfileString() == startPageSettings.defaultPageString) {
            // no reset to the default page since we are already there   
            startPageButton.title = this._nlsResources["page_handler_is_default_page"];
          }
          else {
            startPageButton.title = this._nlsResources["page_handler_is_default_page_with_reset"];
          }
        }
        else {
          // set the plus icon
          dojo.removeClass(startPageButton, "startPageBookmark-selected");
          dojo.addClass(startPageButton, "startPageBookmark-unselected");
          startPageButton.isSelected = false;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "+"; //a11y
          startPageButton.title = this._nlsResources["page_handler_set_start_page_title"];
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler");
        return;
      },


      /**
       * Return a div formatted as a start page button
       * 
       * @type {void}
       */
      createStartPageIcon: function createStartPageIcon_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // create the start page icon/button
        var startPageButton = document.createElement("div");

        startPageButton.tabIndex = "0";
        startPageButton.isSelected = true;
        dojo.addClass(startPageButton, "startPageBookmark-selected");

        var a11yNode = document.createElement("span"); //<span class="bpm-social-a11y">+</span>
        a11yNode.className = "bpm-social-a11y";
        a11yNode.innerHtml = "&#x2713;"; // a11y check mark sign 
        startPageButton.appendChild(a11yNode);

        dijit.setWaiRole(startPageButton, "button");
        dijit.setWaiState(startPageButton, "label", this._nlsResources["page_handler_set_start_page"]);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageButton;
      },


      /**
       * Initialize the dashboard organizer dropdown on the organize tab
       * 
       * @type {void}
       */
      createDashboardOrganizer: function createDashboardOrganizer_PageHandler(openOnCreate){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this.dashboardOrganizer) {
          var div = document.createElement("div");
          dojo.byId("processPortalDashboardOrganizerId").appendChild(div);
          this.dashboardOrganizer = new com.ibm.bpm.social.portal.DashboardOrganizer({"openOnCreate":openOnCreate}, div);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Return the dashboard organizer
       * 
       * @type {DashboardOrganizer}
       */
	  getDashboardOrganizer: function getDashboardOrganizer_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer;
	  },
	  
      /**
       * Return the displayed dashboards
       * 
       * @type {list}
       */
	  getDisplayedDashboards: function getDisplayedDashboards_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer.getDisplayedDashboards();
	  },
	  
      /**
       * Return the hidden dashboards
       * 
       * @type {list}
       */
	  getHiddenDashboards: function getHiddenDashboards_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer.getHiddenDashboards();
	  },

      /**
       * This functions captures a click on the banner to check if the drop down needs to be closed. (because
       * onblur handler of the DashboardOrganizer doesn't work here (for any reason)
       * 
       * @param event
       * @type void
       */

      hideDashboardOrganizer: function hideDashboardOrganizer_PageHandler(event){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        var target = null;
        if (dojo.isIE == 8) {
          target = event.srcElement;
        }
        else {
          target = event.target;
        }

        if (target && target.id && target.id != "processPortalDashboardTabDropDownImg" && target.id != "processPortalDashboardOrganizerLink" 
			&& target.id.indexOf("id_dashboardOrganizerSnapshotIcon")<0 ) {
          if (this.dashboardOrganizer && this.dashboardOrganizer.dropdownOpen) {
            com.ibm.bpm.social.util.common.Log.log(2, "Caught click on banner and dashboard organizer is open. Hiding ...");
            this._hideDashboardOrganizer();
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(3);
        return false;
      },

      /**
       * handle a click on the dashboard organizer dropdown arrow
       * 
       * @type {void}
       */
      onDashboardOrganizerClick: function onDashboardOrganizerClick_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!this.dashboardOrganizer) {
          // creating the drop down also opens it
          this.createDashboardOrganizer(true);
        }
        else if (this.dashboardOrganizer.dropdownOpen) {
          //DashboardOrganizer is already open, close it);
		  this._hideDashboardOrganizer();
        }
        else {
          this.dashboardOrganizer.show();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },
	  
	  _hideDashboardOrganizer: function _hideDashboardOrganizer_PageHandler() {
          this.dashboardOrganizer.hide(true);  //true: save organizer user pref setting
	  },


      /**
       * Event handler for the onDashboardRestart event, delegate this to the dashboard controller
       * 
       * @type {void}
       */
      restartDashboard: function restartDashboard_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: "RESTART_DASHBOARD",
          display: "RESTART_DASHBOARD"
        }]);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
      
      _taskCompletion: function _taskCompletion_PageHandler(taskId) {
        var rc = false;
        if (taskId) {
         	var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
				'taskId': taskId
			});
			if (taskData) {
				if (taskData.errorInfo) {
					// 401 is unathorized, so the task exists and we
					// want to let the default error message show up,
					// otherwise assume the id is bad 
					if (taskData.errorInfo.ioArgs.xhr.status != 401) {
						if (this._workDashboardItem != null) {
							var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["task_not_available"]);
						}
						taskId = null;
					}
				}/*  else {
					// If the work dashboard isn't exposed we need to handle some messaging here.
					if (this._workDashboardItem == null && taskData.state && taskData.state == "STATE_FINISHED") {
						var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(nls["task_already_completed"]);
						taskId = null;
					}
				} */
			} else {
				if (this._workDashboardItem != null) {
					var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
					com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["task_not_available"]);
				}
				taskId = null;
			}
		}
		if (taskId) {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this.showTaskCompletion(taskId,0);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this.showTaskCompletion(taskId,0);
					}
				}));
				com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
			rc = true;
		} 
        return rc;
      },
      
      _processInstance: function _processInstance(piid) {
      	var rc = false;
		if (piid) {
			var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
				'header': true, 
				'instanceId': piid,
				'skipCache': true
			});
			if (instanceData) {
				if (instanceData.errorInfo) {
					// 401 is unathorized, so the process exists and
					// want to let the default error message show up,
					// otherwise assume the id is bad
					if (instanceData.errorInfo.ioArgs.xhr.status != 401) {
						var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["process_instance_not_available"]);
						piid = null;
					}
				}
			} else {
				var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["process_instance_not_available"]);
				piid = null;
			}
		}
		if (piid) {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this.displayLegacyInstanceDetails(piid,0);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this.displayLegacyInstanceDetails(piid,0);
					}
				}));
				com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
			rc = true;
		}
		return rc;
      },
      
      _showSavedSearch: function _showSavedSearch_PageHandler(item,searchFilter) {
		com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
			savedSearchName: item.params.name,
			searchFilter: searchFilter
		}]);
	    dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(item.params)]);
      },
      
      _findSavedSearch: function _findSavedSearch_PageHandler(savedSearchName,searchFilter,errorMessageData,showSearchIfFound) {
       	var item;
		var found = false;
		for ( var i = 0; i < com.ibm.bpm.social.portal.dashboardItems.length; i++) {
			item = com.ibm.bpm.social.portal.dashboardItems[i];
			if (item.params.name == savedSearchName && item.params.type=="queue") {
				found = true;
				break;
			}
		}
		if (found && showSearchIfFound) {
			// If showErrorMessage is true then tell the user they are being redirected to their default start page
			if (errorMessageData && errorMessageData.showErrorMessage && errorMessageData.showErrorMessage==true) {			
				// If currentHash is the string "null" then this code has been reached during login and 
				//   there's no need to display any message.
				// If currentHash is defined but not the string "null", show error message indicating that the 
				//     requested URL is not available and the user's start page will be shown.  
				// If the current hash is available then supply it in the messasge.
				if (errorMessageData.currentHash && errorMessageData.currentHash=="null") {
					// do nothing
				} else {
					if (errorMessageData.currentHash) {
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
							dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [errorMessageData.currentHash]));
					} else {
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
					}
				}
			}
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this._showSavedSearch(item,searchFilter);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this._showSavedSearch(item,searchFilter);
					}
				}));
   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
      	}
      	return found;
      },
      
      _showTaskList: function _showTaskList_PageHandler(state,searchFilter) {
		com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
			state: state,
			searchFilter: searchFilter
		}]);
		if (this._workDashboardItem != null) {
			this._workDashboardItem.params["isWorkTab"]=true;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);
		}
      },
      
      _parseQueryString: function _parseQueryString(url) {
		if (!url) url = "";
		var urlParamString = url.substring(url.indexOf('?')+1);
		var urlParamList = urlParamString.split('&');
		var urlParams = new Object();
		// We need to capture whether or not this is a scoreboard.  We do this
		//   by seeing if the url contains "viewScoreboard.do". If this is false, check again for "serviceName=Scoreboard".
		var isScoreboard = (url.indexOf("viewScoreboard.do")>-1)?true:false;
		if (isScoreboard == false) {
			isScoreboard = (url.indexOf("serviceName=Scoreboard")>-1)?true:false;
		}
		for (var p1=0; p1<urlParamList.length; p1++) {
			var p2 = urlParamList[p1].indexOf('=');
			if (p2 > -1) {
				urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
			}
		}
		urlParams["isScoreboard"] = isScoreboard;
		return urlParams;
      },
      
      
      _findDashboardFromURL: function _findDashboardFromURL_PageHandler(dashboardURL) {
        	var dashboardParams = this._parseQueryString(dashboardURL);
		return this._findDashboardFromParams(dashboardParams);
	 },

	 _compareParams: function _compareParams(param1,param2) {
		var match = false;
		if (param1 == param2) {
			match = true;
		} else if (encodeURIComponent(param1) == param2) {
			match = true;
		} else {
			try {
				var d = decodeURIComponent(param2);
				if (param1 == d) {
					match = true;
				}
			} catch (e) {}
		}
		return match;
	 },

      _findDashboardFromParams: function _findDashboardFromParams_PageHandler(dashboardParams) {
		// Note that a scoreboard URL does not contain a 'snapshot' request param even if the scoreboard
		//    is from a snapshot.   A scoreboard URL just has:      executeServiceByName?processApp=ZZZ&serviceName=Scoreboard&id=2015.71
	     var item = null;
     	   	var found = false;
		if ((dashboardParams.processApp == 'TWP') && (dashboardParams.serviceName == 'Scoreboard') && dashboardParams.id) {
			dashboardParams.processApp = undefined;
			dashboardParams.serviceName = undefined;
			dashboardParams.isScoreboard = true;
		}   
		for ( var i = 0; (i < com.ibm.bpm.social.portal.dashboardItems.length) && !found; i++) {
			item = com.ibm.bpm.social.portal.dashboardItems[i];
			if (item && item.params && item.params.runURL) {
				var runParams = this._parseQueryString(item.params.runURL);
				// if this is the work tab we don't care about the rest of the logic
				if (!item.params.isWorkTab) {
					// Make sure we compare items of the same type
					var typesMatch=(dashboardParams.isScoreboard===runParams.isScoreboard);
					if (typesMatch) {
						if ((dashboardParams.processApp == runParams.processApp) &&
							this._compareParams(dashboardParams.serviceName,runParams.serviceName)) {
							if (!dashboardParams.snapshot && !runParams.snapshot && !dashboardParams.isScoreboard) {
								// No snapshot specified, so use default;  NOTE the check for not scoreboard, since scoreboard URLs don't have a snapshot param
								//  This handles a sneaky case where the Tip is not the top-level item because one of the
								//  snapshots has been set as the default version.
								found = true;
							} else if ((dashboardParams.snapshot && runParams.snapshot) && 
								this._compareParams(dashboardParams.snapshot,runParams.snapshot)) {
								// Anything except a scoreboard, and the snapshot params match
								found = true;
							} else if (dashboardParams.isScoreboard===true && (dashboardParams.id == runParams.id)) {
								// Scoreboard, not in a snapshot, the ids must match
								found=true;
							} else if (item.params.snapshotItems && dojo.isArray(item.params.snapshotItems)) {
								//  This can be a dashboard or scoreboard (not Saved Search).
								//  See if the dashboard URL is found in the snapshots
								var snapItem = null;
								for (var j=0; (j<item.params.snapshotItems.length) && !found; j++) {
									snapItem=item.params.snapshotItems[j];
									if (snapItem && snapItem.params && snapItem.params.runURL) {
										var snapParams = this._parseQueryString(snapItem.params.runURL);
										// Here if this is not a scoreboard....
										if (!dashboardParams.isScoreboard) {
											if (this._compareParams(dashboardParams.snapshot,snapParams.snapshot)) {
												found = true;
											}
										} else {
											if (dashboardParams.id == snapParams.id) {
												found = true;
											}
										}
									}
								}
								if (found) {
									item = snapItem;
								}
							} 
							// defect 137099
							// if a snapshot was not specified and the logic did not find a
							// tip, the default should be used (process server)
							if (!found && !dashboardParams.snapshot && !dashboardParams.isScoreboard) {
								found = true;
							}
						}
					} 
				}
			}
		}
		if (!found) {
		    // Check exposed URLs
			item = null;
			for ( var i = 0; (i < com.ibm.bpm.social.portal.exposedURLs.length) && !found; i++) {
				item = com.ibm.bpm.social.portal.exposedURLs[i];
				if (item && item.params && item.params.runURL) {
					var runParams = this._parseQueryString(item.params.runURL);
					if ((dashboardParams.processApp == runParams.processApp) &&
						this._compareParams(dashboardParams.serviceName,runParams.serviceName)) {
						if (!dashboardParams.snapshot && !runParams.snapshot) {
							// No snapshot specified, so use default;
							found = true;
						} else if ((dashboardParams.snapshot && runParams.snapshot) && 
							this._compareParams(dashboardParams.snapshot,runParams.snapshot)) {
							// snapshot params match
							found = true;
						} else if (item.params.snapshotItems && dojo.isArray(item.params.snapshotItems)) {
							//  See if the URL is found in the snapshots
							var snapItem = null;
							for (var j=0; (j<item.params.snapshotItems.length) && !found; j++) {
								snapItem=item.params.snapshotItems[j];
								if (snapItem && snapItem.params && snapItem.params.runURL) {
									var snapParams = this._parseQueryString(snapItem.params.runURL);
									// Compare snapshots
									if (this._compareParams(dashboardParams.snapshot,snapParams.snapshot)) {
										found = true;
									}
								}
							}
							if (found) {
								item = snapItem;
							}
						}
						// defect 137099
						// if a snapshot was not specified and the logic did not find a
						// tip, the default should be used (process server)
						if (!found && !dashboardParams.snapshot) {
							found = true;
						}
					}
				}
			}
		}
		if (!found) {
			item = null;
		}
		return item;
	},
	
	_showDashboard: function _showDashboard_PageHandler(item,params) {
		if (item && item.params && item.params.isWorkTab && item.params.isWorkTab==true) {
			this.selectWorkTab();
		} else {
			// We need to remove the 'id' attribute in params if this is a scoreboard because it gets erroneously added to the url
			var isScoreboard = (item.params.runURL.indexOf("viewScoreboard.do")>-1)?true:false;
			if (isScoreboard && params && params.id) {
				delete params.id;
			}
			if (!this.selectDashboardsTab(item.params.runURL, item.title, undefined, params)) {			
				dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(item.params)]);
			}
		}
    },

	// This is called by the organizer if the work dashboard is exposed
	setWorkTabInfo: function setWorkTabInfo_PageHandler(workTabItem) {
		this._workDashboardItem = workTabItem;
	},
	
	checkUrlHash: function checkUrlHash_PageHandler(hash) {
	  
		var rc = false;
		if (hash == undefined) {
  			hash = com.ibm.bpm.social.util.common.Util.getCurrentHash(true);
  		}
		dojo.cookie("com_ibm_bpm_process_portal_hash", null, {path: "/"});
		if (hash) {
			com.ibm.bpm.social.util.common.Util.setCurrentHash('');
			if (hash.charAt(0) != '/') {
				hash = '/' + hash;
			}
			var paramString = null;
			var i = hash.indexOf('?');
			if (i != -1) {
				paramString = hash.substring(i+1);
				var j = paramString.indexOf('=');
				var k = paramString.indexOf('?');
				while ((j != -1) && (k != -1) && (j > k)) {
					i = i + k;
					paramString = paramString.substring(k);
					j = paramString.indexOf('=');
					k = paramString.indexOf('?');
				}
				if (j != -1) {
					hash = hash.substring(0,i);
				} else {
					paramString = null;
				}
			}
			hash = hash.split('/');

			// convert legacy urls to new form
			if ((hash.length == 3) && (hash[1] == 'tasks')) {
				var taskId = parseInt(hash[2]);
				hash[1] = 'dashboards';
				hash[2] = 'TWP';
				hash[3] = 'BPM_WORK';
				paramString = 'tw.local.view=taskcompletion&tw.local.taskid='+taskId;
			}
			if ((hash.length == 3) && (hash[1] == 'processes')) {
				var piid = parseInt(hash[2]);
				hash[1] = 'dashboards';
				hash[2] = 'TWP';
				hash[3] = 'Process+Performance';
				paramString = 'tw.local.selectedInstanceId='+piid;
			}
			
			if ((hash.length > 3) && (hash[1] == 'dashboards')) {
				var processApp = hash[2];
				var serviceName = hash[3];
				var snapshot = null;
				if (hash.length > 4) {
					snapshot = hash[4];
					for (var s=5; hash.length > s; s++) {
						snapshot = snapshot + '/' + hash[s];
					}
				}
				var params = new Object();
				if (paramString) {
					var paramList = paramString.split('&');
					for (var i=0; i<paramList.length; i++) {
						var j = paramList[i].indexOf('=');
						if (j > -1) {
							var paramName = paramList[i].substring(0,j);
							if ((processApp == 'TWP') && (serviceName == 'BPM_WORK')) {
								if ((paramName == 'view') || 
									(paramName == 'id') || 
									(paramName == 'taskid') || 
									(paramName == 'searchFilter') ||	
									(paramName == 'state')) {
									paramName = 'tw.local.'+paramName;
								}
							}
							params[paramName] = paramList[i].substring(j+1);
							// if the parameter is a searchFilter, check for escaped ampersand
							// within the string
							while ((paramName == 'tw.local.searchFilter') &&
								   (params[paramName].charAt(params[paramName].length-1) == '\\') &&
								   ((i+1)<paramList.length)) {
								i++;
								params[paramName] = params[paramName] + '&' + paramList[i];
							}
							// if the parameter is a saved search id, the only other allowed
							// parameter is searchFilter and any other ampersands must be part
							// of the saved search name instead of parameter delimiters
							if ((paramName == 'tw.local.id') && (params['tw.local.view'] == 'savedsearch')) {
								while (((i+1)<paramList.length) && 
									(paramList[i+1].indexOf('searchFilter') != 0) &&
									(paramList[i+1].indexOf('tw.local.searchFilter') != 0)) {
									i++;
									params[paramName] = params[paramName] + '&' + paramList[i];
								}
							}
						}
					}
				}
				if (((this._workDashboardItem != null)) && (processApp == 'TWP') && (serviceName == 'BPM_WORK')) {
					if (params['tw.local.view'] == 'taskcompletion') {
						var taskId = params['tw.local.taskid'];
						rc = this._taskCompletion(taskId);
						if (!rc) {
							// log message for bad task id
							com.ibm.bpm.social.util.common.Log.log(2, "The specified task id is invalid: "+params.taskid,"PageHandler");
						} 
					}
					else if (params['tw.local.view'] == 'tasks') {
						if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
							this._showTaskList(params['tw.local.state'],params['tw.local.searchFilter']);
					    } else {
							this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
								if (this.viewReadySub) {
									dojo.unsubscribe(this.viewReadySub);
									this.viewReadySub = null;
									this._showTaskList(params['tw.local.state'],params['tw.local.searchFilter']);
								}
							}));
			   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
						}
						rc = true;
					}
					else if (params['tw.local.view'] == 'savedsearch') {
						rc = this._findSavedSearch(params['tw.local.id'], params['tw.local.searchFilter'], null, true);
						if (!rc) {
							// log message for bad task id
							com.ibm.bpm.social.util.common.Log.log(2, "The specified saved search id is invalid: "+params['tw.local.id'],"PageHandler");
						} 
					}
					else {
						com.ibm.bpm.social.util.common.Log.log(2, "The specified work view is not supported: "+params['tw.local.view'],"PageHandler"); 
					}
				} else if ((dojo.isIE < 9) && (processApp == 'TWP') && ((serviceName == 'Process+Performance') || (serviceName == 'Process Performance')) && params['tw.local.selectedInstanceId']) {
					if (this._workDashboardItem == null) {
						// work tab must be exposed on IE8 for process instance view 
						rc = false; 
					} else {
						// process instance view on IE8
						this._processInstance(params['tw.local.selectedInstanceId']);
						rc = true;		
					}
				} else {
					// find item for dashboard and show it
		        	var dashboardParams = this._parseQueryString(paramString);
					dashboardParams.processApp = processApp;
					dashboardParams.serviceName = serviceName;
					if (snapshot) {
						dashboardParams.snapshot = snapshot;
					}
					var item = this._findDashboardFromParams(dashboardParams);
					if (item) {
						this._showDashboard(item,params);
						rc = true;
					}
				}				
			} else {
				// log message for unsupported fragment
				com.ibm.bpm.social.util.common.Log.log(2, "The specified URL fragment is not supported: #"+hash.join('/'),"PageHandler"); 
			}
		}	
		return rc;
	  },

      isNavigationAllowed: function isNavigationAllowed_PageHandler() {
      	if (this._preventNavigation) {
      		com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this._preventNavigation);
      		return false;
      	}
      	return true;
      },	

      /**
       * Read the start page settings from user profile and show the start page
       * 
       * @type {void}
       */
      showStartPage: function showStartPage_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this.navsub1) {
        	this.navsub1 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.NAVIGATION,dojo.hitch(this,function(msg) {
        		if (msg.indexOf('%') != -1) {
        			try {
        				msg = decodeURIComponent(msg);
        			} catch (e) {
        				// if we cannot decode, it must no be encoded and
        				// just happens to have a % character in the msg
        			}
        		}
        		if (this.isNavigationAllowed()) {
	        		if (msg) {
	        			var rc = this.checkUrlHash(msg);
	        			if (rc != true) {
	        				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
		        			this.showStartPage();
	        			}
	        		} else {
	        			this.showStartPage();
	        		}
        		}
        	}));
        }
        if (!this.navsub2) {
        	this.navsub2 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.PREVENT_NAVIGATION,dojo.hitch(this,function(msg) {
        		this._preventNavigation = msg;
        	}));
        }
        if (!this.navsub3) {
        	this.navsub3 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION,dojo.hitch(this,function() {
        		this._preventNavigation = null;
        	}));
        }
		var message="";
        this._workPageWidget = dijit.byId("workPage");
        this._workPageDiv = dojo.byId("workPageDiv");
        this._dashboardPageWidget = dijit.byId("dashboardPage");
        this._dashboardPageDiv_Div = dojo.byId("dashboardPageDiv");
		// defect 130687
		this._dashboardPageDiv = dojo.byId("dashboardPage");
		if (this._dashboardPageDiv) {
			dojo.style(this._dashboardPageDiv, "overflow", "hidden");
		}
        if (!this.dashboardOrganizer) {
			this.createDashboardOrganizer(false);
            // ensure the drop down is closed
            if (this.dashboardOrganizer.dropdownOpen) {
				this.dashboardOrganizer.hide(false); // false: don't save dashboard order user pref
            }
         }
        if (!com.ibm.bpm.social.portal.exposedURLs) {
			this._getExposedURLList();
         }

		var currentHash = com.ibm.bpm.social.util.common.Util.getCurrentHash(true);
		var retVal = this.checkUrlHash();
		if (retVal != true) {
			var needToShowWorkTab=false;
			var dashboardURL = this._storedStartPageSettings.dashboardURL;

			if (this._storedStartPageSettings.tab === this._storedStartPageSettings.tabPrefixes.DASHBOARDS) {
	        	var dashboardParams = this._parseQueryString(dashboardURL);
				var item = null;
				if ((dojo.isIE < 9) && (dashboardParams.processApp == 'TWP') &&
					((dashboardParams.serviceName == 'Process+Performance') ||
					(dashboardParams.serviceName == 'Process Performance')) &&
					dashboardParams['tw.local.selectedInstanceId']) {
					// default start page is process instance view but the user is using
					// IE8 
					if (this._workDashboardItem != null) {
						item = "IE8_PROCESS_INSTANCE";
					}
				} else {
					item = this._findDashboardFromParams(dashboardParams);
				}
				if (item) {
					// If currentHash is the string "null" then this code has been reached during login and 
					//   there's no need to display any message.
					// If currentHash is defined but not the string "null", show error message indicating that the 
					//     requested URL is not available and the user's start page will be shown.  
					// If the current hash is available then supply it in the messasge.
					if (currentHash && currentHash=="null") {
						// do nothing
					} else {
						if (currentHash) {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
								dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [currentHash]));
						} else {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
						}
					}
					if (item == "IE8_PROCESS_INSTANCE") {
						this._processInstance(dashboardParams['tw.local.selectedInstanceId']);
					} else {
						// Show the dashboard!
						this._showDashboard(item,dashboardParams);
					}
				} else {
					com.ibm.bpm.social.util.common.Log.log(1, "Lookup of dashboard failed.");
					needToShowWorkTab=true;
          		}
	        } 
	        else if (this._storedStartPageSettings.savedSearchName) {
				// If the user's start page is a saved search, try to find it. If it is found then show it (_findSavedSearch() will show it).
				//  If it is not found, show the default start page.
				var errorMessageData={"showErrorMessage":true,"currentHash":currentHash?currentHash:null};
				if (!this._findSavedSearch(this._storedStartPageSettings.savedSearchName, this._storedStartPageSettings.searchFilterString, errorMessageData, true)) {
					needToShowWorkTab=true;
				}
	        }
			else if (this._storedStartPageSettings.tab === this._storedStartPageSettings.tabPrefixes.WORK) {
				// the default start page is the work page;  if it is exposed, then show message;
				//    if it is not exposed, situation will be handled below.
				if (this._workDashboardItem != null) {
					// If currentHash is the string "null" then this code has been reached during login and 
					//   there's no need to display any message.
					if (currentHash && currentHash=="null") {
					} else {
						if (currentHash) {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
								dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [currentHash]));
						} else {
							// defect 127208
							// I do not know why we were showing an error here since there is no
							// hash being used and the work tab is available so I am commenting
							// out the error message for this case
							// com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
						}
					}
				}
				needToShowWorkTab=true; 
	        }
			else {
				// In all other cases switch to work page as default page
				needToShowWorkTab=true; 
	        }
			if (needToShowWorkTab==true) {
				// Make sure work tab is exposed
				if (this._workDashboardItem != null) {
					if (!this._storedStartPageSettings.savedSearchName) {
						// If the start page was actually the work tab, don't show any error
						if (this._storedStartPageSettings.tab != this._storedStartPageSettings.tabPrefixes.WORK) {
							// Show error message indicating that the user's start page is not available and the default start page will be shown.  
							if (dashboardURL && dashboardURL.length>0) {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_no_longer_available"],
									dojo.string.substitute(this._nlsResources["page_no_longer_available_details"], [dashboardURL]));
							} else {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_no_longer_available"]);
							}
						}
					}
					var state='open';
					if (this._storedStartPageSettings.filterConfigId == 'completedTasks') {
						state='close';
					}
					if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
						this._showTaskList(state,this._storedStartPageSettings.searchFilterString);
					} else {
						this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
							if (this.viewReadySub) {
								dojo.unsubscribe(this.viewReadySub);
								this.viewReadySub = null;
								this._showTaskList(state,this._storedStartPageSettings.searchFilterString);
							}
						}));
		   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
					}
				} else {
					com.ibm.bpm.social.util.common.Log.log(1, "Lookup of dashboard failed.");
					// Find the best possible alternative to the work page
					// Get the tab bar widget
					var tabBarWidget = dijit.byId("processPortalTabBar");
					// If there are any visible tabs, display the first in the list
					var fallbackPageFound=false;
					var itemToDisplay=null;
					if (tabBarWidget && tabBarWidget.tabList && tabBarWidget.tabList.length>0) {
						// _showDashboard is expecting an object with attributes called title and params.  So
						//     we'll make a temporary object from the tabList's params
						var itemToDisplay = {"title":tabBarWidget.tabList[0].params.title, "params":tabBarWidget.tabList[0].params};
						fallbackPageFound=true;
					} else {
						// Show first hidden tab, if there are any
						var hiddenDashboards = this.getHiddenDashboards();
						if (hiddenDashboards && hiddenDashboards.length>0) {
							var i=0;
							while( (i<hiddenDashboards.length) && !fallbackPageFound) {
								var hiddenItem=hiddenDashboards[i];
								if (hiddenItem && hiddenItem.data && hiddenItem.data.params) {
								    if (hiddenItem.data.params.runURL) {
										// dashboard or scoreboard
										itemToDisplay = this._findDashboardFromURL(hiddenItem.data.params.runURL);
										if (itemToDisplay) {
											fallbackPageFound=true;	
										}
									} else if (hiddenItem.data.params.type=="queue") {
										// saved search
										// try to find it, but don't show it yet
										var foundSavedSearch=this._findSavedSearch(hiddenItem.data.params.name, "", null, false);
										if (foundSavedSearch) {
											itemToDisplay = {"title":hiddenItem.data.params.name, "params":hiddenItem.data.params};
											fallbackPageFound=true;	
										}
									}
								}
								i++
							}
						}
					}
					// If we found a fallback!
					if (fallbackPageFound==true) {
						// Display message: Here if the work dashboard is not exposed.
						if (this._storedStartPageSettings.savedSearchName) {
							// If the page that wasn't found was a saved search:
							message = this._nlsResources["page_handler_saved_search_not_available"];
							details = this._nlsResources["page_handler_saved_search_not_available_details"];
							details = dojo.string.substitute(details, [this._storedStartPageSettings.savedSearchName]);
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
						} else {
							// Display message: Here if the page that wasn't found was a dashboard or scoreboard
							message = this._nlsResources["page_handler_page_not_available"];
							if (dashboardURL && dashboardURL.length>0) {
								details = this._nlsResources["page_handler_page_not_available_details"];
								details = dojo.string.substitute(details, [dashboardURL]);
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
							} else {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message);
							}
						}
						if (itemToDisplay.params.type == "scoreboardOrDashboard") {
							// If the fallback page is a dashboard or scoreboard:
							this._showDashboard(itemToDisplay);
						} else if (itemToDisplay.params.type == "queue") {
							// If the fallback page is a saved search, find and show:
							this._findSavedSearch(itemToDisplay.params.name,"",null,true);
						}
					} else {
						message = this._nlsResources["page_handler_no_dashboards_available"];
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message);
					}
				}
			}
        }
        
		if (!window.history || !window.history.replaceState) {
			// if the current browser does not support the HTML5 history
			// API then we need to watch for hash changes
			this._hashsub = dojo.subscribe('/dojo/hashchange',this,'_hashChanged');
		}		

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
	  
	  _getExposedURLList: function _getExposedURLList_pageHandler() {
		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		// get exposed URLs
	   	var itemListWidget = new com.ibm.bpm.social.widgets.ExposedItemsList();
		com.ibm.bpm.social.portal.exposedURLs = itemListWidget.getData(exposedItemsArray,["url"]);
	  },
      
      _hashChanged: function _hashChanged_pageHandler() {
      	var h = com.ibm.bpm.social.util.common.Util.getCurrentHash();
      	if (h && (h != com.ibm.bpm.social.util.common.Util.lastHash) && 
      		(h != deocodeURIComponent(com.ibm.bpm.social.util.common.Util.lastHash))) {
      		if (this.isNavigationAllowed()) {
      			this.checkUrlHash(h);
      		} else {
      			// reset hash since navigation is being prevented
      			window.location.hash = '#'+encodeURIComponent(com.ibm.bpm.social.util.common.Util.lastHash);
      		}
      	}
      },


      /**
       * displayInstanceDetails: Used by the dashboards page for cross-site scripting purposes. Opens the
       * process details for a given instance.
       * 
       * @param bpdId
       */
      displayInstanceDetails: function displayInstanceDetails_PageHandler(bpdId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		// make sure work tab is exposed
		if (this._workDashboardItem != null) {
			this.selectWorkTab();
			if (delay) {
				delay = delay + 500;
			} else {
				delay = 500;
			}
			//setTimeout helps with iframe publish issues
			setTimeout(function(){
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
				'instanceId': bpdId
			  }]);
			}, delay);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * displayLegacyInstanceDetails: Opens the legacy process details for a given instance.
       * 
       * @param bpdId
       */
      displayLegacyInstanceDetails: function displayLegacyInstanceDetails_PageHandler(bpdId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		// make sure work tab is exposed
		if (this._workDashboardItem != null) {
			this.selectWorkTab();
			if (delay) {
				delay = delay + 500;
			} else {
				delay = 500;
			}
			//setTimeout helps with iframe publish issues
			setTimeout(function(){
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE, [{
				'instanceId': bpdId
			  }]);
			  if (this._workDashboardItem != null) {
			  	this._workDashboardItem.params["isWorkTab"]=true;
			  	dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);          
	  		  }
			}, delay);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },



      /**
       * 
       * @type {void}
       */
      showTaskCompletion: function showTaskCompletion_PageHandler(tkId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._workPageDiv.getAttribute('data-loaded') == "false" && this._workDashboardItem != null) {
          this._workPageWidget.set("href", ppConfig.portalContextRootJS + '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
        if (delay) {
        	delay = delay + 500;
        } else {
        	delay = 500;
        }
        //setTimeout helps with iframe publish issues
        setTimeout(dojo.hitch(this, function(){
        
        	// defect 124601
        	// save the hash history before selecting the work tab and then
        	// restore it afterward to prevent the work tab from moving to
        	// the top of the history
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
            "taskId": tkId
          }]);

        	var tmpHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	 		com.ibm.bpm.social.util.common.Util._portalHashHistory = new Array();
			com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
			com.ibm.bpm.social.util.common.Util._portalHashHistory = tmpHistory;
			tmpHistory = null;
			
		  if (this._workDashboardItem != null) {
			  this._workDashboardItem.params["isWorkTab"]=true;
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);          
		  }
        }, delay));
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

	  isWorkDataLoaded: function isWorkDataLoaded_PageHandler() {
		return (this._workPageDiv.getAttribute('data-loaded') == "true");
   	  },

	  loadWorkData: function loadWorkData_PageHandler() {
        if (this._workPageDiv.getAttribute('data-loaded') == "false" && this._workDashboardItem != null) {
          this._workPageWidget.set("href", ppConfig.portalContextRootJS+ '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
	  },
	
      /**
       * Open the work tab
       * 
       * @param tab
       */
      selectWorkTab: function selectWorkTab_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		if (this._workDashboardItem != null) {
			this._workDashboardItem.params["isWorkTab"]=true;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);
			this.loadWorkData();
			var wasRealSwitch = this.setSelectedTab('processPortalWorkTab');
			if (!wasRealSwitch) {
			  // Trigger a refresh if the user clicked again on the work tab
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
			}
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },

      /**
       * Open the dashboards tab
       * 
       * @param tab
       */
      selectDashboardsTab: function selectDashboardsTab_PageHandler(dashboardURL, dashboardTitle, tabId, params){
    	  
        if(typeof(dashboardURL)=='undefined' || dashboardURL==null) {
          return false;
        }

        if(typeof(tabId)=="undefined" || tabId==null){
          tabId = 'dashboardOrganizerTab';
        }
    	  
        if (this._dashboardPageDiv_Div.getAttribute('data-loaded') == "false") {
            this._dashboardPageWidget.startup();
            this._dashboardPageDiv_Div.setAttribute('data-loaded', "true");
        }
        
        var dashboardParams = null;
        if (dojo.isIE < 9) {
        	dashboardParams = this._parseQueryString(dashboardURL);
        }
        if (dashboardParams && (dashboardParams.processApp == 'TWP') &&
        	((dashboardParams.serviceName == 'Process+Performance') ||
        	(dashboardParams.serviceName == 'Process Performance')) &&
        	params['tw.local.selectedInstanceId']) { 

			var piid = parseInt(params['tw.local.selectedInstanceId']);
			this._processInstance(piid);
			return true;
        
        }
	     
	    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: dashboardURL,
          display: dashboardTitle,
          params: params
        }]);
        this.setSelectedTab(tabId); // , '/' + ppConfig.portalContextRoot + '/jsp/socialPortal/dashboard.jsp?url=' + escape(dashboardURL) + '&title=' + dashboardTitle);

        return false;
      },
      
      /**
       * Opens the Process Performance Dashboard
       * 
       * @param event (unused, but passed via <b>onclick</b>)
       */
      showProcessDashboard: function showProcessDashboard_PageHandler(event, params){
		var hash = '/dashboards/TWP/Process+Performance';
		var first = true;
		for (var p in params) {
			if (first) {
				hash = hash + '?' + p + '=' + params[p];
				first = false;
			} else {
				hash = hash + '&' + p + '=' + params[p];
			}
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[hash]);
      },

      /**
       * Opens the Team Performance Dashboard
       * 
       * @param event (unused, but passed via <b>onclick</b>)
       */
  	  showTeamDashboard: function showTeamDashboard_PageHandler(event){
		var hash = '/dashboards/TWP/Team+Performance';
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[hash]);
      },

      /**
       * handle a click on the portals primary tabs (Work/Dashboards)
       * 
       * @param id the id of the tab to switch to
       * @return {Boolean} false if previous and next page were the same, true otherwise
       */
      setSelectedTab: function setSelectedTab_PageHandler(id){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // Remove previously selected tab
        dojo.query(".processPortalTab").removeClass("processPortalSelectedTab");
		var node = dojo.byId(id);
		if (node) {
			// Select the new tab
			dojo.addClass(id, "processPortalSelectedTab");
		}
		var previousPage = dojo.query(".processPortalSelectedPage")[0];
		dojo.removeClass(previousPage, "processPortalSelectedPage");
		dojo.addClass(previousPage, "processPortalNonSelectedPage");
		targetPage=null;
		if (id=="processPortalWorkTab") {
			targetPage = dojo.byId("workPageDiv");
		} else {
			targetPage = dojo.byId(dojo.byId(id).getAttribute('data-page'));
		}
		dojo.removeClass(targetPage, "processPortalNonSelectedPage");
		dojo.addClass(targetPage, "processPortalSelectedPage");
		var realRefresh = (previousPage.id != targetPage.id);
		// Publish page switch event
		if (realRefresh) {
		  var target = targetPage.id.replace("Div", "");
		  var previous = previousPage.id.replace("Div", "");
		  dojo.publish("com.ibm.bpm.socialPortal.PageTabSwitchEvent", [{
			showing: target,
			hidding: previous
		  }]);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2, "real refresh: " + realRefresh);
        // indicate that a real page switch was done 
        return (realRefresh);
      }
    });


////////////////////////////////////////
// GLOBALS
////////////////////////////////////////


com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME = "Portal Default Page";

/**
 * Singleton object representing the currently stored startPage and providing the icon
 */
com.ibm.bpm.social.portal.PageHandler.getInstance = function getInstance_PageHandler(){
  com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler", "getInstance");
  var handler;
  if (com.ibm.bpm.social.portal.PageHandler._instance) {
    handler = com.ibm.bpm.social.portal.PageHandler._instance;
  }
  else {
    handler = new com.ibm.bpm.social.portal.PageHandler();
    handler.readFromUserProfile();
    com.ibm.bpm.social.portal.PageHandler._instance = handler;
  }
  com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler", "getInstance");
  return handler;
};

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");






dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: [const] String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		value: "focusNode",
		id: "focusNode",
		tabIndex: "focusNode",
		alt: "focusNode",
		title: "focusNode"
	}),

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "_onMouseDown");
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this._set("disabled", value);
		dojo.attr(this.focusNode, 'disabled', value);
		if(this.valueNode){
			dojo.attr(this.valueNode, 'disabled', value);
		}
		dijit.setWaiState(this.focusNode, "disabled", value);

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._set("hovering", false);
			this._set("active", false);

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex : "focusNode";
			dojo.forEach(dojo.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(dojo.isWebKit || dijit.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');
				}
			}, this);
		}else{
			if(this.tabIndex != ""){
				this.focusNode.setAttribute('tabIndex', this.tabIndex);
			}
		}
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.  Use set('disabled', ...) instead.
		dojo.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	_onFocus: function(e){
		if(this.scrollOnFocus){
			dojo.window.scrollIntoView(this.domNode);
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.  Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled){
			dijit.focus(this.focusNode);
		}
	},

	compare: function(/*anything*/ val1, /*anything*/ val2){
		// summary:
		//		Compare 2 values (as returned by get('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(newValue){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		this._pendingOnChange = this._pendingOnChange
			|| (typeof newValue != typeof this._lastValueReported)
			|| (this.compare(newValue, this._lastValueReported) != 0);
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
			this._lastValueReported = newValue;
			this._pendingOnChange = false;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					clearTimeout(this._onChangeHandle);
				}
				// setTimout allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = setTimeout(dojo.hitch(this,
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}), 0); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			clearTimeout(this._onChangeHandle);
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.  Use set('value', ...) instead.
		dojo.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.  Use get('value') instead.
		dojo.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},
	
	_onMouseDown: function(e){
		// If user clicks on the button, even if the mouse is released outside of it,
		// this button should get focus (to mimics native browser buttons).
		// This is also needed on chrome because otherwise buttons won't get focus at all,
		// which leads to bizarre focus restore on Dialog close etc.
		if(!e.ctrlKey && dojo.mouseButtons.isLeft(e) && this.isFocusable()){ // !e.ctrlKey to ignore right-click on mac
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
				if (this.isFocusable()) {
					this.focus();
				}
				this.disconnect(mouseUpConnector);
			});
		}
	}
});

dojo.declare("dijit.form._FormValueWidget", dijit.form._FormWidget,
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.  See #8484, #8660.
	// TODO: unclear what that {value: ""} is for; FormWidget.attributeMap copies value to focusNode,
	// so maybe {value: ""} is so the value *doesn't* get copied to focusNode?
	// Seems like we really want value removed from attributeMap altogether
	// (although there's no easy way to do that now)

	// readOnly: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "readOnly".
	//		Similar to disabled except readOnly form values are submitted.
	readOnly: false,

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "",
		readOnly: "focusNode"
	}),

	_setReadOnlyAttr: function(/*Boolean*/ value){
		dojo.attr(this.focusNode, 'readOnly', value);
		dijit.setWaiState(this.focusNode, "readonly", value);
		this._set("readOnly", value);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){ // IE won't stop the event with keypress
			this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
		}
		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._lastValueReported = this._resetValue = this.value;
		}
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', value) works.
		// description:
		//		Sets the value of the widget.
		//		If the value has changed, then fire onChange event, unless priorityChange
		//		is specified as null (or false?)
		this._handleOnChange(newValue, priorityChange);
	},

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget has changed.  Saves the new value in this.value,
		//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
		this._set("value", newValue);
		this.inherited(arguments);
	},

	undo: function(){
		// summary:
		//		Restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time
		this._hasBeenBlurred = false;
		this._setValueAttr(this._resetValue, true);
	},

	_onKeyDown: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
			var te;
			if(dojo.isIE){
				e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
				te = document.createEventObject();
				te.keyCode = dojo.keys.ESCAPE;
				te.shiftKey = e.shiftKey;
				e.srcElement.fireEvent('onkeypress', te);
			}
		}
	},

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(dojo.isIE == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(e){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");





dojo.declare("dijit.form.Button",
	dijit.form._FormWidget,
	{
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button dojoType="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Text to display in button.
	//		If the label is hidden (showLabel=false) then and no title has
	//		been specified, then label is also set as title attribute of icon.
	label: "",

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "",

	// type: String
	//		Defines the type of button.  "button", "submit", or "reset".
	type: "button",

	baseClass: "dijitButton",

	templateString: dojo.cache("dijit.form", "templates/Button.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdojoAttachPoint=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "valueNode"
	}),

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			return false;
		}
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary:
		//		Handler when the user activates the button portion.
		if(this._onClick(e) === false){ // returning nothing is same as true
			e.preventDefault(); // needed for checkbox
		}else if(this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a nonform widget needs to be signalled
			for(var node=this.domNode; node.parentNode/*#5935*/; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					break;
				}
			}
		}else if(this.valueNode){
			this.valueNode.click();
			e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
		}
	},

	buildRendering: function(){
		this.inherited(arguments);
		dojo.setSelectable(this.focusNode, false);
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		// TODO: remove the method in 2.0, parser will do it all for me
		if(source && (!this.params || !("label" in this.params))){
			this.set('label', source.innerHTML);
		}
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			dojo.toggleClass(this.containerNode, "dijitDisplayNone", !val);
		}
		this._set("showLabel", val);
	},

	onClick: function(/*Event*/ e){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_clicked: function(/*Event*/ e){
		// summary:
		//		Internal overridable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		dojo.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this._set("label", content);
		this.containerNode.innerHTML = content;
		if(this.showLabel == false && !this.params.title){
			this.titleNode.title = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	},

	_setIconClassAttr: function(/*String*/ val){
		// Custom method so that icon node is hidden when not in use, to avoid excess padding/margin
		// appearing around it (even if it's a 0x0 sized <img> node)

		var oldVal = this.iconClass || "dijitNoIcon",
			newVal = val || "dijitNoIcon";
		dojo.replaceClass(this.iconNode, newVal, oldVal);
		this._set("iconClass", val);
	}
});


dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container, dijit._HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
	// |		<div dojotype="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	dojo.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: dojo.cache("dijit.form", "templates/DropDownButton.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\" dojoAttachPoint=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"focusNode,titleNode,_arrowWrapperNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdojoAttachPoint=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdojoAttachPoint=\"containerNode,_popupStateNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown && this.dropDownContainer){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		if(this.dropDown){
			dijit.popup.hide(this.dropDown);
		}

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
	},

	loadDropDown: function(){
		// Loads our dropdown
		var dropDown = this.dropDown;
		if(!dropDown){ return; }
		if(!this.isLoaded()){
			var handler = dojo.connect(dropDown, "onLoad", this, function(){
				dojo.disconnect(handler);
				this.openDropDown();
			});
			dropDown.refresh();
		}else{
			this.openDropDown();
		}
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button dojoType="dijit.form.ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div dojoType="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: dojo.cache("dijit.form", "templates/ComboButton.html", "<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' role=\"presentation\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" dojoAttachPoint=\"buttonNode\" dojoAttachEvent=\"ondijitclick:_onButtonClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\" role=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdojoAttachPoint=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdojoAttachEvent=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\trole=\"button\" aria-haspopup=\"true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" dojoAttachPoint=\"valueNode\"\n\t\t/></td></tr></tbody\n></table>\n"),

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		id: "",
		tabIndex: ["focusNode", "titleNode"],
		title: "titleNode"
	}),

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			dijit.focus(this._popupStateNode);
			dojo.stopEvent(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			dijit.focus(this.titleNode);
			dojo.stopEvent(evt);
		}
	},
	
	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			dijit.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary:
	//		A button that can be in two states (checked or not).
	//		Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		checked:"focusNode"
	}),

	_clicked: function(/*Event*/ evt){
		this.set('checked', !this.checked);
	},

	_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
		this._set("checked", value);
		dojo.attr(this.focusNode || this.domNode, "checked", value);
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", value);
		this._handleOnChange(value, priorityChange);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary:
		//		Deprecated.  Use set('checked', true/false) instead.
		dojo.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
		this.set('checked', checked);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

}

if(!dojo._hasResource["dijit.form._FormMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormMixin"] = true;
dojo.provide("dijit.form._FormMixin");



dojo.declare("dijit.form._FormMixin", null, {
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)
	// description:
	//		Can extract all the form widgets
	//		values and combine them into a single javascript object, or alternately
	//		take such an object and set the values for all the contained
	//		form widgets

/*=====
	// value: Object
	//		Name/value hash for each child widget with a name and value.
	//		Child widgets without names are not part of the hash.
	//
	//		If there are multiple child widgets w/the same name, value is an array,
	//		unless they are radio buttons in which case value is a scalar (since only
	//		one radio button can be checked at a time).
	//
	//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
	//
	//		Example:
	//	|	{ name: "John Smith", interests: ["sports", "movies"] }
=====*/

	// state: [readonly] String
	//		Will be "Error" if one or more of the child widgets has an invalid value,
	//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
	//		which indicates that the form is ready to be submitted.
	state: "",

	//	TODO:
	//	* Repeater
	//	* better handling for arrays.  Often form elements have names with [] like
	//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
	//
	//

		reset: function(){
			dojo.forEach(this.getDescendants(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features.
			//		1 - it will highlight any sub-widgets that are not
			//			valid
			//		2 - it will call focus() on the first invalid
			//			sub-widget
			var didFocus = false;
			return dojo.every(dojo.map(this.getDescendants(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					dojo.window.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			dojo.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.set('value', dojo.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			dojo.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
			
			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			dojo.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the setTimout(..., 0) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			dojo.forEach(this.getDescendants(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							dojo.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = dojo.getObject(name, false, obj);
							if(value === undefined){
								dojo.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=dojo.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(dojo.isArray(prev)){
							prev.push(value);
						}else{
							dojo.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						dojo.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					} else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					} else{
						// can not set value when there is no name
					}
				} else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(isValid){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = dojo.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return dojo.indexOf(states, "Error") >= 0 ? "Error" :
				dojo.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Remove connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			dojo.forEach(this._childConnections || [], dojo.hitch(this, "disconnect"));
			dojo.forEach(this._childWatches || [], function(w){ w.unwatch(); });
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		Setup connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			//
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			var _this = this;

			// Remove old connections, if any
			this.disconnectChildren();

			this._descendants = this.getDescendants();

			// (Re)set this.value and this.state.   Send watch() notifications but not on startup.
			var set = inStartup ? function(name, val){ _this[name] = val; } : dojo.hitch(this, "_set");
			set("value", this.get("value"));
			set("state", this._getState());

			// Monitor changes to error state and disabled state in order to update
			// Form.state
			var conns = (this._childConnections = []),
				watches = (this._childWatches = []);
			dojo.forEach(dojo.filter(this._descendants,
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget changes validity state - or
				// whenever the disabled attribute on that widget is changed.
				dojo.forEach(["state", "disabled"], function(attr){
					watches.push(widget.watch(attr, function(attr, oldVal, newVal){
						_this.set("state", _this._getState());
					}));
				});
			});

			// And monitor calls to child.onChange so we can update this.value
			var onChange = function(){
				// summary:
				//		Called when child's value or disabled state changes
				
				// Use setTimeout() to collapse value changes in multiple children into a single
				// update to my value.   Multiple updates will occur on:
				//	1. Form.set()
				//	2. Form.reset()
				//	3. user selecting a radio button (which will de-select another radio button,
				//		 causing two onChange events)
				if(_this._onChangeDelayTimer){
					clearTimeout(_this._onChangeDelayTimer);
				}
				_this._onChangeDelayTimer = setTimeout(function(){
					delete _this._onChangeDelayTimer;
					_this._set("value", _this.get("value"));
				}, 10);
			};
			dojo.forEach(
				dojo.filter(this._descendants, function(item){ return item.onChange; } ),
				function(widget){
					// When a child widget's value changes,
					// the efficient thing to do is to just update that one attribute in this.value,
					// but that gets a little complicated when a checkbox is checked/unchecked
					// since this.value["checkboxName"] contains an array of all the checkboxes w/the same name.
					// Doing simple thing for now.
					conns.push(_this.connect(widget, "onChange", onChange));

					// Disabling/enabling a child widget should remove it's value from this.value.
					// Again, this code could be more efficient, doing simple thing for now.
					watches.push(widget.watch("disabled", onChange));
				}
			);
		},

		startup: function(){
			this.inherited(arguments);

			// Initialize value and valid/invalid state tracking.  Needs to be done in startup()
			// so that children are initialized.
			this.connectChildren(true);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.disconnectChildren();
			this.inherited(arguments);
		}

	});

}

if(!dojo._hasResource["dijit._DialogMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._DialogMixin"] = true;
dojo.provide("dijit._DialogMixin");



dojo.declare("dijit._DialogMixin", null,
	{
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		attributeMap: dijit._Widget.prototype.attributeMap,

		execute: function(/*Object*/ formContents){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = dijit._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	}
);

}

if(!dojo._hasResource["dijit.TooltipDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TooltipDialog"] = true;
dojo.provide("dijit.TooltipDialog");






dojo.declare(
		"dijit.TooltipDialog",
		[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin],
		{
			// summary:
			//		Pops up a dialog that appears like a Tooltip

			// title: String
			// 		Description of tooltip dialog (required for a11y)
			title: "",

			// doLayout: [protected] Boolean
			//		Don't change this parameter from the default value.
			//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
			//		is never a child of a layout container, nor can you specify the size of
			//		TooltipDialog in order to control the size of an inner widget.
			doLayout: false,

			// autofocus: Boolean
			// 		A Toggle to modify the default focus behavior of a Dialog, which
			// 		is to focus on the first dialog element after opening the dialog.
			//		False will disable autofocusing. Default: true
			autofocus: true,

			// baseClass: [protected] String
			//		The root className to use for the various states of this widget
			baseClass: "dijitTooltipDialog",

			// _firstFocusItem: [private] [readonly] DomNode
			//		The pointer to the first focusable node in the dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_firstFocusItem: null,

			// _lastFocusItem: [private] [readonly] DomNode
			//		The pointer to which node has focus prior to our dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_lastFocusItem: null,

			templateString: dojo.cache("dijit", "templates/TooltipDialog.html", "<div role=\"presentation\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" role=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\"></div>\n</div>\n"),

			_setTitleAttr: function(/*String*/ title){
				this.containerNode.title = title;
				this._set("title", title)
			},

			postCreate: function(){
				this.inherited(arguments);
				this.connect(this.containerNode, "onkeypress", "_onKey");
			},

			orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
				// summary:
				//		Configure widget to be displayed in given position relative to the button.
				//		This is called from the dijit.popup code, and should not be called
				//		directly.
				// tags:
				//		protected
				var newC = "dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right")
						+ " dijitTooltip"
						+ (corner.charAt(0) == 'T' ? "Below" : "Above");
				
				dojo.replaceClass(this.domNode, newC, this._currentOrientClass || "");
				this._currentOrientClass = newC;
			},

			focus: function(){
				// summary:
				//		Focus on first field
				this._getFocusItems(this.containerNode);
				dijit.focus(this._firstFocusItem);
			},

			onOpen: function(/*Object*/ pos){
				// summary:
				//		Called when dialog is displayed.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected

				this.orient(this.domNode,pos.aroundCorner, pos.corner);
				this._onShow(); // lazy load trigger
			},

			onClose: function(){
				// summary:
				//		Called when dialog is hidden.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected
				this.onHide();
			},

			_onKey: function(/*Event*/ evt){
				// summary:
				//		Handler for keyboard events
				// description:
				//		Keep keyboard focus in dialog; close dialog on escape key
				// tags:
				//		private

				var node = evt.target;
				var dk = dojo.keys;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.containerNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				if(evt.charOrCode == dk.ESCAPE){
					// Use setTimeout to avoid crash on IE, see #10396.
					setTimeout(dojo.hitch(this, "onCancel"), 0);
					dojo.stopEvent(evt);
				}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else if(evt.charOrCode === dk.TAB){
					// we want the browser's default tab handling to move focus
					// but we don't want the tab to propagate upwards
					evt.stopPropagation();
				}
			}
		}
	);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.Avatar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.Avatar"] = true;
// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.ui.Avatar");










dojo.declare("com.ibm.bpm.social.util.ui.Avatar", [dijit._Widget,
                                                   dijit._Templated], {

  templateString: "",
  templateString:"<div class=\"avatarContainer\">\n\t<img dojoAttachPoint=\"img\"></img>\n\t<div class=\"onlineAwarenessIcon\" dojoAttachPoint=\"onlineIcon\"><div class=\"onlineAwarenessIconA11y\">&#9744</div></div>\n</div>\n",

  _onlineTracking: null,
  _userName: null,
  _useBizCard: false,

  /**
   * Initialize the object. Set the user object.
   * 
   * @param {String} userName : username of the user object
   */
  constructor: function constructor_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  postCreate: function postCreate_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //localization
    
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
       
    // Blank image must be set to support background image on the <img> tag
    this.img.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    
    // Accessibility   
    dijit.setWaiRole(this.domNode, "img");
    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the user name.
   * 
   * @param {String} userName : username of the user object
   * @param {String} cssClass: css class for the image, as there are different sizes for example
   *        "bpm-social-default-avatar-medium"
   * @param {Boolean} useBizCard: not all avatars might use the Business Card functionality
   */
  setUser: function setUser_Avatar(user, cssClass, useBizCard){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
   
    dojo.addClass(this.img, cssClass);
    this._user = user;
    this._useBizCard = useBizCard;
    this._cssClass = cssClass;

    this._setUser();   

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  setCSSClass: function setCSSClass_Avatar(CSSClass){
    dojo.addClass(this.img, CSSClass);
  },


  /**
   * Remove the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  unsetCSSClass: function unsetCSSClass_Avatar(CSSClass){
    dojo.removeClass(this.img, CSSClass);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  _setUser: function _setUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // set User
    if (this._user !== null) {
      this.img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this._user["userName"]);
      this.img.alt = this._user["fullName"];
      
      // Accessibility         
      dijit.setWaiState(this.domNode, "label", this._user["fullName"]);

      //
      // Support for Online Awareness
      //      

      if (typeof OnlineTracking === "function") {
        this._deregisterUser();
        this._onlineTrackingConnects = [];
        var myself = this;
        this._onlineTracking = new OnlineTracking(this._user["userName"]);
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOnline", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.addClass(myself.onlineIcon, "online");
          myself.onlineIcon.title=myself._nlsResources["onlinetrackingTooltip"];    
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"] + ". " + myself._nlsResources["onlinetrackingTooltip"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOffline", function(){
          dojo.removeClass(myself.onlineIcon, "online");
          dojo.addClass(myself.onlineIcon, "offline");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onConnectionBroken", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.removeClass(myself.onlineIcon, "online");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        dojox.cometd.batch(function(){
          // Subscribe to appropriate channels
          myself._onlineTracking.activate();
        });

      }
      if (this._useBizCard) { 
    	com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.img, this._user);
    	
    	// Accessibility    	
    	dijit.setWaiState(this.img, "labelledby", this.id);  		
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy all connect from online tracking and also the online tracking object. After that a new
   * 
   * @returns {void} nothing
   */
  _deregisterUser: function _deregisterUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (this._onlineTrackingConnects) {
      for ( var i = 0; i < this._onlineTrackingConnects.length; i++) {
        dojo.disconnect(this._onlineTrackingConnects[i]);
      }
      this._onlineTrackingConnects = null;
    }
    if (this._onlineTracking != null) {
      this._onlineTracking.unregister();
      this._onlineTracking = null;
    }
    if (this._useBizCard) {
      com.ibm.bpm.social.widgets.common.BizCard.unwrapImage(this.img);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy class object
   * 
   * @returns {void} nothing
   */
  destroy: function destroy_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._deregisterUser();
    this.inherited(arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip");









dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-experts-expert-tooltip' role=\"dialog\" aria-describedby=\"activityOwner_tooltip_content\">\n    <div dojoAttachPoint='avatarContent'> </div>\t\n\t<div dojoAttachPoint=\"focusDiv\"></div>\n\t<div id=\"activityOwner_tooltip_content\" dojoAttachPoint=\"content\">\n\t\t<div class='displayName' dojoAttachPoint=\"displayNameField\"></div>\n\t\t<br>\n\t\t<div class='title' dojoAttachPoint=\"titleField\"></div>\n\t\t<div class='phoneNumber' dojoAttachPoint=\"phoneField\"></div>\n\t\t<div class='taskEmailAddress' dojoAttachPoint=\"emailField\"></div>\n\t\t<br>\n\t\t<div class='taskDetails' dojoAttachPoint=\"taskDetailsField\"></div>\n\t\t<div class='chat' dojoAttachPoint=\"chatStringField\"></div>\n\t</div>\n</div>\n",
	tkiid: null,
	livenames: null,	
	popup: null,
	userName: null,
	firstName: null,
	displayName: null,
	avgHours: 0,
	completed: 0,
	title: null,
	taskEmailAddress: null,
	phoneNumber: null,
	taskDetails: null,
	chatString: null,
	includeTaskDetails: true,
	tabInd: 0,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.livenames = {};
		
		if(this.displayName){
			this.displayNameField.innerHTML = this.displayName;
			this.firstName = this.displayName.split(" ")[0];
			//configure BTD on the displayNameField
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.displayNameField);
		}
		
		if(this.title) {
			this.titleField.innerHTML = this.title;
			//configure BTD on the title field
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.titleField);
		}
		
		if(this.phoneNumber) {
			this.phoneField.innerHTML = this.phoneNumber;
		}
		
		if(this.taskEmailAddress) {
			this.emailField.innerHTML = '<a href="mailto:'+this.taskEmailAddress+'">'+this.taskEmailAddress+'</a>';
		}
		
		if(this.includeTaskDetails){
			if(this.completed > 0){
				var data = {
						'userName': this.firstName,
						'average': this.avgHours,
						'completed': this.completed
				};
				this.taskDetailsField.innerHTML = dojo.replace(this.messages.expertsTooltip, data, /\$\{([^\}]+)\}/g);
			} else {
				this.taskDetailsField.innerHTML = this.messages.expertNoTaskData.replace("${user}", this.firstName);
			}
		}
				
		this._updateAvatar();
    	this._updateSametime();
	},
	
	_updateAvatar: function(){
		this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		this.avatar.setUser(user, "bpm-social-default-avatar-small", false);		
	},
	
	_updateSametime: function(){
		if(window.sametime){
			this.displayNameField.innerHTML = "";

			this.livenames[this.userName] = window.sametime.LiveName({'userId': this.userName, 'disableClicks': true, 'disableHoverBizCard': true},dojo.create("span", {}, this.displayNameField));
			this.livenames[this.userName].activate();
			
            this._createChatLinkSentence();            
			
			this._updateStatus();
			this.refresh = setInterval(dojo.hitch(this, this._updateStatus), 2000);
		}
	},

    _createChatLinkSentence: function() {
         //"${chat} ${with} ${user}"
         var s = this.messages.expertsChatTemplate.indexOf("${");
         var e = this.messages.expertsChatTemplate.indexOf("}");
         param = this.messages.expertsChatTemplate.substring(s+2,e);
         this._createChatLinkPart(param);
         for (var i=0; i<2; i++) {         
             s = this.messages.expertsChatTemplate.indexOf("${",s+1);
             e = this.messages.expertsChatTemplate.indexOf("}",e+1);
             param = this.messages.expertsChatTemplate.substring(s+2,e);
             this._createChatLinkPart(param);
         }         
    },

    _createChatLinkPart: function(param) {
         if (param == "chat") {
             this._createChatLink();
         } else if (param == "with") {
             this._createChatWith();
         } else if (param == "user") {
             this._createChatUser();
         } else {
             console.log("_createChatLinkPart WARNING invalid parameter: "+param);
         }
    },

    _createChatLink: function() {
			var link = dojo.create("a", {
				'href': 'javascript:void(0)',
				'username': this.userName,
				'class': 'chatLink',
             'innerHTML': this.messages.expertsChat + "&nbsp;"
			}, this.chatStringField);
			dojo.connect(link, 'onclick', this, this._handleChat);
    },

    _createChatWith: function() {
         dojo.create("span", {
             'innerHTML': this.messages.expertsChatCoda + "&nbsp;"
			}, this.chatStringField);

    },

    _createChatUser: function() {
         dojo.create("span", {
             'innerHTML': this.displayName + "&nbsp;"
         }, this.chatStringField);
	},
	
	_updateStatus: function(){
		dojo.removeClass(this.domNode, "online");
		dojo.removeClass(this.domNode, "offline");
		
		if(this.livenames && this.livenames[this.userName]){
			switch(this.livenames[this.userName].currentTextStyle){
				case "stproxy_liveNameAvailableText":
				case "stproxy_liveNameAwayText":
				case "stproxy_liveNameInMeetingText":
					dojo.addClass(this.domNode, "online");
					this.presenceStatus = "online";
					break;
				case "stproxy_liveNameOfflineText":
				case "stproxy_liveNameDndText":
				default:
					dojo.addClass(this.domNode, "offline");
					this.presenceStatus = "offline";
					break;
			}
		} else {
			dojo.style(this.chatStringField, "display", "none");
		}
		
	},
	
	_handleChat: function(e){
		var uname = e.target.getAttribute('username');
		//Following code was added for defect 112076 where the chat link would not work in IE9 or 10. In IE9 and IE10, 
		//we were getting a not logged in error, so added code to check the login status and login if we were not.
		if (dojo.isIE && !window.stproxy.isLoggedIn) {
			var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
			window.stproxy.login.loginByToken(
				user.userName,
				window.stproxy.awareness.AVAILABLE,
				"",
				dojo.hitch(this, this.loginOK), 
				dojo.hitch(this, this.loginFailed)
			);
		}
		if(this.livenames[uname]){
			this.livenames[uname].startChat();
		}
	},
	
	
	loginOK: function(){
		//console.debug("Login succeeded...");
	},
		
	loginFailed: function(){
		console.error("Login failed...");
	},

	
	//Configure fix for accessibility defect 88867: Need to manually manage tab index and key events
	//Will Return a boolean determining whether the dialog should be closed.
	handleKeyEvent: function(evt) {
		console.log("gettingcalled");
		var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
		if(keypressed == dojo.keys.TAB) {
			console.log("tabkey!");
			if(evt.shiftKey) {
				console.log("decrement");
				this.tabInd--;
			} else {
				console.log("increment");
				this.tabInd++;
			}
			console.log("tabind = " + this.tabInd);
		}
		if (this.tabInd == -1 ||
			this.tabInd == 2 ||
			(this.tabInd == 1 && this.emailField.innerHTML == "")) {
			//close the dialog, because tab order is outside the dialog
			console.log("close");
			return true;
		}
		//otherwise let the tab order do its natural thing
		return false;
	},
				
	destroy: function() {
		if(this.refresh){
			clearInterval(this.refresh);
		}
		
		if (this.avatar) {
		   this.avatar.destroy();
		   this.avatar = null;
		}
		
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.common.BizCard"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.common.BizCard"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.common.BizCard");







com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;

/**
 * Use the method to determine whether or not default actions should be used.
 * The purpose of this method is to abstract the different possible integrations
 * from widget-level code and centralize it in one place.
 */
com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard = function(){
	
	//TODO Possibly allow for parameters to check for only specific integrations...
	
	return typeof(connectionsBridge)=="undefined";
	
}

com.ibm.bpm.social.widgets.common.BizCard._connections = {};

com.ibm.bpm.social.widgets.common.BizCard.wrapElement = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapElement entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections(node, user);
		} 
		if(success) return;
	}
	
	node.innerHTML = "";
	
	dojo.addClass(node, "vcard");
	
	var nameNode = dojo.create("a", null, node);
	nameNode.href = "javascript:void(0);";
	nameNode.innerHTML = (typeof(user["fullName"])!="undefined") ? user["fullName"] : user["userName"];
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.wrapImage = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapImage entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections(node, user);
		}
		if(success) return;
	}
	
	var container = dojo.create("span",{"class": "bpm-bizcard-container"}, node, "before");
	
	var nameNode = dojo.create("a", {"class": "bpm-bizcard-anchor"}, container);
	nameNode.href = "javascript:void(0);";
	
	dojo.place(nameNode, container, "only");
	
	dojo.place(node, nameNode, "only");
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}		
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.unwrapImage = function(node) {
	if(typeof(node)=="undefined" || node == null){
		return;
	}
	
	var connectId = node.getAttribute("connectId");
	if(typeof(connectId) != "undefined" && connectId != null) {
		node.setAttribute("connectId", null);
		dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
	}
	
	if(node.parentNode != null && typeof(node.parentNode) != "undefined" && dojo.hasClass(node.parentNode, "bpm-bizcard-anchor")) {
		var bizCard = node.parentNode.parentNode;
		if(bizCard != null && typeof(bizCard) != "undefined" && dojo.hasClass(bizCard, "bpm-bizcard-container vcard")) {
			if(bizCard.parentNode != null && typeof(bizCard.parentNode) != "undefined") {
				dojo.place(node, bizCard.parentNode);
				dojo.destroy(bizCard);
			}
		}
	}
};

com.ibm.bpm.social.widgets.common.BizCard._hideTooltip = function(){
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
};
/** Listeners to close the default business card **/
dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM,com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
/** End listeners to close the default business card **/

com.ibm.bpm.social.widgets.common.BizCard._showTooltip = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_showTooltip entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
	
	var data = {
			'fullName': user['fullName'],
			'userName': user['userName'], 
			'displayName': user['fullName'],
			'title': typeof(user['userPreferences']['Title'])!='undefined' ? user['userPreferences']['Title'] : '',
			'taskEmailAddress': typeof(user['userPreferences']['Task Email Address'])!='undefined' ? user['userPreferences']['Task Email Address'] : '',
			'phoneNumber': typeof(user['userPreferences']['Phone Number'])!='undefined' ? user['userPreferences']['Phone Number'] : '',
			'includeTaskDetails': false
	};
	
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
	
	com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = new dijit.TooltipDialog({
        content: com.ibm.bpm.social.widgets.common.BizCard._activityOwner.domNode,
        onMouseLeave: function(){
			dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        },
		onMouseEnter: function() {
			//If we get this event it means we definitely arent using an IPAD, so I can
			//cancel the timer going
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
        },
		//handled by bizCardConnect (defect 124992)
        //onClick: function(){
		//    console.log("in onclick");
        //	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        //},
		onKeyDown: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				node.focus();
			}
		},
		onKeyUp: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			//don't start the close timer just because of a key up (defect 124992)
			//com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();
			if(dojo.isIE || com.ibm.bpm.social.util.common.Util.isChrome) {
				var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
				if(keypressed == dojo.keys.TAB) {
					if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
						dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
						node.focus();
					}
				}
			}
		}
    });
	
	//Need to close when user clicks outside of dialog, since we close when they click on the dialog
	//might as well go all the way with this.  Should clear up most issues with IPAD
	var _bizCardConnect = dojo.connect(window, "onmousedown", function(event) {
		try{
		    //only close the window when user clicks outside of the bizcard popup
			//Also use event.target and not event.originalTarget,which
			//is only supported by Firefox.
			//(defect 124992)
			if (dojo.isDescendant(event.target, com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.domNode) == false) {
			//if(event.originalTarget.getAttribute("userName")==null){
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				dojo.disconnect(_bizCardConnect);
			}
		} catch (e) {
			
		}
	});
	com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();

	/** Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var _stack = [];
	while(dijit.popup._stack.length > 0){
		_stack.push(dijit.popup._stack.pop());
	}
	/** End Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	
	dijit.popup.open({
		popup: com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog,
		around: dojo.byId(node)
	});
	
	//Configure fix for accessibility defect 88867
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.setAttribute("tabindex", 0);
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.focus();
	
	
	node.setAttribute("tabindex", 0);
	/** Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var tmp = dijit.popup._stack.pop();
	while(_stack.length > 0){
		dijit.popup._stack.push(_stack.pop());
	}
	dijit.popup._stack.push(tmp);
	/** End Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/

};

/**
 * These timers are primarily used for the ipad, however it also affects desktop browser
 * behavior.  The issue is that if a user opens a dialog, but never enters it, then the dialog
 * just sticks around forever.  This will close the dialog after a certain amount of time
 * if the user doesn't "enter" the dialog
 */
com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer = function() {
	com.ibm.bpm.social.widgets.common.BizCard._closeTimer = 
		window.setTimeout(com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer, 3000);
};	

com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer = function() {
	if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
		window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
	}
	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
};
/**
 * Product-specific integrations below
 * TODO: This could be refactored to allow for dynamic evaluation of available integrations
 */

/**
 * IBM Connections
 */

com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapElement_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["fullName"])!="undefined" 
			&& typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
				&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapElement(node, user["fullName"], user["emailAddress"]);
		}
	}
	return success;
};

com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapImage_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
			&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapImage(node, user["emailAddress"]);
		}
	}
	return success;
};



}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.details.TaskDetailsListItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.details.TaskDetailsListItem"] = true;
/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.details.TaskDetailsListItem");





















dojo.declare("com.ibm.bpm.social.widgets.details.TaskDetailsListItem", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                                        dijit._Templated], {

  templateString: "",
  templateString:"<div class=\"taskDetailsListItem\" role=\"complementary\" aria-label=\"${taskDisplayName}\" tabindex=\"0\">\n<div class=\"taskIconWrapper\">\n\t<img class=\"taskIconImg bpm-social-unfinished-task-avatar\" dojoAttachPoint=\"taskIconImg\" alt=\"\"/>\n</div>\n<div class=\"taskDetailsContent\" dojoAttachPoint=\"taskDetailsContent\">\n\t<div class=\"titleRow\">\n\t\t<a class=\"taskName\" dojoAttachPoint=\"taskNameField\" dojoAttachEvent=\"onclick:_onTaskSubjectClicked,onkeypress:_onTaskSubjectKeyPress\" tabindex=\"0\"></a>\n\t\t<div class=\"taskOwner\" dojoAttachPoint=\"taskOwnerField_1\" id=\"taskOwner_${taskId}\"></div>\n\t    <div dojoAttachPoint=\"avatarContainer\" ></div>\t\t\n\t\t<div class=\"taskOwner taskOwnerSidebar\" dojoAttachPoint=\"taskOwnerField_2\"></div>\t\t\n\t</div>\n\t<div class=\"dateRow\">\n\t\t<div class=\"taskStarted\" dojoAttachPoint=\"taskStartedField\" id=\"taskStarted_${taskId}\"></div>\n\t\t<div class=\"taskCompleted\" dojoAttachPoint=\"taskCompletedField\" id=\"taskCompleted_${taskId}\"></div>\n\t\t<div class=\"taskDue\" dojoAttachPoint=\"taskDueField\" id=\"taskDue_${taskId}\"></div>\n\t</div>\n</div>\n</div>\n",

  taskId: null,
  taskName: null,
  taskAssignee: null,
  taskAssigneeDisplayName: null,
  taskOwner: null,
  taskOwnerDisplayName: null,
  taskStarted: null,
  taskCompleted: null,
  taskDue: null,
  taskStatus: null,
  taskPriority: null,

  sidebar: false,

  _userProfileChangeSubscribeHandle: null,
  _taskDetailsChangeHandle: null,
  _onlineTracking: null,

  postCreate: function(){
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    this.taskIconImg.src = dojo.moduleUrl("dojo", "resources/blank.gif");

    this._applyTaskData();

    this._registerListeners();
  },

  _applyTaskData: function(){

    if (this.taskStatus != null) {
      dojo.addClass(this.domNode, this.taskStatus.toLowerCase());

      if (this.taskStatus.toLowerCase() == "closed") {
        dojo.addClass(this.taskIconImg, "bpm-social-task-complete-avatar");
        dojo.removeClass(this.taskIconImg, "bpm-social-unfinished-task-avatar");
      }

      this.taskIconImg.setAttribute('title', this.taskStatus);
      this.taskIconImg.setAttribute('alt', this.taskStatus);
    }

    this.taskNameField.innerHTML = this.taskDisplayName;
    this.taskNameField.title = this.taskDisplayName;

    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskNameField);

    this.updateTaskParticipants();

    var str, d, ds, dt;
    if (this.taskStarted != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskStarted);
      this.taskStartedField.innerHTML = this.messages.created.replace("${date}", dateStr);
    }

    if (this.taskCompleted != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskCompleted);
      this.taskCompletedField.innerHTML = this.messages.completed.replace("${date}", dateStr);
      dojo.style(this.taskDueField, "display", "none");
      dojo.style(this.taskCompletedField, "display", "block");
    }
    else if (this.taskDue != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskDue);
      this.taskDueField.innerHTML = this.messages.due.replace("${date}", dateStr);
      dojo.style(this.taskCompletedField, "display", "none");
      dojo.style(this.taskDueField, "display", "block");
    }

  },

  updateTaskParticipants: function(){

    var user = null;

    if (this.taskOwner != null) {

      user = com.ibm.bpm.social.util.common.Util.getUser(this.taskOwner);
      this.taskOwnerDisplayName = user.fullName;

      this.taskOwnerField_1.innerHTML = this.taskOwnerDisplayName;
      this.taskOwnerField_2.innerHTML = this.taskOwnerDisplayName;
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskOwnerField_1);
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskOwnerField_2);
    }
    else if (this.taskAssignee != null) {

      user = com.ibm.bpm.social.util.common.Util.getUser(this.taskAssignee);

      this.taskOwnerField_1.innerHTML = this.taskAssigneeDisplayName;
      this.taskOwnerField_2.innerHTML = this.taskAssigneeDisplayName;

   }

    if (user != null && typeof (user["userID"]) != "undefined") {
      /** Business Card integration * */
      com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.taskOwnerField_1, user);
      com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.taskOwnerField_2, user);
      this._avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContainer);
      this._avatar.setUser(user, " bpm-social-default-avatar-small", true);
      /** End Business Card integration * */
    }

  },

  getOnlineTracking: function(){
    if (this._onlineTracking != null) {
      return this._onlineTracking;
    }
  },

  _onTaskSubjectClicked: function(e){

    this._checkCollaborationSessionAvailablity(this.taskId);

    /*dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
    	"taskId": this.taskId,
    	"params": []
    }]);
    */
  },

  _onTaskSubjectKeyPress: function(e){
    if (e.keyCode == dojo.keys.ENTER || e.charOrCode == " ") {
      this._onTaskSubjectClicked(e);
    }
  },

  /* _checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
  _checkCollaborationSessionAvailablity: function(taskID){
    // Response will look something like this:
    // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
    if (taskID) {
      /* /ProcessPortal/notification/roster/task/ */
      dojo.xhrGet({
        url: this.getProxyUrl() +window.endpointURLObject.processPortalNotificationJS +"/roster/task/" + taskID,
        handleAs: "json",
        preventCache: true,
        sync: true,
        headers: {
          "Content-Type": "application/json",
          "Accept": "application/json"
        },
        load: dojo.hitch(this, function(data, xhrArgs){
          this._checkNumberOfParticipants(data, xhrArgs, taskID);
        }),
        error: dojo.hitch(this, this._onXhrErrors)
      });
    }
  },


  _onXhrErrors: function(data, xhrArgs){

    com.ibm.bpm.social.util.common.Log.logEntry(1, "Error communicating with the server.");
    //console.debug([data, xhrArgs]);
  },

  /* _checkNumberOfParticipants: See if there aren't too many participants in this task already. */
  _checkNumberOfParticipants: function(data, xhrArgs, taskID){
    // data will look something like this:
    // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
    // or this:
    // {"message":"NO_SESSION_FOR_TASK"}

    if (!data) {
      return;
    }
    // If there's no collab session, then data won't contain an element called 'participants',
    // and in that case we go ahead and publish the show task topic.
    var doPublishShowTaskCompletionTopic = false;
    if (data.participants) {
      var participants = data.participants;
      var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
      var currentUser = null;
      if (userProfileObj) {
        currentUser = userProfileObj.userName;
      }
      if (participants && (participants.length < 2 || this._currentUserIsAlreadyInSession(currentUser, participants))) {
        // publish the show task topic
        doPublishShowTaskCompletionTopic = true;
      }
      else {
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.messages.collaborationSessionFull,
          null /* details */, function(){
            return;
          });
      }
    }
    else {
      doPublishShowTaskCompletionTopic = true;
    }
    if (doPublishShowTaskCompletionTopic) {
      // publish the show task topic
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
        "taskId": taskID
      }]);
    }
  },

  _currentUserIsAlreadyInSession: function(currentUser, participants){
    if (currentUser && participants) {
      for ( var i in participants) {
        if (participants[i].userName == currentUser) {
          return true;
        }
      }

    }
    return false;
  },

  _registerListeners: function(){
    if (this._userProfileChangeSubscribeHandle == null) {
      this._userProfileChangeSubscribeHandle =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, dojo
          .hitch(this, this._handleUserProfileChange));
    }

    if (this._taskDetailsChangeHandle == null) {
      this._taskDetailsChangeHandle =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this.taskId, dojo.hitch(this,
          this._handleTaskDetailsChange));
    }
  },

  _handleUserProfileChange: function(obj){
    /**
     * Will only trigger off of local changes (based on subscription) by the current user, hence no check for
     * IsCurrentUser *
     */
    if (obj.username && obj.username == this.taskOwner) {
      var user = com.ibm.bpm.social.util.common.Util.getUser(obj.username);
      this._avatar.setUser(user, "bpm-social-default-avatar-medium", true);
    }
  },

  _handleTaskDetailsChange: function(obj){

    var skipCache = true;
    if (typeof (obj.skipCache) != "undefined") {
      skipCache = obj.skipCache;
    }

    var task = com.ibm.bpm.social.util.common.Util.getTaskData({
      taskId: this.taskId,
      skipCache: skipCache
    });

    this.taskName = task.name;
    this.taskDisplayName = task.displayName;
    this.taskAssignee = task.assignedToDisplayName;
    this.taskOwner = task.owner;
    this.taskStarted = task.startTime;
    this.taskCompleted = task.completionTime;
    this.taskDue = task.dueTime;
    this.taskStatus = task.status;
    this.taskPriority = task.priority;

    this._applyTaskData();
  },

  destroy: function(){
    if (this._userProfileChangeSubscribeHandle != null) {
      dojo.unsubscribe(this._userProfileChangeSubscribeHandle);
    }

    if (this._taskDetailsChangeHandle != null) {
      dojo.unsubscribe(this._taskDetailsChangeHandle);
    }

    if (this._avatar) {
      this._avatar.destroy();
      this._avatar = null;
    }
    this.inherited(arguments);
  }

});

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary:
		//		an object which makes a node follow the mouse, or touch-drag on touch devices.
		//		Used as a default mover, and as a base class for custom movers.
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		var pos = e.touches ? e.touches[0] : e;
		this.marginBox = {l: pos.pageX, t: pos.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;
		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following two
			// connects are disconnected
			dojo.connect(d, "onmousemove", this, "onFirstMove"),
			dojo.connect(d, "ontouchmove", this, "onFirstMove"),

			// These are called continually during the drag
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "ontouchmove", this, "onMouseMove"),

			// And these are called at the end of the drag
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			dojo.connect(d, "ontouchend", this, "onMouseUp"),

			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo.stopEvent),
			dojo.connect(d.body, "onselectstart", dojo.stopEvent)
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox,
			pos = e.touches ? e.touches[0] : e;
		this.host.onMove(this, {l: m.l + pos.pageX, t: m.t + pos.pageY}, e);
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		if(dojo.isWebKit && dojo.isMac && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		// 		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = dojo.marginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing dojo.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = dojo.doc.body;
				var bs = dojo.getComputedStyle(b);
				var bm = dojo._getMarginBox(b, bs);
				var bc = dojo._getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}
		
		// Disconnect onmousemove and ontouchmove events that call this function
		dojo.disconnect(this.events.shift());
		dojo.disconnect(this.events.shift());
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



/*=====
dojo.declare("dojo.dnd.__MoveableArgs", [], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dojo.dnd.Mover
});
=====*/

dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.__MoveableArgs?
		//		optional parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			dojo.connect(this.handle, "ontouchstart", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				dojo.connect(this.handle, "onmousemove", this, "onMouseMove"),
				dojo.connect(this.handle, "ontouchmove", this, "onMouseMove"),
				dojo.connect(this.handle, "onmouseup", this, "onMouseUp"),
				dojo.connect(this.handle, "ontouchend", this, "onMouseUp")
			);
			var pos = e.touches ? e.touches[0] : e;
			this._lastX = pos.pageX;
			this._lastY = pos.pageY;
		}else{
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		var pos = e.touches ? e.touches[0] : e;
		if(Math.abs(pos.pageX - this._lastX) > this.delay || Math.abs(pos.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			dojo.disconnect(this.events.pop());
		}
		dojo.stopEvent(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onDragDetected: function(/* Event */ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove");
		dojo.addClass(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover, /* Event */ e){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop, /* Event */ e){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called before every incremental move; can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called after every incremental move; can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




/*=====
dojo.declare("dojo.dnd.move.__constrainedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = dojo._getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
dojo.declare("dojo.dnd.move.__boxConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
dojo.declare("dojo.dnd.move.__parentConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = dojo.getComputedStyle(n),
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dojo.dnd.TimedMoveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.TimedMoveable"] = true;
dojo.provide("dojo.dnd.TimedMoveable");



/*=====
dojo.declare("dojo.dnd.__TimedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// timeout: Number
	//		delay move by this number of ms,
	//		accumulating position changes during the timeout
	timeout: 0
});
=====*/

(function(){
	// precalculate long expressions
	var oldOnMove = dojo.dnd.Moveable.prototype.onMove;
		
	dojo.declare("dojo.dnd.TimedMoveable", dojo.dnd.Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.
		
		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps
	
		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: dojo.dnd.__TimedMoveableArgs
			//		object with additional parameters.
			
			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},
	
		// markup methods
		markupFactory: function(params, node){
			return new dojo.dnd.TimedMoveable(node, params);
		},
	
		onMoveStop: function(/* dojo.dnd.Mover */ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer)
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop)
			}
			dojo.dnd.Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
})();

}

if(!dojo._hasResource["dijit.DialogUnderlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.DialogUnderlay"] = true;
dojo.provide("dijit.DialogUnderlay");





dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		The component that blocks the screen behind a `dijit.Dialog`
		//
		// description:
		// 		A component used to block input behind a `dijit.Dialog`. Only a single
		//		instance of this widget is created by `dijit.Dialog`, and saved as
		//		a reference to be shared between all Dialogs as `dijit._underlay`
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' dojoAttachPoint='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		attributeMap: { id: "domNode" },

		_setDialogIdAttr: function(id){
			dojo.attr(this.node, "id", id + "_underlay");
			this._set("dialogId", id);
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
			this._set("class", clazz);
		},

		postCreate: function(){
			// summary:
			//		Append the underlay to the body
			dojo.body().appendChild(this.domNode);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = dojo.window.getBox();
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.layout();
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay
			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
		}
	}
);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");















// dijit/TooltipDialog required for back-compat.  TODO: remove in 2.0

/*=====
dijit._underlay = function(kwArgs){
	// summary:
	//		A shared instance of a `dijit.DialogUnderlay`
	//
	// description:
	//		A shared instance of a `dijit.DialogUnderlay` created and
	//		used by `dijit.Dialog`, though never created until some Dialog
	//		or subclass thereof is shown.
};
=====*/
dojo.declare(
	"dijit._DialogBase",
	[dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin, dijit._CssStateMixin],
	{
		// summary:
		//		A modal dialog Widget
		//
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)
		//
		// example:
		// |	<div dojoType="dijit.Dialog" href="test.html"></div>
		//
		// example:
		// |	var foo = new dijit.Dialog({ title: "test dialog", content: "test content" };
		// |	dojo.body().appendChild(foo.domNode);
		// |	foo.startup();

		templateString: dojo.cache("dijit", "templates/Dialog.html", "<div class=\"dijitDialog\" role=\"dialog\" aria-labelledby=\"${id}_title\">\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"ondijitclick: onCancel\" title=\"${buttonCancel}\" role=\"button\" tabIndex=\"-1\">\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t</span>\n\t</div>\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n"),
		
		baseClass: "dijitDialog",
		
		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			title: [
				{ node: "titleNode", type: "innerHTML" },
				{ node: "titleBar", type: "attribute" }
			],
			"aria-describedby":""
		}),

		// open: [readonly] Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: dijit.defaultDuration,

		// refocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the moveable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,

		//aria-describedby: String
		//		Allows the user to add an aria-describedby attribute onto the dialog.   The value should
		//		be the id of the container element of text that describes the dialog purpose (usually
		//		the first text in the dialog).
		//		<div dojoType="dijit.Dialog" aria-describedby="intro" .....>
		//			<div id="intro">Introductory text</div>
		//			<div>rest of dialog contents</div>
		//		</div>
		"aria-describedby":"",

		postMixInProperties: function(){
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			dojo.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			dojo.style(this.domNode, {
				display: "none",
				position:"absolute"
			});
			dojo.body().appendChild(this.domNode);

			this.inherited(arguments);

			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overridden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._position();
			if(this.autofocus && dijit._DialogLevelManager.isTop(this)){
				this._getFocusItems(this.domNode);
				dijit.focus(this._firstFocusItem);
			}
			this.inherited(arguments);
		},

		_endDrag: function(){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport
			//		and also adjust position to be fully within the viewport, so user doesn't lose access to handle
			// tags:
			//		private
			var nodePosition = dojo.position(this.domNode),
				viewport = dojo.window.getBox();
			nodePosition.y = Math.min(Math.max(nodePosition.y, 0), (viewport.h - nodePosition.h));
			nodePosition.x = Math.min(Math.max(nodePosition.x, 0), (viewport.w - nodePosition.w));
			this._relativePosition = nodePosition;
			this._position();
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = (dojo.isIE == 6) ?
					new dojo.dnd.TimedMoveable(node, { handle: this.titleBar }) :	// prevent overload, see #5285
					new dojo.dnd.Moveable(node, { handle: this.titleBar, timeout: 0 });
				this.connect(this._moveable, "onMoveStop", "_endDrag");
			}else{
				dojo.addClass(node,"dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": dojo.map(this["class"].split(/\s/), function(s){ return s+"_underlay"; }).join(" ")
			};
		},

		_size: function(){
			// summary:
			// 		If necessary, shrink dialog contents so dialog fits in viewport
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the dojo.marginBox(this.domNode) call below.
			if(this._singleChild){
				if(this._singleChildOriginalStyle){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
				}
				delete this._singleChildOriginalStyle;
			}else{
				dojo.style(this.containerNode, {
					width:"auto",
					height:"auto"
				});
			}

			var mb = dojo._getMarginSize(this.domNode);
			var viewport = dojo.window.getBox();
			if(mb.w >= viewport.w || mb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var w = Math.min(mb.w, Math.floor(viewport.w * 0.75)),
					h = Math.min(mb.h, Math.floor(viewport.h * 0.75));

				if(this._singleChild && this._singleChild.resize){
					this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					this._singleChild.resize({w: w, h: h});
				}else{
					dojo.style(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"	// workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position modal dialog in the viewport. If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the node. Otherwise, use the Dialog's stored relative offset,
			//		and position the node to top: left: values based on the viewport.
			// tags:
			//		private
			if(!dojo.hasClass(dojo.body(), "dojoMove")){	// don't do anything if called during auto-scroll
				var node = this.domNode,
					viewport = dojo.window.getBox(),
					p = this._relativePosition,
					bb = p ? null : dojo._getBorderBox(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
				;
				dojo.style(node,{
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			if(evt.charOrCode){
				var dk = dojo.keys;
				var node = evt.target;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.domNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				// see if we are shift-tabbing from first focusable item on dialog
				if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode || dojo.hasClass(node, "dijitPopup")){
							if(evt.charOrCode == dk.ESCAPE){
								this.onCancel();
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.charOrCode !== dk.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if(!dojo.isOpera){
						try{
							this._firstFocusItem.focus();
						}catch(e){ /*squelch*/ }
					}
				}
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			// returns: dojo.Deferred
			//		Deferred object that resolves when the display animation is complete

			if(this.open){ return; }

			if(!this._started){
				this.startup();
			}

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(window, "onresize", this, function(){
				// IE gives spurious resize events and can actually get stuck
				// in an infinite loop if we don't ignore them
				var viewport = dojo.window.getBox();
				if(!this._oldViewport ||
						viewport.h != this._oldViewport.h ||
						viewport.w != this._oldViewport.w){
					this.layout();
					this._oldViewport = viewport;
				}
			}));
			this._modalconnects.push(dojo.connect(this.domNode, "onkeypress", this, "_onKey"));

			dojo.style(this.domNode, {
				opacity:0,
				display:""
			});

			this._set("open", true);
			this._onShow(); // lazy load trigger

			this._size();
			this._position();

			// fade-in Animation object, setup below
			var fadeIn;

			this._fadeInDeferred = new dojo.Deferred(dojo.hitch(this, function(){
				fadeIn.stop();
				delete this._fadeInDeferred;
			}));

			fadeIn = dojo.fadeIn({
				node: this.domNode,
				duration: this.duration,
				beforeBegin: dojo.hitch(this, function(){
					dijit._DialogLevelManager.show(this, this.underlayAttrs);
				}),
				onEnd: dojo.hitch(this, function(){
					if(this.autofocus && dijit._DialogLevelManager.isTop(this)){
						// find focusable items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						dijit.focus(this._firstFocusItem);
					}
					this._fadeInDeferred.callback(true);
					delete this._fadeInDeferred;
				})
			}).play();
			
			return this._fadeInDeferred;
		},

		hide: function(){
			// summary:
			//		Hide the dialog
			// returns: dojo.Deferred
			//		Deferred object that resolves when the hide animation is complete

			// if we haven't been initialized yet then we aren't showing and we can just return
			if(!this._alreadyInitialized){
				return;
			}
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}

			// fade-in Animation object, setup below
			var fadeOut;

			this._fadeOutDeferred = new dojo.Deferred(dojo.hitch(this, function(){
				fadeOut.stop();
				delete this._fadeOutDeferred;
			}));

			fadeOut = dojo.fadeOut({
				node: this.domNode,
				duration: this.duration,
				onEnd: dojo.hitch(this, function(){
					this.domNode.style.display = "none";
					dijit._DialogLevelManager.hide(this);
					this.onHide();
					this._fadeOutDeferred.callback(true);
					delete this._fadeOutDeferred;
				})
			 }).play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this._set("open", false);

			return this._fadeOutDeferred;
		},

		layout: function(){
			// summary:
			//		Position the Dialog and the underlay
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				if(dijit._underlay){	// avoid race condition during show()
					dijit._underlay.layout();
				}
				this._position();
			}
		},

		destroy: function(){
			if(this._fadeInDeferred){
				this._fadeInDeferred.cancel();
			}
			if(this._fadeOutDeferred){
				this._fadeOutDeferred.cancel();
			}
			if(this._moveable){
				this._moveable.destroy();
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);

			dijit._DialogLevelManager.hide(this);

			this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._DialogBase],
	{}
);

dijit._DialogLevelManager = {
	// summary:
	//		Controls the various active "levels" on the page, starting with the
	//		stuff initially visible on the page (at z-index 0), and then having an entry for
	//		each Dialog shown.

	show: function(/*dijit._Widget*/ dialog, /*Object*/ underlayAttrs){
		// summary:
		//		Call right before fade-in animation for new dialog.
		//		Saves current focus, displays/adjusts underlay for new dialog,
		//		and sets the z-index of the dialog itself.
		//
		//		New dialog will be displayed on top of all currently displayed dialogs.
		//
		//		Caller is responsible for setting focus in new dialog after the fade-in
		//		animation completes.

		var ds = dijit._dialogStack;

		// Save current focus
		ds[ds.length-1].focus = dijit.getFocus(dialog);

		// Display the underlay, or if already displayed then adjust for this new dialog
		var underlay = dijit._underlay;
		if(!underlay || underlay._destroyed){
			underlay = dijit._underlay = new dijit.DialogUnderlay(underlayAttrs);
		}else{
			underlay.set(dialog.underlayAttrs);
		}

		// Set z-index a bit above previous dialog
		var zIndex = ds[ds.length-1].dialog ? ds[ds.length-1].zIndex + 2 : 950;
		if(ds.length == 1){	// first dialog
			underlay.show();
		}
		dojo.style(dijit._underlay.domNode, 'zIndex', zIndex - 1);

		// Dialog
		dojo.style(dialog.domNode, 'zIndex', zIndex);

		ds.push({dialog: dialog, underlayAttrs: underlayAttrs, zIndex: zIndex});
	},

	hide: function(/*dijit._Widget*/ dialog){
		// summary:
		//		Called when the specified dialog is hidden/destroyed, after the fade-out
		//		animation ends, in order to reset page focus, fix the underlay, etc.
		//		If the specified dialog isn't open then does nothing.
		//
		//		Caller is responsible for either setting display:none on the dialog domNode,
		//		or calling dijit.popup.hide(), or removing it from the page DOM.

		var ds = dijit._dialogStack;

		if(ds[ds.length-1].dialog == dialog){
			// Removing the top (or only) dialog in the stack, return focus
			// to previous dialog

			ds.pop();

			var pd = ds[ds.length-1];	// the new active dialog (or the base page itself)

			// Adjust underlay
			if(ds.length == 1){
				// Returning to original page.
				// Hide the underlay, unless the underlay widget has already been destroyed
				// because we are being called during page unload (when all widgets are destroyed)
				if(!dijit._underlay._destroyed){
					dijit._underlay.hide();
				}
			}else{
				// Popping back to previous dialog, adjust underlay
				dojo.style(dijit._underlay.domNode, 'zIndex', pd.zIndex - 1);
				dijit._underlay.set(pd.underlayAttrs);
			}

			// Adjust focus
			if(dialog.refocus){
				// If we are returning control to a previous dialog but for some reason
				// that dialog didn't have a focused field, set focus to first focusable item.
				// This situation could happen if two dialogs appeared at nearly the same time,
				// since a dialog doesn't set it's focus until the fade-in is finished.
				var focus = pd.focus;
				if(!focus || (pd.dialog && !dojo.isDescendant(focus.node, pd.dialog.domNode))){
					pd.dialog._getFocusItems(pd.dialog.domNode);
					focus = pd.dialog._firstFocusItem;
				}
	
				try{
					dijit.focus(focus);
				}catch(e){
					/* focus() will fail if user opened the dialog by clicking a non-focusable element */
				}
			}
		}else{
			// Removing a dialog out of order (#9944, #10705).
			// Don't need to mess with underlay or z-index or anything.
			var idx = dojo.indexOf(dojo.map(ds, function(elem){return elem.dialog}), dialog);
			if(idx != -1){
				ds.splice(idx, 1);
			}
		}
	},

	isTop: function(/*dijit._Widget*/ dialog){
		// summary:
		//		Returns true if specified Dialog is the top in the task
		var ds = dijit._dialogStack;
		return ds[ds.length-1].dialog == dialog;
	}
};

// Stack representing the various active "levels" on the page, starting with the
// stuff initially visible on the page (at z-index 0), and then having an entry for
// each Dialog shown.
// Each element in stack has form {
//		dialog: dialogWidget,
//		focus: returnFromGetFocus(),
//		underlayAttrs: attributes to set on underlay (when this widget is active)
// }
dijit._dialogStack = [
	{dialog: null, focus: null, underlayAttrs: null}	// entry for stuff at z-index: 0
];

}

if(!dojo._hasResource["dijit.form.DropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.DropDownButton"] = true;
dojo.provide("dijit.form.DropDownButton");




}

if(!dojo._hasResource["dijit.Calendar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Calendar"] = true;
dojo.provide("dijit.Calendar");









dojo.declare(
	"dijit.Calendar",
	[dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
		// summary:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//
		// description:
		//		A simple GUI for choosing a date in the context of a monthly calendar.
		//		This widget can't be used in a form because it doesn't serialize the date to an
		//		`<input>` field.  For a form element, use dijit.form.DateTextBox instead.
		//
		//		Note that the parser takes all dates attributes passed in the
		//		[RFC 3339 format](http://www.faqs.org/rfcs/rfc3339.html), e.g. `2005-06-30T08:05:00-07:00`
		//		so that they are serializable and locale-independent.
		//
		// example:
		//	|	var calendar = new dijit.Calendar({}, dojo.byId("calendarNode"));
		//
		// example:
		//	|	<div dojoType="dijit.Calendar"></div>

		templateString: dojo.cache("dijit", "templates/Calendar.html", "<table cellspacing=\"0\" cellpadding=\"0\" class=\"dijitCalendarContainer\" role=\"grid\" dojoAttachEvent=\"onkeypress: _onKeyPress\" aria-labelledby=\"${id}_year\">\n\t<thead>\n\t\t<tr class=\"dijitReset dijitCalendarMonthContainer\" valign=\"top\">\n\t\t\t<th class='dijitReset dijitCalendarArrow' dojoAttachPoint=\"decrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarDecrease\" role=\"presentation\"/>\n\t\t\t\t<span dojoAttachPoint=\"decreaseArrowNode\" class=\"dijitA11ySideArrow\">-</span>\n\t\t\t</th>\n\t\t\t<th class='dijitReset' colspan=\"5\">\n\t\t\t\t<div dojoType=\"dijit.form.DropDownButton\" dojoAttachPoint=\"monthDropDownButton\"\n\t\t\t\t\tid=\"${id}_mddb\" tabIndex=\"-1\">\n\t\t\t\t</div>\n\t\t\t</th>\n\t\t\t<th class='dijitReset dijitCalendarArrow' dojoAttachPoint=\"incrementMonth\">\n\t\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitCalendarIncrementControl dijitCalendarIncrease\" role=\"presentation\"/>\n\t\t\t\t<span dojoAttachPoint=\"increaseArrowNode\" class=\"dijitA11ySideArrow\">+</span>\n\t\t\t</th>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<th class=\"dijitReset dijitCalendarDayLabelTemplate\" role=\"columnheader\"><span class=\"dijitCalendarDayLabel\"></span></th>\n\t\t</tr>\n\t</thead>\n\t<tbody dojoAttachEvent=\"onclick: _onDayClick, onmouseover: _onDayMouseOver, onmouseout: _onDayMouseOut, onmousedown: _onDayMouseDown, onmouseup: _onDayMouseUp\" class=\"dijitReset dijitCalendarBodyContainer\">\n\t\t<tr class=\"dijitReset dijitCalendarWeekTemplate\" role=\"row\">\n\t\t\t<td class=\"dijitReset dijitCalendarDateTemplate\" role=\"gridcell\"><span class=\"dijitCalendarDateLabel\"></span></td>\n\t\t</tr>\n\t</tbody>\n\t<tfoot class=\"dijitReset dijitCalendarYearContainer\">\n\t\t<tr>\n\t\t\t<td class='dijitReset' valign=\"top\" colspan=\"7\">\n\t\t\t\t<h3 class=\"dijitCalendarYearLabel\">\n\t\t\t\t\t<span dojoAttachPoint=\"previousYearLabelNode\" class=\"dijitInline dijitCalendarPreviousYear\"></span>\n\t\t\t\t\t<span dojoAttachPoint=\"currentYearLabelNode\" class=\"dijitInline dijitCalendarSelectedYear\" id=\"${id}_year\"></span>\n\t\t\t\t\t<span dojoAttachPoint=\"nextYearLabelNode\" class=\"dijitInline dijitCalendarNextYear\"></span>\n\t\t\t\t</h3>\n\t\t\t</td>\n\t\t</tr>\n\t</tfoot>\n</table>\n"),
		widgetsInTemplate: true,

		// value: Date
		//		The currently selected Date, initially set to invalid date to indicate no selection.
		value: new Date(""),
		// TODO: for 2.0 make this a string (ISO format) rather than a Date

		// datePackage: String
		//		JavaScript namespace to find Calendar routines.  Uses Gregorian Calendar routines
		//		at dojo.date by default.
		datePackage: "dojo.date",

		// dayWidth: String
		//		How to represent the days of the week in the calendar header. See dojo.date.locale
		dayWidth: "narrow",

		// tabIndex: Integer
		//		Order fields are traversed when user hits the tab key
		tabIndex: "0",
		
		// currentFocus: Date
		//		Date object containing the currently focused date, or the date which would be focused
		//		if the calendar itself was focused.   Also indicates which year and month to display,
		//		i.e. the current "page" the calendar is on.
		currentFocus: new Date(),

		baseClass:"dijitCalendar",

		// Set node classes for various mouse events, see dijit._CssStateMixin for more details
		cssStateNodes: {
			"decrementMonth": "dijitCalendarArrow",
			"incrementMonth": "dijitCalendarArrow",
			"previousYearLabelNode": "dijitCalendarPreviousYear",
			"nextYearLabelNode": "dijitCalendarNextYear"
		},

		_isValidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking that it's a valid date, rather
			//		than blank or NaN.
			// tags:
			//		private
			return value && !isNaN(value) && typeof value == "object" &&
				value.toString() != this.constructor.prototype.value.toString();
		},

		setValue: function(/*Date*/ value){
			// summary:
			//      Deprecated.   Use set('value', ...) instead.
			// tags:
			//      deprecated
			dojo.deprecated("dijit.Calendar:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_getValueAttr: function(){
			// summary:
			//		Support get('value')

			// this.value is set to 1AM, but return midnight, local time for back-compat
			var value = new this.dateClassObj(this.value);
			value.setHours(0, 0, 0, 0);

			// If daylight savings pushes midnight to the previous date, fix the Date
			// object to point at 1am so it will represent the correct day. See #9366
			if(value.getDate() < this.value.getDate()){
				value = this.dateFuncObj.add(value, "hour", 1);
			}
			return value;
		},

		_setValueAttr: function(/*Date|Number*/ value, /*Boolean*/ priorityChange){
			// summary:
			//		Support set("value", ...)
			// description:
			// 		Set the current date and update the UI.  If the date is disabled, the value will
			//		not change, but the display will change to the corresponding month.
			// value:
			//		Either a Date or the number of seconds since 1970.
			// tags:
			//      protected
			if(value){
				// convert from Number to Date, or make copy of Date object so that setHours() call below
				// doesn't affect original value
				value = new this.dateClassObj(value);
			}
			if(this._isValidDate(value)){
				if(!this._isValidDate(this.value) || this.dateFuncObj.compare(value, this.value)){
					value.setHours(1, 0, 0, 0); // round to nearest day (1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
	
					if(!this.isDisabledDate(value, this.lang)){
						this._set("value", value);
		
						// Set focus cell to the new value.   Arguably this should only happen when there isn't a current
						// focus point.   This will also repopulate the grid, showing the new selected value (and possibly
						// new month/year).
						this.set("currentFocus", value);
	
						if(priorityChange || typeof priorityChange == "undefined"){
							this.onChange(this.get('value'));
							this.onValueSelected(this.get('value'));	// remove in 2.0
						}
					}
				}
			}else{
				// clear value, and repopulate grid (to deselect the previously selected day) without changing currentFocus
				this._set("value", null);
				this.set("currentFocus", this.currentFocus);
			}
		},

		_setText: function(node, text){
			// summary:
			//		This just sets the content of node to the specified text.
			//		Can't do "node.innerHTML=text" because of an IE bug w/tables, see #3434.
			// tags:
			//      private
			while(node.firstChild){
				node.removeChild(node.firstChild);
			}
			node.appendChild(dojo.doc.createTextNode(text));
		},

		_populateGrid: function(){
			// summary:
			//      Fills in the calendar grid with each day (1-31)
			// tags:
			//      private

			var month = new this.dateClassObj(this.currentFocus);
			month.setDate(1);

			var firstDay = month.getDay(),
				daysInMonth = this.dateFuncObj.getDaysInMonth(month),
				daysInPreviousMonth = this.dateFuncObj.getDaysInMonth(this.dateFuncObj.add(month, "month", -1)),
				today = new this.dateClassObj(),
				dayOffset = dojo.cldr.supplemental.getFirstDayOfWeek(this.lang);
			if(dayOffset > firstDay){ dayOffset -= 7; }

			// Iterate through dates in the calendar and fill in date numbers and style info
			dojo.query(".dijitCalendarDateTemplate", this.domNode).forEach(function(template, i){
				i += dayOffset;
				var date = new this.dateClassObj(month),
					number, clazz = "dijitCalendar", adj = 0;

				if(i < firstDay){
					number = daysInPreviousMonth - firstDay + i + 1;
					adj = -1;
					clazz += "Previous";
				}else if(i >= (firstDay + daysInMonth)){
					number = i - firstDay - daysInMonth + 1;
					adj = 1;
					clazz += "Next";
				}else{
					number = i - firstDay + 1;
					clazz += "Current";
				}

				if(adj){
					date = this.dateFuncObj.add(date, "month", adj);
				}
				date.setDate(number);

				if(!this.dateFuncObj.compare(date, today, "date")){
					clazz = "dijitCalendarCurrentDate " + clazz;
				}

				if(this._isSelectedDate(date, this.lang)){
					clazz = "dijitCalendarSelectedDate " + clazz;
				}

				if(this.isDisabledDate(date, this.lang)){
					clazz = "dijitCalendarDisabledDate " + clazz;
				}

				var clazz2 = this.getClassForDate(date, this.lang);
				if(clazz2){
					clazz = clazz2 + " " + clazz;
				}

				template.className = clazz + "Month dijitCalendarDateTemplate";
				template.dijitDateValue = date.valueOf();				// original code
				dojo.attr(template, "dijitDateValue", date.valueOf());	// so I can dojo.query() it
				var label = dojo.query(".dijitCalendarDateLabel", template)[0],
					text = date.getDateLocalized ? date.getDateLocalized(this.lang) : date.getDate();
				this._setText(label, text);
			}, this);

			// Repopulate month drop down list based on current year.
			// Need to do this to hide leap months in Hebrew calendar.
			var monthNames = this.dateLocaleModule.getNames('months', 'wide', 'standAlone', this.lang, month);
			this.monthDropDownButton.dropDown.set("months", monthNames);

			// Set name of current month and also fill in spacer element with all the month names
			// (invisible) so that the maximum width will affect layout.   But not on IE6 because then
			// the center <TH> overlaps the right <TH> (due to a browser bug).
			this.monthDropDownButton.containerNode.innerHTML =
				(dojo.isIE == 6 ? "" : "<div class='dijitSpacer'>" + this.monthDropDownButton.dropDown.domNode.innerHTML + "</div>") +
				"<div class='dijitCalendarMonthLabel dijitCalendarCurrentMonthLabel'>" +  monthNames[month.getMonth()] + "</div>";

			// Fill in localized prev/current/next years
			var y = month.getFullYear() - 1;
			var d = new this.dateClassObj();
			dojo.forEach(["previous", "current", "next"], function(name){
				d.setFullYear(y++);
				this._setText(this[name+"YearLabelNode"],
					this.dateLocaleModule.format(d, {selector:'year', locale:this.lang}));
			}, this);
		},

		goToToday: function(){
			// summary:
			//      Sets calendar's value to today's date
			this.set('value', new this.dateClassObj());
		},

		constructor: function(/*Object*/args){
			var dateClass = (args.datePackage && (args.datePackage != "dojo.date"))? args.datePackage + ".Date" : "Date";
			this.dateClassObj = dojo.getObject(dateClass, false);
			this.datePackage = args.datePackage || this.datePackage;
			this.dateFuncObj = dojo.getObject(this.datePackage, false);
			this.dateLocaleModule = dojo.getObject(this.datePackage + ".locale", false);
		},

		postMixInProperties: function(){
			// Parser.instantiate sometimes passes in NaN for IE.  Use default value in prototype instead.
			// TODO: remove this for 2.0 (thanks to #11511)
			if(isNaN(this.value)){ delete this.value; }

			this.inherited(arguments);
		},

		buildRendering: function(){
			this.inherited(arguments);
			dojo.setSelectable(this.domNode, false);

			var cloneClass = dojo.hitch(this, function(clazz, n){
				var template = dojo.query(clazz, this.domNode)[0];
	 			for(var i=0; i<n; i++){
					template.parentNode.appendChild(template.cloneNode(true));
				}
			});

			// clone the day label and calendar day templates 6 times to make 7 columns
			cloneClass(".dijitCalendarDayLabelTemplate", 6);
			cloneClass(".dijitCalendarDateTemplate", 6);

			// now make 6 week rows
			cloneClass(".dijitCalendarWeekTemplate", 5);

			// insert localized day names in the header
			var dayNames = this.dateLocaleModule.getNames('days', this.dayWidth, 'standAlone', this.lang);
			var dayOffset = dojo.cldr.supplemental.getFirstDayOfWeek(this.lang);
			dojo.query(".dijitCalendarDayLabel", this.domNode).forEach(function(label, i){
				this._setText(label, dayNames[(i + dayOffset) % 7]);
			}, this);

			var dateObj = new this.dateClassObj(this.currentFocus);

			this.monthDropDownButton.dropDown = new dijit.Calendar._MonthDropDown({
				id: this.id + "_mdd",
				onChange: dojo.hitch(this, "_onMonthSelect")
			});

			this.set('currentFocus', dateObj, false);	// draw the grid to the month specified by currentFocus

			// Set up repeating mouse behavior for increment/decrement of months/years
			var _this = this;
			var typematic = function(nodeProp, dateProp, adj){
				_this._connects.push(
					dijit.typematic.addMouseListener(_this[nodeProp], _this, function(count){
						if(count >= 0){ _this._adjustDisplay(dateProp, adj); }
					}, 0.8, 500)
				);
			};
			typematic("incrementMonth", "month", 1);
			typematic("decrementMonth", "month", -1);
			typematic("nextYearLabelNode", "year", 1);
			typematic("previousYearLabelNode", "year", -1);
		},

		_adjustDisplay: function(/*String*/ part, /*int*/ amount){
			// summary:
			//      Moves calendar forwards or backwards by months or years
			// part:
			//      "month" or "year"
			// amount:
			//      Number of months or years
			// tags:
			//      private
			this._setCurrentFocusAttr(this.dateFuncObj.add(this.currentFocus, part, amount));
		},

		_setCurrentFocusAttr: function(/*Date*/ date, /*Boolean*/ forceFocus){
			// summary:
			//		If the calendar currently has focus, then focuses specified date,
			//		changing the currently displayed month/year if necessary.
			//		If the calendar doesn't have focus, updates currently
			//		displayed month/year, and sets the cell that will get focus.
			// forceFocus:
			//		If true, will focus() the cell even if calendar itself doesn't have focus

			var oldFocus = this.currentFocus,
				oldCell = oldFocus ? dojo.query("[dijitDateValue=" + oldFocus.valueOf() + "]", this.domNode)[0] : null;

			// round specified value to nearest day (1am to avoid issues when DST shift occurs at midnight, see #8521, #9366)
			date = new this.dateClassObj(date);
			date.setHours(1, 0, 0, 0);

			this._set("currentFocus", date);

			// TODO: only re-populate grid when month/year has changed
			this._populateGrid();

			// set tabIndex=0 on new cell, and focus it (but only if Calendar itself is focused)
			var newCell = dojo.query("[dijitDateValue=" + date.valueOf() + "]", this.domNode)[0];
			newCell.setAttribute("tabIndex", this.tabIndex);
			if(this._focused || forceFocus){
				newCell.focus();
			}

			// set tabIndex=-1 on old focusable cell
			if(oldCell && oldCell != newCell){
				if(dojo.isWebKit){	// see #11064 about webkit bug
					oldCell.setAttribute("tabIndex", "-1");
				}else{
						oldCell.removeAttribute("tabIndex");
				}
			}
		},

		focus: function(){
			// summary:
			//		Focus the calendar by focusing one of the calendar cells
			this._setCurrentFocusAttr(this.currentFocus, true);
		},

		_onMonthSelect: function(/*Number*/ newMonth){
			// summary:
			//      Handler for when user selects a month from the drop down list
			// tags:
			//      protected

			// move to selected month, bounding by the number of days in the month
			// (ex: dec 31 --> jan 28, not jan 31)
			this.currentFocus = this.dateFuncObj.add(this.currentFocus, "month",
				newMonth - this.currentFocus.getMonth());
			this._populateGrid();
		},

		_onDayClick: function(/*Event*/ evt){
			// summary:
			//      Handler for day clicks, selects the date if appropriate
			// tags:
			//      protected
			dojo.stopEvent(evt);
			for(var node = evt.target; node && !node.dijitDateValue; node = node.parentNode);
			if(node && !dojo.hasClass(node, "dijitCalendarDisabledDate")){
				this.set('value', node.dijitDateValue);
			}
		},

		_onDayMouseOver: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse over events on days, sets hovered style
			// tags:
			//      protected

			// event can occur on <td> or the <span> inside the td,
			// set node to the <td>.
			var node =
				dojo.hasClass(evt.target, "dijitCalendarDateLabel") ?
				evt.target.parentNode :
				evt.target;

			if(node && (node.dijitDateValue || node == this.previousYearLabelNode || node == this.nextYearLabelNode) ){
				dojo.addClass(node, "dijitCalendarHoveredDate");
				this._currentNode = node;
			}
		},

		_onDayMouseOut: function(/*Event*/ evt){
			// summary:
			//      Handler for mouse out events on days, clears hovered style
			// tags:
			//      protected
	
			if(!this._currentNode){ return; }
			
			// if mouse out occurs moving from <td> to <span> inside <td>, ignore it
			if(evt.relatedTarget && evt.relatedTarget.parentNode == this._currentNode){ return; }
			var cls = "dijitCalendarHoveredDate";
			if(dojo.hasClass(this._currentNode, "dijitCalendarActiveDate")) {
				cls += " dijitCalendarActiveDate";
			}
			dojo.removeClass(this._currentNode, cls);
			this._currentNode = null;
		},
		
		_onDayMouseDown: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				dojo.addClass(node, "dijitCalendarActiveDate");
				this._currentNode = node;
			}
		},
		
		_onDayMouseUp: function(/*Event*/ evt){
			var node = evt.target.parentNode;
			if(node && node.dijitDateValue){
				dojo.removeClass(node, "dijitCalendarActiveDate");
			}
		},

//TODO: use typematic
		handleKey: function(/*Event*/ evt){
			// summary:
			//		Provides keyboard navigation of calendar.
			// description:
			//		Called from _onKeyPress() to handle keypress on a stand alone Calendar,
			//		and also from `dijit.form._DateTimeTextBox` to pass a keypress event
			//		from the `dijit.form.DateTextBox` to be handled in this widget
			// returns:
			//		False if the key was recognized as a navigation key,
			//		to indicate that the event was handled by Calendar and shouldn't be propogated
			// tags:
			//		protected
			var dk = dojo.keys,
				increment = -1,
				interval,
				newValue = this.currentFocus;
			switch(evt.keyCode){
				case dk.RIGHT_ARROW:
					increment = 1;
					//fallthrough...
				case dk.LEFT_ARROW:
					interval = "day";
					if(!this.isLeftToRight()){ increment *= -1; }
					break;
				case dk.DOWN_ARROW:
					increment = 1;
					//fallthrough...
				case dk.UP_ARROW:
					interval = "week";
					break;
				case dk.PAGE_DOWN:
					increment = 1;
					//fallthrough...
				case dk.PAGE_UP:
					interval = evt.ctrlKey || evt.altKey ? "year" : "month";
					break;
				case dk.END:
					// go to the next month
					newValue = this.dateFuncObj.add(newValue, "month", 1);
					// subtract a day from the result when we're done
					interval = "day";
					//fallthrough...
				case dk.HOME:
					newValue = new this.dateClassObj(newValue);
					newValue.setDate(1);
					break;
				case dk.ENTER:
				case dk.SPACE:
					this.set("value", this.currentFocus);
					break;
				default:
					return true;
			}

			if(interval){
				newValue = this.dateFuncObj.add(newValue, interval, increment);
			}

			this._setCurrentFocusAttr(newValue);

			return false;
		},

		_onKeyPress: function(/*Event*/ evt){
			// summary:
			//		For handling keypress events on a stand alone calendar
			if(!this.handleKey(evt)){
				dojo.stopEvent(evt);
			}
		},

		onValueSelected: function(/*Date*/ date){
			// summary:
			//		Notification that a date cell was selected.  It may be the same as the previous value.
			// description:
			//      Formerly used by `dijit.form._DateTimeTextBox` (and thus `dijit.form.DateTextBox`)
			//      to get notification when the user has clicked a date.  Now onExecute() (above) is used.
			// tags:
			//      protected
		},

		onChange: function(/*Date*/ date){
			// summary:
			//		Called only when the selected date has changed
		},

		_isSelectedDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		Extension point so developers can subclass Calendar to
			//		support multiple (concurrently) selected dates
			// tags:
			//		protected extension
			return this._isValidDate(this.value) && !this.dateFuncObj.compare(dateObject, this.value, "date")
		},

		isDisabledDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to disable certain dates in the calendar e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// tags:
			//      extension
/*=====
			return false; // Boolean
=====*/
		},

		getClassForDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to return CSS classes to associate with the date entry for the given dateObject,
			//		for example to indicate a holiday in specified locale.
			// tags:
			//      extension

/*=====
			return ""; // String
=====*/
		}
	}
);

dojo.declare("dijit.Calendar._MonthDropDown", [dijit._Widget, dijit._Templated], {
	// summary:
	//		The month drop down

	// months: String[]
	//		List of names of months, possibly w/some undefined entries for Hebrew leap months
	//		(ex: ["January", "February", undefined, "April", ...])
	months: [],

	templateString: "<div class='dijitCalendarMonthMenu dijitMenu' " +
		"dojoAttachEvent='onclick:_onClick,onmouseover:_onMenuHover,onmouseout:_onMenuHover'></div>",

	_setMonthsAttr: function(/*String[]*/ months){
		this.domNode.innerHTML = dojo.map(months, function(month, idx){
				return month ? "<div class='dijitCalendarMonthLabel' month='" + idx +"'>" + month + "</div>" : "";
			}).join("");
	},

	_onClick: function(/*Event*/ evt){
		this.onChange(dojo.attr(evt.target, "month"));
	},

	onChange: function(/*Number*/ month){
		// summary:
		//		Callback when month is selected from drop down
	},

	_onMenuHover: function(evt){
		dojo.toggleClass(evt.target, "dijitCalendarMonthLabelHover", evt.type == "mouseover");
	}
});

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormValueWidget,
	{
		// summary:
		//		A base class for textbox form inputs

		// trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		// uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		// lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		// propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		// selectOnClick: [const] Boolean
		//		If true, all text will be selected when focused with mouse
		selectOnClick: false,

		// placeHolder: String
		//		Defines a hint to help users fill out the input field (as defined in HTML 5).
		//		This should only contain plain text (no html markup).
		placeHolder: "",
		
		templateString: dojo.cache("dijit.form", "templates/TextBox.html", "<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" role=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" dojoAttachPoint='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n"),
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" dojoAttachPoint="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: dojo.isIE ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		attributeMap: dojo.delegate(dijit.form._FormValueWidget.prototype.attributeMap, {
			maxLength: "focusNode"
		}),
		
		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == dijit.form.TextBox.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		_setPlaceHolderAttr: function(v){
			this._set("placeHolder", v);
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				/* dijitInputField class gives placeHolder same padding as the input field
				 * parent node already has dijitInputField class but it doesn't affect this <span>
				 * since it's position: absolute.
				 */
				this._phspan = dojo.create('span',{className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(document.createTextNode(v));
			
			this._updatePlaceHolder();
		},
		
		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display=(this.placeHolder&&!this._focused&&!this.textbox.value)?"":"none";
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so get('value') works as we like.
			// description:
			//		For `dijit.form.TextBox` this basically returns the value of the <input>.
			//
			//		For `dijit.form.MappedTextBox` subclasses, which have both
			//		a "displayed value" and a separate "submit value",
			//		This treats the "displayed value" as the master value, computing the
			//		submit value from it via this.parse().
			return this.parse(this.get('displayedValue'), this.constraints);
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so set('value', ...) works.
			//
			// description:
			//		Sets the value of the widget to "value" which can be of
			//		any type as determined by the widget.
			//
			// value:
			//		The visual element value is also set to a corresponding,
			//		but not necessarily the same, value.
			//
			// formattedValue:
			//		If specified, used to set the visual element value,
			//		otherwise a computed visual value is used.
			//
			// priorityChange:
			//		If true, an onChange event is fired immediately instead of
			//		waiting for the next blur event.

			var filteredValue;
			if(value !== undefined){
				// TODO: this is calling filter() on both the display value and the actual value.
				// I added a comment to the filter() definition about this, but it should be changed.
				filteredValue = this.filter(value);
				if(typeof formattedValue != "string"){
					if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
						formattedValue = this.filter(this.format(filteredValue, this.constraints));
					}else{ formattedValue = ''; }
				}
			}
			if(formattedValue != null && formattedValue != undefined && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
				this.textbox.value = formattedValue;
				this._set("displayedValue", this.get("displayedValue"));
			}

			this._updatePlaceHolder();

			this.inherited(arguments, [filteredValue, priorityChange]);
		},

		// displayedValue: String
		//		For subclasses like ComboBox where the displayed value
		//		(ex: Kentucky) and the serialized value (ex: KY) are different,
		//		this represents the displayed value.
		//
		//		Setting 'displayedValue' through set('displayedValue', ...)
		//		updates 'value', and vice-versa.  Otherwise 'value' is updated
		//		from 'displayedValue' periodically, like onBlur etc.
		//
		//		TODO: move declaration to MappedTextBox?
		//		Problem is that ComboBox references displayedValue,
		//		for benefit of FilteringSelect.
		displayedValue: "",

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.  Use get('displayedValue') instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use set('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		_getDisplayedValueAttr: function(){
			// summary:
			//		Hook so get('displayedValue') works.
			// description:
			//		Returns the displayed value (what the user sees on the screen),
			// 		after filtering (ie, trimming spaces etc.).
			//
			//		For some subclasses of TextBox (like ComboBox), the displayed value
			//		is different from the serialized value that's actually
			//		sent to the server (see dijit.form.ValidationTextBox.serialize)

			// TODO: maybe we should update this.displayedValue on every keystroke so that we don't need
			// this method
			// TODO: this isn't really the displayed value when the user is typing
			return this.filter(this.textbox.value);
		},

		setDisplayedValue: function(/*String*/ value){
			// summary:
			//		Deprecated.  Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_setDisplayedValueAttr: function(/*String*/ value){
			// summary:
			//		Hook so set('displayedValue', ...) works.
			// description:
			//		Sets the value of the visual element to the string "value".
			//		The widget value is also set to a corresponding,
			//		but not necessarily the same, value.

			if(value === null || value === undefined){ value = '' }
			else if(typeof value != "string"){ value = String(value) }

			this.textbox.value = value;

			// sets the serialized value to something corresponding to specified displayedValue
			// (if possible), and also updates the textbox.value, for example converting "123"
			// to "123.00"
			this._setValueAttr(this.get('value'), undefined);

			this._set("displayedValue", this.get('displayedValue'));
		},

		format: function(/*String*/ value, /*Object*/ constraints){
			// summary:
			//		Replacable function to convert a value to a properly formatted string.
			// tags:
			//		protected extension
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/*String*/ value, /*Object*/ constraints){
			// summary:
			//		Replacable function to convert a formatted string to a value
			// tags:
			//		protected extension

			return value;	// String
		},

		_refreshState: function(){
			// summary:
			//		After the user types some characters, etc., this method is
			//		called to check the field for validity etc.  The base method
			//		in `dijit.form.TextBox` does nothing, but subclasses override.
			// tags:
			//		protected
		},

		_onInput: function(e){
			if(e && e.type && /key/i.test(e.type) && e.keyCode){
				switch(e.keyCode){
					case dojo.keys.SHIFT:
					case dojo.keys.ALT:
					case dojo.keys.CTRL:
					case dojo.keys.TAB:
						return;
				}
			}
			if(this.intermediateChanges){
				var _this = this;
				// the setTimeout allows the key to post to the widget input box
				setTimeout(function(){ _this._handleOnChange(_this.get('value'), false); }, 0);
			}
			this._refreshState();

			// In case someone is watch()'ing for changes to displayedValue
			this._set("displayedValue", this.get("displayedValue"));
		},

		postCreate: function(){
			if(dojo.isIE){ // IE INPUT tag fontFamily has to be set directly using STYLE
				// the setTimeout gives IE a chance to render the TextBox and to deal with font inheritance
				setTimeout(dojo.hitch(this, function(){
				var s = dojo.getComputedStyle(this.domNode);
				if(s){
					var ff = s.fontFamily;
					if(ff){
						var inputs = this.domNode.getElementsByTagName("INPUT");
						if(inputs){
							for(var i=0; i < inputs.length; i++){
								inputs[i].style.fontFamily = ff;
							}
						}
					}
				}
				}), 0);
			}

			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values should be the same

			this.inherited(arguments);

			if(dojo.isMoz || dojo.isOpera){
				this.connect(this.textbox, "oninput", "_onInput");
			}else{
				this.connect(this.textbox, "onkeydown", "_onInput");
				this.connect(this.textbox, "onkeyup", "_onInput");
				this.connect(this.textbox, "onpaste", "_onInput");
				this.connect(this.textbox, "oncut", "_onInput");
			}
		},

		_blankValue: '', // if the textbox is blank, what value should be reported
		filter: function(val){
			// summary:
			//		Auto-corrections (such as trimming) that are applied to textbox
			//		value on blur or form submit.
			// description:
			//		For MappedTextBox subclasses, this is called twice
			// 			- once with the display value
			//			- once the value as set/returned by set('value', ...)
			//		and get('value'), ex: a Number for NumberTextBox.
			//
			//		In the latter case it does corrections like converting null to NaN.  In
			//		the former case the NumberTextBox.filter() method calls this.inherited()
			//		to execute standard trimming code in TextBox.filter().
			//
			//		TODO: break this into two methods in 2.0
			//
			// tags:
			//		protected extension
			if(val === null){ return this._blankValue; }
			if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		_setBlurValue: function(){
			this._setValueAttr(this.get('value'), true);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this._setBlurValue();
			this.inherited(arguments);

			if(this._selectOnClickHandle){
				this.disconnect(this._selectOnClickHandle);
			}
			if(this.selectOnClick && dojo.isMoz){
				this.textbox.selectionStart = this.textbox.selectionEnd = undefined; // clear selection so that the next mouse click doesn't reselect
			}
			
			this._updatePlaceHolder();
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }

			// Select all text on focus via click if nothing already selected.
			// Since mouse-up will clear the selection need to defer selection until after mouse-up.
			// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
			if(this.selectOnClick && by == "mouse"){
				this._selectOnClickHandle = this.connect(this.domNode, "onmouseup", function(){
					// Only select all text on first click; otherwise users would have no way to clear
					// the selection.
					this.disconnect(this._selectOnClickHandle);

					// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
					// and if not, then select all the text
					var textIsNotSelected;
					if(dojo.isIE){
						var range = dojo.doc.selection.createRange();
						var parent = range.parentElement();
						textIsNotSelected = parent == this.textbox && range.text.length == 0;
					}else{
						textIsNotSelected = this.textbox.selectionStart == this.textbox.selectionEnd;
					}
					if(textIsNotSelected){
						dijit.selectInputText(this.textbox);
					}
				});
			}

			this._updatePlaceHolder();
			
			// call this.inherited() before refreshState(), since this.inherited() will possibly scroll the viewport
			// (to scroll the TextBox into view), which will affect how _refreshState() positions the tooltip
			this.inherited(arguments);

			this._refreshState();
		},

		reset: function(){
			// Overrides dijit._FormWidget.reset().
			// Additionally resets the displayed textbox value to ''
			this.textbox.value = '';
			this.inherited(arguments);
		}
	}
);

dijit.selectInputText = function(/*DomNode*/ element, /*Number?*/ start, /*Number?*/ stop){
	// summary:
	//		Select text in the input element argument, from start (default 0), to stop (default end).

	// TODO: use functions in _editor/selection.js?
	var _window = dojo.global;
	var _document = dojo.doc;
	element = dojo.byId(element);
	if(isNaN(start)){ start = 0; }
	if(isNaN(stop)){ stop = element.value ? element.value.length : 0; }
	dijit.focus(element);
	if(_document["selection"] && dojo.body()["createTextRange"]){ // IE
		if(element.createTextRange){
			var r = element.createTextRange();
			r.collapse(true);
			r.moveStart("character", -99999); // move to 0
			r.moveStart("character", start); // delta from 0 is the correct position
			r.moveEnd("character", stop-start);
			r.select();
		}
	}else if(_window["getSelection"]){
		if(element.setSelectionRange){
			element.setSelectionRange(start, stop);
		}
	}
};

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: dijit.defaultDuration,

		templateString: dojo.cache("dijit", "templates/Tooltip.html", "<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" role='alert'></div\n\t><div class=\"dijitTooltipConnector\" dojoAttachPoint=\"connectorNode\"></div\n></div>\n"),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });
		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.width = "auto";

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, dijit.getPopupAroundAlignment((position && position.length) ? position : dijit.Tooltip.defaultPosition, !rtl), dojo.hitch(this, "orient"));

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected
			this.connectorNode.style.top = ""; //reset to default
			
			//Adjust the spaceAvailable width, without changing the spaceAvailable object
			var tooltipSpaceAvaliableWidth = spaceAvailable.w - this.connectorNode.offsetWidth;

			node.className = "dijitTooltip " +
				{
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];
				
			// reduce tooltip's width to the amount of width available, so that it doesn't overflow screen
			this.domNode.style.width = "auto";
			var size = dojo.contentBox(this.domNode);
			
			var width = Math.min((Math.max(tooltipSpaceAvaliableWidth,1)), size.w);
			var widthWasReduced = width < size.w;
			
			this.domNode.style.width = width+"px";
						
			//Adjust width for tooltips that have a really long word or a nowrap setting
			if(widthWasReduced){
				this.containerNode.style.overflow = "auto"; //temp change to overflow to detect if our tooltip needs to be wider to support the content
				var scrollWidth = this.containerNode.scrollWidth;
				this.containerNode.style.overflow = "visible"; //change it back
				if(scrollWidth > width){
					scrollWidth = scrollWidth + dojo.style(this.domNode,"paddingLeft") + dojo.style(this.domNode,"paddingRight");
					this.domNode.style.width = scrollWidth + "px";
				}
			}
			
			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var mb = dojo.marginBox(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(mb.h > spaceAvailable.h){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = spaceAvailable.h - (aroundNodeCoords.h / 2) - (tooltipConnectorHeight / 2);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						mb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}
			
			return Math.max(0, size.w - tooltipSpaceAvaliableWidth);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
	// summary:
	//		Display tooltip w/specified contents in specified position.
	//		See description of dijit.Tooltip.defaultPosition for details on position parameter.
	//		If position is not specified then dijit.Tooltip.defaultPosition is used.
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode, position, rtl);
};

dijit.hideTooltip = function(aroundNode){
	// summary:
	//		Hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node, the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on position parameter.
		position: [],

		_setConnectIdAttr: function(/*String*/ newId){
			// summary:
			//		Connect to node(s) (specified by id)

			// Remove connections to old nodes (if there are any)
			dojo.forEach(this._connections || [], function(nested){
				dojo.forEach(nested, dojo.hitch(this, "disconnect"));
			}, this);

			// Make connections to nodes in newIds.
			var ary = dojo.isArrayLike(newId) ? newId : (newId ? [newId] : []);
			this._connections = dojo.map(ary, function(id){
				var node = dojo.byId(id);
				return node ? [
					this.connect(node, "onmouseenter", "_onTargetMouseEnter"),
					this.connect(node, "onmouseleave", "_onTargetMouseLeave"),
					this.connect(node, "onfocus", "_onTargetFocus"),
					this.connect(node, "onblur", "_onTargetBlur")
				] : [];
			}, this);
	
			this._set("connectId", newId);

			this._connectIds = ary;	// save as array
		},

		addTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(dojo.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface
			
			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = dojo.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			dojo.forEach(dojo.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		_onTargetMouseEnter: function(/*Event*/ e){
			// summary:
			//		Handler for mouseenter event on the target node
			// tags:
			//		private
			this._onHover(e);
		},

		_onTargetMouseLeave: function(/*Event*/ e){
			// summary:
			//		Handler for mouseleave event on the target node
			// tags:
			//		private
			this._onUnHover(e);
		},

		_onTargetFocus: function(/*Event*/ e){
			// summary:
			//		Handler for focus event on the target node
			// tags:
			//		private

			this._focus = true;
			this._onHover(e);
		},

		_onTargetBlur: function(/*Event*/ e){
			// summary:
			//		Handler for blur event on the target node
			// tags:
			//		private

			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			// keep a tooltip open if the associated element still has focus (even though the
			// mouse moved away)
			if(this._focus){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target, this.position, !this.isLeftToRight());

			this._connectNode = target;
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				dijit.hideTooltip(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		onShow: function(target, position){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		uninitialize: function(){
			this.close();
			this.inherited(arguments);
		}
	}
);

// dijit.Tooltip.defaultPosition: String[]
//		This variable controls the position of tooltips, if the position is not specified to
//		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the following values:
//
//			* before: places tooltip to the left of the target node/widget, or to the right in
//			  the case of RTL scripts like Hebrew and Arabic
//			* after: places tooltip to the right of the target node/widget, or to the left in
//			  the case of RTL scripts like Hebrew and Arabic
//			* above: tooltip goes above target node
//			* below: tooltip goes below target node
//
//		The list is positions is tried, in order, until a position is found where the tooltip fits
//		within the viewport.
//
//		Be careful setting this parameter.  A value of "above" may work fine until the user scrolls
//		the screen so that there's no room above the target node.   Nodes with drop downs, like
//		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
//		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
//		is only room below (or above) the target node, but not both.
dijit.Tooltip.defaultPosition = ["after", "before"];

}

if(!dojo._hasResource["dijit.form.ValidationTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ValidationTextBox"] = true;
dojo.provide("dijit.form.ValidationTextBox");






/*=====
	dijit.form.ValidationTextBox.__Constraints = function(){
		// locale: String
		//		locale used for validation, picks up value from this widget's lang attribute
		// _flags_: anything
		//		various flags passed to regExpGen function
		this.locale = "";
		this._flags_ = "";
	}
=====*/

dojo.declare(
	"dijit.form.ValidationTextBox",
	dijit.form.TextBox,
	{
		// summary:
		//		Base class for textbox widgets with the ability to validate content of various types and provide user feedback.
		// tags:
		//		protected

		templateString: dojo.cache("dijit.form", "templates/ValidationTextBox.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\n\tid=\"widget_${id}\" role=\"presentation\"\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" dojoAttachPoint='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n"),
		baseClass: "dijitTextBox dijitValidationTextBox",

		// required: Boolean
		//		User is required to enter data into this field.
		required: false,

		// promptMessage: String
		//		If defined, display this hint string immediately on focus to the textbox, if empty.
		//		Also displays if the textbox value is Incomplete (not yet valid but will be with additional input).
		//		Think of this like a tooltip that tells the user what to do, not an error message
		//		that tells the user what they've done wrong.
		//
		//		Message disappears when user starts typing.
		promptMessage: "",

		// invalidMessage: String
		// 		The message to display if value is invalid.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the promptMessage instead.
		invalidMessage: "$_unset_$",

		// missingMessage: String
		// 		The message to display if value is empty and the field is required.
		//		The translated string value is read from the message file by default.
		// 		Set to "" to use the invalidMessage instead.
		missingMessage: "$_unset_$",

		// message: String
		//		Currently error/prompt message.
		//		When using the default tooltip implementation, this will only be
		//		displayed when the field is focused.
		message: "",

		// constraints: dijit.form.ValidationTextBox.__Constraints
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},

		// regExp: [extension protected] String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",

		regExpGen: function(/*dijit.form.ValidationTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to generate regExp when dependent on constraints.
			//		Do not specify both regExp and regExpGen.
			// tags:
			//		extension protected
			return this.regExp; // String
		},

		// state: [readonly] String
		//		Shows current state (ie, validation result) of input (""=Normal, Incomplete, or Error)
		state: "",

		// tooltipPosition: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on this parameter.
		tooltipPosition: [],

		_setValueAttr: function(){
			// summary:
			//		Hook so set('value', ...) works.
			this.inherited(arguments);
			this.validate(this._focused);
		},

		validator: function(/*anything*/ value, /*dijit.form.ValidationTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the text input against the regular expression.
			// tags:
			//		protected
			return (new RegExp("^(?:" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== undefined); // Boolean
		},

		_isValidSubset: function(){
			// summary:
			//		Returns true if the value is either already valid or could be made valid by appending characters.
			//		This is used for validation while the user [may be] still typing.
			return this.textbox.value.search(this._partialre) == 0;
		},

		isValid: function(/*Boolean*/ isFocused){
			// summary:
			//		Tests if value is valid.
			//		Can override with your own routine in a subclass.
			// tags:
			//		protected
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary:
			//		Checks for whitespace
			return (this.trim ? /^\s*$/ : /^$/).test(value); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// summary:
			//		Return an error message to show if appropriate
			// tags:
			//		protected
			return (this.required && this._isEmpty(this.textbox.value)) ? this.missingMessage : this.invalidMessage; // String
		},

		getPromptMessage: function(/*Boolean*/ isFocused){
			// summary:
			//		Return a hint message to show when widget is first focused
			// tags:
			//		protected
			return this.promptMessage; // String
		},

		_maskValidSubsetError: true,
		validate: function(/*Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			// tags:
			//		protected
			var message = "";
			var isValid = this.disabled || this.isValid(isFocused);
			if(isValid){ this._maskValidSubsetError = true; }
			var isEmpty = this._isEmpty(this.textbox.value);
			var isValidSubset = !isValid && isFocused && this._isValidSubset();
			this._set("state", isValid ? "" : (((((!this._hasBeenBlurred || isFocused) && isEmpty) || isValidSubset) && this._maskValidSubsetError) ? "Incomplete" : "Error"));
			dijit.setWaiState(this.focusNode, "invalid", isValid ? "false" : "true");

			if(this.state == "Error"){
				this._maskValidSubsetError = isFocused && isValidSubset; // we want the error to show up after a blur and refocus
				message = this.getErrorMessage(isFocused);
			}else if(this.state == "Incomplete"){
				message = this.getPromptMessage(isFocused); // show the prompt whenever the value is not yet complete
				this._maskValidSubsetError = !this._hasBeenBlurred || isFocused; // no Incomplete warnings while focused
			}else if(isEmpty){
				message = this.getPromptMessage(isFocused); // show the prompt whenever there's no error and no text
			}
			this.set("message", message);

			return isValid;
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		Overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			// tags:
			//		extension
			dijit.hideTooltip(this.domNode);
			if(message && this._focused){
				dijit.showTooltip(message, this.domNode, this.tooltipPosition, !this.isLeftToRight());
			}
		},

		_refreshState: function(){
			// Overrides TextBox._refreshState()
			this.validate(this._focused);
			this.inherited(arguments);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.constraints = {};
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			if(!constraints.locale && this.lang){
				constraints.locale = this.lang;
			}
			this._set("constraints", constraints);
			this._computePartialRE();
		},

		_computePartialRE: function(){
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			var partialre = "";
			// parse the regexp and produce a new regexp that matches valid subsets
			// if the regexp is .* then there's no use in matching subsets since everything is valid
			if(p != ".*"){ this.regExp.replace(/\\.|\[\]|\[.*?[^\\]{1}\]|\{.*?\}|\(\?[=:!]|./g,
				function (re){
					switch(re.charAt(0)){
						case '{':
						case '+':
						case '?':
						case '*':
						case '^':
						case '$':
						case '|':
						case '(':
							partialre += re;
							break;
						case ")":
							partialre += "|$)";
							break;
						 default:
							partialre += "(?:"+re+"|$)";
							break;
					}
				}
			);}
			try{ // this is needed for now since the above regexp parsing needs more test verification
				"".search(partialre);
			}catch(e){ // should never be here unless the original RE is bad or the parsing is bad
				partialre = this.regExp;
				console.warn('RegExp error in ' + this.declaredClass + ': ' + this.regExp);
			} // should never be here unless the original RE is bad or the parsing is bad
			this._partialre = "^(?:" + partialre + ")$";
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			if(!this.invalidMessage){ this.invalidMessage = this.promptMessage; }
			if(this.missingMessage == "$_unset_$"){ this.missingMessage = this.messages.missingMessage; }
			if(!this.missingMessage){ this.missingMessage = this.invalidMessage; }
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls attachPoints
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			this.inherited(arguments);	// call FormValueWidget._setDisabledAttr()
			this._refreshState();
		},

		_setRequiredAttr: function(/*Boolean*/ value){
			this._set("required", value);
			dijit.setWaiState(this.focusNode, "required", value);
			this._refreshState();
		},

		_setMessageAttr: function(/*String*/ message){
			this._set("message", message);
			this.displayMessage(message);
		},

		reset:function(){
			// Overrides dijit.form.TextBox.reset() by also
			// hiding errors about partial matches
			this._maskValidSubsetError = true;
			this.inherited(arguments);
		},

		_onBlur: function(){
			// the message still exists but for back-compat, and to erase the tooltip
			// (if the message is being displayed as a tooltip), call displayMessage('')
			this.displayMessage('');

			this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.form.MappedTextBox",
	dijit.form.ValidationTextBox,
	{
		// summary:
		//		A dijit.form.ValidationTextBox subclass which provides a base class for widgets that have
		//		a visible formatted display value, and a serializable
		//		value in a hidden input field which is actually sent to the server.
		// description:
		//		The visible display may
		//		be locale-dependent and interactive.  The value sent to the server is stored in a hidden
		//		input field which uses the `name` attribute declared by the original widget.  That value sent
		//		to the server is defined by the dijit.form.MappedTextBox.serialize method and is typically
		//		locale-neutral.
		// tags:
		//		protected

		postMixInProperties: function(){
			this.inherited(arguments);

			// we want the name attribute to go to the hidden <input>, not the displayed <input>,
			// so override _FormWidget.postMixInProperties() setting of nameAttrSetting
			this.nameAttrSetting = "";
		},

		serialize: function(/*anything*/ val, /*Object?*/ options){
			// summary:
			//		Overridable function used to convert the get('value') result to a canonical
			//		(non-localized) string.  For example, will print dates in ISO format, and
			//		numbers the same way as they are represented in javascript.
			// tags:
			//		protected extension
			return val.toString ? val.toString() : ""; // String
		},

		toString: function(){
			// summary:
			//		Returns widget as a printable string using the widget's value
			// tags:
			//		protected
			var val = this.filter(this.get('value')); // call filter in case value is nonstring and filter has been customized
			return val != null ? (typeof val == "string" ? val : this.serialize(val, this.constraints)) : ""; // String
		},

		validate: function(){
			// Overrides `dijit.form.TextBox.validate`
			this.valueNode.value = this.toString();
			return this.inherited(arguments);
		},

		buildRendering: function(){
			// Overrides `dijit._Templated.buildRendering`

			this.inherited(arguments);

			// Create a hidden <input> node with the serialized value used for submit
			// (as opposed to the displayed value).
			// Passing in name as markup rather than calling dojo.create() with an attrs argument
			// to make dojo.query(input[name=...]) work on IE. (see #8660)
			this.valueNode = dojo.place("<input type='hidden'" + (this.name ? " name='" + this.name.replace(/'/g, "&quot;") + "'" : "") + "/>", this.textbox, "after");
		},

		reset: function(){
			// Overrides `dijit.form.ValidationTextBox.reset` to
			// reset the hidden textbox value to ''
			this.valueNode.value = '';
			this.inherited(arguments);
		}
	}
);

/*=====
	dijit.form.RangeBoundTextBox.__Constraints = function(){
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		// max: Number
		//		Maximum signed value.  Default is +Infinity
		this.min = min;
		this.max = max;
	}
=====*/

dojo.declare(
	"dijit.form.RangeBoundTextBox",
	dijit.form.MappedTextBox,
	{
		// summary:
		//		Base class for textbox form widgets which defines a range of valid values.

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		/*=====
		// constraints: dijit.form.RangeBoundTextBox.__Constraints
		constraints: {},
		======*/

		rangeCheck: function(/*Number*/ primitive, /*dijit.form.RangeBoundTextBox.__Constraints*/ constraints){
			// summary:
			//		Overridable function used to validate the range of the numeric input value.
			// tags:
			//		protected
			return	("min" in constraints? (this.compare(primitive,constraints.min) >= 0) : true) &&
				("max" in constraints? (this.compare(primitive,constraints.max) <= 0) : true); // Boolean
		},

		isInRange: function(/*Boolean*/ isFocused){
			// summary:
			//		Tests if the value is in the min/max range specified in constraints
			// tags:
			//		protected
			return this.rangeCheck(this.get('value'), this.constraints);
		},

		_isDefinitelyOutOfRange: function(){
			// summary:
			//		Returns true if the value is out of range and will remain
			//		out of range even if the user types more characters
			var val = this.get('value');
			var isTooLittle = false;
			var isTooMuch = false;
			if("min" in this.constraints){
				var min = this.constraints.min;
				min = this.compare(val, ((typeof min == "number") && min >= 0 && val !=0) ? 0 : min);
				isTooLittle = (typeof min == "number") && min < 0;
			}
			if("max" in this.constraints){
				var max = this.constraints.max;
				max = this.compare(val, ((typeof max != "number") || max > 0) ? max : 0);
				isTooMuch = (typeof max == "number") && max > 0;
			}
			return isTooLittle || isTooMuch;
		},

		_isValidSubset: function(){
			// summary:
			//		Overrides `dijit.form.ValidationTextBox._isValidSubset`.
			//		Returns true if the input is syntactically valid, and either within
			//		range or could be made in range by more typing.
			return this.inherited(arguments) && !this._isDefinitelyOutOfRange();
		},

		isValid: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.isValid to check that the value is also in range.
			return this.inherited(arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused)); // Boolean
		},

		getErrorMessage: function(/*Boolean*/ isFocused){
			// Overrides dijit.form.ValidationTextBox.getErrorMessage to print "out of range" message if appropriate
			var v = this.get('value');
			if(v !== null && v !== '' && v !== undefined && (typeof v != "number" || !isNaN(v)) && !this.isInRange(isFocused)){ // don't check isInRange w/o a real value
				return this.rangeMessage; // String
			}
			return this.inherited(arguments);
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			if(!this.rangeMessage){
				this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			this.inherited(arguments);
			if(this.focusNode){ // not set when called from postMixInProperties
				if(this.constraints.min !== undefined){
					dijit.setWaiState(this.focusNode, "valuemin", this.constraints.min);
				}else{
					dijit.removeWaiState(this.focusNode, "valuemin");
				}
				if(this.constraints.max !== undefined){
					dijit.setWaiState(this.focusNode, "valuemax", this.constraints.max);
				}else{
					dijit.removeWaiState(this.focusNode, "valuemax");
				}
			}
		},

		_setValueAttr: function(/*Number*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', ...) works.

			dijit.setWaiState(this.focusNode, "valuenow", value);
			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.form._DateTimeTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._DateTimeTextBox"] = true;
dojo.provide("dijit.form._DateTimeTextBox");







new Date("X"); // workaround for #11279, new Date("") == NaN

/*=====
dojo.declare(
	"dijit.form._DateTimeTextBox.__Constraints",
	[dijit.form.RangeBoundTextBox.__Constraints, dojo.date.locale.__FormatOptions], {
	// summary:
	//		Specifies both the rules on valid/invalid values (first/last date/time allowed),
	//		and also formatting options for how the date/time is displayed.
	// example:
	//		To restrict to dates within 2004, displayed in a long format like "December 25, 2005":
	//	|		{min:'2004-01-01',max:'2004-12-31', formatLength:'long'}
});
=====*/

dojo.declare(
	"dijit.form._DateTimeTextBox",
	[ dijit.form.RangeBoundTextBox, dijit._HasDropDown ],
	{
		// summary:
		//		Base class for validating, serializable, range-bound date or time text box.

		templateString: dojo.cache("dijit.form", "templates/DropDownBox.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdojoAttachPoint=\"_buttonNode, _popupStateNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" role=\"textbox\" aria-haspopup=\"true\"\n\t/></div\n></div>\n"),

		// hasDownArrow: [const] Boolean
		//		Set this textbox to display a down arrow button, to open the drop down list.
		hasDownArrow: true,

		// openOnClick: [const] Boolean
		//		Set to true to open drop down upon clicking anywhere on the textbox.
		openOnClick: true,

		/*=====
		// constraints: dijit.form._DateTimeTextBox.__Constraints
		//		Despite the name, this parameter specifies both constraints on the input
		//		(including starting/ending dates/times allowed) as well as
		//		formatting options like whether the date is displayed in long (ex: December 25, 2005)
		//		or short (ex: 12/25/2005) format.  See `dijit.form._DateTimeTextBox.__Constraints` for details.
		constraints: {},
		======*/

		// Override ValidationTextBox.regExpGen().... we use a reg-ex generating function rather
		// than a straight regexp to deal with locale  (plus formatting options too?)
		regExpGen: dojo.date.locale.regexp,

		// datePackage: String
		//		JavaScript namespace to find calendar routines.	 Uses Gregorian calendar routines
		//		at dojo.date, by default.
		datePackage: "dojo.date",

		// Override _FormWidget.compare() to work for dates/times
		compare: function(/*Date*/ val1, /*Date*/ val2){
			var isInvalid1 = this._isInvalidDate(val1);
			var isInvalid2 = this._isInvalidDate(val2);
			return isInvalid1 ? (isInvalid2 ? 0 : -1) : (isInvalid2 ? 1 : dojo.date.compare(val1, val2, this._selector));
		},

		// flag to _HasDropDown to make drop down Calendar width == <input> width
		forceWidth: true,

		format: function(/*Date*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Formats the value as a Date, according to specified locale (second argument)
			// tags:
			//		protected
			if(!value){ return ''; }
			return this.dateLocaleModule.format(value, constraints);
		},

		"parse": function(/*String*/ value, /*dojo.date.locale.__FormatOptions*/ constraints){
			// summary:
			//		Parses as string as a Date, according to constraints
			// tags:
			//		protected

			return this.dateLocaleModule.parse(value, constraints) || (this._isEmpty(value) ? null : undefined);	 // Date
		},

		// Overrides ValidationTextBox.serialize() to serialize a date in canonical ISO format.
		serialize: function(/*anything*/ val, /*Object?*/ options){
			if(val.toGregorian){
				val = val.toGregorian();
			}
			return dojo.date.stamp.toISOString(val, options);
		},

		// dropDownDefaultValue: Date
		//		The default value to focus in the popupClass widget when the textbox value is empty.
		dropDownDefaultValue : new Date(),

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Use get("value") / set("value", val) to manipulate.
		//		When passed to the parser in markup, must be specified according to `dojo.date.stamp.fromISOString`
		value: new Date(""),	// value.toString()="NaN"

		_blankValue: null,	// used by filter() when the textbox is blank

		// popupClass: [protected extension] String
		//		Name of the popup widget class used to select a date/time.
		//		Subclasses should specify this.
		popupClass: "", // default is no popup = text only


		// _selector: [protected extension] String
		//		Specifies constraints.selector passed to dojo.date functions, should be either
		//		"date" or "time".
		//		Subclass must specify this.
		_selector: "",

		constructor: function(/*Object*/ args){
			var dateClass = args.datePackage ? args.datePackage + ".Date" : "Date";
			this.dateClassObj = dojo.getObject(dateClass, false);
			this.value = new this.dateClassObj("");

			this.datePackage = args.datePackage || this.datePackage;
			this.dateLocaleModule = dojo.getObject(this.datePackage + ".locale", false);
			this.regExpGen = this.dateLocaleModule.regexp;
			this._invalidDate = dijit.form._DateTimeTextBox.prototype.value.toString();
		},

		buildRendering: function(){
			this.inherited(arguments);

			if(!this.hasDownArrow){
				this._buttonNode.style.display = "none";
			}

			// If openOnClick is true, we basically just want to treat the whole widget as the
			// button.  We need to do that also if the actual drop down button will be hidden,
			// so that there's a mouse method for opening the drop down.
			if(this.openOnClick || !this.hasDownArrow){
				this._buttonNode = this.domNode;
				this.baseClass += " dijitComboBoxOpenOnClick";
			}
		},

		_setConstraintsAttr: function(/*Object*/ constraints){
			constraints.selector = this._selector;
			constraints.fullYear = true; // see #5465 - always format with 4-digit years
			var fromISO = dojo.date.stamp.fromISOString;
			if(typeof constraints.min == "string"){ constraints.min = fromISO(constraints.min); }
 			if(typeof constraints.max == "string"){ constraints.max = fromISO(constraints.max); }
			this.inherited(arguments);
		},

		_isInvalidDate: function(/*Date*/ value){
			// summary:
			//		Runs various tests on the value, checking for invalid conditions
			// tags:
			//		private
			return !value || isNaN(value) || typeof value != "object" || value.toString() == this._invalidDate;
		},

		_setValueAttr: function(/*Date|String*/ value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Sets the date on this textbox. Note: value can be a JavaScript Date literal or a string to be parsed.
			if(value !== undefined){
				if(typeof value == "string"){
					value = dojo.date.stamp.fromISOString(value);
				}
				if(this._isInvalidDate(value)){
					value = null;
				}
				if(value instanceof Date && !(this.dateClassObj instanceof Date)){
					value = new this.dateClassObj(value);
				}
			}
			this.inherited(arguments);
			if(this.dropDown){
				this.dropDown.set('value', value, false);
			}
		},

		_set: function(attr, value){
			// Avoid spurious watch() notifications when value is changed to new Date object w/the same value
			if(attr == "value" && this.value instanceof Date && this.compare(value, this.value) == 0){
				return;
			}
			this.inherited(arguments);
		},

		_setDropDownDefaultValueAttr: function(/*Date*/ val){
			if(this._isInvalidDate(val)){
				// convert null setting into today's date, since there needs to be *some* default at all times.
				 val = new this.dateClassObj()
						}
			this.dropDownDefaultValue = val;
		},

		openDropDown: function(/*Function*/ callback){
			// rebuild drop down every time, so that constraints get copied (#6002)
			if(this.dropDown){
				this.dropDown.destroy();
			}
			var PopupProto = dojo.getObject(this.popupClass, false),
				textBox = this,
				value = this.get("value");
			this.dropDown = new PopupProto({
				onChange: function(value){
						// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
						dijit.form._DateTimeTextBox.superclass._setValueAttr.call(textBox, value, true);
					},
					id: this.id + "_popup",
					dir: textBox.dir,
					lang: textBox.lang,
				value: value,
				currentFocus: !this._isInvalidDate(value) ? value : this.dropDownDefaultValue,
					constraints: textBox.constraints,
				filterString: textBox.filterString, // for TimeTextBox, to filter times shown

					datePackage: textBox.datePackage,

					isDisabledDate: function(/*Date*/ date){
						// summary:
						// 	disables dates outside of the min/max of the _DateTimeTextBox
						return !textBox.rangeCheck(date, textBox.constraints);
					}
				});

			this.inherited(arguments);
		},

		_getDisplayedValueAttr: function(){
			return this.textbox.value;
		},

		_setDisplayedValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			this._setValueAttr(this.parse(value, this.constraints), priorityChange, value);
		}
	}
);

}

if(!dojo._hasResource["dijit.form.DateTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.DateTextBox"] = true;
dojo.provide("dijit.form.DateTextBox");




dojo.declare(
	"dijit.form.DateTextBox",
	dijit.form._DateTimeTextBox,
	{
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		//
		//		Example:
		// |	new dijit.form.DateTextBox({value: new Date(2009, 0, 20)})
		//
		//		Example:
		// |	<input dojotype='dijit.form.DateTextBox' value='2009-01-20'>

		baseClass: "dijitTextBox dijitComboBox dijitDateTextBox",
		popupClass: "dijit.Calendar",
		_selector: "date",

		// value: Date
		//		The value of this widget as a JavaScript Date object, with only year/month/day specified.
		//		If specified in markup, use the format specified in `dojo.date.stamp.fromISOString`.
		//		set("value", ...) accepts either a Date object or a string.
		value: new Date("")	// value.toString()="NaN"
	}
);

}

if(!dojo._hasResource["dijit._TimePicker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._TimePicker"] = true;
dojo.provide("dijit._TimePicker");




/*=====
dojo.declare(
	"dijit._TimePicker.__Constraints",
	dojo.date.locale.__FormatOptions,
	{
		// clickableIncrement: String
		//		See `dijit._TimePicker.clickableIncrement`
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		See `dijit._TimePicker.visibleIncrement`
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		See `dijit._TimePicker.visibleRange`
		visibleRange: "T05:00:00"
	}
);
=====*/

dojo.declare("dijit._TimePicker",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		A graphical time picker.
		//		This widget is used internally by other widgets and is not available
		//		as a standalone widget due to lack of accessibility support.

		templateString: dojo.cache("dijit", "templates/TimePicker.html", "<div id=\"widget_${id}\" class=\"dijitMenu\"\n    ><div dojoAttachPoint=\"upArrow\" class=\"dijitButtonNode dijitUpArrowButton\" dojoAttachEvent=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&nbsp;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9650;</div></div\n    ><div dojoAttachPoint=\"timeMenu,focusNode\" dojoAttachEvent=\"onclick:_onOptionSelected,onmouseover,onmouseout\"></div\n    ><div dojoAttachPoint=\"downArrow\" class=\"dijitButtonNode dijitDownArrowButton\" dojoAttachEvent=\"onmouseenter:_buttonMouse,onmouseleave:_buttonMouse\"\n\t\t><div class=\"dijitReset dijitInline dijitArrowButtonInner\" role=\"presentation\">&nbsp;</div\n\t\t><div class=\"dijitArrowButtonChar\">&#9660;</div></div\n></div>\n"),

		// baseClass: [protected] String
		//		The root className to use for the various states of this widget
		baseClass: "dijitTimePicker",

		// clickableIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every clickable element in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T00:15:00` creates 15 minute increments
		//		Must divide dijit._TimePicker.visibleIncrement evenly
		clickableIncrement: "T00:15:00",

		// visibleIncrement: String
		//		ISO-8601 string representing the amount by which
		//		every element with a visible time in the time picker increases.
		//		Set in local time, without a time zone.
		//		Example: `T01:00:00` creates text in every 1 hour increment
		visibleIncrement: "T01:00:00",

		// visibleRange: String
		//		ISO-8601 string representing the range of this TimePicker.
		//		The TimePicker will only display times in this range.
		//		Example: `T05:00:00` displays 5 hours of options
		visibleRange: "T05:00:00",

		// value: String
		//		Date to display.
		//		Defaults to current time and date.
		//		Can be a Date object or an ISO-8601 string.
		//		If you specify the GMT time zone (`-01:00`),
		//		the time will be converted to the local time in the local time zone.
		//		Otherwise, the time is considered to be in the local time zone.
		//		If you specify the date and isDate is true, the date is used.
		//		Example: if your local time zone is `GMT -05:00`,
		//		`T10:00:00` becomes `T10:00:00-05:00` (considered to be local time),
		//		`T10:00:00-01:00` becomes `T06:00:00-05:00` (4 hour difference),
		//		`T10:00:00Z` becomes `T05:00:00-05:00` (5 hour difference between Zulu and local time)
		//		`yyyy-mm-ddThh:mm:ss` is the format to set the date and time
		//		Example: `2007-06-01T09:00:00`
		value: new Date(),

		_visibleIncrement:2,
		_clickableIncrement:1,
		_totalIncrements:10,

		// constraints: dijit._TimePicker.__Constraints
		//		Specifies valid range of times (start time, end time)
		constraints:{},

/*=====
		serialize: function(val, options){
			// summary:
			//		User overridable function used to convert the attr('value') result to a String
			// val: Date
			//		The current value
			// options: Object?
			// tags:
			//		protected
		},
=====*/
		serialize: dojo.date.stamp.toISOString,

/*=====
		// filterString: string
		//		The string to filter by
		filterString: "",
=====*/

		setValue: function(/*Date*/ value){
			// summary:
			//		Deprecated.  Used set('value') instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit._TimePicker:setValue() is deprecated.  Use set('value', ...) instead.", "", "2.0");
			this.set('value', value);
		},

		_setValueAttr: function(/*Date*/ date){
			// summary:
			//		Hook so set('value', ...) works.
			// description:
			//		Set the value of the TimePicker.
			//		Redraws the TimePicker around the new date.
			// tags:
			//		protected
			this._set("value", date);
			this._showText();
		},

		_setFilterStringAttr: function(val){
			// summary:
			//		Called by TimeTextBox to filter the values shown in my list
			this._set("filterString", val);
			this._showText();
		},

		isDisabledDate: function(/*Date*/ dateObject, /*String?*/ locale){
			// summary:
			//		May be overridden to disable certain dates in the TimePicker e.g. `isDisabledDate=dojo.date.locale.isWeekend`
			// type:
			//		extension
			return false; // Boolean
		},

		_getFilteredNodes: function(/*number*/ start, /*number*/ maxNum, /*Boolean*/ before, /*DOMnode*/ lastNode){
			// summary:
			//		Returns an array of nodes with the filter applied.  At most maxNum nodes
			//		will be returned - but fewer may be returned as well.  If the
			//		before parameter is set to true, then it will return the elements
			//		before the given index
			// tags:
			//		private
			var
				nodes = [],
				lastValue = lastNode ? lastNode.date : this._refDate,
				n,
				i = start,
				max = this._maxIncrement + Math.abs(i),
				chk = before ? -1 : 1,
				dec = before ? 1 : 0,
				inc = 1 - dec;
			do{
				i -= dec;
				n = this._createOption(i);
				if(n){
					if((before && n.date > lastValue) || (!before && n.date < lastValue)){
						break; // don't wrap
					}
					nodes[before ? "unshift" : "push"](n);
					lastValue = n.date;
				}
				i += inc;
			}while(nodes.length < maxNum && (i*chk) < max);
			return nodes;
		},

		_showText: function(){
			// summary:
			//		Displays the relevant choices in the drop down list
			// tags:
			//		private
			var fromIso = dojo.date.stamp.fromISOString;
			this.timeMenu.innerHTML = "";
			this._clickableIncrementDate=fromIso(this.clickableIncrement);
			this._visibleIncrementDate=fromIso(this.visibleIncrement);
			this._visibleRangeDate=fromIso(this.visibleRange);
			// get the value of the increments and the range in seconds (since 00:00:00) to find out how many divs to create
			var
				sinceMidnight = function(/*Date*/ date){
					return date.getHours() * 60 * 60 + date.getMinutes() * 60 + date.getSeconds();
				},
				clickableIncrementSeconds = sinceMidnight(this._clickableIncrementDate),
				visibleIncrementSeconds = sinceMidnight(this._visibleIncrementDate),
				visibleRangeSeconds = sinceMidnight(this._visibleRangeDate),
				// round reference date to previous visible increment
				time = (this.value || this.currentFocus).getTime();

			this._refDate = new Date(time - time % (clickableIncrementSeconds*1000));
			this._refDate.setFullYear(1970,0,1); // match parse defaults

			// assume clickable increment is the smallest unit
			this._clickableIncrement = 1;
			// divide the visible range by the clickable increment to get the number of divs to create
			// example: 10:00:00/00:15:00 -> display 40 divs
			this._totalIncrements = visibleRangeSeconds / clickableIncrementSeconds;
			// divide the visible increments by the clickable increments to get how often to display the time inline
			// example: 01:00:00/00:15:00 -> display the time every 4 divs
			this._visibleIncrement = visibleIncrementSeconds / clickableIncrementSeconds;
			// divide the number of seconds in a day by the clickable increment in seconds to get the
			// absolute max number of increments.
			this._maxIncrement = (60 * 60 * 24) / clickableIncrementSeconds;

			var
				// Find the nodes we should display based on our filter.
				// Limit to 10 nodes displayed as a half-hearted attempt to stop drop down from overlapping <input>.
				count = Math.min(this._totalIncrements, 10),
				after = this._getFilteredNodes(0, (count >> 1) + 1, false),
				moreAfter = [],
				estBeforeLength = count - after.length,
				before = this._getFilteredNodes(0, estBeforeLength, true, after[0]);
				if(before.length < estBeforeLength){
					moreAfter = this._getFilteredNodes(after.length, estBeforeLength - before.length, false);
				}
			dojo.forEach(before.concat(after, moreAfter), function(n){this.timeMenu.appendChild(n);}, this);
		},

		constructor: function(){
			this.constraints = {}; // create instance object
		},

		postMixInProperties: function(){
			this.inherited(arguments);
			this._setConstraintsAttr(this.constraints); // this needs to happen now (and later) due to codependency on _set*Attr calls
		},

		_setConstraintsAttr: function(/* Object */ constraints){
			// brings in visibleRange, increments, etc.
			dojo.mixin(this, constraints);

			// dojo.date.locale needs the lang in the constraints as locale
			if(!constraints.locale){
				constraints.locale = this.lang;
			}
		},

		postCreate: function(){
			// assign typematic mouse listeners to the arrow buttons
			this.connect(this.timeMenu, dojo.isIE ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			this._connects.push(dijit.typematic.addMouseListener(this.upArrow, this, "_onArrowUp", 33, 250));
			this._connects.push(dijit.typematic.addMouseListener(this.downArrow, this, "_onArrowDown", 33, 250));

			this.inherited(arguments);
		},

		_buttonMouse: function(/*Event*/ e){
			// summary:
			//		Handler for hover (and unhover) on up/down arrows
			// tags:
			//		private

			// in non-IE browser the "mouseenter" event will become "mouseover",
			// but in IE it's still "mouseenter"
			dojo.toggleClass(e.currentTarget, e.currentTarget == this.upArrow ? "dijitUpArrowHover" : "dijitDownArrowHover",
				e.type == "mouseenter" || e.type == "mouseover");
		},

		_createOption: function(/*Number*/ index){
			// summary:
			//		Creates a clickable time option
			// tags:
			//		private
			var date = new Date(this._refDate);
			var incrementDate = this._clickableIncrementDate;
			date.setHours(date.getHours() + incrementDate.getHours() * index,
				date.getMinutes() + incrementDate.getMinutes() * index,
				date.getSeconds() + incrementDate.getSeconds() * index);
			if(this.constraints.selector == "time"){
				date.setFullYear(1970,0,1); // make sure each time is for the same date
			}
			var dateString = dojo.date.locale.format(date, this.constraints);
			if(this.filterString && dateString.toLowerCase().indexOf(this.filterString) !== 0){
				// Doesn't match the filter - return null
				return null;
			}

			var div = dojo.create("div", {"class": this.baseClass+"Item"});
			div.date = date;
			div.index = index;
			dojo.create('div',{
				"class": this.baseClass + "ItemInner",
				innerHTML: dateString
			}, div);

			if(index%this._visibleIncrement<1 && index%this._visibleIncrement>-1){
				dojo.addClass(div, this.baseClass+"Marker");
			}else if(!(index%this._clickableIncrement)){
				dojo.addClass(div, this.baseClass+"Tick");
			}

			if(this.isDisabledDate(date)){
				// set disabled
				dojo.addClass(div, this.baseClass+"ItemDisabled");
			}
			if(this.value && !dojo.date.compare(this.value, date, this.constraints.selector)){
				div.selected = true;
				dojo.addClass(div, this.baseClass+"ItemSelected");
				if(dojo.hasClass(div, this.baseClass+"Marker")){
					dojo.addClass(div, this.baseClass+"MarkerSelected");
				}else{
					dojo.addClass(div, this.baseClass+"TickSelected");
				}

				// Initially highlight the current value.   User can change highlight by up/down arrow keys
				// or mouse movement.
				this._highlightOption(div, true);
			}
			return div;
		},

		_onOptionSelected: function(/*Object*/ tgt){
			// summary:
			//		Called when user clicks an option in the drop down list
			// tags:
			//		private
			var tdate = tgt.target.date || tgt.target.parentNode.date;
			if(!tdate || this.isDisabledDate(tdate)){ return; }
			this._highlighted_option = null;
			this.set('value', tdate);
			this.onChange(tdate);
		},

		onChange: function(/*Date*/ time){
			// summary:
			//		Notification that a time was selected.  It may be the same as the previous value.
			// tags:
			//      public
		},

		_highlightOption: function(/*node*/ node, /*Boolean*/ highlight){
			// summary:
			//		Turns on/off highlight effect on a node based on mouse out/over event
			// tags:
			//		private
			if(!node){return;}
			if(highlight){
				if(this._highlighted_option){
					this._highlightOption(this._highlighted_option, false);
				}
				this._highlighted_option = node;
			}else if(this._highlighted_option !== node){
				return;
			}else{
				this._highlighted_option = null;
			}
			dojo.toggleClass(node, this.baseClass+"ItemHover", highlight);
			if(dojo.hasClass(node, this.baseClass+"Marker")){
				dojo.toggleClass(node, this.baseClass+"MarkerHover", highlight);
			}else{
				dojo.toggleClass(node, this.baseClass+"TickHover", highlight);
			}
		},

		onmouseover: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseover event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			// if we aren't targeting an item, then we return
			if(!dojo.hasClass(tgr, this.baseClass+"Item")){return;}
			this._highlightOption(tgr, true);
		},

		onmouseout: function(/*Event*/ e){
			// summary:
			//		Handler for onmouseout event
			// tags:
			//		private
			this._keyboardSelected = null;
			var tgr = (e.target.parentNode === this.timeMenu) ? e.target : e.target.parentNode;
			this._highlightOption(tgr, false);
		},

		_mouseWheeled: function(/*Event*/ e){
			// summary:
			//		Handle the mouse wheel events
			// tags:
			//		private
			this._keyboardSelected = null;
			dojo.stopEvent(e);
			// we're not _measuring_ the scroll amount, just direction
			var scrollAmount = (dojo.isIE ? e.wheelDelta : -e.detail);
			this[(scrollAmount>0 ? "_onArrowUp" : "_onArrowDown")](); // yes, we're making a new dom node every time you mousewheel, or click
		},

		_onArrowUp: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Removes the bottom time and add one to the top
			// tags:
			//		private
			if(typeof count == "number" && count == -1){ return; } // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[0].index;
			var divs = this._getFilteredNodes(index, 1, true, this.timeMenu.childNodes[0]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
				this.timeMenu.insertBefore(divs[0], this.timeMenu.childNodes[0]);
			}
		},

		_onArrowDown: function(count){
			// summary:
			//		Handler for up arrow key.
			// description:
			//		Remove the top time and add one to the bottom
			// tags:
			//		private
			if(typeof count == "number" && count == -1){ return; } // typematic end
			if(!this.timeMenu.childNodes.length){ return; }
			var index = this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1].index + 1;
			var divs = this._getFilteredNodes(index, 1, false, this.timeMenu.childNodes[this.timeMenu.childNodes.length - 1]);
			if(divs.length){
				this.timeMenu.removeChild(this.timeMenu.childNodes[0]);
				this.timeMenu.appendChild(divs[0]);
			}
		},

		handleKey: function(/*Event*/ e){
			// summary:
			//		Called from `dijit.form._DateTimeTextBox` to pass a keypress event
			//		from the `dijit.form.TimeTextBox` to be handled in this widget
			// tags:
			//		protected
			var dk = dojo.keys;
			if(e.charOrCode == dk.DOWN_ARROW || e.charOrCode == dk.UP_ARROW){
				dojo.stopEvent(e);
				// Figure out which option to highlight now and then highlight it
				if(this._highlighted_option && !this._highlighted_option.parentNode){
					this._highlighted_option = null;
				}
				var timeMenu = this.timeMenu,
					tgt = this._highlighted_option || dojo.query("." + this.baseClass + "ItemSelected", timeMenu)[0];
				if(!tgt){
					tgt = timeMenu.childNodes[0];
				}else if(timeMenu.childNodes.length){
					if(e.charOrCode == dk.DOWN_ARROW && !tgt.nextSibling){
						this._onArrowDown();
					}else if(e.charOrCode == dk.UP_ARROW && !tgt.previousSibling){
						this._onArrowUp();
					}
					if(e.charOrCode == dk.DOWN_ARROW){
						tgt = tgt.nextSibling;
					}else{
						tgt = tgt.previousSibling;
					}
				}
				this._highlightOption(tgt, true);
				this._keyboardSelected = tgt;
				return false;
			}else if(e.charOrCode == dk.ENTER || e.charOrCode === dk.TAB){
				// mouse hover followed by TAB is NO selection
				if(!this._keyboardSelected && e.charOrCode === dk.TAB){
					return true;	// true means don't call stopEvent()
				}

				// Accept the currently-highlighted option as the value
				if(this._highlighted_option){
					this._onOptionSelected({target: this._highlighted_option});
				}

				// Call stopEvent() for ENTER key so that form doesn't submit,
				// but not for TAB, so that TAB does switch focus
				return e.charOrCode === dk.TAB;
			}
			return undefined;
		}
	}
);

}

if(!dojo._hasResource["dijit.form.TimeTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TimeTextBox"] = true;
dojo.provide("dijit.form.TimeTextBox");




/*=====
dojo.declare(
	"dijit.form.TimeTextBox.__Constraints",
	[dijit.form._DateTimeTextBox.__Constraints, dijit._TimePicker.__Constraints]
);
=====*/

dojo.declare(
	"dijit.form.TimeTextBox",
	dijit.form._DateTimeTextBox,
	{
		// summary:
		//		A validating, serializable, range-bound time text box with a drop down time picker

		baseClass: "dijitTextBox dijitComboBox dijitTimeTextBox",
		popupClass: "dijit._TimePicker",
		_selector: "time",

/*=====
		// constraints: dijit.form.TimeTextBox.__Constraints
		constraints:{},
=====*/

		// value: Date
		//		The value of this widget as a JavaScript Date object.  Note that the date portion implies time zone and daylight savings rules.
		//
		//		Example:
		// |	new dijit.form.TimeTextBox({value: dojo.date.stamp.fromISOString("T12:59:59", new Date())})
		//
		//		When passed to the parser in markup, must be specified according to locale-independent
		//		`dojo.date.stamp.fromISOString` format.
		//
		//		Example:
		// |	<input dojotype='dijit.form.TimeTextBox' value='T12:34:00'>
		value: new Date(""),		// value.toString()="NaN"
		//FIXME: in markup, you have no control over daylight savings

		_onKey: function(evt){
			if(this.disabled || this.readOnly){ return; }
			this.inherited(arguments);

			// If the user has backspaced or typed some numbers, then filter the result list
			// by what they typed.  Maybe there's a better way to detect this, like _handleOnChange()?
			switch(evt.keyCode){
				case dojo.keys.ENTER:
				case dojo.keys.TAB:
				case dojo.keys.ESCAPE:
				case dojo.keys.DOWN_ARROW:
				case dojo.keys.UP_ARROW:
					// these keys have special meaning
					break;
				default:
					// setTimeout() because the keystroke hasn't yet appeared in the <input>,
					// so the get('displayedValue') call below won't give the result we want.
					setTimeout(dojo.hitch(this, function(){
						// set this.filterString to the filter to apply to the drop down list;
						// it will be used in openDropDown()
						var val = this.get('displayedValue');
						this.filterString = (val && !this.parse(val, this.constraints)) ? val.toLowerCase() : "";
	
						// close the drop down and reopen it, in order to filter the items shown in the list
						// and also since the drop down may need to be repositioned if the number of list items has changed
						// and it's being displayed above the <input>
						if(this._opened){
							this.closeDropDown();
						}
						this.openDropDown();
					}), 0);
			}
		}
	}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DateTime"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DateTime"] = true;
// DateTime.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.util.DateTime");












dojo.declare("com.ibm.bpm.social.widgets.task.util._DateTimeDateTextBox", dijit.form.DateTextBox, {
  _onDropDownMouseDown: function(/*Event*/e){
    // on iPad when clicking
    // - the text box do not show the drop down but the keyboard
    // - the drop down do not show the keyboard but the calendar
    if (!com.ibm.bpm.social.util.common.Util.isIPad || e.target == this._popupStateNode ||
      e.target.parentNode == this._popupStateNode) {
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        // if the input field is readonly the keyboard is not opened
        this.textbox.setAttribute("readOnly", "readonly");
        window.setTimeout(dojo.hitch(this, function(){
          this.inherited("_onDropDownMouseDown", [e]);
          this.textbox.removeAttribute("readOnly");
        }));
      }
      else {
        this.inherited(arguments);
      }
    }
  },

  closeDropDown: function(/*Boolean*/focus){
    // when closing with iPad focus always needs to be false
    focus = focus && !com.ibm.bpm.social.util.common.Util.isIPad;
    this.inherited("closeDropDown", [focus]);
  }
});

dojo.declare("com.ibm.bpm.social.widgets.task.util._DateTimeTimeTextBox", dijit.form.TimeTextBox, {
  _onDropDownMouseDown: function(/*Event*/e){
    // on iPad when clicking
    // - the text box do not show the drop down but the keyboard
    // - the drop down do not show the keyboard but the calendar
    if (!com.ibm.bpm.social.util.common.Util.isIPad || e.target == this._popupStateNode ||
      e.target.parentNode == this._popupStateNode) {
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        // if the input field is readonly the keyboard is not opened
        this.textbox.setAttribute("readOnly", "readonly");
        window.setTimeout(dojo.hitch(this, function(){
          this.inherited("_onDropDownMouseDown", [e]);
          this.textbox.removeAttribute("readOnly");
        }));
      }
      else {
        this.inherited(arguments);
      }
    }
  },

  closeDropDown: function(/*Boolean*/focus){
    // when closing with iPad focus always needs to be false
    focus = focus && !com.ibm.bpm.social.util.common.Util.isIPad;
    this.inherited("closeDropDown", [focus]);
  }
});

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.DateTime",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: '<div dojoattachpoint="dateTime" class="bpm-task-modify-task-dialog-due">'
        + '<table border="0" style="border-collapse: collapse"><tbody><tr>'
        + ' <td dojoAttachPoint="_dateLabelTd"><label dojoAttachPoint="_dateTime_labelDate"></label></td>'
        + ' <td dojoAttachPoint="_dateTd"><div dojoAttachPoint="_dateTime_date"></div></td>'
        + ' <td dojoAttachPoint="_timeLabelTd"><label dojoAttachPoint="_dateTime_labelTime"></label></td>'
        + ' <td dojoAttachPoint="_timeTd"><div dojoAttachPoint="_dateTime_time"></div></td>' + '</tr></tbody></table></div>',

      _dateTime: null,
      _date: null,
      _time: null,

      _isOnChangeDate: false,
      _isOnChangeTime: false,

      _ignoreNextOnChangeDate: false,
      _ignoreNextOnChangeTime: false,

      /**
       * @param param {
       *        <dl>
       *        <dt>dateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now", default is "now"</dd>
       *        <dt>minDateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now" </dd>
       *        <dt>maxDateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now" </dd>
       *        <dt>datePromptMessage:</dt>
       *        <dd>(optional) property promptMessage for <i>DateTextBox</i> widget</dd>
       *        <dt>dateConstraints:</dt>
       *        <dd>(optional) constraints for <i>DateTextBox</i> widget</dd>
       *        <dt>timePromptMessage:</dt>
       *        <dd>(optional) property promptMessage for <i>TimeTextBox</i> widget</dd>
       *        <dt>timeConstraints:</dt>
       *        <dd>(optional) constraints for <i>TimeTextBox</i> widget</dd>
       *        </dl> }
       * @return {void} nothing
       */
      constructor: function constructor_DateTime(param){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

        this._showNoLabels = param.showNoLabels;

        this._date = {
          properties: {
            required: true,
            intermediateChanges: true
          },
          constraints: {
            formatLength: "short"
          },
          textBox: null,
          isValid: true
        };

        this._time = {
          properties: {
            required: true,
            intermediateChanges: true
          },
          constraints: {
            formatLength: "short",
            visibleIncrement: "T00:15:00",
            clickableIncrement: "T00:15:00",
            visibleRange: "T03:00:00"
          },
          textBox: null,
          isValid: true
        };

        if (param.dateTime) {
          // may be "now"
          this._dateTime = this._parseDateTime(param.dateTime);
        }

        if (param.datePromptMessage) {
          this._date.properties.promptMessage = param.datePromptMessage;
        }
        else {
          this._date.properties.promptMessage = this._nlsResources.KEY_DatePrompt;
        }
        if (param.dateConstraints) {
          dojo.mixin(this._date.constraints, param.dateConstraints);
        }

        if (param.timePromptMessage) {
          this._time.properties.promptMessage = param.timePromptMessage;
        }
        else {
          this._time.properties.promptMessage = this._nlsResources.KEY_TimePrompt;
        }
        if (param.timeConstraints) {
          dojo.mixin(this._time.constraints, param.timeConstraints);
        }

        // initialize min/max constraints
        if (param.minDateTime) {
          this._minDateTime = param.minDateTime;
        }
        if (param.maxDateTime) {
          this._maxDateTime = param.maxDateTime;
        }
        if (this._minDateTime || this.maxDateTime) {
          var constraints = this._getMinMaxConstraints(this._dateTime, this._time.constraints.formatLength);
          if (constraints.minDate) {
            this._date.constraints.min = constraints.minDate;
          }
          if (constraints.maxDate) {
            this._date.constraints.max = constraints.maxDate;
          }
          if (constraints.minTime) {
            this._time.constraints.min = constraints.minTime;
          }
          if (constraints.maxTime) {
            this._time.constraints.max = constraints.maxTime;
          }
        }

        var calType = com.ibm.bpm.social.util.common.Util.getCalendarUserPreference();
        this._datePackage = null;
        if (calType === "hebrew" || calType === "islamic") {
          this._datePackage = "dojox.date." + calType;
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      postCreate: function postCreate_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // create content
        //

        // date
        var id = this.id + "_date"; // unique id
        this._dateTime_date.setAttribute("id", id);
        this._dateTime_labelDate.setAttribute("for", id);
        this._dateTime_labelDate.innerHTML = this._nlsResources.task_dialog_modifyTask_DueSectionTitle;

        var date = null;
        if (this._dateTime !== null) {
          date = dojo.clone(this._dateTime);
          date.setHours(0, 0, 0, 0);
        }

        this._date.textBox = new com.ibm.bpm.social.widgets.task.util._DateTimeDateTextBox({
          value: dojo.date.stamp.toISOString(date, {
            selector: 'date'
          }),
          datePackage: this._datePackage,
          constraints: this._date.constraints
        }, this._dateTime_date);

        dojo.mixin(this._date.textBox, this._date.properties);
        dijit.setWaiState(this._date.textBox.textbox, "label", this._nlsResources.task_dialog_modifyTask_DueSectionTitle);

        if (!this._showNoLabels) {
          dojo.addClass(this._dateLabelTd, "bpm-task-modify-task-dialog-first-td");
        }
        else {
          dojo.addClass(this._dateTd, "bpm-task-modify-task-dialog-first-td");
          this._dateLabelTd.style.display = "none";

        }

        // time
        id = this.id + "_time"; // unique id
        this._dateTime_time.setAttribute("id", id);
        this._dateTime_labelTime.setAttribute("for", id);
        this._dateTime_labelTime.innerHTML = this._nlsResources["task_dialog_modifyTask_DueTime-label"];

        var time = null;
        if (this._dateTime !== null) {
          time = dojo.clone(this._dateTime);
        }
        this._time.textBox = new com.ibm.bpm.social.widgets.task.util._DateTimeTimeTextBox({
          value: time,
          constraints: this._time.constraints
        }, this._dateTime_time);
        dojo.mixin(this._time.textBox, this._time.properties);

        if (this._showNoLabels) {
          this._timeLabelTd.style.display = "none";

        }
        dijit.setWaiState(this._time.textBox.textbox, "label", this._nlsResources["task_dialog_modifyTask_DueTime-label"]);

        // initialize properties
        //
        this._date.isValid = this._date.textBox.validate();
        if (this._date.isValid) {
          this._time.isValid = this._time.textBox.validate();
        }
        else {
          this._time.isValid = true;
        }

        // set handlers
        //
        this._setHandlers();

        // Add tundra style to time and date picker, otherwise the styling is
        // not complete.

        this.connect(this._date.textBox, "openDropDown", function(){
          dojo.addClass(this._date.textBox.dropDown.domNode.parentNode, "tundra");
        });
        this.connect(this._time.textBox, "openDropDown", function(){
          dojo.addClass(this._time.textBox.dropDown.domNode.parentNode, "tundra");
        });

        // Do common UI styling stuff
        dojo.addClass(this._date.textBox.domNode, 'bspace-border');
        dojo.addClass(this._date.textBox.domNode, "bspace-combo-box");
        dojo.addClass(this._time.textBox.domNode, 'bspace-border');
        dojo.addClass(this._time.textBox.domNode, "bspace-combo-box");

        if (!this.isValid()) {
          dojo.addClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
          dojo.addClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
          dojo.addClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
          dojo.addClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
        }

        this.connect(this, "onChange", function(){
          if (this.isValid()) {
            dojo.removeClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.removeClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.removeClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
            dojo.removeClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
          }
          else {
            dojo.addClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.addClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.addClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
            dojo.addClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
          }
        });

        this.connect(this._date.textBox.domNode, "mouseover", function(){
          dojo.addClass(this._date.textBox.domNode, 'bspace-border-over');
          dojo.addClass(this._date.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._date.textBox.domNode, "mouseout", function(){
          dojo.removeClass(this._date.textBox.domNode, 'bspace-border-over');
          dojo.removeClass(this._date.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._date.textBox.textbox, "focus", function(){
          dojo.addClass(this._date.textBox.domNode, 'bspace-border-active');
          dojo.addClass(this._date.textBox.textbox, 'bspace-text-field-active');
          dojo.addClass(this._date.textBox.domNode, 'bspace-combo-box-active');
        });
        this.connect(this._date.textBox.textbox, "blur", function(){
          dojo.removeClass(this._date.textBox.domNode, 'bspace-border-active');
          dojo.removeClass(this._date.textBox.textbox, 'bspace-text-field-active');
          dojo.removeClass(this._date.textBox.domNode, 'bspace-combo-box-active');
        });

        this.connect(this._time.textBox.domNode, "mouseover", function(){
          dojo.addClass(this._time.textBox.domNode, 'bspace-border-over');
          dojo.addClass(this._time.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._time.textBox.domNode, "mouseout", function(){
          dojo.removeClass(this._time.textBox.domNode, 'bspace-border-over');
          dojo.removeClass(this._time.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._time.textBox.textbox, "focus", function(){
          dojo.addClass(this._time.textBox.domNode, 'bspace-border-active');
          dojo.addClass(this._time.textBox.textbox, 'bspace-text-field-active');
          dojo.addClass(this._time.textBox.domNode, 'bspace-combo-box-active');
        });
        this.connect(this._time.textBox.textbox, "blur", function(){
          dojo.removeClass(this._time.textBox.domNode, 'bspace-border-active');
          dojo.removeClass(this._time.textBox.textbox, 'bspace-text-field-active');
          dojo.removeClass(this._time.textBox.domNode, 'bspace-combo-box-active');
        });

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      startup: function startup_DateTime(){

      },

      /**
       * @return {Date} dateTime
       */
      getDateTime: function getDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var dateTime = null;
        if (this.isValid()) {
          dateTime = this._getDateTime();
        }
        com.ibm.bpm.social.util.common.Log.log(3, "com.ibm.bpm.social.widgets.task.util.DateTime.getDateTime ... dateTime=" +
          dateTime);
        com.ibm.bpm.social.util.common.Log.logExit(1);
        return dateTime;
      },

      /**
       * @type {void}
       */
      isValid: function isValid_DateTime(){
        return this._date.isValid && this._time.isValid;
      },

      /**
       * @type {void}
       */
      isDirty: function isDirty_DateTime(){
        if (this.isValid()) {
          return this._isDateTimeChanged();
        }
        return false;
      },

      onChange: function onChange_DateTime(newValue){
      // event fired if value changes
      },

      setDisabled: function setDisabled_DateTime(disabled){
        if (this._date.textBox !== null) {
          this._date.textBox.set('disabled', disabled);
        }
        if (this._time.textBox !== null) {
          this._time.textBox.set('disabled', disabled);
        }
      },

      _setHandlers: function _setHandlers_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        this._disconnectHandlers();

        if (this._date.textBox !== null) {
          this._eventConnections.push(dojo.connect(this._date.textBox, "onChange", this, this._onChangeDate));
        }
        if (this._time.textBox !== null) {
          this._eventConnections.push(dojo.connect(this._time.textBox, "onChange", this, this._onChangeTime));
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
        return;
      },

      _onChangeDate: function _onChangeDate_DateTime(newValue){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._onChangeDate ... newValue=" + newValue);
        if (!this._isOnChangeDate && !this._ignoreNextOnChangeDate) {
          this._isOnChangeDate = !this._isOnChangeTime;
          if (newValue === undefined || (this._date.textBox.required && newValue === null)) {
            this._date.isValid = false;
          }
          else {
            this._date.isValid = true;
            if (!this._isOnChangeTime) {
              // auto fill empty time field
              if (this._time.textBox !== null) {
                if (this._getTime() === null) {
                  // set time to midnight if no time is set
                  var midnight = new Date();
                  midnight.setHours(0, 0, 0, 0);
                  this._setTime(midnight);
                  this._ignoreNextOnChangeTime = true; // ignore next onChange because validation is done here
                }
              }

              // set new min/max values
              this._setMinMaxConstraints();

              // validate date first
              if (this._date.textBox !== null) {
                this._date.isValid = this._date.textBox.validate(true);
              }
              if (this._time.textBox !== null) {
                this._time.isValid = this._time.textBox.validate();
              }
            }
          }

          this.onChange(this.getDateTime());
          this._isOnChangeDate = false;
        }

        this._ignoreNextOnChangeDate = false;

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _onChangeTime: function _onChangeTime_DateTime(newValue){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._onChangeTime ... newValue=" + newValue);
        if (!this._isOnChangeTime && !this._ignoreNextOnChangeTime) {
          this._isOnChangeTime = !this._isOnChangeDate;
          if (newValue === undefined || (this._time.textBox.required && newValue === null)) {
            this._time.isValid = false;
          }
          else {
            this._time.isValid = true;
            if (!this._isOnChangeDate) {
              // auto fill empty date field
              if (this._date.textBox !== null) {
                if (this._getDate() === null) {
                  // set date today or tomorrow depending on current time if no date is set
                  var date = new Date();
                  date.setHours(0, 0, 0, 0);
                  if (dojo.date.compare(this._getTime(), new Date(), "time") < 0) {
                    date = dojo.date.add(date, "day", 1); // tomorrow
                  }
                  this._setDate(date);
                  this._ignoreNextOnChangeDate = true; // ignore next onChange because validation is done here
                }
              }

              // set new min/max values
              this._setMinMaxConstraints();

              // validate time first
              if (this._time.textBox !== null) {
                this._time.isValid = this._time.textBox.validate(true);
              }
              if (this._date.textBox !== null) {
                this._date.isValid = this._date.textBox.validate();
              }
            }
          }

          this.onChange(this.getDateTime());
          this._isOnChangeTime = false;
        }

        this._ignoreNextOnChangeTime = false;

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * @return {Boolean} isChanged
       */
      _isDateTimeChanged: function _isDateTimeChanged_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var isChanged = false;
        var newDateTime = this._getDateTime();
        if (this._dateTime !== null && newDateTime !== null) {
          isChanged = dojo.date.compare(newDateTime, this._dateTime, "datetime") !== 0;
        }
        else {
          isChanged = !(this._dateTime === null && newDateTime === null);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return isChanged;
      },

      /**
       * @return {Date} dateTime
       */
      _getDateTime: function _getDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var dateTime = null;
        dateTime = this._getDateTimeFromDateAndTime(this._getDate(), this._getTime());

        return dateTime;
      },

      /**
       * @return {Date} date
       */
      _getDate: function _getDate_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var date = null;
        if (this._date.textBox !== null) {
          date = this._date.textBox.get('value');
          if (this._datePackage) {
            date = date.toGregorian();
          }
        }
        return date;
      },

      /**
       * @param {Date} date
       */
      _setDate: function _setDate_DateTime(date){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._date.textBox !== null) {
          this._date.textBox.set('value', date);
        }
      },

      /**
       * @return {Date} time
       */
      _getTime: function _getTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var time = null;
        if (this._time.textBox !== null) {
          time = this._time.textBox.get('value');
        }
        return time;
      },

      /**
       * @param {Date} time
       */
      _setTime: function _setTime_DateTime(time){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._time.textBox !== null) {
          this._time.textBox.set('value', time);
        }
      },

      /**
       * @return {Date} dateTime
       */
      _getDateTimeFromDateAndTime: function _getDateTimeFromDateAndTime_DateTime(date, time){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... date=" + date + ", time=" + time);
        var dateTime = null;

        if (date !== undefined && time !== undefined) {
          if (date !== null && time === null) {
            // date only
            dateTime = date;
          }
          else if (time !== null) {
            if (date === null) {
              // time only means today
              date = new Date();
            }
            // combine date and time
            dateTime =
              new Date(date.getFullYear(), date.getMonth(), date.getDate(), time.getHours(), time.getMinutes(), time
                .getSeconds(), time.getMilliseconds());
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log
            .log(3,
              "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... date and/or time is undefined/invalid");
        }

        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... dateTime=" + dateTime);
        com.ibm.bpm.social.util.common.Log.logExit(2);

        return dateTime;
      },

      /**
       * @return {Date} dateTime
       */
      _parseDateTime: function _parseDateTime_DateTime(dateTime){
        var result = dateTime;
        if (result !== null) {
          if (typeof result === "string" && result === "now") {
            result = new Date();
          }
        }
        return result;
      },

      /**
       * Sets the min and max constraints on the textboxes
       * 
       * @returns {void} nothing
       */
      _setMinMaxConstraints: function _setMinMaxConstraints_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var constraints = this._getMinMaxConstraints(this._getDateTime(), this._time.textBox.constraints.formatLength);
        if (this._date.textBox !== null) {
          if (constraints.minDate) {
            this._date.textBox.constraints.min = constraints.minDate;
          }
          if (constraints.maxDate) {
            this._date.textBox.constraints.max = constraints.maxDate;
          }
        }
        if (this._time.textBox !== null) {
          if (constraints.minTime) {
            this._time.textBox.constraints.min = constraints.minTime;
          }
          if (constraints.maxTime) {
            this._time.textBox.constraints.max = constraints.maxTime;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Calculates the new min and max constraints depending on the actual values
       * 
       * @param {Date} dateTime
       * @param {String} timeFormatLength the formatLength property of the time constraints "short", "medium",
       *        "long" or "full", default is "short"
       * @returns {Object} {minDate, maxDate, minTime, maxTime}
       */
      _getMinMaxConstraints: function _getMinMaxConstraints_DateTime(dateTime, timeFormatLength){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var result = {};
        var compareDate = null;
        var compareTime = null;

        // min date, time
        var minDateTime = this._getMinDateTime();
        if (minDateTime !== null) {
          if (!timeFormatLength || timeFormatLength === "short" || timeFormatLength === "medium") {
            minDateTime.setSeconds(0, 0);
          }
          com.ibm.bpm.social.util.common.Log.log(3,
            "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... minDateTime=" + minDateTime);
          // minDate is date limit date, minTime is date limit time
          var minDate = dojo.clone(minDateTime);
          minDate.setHours(0, 0, 0, 0);
          var minTime = dojo.clone(minDateTime);
          if (dateTime) {
            compareDate = dojo.date.compare(dateTime, minDateTime, "date");
            compareTime = dojo.date.compare(dateTime, minDateTime, "time");
            // default constraints: minDate and minTime
            if (compareDate > 0) {
              // no time constraint
              minTime = -Infinity;
            }
            if (compareTime < 0) {
              // minDate is next day, no time constraint
              minDate = dojo.date.add(minDate, "day", 1);
              minTime = -Infinity;
            }
          }
          else {
            // no date time set yet
            // minDate is next day, no time constraint
            minDate = dojo.date.add(minDate, "day", 1);
            minTime = -Infinity;
          }
          result.minDate = minDate;
          result.minTime = minTime;
        }

        // max date, time
        var maxDateTime = this._getMaxDateTime();
        if (maxDateTime !== null) {
          if (!timeFormatLength || timeFormatLength === "short" || timeFormatLength === "medium") {
            maxDateTime.setSeconds(0, 0);
          }
          com.ibm.bpm.social.util.common.Log.log(3,
            "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... maxDateTime=" + maxDateTime);
          // maxDate is date limit date, maxTime is date limit time
          var maxDate = dojo.clone(maxDateTime);
          maxDate.setHours(0, 0, 0, 0);
          var maxTime = dojo.clone(maxDateTime);
          if (dateTime) {
            compareDate = dojo.date.compare(dateTime, maxDateTime, "date");
            compareTime = dojo.date.compare(dateTime, maxDateTime, "time");
            // default constraints: maxDate and maxTime
            if (compareDate < 0) {
              // no time constraint
              maxTime = Infinity;
            }
            if (compareTime > 0) {
              // maxDate is previous day, no time constraint
              maxDate = dojo.date.add(maxDate, "day", -1);
              maxTime = Infinity;
            }
          }
          else {
            // no date time set yet
            // maxDate is previous day, no time constraint
            maxDate = dojo.date.add(maxDate, "day", -1);
            maxTime = Infinity;
          }
          result.maxDate = maxDate;
          result.maxTime = maxTime;
        }

        com.ibm.bpm.social.util.common.Log.logObject(3, result,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... result");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _getMinDateTime: function _getMinDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this._minDateTime) {
          var minDate = null;
          if (this._date.textBox !== null) {
            minDate = this._date.textBox.constraints.min ? this._date.textBox.constraints.min : null;
          }
          var minTime = null;
          if (this._time.textBox !== null) {
            minTime = this._time.textBox.constraints.max ? this._time.textBox.constraints.min : null;
          }
          this._minDateTime = this._getDateTimeFromDateAndTime(minDate, minTime);
        }
        var result = this._parseDateTime(this._minDateTime);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMinDateTime ... minDateTime=" + result);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _getMaxDateTime: function _getMaxDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this._maxDateTime) {
          var maxDate = null;
          if (this._date.textBox !== null) {
            maxDate = this._date.textBox.constraints.max ? this._date.textBox.constraints.max : null;
          }
          var maxTime = null;
          if (this._time.textBox !== null) {
            maxTime = this._time.textBox.constraints.max ? this._time.textBox.constraints.max : null;
          }
          this._maxDateTime = this._getDateTimeFromDateAndTime(maxDate, maxTime);
        }
        var result = this._parseDateTime(this._maxDateTime);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMaxDateTime ... maxDateTime=" + result);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _disconnectHandlers: function _disconnectHandlers_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._eventConnections) {
          for ( var i = 0; i < this._eventConnections.length; i++) {
            dojo.disconnect(this._eventConnections[i]);
          }
        }
        this._eventConnections = [];
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      destroy: function destroy_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        this._disconnectHandlers();

        if (this._date && this._date.textBox) {
          this._date.textBox.destroy();
          this._date.textBox = null;
        }
        if (this._time && this._time.textBox) {
          this._time.textBox.destroy();
          this._time.textBox = null;
        }
        this.inherited("destroy", arguments);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      }

    });

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.ProcessRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.ProcessRestHandler"] = true;
// ProcessRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.ProcessRestHandler");






dojo.declare("com.ibm.bpm.social.util.common.ProcessRestHandler", com.ibm.bpm.social.util.common.RestHandler, {

  constructor: function constructor_ProcessRestHandler(params){},

  /**
   * @param {object}
   * 
   * @param {String} id
   * @param {function} load
   * @param {function} error
   * @param {boolean} async
   * @param {String} parts one of "all", "data", "none" - default is "all"
   * @param {String} systemID federation only
   */
  getProcess: function getProcess_ProcessRestHandler(id, load, error, async, parts, systemID){

    if (this.isApiTypeFederated()) {
      if (!systemID) {
        this.errorNoSystemID();
      }
    }

    var url = "/process";
    url += "/" + id;
    url = this.addSystemID(url, systemID);

    if (parts && parts != "all") {
      url += "?parts=" + parts;
    }

    var sync = true;
    if (async) {
      sync = false;
    }
    var myself = this;
    this.performRequest("get", {
      url: url,
      sync: sync,
      // load: load,
      load: function(process, ioArgs){
        process = myself.addSystemProperties(process, systemID);
        load(process, ioArgs);
      },
      error: error
    });
  },
  
  /**
   *
   * @param startURL
   * @param load
   * @param error
   * @param async
   * @param systemID
   * @returns nothing
   */
  launchProcess: function launchProcess_ProcessRestHandler(startURL, load, error, async, systemID) {
	var inx = startURL.indexOf("/process");
	var url=startURL.substring(inx);
	url = this.addSystemID(url, systemID);
	this.performRequest("post", {
		url: url,
		load: load,
		error: error,
		sync: !async
	});
  },  
  
  
  /**
   * @param {Array} items
   * @param {String} idKey e.g TKIID or tkiid or id
   * @param actionsFilter
   * @param {Function} load
   * @param {Function} error
   * @param async
   * @param {String} domain, optional for federated backend and more than one item
   * @returns {dojo.Deferred}
   */
  getActions: function getActions_TaskRestHandler(items, idKey, actionsFilter, load, error, async, domain){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    if (actionsFilter) {
      com.ibm.bpm.social.util.common.Log.logObject(3, actionsFilter, "actionsFilter");
    }

    var deferred = null;

    var self = this;
    var idString = "";
    dojo.forEach(items, function(item){
      if (idString !== "") {
        idString += ",";
      }
      idString += self.prefixID(item[idKey], domain, item.systemID);
    });

    var url = "/process/actions?";
    var newLoad = load;
    if (this.isApiTypeFederated()) {
      if (domain) {
        url += "domain=" + domain + "&";
      }
      else if (items.length === 1 && items[0].systemID) {
        url += "systemID=" + items[0].systemID + "&";
        // insert the system id, in federated case the system id is not returned if no domain is specified
        newLoad = function(data, ioArgs){
          if (data && data.items) {
            for ( var i = 0; i < data.items.length; i++) {
              data.items[i].systemID = items[0].systemID;
            }
          }
          load(data, ioArgs);
        };

      }
      else {
        this.errorNoDomainOrSystemID();
      }
    }
    url += "instanceIds=" + encodeURIComponent(idString);

    if (actionsFilter) {
      var actionsFilterString = "";
      for ( var i in actionsFilter) {
        if (actionsFilterString !== "") {
          actionsFilterString += ",";
        }
        actionsFilterString += actionsFilter[i];
      }
      url += "&actions=" + actionsFilterString;
    }

    deferred = this.performRequest("get", {
      url: url,
      sync: !async,
      load: load,
      error: error
    });

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return deferred;
  },


  /**
   * @param {object}
   * @param processID
   * @param {Function} load
   * @param {Function} error
   * @param async
   * @param systemID
   * @param domain
   * @returns {void} nothing
   */
  getActionsForProcess: function getActionsForProcess_ProcessRestHandler(processID, load, error, async, domain){

    var url = "/process/" + encodeURIComponent(processID) + "/actions?";

    var newLoad = load;
    if (this.isApiTypeFederated()) {
      if (domain) {
        url += "domain=" + domain + "&";
      }
//      else if (items.length === 1 && items[0].systemID) {
//        url += "systemID=" + items[0].systemID + "&";
//        // insert the system id, in federated case the system id is not returned if no domain is specified
//        newLoad = function(data, ioArgs){
//          if (data && data.items) {
//            for ( var i = 0; i < data.items.length; i++) {
//              data.items[i].systemID = items[0].systemID;
//            }
//          }
//          load(data, ioArgs);
//        };
//      }
      else {
        this.errorNoDomainOrSystemID();
      }
    }

    var deferred = this.performRequest("get", {
      url: url,
      sync: !async,
      load: load,
      error: error
    });

    return deferred;
  },


  /**
   * @param {object} params
   * 
   * @param id
   * @param load
   * @param error
   * @param priority e.g. Highest/10, High/20, Normal/30, Low/40 or Lowest/50
   * @param dueDate in ISO-8601 format (e.g. 2011-01-05T03:15:30.000-00:00)
   * @param systemID
   * @returns {void} nothing
   */
  updateProcessProperties: function updateProcessProperties_ProcessRestHandler(params){
    var id = params.processID;
    var load = params.load;
    var error = params.error;
    var priority = params.priority;
    var dueDate = params.dueDate;
    var systemID = params.systemID;

    var url = "/process";
    url += "/" + id + "?action=update";

    if (priority) {
      url += "&priority=" + priority;
    }

    if (dueDate) {
      url += "&dueDate=" + dueDate;
    }

    url = this.addSystemID(url, systemID);
    this.performRequest("put", {
      url: url,
      sync: true,
      load: load,
      error: error
    });
  },

  runAdHocAction: function runAdHocAction__ProcessRestHandler(params){
    var id = params.processID;
    var actionID = params.actionID;
    var load = params.load;
    var error = params.error;
    var systemID = params.systemID;

    var url = "/process";
    url += "/" + id + "?action=trigger" + "&id=" + actionID;


    url = this.addSystemID(url, systemID);
    this.performRequest("put", {
      url: url,
      sync: true,
      load: load,
      error: error
    });
  },
	
  /**
   * 
   * @param piid
   * @param comment
   * @param load
   * @param error
   * @param async
   * @param systemID
   * @returns {void} nothing
   */	
  postInstanceStreamComment: function postInstanceStreamComment_ProcessRestHandler(piid, comment, load, error, async, systemID) {
		var url = "/process/" + piid;
		
		var url = this.addSystemID(url, systemID);
		
		var content = {
			action: "comment",
			comment: comment
		};
		
		this.performRequest("post", {
			url: url,
			content: content,
			load: load,
			error: error,
			sync: !async
		});		
  },
  
  statics: {
    _instance: null,

    _createInstance: function _createInstance_ProcessRestHandler(params){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.util.common.ProcessRestHandler();
        com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
        // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
      }
      else {
        // programming error
        var error = new Error("RestHandler instance has already been initialized!");
        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
        throw error;
      }
      return this._instance;
    },

    _getInstance: function _getInstance_ProcessRestHandler(){
      if (this._instance === null) {
        // TODO for now return default handler
        this._instance = new com.ibm.bpm.social.util.common.ProcessRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.util.common.ProcessRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.ProcessRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.ProcessRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog");

























dojo.declare("com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog", 	[com.ibm.bspace.common.util.ui.Dialog, com.ibm.bpm.social.widgets.common._RestWidget], {

	templateString:"<div class=\"changeDueDateDialog\">\n\t<div class=\"changeDueDateDialogSubscript\" dojoAttachpoint=\"subscript\" tabindex=\"0\" wairole=\"label\"></div>\n\t<div class=\"changeDueDateDialogLabel\" dojoAttachPoint=\"label\" tabindex=\"0\" wairole=\"label\"></div>\n\t<!-- <label class=\"changeDueDateDialogDatePickerLabel\" dojoAttachPoint=\"dateLabel\" id=\"${id}_dateLabel\"></label> -->\n\t<div class=\"changeDueDateDialogPicker changeDueDateDialogDatePicker\" dojoAttachPoint=\"datepickerNode\" tabindex=\"0\" wairole=\"label\"></div>\n<!-- <label class=\"changeDueDateDialogTimePickerLabel\" dojoAttachPoint=\"timeLabel\" id=\"${id}_timeLabel\"></label>\n\t<div class=\"changeDueDateDialogPicker changeDueDateDialogTimePicker\" dojoAttachPoint=\"timepickerNode\"></div>-->\n\t<div class=\"changeDueDateButtonBar\">\n\t\t<input type=\"button\" id=\"changeDueDateButtonOK\" value=\"\" dojoAttachPoint=\"okButton\" dojoAttachEvent=\"onclick:_submit\" waiRole=\"button\"/>\n\t</div>\n</div>\n",

	instanceId: null,
	
	taskId: null,
	
	dueDate: null,
	
	dateTimePicker: null,
	
	callback: null,
	
	connects: [],
	
	instanceUrl: window.endpointURLObject.bpmRest +"/v1/process/{id}?action=update&dueDate={date}&parts=header",

	taskUrl: window.endpointURLObject.bpmRest +"/v1/task/{id}?action=update&dueDate={date}&parts=none",
	
	postMixInProperties: function(){
		
		this.changeDueDateDialogMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.dialogParams.title = this.changeDueDateDialogMessages.dueDateHeader;
		
		this.dialogParams.draggable = false;
		
		this.inherited(arguments);
	},
	
    postCreate: function() {
		this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
		
		this.subscript.innerHTML = this.changeDueDateDialogMessages.dueDateSubscript;
		this.label.innerHTML = this.changeDueDateDialogMessages.dueDate;
		
		this.dateTimePicker = new com.ibm.bpm.social.widgets.task.util.DateTime({
            dateTime: dojo.date.stamp.fromISOString(this.dueDate),         
            showNoLabels: true
        }, this.datepickerNode);
		this.connects.push(dojo.connect(this.dateTimePicker, "onChange", this, this._onChange));

		this.okButton.value = this.changeDueDateDialogMessages.ok;
		
		/* TODO
		Remove the following if/else once the following 3 lines have made it into theme.css (without lost update):
		.dijitRtl .processPortal .changeDueDateDialog .changeDueDateButtonBar input {
			float: left;
		}
		*/
		// Start delete
		if (document.body.dir == 'rtl') {
			dojo.style(this.okButton, "float", "left");
		}
		else {
			dojo.style(this.okButton, "float", "right");
		}
		// Stop delete
        
		this.inherited(arguments);

		com.ibm.bspace.common.util.ui.Button.setDisabled(this.okButton, true);
		this._onChange();
	},
	
    _onChange: function(){
      var valid = true;
      if (this.dateTimePicker) {
        valid = this.dateTimePicker.isValid();
      }
      com.ibm.bspace.common.util.ui.Button.setDisabled(this.okButton, !valid);
    },
	
	show: function(){
	    this.inherited(arguments);
		dojo.connect(this.popup, 'onHide', this, this.delayedDestroy);
	},
	
    hide: function(){
    	this.inherited(arguments);
		this.destroy();
    },

    _submit: function(arguments){
    	var val = false;
    	if(this.popup && this.popup.isValid){
    		val = this.popup.isValid();
    		if(val){
    			this._changeDueDate(arguments);
    		}
    	}
    },
    
    _changeDueDate: function(arguments){
    	var date = dojo.date.stamp.toISOString(this.dateTimePicker.getDateTime(), {zulu: true});
		
		if(this.instanceId != null) {
			params = {
				processID: this.instanceId,
				load: dojo.hitch(this, "_handleResponse"),
				error: dojo.hitch(this, "_onXhrErrors"),
				dueDate: date
			};
			this.processRestHandler.updateProcessProperties(params);
		} else if(this.taskId != null) {
			this.taskRestHandler.updateTaskProperties(this.taskId, dojo.hitch(this, "_handleResponse"),
				dojo.hitch(this, "_onXhrErrors"), "", date);
		} else {
			this.hide();
		}
    },

	_handleResponse: function(data, xhrArgs){
		var message = new com.ibm.bpm.social.Aether({
			'message': "<div>"+this.changeDueDateDialogMessages.changeDueDateSuccess+"</div>",
			'containerDivId':"messageNotificationDiv",
			'position': 'top-right',
			'fade': true,
			'parent': this,
			'displayCloseIcon': true
		});
		com.ibm.bpm.social.util.common.Util.currentInstanceData[this.instanceId] = data.data;
		
		if(this.callback!=null){
			this.callback(data.data);
		}
		
		dojo.publish(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED+"."+this.instanceId, [data.data]);
		
		this.hide();
	},
	
	_onXhrErrors: function(data,xhrArgs) {
		var errMsg = "";
		if(xhrArgs && xhrArgs.xhr && typeof(xhrArgs.xhr.responseText)!='undefined') {
			var errMsg = "<div>"+dojo.fromJson(xhrArgs.xhr.responseText).Data.errorMessage+"</div>";
		}
		var message = new com.ibm.bpm.social.Aether({
			'message': "<div>"+this.changeDueDateDialogMessages.changeDueDateError+"</div>"+errMsg,
			'containerDivId':"messageNotificationDiv",
			'position': 'top-right',
			'fade': false,
			'parent': this,
			'displayCloseIcon': true
		});
	},
			
	destroy: function() {
		while (this.connects && this.connects.length > 1) {
	          dojo.disconnect(this.connects.pop());
	    }
		
		if(this.dateTimePicker!=null && this.dateTimePicker.destroy){
			this.dateTimePicker.destroy();
			this.dateTimePicker = null;
		}
		
		if(this.popup && this.popup.destroy){
			this.popup.destroy();	
		}
		
		this.inherited(arguments);
	},

	/*
	 * This method is used to destroy this.popup (our dijit.Dialog) after it's animation has finished.
	 * This is only invoked when the 'X' or Cancel button is pressed.
	 */
	delayedDestroy: function(){
		setTimeout(dojo.hitch(this, this.destroy), 1000);
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog");









dojo.declare("com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog", com.ibm.bspace.common.util.ui.FullScreenDialog, {

	templateString: "<div dojoAttachPoint='content'><iframe dojoAttachPoint='frame' src='#' /></div>",
	
	instanceId: null,
	
	instanceName: null,
	
	templateName: null,
	
	auditTrailUrl: window.endpointURLObject.processPortalSupport +"/jsp/processAudit.do?bpdInstanceId={iid}",

	proxyUrl: "",
	
	postMixInProperties: function(){
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.dialogParams.title = this.messages.auditHistoryWindowTitle.replace("${name}", this.templateName);
		
		this.dialogParams.subTitle = this.instanceName;
		
		this.dialogParams.closeButtonTooltip = this.messages.closeAuditHistoryTooltip;
		
		this.inherited(arguments);
	},
		
	show: function(){
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);

        this.frame.setAttribute('frameborder', '0');
		
		this.frame.src = this.proxyUrl +  this.auditTrailUrl.replace("{iid}", this.instanceId);
        this.frame.title =  this.dialogParams.title;//a11y
		
		this.inherited(arguments);
		
		this._sizeContentPane();
		
		dojo.style(this.frame, "width", "100%");
		dojo.style(this.frame, "margin", "0");
		dojo.style(this.frame, "padding", "0");
		dojo.style(this.frame, "position", "absolute");
		dojo.style(this.frame, "top", "0");
		dojo.style(this.frame, "bottom", "0");
		dojo.style(this.frame, "left", "0");
		dojo.style(this.frame, "right", "0");
		
		dojo.connect(this.popup, 'onHide', this, this.delayedDestroy);
		
		dojo.connect(window, 'onresize', this, this._sizeContentPane);

        dojo.connect(this.frame.contentWindow, "onkeypress", this, this._close);//accessibility

        setTimeout(this.frame.contentWindow.focus,1000);//a11y    
      },

      _close: function(e) {
           if (e.charOrCode == dojo.keys.ESCAPE) {
               this.popup.onCancel()
           }
      },
	
	_sizeContentPane: function(){
		var h = 0, w = 0;
		if(dojo.isIE){
			h = document.documentElement.offsetHeight;
			w = document.documentElement.offsetWidth;
		} else {
			h = window.innerHeight;
			w = window.innerWidth;
		}
        if (this.popup && this.popup.domNode) {
		    dojo.query(".dijitDialogPaneContent", this.popup.domNode).style("height", (h-50)+"px").style("width", w+"px"); //50px header height
        }

		if(dojo.isIE < 9){
			dojo.query(".fullScreenDialog").style("left", "0").style("top", "0").style("overflow", "hidden");
		}
		
		if(this.frame){
			dojo.style(this.frame, "height", (h-50)+"px");
            if (dojo.isFF > 19) {
                dojo.style(this.frame, "overflow", "hidden");
            }
		}
	},
	
	/*
	 * This method is used to destroy this.popup (our dijit.Dialog) after it's animation has finished.
	 * This is only invoked when the 'X' or Cancel button is pressed.
	 */
	delayedDestroy: function(){
		setTimeout(dojo.hitch(this, this.destroy), 1000);
	},

    destroy: function(){
        dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
        this.inherited(arguments);
    }
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog");









dojo.declare("com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog", com.ibm.bspace.common.util.ui.FullScreenDialog, {

  templateString: "<div dojoAttachPoint='content'><iframe dojoAttachPoint='frame' src='#'/></div>",

  instanceId: null,

  instanceName: null,

  templateName: null,

  criticalPathUrl: window.endpointURLObject.processPortalSupport +"/jsp/criticalPath.do?cpType=2&bpdInstanceId={iid}",

  proxyUrl: "",

  postMixInProperties: function(){
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    this.dialogParams.title = this.messages.criticalPathManagementWindowTitle.replace("${name}", this.templateName);

    this.dialogParams.subTitle = this.instanceName;

    this.dialogParams.closeButtonTooltip = this.messages.closeCriticalPathManagementTooltip;

    this.inherited(arguments);
  },

  show: function(){
    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);

    this.frame.setAttribute('frameborder', '0');

    this.frame.src = this.proxyUrl + this.criticalPathUrl.replace("{iid}", this.instanceId);
    this.frame.title =  this.dialogParams.title;//a11y

    this.inherited(arguments);

    this._sizeContentPane();

    dojo.style(this.frame, "width", "100%");
    dojo.style(this.frame, "margin", "0");
    dojo.style(this.frame, "padding", "0");
    dojo.style(this.frame, "position", "absolute");
    dojo.style(this.frame, "top", "0");
    dojo.style(this.frame, "bottom", "0");
    dojo.style(this.frame, "left", "0");
    dojo.style(this.frame, "right", "0");

    this.popup.closeButtonNode.tabIndex="0";

    dojo.connect(this.popup, 'onHide', this, this.delayedDestroy);

    dojo.connect(window, 'onresize', this, this._sizeContentPane);

    dojo.connect(this.frame.contentWindow, "onkeypress", this, this._close);//accessibility
    
    setTimeout(this.frame.contentWindow.focus,1000);//a11y
    
    this.tmpHistory = null;
	if (dojo.isIE < 9) {
	  	this.tmpHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	 	com.ibm.bpm.social.util.common.Util._portalHashHistory = new Array();
	}
	
  },

  _close: function(e) {
       if (e.charOrCode == dojo.keys.ESCAPE) {
           this.popup.onCancel()
       }
  },

  _sizeContentPane: function(){
    var h = 0, w = 0;
    if (dojo.isIE) {
      h = document.documentElement.offsetHeight;
      w = document.documentElement.offsetWidth;
    }
    else {
      h = window.innerHeight;
      w = window.innerWidth;
    }
    dojo.query(".dijitDialogPaneContent", this.popup.domNode).style("height", (h - 50) + "px").style("width", w + "px"); //50px header height

    if (dojo.isIE < 9) {
      dojo.query(".fullScreenDialog").style("left", "0").style("top", "0").style("overflow", "hidden");
    }

    if (this.frame) {
      dojo.style(this.frame, "height", (h - 50) + "px");
    }
  },

  /*
   * This method is used to destroy this.popup (our dijit.Dialog) after it's animation has finished.
   * This is only invoked when the 'X' or Cancel button is pressed.
   */
  delayedDestroy: function(){

    dojo.publish(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED + "." + this.instanceId, [{}]);
    dojo.publish(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED, [{}]);

    setTimeout(dojo.hitch(this, this.destroy), 1000);
  },

  destroy: function(){
    dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
    if (this.tmpHistory) {
		var h = this.tmpHistory.pop();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = this.tmpHistory;
		this.tmpHistory = null;
		com.ibm.bpm.social.util.common.Util.setCurrentHash(h);
	}
    this.inherited(arguments);
  }


});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Constants"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Constants"] = true;
// Constants.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2009, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT


// **************************************
//
// Version: 1.30 11/10/18 13:26:25
//
// **************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.Constants");




//config version and widget version
com.ibm.bpm.social.widgets.task.util.Constants.WIDGETVERSION = "7.5.1.0";
com.ibm.bpm.social.widgets.task.util.Constants.CONFIGVERSION = "7.5.1.0";

// Global configuration constants for List Configuration
com.ibm.bpm.social.widgets.task.util.Constants.TITLE_CONFIG = "displayName";
com.ibm.bpm.social.widgets.task.util.Constants.TITLEKEY_CONFIG = "titleKey";
com.ibm.bpm.social.widgets.task.util.Constants.ITEM_CONFIG = "item";
com.ibm.bpm.social.widgets.task.util.Constants.ORDER_BY_CONFIG = "orderBy";
com.ibm.bpm.social.widgets.task.util.Constants.WHERE_CLAUSE_CONFIG = "whereClause";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_CONFIG = "property";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_DISPLAYED_CONFIG = "displayed";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_LABELDISPLAYED_CONFIG = "labelDisplayed";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTIES_CONFIG = "properties";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_CONFIG = "type";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXT_CONFIG = "text";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_DATE_CONFIG = "date";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_NUMBER_CONFIG = "number";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_DECIMAL_CONFIG = "float";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG = "priority";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_STATUS_CONFIG = "status";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXTCHECK_CONFIG = "textcheck";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_ID_CONFIG = "id";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PENDING_CONFIG = "pending";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_BOOLEAN = "boolean";
com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER = "user";
com.ibm.bpm.social.widgets.task.util.Constants.PRIORITIES_CONFIG = "priorities";
com.ibm.bpm.social.widgets.task.util.Constants.MULTIPLE_SELECTION = "multipleSelection";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG = "high";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_TITLEKEY_CONFIG = "highTitleKey";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CSSLCASS_CONFIG = "highCssClass";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_TITLEKEY_CONFIG = "mediumTitleKey";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_CSSCLASS_CONFIG = "mediumCssClass";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG = "low";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_TITLEKEY_CONFIG = "lowTitleKey";
com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CSSLCASS_CONFIG = "lowCssClass";
com.ibm.bpm.social.widgets.task.util.Constants.FILTER_CONFIG = "filterTemplate";
com.ibm.bpm.social.widgets.task.util.Constants.LIST_PROPERTIES = "listProperties";
com.ibm.bpm.social.widgets.task.util.Constants.QUEUE_PROPERTIES_MAP = "queuePropertiesMap";
com.ibm.bpm.social.widgets.task.util.Constants.LIST_TEMPLATE_WHERE_CLAUSE = "templateWhereClause";
com.ibm.bpm.social.widgets.task.util.Constants.ACTIVE_USER_ROLE = "activeUserRole";
com.ibm.bpm.social.widgets.task.util.Constants.DEFAULT_USER_ROLE = "defaultUserRole";
com.ibm.bpm.social.widgets.task.util.Constants.DEFAULT_GROUP = "defaultGroup";
com.ibm.bpm.social.widgets.task.util.Constants.DISABLE_GROUPING = "disableGrouping";
com.ibm.bpm.social.widgets.task.util.Constants.USER_ROLES = "userRoles";
com.ibm.bpm.social.widgets.task.util.Constants.FILTER_TYPE_MAPPING = "pushFilterTypeMapping";
com.ibm.bpm.social.widgets.task.util.Constants.FILTERS = "filters";
com.ibm.bpm.social.widgets.task.util.Constants.DEFAULT_QUEUE = "defaultQueue";
com.ibm.bpm.social.widgets.task.util.Constants.ACTIVE_QUEUE = "activeQueue";
com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE = "com.ibm.task.Task";
com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE = "com.ibm.task.TaskTemplate";
com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE = "com.ibm.bfm.Process";
com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE = "com.ibm.bfm.ProcessTemplate";
com.ibm.bpm.social.widgets.task.util.Constants.ESCALATION_TYPE = "com.ibm.task.Escalation";
com.ibm.bpm.social.widgets.task.util.Constants.NO_ITEMS_FOUND = "noItemsFoundKey";
com.ibm.bpm.social.widgets.task.util.Constants.NO_ITEMS_FOUND_WITH_FILTER_POSTFIX = "_Filter";
com.ibm.bpm.social.widgets.task.util.Constants.VIEW_TYPE = "viewType";
com.ibm.bpm.social.widgets.task.util.Constants.VIEW_TYPE_TABLE = "table";
com.ibm.bpm.social.widgets.task.util.Constants.VIEW_TYPE_LIST = "list";
com.ibm.bpm.social.widgets.task.util.Constants.VIEW_TYPE_TREE = "tree";
com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE = "navigationType";
com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_PAGING = "paging";
com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_SCROLLING = "scrolling";

com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE = "com.ibm.task.WorkBasket";
com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE = "com.ibm.task.BusinessCategory";

//type of the work basket
com.ibm.bpm.social.widgets.task.util.Constants.WB_INFO_TAB_GENERAL_TYPE_OPTION1 = "TYPE_INDIVIDUAL";
com.ibm.bpm.social.widgets.task.util.Constants.WB_INFO_TAB_GENERAL_TYPE_OPTION2 = "TYPE_GROUP";
com.ibm.bpm.social.widgets.task.util.Constants.WB_INFO_TAB_GENERAL_TYPE_OPTION3 = "TYPE_CLEARANCE";
com.ibm.bpm.social.widgets.task.util.Constants.WB_INFO_TAB_GENERAL_TYPE_OPTION4 = "TYPE_TOPIC";

//actions performed on the work basket
com.ibm.bpm.social.widgets.task.util.Constants.ACTION_CREATE = "create";
com.ibm.bpm.social.widgets.task.util.Constants.ACTION_EDIT = "edit";
com.ibm.bpm.social.widgets.task.util.Constants.ACTION_DUPLICATE = "copy";

//name of the default QT of the work basket
com.ibm.bpm.social.widgets.task.util.Constants.WB_DEFAULT_QT = "IBM.DEFAULTALLWORKBASKETSLST";
//name of the default QT of the business category
com.ibm.bpm.social.widgets.task.util.Constants.BC_DEFAULT_QT = "IBM.DEFAULTALLBUSCATSLIST";

com.ibm.bpm.social.widgets.task.util.Constants.TYPE_WORK_BASKET_INFO = "TYPE.WORKBASKET.INFO";
com.ibm.bpm.social.widgets.task.util.Constants.TYPE_BUSINESS_CATEGORY_INFO = "TYPE.BUSINESSCATEGORY.INFO";

com.ibm.bpm.social.widgets.task.util.Constants.TYPE_WORK_BASKET_ENTITY = "TYPE.WORKBASKET.ENTITY";
com.ibm.bpm.social.widgets.task.util.Constants.TYPE_BUSINESS_CATEGORY_ENTITY = "TYPE.BUSINESSCATEGORY.ENTITY";

com.ibm.bpm.social.widgets.task.util.Constants.TASK_LIST_QT_REQU_ATTRIBUTES = "TASK.KIND,TASK.STATE";

com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORIES_DEFAULT_ASSIGNMENT_REASON = "REASON_READER";
com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORIES_DEFAULT_ASSIGNMENT_TYPE = "TYPE_EVERYBODY";

com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKETS_IDENTIFIER_DATABASE_NAME = "WORK_BASKET.WBID";

com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix = function(type){
  switch (type) {
    case com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE :
      return "__ProcessDefinition";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE :
      return "__Process";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE :
      return "__TaskDefinition";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.ESCALATION_TYPE :
      return "__Escalation";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE :
      return "__WorkBasket";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE :
      return "__BusinessCategory";
      break;
    case com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE :
    default:
      return "__Task";
  }
  return "__Task";
};

com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper = function(value){
  var key = null;
  if (value < 15) {
    key = "VeryHigh";
  }
  else if (15 <= value && value < 25) {
    key = "High";
  }
  else if (25 <= value && value < 35) {
    key = "Medium";
  }
  else if (35 <= value && value < 45) {
    key = "Low";
  }
  else if (45 <= value) {
    key = "VeryLow";
  }
  return key;
};

com.ibm.bpm.social.widgets.task.util.Constants.propertyValueMapper = function(key){
  var range = {};
  if (key == "VeryHigh") {
    range = {
      high: 15
    };
  }
  else if (key == "High") {
    range = {
      low: 15,
      high: 25
    };
  }
  else if (key == "Medium") {
    range = {
      low: 25,
      high: 35
    };
  }
  else if (key == "Low") {
    range = {
      low: 35,
      high: 45
    };
  }
  else if (key == "VeryLow") {
    range = {
      low: 45
    };
  }
  return range;
};

com.ibm.bpm.social.widgets.task.util.Constants.PRIORITY_MIN = 0;
com.ibm.bpm.social.widgets.task.util.Constants.PRIORITY_MAX = 2147483647;

com.ibm.bpm.social.widgets.task.util.Constants.getPropertyEnumerations = function(propertyType, systems, domain){
  var rc = null;

  var isWpsOnly = false;
  var isWleOnly = false;
  if (systems) {
    isWpsOnly = systems.isWpsOnly(domain);
    isWleOnly = systems.isWleOnly(domain);
  }

  switch (propertyType) {
    case "TASK.PRIORITY" :
      rc = []; // Not sure about this?
      break;
    case "TASK.STATE" :
      // add state properties depending on backend, keep order of states in "lifecycle"
      //Duplicate values ["STATE_RUNNING", "STATE_CLAIMED"], 
      rc = ["STATE_INACTIVE",
            "STATE_READY",
            "STATE_RUNNING"];
      if (isWleOnly) {
        rc = rc.concat(["STATE_SUSPENDED_BY_PARENT"]);
      }
      rc = rc.concat(["STATE_FINISHED"]);
      if (isWpsOnly) {
        //Duplicate values ["STATE_FAILED", "STATE_EXPIRED"]
        rc = rc.concat(["STATE_FAILED"]);
      }
      rc = rc.concat(["STATE_TERMINATED"]);
      if (isWpsOnly) {
        rc = rc.concat(["STATE_FORWARDED"]);
      }
      break;
    case "TASK.KIND" :
      rc = ["KIND_HUMAN",
            "KIND_ORIGINATING",
            "KIND_PARTICIPATING",
            "KIND_ADMINISTRATIVE"];
      break;
    case "PROCESS_INSTANCE.STATE" :
      // Duplicate values ["STATE_FAILED","STATE_FAILING","STATE_TERMINATED","STATE_TERMINATING"], ["STATE_COMPENSATING","STATE_COMPENSATED"]
      // Unsupported values in WLE ["STATE_TERMINATING", "STATE_COMPENSATED", "STATE_COMPENSATING", "STATE_FAILING", "STATE_COMPENSATION_FAILED"]
      rc = ["STATE_RUNNING",
            "STATE_FINISHED"];
      if (isWpsOnly) {
        rc = rc.concat(["STATE_COMPENSATING"]);
      }
      rc = rc.concat(["STATE_FAILED",
                      "STATE_SUSPENDED"]);
      if (isWpsOnly) {
        rc = rc.concat(["STATE_COMPENSATION_FAILED"]);
      }
      break;
    case "PROCESS_TEMPLATE.EXECUTION_MODE" :
      rc = ["EXECUTION_MODE_MICROFLOW",
            "EXECUTION_MODE_LONG_RUNNING"];
      break;
    case "PROCESS_TEMPLATE.STATE" :
      rc = ["STATE_STARTED",
            "STATE_STOPPED"];
      break;
    case "TASK_TEMPL.KIND" :
      rc = ["KIND_HUMAN",
            "KIND_ORIGINATING",
            "KIND_PARTICIPATING",
            "KIND_ADMINISTRATIVE"];
      break;
    case "TASK_TEMPL.STATE" :
      rc = ["STATE_STARTED",
            "STATE_STOPPED"];
      break;
    case "ESCALATION.ACTION" :
    case "ESC_TEMPL.ACTION" :
      rc = ["ACTION_CREATE_EVENT",
            "ACTION_CREATE_WORK_ITEM",
            "ACTION_SEND_EMAIL"];
      break;
    case "ESCALATION.STATE" :
      rc = ["STATE_INACTIVE",
            "STATE_WAITING",
            "STATE_ESCALATED",
            "STATE_SUPERFLUOUS"];
      break;
    case "ESCALATION.ACTIVATION_STATE" :
    case "ESC_TEMPL.ACTIVATION_STATE" :
      // Duplicate values ["ACTIVATION_STATE_RUNNING", "ACTIVATION_STATE_CLAIMED"],
      rc = ["ACTIVATION_STATE_READY",
            "ACTIVATION_STATE_RUNNING",
            "ACTIVATION_STATE_WAITING_FOR_SUBTASK"];
      break;
    case "ESCALATION.AT_LEAST_EXP_STATE" :
    case "ESC_TEMPL.AT_LEAST_EXP_STATE" :
      rc = ["AT_LEAST_EXPECTED_STATE_CLAIMED",
            "AT_LEAST_EXPECTED_STATE_ENDED",
            "AT_LEAST_EXPECTED_STATE_SUBTASKS_COMPLETED"];
      break;
    case "WORK_BASKET.TYPE" :
      return ["TYPE_GROUP",
              "TYPE_INDIVIDUAL",
              "TYPE_TOPIC",
              "TYPE_CLEARANCE"];
      break;
    default:
      rc = [];
      break;
  }

  return rc;
};

com.ibm.bpm.social.widgets.task.util.Constants.mapPropertyType = function(property, priority){
  var type;
  switch (property.content) {
    case "TASK.OWNER" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER;
      break;
    case "TASK.STARTER" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER;
      break;
    case "TASK.ORIGINATOR" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER;
      break;
    case "PROCESS_INSTANCE.STARTER" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER;
      break;
    case "TASK.PRIORITY" :
      if (priority) {
        if (priority.type == "defaultPriority") {
          type = "defaultPriority";
        }
        else if (priority.type == "priority") {
          type = "priority";
        }
        else {
          type = "numericPriority";
        }
      }
      else {
        type = "numericPriority";
      }
      break;
    case "TASK_DESC.DESCRIPTION" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXTCHECK_CONFIG;
      break;
    case "TASK.IS_WAIT_FOR_SUB_TK" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PENDING_CONFIG;
      break;
    case "TASK.IS_ESCALATED" :
      type = "TASK.IS_ESCALATED";
      break;
    case "TASK.STATE" :
      type = "TASK.STATE";
      break;
    case "TASK.KIND" :
      type = "TASK.KIND";
      break;
    case "PROCESS_INSTANCE.STATE" :
      type = "PROCESS_INSTANCE.STATE";
      break;
    case "PROCESS_TEMPLATE.STATE" :
      type = "PROCESS_TEMPLATE.STATE";
      break;
    case "PROCESS_TEMPLATE.EXECUTION_MODE" :
      type = "PROCESS_TEMPLATE.EXECUTION_MODE";
      break;
    case "TASK_TEMPL.KIND" :
      type = "TASK_TEMPL.KIND";
      break; // Use keys from TASK
    case "TASK_TEMPL.STATE" :
      type = "TASK_TEMPL.STATE";
      break;
    case "TASK_TEMPL_DESC.DESCRIPTION" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXTCHECK_CONFIG;
      break;
    case "ESCALATION.ACTION" :
      type = "ESCALATION.ACTION";
      break;
    case "ESCALATION.STATE" :
      type = "ESCALATION.STATE";
      break;
    case "ESCALATION.ACTIVATION_STATE" :
      type = "ESCALATION.ACTIVATION_STATE";
      break;
    case "ESCALATION.AT_LEAST_EXP_STATE" :
      type = "ESCALATION.AT_LEAST_EXP_STATE";
      break;
    case "ESCALATION_DESC.DESCRIPTION" :
      type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXTCHECK_CONFIG;
      break;
    case "ESC_TEMPL.ACTION" :
      type = "ESC_TEMPL.ACTION";
      break; // Use keys form ESCALATION
    case "ESC_TEMPL.ACTIVATION_STATE" :
      type = "ESC_TEMPL.ACTIVATION_STATE";
      break; // Use keys form ESCALATION
    case "ESC_TEMPL.AT_LEAST_EXP_STATE" :
      type = "ESC_TEMPL.AT_LEAST_EXP_STATE";
      break; // Use keys form ESCALATION
    case "WORK_BASKET.TYPE" :
      type = "WORK_BASKET.TYPE";
      break;
    case "WORK_BASKET.OWNER" :
      type = "usergroup";
      break;
    default:
      switch (property.type) {
        case "TIMESTAMP" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_DATE_CONFIG;
          break;
        case "NUMERIC" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_NUMBER_CONFIG;
          break;
        case "NUMBER" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_NUMBER_CONFIG;
          break;
        case "DECIMAL" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_DECIMAL_CONFIG;
          break;
        case "ID" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_ID_CONFIG;
          break;
        case "BOOLEAN" :
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_BOOLEAN;
          break;
        default:
          type = com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXT_CONFIG;
          break;
      }
  }
  return type;
};

com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString = function(date){
  // create the iso date string in gmt
  var dateString = dojo.date.stamp.toISOString(date, {
    zulu: "true"
  });
  // but the dojo function returns a Z at the end, which
  // is not allowed for the sql select statement
  var pos = dateString.lastIndexOf("Z");
  if (pos > 0) {
    dateString = dateString.substring(0, pos);
  }
  return dateString;
};

//com.ibm.bpm.social.widgets.task.util.Constants.drawAdminDomainsGUI =
//  function(parentNode, idSuffix, labelStyle, controlStyle){
//    var adminDomainTxtBoxID = idSuffix + "~_AdminDomainTextfield_~";
//
//    var nlsResources = dojo.i18n.getLocalization("com.ibm.bpc.widget.bcm", "BusinessCategoryInfo");
//
//    // === admin domain label ===  
//    var adminDomainLabel = document.createElement("label");
//    adminDomainLabel.type = "text";
//    adminDomainLabel.htmlFor = adminDomainTxtBoxID;
//    adminDomainLabel.innerHTML = nlsResources["KEY_CONFIG_BC_GENERAL_ADMDOM"];
//
//    if (labelStyle) {
//      dojo.mixin(adminDomainLabel.style, labelStyle);
//    }
//
//    dijit.setWaiRole(adminDomainLabel, "label");
//    parentNode.appendChild(adminDomainLabel);
//
//    var adminDomainDiv = document.createElement("div");
//    dojo.addClass(adminDomainDiv, "tabContent workBasketInputField");
//
//    if (controlStyle) {
//      dojo.mixin(adminDomainDiv.style, controlStyle);
//    }
//
//    // === admin domain validation textfield ===
//    //set the parameter used for creating the validation text box
//    var parameter =
//      {
//        id: adminDomainTxtBoxID,
//        //required: true, 
//        invalidMessage: nlsResources.KEY_EDIT_INVALID_VALUE_MSG,
//        regExp: "([a-zA-Z_\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][a-zA-Z_.0-9�`\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\u00B7\\u0300-\\u036F\\u203F-\\u2040-]{0,253})",
//        maxLength: 254
//      };
//
//    var adminDomainTxtBox = new dijit.form.ValidationTextBox(parameter);
//    adminDomainTxtBox.domNode.style.marginTop = "-3px";
//    adminDomainTxtBox.domNode.style.marginLeft = "20px";
//    adminDomainTxtBox.domNode.style.width = "214px";
//
//    adminDomainDiv.appendChild(adminDomainTxtBox.domNode);
//    parentNode.appendChild(adminDomainDiv);
//    adminDomainTxtBox.startup();
//  };

}

if(!dojo._hasResource["dojo.data.util.sorter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.sorter"] = true;
dojo.provide("dojo.data.util.sorter");

dojo.getObject("data.util.sorter", true, dojo);

dojo.data.util.sorter.basicComparator = function(	/*anything*/ a,
													/*anything*/ b){
	//	summary:
	//		Basic comparision function that compares if an item is greater or less than another item
	//	description:
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.
	
	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0;
	}else if(a > b || a == null){
		r = 1;
	}
	return r; //int {-1,0,1}
};

dojo.data.util.sorter.createSortFunction = function(	/* attributes array */sortSpec,
														/*dojo.data.core.Read*/ store){
	//	summary:
	//		Helper function to generate the sorting function based off the list of sort attributes.
	//	description:
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	//
	//	sortSpec: array
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	//		{
	//			attribute: "attributeName-string" || attribute,
	//			descending: true|false;   // Default is false.
	//		}
	//	store: object
	//		The datastore object to look up item values from.
	//
	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = dojo.data.util.sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr,
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int
	}; // Function
};

}

if(!dojo._hasResource["dojo.data.util.simpleFetch"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.simpleFetch"] = true;
dojo.provide("dojo.data.util.simpleFetch");


dojo.getObject("data.util.simpleFetch", true, dojo);

dojo.data.util.simpleFetch.fetch = function(/* Object? */ request){
	//	summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems()
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems()
	//		method takes three arguments, the keywordArgs object that was passed
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters
	//		(such as includeChildren).  The _fetchItems() method should create an array of
	//		result items and pass it to the fetchHandler along with the original request object
	//		-- or, the _fetchItems() method may, if it wants to, create an new request object
	//		with other specifics about the request that are specific to the datastore and pass
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo.data.api.Read.fetch()
	request = request || {};
	if(!request.store){
		request.store = this;
	}
	var self = this;

	var _errorHandler = function(errorData, requestObject){
		if(requestObject.onError){
			var scope = requestObject.scope || dojo.global;
			requestObject.onError.call(scope, errorData, requestObject);
		}
	};

	var _fetchHandler = function(items, requestObject){
		var oldAbortFunction = requestObject.abort || null;
		var aborted = false;

		var startIndex = requestObject.start?requestObject.start:0;
		var endIndex = (requestObject.count && (requestObject.count !== Infinity))?(startIndex + requestObject.count):items.length;

		requestObject.abort = function(){
			aborted = true;
			if(oldAbortFunction){
				oldAbortFunction.call(requestObject);
			}
		};

		var scope = requestObject.scope || dojo.global;
		if(!requestObject.store){
			requestObject.store = self;
		}
		if(requestObject.onBegin){
			requestObject.onBegin.call(scope, items.length, requestObject);
		}
		if(requestObject.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
		}
		if(requestObject.onItem){
			for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
				var item = items[i];
				if(!aborted){
					requestObject.onItem.call(scope, item, requestObject);
				}
			}
		}
		if(requestObject.onComplete && !aborted){
			var subset = null;
			if(!requestObject.onItem){
				subset = items.slice(startIndex, endIndex);
			}
			requestObject.onComplete.call(scope, subset, requestObject);
		}
	};
	this._fetchItems(request, _fetchHandler, _errorHandler);
	return request;	// Object
};

}

if(!dojo._hasResource["dojo.data.util.filter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.filter"] = true;
dojo.provide("dojo.data.util.filter");

dojo.getObject("data.util.filter", true, dojo);

dojo.data.util.filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	//	summary:
	//		Helper function to convert a simple pattern to a regular expression for matching.
	//	description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:
	//			ca*   -> /^ca.*$/
	//			*ca*  -> /^.*ca.*$/
	//			*c\*a*  -> /^.*c\*a.*$/
	//			*c\*a?*  -> /^.*c\*a..*$/
	//			and so on.
	//
	//	pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//			* Means match anything, so ca* means match anything starting with ca
	//			? Means match single character.  So, b?b will match to bob and bab, and so on.
	//      	\ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//				To use a \ as a character in the string, it must be escaped.  So in the pattern it should be
	//				represented by \\ to be treated as an ordinary \ character instead of an escape.
	//
	//	ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}
	
};

}

if(!dojo._hasResource["dijit.form.ComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboBox"] = true;
dojo.provide("dijit.form.ComboBox");











dojo.declare(
	"dijit.form.ComboBoxMixin",
	dijit._HasDropDown,
	{
		// summary:
		//		Implements the base functionality for `dijit.form.ComboBox`/`dijit.form.FilteringSelect`
		// description:
		//		All widgets that mix in dijit.form.ComboBoxMixin must extend `dijit.form._FormValueWidget`.
		// tags:
		//		protected

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this ComboBox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: [const] Object
		//		Reference to data provider object used by this ComboBox
		store: null,

		// fetchProperties: Object
		//		Mixin to the dojo.data store's fetch.
		//		For example, to set the sort order of the ComboBox menu, pass:
		//	|	{ sort: [{attribute:"name",descending: true}] }
		//		To override the default queryOptions so that deep=false, do:
		//	|	{ queryOptions: {ignoreCase: true, deep: false} }
		fetchProperties:{},

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on `searchAttr` and the key.
		//		Any reference to the `searchAttr` is ignored.
		query: {},

		// autoComplete: Boolean
		//		If user types in a partial string, and then tab out of the `<input>` box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the `<input>` field
		autoComplete: true,

		// highlightMatch: String
		// 		One of: "first", "all" or "none".
		//
		//		If the ComboBox/FilteringSelect opens with the search results and the searched
		//		string can be found, it will be highlighted.  If set to "all"
		//		then will probably want to change `queryExpr` parameter to '*${0}*'
		//
		//		Highlighting is only performed when `labelType` is "text", so as to not
		//		interfere with any HTML markup an HTML label might contain.
		highlightMatch: "first",

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Search for items in the data store where this attribute (in the item)
		//		matches what the user typed
		searchAttr: "name",

		// labelAttr: String?
		//		The entries in the drop down list come from this attribute in the
		//		dojo.data items.
		//		If not specified, the searchAttr attribute is used instead.
		labelAttr: "",

		// labelType: String
		//		Specifies how to interpret the labelAttr in the data store items.
		//		Can be "html" or "text".
		labelType: "text",

		// queryExpr: String
		//		This specifies what query ComboBox/FilteringSelect sends to the data store,
		//		based on what the user has typed.  Changing this expression will modify
		//		whether the drop down shows only exact matches, a "starting with" match,
		//		etc.  Use it in conjunction with highlightMatch.
		//		dojo.data query expression pattern.
		//		`${0}` will be substituted for the user text.
		//		`*` is used for wildcards.
		//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
		queryExpr: "${0}*",

		// ignoreCase: Boolean
		//		Set true if the ComboBox/FilteringSelect should ignore case when matching possible items
		ignoreCase: true,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button, to display the drop down list.
		//		Defaults to true.
		hasDownArrow: true,

		templateString: dojo.cache("dijit.form", "templates/DropDownBox.html", "<div class=\"dijit dijitReset dijitInlineTable dijitLeft\"\n\tid=\"widget_${id}\"\n\trole=\"combobox\"\n\t><div class='dijitReset dijitRight dijitButtonNode dijitArrowButton dijitDownArrowButton dijitArrowButtonContainer'\n\t\tdojoAttachPoint=\"_buttonNode, _popupStateNode\" role=\"presentation\"\n\t\t><input class=\"dijitReset dijitInputField dijitArrowButtonInner\" value=\"&#9660; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t\t\t${_buttonInputDisabled}\n\t/></div\n\t><div class='dijitReset dijitValidationContainer'\n\t\t><input class=\"dijitReset dijitInputField dijitValidationIcon dijitValidationInner\" value=\"&#935; \" type=\"text\" tabIndex=\"-1\" readonly=\"readonly\" role=\"presentation\"\n\t/></div\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class='dijitReset dijitInputInner' ${!nameAttrSetting} type=\"text\" autocomplete=\"off\"\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" role=\"textbox\" aria-haspopup=\"true\"\n\t/></div\n></div>\n"),

		baseClass: "dijitTextBox dijitComboBox",

		// dropDownClass: [protected extension] String
		//		Name of the dropdown widget class used to select a date/time.
		//		Subclasses should specify this.
		dropDownClass: "dijit.form._ComboBoxMenu",

		// Set classes like dijitDownArrowButtonHover depending on
		// mouse action over button node
		cssStateNodes: {
			"_buttonNode": "dijitDownArrowButton"
		},

		// Flags to _HasDropDown to limit height of drop down to make it fit in viewport
		maxHeight: -1,

		// For backwards compatibility let onClick events propagate, even clicks on the down arrow button
		_stopClickEvents: false,

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			var pos = 0;
			if(typeof(element.selectionStart) == "number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				pos = element.selectionStart;
			}else if(dojo.isIE){
				// in the case of a mouse click in a popup being handled,
				// then the dojo.doc.selection is not the textarea, but the popup
				// var r = dojo.doc.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = dojo.doc.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesn't have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					pos = String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					// If focus has shifted, 0 is fine for caret pos.
				}
			}
			return pos;
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			dijit.selectInputText(element, location, location);
		},

		_setDisabledAttr: function(/*Boolean*/ value){
			// Additional code to set disabled state of ComboBox node.
			// Overrides _FormValueWidget._setDisabledAttr() or ValidationTextBox._setDisabledAttr().
			this.inherited(arguments);
			dijit.setWaiState(this.domNode, "disabled", value);
		},

		_abortQuery: function(){
			// stop in-progress query
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
				this.searchTimer = null;
			}
			if(this._fetchHandle){
				if(this._fetchHandle.abort){ this._fetchHandle.abort(); }
				this._fetchHandle = null;
			}
		},

		_onInput: function(/*Event*/ evt){
			// summary:
			//		Handles paste events
			if(!this.searchTimer && (evt.type == 'paste'/*IE|WebKit*/ || evt.type == 'input'/*Firefox*/) && this._lastInput != this.textbox.value){
				this.searchTimer = setTimeout(dojo.hitch(this, function(){
					this._onKey({charOrCode: 229}); // fake IME key to cause a search
				}), 100); // long delay that will probably be preempted by keyboard input
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles keyboard events

			var key = evt.charOrCode;

			// except for cutting/pasting case - ctrl + x/v
			if(evt.altKey || ((evt.ctrlKey || evt.metaKey) && (key != 'x' && key != 'v')) || key == dojo.keys.SHIFT){
				return; // throw out weird key combinations and spurious events
			}
			
			var doSearch = false;
			var pw = this.dropDown;
			var dk = dojo.keys;
			var highlighted = null;
			this._prev_key_backspace = false;
			this._abortQuery();

			// _HasDropDown will do some of the work:
			//		1. when drop down is not yet shown:
			//			- if user presses the down arrow key, call loadDropDown()
			//		2. when drop down is already displayed:
			//			- on ESC key, call closeDropDown()
			//			- otherwise, call dropDown.handleKey() to process the keystroke
			this.inherited(arguments);

			if(this._opened){
				highlighted = pw.getHighlightedOption();
			}
			switch(key){
				case dk.PAGE_DOWN:
				case dk.DOWN_ARROW:
				case dk.PAGE_UP:
				case dk.UP_ARROW:
					// Keystroke caused ComboBox_menu to move to a different item.
					// Copy new item to <input> box.
					if(this._opened){
						this._announceOption(highlighted);
					}
					dojo.stopEvent(evt);
					break;

				case dk.ENTER:
					// prevent submitting form if user presses enter. Also
					// prevent accepting the value if either Next or Previous
					// are selected
					if(highlighted){
						// only stop event on prev/next
						if(highlighted == pw.nextButton){
							this._nextSearch(1);
							dojo.stopEvent(evt);
							break;
						}else if(highlighted == pw.previousButton){
							this._nextSearch(-1);
							dojo.stopEvent(evt);
							break;
						}
					}else{
						// Update 'value' (ex: KY) according to currently displayed text
						this._setBlurValue(); // set value if needed
						this._setCaretPos(this.focusNode, this.focusNode.value.length); // move cursor to end and cancel highlighting
					}
					// default case:
					// if enter pressed while drop down is open, or for FilteringSelect,
					// if we are in the middle of a query to convert a directly typed in value to an item,
					// prevent submit
					if(this._opened || this._fetchHandle){
						dojo.stopEvent(evt);
					}
					// fall through

				case dk.TAB:
					var newvalue = this.get('displayedValue');
					//	if the user had More Choices selected fall into the
					//	_onBlur handler
					if(pw && (
						newvalue == pw._messages["previousMessage"] ||
						newvalue == pw._messages["nextMessage"])
					){
						break;
					}
					if(highlighted){
						this._selectOption();
					}
					if(this._opened){
						this._lastQuery = null; // in case results come back later
						this.closeDropDown();
					}
					break;

				case ' ':
					if(highlighted){
						// user is effectively clicking a choice in the drop down menu
						dojo.stopEvent(evt);
						this._selectOption();
						this.closeDropDown();
					}else{
						// user typed a space into the input box, treat as normal character
						doSearch = true;
					}
					break;

				case dk.DELETE:
				case dk.BACKSPACE:
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				default:
					// Non char keys (F1-F12 etc..)  shouldn't open list.
					// Ascii characters and IME input (Chinese, Japanese etc.) should.
					//IME input produces keycode == 229.
					doSearch = typeof key == 'string' || key == 229;
			}
			if(doSearch){
				// need to wait a tad before start search so that the event
				// bubbles through DOM and we have value visible
				this.item = undefined; // undefined means item needs to be set
				this.searchTimer = setTimeout(dojo.hitch(this, "_startSearchFromInput"),1);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// 		Fill in the textbox with the first item from the drop down
			// 		list, and highlight the characters that were
			// 		auto-completed. For example, if user typed "CA" and the
			// 		drop down list appeared, the textbox would be changed to
			// 		"California" and "ifornia" would be highlighted.

			var fn = this.focusNode;

			// IE7: clear selection so next highlight works all the time
			dijit.selectInputText(fn, fn.value.length);
			// does text autoComplete the value in the textbox?
			var caseFilter = this.ignoreCase? 'toLowerCase' : 'substr';
			if(text[caseFilter](0).indexOf(this.focusNode.value[caseFilter](0)) == 0){
				var cpos = this._getCaretPos(fn);
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > fn.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					fn.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					dijit.selectInputText(fn, cpos);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				fn.value = text;
				dijit.selectInputText(fn);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			// summary:
			//		Callback when a search completes.
			// description:
			//		1. generates drop-down list and calls _showResultList() to display it
			//		2. if this result list is from user pressing "more choices"/"previous choices"
			//			then tell screen reader to announce new option
			this._fetchHandle = null;
			if(	this.disabled ||
				this.readOnly ||
				(dataObject.query[this.searchAttr] != this._lastQuery)
			){
				return;
			}
			var wasSelected = this.dropDown._highlighted_option && dojo.hasClass(this.dropDown._highlighted_option, "dijitMenuItemSelected");
			this.dropDown.clearResultList();
			if(!results.length && !this._maxOptions){ // if no results and not just the previous choices button
				this.closeDropDown();
				return;
			}

			// Fill in the textbox with the first item from the drop down list,
			// and highlight the characters that were auto-completed. For
			// example, if user typed "CA" and the drop down list appeared, the
			// textbox would be changed to "California" and "ifornia" would be
			// highlighted.

			dataObject._maxOptions = this._maxOptions;
			var nodes = this.dropDown.createOptions(
				results,
				dataObject,
				dojo.hitch(this, "_getMenuLabelFromItem")
			);

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091:
			//		tell the screen reader that the paging callback finished by
			//		shouting the next choice
			if(dataObject.direction){
				if(1 == dataObject.direction){
					this.dropDown.highlightFirstOption();
				}else if(-1 == dataObject.direction){
					this.dropDown.highlightLastOption();
				}
				if(wasSelected){
					this._announceOption(this.dropDown.getHighlightedOption());
				}
			}else if(this.autoComplete && !this._prev_key_backspace
				// when the user clicks the arrow button to show the full list,
				// startSearch looks for "*".
				// it does not make sense to autocomplete
				// if they are just previewing the options available.
				&& !/^[*]+$/.test(dataObject.query[this.searchAttr])){
					this._announceOption(nodes[1]); // 1st real item
			}
		},

		_showResultList: function(){
			// summary:
			//		Display the drop down if not already displayed, or if it is displayed, then
			//		reposition it if necessary (reposition may be necessary if drop down's height changed).

			this.closeDropDown(true);

			// hide the tooltip
			this.displayMessage("");

			this.openDropDown();

			dijit.setWaiState(this.domNode, "expanded", "true");
		},

		loadDropDown: function(/*Function*/ callback){
			// Overrides _HasDropDown.loadDropDown().
			// This is called when user has pressed button icon or pressed the down arrow key
			// to open the drop down.
			
			this._startSearchAll();
		},

		isLoaded: function(){
			// signal to _HasDropDown that it needs to call loadDropDown() to load the
			// drop down asynchronously before displaying it
			return false;
		},

		closeDropDown: function(){
			// Overrides _HasDropDown.closeDropDown().  Closes the drop down (assuming that it's open).
			// This method is the callback when the user types ESC or clicking
			// the button icon while the drop down is open.  It's also called by other code.
			this._abortQuery();
			if(this._opened){
				this.inherited(arguments);
				dijit.setWaiState(this.domNode, "expanded", "false");
				dijit.removeWaiState(this.focusNode,"activedescendant");
			}
		},

		_setBlurValue: function(){
			// if the user clicks away from the textbox OR tabs away, set the
			// value to the textbox value
			// #4617:
			//		if value is now more choices or previous choices, revert
			//		the value
			var newvalue = this.get('displayedValue');
			var pw = this.dropDown;
			if(pw && (
				newvalue == pw._messages["previousMessage"] ||
				newvalue == pw._messages["nextMessage"]
				)
			){
				this._setValueAttr(this._lastValueReported, true);
			}else if(typeof this.item == "undefined"){
				// Update 'value' (ex: KY) according to currently displayed text
				this.item = null;
				this.set('displayedValue', newvalue);
			}else{
				if(this.value != this._lastValueReported){
					dijit.form._FormValueWidget.prototype._setValueAttr.call(this, this.value, true);
				}
				this._refreshState();
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's drop down
			this.closeDropDown();
			this.inherited(arguments);
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			if(!displayedValue){
				displayedValue = this.store.getValue(item, this.searchAttr);
			}
			var value = this._getValueField() != this.searchAttr? this.store.getIdentity(item) : displayedValue;
			this._set("item", item);
			dijit.form.ComboBox.superclass._setValueAttr.call(this, value, priorityChange, displayedValue);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//		a11y code that puts the highlighted option in the textbox.
			//		This way screen readers will know what is happening in the
			//		menu.

			if(!node){
				return;
			}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node == this.dropDown.nextButton ||
				node == this.dropDown.previousButton){
				newValue = node.innerHTML;
				this.item = undefined;
				this.value = '';
			}else{
				newValue = this.store.getValue(node.item, this.searchAttr).toString();
				this.set('item', node.item, false, newValue);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value = this.focusNode.value.substring(0, this._lastInput.length);
			// set up ARIA activedescendant
			dijit.setWaiState(this.focusNode, "activedescendant", dojo.attr(node, "id"));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*Event*/ evt){
			// summary:
			//		Menu callback function, called when an item in the menu is selected.
			if(evt){
				this._announceOption(evt.target);
			}
			this.closeDropDown();
			this._setCaretPos(this.focusNode, this.focusNode.value.length);
			dijit.form._FormValueWidget.prototype._setValueAttr.call(this, this.value, true); // set this.value and fire onChange
		},

		_startSearchAll: function(){
			this._startSearch('');
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value.replace(/([\\\*\?])/g, "\\$1"));
		},

		_getQueryString: function(/*String*/ text){
			return dojo.string.substitute(this.queryExpr, [text]);
		},

		_startSearch: function(/*String*/ key){
			// summary:
			//		Starts a search for elements matching key (key=="" means to return all items),
			//		and calls _openResultList() when the search completes, to display the results.
			if(!this.dropDown){
				var popupId = this.id + "_popup",
				dropDownConstructor = dojo.getObject(this.dropDownClass, false);
				this.dropDown = new dropDownConstructor({
					onChange: dojo.hitch(this, this._selectOption),
					id: popupId,
					dir: this.dir
				});
				dijit.removeWaiState(this.focusNode,"activedescendant");
				dijit.setWaiState(this.textbox,"owns",popupId); // associate popup with textbox
			}
			// create a new query to prevent accidentally querying for a hidden
			// value from FilteringSelect's keyField
			var query = dojo.clone(this.query); // #5970
			this._lastInput = key; // Store exactly what was entered by the user.
			this._lastQuery = query[this.searchAttr] = this._getQueryString(key);
			// #5970: set _lastQuery, *then* start the timeout
			// otherwise, if the user types and the last query returns before the timeout,
			// _lastQuery won't be set and their input gets rewritten
			this.searchTimer=setTimeout(dojo.hitch(this, function(query, _this){
				this.searchTimer = null;
				var fetch = {
					queryOptions: {
						ignoreCase: this.ignoreCase,
						deep: true
					},
					query: query,
					onBegin: dojo.hitch(this, "_setMaxOptions"),
					onComplete: dojo.hitch(this, "_openResultList"),
					onError: function(errText){
						_this._fetchHandle = null;
						console.error('dijit.form.ComboBox: ' + errText);
						_this.closeDropDown();
					},
					start: 0,
					count: this.pageSize
				};
				dojo.mixin(fetch, _this.fetchProperties);
				this._fetchHandle = _this.store.fetch(fetch);

				var nextSearch = function(dataObject, direction){
					dataObject.start += dataObject.count*direction;
					// #4091:
					//		tell callback the direction of the paging so the screen
					//		reader knows which menu option to shout
					dataObject.direction = direction;
					this._fetchHandle = this.store.fetch(dataObject);
					this.focus();
				};
				this._nextSearch = this.dropDown.onPage = dojo.hitch(this, nextSearch, this._fetchHandle);
			}, query, this), this.searchDelay);
		},

		_setMaxOptions: function(size, request){
			 this._maxOptions = size;
		},

		_getValueField: function(){
			// summary:
			//		Helper for postMixInProperties() to set this.value based on data inlined into the markup.
			//		Returns the attribute name in the item (in dijit.form._ComboBoxDataStore) to use as the value.
			return this.searchAttr;
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////

		constructor: function(){
			this.query={};
			this.fetchProperties={};
		},

		postMixInProperties: function(){
			if(!this.store){
				var srcNodeRef = this.srcNodeRef;

				// if user didn't specify store, then assume there are option tags
				this.store = new dijit.form._ComboBoxDataStore(srcNodeRef);

				// if there is no value set and there is an option list, set
				// the value to the first value to be consistent with native
				// Select

				// Firefox and Safari set value
				// IE6 and Opera set selectedIndex, which is automatically set
				// by the selected attribute of an option tag
				// IE6 does not set value, Opera sets value = selectedIndex
				if(!("value" in this.params)){
					var item = (this.item = this.store.fetchSelectedItem());
					if(item){
						var valueField = this._getValueField();
						this.value = this.store.getValue(item, valueField);
					}
				}
			}

			this.inherited(arguments);
		},

		postCreate: function(){
			// summary:
			//		Subclasses must call this method from their postCreate() methods
			// tags:
			//		protected

			// find any associated label element and add to ComboBox node.
			var label=dojo.query('label[for="'+this.id+'"]');
			if(label.length){
				label[0].id = (this.id+"_label");
				dijit.setWaiState(this.domNode, "labelledby", label[0].id);

			}
			this.inherited(arguments);
		},

		_setHasDownArrowAttr: function(val){
			this.hasDownArrow = val;
			this._buttonNode.style.display = val ? "" : "none";
		},

		_getMenuLabelFromItem: function(/*Item*/ item){
			var label = this.labelFunc(item, this.store),
				labelType = this.labelType;
			// If labelType is not "text" we don't want to screw any markup ot whatever.
			if(this.highlightMatch != "none" && this.labelType == "text" && this._lastInput){
				label = this.doHighlight(label, this._escapeHtml(this._lastInput));
				labelType = "html";
			}
			return {html: labelType == "html", label: label};
		},

		doHighlight: function(/*String*/ label, /*String*/ find){
			// summary:
			//		Highlights the string entered by the user in the menu.  By default this
			//		highlights the first occurrence found. Override this method
			//		to implement your custom highlighting.
			// tags:
			//		protected

			var
				// Add (g)lobal modifier when this.highlightMatch == "all" and (i)gnorecase when this.ignoreCase == true
				modifiers = (this.ignoreCase ? "i" : "") + (this.highlightMatch == "all" ? "g" : ""),
				i = this.queryExpr.indexOf("${0}");
			find = dojo.regexp.escapeString(find); // escape regexp special chars
			return this._escapeHtml(label).replace(
				// prepend ^ when this.queryExpr == "${0}*" and append $ when this.queryExpr == "*${0}"
				new RegExp((i == 0 ? "^" : "") + "("+ find +")" + (i == (this.queryExpr.length - 4) ? "$" : ""), modifiers),
				'<span class="dijitComboBoxHighlightMatch">$1</span>'
			); // returns String, (almost) valid HTML (entities encoded)
		},

		_escapeHtml: function(/*String*/ str){
			// TODO Should become dojo.html.entities(), when exists use instead
			// summary:
			//		Adds escape sequences for special characters in XML: &<>"'
			str = String(str).replace(/&/gm, "&amp;").replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
			return str; // string
		},

		reset: function(){
			// Overrides the _FormWidget.reset().
			// Additionally reset the .item (to clean up).
			this.item = null;
			this.inherited(arguments);
		},

		labelFunc: function(/*item*/ item, /*dojo.data.store*/ store){
			// summary:
			//		Computes the label to display based on the dojo.data store item.
			// returns:
			//		The label that the ComboBox should display
			// tags:
			//		private

			// Use toString() because XMLStore returns an XMLItem whereas this
			// method is expected to return a String (#9354)
			return store.getValue(item, this.labelAttr || this.searchAttr).toString(); // String
		}
	}
);

dojo.declare(
	"dijit.form._ComboBoxMenu",
	[dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
		// summary:
		//		Focus-less menu for internal use in `dijit.form.ComboBox`
		// tags:
		//		private

		templateString: "<ul class='dijitReset dijitMenu' dojoAttachEvent='onmousedown:_onMouseDown,onmouseup:_onMouseUp,onmouseover:_onMouseOver,onmouseout:_onMouseOut' style='overflow: \"auto\"; overflow-x: \"hidden\";'>"
				+"<li class='dijitMenuItem dijitMenuPreviousButton' dojoAttachPoint='previousButton' role='option'></li>"
				+"<li class='dijitMenuItem dijitMenuNextButton' dojoAttachPoint='nextButton' role='option'></li>"
			+"</ul>",

		// _messages: Object
		//		Holds "next" and "previous" text for paging buttons on drop down
		_messages: null,
		
		baseClass: "dijitComboBoxMenu",

		postMixInProperties: function(){
			this.inherited(arguments);
			this._messages = dojo.i18n.getLocalization("dijit.form", "ComboBox", this.lang);
		},

		buildRendering: function(){
			this.inherited(arguments);

			// fill in template with i18n messages
			this.previousButton.innerHTML = this._messages["previousMessage"];
			this.nextButton.innerHTML = this._messages["nextMessage"];
		},

		_setValueAttr: function(/*Object*/ value){
			this.value = value;
			this.onChange(value);
		},

		// stubs
		onChange: function(/*Object*/ value){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user clicked an option in the drop down menu.
			//		Probably should be called onSelect.
			// tags:
			//		callback
		},
		onPage: function(/*Number*/ direction){
			// summary:
			//		Notifies ComboBox/FilteringSelect that user clicked to advance to next/previous page.
			// tags:
			//		callback
		},

		onClose: function(){
			// summary:
			//		Callback from dijit.popup code to this widget, notifying it that it closed
			// tags:
			//		private
			this._blurOptionNode();
		},

		_createOption: function(/*Object*/ item, labelFunc){
			// summary:
			//		Creates an option to appear on the popup menu subclassed by
			//		`dijit.form.FilteringSelect`.

			var menuitem = dojo.create("li", {
				"class": "dijitReset dijitMenuItem" +(this.isLeftToRight() ? "" : " dijitMenuItemRtl"),
				role: "option"
			});
			var labelObject = labelFunc(item);
			if(labelObject.html){
				menuitem.innerHTML = labelObject.label;
			}else{
				menuitem.appendChild(
					dojo.doc.createTextNode(labelObject.label)
				);
			}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML == ""){
				menuitem.innerHTML = "&nbsp;";
			}
			menuitem.item=item;
			return menuitem;
		},

		createOptions: function(results, dataObject, labelFunc){
			// summary:
			//		Fills in the items in the drop down list
			// results:
			//		Array of dojo.data items
			// dataObject:
			//		dojo.data store
			// labelFunc:
			//		Function to produce a label in the drop down list from a dojo.data item

			//this._dataObject=dataObject;
			//this._dataObject.onComplete=dojo.hitch(comboBox, comboBox._openResultList);
			// display "Previous . . ." button
			this.previousButton.style.display = (dataObject.start == 0) ? "none" : "";
			dojo.attr(this.previousButton, "id", this.id + "_prev");
			// create options using _createOption function defined by parent
			// ComboBox (or FilteringSelect) class
			// #2309:
			//		iterate over cache nondestructively
			dojo.forEach(results, function(item, i){
				var menuitem = this._createOption(item, labelFunc);
				dojo.attr(menuitem, "id", this.id + i);
				this.domNode.insertBefore(menuitem, this.nextButton);
			}, this);
			// display "Next . . ." button
			var displayMore = false;
			//Try to determine if we should show 'more'...
			if(dataObject._maxOptions && dataObject._maxOptions != -1){
				if((dataObject.start + dataObject.count) < dataObject._maxOptions){
					displayMore = true;
				}else if((dataObject.start + dataObject.count) > dataObject._maxOptions && dataObject.count == results.length){
					//Weird return from a datastore, where a start + count > maxOptions
					// implies maxOptions isn't really valid and we have to go into faking it.
					//And more or less assume more if count == results.length
					displayMore = true;
				}
			}else if(dataObject.count == results.length){
				//Don't know the size, so we do the best we can based off count alone.
				//So, if we have an exact match to count, assume more.
				displayMore = true;
			}

			this.nextButton.style.display = displayMore ? "" : "none";
			dojo.attr(this.nextButton,"id", this.id + "_next");
			return this.domNode.childNodes;
		},

		clearResultList: function(){
			// summary:
			//		Clears the entries in the drop down list, but of course keeps the previous and next buttons.
			while(this.domNode.childNodes.length>2){
				this.domNode.removeChild(this.domNode.childNodes[this.domNode.childNodes.length-2]);
			}
			this._blurOptionNode();
		},

		_onMouseDown: function(/*Event*/ evt){
			dojo.stopEvent(evt);
		},

		_onMouseUp: function(/*Event*/ evt){
			if(evt.target === this.domNode || !this._highlighted_option){
				// !this._highlighted_option check to prevent immediate selection when menu appears on top
				// of <input>, see #9898.  Note that _HasDropDown also has code to prevent this.
				return;
			}else if(evt.target == this.previousButton){
				this._blurOptionNode();
				this.onPage(-1);
			}else if(evt.target == this.nextButton){
				this._blurOptionNode();
				this.onPage(1);
			}else{
				var tgt = evt.target;
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt = tgt.parentNode;
				}
				this._setValueAttr({ target: tgt }, true);
			}
		},

		_onMouseOver: function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			var tgt = evt.target;
			if(!(tgt == this.previousButton || tgt == this.nextButton)){
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt = tgt.parentNode;
				}
			}
			this._focusOptionNode(tgt);
		},

		_onMouseOut: function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			this._blurOptionNode();
		},

		_focusOptionNode: function(/*DomNode*/ node){
			// summary:
			//		Does the actual highlight.
			if(this._highlighted_option != node){
				this._blurOptionNode();
				this._highlighted_option = node;
				dojo.addClass(this._highlighted_option, "dijitMenuItemSelected");
			}
		},

		_blurOptionNode: function(){
			// summary:
			//		Removes highlight on highlighted option.
			if(this._highlighted_option){
				dojo.removeClass(this._highlighted_option, "dijitMenuItemSelected");
				this._highlighted_option = null;
			}
		},

		_highlightNextOption: function(){
			// summary:
			// 		Highlight the item just below the current selection.
			// 		If nothing selected, highlight first option.

			// because each press of a button clears the menu,
			// the highlighted option sometimes becomes detached from the menu!
			// test to see if the option has a parent to see if this is the case.
			if(!this.getHighlightedOption()){
				var fc = this.domNode.firstChild;
				this._focusOptionNode(fc.style.display == "none" ? fc.nextSibling : fc);
			}else{
				var ns = this._highlighted_option.nextSibling;
				if(ns && ns.style.display != "none"){
					this._focusOptionNode(ns);
				}else{
					this.highlightFirstOption();
				}
			}
			// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		highlightFirstOption: function(){
			// summary:
			// 		Highlight the first real item in the list (not Previous Choices).
			var first = this.domNode.firstChild;
			var second = first.nextSibling;
			this._focusOptionNode(second.style.display == "none" ? first : second); // remotely possible that Previous Choices is the only thing in the list
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		highlightLastOption: function(){
			// summary:
			// 		Highlight the last real item in the list (not More Choices).
			this._focusOptionNode(this.domNode.lastChild.previousSibling);
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		_highlightPrevOption: function(){
			// summary:
			// 		Highlight the item just above the current selection.
			// 		If nothing selected, highlight last option (if
			// 		you select Previous and try to keep scrolling up the list).
			if(!this.getHighlightedOption()){
				var lc = this.domNode.lastChild;
				this._focusOptionNode(lc.style.display == "none" ? lc.previousSibling : lc);
			}else{
				var ps = this._highlighted_option.previousSibling;
				if(ps && ps.style.display != "none"){
					this._focusOptionNode(ps);
				}else{
					this.highlightLastOption();
				}
			}
			dojo.window.scrollIntoView(this._highlighted_option);
		},

		_page: function(/*Boolean*/ up){
			// summary:
			//		Handles page-up and page-down keypresses

			var scrollamount = 0;
			var oldscroll = this.domNode.scrollTop;
			var height = dojo.style(this.domNode, "height");
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){
				this._highlightNextOption();
			}
			while(scrollamount<height){
				if(up){
					// stop at option 1
					if(!this.getHighlightedOption().previousSibling ||
						this._highlighted_option.previousSibling.style.display == "none"){
						break;
					}
					this._highlightPrevOption();
				}else{
					// stop at last option
					if(!this.getHighlightedOption().nextSibling ||
						this._highlighted_option.nextSibling.style.display == "none"){
						break;
					}
					this._highlightNextOption();
				}
				// going backwards
				var newscroll=this.domNode.scrollTop;
				scrollamount+=(newscroll-oldscroll)*(up ? -1:1);
				oldscroll=newscroll;
			}
		},

		pageUp: function(){
			// summary:
			//		Handles pageup keypress.
			//		TODO: just call _page directly from handleKey().
			// tags:
			//		private
			this._page(true);
		},

		pageDown: function(){
			// summary:
			//		Handles pagedown keypress.
			//		TODO: just call _page directly from handleKey().
			// tags:
			//		private
			this._page(false);
		},

		getHighlightedOption: function(){
			// summary:
			//		Returns the highlighted option.
			var ho = this._highlighted_option;
			return (ho && ho.parentNode) ? ho : null;
		},

		handleKey: function(evt){
			// summary:
			//		Handle keystroke event forwarded from ComboBox, returning false if it's
			//		a keystroke I recognize and process, true otherwise.
			switch(evt.charOrCode){
				case dojo.keys.DOWN_ARROW:
					this._highlightNextOption();
					return false;
				case dojo.keys.PAGE_DOWN:
					this.pageDown();
					return false;
				case dojo.keys.UP_ARROW:
					this._highlightPrevOption();
					return false;
				case dojo.keys.PAGE_UP:
					this.pageUp();
					return false;
				default:
					return true;
			}
		}
	}
);

dojo.declare(
	"dijit.form.ComboBox",
	[dijit.form.ValidationTextBox, dijit.form.ComboBoxMixin],
	{
		// summary:
		//		Auto-completing text box, and base class for dijit.form.FilteringSelect.
		//
		// description:
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//		If OPTION tags are used as the data provider via markup,
		//		then the OPTION tag's child text node is used as the widget value
		//		when selected.  The OPTION tag's value attribute is ignored.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			this._set("item", null); // value not looked up in store
			if(!value){ value = ''; } // null translates to blank
			dijit.form.ValidationTextBox.prototype._setValueAttr.call(this, value, priorityChange, displayedValue);
		}
	}
);

dojo.declare("dijit.form._ComboBoxDataStore", null, {
	// summary:
	//		Inefficient but small data store specialized for inlined `dijit.form.ComboBox` data
	//
	// description:
	//		Provides a store for inlined data like:
	//
	//	|	<select>
	//	|		<option value="AL">Alabama</option>
	//	|		...
	//
	//		Actually. just implements the subset of dojo.data.Read/Notification
	//		needed for ComboBox and FilteringSelect to work.
	//
	//		Note that an item is just a pointer to the <option> DomNode.

	constructor: function( /*DomNode*/ root){
		this.root = root;
		if(root.tagName != "SELECT" && root.firstChild){
			root = dojo.query("select", root);
			if(root.length > 0){ // SELECT is a child of srcNodeRef
				root = root[0];
			}else{ // no select, so create 1 to parent the option tags to define selectedIndex
				this.root.innerHTML = "<SELECT>"+this.root.innerHTML+"</SELECT>";
				root = this.root.firstChild;
			}
			this.root = root;
		}
		dojo.query("> option", root).forEach(function(node){
			//	TODO: this was added in #3858 but unclear why/if it's needed;  doesn't seem to be.
			//	If it is needed then can we just hide the select itself instead?
			//node.style.display="none";
			node.innerHTML = dojo.trim(node.innerHTML);
		});

	},

	getValue: function(	/*item*/ item,
						/*attribute-name-string*/ attribute,
						/*value?*/ defaultValue){
		return (attribute == "value") ? item.value : (item.innerText || item.textContent || '');
	},

	isItemLoaded: function(/*anything*/ something){
		return true;
	},

	getFeatures: function(){
		return {"dojo.data.api.Read": true, "dojo.data.api.Identity": true};
	},

	_fetchItems: function(	/*Object*/ args,
							/*Function*/ findCallback,
							/*Function*/ errorCallback){
		// summary:
		//		See dojo.data.util.simpleFetch.fetch()
		if(!args.query){ args.query = {}; }
		if(!args.query.name){ args.query.name = ""; }
		if(!args.queryOptions){ args.queryOptions = {}; }
		var matcher = dojo.data.util.filter.patternToRegExp(args.query.name, args.queryOptions.ignoreCase),
			items = dojo.query("> option", this.root).filter(function(option){
				return (option.innerText || option.textContent || '').match(matcher);
			} );
		if(args.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(args.sort, this));
		}
		findCallback(items, args);
	},

	close: function(/*dojo.data.api.Request || args || null*/ request){
		return;
	},

	getLabel: function(/*item*/ item){
		return item.innerHTML;
	},

	getIdentity: function(/*item*/ item){
		return dojo.attr(item, "value");
	},

	fetchItemByIdentity: function(/*Object*/ args){
		// summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback.
		//		Refer to dojo.data.api.Identity.fetchItemByIdentity() for more details.
		//
		// description:
		//		Given arguments like:
		//
		//	|		{identity: "CA", onItem: function(item){...}
		//
		//		Call `onItem()` with the DOM node `<option value="CA">California</option>`
		var item = dojo.query("> option[value='" + args.identity + "']", this.root)[0];
		args.onItem(item);
	},

	fetchSelectedItem: function(){
		// summary:
		//		Get the option marked as selected, like `<option selected>`.
		//		Not part of dojo.data API.
		var root = this.root,
			si = root.selectedIndex;
		return typeof si == "number"
			? dojo.query("> option:nth-child(" + (si != -1 ? si+1 : 1) + ")", root)[0]
			: null;	// dojo.data.Item
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(dijit.form._ComboBoxDataStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["dijit.form.FilteringSelect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.FilteringSelect"] = true;
dojo.provide("dijit.form.FilteringSelect");



dojo.declare(
	"dijit.form.FilteringSelect",
	[dijit.form.MappedTextBox, dijit.form.ComboBoxMixin],
	{
		// summary:
		//		An enhanced version of the HTML SELECT tag, populated dynamically
		//
		// description:
		//		An enhanced version of the HTML SELECT tag, populated dynamically. It works
		//		very nicely with very large data sets because it can load and page data as needed.
		//		It also resembles ComboBox, but does not allow values outside of the provided ones.
		//		If OPTION tags are used as the data provider via markup, then the
		//		OPTION tag's child text node is used as the displayed value when selected
		//		while the OPTION tag's value attribute is used as the widget value on form submit.
		//		To set the default value when using OPTION tags, specify the selected
		//		attribute on 1 of the child OPTION tags.
		//
		//		Similar features:
		//			- There is a drop down list of possible values.
		//			- You can only enter a value from the drop down list.  (You can't
		//				enter an arbitrary value.)
		//			- The value submitted with the form is the hidden value (ex: CA),
		//				not the displayed value a.k.a. label (ex: California)
		//
		//		Enhancements over plain HTML version:
		//			- If you type in some text then it will filter down the list of
		//				possible values in the drop down list.
		//			- List can be specified either as a static list or via a javascript
		//				function (that can get the list from a server)

		// required: Boolean
		//		True (default) if user is required to enter a value into this field.
		required: true,

		_lastDisplayedValue: "",

		_isValidSubset: function(){
			return this._opened;
		},

		isValid: function(){
			// Overrides ValidationTextBox.isValid()
			return this.item || (!this.required && this.get('displayedValue') == ""); // #5974
		},

		_refreshState: function(){
			if(!this.searchTimer){ // state will be refreshed after results are returned
				this.inherited(arguments);
			}
		},

		_callbackSetLabel: function(
						/*Array*/ result,
						/*Object*/ dataObject,
						/*Boolean?*/ priorityChange){
			// summary:
			//		Callback from dojo.data after lookup of user entered value finishes

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// still need to test against _lastQuery in case it came too late
			if((dataObject && dataObject.query[this.searchAttr] != this._lastQuery) || (!dataObject && result.length && this.store.getIdentity(result[0]) != this._lastQuery)){
				return;
			}
			if(!result.length){
				//#3268: don't modify display value on bad input
				//#3285: change CSS to indicate error
				this.valueNode.value = "";
				dijit.form.TextBox.superclass._setValueAttr.call(this, "", priorityChange || (priorityChange === undefined && !this._focused));
				this._set("item", null);
				this.validate(this._focused);
			}else{
				this.set('item', result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			// Callback when a data store query completes.
			// Overrides ComboBox._openResultList()

			// #3285: tap into search callback to see if user's query resembles a match
			if(dataObject.query[this.searchAttr] != this._lastQuery){
				return;
			}
			dijit.form.ComboBoxMixin.prototype._openResultList.apply(this, arguments);

			if(this.item === undefined){ // item == undefined for keyboard search
				// If the search returned no items that means that the user typed
				// in something invalid (and they can't make it valid by typing more characters),
				// so flag the FilteringSelect as being in an invalid state
				this.validate(true);
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook for get('value') to work.

			// don't get the textbox value but rather the previously set hidden value.
			// Use this.valueNode.value which isn't always set for other MappedTextBox widgets until blur
			return this.valueNode.value;
		},

		_getValueField: function(){
			// Overrides ComboBox._getValueField()
			return "value";
		},

		_setValueAttr: function(/*String*/ value, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('value', value) works.
			// description:
			//		Sets the value of the select.
			//		Also sets the label to the corresponding value by reverse lookup.
			if(!this._onChangeActive){ priorityChange = null; }
			this._lastQuery = value;

			if(value === null || value === ''){
				this._setDisplayedValueAttr('', priorityChange);
				return;
			}

			//#3347: fetchItemByIdentity if no keyAttr specified
			var self = this;
			this.store.fetchItemByIdentity({
				identity: value,
				onItem: function(item){
					self._callbackSetLabel(item? [item] : [], undefined, priorityChange);
				}
			});
		},

		_setItemAttr: function(/*item*/ item, /*Boolean?*/ priorityChange, /*String?*/ displayedValue){
			// summary:
			//		Set the displayed valued in the input box, and the hidden value
			//		that gets submitted, based on a dojo.data store item.
			// description:
			//		Users shouldn't call this function; they should be calling
			//		set('item', value)
			// tags:
			//		private
			this.inherited(arguments);
			this.valueNode.value = this.value;
			this._lastDisplayedValue = this.textbox.value;
		},

		_getDisplayQueryString: function(/*String*/ text){
			return text.replace(/([\\\*\?])/g, "\\$1");
		},

		_setDisplayedValueAttr: function(/*String*/ label, /*Boolean?*/ priorityChange){
			// summary:
			//		Hook so set('displayedValue', label) works.
			// description:
			//		Sets textbox to display label. Also performs reverse lookup
			//		to set the hidden value.  label should corresponding to item.searchAttr.

			if(label == null){ label = ''; }

			// This is called at initialization along with every custom setter.
			// Usually (or always?) the call can be ignored.   If it needs to be
			// processed then at least make sure that the XHR request doesn't trigger an onChange()
			// event, even if it returns after creation has finished
			if(!this._created){
				if(!("displayedValue" in this.params)){
					return;
				}
				priorityChange = false;
			}

			// Do a reverse lookup to map the specified displayedValue to the hidden value.
			// Note that if there's a custom labelFunc() this code
			if(this.store){
				this.closeDropDown();
				var query = dojo.clone(this.query); // #6196: populate query with user-specifics
				// escape meta characters of dojo.data.util.filter.patternToRegExp().
				this._lastQuery = query[this.searchAttr] = this._getDisplayQueryString(label);
				// If the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox.   Set the
				// textbox value now so that the impending warning will make
				// sense to the user
				this.textbox.value = label;
				this._lastDisplayedValue = label;
				this._set("displayedValue", label);	// for watch("displayedValue") notification
				var _this = this;
				var fetch = {
					query: query,
					queryOptions: {
						ignoreCase: this.ignoreCase,
						deep: true
					},
					onComplete: function(result, dataObject){
						_this._fetchHandle = null;
						dojo.hitch(_this, "_callbackSetLabel")(result, dataObject, priorityChange);
					},
					onError: function(errText){
						_this._fetchHandle = null;
						console.error('dijit.form.FilteringSelect: ' + errText);
						dojo.hitch(_this, "_callbackSetLabel")([], undefined, false);
					}
				};
				dojo.mixin(fetch, this.fetchProperties);
				this._fetchHandle = this.store.fetch(fetch);
			}
		},

		undo: function(){
			this.set('displayedValue', this._lastDisplayedValue);
		}
	}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect");

//
// -- Class EventQueue
//
dojo.declare("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect", null,
{
  _blankPageURL: dojo.moduleUrl("com.ibm.bpm.social.util.ui", "templates/blankPage.html"),   
  
  _isInDialog: false,
  
  constructor: function(/*string*/ iframePrefix, isInDialog)
  {
    this._lastShimmeredDomNode = null;
    this._isInDialog = isInDialog;
    
    this.IFramePool_size = 3;
    this.IFramePool_idPrefix = iframePrefix;
    this.IFramePool_freeIFrameName = iframePrefix + "FreeOne";
  },

  
  /**
   * Reapplies the background effect on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the effect reapply if the previous size was the same.
   */
  reapplyEffect: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
	  this.reapplyShimmersOn(domNode, optimize);
  },
  
  /** 
   * creates the background effect
   */
  createEffect: function(domNode)
  {
	  this.createShimmer(domNode);
  },
  
  /** 
   * removes the background effect
   */  
  removeEffect: function(domNode) 
  {
	  this.removeShimmer(domNode);
  },  
  
  /**
   * Reapplies a shimmer on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the shimmer reapply if the previous size was the same.
   */
  reapplyShimmersOn: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
    // To optimize, compare with latest size.
    if (!optimize || !this._lastShimmeredDomNode || 
        domNode.offsetWidth != this._lastShimmeredDomNode.width || domNode.offsetHeight != this._lastShimmeredDomNode.height)
    {
      // Remove possible previous shimmer and add a new one with the new size.
      this.removeShimmer(domNode);
      this.createShimmer(domNode);
      
      // Remember the current this.resultsDiv size.
      this._lastShimmeredDomNode = {width:domNode.offsetWidth, height:domNode.offsetHeight};
    }
  },
  
  // @deprecated - use createEffect
  createShimmer: function(domNode)
  {
    this._log("createShimmer() - id: " + domNode.id);
    this._createShimmer_0(domNode);
    //this._createShimmerDashboard(domNode);
    //this._createGenericObjShimmer({domNode:domNode});
  },
  
  // @deprecated - use removeEffect  
  removeShimmer: function(domNode) 
  {
    this._log("removeShimmer() - id: " + domNode.id);
    this._removeShimmer_0(domNode);
    //this._removeShimmerDashboard(domNode);
    //this._removeGenericObjShimmer({domNode:domNode});
  },
  
  
  //////////////////////////////////////////////////////////////////////////////////////////////////
  // IMPLEMENTATION: Hand made version. The iframe appears OVER the div.
  //////////////////////////////////////////////////////////////////////////////////////////////////
  
  _createShimmer_0: function(domNode)
  {
    // LL
    var objParent = domNode.parentNode;    
    if(objParent){  
      try{
        objParent.removeChild(domNode);
        objParent.appendChild(domNode);
      }catch(err){
      }
    }  
    
    // Look for existing shimmer.
    var shimmerId = this._getShimmerId(domNode);
    var shimmer = document.getElementById(shimmerId);
    if(shimmer)
    {
      this.removeShimmer(domNode);
    }
    
    // Create new shimmer (need to use iframe, if use DIV does not work on Lotus Forms)
    shimmer = document.createElement('iframe');
    shimmer.id = shimmerId;
    shimmer.src = this._blankPageURL;
    //shimmer.src = "javascript:'<html></html>';"
    
//    var opacityVal = 0;//0 [0..10]
//    shimmer.style.opacity = opacityVal/10;
//    shimmer.style.filter = 'alpha(opacity=' + opacityVal*10 + ')';
    
    // Set sizes
    var inWidth = domNode.offsetWidth;
    var inHeight = domNode.offsetHeight;
    var inTop = this._getPositionTop(domNode);
    var inLeft = this._getPositionLeft(domNode);
    
    shimmer.style.position='absolute';
    shimmer.style.width=inWidth+"px";
    shimmer.style.height=inHeight+"px";
    shimmer.style.top=inTop+"px";
    shimmer.style.left=inLeft+"px";

    // defect 108900; account specifically for IE8
	//if (dojo.isIE == 8) {
	//	shimmer.style.zIndex='1'; //Defect 109956 Setting this to 1 caused the users to be hidden in IE8
	//							  // in the dropdown, had to backout this fix for this defect.
	//} else if (dojo.isIE && dojo.isIE < 8) {
    if (dojo.isIE) {
		shimmer.style.zIndex='0';
	} else {
		shimmer.style.zIndex='-1';
	}
    
    shimmer.setAttribute('frameborder','0');

    // Trick: Add the shimmer at the end or beginning, depending if this shimmer is used inside or not of a dialog.
    if (this._isInDialog)
    {
      document.body.appendChild(shimmer);
    }
    else
    {
      document.body.insertBefore(shimmer, document.body.firstChild);
    }

  
    this._log("Shimmer added to document body.");
  },
  
  
  /**
   * Local implementation.
   */
  _removeShimmer_0: function(domNode)
  {
    var shimmerId = this._getShimmerId(domNode);
    try 
    {
      var shimmer = document.getElementById(shimmerId);
      if (shimmer) 
      {
        document.body.removeChild(shimmer);
        this._log("Shimmer removed from document body.");
      }
    } 
    catch(e) 
    {
      this._log(e);
    }
  },
  
  
  _getShimmerId: function(domNode)
  {
    var shimmerId = (domNode.id) ? (domNode.id + "_shimmerutil_shimmer_") : "_unnamed_shimmerutil_shimmer_";
    return shimmerId;
  },
  
  
  _getPositionLeft: function(node)
  {
      var curleft = 0;
      if (node.offsetParent) 
      {
        while (1) 
        {
          curleft += node.offsetLeft;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
      } 
      else if (node.x) 
      {
          curleft += node.x;
      }
      return curleft;
  },
  
  
  _getPositionTop: function(node)
  {
    var curtop = 0;
    if (node.offsetParent) 
    {
        while (1) 
        {
          curtop += node.offsetTop;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
    } 
    else if (node.y) 
    {
        curtop += node.y;
    }
    return curtop;
  },
  
  _log: function( message )
  {
    //console.log("ShimmerUtil (" + this.IFramePool_idPrefix + "): " + message);
  }
}
);


}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.UserChooser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.UserChooser"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.UserChooser");





/**
 * Dojo widget that allows the selection of one element from a list of valid ones or also the typing of a new element.
 * <p>
 *
 * The valid elements are provided in the constructor of this class, using the mandatory argument "ucModel".<br>
 * The ucModel should be an object that follows the contract/interface exemplified in the class
 * com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel.
 * <p>
 * Features:
 * <ol>
 *  <li>Embeddable UserChooser or ready to use UserChooserDialog</li>
 *  <li>Configurable model</li>
 *  <li>Configurable initially displayed suggestions</li>
 *  <li>Configurable delay between searchs</li>
 *  <li>Configurable minimum number of letters before starting search</li>
 *  <li>Configurable maximum number of results displayed at a time</li>
 *  <li>Allows to list users and/or groups</li>
 *  <li></li>
 * </ol>
 *
 * Arguments:
 * <ol>
 *  <li>ucModel (Mandatory): the source for the data.</li>
 *  <li>dropDownMaxSize (Optional): The maximum number of elements to display in the dropdown.</li>
 *  <li>usePointerStyle (Optional): Default is false.  If true, the ui will be styled as a pointer.
 *        If false, the ui will be styled as a combo box.</li>
 *  <li>searchDelay (Optional): The time to wait since a key is pressed to when a search is fired. In milliseconds.</li>
 *  <li>searchMinInputLength (Optional): The minimum length on the input to start a search.
 *      Values smaller than this in length will not perform any search.</li>
 *  <li>onChangeHandler (Optional): Handler function to be notified when a selection is done (or undone). 
 *                                  This handler should check
 *                                  the value of the passed in
 *                                  'choice' parameter.  If
 *                                  choice != null, the change
 *                                  is valid.</li>
 *  <li>onEnterHandler (Optional): Handler function for ENTER keystroke.</li>
 *  <li>onEscapeHandler (Optional): Handler function for ESCAPE keystroke.</li>
 * </ol>
 *
 */
dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.UserChooser", [dijit._Widget, dijit._Templated],
{

  templateString:"<div class=\"bpmSocialUserChooser\">\n  <div dojoAttachPoint=\"inputDiv\">\n  </div>\n  <div class=\"hidden noUsersFoundMessage\" dojoAttachPoint=\"noUsersFoundDiv\">\n  </div>  \n  <div class=\"hidden\" dojoAttachPoint=\"dropdownDiv\" > \n      <div class=\"pointer-up\">\n        <div class=\"pointer-up-inner\"></div>             \n        <div class=\"content\">\n            <div dojoAttachPoint=\"resultsDiv\" role=\"option\" class=\"uchDropdown\"></div>            \t\n        </div>            \n    </div>\n  </div>\n</div>\n",
  
  /**
   * Arguments and default values. See description in the header of this class.
   */
  ucModel: null,
  assignTaskidFilter: null,
  collabTaskidFilter: null,
  namesonly: true,
  dropDownMaxSize: (dojo.isSafari && (navigator.userAgent.indexOf('iPad') != -1))? 5:10,
  usePointerStyle: false,
  multiSelect: false,
  showProgressLabel: true,
  searchDelay: 800,
  searchMinInputLength: 1,
  onChangeHandler: null,
  onEnterHandler: null,
  onEscapeHandler: null,
  topic: "com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred",
  preloadInitialSuggestions: false,

  /**
   * Indicates if this instance is inside a USerChooserDialog.
   */
  isInDialog: false,

  /**
   * Private attributes.
   */
  _progressDiv: null,
  _inputWidth: 16,
  _eventConnections: null,
  _multiSelectConnections: null,
  _highlighter: null,
  _chosenItem: null,
  _chosenItems: null,
  _buttonNumber: 1,

  //base text direction user preference
  _btd: "",
  _bodyDir: "",

  /**
   * Indicates if in the middle of a search.
   */
  _performingSearch: false,

  /**
   * The result of the last query.
   */
  _lastQueryItems: [],

  /**
   * The value searched for in the previous attempt. Used to avoid performing the same search twice.
   */
  _lastSearchValue: "",

  _hasFocus: true,
  
  _justOpenedDropdown: true,// Defect 69068

  /**
   * Id of the last scheduled timer for a search.
   */
  _pendingId: null,
  _concurrentSearches: 0,
  _nextSearchId: 0,

  widgetsInTemplate: true,

  constructor: function()
  {
  },
  
  postCreate: function()
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this.inherited("postCreate", arguments);        
    this.setAssignTaskidFilter(this.assignTaskidFilter);
    this.setCollabTaskidFilter(this.collabTaskidFilter);
    if (this.multiSelect) {
        this.usePointerStyle = false;
    }
    //this._debugArguments();
    this._initLocalization();
    this._initUiElements();
    this._setHandlers();
    this._btd = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
    this._bodyDir = document.body.dir;
    if (this._bodyDir) {
        this._bodyDir == this._bodyDir.toLowerCase();
    } else {
        this._bodyDir == "ltr";
    }
    if (dojo.isIE == 8) {
    } else {
        var self = this;
        dojo.subscribe(this.topic, self._handleRestError);
    }
 
    if (this.preloadInitialSuggestions && this.ucModel.getInitialSuggestions) {
    	var model = this.ucModel;
    	var size = this.dropDownMaxSize + this._getMaxSizeFactor();
    	var af = this.assignTaskidFilter;
    	var cf = this.collabTaskidFilter;
        var no = this.namesonly;
    	window.setTimeout(function(){model.getInitialSuggestions(size,af,cf,no);},100);
    }
  },


  _debugArguments: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "dropDownMaxSize=" + this.dropDownMaxSize,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchDelay=" + this.searchDelay,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchMinInputLength=" + this.searchMinInputLength,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "assignTaskidFilter=" + this.assignTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "collabTaskidFilter=" + this.collabTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "namesonly=" + this.namesonly,"UserChooser");
  },


  _initLocalization: function()
  {
    
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
  },
  
  _getFormForElement: function(e) {
  		var form = e.parentNode;
  		while (form && (!form.nodeName || (form.nodeName.toLowerCase() != 'form'))) {
  			form = form.parentNode;
  		}
  		if (!form) {
  			form = document.body;
  		}
  		return form;
	},
  
  _initUiElements: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    if (!this.multiSelect) {
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="uchInput";
        this.chooserInput.autocomplete="off";
        dojo.attr(this.chooserInput,"aria-required","true");
        dojo.place(this.chooserInput, this.inputDiv, "first");
        this._progressDiv = document.createElement("div");
        dojo.place(this._progressDiv,this.inputDiv,"last");
    } else {
        var div = document.createElement("div");             
        div.className = "multiselect-container";
        var ul = document.createElement("ul");
        ul.className = "multiselect-choices";
        ul.id = "userchooserchosenitems";
        div.appendChild(ul);
        this.inputDiv.appendChild(div);
        var li = document.createElement("li");
        li.className = "multiselect-search";
        ul.appendChild(li);
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="multiselect-input";
        this.chooserInput.autocomplete="off";
        li.appendChild(this.chooserInput);
        this._progressDiv = document.createElement("div");
        li.appendChild(this._progressDiv);
    }
    this.noUsersFoundDiv.innerHTML = this._nlsMessages.noUsersFound;
    this.noUsersFoundDiv.id = "noUsersFound_"+this.id;
    this.chooserInput.focus();
    

    if (this.usePointerStyle) {
        dojo.addClass(this.dropdownDiv,"pointer");
    } else {
        dojo.addClass(this.dropdownDiv,"dropdown");
    }

       
    if (!this.chooserInput.id) {
        var labelCounter = 0; 
        while (dojo.byId("bpsace_userChooserLabel_" + labelCounter) != null) {
            labelCounter = labelCounter + 1;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "labelCounter: "+labelCounter,"UserChooser");
        this.chooserInput.id = "bpsace_userChooserLabel_" + labelCounter;        
    }
    
    // RTC 44850 -- for A11y
    this.chooserInput.title = this._nlsMessages.typeInTextboxLabel;

    
    // Init highlighter
    this._highlighter = new com.ibm.bpm.social.util.ui.userchooser.Highlighter( this.resultsDiv, this._lastQueryItems );

		// Init shimmer.
    
    this._shimmerUtil = new com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect("UCIFramePool_", this.isInDialog);
    
    // Set a unique id for some divs.
    if (!this.dropdownDiv.id)
    {
      this.dropdownDiv.id = this._generateUid();
    }
    if (!this.resultsDiv.id)
    {
      this.resultsDiv.id = this._generateUid();
    }    
    
    // Set into initial state.
    this._chooseItem(-1);
    this._hideResults();
    this._exitSearchingMode();
  },

  //removes duplicates from array 
  _unique: function(  /*{cn,userID,fullName,userName}[]*/ origArray, /*boolean*/ twoArrays ) {
		var newArray = [];
        var x,y,prop,entry,found,equal;		
		for (x=0; x < origArray.length; x++) {
		    found = false;
			entry = origArray[x];
            if (twoArrays) {            
    			for (y = 0; y < newArray.length; y++) {
    				equal = true;
    				for (prop in entry) {
    					if (newArray[y][prop] !== undefined) {
                            com.ibm.bpm.social.util.common.Log.log(2, "1["+prop+"]: "+entry[prop],"UserChooser");
                            com.ibm.bpm.social.util.common.Log.log(2, "2["+prop+"]: "+newArray[y][prop],"UserChooser");
    						if (entry[prop] != newArray[y][prop]) {
    							equal = false;
    							break;
    						}
    					} else {
    						equal = false;
    						break;
    					}
    				}//end for prop
    				if (equal) {
    					for (prop in newArray[y]) {
    						if (entry[prop] === undefined) {
    							equal = false;
    							break;
    						}
    					}
    				}
    			    if (equal) {
    				    found = true;
    				    break;
    				}
    			}//end for y
            }//end twoArrays
			if (!found && this.multiSelect) {//if multiSelect, remove items already selected
               if (this._chosenItems && this._chosenItems.length) {
                   for (y in this._chosenItems) {
                       equal = true;
                       for (prop in entry) {
                           if (this._chosenItems[y][prop] !== undefined) {
                               com.ibm.bpm.social.util.common.Log.log(2, "3["+prop+"]: "+entry[prop],"UserChooser");
                               com.ibm.bpm.social.util.common.Log.log(2, "4["+prop+"]: "+this._chosenItems[y][prop],"UserChooser");
                               if (entry[prop] != this._chosenItems[y][prop]) {
                                   com.ibm.bpm.social.util.common.Log.log(2, "5["+prop+"] not equal","UserChooser");
                                   equal = false;
                                   break;
                               }
                           } else {
                               com.ibm.bpm.social.util.common.Log.log(2, "6["+prop+"] undefined","UserChooser");
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           if (entry["cn"] === undefined && (entry["userID"] === undefined || entry["fullName"] === undefined)) {//need these 2 fields to determine a unique user
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           found = true;
                       }
                   }
               }
            }//end !found && multiSelect            
            if (!found) {
			   newArray.push(entry);
			}
		}
	 return newArray;
  },

  _requestSearch: function()
  {
    var searchValue = this._getSearchValue();
    com.ibm.bpm.social.util.common.Log.log(2,  "_requestSearch(): [" + searchValue + "]","UserChooser");
    if ( searchValue.length < this.searchMinInputLength ) {    
      com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. Input should be at least " + this.searchMinInputLength + " characters.","UserChooser");
      this._clearPendingId();
      this._hideResults();
    } else if (this._isInvalidWildcard(searchValue, true)) {
        com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. InvalidWildcard.","UserChooser");
        this._hideResults();
        // Unchoose previous selection
        this._chooseItem(-1);
    } else {
      // Enqueue search.
      this._requestDelayedSearch();
    }

    // Unchoose previous selection
    this._chooseItem(-1);
  },

  _isInvalidWildcard: function(searchString,showMessage) {
       //check for wildcard character in the middle of the search string
       var wildcard = searchString.indexOf('*');
       while (wildcard != -1) {
           if ((wildcard > 0) && (wildcard < (searchString.length-1))) {
               // we do not support wildcards in the middle of the string
               //var dlg = new com.ibm.bspace.common.util.widget.ErrorPopup();
               //dlg.setMessage(this._nlsMessages.invalidWildcardMessage, null, null, null);
               //dlg.destroyOnHide = true;
               //dlg.show();
               if (showMessage) {
                   
                   
                   com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(this._nlsMessages.invalidWildcardMessage);             
               }
               return (true);
           } else {
               wildcard = searchString.indexOf('*',wildcard+1);
           }
       }
       return (false);
  },

 /**
  * Returns search value trimmed of spaces.
  * */
  _getSearchValue: function()
  {
    var searchString = this.chooserInput.value;
    // 76246 convert the % character to * so that we are only using one wildcard char
    var wildcard = searchString.indexOf('%');
    while (wildcard != -1) {
    	searchString = searchString.substring(0,wildcard)+'*'+searchString.substring(wildcard+1);
        wildcard = searchString.indexOf('%',wildcard+1);
    }
    return searchString;
    //72937 return dojo.trim( this.chooserInput.value );
  },

   /**
   *Returns search value trimmed of leading and ending wild cards.
  *  */

  _getValueNoWildCards: function(value)
  {
      if (value.length > 2 && value.charAt(0) == '"' && value.charAt(value.length-1) == '"') {
          value = value.substring(1, value.length-1);
          com.ibm.bpm.social.util.common.Log.log(2, '_getValueNoWildCards quotes found: '+value,"UserChooser");
      }
      if (value.length > 1) {
          if (value.substr(0,1) == '*') {
              value = value.substr(1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards beg * found: "+value,"UserChooser");
          }
          if (value.length > 1 && value.indexOf('*',value.length-1) > -1) {
              value = value.substring(0,value.length-1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards end * found: "+value,"UserChooser");
          }
      }
    return value;    
  },

   /**
    * Returns the id of the input field generated for the user chooser
    */
   getInputId: function()
   {
      return this.chooserInput.id;
   },
   
   /**
    * Sets the aria-labelled by field for the generated input field for the user chooser to the value in the input string
    * which should be the id field of the corresponding label tag
    */
   setAriaLabelledby: function( /*String*/ labelId ) {
   
   	if (this.chooserInput) {
   		
   		dojo.attr(this.chooserInput, "aria-labelledby", labelId);
   	}
   },

  /**
   * Handles a delay before calling _performSearch(). Cancels all pending requests before scheduling a new one.
   */
  _requestDelayedSearch: function()
  {

    // Cancel pending request if any.
    this._clearPendingId();

    // If different from previous search, do a search.
    var searchValue = this._getSearchValue();
    if (searchValue == this._lastSearchValue)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Ignoring search request. Search value is the same as last request: '" + searchValue + "'.","UserChooser");
    }
    else
    {
      if (searchValue == "")
      {
        this._processEmptyState();
      }
      else
      {
        var self = this;
        this._pendingId = window.setTimeout(function() 
                          {
                            self._lastSearchValue = searchValue;
                            self._performSearch(searchValue);
                          }, 
                          this.searchDelay);
      }
    } // else
  },

  
  _enterSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOn";
  },


  _exitSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOff";
  },


  /**
   * Executes a search against VMM.
   */
  _performSearch: function( /*String*/ searchString )
  {
    var searchId = this._nextSearchId++;
    this._concurrentSearches++;
    com.ibm.bpm.social.util.common.Log.log(2, "START    search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches,"UserChooser");

    this._enterSearchingMode();

	//new code to check for mashups wild card setting
	com.ibm.bpm.social.util.common.Log.log(2, "get mashups service","UserChooser");
	var useWildCards = true;
	try {
		var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		if (cs == null)
		   com.ibm.bpm.social.util.common.Log.log(2, "could not find mashups service", "UserChooser");
		else {
		   var autoWildCard = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.AUTO_WILDCARD_DISABLED);
		   com.ibm.bpm.social.util.common.Log.log(2, "autowildcard value =" + autoWildCard,"UserChooser");
		   if (autoWildCard == true) 
		   {
			   useWildCards = false;
		   }
		}
	} catch (e2) {
		// if we can't get mashups, then assume wildcards are ok
        com.ibm.bpm.social.util.common.Log.log(2, "error getting mashups service: "+e2 ,"UserChooser");
	}


	//end code to check for mashups wild card setting
    // Add wildcards
	var searchStringToUse;
	var equalSearchString;
	if (useWildCards == true) 
	{
       searchStringToUse = "*";
       if (searchString && searchString != "") {
    	   searchStringToUse = searchString;
		   if (searchString.length > 2 && searchString.charAt(0) == '"' && searchString.charAt(searchString.length-1) == '"') {
			   searchStringToUse = searchString.substring(1, searchString.length-1);
		   }
		   else {
			   if (searchString.charAt(0) != '*') {
				   searchStringToUse = '*' + searchStringToUse;
			   }
			   if (searchString.charAt(searchString.length-1) != '*') {
				   searchStringToUse = searchStringToUse + '*';
			   }
		   }
       }
	   equalSearchString = searchString;
	}
	else 
	{
	   searchStringToUse = searchString;
	}

	com.ibm.bpm.social.util.common.Log.log(2, "Search string is " + searchStringToUse,"UserChooser");

	var modelItems = [];
    // Request 1 more element than can be displayed, so to identify if there are more elements than we are displaying.
	if (useWildCards == true) {
		//78062 var modelItems1 = this.ucModel.query( equalSearchString, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		var modelItems2 = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		
		/* 78062 if (modelItems1.length > 0) {
		   if (modelItems2.length > 0) {
		      modelItems = this._unique(modelItems1.concat(modelItems2),true);
		   } else {
		      modelItems = this._unique(modelItems1,false);
		   }
		} else */
        if (modelItems2.length > 0) {
			modelItems = this._unique(modelItems2,false);
		}
	} else {
		modelItems = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
        modelItems = this._unique(modelItems,false);
	}

    //var modelItems = [];

    // Are we displaying all the items?
    var allItemsDisplayed = true;
    if ( modelItems.length >= (this.dropDownMaxSize + this._getMaxSizeFactor()) )
    {
      // There are more elements than the ones to be displayed. Set a flag indicating this.
      allItemsDisplayed = false;

      // Remove last unwanted element.
      modelItems.length = this.dropDownMaxSize;
    }
    else if ( modelItems.length <= this.dropDownMaxSize )
    {
      // We are ready to display all query elements. allItemsDisplayed is true by default.
    }
    else if ( modelItems.length <= 0 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Got " + modelItems.length + " results." ,"UserChooser");
      // No matches or ucModel (or VMM in the case of VMMModel) is down.
    }
    else
    {
      // Unexpected situation.
      // TODO use NLS. Use common dialogs.
      com.ibm.bpm.social.util.common.Log.log(2,  "*** ucModel is returning " + modelItems.length + " when the maximum allowed is " + this.dropDownMaxSize ,"UserChooser");
      modelItems.length = this.dropDownMaxSize;
    }

    // This search is finishing in sequence? If not, discard.
    var outOfSequenceSearch = (searchId < this._nextSearchId-1);
    if (outOfSequenceSearch)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Out of sequence search: " + searchId + ". Discarding results.","UserChooser");
    }
    else
    {
      // Populate dropdown with search result.
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
      this._exitSearchingMode();
    }
    
    this._concurrentSearches--;
    if (modelItems.length == 0) {
        dojo.removeClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"true","aria-describedby":this.noUsersFoundDiv.id});

    } else {
        dojo.addClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    com.ibm.bpm.social.util.common.Log.log(2,  "FINISHED search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches + " : " + modelItems.length + " results.","UserChooser");
  },

  _getMaxSizeFactor: function() {
    var num = 1;
    if (this.multiSelect && this._chosenItems) {
        num = this._chosenItems.length + 1;
    }
    com.ibm.bpm.social.util.common.Log.log(2, "_getMaxSizeFactor: "+num,"UserChooser");
    return num;
  },

  _setModelItems: function( /*{cn,userID,fullName,userName}[]*/ modelItems )
  {
    this._lastQueryItems = modelItems;

    // Update highlighter model.
    this._highlighter.updateModel( modelItems );
    this._highlighter.reset();
  },


  _setModelItemsAndUpdateView: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*boolean*/ allItemsDisplayed )
  {
    this._setModelItems( modelItems );
    this._populateDropdown( modelItems, allItemsDisplayed );

    // The component can loose focus before the search is finished, so only show the dropdown if still 
    // with focus.
    if (this._hasFocus)
    {
      this._showResults();
    }
    this._highlighter.reset();

    // For BPM, we are not using the cn or userID fields for searching, so skip this for now
    // If the typed text is a value (cn or userID) in the model, then choose it.
    //this._tryAutomaticChoosing(modelItems);
  },


  /**
   * If the typed text is a value (cn or userID) in the model, then choose it.
   */  
  _tryAutomaticChoosing: function(/*{cn,userID,fullName,userName}[]*/ modelItems)
  {
    var searchValue = this._getValueNoWildCards(this._getSearchValue());
    
    // If there are items, try to choose automatically.
    if (modelItems.length > 0)
    {
      var index = this._searchModelForField( modelItems, searchValue, "cn" );
      com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (cn) on '" + searchValue + "': " + index ,"UserChooser");
      if (index != -1)
      {
        com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
        this._chooseItem( index );
      }
      else
      {
        // Search for id
        var index = this._searchModelForField( modelItems, searchValue, "userID" );
        com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (userID) on '" + searchValue + "': " + index ,"UserChooser");
        if (index != -1)
        {
          com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
          this._chooseItem( index );
        }
      }
    }
    else
    {
      // Set to true to enable free form choice.
      var freeFormChoiceEnabled = false;
      if (freeFormChoiceEnabled)
      {
        // If no values are returned from query (no actuall matches or ucModel down), allow free-form entry.
        com.ibm.bpm.social.util.common.Log.log(2, "Free-form choice made: " + searchValue,"UserChooser");
        this._chooseItemValue( {cn:"", userID:"", fullName: searchValue} );
      }
    }
  },
  

  /**
   * Searchs modelItems for an element with the same value in the searchField as specified.
   * The search is case insensitive.
   * @return The index of the found element or -1 if none.
   */
  _searchModelForField: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*String*/ searchValue, /*String*/ searchField )
  {
    var index = -1;
    if (searchValue && searchValue!="" && searchField && searchField!="")
    {
      var searchValueLC = searchValue.toLowerCase();
      for( var i=0; i<modelItems.length; i++ )
      {
        var currentValue = modelItems[i][searchField];
        if (currentValue && currentValue.toLowerCase && currentValue.toLowerCase() == searchValueLC)
        {
          index = i;
          break;
        }
      }
    }
    return index;
  },

  /**
   * Show the results if there is at least one item. Otherwise ignore.
   */
  _showResults: function()
  {
  	 
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    if ( this._lastQueryItems.length > 0 && this.resultsDiv.childNodes.length > 0 )
    {
      var optimizeShimmer;
      if ( dojo.hasClass(this.dropdownDiv,"hidden") )
      {
        dojo.removeClass(this.dropdownDiv,"hidden");

        // If just opened the dropdown, initialize highlight.
        this._highlighter.reset();
        this._justOpenedDropdown = true;// Defect 69068
        
        // Do not optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Not visible yet. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = false;
      }
      else
      {
        // Already visible and possible changed size. Optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Already visible. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = true;
      }
      this._shimmerUtil.reapplyShimmersOn(this.resultsDiv, optimizeShimmer);
      // Defect 69068 User Chooser in Modify Task/Collaboration prefills the selection with first user from result
      // comment out the next line to prevent prefilling
      // this._chooseItem( this._highlighter.index, dojo.keys.DOWN_ARROW );//allow JAWS to speak this selection
    }
    else
    {
      this._hideResults();
    }
  },


  _hideResults: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    dojo.addClass(this.dropdownDiv,"hidden");
    this._shimmerUtil.removeShimmer(this.resultsDiv);
    this._lastSearchValue = null;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
  },


  /**
   * Generates a UID.
   */
  _generateUid: function() /*string*/
  {
    var newId =  "bspace.userchooser." + new Date().getTime();
    if (newId == this._lastGeneratedUid)
    {
      newId += ".x";
    }
    
    this._lastGeneratedUid = newId;
    return newId;
  },
  
  
  _resultsAreVisible: function()
  {
    return dojo.hasClass(this.dropdownDiv,"hidden") == false;
  },


  /**
   * Populates the dropdown with elements created from the specified items.
   */
  _populateDropdown: function( /*{cn,userID,fullName,userName}[]*/ items, /*boolean*/ allItemsDisplayed )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    // Disconnect previous
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = [];

    // Clean previous values.
    this._removeAllChildren(this.resultsDiv);

    // Add new values
    for( var i=0; i<items.length; i++ )
    {
      var qItem = items[i];

      // Calculate display string.
      var identifier = (qItem.fullName && qItem.fullName!="") 
      						? qItem.fullName 
      						: null;
      //format for BPMModel
      //  "userID":d2.userID,
      //  "fullName":d2.fullName,
      //  "userName":d2.userName} 
      //  var dispHtml = qItem.cn + (identifier ? ("&rlm; [ " + identifier + " ]") : "");
      var dispHtml = identifier;
      var rlm = "";
      if (document.body.dir == "rtl") {
          //rlm = "&rlm;";//after applying BTD, this doesn't seem to anything
      }
      if (identifier && qItem.userName && qItem.userName!="") {
          dispHtml = this._createDisplayName(identifier, qItem.userName );//identifier + (rlm+" [ " + qItem.userName + " ]") ;
      }
/* to test with userID      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + (rlm+" [ " + qItem.userID + " ]") ;
      }
*/
/*BSpace      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + ("&rlm; [ " + qItem.userID + " ]") ;
      }
*/
      //format for other models
      var inputName = "";
      if (!dispHtml) {
          inputName = qItem.cn + (identifier ? rlm: "");
          dispHtml = this._createDisplayName(qItem.cn, identifier);//qItem.cn + (identifier ? (rlm+" [ " + identifier + " ]") : "");
      }
      this._setInputWidth(inputName);
      com.ibm.bpm.social.util.common.Log.log(2, "dispHtml: "+dispHtml,"UserChooser");

      // Bold matching substring
      if (this._lastSearchValue && this._lastSearchValue.length > 0)
      {
        dispHtml = this._emphasize( dispHtml, this._getValueNoWildCards(this._lastSearchValue) );
      }

      // Add element with display string.
      var item = this._addChild( this.resultsDiv, "div", dispHtml);
      item.className = "uchDropdownItem";
      
      
      // Hook event handlers.
      this._dropdownEventConnections.push(dojo.connect( item, "onmouseover", this, this._menuItemMouseOver ));
      this._dropdownEventConnections.push(dojo.connect( item, "onmousedown", this, this._menuItemMouseDown ));
    }
    
    // If needed, add a last DIV with a special message indicating that not all items are being displayed.
    if ( this.showProgressLabel && !allItemsDisplayed )
    {
      var msg = dojo.string.substitute( this._nlsMessages.progressLabel, {1:this.dropDownMaxSize} );
      var item = this._addChild( this.resultsDiv, "div", msg );
      item.className = "uchDropdownMessage";
    }
  },

  _createDisplayName: function(fullname, username) {
       if (this._btd == "default") {
           return fullname + (username ? (" [ " + username + " ]") : "");
       }
       var btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(fullname);
       if (this._btd == "ctx") {//must apply UCC to fullname and username separately
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(username);
               username = this._applyUCC(username, btdDir);
           }
           return fullname + (username ? (this._getMarker()+" [ " + username + " ]") : "");
       } else {
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               username = this._applyUCC(username, btdDir);
           }
           return this._getUCCWrapper() + fullname + (username ? this._getMarker()+" [ " + username + " ]"+com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF : "");
       }
       return fullname + (username ? (" [ " + username + " ]") : "");
  },

  _applyUCC: function(text, btdDir) { 
       var controlChar = (btdDir == "ltr") ? 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

       //wrap the text in the control characters
       text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;
       return text;       
  },

  _getMarker: function() {
    var marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
    if (this._bodyDir == "rtl") {
        marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;
    }    
    return marker;
  },


  _getUCCWrapper: function() {
       if (this._bodyDir == "ltr") {
           return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE;
       } 
       return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;
  },

  _setInputWidth: function(str) {       
       var len = str.length;
       if (len && len > this._inputWidth) {
           this._inputWidth = len;
           dojo.style(this.chooserInput,"minWidth",(len/2.15)+"em");
       }
  },

  /**
   * Emphasizes the matching text in the source text.
   */
  _emphasize: function( /*String*/ source, /*String*/ toMatch )
  {
    // Homogeinize strings to do case-insensitive matching.
    var sourceLC = source.toLowerCase();
    toMatch = toMatch.toLowerCase();
    var replaced = new String();
    var toMatchLength = toMatch.length;
    var idx = 0;
    while( idx < source.length )
    {
      var startIdx = idx;
      // Search on the lowercase version.
      idx = sourceLC.indexOf( toMatch, idx );
      if ( idx == -1 )
      {
        // Add previous text from the original version.
        var preFoundStr = source.substring( startIdx );
        replaced += preFoundStr;
        break;
      }
      else
      {
        // Add previous text
        var preFoundStr = source.substring(startIdx, idx);
        replaced += preFoundStr;

        // Add new replacement. Use text from original str, to match case.
        var foundStr = source.substr(idx, toMatch.length);
        replaced += "<b>" + foundStr + "</b>";

        idx += toMatchLength;
      }
    }

    com.ibm.bpm.social.util.common.Log.log(2, "_replaced('" + source + "', '" + toMatch + ")='" + replaced + "'","UserChooser");
    return replaced;
  },


  _processEmptyState: function()
  {
    if ( this._getSearchValue() == "" || this._isInvalidWildcard(this._getSearchValue()) )
    {
      // Show initial suggestions
      var modelItems = (this.ucModel.getInitialSuggestions) ? this.ucModel.getInitialSuggestions(this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly) : [];
      
      
      // Are we displaying all the items?
      var allItemsDisplayed = true;
      if ( modelItems.length > this.dropDownMaxSize )
      {
        // There are more elements than the ones to be displayed. Set a flag indicating this.
        allItemsDisplayed = false;
  
        // Remove last unwanted element.
        modelItems.length = this.dropDownMaxSize;
      }
      
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
    }
  },


  /**
   * Sets new handlers. The previous handlers will be disconnected.
   */
  _setHandlers: function()
  {
    // Disconnect previous
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = [];
    
    // Hook event handlers.
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeydown", this, this._onKeyDown ));
	this._eventConnections.push(dojo.connect( this.chooserInput, "onkeypress", this, this._onKeyPress ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeyup", this, this._onKeyUp ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onfocus", this, this._onUCFocus ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onblur", this, this._onUCBlur ));
  },



  /**
   * Special treatment to identify the correct DIV in the menu.
   *
   * Because of construction, is necessary to avoid the selection of an inner span in the DIV (this happens with the <B> tags inside the
   * DIV when bolding a part of the string).
   * If a SPAN is identified, is assumed that the parent is the div.
   * Note: this only makes sense according to the construction of the dropdown.
   * If that implementation is changed or the events associated with any other DIV here, this will not work anymore.
   *
   * TODO Find a better solution. This works on FF3 but not on IE7?
   */
  _getSmartTarget: function( /*Event*/ e )
  {
    var smartTarget = null;
    var target = e.target;
    if ( target )
    {
      if ( dojo.isIE )
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - IE" ,"UserChooser");
        smartTarget = target;
        while (smartTarget && (smartTarget.nodeName.toLowerCase() != "div")) {
        	smartTarget = smartTarget.parentNode;
        }
      }
      else
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - not IE" ,"UserChooser");
        if(target.tagName == "B") {
          target = target.parentNode;
        }
        if ( target instanceof HTMLDivElement)
        {
          smartTarget = target;
        }
        else if (((HTMLSpanElement && target instanceof HTMLSpanElement) || target.tagName == "SPAN") && target.parentNode instanceof HTMLDivElement)
        {
          smartTarget = target.parentNode;
        }
        else
        {
          // Ignore
        }
      }
    }

    return smartTarget;
  },


  _menuItemMouseOver: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_menuItemMouseOver()" ,"UserChooser");
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._highlighter.moveSelectionToItem( smartTarget );
    }
  },


  _menuItemMouseDown: function( /*Event*/ e )
  {
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._hideResults();
      this._chooseItem( this._highlighter._getIndexForDivItem( smartTarget ) );
    }
  },

  _onKeyPress: function( /*Event*/ e) {
    com.ibm.bpm.social.util.common.Log.log(2, "keyCode="+e.keyCode+" charCode="+e.charCode,"UserChooser");
    if((e.charCode>0)) {
      setTimeout(dojo.hitch(this, "_requestSearch"), 10);
    }
  },

  _onKeyUp: function( /*Event*/ e )
  {
    if (e.keyCode == dojo.keys.BACKSPACE || e.keyCode == dojo.keys.CLEAR || e.keyCode == dojo.keys.DELETE) {
      this._requestSearch();
    }
  },


  _onKeyDown: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_onKeyDown(): " + e.keyCode ,"UserChooser");
    var key = e.keyCode;
    //var key = (window.event) ? event.keyCode : e.keyCode;

    if (key == dojo.keys.DOWN_ARROW)
    {
      this._showResults();
      //First time dropdown is opened, use the down arrow key to select the first item
      //instead of moving to the next item
      if ( !this._justOpenedDropdown ) {// Defect 69068
      if ( this._highlighter.moveSelectionForward() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
      } else {
          this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
          this._justOpenedDropdown = false;// Defect 69068
      }
    }
    else if (key == dojo.keys.UP_ARROW)
    {
      this._showResults();
      if ( this._highlighter.moveSelectionBack() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
    }
    else if (key == dojo.keys.ENTER)
    {
      if ( this._resultsAreVisible() )
      {
        // Choose the currently selected item.
        this._hideResults();
        this._chooseItem( this._highlighter.index, key );
      }
      else
      {
        this._onEnterHandler();
      }
      dojo.stopEvent(e);
    }
    else if (key == dojo.keys.TAB)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
      }
    }
    else if (key == dojo.keys.ESCAPE)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
        com.ibm.bpm.social.util.common.Log.log(2, "escape detected, unchoose item","UserChooser");
        this._clearPendingId();
        this._chooseItem(-1);//make unchoose item the default behavior for escape
        this._exitSearchingMode();
      }
      else
      {
        this._onEscapeHandler();
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _clearPendingId: function() {
     if (this._pendingId)
     {
        com.ibm.bpm.social.util.common.Log.log(2, "Canceling timer: '" + this._pendingId + "'.","UserChooser");
        window.clearTimeout(this._pendingId);
        this._pendingId = null;
     }
  },

  /**
   * Chooses the specified item as the user choice. The value in this._lastQueryItems in that index will be returned when getChoice
   * is called.
   * If itemIndex is -1, will unchoose any previous selection.
   */
  _chooseItem: function( /*Number*/ itemIndex, /*KeyCode*/ keyCode )
  {
    var item = (itemIndex in this._lastQueryItems) ? this._lastQueryItems[itemIndex] : null;
    this._chooseItemValue(item, keyCode);
  },


  /**
   * Chooses the specified item as the user choice. The specified item will be returned when getChoice is called.
   * If item is null, will unchoose any previous selection.
   */
  _chooseItemValue: function( /*{cn,userID,fullName,userName}*/ item, /*KeyCode*/ keyCode )
  {
    var unchoose = true;
    if (item)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Item chosen: " + item + ":" + dojo.toJson(item),"UserChooser");
      this._chosenItem = item;
      
      // Replace input string with match.
      if (item.cn && item.cn != "")
      {
        this.chooserInput.value = item.cn;
      } else if (item.fullName && item.fullName != "") {
    	this.chooserInput.value = item.fullName;
      }

      if (!keyCode || (keyCode && keyCode === dojo.keys.ENTER)) {
          this._onChangeHandler();
      }
      unchoose = false;
      
      // Forget last searched value.
      this._lastSearchValue = null;
    }

    if ( unchoose )
    {
      this._chosenItem = null;
      this._onChangeHandler();
    }
    //com.ibm.bpm.social.util.ui.TextField.setIncorrectEntry(this.chooserInput,unchoose);
  },

  setAssignTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.collabTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter found collabTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting collabTaskidFilter to null.","UserChooser");
         this.collabTaskidFilter = null;
     }
     this.assignTaskidFilter = taskid;
  },

  setCollabTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.assignTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter found assignTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting assignTaskidFilter to null.","UserChooser");
         this.assignTaskidFilter = null;
     }
     this.collabTaskidFilter = taskid;
  },

  _onChangeHandler: function() {  
    if (this.onChangeHandler) {
      if (this.multiSelect) {
           this._onChangeHandlerMultiselect();
          this.onChangeHandler(this._chosenItems);
      } else {      
          this.onChangeHandler(this._chosenItem);
      }
    }  
  },

  _onChangeHandlerMultiselect: function() {
   if (this._chosenItem != null) {
       if (!this._chosenItems) {
           this._chosenItems = new Array();
           if (!this._multiSelectConnections) {
               this._multiSelectConnections = new Array();
           }
       }
       var li = document.createElement("li");
       li.className = "choice";
       var span = document.createElement("span");
       var displayName =this._chosenItem.fullName;
       if (displayName == undefined) {
           displayName = this._chosenItem.userName;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.cn;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.userID;
       }
       span.innerHTML = displayName;
       li.appendChild(span);
       var button = document.createElement("button");
       button.className = "multiselect-delete-button";
       var idStr = this._getNextButtonId();
       button.id = idStr;
       span.id = idStr+"_span";
       dojo.attr(button, "aria-label", this._nlsMessages.deleteButtonLabel);
       dojo.attr(button, "aria-describedby", idStr+"_span");
       this._multiSelectConnections.push(dojo.connect( button, "onclick", this, this._multiSelectDeleteChoice ));
       var hcSpan = document.createElement("span");//a11y high contrast
       hcSpan.innerHTML = "X";
       hcSpan.className = "multi-select-delete-button-hc";
       button.appendChild(hcSpan);
       li.appendChild(button);
       dojo.place(li,"userchooserchosenitems",this._chosenItems.length);
       this._chosenItem = 
               {"cn":this._chosenItem.cn,
               "userID":this._chosenItem.userID,
               "fullName":this._chosenItem.fullName,
               "userName":this._chosenItem.userName,
               "buttonId": idStr};
       this._chosenItems.push(this._chosenItem);
       this._chosenItem = null;
       this.chooserInput.value = "";
       this._resetInputWidth();
   }
  },

  _resetInputWidth: function(){
       this._inputWidth = 16;
       dojo.style(this.chooserInput,"minWidth",this._inputWidth+"em");
  },

  _multiSelectDeleteChoice: function(e) {
    com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice id: "+e.target.id,"UserChooser");
    dojo.disconnect(e.target);
    //userId and fullName
    var node = document.getElementById(e.target.id);
    if (node) {//button anchor node
      node = node.parentNode;//parent li node
      if (node) {      
          node.parentNode.removeChild(node);
          for (var i in this._chosenItems) {
              var item = this._chosenItems[i];
              if (item.buttonId == e.target.id) {
                  this._chosenItems.splice(i,1);
                  //com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice deleted index: "+i+" _chosenItems.length: "+this._chosenItems.length,"UserChooser");
                 if (this._chosenItems.length == 0) {
                      this._chosenItems = null;
                  }
                  break;
              }
          }
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _getNextButtonId: function() {
       var num = this._buttonNumber++;
       //com.ibm.bpm.social.util.common.Log.log(2, "_getNextButtonId num: "+num,"UserChooser");
       return "chosenitem_"+num;
  },

  _onEscapeHandler: function()
  {
    if (this.onEscapeHandler)
    {
      this.onEscapeHandler();
    } 
  },


  _onEnterHandler: function()
  {
    if (this.onEnterHandler)
    {
      this.onEnterHandler();
    }
  },


  _onUCFocus: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = true;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    if ( this._getSearchValue() == "" )
    {
      this._processEmptyState();
    }
    else if (this._isInvalidWildcard(this._getSearchValue())) {
      this._processEmptyState();
    }
    else
    {
      if (!this._chosenItem) {
    	  this._requestSearch();
      }
    }
  },


  /**
   * Cannot use _onBlur name to avoid collision with Dojo core.
   */
  _onUCBlur: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = false;
    this._hideResults();
    this._exitSearchingMode();
    if (this._chosenItem == null) {//if nothing selected, clear input field
        this.chooserInput.value = "";
    } else {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    dojo.addClass(this.noUsersFoundDiv,"hidden");    

    // Defect 69068 - don't autoselect - may not be what the user wanted
    // If only one item in the last query, choose it.
    //if ( this._lastQueryItems.length == 1 )
    //{      
      //this._chooseItem( 0 );
    //}
  },
  
  _handleRestError: function() {
       com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
       this.chooserInput.value = "";
       this._exitSearchingMode();
       this._clearPendingId();
	   com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },

  /**
   * @return The currently choosen value or null if none.
   */
  getChoice: function() /*{cn,userID,fullName,userName}*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "getChoice(): " + dojo.toJson(this._chosenItem),"UserChooser");
    return this._chosenItem;
  },


  _addHtmlSelectOptions: function( /*DOMSelect*/ domSelect, /*String[]*/ options )
  {
    for( var i=0; i<options.length; i++ )
    {
      var option = document.createElement("option");
      option.value = options[i];
      option.innerHTML = options[i];
      domSelect.appendChild(option);
    }
  },


  _removeAllChildren: function( /*DOMSelect*/ domSelect )
  {
    while( domSelect.hasChildNodes() )
    {
      domSelect.removeChild( domSelect.lastChild );
    }
  },


  _addChild: function( parent, /*String*/ elementName, /*String?*/ innerHTML )
  {
    var newElem = document.createElement(elementName);
    parent.appendChild(newElem);

    if ( innerHTML )
    {
      newElem.innerHTML = innerHTML;
    }
    return newElem;
  },


  destroy: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    // Destroy pending event handlers.
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = null;
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = null;
    dojo.unsubscribe(this.topic);
    this.inherited("destroy", arguments);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },


  _disconnectEvents: function(/*handler[]*/ handlers)
  {
    if (handlers && handlers.length > 0)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Disconnecting " + handlers.length + " event handlers.","UserChooser");
      while(handlers.length)
      {
        dojo.disconnect(handlers.pop());
      }
    }
  },
  
  
  _log: function( message )
  {
    //console.log( message );
  }
}
);


dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.Highlighter", null,
{
  index: null,
  item: null,
  _resultsDiv: null,
  _model: null,

  constructor: function( resultsDiv, model )
  {
    this.index = null;
    this.item = null;
    this._resultsDiv = resultsDiv;
    this._model = model;
  },


  updateModel: function( model )
  {
    this._model = model;
  },


  reset: function()
  {
    // Unselect previous item.
    this.unselectCurrent();

    // If the model has elements, select first one. Otherwise, no one.
    if ( this._model && 0 < this._model.length )
    {
      this.index = 0;
      this.selectItemIndex( this.index );
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  unselectCurrent: function()
  {
    if (this.item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "_unselectCurrent(): " + this.item.innerHTML ,"UserChooser");
      this.item.className = "uchDropdownItem";
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  /**
   * Selects the specified div, remembers the last selection and calculate the index.
   * It is expected that item is in this.resultsDiv.childNodes.
   */
  selectItem: function( /*HTMLDiv*/ item )
  {
    if (item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "selectItem(): "  + item.innerHTML,"UserChooser");
      item.className = "uchDropdownItemSelected";
      this.item = item;
      this.index = this._getIndexForDivItem( item );
    }
  },


  selectItemIndex: function( /*Number*/ itemIndex )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    var divItems = this._resultsDiv.childNodes;
    if ( itemIndex != null && itemIndex < divItems.length )
    {
      this.item = divItems[itemIndex];
      if ( this.item )
      {
        this.item.className = "uchDropdownItemSelected";
        this.index = itemIndex;
      }
      else
      {
        this.index = null;
        com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex ,"UserChooser");
      }
    }
    else
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex + ", divItems.length=" + divItems.length ,"UserChooser");
    }
  },


  moveSelectionToItemIndex: function( /*Number*/ itemIndex )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItemIndex(): " + itemIndex ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItemIndex( itemIndex );
  },


  moveSelectionToItem: function( /*HTMLDiv*/ item )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItem(): " + item.innerHTML ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItem( item );
  },


  _getIndexForDivItem: function( /*HTMLDiv*/ item )
  {
    var index = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf( this._resultsDiv.childNodes, item );
    if ( index == -1 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found: "  + item.innerHTML ,"UserChooser");
      com.ibm.bpm.social.util.common.Log.log(2,  "**: "  + dojo.toJson(item) ,"UserChooser");
    }

    return index;
  },


  moveSelectionForward: function()
  {
    var moved = false;
    if (this.index != null && this.index < this._model.length-1)
    {
      this.index++;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },


  moveSelectionBack: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "moveSelectionBack","UserChooser");
    var moved = false;
    if (this.index != null && 0<this.index)
    {
      this.index--;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },

  
  debug: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2,  "index:" + this.index + ", item:" + this.item.innerHTML ,"UserChooser");
  },


  _log: function( message )
  {
    //console.log( "Highlighter: " + message );
  }
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil");

//
// -- Collection utilities.
//

/**
 * Sorts the specified array of objects using the objects field named as specified by sortField.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy = function( array, sortField ) /*void*/
{
  // Define sorter object.
  var sorter = {
    sortBy: function( a,b)
    {
      if ( a[sortField] == b[sortField] )
        return 0;
      else
        return (a[sortField] < b[sortField]) ? -1 : 1;
    }
  };

  // Sort.
  array.sort( sorter.sortBy );
};



/**
 * Filters the specified array.
 *
 * Creates a new array containing the elements in the original array that evaluates to true when applied the filter.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.filterArray = function( /*Array*/ array, booleanFilter ) /*Array*/
{
  var filtered = new Array();
  for (var i = 0; i<array.length; i++)
  {
    var context = this;
    if ( booleanFilter( array[i], context ) )
    {
      filtered.push( array[i] );
    }
  }

  return filtered;
};


/**
 * Partitions the list based on a specified field.
 * Each time the field value changes from the previous item in the list, the groupHandler is called.
 * For each element in the group, the elementHandler is called.
 *
 * The parameter array should be previously sorted by the specified field.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayInGroupsBy = function( array, field, groupHandler, unassignedGroupMessage )
{
  var currentGroupName = null;
  var currentGroup = null;
  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize group if necessary.
    if ( currentGroupName == null || currentGroupName != fieldValue )
    {
      if ( currentGroup != null )
      {
        // Notify of finished group.
        groupHandler( currentGroupName, currentGroup );
      }
      currentGroupName = fieldValue;
      currentGroup = new Array();
    }

    // Process element.
    currentGroup.push( elem );
  } // for

  // Process last group, if any.
  if ( currentGroup != null )
  {
    groupHandler( currentGroupName, currentGroup );
  }
};


/**
 * Creates a map from an array.
 * The returned map is JSON object with the keys being the values of the specified field
 * in the mapped elements.
 *
 * The parameter array will be sorted as a side effect.
 *
 * @return A not nullable "map", from string to objects originally in array.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayBy = function( array, field, unassignedGroupMessage )
{
  com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy( array, field );
  var map = new String();

  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize bucket if necessary.
    var bucket = map[fieldValue];
    if ( bucket == null )
    {
      // No bucket yet. Create it.
      bucket = new Array();
      map[fieldValue] = bucket;
    }

    // Add element to bucket.
    bucket.push( elem );
  } // for

  return map;
};




/** 
 * Makes all elements in this array unique, i.e., removes all duplicate elements from array.
 * Assumes array is already sorted. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.unique = function(/*Array*/ array, compare)
{
  if (typeof(compare) == 'undefined') 
  {
    compare = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend;
  }
  var dst = 0;  // Destination for elements
  var src = 0;  // Source of elements
  var limit = array.length - 1;
  while (src < limit) 
  {
    while (compare(array[src], array[src + 1]) == 0) 
    {
      if (++src == limit) 
      {
        break;
      }
    }
    array[dst++] = array[src++];
  }
  if (src == limit) 
  {
    array[dst++] = array[src];
  }
  array.length = dst;
}


/** 
 * Adds all the elements in the specified arrays to destArray. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.addAll = function(/*Array*/ destArray, /*Array[]*/ arrays) 
{
  for (var a = 0;  a < arrays.length;  a++) 
  {
    arr = arrays[a];
    for (var i = 0;  i < arr.length;  i++) 
    {
      destArray.push(arr[i]);
    }
  }
}


/** 
 * Compares two objects using built-in JavaScript operators. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend = function(a, b) 
{
  if (a < b)
    return -1;
  else if (a > b)
    return 1;
  return 0;
}



/**
 * Will search the array until it matches the search critera.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf = function( /*Array*/ array, element /*, from*/ ) /*Number*/
{
  var length = array.length;

  var from = Number(arguments[2]) || 0;
  from = (from < 0) ? Math.ceil(from) : Math.floor(from);
  if (from < 0)
    from += length;

  for (; from < length; from++)
  {
    if (array[from] == element)
      return from;
  }
  return -1;
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.RestClient"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.RestClient"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.RestClient");



dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.util.RestClient", null,
{
  REASON_ADMINISTRATOR : 7,
  bpmServiceRoot: window.endpointURLObject.bpmRestJS,
  restgatewayerror: "Error 404: REST Service Gateway:",
  _queryIndex:0,

  constructor: function(bpmServiceRoot, iContext)
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    // Normalize service urls.
    if (bpmServiceRoot) {
        this.bpmServiceRoot = bpmServiceRoot;
    }
//PKL -- TODO bug
    if (iContext)
    {
      this.iContext= iContext;
    }

    this.bpmServiceRoot = this._normalizeUrl( this.bpmServiceRoot );
    this.bpmServiceRoot +="/v1";

    com.ibm.bpm.social.util.common.Log.log(2, "Using bpmServiceRoot=" + this.bpmServiceRoot ,"RestClient");

    this._bpmHeaders = {};
    this._bpmHeaders["accept"] = "application/json"; /*  "text/xml"; */
    if (!this.bpmServiceRoot) {
        this._bpmHeaders["content-type"] = "charset=utf-8";
    } else {
        this._bpmHeaders["content-type"] = "application/json";
    }
	
    
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
    
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");

  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByFilter: function( /*String*/ name, /*int*/ offset, /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes ) 
  {
   //return this._searchBPMUsers( "filter=" + name, offset, size, attributes );
     return this._searchBPMUsers( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, attributes );
  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByName: function( /*string*/ name, /*int*/ offset, /*int*/ size, /*boolean*/ includeGroups, /*boolean*/ includeUsers, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly ) 
  {
    var searchGroups = includeGroups ? (includeGroups == true) : false;	// Default false
    var searchUsers = (includeUsers != null && includeUsers != undefined) ? (includeUsers == true) : true;	// Default true
    var cnsAll = [];
    
	this.errormsg = false;
	if (searchUsers)
	{
	    cnsAll = this.searchBPMByFilter( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly );	
    }
	this.errormsg = false;
	
    // Cut any extra not requested elements.
    if ( cnsAll.length > size )
    {
      cnsAll.length = size;
    }

    return cnsAll;
  },

  /**
   * Generic BPM query. Searchs BPM for users matching the search
   * attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPMUsers: function( /*String*/ searchAttributes, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    var includeAttributes = "";
    var extractor = "";
    return this._searchBPM( "users", searchAttributes, includeAttributes, extractor, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, loadCallback );
  },

  /**
   * Generic BPM query. Searchs BPM for users/groups matching the
   * search attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPM: function( /*String*/ entity, /*String*/ searchAttributes, /*String*/ attributes, /*function*/ extractor, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "Querying BPM for " + entity + ":'" + "filter=" +searchAttributes + "', maxresult="+size+", assignTaskidFilter="+assignTaskidFilter+" , collabTaskidFilter="+collabTaskidFilter+" ,namesonly: "+namesonly+" , using encodeURIComponent on "+searchAttributes ,"RestClient");

    var assignTaskString = "";
    if (assignTaskidFilter != null) {
        assignTaskString = "&assignTaskidFilter="+assignTaskidFilter;
    }
    var collabTaskString = "";
    if (collabTaskidFilter != null) {
        collabTaskString = "&collabTaskidFilter="+collabTaskidFilter;
    }
    var namesonlyString = "&namesonly=true";//performance
    if (namesonly != 'undefined' && (namesonly == false || namesonly == "false")) {
        namesonlyString = "";
    }
    //encode special character of userids in searchAttributes
    var url = this.bpmServiceRoot + "/" + entity + "?" + "filter=" +
              (window.encodeURIComponent(searchAttributes)) +
              "&maxresult=" + size +
              assignTaskString +
              collabTaskString +
              namesonlyString +
              "&parts=none";

    var results = []; // default value
    var self = this;
    var okHandler = function( /*String*/data, ioArgs )
    {
      com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM in okHandler, result is " + ioArgs.xhr.status,"RestClient");
      if (data)
      {
        // Collect desired information from results
        //console.log("_searchBPM data: "+dojo.fromJson(data));
        var d = dojo.fromJson(data);
        if (d.data && d.data.users) {        
            for (var i in d.data.users) {
                var d2 = d.data.users[i];
                if (!d2.isDisabled) {
                    com.ibm.bpm.social.util.common.Log.log(2,"...fullName: "+d2.fullName+", userName: " + d2.userName + ", userID: " + d2.userID,"RestClient");
                    results.push( 
                        {"cn":"",
                        "userID":d2.userID,
                        "fullName":d2.fullName,
                        "userName":d2.userName} 
                    );
                }
                //cn,userID,fullName,userName
            }
        }
      }
      //console.log("_searchBPM loadCallback: "+loadCallback); == undefined
      if (loadCallback) {
          loadCallback(results, ioArgs);
      }
    };
    var errorHandler = function( /*String*/data, ioArgs )
    {
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM In Error Handler, status is " + ioArgs.xhr.status,"RestClient");
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM responsetext " + ioArgs.xhr.responseText,"RestClient");
       var msg; 
       if ((ioArgs.xhr.status != 404) || ((ioArgs.xhr.status == 404) && (ioArgs.xhr.responseText.indexOf(self.restgatewayerror) >= 0)))  {
          if (!self.errormsg) {		     
             self.errormsg = true;
             var msg, msg2=null;
             if (ioArgs.xhr.responseText != null) {                 
                 var r1 = ioArgs.xhr.responseText.split(",");
                 for (var j=0; j<r1.length; j++) {
                      var r2 = r1[j].split(':');
                      if (r2[0] == '"errorMessage"') {
                          msg2 = r1[j].substr(16,r1[j].length-17);
                          break;
                      }
                 }
                 if (msg2 == null) {
                     msg2 = ioArgs.xhr.responseText;//responseText did not contain errorMessage, cannot format it
                 }
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:msg2} );
             } else {
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:ioArgs.xhr.status} );
             }
             dojo.publish("com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred");
             com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM publish errorOccurred topic","RestClient");
             
             com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(msg);             
          } 
        }
    };

    this._executeGet( url, okHandler, errorHandler, (loadCallback == null), this._bpmHeaders );

    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_searchBPM(" + searchAttributes + "," + offset + "," + size + ")=" + results.length ,"RestClient");
    com.ibm.bpm.social.util.common.Log.logEntry(2, "results: " + dojo.toJson( results ) ,"RestClient");
    return results;
  },

  //
  // -------- BPC
  //

  /**
   * Calculates the potential owners of the specified task.
   * <p>
   * Synchronic call.
   */
  queryTaskPotentialOwners: function( /*String*/ taskId, systemId, okHandler, errorHandler ) /*void*/
  {    
    var url = this.bpcServiceRoot + "/task/" + taskId + "/role/REASON_POTENTIAL_OWNER";
    if (systemId) {
        url+= "?systemID="+systemId;
    }
    this._executeGet( url, okHandler, errorHandler, true );
  },

  //
  // -------- Helpers
  //


  _executeGet: function( resourcePath, okHandler, errorHandler, sync, headers )
  {
  	com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    this._queryIndex++;

    //var encodedUrl = encodeURI( resourcePath );
    var encodedUrl = resourcePath;
    
    // Rewrite only if needed.
    var needsRewrite = (encodedUrl.indexOf("http") == 0);

    var url = (needsRewrite && this.iContext) ? this.iContext.io.rewriteURI( encodedUrl ) : encodedUrl;
    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_executeGet(" + url + ")" ,"RestClient");

    if ( !sync )
    {
      sync = false;
    }
    
    var request =
    {
      url: url,
      load: okHandler,
      error: errorHandler,
	  timeout: 5000,
      sync: sync
    };

    if (headers)
    {
      request.headers = headers;
    }

    dojo.xhrGet(request);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");
  },


  _getOperationIdString: function()
  {
    return "[" + this._queryIndex + "]";
  },


  /**
   * If url is not null, tries to remove the trailing '/' character and trim spaces.
   * If null, returns null.
   */
  _normalizeUrl: function( /*String*/ url ) /*String*/
  {
    var newUrl = url;
    if ( url )
    {
      url = dojo.trim( url );
      if( url.lastIndexOf("/") == url.length - 1 )
      {
        newUrl = url.substring(0, url.length - 1);
      }
    }

    return newUrl;
  },


  _log: function( message )
  {
    com.ibm.bpm.social.util.common.Log.log(2, message ,"RestClient");
  }
}
);


}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel");



dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel", null,
{
  constructor: function()
  {
    //this._log("constructor()");
  },
  
  
  /**
   * Searchs some repository for searchText value and returns the specified page.
   * <p>
   * Subclasses may override.
   * 
   * @return {cn,userID}[]
   */
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size ) /*{cn,userID}[]*/
  {
    return [];
  },
  
  
  /**
   * Here can return some initial suggestion, not necessarily bsacked up by VMM.
   * e.g., [{cn:"One",userID:1}, {cn:"Two",userID:2}, {cn:"Three",userID:3}]
   * <p>
   * Subclasses may override.
   * 
   * @param size The maximum number of suggestions.
   * @return {cn,userID}[]
   */
  getInitialSuggestions: function( /*int*/ size ) /*{cn,userID}[]*/
  {
    return [];
  },

  
  _removeWildcards: function( /*String*/ s ) /*String*/
  {
    var cleaned = new String();
    for(var i=0; i<s.length; i++) 
    {
      var c = s.charAt(i);
      if (c!="*")
      {
        cleaned += c;
      }
    }
    return cleaned;
  },  


  _log: function( message )
  {
    console.log( "AbstractModel: " + message );
  }  
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.BPMModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.BPMModel"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.BPMModel");




dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.BPMModel", com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel,
{
  _tasksIds: null,
  _initialSuggestions: null,
  
  constructor: function( /*{bpmServiceRoot: String, includeGroups: boolean, includeUsers: boolean}*/ params )
  {
    //this._log("constructor()");
    this._includeGroups = params.includeGroups ? (params.includeGroups == true) : false;	// Default false
    this._includeUsers = (params.includeUsers != null && params.includeUsers != undefined) 
    							? (params.includeUsers == true) 
    							: true;	// Default true
    this._restClient = new com.ibm.bpm.social.util.ui.userchooser.util.RestClient(params.bpmServiceRoot, null);
  },
  
  
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly ) /*{cn,userID}[]*/
  {
    return this._restClient.searchBPMByName( searchText, offset, size, this._includeGroups, this._includeUsers, assignTaskidFilter, collabTaskidFilter, namesonly );
  },
  
  getInitialSuggestions: function( /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly  ) /*{cn,userID}[]*/
  {
	if (!this._initialSuggestions) {
		this._initialSuggestions = new Object();
	}
	var key = size + "_" + assignTaskidFilter + "_" + collabTaskidFilter;
	if (!this._initialSuggestions[key]) {
		this._initialSuggestions[key] = this._restClient.searchBPMByName( "*", 0, size, this._includeGroups, this._includeUsers, assignTaskidFilter, collabTaskidFilter, namesonly );	
	}
	var rc = new Array();
	for (var i=0; i<this._initialSuggestions[key].length; i++) {
		rc[i] = this._initialSuggestions[key][i];
	}
	return rc;
  },

  _log: function( message )
  {
    console.log( "BPMModel: " + message );
  }  
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler"] = true;
// ModifyTaskHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler");
























dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler",
    null,
    {

      constructor: function(parameters){
        this._connects = [];
        this._wrappedElements = [];
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
        this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();

        this.init(parameters);
      },

      init: function init_ModifyTaskHandler(parameters){

        var task = parameters.task;

        if (typeof task.TKIID === "string") {
          this.mapping = com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.queryMapping;
        }
        else {
          this.mapping = com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.itemMapping;
        }
        this._dueDateTime = task[this.mapping["DUE"]];
        this._priority = task[this.mapping["PRIORITY"]];
        this._availableActions = parameters.availableActions;
        this._tkiid = task[this.mapping["TKIID"]];

        if (parameters.taskDisplayName) {
          this._taskDisplayName = parameters.taskDisplayName;
        }
        else {
          this._taskDisplayName = this._tkiid;
        }

        if (this._availableActions) {
          this.ifReassignBackToGroupOK = dojo.indexOf(this._availableActions, "ACTION_CANCELCLAIM") != -1;
          this.ifTransferToUserOK = dojo.indexOf(this._availableActions, "ACTION_REASSIGNTOUSER") != -1;
          this.ifTransferToGroupOK = dojo.indexOf(this._availableActions, "ACTION_REASSIGNTOGROUP") != -1;
          this.ifModifyPriorityOK = dojo.indexOf(this._availableActions, "ACTION_UPDATEPRIORITY") != -1;
          this.ifModifyDueDateOK = dojo.indexOf(this._availableActions, "ACTION_UPDATEDUEDATE") != -1;
        }
        else {
          this.ifReassignBackToGroupOK = true;
          this.ifTransferToUserOK = true;
          this.ifTransferToGroupOK = true;
          this.ifModifyPriorityOK = true;
          this.ifModifyDueDateOK = true;
        }

        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        this.ifOverallModifyOK =
          this.ifReassignBackToGroupOK || this.ifTransferToUserOK || this.ifModifyPriorityOK ||
            this.ifModifyDueDateOK;

        this.reassignBackToGroupSwitch = null;

        this._destroyUserChooser();
        this._currentOwner = null;
        if (task[this.mapping["OWNER"]]) {
          this._currentOwner = {
            value: task[this.mapping["OWNER"]],
            type: "user"
          };
        }
        else if (task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]]) {
          this._currentOwner = {
            value: task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]],
            type: "group"
          };
        }
        else if (task[this.mapping["ASSIGNED_TO_ROLE"]]) {
          // From Process Portal: determine group display name
          // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
          // So we are just using the last two '_'s to pull the PG name
          var assignedToRole = task[this.mapping["ASSIGNED_TO_ROLE"]];
          if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
            var roleName = assignedToRole;
            var displayName = roleName.substring(0, roleName.lastIndexOf("_"));
            displayName = displayName.substring(0, displayName.lastIndexOf("_"));
            assignedToRole = displayName;
          }
          this._currentOwner = {
            value: assignedToRole,
            type: "group"
          };
        }
        else {
          var error = new Error("Task does not have an owner or a group. ID: " + task[this.mapping["TKIID"]]);
          com.ibm.bpm.social.util.common.Log.consoleError(error.message);
          throw error;
        }
      },

      /**
       * a value in a field has changed, check validation to enable/disable Save button
       */
      _onValueChanged: function _onValueChanged_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var valid = true;

        if (this.ifModifyPriorityOK && this._priorityFilteringSelect) {
          // !! because isValid() returns false or selected option ?!? (Dojo 1.6.1)
          valid = valid && !!this._priorityFilteringSelect.isValid();
        }
        if (this.ifModifyDueDateOK && this.dateTimePicker) {
          valid = valid && this.dateTimePicker.isValid();
        }
        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        if (this.ifTransferToUserOK && this.userChooser &&
          !this.userChooser.chooserInput.disabled) {
          valid =
            valid &&
              (this.userChooser.chooserInput.value.length === 0 || this.userChooser.getChoice() !== null || dojo.hasClass(
                this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help"));
        }

        com.ibm.bspace.common.util.ui.Button.setDisabled(this._saveButton, !valid);

        com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
      },

      destroy: function destroy_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._destroyDialog();
        this._restHandler = null;
        this._errorHandler = null;
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Destroys the controls in the dialog and the dialog itself.
       */
      _destroyDialog: function _destroyDialog_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        while (this._connects && this._connects.length > 1) {
          dojo.disconnect(this._connects.pop());
        }

        if (this._wrappedElements.length > 0) {
          com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements);
          this._wrappedElements = [];
        }

        if (this._priorityFilteringSelect) {
          this._priorityFilteringSelect.destroy();
          this._priorityFilteringSelect = null;
        }

        this._destroyUserChooser();

        if (this.dateTimePicker) {
          this.dateTimePicker.destroy();
          this.dateTimePicker = null;
        }

        if (this.dialog) {
          this.dialog.destroyRecursive();
          this.dialog = null;
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _destroyUserChooser: function _destroyUserChooser_ModifyTaskHandler(){
        if (this.userChooser) {
          this.userChooser.destroy();
          this.userChooser = null;
        }
      },

      /**
       * Assigns the task back to the group, this is the public method that does fire the onTaskModified event
       */
      reassignBackToGroup: function reassignBackToGroup_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        var success = this._reassignBackToGroup();
        if (success) {
          this.onTaskModified(null, null, true);
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Assigns the task back to the group, this is the internal method that does not fire the onTaskModified
       * event
       * 
       * @returns {Boolean} true if the task was successfully reassigned
       */
      _reassignBackToGroup: function reassignBackToGroup_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.cancelTask(this._tkiid, function(){
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            403: {
              key: "KEY_TaskModifyCancelNotAvailable",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            409: {
              key: "KEY_TaskModifyCancelNotAvailable",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            404: {
              key: "KEY_TaskModifyCancelNotExisting",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          myself._errorHandler.showError(data, ioArgs, statusCodeMap, null, function(){
            myself.onError();
          });
        });

        com.ibm.bpm.social.util.common.Log.logExit(2, "result=" + success);

        return success;
      },

      /**
       * Assign task to a specific group
       * 
       * @param group {void} group name to assign the task to
       */
      _assignTaskToGroup: function _assignTaskToGroup_ModifyTaskHandler(group){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.assignTaskToGroup(this._tkiid, function(response){
          myself._currentOwner = {
            value: group,
            type: "group"
          };
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            400: {
              key: "KEY_transferGroupDoesNotExist",
              parameters: {
                1: group
              }
            },
            404: {
              key: "KEY_TransferTask"
            }
          };
          var errorCodeMap = {
            "CWTBG0065E": {
              key: "KEY_transferGroupDoesNotExist"
            },
            "CWTBG0048E": {
              key: "KEY_delegateNotExistingTask"
            }
          };

          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        }, group);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return success;
      },

      /**
       * Assign the task to a specific user
       * 
       * @param user {void} user id to the assign the task to
       */
      _assignTaskToUser: function _assignTaskToUser_ModifyTaskHandler(user){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.assignTaskToUser(this._tkiid, function(response){
          myself._currentOwner = {
            value: user,
            type: "user"
          };
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            400: {
              key: "KEY_TransferUserDoesNotExist",
              parameters: {
                1: user
              }
            },
            404: {
              key: "KEY_delegateNotExistingTask"
            }
          };
          var errorCodeMap = {
            "CWTBG0065E": {
              key: "KEY_TransferUserDoesNotExist",
              parameters: {
                1: user
              }
            },
            "CWTKA0049E": {
              key: "KEY_delegationNotSupported"
            },
            "CWTKA0110E": {
              key: "KEY_delegationForUnauthUser"
            }
          };

          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        }, user);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return success;
      },

      _updateTaskProperties: function _updateTaskProperties_ModifyTaskHandler(newPriority, newDueDateTime){
        var myself = this;
        var hasError = false;

        var errorFunction = function(data, ioArgs){
          hasError = true;

          var statusCodeMap = {
            400: {
              key: "KEY_UpdateTaskError",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            401: {
              key: "KEY_UpdateTaskNotAuthorized",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            404: {
              key: "KEY_UpdateNotExistingTask",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          var errorCodeMap = {
            "CWTBG0073E": {
              key: "KEY_UpdateTaskError",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        };

        // the API does not allow to update both with one API call

        // update due date
        if (newDueDateTime) {
          this._restHandler.updateTaskProperties(this._tkiid, function(response){
            // TODO I do not understand why we need that. The Class will be destroyed afterwards
            if (newDueDateTime) {
              com.ibm.bpm.social.util.common.Log.log(1, "newDueDateTime: " + newDueDateTime, "ModifyTaskHandler",
                "_updateTaskProperties");
              myself._dueDateTime = newDueDateTime;

            }
          }, errorFunction, null, newDueDateTime);
        }

        if (!hasError && newPriority) {
          this._restHandler.updateTaskProperties(this._tkiid, function(response){
            // TODO I do not understand why we need that. The Class will be destroyed afterwards
            if (newPriority) {
              com.ibm.bpm.social.util.common.Log.log(1, "newPriority: " + newPriority, "ModifyTaskHandler",
                "_updateTaskProperties");
              myself._priority = newPriority;
            }
          }, errorFunction, newPriority, null);
        }
        return !hasError;
      },

      showDialog: function showDialog_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // Make sure that the old dialog was destroyed
        this._destroyDialog();

        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, ["modifyTask"]);

        this.dialog = new com.ibm.bspace.common.util.ui.Dialog({
          templateString: '<div dojoAttachPoint="content" ></div>',
          title: this._nlsResources["task_dialog_modifyTask_title"]
        });

        var hideHandle = dojo.connect(this.dialog.popup, "onHide", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, ["modifyTask"]);
          dojo.disconnect(hideHandle);
          this.onHide(this);
        }));

        dojo.addClass(this.dialog.content, "bpm-task-modify-task-dialog");

        var content = this.dialog.content;

        // Description
        var dSection = this._createDescriptionSection(content);

        var formSections = document.createElement("div");
        content.appendChild(formSections);
        dijit.setWaiRole(formSections, "form");
        dijit.setWaiState(formSections, "labelledby", dSection.id);

        // Assignee section
        this._createAssigneeSection(formSections);

        // Priority section
        this._createPrioritySection(formSections);

        // Due date and time section
        this._createDueDateTimeSection(formSections);

        // Create save button
        var saveButton = document.createElement("button");
        saveButton.innerHTML = this._nlsResources["task_dialog_modifyTask_saveButton"]; // Save";
        saveButton.title = this._nlsResources["task_dialog_modifyTask_saveButtonTooltip"];
        if (document.body.dir == 'rtl') {
          dojo.style(saveButton, "float", "left");
        }
        else {
          dojo.style(saveButton, "float", "right");
        }

        saveButton.onclick = dojo.hitch(this, this.onSave);
        var buttonSection = document.createElement("div");
        dojo.style(buttonSection, "background", "white");
        buttonSection.appendChild(saveButton);
        this._saveButton = com.ibm.bspace.common.util.ui.Button.wrapElement(saveButton);
        this._wrappedElements.push(this._saveButton);
        formSections.appendChild(buttonSection);

        // check if the dialog defaults are valid. Else we must disable the save button
        this._onValueChanged();

        this.dialog.show();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _createDescriptionSection: function _createDescriptionSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var descriptionSection = document.createElement("div");
        this._styleSection(descriptionSection);
        dojo.style(descriptionSection, "maxWidth", "35em");
        if (this._taskDisplayName === this._tkiid) {
          descriptionSection.innerHTML = this._nlsResources["task_dialog_modifyTask_description"];
        }
        else {
          descriptionSection.innerHTML =
            dojo.string.substitute(this._nlsResources["task_dialog_modifyTask_description_ext"], {
              1: "<b>" + this._taskDisplayName + "</b>"
            });
        }
        descriptionSection.title = " ";
        descriptionSection.id = this.dialog.id + "_description";
        dijit.setWaiRole(descriptionSection, "heading");

        parentNode.appendChild(descriptionSection);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return descriptionSection;
      },

      /*
       * Returns null, if assignee hasn't changed.
       */
      _getNewAssignee: function _getNewAssignee_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.userChooser) {
          var choice = this.userChooser.getChoice();
          if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
            return choice.userName;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createAssigneeSection: function _createAssigneeSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var self = this;

        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        if (this.ifReassignBackToGroupOK || this.ifTransferToUserOK) {

          var reassignBackToGroup, reassignLabel;

          var assigneeSection = document.createElement("div");
          this._styleSection(assigneeSection);

          var assigneeLabel = document.createElement("div");
          assigneeLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_assigneeSectionTitle"]; // "Task Assignment";
          assigneeLabel.title = this._nlsResources["task_dialog_modifyTask_assigneeSectionTooltip"];
          assigneeLabel.id = this.dialog.id + "_assigneeLabel";
          this._styleSectionTitle(assigneeLabel);
          assigneeSection.appendChild(assigneeLabel);

          // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
          // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
          // support the assignment to a specific group but only the reassignment to the last group. Therefore 
          // do not use this.ifTransferToGroupOK.
          if (this.ifReassignBackToGroupOK && !this.ifTransferToUserOK) {
            // Create checkbox pattern: Just one Checkbox "Reassign back to group"
            // -> No UserChooser 

            reassignBackToGroup = document.createElement("input");
            reassignBackToGroup.name = "ModifyTaskHandler_Assignee";
            reassignBackToGroup.id = this.dialog.id + "_checkbox1";
            reassignBackToGroup.type = "checkbox";
            reassignBackToGroup.tabIndex = "0";
            reassignBackToGroup.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
            assigneeSection.appendChild(reassignBackToGroup);
            this._wrappedElements.push(com.ibm.bspace.common.util.ui.CheckBox.wrapElement(reassignBackToGroup));

            reassignLabel = document.createElement("label");
            reassignLabel.id = this.dialog.id + "_checkbox1_label";
            reassignLabel.htmlFor = reassignBackToGroup.nextSibling.id;
            reassignLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_reassignBackToGroup"];
            reassignLabel.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
            assigneeSection.appendChild(reassignLabel);
            this.reassignBackToGroupSwitch = reassignBackToGroup;
            dijit.setWaiState(reassignBackToGroup, "labelledby", reassignLabel.id);
          }
          else {

            // transfer is  allowed, create a user chooser
            var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
              // includeGroups: this.ifTransferToGroupOK,
              includeGroups: false,
              includeUsers: this.ifTransferToUserOK
            });

            // Create UserChooser 
            self._ucClickedFirstTime = false;
            var args = {
              searchMinInputLength: 0,
              ucModel: ucModel,
              _onUCFocus: function(e){
                if (!self._ucClickedFirstTime) {
                  self._ucClickedFirstTime = true;
                  dojo.removeClass(self.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
                  self.userChooser.chooserInput.value = "";
                }
                com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);
              },
              _onUCBlur: function(e){
                var valid = (self.userChooser.chooserInput.value.length < 0 || self.userChooser.getChoice() !== null);
                if (!valid) {
                  self.userChooser.chooserInput.value = "";
                  self.userChooser._hideResults();
                }
                dojo.addClass(self.userChooser.noUsersFoundDiv, "hidden");
              },
              // defect 103438 
              // performance problem. The user list is loaded twice. 
              // The list will be loaded with the first click.
              preloadInitialSuggestions: false
            };
            this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
            this.userChooser.onChangeHandler = dojo.hitch(this, this._onValueChanged);

            dijit.setWaiState(self.userChooser.chooserInput, "label",
              this._nlsResources["task_dialog_modifyTask_assigneeSectionTitle"]);

            this.userChooser.chooserInput.title = this._nlsResources["task_dialog_modifyTask_assignToTooltip"];
            this.userChooser.chooserInput.tabIndex = "0";
            dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");

            // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
            this.userChooser.setAssignTaskidFilter(this._tkiid);

            if (this.ifReassignBackToGroupOK) {
              // need option pattern: 2 Radio buttons, one for the UserChooser and one for assign_back_to_group
              var table = document.createElement("table");
              dijit.setWaiRole(table, "presentation");
              dijit.setWaiState(table, "describedby", assigneeLabel.id);
              dojo.style(table, "borderCollapse", "collapse");

              var row = table.insertRow(0);
              dijit.setWaiRole(row, "radiogroup");

              // First radio button
              var td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");

              var transferTo = document.createElement("input");
              transferTo.name = "ModifyTaskHandler_Assignee";
              transferTo.type = "radio";
              transferTo.id = this.dialog.id + "_radio1";

              // Enable userChooser when radio button was clicked
              this._connects.push(dojo.connect(transferTo, "onclick", function(){
                if (transferTo.checked) {
                  self.userChooser.chooserInput.disabled = false;
                  dijit.setWaiState(self.userChooser.chooserInput, "disabled", "false");
                }
                self._onValueChanged();
              }));

              td.appendChild(transferTo);
              this._wrappedElements.push(com.ibm.bspace.common.util.ui.RadioButton.wrapElement(transferTo));
              com.ibm.bspace.common.util.ui.RadioButton.setChecked(transferTo, true);
              dijit.setWaiRole(transferTo.nextSibling, "radio");
              dijit.setWaiState(transferTo.nextSibling, "label",
                this._nlsResources["task_dialog_modifyTask_assignUser-label"]);
              dijit.setWaiState(transferTo.nextSibling, "controls", this.userChooser.chooserInput.id);
              transferTo.nextSibling.tabIndex = "0";


//              var radio1Label = document.createElement("label");
//              radio1Label.htmlFor = transferTo.nextSibling.id;
//              radio1Label.innerHTML = this._nlsResources["task_dialog_modifyTask_assignUser-label"];
//              radio1Label.style.display = "none";
//              td.appendChild(radio1Label);

              // User Chooser
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              dijit.setWaiState(self.userChooser.chooserInput, "label",
                this._nlsResources["task_dialog_modifyTask_assignUser-label"]);

              td.appendChild(this.userChooser.domNode);

//              var ucLabel = document.createElement("label");
//              ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
//              ucLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_User-label"];
//              ucLabel.style.display = "none";
//              td.appendChild(ucLabel);

              reassignBackToGroup = document.createElement("input");
              reassignBackToGroup.name = "ModifyTaskHandler_Assignee";
              reassignBackToGroup.type = "radio";
              reassignBackToGroup.id = this.dialog.id + "_reassignBackToGroup";
              reassignBackToGroup.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
              dijit.setWaiRole(transferTo, "radio");

              // Disable userChooser when radio button was clicked
              this._connects.push(dojo.connect(reassignBackToGroup, "onclick", function(){
                if (reassignBackToGroup.checked) {
                  self.userChooser.chooserInput.disabled = true;
                  dijit.setWaiState(self.userChooser.chooserInput, "disabled", "true");
                }
                self._onValueChanged();
              }));

              // Radio button 2
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              td.appendChild(reassignBackToGroup);
              this._wrappedElements.push(com.ibm.bspace.common.util.ui.RadioButton.wrapElement(reassignBackToGroup));
              reassignBackToGroup.nextSibling.tabIndex = "0";
              this.reassignBackToGroupSwitch = reassignBackToGroup;

              // Label for Radio 2
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              reassignLabel = document.createElement("label");
              reassignLabel.id = this.dialog.id + "_checkbox2_label";
              reassignLabel.htmlFor = reassignBackToGroup.nextSibling.id;
              reassignLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_reassignBackToGroup"];
              reassignLabel.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
              td.appendChild(reassignLabel);
              dijit.setWaiState(reassignBackToGroup, "labelledby", reassignLabel.id);

              assigneeSection.appendChild(table);
            }
            else {
              assigneeSection.appendChild(this.userChooser.domNode);
            }
          }

          parentNode.appendChild(assigneeSection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Returns null, if priority hasn't changed.
       */
      _getNewPriority: function _getNewPriority_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.prioritySelection) {
          var selectedPriority =
            (com.ibm.bpm.social.util.common.Util.isIPad ? this.prioritySelection.value : this._priorityFilteringSelect
              .get("value"));
          if (com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(selectedPriority) !== com.ibm.bpm.social.widgets.task.util.Constants
            .propertyKeyMapper(this._priority)) {
            return selectedPriority;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createPrioritySection: function _createPrioritySection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.ifModifyPriorityOK) {
          var prioritySection = document.createElement("div");
          prioritySection.title = this._nlsResources["task_dialog_modifyTask_prioritySectionTooltip"];
          this._styleSection(prioritySection);

          var currentPriority = com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(this._priority);

          var priorityLabel = document.createElement("div");
          priorityLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_prioritySectionTitle"];
          this._styleSectionTitle(priorityLabel);
          priorityLabel.id = this.dialog.id + "_priority";
          prioritySection.appendChild(priorityLabel);

          this.prioritySelection = document.createElement("select");

          var veryLow = document.createElement("option");
          veryLow.value = "50";
          veryLow.innerHTML = this._nlsResources["KEY_PrioVeryLow"]; // "Very Low";
          if (currentPriority === "VeryLow") {
            veryLow.selected = "selected";
          }
          this.prioritySelection.appendChild(veryLow);

          var low = document.createElement("option");
          low.value = "40";
          low.innerHTML = this._nlsResources["KEY_PrioLow"]; // "Low";
          if (currentPriority === "Low") {
            low.selected = "selected";
          }
          this.prioritySelection.appendChild(low);

          var medium = document.createElement("option");
          medium.value = "30";
          medium.innerHTML = this._nlsResources["KEY_PrioMedium"]; // "Medium";
          if (currentPriority === "Medium") {
            medium.selected = "selected";
          }
          this.prioritySelection.appendChild(medium);

          var high = document.createElement("option");
          high.value = "20";
          high.innerHTML = this._nlsResources["KEY_PrioHigh"]; // "High";
          if (currentPriority === "High") {
            high.selected = "selected";
          }
          this.prioritySelection.appendChild(high);

          var veryHigh = document.createElement("option");
          veryHigh.value = "10";
          veryHigh.innerHTML = this._nlsResources["KEY_PrioVeryHigh"]; // "Very High";
          if (currentPriority === "VeryHigh") {
            veryHigh.selected = "selected";
          }
          this.prioritySelection.appendChild(veryHigh);

          if (com.ibm.bpm.social.util.common.Util.isIPad) {
            // on iPad use the native control
            prioritySection.appendChild(this.prioritySelection);
            this._connects.push(dojo.connect(this.prioritySelection, "onchange", this, this._onValueChanged));
          }
          else {
            this._priorityFilteringSelect = new com.ibm.bspace.common.util.ui.FilteringSelect({
              "intermediateChanges": true
            }, this.prioritySelection);
            this._connects.push(dojo.connect(this._priorityFilteringSelect, "onChange", this, this._onValueChanged));
            prioritySection.appendChild(this._priorityFilteringSelect.domNode);
            this._priorityFilteringSelect.textbox.title =
              this._nlsResources["task_dialog_modifyTask_prioritySectionTooltip"];
            dijit.setWaiState(this._priorityFilteringSelect.textbox, "label",
              this._nlsResources["task_dialog_modifyTask_prioritySectionTitle"]);
          }

          parentNode.appendChild(prioritySection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Returns null, if dueDateTime hasn't changed.
       */
      _getNewDueDateTime: function _getNewDueDateTime_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.dateTimePicker && this.dateTimePicker.isValid()) {
          // Ignore seconds, because you cannot define seconds via dateTimePicker.
          var differenceInSeconds =
            dojo.date.difference(this.dateTimePicker.getDateTime(), dojo.date.stamp.fromISOString(this._dueDateTime),
              "second");
          if (Math.abs(differenceInSeconds) >= 60) {
            var dateString = dojo.date.stamp.toISOString(this.dateTimePicker.getDateTime(), {
              zulu: true
            });
            return dateString;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createDueDateTimeSection: function _createDueDateTimeSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.ifModifyDueDateOK) {
          var dueDateSection = document.createElement("div");
          dueDateSection.title = this._nlsResources["task_dialog_modifyTask_DueSectionTooltip"]; // "Due Date";
          this._styleSection(dueDateSection);

          var dueDateLabel = document.createElement("div");
          dueDateLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_DueSectionTitle"]; // "Due Date";
          this._styleSectionTitle(dueDateLabel);

          dueDateSection.appendChild(dueDateLabel);

          this.dateTimePicker = new com.ibm.bpm.social.widgets.task.util.DateTime({
            dateTime: dojo.date.stamp.fromISOString(this._dueDateTime),
            showNoLabels: true
          });

          this._connects.push(dojo.connect(this.dateTimePicker, "onChange", this, this._onValueChanged));

          dueDateSection.appendChild(this.dateTimePicker.domNode);

          parentNode.appendChild(dueDateSection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      onTaskModified: function onTaskModified_ModifyTaskHandler(dueDateTime, priority, assigneeChanged, newAssignee){
        com.ibm.bpm.social.util.common.Log.log(1, "Event onTaskModified fired: dueDateTime=" + dueDateTime + ", priority=" +
          priority + ", assigneeChanged=" + assigneeChanged + ", newAssignee=" + newAssignee);
      },

      /**
       * 
       * @type {void}
       */
      onSave: function onSave_ModifyTaskHandler(e){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var newDueDateTime = this._getNewDueDateTime();
        var newPriority = this._getNewPriority();
        var newAssignee = this._getNewAssignee();

        var updatePropertiesSuccess = false;
        var updateAssigneeSuccess = false;

        // assignee change needs to be handled first to avoid adding a task to the list which does no longer apply to the list
        if (this.reassignBackToGroupSwitch && this.reassignBackToGroupSwitch.checked) {
          com.ibm.bpm.social.util.common.Log.log(1, "assigning back to group");
          updateAssigneeSuccess = this._reassignBackToGroup();
        }
        else {
          if (newAssignee) {
            com.ibm.bpm.social.util.common.Log.log(1, "newAssignee: " + newAssignee);
            updateAssigneeSuccess = this._assignTaskToUser(newAssignee);
            if (!updateAssigneeSuccess) {
              newAssignee = null;
            }
          }
        }

        if (newDueDateTime || newPriority) {
          updatePropertiesSuccess = this._updateTaskProperties(newPriority, newDueDateTime);
          if (!updatePropertiesSuccess) {
            newDueDateTime = null;
            newPriority = null;
          }
        }

        if (updatePropertiesSuccess || updateAssigneeSuccess) {
          this.onTaskModified(newDueDateTime, newPriority, updateAssigneeSuccess, newAssignee);
        }

        this.dialog.hide();

        com.ibm.bpm.social.util.common.Log.logExit(1);
        return;
      },

      _styleSection: function _styleSection_ModifyTaskHandler(node){
        dojo.style(node, "padding", "10px 0");
        dojo.style(node, "background", "white");
      },

      _styleSectionTitle: function _styleSectionTitle_ModifyTaskHandler(node){
        dojo.style(node, "fontWeight", "bold");
        dojo.style(node, "padding", "0 0 10px 0");
      },

      onError: function onError_ModifyTaskHandler(){},

      onHide: function onHide_ModifyTaskHandler(modifyTaskHandler){}

    });


com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.queryMapping = {
  "DUE": "DUE",
  "PRIORITY": "PRIORITY",
  "TKIID": "TKIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.itemMapping = {
  "DUE": "dueTime",
  "PRIORITY": "priority",
  "TKIID": "tkiid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.PropertyFormatter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.PropertyFormatter"] = true;
// PropertyFormatter.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

// **************************************
//
// Version: 1.40 11/10/18 13:26:26
//
// **************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.PropertyFormatter");







// TODO ??? dojo.require("com.ibm.bpm.social.widgets.task.util.SametimeService");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.PropertyFormatter",
    null,
    {

      STATE_STATUS_MAP: {
//        "KEY_TASK_STATE_INACTIVE": "KEY_StateInActive",
//        "KEY_TASK_STATE_READY": "KEY_StateReady",
//        "KEY_TASK_STATE_RUNNING": "KEY_StateRunning",
//        "KEY_TASK_STATE_FINISHED": "KEY_StateSuccess",
//        "KEY_TASK_STATE_FAILED": "KEY_StateFailed",
//        "KEY_TASK_STATE_TERMINATED": "KEY_StateTerminated",
//        "KEY_TASK_STATE_CLAIMED": "KEY_StateRunning",
//        "KEY_TASK_STATE_EXPIRED": "KEY_StateFailed",
//        "KEY_TASK_STATE_FORWARDED": "KEY_StateForwarded",
//        "KEY_TASK_STATE_STARTED": "KEY_PreDraft", // For the details tab of a task template 
//        "KEY_TASK_STATE_STOPPED": "KEY_StateStopped", // For the details tab of a task template
//        "KEY_TASK_STATE_SUSPENDED_BY_PARENT": "KEY_StateSuspendedByParent", // WLE only
//        "KEY_ESCALATION_ACTIVATION_STATE_READY": "KEY_StateReady",
//        "KEY_ESCALATION_ACTIVATION_STATE_RUNNING": "KEY_StateRunning",
//        "KEY_ESCALATION_ACTIVATION_STATE_CLAIMED": "KEY_StateRunning",
//        "KEY_ESCALATION_ACTIVATION_STATE_WAITING_FOR_SUBTASK": "KEY_StateWaiting",
//        "KEY_ESCALATION_AT_LEAST_EXPECTED_STATE_CLAIMED": "KEY_StateRunning",
//        "KEY_ESCALATION_AT_LEAST_EXPECTED_STATE_ENDED": "KEY_StateFinished",
//        "KEY_ESCALATION_AT_LEAST_EXPECTED_STATE_SUBTASKS_COMPLETED": "KEY_ESCALATION_AT_LEAST_EXPECTED_STATE_SUBTASKS_COMPLETED"
      },

      ICON_MAP: {
//        TASK_STATE_INACTIVE: "taskStateInactive.gif",
//        TASK_STATE_READY: "taskStateReady.gif",
//        TASK_STATE_RUNNING: "taskSateRunning.gif",
//        TASK_STATE_FINISHED: "taskStateFinished.gif",
//        TASK_STATE_FAILED: "taskStateFailed.gif",
//        TASK_STATE_TERMINATED: "taskStateTerminated.gif",
//        TASK_STATE_CLAIMED: "taskSateRunning.gif",
//        TASK_STATE_EXPIRED: "taskStateFailed.gif",
//        TASK_STATE_FORWARDED: "taskStateForwarded.gif",
//        TASK_STATE_SUSPENDED_BY_PARENT: "processStateSuspended.gif",
//        TASK_KIND_ADMINISTRATIVE: "taskKindAdministrative.gif",
//        TASK_KIND_HUMAN: "taskKindHuman.gif",
//        TASK_KIND_ORIGINATING: "taskKindOriginating.gif",
//        TASK_KIND_PARTICIPATING: "taskKindParticipating.gif",
//        PROCESS_INSTANCE_STATE_DELETED: "processStateDeleted.gif",
//        PROCESS_INSTANCE_STATE_READY: "processStateReady.gif",
//        PROCESS_INSTANCE_STATE_RUNNING: "processStateRunning.gif",
//        PROCESS_INSTANCE_STATE_FINISHED: "processStateFinished.gif",
//        PROCESS_INSTANCE_STATE_COMPENSATING: "processStateCompensated.gif",
//        PROCESS_INSTANCE_STATE_FAILED: "processStateFailed.gif",
//        PROCESS_INSTANCE_STATE_TERMINATED: "processStateFailed.gif",
//        PROCESS_INSTANCE_STATE_COMPENSATED: "processStateCompensated.gif",
//        PROCESS_INSTANCE_STATE_TERMINATING: "processStateFailed.gif",
//        PROCESS_INSTANCE_STATE_FAILING: "processStateFailed.gif",
//        PROCESS_INSTANCE_STATE_INDOUBT: "processStateInDoubt.gif",
//        PROCESS_INSTANCE_STATE_SUSPENDED: "processStateSuspended.gif",
//        PROCESS_INSTANCE_STATE_COMPENSATION_FAILED: "processStateCompensationFailed.gif",
//        TASK_TEMPL_STATE_STARTED: "taskTemplStateStarted.gif",
//        TASK_TEMPL_STATE_STOPPED: "taskTemplStateStopped.gif",
//        ESCALATION_STATE_ESCALATED: "escalationStateEscalated.gif",
//        ESCALATION_STATE_INACTIVE: "escalationStateInactive.gif",
//        ESCALATION_STATE_SUBTASKS_COMPLETED: "escalationStateSubtasksCompleted.gif",
//        ESCALATION_STATE_SUPERFLUOUS: "escalationStateSuperfluous.gif",
//        ESCALATION_STATE_WAITING: "escalationStateWaiting.gif",
//        ESCALATION_ACTION_CREATE_EVENT: "escalationActionCreateEvent.gif",
//        ESCALATION_ACTION_CREATE_WORK_ITEM: "escalationActionGrantReadAccess.gif",
//        ESCALATION_ACTION_SEND_EMAIL: "escalationActionSendEMail.gif",
//        ESCALATION_ACTIVATION_STATE_READY: "taskStateReady.gif",
//        ESCALATION_ACTIVATION_STATE_RUNNING: "taskSateRunning.gif",
//        ESCALATION_ACTIVATION_STATE_CLAIMED: "taskSateRunning.gif",
//        ESCALATION_ACTIVATION_STATE_WAITING_FOR_SUBTASK: "waiting.gif",
//        ESCALATION_AT_LEAST_EXPECTED_STATE_CLAIMED: "taskSateRunning.gif",
//        ESCALATION_AT_LEAST_EXPECTED_STATE_ENDED: "taskStateFinished.gif",
//        ESCALATION_AT_LEAST_EXPECTED_STATE_SUBTASKS_COMPLETED: "escalationStateSubtasksCompleted.gif",
//        WORKBASKET_TYPE_GROUP: "workbasketTypeGroup.gif",
//        WORKBASKET_TYPE_INDIVIDUAL: "workbasketTypeIndividual.gif",
//        WORKBASKET_TYPE_TOPIC: "workbasketTypeTopic.gif",
//        WORKBASKET_TYPE_CLEARANCE: "workbasketTypeClearance.gif"
      },

      constructor: function(parameters){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

        if (parameters) {
          this._queueConfig = parameters.queueConfig;
          if (typeof parameters.attributesForQueryTableFunc != "undefined") {
            this._attributesForQueryTableFunc = parameters.attributesForQueryTableFunc;
          }
          if (parameters.config) {
            this._prioritySettings = parameters.config.priorities;
            this._type = parameters.config.type;
          }

          this._widgetId = parameters.widgetId;
        }
      },

      getIconFormatter: function(type){
        var self = this;
        switch (type) {
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_BOOLEAN :
            return dojo.hitch(this, this.booleanIconFormatter);
            break;
          case "TASK.KIND" :
          case "TASK.STATE" :
          case "TASK_TEMPL.KIND" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              return self.genericIconFormatter("TASK", propertyValue);
            };
            break;
          case "PROCESS_INSTANCE.STATE" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              return self.genericIconFormatter("PROCESS_INSTANCE", propertyValue);
            };
            break;
          case "TASK_TEMPL.STATE" :
          case "PROCESS_TEMPLATE.STATE" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              return self.genericIconFormatter("TASK_TEMPL", propertyValue);
            };
            break;
          case "PROCESS_TEMPLATE.EXECUTION_MODE" :
            return function(propertyValue){
              if ("EXECUTION_MODE_LONG_RUNNING" == propertyValue) {
                return self.booleanIconFormatter(true);
              }
              else {
                return self.booleanIconFormatter(false);
              }
            };
            break;
//          case "ESCALATION.ACTION" :
//          case "ESCALATION.STATE" :
//          case "ESCALATION.ACTIVATION_STATE" :
//          case "ESCALATION.AT_LEAST_EXP_STATE" :
//          case "ESC_TEMPL.ACTION" :
//          case "ESC_TEMPL.ACTIVATION_STATE" :
//          case "ESC_TEMPL.AT_LEAST_EXP_STATE" :
//          return function(propertyValue){
//            // non-trivial closure needed for type spezialization
//            return self.genericIconFormatter("ESCALATION", propertyValue);
//          };
//          break;
          default:
            // Other types do not support an icon.
            break;
        }
        return null;
      },

      /**
       * Returns a hash {url: url, title: title}
       * 
       * @param {Object} type
       * @param {Object} displayName
       */
      getFormatter: function(type, verboseBoolean, displayName){
        var self = this;
        switch (type) {
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_USER :
            // TODO ???
//        if (com.ibm.bpm.social.widgets.task.util.SametimeService.getInstance().isRunning()) {
//          return dojo.hitch(this, this.liveNameFormatter);
//        }
//        else {
            return null;
//        }
            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_DATE_CONFIG :
            return dojo.hitch(this, this.dateFormatter);
            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXTCHECK_CONFIG :
            return dojo.hitch(this, this.textcheckFormatter);
            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_TEXT_CONFIG :
            return dojo.hitch(this, this.textFormatter);
            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_NUMBER_CONFIG :
          case "numericPriority" :
            return dojo.hitch(this, this.numberFormatter);
            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG :
          case "defaultPriority" :
            return dojo.hitch(this, this.priorityFormatter);
            break;
//          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PENDING_CONFIG :
//            if (verboseBoolean !== false) {
//              return dojo.hitch(this, this.pendingVerboseFormatter);
//            }
//            else {
//              return dojo.hitch(this, this.pendingFormatter);
//            }
//            break;
          case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_BOOLEAN :
            return function(propertyValue){
              // non-trivial closure needed for display name specification
              var info = self.booleanIconFormatter(propertyValue);
              return self._createDomNode(info, displayName, verboseBoolean);
            };
            break;
          case "PROCESS_TEMPLATE.EXECUTION_MODE" :
            return function(propertyValue){
              var info = null;
              if ("EXECUTION_MODE_LONG_RUNNING" == propertyValue) {
                info = self.booleanIconFormatter(true);
              }
              else {
                info = self.booleanIconFormatter(false);
              }
              return self._createDomNode(info, displayName, verboseBoolean);
            };
            break;
          case "TASK.KIND" :
          case "TASK.STATE" :
          case "TASK_TEMPL.KIND" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              var info = self.genericIconFormatter("TASK", propertyValue);
              return self._createDomNode(info);
            };
            break;
//          case "TASK.IS_ESCALATED" :
//            if (verboseBoolean !== false) {
//              return dojo.hitch(this, this.escalatedVerboseFormatter);
//            }
//            else {
//              return dojo.hitch(this, this.escalatedFormatter);
//            }
//            break;
//          case "TASK.SUSPENDED" :
//            if (verboseBoolean !== false) {
//              return dojo.hitch(this, this.suspendedVerboseFormatter);
//            }
//            else {
//              return dojo.hitch(this, this.suspendedFormatter);
//            }
//            break;
          case "PROCESS_INSTANCE.STATE" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              var info = self.genericIconFormatter("PROCESS_INSTANCE", propertyValue);
              return self._createDomNode(info);
            };
            break;
          case "TASK_TEMPL.STATE" :
          case "PROCESS_TEMPLATE.STATE" :
            return function(propertyValue){
              // non-trivial closure needed for type spezialization
              var info = self.genericIconFormatter("TASK_TEMPL", propertyValue);
              return self._createDomNode(info);
            };
            break;
//          case "ESCALATION.ACTION" :
//          case "ESCALATION.STATE" :
//          case "ESCALATION.ACTIVATION_STATE" :
//          case "ESCALATION.AT_LEAST_EXP_STATE" :
//          case "ESC_TEMPL.ACTION" :
//          case "ESC_TEMPL.ACTIVATION_STATE" :
//          case "ESC_TEMPL.AT_LEAST_EXP_STATE" :
//            return function(propertyValue){
//              // non-trivial closure needed for type spezialization
//              var info = self.genericIconFormatter("ESCALATION", propertyValue);
//              return self._createDomNode(info);
//            };
//            break;
//          case "WORK_BASKET.TYPE" :
//            return function(propertyValue){
//              // non-trivial closure needed for type spezialization
//              var info = self.genericIconFormatter("WORKBASKET", propertyValue);
//              return self._createDomNode(info);
//            };
//            break;
          case "usergroup" :
            return dojo.hitch(this, this.userGroupFormatter);
          case "ASSIGNED_TO_ROLE" :
            return dojo.hitch(this, this.assigneeFormatter);
          case "ASSIGNED_TO_ROLE_DISPLAY_NAME" :
            return dojo.hitch(this, this.groupFormatter);
          case "ASSIGNED_TO_PARTICIPANT" :
            return dojo.hitch(this, this.participantFormatter);
          default:
            return null;
        }
        return null;
      },

      booleanIconFormatter: function(propertyValue){
        // TODO: rework css and images
        var url = null;
        var title = null;
        if (propertyValue) {
          url = dojo.moduleUrl("com.ibm.bpm.social.img", "booleantrue.gif");
          title = this._nlsResources["KEY_yes"];
        }
        else {
          url = dojo.moduleUrl("com.ibm.bpm.social.img", "booleanfalse.gif");
          title = this._nlsResources["KEY_no"];
        }
        return {
          url: url,
          title: title
        };
      },

      genericIconFormatter: function(type, propertyValue){
        //
        // Generic formatter that uses a closure function that accesses the type 
        // and a generic key format. Thereby no value to key mapping is necessary.
        //
//        var title = null;
//        var url = null;
//        var imageKey = null;
//        if (propertyValue && propertyValue != "") {
//          // Mapping between state and key is trivial
//          imageKey = type + "_" + propertyValue;
//          var key = "KEY_" + imageKey;
//          key = key.replace(/\./g, "_");
//          //
//          // Make sure that the same label is used for values that are not distinquished.
//          //
//          if (key != null &&
//            (key.indexOf("KEY_TASK_STATE") != -1 || key.indexOf("KEY_ESCALATION_ACTIVATION_STATE") != -1 || key
//              .indexOf("KEY_ESCALATION_AT_LEAST_EXPECTED_STATE") != -1)) {
//            key = this.STATE_STATUS_MAP[key];
//          }
//          else if (key != null && key.indexOf("PROCESS_INSTANCE_STATE") != -1) {
//            switch (key) {
//              case "KEY_PROCESS_INSTANCE_STATE_FAILED" :
//              case "KEY_PROCESS_INSTANCE_STATE_FAILING" :
//              case "KEY_PROCESS_INSTANCE_STATE_TERMINATED" :
//              case "KEY_PROCESS_INSTANCE_STATE_TERMINATING" :
//                key = "KEY_PROCESS_INSTANCE_STATE_FAILED";
//                break;
//              case "KEY_PROCESS_INSTANCE_STATE_COMPENSATING" :
//              case "KEY_PROCESS_INSTANCE_STATE_COMPENSATED" :
//                key = "KEY_PROCESS_INSTANCE_STATE_COMPENSATED";
//                break;
//            }
//          }
//          // Lookup value
//          title = this._nlsResources[key] ? this._nlsResources[key] : propertyValue;
//        }
//        //
//        // Check for image 
//        //         
//        if (imageKey != null) {
//          var imageName = this.ICON_MAP[imageKey];
//          if (imageName != null) {
//            url = dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/" + imageName);
//          }
//        }
//        if (url != null) {
//          return {
//            url: url,
//            title: title
//          };
//        }
//        else {
        // TODO: right now we do not have the mappings for the states, this use the propertyValue
        var title = propertyValue;
        return {
          title: title
        };
//        }
      },

      _createImage: function(url, title){
        var img = document.createElement("img");
        img.src = url;
        img.alt = title;
        img.style.verticalAlign = "middle";
        return img;
      },

      _createDomNode: function(info, displayName, verbose){
        var div = document.createElement("div");
        var title = info.title;
        if (typeof info.url != "undefined") {
          var img = this._createImage(info.url, title);
          div.appendChild(img);
          var text = title;
          if (typeof displayName != "undefined") {
            text = displayName;
          }
          if (typeof verbose == "undefined" || verbose) {
            var span = document.createElement("span");
            span.appendChild(document.createTextNode(text));
            span.style.verticalAlign = "middle";
            span.style.paddingLeft = "5px";
            span.style.paddingRight = "5px";
            div.appendChild(span);
          }
          else {
            img.style.display = "block";
            img.style.marginLeft = "auto";
            img.style.marginRight = "auto";
          }
          div.title = title;
        }
        else {
          div.appendChild(document.createTextNode(title));
        }
        return div;
      },

      dateFormatter: function(propertyValue){
        if (propertyValue) {
          var d = dojo.date.stamp.fromISOString(propertyValue);
          if (d == null) {
            this._checkProperties();
            return null;
          }
          else {
            propertyValue = dojo.date.locale.format(d, {
              formatLength: "short"
            });
          }
          var node = document.createElement("div");
          node.innerHTML = propertyValue;
          return node;
        }
        else {
          return null;
        }
      },

      userGroupFormatter: function(propertyValue){
        //make sure no code injection happens        
        if (propertyValue) {
          if (propertyValue.indexOf("cn=") == 0) {
            propertyValue = propertyValue.substring(3, propertyValue.indexOf(","));
          }
          propertyValue = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(propertyValue);
        }
        var node = document.createElement("div");
        node.innerHTML = propertyValue;
        node.title = propertyValue;
        return node;
      },


      participantFormatter: function(assignedToRole){
        return this.assignedToRole(assignedToRole, false);
      },

      groupFormatter: function(assignedToRoleDisplayName, hideIcon){
        if (assignedToRoleDisplayName) {
          var node = document.createElement("div");
          if (hideIcon !== true) {
            var n2 = document.createElement("div");
            n2.className = "bpm-task-group bpm-social-group";
            node.appendChild(n2);
            var n3 = document.createElement("span");
            node.appendChild(n3);
            n3.innerHTML = assignedToRoleDisplayName;
            n3.title = assignedToRoleDisplayName;
          }
          else {
            node.innerHTML = assignedToRoleDisplayName;
            node.title = assignedToRoleDisplayName;
          }
          return node;
        }
      },

      assigneeFormatter: function(assignedToRole, hideIcon){
        if (assignedToRole) {
          // From Process Portal: determine group display name
          // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
          // So we are just using the last two '_'s to pull the PG name
          if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
            var displayName = assignedToRole.substring(0, assignedToRole.lastIndexOf("_"));
            displayName = displayName.substring(0, displayName.lastIndexOf("_"));
            assignedToRole = displayName;
          }
          else {
            hideIcon = true;
          }
          var node = document.createElement("div");
          if (hideIcon !== true) {
            var n2 = document.createElement("div");
            n2.className = "bpm-task-group bpm-social-group";
            node.appendChild(n2);
            var n3 = document.createElement("span");
            node.appendChild(n3);
            n3.innerHTML = assignedToRole;
            n3.title = assignedToRole;
          }
          else {
            node.innerHTML = assignedToRole;
            node.title = assignedToRole;
          }
          return node;
        }
      },

      textFormatter: function(propertyValue){
        //make sure no code injection happens 

        // Remember title value, because it will not be interpreted as HTML.
        var titleValue = propertyValue;

        if (propertyValue) {
          propertyValue = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(propertyValue);
        }
        var node = document.createElement("div");
        node.innerHTML = propertyValue;
        node.title = titleValue;
        return node;
      },

      // TODO ???
//      liveNameFormatter: function(userShortName){
//        var liveNameContainer = document.createElement("div");
//        liveNameContainer.innerHTML = userShortName;
//        com.ibm.bpm.social.widgets.task.util.SametimeService.getInstance().createLiveName(this._widgetId, liveNameContainer,
//          userShortName);
//        return liveNameContainer;
//      },

      textcheckFormatter: function(propertyValue){
        if (propertyValue) {
          // 	 (Dot). Match any character except newline or another 
          //   Unicode line terminator.  Global search, ignore case.
          propertyValue = propertyValue.toString().replace(/%htm(.*?)%/gi, this._nlsResources["KEY_TaskVariableReplace"]);
          propertyValue = propertyValue.toString().replace(/%bfm(.*?)%/gi, this._nlsResources["KEY_TaskVariableReplace"]);
          propertyValue = propertyValue.toString().replace(/%wf(.*?)%/gi, this._nlsResources["KEY_TaskVariableReplace"]);
          //make sure no code injection happens

          // Remember title value, because it will not be interpreted as HTML.
          var titleValue = propertyValue;

          propertyValue = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(propertyValue);
          var node = document.createElement("div");
          node.innerHTML = propertyValue;
          node.title = titleValue;
          return node;
        }
        else {
          return null;
        }
      },

      numberFormatter: function(propertyValue){
        if (propertyValue !== null && propertyValue !== "") {
          var number = dojo.number.parse(propertyValue);
          if (number !== null && number != "NaN") {
            propertyValue = dojo.number.format(number);
          }
          else {
            this._checkProperties();
          }
          var node = document.createElement("div");
          dojo.addClass(node, "numericDataCell");
          node.innerHTML = propertyValue;
          return node;
        }
        else {
          return null;
        }
      },

      priorityFormatter: function(propertyValue){
        // TODO: rework function to use the new css classes
        if (propertyValue !== null && propertyValue !== "") {
          var node = null;
          if (this._prioritySettings == null) {
            node = document.createElement("div");
            node.innerHTML = propertyValue;
          }
          else if (this._prioritySettings.type == "priority") {
            // TODO : check for number
            var high = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
            var low = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
            // find the priority range for the current value            
            if (low !== null && high !== null) {
              var cssClass = null;
              var titleKey = null;
              var tooltip = null;
              // high is greater then low
              if (high > low) {
                if (propertyValue >= high) {
                  // high
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CSSLCASS_CONFIG];
                }
                else if (propertyValue > low) {
                  // medium
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass =
                    this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_CSSCLASS_CONFIG];
                }
                else {
                  // low
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CSSLCASS_CONFIG];
                }
              }
              else {
                // high is less or equal then low
                if (propertyValue <= high) {
                  // high
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CSSLCASS_CONFIG];
                }
                else if (propertyValue < low) {
                  // medium
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass =
                    this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOMEDIUM_CSSCLASS_CONFIG];
                }
                else {
                  // low
                  titleKey =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_TITLEKEY_CONFIG]];
                  tooltip =
                    this._nlsResources[this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_TITLEKEY_CONFIG] +
                      "Tooltip"];
                  cssClass = this._prioritySettings[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CSSLCASS_CONFIG];
                }
              }

              // Map css class to priority icon.
              var imageName = null;
              switch (cssClass) {
                case "itemPriorityLow" :
                  imageName = "priority-low.gif";
                  break;
                case "itemPriorityMedium" :
                  imageName = "priority-medium.gif";
                  break;
                case "itemPriorityHigh" :
                  imageName = "priority-high.gif";
                  break;
              }

              if (imageName) {
                var url = dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/" + imageName);
                var info = {
                  url: url,
                  title: tooltip
                };
                node = this._createDomNode(info, titleKey, true);
                dojo.addClass(node, "itemVisualProperty");
              }
              else {
                node = document.createElement("div");
                node.innerHTML = propertyValue;
              }
            }
          }
          else if (this._prioritySettings.type == "defaultPriority") {
            var key = com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(propertyValue);
            var titleKey = this._nlsResources["KEY_Prio" + key];
            var tooltip = this._nlsResources["KEY_Prio" + key + "Tooltip"];
            // var cssClass = "itemPriority" + key;
            var imageName = "priority-" + key.toLowerCase() + ".gif";

            if (this._prioritySettings.showImage === true) {
              info = {
                url: dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/" + imageName),
                title: tooltip
              };
              node = this._createDomNode(info, titleKey, true);
              dojo.addClass(node, "itemVisualProperty");
            }
            else {
              node = document.createElement("div");
              node.innerHTML = titleKey;
              node.title = tooltip;
            }
          }

          return node;
        }
        else {
          return null;
        }
      },

//      pendingFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var info = this.booleanIconFormatter(propertyValue);
//          return this._createDomNode(info, "", false);
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

//      pendingVerboseFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var url = dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/waiting.gif");
//          var info = {
//            url: url,
//            title: this._nlsResources["KEY_waitingForSubtasks"]
//          };
//          var node = this._createDomNode(info, this._nlsResources["KEY_waitingForSubtasksFilter"], true);
//          dojo.addClass(node, "itemVisualProperty");
//          return node;
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

//      escalatedFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var info = this.booleanIconFormatter(propertyValue);
//          return this._createDomNode(info, "", false);
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

//      escalatedVerboseFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var url = dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/escalationStateEscalated.gif");
//          var info = {
//            url: url,
//            title: this._nlsResources["KEY_escalatedTask"]
//          };
//          var node = this._createDomNode(info, this._nlsResources["KEY_escalatedTaskFilter"], true);
//          dojo.addClass(node, "itemVisualProperty");
//          return node;
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

//      suspendedFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var info = this.booleanIconFormatter(propertyValue);
//          return this._createDomNode(info, "", false);
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

//      suspendedVerboseFormatter: function(propertyValue){
//        if (propertyValue && propertyValue !== false) {
//          var url = dojo.moduleUrl("com.ibm.bpm.social.widgets.task.list.themes", "images/taskSuspended.gif");
//          var info = {
//            url: url,
//            title: this._nlsResources["KEY_suspendedTask"]
//          };
//          var node = this._createDomNode(info, this._nlsResources["KEY_suspendedTaskFilter"], true);
//          dojo.addClass(node, "itemVisualProperty");
//          return node;
//        }
//        else {
//          return document.createElement("div");
//        }
//      },

      _checkProperties: function(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this._queueConfig || !this._attributesForQueryTableFunc) {
          com.ibm.bpm.social.util.common.Log.logError(1, "Either _queueConfig or ._attributesForQueryTableFunc not set");
          return;
        }
        var attributes = null;

        var processAppName = this._queueConfig.processAppName; // may be null, or undefined, or a valid String
        this._attributesForQueryTableFunc(this._type, this._queueConfig.getPlainQueueName(), processAppName,
          function(result){
            attributes = result;
          }, null, null, this._queueConfig.getDomain());
        if (attributes) {
          // check if the property is already in the list,
          // then it is a displayed property;      
          for ( var k in this._queueConfig.item[com.ibm.bpm.social.widgets.task.util.Constants.PROPERTIES_CONFIG]) {
            var found = false;
            var property = (this._queueConfig.item[com.ibm.bpm.social.widgets.task.util.Constants.PROPERTIES_CONFIG])[k];
            for ( var i in attributes.items) {
              var propertyMetaData = attributes.items[i];
              if ((property[com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_CONFIG] == propertyMetaData.name) &&
                (property.type == com.ibm.bpm.social.widgets.task.util.Constants.mapPropertyType(propertyMetaData)) &&
                (property.content == propertyMetaData.content)) {
                found = true;
              }
            }
            if (!found) {
              var queueName = this._queueConfig.getDisplayName();
              this._queueConfig.exception =
                {
                  type: "list",
                  messageKey: "KEY_queuesHasChanged" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(this._type),
                  severity: "WARNING",
                  parameters: {
                    1: queueName,
                    2: property.title
                  }
                };
              // looks like it is necessary to throw an object
              // and not this._queueConfig.exception
              throw ({
                type: "list",
                messageKey: "KEY_queuesHasChanged" +
                  com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(this._type),
                severity: "WARNING",
                parameters: {
                  1: queueName,
                  2: property.title
                }
              });
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      }
    });

com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText =
  function(str){
    if (!str) {
      return str;
    }
    var result =
      str.toString().replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
    return result;
  };

com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL = function(str){
  if (!str) {
    return str;
  }
  var result = str.toString().replace(/'/gm, "''");
  return result;
};

com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatDate = function(text, options){
  if (!options) {
      options = {formatLength: "short"};
  }
  var d = dojo.date.stamp.fromISOString(text);
  return com.ibm.bpm.social.util.common.Util.formatDateObject(d, options);
};

com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatNumber = function(number){
  return dojo.number.format(number);
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ClaimDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ClaimDialog"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.task.util.ClaimDialog");








dojo.declare("com.ibm.bpm.social.widgets.task.util.ClaimDialog", com.ibm.bspace.common.util.ui.Dialog, {

  templateString: '<div dojoAttachPoint="container"></div>',

  _tkiid: null,

  preamble: function(params){
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this.title = this._nlsResources["task_dialog_confirm_claim_title"];
    params.title = this.title;
    return [params];
  },

  /**
   * Prerequisites:
   * <ol>
   * <li> restHandler a rest handler to claim the task </li>
   * </ol>
   * 
   * @param params {void} nothing
   */
  constructor: function(params){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "", "ClaimDialog");
    this._restHandler = params.restHandler;
    //
    // Instanciate an error handler.
    //
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Assemble the dialog. {void} nothing
   */
  postCreate: function postCreate_ClaimDialog(){
    this.inherited("postCreate", arguments);
    this.container.className = "bpm-task-dialog-container";
    this.container.title = ""; // Unset the title, else we see the tooltip from the dialog on all controlls that don't have a title

    this.description = document.createElement("div");
    this.description.innerHTML = this._nlsResources["task_dialog_confirm_claim_description"];
    this.description.className = "bpm-task-dialog-description";
    this.container.appendChild(this.description);

    var t = document.createElement("table");
    var tb = document.createElement("tbody");
    var tr = document.createElement("tr");
    var td1 = document.createElement("td");
    var td2 = document.createElement("td");

    t.appendChild(tb);
    tb.appendChild(tr);
    tr.appendChild(td1);
    tr.appendChild(td2);

    this.container.appendChild(t);
    this.dontShowAgainCheckbox = document.createElement("input");
    this.dontShowAgainCheckbox.type = "checkbox";
    td1.appendChild(this.dontShowAgainCheckbox);

    this.dontShowAgainMessage = document.createElement("span");
    this.dontShowAgainMessage.innerHTML = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];
    this.dontShowAgainMessage.title = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];

    td2.appendChild(this.dontShowAgainMessage);

    this.claimButtonDiv = document.createElement("div");
    this.claimButtonDiv.className = "bpm-task-dialog-claimButtonDiv";
    this.container.appendChild(this.claimButtonDiv);

    this.claimButton = document.createElement("button");
    this.claimButton.innerHTML = this._nlsResources["task_dialog_confirm_claim_button"];
    this.claimButton.title = this._nlsResources["task_dialog_confirm_claim_button_tooltip"];
    this.claimButtonDiv.appendChild(this.claimButton);


    this.connect(this.claimButton, "onclick", "_onClaimConfirmed");

    com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this.dontShowAgainCheckbox);
    // ASZ: Title is not shown if set before wrap
    this.dontShowAgainCheckbox.nextSibling.title = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];
    com.ibm.bspace.common.util.ui.Button.wrapElement(this.claimButton);

    this.set("content", this.container);
    this.connect(this.popup, "onCancel", "_dialogCancel");
    this.connect(this.popup, "onHide", "_onHide");

  },

  _dialogCancel: function _dialogCancel_ClaimDialog(){
    if (this.taskId) {
      this.onCancelled(this.taskId);
    }
    else {
      this.onCancelled();
    }
  },

  _onHide: function _onHide_ClaimDialog(){
    dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
  },

  /**
   * Requires a task object either one with a TKIID and TAD_DISPLAY_NAME property or one with a tkiid and
   * displayName property.
   * 
   * @param task The task object.
   * @param systemId The system id.
   * @returns {showConfirmationDialog} {void} nothing
   */
  showConfirmationDialog: function showConfirmationDialog_ClaimDialog(task /* object */, systemId){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);

    this.taskId = task.tkiid || task["TASK.TKIID"];
    this.displayName = task.displayName || task["TAD_DISPLAY_NAME"] || task["NAME"];

    this.systemId = systemId;
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog About to show dialog for " + this.displayName + " (" +
      this.taskId + ")");
    // Reset the combo box to unchecked before displaying the dialog.
    com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.dontShowAgainCheckbox, false);
    this.show();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _onClaimConfirmed: function _onClaimConfirmed_TasksListController(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    //
    // Hide the dialog
    //
    this.hide();
    //
    // Set global user preferences if necessary
    //
    if (this.dontShowAgainCheckbox.checked === true) {
      com.ibm.bpm.socialportal.GlobalAssets.setUserPreference('Alert On Assign And Run', "false");
    }
    this._claim();
    com.ibm.bpm.social.util.common.Log.logExit(1);
    return;
  },

  _claim: function _claim_TasksListController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var claimed = false;
    var self = this;
    this._restHandler.claimTask(this.taskId, function(responseObject, ioArgs){
      claimed = true;
    }, function(data, ioArgs){
      if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status === 409 && ioArgs.xhr.responseText) {
        var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
        if (response.errorNumber === "CWTBG0014E") {
          try {
            // Check whether the task has been claimed since last list refresh
            // Check if user is allowed to complete
            // Refresh available actions for this item synchronously          
            var availableActions = [];
            self._restHandler.getActionsForTask(self.taskId, ["ACTION_COMPLETE"], function(responseObject){
              if (responseObject.tasks.length === 1) {
                availableActions = responseObject.tasks[0].actions || [];
              }
            }, null /* no error handling */, self.systemId);
            if (dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1) {
              // continue normal processing
              claimed = true;
            }
          }
          catch (e) {
            // This is just logic for checking whether an error condition can be ignored.
            // Therefore ignore any errors either.
          }
        }
      }
      if (!claimed) {
        // handle errors
        var name = self.displayName;
        var statusCodeMap = {
          409: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          401: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          403: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          404: {
            key: "KEY_ClaimNotExistingTask",
            parameters: {
              1: name
            }
          }
        };
        var errorCodeMap = {
          "CWTKA0072E": {
            key: "KEY_ClaimSuspendedTask",
            parameters: {
              1: name
            }
          }
        };

        self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
      }
    }, this.systemId ? this.systemId : null);
    com.ibm.bpm.social.util.common.Log.logExit(2, claimed);
    if (claimed) {
      this.onClaimed(this.taskId);
    }
    else {
      this.onFailed(this.taskId);
    }
  },

  onClaimed: function onClaimed_ClaimDialog(taskId /* string */){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onClaimed " + this.taskId);
    dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_CLAIMED + "." + this.taskId, [{}]);
  },


  onCancelled: function onCancelled_ClaimDialog(taskId){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onCancelled " + this.taskId);
  },

  onFailed: function onFailed_ClaimDialog(taskId /* string */){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onFailed " + this.taskId);
  },

  destroy: function(){
    com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._confirmClaimDialog.dontShowAgainCheckbox);
    com.ibm.bspace.common.util.ui.Button.unwrapElement(this._confirmClaimDialog.claimButton);
    this.inherited("destroy", arguments);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.experts.DialogExpertTile"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.experts.DialogExpertTile"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.DialogExpertTile");












/**
 * TODO This class needs to be consolidate with the ExpertTile.
 */
dojo.declare("com.ibm.bpm.social.widgets.experts.DialogExpertTile", [dijit._Widget,
                                                                     dijit._Templated], {

  templateString: "",
  templateString:"<div class='bpm-social-experts-expert' dojoAttachPoint='expert' dojoAttachEvent='onmouseenter:_doMouseEnter, onmouseleave:_doMouseLeave, onkeydown:_onKeyDown'>\n    <div dojoAttachPoint='avatarContent'> </div>\t\n\t<div class='bpm-social-experts-expert-data-div'>\t    \n\t\t<div class='bpm-social-experts-expert-name' dojoAttachPoint='expertname' role=\"listitem\"></div>\n\t\t<div class='bpm-social-experts-expert-details taskDetails' dojoAttachPoint='taskDetails'></div>\n\t\t<div class='bpm-social-experts-expert-actions-div'>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-mention' dojoAttachPoint=\"mentionDiv\">\n\t\t\t\t<a dojoAttachPoint='mention' href='javascript:void(0)' dojoAttachEvent='onclick:_handleMention'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator1' class='vertical-bar' style='display:block'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-help' dojoAttachPoint=\"helpDiv\">\n\t\t\t\t<a dojoAttachPoint='gethelp' href='javascript:void(0)' dojoAttachEvent='onclick:_handleGetHelp'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator2' class='vertical-bar'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-chat' dojoAttachPoint=\"chatDiv\">\n\t\t\t\t<a dojoAttachPoint='startchat' tabIndex='-1' href='javascript:void(0)' dojoAttachEvent='onclick:_handleChat'></a>\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n",

  /**
   * Input provided in the constructor based on the experts call.
   */
  avgHours: 0,
  completed: 0,
  email: null,
  fullName: null,
  workload: 0,
  userName: null,

  /**
   * Added information for fine tuning available functions also provided using the constructor.
   */
  categoryName: null, // If set, will be shown in the taskDetails section
  collaborate: false, // If true, collaboration action is shown  
  includeTaskDetails: false, // If true, shows the task details i.e. the statistics (avgHours, completed...)
  sidebar: false, // If true, assumes that it is rendered in the sidebar.
  taskId: null, // Added to allow for collaboration... required if includeTaskDetails == true


  livename: null, // Set by sametime integration logic
  imageUrl: null, // ??
  phoneNumber: null, // ??
  title: "", // ?
  refresh: null, // Used for timeout  handle
  
  /**
   * 
   */
  postCreate: function(){
    this.inherited(arguments);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    
    //
    // extends of template
    //
    var div = document.createElement("div");
    dojo.addClass(div, "bpm-social-experts-expert-icon");
    this.expert.appendChild(div);
    var divA11y = document.createElement("div");
    dojo.addClass(divA11y, "bpm-social-experts-expert-icon-a11y");
    this.domNodeA11y = divA11y;
    div.appendChild(divA11y);
       
    //
    // Basic Setup
    //
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Create a unique id to be able to refer to the name field
    // It is dijit._Widget that has a unique id.
    var nameId = this.id + "_name";
    dojo.attr(this.expertname, "id", nameId);
    this.domNode.title = "";
    //
    // Retrieve preferences of current user -> this.userData
    // Retrieve user avatar and add it the div[#expert-avatar]
    //
    this._getUserData();
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    if (!this.fullName && user && user.fullName) {
      this.fullName = user.fullName;
    }
    this.expertname.innerHTML = this.fullName;

    if (window.sametime && this.fullName) {
      // the text-overflow:ellipsis style does not function properly
      // when there are span elements within the div, so clip the
      // text instead if we are using sametime
      this.expertname.style.textOverflow = 'clip';
      // Overwrite name and add the live name
      this.expertname.innerHTML = "";
      this.livename = window.sametime.LiveName({
        'userId': this.userName,
        'disableClicks': true,
        'disableHoverBizCard': true
      }, dojo.create("span", null, this.expertname));
      this.livename.activate();

      this._updateStatus();
      this.refresh = window.setInterval(dojo.hitch(this, this._updateStatus), 2000);
    }
    
    this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
    this.avatar.setUser(user, "bpm-social-default-avatar-medium", false);
    
    /** Business Card integration * */
    /** Default * */
    var node = this.expertname;
    if (this.livename != null) {
      var nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
      if (nodes.length == 1) {
        node = nodes[0];
        var img = dojo.create("img", {
          "src": dojo.moduleUrl("dojo", "resources/blank.gif"),
          "class": "connectionsOverlayImage"
        }, dojo.query(".stproxy_general", this.domNode)[0], "first");
        com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
        var divA11y = document.createElement("div");
        divA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
        divA11y.addClass("bpm-social-experts-expert-name-a11y");
        this.onlineTrackingDomNodeA11y = divA11y;
        img.appendChild(divA11y);
      }
    }

    /*
    // Do not show the BizCard as clicking on the tile selects the user and 
    // should not show the BizCard
    com.ibm.bpm.social.widgets.common.BizCard.wrapElement(node, user);
    com.ibm.bpm.social.widgets.common.BizCard.wrapImage(dojo.query("img", this.img)[0], user);
    */
    /** End Business Card integration * */


    //
    // Adjusting functionality based on configuration of instance.
    // Processing according to display in rtl direction
    //
    if (this.categoryName) {
      this.taskDetails.appendChild(document.createTextNode(this.categoryName));
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    else if (this.includeTaskDetails) {
      // If includeTaskDetails add content to dojoAttachPoint taskDetails
      var data;
      var firstName = this.fullName.split(" ")[0];
      this._updateUserData();
      if (this.completed == 0) {
        this.taskDetails.innerHTML = this.messages.expertNoTaskData.replace("${user}", firstName);
      }
      else {
        data = {
          'userName': firstName,
          'average': this.avgHours,
          'completed': this.completed
        };
        //defect 149810 Fix Expert tooltip wording
        this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip_short, data, /\$\{([^\}]+)\}/g);
      }
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    // 
    // Add actions for mention | collaborate | chat
    // Remove mention if not in sidebar
    // 
    this.mention.innerHTML = "@" + this.messages.expertsMention;
    this.gethelp.innerHTML = this.messages.expertsCollaborate;
    this.startchat.innerHTML = this.messages.expertsChat;
    // Refering to the name field
    dijit.setWaiState(this.mention, "describedby", nameId);
    dijit.setWaiState(this.gethelp, "describedby", nameId);
    dijit.setWaiState(this.startchat, "describedby", nameId);
    this.actionSeparator1.innerHTML = this.messages.expertsSeparator;
    this.actionSeparator2.innerHTML = this.messages.expertsSeparator;
    // Downgrade actions if not in sidebar
    if (!this.sidebar) {
      this.mention.style.display = 'none';
      this.actionSeparator1.style.display = 'none';
    }
    //
    // The collaborate constructor parameter allows to configure whether collaboration action is supported.
    //
    if (!this.collaborate) {
      this.actionSeparator2.style.display = 'none';
      this.helpDiv.style.display = 'none';
    }
    //
    // Remove chat action
    //
    if (!(window.sameTime && this.fullName)) {
      dojo.style(this.chatDiv, "display", "none");
    }
  },

  _handleMention: function(e){
    var taskCompletionView = this._getTaskCompletionView();
    if (taskCompletionView) {
      // switch to the stream tab
      taskCompletionView.sidebar.selectTab(1);
      // set focus to input box
      taskCompletionView.stream._handlePostInitialFocus();
      // tag user
      taskCompletionView.stream.tagUser(this.userName, this.fullName);
    }
    dojo.stopEvent(e);
  },

  _handleGetHelp: function(e){
    var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
      "taskId": this.taskId
    });
    com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(taskData, {
      userName: this.userName,
      fullName: this.fullName
    });
    dojo.stopEvent(e);
  },

  _handleChat: function(e){
    if (this.livename && this.presenceStatus == "online") {
      this.livename.startChat();
    }
    dojo.stopEvent(e);
  },

  _showTooltip: function(){
    return;
  },

  _hideTooltip: function(){
    if (this.myTooltipDialog) {
      this.myTooltipDialog = null;
    }

  },

  _updateUserData: function(){
    var self = this;
    this.socialRestHandler.getExpertUserData(this.taskId, this.userName, function(data, xhrArgs){
      self.avgHours = data.avgHours;
      self.completed = data.completed;
    }, "", false);
  },

  _getUserData: function(){
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    /*if(user && user.userPreferences && user.userPreferences.Image){
    	this.img.innerHTML = "<img alt='"+this.fullName+"' src='"+user.userPreferences.Image+"' />";
    }*/
    this.userData = user.userPreferences;
    if (!this.userData) {
      this.userData = new Object();
    }
  },

  _updateStatus: function(){
    dojo.removeClass(this.domNode, "online");
    dojo.removeClass(this.domNode, "offline");

    if (this.livename) {
      switch (this.livename.currentTextStyle) {
        case "stproxy_liveNameAvailableText" :
        case "stproxy_liveNameAwayText" :
        case "stproxy_liveNameInMeetingText" :
          dojo.addClass(this.domNode, "online");
          this.presenceStatus = "online";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-online"];
          }
          break;
        case "stproxy_liveNameOfflineText" :
        case "stproxy_liveNameDndText" :
        default:
          dojo.addClass(this.domNode, "offline");
          this.presenceStatus = "offline";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
          }
          break;
      }
    }
    else {
      dojo.style(this.chatDiv, "display", "none");
    }

    /** Business Card integration * */
    if (!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard() && this.livename != null) {
      /** Account for Sametime DOM structure, which can be delayed in being created due to network latency * */
      var nodes = dojo.query(".stproxy_liveNamePadding .hasHover", this.expertname);
      if (nodes.length == 0) {
        nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
        if (nodes.length == 1) {
          var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
          if (!this.fullName) {
            this.fullName = user.fullName;
          }
          com.ibm.bpm.social.widgets.common.BizCard.wrapElement(nodes[0], user);
        }
      }
    }
    /** End Business Card integration * */

  },


  _getTaskCompletionView: function(){
    var node = this.domNode;
    while (node && !dojo.hasClass(node, 'workViewCompletion')) {
      node = node.parentNode;
    }
    if (node == null) {
      node = dojo.query('.workViewCompletion')[0];
    }
    var taskCompletionView = dijit.byId(node.id);
    return taskCompletionView;
  },


  _doMouseEnter: function(e){
    dojo.addClass(this.expert, 'hover');
  },

  _doMouseLeave: function(e){
    dojo.removeClass(this.expert, 'hover');
  },

  _onFocus: function(){
    dojo.addClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onBlur: function(){
    dojo.removeClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onKeyDown: function(e){
    if (e.keyCode === dojo.keys.RIGHT_ARROW && this.livename) {
      this.startchat.focus();
    } 
  },
 
  destroy: function(){
    if (this.refresh) {
      window.clearInterval(this.refresh);
    }
    
    this._hideTooltip();

    if (this.avatar) {
        this.avatar.destroy();
        this.avatar = null;
    }

    this.inherited(arguments);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.InvitationDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.InvitationDialog"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012,2013. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.InvitationDialog");











/**
 * Invitation dialog used when inviting people to a group task.
 * 
 */
dojo.declare("com.ibm.bpm.social.widgets.dialogs.InvitationDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString:"<div dojoAttachPoint='content' class=\"bpm-task-invitation-dialog\">\t\t\n\t<div dojoAttachPoint=\"invitedUserSection\" class=\"bpm-task-invitation-dialog-invitedUsers-section\">\t\t\n\t\t<h2 title=\"${labelForInvitedUsersSubTitle}\" class=\"bpm-task-invitation-dialog-invitedUsers-title\" dojoAttachPoint=\"userChooserLabel\">${labelForInvitedUsersSubTitle}</h2>\t\t\t\t\t\t\t\t\t\n   </div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"collaboratorsSectionDummy\"></div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"suggestedCollaboratorsSection\" class=\"bpm-task-invitation-dialog-suggestions-section\" title=\"${labelForSuggestedCollaboratorTooltip}\">\n\t      <h2 class=\"bpm-task-invitation-dialog-suggestions-title\" dojoAttachPoint=\"suggestionsTitle\">${labelForSuggestedCollaboratorTitle}</h2>\t\t\n\t\t  <div dojoAttachPoint=\"suggestedCollaborators\"></div>\n\t\t<div>\n\t\t\t<div tabIndex=\"0\" dojoAttachPoint=\"showMore\" class=\"bpm-task-invitation-dialog-suggestions-more\">${labelForMore}</div>\n\t\t\t<div tabIndex=\"0\" dojoAttachPoint=\"showBack\" class=\"bpm-task-invitation-dialog-suggestions-back\">${labelForBack}</div>\t\t\t\n\t\t</div>\n\t\t<div class=\"bpm-task-invitation-dialog-suggestions-clear\"></div>\n\t</div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"invitedCollaboratorsSection\" class=\"bpm-task-invitation-dialog-suggestions-section\" title=\"${labelForInvitedCollaboratorTooltip}\">   \n\t      <h2 class=\"bpm-task-invitation-dialog-suggestions-title\">${labelForInvitedCollaboratorTitle}</h2>\t\t\n\t\t  <div dojoAttachPoint=\"invitedCollaborators\"></div>\n\t\t  \t\t<div>\n\t\t\t<div dojoAttachPoint=\"showMoreInvited\" class=\"bpm-task-invitation-dialog-suggestions-more\">${labelForMore}</div>\n\t\t\t<div dojoAttachPoint=\"showBackInvited\" class=\"bpm-task-invitation-dialog-suggestions-back\">${labelForBack}</div>\t\t\t\n\t\t</div>\n\t\t<div class=\"bpm-task-invitation-dialog-suggestions-clear\"></div>\n    </div>\t\n\t\t\n\t<div dojoAttachPoint=\"messageSection\" class=\"bpm-task-invitation-dialog-message-section\">\n\t\t<h2 class=\"bpm-task-invitation-dialog-message-title\">${labelForMessageSection}</h2>\n\t\t<label dojoAttachPoint=\"messageBoxLabel\" style=\"display: none\">${labelForMessageSection}</label>\n\t\t<textarea dojoAttachPoint=\"messageBox\" rows=\"3\" cols=\"85\" placeholder=\"${labelForMessageSectionPlaceholder}\"></textarea>\n\t</div>\n\t\n\t<div class=\"bpm-task-invitation-dialog-button-section\" style=\"background: white\">\n\t\t<button dojoAttachPoint=\"inviteButton\" class=\"bpm-task-invitation-dialog-button-invite\" title=\"${labelForInviteButtonTitle}\">${labelForInviteButton}</button>\n\t</div>\n\t\t\n</div>\n",

  constructor: function(parameters){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Life cycle management:
    // As dijit._Widget this.connect should be used in order have everything freed upon destruction.
    // The destroy method will be called in event handler to onHide.
    // Might not be used as the com.ibm.bspace.common.util.ui.Dialog.postCreate already calls Helper.wrapElement that 
    // wraps the elements out of the template and should take care of them when destroying.
    //
    this._wrappedElements = [];

    // TODO There could be a configurable limit. So far it is 5. If it changes we need to ask for that limit.
    this.COLLAB_LIMIT = 5;
    //
    // Looking up helper
    //
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Processing of parameters
    //
    var task = parameters.task;

    //
    // labels used in the template
    //    
    this.labelForInvitedUsers = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTitle"];
    this.labelForInvitedUsersSubTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle"];    
    this.labelForInvitedUsersSubTitleWithoutSection =
    this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle_WithoutSection"];
    this.labelForInvitedUsersTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTooltip"];
    this.labelForMessageSection = this._nlsResources["task_dialog_startCollaboration_messageSectionTitle"];
    this.labelForMessageSectionPlaceholder = this._nlsResources["task_dialog_startCollaboration_messageSectionPlaceholder"];
    // this.labelForMessageSectionTitle = this._nlsResources["task_dialog_startCollaboration_messageSectionTooltip"];
    this.labelForInviteButton = this._nlsResources["task_dialog_startCollaboration_saveButton"];
    this.labelForInviteButtonTitle = this._nlsResources["task_dialog_startCollaboration_saveButtonTooltip"];
    this.labelForSuggestedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_suggestedCollaborators"];
    this.labelForInvitedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_invitedCollaborators"];
    this.labelForSuggestedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_suggestedCollaboratorsTooltip"];
    this.labelForInvitedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_invitedCollaboratorsTooltip"];
    this.labelForMore = this._nlsResources["task_dialog_startCollaboration_more"];
    this.labelForBack = this._nlsResources["task_dialog_startCollaboration_back"];
    //
    //
    // Normalizing properties because the properties used by query are different from
    // the properties returned by GET /task
    // Although there is currently no use case where the provided task is retrieved using a query.
    //
    if (typeof task.TKIID === "string") {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping;
    }
    else {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping;
    }
    this._availableActions = parameters.availableActions;
    this._tkiid = task[this.mapping["TKIID"]];
    this._taskName = task[this.mapping["NAME"]];
    this._taskDisplayName = task[this.mapping["DISPLAY_NAME"]];
    this._processInstanceName = task[this.mapping["PI_NAME"]];
    this._piid = task[this.mapping["PIID"]];
    // Remember whether a user is expert or frequent collaborator
    this._userCategory = {};
    //
    // Assumption: task has already an owner. Otherwise inviting does not make sense.
    // 
    this._currentOwner = {
      value: task[this.mapping["OWNER"]],
      type: "user"
    };
    this.placeHolderState = {
      modified: true
    };
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  postCreate: function postCreate_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calling com.ibm.bspace.common.util.ui.Dialog.postCreate and thereby wrapping the radio buttons and buttons.
    // Because the Dialog.postCreate method calls com.ibm.bspace.common.util.ui.Helper.wrapElement which
    // finds all markup that needs to be wrapped.
    //
    this.inherited("postCreate", arguments);
    this.connect(this.popup, 'onHide', function(){
      // Basically this is also a dijit._Widget with a meaningful destroy method.
      // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
      setTimeout(dojo.hitch(this, this.destroy), 1000);
    });
    
    // Accessibility
    dijit.setWaiRole(this.suggestedCollaborators, "listbox");
    var suggestionsTitleId = this.id + "_suggestionsTitle";
    this.suggestionsTitle.id = suggestionsTitleId;
    dijit.setWaiState(this.suggestedCollaborators, "labelledby", suggestionsTitleId);
    dijit.setWaiRole(this.showMore, "button");     
    dijit.setWaiRole(this.showBack, "button");
    
    //
    // Custom Logic Starts Here
    //    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  showDialog: function showDialog_InvitationDialog(user){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var expertsBoxDim = this._calculateSize();
    // get the number of experts per line before the invited users are inserted
    this._calculateNumberOfExperts();
    this._setupUserChooser(user);
    if (user == null) {
      // user is not already known
      this._setupSuggestedCollaborators();
    }
    // insert the invited users in the dialog, that can be done independent of the experts callback
    // this._setupInvitedUsers();
    this.connect(this.inviteButton, "onclick", "onInvite");
    // if there is a limit of allowed collaborators then we do not need to create the whole dialog. 
    // In that case do nothing.     
    this._setupMessageBox();
    // button is disabled 
    com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, true);
    // Remove the tooltip for the dialog
    this.domNode.removeAttribute("title");
    this.show();
    // Set the width of the dialog to assure that there is no white space,
    // if the title is too long.
    dojo.style(this.popup.domNode, "width", expertsBoxDim.w + "px");
    this._connectKey = dojo.connect(this.domNode, "onkeydown", this, "_onKeyDown");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  destroy: function destroy_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    //
    // Assumption: Connections established with this.connect are disconnected in the destroy method of dijij._Widget
    //
    if (this._connectKey) {
      dojo.disconnect(this._connectKey);
      this._connectKey = null;
    }

    if (this._wrappedElements.length > 0) {
      for ( var i = 0; i < this._wrappedElements.length; i++) {
        com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements);
      }
    }
    if (this._expertsWidgets != null) {
      for ( var j = 0; j < this._expertsWidgets.length; j++) {
        var widget = this._expertsWidgets[j];
        widget.destroy();
      }
      this._expertsWidgets = null;
    }
    if (this._invitedWidgets != null) {
      for ( var j = 0; j < this._invitedWidgets.length; j++) {
        var widget = this._invitedWidgets[j];
        widget.destroy();
      }
      this._invitedWidgets = null;
    }
    this._restHandler = null;
    this._errorHandler = null;
    this._messageField = null;
    this.inherited("destroy", arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _onKeyDown: function _onKeyDown_InvitationDialog(e){
    var key = e.keyCode;
    if (key == dojo.keys.ESCAPE) {
      this.hide();
    }
  },
 
  _calculateSize: function _calculateSize_InvitationDialog(){
    // calculate the size of the area necessary for the experts/invitee box
    var size = dojo.window.getBox();
    var width = size.w;
    var height = size.h;
    // this is the size we should use. Otherwise the dialog is smaller and has scrollbars
    var expertsBoxHeight = 300;
    var expertsBoxWidth = 675;
    // the width of the window plus 32 for the rest of the dialog need to be there
    if (width - 32 < expertsBoxWidth) {
      expertsBoxWidth = (width - 32) > 0 ? width - 32 : width;
    }
    // the height of the window plus 250 for the rest of the dialog needs to be there 
    if (height - 250 < expertsBoxHeight) {
      expertsBoxHeight = (height - 250) > 0 ? height - 250 : height;
    }
    // make the dummy element that size and make it visible till the dialog is shown
    dojo.style(this.content, "width", expertsBoxWidth + "px");
    dojo.style(this.collaboratorsSectionDummy, "height", expertsBoxHeight + "px");
    dojo.style(this.collaboratorsSectionDummy, "display", "block");
    
    return {"h": expertsBoxHeight, "w": expertsBoxWidth};
  },

  _calculateNumberOfExperts: function _calculateNumberOfExperts_InvitationDialog(){
    var expertsBoxWidth = 630;
    if (this.content && this.content.offsetWidth) {
      // the scrollbar also needs some px this reduce 15
      expertsBoxWidth = this.content.offsetWidth - 25;
    }
    // for 3 experts tiles we need 600
    if (expertsBoxWidth >= 630) {
      this._numberExpertsPerLine = 3;
    }
    else {
      // otherwise calculate how many fit
      var mod = expertsBoxWidth % 210;
      this._numberExpertsPerLine = ((expertsBoxWidth - mod) / 210);
    }
    // now make room for the real experts or just make it small
    dojo.style(this.collaboratorsSectionDummy, "height", 0 + "px");
  },

  _checkButtonEnablement: function _checkButtonEnablement_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var valid = true;
    valid = (this.userChooser.chooserInput.value.length > 0 && this.userChooser.getChoice() !== null);
    if (this.inviteButton) {
      com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, !valid);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
  },

  _setupUserChooser: function _setupUserChooser_InvitationDialog(prepopulatedUseridString){
    //
    // TODO Make sure that (de-) selecting suggested collaborators plus entering users in the user chooser is synchronized.
    //
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Create and configure the user chooser
    //
    var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
      includeGroups: false,
      includeUsers: true
    });
    var myself = this;
    myself._ucClickedFirstTime = false;
    var args = {
      searchMinInputLength: 0,
      ucModel: ucModel,
      _onUCFocus: function(e){
        if (!myself._ucClickedFirstTime) {
          myself._ucClickedFirstTime = true;
          dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
          myself.userChooser.chooserInput.value = "";
        }
        com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);

        // myself.userChooser.chooserInput.value = "";
        myself._checkButtonEnablement();
        for ( var j = 0; j < myself._expertsWidgets.length; j++) {
          var widget = myself._expertsWidgets[j];
          if (widget.isAdded) {
            widget.isAdded = false;
            dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
            widget.domNodeA11y.innerHTML = "";
          }
        }
      },
      _onUCBlur: function(e){
        var choice = myself.userChooser.getChoice();
        var valid = (myself.userChooser.chooserInput.value.length < 0 || choice !== null);
        if (!valid) {
          myself.userChooser.chooserInput.value = "";
          myself.userChooser._hideResults();
          myself._checkButtonEnablement();
        }
        else {
          // make sure only the user specified in the user chooser is marked for invite
          for ( var j = 0; j < myself._expertsWidgets.length; j++) {
            var widget = myself._expertsWidgets[j];
            if (widget.userName == choice.userName) {
              dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "&#10004;";
              widget.isAdded = true;
            }
            else if (widget.isAdded) {
              widget.isAdded = false;
              dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "";
            }
          }
        }
        dojo.addClass(myself.userChooser.noUsersFoundDiv, "hidden");
      }
    };
    this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
    this.connect(this.userChooser, "onChangeHandler", "_checkButtonEnablement");
    //
    // initially show a help text in the input field
    //      
    this.userChooser.chooserInput.value = this._nlsResources["task_dialog_startCollaboration_invitedUseridFieldHelp"];
    dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    //
    // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
    //
    this.userChooser.setCollabTaskidFilter(this._tkiid);

    // allow to set a string in the UserChooser field, e.g. for calling this dialog
    // from the Experts List, thereby pre-populating the selected user from the Experts list.
    if (prepopulatedUseridString) {
      if (prepopulatedUseridString.fullName) {
        this.userChooser._chooseItemValue(prepopulatedUseridString);
      }
      else {
        this.userChooser.chooserInput.value = prepopulatedUseridString;
      }
      myself._ucClickedFirstTime = true;
      dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    }

    // A11y: add label for user chooser
    var ucLabel = document.createElement("label");
    ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
    ucLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_invitedUseridField_label"];
    ucLabel.style.display = "none";

    this.invitedUserSection.appendChild(ucLabel);
    this.invitedUserSection.appendChild(this.userChooser.domNode);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setupMessageBox: function _setupMessageBox_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calculate the default invitation message
    //
    var taskLabel = this._taskDisplayName;
    if (!taskLabel || taskLabel === "") {
      taskLabel = this._taskName;
    }
    var instanceName = this._processInstanceName;
    if (!instanceName || instanceName === "") {
      instanceName = this._piid;
    }
    // Set id and for appropriately
    this.messageBox.id = this.popup.id + "_MessageBox";
    this.messageBoxLabel.htmlFor = this.popup.id + "_MessageBox";
    // Use placeholder instead
    /*
    var msg = dojo.string.substitute(this._nlsResources["task_dialog_startCollaboration_messageDefault"], {
        1: taskLabel,
        2: instanceName
      });
      this.messageBox.appendChild(document.createTextNode(msg));
    */
    //configure base text direction here:
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.messageBox);
    if (dojo.isIE) {
      this._addPlaceholder(this.messageBox, this.labelForMessageSectionPlaceholder, this.placeHolderState);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _addPlaceholder: function(textarea, placeHolderText, placeHolderState){
    var fontStyle = "normal";
    if (textarea.style.fontStyle != null) {
      fontStyle = textarea.style.fontStyle;
    }
    var setPlaceHolderText = function(){
      textarea.value = placeHolderText;      
      textarea.style.fontStyle = "italic";
    };
    var removePlaceHolderText = function(){
      textarea.value = "";      
      textarea.style.fontStyle = fontStyle;
    };
    // Initial Setup 
    placeHolderState.modified = false;
    setPlaceHolderText();
    // Focus received
    dojo.connect(textarea, "onfocus", function(){
      if (!placeHolderState.modified) {
        removePlaceHolderText();
      }
    });
    // Focus lost
    dojo.connect(textarea, "onblur", function(){
      if (textarea.value != "") {
        placeHolderState.modified = true;
      }
      else {
        placeHolderState.modified = false;
        setPlaceHolderText();
      }
    });

  },

  _setupSuggestedCollaborators: function _setupSuggestedCollaborators_InvitationDialog(){
    this._socialRestHandler.getExpertsForTask(this._tkiid, dojo.hitch(this, this._expertsCallback), function(error){
    // TODO Error Handling for experts call
    }, false /* not async */);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _removeCurrentUser: function _removeCurrentUser_InvitationDialog(group){
    if (group && this._currentUser) {
      for ( var i = 0; i < group.length; i++) {
        if (group[i].userName == this._currentUser) {
          group.splice(i, 1);
          i--;
        }
      }
    }
  },

  _removeDuplicateUsers: function _removeDuplicateUsers_InvitationDialog(systemIdentified, expertParticipant){
    // calculate the invited users to delete them form the experts lists
    // they are displayed separately.	
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    if (expertParticipant && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < expertParticipant.length; i++) {
        var userName = expertParticipant[i].userName;
        /*
            var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);        
            if (filter > -1) {
              expertParticipant.splice(i, 1); // splicing the current item shift the items to the left
              i--;
            }
        */
        this._userCategory[userName] = categoryName;
      }
    }
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    if (systemIdentified && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < systemIdentified.length; i++) {
        var userName = systemIdentified[i].userName;
        /*
        var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);
        */
        if (typeof this._userCategory[userName] === "string") {
          // Already an expert 
          systemIdentified.splice(i, 1); // splicing the current item shift the items to the left     
          i--;
        }
        if (typeof this._userCategory[userName] === "undefined") {
          // Rememeber the type which is not already "Expert"
          this._userCategory[userName] = categoryName;
        }
      }
    }
  },

  _expertsCallback: function _expertsCallback_InvitationDialog(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // Establish user's name
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    if (userProfileObj) {
      this._currentUser = userProfileObj.userName;
    }

    var anker = this.suggestedCollaborators;
    var taskId = this._tkiid;
    var counter = 0;
    var systemIdentified = data["systemIdentifiedUsers"];
    var expertParticipantGroup = data["expertParticipantGroupUsers"];
    var numberExpertsPerLine = this._numberExpertsPerLine;
    var displayedUsers = {};
    // Keep track of ExpertTile objects in order to destroy them later
    var expertsWidgets = this._expertsWidgets = [];
    // TODO Check whether the categories (Frequent Collaborator, Experts) can also be user defined in the plugable member service
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    // remove the current user from the lists
    this._removeCurrentUser(systemIdentified);
    this._removeCurrentUser(expertParticipantGroup);
    // remove the user which are already invited
    // insert system suggested users        
    this._removeDuplicateUsers(systemIdentified, expertParticipantGroup);
    var tileLabel = this._nlsResources["task_dialog_startCollaborationSuggested_title"];
    var myself = this;
    dojo.forEach(systemIdentified, function(item){     
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    // insert modeled experts
    dojo.forEach(expertParticipantGroup, function(item){      
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
        
    for ( var j = 0; j < this._expertsWidgets.length; j++) {
      var widget = this._expertsWidgets[j];
      this.connect(widget, "onClick", "_expertClicked");
    }
    
    if (counter == 0) {
      dojo.style(this.suggestedCollaboratorsSection, "display", "none");
      this.userChooserLabel.innerHTML=this.labelForInvitedUsersSubTitleWithoutSection;      
      this.userChooserLabel.setAttribute("title",this.labelForInvitedUsersSubTitleWithoutSection);
      com.ibm.bpm.social.util.common.Log.logExit(2);
      return;
    }
    else {
      dojo.style(this.suggestedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMore, "display", "none");
    }
    else {
      dojo.style(this.showMore, "display", "block");
    }
    
    this.connect(this.showMore, "onclick", function(){
    	this._showMore(anker);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });
    this.connect(this.showMore, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showMore(anker);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });
    
    this.connect(this.showBack, "onclick", function(){
    	this._showLess(anker, numberExpertsPerLine);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });   
    this.connect(this.showBack, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showLess(anker, numberExpertsPerLine);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },
  
  _showMore: function _showMore_InvitationDialog(anker) {
	  dojo.forEach(anker.childNodes, function(node){
	        dojo.style(node, "display", "inline-block");
	  });
	  dojo.style(this.showBack, "display", "block");
	  dojo.style(this.showMore, "display", "none"); 
  },
  
  _showLess: function _showLess_InvitationDialog(anker, numberExpertsPerLine) {
	  var i = 0;
      dojo.forEach(anker.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBack, "display", "none");
      dojo.style(this.showMore, "display", "block");
  },
  
  _createAccForDialogExpertTile: function _setWaiForDialogExpertTile_InvitationDialog(widget, pos) {
	  dijit.setWaiRole(widget.domNode,"option");
	  dijit.setWaiState(widget.domNode, "selected", "false");	   
	  dijit.setWaiState(widget.domNode, "labelledby", widget.avatar.id + " " + widget.taskDetails.id);
      if(pos === 0) {
        dojo.attr(widget.domNode, 'tabindex', "0");
      } else {
        dojo.attr(widget.domNode, 'tabindex', "-1");    	  
      } 
    
      // Keyboard handling for list navigation
      // Key down: next expert
      // Key up: previous expert 
      // Key space: select      
      this.connect(widget, "onKeyDown", function(e) {
    	  var numberOfExperts = this._expertsWidgets.length;
    	  if(e.keyCode === dojo.keys.DOWN_ARROW && pos < numberOfExperts - 1) {
            this._expertsWidgets[pos + 1].domNode.focus(); 
    	  } else if(e.keyCode === dojo.keys.UP_ARROW && pos > 0) {
    	    this._expertsWidgets[pos - 1].domNode.focus(); 
      	  } else if(e.keyCode === dojo.keys.SPACE) {
      		 this._selectWidget(widget, false); 
      	  }
      });
  },

  _setupInvitedUsers: function _invitedUsers_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // show invited users in a box
    var anker2 = this.invitedCollaborators;
    var taskId = this._tkiid;
    // Keep track of ExpertTile objects in order to destroy them later
    var invitedWidgets = this._invitedWidgets = [];
    // create a list of the invited users
    var invitedUsers = [];
    if (this.task.collaboration.currentUsers && this.task.collaboration.currentUsers.length > 0) {
      for ( var i = 0; i < this.task.collaboration.currentUsers.length; i++) {
        if (this.task.collaboration.currentUsers[i] != this._currentUser) {
          var userName = this.task.collaboration.currentUsers[i];
          var user = {
            "userName": userName
          };
          if (typeof this._userCategory[userName] === "string") {
            user["categoryName"] = this._userCategory[userName];
          }
          invitedUsers.push(user);
        }
      }
    }
    var numberExpertsPerLine = this._numberExpertsPerLine;
    // insert invited users
    var myself = this;
    var counter = 0;
    if (invitedUsers.length > 0) {
      //anker2.style.display = "block";
      // if the box is already full then do not show these entries
      if (counter > numberExpertsPerLine) {
        this.invitedCollaboratorsSection.style.display = "none";
      }
      else {
        this.invitedCollaboratorsSection.style.display = "block";
      }
      dojo.forEach(invitedUsers, function(item){
        counter++;
        item['taskId'] = taskId;
        item['includeTaskDetails'] = false;
        item['sidebar'] = false;
        item['collaborate'] = false;
        var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
        ee.domNode.title = myself._nlsResources["task_dialog_startCollaborationInvited_title"];
        invitedWidgets.push(ee);
        anker2.appendChild(ee.domNode);
        dojo.addClass(ee.domNode, "bpm-social-experts-expert-selected");
        ee.domNodeA11y.innerHTML = "&#10004;";
        if (counter > numberExpertsPerLine) {
          dojo.style(ee.domNode, "display", "none");
        }
      });
      for ( var j = 0; j < invitedWidgets.length; j++) {
        var widget = invitedWidgets[j];
        this.connect(widget, "onClick", "_invitedUserClicked");
      }
    }
    if (counter == 0) {
      dojo.style(this.invitedCollaboratorsSection, "display", "none");
    }
    else {
      dojo.style(this.invitedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMoreInvited, "display", "none");
    }
    else {
      dojo.style(this.showMoreInvited, "display", "block");
    }
    
    // show more functions for invited 
    this.connect(this.showMoreInvited, "onclick", function(){
      dojo.forEach(anker2.childNodes, function(node){
        dojo.style(node, "display", "inline-block");
      });
      dojo.style(this.showBackInvited, "display", "block");
      dojo.style(this.showMoreInvited, "display", "none");
    });
    this.connect(this.showBackInvited, "onclick", function(){
      var i = 0;
      dojo.forEach(anker2.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBackInvited, "display", "none");
      dojo.style(this.showMoreInvited, "display", "block");
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _expertClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, false);
  },

  _invitedUserClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, true);
  },

  _userClicked: function _userClicked_InvitationDialog(event, alreadyInviteed){
    var widget = dijit.getEnclosingWidget(event.target);
    if (widget instanceof com.ibm.bpm.social.util.ui.Avatar) {
      widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
    }
    this._selectWidget(widget, alreadyInviteed);
  },
  
  _selectWidget: function _selectWidget_InvitationDialog(widget, alreadyInviteed){
    if (widget != null) {
      if (!alreadyInviteed && widget.isAdded) {
        // Remove
        widget.isAdded = false;
        dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "";
        dijit.setWaiState(widget.domNode, "selected", "false");
        // Is there a clear function on the user chooser
        this.userChooser.chooserInput.value = "";
        this._checkButtonEnablement();       
      }
      else {
        // only one invitee can be selected. Therefore deselect all the others
        // TODO : grey out the already invited users. They should be display only. 	
        for ( var j = 0; j < this._expertsWidgets.length; j++) {
          var w = this._expertsWidgets[j];
          if (w.userName != widget.userName) {
            w.isAdded = false;
            dojo.removeClass(w.domNode, "bpm-social-experts-expert-selected");
            dijit.setWaiState(w.domNode, "selected", "false");
            w.domNodeA11y.innerHTML = "";
          }
        }
        widget.isAdded = true;
        dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "&#10004;";
        dijit.setWaiState(widget.domNode, "selected", "true");
        var user = {
          "userName": widget.userName,
          "fullName": widget.fullName
        };
        //
        // TODO Validate logic for setting a user on the user chooser
        //
        if (user != null) {
          if (user.fullName != null) {
            this.userChooser._chooseItemValue(user);
          }
          else {
            // Does this make sense given that user is an object !              
            this.userChooser.chooserInput.value = user;
          }
        }
      }      
    }
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getMessageText: function _getMessageText_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var messageText = "[no message specified]"; // User should never see this - no NLS necessary
    if (this.messageBox) {
      if (this.placeHolderState.modified) {
        messageText = this.messageBox.value;
      }
      else {
        messageText = "";
      }
    }
    if (dojo.trim(messageText) === "") {
      messageText = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return messageText;
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getInvitedUserid: function _getInvitedUserid_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // TODO Check implications of selecting/entering groups instead of single user
    var choice = null;
    if (this.userChooser) {
      choice = this.userChooser.getChoice();
      if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
        return {
          value: choice.userName,
          type: "user"
        };
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return choice;
  },

  onInvite: function onInvite_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    var myself = this;

    // if nothing can be don in the dialog it is an cancel button

    var invitedUser = this._getInvitedUserid();
    var userid = "[not selected yet]";
    if (invitedUser) {
      userid = invitedUser.value;
    }
    var message = this._getMessageText();
    var tkiid = this._tkiid; // Need for error handler below
    // 
    // Rest call to backend 
    //    
    this._restHandler.inviteUserToCollaborate(this._tkiid, function(response){
      // Good case => not action necessary
      // insert the invited user in task object.
      if (myself.task.collaboration && myself.task.collaboration.currentUsers) {
        if (dojo.indexOf(myself.task.collaboration.currentUsers, userid) === -1) {
          myself.task.collaboration.currentUsers.push(userid);
        }
      }
    }, function(data, ioArgs){
      var messageParameters = {
        1: tkiid
      };
      // the errorCodeMap will overrule the statusCodeMap
      var errorCodeMap = {
        "CWTBG0007E": {
          key: "KEY_TaskInvitedUserDoesNotExist",
          parameters: {
            1: userid
          }
        },
        "CWTBG0561E": {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        "CWTBG0018E": {
          key: "KEY_TaskInviteTaskNotClaimed"
        },
        "CWTBG0048E": {
          key: "KEY_TaskInviteTaskNotFound"
        },
        "CWTBG0540E": {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      // for error codes not found in the errorCodeMapm use the statusCodeMap
      var statusCodeMap = {
        404: {
          key: "KEY_TaskInviteNotAllowedForTask"
        },
        403: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        401: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        400: {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      var errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
      errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
    }, userid, message);

    this.hide();
    com.ibm.bpm.social.util.common.Log.logExit(1);
  }

});


com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping = {
  "TKIID": "TKIID",
  "NAME": "NAME",
  "DISPLAY_NAME": "DISPLAY_NAME",
  "PI_NAME": "PI_NAME",
  "PIID": "PIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping = {
  "TKIID": "tkiid",
  "NAME": "name",
  "DISPLAY_NAME": "displayName",
  "PI_NAME": "processInstanceName",
  "PIID": "piid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};


/**
 * Opens the InvitationDialog
 * 
 * @param task
 * @param user An object with a userName and fullName property
 * @param user {void} nothing
 */
com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog = function(task, user){
  var nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
  var dialog = new com.ibm.bpm.social.widgets.dialogs.InvitationDialog({
    "title": nlsResources["task_dialog_startCollaboration_title"],
    //"style": "min-width: 50%",
    //"style": "min-height: 50%",
    "task": task
  });
  dialog.showDialog(user);
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.form.Renderer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.form.Renderer"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2011. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.task.form.Renderer");


/**
 * A Renderer is responsible for rendering a form within the Task Information Widget based on the
 * clientSettings of a task or a task template. The result is displayed in the form tab of the task within the
 * Task Information Widget.<br />
 * The purpose of the form depends on the mode in which the task has been opened in the Task Information
 * Widget: <br />
 * <ul>
 * <li>"init" - the form is used to collect the input message for creating or creating and starting a task.</li>
 * <li>"edit" - the form is used to collect the output message while editing a task or completing a task.</li>
 * <li>"view" - the form is used to show the content of the input and output message.</li>
 * </ul>
 * In order to use a Renderer, the Renderer need to be associated with a clientType or defined as a default
 * Renderer using Extension Service. <br />
 * An instance of this module is instanciated for each task form which this Renderer is configured for. <br />
 * <br />
 * The following methods of the Renderer are called by the Task Information Widget and need to be implemented
 * by any Dojo module derived from Renderer:
 * <ul>
 * <li>load()</li>
 * <li>unload()</li>
 * <li>resize(dim)</li>
 * <li>show()</li>
 * <li>hide()</li>
 * <li>isSaveSupported()</li>
 * <li>save() but only if isSaveSupported() returns true</li>
 * <li>enableActions()</li>
 * <li>disableActions()</li>
 * <li>isValid()</li>
 * </ul>
 * <br />
 * The following methods are used to call back to the Task Information Widget. I.e. the Task Information
 * Widget connects to these methods in order get updates about the form and its state.:
 * <ul>
 * <li>onLoaded(tkiid)</li>
 * <li>onSubmitted(tkiid, nextTkiid?)</li>
 * <li>onSaved(tkiid)</li>
 * <li>onCanceled(tkiid)</li>
 * <li>onErrorOccurred(message, description, details, action)</li>
 * <li>onLoadNewTask(followOnTkiid) but only if follow-on task need to ne opened</li>
 * <li>onDirty()</li>
 * <li>onReset()</li>
 * <li>onValid()</li>
 * <li>onInValid()</li>
 * </ul>
 */
dojo.declare("com.ibm.bpm.social.widgets.task.form.Renderer", null, {

  /**
   * Provides the context for the form rendering. <br />
   * <br />
   * 
   * 
   * The properties of the parameter object: <br />
   * <ul>
   * <li>iContext - The iContext object that allows to interact with the environment. </li>
   * <li>htmServiceUrl - The service url for the HTM Rest API</li>
   * <li>bfmServiceUrl - The service url for the BFM Rest API</li>
   * <li>clientType - The client type the renderer was registered for. Might be null for a default renderer.</li>
   * <li>clientSettings - The client settings for the client type. Might be null for a default renderer.</li>
   * <li>task - The task object if available. </li>
   * <li>taskTemplate - The task template object. In case no task is yet available.</li>
   * <li>mode - The mode of operation. </li>
   * </ul>
   * <br />
   * The clientType and its clientSettings provide information about the configuration of the form. <br />
   * The task is not available if the form has been opened to create or create and start a task for a task
   * template. The task template object is provided in case no task is yet available. <br />
   * <br />
   * Possible values for mode are:
   * <ul>
   * <li>"init" - for creating and starting a task.</li>
   * <li>"edit" - for working on a task.</li>
   * <li>"view" - for readonly rendering of the task form.</li>
   * </ul>
   * <br />
   * <br />
   * 
   * @param {object} parameters The context parameter.
   */
  constructor: function(parameters){},

  // ****************** Lifecycle Methods Begin ******************** 

  /**
   * The Task Information Widget calls load as first method after the form renderer has been instantiated. The
   * form is consider to be clean and valid at the beginning. Subsequently the resize function is called with
   * the dimensions. The dimensions allow to establish a block with a fixed size that can be scrolled. (For
   * example, one might not want that the whole form including its buttons can be scrolled.) <br />
   * <br />
   * 
   * @param {DOMElement} element A reference to the element where the form should appear.
   * @type {void}
   */
  load: function(element /* DOM element */){},

  /**
   * Task Information Widget calls unload if the Renderer needs to be destroyed.
   * 
   * This method should also be used to release any resources.
   * 
   * @type {void}
   */
  unload: function(){},

  /**
   * The Task Information Widget calls resize if the widget need to be resized. For example, this allows to
   * implement a block with an absolute height or width which can be scrolled. Resize will be called at least
   * once after the invocation of the load method.
   * 
   * @param {object} dim The dimensions {width:int, height: int}
   * @type {void}
   */
  resize: function(dim /*{width:1,height:1} */){},

  /**
   * The Task Information Widget calls show if the form should become visible.
   * 
   * @type {void}
   */
  show: function(){},

  /**
   * The Task Information Widget calls hide if the form should become hidden.
   * 
   * @type {void}
   */
  hide: function(){},

  // ****************** Lifecycle Methods End ******************** 

  // ****************** Actions Begin *************************** 

  /**
   * The Task Information Widget calls isSaveSupported in order to decide whether it need to render a Save
   * button.
   * 
   * If save is supported, the Task Information Widget provides a save button which triggers the save method
   * of this Renderer if possible. For example, a user might only be entitled to see a task but not entitled
   * to save any changes in which case the save button of the Task Information Widget would be disabled.
   * 
   * @returns {boolean} true if supported.
   */
  isSaveSupported: function(){
    return true; // Default is true
  },

  /**
   * The Task Information Widget calls save if the form content need to be saved.
   * 
   * The method will only be called if isSaveSupported returned true. In case of success, onSaved should be
   * called otherwise onErrorOccurred.
   * 
   * @type {void}
   */
  save: function(){},

  /**
   * The Task Information Widget calls enableActions if the task can be started in case of "init" mode or if
   * the task can be completed in case of "edit" mode.
   * 
   * <br />
   * Basically this method is called if no reasons for disabling the main action do not exist any more. In
   * additino to that the Renderer need to take into account additional pieces of information before enabling
   * actions. For example, the Renderer should only enable actions related to the form content if the provided
   * input is valid.
   * 
   * 
   * @type {void}
   */
  enableActions: function(){},

  /**
   * The Task Information Widget calls disableActions if the task cannot be started in case of "init" mode or
   * if the task cannot be completed in case of "edit" mode.
   * 
   * This might be the case if the task is suspended or waiting for subtasks. Basically this method is called
   * by the Task Information Widget as callback, if any reasons for disabling the main action exist.
   * 
   * @type {void}
   */
  disableActions: function(){},

  // ****************** Actions End *************************** 

  // ****************** Notification Begin ******************** 


  /**
   * onLoaded has to be called if the form has been completely loaded. Added with version 8.0.
   * 
   * This method allows to asynchronously report that the form has been completely loaded.
   * 
   * @param {string} tkiid of the task.
   * @type {void}
   */
  onLoaded: function(tkiid /* string */){},

  /**
   * onSubmitted has to be called if the main action was successful.
   * 
   * These are the scenarios: <br />
   * <br />
   * <ul>
   * <li>Mode is "init" and create and start task has run successfully. The tkiid of the started task is
   * provided.</li>
   * <li> Mode is "edit" and complete task has run successfully. The tkiid of the completed task is provided.</li>
   * </ul>
   * <br />
   * 
   * If a follow-on task should be opened in the Task Information Widget, the onLoadNewTask(followOnTkiid) can
   * be called. Where followOnTkiid is the id of the next task.
   * 
   * In case the next task is shown within the renderer, the tkiid of that task can be reported using the
   * optional nextTkiid parameter. The tab where the form is shown need not be opened again in contrast to a
   * follow-on task. Instead only the details about the task need to be updated.
   * 
   * @param {string} tkiid of the task.
   * @param {string] nextTkiid of the task.
   * @type {void}
   */
  onSubmitted: function(tkiid /* string */, nextTkiid /* string */){},

  /**
   * onSave has to be called called if the task was either created or saved.
   * 
   * These are the scenarios: <br />
   * <br />
   * <ul>
   * <li>Mode is "init" and create task has run successfully. The tkiid of the created task is provided.</li>
   * <li>Mode is "edit" and save task has run successfully. The tkiid of the task is provided.</li>
   * </ul>
   * 
   * @param {string} tkiid The task instance if of the task.
   * @type {void}
   */
  onSaved: function(tkiid /* object */){},

  /**
   * onCanceled has to be called if the user interaction is canceled.
   * 
   * This is the following scenario: <br />
   * <br />
   * <ul>
   * <li>If user canceled his or her work on the form and tab needs to be closed.</li>
   * </ul>
   * 
   * @param {string} tkiid The task instance if of the task.
   * @type {void}
   */
  onCanceled: function(tkiid /* object */){},

  /**
   * onErrorOccurred has to be called if an error occurred.
   * 
   * The Task Information Widget will shown an error popup. Prior to calling this method any controls should
   * be disabled. The parameter allows to specify the information shown in the error popup.
   * 
   * @param {string} message The message to be shown.
   * @param {string} description The description to be shown.
   * @param {string} details The details to be shown.
   * @param {string} action The actions to be shown.
   * @type {void}
   */
  onErrorOccurred: function(message, description, details, action){},

  // ****************** Notification End ********************

  // ****************** Pageflow Handling Begin ******************** 

  /**
   * onLoadNewTask has to be called if a new task should be opened in the Task Information Widget.
   * 
   * This method is particularly helpful if a follow-on task need to be displayed in the Task Information
   * Widget.
   * 
   * @param {string} followOnTkiid The task instance if of the new task.
   * @type {void}
   */
  onLoadNewTask: function(followOnTkiid /* object */){},

  // ****************** Pageflow Handling End ******************** 


  // ****************** Validation and dirty Handling Begin ******************** 

  /**
   * onDirty has to be called if the form has been modified and unsaved work would be lost if the tab is
   * closed.
   * 
   * For dirty forms, closing the tab triggers a confirmation dialog by the Task Information Widget.
   * 
   * @type {void}
   */
  onDirty: function(){},

  /**
   * onReset has to be called if the form does not contain any unsaved information any more.
   * 
   * Basically the form can now be closed without confirmation dialog.
   * 
   * @type {void}
   */
  onReset: function(){},

  /**
   * onValid has to be called if the form is valid.
   * 
   * The method need to be called at the point in time when the form gets valid.
   * 
   * @type {void}
   */
  onValid: function(){},

  /**
   * onInvalid has to be called if the form is invalid.
   * 
   * The method need to be called at the point in the time when the form gets invalid. <br />
   * The load method makes the assumption that the form is initially valid. Therefore onInvalid also need to
   * be called after load() if the form is initially invalid.
   * 
   * @type {void}
   */
  onInvalid: function(){},

  /**
   * Returns whether the form is valid.
   * 
   * @returns boolean true if valid.
   */
  isValid: function(){
    return true;
  }

// ****************** Validation and dirty Handling End ******************** 


});


/**
 * Constant for the "init" mode of the Task Information Widget. In the "init" mode, only the input message is
 * usually rendered. The input message needs to be entered for creating and starting a task instance.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT = "init";

/**
 * Constant for the "edit" mode of the Task Information Widget. In the "edit" mode, the input and the output
 * message are usually rendered. Only the output message needs to be entered or modified for saving and
 * completing the task instance.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT = "edit";

/**
 * Constant for the "view" mode of the Task Information Widget. In the "view" mode, the input and the output
 * message are usually rendered. Neither message needs to be modified.
 */
com.ibm.bpm.social.widgets.task.form.Renderer.MODE_VIEW = "view";

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.form.CoachRenderer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.form.CoachRenderer"] = true;
// CoachRenderer.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.form.CoachRenderer");










/**
 * All logic that is specific to the iWidget implementation or the Process Portal implementation is part of
 * this class which is added as mixin (by adding it to the constructor of the CoachRenderer).
 * 
 * This should help to keep the implementation of
 * 
 * com.ibm.bpc.widget.editor.form.CoachRenderer
 * 
 * and
 * 
 * com.ibm.bpm.social.widgets.task.form.CoachRenderer
 * 
 * in sync.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.form.CoachRendererProcessPortalHelper", null, {

  MODE_INIT: com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT,
  MODE_EDIT: com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT,

  constructor: function(parameters){
    // Helper for logging.
    this.logHelper = com.ibm.bpm.social.util.common.Log;
    // Is there already a callback object?        
    if (typeof window["BPM_CoachCallback"] != "object") {
      //
      // The used correlation token is either the taskId or generated correlationToken.
      // 
      window["BPM_CoachCallback"] = com.ibm.bpm.social.widgets.task.form.RendererPool.getInstance();
    }
  },
  setupRestHandler: function(){
    // Set the rest urls
    this.restHandler = new com.ibm.bpm.social.util.common.TaskRestHandler();
  },

  _getTaskState: function(){
    var state = null;
    var taskObject = null;
    this.restHandler.getTask(this.task.tkiid, function(responseObject, ioArgs){
      taskObject = responseObject;
    }, function(data, ioArgs){
    // Problems retrieving the task...
    }, false, null, this.task.systemID);
    if (taskObject !== null) {
      state = taskObject.state;
    }
    return state;
  },

  _log: function log_CoachRenderer(level, message){
    this.logHelper.log(level, message);
  },

  /**
   * calls exitCollaboration on coach
   * 
   * @returns {void} nothing
   */
  exitTaskCollaboration: function exitTaskCollaboration_CoachRenderer(){
    try {
      //exit task collaboration
      if (this.frame && this.frame.contentWindow && this.frame.contentWindow["exitCollaboration"]) {
        this.frame.contentWindow["exitCollaboration"](true);
      }
    }
    catch (e) {
      // In case there are any security violations due to different domains
    }
  }

});

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.form.CoachRenderer",
    [dijit._Widget,
     dijit._Templated,
     com.ibm.bpm.social.widgets.task.form.Renderer,
     com.ibm.bpm.social.widgets.task.form.CoachRendererProcessPortalHelper],
    {

      templateString: '<div dojoAttachPoint="root">'
        + '<iframe dojoAttachPoint="frame" style="border:0px;width:100%;height:100%;min-width:0px; min-height:0px;overflow: auto" frameborder="0"></iframe>'
        + '</div>',

      isNGCoach: false,

      /**
       * Constructor that sets all the parameter that are required according to the interface description.
       * 
       * @param parameters
       * @returns {void} nothing
       */
      constructor: function(parameters){
        this.iContext = parameters.iContext;
        this.task = parameters.task;
        this.taskTemplate = parameters.taskTemplate;
        this.mode = parameters.context;
        this.clientType = parameters.clientType;
        this.clientSettings = parameters.clientSettings;
        this.parentTKIID = parameters.parentTKIID;
        this.setupRestHandler();
        this._isTryCoachReload = false;
        this.objectId = this.task ? this.task.tkiid : this.taskTemplate.tktid;
        this._isDialog = parameters.dialog;
      },

      _createCallbackAdapter: function _createCallbackAdapter_CoachRenderer(){
        this.logHelper.logEntry(2);
        var callbackObject = window["BPM_CoachCallback"];
        this.connect(callbackObject, "onInitialized", "_onInitialized");
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.connect(callbackObject, "onInitialized", "_onInitialized4Ipad");
        }
        this.connect(callbackObject, "onCompleted", "_onCompleted");
        this.connect(callbackObject, "onError", "_onError");
        this.connect(callbackObject, "onSuspended", "_onSuspended");

        // this is a workaround for IE to handle the iframe's onload event correctly

        if (this.frame.attachEvent) {
          this._onResourceLoadedImplFn = dojo.hitch(this, "_onResourceLoadedImpl");
          this.frame.attachEvent("onload", this._onResourceLoadedImplFn);
        }
        else {
          this.connect(this.frame, "onload", "_onResourceLoadedImpl");
        }

        this.logHelper.logExit(2);
      },

      _onResourceLoadedImpl: function _onResourceLoadedImpl_CoachRenderer(){
        var source = "";
        try {
          source = this.frame.contentWindow.location.pathname;
        }
        catch (e) {
          // Security violance due to Same Origin Policy for iFrame should not 
          // happen as for Process Portal, the server and port should be the same.
          // For Business Space a proxy is used.
          source = "";
        }
        this.onResourceLoaded(source);
      },

      /**
       * Which resource has been loaded is unknown. {void} nothing
       */
      onResourceLoaded: function onResourceLoaded_CoachRenderer(url){},

      _onInitialized: function _onInitialized_CoachRenderer(frame, taskID, isOldCoach){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onInitialized Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          var wasOldCoach = !this.isNGCoach;
          this.isNGCoach = !isOldCoach;
          if (typeof taskID === "string") {
            // Only heritage coaches (=old coaches) do not notify about auto flow to next task. 
            if (taskID.substr(0, 1) == "t" && wasOldCoach) {
              // New taskID encountered => autoflow for old-style coaches
              // Remove the prefix
              taskID = taskID.substring(1);
              if (taskID != this.objectId) {
                var originalID = this.objectId;
                this.objectId = taskID;
                this.onSubmitted(originalID, taskID);
              }
            }
            else if (taskID.substr(0, 1) == "p") {
              this._log(2, "Initiating task " + taskID + " encountered");
            }
          }
          this.onLoaded(this.objectId, !isOldCoach);
        }
        this._log(2, "CoachRenderer._onInitialized Exit " + this.objectId);
      },

      _onCompleted: function _onCompleted_CoachRenderer(frame, nextTkiid){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onCompleted Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          
          // Make sure that the keyboard is closed after the coach was completed.
          if (com.ibm.bpm.social.util.common.Util.isIPad) {
        	frame.document.activeElement.blur();
          }
          
          if (typeof nextTkiid == "string") {
            if (nextTkiid.substr(0, 1) == "t") {
              // Remove the prefix
              nextTkiid = nextTkiid.substring(1);
            }
            var originalID = this.objectId;
            this.objectId = nextTkiid;
            this.onSubmitted(originalID, nextTkiid);
          }
          else {
            if (this.mode === this.MODE_INIT) {
              // A human service was started, as the human service does not get persisted and 
              // we do not know anything about the implementation we just want to close 
              // the tab without firing any event
              this.onCanceled(this.objectId);
            }
            else {
              // Do we need to check the state of the task?
              // Is it possible to cancel a coach using a submit button of the form?
              this.onSubmitted(this.objectId);
            }
          }
        }
        this._log(2, "CoachRenderer._onCompleted Exit " + this.objectId);
      },

      _onError: function _onError_CoachRenderer(frame, message){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onError Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          if (message === "tm_task_closed") {
            this.onErrorOccurred("task-row-details-error-not-available");
          }
          else if (message === "tm_task_not_owned") {
            this.onErrorOccurred("user_not_authorized_for_task");
          }
          else {
            this.onErrorOccurred(message);
          }
        }
        this._log(2, "CoachRenderer._onError Exit " + this.objectId);
      },

      _onSuspended: function _onSuspended_CoachRenderer(frame){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onSuspended Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          this.onCanceled();
        }
        this._log(2, "CoachRenderer._onSuspended Exit " + this.objectId);
      },

      /*
       * Because there are problems with bounded iFrames on iPad, the coach will 
       * be displayed in full size and the viewport will be changed so that the user
       * can scroll the entire page to see the complete coach.
       */
      _onInitialized4Ipad: function _onInitialized4Ipad_CoachRenderer(frame, taskID, isOldCoach){
        if (frame == this.frame.contentWindow) {
          var viewport = document.querySelector("meta[name=viewport]");
        	 
      	  // In case of a coach flow, this method is called for each coach step. To assure that the 
          // previous viewport calculation does not influence the following, the original settings are 
          // restored.
          if (this.processPortalVPSettings) {
   	 		// Make sure all calculated width and height settings are removed...
   	 		viewport.setAttribute('content', 'width = device-width, height = device-height');
   	 		//... and then return to the old settings
   	 		viewport.setAttribute('content', this.processPortalVPSettings);
   	 	  } else {
   	 		// Store the process portal viewport settings to restore it sometimes 
            this.processPortalVPSettings = viewport.getAttribute("content");  			  
   		  }	
          	
          var frameWidth = frame.document.body.clientWidth;
          var frameHeight = frame.document.body.clientHeight;          

  		  var newWidth = "device-width";
          // 394 = 314 (sidebar) + 2x35 (margins) + 10 (random pixels)
  		  var frameWidthDelta = 394;
  		  if (frameWidth > window.innerWidth - frameWidthDelta) {
  			  // pixels are hardcoded, because out of some reasons dynamic calculation didn't work
  			  // properly here
              newWidth = frameWidth + frameWidthDelta;
            }

            var newHeight = "device-height";
            // 219 = 56 (top bar) + 55 (coach header) + 28 (bottom bar) + 2x35 (margins) + 10 (random pixels)
            var frameHeightDelta = 219;
            if (frameHeight > window.innerHeight - frameHeightDelta) {
			  // pixels are hardcoded, because out of some reasons dynamic calculation didn't work
			  // properly here
              newHeight = frameHeight + frameHeightDelta;
            }

            viewport.setAttribute('content', 'width = ' + newWidth + ', height = ' + newHeight +
              ', initial-scale = 1.0, minimum-scale = 1.0');

            // Scroll to the right border, if the width was changed in rtl mode.
            if (newWidth !== "device-width" && document.body.dir === "rtl") {
              window.scrollTo(newWidth, 0);
            }
        }
      },

      /**
       * Creates the form. Beware that the elem is not part of the document yet.
       * 
       * @param elem - anchor element where the form is added.
       * 
       * @returns {void} nothing
       */
      load: function(elem /* DOM element */){
        elem.appendChild(this.domNode);
        this._createCallbackAdapter();
        // Retrieve the url and set it on the iframe
        var url = this.clientSettings.url;
        // Get the coach debug trace settings from the cookie and set them on the url
        var coachDebug = dojo.cookie("com_ibm_bpm_coach_debug_logger");
        if (!!coachDebug) {
          url += coachDebug;
        }

        // A11y: set the title of the iFrame
        if (this.task) {
          this.frame.title = this.task.displayName;
        }
        else if (this.taskTemplate) {
          this.frame.title = this.taskTemplate.displayName;
        }

        this.frame.src = url;
      },

      /**
       * Resizes the form in order to allow sophisticated scrolling.
       * 
       * @param dim
       * @returns {void} nothing
       */
      resize: function(dim /*{width:1,height:1} */){
        //Blocked due to mashups defect 18110:
        //TODO: handle case when widget is maximized
        //TODO: handle case where Coach input requires more height than the default
        //
        // Calculate width, height. Math.max makes sure that the numbers are positive.
        //

        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          // Because the viewport matches, auto width is ok here.
          this.root.style.width = "auto";
        }
        else {
          var frameWidth = "100%";
          if (typeof dim.width == "number" && isNaN(dim.width) !== true) {
            frameWidth = Math.max(0, dim.width) + "px";
          }
          this.frame.style.width = frameWidth;
        }

        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          // Because the viewport matches, auto height is ok here.
          this.root.style.height = "auto";
        }
        else {
          var frameHeight = "100%";
          if (typeof dim.height == "number" && isNaN(dim.height) !== true) {
            frameHeight = Math.max(0, dim.height) + "px";
          }
          this.frame.style.height = frameHeight;
        }
      },

      /**
       * Destroys the widget.
       * 
       * 
       * @returns {void} nothing
       */
      unload: function unload_CoachRenderer(){
        if (com.ibm.bpm.social.util.common.Util.isIPad && this.processPortalVPSettings) {
          var viewport = document.querySelector("meta[name=viewport]");
          // Make sure all calculated width and height settings are removed...
          viewport.setAttribute('content', 'width = device-width, height = device-height');
          //... and then return to the old settings
          viewport.setAttribute('content', this.processPortalVPSettings);
        }

        this.destroyRecursive();
      },

      /**
       * Shows the form.
       * 
       * @returns {void} nothing
       */
      show: function(){
        this.root.style.visibility = "visible";
      },

      /**
       * Hides the form.
       * 
       * @returns {void} nothing
       */
      hide: function(){
        this.root.style.visibility = "hidden";
      },

      /**
       * Returns whether the form is valid.
       * 
       * @returns boolean - true if valid.
       */
      isValid: function(){
        return true;
      },

      /**
       * Disables any controls.
       * 
       * @param message
       * @param details
       * @param action
       * @returns {void} nothing
       */
      onErrorOccurred: function(message, description, details, action){

      },

      /**
       * Save is not supported
       * 
       * @returns {boolean} false
       */
      isSaveSupported: function(){
        return false;
      },

      /**
       * Implementation of save in "edit" mode.
       * 
       * @returns {void} nothing
       */
      save: function(){
      // Not supported
      },


      /**
       * Called by the Task Information Widget if the task can be started in case of init mode or if the task
       * can be completed in case of edit mode.
       * 
       * @returns {void} nothing
       */
      enableActions: function(){

      },

      /**
       * Called by the Task Information Widget if the task cannot be started in case of init mode or if the
       * task cannot be completed in case of edit mode. This might be the case if the task is suspended or the
       * waiting for subtasks.
       * 
       * @returns {void} nothing
       */
      disableActions: function(){

      },

      /**
       * ************************** Technical methods that are not part of the interface
       * ************************
       */

      /**
       * Destroys all the widgets.
       * 
       * @returns {void} nothing
       */
      destroy: function(){
        this.exitTaskCollaboration();
        this.inherited("destroy", arguments);
      }

    });

// 
// Can be removed in the context of the pending tech-debt items.
//
dojo.declare("com.ibm.bpm.social.widgets.task.form.RendererPool", null, {
  unkownToken: 0,
  knownToken: {},
  handle: null,

  constructor: function(){
    // Establish listener for onMessage events for the HTML5 postMessage events.
	// *****  When cross document messaging is turned off this connect won't work. In that
	//        situation, the _dispatch function will be called directly by com.ibm.bpm.socialportal.onMessage() which
	//        is defined in modules\bpmsocial\src\main\webapp\jsp\index.jsp  ******
    this.handle = dojo.connect(window, "message", this, "_dispatch");
  },

  destroy: function(){
    if (this.handle) {
      dojo.disconnect(this.handle);
    }
  },

  _dispatch: function _dispatch_RendererPool(event){
    var payloadLiteral = event.data;
    try {
      //
      // Only take the event seriously if the payload is json and
      // there is a property "name" of type string.
      // Otherwise this is probably some other event posted using postMessage in 
      // this instance of the browser.
      //
      var payload = dojo.fromJson(payloadLiteral);
      if (typeof payload.name == "string") {
        var eventType = payload["name"];
        var taskID = null;
        if (typeof payload["taskID"] === "string") {
          // taskID is optional
          taskID = payload["taskID"];
        }
        var broadcastingWindow = event.source;
        var firstParameter = null;
        var secondParameter = null;
        if (payload.parameters && payload.parameters.length > 0 && typeof payload.parameters[0] === "string") {
          if (payload.parameters[0].length > 0) {
            firstParameter = payload.parameters[0];
          }
          if (payload.parameters[1] && typeof payload.parameters[1] === "string") {
            secondParameter = payload.parameters[1];
          }
        }
        switch (eventType) {
          case "onInitialized" :
            if (firstParameter == "oldCoach") {
              this.onInitialized(broadcastingWindow, taskID, true /* old Coach */);
            }
            else {
              this.onInitialized(broadcastingWindow, taskID, false /* new Coach */);
            }
            break;
          case "onCompleted" :
            if (firstParameter) {
              // autoflow to next tkiid
              var nextTkiid = firstParameter;
              this.onCompleted(broadcastingWindow, nextTkiid);
            }
            else {
              this.onCompleted(broadcastingWindow, null);
            }
            break;
          case "onError" :
            if (secondParameter !== null) {
              this.onError(broadcastingWindow, secondParameter);
            }
            else {
              this.onError(broadcastingWindow, firstParameter);
            }
            break;
          case "onSuspended" :
            this.onSuspended(broadcastingWindow);
            break;
          default:
            this._log(2, "Ignoring unknown event type: " + eventType);
            break;
        }
      }
    }
    catch (e) {
      // Ignore the event. There might be other events using postMessage.
      this._log(2, "Ignoring non-JSON event: " + payloadLiteral);
    }
  },

  onInitialized: function(/* window */broadcastingWindow,/* string */taskID, /* boolean */isOldCoach){},

  onCompleted: function(/* window */broadcastingWindow,/* string */newTkiid){},

  onError: function(/* window */broadcastingWindow,/* string */message){},

  onSuspended: function(/* window */broadcastingWindow){},

  _log: function(level, message){
    com.ibm.bpm.social.util.common.Log.log(level, message);
  },

  statics: {
    _instance: null,
    _getInstance: function(){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.widgets.task.form.RendererPool();
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.widgets.task.form.RendererPool.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.form.RendererPool.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["dijit.layout.BorderContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.BorderContainer"] = true;
dojo.provide("dijit.layout.BorderContainer");





dojo.declare(
	"dijit.layout.BorderContainer",
	dijit.layout._LayoutWidget,
{
	// summary:
	//		Provides layout in up to 5 regions, a mandatory center with optional borders along its 4 sides.
	//
	// description:
	//		A BorderContainer is a box with a specified size, such as style="width: 500px; height: 500px;",
	//		that contains a child widget marked region="center" and optionally children widgets marked
	//		region equal to "top", "bottom", "leading", "trailing", "left" or "right".
	//		Children along the edges will be laid out according to width or height dimensions and may
	//		include optional splitters (splitter="true") to make them resizable by the user.  The remaining
	//		space is designated for the center region.
	//
	//		The outer size must be specified on the BorderContainer node.  Width must be specified for the sides
	//		and height for the top and bottom, respectively.  No dimensions should be specified on the center;
	//		it will fill the remaining space.  Regions named "leading" and "trailing" may be used just like
	//		"left" and "right" except that they will be reversed in right-to-left environments.
	//
	//		For complex layouts, multiple children can be specified for a single region.   In this case, the
	//		layoutPriority flag on the children determines which child is closer to the edge (low layoutPriority)
	//		and which child is closer to the center (high layoutPriority).   layoutPriority can also be used
	//		instead of the design attribute to conrol layout precedence of horizontal vs. vertical panes.
	// example:
	// |	<div dojoType="dijit.layout.BorderContainer" design="sidebar" gutters="false"
	// |            style="width: 400px; height: 300px;">
	// |		<div dojoType="dijit.layout.ContentPane" region="top">header text</div>
	// |		<div dojoType="dijit.layout.ContentPane" region="right" splitter="true" style="width: 200px;">table of contents</div>
	// |		<div dojoType="dijit.layout.ContentPane" region="center">client area</div>
	// |	</div>

	// design: String
	//		Which design is used for the layout:
	//			- "headline" (default) where the top and bottom extend
	//				the full width of the container
	//			- "sidebar" where the left and right sides extend from top to bottom.
	design: "headline",

	// gutters: [const] Boolean
	//		Give each pane a border and margin.
	//		Margin determined by domNode.paddingLeft.
	//		When false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.
	gutters: true,

	// liveSplitters: [const] Boolean
	//		Specifies whether splitters resize as you drag (true) or only upon mouseup (false)
	liveSplitters: true,

	// persist: Boolean
	//		Save splitter positions in a cookie.
	persist: false,

	baseClass: "dijitBorderContainer",

	// _splitterClass: String
	// 		Optional hook to override the default Splitter widget used by BorderContainer
	_splitterClass: "dijit.layout._Splitter",

	postMixInProperties: function(){
		// change class name to indicate that BorderContainer is being used purely for
		// layout (like LayoutContainer) rather than for pretty formatting.
		if(!this.gutters){
			this.baseClass += "NoGutter";
		}
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }
		dojo.forEach(this.getChildren(), this._setupChild, this);
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget._setupChild().

		var region = child.region;
		if(region){
			this.inherited(arguments);

			dojo.addClass(child.domNode, this.baseClass+"Pane");

			var ltr = this.isLeftToRight();
			if(region == "leading"){ region = ltr ? "left" : "right"; }
			if(region == "trailing"){ region = ltr ? "right" : "left"; }

			// Create draggable splitter for resizing pane,
			// or alternately if splitter=false but BorderContainer.gutters=true then
			// insert dummy div just for spacing
			if(region != "center" && (child.splitter || this.gutters) && !child._splitterWidget){
				var _Splitter = dojo.getObject(child.splitter ? this._splitterClass : "dijit.layout._Gutter");
				var splitter = new _Splitter({
					id: child.id + "_splitter",
					container: this,
					child: child,
					region: region,
					live: this.liveSplitters
				});
				splitter.isSplitter = true;
				child._splitterWidget = splitter;

				dojo.place(splitter.domNode, child.domNode, "after");

				// Splitters aren't added as Contained children, so we need to call startup explicitly
				splitter.startup();
			}
			child.region = region;	// TODO: technically wrong since it overwrites "trailing" with "left" etc.
		}
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		this._layoutChildren();
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Override _LayoutWidget.addChild().
		this.inherited(arguments);
		if(this._started){
			this.layout(); //OPT
		}
	},

	removeChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget.removeChild().

		var region = child.region;
		var splitter = child._splitterWidget
		if(splitter){
			splitter.destroy();
			delete child._splitterWidget;
		}
		this.inherited(arguments);
		
		if(this._started){
			this._layoutChildren();
		}
		// Clean up whatever style changes we made to the child pane.
		// Unclear how height and width should be handled.
		dojo.removeClass(child.domNode, this.baseClass+"Pane");
		dojo.style(child.domNode, {
			top: "auto",
			bottom: "auto",
			left: "auto",
			right: "auto",
			position: "static"
		});
		dojo.style(child.domNode, region == "top" || region == "bottom" ? "width" : "height", "auto");
	},

	getChildren: function(){
		// Override _LayoutWidget.getChildren() to only return real children, not the splitters.
		return dojo.filter(this.inherited(arguments), function(widget){
			return !widget.isSplitter;
		});
	},

	// TODO: remove in 2.0
	getSplitter: function(/*String*/region){
		// summary:
		//		Returns the widget responsible for rendering the splitter associated with region
		// tags:
		//		deprecated
		return dojo.filter(this.getChildren(), function(child){
			return child.region == region;
		})[0]._splitterWidget;
	},

	resize: function(newSize, currentSize){
		// Overrides _LayoutWidget.resize().

		// resetting potential padding to 0px to provide support for 100% width/height + padding
		// TODO: this hack doesn't respect the box model and is a temporary fix
		if(!this.cs || !this.pe){
			var node = this.domNode;
			this.cs = dojo.getComputedStyle(node);
			this.pe = dojo._getPadExtents(node, this.cs);
			this.pe.r = dojo._toPixelValue(node, this.cs.paddingRight);
			this.pe.b = dojo._toPixelValue(node, this.cs.paddingBottom);

			dojo.style(node, "padding", "0px");
		}

		this.inherited(arguments);
	},

	_layoutChildren: function(/*String?*/ changedChildId, /*Number?*/ changedChildSize){
		// summary:
		//		This is the main routine for setting size/position of each child.
		// description:
		//		With no arguments, measures the height of top/bottom panes, the width
		//		of left/right panes, and then sizes all panes accordingly.
		//
		//		With changedRegion specified (as "left", "top", "bottom", or "right"),
		//		it changes that region's width/height to changedRegionSize and
		//		then resizes other regions that were affected.
		// changedChildId:
		//		Id of the child which should be resized because splitter was dragged.
		// changedChildSize:
		//		The new width/height (in pixels) to make specified child

		if(!this._borderBox || !this._borderBox.h){
			// We are currently hidden, or we haven't been sized by our parent yet.
			// Abort.   Someone will resize us later.
			return;
		}

		// Generate list of wrappers of my children in the order that I want layoutChildren()
		// to process them (i.e. from the outside to the inside)
		var wrappers = dojo.map(this.getChildren(), function(child, idx){
			return {
				pane: child,
				weight: [
					child.region == "center" ? Infinity : 0,
					child.layoutPriority,
					(this.design == "sidebar" ? 1 : -1) * (/top|bottom/.test(child.region) ? 1 : -1),
					idx
				]
			};
		}, this);
		wrappers.sort(function(a, b){
			var aw = a.weight, bw = b.weight;
			for(var i=0; i<aw.length; i++){
				if(aw[i] != bw[i]){
					return aw[i] - bw[i];
				}
			}
			return 0;
		});

		// Make new list, combining the externally specified children with splitters and gutters
		var childrenAndSplitters = [];
		dojo.forEach(wrappers, function(wrapper){
			var pane = wrapper.pane;
			childrenAndSplitters.push(pane);
			if(pane._splitterWidget){
				childrenAndSplitters.push(pane._splitterWidget);
			}
		});

		// Compute the box in which to lay out my children
		var dim = {
			l: this.pe.l,
			t: this.pe.t,
			w: this._borderBox.w - this.pe.w,
			h: this._borderBox.h - this.pe.h
		};

		// Layout the children, possibly changing size due to a splitter drag
		dijit.layout.layoutChildren(this.domNode, dim, childrenAndSplitters,
			changedChildId, changedChildSize);
	},

	destroyRecursive: function(){
		// Destroy splitters first, while getChildren() still works
		dojo.forEach(this.getChildren(), function(child){
			var splitter = child._splitterWidget;
			if(splitter){
				splitter.destroy();
			}
			delete child._splitterWidget;
		});

		// Then destroy the real children, and myself
		this.inherited(arguments);
	}
});

// This argument can be specified for the children of a BorderContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// region: [const] String
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Values: "top", "bottom", "leading", "trailing", "left", "right", "center".
	//		See the `dijit.layout.BorderContainer` description for details.
	region: '',

	// layoutPriority: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Children with a higher layoutPriority will be placed closer to the BorderContainer center,
	//		between children with a lower layoutPriority.
	layoutPriority: 0,

	// splitter: [const] Boolean
	//		Parameter for child of `dijit.layout.BorderContainer` where region != "center".
	//		If true, enables user to resize the widget by putting a draggable splitter between
	//		this widget and the region=center widget.
	splitter: false,

	// minSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a minimum size (in pixels) for this widget when resized by a splitter.
	minSize: 0,

	// maxSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a maximum size (in pixels) for this widget when resized by a splitter.
	maxSize: Infinity
});

dojo.declare("dijit.layout._Splitter", [ dijit._Widget, dijit._Templated ],
{
	// summary:
	//		A draggable spacer between two items in a `dijit.layout.BorderContainer`.
	// description:
	//		This is instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create it directly.
	// tags:
	//		private

/*=====
 	// container: [const] dijit.layout.BorderContainer
 	//		Pointer to the parent BorderContainer
	container: null,

	// child: [const] dijit.layout._LayoutWidget
	//		Pointer to the pane associated with this splitter
	child: null,

	// region: [const] String
	//		Region of pane associated with this splitter.
	//		"top", "bottom", "left", "right".
	region: null,
=====*/

	// live: [const] Boolean
	//		If true, the child's size changes and the child widget is redrawn as you drag the splitter;
	//		otherwise, the size doesn't change until you drop the splitter (by mouse-up)
	live: true,

	templateString: '<div class="dijitSplitter" dojoAttachEvent="onkeypress:_onKeyPress,onmousedown:_startDrag,onmouseenter:_onMouse,onmouseleave:_onMouse" tabIndex="0" role="separator"><div class="dijitSplitterThumb"></div></div>',

	postMixInProperties: function(){
		this.inherited(arguments);

		this.horizontal = /top|bottom/.test(this.region);
		this._factor = /top|left/.test(this.region) ? 1 : -1;
		this._cookieName = this.container.id + "_" + this.region;
	},

	buildRendering: function(){
		this.inherited(arguments);

		dojo.addClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V"));

		if(this.container.persist){
			// restore old size
			var persistSize = dojo.cookie(this._cookieName);
			if(persistSize){
				this.child.domNode.style[this.horizontal ? "height" : "width"] = persistSize;
			}
		}
	},

	_computeMaxSize: function(){
		// summary:
		//		Return the maximum size that my corresponding pane can be set to

		var dim = this.horizontal ? 'h' : 'w',
			childSize = dojo.marginBox(this.child.domNode)[dim],
			center = dojo.filter(this.container.getChildren(), function(child){ return child.region == "center";})[0],
			spaceAvailable = dojo.marginBox(center.domNode)[dim];	// can expand until center is crushed to 0

		return Math.min(this.child.maxSize, childSize + spaceAvailable);
	},

	_startDrag: function(e){
		if(!this.cover){
			this.cover = dojo.doc.createElement('div');
			dojo.addClass(this.cover, "dijitSplitterCover");
			dojo.place(this.cover, this.child.domNode, "after");
		}
		dojo.addClass(this.cover, "dijitSplitterCoverActive");

		// Safeguard in case the stop event was missed.  Shouldn't be necessary if we always get the mouse up.
		if(this.fake){ dojo.destroy(this.fake); }
		if(!(this._resize = this.live)){ //TODO: disable live for IE6?
			// create fake splitter to display at old position while we drag
			(this.fake = this.domNode.cloneNode(true)).removeAttribute("id");
			dojo.addClass(this.domNode, "dijitSplitterShadow");
			dojo.place(this.fake, this.domNode, "after");
		}
		dojo.addClass(this.domNode, "dijitSplitterActive dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
		if(this.fake){
			dojo.removeClass(this.fake, "dijitSplitterHover dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover");
		}

		//Performance: load data info local vars for onmousevent function closure
		var factor = this._factor,
			isHorizontal = this.horizontal,
			axis = isHorizontal ? "pageY" : "pageX",
			pageStart = e[axis],
			splitterStyle = this.domNode.style,
			dim = isHorizontal ? 'h' : 'w',
			childStart = dojo.marginBox(this.child.domNode)[dim],
			max = this._computeMaxSize(),
			min = this.child.minSize || 20,
			region = this.region,
			splitterAttr = region == "top" || region == "bottom" ? "top" : "left",	// style attribute of splitter to adjust
			splitterStart = parseInt(splitterStyle[splitterAttr], 10),
			resize = this._resize,
			layoutFunc = dojo.hitch(this.container, "_layoutChildren", this.child.id),
			de = dojo.doc;

		this._handlers = (this._handlers || []).concat([
			dojo.connect(de, "onmousemove", this._drag = function(e, forceResize){
				var delta = e[axis] - pageStart,
					childSize = factor * delta + childStart,
					boundChildSize = Math.max(Math.min(childSize, max), min);

				if(resize || forceResize){
					layoutFunc(boundChildSize);
				}
				// TODO: setting style directly (usually) sets content box size, need to set margin box size
				splitterStyle[splitterAttr] = delta + splitterStart + factor*(boundChildSize - childSize) + "px";
			}),
			dojo.connect(de, "ondragstart", dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent),
			dojo.connect(de, "onmouseup", this, "_stopDrag")
		]);
		dojo.stopEvent(e);
	},

	_onMouse: function(e){
		var o = (e.type == "mouseover" || e.type == "mouseenter");
		dojo.toggleClass(this.domNode, "dijitSplitterHover", o);
		dojo.toggleClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover", o);
	},

	_stopDrag: function(e){
		try{
			if(this.cover){
				dojo.removeClass(this.cover, "dijitSplitterCoverActive");
			}
			if(this.fake){ dojo.destroy(this.fake); }
			dojo.removeClass(this.domNode, "dijitSplitterActive dijitSplitter"
				+ (this.horizontal ? "H" : "V") + "Active dijitSplitterShadow");
			this._drag(e); //TODO: redundant with onmousemove?
			this._drag(e, true);
		}finally{
			this._cleanupHandlers();
			delete this._drag;
		}

		if(this.container.persist){
			dojo.cookie(this._cookieName, this.child.domNode.style[this.horizontal ? "height" : "width"], {expires:365});
		}
	},

	_cleanupHandlers: function(){
		dojo.forEach(this._handlers, dojo.disconnect);
		delete this._handlers;
	},

	_onKeyPress: function(/*Event*/ e){
		// should we apply typematic to this?
		this._resize = true;
		var horizontal = this.horizontal;
		var tick = 1;
		var dk = dojo.keys;
		switch(e.charOrCode){
			case horizontal ? dk.UP_ARROW : dk.LEFT_ARROW:
				tick *= -1;
//				break;
			case horizontal ? dk.DOWN_ARROW : dk.RIGHT_ARROW:
				break;
			default:
//				this.inherited(arguments);
				return;
		}
		var childSize = dojo._getMarginSize(this.child.domNode)[ horizontal ? 'h' : 'w' ] + this._factor * tick;
		this.container._layoutChildren(this.child.id, Math.max(Math.min(childSize, this._computeMaxSize()), this.child.minSize));
		dojo.stopEvent(e);
	},

	destroy: function(){
		this._cleanupHandlers();
		delete this.child;
		delete this.container;
		delete this.cover;
		delete this.fake;
		this.inherited(arguments);
	}
});

dojo.declare("dijit.layout._Gutter", [dijit._Widget, dijit._Templated],
{
	// summary:
	// 		Just a spacer div to separate side pane from center pane.
	//		Basically a trick to lookup the gutter/splitter width from the theme.
	// description:
	//		Instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create directly.
	// tags:
	//		private

	templateString: '<div class="dijitGutter" role="presentation"></div>',

	postMixInProperties: function(){
		this.inherited(arguments);
		this.horizontal = /top|bottom/.test(this.region);
	},

	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "dijitGutter" + (this.horizontal ? "H" : "V"));
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2011, 2012. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget");























dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: dojo.cache("com.ibm.bpm.social.widgets.task.form", "templates/TaskCompletionWidget.html", "\n<div class=\"bpm-task-coach-widget\">\t\t\n\t<div dojoAttachPoint=\"header\" class=\"bpm-task-coach-header\">\t\t\n\t    <div class=\"bpm-task-header-subject\">\n\t    \t<div dojoAttachPoint=\"fullSizeControl\" class=\"bpm-social-full-screen\" tabIndex=\"0\"><span dojoAttachPoint=\"fullSizeControlA11y\"></span></div>\n\t\t    <div class=\"bpm-task-header-due\" dojoAttachPoint=\"dueDateSection\">\n\t\t\t   <span>${due_date_label} </span><em dojoAttachPoint=\"dueDateArea\"></em>\n\t\t    </div>\n\t    \t<h2 dojoAttachPoint=\"subjectArea\"></h2>\n\t    \t<div dojoAttachPoint=\"menuAnker\" class=\"bpm-social-drop-down\" aria-label=\"Actions\" tabIndex=\"0\" style=\"display:none\" waiRole=\"button\"><div class=\"bpm-social-drop-down-text\">&#9660;</div></div>\n\t    </div>\t\t\n\t\t<div style=\"clear: both\"></div>\n\t</div>\t\t\n\t<div  dojoAttachPoint=\"loadingSection\" class=\"bpm-task-coach-loading\" style=\"display: none\"><div>${loading_label}</div></div>\n\t<div dojoAttachPoint=\"coachContainerAnker\" class=\"bpm-task-coach-container\"></div>\n</div> \n"),
      widgetsInTemplate: true,

      //
      // dojoAttachPoints out of template:
      //
      // 0) The header area.
      // 2) The full size control
      // 3) Subject area
      // 4) Menu anker
      // 5) The due date section
      // 6) The actual due date control
      // 7) Anker for coach container
      header: null,
      fullSizeControl: null,
      subjectArea: null,
      menuAnker: null,
      dueDateSection: null,
      dueDateArea: null,
      coachContainerAnker: null,
	  fullSize: false,
      //
      // Properties referenced in the template:
      // (Which are set in the constructor)
      //
      due_date_label: "",
      back_link_to_work: "",
      maximize_label: "",
      restore_label: "",
      loading_label: "",

      // State of the fullSizeControl 
      // For template fullSize is always true therefore the user preference (for this session) need to be stored separately
      fullSize: false,
      fullSizeUserSelection: false,

      // The coachRenderer that listens to the coach.     
      coachRenderer: null,

      // Utility for making calls against the rest api.
      restHandler: null,
      // If the header has been updated already, it stays there during an error message.
      headerUpdated: false,

      //
      // Set of properties which need to be kept between the asynchronous calls to 
      // a) fetch the clientSetttings
      // b) fetch the actions of the task (Authorization verification)
      // which are cleared with the _clear method.
      //
      _id: "", // Used for tracing purposes
      _task: null,
      _taskTemplate: null,
      _clientType: null,
      _clientSettings: null,
      _actions: null,
      _ngCoach: false,

      // For resource allocation
      _modifyTaskHandler: null,

      //
      // Is the widget displayed in dialog mode?
      // Default is no dialog.
      //
      _isDialog: false,
      /**
       * The TaskCompletionWidget can be used for several coaches. (Any state object are reseted at the end)
       * Assumption: The task or task template could be retrieved with HTTP code 200 (Otherwise
       * TaskCompletionView does the error handling)
       * 
       * <ol>
       * <li>loadCoach(task) or loadCoach4Template(taskDefinition) => triggers the rest calls for
       * clientSettings and actions</li>
       * <li>onLoaded() => Coach is loaded </li>
       * <li>onCompleted(newTkiid) => If newTkiid == null => next page, newTkiid != null => updateTask
       * (nextTkiid in case of autoflow) </li>
       * </ol>
       * 
       * Alternate Flow and Error case:
       * <ol>
       * <li>task is already completed => onTaskAlreadyCompleted </li>
       * <li>an error occurred => for old-style coaches => onCompleted for next generation coaches =>
       * onCanceled stay on page</li>
       * </ol>
       * 
       * Remark: The different behaviour for old-style and next-gen coaches is due to the error message in the
       * jsps for old-style coaches which mention that the task list is shown afterwards.
       * 
       * The following error situations can happen in course of the processing:
       * <ul>
       * <li>REST call does not return 200 (OK) => general error message
       * (server_not_available,server_not_available_action) </li>
       * <li>REST calls ok, but _isTaskAndUserFit == false => missing authorization =>
       * user_not_authorized_for_task </li>
       * <li>(Might be catched before) Old style coach is assigned to different person =>
       * _closeCoachDueToError called => user_not_authorized_for_task </li>
       * </ul>
       * 
       */
      constructor: function constructor_TaskCompletionWidget(params){
        // Are we used in dialog mode?
        if (typeof params.dialog === "boolean") {
          // If yes, the header will have display:hidden and no additional popups are shown.
          // Neither will there be any action drop down.
          this._isDialog = params.dialog;
        }
        // PII is loaded before they are used in _Widget.buildRendering.
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        //
        // Set labels used in the template.
        //
        this.due_date_label = this._nlsResources["task_header_due"];
        this.back_link_to_work = this._nlsResources["back_link_to_work"];
        this.maximize_label = this._nlsResources["task_completion_maximize_coach"];
        this.restore_label = this._nlsResources["task_completion_restore_coach"];
        this.loading_label = this._nlsResources["KEY_Loading"];
        //
        // Lookup helper for dropDown menu and rest handler.
        //
        this._dropDownHelper = new com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();
        this.restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
      },

      postCreate: function(){

        if (this._isDialog) {
          // Do not show the header
          this.header.style.display = "none";
          // Set the correct style
          dojo.removeClass(this.domNode, "bpm-task-coach-widget");
          dojo.addClass(this.domNode, "bpm-task-coach-popup");
        }

        // Do not show the maximize button, if device is an iPad, because in this case the
        // coach is always shown in full size (due to problems with bounded iFrames).
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.fullSizeControl.style.display = "none";
        }

        // Default is fullSize == false
        this.fullSizeControl.title = this._nlsResources["fullsize-icon-collapsed"];
        this.fullSizeControlA11y.innerHTML = this.maximize_label;

        // retrieve full size controls from banner
        this.fullSizeRestore = dojo.byId("processPortalFullScreenCoachIcon");
        this.fullSizeRestore.title = this._nlsResources["fullsize-icon-expanded"];
        this.fullSizeRestoreA11y = dojo.byId("processPortalFullScreenCoachIconA11y");
        this.fullSizeRestoreA11y.innerHTML = this.restore_label;
        dijit.setWaiRole(this.fullSizeRestoreA11y, "button");
        dijit.setWaiState(this.fullSizeRestoreA11y, "label", this.restore_label);
        this.fullSizeSubject = dojo.byId("processPortalFullScreenCoachSubject");
        this.fullSizeDue = dojo.byId("processPortalFullScreenCoachDue");
        this.fullSizeDueLabel = dojo.byId("processPortalFullScreenCoachDueLabel");
        this.fullSizeDueLabel.innerHTML = this.due_date_label + " ";
        this.fullSizeDueDate = dojo.byId("processPortalFullScreenCoachDueDate");

		// defect 135784
		dojo.removeClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');
		
        //
        // Add listener to the full size icon
        //
        this.connect(this.fullSizeControl, "click", function(){
          this.expandToFullSize();
          this.fullSizeUserSelection = this.fullSize;
        });
        this.connect(this.fullSizeControl, "keypress", function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this.expandToFullSize();
            this.fullSizeUserSelection = this.fullSize;
          }
        });
        //
        // Add listener to the restore icon
        //
        this.connect(this.fullSizeRestore, "click", function(){
          this.collapseToNormalSize();
          this.fullSizeUserSelection = this.fullSize;
        });
        this.connect(this.fullSizeRestore, "keypress", function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this.collapseToNormalSize();
            this.fullSizeUserSelection = this.fullSize;
          }
        });
      },

      _showLoadingSection: function _showLoadingSection_TaskCompletionWidget(){
        if (this.loadingTimeOut != null) {
          clearTimeout(this.loadingTimeOut);
          this.loadingTimeOut = null;
        }
        // Only show after some time the loading indicator because it might irritate the user
        // if the coach loads quickly anyway.
        this.loadingTimeOut = setTimeout(dojo.hitch(this, this._showLoadingSectionImpl), 1000);
      },

      _showLoadingSectionImpl: function _showLoadingSectionImpl_TaskCompletionWidget(){
        this.loadingSection.style.display = "block";
        this.loadingTimeOut = null;
      },

      _clearLoadingSection: function _clearLoadingSection_TaskCompletionWidget(){
        if (this._task) {
          this._createActionDropDown(this._actions);
        }
        if (this.loadingTimeOut != null) {
          clearTimeout(this.loadingTimeOut);
          this.loadingTimeOut = null;
        }
        this.loadingSection.style.display = "none";
      },

      show: function(){
        this.domNode.style.visibility = "";
      },

      hide: function(){
        this.domNode.style.visibility = "hidden";
      },

      expandToFullSize: function(){
        // Publish a maximize request which the controller is listing to.
        dojo.publish(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", [{}]);
        // hide header
        this.header.style.display = "none";

        this._modifyBanner();

        this.fullSize = true;
        this.resize();

		// defect 135784
		dojo.addClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');

      },

      collapseToNormalSize: function(){
        // Publish a restore request which the controller is listing to.
        // Do not show a side bar for task templates.
        if (!this._taskTemplate) {
          dojo.publish(com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT + "-TaskCompletionView", [{}]);
        }
        // show header
        this.header.style.display = "block";

        this._restoreBanner();
        this.fullSize = false;
        this.resize();

		// defect 135784
		dojo.removeClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');
      },

      _modifyBanner: function _modifyBanner_TaskCompletionWidget(){
        dojo.style(dojo.byId("processPortalTabBar"), "display", "none");
        dojo.style(dojo.byId("processPortalUser"), "display", "none");
        dojo.style(dojo.byId("processPortalFullScreenCoach"), "display", "block");
      },

      _restoreBanner: function _restoreBanner_TaskCompletionWidget(){
        dojo.style(dojo.byId("processPortalFullScreenCoach"), "display", "none");
        dojo.style(dojo.byId("processPortalTabBar"), "display", "block");
        dojo.style(dojo.byId("processPortalUser"), "display", "block");
      },

      _initFullSizeMode: function _initFullSizeControl(isTemplate /* boolean */){
        if (this._isDialog || com.ibm.bpm.social.util.common.Util.isIPad) {
          // Assumption the dialog contains its own instance of this class which is not 
          // used in fullscreen mode.
          // => no manipulations of banner are necessary
          return;
        }
        // Reset to "" in case of task
        // this.fullSizeControl.style.display = isTemplate ? "none" : "";
        this.fullSizeControl.style.display = "";
        this.fullSizeRestore.style.display = this.fullSizeControl.style.display;

        if (isTemplate && !this.fullSizeUserSelection) {
          // In order to make sure that the publish for the MAXIMIZE_CONTENT topic occurs.
          dojo.publish(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", [{}]);
          this.header.style.position = "static";
        }
        else {
        	// We have different behaviour depending on where this is launched (My Tasks or Launch pane). Defect 131867 removed the
        	// follwoing style on fixed it's issue when opening from the right hand pane, however broke it in the case of opening
        	// via my task. It seems like the MAXIMIZE_CONTENT in the above short circuits the maximize coming from the maximize button
        	// For now if we launch via the my task add the styling below, remove it otherwise
            this.header.style.position = "absolute";
            this.header.style.top = 0;
            this.header.style.left = 0;
        }
        // Check if last opened coach was a startable human service which is always full size but user wants normal size
        if (!this.fullSizeUserSelection) {
          // Always collapse in order to make sure that the publish for the RESTORE_CONTENT topic occurs.            
          this.collapseToNormalSize();
        }
        else {
          // Always expand in order to make sure that the publish for the MAXIMIZE_CONTENT topic occurs.
          this.expandToFullSize();
        }
      },

      _clear: function(updateMode /* boolean */){
        if (this.coachRenderer && !updateMode) {
          this.unloadCoach();
        }
        // Clear cached objects    
        this._id = "";
        this._task = null;
        this._taskTemplate = null;
        this._clientType = null;
        this._clientSettings = null;
        this._actions = null;
        this._ngCoach = null;

        if (this._menu) {
          // Unwrap and remove the element out of the DOM. Otherwise it does not disappear.     
          com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this.menuAnker);
          this.menuAnker.style.display = "none";
          this._menu.destroyRecursive();
          this._menu = null;
        }
        // clear header
        this.headerUpdated = false;
        this.subjectArea.innerHTML = "";
        this.dueDateArea.innerHTML = "";
        this.fullSizeSubject.innerHTML = "";
        this.fullSizeDueDate.innerHTML = "";
        if (this._modifyTaskHandler) {
          this._modifyTaskHandler.destroy();
          this._modifyTaskHandler = null;
        }
      },

      unloadCoach: function unloadCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        // Coach need to be removed before loading the next one
        if (this.coachRenderer) {
          this.coachRenderer.unload();
          this.coachRenderer = null;
        }
        com.ibm.bpm.social.util.common.Log.logExit(2, this._id);
      },

      /**
       * Check authorization for working on the task by retrieving the actions and the clientSettings for the
       * task.
       * 
       * In case of error, the error is shown and onCancel is called.
       * 
       * @param {object} The task object.
       */
      loadCoach: function loadCoach_TaskCompletionWidget(/* object */task){
        com.ibm.bpm.social.util.common.Log.logEntry(2, task.tkiid);
        var myself = this;
        this._clear(false /* no update */);
        this._task = task;
        var message;
        // 
        // Check prerequisites.
        //
        if (!task || typeof task.tkiid != "string" || typeof task.state != "string") {
          com.ibm.bpm.social.util.common.Log.log(2, "The task not available");
          message = this._nlsResources["task_not_available"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onTaskDoesNotExist();
          });
          return;
        }
        this._initFullSizeMode(false);
        this._id = this._task.tkiid; // for tracing purposes
        var taskId = task.tkiid;
        this._updateHeader(this._task);

        //
        // Is task already completed
        //
        if (this._task.state == "STATE_FINISHED") {
          // In case the TaskCompletionView stays on this page.
          // fullSizeUserSelection will not be changed
          this.collapseToNormalSize();
          var piid = null;
          if (this._task.piid) {
            piid = this._task.piid;
          }
          com.ibm.bpm.social.util.common.Log.log(2, "The task " + taskId + " with piid " + piid + " is already completed");
          message = this._nlsResources["task_already_completed"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, null /* details */, function(){
            // TaskCompletionView can decided based on (piid != null) which view to show.
            myself.onTaskAlreadyCompleted(piid);
          });
          return;
        }
        // The client type is always the same. (both for heritage coaches and for next generation coaches)            
        this._clientType = "IBM_WLE_Coach";
        // Make the asynchronous calls to
        // 1) Retrieve the clientSettings
        // 2) Retrieve authorization information        
        this.restHandler.getClientSetting(null, taskId, this._clientType, function(response){
          myself._setClientSettings(response);
        }, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, true, task.systemID);
        this._retrieveActions(taskId, false, true /* async */);
        //
        // The header can already be shown
        //
        this._showLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Retrieve client settings and display the coach.
       * 
       * @param {object} The task object.
       */
      loadCoach4Template: function loadCoach4Template_TaskCompletionWidget(/* object */taskTemplate){
        com.ibm.bpm.social.util.common.Log.logEntry(2, taskTemplate.tktid);
        var myself = this;
        this._clear(false /* no update */);
        this._taskTemplate = taskTemplate;
        // 
        // Check prerequisites.
        //
        if (!taskTemplate || typeof taskTemplate.tktid != "string" || typeof taskTemplate.state != "string") {
          com.ibm.bpm.social.util.common.Log.log(2, "The task template not available");
          var message = this._nlsResources["task_template_not_available"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onTaskDoesNotExist();
          });
          return;
        }
        this._initFullSizeMode(true /* isTemplate */);
        this._id = this._taskTemplate.tktid; // For tracing purposes        
        this._updateHeader4Template(this._taskTemplate);
        var taskTemplateId = taskTemplate.tktid;
        // The client type is always the same. (both for heritage coaches and for next generation coaches)                      
        this._clientType = "IBM_WLE_Coach";
        // Make the asynchronous calls to
        // 1) Retrieve the clientSettings
        // 2) Retrieve authorization information        
        this.restHandler.getClientSetting(taskTemplateId, null, this._clientType, function(response){
          myself._setClientSettings(response);
        }, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, true, taskTemplate.systemID);
        //
        // The header can already be shown
        //
        this._showLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _handleTaskDetailsChange: function handleTaskDetailsChange_TaskCompletionWidget(task){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._task.dueTime = task.dueDateTime;
        this._updateHeader(this._task);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },
      /**
       * Updates the meta data of the coach container. Used in case of autoflow. The coach itself is kept.
       * 
       * @param task
       * @returns {updateTask_TaskCompletionWidget} {void} nothing
       */
      updateTask: function updateTask_TaskCompletionWidget(task /* object */){
        com.ibm.bpm.social.util.common.Log.logEntry(2, task.tkiid);
        this._clear(true /* update */);
        this._task = task;
        this._id = this._task.tkiid; // for tracing purposes        
        this._updateHeader(this._task);
        var taskId = task.tkiid;
        // 1) Retrieve authorization information
        // retrive actions synchronously because they are required in the next step
        this._retrieveActions(taskId, false, false /* async */);
        this._clearLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _updateHeader: function(task){
        // Set the due date the subject of the activity.
        this.headerUpdated = true;
        this.subjectArea.innerHTML = task.displayName;
        this.fullSizeSubject.innerHTML = task.displayName;

        //apply btd to the subject area field
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subjectArea);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.fullSizeSubject);

        this.dueDateSection.style.visibility = "";
        this.dueDateArea.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(task.dueTime);
        this.fullSizeDueDate.innerHTML = this.dueDateArea.innerHTML;
      },

      _updateHeader4Template: function(taskTemplate){
        // Set the due date the subject of the activity.
        this.headerUpdated = true;
        this.subjectArea.innerHTML = taskTemplate.displayName;
        this.fullSizeSubject.innerHTML = taskTemplate.displayName;
        
        //apply btd to the subject area field
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subjectArea);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.fullSizeSubject);
        
        this.dueDateSection.style.visibility = "hidden";
        this.fullSizeDue.style.visibility = "hidden";
      },

      _retrieveActions: function _retrieveActions_TaskCompletionWidget(taskId, recall /* boolean */, async){
        var myself = this;
        this.actions = null;
        this.restHandler.getActionsForTask(taskId, com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget.TASK_ACTIONS,
          function(response){
            var data = response;
            if (data.tasks.length === 1) {
              var actions = data.tasks[0].actions || [];
              myself._setActions(actions, false);
            }
            else {
              myself._showErrorAndCancel(myself._nlsResources["user_not_authorized_for_task"]);
            }
          }, function(error, ioArgs){
            myself._showErrorAndCancel();
          }, async, this._task.systemID, this._task.domain);
      },

      _setClientSettings: function _setClientSettings_TaskCompletionWidget(clientSettings){
        com.ibm.bpm.social.util.common.Log.log(2, "Client Settings received.");
        // APAR JR45854 : show an error message if there is no coach available for a task. For example the 
        // task is implemented by an external activity.               
        if (!clientSettings || !clientSettings.url || clientSettings.url==""){
          var message= this._nlsResources["task_completion_no_coach_available"];
          this._clientSettings=null;
          this._showErrorAndCancel(message);          
        } 
        else {        
          this._clientSettings = clientSettings;
           // If we are dealing with a task template or the actions are also available, the 
           // necessary information are retrieved and the coach can be shown.
          if (this._taskTemplate || this._actions) {
             this._showCoach();
          }
        }
      },

      _setActions: function _setActions_TaskCompletionWidget(/* object */actions,/* boolean */recall){
        com.ibm.bpm.social.util.common.Log.log(2, "Task Actions received.");
        if (actions && dojo.indexOf(actions, "ACTION_CLAIM") != -1) {
          // If this is already a recall, continue and rely on the isUserAndTaskFit check later
          if (!recall) {
            // Not authorized yet
            this._actions = null;
            var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
            if (userProfile && userProfile.userPreferences &&
              userProfile.userPreferences['Alert On Assign And Run'] === "true") {
              com.ibm.bpm.social.util.common.Log.log(2, "Show confirmation dialog for claim");
              if (!this._confirmClaimDialog) {
                this._createClaimDialog();
              }
              this._confirmClaimDialog.showConfirmationDialog(this._task);
            }
            else {
              // synchronous call
              this._autoClaimTask();
            }
          }
        }
        else {
          this._actions = actions;
        }
        // Do we have both results ??
        if (this._clientSettings && this._actions) {
          this._showCoach();
        }
      },

      _autoClaimTask: function _autoClaimTask_TaskCompletionWidget(){
        var myself = this;
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        // Synchronous calls because we need to first claim before checking the permission again.
        this.restHandler.claimTask(this._task.tkiid, null, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, this._task.systemID);
        // Retrieve the actions again.
        this._retrieveActions(this._task.tkiid, true /* recall */, true /* async */);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createActionDropDown: function _createActionDropDown_TaskCompletionWidget(availableActions /* object */){
        if (this._isDialog || this._menu) {
          // No header => no action drop down
          return;
        }
        var modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler({
          task: this._task,
          availableActions: availableActions
        });
        this._modifyTaskHandler = modifyTaskHandler; // In order to destroy in case it is a dijit._Widget
        this.connect(modifyTaskHandler, "onTaskModified", function(dueDateTime, priority, assignee){
          dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this._id, [{
            "dueDateTime": dueDateTime,
            "priority": priority,
            "assignee": assignee
          }]);
        });
        //
        // Create the menu and add the task actions
        //
        this._menu = this._dropDownHelper.createDropDownForTaskCompletion(this.menuAnker, this._task, availableActions, {
          modifyTask: function(){
            modifyTaskHandler.showDialog();
          },
          reassignBackToGroup: function(){
            modifyTaskHandler.reassignBackToGroup();
          },
          collaborate: dojo.hitch(this, "_showInvitationDialog")
        });
        //
        // Add the process actions if necessary
        //
        if (this._menu && typeof this._task.piid == "string") {
          this._dropDownHelper.addProcessActions(this._menu, {
            "piid": this._task.piid
          }, {
            onCompletedCallback: function(){
            // No action necessary
            },
            onCanceledCallback: function(){
            // No action necessary
            }
          }, true, false, false, true /* omitCriticalPath */);
        }
        if (this._menu) {
          com.ibm.bspace.common.util.ui.Helper.wrapElement(this.menuAnker);
          dojo.style(this.menuAnker, "display", "inline-block");
        }
      },

      _showInvitationDialog: function _showInvitationDialog_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(this._task);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createClaimDialog: function _createClaimDialog_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        this._confirmClaimDialog = new com.ibm.bpm.social.widgets.task.util.ClaimDialog({
          "restHandler": this.restHandler
        });
        this.connect(this._confirmClaimDialog, "onClaimed", function(){
          com.ibm.bpm.social.util.common.Log.logEntry(2);
          myself._retrieveActions(myself._task.tkiid, true /* recall */, true /* async */);
        });
        this.connect(this._confirmClaimDialog, "onFailed", function(){
          myself.onCanceled();
        });
        this.connect(this._confirmClaimDialog, "onCancelled", function(){
          myself.onCanceled();
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _isTaskAndUserFit: function _isTaskAndUserFit_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        //
        // How to deal with collaboration invitations??    
        // Assumption the authorization is transparently handled by the actions call (taking into account collaboration invitations).
        // ACTION_SETOUTPUTMESSAGE is used for invited user.
        // ACTION_COMPLETE is used for owner. 
        // We need to distinguish in this way, because only the owner is allowed to actually complete the task.
        // 
        if (this._actions &&
          (dojo.indexOf(this._actions, "ACTION_COMPLETE") !== -1 || dojo.indexOf(this._actions, "ACTION_SETOUTPUTMESSAGE") !== -1)) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "true");
          return true;
        }
        else {
          com.ibm.bpm.social.util.common.Log.logExit(2, "false");
          return false;
        }
      },

      _showCoach: function _showCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        var myself = this;
        // Check preconditions only for tasks
        if (this._task && !this._isTaskAndUserFit(this._task)) {
          var message = this._nlsResources["user_not_authorized_for_task"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onCanceled();
          });
          com.ibm.bpm.social.util.common.Log.logExit(2);
          return false;
        }
        // Delegate the rendering to the CoachRenderer which is also used in Business Space.        
        var mode = com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT;
        mode =
          this._task && this._task.piid ? com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT
            : com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT;

        var parameters = {
          serviceURL: window.endpointURLObject.bpmRest +"/v1/",
          mode: mode,
          clientType: this._clientType,
          clientSettings: this._clientSettings,
          resizeHeightAutomatically: false,
          dialog: this._isDialog
        };
        if (this._task) {
          parameters["task"] = this._task;
        }
        if (this._taskTemplate) {
          parameters["taskTemplate"] = this._taskTemplate;
        }
        this.coachRenderer = new com.ibm.bpm.social.widgets.task.form.CoachRenderer(parameters);
        // Establish callbacks...              
        // Connect using the dijit._Widget whose lifecycle is bound to the methods. Thereby the connects get automatically disconnected.
        this.coachRenderer.connect(this.coachRenderer, "onSubmitted", function(tkiid, nextTkiid){
          com.ibm.bpm.social.util.common.Log.log("onSubmitted(" + tkiid + "," + nextTkiid + ")");
          if (nextTkiid) {
            myself.onCompleted(nextTkiid);
          }
          else {
            myself._closeCoach();
          }
        });
        this.coachRenderer.connect(this.coachRenderer, "onCanceled", function(){
          myself._closeCoach();
        });
        this.coachRenderer.connect(this.coachRenderer, "onLoaded", function onLoadedCallback_TaskCompletionWidget(
          tkiid /* string */, newGeneration /* boolean */){
          // In case the coach has been hidden before
          myself.coachRenderer.show();
          if (newGeneration) {
            com.ibm.bpm.social.util.common.Log.log(2, myself._id + " uses next generation coach.");
            myself._ngCoach = true;
            myself.onLoaded();
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(2, myself._id + " uses heritage coach.");
            myself._ngCoach = false;
            myself.onLoaded();
          }
        });
        this.coachRenderer.connect(this.coachRenderer, "onErrorOccurred", function(message, description, details, action){
          myself._closeCoachDueToError(message);
        });
        this.coachRenderer.connect(this.coachRenderer, "onResourceLoaded", function(url){
          myself._clearLoadingSection();
        });
        this._applyHeight();
        this.coachRenderer.load(this.coachContainerAnker);
        this.coachRenderer.enableActions();
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return true;
      },

      _closeCoachDueToError: function _closeCoachDueToError_TaskCompletionWidget(message){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.unloadCoach();
        var errorMessage = this._nlsResources[message];
        if (!errorMessage) {
          errorMessage = message;
        }
        this._showErrorAndCancel(errorMessage);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _closeCoach: function _closeCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.coachRenderer) {
          this.unloadCoach();
          // blue sky scenario: The task has been properly completed.
          this.onCompleted();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      exitTaskCollaboration: function exitTaskCollaboration_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.coachRenderer) {
          this.coachRenderer.exitTaskCollaboration();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _applyHeight: function _applyHeight_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //
        // The coachRenderer should take all the space available below the header.
        // Shall we set some minimal sizes?
        //
        var availableSpace = dojo._getContentBox(this.domNode).h - 35 - 35; //defect 121150
        
        // For some reason in expanded mode the size drops to the bottom of the windows (maybe not taking into account the page footer).
        // 124647
        //if (this.fullSize) {  // remove this condition to factor in the page footer in all the calculations (133663)
       		availableSpace = availableSpace - 25;
        //}
        
        if (!this.fullSize) {
          var headerHeight = dojo._getBorderBox(this.header).h;
          availableSpace = availableSpace - headerHeight;
        }
        if (this.coachRenderer) {
          this.coachRenderer.resize({
            height: availableSpace
          });
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      resize: function resize_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._applyHeight();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _showErrorAndCancel: function _showErrorAndCancel_TaskCompletionWidget(message, action){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        var myself = this;
        if (message === null) {
          // default message => server not available
          message = this._nlsResources["server_not_available"];
        }
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, null /* details */, function(){
          myself.onCanceled();
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * The coach for task with id tkiid has been successfully loaded.
       * 
       * @param {string} The tkiid of the task.
       */
      onLoaded: function onLoaded_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.log(2, "called for " + this._id);
        this._clearLoadingSection();
      },

      /**
       * The interaction with the coaches has been finished.
       */
      onCompleted: function onCompleted_TaskCompletionWidget(/* string */newTkiid){
        if (this.fullSize) {
          this._restoreBanner();
        }
        if (newTkiid) {
          com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted(" + newTkiid + ") for " + this._id);
        }
        else {
          if (this._task) {
            com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted() for " + this._id);
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted() for template " + this._id);
          }
        }
      },

      /**
       * Loading of the coach was stopped
       */
      onCanceled: function onCanceled_TaskCompletionWidget(){
        if (this.fullSize) {
          this._restoreBanner();
        }
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCanceled()");
      },

      /**
       * onTaskAlreadyCompleted used for triggering the instance view. {void} nothing
       */
      onTaskAlreadyCompleted: function onShowInstance_TaskCompletionWidget(piid){
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onTaskAlreadyCompleted(" + piid + ")");
      },

      /**
       * Task does not exist. {void} nothing
       */
      onTaskDoesNotExist: function onTaskDoesNotExist_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onTaskDoesNotExist()");
      },

      destroy: function destroy_TaskCompletionWidget(){
        // Set references to null
        this._clear(false /* no update */);
        // Process _Widget.destroy which unconnects all connects.
        this.inherited(arguments);
      }

    });

com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget.TASK_ACTIONS = ["ACTION_CLAIM",
                                                                          "ACTION_COMPLETE",
                                                                          "ACTION_CANCELCLAIM",
                                                                          "ACTION_INVITE",
                                                                          "ACTION_REASSIGNTOUSER",
                                                                          "ACTION_REASSIGNTOGROUP",
                                                                          "ACTION_SETOUTPUTMESSAGE",
                                                                          "ACTION_UPDATEPRIORITY",
                                                                          "ACTION_UPDATEDUEDATE"];

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog");






dojo.declare("com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString: "<div dojoAttachPoint='content'><div dojoAttachPoint='completionContent'></div></div>",

  _task: null,
  _onCompletedCalled: false,

  /**
   * Creates a popup for working on coaches of a task.
   * 
   * @param taskId The task id of the task to be shown. *
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    // There is no reuse of the dialog, therefore store the task object internally.
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._task = params.task;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  postCreate: function postCreate_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited("postCreate", arguments);
    //
    // Create the TaskCompletionWidget and associate it with the div "completionContent" in the template.
    // Establish all necessary callbacks.
    //        
    this.coachContainer = new com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget({
      dialog: true
    }, this.completionContent);
    this.connect(this.coachContainer, "onLoaded", "_coachLoaded");
    this.connect(this.coachContainer, "onCanceled", "_coachCanceled");
    this.connect(this.coachContainer, "onCompleted", "_taskCompleted");
    this._sizeContent();
    this.coachContainer.resize();
    this.coachContainer.loadCoach(this._task);
    //
    // In case the dialog is just closed, call the _finish method.
    // Which in turn calls onCanceled and makes sure everything is destroyed.
    //
    this.connect(this.popup, 'onHide', "_finish");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _finish: function _finish_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    // Either call onCompleted or onCanceled
    if (!this._onCompletedCalled) {
      this.onCanceled();
    }
    // Basically this is also a dijit._Widget with a meaningful destroy method.
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  destroy: function destroy_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited("destroy", arguments);
    if (this.coachContainer != null) {
      this.coachContainer.destroy();
      this.coachContainer = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Event handling hook for the case that the dialog has been simply closed.
   */
  onCanceled: function onCanceled_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * 
   * Event handling hook for the case that the coaches interaction has been finished.
   */
  onCompleted: function onCompleted_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._onCompletedCalled = true;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _coachLoaded: function _coachLoaded_TaskCompletionDialog(tkiid /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, tkiid);
    // Any logic that needs to be triggered upon loading of the coach can be added here.
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _coachCanceled: function _coachCanceled_TaskCompletionDialog(message /* string */, action /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Close the dialog and return
    //
    this.hide();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _taskCompleted: function(nextTkiid /* auto flow task */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, nextTkiid);
    // Switch back to task list.
    // Is there a smarter solution which switches back to where we came from?
    if (nextTkiid !== null && nextTkiid !== undefined) {
      //
      // Update header
      //
      console.log("Update header");
    }
    else {
      // Make sure that hide does not trigger onCanceled.
      this._onCompletedCalled = true;
      this.hide();
      this.onCompleted();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, nextTkiid);
  },

  show: function(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    // The size of the content has already been set.
    this.connect(dojo.isIE && dojo.isIE < 9 ? dojo.body() : dojo.global, 'onresize', "_sizeContent");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _sizeContent: function(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var viewport = dojo.window.getBox();
    // Minimal width of 300px and minimal height of 200px.
    // Take 66% of the viewport.
    var width = Math.max(viewport.w * 2 / 3, 300);
    var height = Math.max(viewport.h * 2 / 3, 200);
    this.coachContainer.domNode.style.width = width + "px";
    this.coachContainer.domNode.style.height = height + "px";
    if (this.coachContainer != null) {
      // The size of the iframe need to be adjusted.
      this.coachContainer.resize();
    }
    /*
    dojo.query(".dijitDialogPaneContent", this.popup.domNode).style("height", "400px").style("width", "500px").style(
      "padding", "0"); //35px header height + 15px padding
    */
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }

});

/**
 * Retrieves the task object and a dialog with the coach of the task.
 * 
 * When the coach(es) is(are) properly completed, the onCompletedCallback is called. When the dialog is closed
 * the onCanceledCallback is called.
 * 
 * @param taskId
 * @param onCompletedCallback Called upon completion of the coaches interaction.
 * @param onCanceledCallback Called if dialog is closed.
 */
com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog.showCoach =
  function(taskId /* string */, onCompletedCallback /* function */, onCanceledCallback /* function */){
    var restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    restHandler.getTask(taskId, function(task, ioArgs){
      var dialog = new com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog({
        "task": task,
        "title": task.displayName
      });
      if (typeof onCompletedCallback === "function") {
        dialog.connect(dialog, "onCompleted", onCompletedCallback);
      }
      if (typeof onCanceledCallback === "function") {
        dialog.connect(dialog, "onCanceled", onCanceledCallback);
      }
      dialog.show();
    }, function(errorDetails, ioArgs){

    }, false);
  };

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.ActionDropDownMenu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.ActionDropDownMenu"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.ActionDropDownMenu");



dojo.declare(
	"com.ibm.bpm.social.widgets.ActionDropDownMenu",
	[dijit.Menu],
	{

    /**
     * Override _scheduleOpen to allow positioning of popup below the
     * anchor in same place each time.
     */
        _scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
            // summary:
            //		Set timer to display myself.  Using a timer rather than displaying immediately solves
            //		two problems:
            //
            //		1. IE: without the delay, focus work in "open" causes the system
            //		context menu to appear in spite of stopEvent.
            //
            //		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
            //		even after a dojo.stopEvent(e).  (Shift-F10 on windows doesn't generate the
            //		oncontextmenu event.)

            var c = dojo.position(target);
            if (document.body.dir !== "rtl") {
                coords = {
                    x: c.x+1,
                    y: c.y+17,
                    h: c.h,
                    w: c.w
                };
            } else {
                coords = {
                    x: c.x+15,
                    y: c.y+17,
                    h: c.h,
                    w: c.w
                };
            }
            if(!this._openTimer){
                this._openTimer = setTimeout(dojo.hitch(this, function(){
                    delete this._openTimer;
                    this._openMyself({
                        target: target,
                        iframe: iframe,
                        coords: coords
                    });
                }), 1);
            }
        }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog"] = true;
dojo.provide("com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog");










dojo.declare("com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog", com.ibm.bspace.common.util.ui.FullScreenDialog, {

	templateString:"<div class='bpm-social-popup-participants'>\n\t<div dojoAttachPoint='wrapper'></div>\n</div>\n",
	
	instanceId: null,
	
	callback: null,
	
	diagram: null,
	
	criticalPathFlowIds: null,
	
	constructor: function(){
		/**** BEGIN Loading Process Diagram-specific modules ****/
		try{
			dojo.registerModulePath('ibm_ilog.diagram', ppConfig.portalContextRootJS+'/layers/'+socialversion+'/ibm_ilog/diagram');
			dojo["require"]("ibm_ilog.diagram.ibm_ilog_diagram_base"); //Alternate syntax used to avoid dojo build detection
		} catch (e) { 
			console.warn(e);
			dojo.registerModulePath('ibm_ilog.diagram', ppConfig.webviewerContextRootJS+'/widgets/ibm_ilog/diagram');
		}
		
		try{
			dojo.registerModulePath('com.ibm.bpm.wpd', ppConfig.portalContextRootJS+'/layers/'+socialversion+'/com/ibm/bpm/wpd'); //Needed, in addition to /com/ibm/bpm, for img & css files
			dojo["require"]("com.ibm.bpm.wpd_base"); //Alternate syntax used to avoid dojo build detection
		} catch (e) { 
			console.warn(e);
			dojo.registerModulePath('com.ibm.bpm.wpd', ppConfig.webviewerContextRootJS+'/widgets/com/ibm/bpm/wpd');
		}
		
		try{
			dojo["require"]("com.ibm.bpm.social_diagram"); //Alternate syntax used to avoid dojo build detection
		} catch (e) { 
			console.warn(e);
			dojo.registerModulePath('com.ibm.bpm.social', ppConfig.portalContextRootJS+'/com/ibm/bpm/social');
		}
		/**** END Loading Process Diagram-specific modules ****/
		
		//this.inherited(arguments); /* Removed because constructor is already called automatically via dojo */
	},
	
	postMixInProperties: function(){
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.dialogParams.closeButtonTooltip = this.messages.closeProcessDiagramTooltip;
		
		this.inherited(arguments);
	},	
		
	show: function(){
		
		this.inherited(arguments);
		//apply BTD to the title bar
		var titleBar = this.popup.titleBar;
		var titleDiv = this.popup.titleNode;
		var subTitleDiv;

		if(titleBar) {
			subTitleDiv = dojo.query(".dijitDialogSubtitle", titleBar)[0];
		}
		
		if(titleDiv) {
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(titleDiv);
		}

		if(subTitleDiv) {
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(subTitleDiv);
		}

		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);

		this._sizeContent();

		dojo.style(this.wrapper, "width", "100%");
		dojo.style(this.wrapper, "margin", "0");
		dojo.style(this.wrapper, "padding", "0");
		dojo.style(this.wrapper, "position", "absolute");
		dojo.style(this.wrapper, "top", "0");
		dojo.style(this.wrapper, "bottom", "0");
		dojo.style(this.wrapper, "left", "0");
		dojo.style(this.wrapper, "right", "0");
		
		dojo["require"]("com.ibm.bpm.social.widgets.diagram.Participants");
		this.diagram = new com.ibm.bpm.social.widgets.diagram.Participants({
			instanceid: this.instanceId,
			criticalPathFlowIds: (this.criticalPathFlowIds!=null ? this.criticalPathFlowIds: [] )
		}, dojo.create("div", {}, this.wrapper));
		
		dojo.connect(this.popup, 'onHide', this, this.delayedDestroy);
		
		dojo.connect(window, 'onresize', this, this._sizeContent);
		
		dojo.subscribe(com.ibm.bpm.social.portal.Topics.CLOSE_PROCESS_DIAGRAM, dojo.hitch(this, this.delayedDestroy));
		this.refreshSubHandler = dojo.subscribe(com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM, dojo.hitch(this, this.refresh));
		
		this.domNode.setAttribute("title", "");
	},
	
	refresh: function(){
		if(this.diagram){
			if(this.refreshSubHandler) {
				dojo.unsubscribe(this.refreshSubHandler);
			}
			this.diagram.destroy();
			this.diagram = null;
			setTimeout(dojo.hitch(this, this.show), 1500);
		}
	},
	
	_sizeContent: function(){
		var h = 0, w = 0;
		if(dojo.isIE){
			h = document.documentElement.offsetHeight;
			w = document.documentElement.offsetWidth;
		} else {
			h = window.innerHeight;
			w = window.innerWidth;
		}

		dojo.style(this.popup.domNode, "height", h+"px");
		dojo.style(this.popup.domNode, "width", w+"px");
		dojo.style(this.popup.domNode, "top", "0");
		dojo.style(this.popup.domNode, "left", "0");
		
		dojo.query(".dijitDialogPaneContent", this.popup.domNode).style("height", (h-50)+"px").style("width", w+"px").style("padding", "0"); //50px header height
		
		if(dojo.isIE < 9){
			dojo.query(".fullScreenDialog").style("left", "0").style("top", "0").style("overflow", "hidden");
			dojo.style(dojo.body(), "overflow", "hidden");
		}
		
		//dojo.style(this.domNode, "height", (h-50)+"px"); //35px header height + 15px padding
		//dojo.style(this.domNode, "width", (w)+"px"); //20px scrollbar?
		
		if(this.diagram && this.diagram.resize){
			this.diagram.resize();
		}
	},
	
	destroy: function() {
    	
		if(this.diagram && this.diagram.destroy){
			this.diagram.destroy();
			this.diagram = null;
		}
		
		this.inherited(arguments);
		
		if(dojo.isIE < 9){
			dojo.style(dojo.body(), "overflow", "auto");
		}
		
		if(this.popup && this.popup.destroy){
			this.popup.destroy();
		}

        dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
	},
	
	/*
	 * This method is used to destroy this.popup (our dijit.Dialog) after it's animation has finished.
	 * This is only invoked when the 'X' or Cancel button is pressed.
	 */
	delayedDestroy: function(){
		this.diagram._closeOverlays();
		setTimeout(dojo.hitch(this, this.destroy), 1000);
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.ActionDropDownHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.ActionDropDownHelper"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.ActionDropDownHelper");













dojo
  .declare(
    "com.ibm.bpm.social.widgets.ActionDropDownHelper",
    [com.ibm.bpm.social.widgets.common._RestWidget],
    {

      processActionsOrder: ["ACTION_CHANGE_INSTANCE_DUEDATE",
                            "ACTION_VIEW_PROCESS_AUDIT",
                            "ACTION_CHANGE_CRITICAL_PATH",
                            "ACTION_VIEW_PROCESS_DIAGRAM"],

      constructor: function(params){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
      },

      _calculateActions: function(task /* object */, availableActions /* object */, callbacks /* callbacks */,
        withCollaborate /* boolean */){
        var result = {
          taskAction: [],
          processAction: [],
          piid: ""
        };
        //
        // Which task actions qualify?
        //
        var ifReassignBackToGroupOK = dojo.indexOf(availableActions, "ACTION_CANCELCLAIM") != -1;
        var ifTransferToUserOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOUSER") != -1;
        var ifModifyPriorityOK = dojo.indexOf(availableActions, "ACTION_UPDATEPRIORITY") != -1;
        var ifModifyDueDateOK = dojo.indexOf(availableActions, "ACTION_UPDATEDUEDATE") != -1;
        var ifCollaborate = dojo.indexOf(availableActions, "ACTION_INVITE") != -1;
        //
        // MODIFY_TASK
        //
        if (ifTransferToUserOK || ifModifyPriorityOK || ifModifyDueDateOK) {
          if (typeof callbacks.modifyTask === "function") {
            result.taskAction.push("MODIFY_TASK");
          }
        }
        //
        //  REASSIGN_BACK_TO_GROUP
        //
        if (ifReassignBackToGroupOK) {
          if (typeof callbacks.reassignBackToGroup === "function") {
            result.taskAction.push("REASSIGN_BACK_TO_GROUP");
          }
        }
        //
        // COLLABORATE
        //
        if (withCollaborate && ifCollaborate) {
          if (typeof callbacks.collaborate === "function") {
            result.taskAction.push("COLLABORATE");
          }
        }
        //
        // VIEW_INSTANCE
        //
        var piid = null;
        if (typeof task["piid"] !== "undefined") {
          piid = task.piid;
        }
        else if (typeof task["PI_PIID"] !== "undefined") {
          piid = task["PI_PIID"];
        }
        if (piid != null) {
          result.piid = piid;
          result.processAction.push("VIEW_INSTANCE");
        }
        return result;
      },

      _addModifyTaskEntry: function(menu /* dijit.Menu */, callback /* function */){
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["task_menu_modify_task"],
          iconClass: "bpm-social-modify-task",
          onClick: callback
        }));
      },

      _addReassignBackToGroupEntry: function(menu /* dijit.Menu */, callback /* function */){
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["task_menu_cancel"],
          iconClass: "bpm-social-reassign-back-to-group",
          onClick: callback
        }));
      },

      _addCollaborationEntry: function(menu /* dijit.Menu */, callback /* function */){
          if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) { // Prevent collaboration menu option if user is using Safari on iPad but allow for Chrome on iPad
            return
          }

          menu.addChild(new dijit.MenuItem({
            label: this._nlsResources["task_menu_collaborate"],
            iconClass: "bpm-social-collaborate",
            onClick: callback
         }));
      },

      _addViewInstanceEntry: function(menu /* dijit.Menu */, piid /* string */, callback /* function */){
        if (typeof callback !== "function") {
          callback = function(){
            dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
              instanceId: piid
            }]);
          };
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_view_instance"],
          iconClass: "bpm-social-view-instance",
          onClick: callback
        }));
      },

      _addModifyInstanceEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var showF = function(process){
            var cddd = new com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog({
              "instanceId": process.piid,
              "dueDate": process.dueDate,
              "callback": function(data){}
            }, dojo.create("div"));
            cddd.show();
          };
          var myself = this;
          if (processObject.dueDate != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              //var xhr = myself._createProcessXHR(piid);
              //xhr.addCallback(showF);

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        //
        // Different labels for side bar and task completion ui, task row.
        // If processObject is entire process object => side bar. 
        //
        var label =
          processObject.executionState != null ? this._nlsResources["process_menu_change_due_date_instance"]
            : this._nlsResources["process_menu_modify_instance"];
        menu.addChild(new dijit.MenuItem({
          label: label,
          iconClass: "bpm-social-modify-instance",
          onClick: callback
        }));
      },

      _addViewAuditHistoryEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            var ahDialog = new com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog({
              'instanceId': process.piid,
              'templateName': process.processTemplateName,
              'instanceName': process.name,
              'proxyUrl': myself.getProxyUrl()
            });
            ahDialog.show();
          };
          if (processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              // var xhr = myself._createProcessXHR(piid);
              // xhr.addCallback(showF);

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["prcesss_menu_view_audit_history"],
          iconClass: "bpm-social-view-audit-history",
          onClick: callback
        }));
      },

      _addCriticalPathEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            var cpmDialog = new com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog({
              'instanceId': process.piid,
              'templateName': process.processTemplateName,
              'instanceName': process.name,
              'proxyUrl': myself.getProxyUrl()
            });
            cpmDialog.show();
          };
          if (processObject.piid != null && processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred
              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_critical_path_management"],
          iconClass: "bpm-social-critical-path-management",
          onClick: callback
        }));
      },

      _addShowProcessDiagram: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            //
            // Load required libraries
            //
            /** ** BEGIN Loading Process Diagram-specific modules *** */
            /* NO LONGER REQUIRED AS OF 70752 - loading moved inside ProcessDiagramDialog widget
              try {
                dojo.registerModulePath('ibm_ilog.diagram', '/ProcessPortal/layers/' + socialversion + '/ibm_ilog/diagram');
                dojo["require"]("ibm_ilog.diagram.ibm_ilog_diagram_base"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('ibm_ilog.diagram', '/WebViewer/widgets/ibm_ilog/diagram');
              }

              try {
                dojo.registerModulePath('com.ibm.bpm.wpd', '/ProcessPortal/layers/' + socialversion + '/com/ibm/bpm/wpd'); //Needed, in addition to /com/ibm/bpm, for img & css files
                dojo["require"]("com.ibm.bpm.wpd_base"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('com.ibm.bpm.wpd', '/WebViewer/widgets/com/ibm/bpm/wpd');
              }

              try {
                dojo["require"]("com.ibm.bpm.social_diagram"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('com.ibm.bpm.social', '/ProcessPortal/com/ibm/bpm/social');
              }
              */
            /** ** END Loading Process Diagram-specific modules *** */
            //
            // Open process diagram
            //
            
            var pdd = new com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog({
              "instanceId": process.piid,
              "title": process.processTemplateName,
              "subTitle": process.name
            }, dojo.create("div"));
            pdd.show();
          };
          if (processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }

        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_show_process_diagram"],
          iconClass: "bpm-social-view-process-diagram",
          onClick: callback
        }));
      },

      _addLoadingEntry: function(menu /* dijit.Menu */){
        var item = new dijit.MenuItem({
          label: this._nlsResources["KEY_Loading"]
        });
        menu.addChild(item);
        menu._loadingEntry = item;
      },

      _removeLoadingEntry: function(menu /* dijit.Menu */){
        if (typeof menu._loadingEntry === "object") {
          //menu.removeChild(menu._loadingEntry);
          //menu._loadingEntry = null;
          dojo.style(menu._loadingEntry.domNode, "display", "none");

        }
      },

      _addSeparatorEntry: function(menu /* dijit.Menu */){
        if (menu.containerNode.children.length) {
          if (!dojo.hasClass(menu.containerNode, "actionDropDown")) {
            dojo.addClass(menu.containerNode, "actionDropDown");
          }
          menu.addChild(new dijit.MenuSeparator());
        }
      },

      _addAdHocEntry: function _addAdHocEntry_ActionDropDownHelper(menu, process, action, dialog, onCompletedCallback,
        onCanceledCallback, onMessageClosedCallback){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var piid = process.piid;
        var myself = this;
        if (dialog === undefined) {
          // Default
          dialog = false;
        }
        var actionId = action["ID"];
        var displayName = action["displayName"];
        menu.addChild(new dijit.MenuItem({
          label: action.displayName,
          iconClass: "bpm-social-run-ad-hoc-event",
          onClick: function(){

            var params =
              {
                processID: piid,
                actionID: actionId,
                load: function(response){
                  if (response.data && response.data.tkiid && response.data.tkiid.length > 0) {
                    if (dialog) {
                      com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog.showCoach(response.data.tkiid[0],
                        onCompletedCallback, onCanceledCallback);
                    }
                    else {
                      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
                        taskId: response.data.tkiid[0]
                      }]);
                    }
                  }
                  else {
                    var message = myself._nlsResources["process_ad_hoc_event_started"];
                    message = dojo.string.substitute(message, [displayName]);
                    com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, null, onMessageClosedCallback);
                  }
                },
                error: function(){
                  var message = myself._nlsResources["process_ad_hoc_event_failed"];
                  message = dojo.string.substitute(message, [displayName]);
                  com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message);
                }
              };
            myself.processRestHandler.runAdHocAction(params);
          }
        }));
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createDropDownForTask: function _createDropDownForTask_ActionDropDownHelper(anchor /* DomNode */,
        task /* object */, availableActions /* object */, callbacks /* object */, withCollaborate /* boolean */,
        omitViewInstance /* boolean */){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //
        // Calculate the actions
        // 
        var actions = this._calculateActions(task, availableActions, callbacks, withCollaborate);
        if ((actions.taskAction.length + actions.processAction.length === 0) && !com.ibm.bpm.social.util.common.Util.isIPad) {
          // In case there is a process instance associated VIEW_INSTANCE should be available
          //However, on the iPad, we use the context menu to display the link for read-only Activity Details which all users have access to.
          //We account for this case by extending the logic to validate when we are on the iPad and in the Diagram view.
          return null;
        }
        // There are actions for the drop down
        var menu = new com.ibm.bpm.social.widgets.ActionDropDownMenu({
          leftClickToOpen: true
        });

        if (com.ibm.bpm.social.util.common.Util.isIPad && omitViewInstance == true) {
          menu.addChild(new dijit.MenuItem({
            label: this._nlsResources["process_diagram_activity_details"],
            iconClass: "bpm-social-cancel-enrollment",
            onClick: callbacks.viewActivityDetails
          }));
        }

        for ( var i = 0; i < actions.taskAction.length; i++) {
          switch (actions.taskAction[i]) {
            case "MODIFY_TASK" :
              // Create Modify Task entry       
              this._addModifyTaskEntry(menu, callbacks.modifyTask);
              break;
            case "REASSIGN_BACK_TO_GROUP" :
              // Create Reassign back to group entry
              this._addReassignBackToGroupEntry(menu, callbacks.reassignBackToGroup);
              break;
            case "COLLABORATE" :
              // Create Collaboration entry
              this._addCollaborationEntry(menu, callbacks.collaborate);
              break;
            default:
              com.ibm.bpm.social.util.common.Log.log(1, "Unexpected action: " + actions.taskAction[i]);
              break;
          }
        }

        if (omitViewInstance === undefined) {
          //Default is to include 'View Instance' link
          omitViewInstance = false;
        }

        if (actions.piid && omitViewInstance === false) {
          // The "View Instance" entry is always possible if there is an associated instance, unless we are viewing the Process Diagram
          this._addSeparatorEntry(menu);
          this._addViewInstanceEntry(menu, actions.piid, callbacks.viewInstance);
        }
        menu.bindDomNode(anchor);

        // a11y; dijit.Menu behaves like a context menu, which means it is opened using Shift+F10, our context menu behaves like a button and should react on Enter
        this._connectEnter(menu, anchor);

        com.ibm.bpm.social.util.common.Log.logExit(2);

        return menu;
      },

      /**
       * connects the ENTER key to open the Menu, this normally does not happen, as a dijit.Menu only reacts
       * on Shift+F10
       * 
       * @param menu The menu that should be opened on ENTER.
       * @param anchor The DOM node that should be used as anchor
       */
      _connectEnter: function _connectEnter_ActionDropDownHelper(/*dijit.Menu*/menu, /*DOMElement*/anchor){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (menu && menu._bindings && menu._bindings[0] && menu._bindings[0].connects) {
          menu._bindings[0].connects.push(dojo.connect(anchor, "onkeydown", menu, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              this._scheduleOpen(evt.target, menu._bindings[0].iframe);
            }
          }));
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1, "Cannot connect ENTER key to Menu anchor.");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Adds the process related entries.
       * 
       * @param menu The menu to which the entries are added.
       * @param result The data object of the process/<process id>/actions call.
       * @param process The process object (having at least a piid property).
       * @param dialog Whether a dialog for ad hoc event is necessary.
       * @param callbacks A callback object that is used in case special callback handler are needed.
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       */
      _addProcessEntries: function _addProcessEntries_ActionDropDownHelper(menu /* menu */, result /* object */,
        process /* object */, dialog /* boolean */, callbacks /* object */, omitViewInstance /*boolean*/,
        omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
            omitCriticalPath = false;
        }

        var actions = result.actions;
        var standardActions = result.standardActions;
        var i;
        if (standardActions &&
          standardActions.length > 0 &&
          (dojo.indexOf(standardActions, "ACTION_CHANGE_INSTANCE_DUEDATE") > -1 ||
            dojo.indexOf(standardActions, "ACTION_VIEW_PROCESS_AUDIT") > -1 ||
            dojo.indexOf(standardActions, "ACTION_CHANGE_CRITICAL_PATH") > -1 || dojo.indexOf(standardActions,
            "ACTION_VIEW_PROCESS_DIAGRAM") > -1)) {

          // Separator not needed as in case of task actions there is already the View Instance entry for processes
          // If we only have one MenuItem, it's the "Loading..." MenuItem that is hidden.  We don't need an initial separator in this case.
          if (omitViewInstance === true && menu.getChildren().length > 1) {
            this._addSeparatorEntry(menu);
          }
          for (i = 0; i < this.processActionsOrder.length; i++) {
            var standardAction = this.processActionsOrder[i];
            if (dojo.indexOf(standardActions, standardAction) != -1) {
              switch (standardAction) {
                case "ACTION_CHANGE_INSTANCE_DUEDATE" :
                  // Create Modify Instance entry
                  var modifyInstanceCallback = callbacks.modifyInstance || null;
                  this._addModifyInstanceEntry(menu, process, modifyInstanceCallback);
                  break;
                case "ACTION_VIEW_PROCESS_AUDIT" :
                  // Create View Audit History entry
                  this._addViewAuditHistoryEntry(menu, process, callbacks.viewAuditHistory);
                  break;
                case "ACTION_CHANGE_CRITICAL_PATH" :
                  // Create Critical Path Management entry
                  if (omitCriticalPath !== true) {
                  	this._addCriticalPathEntry(menu, process, callbacks.criticalPathManagement);
                  }
                  break;
                case "ACTION_VIEW_PROCESS_DIAGRAM" :
                  // Process Diagrqamm entry
                  if (omitProcessDiagram !== true) {
                    this._addShowProcessDiagram(menu, process, callbacks.viewProcessDiagram);
                  }
                  break;
                default:
                  com.ibm.bpm.social.util.common.Log.log(1, "Unexpected action: " + standardAction);
                  break;
              }
            }
          }
        }
        if (actions && actions.length > 0) {
          var onCompletedCallback = null;
          if (typeof callbacks.onCompletedCallback === "function") {
            onCompletedCallback = callbacks.onCompletedCallback;
          }
          var onCanceledCallback = null;
          if (typeof callbacks.onCanceledCallback === "function") {
            onCanceledCallback = callbacks.onCanceledCallback;
          }
          var onClosedCallback = null;
          if (typeof callbacks.onMessageClosedCallback === "function") {
            onClosedCallback = callbacks.onMessageClosedCallback;
          }
          this._addSeparatorEntry(menu);
          for (i = 0; i < actions.length; i++) {
            var action = actions[i];
            this._addAdHocEntry(menu, process, action, dialog, onCompletedCallback /* function */,
              onCanceledCallback /* function */, onClosedCallback /* function */);
          }
        }
        //This case should only be encountered on the Process Diagram.  All others will have at least "View Instance" option.
        if (menu.getChildren().length == 1) {
          menu._loadingEntry.set('label', this._nlsResources["no_actions_available"]);
          menu._loadingEntry.set('disabled', true);
          dojo.style(menu._loadingEntry.domNode, "display", "block");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Creates a drop down menu for the action menu. Which can be used for the latter calls.
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @return The dijit.Menu instance which need to be destroyed by the calling class.
       */
      createDropDown: function createDropDown_ActionDropDownHelper(anchor /* DomNode */){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // There are actions for the drop down
        var menu = new com.ibm.bpm.social.widgets.ActionDropDownMenu({
          leftClickToOpen: true
        });

        menu.bindDomNode(anchor);

        // a11y; dijit.Menu behaves like a context menu, which means it is opened using Shift+F10, our context menu behaves like a button and should react on Enter
        this._connectEnter(menu, anchor);

        com.ibm.bpm.social.util.common.Log.logExit(2);

        return menu;
      },

      /**
       * Creates a dijit.Menu for the Task Row.
       * 
       * The following callbacks are excepted:
       * <ul>
       * <li>modifyTask</li>
       * <li>reassignBackToGroup</li>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <ul>
       * 
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @param task The associated task.
       * @param availableActions The availableActions of the task.
       * @param callbacks The callbacks for the actions.
       * @return The dijit.Menu instance which need to be destroyed by the calling class.
       * 
       */
      createDropDownForTaskRow: function(anchor /* DomNode */, task /* object */, availableActions /* object */,
        callbacks /* object */){
        return this._createDropDownForTask(anchor, task, availableActions, callbacks, false /* no collaboration */);
      },

      /**
       * Creates a dijit.Menu for the Task Completion UI.
       * 
       * The following callbacks are excepted:
       * <ul>
       * <li>modifyTask</li>
       * <li>reassignBackToGroup</li>
       * <li>collaborate</li>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <ul>
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @param task The associated task.
       * @param availableActions The availableActions of the task.
       * @param callbacks The callbacks for the actions.
       * @returns The dijit.Menu instance which need to be destroyed by the calling class.
       */
      createDropDownForTaskCompletion: function(anchor /* DomNode */, task /* object */, availableActions /* object */,
        callbacks /* object */){
        return this._createDropDownForTask(anchor, task, availableActions, callbacks, true /* including collaboration */);
      },

      /**
       * Make sure that the process actions are added if the menu gets focus.
       * 
       * @param menu
       * @param process A process object which need to have at least the piid property
       * @param dialog Whether a dialog for ad hoc event is necessary.
       * @param callbacks {void} nothing
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       * 
       * @return A deferred that provides the result of the process action call on success.
       */
      addProcessActionsOnFocus: function addProcessActionsOnFocus_ActionDropDownHelper(menu /* Menu */,
        process /* object */, callbacks /* object */, omitViewInstance /*boolean*/, omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
          omitCriticalPath = false;
        }
        var myself = this;
        // Provide a Deferred that allow other interessted parties to take part in using the result.
        // Requires that all callback return the result to keep the callback chain.
        var deferred = new dojo.Deferred();
        var piid = process.piid;
        menu._processActionsPending = true;
        menu._loadingEntryAdded = false;
        //
        // Define the logic for processing the result of process/{piid}/action as callback on 
        // a deferred to make it independent of the trigger. 
        // (onFocus or some other circumenstances outside this module)
        //
        deferred.then(function(result){
          // A process action call has been conducted, we do not need another one onFocus.
          menu._processActionsPending = false;
          if (menu._loadingEntryAdded) {
            myself._removeLoadingEntry(menu);
          }
          myself._addProcessEntries(menu, result, process, false, callbacks, omitViewInstance, omitProcessDiagram, omitCriticalPath);
          return result;
        });
        menu.connect(menu, "onFocus", function(){
          // Only call the process/{piid}/actions if it has not been done already.
          if (menu._processActionsPending) {
            menu._loadingEntryAdded = true;
            myself._addLoadingEntry(menu);

            // var xhr = myself._createProcessActionXHR(piid);
            // xhr.then(function(result){
            // // Trigger all required actions              
            // deferred.resolve(result);
            // });

            myself.processRestHandler.getActionsForProcess(piid, function(result){
              deferred.resolve(result);
            }, myself._onXhrErrors, true);

          }
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return deferred;
      },

      /**
       * In contrast to the OnFocus method the process actions are immediately retrieved.
       * 
       * Which is used in the TaskCompletionUI where only one task is under consideration. In the task list
       * are potentially hundreds of items shown for which the process action call cannot be done in advance
       * without degrading the performance. The following callback entries can be used to define callbacks for
       * completion of ad hoc event dialog and for cancelation of an ad hoc event dialog.
       * 
       * <ul>
       * <li>onCompletedCallback</li>
       * <li>onCanceledCallback</li>
       * </ul>
       * 
       * In addition to that the following entries are also used
       * 
       * <ul>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <li>viewProcessDiagram</li>
       * <ul>
       * 
       * 
       * @param menu
       * @param process A process object which need to have at least the piid property
       * @param callbacks
       * @param dialog Whether the coaches for ad hoc event should be shown in a dialog. {void} nothing
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       */
      addProcessActions: function addProcessActions_ActionDropDownHelper(menu /* Menu */, process /* object */,
        callbacks /* object */, dialog /* boolean */, omitViewInstance /*boolean*/, omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var piid = process.piid;
        if (dialog === undefined) {
          // Default is no dialog
          dialog = false;
        }
        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
          omitCriticalPath = false;
        }
        var myself = this;
        myself._addLoadingEntry(menu);
        // Immediately retrieve the process actions

        var async = true;
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          async = false;
        }
        myself.processRestHandler.getActionsForProcess(piid,
          function(result){
            if ((result.standardActions && result.standardActions.length > 0) ||
              (result.actions && result.actions.length > 0)) {
              dojo.publish(com.ibm.bpm.social.portal.Topics.PROCESS_ACTIONS_AVAILABLE, [result]);
            }
            myself._removeLoadingEntry(menu);
            myself._addProcessEntries(menu, result, process, dialog, callbacks, omitViewInstance, omitProcessDiagram, omitCriticalPath);
            return result;
          }, myself._onXhrErrors, async);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _onXhrErrors: function _onXhrErrors_ActionDropDownHelper(data, xhrArgs){
        console.debug(["ActionDropDownHelper experienced an error.",
                       data,
                       xhrArgs]);
      },

      statics: {
        _instance: null,
        _getInstance: function(){
          if (this._instance === null) {
            this._instance = new com.ibm.bpm.social.widgets.ActionDropDownHelper();
          }
          return this._instance;
        }
      }
    });


com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance = function(){
  return com.ibm.bpm.social.widgets.ActionDropDownHelper.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TaskRow"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TaskRow"] = true;
// TaskRow.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TaskRow");
























dojo.declare("com.ibm.bpm.social.widgets.task.list.TaskRow", [dijit._Widget,
                                                              dijit._Templated], {

  templateString: dojo.cache("com.ibm.bpm.social.widgets.task", "list/templates/TaskRow.html", "<div class=\"bpm-social-task-row\">\n\t<div class=\"bpm-social-task-box\" dojoAttachPoint=\"taskBoxAttach\">\n\n\t\t<div class=\"bpm-social-task-row-status-img-div\">\n\t\t\t<div class=\"bpm-social-task-row-status-img\"\n\t\t\t\tdojoAttachPoint=\"imgStatus\" waiRole=\"img\"></div>\n\t\t</div>\n\n\t\t<div class=\"bpm-social-task-row-data-div\" dojoAttachPoint=\"focusNode\"\n\t\t\tdojoAttachEvent=\"ondijitclick:toggle\" tabIndex=\"0\" role=\"listitem\">\n\n\t\t\t<div class=\"bpm-social-task-row-data-header\">\n\t\t\t\t<span class=\"bpm-social-task-row-data-subject\" ><div dojoAttachPoint=\"headerLabel\" style=\"display:none\"></div> <a dojoAttachPoint=\"subject\"\n\t\t\t\t\trole=\"button\" href=\"javascript:void(0)\"></a> </span>\n\t\t\t\t<div dojoAttachPoint=\"menuButton\" class=\"bpm-social-drop-down\" tabIndex=\"0\" style=\"display:none\"  waiRole=\"button\"><div class=\"bpm-social-drop-down-text\">&#9660;</div></div>\n\t\t\t\t<span class=\"bpm-social-task-row-data-due\" dojoAttachPoint=\"due\">\n\t\t\t\t\t<span class=\"bpm-social-task-row-data-due-label\"\n\t\t\t\t\tdojoAttachPoint=\"dueLabel\"></span> <span\n\t\t\t\t\tclass=\"bpm-social-task-row-data-due-date\" dojoAttachPoint=\"dueDate\"></span>\n\t\t\t\t</span>\n\t\t\t\t<span class=\"bpm-social-task-row-data-complete\" dojoAttachPoint=\"complete\">\n\t\t\t\t\t<span class=\"bpm-social-task-row-data-complete-label\"\n\t\t\t\t\tdojoAttachPoint=\"completeLabel\"></span> <span\n\t\t\t\t\tclass=\"bpm-social-task-row-data-complete-date\" dojoAttachPoint=\"completeDate\"></span>\n\t\t\t\t</span>\n\t\t\t</div>\n\t\t\t\n\t\t\t<div style=\"clear: both\"></div>\n\n\t\t\t<div class=\"bpm-social-task-row-data-second-line\">\n\t\t\t    <span class=\"bpm-social-task-row-data-due bpm-social-a11y\"> \n\t\t\t       <div class=\"bpm-social-task-row-data-due-label\"  dojoAttachPoint=\"priorityA11yLabel\"></div>\t\t\t\t\n\t\t\t       <span class=\"bpm-social-task-row-data-due-date\" dojoAttachPoint=\"priorityA11y\" ></span>\n\t\t\t    </span>\n\t\t\t</div>\n\t\t\t\n\t\t\t<div style=\"clear: both\"></div>\n\t\t\t\n\t\t\t<div class=\"bpm-social-task-row-data-second-line\">\n\t\t\t\t<span class=\"bpm-social-task-row-data-instance bpm-social-task-row-data-second-line\"\n\t\t\t\t\tdojoAttachPoint=\"instance\"></span>\n\t\t\t\t<div dojoAttachPoint=\"instanceLabel\" style=\"display:none\"></div>\t  \n\t\t\t\t<span class=\"bpm-social-task-row-data-group-div\" >\n\t\t\t\t\t<div class=\"bpm-social-task-row-data-group-img\"\n\t\t\t\t\t\tdojoAttachPoint=\"imgGroup\"></div> \n\t\t\t\t\t<span class=\"bpm-social-task-row-data-group bpm-social-task-row-data-second-line\"\n\t\t\t\t\tdojoAttachPoint=\"group\"> </span>\n\t\t\t\t\t<div dojoAttachPoint=\"groupLabel\" style=\"display:none\"></div>\t  \t\t\t\t\t\t\t\t\t\t\t\t\t\n\t\t\t\t</span>\t\t\n\t\t\t</div>\n\t\t\t<div class=\"bpm-social-task-row-data-expanded\"\n\t\t\t\tdojoAttachPoint=\"expanded\"></div>\t\t\t\n\t\t</div>\n\t\t\t\t\n\t\t<div class=\"bpm-social-task-row-priority-img-div bpm-social-no-a11y\">\n\t\t\t<div class=\"bpm-social-task-row-priority-img\"\n\t\t\t\tdojoAttachPoint=\"imgPriority\"></div>\n\t\t\t<label dojoAttachPoint=\"imgPriorityLabel\" style=\"display:none\"></label>\t\t\t\t\t\n\t\t</div>\n\t</div>\t\n</div>\n\n"),

  open: false,

  _processActionsReceived: false,
  _processActionsDeferred: null,
  _postInstanceComment: false,

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    //
    // Lookup helper for dropDown menu.
    //  
    this._dropDownHelper = com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * postCreate
   * 
   */
  postCreate: function postCreate_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);

    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * make sure all data is set with defaults or the incoming values, thus all old values in a cached row
   * widget are replaced
   */
  initRow: function initRow_TaskRow(param){
    com.ibm.bpm.social.util.common.Log.logEntry(2, param.model.TKIID);

    this.model = param.model;
    this.controller = param.controller;
    this.displayedProperties = param.displayedProperties;

    // Status
    this.setStatus(this.model.categoryId);

    // Subject
    this.subject.innerHTML = "";
    if (this._isDisplayedProperty("TAD_DISPLAY_NAME")) {
      this.subject.innerHTML =
        com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this.model.TAD_DISPLAY_NAME);

      //configure BTD for Subject
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subject);

      this.headerLabel.innerHTML = this._nlsResources["task-row-task-subject-label"];
      this.headerLabel.setAttribute("id", "taskrowsubject" + this.id);
      dijit.setWaiState(this.subject, "labelledby", this.headerLabel.id);
    }
    else {
      // set default values
      this.subject.innerHTML = "";
      this.headerLabel.innerHTML = "";
    }
    dijit.setWaiRole(this.subject, "button");

    if (param.controller.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
      this.subject.title = this._nlsResources["task_action_view_instance_details_description"];
    }
    else {
      this.subject.title = this._nlsResources["task_action_run_task_description"];
    }

    dijit.setWaiState(this.menuButton, "label", this._nlsResources["task_actions_title"]);

    // Priority
    if (this._isDisplayedProperty("PRIORITY")) {
      this._setPriority(this.model.PRIORITY);
    }
    else {
      // set default values
      this._setPriority(null);
    }

    // Due
    if (this._isDisplayedProperty("DUE")) {
      this._setDue(this.model.DUE);
    }
    else {
      // set default values
      this._setDue(null);
    }

    // Instance
    this.instance.innerHTML = "";
    if (this._isDisplayedProperty("PI_DISPLAY_NAME")) {
      var text = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this.model.PI_DISPLAY_NAME);
      //configure BTD for the instance
      text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(text);
      this.instance.innerHTML = text;
      this.instanceLabel.setAttribute("id", "taskrowinstancelabel" + this.id);
      dijit.setWaiState(this.instance, "labelledby", this.instanceLabel.id);
      this.instanceLabel.innerHTML = this._nlsResources["task-row-instance-name-label"];
    }
    else {
      // set default values
      this.instance.innerHTML = "";
      this.instanceLabel.innerHTML = "";
    }

    // assigned to group
    if (this._isDisplayedProperty("ASSIGNED_TO_ROLE_DISPLAY_NAME")) {
      this._setAssignee(this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME);
    }
    else {
      // set default values
      this._setAssignee(null);
    }
    // complete date
    if (this._isDisplayedProperty("COMPLETED")) {
      this._setCompleteDate(this.model.COMPLETED);
    }
    else {
      // set default values
      this._setCompleteDate(null);
    }

    // Clean up expanded area
    this._clearDetailsArea();

    // is the Modify Task dialog open?
    if (this._modifyTaskHandler && this._modifyTaskHandler.dialog && this._modifyTaskHandler.dialog.popup &&
      this._modifyTaskHandler.dialog.popup.open) {
      // use a new ModifyTaskHandler 
      // otherwise the properties of the existing ModifyTaskHandler will be updated and modifications will be done to the new task
      // the existing one will be destroyed when the dialog is hidden
      this._modifyTaskHandler = null;
    }

    // actions drop down
    this._clearMenuButton();
    this._callback = dojo.hitch(this, "_handleAvailableActions");
    this.controller.setActionsCallbackSuccess(this.model, this._callback);

    // task details
    if (param.open) {
      this._expand(true); // retrieve task details
    }
    else {
      this.open = false; // initial default state is collapsed
      this._collapse();
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * update model and widget with new data
   * <p>
   * TODO can the task subject and the process instance name change?
   * 
   * @param model updated model
   * @param actionsNeeded callback for actions needs to be established
   * @returns {boolean} synchronizationNeeded
   */
  updateRow: function updateRow_TaskRow(model, actionsNeeded){
    com.ibm.bpm.social.util.common.Log.logEntry(2, model.TKIID);

    var synchronizationNeeded = false;
    var deleteRow = false;

    // assigned to another user
    if (model.OWNER !== undefined) {
      if (this.model.OWNER !== model.OWNER) {
        this.model.OWNER = model.OWNER;
        this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME = null;
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        if (this.model.OWNER !== user.userName) {
          // remove row
          deleteRow = true;
          this.onDeleteRowNeeded(this.model.TKIID);
        }
        else {
          this._setAssignee(null);
        }
      }
    }

    if (!deleteRow) {
      // Priority
      if (model.PRIORITY !== undefined) {
        if (this._isDisplayedProperty("PRIORITY") && this.model.PRIORITY !== model.PRIORITY) {
          this.model.PRIORITY = model.PRIORITY;
          this._setPriority(this.model.PRIORITY);
        }
      }

      // Due
      if (model.DUE !== undefined) {
        if (this._isDisplayedProperty("DUE") && this.model.DUE !== model.DUE) {
          this.model.DUE = model.DUE;
          this._setDue(this.model.DUE);

          // If the task is not in the atRisk group, set the atRisk flag to false 
          // and wait for any notification to fix this.
          if (this.model.categoryId !== com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
            this.model.IS_AT_RISK = false;
          }

          synchronizationNeeded = true;
        }
      }

      // category
      if (model.categoryId !== undefined) {
        if (this.model.categoryId !== model.categoryId) {
          this.model.categoryId = model.categoryId;
          synchronizationNeeded = true;
        }
      }

      // Status
      if (model.DUE !== undefined && model.IS_AT_RISK !== undefined) {
        if (this.model.DUE !== model.DUE || this.model.IS_AT_RISK !== model.IS_AT_RISK) {
          this.model.IS_AT_RISK = model.IS_AT_RISK;
          this.setStatus(this.model.categoryId);
          synchronizationNeeded = true;
        }
      }

      // assigned to group
      if (model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== undefined) {
        if (this._isDisplayedProperty("ASSIGNED_TO_ROLE_DISPLAY_NAME") &&
          this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== model.ASSIGNED_TO_ROLE_DISPLAY_NAME) {
          this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME = model.ASSIGNED_TO_ROLE_DISPLAY_NAME;
          this.model.OWNER = null;
          this._setAssignee(this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME);
        }
      }

      // complete date
      if (model.COMPLETED !== undefined) {
        if (this._isDisplayedProperty("COMPLETED") && this.model.COMPLETED !== model.COMPLETED) {
          this.model.COMPLETED = model.COMPLETED;
          this._setCompleteDate(this.model.COMPLETED);
          synchronizationNeeded = true;
        }
      }

      // actions drop down 
      if (actionsNeeded) {
        this._clearMenuButton();
        this._callback = dojo.hitch(this, "_handleUpdateAvailableActions");
        this.controller.setActionsCallbackSuccess(this.model, this._callback);
      }
    }

    // KJ: Removed, because the flickering was quite confusing in the
    // Modify Task Dialog case PLUS if the details are not refreshed,
    // the in-line completion task content will not be lost! 
    // Anyway: still have to discuss this...
    /*
    // TODO task details
    if (this.open) {
      this._expand(true); // retrieve task details ?
    }*/

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return synchronizationNeeded;
  },

  /**
   * checks if the new model requires new actions
   * <p>
   * This is the case if owner or state changed
   * 
   * @param model
   * @returns {boolean} actionsNeeded
   */
  actionsNeeded: function actionsNeeded_TaskRow(model){
    com.ibm.bpm.social.util.common.Log.logEntry(2, model.TKIID);

    var actionsNeeded = false;

    if (model.STATE !== undefined) {
      actionsNeeded = (this.model.STATE !== model.STATE);
    }

    // assigned to group
    if (model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== undefined &&
      this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== model.ASSIGNED_TO_ROLE_DISPLAY_NAME) {
      actionsNeeded = true;
    }

    // assigned to another user
    if (model.OWNER !== undefined) {
      if (this.model.OWNER !== model.OWNER) {
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        if (model.OWNER !== user.userName) {
          // task row will be removed
          actionsNeeded = false;
        }
        else {
          actionsNeeded = true;
        }
      }
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, actionsNeeded);
    return actionsNeeded;
  },

  _isDisplayedProperty: function _isDisplayedProperty_TaskRow(key){
    if (this.displayedProperties && dojo.indexOf(this.displayedProperties, key) !== -1 && this.model[key] !== undefined) {
      return true;
    }
    else if (!this.displayedProperties && this.model[key] !== undefined) {
      return true;
    }
    return false;
  },

  destroy: function destroy_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    if (this._modifyTaskHandler) {
      this._modifyTaskHandler.destroy();
      this._modifyTaskHandler = null;
    }
    if (this._menu) {
      this._clearMenuButton();
      com.ibm.bspace.common.util.ui.Helper.unwrapElementList([this.menuButton]);
    }
    this._clearDetailsArea();
    if (this._connectsCallback) {
      for ( var i = 0; i < this._connectsCallback.length; i++) {
        dojo.disconnect(this._connectsCallback[i]);
      }
      this._connectsCallback = null;
    }
    this._dropDownHelper = null;
    this._nlsResources = null;

    this.controller = null;
    this.model = null;
    this._callback = null;
    this.inherited('destroy', arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setDue: function _setDue_TaskRow(dueDate){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + dueDate);
    if (dueDate) {
      this.dueLabel.innerHTML = this._nlsResources["task_row_due"];
      this.dueDate.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(dueDate);
      this.dueDate.setAttribute("id", "taskrowduedate" + this.id);
      this.dueLabel.setAttribute("id", "taskrowduelabel" + this.id);
      this.dueDate.setAttribute("aria-labelledby", "taskrowduelabel" + this.id);
    }
    else {
      this.dueLabel.innerHTML = "";
      this.dueDate.innerHTML = "";
    }
  },

  _setCompleteDate: function _setCompleteDate_TaskRow(completeDate){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + completeDate);
    if (completeDate) {
      this.completeLabel.innerHTML = this._nlsResources["task_row_completed"];
      this.completeDate.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(completeDate);
      this.completeDate.setAttribute("id", "taskrowcompletedate" + this.id);
      this.completeLabel.setAttribute("id", "taskrowcompletelabel" + this.id);
      this.completeDate.setAttribute("aria-labelledby", "taskrowcompletelabel" + this.id);
    }
    else {
      this.completeLabel.innerHTML = "";
      this.completeDate.innerHTML = "";
    }
  },

  _setPriority: function _setPriority_TaskRow(priority){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + priority);
    if (priority) {
      // get the new tooltip and css class
      var priorityText = com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(priority);
      var priorityClass = null;
      var priorityTooltip = null;
      var priorityLabel = null;
      var priorityA11y = null;
      switch (priorityText) {
        case "VeryHigh" :
          priorityClass = "bpm-social-very-high";
          priorityTooltip = "task-row-priority-very-high-tooltip";
          priorityLabel = "KEY_PrioVeryHighTooltip";
          priorityA11y = "KEY_PrioVeryHigh";
          break;
        case "High" :
          priorityClass = "bpm-social-high";
          priorityTooltip = "task-row-priority-high-tooltip";
          priorityLabel = "KEY_PrioHighTooltip";
          priorityA11y = "KEY_PrioHigh";
          break;
        case "Medium" :
          priorityClass = "bpm-social-medium";
          priorityTooltip = "task-row-priority-normal-tooltip";
          priorityLabel = "KEY_PrioMediumTooltip";
          priorityA11y = "KEY_PrioMedium";
          break;
        case "Low" :
          priorityClass = "bpm-social-low";
          priorityTooltip = "task-row-priority-low-tooltip";
          priorityLabel = "KEY_PrioLowTooltip";
          priorityA11y = "KEY_PrioLow";
          break;
        case "VeryLow" :
          priorityClass = "bpm-social-very-low";
          priorityTooltip = "task-row-priority-very-low-tooltip";
          priorityLabel = "KEY_PrioVeryLowTooltip";
          priorityA11y = "KEY_PrioVeryLow";
          break;
        default:
          break;

      }
      //// remove the old values
      // remove the old class and set the new one
      dojo.replaceClass(this.imgPriority, priorityClass, ["bpm-social-very-low",
                                                          "bpm-social-low",
                                                          "bpm-social-medium",
                                                          "bpm-social-high",
                                                          "bpm-social-very-high"]);
      // normal mode 
      this.imgPriority.title = this._nlsResources[priorityTooltip];
      this.imgPriorityLabel.innerHTML = this._nlsResources[priorityLabel];
      this.imgPriorityLabel.setAttribute("id", "taskrowpriority" + this.id);
      dijit.setWaiState(this.imgPriority, "labelledby", this.imgPriorityLabel.id);


      // high contrast mode
      this.priorityA11yLabel.innerHTML = this._nlsResources["task-row-priority-label"] + " : ";
      this.priorityA11yLabel.setAttribute("id", "taskrowprioritya11y_label" + this.id);

      this.priorityA11y.innerHTML = this._nlsResources[priorityA11y];
      dijit.setWaiState(this.priorityA11y, "labelledby", this.priorityA11yLabel.id);

    }
    else {
      //// set default values
      this.imgPriority.title = "";
      this.imgPriorityLabel.innerHTML = "";
      // high contrast mode
      this.priorityA11y.innerHTML = "";
      this.priorityA11yLabel.innerHTML = "";
      // remove the old class
      dojo.removeClass(this.imgPriority, ["bpm-social-very-low",
                                          "bpm-social-low",
                                          "bpm-social-medium",
                                          "bpm-social-high",
                                          "bpm-social-very-high"]);
    }

  },

  _setAssignee: function _setAssignee_TaskRow(assignedToRole){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + assignedToRole);
    this.group.innerHTML = "";
    if (assignedToRole) {
      // From Process Portal: determine group display name
      // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
      // So we are just using the last two '_'s to pull the PG name
      if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
        var roleName = assignedToRole;
        var displayName = roleName.substring(0, roleName.lastIndexOf("_"));
        displayName = displayName.substring(0, displayName.lastIndexOf("_"));
        assignedToRole = displayName;
      }
      this.groupLabel.setAttribute("id", "taskrowgrouplabel" + this.id);
      this.groupLabel.innerHTML = this._nlsResources["task_row_owner-label"];
      this.group.innerHTML = assignedToRole;
      dijit.setWaiState(this.group, "labelledby", this.groupLabel.id);
      dojo.addClass(this.imgGroup, "bpm-social-group");
    }
    else {
      //// set default values
      this.group.innerHTML = "";
      this.groupLabel.innerHTML = "";
      dojo.removeClass(this.imgGroup, "bpm-social-group");
    }
  },

  setStatus: function setStatus_TaskRow(categoryId){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + categoryId);
    //// remove the old one
    if (this._statusClass) {
      dojo.removeClass(this.imgStatus, this._statusClass);
    }
    var statusTooltip = null;
    switch (categoryId) {
      case com.ibm.bpm.social.widgets.task.list.Category.OVERDUE :
        this._statusClass = "bpm-social-overdue";
        statusTooltip = "task-row-status-overdue-tooltip";
        break;
      case com.ibm.bpm.social.widgets.task.list.Category.AT_RISK :
        this._statusClass = "bpm-social-at-risk";
        statusTooltip = "task-row-status-at-risk-tooltip";
        break;
      default:
        this._statusClass = "bpm-social-normal-task";
        statusTooltip = "task-row-status-normal-tooltip";
        break;
    }
    //// set the new one
    dojo.addClass(this.imgStatus, this._statusClass);
    // normal mode
    this.imgStatus.title = this._nlsResources[statusTooltip];
    this.imgStatus.setAttribute("id", "imgStatus" + this.id);
    this.imgStatus.setAttribute("alt", this.imgStatus.title);
    dijit.setWaiState(this.imgStatus, "label", this._nlsResources["task-row-status-label"]);
  },

  _workOn: function _workOn_TaskRow(e){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID, "TaskRow");

    dojo.stopEvent(e);
    this.onWorkOn(this.model);

    com.ibm.bpm.social.util.common.Log.logExit(2, this.model.TKIID, "TaskRow");
  },

  onWorkOn: function onWorkOn_TaskRow(model){},

  /**
   * Sets details for task and append the simple approval user input form if the user has the necessary rights
   * to complete the task
   * 
   */
  setDetailsSimpleApproval: function setDetailsSimpleApproval_TaskRow(taskId, details){
    var myself = this;
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details
        myself.setDetails(details);
      }
      else {
        myself._setDetailsSimpleApprovalImpl(taskId, details, false /* choiceFallback */);
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  setDetailsSimpleCompletion: function setDetailsSimpleCompletion_TaskRow(taskId, details){
    var myself = this;
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details        
        myself.setDetails(details);
      }
      else {
        myself._setDetailsSimpleCompletionImpl(taskId, details);
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  setDetailsSimpleChoice: function setDetailsSimpleChoice_TaskRow(taskId, details){
    var myself = this;
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details
        myself.setDetails(details);
      }
      else {
        if (details.variables && details.variables.choices && details.variables.choices.items &&
          details.variables.choices.items.length > 0) {
          myself._setDetailsSimpleChoiceImpl(taskId, details);
        }
        else {
          myself._setDetailsSimpleApprovalImpl(taskId, details, true /* choiceFallback */);
        }
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _handleErrorForInlineCompletion: function _handleErrorForInlineCompletion_TaskRow(taskId, displayName){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    var myself = this;
    var handled = false;
    myself._restHandler.getActionsForTask(taskId, ["ACTION_CLAIM",
                                                   "ACTION_COMPLETE"],
      function(data){
        if (data.tasks.length === 1) {
          var availableActions = data.tasks[0].actions || [];
          if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 &&
            dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
            handled = true;
            myself._errorHandler.showWarningDialog("KEY_TaskInlineAlreadyAssigned", {
              1: displayName
            }, null);
          }
        }
      }, function(){
      // Ignore
      }, false /* synchronous call */);
    if (!handled) {
      this._errorHandler.showErrorDialog("KEY_TaskInlineCouldNotBeCompleted", {
        1: displayName
      }, null);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _createInlineForm: function _createInlineForm_TaskRow(styleClass /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var inlineForm = dojo.create("div", {}, this.expanded, "last");
    dojo.addClass(inlineForm, styleClass);
    //
    // Make sure that the task details are not closed when clicking in the inline input form
    // or while entering values.
    // 
    // Clicking on the details area would close the details. Prevent this for the controls.    
    //
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(inlineForm, "onclick", function(event){
      dojo.stopEvent(event);
      event.preventDefault();
    }));
    this._expandedConnects.push(dojo.connect(inlineForm, "onkeydown", function(event){
      event.stopPropagation();
    }));
    this._expandedConnects.push(dojo.connect(inlineForm, "onkeyup", function(event){
      event.stopPropagation();
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return inlineForm;
  },

  _createCommentField: function _createCommentField_TaskRow(inlineForm /* DOM Element*/){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // label for accessibility
    var label = dojo.create("label", {
      "for": this.id + "_InlineCommentField",
      "style": "display: none"
    }, inlineForm, "last");
    label.innerHTML = this._nlsResources["task-inline-comment-placeholder"];
    var commentField = dojo.create("textarea", {
      id: this.id + "_InlineCommentField",
      rows: "2",
      placeholder: this._nlsResources["task-inline-comment-placeholder"],
      title: this._nlsResources["task-inline-comment-title"]
    }, inlineForm, "last");
    dojo.addClass(commentField, "bpm-task-inline-comment");
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return commentField;
  },

  _setDetailsSimpleApprovalImpl: function _setDetailsSimpleApprovalImpl_TaskRow(taskId, details, choicesFallback){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a reject and approval button.
    // 
    // a) div for container => bpm-task-approval-form
    // b) input[type=text] for comment => bpm-task-approval-form-comment, bpm-task-inline-comment
    // c) input[type=button] for reject button => bpm-task-approval-form-reject, bpm-task-inline-red-button
    // d) input[type=button] for approve button => bpm-task-approval-form-approve, bpm-task-inline-green-button
    // e) technical div for closing floating area => bpm-task-approval-form-closing
    var inlineForm = this._createInlineForm("bpm-task-approval-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-approval-form-comment");
    var rejectButton = dojo.create("input", {
      type: "button",
      name: "SimpleApproval_Reject",
      disabled: "disabled",
      title: this._nlsResources["task-inline-reject-title-disabled"],
      value: this._nlsResources["task-inline-reject"]
    }, inlineForm, "last");
    rejectButton.dataTitleNoComment = this._nlsResources["task-inline-reject-title-disabled"];
    rejectButton.dataTitleWithComment = this._nlsResources["task-inline-reject-title-enabled"];
    dojo.addClass(rejectButton, "bpm-task-approval-form-reject bpm-task-inline-red-button-disabled");
    var approveButton = dojo.create("input", {
      type: "button",
      name: "SimpleApproval_Accept",
      title: this._nlsResources["task-inline-approve-title"],
      value: this._nlsResources["task-inline-approve"]
    }, inlineForm, "last");
    dojo.addClass(approveButton, "bpm-task-approval-form-approve bpm-task-inline-green-button");
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-approval-form-closing");
    //
    // 2) Establish logic for reject button handling. (I.e. rejecting requires comment according to design)
    // 
    // Clicking on the details area would close the details. Prevent this for the controls.
    // Reject is only enabled if a comment has been entered. This logic is implemented in private function updateCommentField.
    // 
    var updateCommentField = function(){
      if (commentField.value.length > 0) {
        rejectButton.disabled = false;
        rejectButton.title = rejectButton.dataTitleWithComment;
        dojo.replaceClass(rejectButton, "bpm-task-inline-red-button", "bpm-task-inline-red-button-disabled");
      }
      else {
        rejectButton.disabled = true;
        rejectButton.title = rejectButton.dataTitleNoComment;
        dojo.replaceClass(rejectButton, "bpm-task-inline-red-button-disabled", "bpm-task-inline-red-button");
      }
    };
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(commentField, "onclick", function(event){
      updateCommentField();
    }));
    // Make sure that the button enablement/disablement happen at each key stroke.
    this._expandedConnects.push(dojo.connect(commentField, "onkeyup", function(event){
      updateCommentField();
    }));
    this._expandedConnects.push(dojo.connect(commentField, "onchange", function(event){
      updateCommentField();
    }));
    //
    // 3) Finally trigger the completion of the Simple Approval Huamn Service.
    //  
    var myself = this;
    var property = choicesFallback ? "decision" : "approved";
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    this._expandedConnects.push(dojo.connect(approveButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value || "";
      myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, property, choicesFallback ? "true" : true, comment,
        function(response){
          var streamMessage =
            comment.length > 0 ? myself._nlsResources["task-inline-simple-approval-comment"]
              : myself._nlsResources["task-inline-simple-approval"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
    }));
    this._expandedConnects.push(dojo.connect(rejectButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value ? commentField.value : "";
      myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, property, choicesFallback ? "false" : false, comment,
        function(response){
          var streamMessage = myself._nlsResources["task-inline-simple-approval-rejection-comment"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Sets details for task and append the simple completion user input form if the user has the necessary
   * rights to complete the task
   */
  _setDetailsSimpleCompletionImpl: function _setDetailsSimpleCompletionImpl_TaskRow(taskId, details, choiceFallback){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a complete button.
    // 
    // a) div for container => bpm-task-completion-form
    // b) input[type=text] for comment => bpm-task-completion-form-comment, bpm-task-inline-comment
    // c) input[type=button] for complete button => bpm-task-completion-form-complete, bpm-task-inline-blue-button
    // d) technical div for closing floating area => bpm-task-completion-form-closing
    var inlineForm = this._createInlineForm("bpm-task-completion-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-completion-form-comment");
    var completeButton = dojo.create("input", {
      type: "button",
      name: "SimpleCompletion_Accept",
      title: this._nlsResources["task-inline-complete-title"],
      value: this._nlsResources["task-inline-complete"]
    }, inlineForm, "last");
    dojo.addClass(completeButton, "bpm-task-completion-form-complete bpm-task-inline-blue-button");
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-completion-form-closing");
    //
    // 2) Finally trigger the completion of the Simple Completion Human Service    
    //
    var myself = this;
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(completeButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value || "";
      myself._restHandler.completeSimpleCompletionTask(taskId, unclaimed, comment, function(response){
        var streamMessage =
          comment.length > 0 ? myself._nlsResources["task-inline-simple-completion-comment"]
            : myself._nlsResources["task-inline-simple-completion"];
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        var fullName = user.fullName;
        var parameters = [fullName,
                          myself.model["TAD_DISPLAY_NAME"],
                          comment];
        myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
        myself.onDeleteRowNeeded(taskId);
      }, function(error, ioArgs){
        myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
        myself.onRefreshNeeded();
      });
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * Sets details for task and append the simple choice user input form if the user has the necessary rights
   * to complete the task.
   */
  _setDetailsSimpleChoiceImpl: function _setDetailsSimpleChoiceImpl_TaskRow(taskId, details){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    var myself = this;
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a complete button.
    // 
    // a) div for container => bpm-task-choice-form
    // b) input[type=text] for comment => bpm-task-choice-form-comment, bpm-task-inline-comment
    // c) input[type=button] for each option button => bpm-task-choice-form-complete, bpm-task-inline-blue-button
    // d) select for the available options => bpm-task-choice-form-options
    // e) technical div for closing floating area => bpm-task-choice-form-closing
    var inlineForm = this._createInlineForm("bpm-task-choice-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-choice-form-comment");
    var submitButtons = [];
    var numberOfChoices = details.variables.choices.items.length;
    for ( var i = numberOfChoices - 1; i >= 0; i--) {
      var choice = details.variables.choices.items[i];
      var completeButton = dojo.create("input", {
        type: "button",
        name: "SimpleChoice_Accept",
        title: myself._nlsResources["task-inline-complete-title"],
        value: choice
      }, inlineForm, "last");
      dojo.addClass(completeButton, "bpm-task-choice-form-complete bpm-task-inline-blue-button");
      submitButtons.push(completeButton);
    }
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-choice-form-closing");
    //
    // 2) Finally trigger the completion of the Simple Choice Human Service    
    //           
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    var clickHandler =
      function(decision){
        // Provide the data as escaped json to REST call
        var comment = commentField.value || "";
        myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, "decision", decision, comment, function(){
          var streamMessage =
            comment.length > 0 ? myself._nlsResources["task-inline-simple-choice-comment"]
              : myself._nlsResources["task-inline-simple-choice"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            decision,
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
      };
    dojo.forEach(submitButtons, function(completeButton){
      myself.connect(completeButton, "onclick", function(){
        clickHandler(completeButton.value);
      });
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _postMessage: function _postMessage_TaskRow(piid, streamMessage, parameters){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // Current user is cached by singleton function.
    streamMessage = dojo.replace(streamMessage, parameters);
    var restHandler = this._restHandler;
    // Make sure the process actions have already been retrieved
    var myself = this;
    if (this._processActionsDeferred != null) {
      this._processActionsDeferred.then(function(){
        if (myself._postInstanceComment) {
          restHandler.publishOnInstanceStream(piid, streamMessage, function(){}, function(){});
        }
      });
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * sets detailed information for specific task as description and specific key value pairs
   * 
   * Expected details format: {narrative: "...", businessData: [{label: "myLabel", value: "myValue", type:
   * String|Integer|Boolean|Date|Time|Decimal, name: "myVariableName"}, ...]}
   */
  setDetails: function setDetails_TaskRow(details){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // Clean up expanded area
    this._clearDetailsArea();

    var isDetailsFound = false;

    if (details) {

      if (!(details.narrative === null || details.narrative === "")) {
        var narrativeContainer = document.createElement("div");
        narrativeContainer.innerHTML = com.ibm.bpm.social.util.common.Util.createSecureInnerHtml(details.narrative);
        dijit.setWaiState(narrativeContainer, "label", this._nlsResources["task-row-task-definition-label"]);

        //Apply base text direction to the narrative container
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(narrativeContainer);

        narrativeContainer.setAttribute("id", "narrativecontainer" + this.id);

        dojo.style(narrativeContainer, "marginBottom", "10px");
        dojo.addClass(narrativeContainer, "bpm-task-businessData");
        this.expanded.appendChild(narrativeContainer);
        isDetailsFound = true;
      }

      if (details.businessData) {
        var self = this;
        var businessDataContainer = document.createElement("div");
        dijit.setWaiState(businessDataContainer, "label", this._nlsResources["task-row-task-details-label"]);
        businessDataContainer.setAttribute("id", "businessdata" + this.id);
        //dojo.style(businessDataContainer, "position", "relative");
        dojo.addClass(businessDataContainer, "bpm-task-businessDataContainer");
        var labelString = this._nlsResources["task-row-task-details-business-data-label"];
        dojo.forEach(details.businessData, function(businessData, index){
          // Labels of business data with null or empty string value will 
          // not be displayed.
          if (typeof businessData.value != "undefined" && businessData.value !== null && businessData.value !== "") {
            var labelAndValue = document.createElement("div");
            dojo.addClass(labelAndValue, "bpm-task-businessDataContainer-entry");
            var label = document.createElement("span");
            label.innerHTML = dojo.string.substitute(labelString, [businessData.label]) + "&nbsp;";
            dojo.addClass(label, "bpm-task-businessDataLabel bpm-task-businessData bpm-task-businessData-string");
            if (businessData.alias) {
              label.title = businessData.alias.toLowerCase();
            }
            labelAndValue.appendChild(label);
            var formattedValue = self.formatBusinessData(businessData.type, businessData.value);
            labelAndValue.appendChild(formattedValue);
            //appropriately tag all string business data elements for base text direction application
            if (businessData.type.toLowerCase() == "string") {
              dojo.addClass(formattedValue, "bpm-task-businessData-string");
            }
            businessDataContainer.appendChild(labelAndValue);
            isDetailsFound = true;
          }
        });

        //query all string business data elements if pref is not "Default"
        var btdpref = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
        if (btdpref != "default") {
          dojo.query(".bpm-task-businessData-string", businessDataContainer).forEach(function(node, index, arr){
            //configure BTD for each element
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(node);
          });
        }
        var clearDiv = document.createElement("div");
        dojo.style(clearDiv, "clear", "both");
        businessDataContainer.appendChild(clearDiv);

        this.expanded.appendChild(businessDataContainer);
      }
    }

    if (!isDetailsFound) {
      this.showMessage(this._nlsResources["task-row-details-not-found"]);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  formatBusinessData: function formatBusinessData_TaskRow(type, value){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var valueContainer = document.createElement("span");
    dojo.addClass(valueContainer, "bpm-task-businessDataValue bpm-task-businessData");

    switch (type) {
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DATE_TYPE :
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_TIME_TYPE :
        valueContainer.innerHTML = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatDate(value);
        break;
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DECIMAL_TYPE :
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_INTEGER_TYPE :
        valueContainer.innerHTML = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatNumber(value);
        break;
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_BOOLEAN_TYPE :
        valueContainer.innerHTML =
          (value ? this._nlsResources["task-row-details-boolean-true"]
            : this._nlsResources["task-row-details-boolean-false"]);
        break;
      default:
        var valueParts = value.toString().split("\n");
        for ( var i = 0; i < valueParts.length; i++) {
          valueContainer.appendChild(document.createTextNode(valueParts[i]));
          if (i + 1 != valueParts.length) {
            valueContainer.appendChild(document.createElement("br"));
          }
        }
        break;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return valueContainer;
  },

  _expand: function _expand_TaskRow(evt){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    dojo.style(this.expanded, "display", "block");
    dojo.replaceClass(this.taskBoxAttach, "bpm-social-task-box-selected", ["bpm-social-task-box"]);
    this.open = true;
    if (evt) {
      this.onDetailsRequested(this.model);
    }
    this.onExpanded(this.model);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  onDetailsRequested: function onDetailsRequested_taskRow(model){},

  _collapse: function _collapse_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    dojo.style(this.expanded, "display", "none");
    dojo.replaceClass(this.taskBoxAttach, "bpm-social-task-box", ["bpm-social-task-box-selected"]);
    if (this.open) {
      this.open = false;
      this.onCollapsed(this.model);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  toggle: function toggle_TaskRow(evt){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    if (this.open) {
      // collapse
      this._collapse();
    }
    else {
      // expand
      this._expand(evt);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _clearDetailsArea: function _clearDetailsArea_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    if (this._expandedConnects) {
      for ( var i = 0; i < this._expandedConnects.length; i++) {
        dojo.disconnect(this._expandedConnects[i]);
      }
      this._expandedConnects = null;
    }
    if (this.expanded.hasChildNodes()) {
      while (this.expanded.childNodes.length >= 1) {
        //this.expanded.firstChild.destroyRecursive(true);
        dojo.destroy(this.expanded.firstChild);
        //this.expanded.removeChild(this.expanded.firstChild);
      }
    }
    dojo.style(this.expanded, null);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  showMessage: function showMessage_TaskRow(message){
    this._clearDetailsArea();
    this.expanded.innerHTML = message;
  },

  _createMenuButton: function _createMenuButton_TaskRow(){
    var myself = this;
    if (this.model && this.model.availableActions) {

      var params = {
        task: this.model,
        availableActions: this.model.availableActions,
        taskDisplayName: this.model.TAD_DISPLAY_NAME
      };
      if (this._modifyTaskHandler) {
        this._modifyTaskHandler.init(params);
      }
      else {
        this._modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler(params);

        this.connect(this._modifyTaskHandler, "onTaskModified", this._handleTaskModifyUpdates);

        this.connect(this._modifyTaskHandler, "onError", this.onRefreshNeeded);

        this.connect(this._modifyTaskHandler, "onHide", dojo.hitch(this, function(modifyTaskHandler){
          // destroy this handler if task row has been reused while dialog was open
          if (this._modifyTaskHandler != modifyTaskHandler) {
            modifyTaskHandler.destroy();
          }
        }));
      }

      if (this._menu) {
        this._clearMenuButton();
      }

      this._menu = this._dropDownHelper.createDropDownForTaskRow(this.menuButton, this.model, this.model.availableActions, {
        modifyTask: function(){
          myself._modifyTaskHandler.showDialog();
        },
        reassignBackToGroup: function(){
          myself._modifyTaskHandler.reassignBackToGroup();
        }
      });

      if (this._menu) {        
        this._menu.connect(this._menu, "onOpen", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);
        }));
        this._menu.connect(this._menu, "onClose", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
        }));

        // task is associated with process
        if (typeof this.model["PI_PIID"] == "string") {
          var piid = this.model["PI_PIID"];
          this._processActionsDeferred = this._dropDownHelper.addProcessActionsOnFocus(this._menu, {
            "piid": piid
          }, {}, false, false, true /*omitCriticalPath*/);
          this._setupProcessActionsDeferred(this._processActionsDeferred);
        }
        com.ibm.bspace.common.util.ui.Helper.wrapElement(this.menuButton);
        dojo.style(this.menuButton, "display", "block");
      }
    }
  },

  _clearMenuButton: function _clearMenuButton_TaskRow(){
    if (this._menu) {
      this._menu.destroyRecursive();
      this._menu = null;
    }
  },

  onRefreshNeeded: function onRefreshNeeded_TaskRow(){},

  /**
   * @param model updated model
   */
  onModelUpdate: function onModelUpdate_TaskRow(model){},

  /**
   * 
   */
  onDeleteRowNeeded: function onDeleteRowNeeded_TaskRow(tkiid){},

  /*
   * Callback function to be executed when the update of the available actions 
   * for the item is finished successfully.
   * 
   * @param data response data
   * @returns {void} nothing
   */
  _handleUpdateAvailableActions: function _handleUpdateAvailableActions_TaskRow(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    var myself = this;
    var tasksForId = dojo.filter(data.tasks, function(task){
      return task.tkiid === myself.model.TKIID;
    });
    if (tasksForId.length > 0) {
      this.model.availableActions = tasksForId[0].actions;
      this._handleAvailableActions(data);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * callback function to be executed when the retrieval of available actions for the item is finished
   * successfully
   * 
   * @param data response data, ignored here because available actions are available on
   *        this.model.availableActions
   * @returns {void} nothing
   */
  _handleAvailableActions: function _handleAvailableActions_TaskRow(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    // enable menu
    this._createMenuButton();

    // enable onclick and onkeypress Enter on subject
    if (!this._connectsCallback) {
      this._connectsCallback = [];
      this._connectsCallback.push(dojo.connect(this.subject, "onclick", this, this._workOn));
      this._connectsCallback.push(dojo.connect(this.subject, "onkeydown", this, function(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
          this._workOn(evt);
        }
      }));
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setupProcessActionsDeferred: function _setupProcessActionsDeferred(deferred){
    if (deferred != null) {
      deferred.then(dojo.hitch(this, "_handleAvailableProcessActions"));
    }
  },

  _triggerProcessActionsDeferred: function _triggerProcessActionsDeferred(piid /* string */, systemId){
    if (this._processActionsDeferred != null) {
      if (!this._processActionsReceived) {
        var deferred = this._processActionsDeferred;
        this._restHandler.getProcessActions(piid, function(result){
          deferred.resolve(result);
        }, function(){}, true /* async */, null /* parts */, systemId);
      }
    }
  },

  /**
   * Callback function to be executed when the retrieval of the available process actions of the itm
   * 
   * @param result
   * @returns {_handleAvailableProcessActions_TaskRow} {void} nothing
   */
  _handleAvailableProcessActions: function _handleAvailableProcessActions_TaskRow(result){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    this._processActionsReceived = true;
    if (result.standardActions != null) {
      if (dojo.indexOf(result.standardActions, "ACTION_ADD_COMMENT") != -1) {
        this._postInstanceComment = true;
      }
      else {
        this._postInstanceComment = false;
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return result;
  },

  onExpanded: function onExpanded_TaskRow(model){},

  onCollapsed: function onCollapsed_TaskRow(model){},

  _handleTaskModifyUpdates: function _handleTaskModifyUpdates_TaskRow(dueDateTime, priority, assigneeChanged, newAssignee){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var updatedValues = {
      "TASK.TKIID": this.model.TKIID,
      "TKIID": this.model.TKIID
    };
    if (dueDateTime) {
      updatedValues["DUE"] = dueDateTime;
    }
    if (priority) {
      updatedValues["PRIORITY"] = priority;
    }

    if (newAssignee) {
      updatedValues["OWNER"] = newAssignee;
      this.onModelUpdate(updatedValues, true);
    }
    else if (assigneeChanged) {
      // will be updated after push event is received
      this.group.innerHTML = this._nlsResources["task-row-details-loading"];
    }
    else {
      this.onModelUpdate(updatedValues, true);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }
});

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_STRING_TYPE = "String";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_INTEGER_TYPE = "Integer";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DECIMAL_TYPE = "Decimal";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DATE_TYPE = "Date";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_TIME_TYPE = "Time";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_BOOLEAN_TYPE = "Boolean";

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.details.InstanceDetails"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.details.InstanceDetails"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.details.InstanceDetails");



























dojo.declare("com.ibm.bpm.social.widgets.details.InstanceDetails", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                                    dijit._Templated], {

  templateString: "",
  templateString:"<div class=\"instanceDetails\" role=\"complementary\" dojoAttachPoint=\"instanceDetailsDiv\" tabindex=\"0\">\n<div class=\"headerWrapper\" dojoAttachPoint=\"headerWrapper\">\n\t<div class=\"ie8messageDiv\">\n\t<span class=\"ie8message\" id=\"ie8message\" dojoAttachPoint=\"ie8message\"></span>\n\t<a id=\"ie8messageLink\" class=\"ie8messageLink\" href=\"/BusinessSpaceHelp/index.jsp?topic=/com.ibm.wbpm.portal.help.doc/topics/cport_ie8.html\" target=\"_blank\" aria-labelledBy=\"ie8message\">\n\t<img class=\"ie8messageIcon\" dojoAttachPoint=\"ie8messageIcon\"></img>\n\t<span class=\"ie8messageA11y\">(?)</span>\n\t</a>\n\t</div>\n\t<a class=\"instanceName\" dojoAttachPoint=\"instanceName\" dojoAttachEvent=\"onclick:_onInstanceClick\" tabindex=\"0\" aria-describedby=\"instancedetails_duedatewrapper\">_</a>\n\t<div class=\"followedAnchor\" role=\"button\" dojoAttachPoint=\"followedAnchor\" dojoAttachEvent=\"onclick:_onFollowedClick, onkeypress:_onFollowedKeyPress\" \n\ttabindex=\"0\" aria-pressed=\"false\"><span dojoAttachPoint=\"followAltText\" class=\"hi-contrast-icon\">&#9734;</span></div>\n\t<div class=\"dropdownAnchor bpm-social-drop-down\" style=\"display:none;\" dojoAttachPoint=\"dropdownWrapper\" tabindex=\"0\">\n\t\t<span class=\"dropDownText\">&#9660;</span>\n\t</div>\n\t<div class=\"detailsSeparator\"></div>\n\t<span id=\"instancedetails_duedatewrapper\" class=\"dueDateWrapper\">\n\t\t<img class=\"dueDateOverdue bpm-social-overdue\" dojoAttachPoint=\"dueDateOverdue\"/>\n\t\t<div class=\"dueDateLabel\" dojoAttachPoint=\"dueDateLabel\"></div>\n\t\t<div class=\"dueDateValue\" dojoAttachPoint=\"dueDateValue\"></div>\n\t</span>\n</div>\n<div class=\"bizDataWrapper\" dojoAttachPoint=\"bizDataWrapper\"></div>\n<div class=\"diagramLinkWrapper\" dojoAttachPoint=\"diagramLinkWrapper\" style=\"display: none;\">\n\t<a href=\"#\" dojoAttachPoint=\"diagramLinkAnchor\" dojoAttachEvent=\"onclick:_viewProcessDiagram\" tabindex=\"0\">\n\t\t<img class=\"diagramLinkImg bpm-social-process-icon\" dojoAttachPoint=\"diagramLinkImg\" alt=\"\"/>\n\t\t<span class=\"diagramLinkText\" dojoAttachPoint=\"diagramLinkText\"></span>\n\t</a>\n</div>\n<div class=\"taskDetailsWrapper\" role=\"complementary\" dojoAttachPoint=\"taskDetailsWrapper\" tabindex=\"0\">\n\t<div class=\"taskDetailsHeader\" dojoAttachPoint=\"taskDetailsHeader\"></div>\n\t<ul id=\"instancedetails_taskdetailslist\" class=\"taskDetailsList\" dojoAttachPoint=\"taskDetailsList\"><li style=\"display:none\">&nbsp;</li></ul>\n</div>\n</div>\n",
  sidebar: false,
  instanceId: null,
  followed: false,
  fullWidth: 0,
  instanceData: null,
  counter: 0,
  handle: null,
  taskDetailsItems : null,

  //Enclosed dijit widgets
  menu: null,
  dropdownButton: null,
  processActionsAvailable: false,
  taskDetailsListItems: null,

  //JSP URLs
  //auditTrailUrl: "/portal/jsp/processAudit.do?bpdInstanceId={iid}",
  //criticalPathUrl: "/portal/jsp/criticalPath.do?cpType=2&bpdInstanceId={iid}",
  instanceDataUrl: window.endpointURLObject.bpmRest +"/v1/process/{piid}?parts=header%7Cdata",
  instanceFollowingUrl: window.endpointURLObject.bpmRest +"/v1/social/instance/{piid}/following",

  postCreate: function(){
    this.taskDetailsItems = new Array();
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    
	this.ie8message.innerHTML = this._nlsResources.ie8message;
	this.ie8messageIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
	
    dojo.attr(this.instanceDetailsDiv, "aria-label", this.messages.details);

    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();

    this.followedAnchor.title = this.messages.notFollowing;

    this.diagramLinkImg.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    // RPT: Image alt text must not be redundant with adjacent link text
    // this.diagramLinkImg.alt = this.messages.viewProcessDiagram;  

    if (this.sidebar) {
      dojo.addClass(this.domNode, 'sidebar');
      dojo.place(this.followedAnchor, this.instanceName, "before");
    }

    dojo.subscribe(com.ibm.bpm.social.portal.Topics.PROCESS_ACTIONS_AVAILABLE, dojo.hitch(this,
      this._processActionsAvailable));

    if (this.instanceId != null) {
      this._getInstanceDetails();
    }

    //Add Instance Due Date to header with optional overdue icon
    this.dueDateLabel.innerHTML = this.messages.due.replace("${date}", "");
    this.dueDateLabel.id = 'dueDateLabel_' + this.get("id");
    this.dueDateValue['aria-labelledby'] = 'dueDateLabel_' + this.get("id");

    this.dueDateOverdue.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    this.dueDateOverdue.alt = this.messages.overdue;

  },

  refresh: function(obj){
    if (!(obj.instanceId)) {
      return;
    }
    this.instanceId = obj.instanceId;
    this.processActionsAvailable = false;
    this._clearTaskDetailsItems();
    this._getInstanceDetails();
  },
  
  _clearTaskDetailsItems: function() {
		dojo.forEach(this.taskDetailsItems , function(ti){
			if(ti.destroy){
				ti.destroy();
			}
		});
		// Do not destroy the task details items several times.
		this.taskDetailsItems  = new Array();		
  },

  _getInstanceDetails: function(){

    this.processRestHandler.getProcess(this.instanceId, dojo.hitch(this, "_loadInstanceDetails"), dojo.hitch(this,
      "_xhrErrors"), true, "header%7Cdata");
  },

  _loadInstanceDetails: function(data, xhrArgs){

    this.instanceData = data;

    //Apply styles
    this.fullWidth = dojo.style(this.domNode, "width") - 20;
    if (!this.sidebar) {
      //dojo.style(this.headerWrapper, "width", this.fullWidth+"px"); //Account for 100% width with padding
      dojo.style(this.instanceName, "maxWidth", Math.max(0, (this.fullWidth - 70)) + "px"); //Buffer for long instance names
    }

    dojo.connect(window, "onresize", this, dojo.hitch(this, function(){
      this.fullWidth = dojo.style(this.domNode, "width") - 20;
      if (!this.sidebar) {
        //dojo.style(this.headerWrapper, "width", this.fullWidth+"px"); //Account for 100% width with padding
        dojo.style(this.instanceName, "maxWidth", Math.max(0, (this.fullWidth - 70)) + "px"); //Buffer for long instance names
      }
    }));

    //Update header information
    var text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.name);
    this.instanceName.innerHTML = text;
    this.instanceName.title = text;

    if (typeof data.dueDate != "undefined" && data.dueDate != null) {
      this.dueDateValue.innerHTML = this._determineDueDate(data.dueDate);
      if (this.dueDateValue.innerHTML == this.messages.today || this.dueDateValue.innerHTML == this.messages.yesterday ||
        this.dueDateValue.innerHTML == this.messages.tomorrow) {
        this.dueDateValue.title = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatDate(data.dueDate, {
          formatLength: "long"
        });
      }

      var now = new Date();
      var due = dojo.date.stamp.fromISOString(data.dueDate);

      if (dojo.date.compare(now, due) >= 0) {
        dojo.addClass(this.domNode, "overdue");
      }
      else {
        dojo.removeClass(this.domNode, "overdue");
      }
    }
    else {
      this.dueDateValue.innerHTML = "";
    }

    this._getInstanceFollowed();

    //Update business data information
    if (data.businessData) {

      dojo.empty(this.bizDataWrapper);

      var btdpref = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
      for ( var item in data.businessData) {
        var value = data.businessData[item].value;
        if (typeof data.businessData[item].value != "undefined" && data.businessData[item].value !== null &&
          data.businessData[item].value !== "") {
          var bd = dojo.create("div", {
            "class": "bizDataItem",
            "wairole": "label",//RPT violation: Elements that use a 'role' must reference a valid WAI-ARIA role
            "tabindex": "0"
          }, this.bizDataWrapper);
          this._setBidiLangDir(bd);
          var title = "";
          if (data.businessData[item].alias) {
            title = data.businessData[item].alias.toLowerCase();
          }
          else if (data.businessData[item].name) {
            title = data.businessData[item].name;
          }

          var span = dojo.create("span", {
            "class": "bizDataItemLabel",
            innerHTML: data.businessData[item].label,
            "wairole": "label",//RPT violation: Elements that use a 'role' must reference a valid WAI-ARIA role
            "title": title
          }, bd);
          this._setDirAttribute(btdpref, data.businessData[item].label, span);
          span = null;
          dojo.create("span", {
            "class": "bizDataItemSeparator",
            innerHTML: this.messages.colon_separator
          }, bd);
          var formattedValue = value;
          var type = data.businessData[item].type;
          var applyBTD = false;
          switch (type) {
            case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DATE_TYPE :
            case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_TIME_TYPE :
              formattedValue = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatDate(value);
              applyBTD = true;
              break;
            case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DECIMAL_TYPE :
            case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_INTEGER_TYPE :
              formattedValue = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatNumber(value);
              break;
            case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_BOOLEAN_TYPE :
              formattedValue =
                (value ? this._nlsResources["task-row-details-boolean-true"]
                  : this._nlsResources["task-row-details-boolean-false"]);
              break;
            default:
              formattedValue = null;
              var valueParts = value.toString().split("\n");
              for ( var i = 0; i < valueParts.length; i++) {
                if (i == 0) {
                  dojo.create("span", {
                    innerHTML: "&nbsp;"
                  }, bd);
                }
                var span = document.createElement("span");
                span.innerHTML = valueParts[i];
                this._setDirAttribute(btdpref, valueParts[i], span);
                bd.appendChild(span);
                if (i + 1 != valueParts.length) {
                  bd.appendChild(document.createElement("br"));
                }
                span = null;
              }
              break;
          }
          if (formattedValue != null) {
            dojo.create("span", {
              innerHTML: "&nbsp;"
            }, bd);
            var span = dojo.create("span", {
              "class": "bizDataItemValue",
              "title": formattedValue,
              "innerHTML": formattedValue
            }, bd);
            if (applyBTD) {
              this._setDirAttribute(btdpref, formattedValue, span);
            }
            span = null;
          }
        }
      }
    }

    //Update process diagram link
    // RPT: Image alt text must not be redundant with adjacent link text
    // this.diagramLinkImg.alt = this.messages.viewProcessDiagram;
    this.diagramLinkText.innerHTML = this.messages.viewProcessDiagram;

    //Update task details list
    if (data.tasks) {

      // Delete all children (widgets and nodes) of this.taskDetailsList 
      while (this.taskDetailsList.firstChild) {
        var cList = this.taskDetailsList.firstChild;
        while (cList.firstChild) {
          var cTaskDetails = cList.firstChild;
          var cTaskDetailsWidget = dijit.byId(cTaskDetails.id);
          if (cTaskDetailsWidget && cTaskDetailsWidget.destroy) {
            cTaskDetailsWidget.destroy();
          }
          else {
            cList.removeChild(cTaskDetails);
            delete cTaskDetails;
          }
        }
        this.taskDetailsList.removeChild(cList);
        delete cList;
      }

      //Sort tasks in reverse order by dueDate
      data.tasks.sort(function(a, b){
        var ad = new Date(a.dueTime);
        var bd = new Date(b.dueTime);
        return bd - ad;
      });

      this.taskDetailsHeader.innerHTML = this.messages.tasks;
      this.taskDetailsHeader.id = "taskDetailsHeader_" + this.get("id");
      this.taskDetailsList['aria-labelledby'] = "taskDetailsHeader_" + this.get("id");
      dojo.attr(this.taskDetailsWrapper, "aria-labelledby", this.taskDetailsHeader.id);

	  var onlineTrackingItems = [];
	  
      if(!this.taskDetailsListItems) {
          this.taskDetailsListItems = new Array();
      } else {
        for(var i=0; i < this.taskDetailsListItems.length; i++) {
          this.taskDetailsListItems[i].destroy();
        }
        this.taskDetailsListItems = new Array();
      }
      for ( var item in data.tasks) {
        var describedby = "";
        var li = dojo.create("li", {}, this.taskDetailsList);

        var tdli = new com.ibm.bpm.social.widgets.details.TaskDetailsListItem({
          taskId: data.tasks[item].tkiid,
          taskName: data.tasks[item].name,
          taskDisplayName: data.tasks[item].displayName,
          taskAssignee: data.tasks[item].assignedTo,
          taskAssigneeDisplayName: data.tasks[item].assignedToDisplayName,
          taskOwner: data.tasks[item].owner,
          taskOwnerDisplayName: data.tasks[item].ownerDisplayName,
          taskStarted: data.tasks[item].startTime,
          taskCompleted: data.tasks[item].completionTime,
          taskDue: data.tasks[item].dueTime,
          taskStatus: data.tasks[item].status,
          sidebar: this.sidebar
        }, dojo.create("div", {}, li));
		
		var onlineTracking = tdli.getOnlineTracking();
		if (onlineTracking != null) {
			onlineTrackingItems.push(onlineTracking);
		}
		this.taskDetailsItems.push(tdli);

        this.taskDetailsListItems.push(tdli);
        describedby = describedby + "taskOwner_" + data.tasks[item].tkiid + " ";
        describedby = describedby + "taskStarted_" + data.tasks[item].tkiid + " ";
        describedby = describedby + "taskCompleted_" + data.tasks[item].tkiid + " ";
        describedby = describedby + "taskDue_" + data.tasks[item].tkiid;
        dojo.attr(tdli.domNode, "aria-describedby", describedby);
      }
	  
	  if (onlineTrackingItems.length > 0) {
		dojox.cometd.batch(function() {
			for(var i = 0; i < onlineTrackingItems.length; i++) {
				// Subscribe to appropriate channels
				onlineTrackingItems[i].activate();
			}
		});
	  }
    }

    if (!this.sidebar) {
      //Only show actions drop down when viewed in Instance Details page and not Task Completion sidebar.
      this._updateActionsDropdown();
    }

  },

  /*
   * Determine whether to show due date, Today, Tomorrow, or Yesterday.
   * Same pattern as used by TasksListController.js::_addCategories with less outcomes 
   */

  _determineDueDate: function(d){

    var result = com.ibm.bpm.social.util.common.Util.formatDate(d);
    var now = new Date();
    var endToday = new Date(now);
    endToday.setHours(23, 59, 59, 0);
    var endTom = dojo.date.add(endToday, "day", 1);

    var startToday = new Date(now);
    startToday.setHours(0, 0, 0, 0);

    var startYest = dojo.date.add(now, "day", -1);
    startYest.setHours(0, 0, 0, 0);

    var due = dojo.date.stamp.fromISOString(d);

    if (dojo.date.compare(now, due) >= 0) {
      dojo.addClass(this.domNode, "overdue");
    }
    else {
      dojo.removeClass(this.domNode, "overdue");
    }

    /*
    console.debug("***Check for Today;")
    console.debug("dojo.date.compare(due, startToday) >= 0: ",dojo.date.compare(due, startToday) >= 0);
    console.debug("dojo.date.compare(endToday, due) > 0: ",dojo.date.compare(endToday, due) > 0 );
    
    console.debug("***Check for Tomorrow");
    console.debug("dojo.date.compare(due, endToday) >= 0: ", dojo.date.compare(due, endToday) >= 0);
    console.debug("dojo.date.compare(endTom, due) > 0: ",dojo.date.compare(endTom, due) > 0);
    
    console.debug("***Check for Yesterday");
    console.debug("dojo.date.compare(due, startYest) >= 0: ",dojo.date.compare(due, startYest) >= 0);
    console.debug("dojo.date.compare(startToday, due) > 0: ", dojo.date.compare(startToday, due) > 0);
    */

    if (dojo.date.compare(due, startToday) >= 0 && dojo.date.compare(endToday, due) > 0) {
      result = this.messages.today;
    }
    else if (dojo.date.compare(due, endToday) >= 0 && dojo.date.compare(endTom, due) > 0) {
      result = this.messages.tomorrow;
    }
    else if (dojo.date.compare(due, startYest) >= 0 && dojo.date.compare(startToday, due) > 0) {
      result = this.messages.yesterday;
    }

    return result;
  },

  /*
  _formatDate: function(str){
    var date,d1,t1;
    
    str = str.replace("Z", ".000+0000");
    date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
    
    d1 = dojo.date.locale.format(date, {formatLength: "long", selector: "date"});
    t1 = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});

    return d1; //d1+" "+t1
  },
  */

  _updateActionsDropdown: function(){

    var _dd = com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();

    this.menu = _dd.createDropDown(this.dropdownWrapper);

    _dd.addProcessActions(this.menu, this.instanceData, {
      'modifyInstance': dojo.hitch(this, this._launchChangeDueDateDialog),
      'viewProcessDiagram': dojo.hitch(this, this._viewProcessDiagram)
    }, /* dialog = */false, /* omitViewInstance = */true, /* omitProcessDiagram = */true);

  },

  _processActionsAvailable: function(data){
    if ((data.standardActions && data.standardActions.length > 0) || (data.actions && data.actions.length > 0)) {
      if (this.processActionsAvailable == false) {
        if (data.standardActions != null &&
          this.sidebar == false &&
          (dojo.indexOf(data.standardActions, "ACTION_CHANGE_INSTANCE_DUEDATE") > -1 ||
            dojo.indexOf(data.standardActions, "ACTION_VIEW_PROCESS_AUDIT") > -1 || dojo.indexOf(data.standardActions,
            "ACTION_CHANGE_CRITICAL_PATH") > -1)) {
          com.ibm.bspace.common.util.ui.Helper.wrapElement(this.dropdownWrapper);
          dojo.style(this.dropdownWrapper, "display", "block");
        }

        if (data.standardActions != null && dojo.indexOf(data.standardActions, "ACTION_VIEW_PROCESS_DIAGRAM") > -1) {
          dojo.style(this.diagramLinkWrapper, "display", "block");
        }
        this.processActionsAvailable = true;
      }
    }
  },

  _launchChangeDueDateDialog: function(){
    var cddd = new com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog({
      "instanceId": this.instanceData.piid,
      "dueDate": this.instanceData.dueDate,
      "callback": dojo.hitch(this, this._setInstanceData)
    }, dojo.create("div"));
    cddd.show();
  },

  /*
   * Callback method to update local instanceData variable with result of changeDueDate REST call.
   */
  _setInstanceData: function(data){
    if (typeof (data) == "undefined" || typeof (data.piid) == "undefined") {
      data = com.ibm.bpm.social.util.common.Util.getInstanceData({
        instanceId: this.instanceId,
        header: true,
        skipCache: true,
        updateTaskData: true
      });

      var t, a;
      for ( var i = 0; i < data.tasks.length; i++) {
        t = data.tasks[i];
        a = (t.assignedToDisplayName ? t.assignedToDisplayName : t.assignedTo);
        dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + t.tkiid, [{
          dueDateTime: t.dueTime,
          priority: t.priority,
          assignee: a,
          skipCache: false
        }]);
      }
    }

    this.instanceData = data;
    this.dueDateValue.innerHTML = this._determineDueDate(this.instanceData.dueDate);

    if (this.dueDateValue.innerHTML == this.messages.today || this.dueDateValue.innerHTML == this.messages.yesterday ||
      this.dueDateValue.innerHTML == this.messages.tomorrow) {
      this.dueDateValue.title = dojo.date.locale.format(dojo.date.stamp.fromISOString(this.instanceData.dueDate), {
        formatLength: "long"
      });
    }
  },

  _getInstanceFollowed: function(){

    this.socialRestHandler.isFollowingInstance(this.instanceId, dojo.hitch(this, function(data, xhrArgs){
      this._updateFollowedUI(data.followed);
      this.followed = data.followed;
    }), dojo.hitch(this, "_xhrErrors"), true);
  },

  _setInstanceFollowed: function(followed){

    var load = dojo.hitch(this, function(data, xhrArgs){
      this._updateFollowedUI(data.followed);
      dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_FOLLOW_LIST, [{}]);
    });
    var error = dojo.hitch(this, "_xhrErrors");

    if (followed) {
      this.socialRestHandler.followInstance(this.instanceId, load, error, true);
    }
    else {
      this.socialRestHandler.unfollowInstance(this.instanceId, load, error, true);
    }

  },

  _updateFollowedUI: function(followed){
    if (followed) {
      dojo.removeClass(this.followedAnchor, 'bpm-social-unfollowed-star');
      dojo.addClass(this.followedAnchor, 'bpm-social-followed-star');
      //this.followedAnchor.alt = this.messages.followed;
      this.followedAnchor.title = this.messages.followed;
      dojo.attr(this.followAltText, "innerHTML", "&#9733;");
    }
    else {
      dojo.removeClass(this.followedAnchor, 'bpm-social-followed-star');
      dojo.addClass(this.followedAnchor, 'bpm-social-unfollowed-star');
      //this.followedAnchor.alt = this.messages.notFollowed;
      this.followedAnchor.title = this.messages.notFollowed;
      dojo.attr(this.followAltText, "innerHTML", "&#9734;");
    }
  },

  _onFollowedClick: function(e){
    if (this.followed) {
      this.followed = false;
      this.followedAnchor.setAttribute("aria-pressed", "false");
    }
    else {
      this.followed = true;
      this.followedAnchor.setAttribute("aria-pressed", "true");
    }
    this._setInstanceFollowed(this.followed);
  },

  _onFollowedKeyPress: function(e){
    console.warn(e);
    if (e.keyCode == dojo.keys.ENTER) {
      this._onFollowedClick(e);
      dojo.stopEvent(e);
    }
  },

  _viewProcessDiagram: function(e){

    /** ** BEGIN Loading Process Diagram-specific modules *** */
    /*
    try{
      dojo.registerModulePath('ibm_ilog.diagram', '/ProcessPortal/layers/'+socialversion+'/ibm_ilog/diagram');
      dojo["require"]("ibm_ilog.diagram.ibm_ilog_diagram_base"); //Alternate syntax used to avoid dojo build detection
    } catch (e) { 
      console.warn(e);
      dojo.registerModulePath('ibm_ilog.diagram', '/WebViewer/widgets/ibm_ilog/diagram');
    }
    
    try{
      dojo.registerModulePath('com.ibm.bpm.wpd', '/ProcessPortal/layers/'+socialversion+'/com/ibm/bpm/wpd'); //Needed, in addition to /com/ibm/bpm, for img & css files
      dojo["require"]("com.ibm.bpm.wpd_base"); //Alternate syntax used to avoid dojo build detection
    } catch (e) { 
      console.warn(e);
      dojo.registerModulePath('com.ibm.bpm.wpd', '/WebViewer/widgets/com/ibm/bpm/wpd');
    }
    
    try{
      dojo["require"]("com.ibm.bpm.social_diagram"); //Alternate syntax used to avoid dojo build detection
    } catch (e) { 
      console.warn(e);
      dojo.registerModulePath('com.ibm.bpm.social', '/ProcessPortal/com/ibm/bpm/social');
    }
    */
    /** ** END Loading Process Diagram-specific modules *** */

    this._viewProcessDiagram_loaded(e);
  },

  _viewProcessDiagram_loaded: function(e){

	var tmpHistory = null;
	if (dojo.isIE < 9) {
	  	tmpHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	 	com.ibm.bpm.social.util.common.Util._portalHashHistory = new Array();
	}
	
    /**
     * This will not fire an "open diagram" action, but will close all necessary popups before opening the
     * diagram. *
     */
    dojo.publish(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM, [{
      "instanceId": this.instanceId
    }]);

    
    var pdd = new com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog({
      "instanceId": this.instanceId,
      "title": this.instanceData.processTemplateName,
      "subTitle": this.instanceData.name
    }, dojo.create("div"));
    pdd.show();

	if (tmpHistory) {
		var h = tmpHistory.pop();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = tmpHistory;
		tmpHistory = null;
		window.setTimeout(function() {
			com.ibm.bpm.social.util.common.Util.setCurrentHash(h);
		}, 250);
	}
	    
  },

  _onInstanceClick: function(e){
    if (this.sidebar) {
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        'instanceId': this.instanceId
      }]);
    }
  },

  _xhrErrors: function(xhrArgs, data){
    console.error(xhrArgs, data);
  },

  /**
   * Determine the base text direction dir attribute for a string based on user preference. Set the dir to
   * either rtl or ltr. Do nothing if pref is default.
   */
  _setDirAttribute: function(btdpref, str, node){
    if (btdpref != "default") {
      var dir = "";
      if (btdpref == "ltr" || btdpref == "rtl") {
        dir = btdpref;
      }
      else {
        dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection("", str);
      }
      if (dir != "") {
        node.dir = dir;
        //console.log("InstanceDetails _setDirAttribute str: "+str+", dir: "+dir+", node: "+node);
      }
    }
  },

  _setBidiLangDir: function(node){
    if (document.body.dir == "rtl") {
      node.dir = "rtl";
    }
  },

  destroy: function(){
    this._clearTaskDetailsItems();
    if (this.menu != null && this.menu.destroy) {
	  
      this.menu.destroy();
      this.menu = null;
    }

    if (this.taskDetailsListItems) {
        for(var i=0; i < this.taskDetailsListItems.length; i++) {
          this.taskDetailsListItems[i].destroy();
        }
        this.taskDetailsListItems = null;
    }
    this.inherited(arguments);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.stream.StreamInputBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.stream.StreamInputBox"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.StreamInputBox");





dojo.declare(
	"com.ibm.bpm.social.widgets.stream.StreamInputBox",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div>\n<div class='bpm-social-stream-editable-div' tabIndex=0 contentEditable dojoAttachPoint='postInput' dojoAttachEvent='onkeypress:_handlePostInputKeyPress, onblur: _handlePostInputBlur, onkeyup:_handlePostInputKeyUp' waiRole=\"textbox\"></div>\n<div class='bpm-social-stream-user-popup-div' style='display:none'>\n<div dojoAttachPoint='userSearchDiv' class='bpm-social-stream-user-popup'></div>\n<div class='bpm-social-stream-user-popup-arrow'></div>\n</div>\n</div>\n",
	userSearchList: undefined,
	userSearchConnections: null,
	userSearchValue: undefined,
	userSearchTimer: undefined,
	userSearchSelected: undefined,
	restClient: undefined,
	dropDownMaxSize: (dojo.isSafari && (navigator.userAgent.indexOf('iPad') != -1))? 5:10,
			
	constructor: function() {
		this.userSearchConnections = new Array();
	},
			
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.restClient = new com.ibm.bpm.social.util.ui.userchooser.util.RestClient();
		
		//Try to configure BTD here?
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.postInput);
	},
	
	getPostInput: function () {
		var text = '';
		var child = this.postInput.firstChild;
		while (child) {
			if ((child.nodeName.toLowerCase() == 'span') && dojo.hasClass(child,'bpm-user-tag')) {
				text = text + '<taggedUser>'+child.id+'</taggedUser>';
			} else if (child.textContent) {
				text = text + child.textContent;
			} else if (child.innerText) {
				text = text + child.innerText;
			} else if (child.innerHTML) {
				text = text + child.innerHTML;
			} else if (child.nodeValue) {
				text = text + child.nodeValue;
			}
			child = child.nextSibling;
		}
		return text;
	},

	clearInput: function(setFocus) {
		this._cancelUserSearch();
		this.postInput.innerHTML = '';
		if (setFocus) {
			this.postInput.focus();
		}
		
	},
	
	_getNameForUser: function(user) {
		var name = user.fullName;
		if (!name) {
			name = user.cn;
		}
		if (!name) {
			name = user.userName;
		}
		return name;
	},
	
	_getNodeText: function(node) {
		var text = '';
		var child = node.firstChild;
		while (child) {
			if ((child.nodeName.toLowerCase() == 'span') && dojo.hasClass(child,'bpm-user-tag')) {
				text = text + '<span id="'+child.id+'" class="bpm-user-tag">'+child.innerHTML+'</span>';
			} else if (child.nodeName.toLowerCase() == 'span') {
				text = text + this._getNodeText(child);
			} else if (child.textContent) {
				text = text + dojox.html.entities.encode(child.textContent);
			} else if (child.innerHTML){
				text = text + child.innerHTML;
			} else if (child.nodeValue) {
				text = text + dojox.html.entities.encode(child.nodeValue);
			}
			child = child.nextSibling;
		}
		return text;
	},
	
	_getPostInput: function() {
		return this._getNodeText(this.postInput);
	},
	
	tagUser: function(userName,fullName) {
		var span = document.createElement('span');
		span.id = userName;
		span.className = 'bpm-user-tag';
		span.contentEditable = 'false';
		span.innerHTML = fullName;
		this.postInput.appendChild(span);
		span = document.createElement('span');
		span.innerHTML = '&nbsp;'
		this.postInput.appendChild(span);
		if (window.getSelection) {
			window.getSelection().collapse(span,1);
		}
	},
	
	_selectUserDiv: function(userDiv) {
		if (this.userSearchSelected) {
			dojo.removeClass(this.userSearchSelected,'bpm-social-stream-user-selected');
		}
		this.userSearchSelected = userDiv;
		if (this.userSearchSelected) {
			dojo.addClass(this.userSearchSelected,'bpm-social-stream-user-selected');
    		var user = this.userSearchList[eval(this.userSearchSelected.id)];
    		var name = this._getNameForUser(user);
    		this._screenReaderReadText(name);
		}
	},
	
	_getIndexOfTagChar: function(txt) {
		// need to ignore any existing user tags in the string
		var tmp = txt;
		var tagStart = tmp.indexOf('<span');
		while (tagStart != -1) { 
			var tagStop = tmp.indexOf('</span>',tagStart);
			var prefix = tmp.substring(0,tagStart);
			while (prefix.length < tagStop) {
				prefix = prefix + 'x';
			}
			tmp = prefix + tmp.substring(tagStop);
			tagStart = tmp.indexOf('<span',tagStop);
		}
		return tmp.indexOf('@');
	},
	
	_handleEnterKey: function(e) {
		if (this.userSearchList) {
			if (this.userSearchSelected) {
				var user = this.userSearchList[eval(this.userSearchSelected.id)];
				var text = this._getPostInput();
				var index = this._getIndexOfTagChar(text);
				if (index != -1) {
					this.postInput.innerHTML = text.substring(0,index);
					this.tagUser(user.userName,this._getNameForUser(user));
					this._cancelUserSearch();
					dojo.stopEvent(e);
				}
			}
		}
	},
	
	_handlePostInputKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			this._handleEnterKey(e);
		}
		if (this.userSearchList) {
			if (e.keyCode == dojo.keys.UP_ARROW) {
				if (this.userSearchSelected.previousSibling) {
					this._selectUserDiv(this.userSearchSelected.previousSibling);
				}
				dojo.stopEvent(e);
			}
			if (e.keyCode == dojo.keys.DOWN_ARROW) {
				if (this.userSearchSelected.nextSibling) {
					this._selectUserDiv(this.userSearchSelected.nextSibling);
				}
				dojo.stopEvent(e);
			}
		}
		if (e.keyCode == dojo.keys.BACKSPACE) {
			var sel = null;
			if (window.getSelection) {
				sel = window.getSelection();
				if (sel && sel.focusNode && (sel.focusOffset == 0)) {
					var node = sel.focusNode;
					while (node && (node.parentNode != this.postInput)) {
						node = node.parentNode;
					}
					if (node && node.previousSibling && dojo.hasClass(node.previousSibling,'bpm-user-tag')) {
						var span = node.previousSibling;
						this.postInput.removeChild(span);
						delete span;
						dojo.stopEvent(e);
					} 
				} else if (sel && sel.focusNode && sel.focusNode.parentNode && dojo.hasClass(sel.focusNode.parentNode,'bpm-user-tag')) {
					var span = sel.focusNode.parentNode;
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					if (!dojo.isIE) {
						window.getSelection().collapse(span,0);
					}
					dojo.stopEvent(e);
				}
			} else if (document.selection) {
	            var range = document.selection.createRange();
	            if (dojo.hasClass(range.parentElement(),'bpm-user-tag')) {
					var span = range.parentElement();
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					dojo.stopEvent(e);
	            }
			}
		}
		if (e.keyCode == dojo.keys.DELETE) {
			if (window.getSelection) {
				var sel = window.getSelection();
				if (sel && sel.focusNode && (sel.focusNode.textContent != undefined) && (sel.focusOffset == sel.focusNode.textContent.length)) {
					var node = sel.focusNode;
					while (node && (node.parentNode != this.postInput)) {
						node = node.parentNode;
					}
					if (node && node.nextSibling && dojo.hasClass(node.nextSibling,'bpm-user-tag')) {
						var span = node.nextSibling;
						this.postInput.removeChild(span);
						delete span;
						dojo.stopEvent(e);
					} 
				} else if (sel && sel.focusNode && sel.focusNode.parentNode && dojo.hasClass(sel.focusNode.parentNode,'bpm-user-tag')) {
					var span = sel.focusNode.parentNode;
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					window.getSelection().collapse(span,0);
					dojo.stopEvent(e);
				}
			}
		}
		else if (e.charCode > 0) {
			setTimeout(dojo.hitch(this, "_startUserSearch", 10));
		}
		else if (document.selection) {
            var range = document.selection.createRange();
            if (dojo.hasClass(range.parentElement(),'bpm-user-tag')) {
				var span = range.parentElement();
				span.id = '';
				span.className = '';
				span.innerHTML = '';
				dojo.stopEvent(e);
            }
		}
	},
	
	_handlePostInputKeyUp: function(e) {
		if(e.keyCode == dojo.keys.BACKSPACE || e.keyCode == dojo.keys.CLEAR || e.keyCode == dojo.keys.DELETE) {
			this._startUserSearch();
		}
	},
	
	_startUserSearch: function() {
		var txt = this._getPostInput();
		this.userSearchIndex = this._getIndexOfTagChar(txt);
		if (this.userSearchIndex == -1) {
			this._cancelUserSearch();
		} else {
			var newUserSearchValue = txt.substring(this.userSearchIndex+1);
			if (this.userSearchValue != newUserSearchValue) {
				this._cancelUserSearch();
				this.userSearchValue = newUserSearchValue;
				this.userSearchTimer = window.setTimeout(dojo.hitch(this,'_doUserSearch'),1000);
			}
		}
	},
	
	_cancelUserSearch: function() {
		this.userSearchValue = undefined;
		if (this.userSearchTimer) {
			window.clearTimeout(this.userSearchTimer);
		}
		if (this.userSearchDiv) {
			this._hideUserSearchDiv();
		}
		if (this.userSearchConnections) {
			while(this.userSearchConnections.length) {
				dojo.disconnect(this.userSearchConnections.pop());
			}
		}
		this.userSearchList = undefined;
		this.userSearchSelected = undefined;
	},
	
	_highlightText: function(fullText,highlightText) {
		var f = fullText.toLowerCase();
		var h = highlightText.toLowerCase();
		var result = '';
		var start = 0;
		var index = f.indexOf(h);
		while (index != -1) {
			result = result + fullText.substring(start,index) +
					'<span class="bpm-social-stream-search-text">' + 
					fullText.substring(index,index+h.length) + '</span>';
			start = index + h.length;
			index = f.indexOf(h,start);
		}
		result = result + fullText.substring(start);
		return result;
	},
	
	_isInvalidWildcard: function(searchString) {
		//check for wildcard character in the middle of the search string
		var wildcard = searchString.indexOf('*');
		while (wildcard != -1) {
			if ((wildcard > 0) && (wildcard < (searchString.length-1))) {
				// we do not support wildcards in the middle of the string
				return true;
			} else {
				wildcard = searchString.indexOf('*',wildcard+1);
			}
		}
		return false;
	},
	
	_doUserSearch: function() {
		this.userSearchTimer = undefined;
		if (this.userSearchValue) {
			var tmp = this.userSearchValue;
			// convert % to * for consistency
		    var wildcard = tmp.indexOf('%');
		    while (wildcard != -1) {
		    	tmp = tmp.substring(0,wildcard)+'*'+tmp.substring(wildcard+1);
		        wildcard = tmp.indexOf('%',wildcard+1);
		    }
		    // check for invalid wildcard usage
		    if (this._isInvalidWildcard(tmp)) {
		    	// clear previous results before showing error
				while(this.userSearchConnections.length) {
					dojo.disconnect(this.userSearchConnections.pop());
				}
				this._selectUserDiv(null);
				while(this.userSearchDiv.firstChild) {
					var child = this.userSearchDiv.firstChild;
					this.userSearchDiv.removeChild(child);
					delete child;
				}
				// use message already in the user chooser
			    
			    var ucMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
				this.userSearchDiv.innerHTML = '<span class="bpm-social-stream-no-users-msg">'+ucMessages.invalidWildcardMessage+'</span>';
				this.userSearchSelected = undefined;
				this._showUserSearchDiv();
				return;
		    } 
			// check for mashups wild card setting
			var useWildCards = true;
			try {
				var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
				if (cs != null) {
				   var autoWildCard = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.AUTO_WILDCARD_DISABLED);
				   if (autoWildCard == true) 
				   {
					   useWildCards = false;
				   }
				}
			} catch (err) {
				// if error, then assume wildcards are ok
			}
		    if (useWildCards) {
				if (tmp.length > 2 && tmp.charAt(0) == '"' && tmp.charAt(tmp.length-1) == '"') {
					tmp = tmp.substring(1, tmp.length-1);
				}
				else {
					if (tmp.charAt(0) != '*') tmp = '*' + tmp;
					if (tmp.charAt(tmp.length-1) != '*') tmp = tmp + '*';
				}
		    }
		    
			var users = this.restClient.searchBPMByName(tmp, 0, this.dropDownMaxSize, false, true );
			while(this.userSearchConnections.length) {
				dojo.disconnect(this.userSearchConnections.pop());
			}
			this._selectUserDiv(null);
			while(this.userSearchDiv.firstChild) {
				var child = this.userSearchDiv.firstChild;
				this.userSearchDiv.removeChild(child);
				delete child;
			}
			if (users.length == 0) {
				this.userSearchDiv.innerHTML = '<span class="bpm-social-stream-no-users-msg">'+this.messages.noUsersFound+'</span>';
				this.userSearchSelected = undefined;
			} else {
				this.userSearchList = users;
				for (i=0; i<users.length; i++) {
					var name = this._getNameForUser(users[i]);
					var child = document.createElement('div');
					dojo.addClass(child,'bpm-social-stream-user');
					child.innerHTML = this._highlightText(name,this.userSearchValue) +
						'<br><span class="bpm-social-stream-user-name">'+this._highlightText(users[i].userName,this.userSearchValue)+'</span>';					
					child.id = ''+i;
					this.userSearchDiv.appendChild(child);
					if (this.userSearchDiv.firstChild == child) {
						this._selectUserDiv(child);
					}
					this.userSearchConnections.push(dojo.connect(child,'onmouseover',this,'_handleUserMouseOver'));
					this.userSearchConnections.push(dojo.connect(child,'onmousedown',this,'_handleUserMouseDown'));
				}
			}
			this._showUserSearchDiv();
		}
	},
	
	_getUserDiv: function(e) {
		var div = e.target;
		while (div && !dojo.hasClass(div,'bpm-social-stream-user')) {
			div = div.parentNode;
		}
		return div;
	},

	_handleUserMouseOver: function(e) {
		this._selectUserDiv(this._getUserDiv(e));
		if (this.userSearchSelected && !this.userSearchSelected.title && (this.userSearchSelected.scrollWidth > this.userSearchSelected.offsetWidth)) { 
    		var user = this.userSearchList[eval(this.userSearchSelected.id)];
    		var name = this._getNameForUser(user);
			this.userSearchSelected.title = name;
		}
	},

	_handleUserMouseDown: function(e) {
		window.setTimeout(dojo.hitch(this,'_setPostInputFocus'),100);
		this.userSearchSelected = this._getUserDiv(e);
		this._handleEnterKey(e);
	},
	
	_setPostInputFocus: function() {
		this.postInput.focus();
	},
	
	_handlePostInputBlur: function() {
		window.setTimeout(dojo.hitch(this,'_cancelUserSearch'),100);
	},
	
	_showUserSearchDiv: function() {
		var div = this.userSearchDiv.parentNode;
		div.style.display = 'block';
		if (!div.parentNode) {
			this.domNode.appendChild(div);
		}
		var coords = dojo.coords(div);
		div.parentNode.removeChild(div);
		div.style.width = this.domNode.offsetWidth + 'px';
		div.style.left = (coords.x + document.body.scrollLeft) + 'px';
		div.style.top = (coords.y + document.body.scrollTop) + 'px';
		div.style.position = 'absolute'; 
		document.body.appendChild(div);
	},
	
	_hideUserSearchDiv: function() {
		var div = this.userSearchDiv.parentNode;
		div.style.display = 'none';
		if (div.parentNode) {
			div.parentNode.removeChild(div);
		}
		div.style.left = '0px';
		div.style.top = '0px';
		div.style.position = 'relative'; 
		this.domNode.appendChild(div);
	},
	
	_screenReaderReadText: function(txt) {
		var div = dojo.byId('screenReaderDiv');
		if (!div) {
			div = document.createElement('div');
			div.id = 'screenReaderDiv';
			div.style.position = 'absolute';
			div.style.top = '-10px';
			div.style.width = '1px';
			div.style.height = '1px';
			div.style.overflow = 'hidden';
			dijit.setWaiRole(div,'note');
			dijit.setWaiState(div,'live','polite');
			document.body.appendChild(div);
		}
		div.innerHTML = '';
		div.innerHTML = txt;
	},

	destroy: function() {
	    try {
			this._cancelUserSearch();
		} catch (error) {
			//throwing error on log out
		}
		this.inherited('destroy',arguments);
	}
	
});


}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.stream.StreamEvent"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.stream.StreamEvent"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.StreamEvent");















dojo.declare("com.ibm.bpm.social.widgets.stream.StreamEvent", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                               dijit._Templated], {

  templateString: "",
  templateString:"<div class='bpm-social-stream-event'>\n<div class='bpm-social-stream-event-img-div'> \n <div dojoAttachPoint='avatarContent'> </div>\n</div>\n<div class='bpm-social-stream-event-data-div'>\n<div dojoAttachPoint=\"actor\" class='bpm-social-stream-event-data-actor'>\n</div>\n<div dojoAttachPoint=\"text\" dojoAttachEvent='onmouseover:_handleOnMouseOverText'>\n</div>\n<div class='bpm-social-subscript'>\n<a dojoAttachPoint='comment' href='javascript:void(0)' dojoAttachEvent='onclick:_handleComment' style='display:none' waiRole='button'></a>\n<span dojoAttachPoint='commentDivider' style='display:none'>&nbsp;|&nbsp;</span>\n<a dojoAttachPoint='showDiscussion' href='javascript:void(0)' dojoAttachEvent='onclick:_showDiscussion' waiRole='button'></a>\n<a dojoAttachPoint='hideDiscussion' href='javascript:void(0)' dojoAttachEvent='onclick:_hideDiscussion' waiRole='button'></a>\n<span class='bpm-social-event-discussion-icons' dojoAttachPoint='discussionIcons' dojoAttachEvent='onmouseover:_mouseOver, onmousemove:_mouseMove, onmouseout:_mouseOut'>&nbsp;</span>\n<span class='bpm-social-event-discussion-icons' dojoAttachPoint='discussionCount'></span>\n</div>\n<div dojoAttachPoint='comments' class='bpm-social-stream-event-comments' dojoAttachEvent='onmouseover:_handleOnMouseOverText'>\n<div dojoAttachPoint='newComment' class='bpm-social-stream-event-new-comment'>\n<div dojoAttachEvent='onkeypress:_handlePostInputKeyPress'>\n<div dojoAttachPoint=\"postInput\"></div>\n</div>\n<div class='bpm-social-event-new-comment-button-div'>\n<div class='bpm-social-button-left bpm-social-stream-event-new-comment-button'>\n<div class='bpm-social-button-right'> \n<button dojoAttachPoint='postButton' dojoAttachEvent='onclick:_handlePost' class='bpm-social-button-middle' waiRole='button'></button>\n</div>\n</div>\n</div>\n</div>\n</div>\n</div>\n<div class='bpm-social-stream-event-iequirks-border'>\n</div>\n</div>\n",
  events: null,
  overIcon: null,
  overIconTimer: null,
  overIconDelay: 1000,
  overIconTooltip: null,
  taskid: null,
  instanceid: null,
  //commentUrl: "/ProcessPortal/api/rest/task/{tid}/comment",
  commentUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/comment",
  auditTrailUrl: window.endpointURLObject.processPortalSupport +"/jsp/processAudit.do?bpdInstanceId=${iid}",
  streamInputBox: undefined,

  constructor: function(){
    this.events = new Array();
  },

  postCreate: function(){
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();

    dojo.addClass(this.domNode, 'bpm-social-stream-grouped-event-expanded');

    this.comment.innerHTML = this.messages.comment;
    this.showDiscussion.innerHTML = this.messages.showComments;
    this.hideDiscussion.innerHTML = this.messages.hideComments;
    this.postButton.innerHTML = this.messages.post;

    this.showDiscussion.style.display = 'none';
    this.hideDiscussion.style.display = 'none';
    this.comments.style.display = 'none';
    this.newComment.style.display = 'none';
    this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
    this._avatar=[];

    if (this.event) {
      this.events.push(this.event);
    }

    this._showActor();
    for ( var i = 0; i < this.events.length; i++) {
      this._showEvent(this.events[i]);
    }

    this.streamInputBox = new com.ibm.bpm.social.widgets.stream.StreamInputBox({}, this.postInput);
    dijit.setWaiState(this.streamInputBox.postInput, "label", this.messages.comment);

    this.text.tabIndex = '0';
    this.connect(this.text, 'onfocus', '_handleTextFocus');
  },

  _handleTextFocus: function(e){
    if (e.target == this.text) {
      this._screenReaderReadText(this.text.innerHTML);
    }
  },

  getActorId: function(e){
    return this.events[0].actor.id;
  },

  updateAvatarUrlForUserid: function(url, userid){
    // update actor avatar
    if (this.events[0].actor.id == userid) {
      if (!this.events[0].actor.image) {
        this.events[0].actor.image = new Object();
      }
      this.events[0].actor.image.url;

      /** Business Card integration * */
      if (this.events[0].actor.id != 'ibm.bpm.system') {
        var user = com.ibm.bpm.social.util.common.Util.getUser(userid);
        if (typeof (user["userID"]) != "undefined") {
          this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);
        }
      }
      /** End Business Card integration * */
    }

    // update discussion avatars
    var updateDiscussion = false;
    for ( var i = 0; i < this.events.length; i++) {
      var e = this.events[i];
      if (e.replies && e.replies.items && e.replies.items.length) {
        for ( var j = 0; j < e.replies.items.length; j++) {
          var r = e.replies.items[j];
          if (r.author && (r.author.id == userid)) {
            if (!r.author.image) {
              r.author.image = new Object();
            }
            //r.author.image.url = url;
            r.author.image.url = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(r.author.id);
            updateDiscussion = true;
          }
        }
      }
    }
    if (updateDiscussion) {
      this.updateComments();
    }
  },

  isEventCollapsable: function(e){
    var rc = false;
    // check if the same actor
    if (this.getActorId() == e.actor.id) {
      // check if file or link
      if (e.object && e.object.objectType && (e.object.objectType.toLowerCase() == 'file')) {
        // check if attached within 10 seconds of the previous event
        var lastDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[this.events.length - 1].published);
        var newDate = com.ibm.bpm.social.util.common.Util.getDateFromString(e.published);
        if ((newDate.getTime() - lastDate.getTime()) < 10000) {
          rc = true;
        }
      }
    }
    return rc;
  },

  addEvent: function(e){
    // add attachment event 
    this._showAttachmentWithLastEvent(e);
    this.events.push(e);
  },

  _showActor: function(){
    var url = dojo.moduleUrl("dojo", "resources/blank.gif");
    var alt = '';
    if (this.events[0].actor.objectType.toLowerCase() == 'service') {
      if (this.events[0].object.objectType.toLowerCase().indexOf('overdue') != -1) {
        this.avatar.setCSSClass('bpm-social-overdue-avatar');
      }
      else {
        this.avatar.setCSSClass('bpm-social-generic-system-avatar');
      }
    }
    else if (this.events[0].actor.objectType.toLowerCase() == 'person') {
      if (this.events[0].actor.image && this.events[0].actor.image.url) {
        url = this.events[0].actor.image.url;
      }
      else {
        this.avatar.setCSSClass('bpm-social-default-avatar-medium');
      }
      alt = this.events[0].actor.displayName;
    }
    else if (this.events[0].actor.objectType.toLowerCase() == 'group') {
      // no group avatar yet, so use default for now
      this.avatar.setCSSClass('bpm-social-default-avatar-medium');
    }

    this.actor.innerHTML = "";
    if (this.events[0].actor.displayName && (this.events[0].actor.id != 'ibm.bpm.system')) {
      this.actor.innerHTML =
        "<span class='bpm-social-user' userId='" + this.events[0].actor.id + "'>" + this.events[0].actor.displayName +
          "</span> ";

      /** Business Card integration * */
      var user = com.ibm.bpm.social.util.common.Util.getUser(this.events[0].actor.id);
      if (typeof (user["fullName"]) == "undefined" && typeof (this.events[0].actor.displayName) != "undefined") {
        user["fullName"] = this.events[0].actor.displayName;
      }

      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user", this.actor)[0], user);
        this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);
      }
      /** End Business Card integration * */
    }

  },

  _showAttachmentWithLastEvent: function(e){
    var text = e.content;

    var tmp = e.object.url;
    var i = text.indexOf(tmp);
    if (i == -1) {
      tmp = e.object.displayName;
      i = text.indexOf(tmp);
    }
    if (i != -1) {
      text =
        text.substring(0, i) + '<a href="' + e.object.url + '" target="_blank">' + tmp + '</a>' +
          text.substring(i + tmp.length);
    }

    var div = this.text.firstChild;
    div.innerHTML = div.innerHTML + '<br>' + text;
  },

  _showEvent: function(e){
    if (e.content) {
      //create the dom nodes
      var div = document.createElement('div');
      div.className = 'bpm-social-stream-event-data-text';
      var textNode = document.createElement("div");
      dojo.addClass(textNode, "bpm-social-stream-event-text-token");

      var text = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(e.content);
      text = text.replace(/<user/g, "<span class='bpm-social-user'");
      text = text.replace(/<\/user>/g, "</span>");

      //default to this text, however further code may override this with modified text
      textNode.innerHTML = text;

      //TODO Temporarily pre-pend user to beginning of comment
      //This will be updated in US-55939
      if (e.object && e.object.objectType.toLowerCase() == 'ibm.bpm.comment') {

        //Adjust here for mirroring make a user node if one is needed
        if (e.actor && e.actor.displayName) {
          var userNode = dojo.create("span", {
            //style: {
            //  "margin-right": "4px"
            //},
            innerHTML: e.actor.displayName + ":"
          });
          dojo.addClass(userNode,
            "bpm-social-user proper-noun bpm-social-stream-event-comment-author bpm-social-stream-event-text-token");
          //Append the user node and the text and apply btd separately for each
          div.appendChild(userNode);
          com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(userNode);
        }

		// Defect 132580
        textNode.innerHTML = dojo.string.substitute(
					this.messages["taggedProcessList-quotedText"], {
					'text': text
				});
        //For User Comments we need to apply base text direction to the content node
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(textNode);
      }

      var auditLink = null;
      if (e.object && e.object.objectType.toLowerCase() == 'ibm.bpm.task') {
        if (e.content.indexOf('completed the task') > -1) {
          //TODO:  Need a better way of signifying specific TASK & INSTANCE based status / actions
          var iid = this.instanceid;

          auditLink = dojo.create("a", {
            'onclick': dojo.hitch(this, function(){

              var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
                "instanceId": iid,
                "header": true
              });

              var ahDialog = new com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog({
                'instanceId': instanceData.piid,
                'templateName': instanceData.processTemplateName,
                'instanceName': instanceData.name,
                'proxyUrl': this.getProxyUrl()
              });
              ahDialog.show();

            }),
            'innerHTML': this.messages.showAudit,
            'style': 'cursor: pointer; margin-left: 10px;'
          });

        }
        //make a node to hold the comment text
        textNode.innerHTML = text;
      }

      if (e.object && e.object.objectType && (e.object.objectType.toLowerCase() == 'file')) {
        var tmp = e.object.url;
        var i = text.indexOf(tmp);
        if (i == -1) {
          tmp = e.object.displayName;
          i = text.indexOf(tmp);
        }
        if (i != -1) {
          text =
            text.substring(0, i) + '<a href="' + e.object.url + '" target="_blank">' + tmp + '</a>' +
              text.substring(i + tmp.length);
        }
        //make a node to hold the comment text, will be btd'ed
        textNode.innerHTML = text;
      }

      div.appendChild(textNode);

      //add a div to fix floating issue where dates creep up onto same line as comments
      var noFloatDiv = dojo.create("div");
      dojo.addClass(noFloatDiv, "nofloat");
      div.appendChild(noFloatDiv);

      var user = null;

      /** Business Card integration * */
      var properNouns = dojo.query(".proper-noun", div);
      for ( var x = 0; x < properNouns.length; x++) {

        //Handle BTD for each of the proper nouns (Apply UCC) 
        //Do this before BizCard wrap so we don't lose dojo connects
        this._applyUCCproperNoun(properNouns[x]);

        var objId = properNouns[x].getAttribute("objId");
        if (typeof (objId) != "undefined" && objId != null && objId != "") {
          user = com.ibm.bpm.social.util.common.Util.getUser(objId);
          if (typeof (user["userID"]) != "undefined") {
            com.ibm.bpm.social.widgets.common.BizCard.wrapElement(properNouns[x], user);
          }
        }
      }

      if (e.actor.id != "ibm.bpm.system") {
        user = com.ibm.bpm.social.util.common.Util.getUser(e.actor.id);
        if (typeof (user["fullName"]) == "undefined" && typeof (e.actor.displayName) != "undefined") {
          user["fullName"] = e.actor.displayName;
        }
        if (typeof (user["userID"]) != "undefined") {
          com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user.proper-noun", div)[0], user);
        }
      }

      var bpmUserTags = dojo.query(".bpm-user-tag", div);
      for ( var x = 0; x < bpmUserTags.length; x++) {
        var objId = bpmUserTags[x].getAttribute("objId");
        if (typeof (objId) != "undefined" && objId != null && objId != "") {
          user = com.ibm.bpm.social.util.common.Util.getUser(objId);
          if (typeof (user["userID"]) != "undefined") {
            com.ibm.bpm.social.widgets.common.BizCard.wrapElement(bpmUserTags[x], user);
          }
        }
      }
      /** End Business Card integration * */

      if (auditLink != null) {
        dojo.place(auditLink, div, "last");
      }

      if (e.published) {
        var timestamp = document.createElement('div');
        timestamp.className = 'bpm-social-stream-event-timestamp';
        timestamp.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(e.published);
        this.text.insertBefore(timestamp, this.text.firstChild);
      }
      this.text.insertBefore(div, this.text.firstChild);
    }
    if (e.replies && e.replies.items && e.replies.items.length) {
      //this._hideDiscussion(undefined,e);
      this._showDiscussion(undefined, e);
    }
    if (com.ibm.bpm.social.util.common.Util.isCommentSupported(e)) {
      this.comment.style.display = 'inline';
      dojo.addClass(this.domNode, 'bpm-social-stream-event-show-subscript');
    }
    var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
    if (e.target && (e.target.id == user.userName)) {
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.created') {
        dojo.addClass(this.domNode, 'bpm-social-help-request');
        this.comment.style.display = 'none';
      }
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.replied') {
        dojo.addClass(this.domNode, 'bpm-social-help-request-completed');
      }
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.ignored') {
        dojo.addClass(this.domNode, 'bpm-social-help-request-ignored');
      }
    }
    if (e.object.objectType.toLowerCase() == 'ibm.bpm.instance.overdue') {
      dojo.addClass(this.domNode, 'bpm-social-stream-event-overdue');
    }
  },

  _applyUCCproperNoun: function(node){
    //ensure span passed in
    if (typeof node == "undefined" || node == "" || node == null) {
      return;
    }

    //check if link inside of span (will happen if username bizcard wrapped)
    var html = node.innerHTML;
    if (html == "") {
      return;
    }

    var endElemInd = -1;
    var closeElemStartInd = -1;
    var isLink = false;
    var htmlText = html;

    if (html.indexOf("<a") != -1) {
      var endElemInd = html.indexOf(">");
      if (endElemInd == -1) {
        return;
      }
      var closeElemStartInd = html.indexOf("<", endElemInd);
      if (closeElemStartInd == -1 || closeElemStartInd < endElemInd) {
        return;
      }

      //determine the necessary direction (may differ based on user pref)
      isLink = true;
      htmlText = html.substring(endElemInd + 1, closeElemStartInd);
    }

    //Apply The UCC characters and fix the parenthesis to the text
    htmlText = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(htmlText);

    //Add the link html back, if there was any
    if (isLink) {
      htmlText = html.substring(0, endElemInd + 1) + htmlText + html.substring(closeElemStartInd, html.length + 1);
    }

    //Apply the updated HTML
    node.innerHTML = htmlText;
  },

  _showComment: function(event, i){
    var div = document.createElement('div');
    div.className = 'bpm-social-stream-event';
    var reply = event.replies.items[i];

    //Determine the btd and generate the html attribute string for the userid of the comment
    var userbtdstring = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(reply.author.displayName);
    if (userbtdstring != "default") {
      userbtdstring = "dir=\"" + userbtdstring + "\"";
    }
    else {
      userbtdstring = "";
    }

    //Determine the btd and generate the html attribute string for the content of the comment
    var replyContent = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(reply.content);
    var contbtdstring = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(replyContent);
    if (contbtdstring != "default") {
      contbtdstring = "dir=\"" + contbtdstring + "\"";
    }
    else {
      contbtdstring = "";
    }

    div.innerHTML =
      "<div class='bpm-social-stream-event-img-div'tabIndex='0'></div>" +
      "<div class='bpm-social-stream-event-data-div' tabIndex='0'>" +
         "<div class='bpm-social-stream-event-comment-text'>" +
           "<span class='bpm-social-user'" + userbtdstring + " userId='" + reply.author.id + "'>" + 
               reply.author.displayName +
           "</span><br><span " + contbtdstring + ">" + replyContent + "</span></div>" +
          "<span class='bpm-social-stream-event-timestamp'>" +
           com.ibm.bpm.social.util.common.Util.formatDate(reply.published) + 
        "</span></div>";
    var avatar= new com.ibm.bpm.social.util.ui.Avatar();
    this._avatar.push(avatar);
    div.firstChild.appendChild(avatar.domNode);
    this.comments.insertBefore(div, this.newComment);
    

    /** Business Card integration * */
    if (reply.author.id != "ibm.bpm.system") {
      var user = com.ibm.bpm.social.util.common.Util.getUser(reply.author.id);
      if (typeof (user["fullName"]) == "undefined" && typeof (reply.author.displayName) != "undefined") {
        user["fullName"] = reply.author.displayName;
      }

      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user", div)[0], user);
        avatar.setUser(user, "bpm-social-default-avatar-small", true);
      }
    }
    /** End Business Card integration * */
  },

  updateComments: function(){
    if (this.showDiscussion.style.display == 'none') {
      while (this.comments.firstChild != this.newComment) {
        var c = this.comments.firstChild;
        this.comments.removeChild(c);
        delete c;
      }
      this._showDiscussion();
    }
    else {
      while (this.discussionIcons.firstChild) {
        var c = this.discussionIcons.firstChild;
        this.discussionIcons.removeChild(c);
        delete c;
      }
      this._hideDiscussion();
    }
  },

  _showDiscussion: function(mouseEvent, e){
    this.commentDivider.style.display = 'inline';
    this.showDiscussion.style.display = 'none';
    this.hideDiscussion.style.display = 'inline';
    while (this.discussionIcons.firstChild) {
      var c = this.discussionIcons.firstChild;
      this.discussionIcons.removeChild(c);
      delete c;
    }
    this.discussionCount.innerHTML = "";
    for ( var i = 0; i < this.events.length; i++) {
      if (this.events[i].replies && this.events[i].replies.items && this.events[i].replies.items.length) {
        for ( var j = 0; j < this.events[i].replies.items.length; j++) {
          if ((e == undefined) || (e == this.events[i])) {
            this._showComment(this.events[i], j);
          }
        }
      }
    }
    this.comments.style.display = 'block';
  },

  _addDiscussionIcon: function(eventIndex, commentIndex){
    var event = this.events[eventIndex];
    var img = document.createElement('img');
    img.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_18x18.png");
    var author = this.events[eventIndex].replies.items[commentIndex].author;
    if (author && author.image && author.image.url) {
      img.src = author.image.url;
    }
    img.id = eventIndex + '_' + commentIndex + '_' + this.domNode.id;
    img.alt = "";
    img.style.width = '16px';
    img.style.height = '16px';
    this.discussionIcons.appendChild(img);

    /** Business Card integration * */
    if (author.id != "ibm.bpm.system") {
      img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(author.id);
      var user = com.ibm.bpm.social.util.common.Util.getUser(author.id);
      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
      }
    }
    /** End Business Card integration * */

  },

  _mouseOver: function(e){
    if (e.target.src && !this.overIcon) {
      this.overIcon = e.target;
      this.overIconTimer = window.setTimeout(dojo.hitch(this, '_showDiscussionIconTooltip'), this.overIconDelay);
    }
  },

  _mouseMove: function(e){
    if (this.overIconTimer) {
      window.clearTimeout(this.overIconTimer);
      this.overIconTimer = window.setTimeout(dojo.hitch(this, '_showDiscussionIconTooltip'), this.overIconDelay);
    }
  },

  _mouseOut: function(e){
    if (this.overIconTooltip) {
      this.overIconTooltip.parentNode.removeChild(this.overIconTooltip);
      delete this.overIconTooltip;
      this.overIconTooltip = null;
      dojo.removeClass(this.overIcon, 'bpm-social-event-comment-active');
    }
    if (this.overIconTimer) {
      window.clearTimeout(this.overIconTimer);
      this.overIconTimer = null;
    }
    this.overIcon = null;
  },

  _showDiscussionIconTooltip: function(){
    this.overIconTimer = null;
    this.overIconTooltip = document.createElement('div');
    this.overIconTooltip.className = 'bpm-social-stream-event-tooltip';
    var div = document.createElement('div');
    div.className = 'bpm-social-stream-event-img-div';
    this.overIconTooltip.appendChild(div);
    var img = document.createElement('img');
    img.className = 'bpm-social-stream-event-img';
    img.alt = '';
    img.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
    div.appendChild(img);
    div = document.createElement('div');
    div.className = 'bpm-social-stream-event-data-div';
    this.overIconTooltip.appendChild(div);
    var tmp = this.overIcon.id.split('_');
    var i = eval(tmp[0]);
    var j = eval(tmp[1]);
    var reply = this.events[i].replies.items[j];
    if (reply.author && reply.author.image && reply.author.image.url) {
      //img.src = reply.author.image.url;
      img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(reply.author.id);
    }
    var replyContent = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(reply.content);
    div.innerHTML =
      "<span class='bpm-social-user' userId='" + reply.author.id + "'>" + reply.author.displayName + "</span><br>" +
        replyContent + "<br>" + "<span class='bpm-social-stream-event-timestamp'>" +
        com.ibm.bpm.social.util.common.Util.getDateFromString(reply.published).toLocaleString() + "</span>";
    document.body.appendChild(this.overIconTooltip);
    var coords = dojo.coords(this.overIcon, true);
    var x = coords.x - 50;
    var y = coords.y + 24;
    this.overIconTooltip.style.left = '0px';
    this.overIconTooltip.style.top = y + 'px';
    this.overIconTooltip.style.visibility = 'hidden';
    document.body.appendChild(this.overIconTooltip);
    if ((x + this.overIconTooltip.offsetWidth + 5) > document.body.offsetWidth) {
      x = document.body.offsetWidth - this.overIconTooltip.offsetWidth - 5;
    }
    this.overIconTooltip.style.left = x + 'px';
    this.overIconTooltip.style.visibility = 'visible';
    dojo.addClass(this.overIcon, 'bpm-social-event-comment-active');
  },

  _hideDiscussion: function(mouseEvent, e){
    this.commentDivider.style.display = 'inline';
    this.showDiscussion.style.display = 'inline';
    this.hideDiscussion.style.display = 'none';
    while (this.comments.firstChild != this.newComment) {
      var c = this.comments.firstChild;
      this.comments.removeChild(c);
      delete c;
    }
    if (this.newComment.style.display == 'none') {
      this.comments.style.display = 'none';
    }
    var len = 0;
    for ( var i = 0; i < this.events.length; i++) {
      if (this.events[i].replies && this.events[i].replies.items) {
        for ( var j = 0; j < this.events[i].replies.items.length; j++) {
          if ((e == undefined) || (e == this.events[i])) {
            if (this.discussionIcons.children.length == 5) {
              this.discussionIcons.appendChild(document.createTextNode('...'));
            }
            if (this.discussionIcons.children.length < 5) {
              this._addDiscussionIcon(i, j);
            }
          }
          len++;
        }
      }
    }
    this.discussionCount.innerHTML = "(" + len + ")";
  },

  _handleComment: function(e){
    if (this.newComment.style.display == 'none') {
      this.comments.style.display = 'block';
      this.newComment.style.display = 'block';
      this.streamInputBox._setPostInputFocus();
    }
    else {
      this.newComment.style.display = 'none';
      if (this.hideDiscussion.style.display == 'none') {
        this.comments.style.display = 'none';
      }
    }
    if (e) {
    	dojo.stopEvent(e);
    }
  },

  _handleShowAudit: function(){
    this.audit.style.display = 'block';
    this.hideAuditButton.style.display = 'inline';
    this.showAuditButton.style.display = 'none';
    this.hideAuditButton.focus();
  },

  _handleHideAudit: function(){
    this.audit.style.display = 'none';
    this.hideAuditButton.style.display = 'none';
    this.showAuditButton.style.display = 'inline';
    this.showAuditButton.focus();
  },

  _handlePostInputKeyPress: function(e){
    if (e.keyCode == dojo.keys.ENTER) {
      this._handlePost();
      dojo.stopEvent(e);
    }
  },

  _getEventId: function(){
    var id = null;
    for ( var i = 0; (id == null) && (i < this.events.length); i++) {
      if (com.ibm.bpm.social.util.common.Util.isCommentSupported(this.events[i])) {
        id = this.events[i].object.id;
        if (id.indexOf('.') != -1) {
          id = id.substring(id.indexOf('.') + 1);
        }
      }
    }
    return id;
  },

  _getEventTargetName: function(){
    var name = null;
    for ( var i = 0; (name == null) && (i < this.events.length); i++) {
      if (com.ibm.bpm.social.util.common.Util.isCommentSupported(this.events[i])) {
        name = this.events[i].target.displayName;
        if (!name) {
          name = this.events[i].target.id;
        }
      }
    }
    return name;
  },

  _handlePost: function(){
    var txt = this.streamInputBox.getPostInput();
    if (txt) {
      var data = {
        message: txt
      };
      var id = this._getEventId();
      dojo.addClass(this.domNode, 'bpm-social-event-post-in-progress');

      this.socialRestHandler.postTaskStreamComment(id, dojo.toJson(data), dojo.hitch(this, function(data, xhrArgs){
        if (data.posted) {
          this.getEventUpdates();
        }
        else {
          dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
        }
      }), dojo.hitch(this, function(data, xhrArgs){
        dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
      }), true);

    }
    this.streamInputBox.clearInput();
    this._handleComment();
  },

  getEventUpdates: function(){
    dojo.publish("/com/ibm/bpm/social/updateStream/" + this.instanceid, [{}]);
    dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
  },

  updateEvent: function(event){
    for ( var j = 0; j < this.events.length; j++) {
      if (event.published && (event.published == this.events[j].published)) {
        if (this.events[j].object && (this.events[j].object.objectType == 'ibm.bpm.helprequest.created')) {
          if (event.object && (event.object.objectType != 'ibm.bpm.helprequest.created')) {
            this.events[j].object.objectType = event.object.objectType;
            dojo.removeClass(this.domNode, 'bpm-social-help-request');
            dojo.removeClass(this.domNode, 'bpm-social-help-request-reply-in-progress');
            this.comment.style.display = 'inline';
            if (event.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.replied') {
              dojo.addClass(this.domNode, 'bpm-social-help-request-completed');
            }
            if (event.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.ignored') {
              dojo.addClass(this.domNode, 'bpm-social-help-request-ignored');
            }
          }
        }
        if (event.replies && event.replies.items && event.replies.items.length) {
          var needsUpdate = true;
          if (this.events[j].replies && this.events[j].replies.items &&
            (this.events[j].replies.items.length == event.replies.items.length)) {
            needsUpdate = false;
          }
          if (needsUpdate) {
            this.events[j].replies = event.replies;
            this.updateComments();
          }
        }
      }
    }
  },

  _screenReaderReadText: function(txt){
    var div = dojo.byId('screenReaderDiv');
    if (!div) {
      div = document.createElement('div');
      div.id = 'screenReaderDiv';
      div.style.position = 'absolute';
      div.style.top = '-10px';
      div.style.width = '1px';
      div.style.height = '1px';
      div.style.overflow = 'hidden';
      dijit.setWaiRole(div, 'note');
      dijit.setWaiState(div, 'live', 'polite');
      document.body.appendChild(div);
    }
    div.innerHTML = '';
    div.innerHTML = txt;
  },

  _handleOnMouseOverText: function(e){
    var node = e.target;
    // check if the mouse is over a user tag
    if (node && dojo.hasClass(node, 'bpm-user-tag')) {
      // if the user name is clipped, provide hover help
      if (node.offsetWidth > node.parentNode.offsetWidth) {
        node.title = node.innerHTML;
      }
    }
  },

  destroy: function(){
    this._mouseOut();
    this.events = null;
    this.streamInputBox.destroy();
    if (this.avatar){
       this.avatar.destroy();
       this.avatar=null;
    }
    if (this._avatar){
      for (var i=0; i<this._avatar.length; i++){
        this._avatar[i].destroy();
      }
      this._avatar=null;
    }
    this.inherited('destroy', arguments);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.stream.Stream"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.stream.Stream"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.Stream");










dojo.declare(
	"com.ibm.bpm.social.widgets.stream.Stream",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-stream' dojoAttachEvent='onkeypress:_handleKeyPress' style=\"overflow-y:hidden; height:100%\">\n<div class='bpm-social-stream-title' dojoAttachPoint=\"streamTitle\"></div>\n<div class='bpm-social-popup-content' style=\"height:inherit\">\n<div class='bpm-social-stream-header bpm-social-stream-header-init' dojoAttachPoint='header'>\n<div class='bpm-social-stream-progress-indicator' dojoAttachPoint='progressIndicator'></div>\n<div class='bpm-social-stream-header-input-div' dojoAttachEvent='onkeypress:_handlePostInputKeyPress'>\n<input type='text' dojoAttachPoint='postInitial' dojoAttachEvent='onfocus:_handlePostInitialFocus' size=\"63\" waiRole=\"textbox\"/>\n<div dojoAttachPoint=\"postInput\"></div>\n</div>\n<div class='bpm-social-subscript'>\n<a href='javascript:void(0)' dojoAttachPoint='attachFile' dojoAttachEvent='onclick:_attachFile' class='bpm-social-stream-attach-file' waiRole=\"button\">\n\t<img dojoAttachPoint='attachFileIcon' class='bpm-social-attachment-icon' waiRole='button'/>\n\t<span dojoAttachPoint='attachFileSpan' waiRole='button'></span>\n</a>\n<a href='javascript:void(0)' dojoAttachPoint='attachLink' dojoAttachEvent='onclick:_attachLink' class='bpm-social-stream-attach-link' waiRole=\"button\">\n\t<img dojoAttachPoint='attachLinkIcon' class='bpm-social-attachment-icon' waiRole='button'/>\n\t<span dojoAttachPoint='attachLinkSpan' waiRole='button'></span>\n</a>\n<div id='bpm_social_button_left' class='bpm-social-button-left'>\n<div class='bpm-social-button-right'> \n<button dojoAttachPoint='postButton' dojoAttachEvent='onclick:_handlePost,onfocus:_setPostFocusStyle,onblur:_removePostFocusStyle' class='bpm-social-button-middle' waiRole='button'></button>\n</div>\n</div>\n</div>\n<div style='overflow:hidden'>\n</select>\n<div style='display:none' dojoAttachPoint='hiddenIFramesDiv'></div>\n<div dojoAttachPoint='attachFileDiv' style='display:none' class='bpm-social-stream-header-attach-file-div'>\n<form enctype=\"multipart/form-data\" action=\"/rest/bpm/wle/v1/process/\" method=\"POST\" target=\"bpmSocialFormSubmitFrame\" dojoAttachPoint=\"attachFileForm\" waiRole=\"form\" accept-charset=\"iso-8859-1\">\n<input type=\"hidden\" name=\"instanceId\" value=\"\" dojoAttachPoint=\"attachFileInstanceId\" />\n<input type=\"hidden\" name=\"name\" value=\"\" dojoAttachPoint=\"attachFileName\" />\n<input type=\"hidden\" name=\"docType\" value=\"file\" />\n<table cellSpacing=0 cellPadding=0 dojoAttachPoint=\"attachFileTable\" role=\"presentation\">\n<tbody>\n<tr>\n<td width=\"*\" noWrap=\"true\">\n<span class='bpm-social-stream-header-label' dojoAttachPoint='attachFileLabel'></span>\n</td>\n<td width=\"*\" noWrap=\"true\">\n<div class='bpm-social-file-input-field'>\n<div>\n<input type=\"text\" value=\"\" size=\"25\" dojoAttachPoint=\"attachFileText\" tabIndex=\"-1\" disabled=\"true\" waiRole=\"textbox\" />\n<div id=\"attach_file_button\" class='bpm-social-button-left' dojoAttachPoint=\"attachFileButton\">\n<div class='bpm-social-button-right'> \n<button dojoAttachPoint='browseButton' class=\"bpm-social-button-middle\" tabIndex=\"-1\"></button>\n</div>\n</div>\n</div>\n<input type=\"file\" value=\"\" name=\"data\" dojoAttachPoint=\"attachFileInput\" dojoAttachEvent='onchange:_fileInputChanged,onfocus:_setBrowseFocusStyle,onblur:_removeBrowseFocusStyle,onkeydown:_handleBrowseFileKeyPress' waiRole=\"textbox\" />\n</div>\n</td>\n<td width=\"1px\" noWrap=\"true\">\n<button dojoAttachPoint=\"cancelFileButton\" type=\"button\" dojoAttachEvent='onclick:_cancelFile' class='bpm-social-stream-attachment-cancel' waiRole='button'>&times;</button>\n</td>\n</tr>\n</tbody>\n</table>\n</form>\n</div>\n<div dojoAttachPoint='attachLinkDiv' style='display:none'  class='bpm-social-stream-header-attach-link-div'>\n<form enctype=\"multipart/form-data\" action=\"/rest/bpm/wle/v1/process/\" method=\"POST\" target=\"bpmSocialFormSubmitFrame\" dojoAttachPoint=\"attachLinkForm\" waiRole=\"form\">\n<input type=\"hidden\" name=\"instanceId\" value=\"\" dojoAttachPoint=\"attachLinkInstanceId\" />\n<input type=\"hidden\" name=\"name\" value=\"\" dojoAttachPoint=\"attachLinkName\" />\n<input type=\"hidden\" name=\"docType\" value=\"url\" />\n<table cellSpacing=0 cellPadding=0 dojoAttachPoint=\"attachLinkTable\" role=\"presentation\">\n<tbody>\n<tr>\n<td width=\"1px\" noWrap=\"true\">\n<span class='bpm-social-stream-header-label' dojoAttachPoint='attachLinkLabel'></span>\n<td width=\"*\" noWrap=\"true\">\n<input type=\"text\" value=\"\" size=\"50\" name=\"docUrl\" dojoAttachPoint=\"attachLinkInput\" dojoAttachEvent='onkeypress:_handleLinkKeyPress' waiRole=\"textbox\">\n</td>\n<td width=\"1px\" noWrap=\"true\">\n<button dojoAttachPoint=\"cancelLinkButton\" type=\"button\" dojoAttachEvent='onclick:_cancelLink' class='bpm-social-stream-attachment-cancel' waiRole='button'>&times;\n</button>\n</td>\n</tr>\n</tbody>\n</table>\n</form>\n</div>\n</div> \n</div>\n<div dojoAttachPoint='eventDiv' style=\"overflow-y:auto; height:inherit\">\n</div>\n</div>\n</div>\n",
	showSelection: 'all',	
	events: null,
	fetchsize: 50,
	more: null,
	moreConnect: null,
	userid: null,
	taskid: null,
	instanceid: null,
	lastStreamEvent: null,
	sub: null,
	timer: null,
	sidebar: false,
	//streamUrl: "/ProcessPortal/api/rest/task/{tid}/stream?preventDefault={preventCache}", //IBPM
	streamUrl: window.endpointURLObject.bpmRestJS +"/v1/social/instance/{iid}/stream?preventDefault={preventCache}", //IBPM
	commentUrl: window.endpointURLObject.bpmRestJS +"/v1/process/{iid}",
	streamInputBox: undefined,
    _userProfileChangeSubscribeHandle: null,
    _fixStreamHeightTimer: null,
	
	constructor: function() {
		this.progressCount = 0;
	},
			
    postCreate: function() {

    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    	this.postInitial.value = this.messages.post_ellipse;
    	this.postButton.innerHTML = this.messages.post;
    	this.attachFileSpan.innerHTML = this.messages.file;
    	this.attachFileIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    	this.attachFileIcon.alt = "";
    	this.attachLinkSpan.innerHTML = this.messages.link;
    	this.attachLinkIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    	this.attachLinkIcon.alt = "";
		this.attachFileLabel.innerHTML = this.messages.file_attach;
		this.attachLinkLabel.innerHTML = this.messages.link_attach;
		//this.cancelFileImg.alt = this.messages.cancelFile;
		//this.cancelLinkImg.alt = this.messages.cancelLink;
		this.browseButton.innerHTML = this.messages.browse;
		//this.cancelFileImg.src = dojo.moduleUrl('dojo','resources/blank.gif'); 
		//this.cancelLinkImg.src = dojo.moduleUrl('dojo','resources/blank.gif');
		this.streamTitle.innerHTML = this.messages.stream;
		
		dijit.setWaiState(this.postInitial, "label", this.messages.post);
		dijit.setWaiState(this.attachFileText, "label", this.messages.file_attach);
		dijit.setWaiState(this.attachFileInput, "label", this.messages.file_attach);
		dijit.setWaiState(this.attachLinkInput, "label", this.messages.link_attach);
		dijit.setWaiState(this.cancelFileButton, "label", this.messages.cancelFile);
		dijit.setWaiState(this.cancelLinkButton, "label", this.messages.cancelLink);
		
		this.events = new Array();

		if(this.sidebar){
			dojo.addClass(this.domNode, 'sidebar');
			//dojo.style(this.streamTitle, 'display', 'none');
		}
		
    	this.streamInputBox = new com.ibm.bpm.social.widgets.stream.StreamInputBox({},this.postInput);
    	dijit.setWaiState(this.streamInputBox.postInput, "label", this.messages.post);
    	
		if(this.instanceid!=null){
			this._clearEvents();
			this._getEvents();
			this.sub = dojo.subscribe("/com/ibm/bpm/social/updateStream/"+this.instanceid,dojo.hitch(this,'_getNewEvents'));
		}
	
		// subscribe to user profile changes
		if(this._userProfileChangeSubscribeHandle == null){
        	dojo.subscribe(com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, dojo
          	.hitch(this, this._handleUserProfileChange));
        }

		// do not show the "Attachment" option for iPad
		if (dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
			this.attachFile.style.display = 'none';
		}
		
		// use timer to adjust the stream height to fill available space
		this._fixStreamHeightTimer = setInterval(dojo.hitch(this, '_fixStreamHeight'), 250);
		
		//ensure that the file attachement form is always ltr aligned.
		this.attachFileText.dir = "ltr";
	},
	
	_fixStreamHeight: function() {
		if (this.eventDiv.parentNode.offsetHeight) {
			var h = this.eventDiv.parentNode.offsetHeight - this.header.offsetHeight;
			if (this.streamTitle.offsetHeight) {
				h = h - this.header.offsetTop;
			}
			if (h < 0) h = 0;
			this.eventDiv.style.height = h + 'px';
		}
	},
	
	_createHiddenIFrame: function() {
		var name = 'frame'+(new Date().getTime());
		var iframe = document.createElement('iframe');
		iframe.name=name;
		iframe.style.display="none";
		iframe.title=name;
		iframe.id=name;
		this.hiddenIFramesDiv.appendChild(iframe);
		var fn = dojo.hitch(this,function() {
			if (iframe.contentWindow.location.href != 'about:blank') {
				this._getNewEvents();
				this._hideProgressIndicator();
				if (iframe.attachEvent) {
					iframe.detachEvent('onload',fn);
				} else {
					iframe.removeEventListener('onload',fn,false);
				}
				if (iframe.contentDocument &&
					iframe.contentDocument.documentElement &&
					(iframe.contentDocument.documentElement.textContent || 
					iframe.contentDocument.documentElement.innerText)) {
					var txt = iframe.contentDocument.documentElement.textContent;
					if (!txt) {
						txt = iframe.contentDocument.documentElement.innerText;
					}
					var result = dojo.fromJson(txt);
					if (result && result.status) {
						if (result.status == "error") {
							// Display error!
							
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(result.Data.errorMessage);
						}
					}
				}
				this.hiddenIFramesDiv.removeChild(iframe);
				delete iframe;
			}
		});
		if (iframe.attachEvent) {
			iframe.attachEvent('onload',fn);
		} else {
			iframe.addEventListener('load',fn,false);
		}
		return iframe;
	},
	
	_handleUserProfileChange: function(obj){
		var currUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile().userName;
		var avatarURL = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(currUser);
		var se = null;
		for (var i=0; i<this.events.length; i++) {
			var e = this.events[i];
			if (e.streamEventObject && (e.streamEventObject != se)) {
				se = e.streamEventObject;
				se.updateAvatarUrlForUserid(avatarURL,currUser);
			}
		}
	},

	refresh: function(obj){
		if(! (obj.instanceid && obj.taskid) ){
			return;
		}
		
		dojo.unsubscribe(this.sub);
		
		this.instanceid = obj.instanceid;
		
		this.taskid = obj.taskid;
		
		this._clearEvents();
		
		this._getEvents();
		
		this.streamInputBox.clearInput(false);
		this._toggleHeaderIfCommentsProhibited();
		this._cancelFile();
		this._cancelLink();
		
		this.sub = dojo.subscribe("/com/ibm/bpm/social/updateStream/"+this.instanceid,dojo.hitch(this,'_getNewEvents'));
	},
	
	_toggleHeaderIfCommentsProhibited: function() {
		if(this.instanceid != null) {
			this._restHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
			this._restHandler.getActionsForProcess(this.instanceid, dojo.hitch(this, function(result) {
				if(result && result.standardActions && result.standardActions.indexOf("ACTION_ADD_COMMENT") >= 0) {
					this._showPostCommentHeader();
				}
				else if(result && result.standardActions) {
					this._hidePostCommentHeader();
				}
			}),function(){}, true, null, null);
		}
	},
	
	_showPostCommentHeader: function() {
		dojo.style(this.header, "display", "");
	},
	
	_hidePostCommentHeader: function() {
		dojo.style(this.header, "display", "none");
	},
	
	_showProgressIndicator: function() {
		this.progressCount++;
		this.progressIndicator.style.display = 'block';	
	},
	
	_hideProgressIndicator: function() {
		if (this.progressCount > 0) {
			this.progressCount--;
			if (this.progressCount == 0) {
				this.progressIndicator.style.display = 'none';
			}
		}
	},
	
	_clearEvents: function() {
		//destroy the events
		for(var i=0; i<this.events.length; i++) {
			if(this.events[i] && this.events[i].streamEventObject) {
				this.events[i].streamEventObject.destroy();
			}
		}
		//destroy the array
		delete this.events;
		this.events = new Array();
		if (this.more) {
			dojo.disconnect(this.moreConnect);
			this.more = null;
		}
		while (this.eventDiv.firstChild) {
			var c = this.eventDiv.firstChild;
			var se = dijit.byId(c.id);
			if (se && se.destroy) {
				se.destroy();
			} else {
				this.eventDiv.removeChild(c);
				delete c;
			}
		}
		this.lastStreamEvent = null;
	},
	
	_fixActorAvatar: function(actor) {
		if (actor && actor.objectType) {
			if ((actor.objectType.toLowerCase() == 'person') && actor.image) {
				actor.image.url = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(actor.id);
			}
		}
	},
	
	_fixAvatars: function(data) {
		for (var i=0; i<data.items.length; i++) {
			var event = data.items[i];
			this._fixActorAvatar(event.actor);
			if (event.replies && event.replies.items) {
				for (var j=0; j<event.replies.items.length; j++) {
					this._fixActorAvatar(event.replies.items[j].author);
				}
			}
		}
	},
	
	_getNewEvents: function() {
		if (this.timer) {
			window.clearTimeout(this.timer);
			this.timer = null;
			this._showProgressIndicator();
			var url = this.getProxyUrl() + this.streamUrl.replace("{iid}", this.instanceid).replace("{preventCache}",(new Date().getTime()) );
			if (this.events.length > 0) {
				url = url + '&since=' + this.events[this.events.length-1].published;
			}
			// read data
			dojo.xhrGet({
				url: url,
				handleAs: "json",
				headers: {
					"Content-Type": "application/json", 
					"Accept":"application/json"
				},
				load: dojo.hitch(this,'_loadNewEvents'), 
				error: dojo.hitch(this,'_errorEvents')
			});
		}
	},
	
	_getEvents: function(last) {
		if (this.taskid) {
			// read data
			this._showProgressIndicator();
			
			this.socialRestHandler.getStreamEvents(this.instanceid, 1, this.fetchsize, 
				dojo.hitch(this, '_loadEvents'), dojo.hitch(this, '_errorEvents'), true);
		}
	},
	
	_loadEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			for (var i=0; i<data.items.length; i++) {
				events[i] = data.items[i];
			}
		}
		var len = events.length;
		this._addEvents(events);
		if (len == this.fetchsize) {
			this.more = document.createElement('div');
			this.more.className = 'bpm-social-stream-event-data-div bpm-social-stream-more';
			this.eventDiv.appendChild(this.more);
			var a = document.createElement('a');
			a.href = 'javascript:void(0)';
			this.more.appendChild(a);
			a.innerHTML = this.messages.more;
			this.moreConnect = dojo.connect(a,'onclick',this,'_getMoreEvents');
		}
	},
	
	_loadMoreEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			for (var i=0; i<data.items.length; i++) {
				events[i] = data.items[i];
			}
		}
		var len = events.length;
		if (len) {
			var testDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[0].published);
			if (testDate) {
				var testTime = testDate.getTime();
				var d = com.ibm.bpm.social.util.common.Util.getDateFromString(events[0].published);
				while (d && (d.getTime() >= testTime)) {
					events.splice(0,1);
					if (events.length > 0) {
						d = com.ibm.bpm.social.util.common.Util.getDateFromString(events[0].published);
					} else {
						d = null;
					}
				}
			} 	
		}
		this._addEvents(events,true);
		if (len == this.fetchsize) {
			this.more = document.createElement('div');
			this.more.className = 'bpm-social-stream-event-data-div bpm-social-stream-more';
			this.eventDiv.appendChild(this.more);
			var a = document.createElement('a');
			a.href = 'javascript:void(0)';
			this.more.appendChild(a);
			a.innerHTML = this.messages.more;
			this.moreConnect = dojo.connect(a,'onclick',this,'_getMoreEvents');
		}
	},
	
	_checkForEventUpdate: function(event) {
		var found = false;
		for (var i=0; (i<this.events.length) && !found; i++) {
			// use timestamp to find match (may need better logic)
			if (this.events[i].published == event.published) {
				var se = this.events[i].streamEventObject;
				if (se) {
					se.updateEvent(event);
				}
			}
		}
	},
	
	_loadNewEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			var lastDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[this.events.length-1].published);
			for (var i=0; i<data.items.length; i++) {
				if (data.items[i].published) {
					var testDate = com.ibm.bpm.social.util.common.Util.getDateFromString(data.items[i].published);
					if (lastDate && testDate && (lastDate.getTime() < testDate.getTime())) {				
						events[events.length] = data.items[i];
					} else {
						this._checkForEventUpdate(data.items[i]);
					}
				}
			}
		}
		this._addEvents(events);
	},
	
	_errorEvents: function(data,xhrArgs) {
		// TODO: how do we show errors?
		this._hideProgressIndicator();
	},
	
	_isEventCollapsable: function(e, se) {
		var collapse = false;
		if (se) {
			collapse = se.isEventCollapsable(e);
		}
		return collapse;
	},
	
	_addEvents: function(events,more) {
		var size = events.length;
		var se = this.lastStreamEvent;
		var moreindex = 0;
		if (more) {
			se = null;
		}
		while (events.length > 0) {
			var e = events.pop();
			if (this._isEventCollapsable(e,se)) {
				se.addEvent(e);
			} else {
				var node = this.eventDiv.firstChild;
				if (more) {
					node = null;
					if (se) {
						node = se.domNode;
					}
				} 
				se = new com.ibm.bpm.social.widgets.stream.StreamEvent({event:e, taskid:this.taskid, instanceid:this.instanceid});
				this.eventDiv.insertBefore(se.domNode,node);
			}
			
			if(e.actor && e.actor.id && e.actor.displayName && e.actor.image && e.actor.image.url){
				com.ibm.bpm.social.util.common.Util.primeUserCache(e.actor.id, e.actor.displayName, e.actor.image.url);
			}
			
			e.streamEventObject = se;
			if (more) {
				this.events.splice(moreindex,0,e);
				moreindex++;
			} else {
				this.events.push(e);
			}
		}
		if (more) {
			// if the event immediately after the last inserted event
			// can be collapsed, we need to remove the old markup and
			// collpase the events together
			if (this._isEventCollapsable(this.events[moreindex],se)) {
				var se2 = this.events[moreindex].streamEventObject;
				var tmp = se2.events;
				se2.destroy();
				while (tmp.length > 0) {
					var e = tmp.pop();
					se.addEvent(e);
					e.streamEventObject = se;
				}
			}
		}
		this._hideProgressIndicator();
		if (!more) {
			this.lastStreamEvent = se;
			this.timer = window.setTimeout(dojo.hitch(this,'_getNewEvents'),300000);
		}
	},
	
	_getMoreEvents: function() {
		dojo.disconnect(this.moreConnect);
		this.eventDiv.removeChild(this.more);
		delete this.more;
		this.more = null;
		var page = Math.floor(this.events.length/this.fetchsize) + 1;
		this._showProgressIndicator();
		
		this.socialRestHandler.getStreamEvents(this.instanceid, page, this.fetchsize,
			dojo.hitch(this, '_loadMoreEvents'), dojo.hitch(this, '_errorEvents'));
	},
	
	
	_fixAttachFileSize: function() {
		this.attachFileTable.style.width = this.header.offsetWidth + 'px';
		this.attachFileText.size = 1;
		this.attachFileInput.size = 30;
		var size = 30;
		if (dojo.isChrome) {
			var inputSize = this.header.offsetWidth;
			this.attachFileInput.style.width = inputSize+'px'; 
			while ((inputSize > 100) && (this.attachFileTable.offsetWidth > this.header.offsetWidth)) {
				inputSize = inputSize - 5;
				this.attachFileInput.style.width = inputSize+'px'; 
			}
		} else {
			while ((size > 5) && (this.attachFileTable.offsetWidth > this.header.offsetWidth)) {
				size--;
				this.attachFileInput.size = size; 
			}
		}
		this.attachFileText.size = size;
		while ((size > 5) && ((this.attachFileText.offsetWidth + this.attachFileButton.offsetWidth + 5) > this.attachFileInput.offsetWidth)) {
			size--;
			this.attachFileText.size = size; 
		}
		if (dojo.hasClass(document.documentElement,'dijitRtl')) {
			dojo.attr(this.attachFileDiv, 'dir', 'ltr');
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.browseButton);
		} else {
			if (dojo.isIE) {
				this.attachFileButton.style.left = this.attachFileText.offsetWidth + 'px';
			} else {
				this.attachFileButton.style.right = '-' +  (this.attachFileButton.offsetWidth+5) + 'px';
			}
		}
	},
	
	_fixAttachLinkSize: function() {
		this.attachLinkTable.style.width = this.header.offsetWidth + 'px';
		this.attachLinkInput.size = 40;
		var size = 40;
		while (this.attachLinkTable.offsetWidth > this.header.offsetWidth) {
			size--;
			this.attachLinkInput.size = size; 
		}
	},
	
	_attachFile: function() {
		this.attachFileDiv.style.display = 'block';
		if (!dojo.isSafari)	this.attachFileInput.style.width = "auto";
		this._fixAttachFileSize();
		this.attachFileText.style.width = "auto";
		this.attachFile.disabled = true;
		dojo.addClass(this.attachFile,'bpm-social-link-disabled');
		if (this.attachLinkDiv.style.display == 'block') {
			this._fixAttachLinkSize();
		}
		window.setTimeout(dojo.hitch(this, function(){
			this.attachFileInput.focus();
			this._fixAttachFileSize();
		}),100);
	},
	
	
	_attachLink: function() {
		this.attachLinkDiv.style.display = 'block';
		this.attachLinkInput.style.width = "auto";
		this._fixAttachLinkSize();
		this.attachLink.disabled = true;
		dojo.addClass(this.attachLink,'bpm-social-link-disabled');
		if (this.attachFileDiv.style.display == 'block') {
			this._fixAttachFileSize();
		}
		window.setTimeout(dojo.hitch(this, function(){this.attachLinkInput.focus();}),100);
	},
	
	_handlePost: function() {
		var txt = this.streamInputBox.getPostInput();
		if (txt) {
			this._showProgressIndicator();
			
			this.processRestHandler.postInstanceStreamComment(this.instanceid, txt,
				dojo.hitch(this, function(data, xhrArgs) {
					if (data.status == '200') {
						this._getNewEvents();
						this.streamInputBox.clearInput(true);
					} else {
						//TODO: error handling
					}
					this._hideProgressIndicator();
				}), dojo.hitch(this, function(data, xhrArgs) {
					this._hideProgressIndicator();
				}), false);  /*false means XHR is synchronous; this fixes problem where stream text and attached file or URL might end up in different stream posts.*/
		}
		if (this.attachFileInput.value && this.attachFileText.value) {
			this._showProgressIndicator();
			name = this.attachFileInput.value;
			name = name.substring(name.lastIndexOf('/')+1);
			name = name.substring(name.lastIndexOf('\\')+1);
			this.attachFileName.value = name;
			this.attachFileForm.action = window.endpointURLObject.bpmRestJS +'/v1/process/'+this.instanceid+'?action=addDocument&docType=file&parts=all&accept=application/json&override-content-type=text/plain&name='+encodeURIComponent(name);
			this.attachFileInstanceId.value = this.instanceid;
			this.attachFileForm.target = this._createHiddenIFrame().name;
			this.attachFileForm.submit();
			this.attachFileText.value = '';
			if (this.attachFileInput.form.reset) {
				try {
					this.attachFileInput.form.reset();
				} catch(e) {
				}
			}
		}
		if (this.attachLinkInput.value) {
			this._showProgressIndicator();
			name = this.attachLinkInput.value;
			if (name.indexOf('://') == -1) {
				this.attachLinkInput.value = 'http://'+name;
				name = this.attachLinkInput.value;
			}
			if (name.length > 50) {
				name = name.substring(0,47)+"...";
			}
			this.attachLinkName.value = name;
			this.attachLinkForm.action = window.endpointURLObject.bpmRestJS +'/v1/process/'+this.instanceid+'?action=addDocument&docType=url&parts=all&accept=application/json&override-content-type=text/plain&name='+encodeURIComponent(name)+'&docUrl='+encodeURIComponent(this.attachLinkInput.value);
			this.attachLinkInstanceId.value = this.instanceid;
			this.attachLinkForm.target = this._createHiddenIFrame().name;
			this.attachLinkForm.submit();
			this.attachLinkInput.value = '';
		}
		this.attachFileDiv.style.display = 'none';

		this.attachFile.disabled = false;
		dojo.removeClass(this.attachFile,'bpm-social-link-disabled');
		this.attachLinkDiv.style.display = 'none';
		this.attachLink.disabled = false;
		dojo.removeClass(this.attachLink,'bpm-social-link-disabled');
	},
	
	_cancelFile: function(e) {
		this.attachFileText.value = '';
		// this.attachFileInput.form.reset(); used to be called here, but it blew up on IE9 -totally crashed the browser.
		this.attachFileDiv.style.display = 'none';
		this.attachFile.disabled = false;
		dojo.removeClass(this.attachFile,'bpm-social-link-disabled');
		if (e) {
			dojo.stopEvent(e);
		}
		return false;
	},
	
	_cancelLink: function(e) {
		this.attachLinkInput.value = '';
		this.attachLinkDiv.style.display = 'none';
		this.attachLink.disabled = false;
		dojo.removeClass(this.attachLink,'bpm-social-link-disabled');
		if (e) {
			dojo.stopEvent(e);
		}
		return false;
	},
	
	_handlePostInputKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			this._handlePost();
			dojo.stopEvent(e);
		}
	},
	
	_handleLinkKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			dojo.stopEvent(e);
		}
	},

	_handleBrowseFileKeyPress: function(e) {
		// In IE, enter key tries to submit the form by default, so let's prevent this
		// (Space bar opens the file browser)
		if (dojo.isIE) {
			if (e.keyCode == dojo.keys.ENTER) {
				if(e.preventDefault) {
					// IE 9
					e.preventDefault();
				} else {
					// IE < 9
					e.cancelBubble = true;
					e.returnValue = false;
				}
			}			
		}
	},
	
	_handleKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ESCAPE) {
			// Accessibility 
			// If user presses escape, change focus out to main content
			try { // prevent default behavior - in Firefox, ESC stops the page
				e.preventDefault(); // Non-IE
			} catch(x) {
				e.returnValue = false; // IE
			}
			
			if (dojo.style(dojo.query(".workViewCompletion")[0], "display") === "none") {
				var node = dojo.query(".detailsContent .instanceName")[0];
				dojo.attr(node, "tabindex", "-1");
				var blur = dojo.connect(node, "onblur", function() {
					dojo.attr(this, "tabindex", "0");
					dojo.disconnect(this, blur);
				});
				node.focus();
			} else {
				dojo.query(".processPortalSelectedPage .bpm-task-header-subject .bpm-social-drop-down")[0].focus();
			}
		}
	},
	
	_handlePostInitialFocus: function() {
		dojo.removeClass(this.header,'bpm-social-stream-header-init');
		this.streamInputBox.clearInput(true);
	},
	
	_fileInputChanged: function() {
		// PMR 138549 - this will be called via dojo when the onchange event is fired; and also when the input gets focus - see _fileInputOnFocus() below.
		// This covers browsers that don't fire onchange when the same file is picked twice in a row in the file dialog.
		name = this.attachFileInput.value;
		name = name.substring(name.lastIndexOf('/')+1);
		name = name.substring(name.lastIndexOf('\\')+1);
		this.attachFileText.value = name;
	},
	
	_fileInputOnFocus: function() {
		// PMR 138549 - using onfocus enables us to know when the file dialog is closed.
		this._fileInputChanged();
	},
	
	tagUser: function(userName,fullName) {
		this.streamInputBox.clearInput();
		this.streamInputBox.tagUser(userName,fullName);
	},
	
    _setPostFocusStyle: function(e) {
         dojo.addClass("bpm_social_button_left","bpm-social-button-focus");

    },

    _removePostFocusStyle: function(e) {
         dojo.removeClass("bpm_social_button_left","bpm-social-button-focus");
    },

    _setBrowseFocusStyle: function(e) {
         dojo.addClass("attach_file_button","bpm-social-button-focus");
		this._fileInputOnFocus();

    },

    _removeBrowseFocusStyle: function(e) {
         dojo.removeClass("attach_file_button","bpm-social-button-focus");
    },

	destroy: function() {
		this.streamInputBox.destroy();
		if (this._fixStreamHeightTimer) {
			window.clearInterval(this._fixStreamHeightTimer);
			this._fixStreamHeightTimer = null;
		}
		if (this.timer) {
			window.clearTimeout(this.timer);
			this.timer = null;
		}
		if (this.sub) {
			dojo.unsubscribe(this.sub);
			this.sub = null;
		}
		this._clearEvents();
		if (this.more) {
			dojo.disconnect(this.moreConnect);
			this.more = null;
		}
		if (this._userProfileChangeSubscribeHandle) {
			dojo.unsubscribe(this._userProfileChangeSubscribeHandle);
			this._userProfileChangeSubscribeHandle = null;
		}
		this.inherited('destroy',arguments);
	}
	
});


}

if(!dojo._hasResource["com.ibm.bpm.social.portal.views.InstanceDetailsView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.views.InstanceDetailsView"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.InstanceDetailsView");




















dojo.declare(
	"com.ibm.bpm.social.portal.views.InstanceDetailsView",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: dojo.cache("com.ibm.bpm.social.portal.views", "templates/InstanceDetailsView.html", "<div dojoAttachPoint=\"container\" class=\"workView workViewInstance\">\n\t<div class=\"sidebarContent streamContent\" dojoAttachPoint=\"streamContent\"></div>\n\t<div class=\"mainContent detailsContent\" dojoAttachPoint=\"detailsContent\"></div>\n</div>\n"),
	
	CLASS_NAME: "InstanceDetailsView",
	
	LOG_LEVEL: 1,
    
    active: false,
    
    sidebarWidth: 0,
    
    instanceId: 0,
    
    postCreate: function(arguments) {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		dojo.create("div", {
			'id': 'details_content_widget_target',
			'innerHTML': 'Details Container - Please clobber me with a widget!'
		}, this.detailsContent);

		dojo.create("div", {
			'id': 'details_stream_widget_target',
			'innerHTML': 'Stream Container - Please clobber me with a widget!'
		}, this.streamContent);
			
		this.initializeContentWidgets();
	},

	initializeContentWidgets: function(){
		var FUNC_NAME = "initializeContentWidgets";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		//*** Content teams place widget initialization here ***//
		this.details = new com.ibm.bpm.social.widgets.details.InstanceDetails({}, 'details_content_widget_target');
		
		this.stream = new com.ibm.bpm.social.widgets.stream.Stream({}, 'details_stream_widget_target');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	load: function(arguments){
		var FUNC_NAME = "load";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		//*** Called from the WorkViewContainer every time immediately before the view is displayed ***//

		if(! (arguments.instanceId) ){
			return;
		}
		
		this.active = true;
		this.instanceId = arguments.instanceId;
		
		// disconnect from instance updates
		if (this.instanceUpdateHandle) {
			dojo.unsubscribe(this.instanceUpdateHandle);
			this.instanceUpdateHandle = null;
		}

		//TODO Need some way to invoke stream via InstanceId
		var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
			'header': true, 
			'instanceId': this.instanceId,
			'skipCache': true
		});
		
		if (typeof instanceData.errorInfo === "object") {
			//
			// Error occured
			//
			var errorInfo = instanceData.errorInfo;
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error occurred gettting instanceData", this.CLASS_NAME, FUNC_NAME);
			this._showErrorMessage(errorInfo);
		}else{
	  
			//TODO get the latest task ID in the list and use. only temporary until Stream service is instance-based
			var task = null;
			if(instanceData.tasks.length>0){
				task = instanceData.tasks[instanceData.tasks.length-1];
				com.ibm.bpm.social.util.common.Util.currentTaskData = task;
			}
			var tid = -1;
			if(task!=null) {
				tid = dojo.number.parse(task.tkiid);
			}
			
			
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Refreshing details...", this.CLASS_NAME, FUNC_NAME);
			this.details.refresh({
				instanceId: this.instanceId
			});
			
			
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Refreshing stream...", this.CLASS_NAME, FUNC_NAME);
			this.stream.refresh({
				taskid:tid,
			    instanceid:this.instanceId
			});

			// connect to instance updates
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Subscribing to instance due date updates...", this.CLASS_NAME, FUNC_NAME);
			this.instanceUpdateHandle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED + "." + this.instanceId, dojo.hitch(this,
				function(tkiid) {
					if(this.active) {
						this.details.refresh({
							instanceId: this.instanceId
						});
						this.stream.refresh({
							taskid: tkiid,
							instanceid: this.instanceId
						});
					}
				}, tid));
		
			var newHash = 'dashboards/TWP/Process+Performance?tw.local.selectedInstanceId='+this.instanceId;
			com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
		}
		
		dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{"viewType":"InstanceDetailsView"}]);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	 _showErrorMessage: function(errorInfo){
		var errorMessage = errorInfo.data.message;
		var statusCode = errorInfo.ioArgs.xhr.status;
		var errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
		//var statusCode = errorInfo.ioArgs.xhr.status;
		if (statusCode == 401) {
		  // 401 Unauthorized
		  errorHandler.showWarningDialog("KEY_REST_UNAUTHORIZED", {
			1: "UNKOWN"
		  }, function(){
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
		  });
		}else{
			com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(errorMessage, null /* details */, function(){
				dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
			});
		}
		com.ibm.bpm.social.util.common.Log.logError(this.LOG_LEVEL, ""+this.CLASS_NAME+" experienced an error: "+errorMessage);
	  },
	
	unload: function(arguments){
		var FUNC_NAME = "unload";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.active = false;
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	destroy: function(){
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Destroying details...", this.CLASS_NAME, FUNC_NAME);
		if(this.details && this.details.destroy){
			this.details.destroy();
			this.details = null;
		}
		// disconnect from instance updates
		if (this.instanceUpdateHandle) {
			dojo.unsubscribe(this.instanceUpdateHandle);
			this.instanceUpdateHandle = null;
		}
		
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Destroying stream...", this.CLASS_NAME, FUNC_NAME);
		if(this.stream && this.stream.destroy){
			this.stream.destroy();
			this.stream = null;
		}
		
		this.inherited(arguments);
		
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.experts.ExpertTile"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.experts.ExpertTile"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.ExpertTile");














dojo.declare(
	"com.ibm.bpm.social.widgets.experts.ExpertTile",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-experts-expert' dojoAttachPoint='expert' dojoAttachEvent='onmouseenter:_doMouseEnter, onmouseleave:_doMouseLeave'>\n\t<div dojoAttachPoint='avatarContent'> </div>\t\n\t<div class='bpm-social-experts-expert-data-div'>\t    \n\t\t<div class='bpm-social-experts-expert-name' dojoAttachPoint='name' role=\"listitem\" tabindex=\"0\"></div>\n\t\t<div tabIndex='0' dojoAttachEvent='onfocus:_focusStart' class='bpm-social-experts-focus-control'></div>\n\t\t<div class='bpm-social-experts-expert-details taskDetails' dojoAttachPoint='taskDetails'></div>\n\t\t<div class='bpm-social-experts-expert-actions-div'>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-mention' dojoAttachPoint=\"mentionDiv\">\n\t\t\t\t<a dojoAttachPoint='mention' href='javascript:void(0)' dojoAttachEvent='onclick:_handleMention'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator1' class='vertical-bar' style='display:block'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-help' dojoAttachPoint=\"helpDiv\">\n\t\t\t\t<a dojoAttachPoint='gethelp' href='javascript:void(0)' dojoAttachEvent='onclick:_handleGetHelp'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator2' class='vertical-bar'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-chat' dojoAttachPoint=\"chatDiv\">\n\t\t\t\t<a dojoAttachPoint='startchat' href='javascript:void(0)' dojoAttachEvent='onclick:_handleChat'></a>\n\t\t\t</div>\n\t\t</div>\n\t\t<div tabIndex='0' dojoAttachEvent='onfocus:_focusEnd' class='bpm-social-experts-focus-control'></div>\n\t</div>\n</div>\n",
	completed: 0,
	avgHours: 0,
	workload: 0,
	userName: null,
	fullName: null,
	phoneNumber: null,
	email: null,
	imageUrl: null,
	taskId: null,
	livename: null,
	refresh: null,
	title: "",
	includeTaskDetails: false,
	sidebar: false,
	userDataUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/userdata?username={username}",
	collaborate: false,
	
    postCreate: function() {
    	this.inherited(arguments);
    	
    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.mention.innerHTML = "@"+this.messages.expertsMention;
		this.gethelp.innerHTML = this.messages.expertsCollaborate;
		this.startchat.innerHTML = this.messages.expertsChat;
		
		this.name.innerHTML = this.fullName;
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.name);
		
		var curdate = new Date();
		var millisec = curdate.getTime();
		var nameId = "expertTile_name" + millisec;
		dojo.attr(this.name, "id", nameId);	
		dijit.setWaiState(this.mention, "describedby", nameId);
		dijit.setWaiState(this.gethelp, "describedby", nameId);
		dijit.setWaiState(this.startchat, "describedby", nameId);
		
		this.actionSeparator1.innerHTML = this.messages.expertsSeparator;
		this.actionSeparator2.innerHTML = this.messages.expertsSeparator;
		
		if (!this.sidebar) {
			this.mention.style.display = 'none';
			this.actionSeparator1.style.display = 'none';
		}
		
    	this._getUserData();
    	
    	this.domNode.title = "";
		
		if (!this.collaborate) {
			this.actionSeparator1.style.display = 'none';
			this.helpDiv.style.display = 'none';
		}
    	
		if(window.sametime && this.fullName){
			// the text-overflow:ellipsis style does not function properly
			// when there are span elements within the div, so clip the
			// text instead if we are using sametime
			this.name.style.textOverflow = 'clip';
			
			this.name.innerHTML = "";
			this.livename = window.sametime.LiveName({'userId': this.userName, 'disableClicks': true, 'disableHoverBizCard': true},dojo.create("span", null, this.name));
			this.livename.activate();
			
			this._updateStatus();
			this.refresh = setInterval(dojo.hitch(this, this._updateStatus), 2000);
		} else {
			dojo.style(this.chatDiv, "display", "none");
		}			   

		/** Business Card integration **/
		/** Default **/
		var node = this.name;
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		
		if(this.livename!=null){
			var nodes = dojo.query(".stproxy_liveNamePadding", this.name);
			if(nodes.length==1){
				node = nodes[0];
				var img = dojo.create("img", {
					"src": dojo.moduleUrl("dojo", "resources/blank.gif"),
					"class": "connectionsOverlayImage"
					},
					dojo.query(".stproxy_general", this.domNode)[0],
					"first");
				com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
			}
		}
		com.ibm.bpm.social.widgets.common.BizCard.wrapElement(node, user);
		com.ibm.bpm.social.widgets.common.BizCard.wrapImage(dojo.query("img", this.img)[0], user);

		/** End Business Card integration **/
		
		this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
		this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);

		this._updateUserData();
		var data;
		var firstName = this.fullName.split(" ")[0]; 
        firstName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(firstName);
		if(this.includeTaskDetails){
			if(this.completed==0){
				this.taskDetails.innerHTML = this.messages.expertNoTaskData.replace("${user}", firstName);
			} else {
				data = {
					'userName': firstName,
					'average': this.avgHours,
					'completed': this.completed
				};
				//defect 149810 Fix Expert tooltip wording
				if(this.sidebar){
				   this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip, data, /\$\{([^\}]+)\}/g);
				} else {
				   this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip_short, data, /\$\{([^\}]+)\}/g);
				}
			}
            dojo.attr(this.taskDetails,"id","taskDetails_"+nameId);
            dijit.setWaiState(this.name,"describedby","taskDetails_"+nameId);
		}
	},
	  
	_showTooltip: function(){
		
		return;
		/*
		if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
			** Do not show if Connections is enabled **
			return;
		}
		
		if (this.myTooltipDialog) {
			this.myTooltipDialog = null;
		}
		
		this._updateUserData();
		
		var firstName = this.fullName.split(" ")[0]; 
		
		var data = {
				'fullName': this.fullName,
				'userName': this.userName, 
				'avgHours': this.avgHours,
				'completed': this.completed,
				'displayName': this.fullName,
				'title': typeof(this.userData['Title'])!='undefined' ? 
						this.userData['Title'] : '',
				'taskEmailAddress': typeof(this.userData['Task Email Address'])!='undefined' ? 
						this.userData['Task Email Address'] : '',
				'phoneNumber': typeof(this.userData['Phone Number'])!='undefined' ? this.userData['Phone Number'] : '',
				'taskDetails': "",
				'chatString': '',
				'hr': "",
				'includeTaskDetails': false
		};
		
		this.activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
		
		this.myTooltipDialog = new dijit.TooltipDialog({
            content: this.activityOwner.domNode,
            onMouseLeave: function(){
                dijit.popup.close(this.myTooltipDialog);
            }
        });
		dijit.popup.open({
			popup: this.myTooltipDialog,
			around: dojo.byId('expert-avatar')
		});

		*/
		
	},
	
	_hideTooltip: function() {
		if (this.myTooltipDialog) {
			this.myTooltipDialog = null;
		}

	},
	
	//defect 149810 fix so that task completed is an integer and does not have decimal places
	_updateUserData: function() {
		var self = this;
		
		this.socialRestHandler.getExpertUserData(this.taskId, this.userName,
			function(data, xhrArgs) {
				if(data.avgHours < 1){
					self.avgHours = dojo.number.format( data.avgHours, {type: "decimal", pattern: "0.#", places: 1});
				} else {
					self.avgHours = dojo.number.format( data.avgHours, {places: 1 } );
				}
				self.completed = dojo.number.format( data.completed, {places: 0 } ); 
			}, "", false);
	},
	
	_getUserData: function(){
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		this.userData = user.userPreferences;
		if (!this.userData) {
			this.userData = new Object();
		}		
	},
	
	_updateStatus: function(){
		dojo.removeClass(this.domNode, "online");
		dojo.removeClass(this.domNode, "offline");
		
		if(this.livename){
			switch(this.livename.currentTextStyle){
				case "stproxy_liveNameAvailableText":
				case "stproxy_liveNameAwayText":
				case "stproxy_liveNameInMeetingText":
					dojo.addClass(this.domNode, "online");
					this.presenceStatus = "online";
					break;
				case "stproxy_liveNameOfflineText":
				case "stproxy_liveNameDndText":
				default:
					dojo.addClass(this.domNode, "offline");
					this.presenceStatus = "offline";
					break;
			}
		} else {
			dojo.style(this.chatDiv, "display", "none");
		}
		
		/** Business Card integration **/
		if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard() && this.livename!=null){
			/** Account for Sametime DOM structure, which can be delayed in being created due to network latency **/
			var nodes = dojo.query(".stproxy_liveNamePadding .hasHover", this.name);
			if(nodes.length==0){
				nodes = dojo.query(".stproxy_liveNamePadding", this.name);
				if(nodes.length==1){
					var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
					com.ibm.bpm.social.widgets.common.BizCard.wrapElement(nodes[0], user); 
				}
			}
		}
		/** End Business Card integration **/
		
	},
	
	_handleGetHelp: function(e){
		var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({"taskId": this.taskId});
		com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(taskData, {
			userName: this.userName,
			fullName: this.fullName
		});		
		dojo.stopEvent(e);
	},
	
	_handleChat: function(e){
		if(this.livename && this.presenceStatus=="online"){
			this.livename.startChat();
		}
		dojo.stopEvent(e);
	},
	
	_getTaskCompletionView: function() {
		var node = this.domNode;
		while (node && !dojo.hasClass(node,'workViewCompletion')) {
			node = node.parentNode;
		}
		if (node == null) {
			node = dojo.query('.workViewCompletion')[0];
		}
		var taskCompletionView = dijit.byId(node.id);
		return taskCompletionView;
	},
	
	_handleMention: function(e){
		var taskCompletionView = this._getTaskCompletionView();
		if (taskCompletionView) {
			// switch to the stream tab
			taskCompletionView.sidebar.selectTab(1);
			// set focus to input box
			taskCompletionView.stream._handlePostInitialFocus();
			// tag user
			taskCompletionView.stream.tagUser(this.userName,this.fullName);
		}
		dojo.stopEvent(e);
	},
	
	_doMouseEnter: function(e){
		dojo.addClass(this.expert, 'hover');
	},
	
	_doMouseLeave: function(e){
		dojo.removeClass(this.expert, 'hover');
	},
	
	_onFocus: function() {
		dojo.addClass(this.domNode,'bpm-social-experts-focus');
	},
	
	_onBlur: function() {
		dojo.removeClass(this.domNode,'bpm-social-experts-focus');
	},
	
	_focusStart: function() {
		if (this.sidebar) {
			this.mention.focus();
		} else {
			this.gethelp.focus();
		}
	},
	
	_focusEnd: function() {
		if(this.livename){
			this.startchat.focus();
		} else {
			this.gethelp.focus();
		}
	},
	
	destroy: function() {
		if(this.refresh){
			clearInterval(this.refresh);
		}
		this._hideTooltip();
	    if (this.avatar) {
	        this.avatar.destroy();
	        this.avatar = null;
	    }
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.experts.Experts"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.experts.Experts"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.Experts");















dojo.declare(
	"com.ibm.bpm.social.widgets.experts.Experts",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-experts' dojoAttachPoint='expertsWidgetContainer'>\n<div class='bpm-social-popup-experts-content'>\n\t<div class='bpm-social-experts-subscript' dojoAttachPoint='autoSubscript'></div>\n\t<div class='bpm-social-experts-layout bpm-social-experts-layout-auto' dojoAttachPoint='autoExperts'></div>\n\t<div style=\"clear:both\"><a class='bpm-social-experts-auto-more' dojoAttachPoint='autoMore' dojoAttachEvent='onclick:_handleAutoMore' style='display:none'></a></div>\n\t<div class='bpm-social-experts-subscript' dojoAttachPoint='manualSubscript'></div>\n\t<div class='bpm-social-experts-layout bpm-social-experts-layout-manual' dojoAttachPoint='manualExperts'></div>\n\t<div style=\"clear:both\"><a class='bpm-social-experts-manual-more' dojoAttachPoint='manualMore' dojoAttachEvent='onclick:_handleManualMore' style='display:none'></a></div>\n\t<!-- <div class='help-link-wrapper'><a href='' class='help-link' dojoAttachPoint='helpLink' target='_blank'></a></div> -->\n</div>\n</div>\n",
	popup: null,
	userid: null,
	taskid: null,
	AUTO_EXPERTS: "auto",
	MANUAL_EXPERTS: "manual",
	expertTiles: null,
	sidebar: false,
	expertsUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/experts",
	_taskClaimedHandle: null,
	_refreshInProgress: false,
	_refreshNeeded: false,
	_currentUser: null,
		
    postCreate: function() {
		
		this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		dijit.setWaiState(this.expertsWidgetContainer, "live", "polite");
		dijit.setWaiState(this.expertsWidgetContainer, "atomic", "true");
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.taskMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
		
		this.autoSubscript.innerHTML = this.messages.expertsAutoHeader;
		this.manualSubscript.innerHTML = this.messages.expertsManualHeader;
		this.autoMore.innerHTML = '+ '+this.messages.showAll;
		this.manualMore.innerHTML = '+ '+this.messages.showAll;
		//this.helpLink.innerHTML = this.messages.expertsHelpText;
		//this.helpLink.href = "/BPMHelp/index.jsp?topic=/com.ibm.wbpm.wle.widget.doc/portalhelp/topic/experts.html";
		
		if(this.sidebar){
			dojo.addClass(this.domNode, 'sidebar');
		}
		
		var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if (userProfileObj) {
			this._currentUser = userProfileObj.userName;
		} 

		this.expertTiles = new Array();
		if(this.taskid!=null){
			this.refresh({taskid:this.taskid});
		}
	},
	
	showExpertsList: function() {
		this.refresh({taskid: this.taskid});
	},
	refresh: function(obj){
		if(! (obj.taskid) ){
			return;
		}
		this.taskid = obj.taskid;
		if(this.expertsWidgetContainer.style.display == "none") {
			return;
		}
		if (this._refreshInProgress) {
			this._refreshNeeded = true;
		} else {
			this._refreshInProgress = true;
			this._clearExperts();
			this._showLoading();
			this._getExperts();
			this._subscribeToTaskClaimed();
		}
	},
	
	_clearExperts: function(){
		
		dojo.forEach(this.expertTiles, function(ee){
			if(ee.destroy){
				ee.destroy();
			}
		});
		// Do not destroy the tiles several times.
		this.expertTiles = new Array();
		
		dojo.empty(this.autoExperts);
		
		dojo.empty(this.manualExperts);
		
	},
	
	_getExperts: function() {
		
		this.socialRestHandler.getExpertsForTask(this.taskid, 
			dojo.hitch(this, "_getExpertsResponse"), dojo.hitch(this, "_onXhrErrors"), true);
	},
	
	_isCollaborationSupported: function() {
		var col = false;
		this.taskRestHandler.getActionsForTask(this.taskid, null, function(data) {
			var actions = data.tasks[0].actions;
			for (var i=0; i<actions.length; i++) {
				if (actions[i] == 'ACTION_INVITE') {
					col = true;
				}
			}
		}, function() {}, false, null, null);
		return col;
	},
	
	_removeCurrentUser: function(group) {
		if(group && this._currentUser) {
			for (var i=0; i<group.length; i++) {
				if (group[i].userName == this._currentUser) {
					group.splice(i,1);
					i--;
				}
			}
		}
	},
	
	_getExpertsResponse: function(data,xhrArgs) {
		this._hideLoading();
		
		var ee = null;
		var cnt = 0;
		var tmpUserCache = {}; //Hashmap of automatic users to easily track task data if needed for manual users

		var col = this._isCollaborationSupported();
		
		this.autoSubscript.innerHTML = this.messages.expertsAutoHeader;
		dojo.removeClass(this.autoExperts,'bpm-social-experts-more');
		this.autoMore.style.display = 'none';
		
		var systemIdentified = data["systemIdentifiedUsers"];
		this._removeCurrentUser(systemIdentified);
		if(!systemIdentified || !systemIdentified.length){
			this.autoSubscript.innerHTML = this.messages.expertsAutoHeader + " (0)";
			dojo.addClass(this.autoExperts, 'no-experts');
			this.autoExperts.innerHTML = this.messages.noExperts_AutoHeader;
		} else {
			cnt = 0;
			dojo.forEach(systemIdentified, dojo.hitch(this, function(item){
				if(cnt < 6) { //max of 6 system-identified experts (fix for RTC defect 100772)
					item['taskId'] = this.taskid;
					item['includeTaskDetails'] = true;
					item['sidebar'] = this.sidebar;
					item['collaborate'] = col;
					tmpUserCache[item['userName']] = item;
					ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(item);
					this.autoExperts.appendChild(ee.domNode);
					this.expertTiles.push(ee);
					cnt++;
					if (cnt >5) {
						dojo.addClass(ee.domNode,'bpm-social-experts-all');
						dojo.addClass(this.autoExperts,'bpm-social-experts-more');
						this.autoMore.style.display = 'block';
					}
				}
			}));
			this.autoSubscript.innerHTML = this.messages.expertsAutoHeader + " ("+cnt+")"
		}
		
		this.manualSubscript.innerHTML = this.messages.expertsManualHeader;
		dojo.removeClass(this.manualExperts,'bpm-social-experts-more');
		this.manualMore.style.display = 'none';

		var expertParticipantGroup = data["expertParticipantGroupUsers"];
		this._removeCurrentUser(expertParticipantGroup);
		if(!expertParticipantGroup || !expertParticipantGroup.length){
			this.manualSubscript.innerHTML = this.messages.expertsManualHeader+ " (0)";
			dojo.addClass(this.manualExperts, 'no-experts');
			this.manualExperts.innerHTML = this.messages.noExperts_ManualHeader;
		} else {
			this.manualSubscript.innerHTML = this.messages.expertsManualHeader + " ("+expertParticipantGroup.length+")";
			cnt = 0;
			dojo.forEach(expertParticipantGroup, dojo.hitch(this, function(item){
				var tmp;
				if(tmpUserCache[item['userName']]){
					tmp = tmpUserCache[item['userName']];
				} else {
					item['taskId'] = this.taskid;
					item['includeTaskDetails'] = true;
					item['sidebar'] = this.sidebar;
					item['collaborate'] = col;
					tmp = item;
				}
				ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(tmp);
				this.manualExperts.appendChild(ee.domNode);
				this.expertTiles.push(ee);
				cnt++;
				if (cnt >5) {
					dojo.addClass(ee.domNode,'bpm-social-experts-all');
					dojo.addClass(this.manualExperts,'bpm-social-experts-more');
					this.manualMore.style.display = 'block';
				}
			}));
		}
				
		this._refreshInProgress = false;
		if (this._refreshNeeded) {
			this._refreshNeeded = false;
			this.refresh({taskid:this.taskid});
		}
	},
	
	_handleAutoMore: function() {
		dojo.removeClass(this.autoExperts,'bpm-social-experts-more');
		this.autoMore.style.display = 'none';
	},

	_handleManualMore: function() {
		dojo.removeClass(this.manualExperts,'bpm-social-experts-more');
		this.manualMore.style.display = 'none';
	},

	_onXhrErrors: function(data,xhrArgs) {
		// TODO: how do we show errors?
		this._refreshInProgress = false;
		if (this._refreshNeeded) {
			this._refreshNeeded = false;
			this.refresh({taskid:this.taskid});
		}
	},

	_subscribeToTaskClaimed: function() {
		if (this._taskClaimedHandle) {
			dojo.unsubscribe(this._taskClaimedHandle);
			this._taskClaimedHandle = null;
		}
		this._taskClaimedHandle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_CLAIMED+"."+this.taskid, dojo.hitch(this, '_taskClaimed'));
	},
	
	_taskClaimed: function() {
		this.refresh({taskid:this.taskid});
	},
	
	_showLoading: function() {
		this.autoExperts.innerHTML = this.taskMessages["KEY_Loading"];
		this.manualExperts.innerHTML = this.taskMessages["KEY_Loading"];
	},

	_hideLoading: function() {
		this.autoExperts.innerHTML = "";
		this.manualExperts.innerHTML = "";
	},
	destroy: function() {
	    this._clearExperts();
		if (this._taskClaimedHandle) {
			dojo.unsubscribe(this._taskClaimedHandle);
			this._taskClaimedHandle = null;
		}
		this.inherited(arguments);
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.views.TaskCompletionView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.views.TaskCompletionView"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.TaskCompletionView");























dojo.declare("com.ibm.bpm.social.portal.views.TaskCompletionView", [dijit._Widget,
                                                                    dijit._Templated], {

  templateString: dojo.cache("com.ibm.bpm.social.portal.views", "templates/TaskCompletionView.html", "<div dojoAttachPoint=\"container\" class=\"workView workViewCompletion\">\n\t<div class=\"sidebarContent\" dojoAttachPoint=\"sidebarContent\"></div>\n\t<div class=\"mainContent completionContent\" dojoAttachPoint=\"completionContent\"></div>\n</div>\n"),
  CLASS_NAME: "TaskCompletionView",
  LOG_LEVEL: 1,

  active: false,
  taskId: 0,
  lastListOrInstanceTopic: null,
  lastInstancePayload: null,
  lastTaskAlreadyCompleted: "",

  postCreate: function(arguments){
	var FUNC_NAME = "postCreate";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    
	  
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
	  this.taskMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

    dojo.create("div", {
      'id': 'completion_content_widget_target'
    }, this.completionContent);

    dojo.create("div", {
      'id': 'completion_sidebar_widget_target'
    }, this.sidebarContent);

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Subscribing to TaskCompletionView topics...", this.CLASS_NAME, FUNC_NAME);
    dojo.subscribe(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", dojo.hitch(this,
      function(args){
        dojo.addClass(this.domNode, "maximizedContent");
        dojo.addClass(dojo.body(), "socialPortalWorkMaximized");
        dojo.query(".workViewContainer").addClass("maximizedContent");
      }));

    dojo.subscribe(com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT + "-TaskCompletionView", dojo.hitch(this,
      function(args){
        dojo.removeClass(this.domNode, "maximizedContent");
        dojo.removeClass(dojo.body(), "socialPortalWorkMaximized");
        dojo.query(".workViewContainer.maximizedContent").removeClass("maximizedContent");
      }));

    this.initializeContentWidgets();

    //Determine width of our main content
    this._sizeContentContainer();

    // Add tabindex of -1 for accessibility so we can change focus to this tab through dojo
    dojo.query('#completion_content_widget_target').attr("tabindex", "-1");

    // Logic for triggering size adjustments.
    // 	Might be temporary depending on what the shell is going to provide.
    this.connect(dojo.isIE && dojo.isIE < 9 ? this.domNode : dojo.global, 'onresize', "_sizeContentContainer");
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _returnToLastView: function(){
	var FUNC_NAME = "_returnToLastView";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _sizeContentContainer: function(){
    // Browser automatically sizes the coachContainer due to using a layout with 100% 
    // Only the allocation of the available space need to be recalculated using JavaScript.
    if (this.coachContainer) {
      this.coachContainer.resize();
    }
  },

  _coachLoaded: function(tkiid /* string */){
  // Any logic that needs to be triggered upon loading of the coach can be added here.
  },

  _coachCanceled: function(){
    //
    // Stay on the page since the side bar still provides meaningful information.
    //
    this.coachContainer.hide();
    // Return to the last view
	this.taskId = null;
	this.taskTemplateId = null;
    this._returnToLastView();
  },

  _taskCompleted: function(nextTkiid /* auto flow task */){
	var FUNC_NAME = "_taskCompleted";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    // Switch back to task list.
    // Is there a smarter solution which switches back to where we came from?
    if (nextTkiid) {
    	// autoflow case
		// remove this task from the history list (since it finished)
		var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
		if (portalHashHistory && (portalHashHistory.length > 0)) {
			portalHashHistory.pop();
		}	
		// load next task and update sidebar
		this.load({
			"taskId": nextTkiid,
			"update": true
		});
    }
    else {
   		this.taskId = null;
	    this.taskTemplateId = null;

		// force process performance dashboard to use a new instance
		dojo.publish(com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD,['TWP','Process+Performance']);

		var src = null;
		var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
		if (portalHashHistory && (portalHashHistory.length > 1)) {
			// remove this task from the history list (since it finished)
			portalHashHistory.pop();
			// check if the previous window was the process performance dashboard
			var src = portalHashHistory.pop();
			var tmp = src;
			if (tmp.charAt(0) != '/') {
				tmp = '/' + tmp;
			}
			var i = tmp.indexOf('?');
			if (i != -1) {
				tmp = tmp.substring(0,i);
			}
			var tmp = tmp.split('/');
			if (tmp && (tmp.length > 3) && (tmp[2] == 'TWP') && ((tmp[3] == 'Process+Performance') || (tmp[3] == 'Process Performance'))) {
				// go to the task list instead of the process perfomance dashboard
				src = '/dashboards/TWP/BPM_WORK?tw.local.view=tasks';
			}
			dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[src]);
		} else {
			// let navigation logic figure out where to go
			com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
		}
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _updateInstance: function(){
    if (this.active && this.taskId) {
      // we need to update everything because in Critical path simply everything might be changed
      this.load({
        "taskId": this.taskId,
        "update": true
      });
    }
  },

  _updateTask: function(changes){
    if (this.active && this.taskId) {
      if (changes && changes.assignee) {
        // if the assignee is changed, one cannot work on the task anymore
        if (this.coachContainer) {
          this.coachContainer.onCompleted();
        }
      }
      else {
        this.load({
          "taskId": this.taskId,
          "update": true
        });
      }
    }
  },

  _taskAlreadyCompleted: function(piid /* string */){
	this.taskId = null;
    this.taskTemplateId = null;
    if (piid) {
      // Show instance view
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        instanceId: piid
      }]);
    }
    else {
      // Stay on page
      this.coachContainer.hide();
    }
  },
  _taskDoesNotExist: function(){
    // Show the task list again.
    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
  },
  initializeContentWidgets: function(){
	var FUNC_NAME = "initializeContentWidgets";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Content teams place widget initialization here ***//
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TaskCompletionWidget....", this.CLASS_NAME, FUNC_NAME);
    this.coachContainer = new com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget({
      dialog: false
    }, 'completion_content_widget_target');
    this.connect(this.coachContainer, "onLoaded", "_coachLoaded");
    this.connect(this.coachContainer, "onCanceled", "_coachCanceled");
    this.connect(this.coachContainer, "onCompleted", "_taskCompleted");
    this.connect(this.coachContainer, "onTaskAlreadyCompleted", "_taskAlreadyCompleted");
    this.connect(this.coachContainer, "onTaskDoesNotExist", "_taskDoesNotExist");

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the PortalSidebar....", this.CLASS_NAME, FUNC_NAME);
    this.sidebar = new com.ibm.bpm.social.portal.PortalSidebar({
      showWidgetTopic: com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION
    }, 'completion_sidebar_widget_target');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the InstanceDetails for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.details = new com.ibm.bpm.social.widgets.details.InstanceDetails({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.details, this.details, 'bpm-social-details-alt-icon', '&Dagger;');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the Stream for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.stream = new com.ibm.bpm.social.widgets.stream.Stream({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.stream, this.stream, 'bpm-social-stream-alt-icon', ':=');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the Experts for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.experts = new com.ibm.bpm.social.widgets.experts.Experts({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.experts, this.experts, 'bpm-social-experts-alt-icon', '?');

    this.sidebar.startup();
    var expertsTabButton = dojo.query("li[role=\"tab\"][actual-title=\""+this.messages.experts+"\"]")[0];
    //connect onclick and keydown handlers to refresh experts list
    dojo.connect(expertsTabButton, "onclick", dojo.hitch(this.experts, this.experts.showExpertsList));
    dojo.connect(expertsTabButton, "onkeydown", dojo.hitch(this, function(event) {
      if(event.keyCode == 13 /*Enter Key */) {
          this.experts.showExpertsList();
      }
    }));
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _loadTask: function(taskData, update){
  var taskAlreadyCompleted = false;
  if (taskData && taskData.state && taskData.state == "STATE_FINISHED") {
	if (this.lastTaskAlreadyCompleted != taskData.tkiid) {
		this.lastTaskAlreadyCompleted = taskData.tkiid;
	} else {
		this.lastTaskAlreadyCompleted = "";
		taskAlreadyCompleted = true;
	}
  }
  if (!taskAlreadyCompleted) {
		if (update) {
		  // Only update the meta data of the coach container
		  this.coachContainer.updateTask(taskData);
		  this._sizeContentContainer(); // Due to problems with IE
		}
		else {
		  // 
		  // Show the task completion ui.
		  //
		  this.coachContainer.loadCoach(taskData);
		  this._sizeContentContainer(); // Due to problems with IE
		}

		if (this.taskId) {
		  this.details.refresh({
			instanceId: taskData.piid
		  });

		  this.stream.refresh({
			taskid: this.taskId,
			instanceid: taskData.piid
		  });

		  this.experts.refresh({
			taskid: this.taskId
		  });
		  var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=taskcompletion&tw.local.taskid='+this.taskId;
		  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);

		  // connect to task updates
		  this.taskUpdateHandle =
			dojo
			  .subscribe(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this.taskId, dojo.hitch(this, this._updateTask));

		  // connect to instance updates
		  this.instanceUpdateHandle =
			dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED + "." + taskData.piid, dojo.hitch(this,
			  this._updateInstance));
		} 
	} else {
		  var newHash2 = 'dashboards/TWP/BPM_WORK?tw.local.view=tasks';
		  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash2);
		  var params={"isWorkTab":true};
		  var msg={"runURL":newHash2,"params":params};
		  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [msg]);
	}
  },

  _loadTaskTemplate: function(taskTemplateData){
    // Prereqs are checked by the TaskCompletionWidget
    this.coachContainer.loadCoach4Template(taskTemplateData);
    this._sizeContentContainer(); // Due to problems with IE
  },

  _showErrorMessage: function(errorInfo){
    var errorMessage = errorInfo.data.message;
    var statusCode = errorInfo.ioArgs.xhr.status;
    //
    // Is it a well-known error code?
    //
	this.taskId = null;
	this.taskTemplateId = null;
    if (statusCode == 401) {
        // 401 Unauthorized
	      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.taskMessages["user_not_authorized_for_task"], null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });
    }
    else if (statusCode == 404) {
        // 404 Not Found
	      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.taskMessages["task_not_available"], null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });      
    }
    else {
      //
      // Otherwise just show the error message.
      //
      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(errorMessage, null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });
    }
	com.ibm.bpm.social.util.common.Log.logError(this.LOG_LEVEL, ""+this.CLASS_NAME+" experienced an error: "+errorMessage);
  },

  load: function(arguments){
	var FUNC_NAME = "load";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Called from the WorkViewContainer every time immediately before the view is displayed ***//

    //
    // Either the taskId or the taskTemplateId should be provided
    //
    if (typeof arguments.taskId === "undefined" && typeof arguments.taskTemplateId === "undefined") {
	  com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "No taskId or taskTemplateId", this.CLASS_NAME, FUNC_NAME);
      return;
    }

    // disconnect from update handlers
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Disconnecting update handlers...", this.CLASS_NAME, FUNC_NAME);
    if (this.instanceUpdateHandle) {
      dojo.unsubscribe(this.instanceUpdateHandle);
      this.instanceUpdateHandle = null;
    }
    if (this.taskUpdateHandle) {
      dojo.unsubscribe(this.taskUpdateHandle);
      this.taskUpdateHandle = null;
    }

    this.active = true;

    //
    // Initially set all the ids to null.
    // Is it a task or a taskTemplate Id ??   If yes, call the REST api and retrieve the object.
    //
    this.taskId = null;
    this.taskTemplateId = null;
    var taskData = null;
    var taskTemplateData = null;
    var result = null;
    if (arguments.taskId) {
      this.taskId = arguments.taskId;
      this.taskTemplateId = null;
      // Always retrieve an updated task since the state could have changed in the meantime
      result = com.ibm.bpm.social.util.common.Util.getTaskData({
        "taskId": this.taskId,
        "skipCache": true
      });
      taskData = result;
    }
    else {
      this.taskId = null;
      this.taskTemplateId = arguments.taskTemplateId;
      result = com.ibm.bpm.social.util.common.Util.getTaskTemplateData({
        "taskTemplateId": this.taskTemplateId
      });
      taskTemplateData = result;
    }
    //
    // Processing of the result of either the getTaskData or the getTaskTemplateData call.
    //
    if (result) {
      //
      // Did an error occur?
      // Since task definitions are also tasks (which are startable), the same error message can be used.
      //
      if (typeof result.errorInfo === "object") {
        //
        // Error occurred
        //
        var errorInfo = result.errorInfo;
		if(this.taskId != null) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error loading task "+this.taskId, this.CLASS_NAME, FUNC_NAME);
		}
		else {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error loading task "+this.taskTemplateId, this.CLASS_NAME, FUNC_NAME);
		}
        this._showErrorMessage(errorInfo);
      }
      else {
        //
        // The rest call was successful
        //
        if (taskData) {
          this._loadTask(taskData, typeof arguments.update === "boolean" && arguments.update);
        }
        else if (taskTemplateData) {
          this._loadTaskTemplate(taskTemplateData);
        }
      }
    }

    dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{
      "viewType": "TaskCompletionView"
    }]);
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  unload: function(arguments){
	var FUNC_NAME = "unload";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    this.active = false;
    if (this.coachContainer) {
      this.coachContainer.exitTaskCollaboration();
      this.coachContainer.unloadCoach();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  destroy: function(){
	var FUNC_NAME = "destroy";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    if (this.instanceUpdateHandle) {
      dojo.unsubscribe(this.instanceUpdateHandle);
      this.instanceUpdateHandle = null;
    }
    if (this.taskUpdateHandle) {
      dojo.unsubscribe(this.taskUpdateHandle);
      this.taskUpdateHandle = null;
    }
    if (this.coachContainer) {
      this.coachContainer.exitTaskCollaboration();
    }
    this.inherited(arguments);

	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.ProcessList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.ProcessList"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.ProcessList");





/* ProcessList - A dojo widget responsible for displaying a list of processes. 
	Originally created for the FollowedProcessList and LaunchableProcessList
	widgets. Each item in a process list has a clickable icon and title. 
	SmartList was later created as a base class to allow for more generic, non-
	process lists (such as TaggedProcessist, which  isn't a ProcessList 
	proper).
	
	Updated: 1/9/2012
	Experts: Ben Myers @ Raleigh, Scott Johnson @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.ProcessList",
	[com.ibm.bpm.social.widgets.SmartList, dijit._Templated],
	{
	
	templateString: "",
	templateString:"<div class=\"bpm-social-smartlist bpm-social-processlist\">\n\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t<ul class=\"process-list\" dojoAttachPoint=\"listElement\"><li style=\"display:none\">&nbsp;</li></ul>\n</div>\n",
	constructor: function() {
		//dojo event connectors are added to an array so they can be disconnected
		this.processListEventConnectors = [];
		
		//used with _restoreFocusedItem and related event handlers
		this.currFocusIndex = null;
		this.currFocusIsIcon = false;
	},
	
	postCreate: function() {
		//localization
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.processListEventConnectors = [];
		
		this.inherited(arguments);
	},
	
	/* addProcessItem: Adds an item to the ProcessList with title as its text. 
		When the item is clicked any params it was created with will be 
		exposed, so the event can be better handled. */
	addProcessItem: function(title, params) {
		if(typeof(params)=="undefined" ) {
			params = null;
		}
		return this.addItem(params, {'title':title});
	},
	
	/* _createItem: Handles creating each process item's HTML elements. Params 
		is provided when addProcessItem is called. */
	_createItem: function(index, li, params, createParams) {
		var title = "<i>Unknown</i>";
		if(createParams!=null && typeof(createParams)!='undefined') {
			title = createParams['title'];
		}
		
		//create the HTML elements
		var icon = dojo.create("div", {'class':"process-icon "+params['iconClass'], 'tabIndex':"0"}, li);
		dijit.setWaiRole(icon, "button");
		var text = dojo.create("a", {'class':"process-text", 'innerHTML':title}, li);
		dijit.setWaiRole(text, "link");
		
		//add dojo event connectors
		this.processListEventConnectors.push(dojo.connect(icon, 'keypress', this, "_processIconKeyPressEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onclick', this, "_processIconClickEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onfocus', this, "_processIconFocusEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onblur', this, "_processIconBlurEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onmouseenter', this, "_processIconMouseoverEvent"));
		this.processListEventConnectors.push(dojo.connect(icon, 'onmouseleave', this, "_processIconMouseoutEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'keypress', this, "_processTextKeyPressEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onclick', this, "_processTextClickEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onfocus', this, "_processTextFocusEvent"));
		this.processListEventConnectors.push(dojo.connect(text, 'onblur', this, "_processTextBlurEvent"));
	},
	
	/* Below are event handlers that do little other than handle basic 
		interactions with the ProcessList and usually end up calling 
		_onProcessIconClicked or _onProcessTextClicked. */
	_processIconMouseoverEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['li']!=null) {
			dojo.addClass(dojo.query(".process-icon", info['li'])[0], "hover");
		}
	},
	_processIconMouseoutEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['li']!=null) {
			dojo.removeClass(dojo.query(".process-icon", info['li'])[0], "hover");
		}
	},
	_processIconClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['index']!=null && info['li']!=null) {
			this._onProcessIconClicked(info['index'], info['li'], info['params']);
		}
	},
	_processIconKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt);
			if(info['index']!=null && info['li']!=null) {
				this._onProcessIconClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_processTextClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		if(info['index']!=null && info['li']!=null) {
			this._onProcessTextClicked(info['index'], info['li'], info['params']);
		}
	},
	_processTextKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt);
			if(info['index']!=null && info['li']!=null) {
				this._onProcessTextClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	
	/* Below are event handlers that allow the ProcessList to remember what 
		element was focused even after the list is torn apart by clearList. 
		This is espcially useful for keyboard-only users. */
	_processIconFocusEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		this.currFocusIndex = info['index'];
		this.currFocusIsIcon = true;
	},
	_processTextFocusEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt);
		this.currFocusIndex = info['index'];
		this.currFocusIsIcon = false;
	},
	_processIconBlurEvent: function(evt) {
		this.currFocusIndex = null;
	},
	_processTextBlurEvent: function(evt) {
		this.currFocusIndex = null;
	},
	
	/* _restoreFocusedItem: Since DOM elements may be deleted and recreated 
		when the ProcessList is refreshed, it is necessary to restore focus 
		to the last element the user interacted with to support 
		keyboard-only users. Subclasses of ProcessList should call this method 
		anytime the process list is rebuilt. */
	_restoreFocusedItem: function() {
		var item = this.getItem(this.currFocusIndex);
		if(item!=null && item['li']!=null) {
			var focusTarget = null;
			if(this.currFocusIsIcon) {
				focusTarget = dojo.query(".process-icon", item['li'])[0];
			}
			else {
				focusTarget = dojo.query(".process-text", item['li'])[0];
			}
			if(focusTarget!=null) {
				focusTarget.focus();
			}
		}
	},
	
	/* _onProcessIconClicked: Called whenever an item's icon is clicked. The 
		params argument is provided when the item is created and should hold 
		whatever information is required to properly handle the event. */
	_onProcessIconClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onProcessIconClicked: Called whenever an item's text is clicked. The 
		params argument is provided when the item is created and should hold 
		whatever information is required to properly handle the event. */
	_onProcessTextClicked: function(index, li, params) {}, //OVERRIDE ME!
	
	/* _onItemMouseOver: Called whenver an item is moused over. Overridden to 
		set the process icon's class to an appropriate css sprite class. */
	_onItemMouseOver: function(index, li, params) {
		var icon = dojo.query(".process-icon", li)[0];
		dojo.removeClass(icon, params['iconClass']);
		dojo.addClass(icon, params['iconClassHighlight']);
	},
	
	/* _onItemMouseOut: Called whenver an item is moused out. Overridden to 
		set the process icon's class to an appropriate css sprite class. */
	_onItemMouseOut: function(index, li, params) {
		var icon = dojo.query(".process-icon", li)[0];
		dojo.removeClass(icon, params['iconClassHighlight']);
		dojo.addClass(icon, params['iconClass']);
	},
	
	/* clearList: Make sure to clear extra event connectors. */
	clearList: function() {
		this.inherited(arguments);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.processListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
	},
	
	destroy: function(finalize) {
		this.inherited(arguments);
	}
	
});



}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.LaunchableProcessList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.LaunchableProcessList"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.LaunchableProcessList");


















/* LaunchableProcessList - Extends ProcessList. Displays a list of processes 
	that have been exposed to the user. Clicking an item will launch a new 
	instance of the process.
	
	Created: November 2011
*/
dojo
  .declare(
    "com.ibm.bpm.social.widgets.LaunchableProcessList",
    [com.ibm.bpm.social.widgets.ProcessList,
     com.ibm.bpm.social.widgets.common._RestWidget,
     dijit._Templated],
    {

      templateString: "",
      templateString:"<div class=\"bpm-social-smartlist bpm-social-processlist\">\n\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t<ul class=\"process-list\" dojoAttachPoint=\"listElement\"><li style=\"display:none\">&nbsp;</li></ul>\n</div>\n",
      constructor: function(){
        this.showProcessTopic = "defaultShowProcessTopic";
        this.showTaskTopic = "defaultShowTaskTopic";
        this.widgetsToCleanUp = [];
        this.processListEventConnectors = [];
        this.menuHasBeenHovered = false;

        this.systemRestHandler = com.ibm.bpm.social.util.common.SystemRestHandler.getInstance();
        this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
      },

      postCreate: function(){
        this.inherited(arguments);

        
        this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

        //set a message to appear when the list is empty
        this.setEmptyMessage(this.messages['launchableProcessList-empty']);

        //LaunchableProcessList uses the parent's template, but adds a class 
        // so styles can be applied specifically to it (such as for cogs)
        dojo.addClass(this.domNode, "bpm-social-launchableprocesslist");

        //get the user profile and preferences
        this.currentUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
      },

      /* refresh: Inherited from SmartList. Updates the information displayed in 
      	the widget by requesting a new set of data, clearing the list, and 
      	repopulating it. */
      refresh: function(){
        dojo.attr(this.listElement, "aria-busy", "true");

		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		this._doRefresh(exposedItemsArray);
      },
      _doRefresh: function(data){
		this.itemList = new com.ibm.bpm.social.widgets.ExposedItemsList();

        this.cleanUpMenusAndWidgets();
		var items = this.itemList.getData(data,["processOrService"]);

        //add new DOM elements using the given data
        dojo.forEach(items, dojo.hitch(this, function(item){
          this.addProcessItem(item.title, item.params);
        }));

        //restore focus to the item the keyboard-only user was focused on
        this._restoreFocusedItem();

        dojo.attr(this.listElement, "aria-busy", "false");
      },

      /* addProcessItem: Adds an item to the ProcessList. Inherited from 
      	ProcessList. */
      addProcessItem: function(title, params){
        params['iconClass'] = 'bpm-social-gray-cog';
        params['iconClassHighlight'] = 'bpm-social-white-cog';
        this.inherited(arguments);
      },

      /* _createItem: Handles creating each process item's HTML elements. Params 
      	is provided when addProcessItem is called. */
      _createItem: function(index, li, params, createParams){
        //create the HTML elements
        if (document.body.dir == 'rtl') {
          this._createItemNameElement(index, li, params, createParams);
          this._createLaunchIconElement(index, li, params, createParams);
        }
        else {
          this._createLaunchIconElement(index, li, params, createParams);
          this._createItemNameElement(index, li, params, createParams);
        }
      },

      _createLaunchIconElement: function(index, li, params, createParams){
        var icon = dojo.create("div", {
          'class': "process-icon " + params['iconClass'],
          'tabIndex': "0",
          'aria-label': "Launch process " + createParams['title'],
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, li);
        dijit.setWaiRole(icon, "button");
        this.processListEventConnectors.push(dojo.connect(icon, 'keypress', this, "_processIconKeyPressEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onclick', this, "_processIconClickEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onfocus', this, "_processIconFocusEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onblur', this, "_processIconBlurEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onmouseenter', this, "_processIconMouseoverEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onmouseleave', this, "_processIconMouseoutEvent"));
      },

      _createItemNameElement: function(index, li, params, createParams){
        var title = "<i>Unknown</i>";
        if (createParams != null && typeof (createParams) != 'undefined') {
          title = createParams['title'];
        }
        // If there are snapshots for this item, the tip name will be inline
        var text = dojo.create("a", {
          'class': "process-text",
          'innerHTML': title,
          'aria-label': "Choose a snapshot to launch for process " + title + "."
        }, li);
        dijit.setWaiRole(text, "link");

        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text);
		
        // If there are snapshots for this item, create a dropdown menu to contain them
        if (params.snapshotItems && params.snapshotItems.length > 0) {
          var menuIcon = dojo.create("img", {
            'class': "bpm-social-white-drop-arrow",
            'border': "none",
            'tabIndex': "0",
            'alt': "",
            'id': "id_" + params.baseID
          }, text);
          var menuIconAltText = dojo.create("span", {
            'class': "hi-contrast-icon dropdown-text",
            'innerHTML': "v",
            'id': "id_alttext_" + params.baseID

          }, text);
          menuIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
          dijit.setWaiRole(menuIcon, "button");
		  var menuDropdown = new dijit.Menu({
			"style": "display:none",
            leftClickToOpen: true
          });
          menuDropdown.bindDomNode(menuIcon);		  
          params["menuDropdownWidget"] = menuDropdown;
          this.widgetsToCleanUp.push(menuDropdown);
          dojo.addClass(menuDropdown.domNode, "processPortalDropDownMenu");

          var connectFunc = dojo.connect(menuDropdown.domNode, "onmouseenter", this, function(evt){
            this._onMenuDropdownHovered(menuDropdown, evt);
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuDropdown.domNode, "onmouseleave", this, function(evt){
            this._onMenuDropdownUnhovered(menuDropdown, evt);
          });
          this.processListEventConnectors.push(connectFunc);

          connectFunc = dojo.connect(menuIcon, 'onmouseover', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIcon, 'onclick', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIcon, 'keypress', this, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              menuDropdown._scheduleOpen(evt.target, null);
              this.menuHasBeenHovered = false;
            }
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'onmouseover', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'onclick', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'keypress', this, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              menuDropdown._scheduleOpen(evt.target, null);
              this.menuHasBeenHovered = false;
            }
          });
          this.processListEventConnectors.push(connectFunc);

          for ( var i = 0; i < params.snapshotItems.length; i++) {
            this._addSnapshotMenuItem(menuDropdown, params.snapshotItems[i].params["snapshotName"], params.snapshotItems[i]);
          }
          menuDropdown.startup();
        }
        this.processListEventConnectors.push(dojo.connect(text, 'keypress', this, "_processTextKeyPressEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onclick', this, "_processTextClickEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onfocus', this, "_processTextFocusEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onblur', this, "_processTextBlurEvent"));
      },

      _onMenuDropdownHovered: function(menuDropdown, evt){
        this.menuHasBeenHovered = true;
      },
      _onMenuDropdownUnhovered: function(menuDropdown, evt){
        this.menuHasBeenHovered = false;
        dijit.popup.close(menuDropdown);
      },
      _onItemMouseOut: function(index, li, params){
        this.inherited(arguments);
        if (params["menuDropdownWidget"]) {
          var menu = params["menuDropdownWidget"];
          window.setTimeout(dojo.hitch(this, function(){
            if (!this.menuHasBeenHovered) dijit.popup.close(menu);
          }), 750);
        }
      },

      _addSnapshotMenuItem: function(menuDropdown, display, params){
        // params is the snapshot information
        if (display && params) {
          var self = this;
		  
		  var btd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(display);

          var item = new dijit.MenuItem({
            label: display,
            onClick: dojo.hitch(this, "_menuItemClick"),
            dir: btd
          });
          this.widgetsToCleanUp.push(item);
          menuDropdown.addChild(item);
          // for simplicity and because the snapshots are launched from a menu and not from
          // the launchable items list, keep an array of snapshot information for easy lookup
          // when a menu item is selected
          if (!this.snapshotMenuItems) {
            this.snapshotMenuItems = {};
          }

          this.snapshotMenuItems[item.id] = {
            "display": display,
            "params": params
          };

        }
      },

      _menuItemClick: function(evt){
        var itemData = this.snapshotMenuItems[evt.currentTarget.id];
        this._onProcessItemSelected(itemData.params.params);
      },

      /* _onProcessIconClicked: Called when a cog is clicked. When clicked, 
      	launch a new instance of the process, or publish message for task template. Same functionality as 
      	_onProcessTextClicked.  */
      _onProcessIconClicked: function(index, li, params){
        this._onProcessItemSelected(params);
      },

      /* _onProcessTextClicked: Called when the text for an item is clicked. 
      	When clicked, launch a new instance of the process. Same functionality 
      	as _onProcessiconClicked. */
      _onProcessTextClicked: function(index, li, params){
        this._onProcessItemSelected(params);
      },

      _processTextKeyPressEvent: function(evt){
        // if user pressed on img, don't handle that here
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user pressed on img, don't handle that here
        if (theTag != "img") {
          if (evt.keyCode == dojo.keys.ENTER) {
            var info = this._extractItemInfoFromEvent(evt);
            if (info['index'] != null && info['li'] != null) {
              this._onProcessTextClicked(info['index'], info['li'], info['params']);
            }
          }
        }
      },

      _processTextClickEvent: function(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user clicked on img, don't handle that here
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
          if (info['index'] != null && info['li'] != null) {
            this._onProcessTextClicked(info['index'], info['li'], info['params']);
          }
        }
      },

      _onProcessItemSelected: function(params){
        if (params.type == "exposedprocess") {
          this._launchProcess(params["startURL"]);
        }
        else if (params.type == "tasktemplate") {
          this._openTaskTemplateCoach(params["baseID"]);
        }
      },

      /* _launchProcess: Called when an item is clicked. Launch a new instance 
      	of the process. Once the process is launched the user will either be 
      	taken to a coach or will receive a notification. */
      _launchProcess: function(startURL){
        //make a rest call to launch the process
        this.processRestHandler.launchProcess(startURL, dojo.hitch(this, this._doLaunchProcess), dojo.hitch(this, this._onLaunchProcessError), true, null);
      },
      _doLaunchProcess: function(data, xhrArgs){
        if (typeof (data) == 'undefined' || data == null || typeof (data.data) == 'undefined' || data.data == null ||
          typeof (data.data.tasks) == 'undefined' || data.data.tasks == null) {
          return;
        }

        var tasks = data.data.tasks;

        //check to see if the task is assigned to the last user in the lane
        if (this.currentUser != null) {
          for ( var i = 0; i < tasks.length; i++) {
            if (tasks[i].owner == this.currentUser.userName) {
              //if so, open it and let the user fill out the coach
              this._openProcessCoach(tasks[i]["tkiid"]);
              return;
            }
          }
        }

        //otherwise just display a notification that the process started
        this._displayProcessNotification(data.data.piid, data.data.name, tasks.length == 0);
      },
      _onLaunchProcessError: function(data, xhrArgs){
        var errMsg = "";
        if (xhrArgs && xhrArgs.xhr && typeof (xhrArgs.xhr.responseText) != 'undefined') {
          var responseTextJson = dojo.fromJson(xhrArgs.xhr.responseText);
          var errMsg = "<div>" + responseTextJson.errorMessage + "</div>";
        }
        var message = new com.ibm.bpm.social.Aether({
          'message': "<div>" + this.messages["launchableprocesslist-launchProcessFailed"] + "</div>" + errMsg,
          'containerDivId': "messageNotificationDiv",
          'position': 'top-right',
          'fade': true,
          'parent': this,
          'displayCloseIcon': true
        });
      },

      /* _openProcessCoach: Called when a process is launched and the user needs 
      	to fill out a coach. */
      _openProcessCoach: function(taskId){
        dojo.publish(this.showTaskTopic, [{
          "taskId": taskId
        }]);
      },

      /* _openTaskTemplateCoach: Called when a human service is launched and the 
      	user needs to fill out a coach. */
      _openTaskTemplateCoach: function(baseID){
        dojo.publish(this.showTaskTopic, [{
          "taskTemplateId": baseID
        }]);
      },

      /* _displayProcessNotification: Called when a process is launched and the 
      	user isn't responsible for filling it out. */
      _displayProcessNotification: function(processID, processName, hideLink){
        if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) { // Prevent notification if user is using Safari on iPad but allow if Chrome on iPad
          return
        }  
        
        if (typeof (hideLink) == 'undefined') {
          hideLink = false;
        }
        var link = processName;
        if (link == null || link.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
          link = this.messages["launchableprocesslist-defaultProcess"].replace("${id}", processID);
        }
        link = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(link);
        if (!hideLink) {
          link = "<a href='#' dojoattachevent='onclick: _handleActionClick'>" + link + "</a>";
        }
        var msg = this.messages['launchableProcessList-started'].replace("${process}", link);

        var message = new com.ibm.bpm.social.Aether({
          'message': "<div>" + msg + "</div>",
          'containerDivId': "messageNotificationDiv",
          'position': 'top-right',
          'fade': true,
          'parent': this,
          'fireEvent': dojo.hitch(this, function(processID){
            dojo.publish(this.showProcessTopic, [{
              "instanceId": processID
            }]);
          }, processID),
          'displayCloseIcon': true
        });
      },

      _onXhrErrors: function(data, xhrArgs){
        if (this.listElement) {
          dojo.attr(this.listElement, "aria-busy", "false");
        }
        console.debug(["LaunchableProcessList experienced an error.",
                       data,
                       xhrArgs]);
      },

      /* clearList: Make sure to clear extra event connectors. */
      clearList: function(){
        this.inherited(arguments);
        this.cleanUpConnects();
      },

      cleanUpConnects: function(){
        //disconnect all dojo event connectors
        if (this.processListEventConnectors) {
          dojo.forEach(this.processListEventConnectors, function(connector){
            dojo.disconnect(connector);
          });
        }
        this.processListEventConnectors = [];
      },

      cleanUpMenusAndWidgets: function(){
        if (this.widgetsToCleanUp && this.widgetsToCleanUp.length > 0) {
          for ( var i = 0; i < this.widgetsToCleanUp.length; i++) {
            if (this.widgetsToCleanUp[i].destroyRecursive) {
              this.widgetsToCleanUp[i].destroyRecursive();
            }
            else if (this.widgetsToCleanUp[i].destroy) {
              this.widgetsToCleanUp[i].destroy();
            }
          }
        }
        this.widgetsToCleanUp = [];
        if (this.snapshotMenuItems) {
          delete this.snapshotMenuItems;
        }
        this.cleanUpConnects();
      },

      destroy: function(finalize){
        this.cleanUpMenusAndWidgets();
      }

    });

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.UnfollowDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.UnfollowDialog"] = true;
/* 
	 Licensed Materials - Property of IBM
	 5725-C96
	 Copyright IBM Corporation 2012. All Rights Reserved.
	 US Government Users Restricted Rights- Use, duplication or disclosure
	 restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.UnfollowDialog");



dojo.declare(
	"com.ibm.bpm.social.widgets.dialogs.UnfollowDialog", 
	[com.ibm.bspace.common.util.ui.Dialog], {
	
	templateString: '<div dojoAttachPoint="container"></div>',
	
	preamble: function(params) {
    	
		this.unfollowDialogMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		if(typeof(params) == 'undefined' || params == null) {
			params = {};
		}
		params.title = this.unfollowDialogMessages["followedProcessList-unfollowDialogTitle"];
		return [params];
	},
	
	constructor: function() {
		this.currInstanceId = null;
		this.parentWidget = null;
	},            	
	
    postCreate: function() {
		this.inherited(arguments);
		
		this.container.className = "bpm-task-dialog-container";
		this.container.title = ""; // Unset the title, else we see the tooltip from the dialog on all controls that don't have a title
		
		this.description = document.createElement("div");
		this.description.innerHTML = this.unfollowDialogMessages["followedProcessList-unfollowDialog"];
		this.description.className = "bpm-task-dialog-description";
		this.container.appendChild(this.description);
		
		var t = document.createElement("table");
		var tb = document.createElement("tbody");
		var tr = document.createElement("tr");
		var td1 = document.createElement("td");
		var td2 = document.createElement("td");
		
		t.appendChild(tb);
		tb.appendChild(tr);
		tr.appendChild(td1);
		tr.appendChild(td2);
		
		this.container.appendChild(t);
		this.dontShowAgainCheckbox = document.createElement("input");
		this.dontShowAgainCheckbox.type = "checkbox";
		td1.appendChild(this.dontShowAgainCheckbox);
		
		this.dontShowAgainMessage = document.createElement("span");
		this.dontShowAgainMessage.innerHTML = this.unfollowDialogMessages["followedProcessList-dontShowAgain"];
		
		td2.appendChild(this.dontShowAgainMessage);
		
		this.unfollowButtonDiv = document.createElement("div");
		this.unfollowButtonDiv.className = "bpm-task-dialog-claimButtonDiv";
		this.container.appendChild(this.unfollowButtonDiv);
		
		this.unfollowButton = document.createElement("button");
		this.unfollowButton.innerHTML = this.unfollowDialogMessages["followedProcessList-unfollowButton"];
		this.unfollowButton.title = this.unfollowDialogMessages["followedProcessList-unfollowHover"];
		this.unfollowButtonDiv.appendChild(this.unfollowButton);
		
		
		this.connect(this.unfollowButton, "onclick", "_onUnfollowConfirmed");
		
		com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this.dontShowAgainCheckbox);
		com.ibm.bspace.common.util.ui.Button.wrapElement(this.unfollowButton);
		
		this.set("content", this.container);
	},
	
	showConfirmationDialog: function showConfirmationDialog_ClaimDialog(instanceId, widget) {
		this.currInstanceId = null;
		this.parentWidget = null;
		if(typeof(widget) != "undefined" && widget != null && typeof(instanceId) != "undefined" && instanceId != null) {
			this.currInstanceId = instanceId;
			this.parentWidget = widget;
			// Reset the combo box to unchecked before displaying the dialog.
			com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.dontShowAgainCheckbox, false);
			this.show();
		}
	},
	
	_onUnfollowConfirmed: function _onClaimConfirmed_TasksListController() {
		// Hide the dialog
		this.hide();
		
		// Set global user preferences if necessary
		if(this.dontShowAgainCheckbox.checked === true) {
			com.ibm.bpm.socialportal.GlobalAssets.setUserPreference('Show Unfollow Messages', "false");
		}
		
		if(this.parentWidget != null && this.currInstanceId != null) {
			this.parentWidget.unfollow(this.currInstanceId);
		}
	},
	
	_unfollow: function _claim_TasksListController() {
		/*com.ibm.bpm.social.util.common.Log.logEntry(2);
		var claimed = false;
		var self = this;
		this._restHandler.claimTask(this.taskId, function(responseObject, ioArgs){
			claimed = true;
		}, function(data, ioArgs){
			if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status === 409 && ioArgs.xhr.responseText) {
				var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
				if (response.errorNumber === "CWTBG0014E") {
					try {
						// Check whether the task has been claimed since last list refresh
						// Check if user is allowed to complete
						// Refresh available actions for this item synchronously					
						var availableActions = [];
						self._restHandler.getActionsForTask(self.taskId, ["ACTION_COMPLETE"], function(responseObject, ioArgs){
							var data = response;
							if (data.tasks.length === 1) {
								availableActions = data.tasks[0].actions;
							}
						}, null, self.systemId);
						if (availableActions) {
							if (dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1) {
								// continue normal processing
								claimed = true;
							}
						}
					}
					catch (e) {
						// This is just logic for checking whether an error condition can be ignored.
						// Therefore ignore any errors either.
					}
				}
			}
			if (!claimed) {
				// handle errors
				var name = self.displayName;
				var statusCodeMap = {
					409: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					401: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					403: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					404: {
						key: "KEY_ClaimNotExistingTask",
						parameters: {
							1: name
						}
					}
				};
				var errorCodeMap = {
					"CWTKA0072E": {
						key: "KEY_ClaimSuspendedTask",
						parameters: {
							1: name
						}
					}
				};
				
				self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
			}
		}, this.systemId ? this.systemId : null);
		com.ibm.bpm.social.util.common.Log.logExit(2, claimed);
		if (claimed) {
			this.onClaimed(this.taskId);
		}
		else {
			this.onFailed(this.taskId);
		}*/
	},
	
	destroy: function(){
	    if (this._confirmClaimDialog) {
		    if (this._confirmClaimDialog.dontShowAgainCheckbox) {
				com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._confirmClaimDialog.dontShowAgainCheckbox);
			}
			if (this._confirmClaimDialog.unfollowButton) {
				com.ibm.bspace.common.util.ui.Button.unwrapElement(this._confirmClaimDialog.unfollowButton);
			}
		}	
		this.inherited("destroy", arguments);
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.FollowedProcessList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.FollowedProcessList"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.FollowedProcessList");











/* FollowedProcessList - Extends ProcessList. Displays a list of processes 
	that the user has marked as following. Clicking an item will take the user 
	to the task details for that process. Clicking an icon will remove it from 
	the list.
	
	Updated: 2/6/2012
	Experts: Ben Myers @ Raleigh, Scott Johnson @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.FollowedProcessList",
	[com.ibm.bpm.social.widgets.ProcessList, com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class=\"bpm-social-smartlist bpm-social-processlist\">\n\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t<ul class=\"process-list\" dojoAttachPoint=\"listElement\"><li style=\"display:none\">&nbsp;</li></ul>\n</div>\n",
	constructor: function() {
		this.sortBy = "instanceDueDate"; //database column to order the list by
		
		//the topic that will be published to when a task is clicked - the 
		// expected behavior is this will take the user to the task details
		this.showProcessTopic = "defaultProcessListTopic";
		
		//the topic to subscribe to for refreshes
		this.refreshTopic = "defaultRefreshTopic";
		
		//the topic to subscribe to when the user profile preferences change
		this.profileChangedTopic = "defaultProfileChangedTopic";
		this.profilePreferences = null;
		
		this.followedProcessListSubscriptions = [];
		
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
	},            	
	
    postCreate: function() {
		this.inherited(arguments);
		
		//localization
    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//set a message to appear when the list is empty
		this.setEmptyMessage(this.messages['followedProcessList-empty']);
		
		//FollowedProcessList uses the parent's template, but adds a class so 
		// styles can be applied specifically to it (such as for stars)
		dojo.addClass(this.domNode, "bpm-social-followedprocesslist");
		
		//get the user profile and preferences
		this.profilePreferences = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if(this.profilePreferences != null) {
			this.profilePreferences = this.profilePreferences.userPreferences;
		}
		
		//subscribe to refresh channel and refresh when received
		this.followedProcessListSubscriptions.push(dojo.subscribe(this.refreshTopic, dojo.hitch(this, function(data) {
			this.refresh();
		})));
		
		//subscribe to profile changes and update profile when received
		this.followedProcessListSubscriptions.push(dojo.subscribe(this.profileChangedTopic, dojo.hitch(this, function(data) {
			this.profilePreferences = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			if(this.profilePreferences != null) {
				this.profilePreferences = this.profilePreferences.userPreferences;
			}
		})));
		
		//create the unfollow dialog
		this.dialog = new com.ibm.bpm.social.widgets.dialogs.UnfollowDialog();
	},
    
	/* _createItem: Inherited from ProcessList. Adds a dojo tooltip to the 
		list's stars. */
	_createItem: function(index, li, params, createParams) {
		this.inherited(arguments);
		//add tooltip
		var icon = dojo.query(".process-icon", li)[0];
		dojo.attr(icon, "title", this.messages["followedProcessList-unfollowHover"]);
		dojo.attr(icon, "aria-label", "Stop following process "+createParams['title']);
		dojo.attr(icon, 'innerHTML', '<span class="hi-contrast-icon">&#9733;</span>');
		var text = dojo.query(".process-text", li)[0];
		dojo.attr(text, "tabindex", "0");
		dojo.attr(text, "aria-label", "Go to process "+createParams['title']);
		
		//configure base text direction here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text);
	},
	
	/* refresh: Inherited from SmartList. Updates the information displayed in 
		the widget by requesting a new set of data, clearing the list, and 
		repopulating it. */
    refresh: function() {
		
		this.socialRestHandler.getFollowedInstances(dojo.hitch(this, this._doRefresh),
			dojo.hitch(this, this._onXhrErrors), true);
    },
    _doRefresh: function(data, xhrArgs) {
    	
		//clear all the DOM elements from the ProcessList
		this.clearList();
		
		//add new DOM elements using the given data
		var items = data.data.data;
    	dojo.forEach(items, dojo.hitch(this, function(item) {
			if(item.instanceName == null || item.instanceName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
				this.addProcessItem(this.messages["followedProcessList-defaultProcess"].replace("${id}", item.instanceId), {'instanceId': item.instanceId});
			}
			else {
				this.addProcessItem(item.instanceName, {'instanceId': item.instanceId});
			}
		}));
		
		//restore focus to the item the keyboard-only user was focused on
		//this._restoreFocusedItem();
		
		//dojo.attr(this.listElement, "aria-busy", "false");
    },
	
	/* addProcessItem: Adds an item to the ProcessList. Inherited from 
		ProcessList. */
	addProcessItem: function(title, params) {
		params['iconClass'] = 'bpm-social-followed-star';
		params['iconClassHighlight'] = 'bpm-social-followed-star';
		this.inherited(arguments);
	},
	
	/* _onProcessIconClicked: Called when a star is clicked. When clicked, 
		unfollow the item. */
	_onProcessIconClicked: function(index, li, params) {
		var instanceId = params['instanceId'];
		this.unfollow(instanceId);
	},
	
	/* _onProcessTextClicked: Called when the text for an item is clicked. 
		When clicked, take the user to the details for that process. */
	_onProcessTextClicked: function(index, li, params) {
		//publish a dojo topic to display the details
		if(params!=null && params['instanceId']!=null) {
			dojo.publish(this.showProcessTopic, [{'instanceId': params['instanceId']}]);
		}
	},
	
	/* unfollow: Called when a star is clicked. Remove the item from the list 
		and mark it as no longer being followed. */
	unfollow: function(instanceId) {
		if(instanceId!=null) {
			//make a rest call to unfollow the process
			
			this.socialRestHandler.unfollowInstance(instanceId, 
				dojo.hitch(this, this.refresh),
				dojo.hitch(this, function(data, xhrArgs) {
					this.refresh(); 
					this._onXhrErrors(data, xhrArgs);
				}),
				true);
		}
    },
    
    _onXhrErrors: function(data, xhrArgs) {
		if(this.listElement) {
			dojo.attr(this.listElement, "aria-busy", "false");
		}
    	console.debug(["FollowedProcessList experienced an error.", data, xhrArgs]);
    },
	
	destroy: function(finalize) {
		for(var i=0; i<this.followedProcessListSubscriptions.length; i++) {
			dojo.unsubscribe(this.followedProcessListSubscriptions[i]);
		}
		this.dialog.destroy();
		this.inherited(arguments);
	}
    
});



}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.TaggedProcessList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.TaggedProcessList"] = true;
/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.TaggedProcessList");



















/* TaggedProcessList - Extends SmartList. Displays posts in which the user was
	tagged. Clicking a link will take the user to an activity stream in most
	cases. Note that although this is named TaggedPROCESSLIST, it actually
	doesn't extend ProcessList - it extends SmartList. This is because, due to
	design changes, it no longer displays a list of processes. It's more like
	a list of posts linking to processes. Note also that "tagged" is also
	referred to as "mentioned" and this widget may be more properly labeled a
	"Mentions" list at the end of the day.
	
	Created: October 2011
	Experts: Ben Myers @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.TaggedProcessList",
	[com.ibm.bpm.social.widgets.SmartList, com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class=\"bpm-social-smartlist bpm-social-taggedprocesslist\">\n\t<p class=\"empty-message\" dojoAttachPoint=\"emptyMessageParagraph\"></p>\n\t<p class=\"clear-list-message\" dojoAttachPoint=\"clearListMessage\">\n\t\t<a tabindex=\"0\" dojoAttachPoint=\"clearListLink\"></a>\n\t\t<span class=\"clear-list-icon bpm-social-close-x\" dojoAttachPoint=\"clearListIcon\"></span>\n\t</p>\n\t<ul dojoAttachPoint=\"listElement\"><li style=\"display:none\">&nbsp;</li></ul>\n\t<p class=\"show-more-message\">\n\t\t<a tabindex=\"0\" dojoAttachPoint=\"showMoreLink\"></a>\n\t\t<div dojoAttachPoint=\"loadingMessageIcon\" class=\"loading-icon\"></div>\n\t\t<span dojoAttachPoint=\"loadingMessageSpan\" class=\"loading-message\"></span>\n\t</p>\n</div>\n",
	_userProfileChangeSubscribeHandle: null,
	constructor: function() {
	
		//topics to subscribe to or publish to
		this.showProcessTopic = "defaultProcessListTopic";
		this.showTaskTopic = "defaultShowTaskTopic";
		this.refreshTopic = "defaultRefreshTopic";
		this.taggedProcessListSubscriptions = [];
		
		//event connectors are added to an array so they can be disconnected
		this.taggedProcessListEventConnectors = [];
		this.taggedProcessListStaticEventConnectors = []; //for the Clear All button, since it won't be destroyed on refresh
		
		//used for markTabAsNew
		this.parentTabNumber = null;
		this.parentContainer = null;
		
		//used for paging
		this.numItemsToRequest = 50;
		
		//the rest handler
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		// For unread message marker
		this.visible = false;
        this.animations = [];
		this.cachedPortalMentionsTimestamp = new Date(); 		
		this.portalMentionsTimestampAvailable = true; 		
		// To account for difference in clocks on the server and on the client:
		// It is important to only compare timestamps from the server.
		// Therefore we set the user attribute "Portal Mentions Timestamp" value to the published value of the most recent message
		// which is available to the client. Therefore we need to remember the most recent published date.
		this.mostRecentPublishedDate = null;
		
		this._newItems = [];
	},
	
    postCreate: function() {
		//hide certain elements
		dojo.style(this.showMoreLink, "display", "none");
		dojo.style(this.loadingMessageSpan, "display", "none");
		dojo.style(this.loadingMessageIcon, "display", "none");
		dojo.style(this.clearListLink, "display", "none");
		
		this.inherited(arguments);
		
		//localization
    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.taggedProcessListEventConnectors = [];
		this.taggedProcessListStaticEventConnectors = [];
		this.parentTabNumber = null;
		this.parentContainer = null;
		
		//set a message to appear when the list is empty
		this.setEmptyMessage(this.messages['taggedProcessList-empty']);
		
		//set the message for the "Clear all" link and add events to it
		dojo.attr(this.clearListLink, "innerHTML", this.messages['taggedProcessList-clearList']);
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onclick', this, "_clearAllClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'keypress', this, "_clearAllKeyPressEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x-highlight");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onclick', this, "_clearAllClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'keypress', this, "_clearAllKeyPressEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListLink, "linkHover");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x");
			dojo.removeClass(this.clearListLink, "linkHover");
		})));
		
		//set the message for the "Show more" link and add events to it
		dojo.attr(this.showMoreLink, "innerHTML", this.messages['taggedProcessList-showMore']);
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.showMoreLink, 'onclick', this, "_showMoreClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.showMoreLink, 'keypress', this, "_showMoreKeyPressEvent"));
		
		//set the message for the "Loading more..." message
		dojo.attr(this.loadingMessageSpan, "innerHTML", this.messages['taggedProcessList-loadingMore']);
		
		//subscribe to refresh channel and refresh when received
		this.taggedProcessListSubscriptions.push(dojo.subscribe(this.refreshTopic, dojo.hitch(this, function(data) {
			this.refresh();
		})));
		
		this.taggedProcessListSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED, dojo.hitch(this, function(data) {
			this.refresh();
		})));
		
		// subscribe to user profile changes
		if(this._userProfileChangeSubscribeHandle == null){
        	dojo.subscribe(com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, dojo
          	.hitch(this, this._handleUserProfileChange));
        }
    },
    
    _handleUserProfileChange: function(obj){
    	this.refresh();
    },
	
	/* addTaggedItem: Adds an item of the form "tagged you in a post". Called
		when the referesh() REST call gets data about users tagging the
		current user. */
	addTaggedItem: function(id, author, authorId, processId, processName, text, date) {
		//format the date if it is valid
		if(typeof(date)=='undefined' || date==null || date=='') {
			date = null;
		}
		else {
			date = com.ibm.bpm.social.util.common.Util.formatDate(date);
		}
		
		if(processName==null || processName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			processName = this.messages["taggedProcessList-defaultProcess"].replace("${id}", processId);
		}
		
		return this.addItem(
		//params - parameters for event handlers
		{	'id': id,
			'type': "tagged",
			'processId': processId,
			'processName': processName
		},
		//createParams - parameters for item creation
		{	'author': author,
			'authorId': authorId,
			'text': text,
			'date': date
		});
	},
	
	/* addCollaborationItem: Adds an item of the form "invited you to
		collaborate". Called when the referesh() REST call gets data about
		users wanting to collaborate with the current user. */
	addCollaborationItem: function(id, author, authorId, processId, processName, taskId, taskName, text, date) {
		//format the date if it is valid
		if(typeof(date)=='undefined' || date==null || date=='') {
			date = null;
		}
		else {
			date = com.ibm.bpm.social.util.common.Util.formatDate(date);
		}
		
		if(processName==null || processName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			processName = this.messages["taggedProcessList-defaultProcess"].replace("${id}", processId);
		}
		if(taskName==null || taskName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			taskName = this.messages["taggedProcessList-defaultTask"].replace("${id}", taskId);
		}
		
		return this.addItem(
		//params - parameters for event handlers
		{	'id': id,
			'type': "collaboration",
			'processId': processId,
			'processName': processName,
			'taskId': taskId,
			'taskName': taskName
		},
		//createParams - parameters for item creation
		{	'author': author,
			'authorId': authorId,
			'text': text,
			'date': date
		});
	},
	
	/* _onItemAdded: Called whenever an item is added. */
	_onItemAdded: function(index, li, params) {
		this.inherited(arguments);
		
		//display the "Clear All" option
		dojo.style(this.clearListLink, "display", "");
		dojo.style(this.clearListIcon, "display", "");
	},
	
	/* _onItemRemoved: Hide the "Clear All" link if the list is empty. */
	_onItemRemoved: function(index, li, params) {
		if(this.numItems()==0) {
			dojo.style(this.clearListLink, "display", "none");
			dojo.style(this.clearListIcon, "display", "none");
		}
	},
	
	/* _createItem: Handles creating each item's HTML elements. */
	_createItem: function(index, li, params, createParams) {
		dojo.attr(li, 'tabindex', "0");
		
		// add the picture of the user that tagged you		
	   this._avatar = new com.ibm.bpm.social.util.ui.Avatar();
       var user = com.ibm.bpm.social.util.common.Util.getUser(createParams.authorId);
	   li.appendChild(this._avatar.domNode);		

		//if the message is "tagged you in a post"...
		if(params.type == "tagged") {
         dojo.addClass(li, "bpm-social-taggedprocesslist-tagged");

         //check for user names in the text, then set the message accordingly
         // but make sure to not use the mentions text for other user then the logged in user.
         var owner= com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
         var messageText = "taggedProcessList-tagNotification";
         if (createParams.text && (createParams.text.indexOf("\"bpm-user-tag\"") < 0  ||
             createParams.text.indexOf("objId=\""+owner.userName+"\"") < 0)) {
           messageText = "taggedProcessList-tagNotificationComment";
         }
		   //create the tagged notification message
         var p =
            dojo.create("p", {
              'class': "bpm-social-taggedprocesslist-message",
              'innerHTML': dojo.string.substitute(this.messages[messageText], {
                'user': '<span class="bpm-social-taggedprocesslist-user">' + createParams.author + '</span>',
                'process': '<a class="bpm-social-taggedprocesslist-process" tabindex="0" aria-label="' + params.processName +
                  '">' + params.processName + '</a>'
               })
            }, li);
		   var userName = dojo.query("span.bpm-social-taggedprocesslist-user", p)[0];
		   this.taggedProcessListEventConnectors.push(dojo.connect(userName, 'onmouseenter', this, "_addTitleIfNecessary"));
		   var processLink = dojo.query("a.bpm-social-taggedprocesslist-process", p)[0];
		   this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'onclick', this, "_processLinkClickEvent"));
		   this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'keypress', this, "_processLinkKeyPressEvent"));

			this._createUnseenMarkerIcon(li);
		   
			//create the date & time
			if(createParams.date!=null) {	
				dojo.create("p", {
					"class": "bpm-social-taggedprocesslist-timestamp", 
					"innerHTML": createParams.date
				}, li);
			}
						
			if(createParams.text && dojo.trim(createParams.text) !== "") {
				//prevent cross site scripting
				var fixedText = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(createParams.text); 
			
	     	   	//create the post the user typed
				var p2 = dojo.create("p", {'class': "bpm-social-taggedprocesslist-text", 'innerHTML':
				dojo.string.substitute(
					this.messages["taggedProcessList-quotedText"], {
					'text': fixedText
				})
				}, li);
			
				//apply base text direction to the mentioned post
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(p2);
			
				var yourUserName = dojo.query("span.bpm-user-tag", p2)[0];
				if(yourUserName) {
					this.taggedProcessListEventConnectors.push(dojo.connect(yourUserName, 'onmouseenter', this, "_addTitleIfNecessary"));
				}
			}
		}
		
		//if the message is "invited you to collaborate"...
		else if(params.type == "collaboration") {
			dojo.addClass(li, "bpm-social-taggedprocesslist-collaboration");
			
			//create the collaboration invitation message
			var p = dojo.create("p", {'class': "bpm-social-taggedprocesslist-message", 'innerHTML':
				dojo.string.substitute(
					this.messages["taggedProcessList-collaborationInvitation"], {
					'user': '<span class="bpm-social-taggedprocesslist-user">'+createParams.author+'</span>',
					'task': '<a class="bpm-social-taggedprocesslist-task" tabindex="0">'+params.taskName+'</a>',
					'process': '<a class="bpm-social-taggedprocesslist-process" tabindex="0">'+params.processName+'</a>'
				})
			}, li);
			var userName = dojo.query("span.bpm-social-taggedprocesslist-user", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(userName, 'onmouseenter', this, "_addTitleIfNecessary"));
			var taskLink = dojo.query("a.bpm-social-taggedprocesslist-task", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(taskLink, 'onclick', this, "_taskLinkClickEvent"));
			this.taggedProcessListEventConnectors.push(dojo.connect(taskLink, 'keypress', this, "_taskLinkKeyPressEvent"));
			var processLink = dojo.query("a.bpm-social-taggedprocesslist-process", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'onclick', this, "_processLinkClickEvent"));
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'keypress', this, "_processLinkKeyPressEvent"));

			this._createUnseenMarkerIcon(li);
			
			//create the date & time
			if(createParams.date!=null) {
				dojo.create("p", {
					"class": "bpm-social-taggedprocesslist-timestamp",
					"innerHTML": createParams.date
				}, li);	
			}

			if(createParams.text && dojo.trim(createParams.text) !== "") {
				//prevent cross site scripting
				var fixedText = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(createParams.text);
			
				//create the post the user typed			
				var p2 = dojo.create("p", {'class': "bpm-social-taggedprocesslist-text", 'innerHTML':
					dojo.string.substitute(
						this.messages["taggedProcessList-quotedText"], {
						'text': fixedText
					})
				}, li);
				var yourUserName = dojo.query("span.bpm-user-tag", p2)[0];
				if(yourUserName) {
					this.taggedProcessListEventConnectors.push(dojo.connect(yourUserName, 'onmouseenter', this, "_addTitleIfNecessary"));
				}
			}
		}
		
		/** Business Card integration **/		
		if(typeof(user["fullName"])=="undefined" && typeof(createParams.author)!="undefined"){
			user["fullName"] = createParams.author;
		}
	        com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query("span.bpm-social-taggedprocesslist-user", p)[0],
	          user);
	        this._avatar.setUser(user, "bpm-social-taggedprocesslist-user-img bpm-social-default-avatar-medium", true);
		
		var bpmUserTags = dojo.query(".bpm-user-tag", li);
		for(var x = 0; x < bpmUserTags.length; x++){
			var objId = bpmUserTags[x].getAttribute("objId");
			if(typeof(objId)!="undefined" && objId!=null && objId!=""){
				user = com.ibm.bpm.social.util.common.Util.getUser(objId);
				com.ibm.bpm.social.widgets.common.BizCard.wrapElement(bpmUserTags[x], user);
			}
		}
		/** End Business Card integration **/
		
		//create the remove button
		var remove = dojo.create("span", {
			'style': "width: 12px; height: 12px",
			'class': "bpm-social-taggedprocesslist-closeButton bpm-social-close-x",
			'tabindex': "0",
			'title': this.messages["taggedProcessList-remove"],
			'innerHTML': '<span class="hi-contrast-icon">X</span>'
		}, li);
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onclick', this, "_removeClickEvent"));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'keypress', this, "_removeKeyPressEvent"));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.addClass(remove, "hover");
			dojo.removeClass(remove, "bpm-social-close-x");
			dojo.addClass(remove, "bpm-social-close-x-highlight");
		})));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(remove, "hover");
			dojo.removeClass(remove, "bpm-social-close-x-highlight");
			dojo.addClass(remove, "bpm-social-close-x");
		})));
	},
	
	/*
	 * Creates the element for the unseen mentions marker
	 */
	_createUnseenMarkerIcon: function(container) {
		var unseenMarker = dojo.create("div", {
			'class': 'bpm-social-taggedprocesslist-unseen-marker',
			'title': this.messages["taggedProcessList-newNotification"]
		}, container);
	    dijit.setWaiRole(unseenMarker, "img");
	    dijit.setWaiState(unseenMarker, "label", this.messages["taggedProcessList-newNotification"]);
		var a11yUnseenMarker = dojo.create("div", {"class": "bpm-social-taggedprocesslist-unseen-marker-a11y"}, unseenMarker);
		a11yUnseenMarker.innerHTML = "&#42;";
	},

	/* _addTitleIfNecessary: Adds a title to a span if its content is too long. */
	_addTitleIfNecessary: function(e) {
		var node = e.target;
		// check if the mouse is over a user tag
		if (node && (dojo.hasClass(node,'bpm-user-tag') || dojo.hasClass(node, 'bpm-social-taggedprocesslist-user'))) {
			// if the user name is clipped, provide hover help
			if (node.offsetWidth > node.parentNode.offsetWidth) {
				node.title = node.innerHTML;
			}
		}
	},
	
	/* Below are event handlers that do little other than handle basic
		interactions with the TaggedProcessList and usually end up calling
		_onProcessLinkClicked or _onTaskLinkClicked. */
	_processLinkClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onProcessLinkClicked(info['index'], info['li'], info['params']);
		}
	},
	_taskLinkClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onTaskLinkClicked(info['index'], info['li'], info['params']);
		}
	},
	_removeClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onRemoveClicked(info['index'], info['li'], info['params']);
		}
	},
	_clearAllClickEvent: function(evt) {
		this._onClearAllClicked();
	},
	_showMoreClickEvent: function(evt) {
		this._onShowMoreClicked();
	},
	_processLinkKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onProcessLinkClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_taskLinkKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onTaskLinkClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_removeKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onRemoveClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_clearAllKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			this._onClearAllClicked();
		}
	},
	_showMoreKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			this._onShowMoreClicked();
		}
	},
	
	/* _onProcessLinkClicked: Called whenever a process link is clicked.
		Should redirect the user to the process details page. */
	_onProcessLinkClicked: function(index, li, params) {
		if(params!=null && params['processId']!=null) {
			dojo.publish(this.showProcessTopic, [{'instanceId':  params['processId']}]);
		}
	},
	
	/* _onTaskLinkClicked: Called whenever a task link is clicked. Should
		redirect the user to the task details page if collaboration session isn't full. */
	_onTaskLinkClicked: function(index, li, params) {
		if(params!=null && params['taskId']!=null) {
			this._checkCollaborationSessionAvailablity(params['taskId']);
		}
	},
	
	/* _checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
    _checkCollaborationSessionAvailablity: function(taskID) {
		// Response will look something like this:
		// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
		if (taskID) {
			/* /bpm/notification/roster/task/ */
			dojo.xhrGet({
				url: this.getProxyUrl() +window.endpointURLObject.processPortalNotificationJS +"/roster/task/"+taskID,
				handleAs: "json",
				preventCache: true,
				sync: true,
				headers: {
					"Content-Type": "application/json",
					"Accept": "application/json"
				},
				load: dojo.hitch(this, function(data, xhrArgs) { this._checkNumberOfParticipants(data, xhrArgs, taskID); }),
				error: dojo.hitch(this, this._onXhrErrors)
			});
		}
    },
	
	
	/* _checkNumberOfParticipants: See if there aren't too many participants in this task already. */
	_checkNumberOfParticipants: function (data, xhrArgs, taskID) {
		// data will look something like this:
		// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
		// or this:
		// {"message":"NO_SESSION_FOR_TASK"}
		if (!data) {
			return;
		}
		// If there's no collab session, then data won't contain an element called 'participants',
		// and in that case we go ahead and publish the show task topic.
		var doPublishShowTaskTopic = false;
		if (data.participants) {
			var participants = data.participants;
			var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			var currentUser;
			if (userProfileObj) {
				currentUser = userProfileObj.userName;
			}
			if (participants && (participants.length < 2||this._currentUserIsAlreadyInSession(currentUser,participants))) {
				// publish the show task topic
				doPublishShowTaskTopic=true;
			} else {
				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.messages.collaborationSessionFull, null /* details */, function(){
				  return;
				});
			}
		} else {
			doPublishShowTaskTopic = true;
		}
		if (doPublishShowTaskTopic) {
			// publish the show task topic
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{"taskId": taskID}]);
		}
	},
	
	
	_currentUserIsAlreadyInSession:function(currentUser, participants){
		if(currentUser&&participants){
			for(var i in participants){
				if(participants[i].userName == currentUser)
					return true;
			}
		
		}
		return false;	
	},
	
	/* _onRemoveClicked: Called whenever a remove button is clicked. Should
		permanently remove the item from the list. */
	_onRemoveClicked: function(index, li, params) {
		this.removeItem(index);
		this.numItemsToRequest--;
		if(this.numItemsToRequest <= 0) {
			this.numItemsToRequest = 50;
		}
		
		this.socialRestHandler.deleteMentions(params.id, dojo.hitch(this, function(data, xhrArgs) {
				if(this.numItems()==0) {
					this.refresh();
				}
			}), dojo.hitch(this, this._onXhrErrors), true);
	},
	
	/* _onClearAllClicked: Called whenever the Clear All link is clicked.
		Should permanently remove all visible items from the list. */
	_onClearAllClicked: function() {
		this.clearList();
		this.numItemsToRequest = 50;
		
		this.socialRestHandler.deleteAllMentions("", dojo.hitch(this, this._onXhrErrors), true);
	},
	
	/* _onShowMoreClicked: Called whenever the Show More link is clicked.
		Should show additional paged items if there are any. */
	_onShowMoreClicked: function() {
		dojo.style(this.showMoreLink, "display", "none");
		dojo.style(this.loadingMessageSpan, "display", "");
		dojo.style(this.loadingMessageIcon, "display", "");
		this.numItemsToRequest += 50;
		this.refresh();
	},
	
	/* refresh: Inherited from SmartList. Updates the information displayed in
		the widget by requesting a new set of data, clearing the list, and
		repopulating it. */
    refresh: function() {
		
		this.socialRestHandler.getMentions(1, this.numItemsToRequest,
			dojo.hitch(this, this._doRefresh), dojo.hitch(this, this._onXhrErrors), true);
	},
	
	_doRefresh: function(data, xhrArgs) {
		this.clearList();
		
		var items = data.items;
		var hasMoreItems = data.more;
		
		var mentionsTimestamp = this._getMentionsTimestamp();
        var unreadMessageCount = 0;
        this.animations = [];
		
		this.mostRecentPublishedDate = null;
		for(var i=0; i<items.length; i++) {
			var item = items[i];
			var actorData = com.ibm.bpm.social.util.common.Util.getUser(item.actor.id);
			var processId = null;
			var processName = this.messages['taggedProcessList-unknownProcess'];
			if(typeof(item.ibmbpmext)!='undefined') {
				//take the substring in case the process begins with "2072." or something similar
				processId = item.ibmbpmext.processId.substring(item.ibmbpmext.processId.indexOf('.')+1);
				processName = item.ibmbpmext.processName;
			}
			var index = null;
			if(item.verb=="POST") {
				index = this.addTaggedItem(
					item.object.id,
					item.actor.displayName,
					item.actor.id,
					processId,
					processName,
					item.ibmbpmext.content,
					item.published);
			}
			else if(item.verb=="INVITE") {
				var taskId = null;
				var taskName = this.messages['taggedProcessList-unknownTask'];
				if(typeof(item.ibmbpmext)!='undefined') {
					//take the substring in case the process begins with "2078." or something similar
					taskId = item.ibmbpmext.taskId.substring(item.ibmbpmext.taskId.indexOf('.')+1);
					taskName = item.ibmbpmext.taskName;
				}
				index = this.addCollaborationItem(
					item.object.id,
					item.actor.displayName,
					item.actor.id,
					processId,
					processName,
					taskId,
					taskName,
					item.ibmbpmext.content,
					item.published);
			}
			var published = dojo.date.stamp.fromISOString(item.published);
			if (i == 0) {
				// Assumption: Result Set of @Mentions call is ordered by published and "paging" is based on "more".
				// I.e. including all the messages starting with the most recent one.
				this.mostRecentPublishedDate = published;
			}
			
			var itemNode = this.getItem(index);

			// Check if the item is unseen
			if (mentionsTimestamp == null || dojo.date.compare(published, mentionsTimestamp) === 1) {
				unreadMessageCount++;
				dojo.addClass(itemNode.li, "bpm-social-taggedprocesslist-new-item");
				dojo.style(itemNode.li, "opacity", "0");
				// Add an animation for the new entry
				this.animations.push(dojo.fadeIn({
					node: itemNode.li,
					duration: 2000
				}));
				// Push the notification id to the newItems array (and not decorate the item node directly) to assure that
				// mentions keep the unseen marker in case that the @Mentions tab is open and new mentions fade in.
				this._newItems.push(item.object.id);
			}
			
			// If the item is unseen, decorate it with an icon
			if (dojo.indexOf(this._newItems, item.object.id) !== -1) {
				dojo.addClass(itemNode.li, "bpm-social-taggedprocesslist-new-item");
			}
		}
		//show the "Show More" link if more items exist
		if(hasMoreItems) {
			dojo.style(this.showMoreLink, "display", "");
		}
		else {
			dojo.style(this.showMoreLink, "display", "none");
		}
		dojo.style(this.loadingMessageSpan, "display", "none");
		dojo.style(this.loadingMessageIcon, "display", "none");
		
		if (this.visible && unreadMessageCount > 0) {
			this.updateMentionsTimestamp();
			dojo.fx.combine(this.animations).play();
			this.animations = [];
			unreadMessageCount = 0;
        } else if (unreadMessageCount == 0) {
			// In case of tracking clicks on the Aether items
			// one would need to update the stamp if the count stays on zero.
		}

		this._displayUnreadMessageCount(unreadMessageCount);		
	},
	
	onSelected: function(){
		this.onShow(true);
	},
	
	onDeSelected: function(){
		this.onHide(true);
	},
	
	onShow: function(selected) {
		this.visible = true;
		if (selected) {
			// Fallback mode without user property			
			this.cachedPortalMentionsTimestamp = this.mostRecentPublishedDate != null ? this.mostRecentPublishedDate : new Date();
		}		
        this.updateMentionsTimestamp();
        // Play any animations that are pending because the element was hidden before.
        dojo.fx.combine(this.animations).play();
        this.animations = [];
        this._displayUnreadMessageCount(0);
	},

	onHide: function(deSelected) {
		this.visible = false;
		
		// Remove unseen markers again
		var allItemNodes = this.getItems();
		for(var k = 0; k < allItemNodes.length; k++) {
			var itemNode = allItemNodes[k];
			dojo.removeClass(itemNode.li, "bpm-social-taggedprocesslist-new-item");			
		}
		this._newItems = [];
	},

	_getMentionsTimestamp: function(){
		var stamp = null;
		try {
			var preferences = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			if (preferences != null && this.portalMentionsTimestampAvailable) {
				if (typeof preferences["userPreferences"][com.ibm.bpm.social.widgets.TaggedProcessList.PORTAL_MENTIONS_TIMESTAMP] === "string") {
					stamp = dojo.date.stamp.fromISOString(preferences["userPreferences"][com.ibm.bpm.social.widgets.TaggedProcessList.PORTAL_MENTIONS_TIMESTAMP]);
					// toISOString removes the milliseconds => add milliseconds again
					// Try to recover milliseconds. Are they locally cached?
					if (this.cachedPortalMentionsTimestamp != null && dojo.date.difference(stamp,this.cachedPortalMentionsTimestamp,"second") == 0) {
						stamp = this.cachedPortalMentionsTimestamp;
					} else {
						// Guessing milliseconds...
						stamp = dojo.date.add(stamp,"millisecond",999);
					}
				}
			}
        }
        catch (e) {
			this.portalMentionsTimestampAvailable = false;
			com.ibm.bpm.social.util.common.Log.logObject(2,e);
        }
        return stamp;
	},

	updateMentionsTimestamp: function(){
		// Edge case: There are no @Mentions yet, use the actual Date and accept that there might be slight differences in clocks between client and server.
		var stamp = this.mostRecentPublishedDate != null ? this.mostRecentPublishedDate : new Date();
		try {
			if (this.portalMentionsTimestampAvailable) {
				com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.widgets.TaggedProcessList.PORTAL_MENTIONS_TIMESTAMP, dojo.date.stamp.toISOString(stamp));
			}
			// Fallback mode without user property and caching for milliseconds that are lost with toISOString			
			this.cachedPortalMentionsTimestamp = stamp;			
		} catch(e) {
			this.portalMentionsTimestampAvailable = false;
			com.ibm.bpm.social.util.common.Log.logObject(2,e);			
		}
	},

	_displayUnreadMessageCount: function(unreadMessageCount){
		try {
			//
			// TODO can we search the node using based on the template...?
			// TODO instead of using a convoluted CSS selector with dojo.query
			// TODO browser dependencies in dojo.query caused problems in the past...
			//
			var node = null;			
			if(document.body.dir == 'rtl') {
				node = dojo.query(".sidebarContent .tab-list li:first-child .tab-text");
			} else {
				node = dojo.query(".sidebarContent .tab-list li:nth-child(3) .tab-text");
			}
			
			if (node != null && node[0] != null) {
				node = node[0];
			}
			else {
				return;
			}
			//
			// Lookup the counter
			//
			var counterNode = document.getElementById("counterNode");
			if (counterNode == null) {
				counterNode = document.createElement("div");
				counterNode.setAttribute("id", "counterNode");
				counterNode.className = "unreadMentionsCounter";
				counterNode.title = this.messages["taggedProcessList-newMentionsAvailable"];
				counterNode.style.display = "none";
			    dijit.setWaiRole(counterNode, "alert");
			    dijit.setWaiState(counterNode, "label", this.messages["taggedProcessList-newMentionsAvailable"]);
				var container = node.parentNode.parentNode;
				container.appendChild(counterNode);
			}
			if (counterNode.childNodes.length > 0) {
				// Remove TextNode and add a new one
				counterNode.removeChild(counterNode.childNodes[0]);
			}
			
			var unreadMessageCountTxt = unreadMessageCount;
			
			// Counter is visible if and only if unreadMessageCounter != 0
			if (unreadMessageCount == 0) {
				counterNode.style.display = "none";
			}
			else {
				counterNode.style.display = "block";
				if(unreadMessageCount >=50) {
					unreadMessageCountTxt = this.messages["taggedProcessList-50+"];
				}
			}
			
			counterNode.appendChild(document.createTextNode(unreadMessageCountTxt));
        }
        catch (e) {			
			com.ibm.bpm.social.util.common.Log.logObject(2,e);
		}
	},
	
	/* claimControlofTab: Gives the TaggedProcessList a ProtalSidebar for it
		to call markTabAsNew on in the future. Should be set once after the
		widget is created. There is no harm in choosing not to use this
		function, other than *NEW* messages won't be displayed. */
	claimControlOfTab: function(container, tabNumber) {
		this.parentContainer = container;
		this.parentTabNumber = tabNumber;
	},
	
	/* markTabAsNew: If the TaggedProcessList is contained by a PortalSidebar,
		it will add a small *NEW* message to the tab containing it. This
		allows the current user to see when new messages have arrived even if
		not currently viewing the tab. */
	markTabAsNew: function() {
		if(this.parentContainer!=null && this.parentTabNumber!=null) {
			this.parentContainer.setTabTag(this.parentTabNumber, this.messages["taggedProcessList-newMessages"], true);
		}
	},
	
    _onXhrErrors: function(data, xhrArgs) {
		com.ibm.bpm.social.util.common.Log.logObject(2,[data, xhrArgs]);    	
    },
	
	/* clearList: Make sure to clear extra event connectors. */
	clearList: function() {
		this.inherited(arguments);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.taggedProcessListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		//hide the "Clear All" option
		dojo.style(this.clearListLink, "display", "none");
		dojo.style(this.clearListIcon, "display", "none");
		dojo.style(this.showMoreLink, "display", "none");
	},
	
	destroy: function(finalize) {
		//disconnect all dojo event connectors
		dojo.forEach(this.taggedProcessListStaticEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		for(var i=0; i<this.taggedProcessListSubscriptions.length; i++) {
			dojo.unsubscribe(this.taggedProcessListSubscriptions[i]);
		}
	        if (this._avatar) {
	          this._avatar.destroy();
	          this._avatar = null;
	        }
		if (this._userProfileChangeSubscribeHandle) {
			dojo.unsubscribe(this._userProfileChangeSubscribeHandle);
			this._userProfileChangeSubscribeHandle = null;
		}
		this.inherited(arguments);
	}
	
});

com.ibm.bpm.social.widgets.TaggedProcessList.PORTAL_MENTIONS_TIMESTAMP = "Portal Mention Timestamp"; // "mentionsTimestamp";

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.List"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.List"] = true;
// List.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.list.List");











dojo.declare("com.ibm.bpm.social.widgets.task.list.List", [dijit._Widget,
                                                           dijit._Templated], {

  FADING_DURATION: 2000,

  templateString: '<div class="bpm-task-list" dojoAttachPoint="itemList"></div>',

  widgetsInTemplate: false,

  model: null,

  modelEffectiveSize: 0,

  pageSize: 0,

  _categoryWidgets: null,

  _rowWidgets: null,

  _rowCache: [],

  _connectRowHandlers: null,

  _connectCategoryHandlers: null,

  _lastSeenCategoryNumVal: null,

  _animations: [],

  _rowState: {}, // keeps allExpanded or an array of expanded/collapsed tkiids for each filter id (openTasks, completedTasks)

  _initializingRows: false,

  _focus: {}, // keeps last focus info for each filter id (openTasks, completedTasks)

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_List(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._categoryWidgets = [];
    this._rowWidgets = {};
    this._connectRowHandlers = [];
    this._connectCategoryHandlers = [];

    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the Layout config of the table Setting the config purges the content
   * 
   * @param {Object} config
   */
  setConfig: function setConfig_List(config){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.config = config;

    this.setKeepSavedFocus(false);

    // clear all data in the categories
    this._clearList();
    // also hide all categories now
    this._hideCategories();

    // initialize expanded row settings
    this._initRowState();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Resize the table (Not necessarily implemented by children)
   * 
   * @param {Object} dimensions JSON object {"h" : number, "w" : number }
   */
  resize: function(args){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (args && args.h > 0) {
      this.domNode.style.height = args.h + "px";
    }

    if (args && args.w > 0) {
      this.domNode.style.width = args.w + "px";
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * destroy the view, Not necessarily implemented by children
   */
  destroy: function destroy_List(config){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    var i;
    if (this._connectRowHandlers) {
      for (i = 0; i < this._connectRowHandlers.length; i++) {
        dojo.disconnect(this._connectRowHandlers[i]);
      }
      this._connectRowHandlers = null;
    }
    if (this._rowWidgets) {
      for ( var categoryId in this._rowWidgets) {
        for (i = 0; i < this._rowWidgets[categoryId].length; i++) {
          this._rowWidgets[categoryId][i].destroy();
        }
      }
      this._rowWidgets = null;
    }
    if (this._connectCategoryHandlers) {
      for (i = 0; i < this._connectCategoryHandlers.length; i++) {
        dojo.disconnect(this._connectCategoryHandlers[i]);
      }
      this._connectCategoryHandlers = null;
    }
    if (this._categoryWidgets) {
      for (i = 0; i < this._categoryWidgets.length; i++) {
        this._categoryWidgets[i].destroyDescendants();
        this._categoryWidgets[i].destroy();
      }
      this._categoryWidgets = null;
    }

    this.inherited('destroy', arguments);
    com.ibm.bpm.social.util.common.Log.logExit(1);
  },


  /**
   * postCreate
   */
  postCreate: function(){

    this._rowCache = new Array();

    this.containerNode = this.domNode;
    dijit.setWaiRole(this.domNode, "list");

    // initialize expanded row settings
    this._initRowState();

  },


  onDetailsRequested: function(item){},

  onCategoryCollapsed: function(categoryId, count){},

  onCategoryExpanded: function(categoryId, count){},

  onFilter: function(){},

  /**
   * Set the page size which is later used to decide whether a trailing section should be displayed
   * 
   * @param {int} actual page size requested by user, initially 25, doubled by "Show more"
   * @type {void}
   */
  setPageSize: function setPageSize_List(pageSize){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.pageSize = pageSize;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the model(data) to be displayed
   * 
   * @param {Array} items Array of items
   * @type {void}
   */
  setModel: function setModel_List(model){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._initializingRows = true;

    // PERFORMANCE TEST
    // PERFORMANCE TEST this._rowsFromCache = 0;
    // PERFORMANCE TEST this._newRows = 0;

    // Stop all current animations, otherwise there might be fading effects on the 
    // full reloaded list.
    var item, k;
    for (k = 0; k < this._animations.length; k++) {
      this._animations[k].stop(false);
    }
    this._animations = [];

    this.setKeepSavedFocus(false);

    this.model = model;
    this.modelEffectiveSize = 0;

    // clear all data in the categories
    // TODO: check, if we can do the clean and reload better for expand and collapse, 
    // right now it is a bit slow. 
    this._clearList();

    var myself = this;
    if (this.model) {
      if (this.config && this.config.filter.categories) {

        // add the categories to the model
        this._addCategoriesToModel(model);

        dojo.attr(this.domNode, "aria-busy", "true");

        // find category with highest index.
        // This will be needed later to identify and suppress trailing categories
        myself._lastSeenCategoryNumVal = -1;
        for (k = 0; k < this.model.length; k++) {
          item = this.model[k];
          // TODO   Optimization possible: we get items in sorted order, so rather than doing this loop,
          // it would be ok just to check the category of the last non-extra item.
          // Loop over 25 (or few hundred) iterations is quite cheap though, so until we are sure that all backends sort correctly
          // we can use this defensive way.   
          if (item && item.isExtra === true) {
            // skip this item
            com.ibm.bpm.social.util.common.Log.log(3, "Extra item: " + item.TKIID);
          }
          else {
            myself.modelEffectiveSize++;
            // maintain highest due date seen: memorize highest one
            // TODO  Optimization: memorize the cat id (string) first, transforming to a number only on mismatch
            // In most cases, the string will already match. 
            var actualCatNumVal = com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal(item.categoryId);
            if (actualCatNumVal > myself._lastSeenCategoryNumVal) {
              myself._lastSeenCategoryNumVal = actualCatNumVal;
            }
          }
          //});

        }
        // create new content       
        if (this.model) {
          dojo.forEach(this.config.filter.categories, function(categoryId){
            var tasksInCategory = dojo.filter(myself.model, function(item){
              return item.categoryId === categoryId;
            });
            var taskContainer = document.createElement("div");

            if (!myself._rowWidgets[categoryId]) {
              myself._rowWidgets[categoryId] = [];
            }

            for (k = 0; k < tasksInCategory.length; k++) {
              var item = tasksInCategory[k];
              if (item && item.isExtra !== true) {
                // the row destroy is called through the category destroyDescendants call               
                var itemWidget = myself._getNewRowWidget(item, categoryId);
                taskContainer.appendChild(itemWidget.domNode);
              }
            }

            var categoryWidget = myself._getCategoryWidget(categoryId);

            // compute effective length; this may be different than tasksInCategory.length 
            // because we have tried to fetch one extra item 
            var moreAvailable = false;
            var effectiveLength = tasksInCategory.length;
            if (tasksInCategory.length > 0 && tasksInCategory[tasksInCategory.length - 1] &&
              tasksInCategory[tasksInCategory.length - 1].isExtra === true) {
              moreAvailable = true;
              effectiveLength = tasksInCategory.length - 1;
            }
            // show or hide the widget
            // trailing categories (i.e. those who have a numerically "higher" category than the last task fetched from the backend)
            // are not shown, neither collapsed nor in expanded state.
            // Exception to this: when the page is not full (less items retrieved from the server page size)
            var categoryIsTrailing = false;
            if (categoryWidget.id && categoryWidget.id !== com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
              // Overdue is never trailing
              if (myself._lastSeenCategoryNumVal && myself._lastSeenCategoryNumVal > 0 && categoryWidget.id) {
                // Transform current category name into a comparable numeric value:
                var categoryNumVal = com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal(categoryWidget.id);
                if (categoryNumVal > myself._lastSeenCategoryNumVal) {
                  categoryIsTrailing = true;
                  if (!categoryWidget.isExpanded()) {
                    // Here, we can auto-expand collapsed categories that are beyond the screen.
                    // auto-expand invisible categories; otherwise, there may be strange effects,
                    // for example, if we expand a category which has 0 items in the meantime
                    // in which case the "expand" operation would make the category disappear - very confusing.
                    // This auto-expand is currently disabled here:
                    // categoryWidget.toggle(true); // This "true" means "silent", i.e. do not cause another fetch from backend  
                  }
                }
              }
            }

            // find out whether there are more items on the server available than the current page size
            // here, the where clause (that was built to reflect the expanded sections) will affect the items available on the server  
            // Because we retrieve one extra record, this is visible in the fact whether that extra item was retrieved 
            // (current page size is initially 25, then potentially doubled by "show more" requests)
            var pageIsFull = false;
            if (myself.modelEffectiveSize && myself.pageSize && myself.modelEffectiveSize === myself.pageSize) {
              pageIsFull = true;
            }

            // Hide categories if one (or both) of these conditions are true:
            // 1. an expanded category has 0 items (note that collapsed categories have always 0 items due to the where clause we use)
            // 2. trailing categories (collapsed or expanded) in case that they would start beyond the page size because page is full  
            if ((effectiveLength === 0 && categoryWidget.isExpanded()) || (categoryIsTrailing && pageIsFull === true)) {
              categoryWidget.hide();
              com.ibm.bpm.social.util.common.Log.log(1, "Hiding category '" + categoryWidget.id + "', eff size: " +
                myself.modelEffectiveSize + ", pageSize: " + myself.pageSize, "List", "setModel");
            }
            else {
              categoryWidget.set("content", taskContainer);
              categoryWidget.setCount(effectiveLength, moreAvailable);
              categoryWidget.show();
            }
          });
        }
      }
      else {
        if (this.model) {
          // delete old content

          this.pageSize = 0;
          this._lastSeenCategoryNumVal = null;

          // hide all categories
          this._hideCategories();

          // no category, so use pseudo category 1
          if (!this._rowWidgets[1]) {
            this._rowWidgets[1] = [];
          }
          // Display a plain list of items
          for (k = 0; k < this.model.length; k++) {
            item = this.model[k];
            if (item.isExtra !== true) {
              var itemWidget = myself._getNewRowWidget(item, 1);
              myself.domNode.appendChild(itemWidget.domNode);
            }
            else {
              com.ibm.bpm.social.util.common.Log.log(3, "Extra item: " + item.TKIID);
            }
          }
        }
      }

      // maintain expand/collapse state of rows
      var filterId = this._getFilterId();
      if (this._rowState[filterId]) {
        if (this._rowState[filterId].allExpanded) {
          this.expandAll(true);
          this.onShowCollapseAllButton();
        }
        else {
          this.collapseAll(true);
        }
      }

      dojo.attr(this.domNode, "aria-busy", "false");

    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "no items");
    }


    this._initializingRows = false;

    // PERFORMANCE TEST console.debug("from cache: " + this._rowsFromCache + ", new rows: " + this._newRows);

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _getNewRowWidget: function _getNewRowWidget_List(item, categoryId){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "tkiid=" + item.TKIID + ", category=" + categoryId);
    // get the row from the cache if possible
    var itemWidget = null;
    var newRow = false;
    if (this._rowCache.length > 0) {
      itemWidget = this._rowCache.pop();
      com.ibm.bpm.social.util.common.Log.log(3, "Using row from cache.");
      // PERFORMANCE TEST
      // PERFORMANCE TEST this._rowsFromCache++;
    }
    else {
      var params = {
        model: item,
        open: false,
        controller: this.config.controller,
        displayedProperties: this.config.filter.displayedProperties
      };
      if (this.config.type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
        itemWidget = new com.ibm.bpm.social.widgets.task.list.TaskRow(params);
      }
      else {
        var e = new Error("Unexpected config type: " + this.config.type);
        com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected config type.", e);
        throw e;
      }
      newRow = true;
      // PERFORMANCE TEST
      // PERFORMANCE TEST this._newRows++;
    }

    itemWidget.initRow({
      model: item,
      open: false,
      controller: this.config.controller,
      displayedProperties: this.config.filter.displayedProperties
    });

    // store the row widget for clean up and update
    if (!this._rowWidgets[categoryId]) {
      this._rowWidgets[categoryId] = new Array();
    }
    this._rowWidgets[categoryId].push(itemWidget);

    // create connects for new rows 
    if (newRow) {
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onDetailsRequested", dojo.hitch(this, function(rowModel){
        this.onDetailsRequested(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onWorkOn", dojo.hitch(this, function(rowModel){
        this.onWorkOn(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onRefreshNeeded", this, "onRefreshNeeded"));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onExpanded", this, dojo.hitch(this, function(rowModel){
        this._onTaskRowExpanded(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onCollapsed", this, dojo.hitch(this, function(rowModel){
        this._onTaskRowCollapsed(rowModel);
      })));
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onModelUpdate", this, dojo.hitch(this, function(model){
        // Row update triggered by "Modify Task"
        this.updateModel([model], true);
      })));
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onDeleteRowNeeded", this, dojo.hitch(this, function(tkiid){
        // get task row widget
        var row = this._getTaskWidget(tkiid);
        // Remove row from the list if not already removed by notification message
        if (row !== null) {
          this._deleteRowWidget(row, false);
        }
      })));
    }

    // now make it visible
    dojo.style(itemWidget.domNode, "display", "block");

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return itemWidget;
  },

  /**
   * Update the list model and task row
   * 
   * @param items updated tasks
   * @param manualUpdateMode row change has been triggered by the user
   * @param searchFilterApplied the task list is filtered with a search string
   * @param notNewAssignment
   * @returns {void} nothing
   */
  updateModel: function updateModel_List(items, manualUpdateMode, searchFilterApplied, notNewAssignment){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    if (items) {
      com.ibm.bpm.social.util.common.Log.log(3, "Updates for " + items.length + " task(s).");

      var filterId = this._getFilterId();

      var itemsToAdd = new Array();
      var itemIdsToUpdate = new Array();
      var itemsToUpdate = {}; // tkiid: {"item": item, "rowWidget": rowWidget, "actionsNeeded": true|false}
      var itemsNeedingActions = new Array();
      var i;
      var item;
      var extraItem = this._getExtraItem();

      for (i = 0; i < items.length; i++) {
        item = items[i];
        if (item) {

          com.ibm.bpm.social.util.common.Log.log(2, "Update for task " + item.TKIID + ": " + dojo.toJson(item));

          var isExtraItem = (extraItem !== null && extraItem.TKIID === item.TKIID);
          var fullRefresh = false;

          // get task row widget
          var row = this._getTaskWidget(item["TKIID"]);

          if (searchFilterApplied && row === null) {
            // ignore new tasks if search filter has been applied
            com.ibm.bpm.social.util.common.Log.log(2, "Search filter applied: Ignoring new task " + item.TKIID);
            continue;
          }

          //
          //check if row is visible on this list
          var inFilter = true;
          var inScope = true;

          if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
            if (item.STATE && item.STATE !== "STATE_READY" && item.STATE !== "STATE_CLAIMED") {
              // item does no longer match to state filter
              if (isExtraItem) {
                com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                  " is extra item, state does no longer apply to filter " + filterId);
                fullRefresh = true;
              }
              else {
                com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
                inFilter = false;
              }
            }
            else if (item.OWNER && item.OWNER !== com.ibm.bpm.social.util.common.Util.getCurrentUser().userName) {
              // item does not match filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (extraItem !== null && item.DUE) {
              var itemDueDate = dojo.date.stamp.fromISOString(item.DUE);
              var extraItemDueDate = dojo.date.stamp.fromISOString(extraItem.DUE);
              if (isExtraItem) {
                if (dojo.date.compare(itemDueDate, extraItemDueDate) !== 0) {
                  // due date changed for extra item
                  com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                    " is extra item, due date changed for filter " + filterId);
                  fullRefresh = true;
                }
              }
              else {
                // behind extra item?
                if ((dojo.date.compare(itemDueDate, extraItemDueDate) === 0 && (item.TKIID > extraItem.TKIID)) ||
                  dojo.date.compare(itemDueDate, extraItemDueDate) > 0) {
                  com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " is behind extra item for filter " +
                    filterId);
                  inScope = false;
                }
              }
            }
          }
          else if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
            if (item.COMPLETED === null) {
              // item does not match to completed filter
              inFilter = false;
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
            }
            else if (item.STATE && item.STATE !== "STATE_FINISHED") {
              // item does not match to completed filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (!item.OWNER || item.OWNER !== com.ibm.bpm.social.util.common.Util.getCurrentUser().userName) {
              // item does not match to completed filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (extraItem !== null && item.COMPLETED) {
              var itemCompletedDate = dojo.date.stamp.fromISOString(item.COMPLETED);
              var extraItemCompletedDate = dojo.date.stamp.fromISOString(extraItem.COMPLETED);
              if (isExtraItem) {
                if (dojo.date.compare(itemCompletedDate, extraItemCompletedDate) !== 0) {
                  // completed changed for extra item
                  com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                    " is extra item, completed changed for " + filterId);
                  fullRefresh = true;
                }
              }
              else {
                // behind extra item?
                if ((dojo.date.compare(itemCompletedDate, extraItemCompletedDate) === 0 && (item.TKIID > extraItem.TKIID)) ||
                  dojo.date.compare(itemCompletedDate, extraItemCompletedDate) < 0) {
                  com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " is behind extra item for filter " +
                    filterId);
                  inScope = false;
                }
              }
            }
          }

          if (fullRefresh) {
            // trigger full refresh because extra item was modified
            this.onFullRefreshNeeded();
            com.ibm.bpm.social.util.common.Log.logExit(2, "Full Refresh");
            return;
          }

          if (row === null && manualUpdateMode && !item.OWNER) {
            // manual update may update properties of a task which has already been deleted
            com.ibm.bpm.social.util.common.Log.logExit(2, "Task " + item.TKIID +
              " is already deleted, properties cannot be updated.");
            return;
          }

          if (row === null) {
            //
            // new items
            if (inScope && inFilter) {
              // add new row
              itemsToAdd.push(item);
              itemsNeedingActions.push(item);
              if (!this.model) {
                this.model = new Array();
              }
              // add new item to model - keep eventual extra item at the end
              this.model.splice(0, 0, item);
            }

            // TODO: rework special logic for completed tasks to increase global counter.
            if (inFilter &&
              (!notNewAssignment || (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS))) {
              this.onIncreaseGlobalCount();
            }
          }
          else {
            //
            // existing rows
            if (inScope && inFilter) {
              // update existing task row
              if (itemsToUpdate[item.TKIID] === undefined) {
                // add only once but update data if another message is received for the same tkiid
                itemIdsToUpdate.push(item.TKIID);
              }
              itemsToUpdate[item.TKIID] = {
                "item": item,
                "row": row,
                "actionsNeeded": false
              };
              if (row.actionsNeeded(item)) {
                itemsNeedingActions.push(item);
                itemsToUpdate[item.TKIID].actionsNeeded = true;
              }
            }
            else {
              // Remove row from the list            
              this._deleteRowWidget(row, inFilter && (manualUpdateMode || !inScope));
            }
          }
        }
      }

      // trigger actions call - this needs to be done before creating/updating the row, 
      // because the deferred callback can only be registered after the actions call has been done
      if (itemsNeedingActions.length > 0) {
        // retrieve actions async
        this.config.controller.retrieveAvailableActionsForItems(itemsNeedingActions);
      }

      // add items
      if (itemsToAdd.length > 0) {
        if (this.config && this.config.filter.categories) {
          // add the categories to the model
          this._addCategoriesToModel(itemsToAdd);
        }
        for (i = 0; i < itemsToAdd.length; i++) {
          item = itemsToAdd[i];
          if (item) {
            var categoryId = item.categoryId ? item.categoryId : 1;
            var categoryWidget = this._getCategoryWidget(categoryId);

            // Do not synchronize for a task in a collapsed group.
            if (!categoryWidget || categoryWidget.isExpanded()) {
              row = this._getNewRowWidget(item, categoryId, true);
              this._synchronize(row, true);
            }
          }
        }
        if (this._rowState[filterId].allExpanded) {
          // expand new rows
          this._initializingRows = true;
          this.expandAll(true);
          this._initializingRows = false;
        }
      }

      //
      // update items
      for (i = 0; i < itemIdsToUpdate.length; i++) {
        var tkiid = itemIdsToUpdate[i];
        item = itemsToUpdate[tkiid].item;
        if (item) {
          var rowDeleted = false;
          var rowToUpdate = itemsToUpdate[tkiid].row;
          var actionsNeeded = itemsToUpdate[tkiid].actionsNeeded;
          if (rowToUpdate.updateRow(item, actionsNeeded)) {
            this._synchronize(rowToUpdate, false);
          }
        }
      }

    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Updates the UI (row position) and the model.
   * 
   * @param row task row
   * @param addRow a new row has been added
   * @returns {void} nothing
   */
  _synchronize: function _synchronize_List(row, addRow){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    // The following algorithm is applied to the task rows:
    // -> show task reordered in current category:    
    //      Find current category
    //      If category not visible -> Create category 
    //      Move task to it and sort

    var currentCategoryId = row.model.categoryId ? row.model.categoryId : 1; // 1 is pseudo category for completedTasks
    var oldPosition = dojo.indexOf(this._rowWidgets[currentCategoryId], row);
    if (!addRow) {
      com.ibm.bpm.social.util.common.Log.log(3, "Move row for task " + row.model.TKIID);
      // Find current category
      var oldCategoryId = row.model.categoryId;
      com.ibm.bpm.social.util.common.Log.log(3, "Old category = " + oldCategoryId + ", old position = " * oldPosition);
      if (this.config && this.config.filter.categories) {
        this._addCategoriesToModel([row.model]);
        currentCategoryId = row.model.categoryId;

        // If categories are different, remove from old category storage and add to new 
        // category storage.          
        if (oldCategoryId !== currentCategoryId) {
          var tasksInOldCategory = this._rowWidgets[oldCategoryId];
          var indexOfTask = dojo.indexOf(tasksInOldCategory, row);
          tasksInOldCategory.splice(indexOfTask, 1);
          this._rowWidgets[currentCategoryId].push(row);
        }
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(3, "Add row for task " + row.model.TKIID);
    }

    // used filter openTasks or completedTasks
    var filterId = this._getFilterId();

    // Change order in _rowWidgets cache
    var itemADate, itemBDate;
    var tasksInCurrentCategory = this._rowWidgets[currentCategoryId];
    tasksInCurrentCategory.sort(function(itemA, itemB){
      if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
        itemADate = dojo.date.stamp.fromISOString(itemA.model.DUE);
        itemBDate = dojo.date.stamp.fromISOString(itemB.model.DUE);
        if (dojo.date.compare(itemADate, itemBDate) === 0) {
          return itemA.model.TKIID > itemB.model.TKIID;
        }
        return dojo.date.compare(itemADate, itemBDate);
      }
      else if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
        itemADate = dojo.date.stamp.fromISOString(itemA.model.COMPLETED);
        itemBDate = dojo.date.stamp.fromISOString(itemB.model.COMPLETED);
        if (dojo.date.compare(itemADate, itemBDate) === 0) {
          return itemA.model.TKIID > itemB.model.TKIID;
        }
        return dojo.date.compare(itemBDate, itemADate);

      }
    });
    var newPosition = dojo.indexOf(tasksInCurrentCategory, row);
    com.ibm.bpm.social.util.common.Log.log(3, "New category = " + currentCategoryId + ", new position = " + newPosition);

    //
    // Now change order in dom tree
    //
    if (addRow || oldPosition !== newPosition || currentCategoryId !== oldCategoryId) {
      var myself = this;

      var currentCategoryWidget = this._getCategoryWidget(currentCategoryId);

      var fadeOutArgs = null;
      if (!addRow) {
        fadeOutArgs =
          {
            node: row.domNode,
            duration: this.FADING_DURATION,
            onStop: function(){
              com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeOut_synchronize");
              // Remove opacity settings, because this row might be reused.       
              myself._removeOpacitySettings(row.domNode);
              com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeOut_synchronize");
            },
            onEnd: function(){
              com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onEndFadeOut_synchronize");

              if (!this._keepSavedFocus) {
                myself.saveFocus();
              }

              if (currentCategoryWidget !== null) {
                // openTasks
                // handle categories
                row.setStatus(currentCategoryId);

                var oldCategoryWidget = null;

                // Move row to new position
                if (currentCategoryId !== oldCategoryId) {
                  oldCategoryWidget = myself._getCategoryWidget(oldCategoryId);
                  oldCategoryWidget.containerNode.childNodes[0].removeChild(row.domNode);

                  // If old category is empty, hide it.
                  if (tasksInOldCategory.length === 0) {
                    com.ibm.bpm.social.util.common.Log.log(3, "Hide old category " + oldCategoryId);
                    oldCategoryWidget.hide();
                  }
                  else {
                    // Decrease category count
                    oldCategoryWidget.setCount(oldCategoryWidget.count - 1);
                  }

                  // If current category is not visible, make it visible.
                  if (!currentCategoryWidget.isDisplayed()) {
                    com.ibm.bpm.social.util.common.Log.log(3, "Show new category " + currentCategoryId);
                    currentCategoryWidget.setCount(1, false);
                    currentCategoryWidget.show();
                    currentCategoryWidget.set('content', document.createElement("div"));
                  }
                  else {
                    // Increment the count
                    currentCategoryWidget.setCount(currentCategoryWidget.count + 1);
                  }
                }
                else {
                  // same category, new position
                  currentCategoryWidget.containerNode.childNodes[0].removeChild(row.domNode);
                }
              }
              else {
                // completedTasks
                myself.domNode.removeChild(row.domNode);
              }

              //
              // Insert the row to the new position
              var rowParentNode = null;
              if (currentCategoryWidget !== null) {
                // openTasks
                rowParentNode = currentCategoryWidget.containerNode.childNodes[0];
              }
              else {
                // completedTasks
                rowParentNode = myself.domNode;
              }

              if (tasksInCurrentCategory.length - 1 === newPosition) {
                rowParentNode.appendChild(row.domNode);
              }
              else {
                // IE 9 requires the nextSibling to be null!
                var nextSibling =
                  newPosition < rowParentNode.childNodes.length ? rowParentNode.childNodes[newPosition] : null;
                rowParentNode.insertBefore(row.domNode, nextSibling);
              }

              if (currentCategoryWidget.isExpanded()) {
                if (oldCategoryWidget !== null && !oldCategoryWidget.isExpanded()) {
                  // increment local count
                  myself.onIncreaseLocalCount();
                }
              }
              else {
                if (oldCategoryWidget !== null && oldCategoryWidget.isExpanded()) {
                  // Decrement local count. 
                  myself.onDecreaseLocalCount();
                }
              }

              myself._restoreFocusIfNeeded(row, currentCategoryWidget);

              // Remove animation from internal storage
              myself._animations.splice(dojo.indexOf(myself._animations, this), 1);
              com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onEndFadeOut_synchronize");
            }
          };
      }
      var fadeInArgs = {
        node: row.domNode,
        duration: this.FADING_DURATION,
        onStop: function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeIn_synchronize");
          myself._removeOpacitySettings(row.domNode);
          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeIn_synchronize");
        }
      };
      if (addRow) {
        fadeInArgs.beforeBegin = function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
          if (currentCategoryWidget !== null) {
            // If current category is not visible, make it visible.
            if (!currentCategoryWidget.isDisplayed()) {
              com.ibm.bpm.social.util.common.Log.log(3, "Show new category " + currentCategoryId);
              currentCategoryWidget.setCount(1, false);
              currentCategoryWidget.show();
              currentCategoryWidget.set('content', document.createElement("div"));
            }
            else {
              // Increment the count
              currentCategoryWidget.setCount(currentCategoryWidget.count + 1);
            }
          }
          // move the row to the correct position and then fade in
          myself._addOpacitySettings(row.domNode);
          var rowParentNode = null;
          if (currentCategoryWidget !== null) {
            // openTasks
            rowParentNode = currentCategoryWidget.containerNode.childNodes[0];
          }
          else {
            // completedTasks
            rowParentNode = myself.domNode;
          }
          // IE 9 requires the nextSibling to be null!
          var nextSibling = newPosition < rowParentNode.childNodes.length ? rowParentNode.childNodes[newPosition] : null;
          rowParentNode.insertBefore(row.domNode, nextSibling);

          if (currentCategoryWidget === null || currentCategoryWidget.isExpanded) {
            // increment local count
            myself.onIncreaseLocalCount();
          }

          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "beforeBeginFadeIn");
        };
      }
      else {
        fadeInArgs.beforeBegin = function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
          myself._addOpacitySettings(row.domNode);
          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
        };
      }

      var animation = null;
      // In IE 8 the opacity settings for child elements with position 
      // absolute or relative will be ignored, so all such elements will 
      // be listed explicitly to fade out and in.
      if (dojo.isIE < 9) {
        var fadeOutElements = null;
        if (!addRow) {
          fadeOutElements = [dojo.fadeOut(fadeOutArgs),
                             dojo.fadeOut({
                               node: row.imgPriority,
                               duration: this.FADING_DURATION
                             }),
                             dojo.fadeOut({
                               node: row.imgStatus,
                               duration: this.FADING_DURATION
                             }),
                             dojo.fadeOut({
                               node: row.imgGroup,
                               duration: this.FADING_DURATION
                             })];
        }
        var fadeInElements = [dojo.fadeIn(fadeInArgs),
                              dojo.fadeIn({
                                node: row.imgPriority,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgPriority, "filter", null);
                                }

                              }),
                              dojo.fadeIn({
                                node: row.imgStatus,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgStatus, "filter", null);
                                }

                              }),
                              dojo.fadeIn({
                                node: row.imgGroup,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgGroup, "filter", null);
                                }
                              })];

        if (addRow) {
          animation = dojo.fx.combine(fadeInElements);

        }
        else {
          animation = dojo.fx.chain([dojo.fx.combine(fadeOutElements),
                                     dojo.fx.combine(fadeInElements)]);
        }
      }
      else {
        if (addRow) {
          animation = dojo.fadeIn(fadeInArgs);

        }
        else {
          animation = dojo.fx.chain([dojo.fadeOut(fadeOutArgs),
                                     dojo.fadeIn(fadeInArgs)]);
        }
      }
      this._animations.push(animation);
      animation.play();
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * delete a row widget
   * 
   * @param row
   * @param keepGlobal no event for decrement of the global count, e.g. if task is in filter but not in list
   *        scope
   * @returns {void} nothing
   */
  _deleteRowWidget: function _deleteRowWidget_List(row, keepGlobal){
    com.ibm.bpm.social.util.common.Log.logEntry(2, row.model.TKIID);

    // Remove the task from the category and push it to the cache
    // This needs to be done immediately to make sure that for manual updates 
    // the delete is not triggered again for the push event
    var tasksInSameCategory = this._rowWidgets[row.model.categoryId];
    var indexOfTask = dojo.indexOf(tasksInSameCategory, row);
    tasksInSameCategory.splice(indexOfTask, 1);

    var myself = this;
    var fadeArgs = {
      node: row.domNode,
      duration: this.FADING_DURATION,
      onStop: function(){
        com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeOut_deleteRowWidget");
        // Remove opacity settings, because this row might be reused.
        myself._removeOpacitySettings(row.domNode);
        com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeOut_deleteRowWidget");
      },
      onEnd: function(){
        com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onEndFadeOut_deleteRowWidget");

        if (!this._keepSavedFocus) {
          myself.saveFocus();
        }

        var categoryWidget = myself._getCategoryWidget(row.model.categoryId);
        categoryWidget.containerNode.childNodes[0].removeChild(row.domNode);

        // Remove opacity settings, because this row might be reused.       
        myself._removeOpacitySettings(row.domNode);

        var widget = null;
        // If the category is empty, hide it
        if (tasksInSameCategory.length === 0) {
          categoryWidget.hide();
        }
        else {
          // Decrease category count
          categoryWidget.setCount(categoryWidget.count - 1);
        }

        if (row.open) {
          // row is expanded -> collapse
          row.toggle();
        }

        myself._rowCache.push(row);

        // Synchronize model     
        myself.model.splice(dojo.indexOf(myself.model, row.model), 1);

        if (categoryWidget.isExpanded()) {
          // decrement local count
          myself.onDecreaseLocalCount();
        }
        if (!keepGlobal) {
          // decrement global count
          myself.onDecreaseGlobalCount();
        }

        myself._restoreFocusIfNeeded(row, categoryWidget);

        // Remove animation from internal storage
        myself._animations.splice(dojo.indexOf(myself._animations, this), 1);
        com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onEndFadeOut_deleteRowWidget");
    }
    };

    var animation = null;
    // In IE 8 the opacity settings for child elements with position 
    // absolute or relative will be ignored, so all such elements will 
    // be listed explicitly to fade out.
    if (dojo.isIE < 9) {
      var fadeOutElements = [dojo.fadeOut(fadeArgs),
                             dojo.fadeOut({
                               node: row.imgPriority,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgPriority, "filter", null);
                               }
                             }),
                             dojo.fadeOut({
                               node: row.imgStatus,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgStatus, "filter", null);
                               }

                             }),
                             dojo.fadeOut({
                               node: row.imgGroup,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgGroup, "filter", null);
                               }
                             })];


      animation = dojo.fx.combine(fadeOutElements);
    }
    else {
      animation = dojo.fadeOut(fadeArgs);
    }
    this._animations.push(animation);
    animation.play();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  collapseAll: function collapseAll_List(keepRowState){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var filterId = this._getFilterId();
    if (!keepRowState) {
      // set all collapsed row state
      if (this._rowState[filterId]) {
        this._rowState[filterId].allExpanded = false;
        this._rowState[filterId].expandedRows = new Array();
        this._rowState[filterId].collapsedRows = null;
      }
    }

    // collapse rows
    var self = this;
    var tkiidList = [];
    // get the tkiids for list of shown tasks for the bulk api call we do later on.
    if (this.config.filter.categories) {
      dojo.forEach(this._categoryWidgets, function(categoryWidget){
        dojo.forEach(categoryWidget.getChildren(), function(taskWidget){
          if (taskWidget.open) {
            // row is expanded -> collapse
            taskWidget.toggle();

          }
          else if (dojo.indexOf(self._rowState[filterId].expandedRows, taskWidget.model.TKIID) !== -1) {
            // row is collapsed but part of expandedRows -> expand
            tkiidList.push(taskWidget.model.TKIID);
            taskWidget.toggle();
          }
        });
      });
    }
    else {
      dojo.forEach(this._rowWidgets[1], function(taskWidget){
        if (taskWidget.open) {
          // row is expanded -> collapse
          taskWidget.toggle();
        }
        else if (dojo.indexOf(self._rowState[filterId].expandedRows, taskWidget.model.TKIID) !== -1) {
          // row is collapsed but part of expandedRows -> expand
          tkiidList.push(taskWidget.model.TKIID);
          taskWidget.toggle();
        }
      });
    }

    // Now trigger the bulk api call to receive the task details
    if (tkiidList.length > 0) {
      this.onTaskDetailsNeeded(tkiidList);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  getExpandedCategoriesCount: function getExpandedCategoriesCount_List(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var count = 0;

    if (this._categoryWidgets) {
      for ( var i = 0; i < this._categoryWidgets.length; i++) {
        if (this._categoryWidgets[i].isExpanded()) {
          count++;
        }
      }
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, "count=" + count);
    return count;
  },

  expandAll: function expandAll_List(keepRowState){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var filterId = this._getFilterId();
    if (!keepRowState) {
      // set all expanded row state
      if (this._rowState[filterId]) {
        this._rowState[filterId].allExpanded = true;
        this._rowState[filterId].expandedRows = null;
        this._rowState[filterId].collapsedRows = new Array();
      }
    }

    var self = this;
    var tkiidList = [];
    // get the tkiids for list of shown tasks for the bulk api call we do later on.
    // But get only the collapsed ones, as otherwise there may be lost data:
    // this is in the case that one (expanded) inline task already has (unsaved) data filled in, refresh would loose those
    // TODO: Possible optimitation is to do more parallelism, using this sequence
    //  1) get the list of tkiids,
    //  2) trigger the bulk call (async)
    //  3) to the toggle
    // thus 2 and 3 could run in parallel.
    if (this.config.filter.categories) {
      dojo.forEach(this._categoryWidgets, function(categoryWidget){
        if (categoryWidget.open) {
          dojo.forEach(categoryWidget.getChildren(), function(taskWidget){
            if (!taskWidget.open && dojo.indexOf(self._rowState[filterId].collapsedRows, taskWidget.model.TKIID) === -1) {
              // row is collapsed and not part of collapsedRows
              tkiidList.push(taskWidget.model.TKIID);
              taskWidget.toggle();
            }
          });
        }
      });
    }
    else {
      dojo.forEach(this._rowWidgets[1], function(taskWidget){
        if (!taskWidget.open && dojo.indexOf(self._rowState[filterId].collapsedRows, taskWidget.model.TKIID) === -1) {
          // row is collapsed and not part of collapsedRows
          tkiidList.push(taskWidget.model.TKIID);
          taskWidget.toggle();
        }
      });
    }

    if (tkiidList.length > 0) {
      // Now trigger the bulk api call to receive the task details
      this.onTaskDetailsNeeded(tkiidList);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  //
  // Render details + simple approval input form and controls.
  //
  setDetailsSimpleApproval: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleApproval(taskId, details);
  },
  //
  // Render details + simple completion input form and controls.
  //
  setDetailsSimpleCompletion: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleCompletion(taskId, details);
  },
  //
  // Render details + simple choice input form and controls.
  //
  setDetailsSimpleChoice: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleChoice(taskId, details);
  },
  setDetails: function setDetails_List(taskId, details){
    this._getTaskWidget(taskId).setDetails(details);
  },

  // remove the rows from all categories and cache them
  _clearList: function _clearList_List(){
    if (this._rowWidgets) {
      for ( var categoryId in this._rowWidgets) {
        for ( var i = 0; i < this._rowWidgets[categoryId].length; i++) {
          // put row into cache and remove from parent in DOM tree
          var row = this._rowWidgets[categoryId][i];
          dojo.style(row.domNode, "display", "none");
          row.domNode.parentNode.removeChild(row.domNode);
          this._rowCache.push(row);
        }
        this._rowWidgets[categoryId] = [];
      }
    }
  },

  _hideCategories: function _hideCategories_List(){
    if (this._categoryWidgets) {
      for ( var i = 0; i < this._categoryWidgets.length; i++) {
        this._categoryWidgets[i].hide();
      }
    }
  },

  _getTaskWidget: function _getTaskWidget_List(taskId){
    var taskWidget = null;
    for ( var categoryId in this._rowWidgets) {
      if (this._rowWidgets[categoryId] && !taskWidget) {
        for ( var i = 0; i < this._rowWidgets[categoryId].length; i++) {
          if (this._rowWidgets[categoryId][i].model.TKIID === taskId) {
            taskWidget = this._rowWidgets[categoryId][i];
            break;
          }
        }
      }
    }
    return taskWidget;
  },

  /*
   * Returns existing category widget or creates new one.
   */
  _getCategoryWidget: function _getCategoryWidget_List(categoryId){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var categoryWidget = null;
    if (this.config && this.config.filter.categories) {
      var existingCategoryWidgets = dojo.filter(this._categoryWidgets, function(category){
        return (category.id === categoryId);
      });

      if (existingCategoryWidgets.length > 0) {
        categoryWidget = existingCategoryWidgets[0];
      }
      // create a new widget, no wipe in for resize calculation!
      else {
        categoryWidget = new com.ibm.bpm.social.widgets.task.list.Category({
          id: categoryId,
          duration: 0
        });

        categoryWidget.set("content", document.createElement("div"));

        // store the connections per category, as they get deleted by category
        this._connectCategoryHandlers.push(dojo.connect(categoryWidget, "onCollapsed", this, function(count){
          this.saveFocus();
          this.onCategoryCollapsed(categoryId, count);
        }));

        this._connectCategoryHandlers.push(dojo.connect(categoryWidget, "onExpanded", this, function(count){
          this.saveFocus();
          this.onCategoryExpanded(categoryId, count);
        }));

        this._categoryWidgets.push(categoryWidget);
        this.domNode.appendChild(categoryWidget.domNode);
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return categoryWidget;
  },

  _getExtraItem: function _getExtraItem_List(){
    var item = null;
    if (this.model && this.model.length > 1) {
      // get last item
      item = this.model[this.model.length - 1];
      if (!item.isExtra) {
        item = null;
      }
    }
    return item;
  },

  // To comply with table view, the taskID must be the second parameter
  showMessage: function showMessage_List(message, taskId){
    this._getTaskWidget(taskId).showMessage(message);
  },


  /**
   * Group tasks by: Overdue, At Risk(later), Due Today, Due Tomorrow, Due This Week, Due Later
   * 
   * @param model
   */
  _addCategoriesToModel: function _addCategoriesToModel_List(model){
    var categories = null;
    if (this.config && this.config.filter.categories) {
      categories = this.config.filter.categories;
    }
    if (categories !== null) {
      var now = new Date();
      var endOfToday = new Date(now);
      endOfToday.setHours(23, 59, 59, 0);
      var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

      // getting the end of this week needs some more computation: depends on current weekday and locale
      var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
      // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
      firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
      var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
      var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);
      var myself = this;
      dojo.forEach(model, function(item){
        var due = dojo.date.stamp.fromISOString(item.DUE);

        if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) && dojo.date.compare(now, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.OVERDUE;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) && item.IS_AT_RISK === true) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.AT_RISK;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) &&
          dojo.date.compare(endOfToday, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) &&
          dojo.date.compare(endOfTomorrow, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) &&
          dojo.date.compare(endOfThisWeek, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER)) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER;
        }
      });
    }
  },

  /**
   * Indicates whether a particular category should be rendered according to the current configuration
   * 
   * @param id String the category name which is to be looked up in the configuration
   */
  _hasCategory: function _hasCategory_List(id){
    var filteredArr = dojo.filter(this.config.filter.categories, function(categoryId){
      return categoryId == id;
    });
    return filteredArr.length > 0;
  },

  _getFilterId: function _getFilterId_List(){
    if (this.config && this.config.filter && this.config.filter.id) {
      return this.config.filter.id;
    }
    return null;
  },

  _initRowState: function _initRowState_List(){
    var filterId = this._getFilterId();
    if (filterId !== null) {
      if (this._rowState[filterId] === undefined) {
        this._rowState[filterId] = {
          allExpanded: false,
          expandedRows: new Array(),
          collapsedRows: null
        };
      }
    }
  },

  _onTaskRowExpanded: function _onTaskRowExpanded_List(item){
    this.onTaskRowExpanded(item);
    if (!this._initializingRows) {
      // maintain row state
      var filterId = this._getFilterId();
      if (filterId !== null && this._rowState[filterId]) {
        if (this._rowState[filterId].allExpanded) {
          // all expanded
          var index = dojo.indexOf(this._rowState[filterId].collapsedRows, item.TKIID);
          if (index !== -1) {
            // remove from collapsedRows
            this._rowState[filterId].collapsedRows.splice(index, 1);
          }

        }
        else if (dojo.indexOf(this._rowState[filterId].expandedRows, item.TKIID) === -1) {
          // not all expanded, add to expandedRows
          this._rowState[filterId].expandedRows.push(item.TKIID);
        }
      }
    }
  },

  _onTaskRowCollapsed: function _onTaskRowCollapsed_List(item){
    this.onTaskRowCollapsed(item);
    if (!this._initializingRows) {
      // maintain row state
      var filterId = this._getFilterId();
      if (filterId !== null && this._rowState[filterId]) {
        if (!this._rowState[filterId].allExpanded) {
          // not all expanded
          var index = dojo.indexOf(this._rowState[filterId].expandedRows, item.TKIID);
          if (index !== -1) {
            // remove from expandedRows
            this._rowState[filterId].expandedRows.splice(index, 1);
          }
        }
        else if (dojo.indexOf(this._rowState[filterId].collapsedRows, item.TKIID) === -1) {
          // all expanded, add to collapsedRows
          this._rowState[filterId].collapsedRows.push(item.TKIID);
        }
      }
    }
  },

  onTaskDetailsNeeded: function onTaskDetailsNeeded_List(tkiidList){},

  onWorkOn: function onWorkOn_List(item){},

  onRefreshNeeded: function onRefreshNeeded_List(){},

  onTaskRowExpanded: function onTaskRowExpanded_List(item){},

  onTaskRowCollapsed: function onTaskRowCollapsed_List(item){},

  /**
   * event triggered if a task is removed from the list (global count)
   */
  onDecreaseGlobalCount: function onDecreaseGlobalCount_List(){},

  /**
   * event triggered if a task is added to the list (global count)
   */
  onIncreaseGlobalCount: function onIncreaseGlobalCount_List(){},

  /**
   * event triggered if a task is moved from an expanded to a collapsed category (local count)
   */
  onDecreaseLocalCount: function onDecreaseLocalCount_List(){},

  /**
   * event triggered if a task is moved from a collapsed to an expanded category (local count)
   */
  onIncreaseLocalCount: function onIncreaseLocalCount_List(){},

  /**
   * event triggered if the task list is in all rows expanded state to show the button in the correct state
   */
  onShowCollapseAllButton: function onShowCollapseAllButton_List(){},

  /**
   * event triggered if the extra item is removed or the due date or state of the extra item is changed
   */
  onFullRefreshNeeded: function onFullRefreshNeeded_List(){},

  onBlur: function onBlur_List(){
    if (!this._keepSavedFocus) {
      this._clearSavedFocus();
    }
  },

  /*
   * event triggered if list has no more visible element to set the focus to
   */
  onFocusNeeded: function onFocusNeeded_List(){},

  /**
   * Save the information about the focus, reset saved focus if focus is not on a task row or category
   */
  saveFocus: function saveFocus_List(){
    com.ibm.bpm.social.util.common.Log.logExit(2);
    var focus = null;
    var foc = dijit.getFocus();
    var focusedNode = null;
    if (foc.node) {
      focusedNode = foc.node;
    }
    else if (foc.bookmark && foc.bookmark.mark && foc.bookmark.mark.startContainer) {
      focusedNode = foc.bookmark.mark.startContainer;
    }
    if (focusedNode) {
      focus = {};
      var widget = dijit.getEnclosingWidget(focusedNode);
      if (widget) {
        if (widget instanceof com.ibm.bpm.social.widgets.task.list.TaskRow && widget.model && widget.model.TKIID) {
          // task row widget
          focus.TKIID = widget.model.TKIID;
          focus.categoryId = widget.model.categoryId;
          var previousNode = widget.domNode.previousSibling;
          var nextNode = widget.domNode.nextSibling;
          if (previousNode) {
            widget = dijit.getEnclosingWidget(previousNode);
            if (widget && widget.model && widget.model.TKIID) {
              focus.previousTKIID = widget.model.TKIID;
            }
          }
          else {
            focus.previousTKIID = null;
          }
          if (nextNode) {
            widget = dijit.getEnclosingWidget(nextNode);
            if (widget && widget.model && widget.model.TKIID) {
              focus.nextTKIID = widget.model.TKIID;
            }
          }
          else {
            focus.nextTKIID = null;
          }
          com.ibm.bpm.social.util.common.Log.log(3, "Focused item is task " + focus.TKIID);
        }
        else if (widget instanceof com.ibm.bpm.social.widgets.task.list.Category) {
          // category Widget
          focus.categoryId = widget.id;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused item is category " + focus.categoryId);
        }
      }
    }

    // save the focus settings
    if (!focus || (!focus.TKIID && !focus.categoryId)) {
      focus = null;
    }
    var filterId = this._getFilterId();
    this._focus[filterId] = focus;

    com.ibm.bpm.social.util.common.Log.logExit(2, filterId + ": " + dojo.toJson(this._focus[filterId]));
  },

  /**
   * Restore the focus only if row was the focused row or if the focused group has been hidden
   * 
   * @param row the row widget which has been moved or removed
   * @param categoryWidget the group widget in which the row was before move or remove
   * @returns {Boolean} true if focus has been restored
   */
  _restoreFocusIfNeeded: function _restoreFocusIfNeeded_List(row, categoryWidget){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var focusRestored = false;
    var savedFocus = this._getSavedFocus();
    if (savedFocus) {
      if ((row && savedFocus.TKIID === row.model.TKIID) ||
        (categoryWidget && savedFocus.categoryId === categoryWidget.id && !categoryWidget.isDisplayed())) {
        focusRestored = this.restoreFocus(true);
      }
      else {
        this._clearSavedFocus();
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, focusRestored);
    return focusRestored;
  },

  /**
   * Restore the focus if possible and scroll the focused item into the view
   * <p>
   * Same task row if it still exists <br>
   * Next task row in same group if it (still) exists <br>
   * Previous task row in same group if it (still) exists <br>
   * Next group header if available <br>
   * Previous group header if available
   * <p>
   * saved focus is updated if changed<br>
   * saved focus is cleared if it cannot be restored or if not in autoRefresh mode
   * 
   * @param {Boolean} autoRefresh The saved focus is cleared if we are not in autoRefresh mode, e.g. on return
   *        from Task Completion View, Instance View
   * @returns {Boolean} true if focus has been restored
   */
  restoreFocus: function restoreFocus_List(autoRefresh){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var focusRestored = false;
    var focus = this._getSavedFocus();
    var saveNewFocus = true;
    if (focus) {
      com.ibm.bpm.social.util.common.Log.log(3, dojo.toJson(focus));
      var focusNode = null;
      if (focus.TKIID) {
        var row = this._getTaskWidget(focus.TKIID);
        if (row) {
          // same row
          focusNode = row.focusNode;
          saveNewFocus = false;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is same task " + focus.TKIID);
        }
      }
      if (!focusNode && focus.nextTKIID) {
        row = this._getTaskWidget(focus.nextTKIID);
        if (row) {
          // next row
          focusNode = row.focusNode;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is next task " + focus.nextTKIID);
        }
      }
      if (!focusNode && focus.previousTKIID) {
        row = this._getTaskWidget(focus.previousTKIID);
        if (row) {
          // previous row
          focusNode = row.focusNode;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is previous task " + focus.previousTKIID);
        }
      }
      if (!focusNode && focus.categoryId) {
        // find category widget
        var categoryWidget = this._getCategoryWidget(focus.categoryId);
        if (categoryWidget != null) {
          if (categoryWidget.isDisplayed()) {
            focusNode = categoryWidget.focusNode;
            saveNewFocus = (focus.TKIID !== undefined);
            com.ibm.bpm.social.util.common.Log.log(3, "Focused node is actual group " + focus.categoryId);
          }
          else {
            // find next displayed category
            var widget = null;
            var node = categoryWidget.domNode.nextSibling;
            while (node) {
              widget = dijit.getEnclosingWidget(node);
              if (widget.isDisplayed()) {
                focusNode = widget.focusNode;
                com.ibm.bpm.social.util.common.Log.log(3, "Focused node is next displayed group " + widget.id);
                break;
              }
              node = node.nextSibling;
            }
            if (node === null) {
              // find previous displayed category
              node = categoryWidget.domNode.previousSibling;
              while (node) {
                widget = dijit.getEnclosingWidget(node);
                if (widget.isDisplayed()) {
                  focusNode = widget.focusNode;
                  com.ibm.bpm.social.util.common.Log.log(3, "Focused node is previous displayed group " + widget.id);
                  break;
                }
                node = node.previousSibling;
              }
            }
          }
        }
      }
      if (focusNode !== null) {
        // set focus to node
        dojo.window.scrollIntoView(focusNode);
		// Defect 135428 - IE 8 unable to focus on nodes that are hidden, display = none
		try {
			focusNode.focus();
			focusRestored = true;
		} catch(e) {
			com.ibm.bpm.social.util.common.Log.log(3, "Unable to focus on node");
			// focus could not be restored, let the container set the focus
			this.onFocusNeeded();
		}
      }
      else {
        // focus could not be restored, let the container set the focus
        com.ibm.bpm.social.util.common.Log.log(3, "No node found to focus!");
        this.onFocusNeeded();
      }
    }
    else {
      // no saved focus, focus was somewhere else in the list - do not change the list focus
      com.ibm.bpm.social.util.common.Log.log(3, "No saved list focus - keep the actual focus");
    }
    // update saved focus
    if (!focusRestored || !autoRefresh) {
      this._clearSavedFocus();
    }
    else if (saveNewFocus) {
      this.saveFocus();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, focusRestored);
    return focusRestored;
  },

  _getSavedFocus: function _getSavedFocus_List(){
    var filterId = this._getFilterId();
    com.ibm.bpm.social.util.common.Log.log(2, filterId);
    return this._focus[filterId];
  },

  _clearSavedFocus: function _clearSavedFocus_List(){
    var filterId = this._getFilterId();
    com.ibm.bpm.social.util.common.Log.log(2, filterId);
    this._focus[filterId] = null;
  },

  setKeepSavedFocus: function setKeepSavedFocus_List(keep){
    com.ibm.bpm.social.util.common.Log.log(2, keep);
    this._keepSavedFocus = keep;
  },

  _addOpacitySettings: function _addOpacitySettings_List(node){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (dojo.isIE < 9) {
      dojo.style(node, "filter", "0");
    }
    else {
      dojo.style(node, "opacity", "0");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _removeOpacitySettings: function _removeOpacitySettings_List(node){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (dojo.isIE < 9) {
      dojo.style(node, "filter", null);
    }
    else {
      dojo.style(node, "opacity", null);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.Table"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.Table"] = true;
// Table.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.list.Table");










dojo.declare("com.ibm.bpm.social.widgets.task.list.Table", [dijit._Widget,
                                                            dijit._Templated],
  {

    templateString: dojo.cache("com.ibm.bpm.social.widgets.task", "list/templates/Table.html", "<div dojoAttachPoint=\"Table\" class=\"tableView\" role=\"grid\">\n\t<div class=\"tableViewCenter\" dojoAttachPoint=\"Center\" role=\"presentation\" tabIndex=\"-1\">\n\t    <!-- Please note that table gets additional text nodes if '>' is the last character on the line, user >< at the beginning instead -->\n\t\t<table dojoAttachPoint=\"HeaderTable\" class=\"tableViewHeaderTable bspace-style\" cellspacing=\"0\" cellpadding=\"0\" role=\"rowgroup\"\n\t\t\t><tbody dojoAttachPoint=\"_dnHeaderBody\" role=\"presentation\"></tbody\n\t\t></table>\n\t\t<div class=\"tableViewCenterScroller\" dojoAttachPoint=\"BodyCenter\" role=\"presentation\">\n\t\t\t<table class=\"tableViewCenterTable bspace-style\" dojoAttachPoint=\"DataTable\" cellspacing=\"0\" cellpadding=\"0\" role=\"rowgroup\"\n\t\t\t\t><tbody dojoAttachPoint=\"_dnTableBody\" role=\"presentation\"></tbody\n\t\t\t></table>\n\t\t</div>\n\t</div>\n\t<div dojoAttachPoint=\"Right\" class=\"tableViewFixedColumns\" role=\"presentation\">\n\t\t<div class=\"tableViewActionsTitle\" dojoAttachPoint=\"HeaderRight\" role=\"presentation\">\n\t\t\t<div class=\"headerInnerText\" dojoAttachPoint=\"HeaderRightFocusNode\" role=\"presentation\"></div>\n\t\t</div>\n\t\t<div class=\"tableViewFixedColumnScroller\" dojoAttachPoint=\"BodyRight\" role=\"presentation\">\n\t\t\t<table dojoAttachPoint=\"ActionTable\" class=\"tableViewFixedColumnTable bspace-style\" cellspacing=\"0\" cellpadding=\"0\" role=\"presentation\"\n\t\t\t\t><tbody dojoAttachPoint=\"ActionTableBody\" role=\"presentation\"></tbody\n\t\t\t></table>\n\t\t</div>\n\t</div>\n\t<div dojoAttachPoint=\"VeryRight\" style=\"width: 20px;\" class=\"tableViewFixedColumns\" role=\"presentation\">\n\t\t<div class=\"tableViewUpperCorners\" dojoAttachPoint=\"HeaderVeryRight\" role=\"presentation\"></div>\n\t\t<div class=\"tableViewVScroller\" dojoAttachPoint=\"VScrollerContainer\" tabIndex=\"-1\">\n\t\t\t<div style=\"width: 20px\" dojoAttachPoint=\"VScroller\"></div>\n\t\t</div>\n\t</div>\n</div>\n"),

    MAX_CELL_WIDTH: 150,
    CELL_PADDING: 13,

    /**
     * constructor
     * 
     * @param {Object} parameters members: config, model
     */

    constructor: function constructor_Table(parameters){


      this._sizeCache = {
        ScrollbarSize: 0,
        RowHeight: 0,
        HeaderHeight: 0,
        LeftWidth: 0,
        RightWidth: 0,
        ScrollerWidth: 0
      };

      this.scrollbarSize = this.getScrollbarSize();
      // private members

      //this._groupHeight = 0;
      // table DOM elements
      this._dnTableHeader = null;
      this._columns = []; // store this._config.displayedProperties
//        this._selectAllCheckbox = null;
//        this._showSelectAllCheckbox = true;
      this._filterOnProperties = {}; // keep track of filtered properties
      this._config = null; // Object specifying our layout and supportede DND types
      this._model = []; // List of JSON objects, each forming a row
      this._configurationApplied = false; // indicate if the current config has been set and rendered
      this._currentSelection = -1;
      //this._groupHeaders = []; // keep track of pseudo rows (=Group headers)
      this._lastPublishedSelection = [];
      this._lastChecked = null;
      this._lastOver = null;

      this._availableTableSpace = null; // set from the outside, is the max size the table can fill with data
      this._modelRepresentation = []; // List of JSON objects specifying the elements that belong to one row.
      // One row in a model is represented by multiple elements in the table. Here they are connected to access
      // easily all elements of a specific row.
      //  {dataRow:    <tr DOM node in data table>
      //   checkboxRow: <tr DOM node in checkbox table>
      //   checkbox:  <checkbox for that row>
      //   actionRow: <tr DOM node in that row>
      //   actions:  <list of inline actions in that row>
      //   isGroupHeader: <flag for group header>,
      //   offset:    <group header offset> // group headers are in the local model,
      //                                    // but not in the list, this is the offset
      //                    // of the accumulated group headers   }
      this._tableConnects = new Array();
      this._tableContentConnects = new Array();
      //this._groupHeaderOffset = -1;
      this.focusedRow = -1;
      this.focusedColumn = 0;
      this.focusNode = null;

      this.maxCellWidth = this.MAX_CELL_WIDTH;

      // cache the config & model if we got one
      if (parameters) {
        if (parameters.config) {
          this._config = parameters.config;
        }
        if (parameters.model) {
          this._model = parameters.model;
        }
      }
      this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    },


    /**
     * postCreate
     */
    postCreate: function postCreate_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (dojo.isIE && (dojo.isIE == 7)) {
        this.Center.style.width = "10000px";
      }


      this._dnTableHeader = document.createElement('tr');
      dijit.setWaiRole(this._dnTableHeader, "row"); // a11y
      this._dnHeaderBody.appendChild(this._dnTableHeader);
//        if (!this._showSelectAllCheckbox) {
//          dojo.addClass(this.CheckboxTableBody, 'bSpaceTableNoCheckBox');
//        }

      // perform actions on the objects we got in the constructor
      if (this._config) {
        this.setConfig(this._config);
        if (this._model && this._model.length > 0) {
          this.setModel(this._model);
        }
      }

      this._sizeCache.TableHeight = 300; // ensure non-zero defaults
      this._sizeCache.TableWidth = 300; // ensure non-zero defaults

// will be done in cleanup        
//        if (!com.ibm.bpm.social.util.common.Util.isIPad) {
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseout", this, "_onTableMouseOut"));
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseover", this, "_onTableMouseOver"));
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClick"));
//        }
//        else {
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClickDelayed"));
//        }
//        this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mousedown", this, "_onTableMouseDown"));
//        this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onkeydown", this, "_onRowKeyEvent"));


//        this._dndConnects = new Array();
//        this._dndConnects.push(dojo.subscribe("/dnd/drop", dojo.hitch(this, this._onDnDNotification)));
//        this._dndConnects.push(dojo.subscribe("/dnd/start", dojo.hitch(this, this._onDnDStart)));

      this._lastOver = null;
      this.HeaderRightFocusNode.innerHTML = this._nlsResources["task_actions_title"];
      this.HeaderRightFocusNode.title = this._nlsResources["task_actions_title"];
      dijit.setWaiState(this.domNode, "label", this._nlsResources["task_saved_searches_table_description"]);

      // maintain focus in table content when focus gets outside of content but inside of table
      this._tableConnects.push(dojo.connect(this.Table, "onfocus", this, "_onOuterTableFocus"));
      this._tableConnects.push(dojo.connect(this.VScrollerContainer, "onfocus", this, "_onOuterTableFocus"));
      this._tableConnects.push(dojo.connect(this.Center, "onfocus", this, "_onOuterTableFocus"));

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    ///////////////////////////////////////
    // public methods / Renderer interface
    ///////////////////////////////////////

    /**
     * Set the model(data) to be displayed and triggers a refresh
     * 
     * @param {Array} items Array of items
     * @param {Boolean} callRenderTable implicitly call renderTable
     * @type {void}
     */
    setModel: function setModel_Table(items, callRenderTable){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");
      try {
        this._model = items;

        this._cleanupTableContent();

        var i;
        var wasDirty = false;

        if (this._model.length > 0) {
          // check if we must render the header also
          if (this._config) {
            if (this._configurationApplied === false) {
              this._setColumns(this._config.displayedProperties);
              this._configurationApplied = true;
              wasDirty = true;
            }

            //this._model = this._prepareModelForGroupBy(items);

//            if (this._selectAllCheckbox) {
//              com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//            }
            //this._initializeGroupFormatter();

//            var itemsCreatedByDND = false;

//            if (!itemsCreatedByDND) {
            for (i = 0; i < this._model.length; i++) {
              this._renderRow(i);
            }
//            }

            // synchronize the row heights
            var rows = this._modelRepresentation;
            if (rows.length > 0) {
              // take largest row height
              this._sizeCache.RowHeight = Math.max(rows[0].dataRow.offsetHeight, rows[0].actionRow.offsetHeight);
              com.ibm.bpm.social.util.common.Log.log(1, "this._sizeCache.RowHeight=" + this._sizeCache.RowHeight + "px");
//            if (dojo.isFF || (dojo.isIE) || dojo.isChrome) {
              for ( var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var height = 0;
//                    if (row.isGroupHeader) {
//                      height = this.getGroupHeaderHeight();
//                    }
//                    else {
                height = this._sizeCache.RowHeight;
                row.dataRow.style.height = height + 'px';
//                    }
//                    if (row.checkboxRow) {
//                      row.checkboxRow.style.height = height + 'px';
//                    }
                row.actionRow.style.height = height + 'px';

              }
            }
//                else if (dojo.isIE && dojo.isIE <= 7) {
//                  // this is a bit complicated in IE since the table has not much interest in the height settings
//                  for (i = 0; i < rows.length; i++) {
//                    row = rows[i];
//                    height = -1;
////                    if (row.isGroupHeader) {
////                      if (i < rows.length - 1) {
////                        height = this.getGroupHeaderHeight(); //  + 1;
////                      }
////                    }
////                    else {
//                      height = this._sizeCache.RowHeight;
//                      height -= 3;
////                    }
//                    if (height >= 0) {
//                      row.dataRow.style.height = height + 'px';
//                      if (row.checkboxRow) {
//                        row.checkboxRow.style.height = height + 'px';
//                      }
//                      row.actionRow.style.height = height + 'px';
//                    }
//                  }
//                }
//                else {
//                  // webkit goes here
//                  for (i = 0; i < rows.length; i++) {
//                    row = rows[i];
//                    height = 0;
//                    //if (dojo.hasClass(rows[i], "groupBanner")) {
//                    if (row.isGroupHeader) {
//                      height = this.getGroupHeaderHeight();
//                      if (row.checkboxRow) {
//                        row.checkboxRow.style.height = height + 'px';
//                      }
//                      row.actionRow.style.height = height + 'px';
//                    }
//                }
//          }

            // Reset sort if setModel w/o setConfig is called
            if (wasDirty === false) {
              for (i = 0; i < this._config.displayedProperties.length; i++) {
                var prop = this._config.displayedProperties[i];
                prop.isSort = false;
                for ( var j = 0; j < this._config.sortProperties.length; j++) {
                  if (this._config.sortProperties[j].property == prop.id) {
                    prop.isSort = true;
                  }
                }
              }
            }
            this._setSortIndicators();

            // if search called in the meantime, display it now
            for (i in this._filterOnProperties) {
              if (i) {
                this.setSearchActive(i, true);
              }
            }

//            this.setLastFocus()
            if (this._refreshedBySort === true) {
              this._setFocusInHeader(true);
              this._refreshedBySort = false;

            }
            else {
              // set initial focus
              this._setFocusInHeader(false);
            }
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(1, "called with empty config -> noop!");
          }

//          if (this._config.maintainSelection) {
//            this._checkSelectAllStatus();
//            this._lastPublishedSelection = this.getCheckedData();
//          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(2, "no items");
        }

        if (callRenderTable) {
          this._afterRendering(true);
        }
      }
      catch (xcp) {
        com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
        throw (xcp);
      }
      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },


//    setLastFocus: function setLastFocus_Table(){
//      // set focus to filter icon in header of the last filter property
//      // otherwise we would lose the focus because the header is destroyed when refreshing
//      if (this.lastFilterProperty) {
//        var column = this._getColumnForProperty(this.lastFilterProperty);
//        if (column) {
//          var gif = dojo.query(".magnifier_hidden", column);
//          if (gif && gif.length > 0) {
//            window.setTimeout(function(){
//              gif[0].focus();
//            }, 100);
//          }
//        }
//        this.lastFilterProperty = null;
//      }
//    },


    /**
     * 
     * @type {void}
     */
    setSortProperties: function setSortProperties_Table(props){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (props) {
        this._config.sortProperties = props;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2);
      return;
    },

    /**
     * Set the Layout config of the table Setting the config purges the content
     * 
     * @param {Object} config
     */

    setConfig: function setConfig_Table(config){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");
      try {
        if (config) {
          this._config = config;

//            // ASZ@MoveToSocial prevent DND
//            this._config.supportedDndTypes = "";
//            this._config.publishedDndTypes = "";

          // ASZ@MoveToSocial prevent Grouping
          //this._config.activeGroupByProperty = null;

          if (!this._config.sortProperties) {
            this._config.sortProperties = [];
          }

          this._filterOnProperties = {};
          this._configurationApplied = false; // render header on setModel to prevent dirty display
          // ensure we have a dndSource but create only a new DnDSource if something has changed
//            if (!this._isPrototypTable) {
//              this._createDndTarget();
//              this._createDndSource();
//            }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1, "empty config received!");
        }
      }
      catch (xcp) {
        com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
        throw (xcp);
      }

      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },

    /**
     * Resize the table
     * 
     * @param {Object} dimensons JSON object {"h" : number, "w" : number }
     */
    resize: function resize_Table(dimensons){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");

      if (!dimensons) {
        return;
      }
      // initially we get a resize, but table sizes are not set.
      if (!this._sizeCache.LeftWidth || !this._sizeCache.RightWidth || !this._sizeCache.ScrollerWidth) {
        this._setTableSize();
      }
      if (dimensons.w && (dimensons.w > -1)) {
        var centerWidth =
          dimensons.w - this._sizeCache.LeftWidth - this._sizeCache.RightWidth - this._sizeCache.ScrollerWidth;
        if (centerWidth > this._sizeCache.CenterWidth) {
          // enlarge outer div first 
          this.domNode.style.width = dimensons.w + "px";
          this.Center.style.width = centerWidth + 'px';
        }
        else {
          // reduce inner div first
          this.Center.style.width = centerWidth + 'px';
          this.domNode.style.width = dimensons.w + "px";
        }

        this._sizeCache.CenterWidth = centerWidth;
        this._sizeCache.TableWidth = dimensons.w;
        com.ibm.bpm.social.util.common.Log.log(1, "New width " + dimensons.w + "px");
      }

      if (dimensons.h && (dimensons.h > -1)) {
        this.domNode.style.height = dimensons.h + "px";
        this._sizeCache.TableHeight = dimensons.h;
        com.ibm.bpm.social.util.common.Log.log(1, "New height " + dimensons.h + "px");
      }
      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },

    getScrollbarSize: function getScrollbarSize_Table(){

      var size;

      if ((size = this._sizeCache.ScrollbarSize)) {
        return size;
      }

      var div = document.createElement("div");
      div.style.visibility = "hidden";
      div.style.overflow = "scroll";
      div.style.position = "absolute";
      div.style.height = 100 + 'px';
      div.style.width = 100 + 'px';
      div.style.top = "-1000px";
      div.style.left = "-1000px";
      document.body.appendChild(div);
      size = div.offsetWidth - div.clientWidth;
      document.body.removeChild(div);
      this._sizeCache.ScrollbarSize = size + 1;
      // need to add 1px, otherwise it does not work
      // in some environments
      return size;
    },

//      /**
//       * Return the item displayed at index rowIndex (count starting at 0)
//       * 
//       * @type {void}
//       */
//      deselectAll: function deselectAll_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // check if we need to do anything
//        if (this.getCheckedData().length > 0) {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//          this._selectAllCheckboxClicked();
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//
//      },


    /**
     * activate / highlight rows (different to highlight from mouseclick) <br>
     * NOTE: should only be called with the indices from the original model that does not include any group
     * headers.
     * 
     * @param {Array} indices
     * @param {Boolean} switchOff if present, remove highlight instead of adding.
     */
    highlightItems: function highlightItems_Table(indices, switchOff){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var row = null;

      if (this._model.length > 0) {
        if (indices) {
          if (!(indices instanceof Array)) {
            indices = [indices];
          }

          for ( var i = 0; i < indices.length; i++) {
            // the index comes from the list and does not include the group headers.
            // thus we have to add an offset for potential group headers
            var index = (this._modelRepresentation[indices[i]].offset) + (indices[i]);
            if (index < this._modelRepresentation.length) {
              row = this._modelRepresentation[index];
              // For group headers we need to move one further
              //while (this._modelRepresentation.length - 1 > index && row.isGroupHeader) {
              while (this._modelRepresentation.length - 1 > index) {
                index++;
                row = this._modelRepresentation[index];
              }
              if (this._modelRepresentation.length > index && row !== null) {
                if (!switchOff) {
                  dojo.addClass(row.dataRow, "taskActive");
//                    if (this._config.isSelectable !== false) {
//                      dojo.addClass(row.checkboxRow, "taskActive");
//                    }
                  dojo.addClass(row.actionRow, "taskActive");
                }
                else {
                  dojo.removeClass(row.dataRow, "taskActive");
//                    if (this._config.isSelectable !== false) {
//                      dojo.removeClass(row.checkboxRow, "taskActive");
//                    }
                  dojo.removeClass(row.actionRow, "taskActive");
                }
              }
            }
          }
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /*
     * Delete all rows in the table
     */
    _cleanupTableContent: function _cleanupTableContent_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      // remove all event handlers on the table content
      this._lastOver = null;

      //this._groupHeaderOffset = -1;

      if (this._tableContentConnects !== null) {
        for ( var k = 0; k < this._tableContentConnects.length; k++) {
          if (this._tableContentConnects[k][0] != this._dnTableBody) {
            // remove all connections except the body which we can't purge because auf DnD
            dojo.disconnect(this._tableContentConnects[k]);
          }
        }
      }

      // Uncheck the row, otherwise some visual check indicator might remain.
      //        for(var i=0; i<this._modelRepresentation.length; i++){
      //          if (this._modelRepresentation.checkbox){
      //             com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._modelRepresentation.checkbox, false);
      //            }
      //        }
      // TODO : AB make sure all widgets are deleted. The grouping elements include dojo widgets.!!!!
      // remove the representation layer
      if (this._modelRepresentation) {
        delete this._modelRepresentation;
        this._modelRepresentation = [];
      }

      // delete all the data from the table
      // but we must keep the table body, because the DnDSource is bound to it!
      this.DataTable.removeChild(this._dnTableBody);
      this._dnTableBody = document.createElement('tbody');
      this.DataTable.appendChild(this._dnTableBody);

//        if (this.dndSource) {
//          var nodes = this.dndSource.getAllNodes();
//          for ( var i = 0; i < nodes.length; i++) {
//            this._dnTableBody.removeChild(nodes[i]);
//          }
//          this.dndSource.clearItems();
//        }

//        // cleanup any items not listed as DnD items
      while (this._dnTableBody.hasChildNodes()) {
        this._dnTableBody.removeChild(this._dnTableBody.firstChild);
      }

      this.ActionTable.removeChild(this.ActionTableBody);
      this.ActionTableBody = document.createElement('tbody');
      this.ActionTable.appendChild(this.ActionTableBody);

//        var tbody = this.CheckboxTable.removeChild(this.CheckboxTableBody);
//        var checkboxes = dojo.query("input", tbody);
//        if (checkboxes) {
//          for (i = 0; i < checkboxes.length; i++) {
//            com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(checkboxes[i]);
//          }
//        }


//        this.CheckboxTableBody = document.createElement('tbody');
//        this.CheckboxTable.appendChild(this.CheckboxTableBody);

      // create the event handlers again for the table content
      if (!com.ibm.bpm.social.util.common.Util.isIPad) {
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "mouseover", this, "_onTableMouseOver"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "mouseover", this, "_onTableMouseOver"));
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "mouseout", this, "_onTableMouseOut"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "mouseout", this, "_onTableMouseOut"));
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onclick", this, "_onTableClick"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onclick", this, "_onTableClick"));
      }
      else {
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onclick", this, "_onTableClickDelayed"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onclick", this, "_onTableClickDelayed"));
      }
/////////////////////////////////////////////////
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseout", this, "_onTableMouseOut"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseover", this, "_onTableMouseOver"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClick"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onkeydown", this, "_onKeyEvent"));
/////////////////////////////////////////////////

      this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onkeydown", this, "_onKeyEvent"));
//        this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onkeydown", this, "_onRowKeyEvent"));

      // event handlers for header rows
      this._tableContentConnects.push(dojo.connect(this.HeaderTable, "onkeydown", this, "_onKeyEvent"));
      this._tableContentConnects.push(dojo.connect(this.HeaderRight, "onkeydown", this, "_onKeyEvent"));
      this._tableContentConnects.push(dojo.connect(this.HeaderTable, "onclick", this, "_onHeaderClick"));
      this._tableContentConnects.push(dojo.connect(this.HeaderRight, "onclick", this, "_onHeaderClick"));

      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    /**
     * this function is not available for the table, a message should be displayed for a task
     */
    showMessage: function showMessage_Table(message, taskID){
      com.ibm.bpm.social.util.common.Log.logEntry(1);

      com.ibm.bpm.social.util.common.Log.logError(1, "This function is not available for the Table.");

      com.ibm.bpm.social.util.common.Log.logExit(1);
    },

    _getModelIndex: function _getModelIndex_Table(itemIndex){
      var numItems = Number(itemIndex) + 1;
      var count = 0;
      var modelIndex = 0;

      while (count < numItems) {
//          if (!(this._model[modelIndex] instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader)) {
//            count++;
//          }
        if (count < numItems) {
          modelIndex++;
        }
      }

      return modelIndex;
    },

    ///////////////////////////////////////
    // public events
    // (to connect to using dojo.connect)
    ///////////////////////////////////////

    /**
     * Sort triggered by user
     * 
     * @param {String} property column / property used as sort criteria
     * @param {String} direction one of {ASC|DESC}
     */
    onSort: function onSort_Table(property, direction){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. Sort on '" + property + "', " + direction);
    },


    /**
     * User dropped one (ToDo: or more?) item onto the table
     * 
     * @param {Array} objects list of objects that are dropped
     * @param {String} type the type of the drop
     */
    onDrop: function onDrop_Table(objects, type){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. dropped " + objects.length + " of type " + type);
    },


    /**
     * User clicked on an action icon
     * 
     * @param {String} action the id of the action clicked
     * @param {Object} item the item
     */
    onActionRequested: function onActionClicked_Table(action, item){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. Action : " + action);

    },


    /**
     * User selected one or more items <br>
     * Note: use the wrapper _onSelectionChanged to prevent double event emission.
     * 
     * @param {Array} items an array of 0...n items
     */
    onSelectionChanged: function onSelectionChanged_Table(items){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired, selected items:" + items.length);
    },


    /**
     * onFocusChanged
     * 
     * @param {Object} item
     */
    onFocusChanged: function onFocusChanged_Table(item){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired");
      com.ibm.bpm.social.util.common.Log.logObject(3, item);
    },


    /**
     * onFilter
     * 
     * @param {Object} property
     * @param {Object} node the dom node
     */
    onFilter: function onFilter_Table(property, node){
      this.lastFilterProperty = property;
      com.ibm.bpm.social.util.common.Log.log(1, "event fired: property=" + property);
    },

    ////////////////////////////////
    /* End of Renderer interface  */
    ////////////////////////////////
    /**
     * Activate the magnifier icon on a column
     * 
     * @param {Object} property
     * @param {Object} active optional, if missing, set search active for property
     */
    setSearchActive: function setSearchActive_Table(property, active){
      com.ibm.bpm.social.util.common.Log.log(1, "called: property=" + property + ", active=" + active);

      var column = null;
      var gifs = null;

      if (!property) {
        com.ibm.bpm.social.util.common.Log.log(1, "Error, no property received !");
        return;
      }

      if (active === null) {
        active = false;
      }

      column = this._getColumnForProperty(property);
      if (column) {
        gifs = dojo.query(".magnifier_hidden", column);
      }
      else {
        com.ibm.bpm.social.util.common.Log.log(1, "Error, no column found for property !");
      }

      // class for the magnifier icon
      if (gifs && gifs[0]) {
        if (active) {
          dojo.addClass(gifs[0], 'magnifier_active');
          this._filterOnProperties[property] = true;
        }
        else {
          dojo.removeClass(gifs[0], 'magnifier_active');
          delete (this._filterOnProperties[property]);
        }
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    /* Helpers for the controller */

    hasHScrollbar: function hasHScrollbar_Table(plannedWidth){
      var width = this._sizeCache.TableWidth;
      if (plannedWidth) {
        width = plannedWidth;
      }
      var leftWidth = 26;
      var rightWidth = 26;
      var hasHScroller = (this.DataTable.offsetWidth > width - leftWidth - rightWidth - this.scrollbarSize);

      var hScroll = hasHScroller;
      com.ibm.bpm.social.util.common.Log.log(1, "returns " + hScroll);
      return hScroll;

    },

    hasVScrollbar: function hasVScrollbar_Table(){
      var height = this._sizeCache.TableHeight;
      var width = this._sizeCache.TableWidth;

      var headerHeight = this._sizeCache.HeaderHeight;
      if (!this._sizeCache.HeaderHeight) {
        this._sizeCache.HeaderHeight = headerHeight;
      }

      var leftWidth = 26;
      var rightWidth = 26;
      var hasHScroller = (this.DataTable.offsetWidth > width - leftWidth - rightWidth - this.scrollbarSize);
      var scrollerHeight = hasHScroller ? this.scrollbarSize : 0;
      var hasVScroller = (this.DataTable.offsetHeight > height - headerHeight - scrollerHeight);

      var vScroll = hasVScroller;
      com.ibm.bpm.social.util.common.Log.log(1, "returns " + vScroll);
      return vScroll;

    },

    getTableHeights: function getTableHeights_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");


      var rowHeight = this.getRowHeight();
      com.ibm.bpm.social.util.common.Log.log(1, "rowHeight=" + rowHeight);

      var headHeight = this.getHeaderHeight();
      com.ibm.bpm.social.util.common.Log.log(1, "headHeight=" + headHeight);

      //var groupHeight = this.getGroupHeaderHeight();
      //com.ibm.bpm.social.util.common.Log.log(1, "groupHeight=" + groupHeight);


      var _tableHeights = {
        headHeight: headHeight,
        //groupHeight: groupHeight,
        rowHeight: rowHeight
      };


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return _tableHeights;
    },


    getHeaderHeight: function getHeaderHeight_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var height;
      if (this._sizeCache.HeaderHeight) {
        height = this._sizeCache.HeaderHeight;
      }
      else {
        if (!this._configurationApplied) {
          this._prototypeTable();
        }
        height = this.HeaderTable.offsetHeight;
      }
      if (!height || height < 10) {
        height = 26;
      }
      this._sizeCache.HeaderHeight = height;

      com.ibm.bpm.social.util.common.Log.logExit(2, "HeaderHeight is " + height);
      return height;
    },


    getRowHeight: function getRowHeight_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var height = null;

      if (this._sizeCache.RowHeight) {
        height = this._sizeCache.RowHeight;
      }
      else {
        if (this._modelRepresentation && this._modelRepresentation.length > 0) {
          height = this._modelRepresentation[0].dataRow.offsetHeight;
        }
        else {
          this._prototypeTable();
          if (this._modelRepresentation && this._modelRepresentation.length > 0) {
            height = this._modelRepresentation[0].dataRow.offsetHeight;
          }
          // remove any prototype rows
          //AB: A complete cleanup is necessary
          this._cleanupTableContent();
        }
      }
      if (!height || height < 10) {
        // ensure a number, just a default
        height = 25;
      }

      this._sizeCache.RowHeight = height;
      com.ibm.bpm.social.util.common.Log.logExit(2, "RowHeight is :" + height + "px");
      return height;
    },

//      getGroupHeaderHeight: function getGroupHeaderHeight_Table(){
//
//        if (this._config && this._config.groupFormatter) {
//          this._groupHeight = this._config.groupFormatter.getHeaderHeight();
//        }
//        return this._groupHeight;
//      },

    /**
     * calculate how many rows will fit in the given height
     * 
     * @param {Number} height
     */
    getMaxRowCount: function getMaxRowCount_Table(height){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      // calculate the maximal number of rows new, because available space changed.
      // As long as the height does not change, the max row count will not change.
      if (!this._availableTableSpace || height != this._availableTableSpace) {
        var tableHeights = this.getTableHeights();
        var headHeight = this._sizeCache.HeaderHeight;

        if (!this._sizeCache.HeaderHeight) {
          this._sizeCache.HeaderHeight = this.HeaderTable.offsetHeight;
        }
        var rowHeight = tableHeights.rowHeight;
//          var groupHeight = 0;
//          if (this._config && this._config.activeGroupByProperty) {
//            // we have at least one group header
//            groupHeight = tableHeights.groupHeight;
//          }
        com.ibm.bpm.social.util.common.Log.log(1, "headHeight=" + headHeight + "; rowHeight=" + rowHeight);

//          var availableRowSpace = height - headHeight - groupHeight - 1; // - 1 since the first row has an additional 1px border-top
        var availableRowSpace = height - headHeight - 1; // - 1 since the first row has an additional 1px border-top
        com.ibm.bpm.social.util.common.Log.log(1, "availableRowSpace (after subtracting headHeight)=" + availableRowSpace);

        // FLM: if we have a HScroller we will most likely have one every page
        var scrollHeight = this.hasHScrollbar() ? this.scrollbarSize : 0;
        this._maxRowCount = Math.floor((availableRowSpace - scrollHeight) / (rowHeight));
        // we need to calculate the max row count new, when we do not have actual data right now.
        // because there can be an horizontal scrollbar later
        if (this._modelRepresentation.length > 0 && !this._isPrototypTable) {
          this._availableTableSpace = height;
        }
      }
      com.ibm.bpm.social.util.common.Log.log(1, "exit, maxRowCount=" + this._maxRowCount);
      return this._maxRowCount;
    },

    /*
     *  Calculate how many rows fit the space really. Max Row Count is a guess, when the
     *  model is not set. But with the model the real number can be calculated. Because
     *  for grouping the number of fitting rows can be less then max row count.
     */
    getRowCount: function getRowCount_Table(height, removeFromTop){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var rowCount = 0;
      // if we have grouping and not a prototype table and an actual model representation
//        if (this._config && this._config.activeGroupByProperty && !this._isPrototypTable &&
      if (this._config && !this._isPrototypTable && this._modelRepresentation.length > 0) {
        var tableHeights = this.getTableHeights();
        var headHeight = this._sizeCache.HeaderHeight;
        if (!this._sizeCache.HeaderHeight) {
          this._sizeCache.HeaderHeight = this.HeaderTable.offsetHeight;
        }
        var rowHeight = tableHeights.rowHeight;
        // FLM: if we have a HScroller we will most likely have one every page
        var scrollHeight = this.hasHScrollbar() ? this.scrollbarSize : 0;
        var availableRowSpace = height - scrollHeight - headHeight - 1; // - 1 since the first row has an additional 1px border-top
        // calculate the group header height
        //var groupHeight = tableHeights.groupHeight;

        // we have at least one group header and not a prototype table
        // now start calculating the number of rows.
        if (!removeFromTop) {
          // Start at the top of the list to check, how many elements fit into the
          // table. The elements at the end of the list that don't fit are removed.
          var i = 0;
          while (availableRowSpace > 0 && i < this._modelRepresentation.length) {
//              if (this._modelRepresentation[i].isGroupHeader) {
//                availableRowSpace = availableRowSpace - groupHeight;
//              }
//              else {
            availableRowSpace = availableRowSpace - rowHeight;
            rowCount++;
//              }
            i++;
          }
          // If there is not enough space for the last row, then reduce the row count.
          // A group header does not count, but we have to make sure to call the remove later in the entityList
          // Thus the posRow is used instead for the correct position.
//            if (availableRowSpace < 0 && rowCount > 0 && !this._modelRepresentation[i - 1].isGroupHeader) {
          if (availableRowSpace < 0 && rowCount > 0) {
            rowCount--;
          }
          // now remove the elements between i and modelRepresentation.length, thus store the pos, we can not
          // remove here, because that is called from entity list.
          // We can delete this direction quite easy, but the other one we can not. So this
          // direction will be deleted by the table removeLastRows and the other by a new setModel call.
          i--;
          this._posRemove = i;
        }
        else {
          // start at the bottom of the list and check how many items fit into the table
          // everything above that are removed.
          i = this._modelRepresentation.length - 1;
          while (i > 0 && availableRowSpace > 0) {
//              if (this._modelRepresentation[i].isGroupHeader) {
//                if (availableRowSpace - groupHeight > 0) {
//                  availableRowSpace = availableRowSpace - groupHeight;
//                }
//                else {
//                  // should never happen
//                  if (rowCount > 0) {
//                    rowCount--;
//                  }
//                  availableRowSpace = 0;
//                }
//              }
//              else {
            if (availableRowSpace - rowHeight > 0) {
//                if (availableRowSpace - groupHeight - rowHeight > 0) {
              availableRowSpace = availableRowSpace - rowHeight;
              rowCount++;
            }
            else {
              // if the predecessor is a group header, then we can just stop here
              availableRowSpace = 0;
//                  if (rowCount > 0 && !this._modelRepresentation[i + 1].isGroupHeader) {
              if (rowCount > 0) {
                rowCount--;
              }
            }
//              }
            i--;
          }
        }
      }
      else {
        rowCount = this.getMaxRowCount(height);
      }
      com.ibm.bpm.social.util.common.Log.log(1, "exit, rowCount=" + rowCount + "  pos " + i);
      return rowCount;

    },


    /*
     *  Remove the specified number of rows.
     */
    removeLastRows: function(count){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var len = this._modelRepresentation.length;
      var pos = this._posRemove;
      var i, k;
      this._posRemove = null;
      // if we have stored the exact position from where to delete, then use this one. Because then
      // we do not need to look for the last element.
      if (pos > 0) {
        // delete the rows but not the group header, if there are more elements still for this group
        for (k = pos; k < len; k++) {
          var row = this._modelRepresentation.pop();
          this._model.pop();
//            if (row.isGroupHeader) {
//              // remove the last element from the array
//              this._groupHeaders.pop();
//            }
          this._dnTableBody.removeChild(row.dataRow);
          this.ActionTableBody.removeChild(row.actionRow);
//            this.CheckboxTableBody.removeChild(row.checkboxRow);
        }
      }
      else {
        // use the count to delete the rows
        k = 0; // count the model elements
        i = 0; // count removed rows
        while (i < count && k < len) {
          row = this._modelRepresentation.pop();
          this._model.pop();
//            if (row.isGroupHeader) {
//              // remove the last element from the array
//              this._groupHeaders.pop();
//            }
//            else {
          k++;
//            }
          i++;
          this._dnTableBody.removeChild(row.dataRow);
          this.ActionTableBody.removeChild(row.actionRow);
//            this.CheckboxTableBody.removeChild(row.checkboxRow);
        }
      }

//        // check if the last element in the table is now a group header
//        var last = this._modelRepresentation.length - 1;
//        if (last >= 0 && this._modelRepresentation[last].isGroupHeader) {
//          row = this._modelRepresentation.pop();
//          this._model.pop();
//          // remove the last element from the array
//          this._groupHeaders.pop();
//          this._dnTableBody.removeChild(row.dataRow);
//          this.ActionTableBody.removeChild(row.actionRow);
//          this.CheckboxTableBody.removeChild(row.checkboxRow);
//        }


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },
    ///////////////////////////////////////
    // dojo lifecycle functions
    ///////////////////////////////////////
    /**
     * Call this method to trigger rendering after model has been set. Call is also after the table has been
     * modified.
     */
    renderTable: function renderTable_Table(delayedExecution){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (delayedExecution === null || delayedExecution === undefined) {
        delayedExecution = true;
      }
      this._afterRendering(delayedExecution);
      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    _afterRendering: function _afterRendering_Table(delayedExecution){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var self = this;


      if (delayedExecution) {
        // do calculation in new thread.
        // this gives the browser a chance to render the table before measurement takes place.
        // neede for iPad support but does not seem to harm other platforms
        window.setTimeout(function(){
          self._afterRendering(false);
        }, 1);
        com.ibm.bpm.social.util.common.Log.logExit(2, "delayedExecution, exiting w/o actions", "Table");
        return;
      }


      if (!this._tableHeights) {
        this.getTableHeights();
      }

      this.maxCellWidth =
        Math.max(this.MAX_CELL_WIDTH, Math.floor(this._sizeCache.CenterWidth / this._config.displayedProperties.length));
      com.ibm.bpm.social.util.common.Log.log(2, "maxCellWidth is now " + this.maxCellWidth);


      if (this._modelRepresentation && this._modelRepresentation.length > 0) {
        this._handleCellWidth();
      }

      this._setTableSize();

      if (this._modelRepresentation && this._modelRepresentation.length > 0) {
        //if (rows.length > 0) {
        this._handleLargeCells();
      }

      if (!this.vScrollEvent) {
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.BodyCenter.style.overflowY = "auto";
          this.vScrollEvent = dojo.connect(this.BodyCenter, 'onscroll', this, "_moveVerticalTouch");
        }
        else {
          this.vScrollEvent = dojo.connect(this.VScrollerContainer, 'onscroll', this, "_moveVertical");
        }
      }

      if (!this.hScrollEvent) {
        this._lastHScrollLeft = this.Center.scrollLeft;
        this.hScrollEvent = dojo.connect(this.Center, 'onscroll', this, "_moveHorizontal");
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _moveVertical: function _moveVertical_Table(e){
      var y = this.VScrollerContainer.scrollTop;
      // check focus if view is scrolled
      com.ibm.bpm.social.util.common.Log.log(3, "scrollTop " + y + ", DataTable.offsetTop " + this.DataTable.offsetTop);
      var checkFocus = this.DataTable.offsetTop !== -y;
      this.DataTable.style.top = -y + 'px';
      this.ActionTable.style.top = -y + 'px';
//        this.CheckboxTable.style.top = -y + 'px';
      if (checkFocus) {
        this._focusNextVisibleRow();
      }
    },

    _moveVerticalTouch: function _moveVerticalTouch_Table(e){
      var y = this.BodyCenter.scrollTop;
      com.ibm.bpm.social.util.common.Log.log(3, "scrollTop " + y + ", ActionTable.offsetTop " + this.ActionTable.offsetTop);
      var checkFocus = this.ActionTable.offsetTop !== -y;
      //this.DataTable.style.top = -y + 'px';
      this.ActionTable.style.top = -y + 'px';
//        this.CheckboxTable.style.top = -y + 'px';
      if (checkFocus) {
        this._focusNextVisibleRow();
      }
    },

//    /**
//     * Move the viewport so that the last item is shown
//     * 
//     * @type {void}
//     */
//    scrollToLastItem: function scrollToLastItem_Table(){
//      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//      var delta = this.VScrollerContainer.scrollHeight - this.VScrollerContainer.clientHeight;
//      this.DataTable.style.top = -delta + 'px';
//      this.ActionTable.style.top = -delta + 'px';
////        this.CheckboxTable.style.top = -delta + 'px';
//      this.VScroller.style.top = -delta + 'px';
//      this.VScrollerContainer.style.top = -delta + 'px';
//      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      return;
//    },

    /**
     * Change the focus if the focused row scrolls out of the visible area
     */
    _focusNextVisibleRow: function _focusNextVisibleRow_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusedRow !== -1) {
        com.ibm.bpm.social.util.common.Log.log(3, "focused row " + this.focusedRow);
        var index = this.focusedRow;
        var scrollHeight = this.Table.offsetHeight - this.HeaderTable.offsetHeight;
        var focusedRow = this._modelRepresentation[index].dataRow;
        var focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
        var focusedRowBottom = focusedRowTop + focusedRow.offsetHeight;
        com.ibm.bpm.social.util.common.Log.log(3, "scrollHeight " + scrollHeight + ", focusedRowTop " + focusedRowTop +
          ", focusedRowBottom " + focusedRowBottom);
        if (focusedRowBottom > scrollHeight) {
          // set focus to previous visible row
          while (focusedRowBottom > scrollHeight) {
            index--;
            focusedRow = this._modelRepresentation[index].dataRow;
            focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
            focusedRowBottom = focusedRowTop + focusedRow.offsetHeight;
          }
          this._setFocusedRow(this.focusedRow, index);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to row " + index);
        }
        else if (focusedRowTop <= 0) {
          // set focus to next visible row
          while (focusedRowTop < 0) {
            index++;
            focusedRow = this._modelRepresentation[index].dataRow;
            focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
          }
          this._setFocusedRow(this.focusedRow, index);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to row " + index);
        }
        else {
          this._setFocus(false);
        }
      }
      else {
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _moveHorizontal: function _moveHorizontal_Table(e){
      // check focus if view is scrolled
      com.ibm.bpm.social.util.common.Log.log(3, "lastHScrollLeft " + this._lastHScrollLeft + " scrollLeft " +
        this.Center.scrollLeft);
      if (this._lastHScrollLeft !== this.Center.scrollLeft) {
        this._focusNextVisibleColumn();
        this._lastHScrollLeft = this.Center.scrollLeft;
      }
    },

    /**
     * Change the focus if the focused column scrolls out of the visible area
     */
    _focusNextVisibleColumn: function _focusNextVisibleColumn_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "focused row " + this.focusedRow + " focusedColumn " + this.focusedColumn);
      if (this.focusedColumn < this._getChildElementCount(this.HeaderTable.rows[0])) {
        // focused column is in scrollable view
        var focusedRow = null;
        if (this.focusedRow === -1) {
          focusedRow = this.HeaderTable.rows[0];
        }
        else {
          focusedRow = this._modelRepresentation[this.focusedRow].dataRow;
        }
        var index = this.focusedColumn;
        var focusColumn = focusedRow.childNodes[index];
        var scrollLeft = this.Center.scrollLeft;
        var scrollRight = scrollLeft + this.Center.offsetWidth;
        var focusColumnLeft = focusColumn.offsetLeft;
        var focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
        com.ibm.bpm.social.util.common.Log.log(3, "scrollLeft " + scrollLeft + " scrollRight " + scrollRight +
          " focusColumnLeft " + focusColumnLeft + " focusColumnRight " + focusColumnRight);
        if (this._lastHScrollLeft < scrollLeft && focusColumnLeft < scrollLeft) {
          // left of visible area - move focus to the right
          while (focusColumnLeft < scrollLeft) {
            index++;
            focusColumn = focusedRow.childNodes[index];
            focusColumnLeft = focusColumn.offsetLeft;
            //focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
          }
          this.focusedColumn = index;
          this._setFocusToNode(focusColumn.focusNode, true);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to column " + index);
        }
        else if (this._lastHScrollLeft > scrollLeft && focusColumnRight > scrollRight) {
          // right of visible area - move focus to the left            
          while (focusColumnRight > scrollLeft + this.Center.offsetWidth) {
            index--;
            focusColumn = focusedRow.childNodes[index];
            //focusColumnLeft = focusColumn.offsetLeft;
            focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
          }
          this.focusedColumn = index;
          this._setFocusToNode(focusColumn.focusNode, true);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to column " + index);
        }
        else {
          this._setFocus(false);
        }
      }
      else {
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _setTableSize: function _setTableSize_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var width = this._sizeCache.TableWidth;
      var height = this._sizeCache.TableHeight;

//        for ( var i = 0; i < this._groupHeaders.length; i++) {
//          // hide group Headers since they disrupt measuring
//          this._groupHeaders[i].style.display = "none";
//        }


      this.BodyCenter.style.width = "";
      this.BodyCenter.style.height = "";
      this.DataTable.style.display = "inline-table";
      this.HeaderTable.style.display = "inline-table";
      this.DataTable.style.width = "";
      // for some reason this does not work in FF
      if ((dojo.isIE && (dojo.isIE < 8))) {
        this.DataTable.style.height = "";
      }
      this.HeaderTable.style.width = "";
      this.HeaderTable.style.height = "";

      // start measuring
      var headerHeight = this.HeaderTable.offsetHeight;
      this._sizeCache.HeaderHeight = headerHeight;

      var dataWidth = this.DataTable.firstChild.offsetWidth;
      //var dataHeight = this.DataTable.firstChild.offsetHeight + this._groupHeaders.length * this._groupHeight;
      var dataHeight = this.DataTable.firstChild.offsetHeight;
      var headerWidth = this.HeaderTable.offsetWidth;
      var leftWidth = 0; // this.HeaderLeft.offsetWidth;
      var rightWidth = this.ActionTable.offsetWidth;

      com.ibm.bpm.social.util.common.Log.log(2, "Measures: dataHeight:" + dataHeight + "px, dataWidth:" + dataWidth +
        "px, headerWidth:" + headerWidth + "px, headerHeight:" + headerHeight + "px, leftWidth:" + leftWidth +
        "px, rightWidth:" + rightWidth + "px");

      // end measuring


      // some sanity checks
      if (rightWidth > 100) {
        // to wide, make no sense
        rightWidth = 28;
      }
      else if (rightWidth < 20) {
        // to small, no inline action at all 
        rightWidth = 0;
      }

//        // left contains only checkboxes, they cant be bigger than 30 (?) px
//        if (leftWidth > 30) {
//          leftWidth = 24;
//        }
//
//        // if we have no checkboxes at all, don't show the left side at all
//        if (this._config.isSelectable === false) {
      leftWidth = 0;
//        }

//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          if (headerHeight > 25) {
//            headerHeight = 20;
//          }
//        }
      // }

      this.DataTable.style.display = "block";
      this.HeaderTable.style.display = "block";


      // if the "no items" message is shown
      dataWidth = Math.max(dataWidth, headerWidth);
      headerWidth = dataWidth;
      this._sizeCache.ContentWidth = dataWidth;

      this.DataTable.style.width = dataWidth + 'px';

//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          this.DataTable.style.height = dataHeight + 'px';
//          this.ActionTable.style.height = dataHeight + 'px';
////          this.CheckboxTable.style.height = dataHeight + 'px';
//        }


      this.HeaderTable.style.width = dataWidth + 'px';

      this.BodyCenter.style.width = dataWidth + 'px';


      if (this._config.fitToContent) {
        width = dataWidth + leftWidth + rightWidth + 2;
        height = dataHeight + headerHeight + 2;
        this.domNode.style.width = width + 'px';
        this.domNode.style.height = height + 'px';
        this._sizeCache.TableHeight = height;
        this._sizeCache.TableWidth = width;
      }

      var hasHScroller = (dataWidth > width - leftWidth - rightWidth - this.scrollbarSize);
      var scrollerHeight = hasHScroller ? this.scrollbarSize : 0;

      var hasVScroller = (dataHeight > height - headerHeight - scrollerHeight);
      var scrollerWidth = hasVScroller ? this.scrollbarSize : 0;

      if (this._config.fitToContent) {
        scrollerHeight = 0;
        scrollerWidth = 0;
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "Sizes before calculating centerWidth: width:" + width +
        "px, leftWidth:" + leftWidth + "px, rightWidth:" + rightWidth + "px, scrollerWidth:" + scrollerWidth + "px");
      var centerWidth = width - leftWidth - rightWidth - scrollerWidth;

      this._sizeCache.CenterWidth = centerWidth;

      if (!hasVScroller) {
        this.VeryRight.style.display = "none";
        this.DataTable.style.top = '0px';
        this.ActionTable.style.top = '0px';
//          this.CheckboxTable.style.top = '0px';
      }
      else {
        this.VeryRight.style.display = "block";
      }

      this.Center.style.height = height + 'px';
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        this.BodyCenter.style.height = (height - headerHeight) + 'px';
      }
      // this.Left.style.width = leftWidth + 'px';
      this.Right.style.width = rightWidth + 'px';
      this.Center.style.width = centerWidth + 'px';
      this.VeryRight.style.width = scrollerWidth + 'px';
      this.VScroller.style.width = scrollerWidth + 'px';
      this.VScrollerContainer.style.width = (scrollerWidth + 1) + 'px';
      this.VScroller.style.height = dataHeight + 'px';
      this._appendOrResizeVScrollerImage(dataHeight, scrollerWidth);

//        if (leftWidth > 2 && headerHeight > 2) {
//          this.HeaderLeft.style.width = (leftWidth - 2) + 'px'; // -2 is the border
//          this.HeaderLeft.style.height = (headerHeight - 2) + 'px';
//        }
//        else {
//          this.HeaderLeft.style.width = '0px';
//          this.HeaderLeft.style.height = '0px';
//        }

      if (rightWidth > 2 && headerHeight > 2) {
        this.HeaderRight.style.width = (rightWidth - 2) + 'px';
        // this.HeaderRight.style.height = (headerHeight - 2) + 'px';
        if (dojo.isFF) {
          this.HeaderRight.style.height = headerHeight + 'px'; // bug: using "display: table-cell" reduces height in FF   
        }
        else {
          this.HeaderRight.style.height = (headerHeight - 2) + 'px';
        }

      }
      else {
        this.HeaderRight.style.width = '0px';
        this.HeaderRight.style.height = '0px';
      }

      this.HeaderVeryRight.style.width = (scrollerWidth > 2 ? scrollerWidth - 2 : scrollerWidth) + 'px'; // 1 px padding left and right
      this.HeaderVeryRight.style.height = (headerHeight > 2 ? headerHeight - 2 : headerHeight) + 'px'; // 1 px padding top and bottom
      // this.BodyLeft.style.width = leftWidth + 'px';
      this.BodyRight.style.width = rightWidth + 'px';

      var newHeight = height - headerHeight - scrollerHeight;
      if (newHeight < 0) {
        newHeight = 0;
      }
      // this.BodyLeft.style.height = newHeight + 'px';
      this.BodyRight.style.height = newHeight + 'px';
      if (newHeight > 2) {
        // HeaderVeryRight placeholder has 2px padding
        this.VScrollerContainer.style.height = (newHeight - 2) + 'px';
      }
      else {
        this.VScrollerContainer.style.height = newHeight + 'px';
      }

      this._sizeCache.LeftWidth = leftWidth;
      this._sizeCache.CenterWidth = centerWidth;
      this._sizeCache.RightWidth = rightWidth;
      this._sizeCache.ScrollerWidth = scrollerWidth;


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return true;
    },

    // get first row, that is not a group header
    _getFirstDataRow: function(rows){
      var k = 0;
//        while (rows[k].isGroupHeader === true && k < rows.length) {
//          k++;
//        }
//        if (k >= rows.length) {
//          return null;
//        }
//        else {
      return rows[k].dataRow;
//        }
    },

    _handleCellWidth: function _handleCellWidth_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var widthMap = [];
      this.actualWidthMap = [];
      var cell, node, headerCells, headerText, t, i, row;

      var rows = this._modelRepresentation;
      var headerRow = this._dnHeaderBody.firstChild;
      // determine size
      // Each column in the data table has now the size to fit the content, 
      // so we need to measure only the first rows cells


      var padding = this.CELL_PADDING;

      if (rows.length > 0) {
        // find the first row with real data in it
        var dataRow = this._getFirstDataRow(rows);
        if (dataRow !== null) {

          // iterate over all columns of the first row
          for (t = 0; t < dataRow.childNodes.length; t++) {

            var cellWidth = dataRow.childNodes[t].offsetWidth;
            if (cellWidth < padding) {
              // avoid problems with negative values
              cellWidth = padding;
            }

            if (cellWidth - padding > this.maxCellWidth) {
              // the column is wider than max size, we need to backup its original size
              // for use in _handleLargeCells later
              for (i = 0; i < rows.length; i++) {
                row = rows[i];
//                  if (!row.isGroupHeader) {
                cell = row.dataRow.childNodes[t];
                node = cell.firstChild;
                if (node) {
                  var width = cellWidth - padding;
                  if (dataRow !== row) {
                    width = node.offsetWidth; // this saves one set of offsetWidth
                  }
                  cell.originalWidth = width;
                  if (!widthMap[t] || width > widthMap[t]) {
                    widthMap[t] = width;
                  }
                  if (widthMap[t] > this.maxCellWidth) {
                    widthMap[t] = this.maxCellWidth;
                  }
                }
//                  }
              }
            }
            else {
              if (!widthMap[t] || cellWidth - padding > widthMap[t]) {
                widthMap[t] = cellWidth - padding;
              }
            }
          }
        }
      } // end if (rows.length > 0)

      // determine header width
      for (t = 0; t < headerRow.childNodes.length; t++) {
        cell = headerRow.childNodes[t];
        node = cell.firstChild;
        if (node.tagName == "TABLE") {
          // in webkit only the table body has a width, not the table itself
          node = node.firstChild;
        }
        if (node) {
          width = node.offsetWidth - 2;
          if (width < 0) {
            // avoid problems with negative sizes
            width = 0;
          }
          cell.originalWidth = width;
          if (!widthMap[t] || width > widthMap[t]) {
            widthMap[t] = width;
          }
        }
      }

      // set size
      // header
      for (t = 0; t < headerRow.childNodes.length; t++) {
        cell = headerRow.childNodes[t];
        node = cell.firstChild;
        if (node && dojo.hasClass(cell, "tableCell")) {
          width = widthMap[t];
          // save the actual size for restoring after refresh
          this.actualWidthMap[t] = width;
          node.style.width = (width + 2) + 'px';
          node.style.display = "block";

          // handle large text
          headerCells = node.getElementsByTagName("td");
          if (headerCells && headerCells.length > 2) {
            headerText = headerCells[1].firstChild;
            if (headerText) {
              if (width < 32) {
                // avoid problems with negative sizes
                width = 32;
              }
              if (this._config.isFilterable) {
                // cell width - sort div - filter div
                headerText.style.width = (width - 32) + "px";
              }
              else {
                //  filter div is hidden
                headerText.style.width = (width - 16) + "px";
              }
            }
          }
        }
      }

      // data table
      for (i = 0; i < rows.length; i++) {
        row = rows[i];
//          if (!row.isGroupHeader) {
        for (t = 0; t < row.dataRow.childNodes.length; t++) {
          cell = row.dataRow.childNodes[t];
          node = cell.firstChild;
          if (node && dojo.hasClass(cell, "tableData")) {
            width = widthMap[t];
            if (width > 0) {
              node.style.width = width + 'px';
              node.style.display = "block";
              if (dojo.hasClass(cell, "bSpaceBlankImageHolder")) {
                node.style.height = '1px';
              }
            }
          }
//            }
        }
      }

      this.widthMap = widthMap;
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * If we have free horizontal space give it to all cells who need it. If the cell is still to large, make
     * an ellipsis.
     */
    _handleLargeCells: function _handleLargeCells_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      // do we have free space?
      var freeSpace = this._sizeCache.CenterWidth - this.DataTable.offsetWidth;
      var initialFreeSpace = freeSpace;
      var widthMap = this.widthMap;
      var monsterCells = [];
      var monsterHeaders = [];

      // handle too large cells
      var headerRow = this._dnHeaderBody.firstChild;
      var rows = this._modelRepresentation;

      for ( var t = 0; t < headerRow.childNodes.length; t++) {
        var addWidth = 0;

        if (widthMap[t] > this.maxCellWidth) {
          addWidth = Math.min(widthMap[t] - this.maxCellWidth, freeSpace);
          if (addWidth < 0) {
            addWidth = 0;
          }
          if (addWidth < Math.min(widthMap[t] - this.maxCellWidth)) {
            // check for monstercells
            // check in header
            var cell = headerRow.childNodes[t];
            var node = cell.firstChild;
            if (cell.originalWidth > this.maxCellWidth + addWidth) {
              node.maxWidth = widthMap[t] + this.CELL_PADDING;
              monsterCells.push(node);
            }
            // check in data cells
            for ( var i = 0; i < rows.length; i++) {
//                if (!rows[i].isGroupHeader) {
              //if (!dojo.hasClass(rows[i], "groupBanner")) {
              cell = rows[i].dataRow.childNodes[t];
              //var cell = rows[i].childNodes[t];
              node = cell.firstChild;
              if (cell.originalWidth > this.maxCellWidth + addWidth) {
                node.maxWidth = widthMap[t] + this.CELL_PADDING;
                monsterCells.push(node);
              }
//                }
            }
          }
          freeSpace = freeSpace - addWidth;
        }
        if (freeSpace > 0) {
          if (t == headerRow.childNodes.length - 1) {
            addWidth += freeSpace;
            freeSpace = 0;
          }
        }
        if (addWidth > 0) {
          // add free space to header
          cell = headerRow.childNodes[t];
          node = cell.firstChild;
          var width = Math.min(widthMap[t], this.maxCellWidth) + addWidth;
          this.actualWidthMap[t] = width;
          node.style.width = (width + 2) + 'px';

          // handle large text
          var headerCells = node.getElementsByTagName("td");
          if (headerCells && headerCells.length > 2) {
            var headerText = headerCells[1].firstChild;
            if (headerText) {
              if (width < 32) {
                // avoid problems with negative sizes
                width = 32;
              }
              if (this._config.isFilterable) {
                // cell width - sort div - filter div
                headerText.style.width = (width - 32) + "px";
              }
              else {
                //  filter div is hidden
                headerText.style.width = (width - 16) + "px";
              }
            }
          }

          // add free space to data cell
          for (i = 0; i < rows.length; i++) {
//              if (!rows[i].isGroupHeader) {
            cell = rows[i].dataRow.childNodes[t];
            node = cell.firstChild;
            width = Math.min(widthMap[t], this.maxCellWidth) + addWidth;
            node.style.width = width + 'px';
          }
//            }
        }
      }

      // add ellipsis
      for ( var k = 0; k < monsterCells.length; k++) {
        cell = monsterCells[k];
        this._addEllipsis(cell);
      }

      // size headers
//        if (rows.length > 0 && this._config.activeGroupByProperty) {
//          var groupHeaders = this._groupHeaders;
//          var style = (dojo.isIE ? "block" : "table-row");
//          for (k = 0; k < groupHeaders.length; k++) {
//            groupHeaders[k].style.display = style;
//            var gWidth = Math.max(this._sizeCache.CenterWidth, this._sizeCache.ContentWidth);
//            this._config.groupFormatter.resizeWidth(groupHeaders[k].firstChild.firstChild, gWidth);
//          }
//        }
      if (initialFreeSpace > 0) {
        this.DataTable.style.width = this._sizeCache.CenterWidth + 'px';
        this.BodyCenter.style.width = this._sizeCache.CenterWidth + 'px';
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _addEllipsis: function _addEllipsis_Table(cell){
      var cellWidth = cell.offsetWidth;
      var parentTd = cell.parentNode;
      // Typically, ellipsis is done via css. Only older brwosers don't support this
      if (dojo.isFF && (dojo.isFF < 7) && dojo.hasClass(parentTd, "tableData")) {
        // don't do it twice
        if (cell.tagName != "TABLE") {
          // Set display back to block, otherwise the width setting would not be used.
          var inlineTable = document.createElement("table");
          dijit.setWaiRole(inlineTable, "presentation"); //a11y
          inlineTable.style.tableLayout = "fixed";
          inlineTable.style.display = "block";
          inlineTable.style.width = cellWidth + "px";
          inlineTable.style.borderCollapse = "collapse";
          inlineTable.style.borderWidth = 0;
          parentTd.replaceChild(inlineTable, cell);
          var row = inlineTable.insertRow(0); //a11y: none
          var content = row.insertCell(0); //a11y: none
          content.vAlign = "top";
          content.style.padding = 0;
          cell.style.width = (cellWidth - 10) + "px";
          content.appendChild(cell);
          var ellipsis = row.insertCell(1);
          ellipsis.vAlign = "top";
          ellipsis.style.padding = 0;
          ellipsis.innerHTML = "...";
        }
        else {
          cell.style.width = cellWidth + "px";
          content = cell.firstChild.firstChild.firstChild.firstChild;
          content.style.width = (cellWidth - 10) + "px";
        }
      }
    },

    /**
     * destroy
     */
    destroy: function destroy_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      this._lastOver = -1;
      this._currentSelection = -1;

//        if (this._dndConnects) {
//          for ( var i = 0; i < this._dndConnects.length; i++) {
//            dojo.unsubscribe(this._dndConnects[i]);
//          }
//          delete this._dndConnects;
//          this._dndConnects = null;
//        }

      if (this.vScrollEvent) {
        dojo.disconnect(this.vScrollEvent);
        this.vScrollEvent = null;
      }
      if (this.hScrollEvent) {
        dojo.disconnect(this.hScrollEvent);
        this.hScrollEvent = null;
      }
      if (this._tableConnects !== null) {
        for ( var k = 0; k < this._tableConnects.length; k++) {
          dojo.disconnect(this._tableConnects[k]);
        }
        delete this._tableConnects;
        this._tableConnects = null;
      }

      if (this._tableContentConnects !== null) {
        for ( var k = 0; k < this._tableContentConnects.length; k++) {
          dojo.disconnect(this._tableContentConnects[k]);
        }
        delete this._tableContentConnects;
        this._tableContentConnects = null;
      }

      // TODO : AB make sure all widgets are deleted. The grouping elements include dojo widgets.!!!!
      // remove the representation layer
      if (this._modelRepresentation) {
        delete this._modelRepresentation;
        // AB: do not set it to null, because maybe there are still events accessing the array
        this._modelRepresentation = [];
      }
      if (this._columns) {
        delete this._columns;
        this._columns = null;
      }
//        if (this._selectAllCheckbox) {
//          com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._selectAllCheckbox);
//          delete this._selectAllCheckbox;
//          this._selectAllCheckbox = null;
//        }
      if (this._model) {
        delete this._model;
        this._model = null;
      }

//        // Remove dnd objects.
//        if (this.dndSource) {
//          this.dndSource.destroy();
//          this.dndSource = null;
//        }
//        if (this.dndTarget) {
//          this.dndTarget.destroy();
//          this.dndTarget = null;
//        }

      this.inherited('destroy', arguments);
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");

    },

    //////////////////////////////////////////
    // public interface
    //////////////////////////////////////////


    /**
     * return array of data items of selected rows
     */
//      getCheckedData: function getCheckedData_Table(){
//        var checked = [];
//        var rows = this._modelRepresentation;
//        for ( var i = 0; i < rows.length; i++) {
//          var row = rows[i];
////          if (!row.isGroupHeader) {
//          if (row.checkbox && row.checkbox.checked) {
//            checked.push(this._model[i]);
//          }
////          }
//        }
//        return checked;
//      },
//      /**
//       * enable / disable select all checkbox
//       */
//      setShowSelectAllCheckbox: function setShowSelectAllCheckbox_Table(bool){
//        this._showSelectAllCheckbox = bool;
//        if (!this._showSelectAllCheckbox) {
//          dojo.addClass(this.Table, 'bSpaceTableNoCheckBox');
//        }
//        else {
//          dojo.removeClass(this.Table, 'bSpaceTableNoCheckBox');
//        }
//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          // rerender
//          this._configurationApplied = false;
//          this.setModel(this._model);
//        }
//      },
    // row specifics
    /**
     * return the model/data/rows the table displays
     */
    getModel: function getModel_Table(){
      return this._model;
    },

    // some functions to move rows up/down

    rowMoveUp: function rowMoveUp_Table(rowIndex, animate){
      if (rowIndex > 0) {
        return this.swapRows(rowIndex - 1, rowIndex, animate);
      }
      else {
        return false;
      }
    },

    rowMoveDown: function rowMoveUp_Table(rowIndex, animate){
      if (rowIndex < this._model.length - 1) {
        return this.swapRows(rowIndex, rowIndex + 1, animate);
      }
      else {
        return false;
      }
    },

    /**
     * Connect to this method for notification if the row swap animation is finished. {void} nothing
     */
    onSwapFinished: function(){

    },

    swapRows: function swapRows_Table(rowIndex1, rowIndex2, animate){
//        if (rowIndex1 < 0 || rowIndex2 < 0 || rowIndex1 == rowIndex2 || rowIndex1 > this._model.length - 1 ||
//          rowIndex2 > this._model.length - 1 || this.animationInAction ||
//          this._modelRepresentation[rowIndex1].isGroupHeader || this._modelRepresentation[rowIndex2].isGroupHeader) {
      if (rowIndex1 < 0 || rowIndex2 < 0 || rowIndex1 == rowIndex2 || rowIndex1 > this._model.length - 1 ||
        rowIndex2 > this._model.length - 1 || this.animationInAction) {
        return false;
      }

      this.animationInAction = true;

      if (rowIndex1 > rowIndex2) {
        var temp = rowIndex2;
        rowIndex2 = rowIndex1;
        rowIndex1 = temp;
      }

      // unhover rows if necessary
      this._unsetHoverForRow(this._modelRepresentation[rowIndex1], rowIndex1);
      this._unsetHoverForRow(this._modelRepresentation[rowIndex2], rowIndex2);

      var actionNodes = this._modelRepresentation[rowIndex1].actions;
      var actions = this._model[rowIndex1].inlineActions;
      if (actionNodes) {
        for ( var k = 0; k < actionNodes.length; k++) {
          dojo.removeClass(actionNodes[k], actions[k].hoverClass);
        }
      }

      this._lastOver = null;

      // swap model
      var tempModelItem = this._model[rowIndex1];
      this._model[rowIndex1] = this._model[rowIndex2];
      this._model[rowIndex2] = tempModelItem;

      // swap representation
      var tempRepItem = this._modelRepresentation[rowIndex1];
      this._modelRepresentation[rowIndex1] = this._modelRepresentation[rowIndex2];
      this._modelRepresentation[rowIndex2] = tempRepItem;


      tempRepItem = this._modelRepresentation[rowIndex1];
      if (tempRepItem.actionRow) {
        this._setRowId(tempRepItem.actionRow, rowIndex1, tempRepItem.actionRow._parentId);
      }
      if (tempRepItem.dataRow) {
        this._setRowId(tempRepItem.dataRow, rowIndex1, tempRepItem.dataRow._parentId);
      }
//        if (tempRepItem.checkboxRow) {
//          this._setRowId(tempRepItem.checkboxRow, rowIndex1, tempRepItem.checkboxRow._parentId);
//        }

      tempRepItem = this._modelRepresentation[rowIndex2];
      if (tempRepItem.actionRow) {
        this._setRowId(tempRepItem.actionRow, rowIndex2, tempRepItem.actionRow._parentId);
      }
      if (tempRepItem.dataRow) {
        this._setRowId(tempRepItem.dataRow, rowIndex2, tempRepItem.dataRow._parentId);
      }
//        if (tempRepItem.checkboxRow) {
//          this._setRowId(tempRepItem.checkboxRow, rowIndex2, tempRepItem.checkboxRow._parentId);
//        }

      if (!animate) {
        this._swapRowsDOMQuick(rowIndex1, rowIndex2);
      }
      else {
        this._swapRowsDOMAnimated(rowIndex1, rowIndex2);
      }

      return true;
    },

    _swapRowsDOMQuick: function(rowIndex1, rowIndex2){
      var row1 = this._modelRepresentation[rowIndex1];
      var row2 = this._modelRepresentation[rowIndex2];

//        if (row2.checkboxRow) {
//          this.CheckboxTableBody.removeChild(row1.checkboxRow);
//          dojo.place(row1.checkboxRow, row2.checkboxRow, "replace");
//          dojo.place(row2.checkboxRow, this.CheckboxTableBody, rowIndex2);
//        }
      if (row2.dataRow) {
        this._dnTableBody.removeChild(row1.dataRow);
        dojo.place(row1.dataRow, row2.dataRow, "replace");
        dojo.place(row2.dataRow, this._dnTableBody, rowIndex2);
      }
      if (row2.actionRow) {
        this.ActionTableBody.removeChild(row1.actionRow);
        dojo.place(row1.actionRow, row2.actionRow, "replace");
        dojo.place(row2.actionRow, this.ActionTableBody, rowIndex2);
      }
      this.animationInAction = false;
      this.onSwapFinished();
    },

    _swapRowsDOMAnimated: function(rowIndex1, rowIndex2, endAt, current){
      var row1 = this._modelRepresentation[rowIndex1];
      var row2 = this._modelRepresentation[rowIndex2];
      if (!endAt) {
        endAt = row1.dataRow.offsetTop - row2.dataRow.offsetTop;
        current = 1;
      }

//        if (row1.checkboxRow) {
//          this._positionRowContent(row1.checkboxRow, -current);
//        }
      if (row1.dataRow) {
        this._positionRowContent(row1.dataRow, -current);
      }
      if (row1.actionRow) {
        this._positionRowContent(row1.actionRow, -current);
      }

      if (row2.checkboxRow) {
        this._positionRowContent(row2.checkboxRow, current);
      }
      if (row2.dataRow) {
        this._positionRowContent(row2.dataRow, current);
      }
      if (row2.actionRow) {
        this._positionRowContent(row2.actionRow, current);
      }

      if (current === 0) {
        this.animationTimer = null;
        this._swapRowsDOMQuick(rowIndex1, rowIndex2);
        return;
      }

      current = current + Math.ceil(endAt / 10);
      var self = this;
      this.animationTimer = window.setTimeout(function(){
        if (endAt - current < 0) {
          current = 0;
          // loop stops if current = 0
        }
        self._swapRowsDOMAnimated(rowIndex1, rowIndex2, endAt, current);
      }, 50);
    },

    _positionRowContent: function(row, pos){
      if (row) {
        var tds = row.childNodes;
        if (tds) {
          for ( var i = 0; i < tds.length; i++) {
            var node = tds[i].firstChild;
            if (node) {
              if (pos !== 0) {
                node.style.position = "relative";
                node.style.top = pos + "px";
              }
              else {
                node.style.position = "";
                node.style.top = "";

              }
            }
          }
        }
      }
    },

    ///////////////////////////////////////
    // internals
    ///////////////////////////////////////

    _setColumns: function _setColumns_Table(columns){
      this._columns = columns;
      while (this._dnTableHeader.firstChild) {
        this._dnTableHeader.removeChild(this._dnTableHeader.firstChild);
      }
//        while (this.HeaderLeft.firstChild) {
//          this.HeaderLeft.removeChild(this.HeaderLeft.firstChild);
//        }
      this._renderHeaderRow();
    },


    /**
     * display the sort indicator
     */
    _setSortIndicators: function _setSortIndicators_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2);

      // sort props are given in this._config.sortProperties
      // remove sort indicators from columns
      var cols = dojo.query("div", this._dnTableHeader);
      for ( var i = 0; i < cols.length; i++) {
        dojo.removeClass(cols[i], "sortAscPrimary");
        dojo.removeClass(cols[i], "sortAscSecondary");
        dojo.removeClass(cols[i], "sortDescSecondary");
        dojo.removeClass(cols[i], "sortDescPrimary");
      }

      var column = null;
      var sortProp, div;

      if (this._config.sortProperties && this._config.sortProperties.length > 0) {

        // primary sort
        sortProp = this._config.sortProperties[0];
        if (sortProp.property) {
          column = this._getColumnForProperty(sortProp.property);
          if (column) {
            // show sort icons
            div = dojo.query(".sortHidden", column)[0];
            if (sortProp.sortDir == 'ASC') {
              div.parentNode.title = this._nlsResources["KEY_SortDirectionDesc"];
              dojo.addClass(div, 'sortAscPrimary');
            }
            else {
              div.parentNode.title = this._nlsResources["KEY_SortDirectionAsc"];
              dojo.addClass(div, 'sortDescPrimary');
            }
          }
        }

        // secondary sort
        if (this._config.sortProperties.length > 1) {
          sortProp = this._config.sortProperties[1];
          if (sortProp.property) {
            column = this._getColumnForProperty(sortProp.property);
            if (column) {
              // show sort icons
              div = dojo.query(".sortHidden", column)[0];
              if (sortProp.sortDir == 'ASC') {
                div.parentNode.title = this._nlsResources["KEY_SortDirectionDesc"];
                dojo.addClass(div, 'sortAscSecondary');
              }
              else {
                div.parentNode.title = this._nlsResources["KEY_SortDirectionAsc"];
                dojo.addClass(div, 'sortDescSecondary');
              }
            }
          }
        }
      }
    },


    /**
     * Render all cells in the header row of the table, including the checkbox row (first row)
     */
    _renderHeaderRow: function _renderHeaderRow_Table(){

      if (this._config.isSelectable !== false) {
        // first row is selectAll checkbox
        // dijit.setWaiRole(this.HeaderLeft, "gridcell"); // a11y

//          this._selectAllCheckbox = document.createElement('input');
//          dijit.setWaiRole(this._selectAllCheckbox, "checkbox"); // a11y
//          this._selectAllCheckbox.type = 'checkbox';
//          this.HeaderLeft.appendChild(this._selectAllCheckbox);

//          com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this._selectAllCheckbox);
//          this.connect(this._selectAllCheckbox, "click", "_selectAllCheckboxClicked");

        // keyboard navigation is done on the bSpace Checkbox button, which is next sibling to the real xbox
//          if (this._selectAllCheckbox.nextSibling) {
//            var button = this._selectAllCheckbox.nextSibling;
//            button.title = this._nlsResources["KEY_selectAllItems"];
//            this.focusNode = button; // focus node of the whole table
//            this.connect(button, "onfocus", "_onFocus");
//            button.tabIndex = "0"; // enable tab stop. Note: This is the only tabstop .
//          }
      }

      for ( var i = 0; i < this._columns.length; i++) {
        this._renderHeaderForColumn(this._columns[i]);
        this._dnTableHeader.lastChild._focusColumnId = i;

      }
    },

    /**
     * Reactivate last focus or set initial focus when table gets focus
     */
    onFocus: function onFocus_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusNode !== null) {
        com.ibm.bpm.social.util.common.Log.log(3, "row " + this.focusedRow + "  column " + this.focusedColumn);
        // reactivate focus node
        this._setFocusToNode(this.focusNode);
        if (this.focusedRow != -1) {
          // highlight focused row
          var dataRow = this._modelRepresentation[this.focusedRow].dataRow;
          this._onTableMouseOver({
            target: dataRow
          });
        }
      }
      else {
        // set focus node
        if (this.focusedRow >= 0) {
          this._setFocusInRow();
        }
        else {
          this._setFocusInHeader();
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

//    /**
//     * highlite a cell when navigating with keyboard
//     * 
//     * @param {Object} node
//     */
//    _focusOnNode: function _focusOnNode_Table(node){
//      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//      if (node) {
//        // changing className in FF changes focus
//        var activeDoc = document.activeElement;
//        dojo.addClass(node, "bSpaceTableOver");
//        activeDoc.focus();
//      }
//      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//    },

    /**
     * Sets the focus to the node
     * 
     * @param node new focus node
     * @param delayed delayed execution of focus setting
     */
    _setFocusToNode: function _setFocusToNode_Table(node, delayed){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (node) {
        this._setFocusNode(node);
        if (delayed) {
          window.setTimeout(dojo.hitch(this, function(){
            this._setFocus(true);
          }, 100));
        }
        else {
          this._setFocus(true);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * The focus node is the only node within the table with a tabIndex of 0. The tabIndex of the last focus
     * node must be reset before setting focus to another node.
     */
    _setFocusNode: function _setFocusNode_Table(node){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "focusedRow " + this.focusedRow + "  focusedColumn " + this.focusedColumn);
      if (node) {
        if (this.focusNode) {
          this.focusNode.tabIndex = -1;
        }
        node.tabIndex = 0;
        this.focusNode = node;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _setFocus: function _setFocus_Table(scrollIntoView){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusNode) {
        if (scrollIntoView) {
          this._scrollIntoView();
        }
        this.focusNode.focus();
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * Scrolls the table cell of the focused node into the view
     */
    _scrollIntoView: function _scrollIntoView_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var focusedRow = null;
      if (this.focusedRow === -1) {
        focusedRow = this.HeaderTable.rows[0];
      }
      else {
        focusedRow = this._modelRepresentation[this.focusedRow].dataRow;
      }
      if (focusedRow) {
        var focusColumn = focusedRow.childNodes[this.focusedColumn];
        if (focusColumn) {
          dojo.window.scrollIntoView(focusColumn);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

//      /**
//       * select/deselect all row checkboxes (w/o) sending event
//       * 
//       * @param {bool} select select or unselect
//       */
//      _selectAllRowCheckboxes: function _selectAllRowCheckboxes_Table(select){
//
//        for ( var i = 0; i < this._modelRepresentation.length; i++) {
//          this._checkRow(i, select);
//        }
//        if (select === false && this._config.maintainSelection) {
//          // for maintainSelection mode, we must lowlight manually
//          this._lowLightAll();
//        }
//
//        this._currentSelection = -1;
//      },


//      /**
//       * handle the select all event
//       * 
//       * @param {Object} e not used
//       */
//      _selectAllCheckboxClicked: function _selectAllCheckboxClicked_Table(e){
//        this._selectAllRowCheckboxes(this._selectAllCheckbox.checked);
//        // fire onSelectionChanged event
//        this._onSelectionChanged(this.getCheckedData());
//      },


//      /**
//       * handle the select group event
//       * 
//       * @param {Object} index not used
//       */
//      _selectGroupCheckboxClicked: function _selectGroupCheckboxClicked_Table(index){
//        var rows = this._modelRepresentation;
//        var checkbox = rows[index].checkbox;
//        var select = checkbox.checked;
//
//        for ( var i = Number(index) + 1; i < rows.length; i++) {
//          if (rows[i].isGroupHeader) {
//            break;
//          }
//          this._checkRow(i, select);
//        }
//        this._onSelectionChanged(this.getCheckedData());
//        this._checkSelectAllStatus();
//      },

    /**
     * Set focus in table content if focus gets outside of table content, e.g. Scrollers
     */
    _onOuterTableFocus: function _onOuterTableFocus_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "activeElement.className " + document.activeElement.className);
      if (document.activeElement !== this.focusNode) {
        com.ibm.bpm.social.util.common.Log.log(3, "Setting focus to focusNode");
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Set focus node in header if header is clicked
     */
    _onHeaderClick: function _onHeaderClick(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      this.focusedRow = -1;
      var headerNode = this._getParentWithClass(event.target, 'bSpaceTableHeader');
      if (!headerNode) {
        this.focusedColumn = this._getChildElementCount(this.HeaderTable.rows[0]);
        this._setFocusToNode(event.target);
      }
      else if (headerNode._focusColumnId !== undefined) {
        this.focusedColumn = headerNode._focusColumnId;
        this._setFocusToNode(headerNode.focusNode, true);
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /*
     * For iPad we combine mouseOver and onClick. We have to start a new thread to get the
     * mouseover effect rendered. Otherwise the inline action would highlight after it is processed
     * (which can be quite a while on the iPad).
     */
    _onTableClickDelayed: function _onTableClickDelayed_Table(event){
      this._onTableMouseOver(event);
      var self = this;
      setTimeout(function(){
        self._onTableClick(event);
        setTimeout(function(){
          self._onTableMouseOut(event);
        }, 1000);
      }, 100);
    },

    /**
     * Handle click event in table area. Select a row or send an action request. Sets focus to clicked cell.
     */
    _onTableClick: function _onTableClick_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no active table data");
        return;
      }

      // check for the necessary information of the event to process any further
      var node = event.target;
      if (!node) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no event target or no rowId");
        return;
      }
      if (node._focusColumnId !== undefined) {
        this.focusedColumn = node._focusColumnId;
      }
      var rowId = node["_rowId"];
      if (rowId !== undefined && rowId >= 0) {
        var row = this._modelRepresentation[rowId];

//          if (com.ibm.bpm.social.util.common.Util.isIPad) {
//            // necessary for highlighting since we disabled the over/out stuff for iPad
//            if (this._config.isSelectable !== false && dojo.hasClass(event.target, "bSpaceTableCheckBox")) {
//              // click into the checkbox cell, but checkbox is still display=none
//              node = row.checkbox;
//              com.ibm.bspace.common.util.ui.CheckBox.setChecked(row.checkbox, !row.checkbox.checked);
//            }
//          }


        // if the click was on an action, then return afterwards. There is no more to do
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          var actions = this._model[rowId].inlineActions;
          var action = actions[actionId];

          // fire only if enabled
          if (action.enabled) {
            this.onActionRequested(action.id, this._model[rowId]);
            return;
          }
        }
        // if there is no check box, then set only the focus to the row
        if (this._config.isSelectable === false) {
          this._setFocusedRow(this.focusedRow, rowId);
          return;
        }
//          // group header check box was clicked, thus mark the group members
//          if (row.isGroupHeader === true) {
//            if (row.checkbox === node) {
//              this._selectGroupCheckboxClicked(rowId);
//            }
//          }
//          else {
        if ((event.ctrlKey) && (this._selectionPending != rowId)) {
//          if ((event.ctrlKey || (node.type && node.type == "checkbox")) && (this._selectionPending != rowId)) {

//            // for the click event on the checkbox the current value only has to be
//            // set to make sure the selection is set correct. The checkbox status is
//            // set by the widget itself. For the ctrl key the status has to toggle.
//            // toggle the check box when ctrl key is pressed
//            var value = node.type == "checkbox" ? row.checkbox.checked : !row.checkbox.checked;
//            this._checkRow(rowId, value);
//            // remember last selected element for shift selection
//            if (row.checkbox.checked) {
//              // set the selection for the checkbox here, as it is not handled in mouse down.
//              // But that is ok as by clicking the checkbox the DnD is also not possible
//              // and therefore setting the selection and sending on focus changed can 
//              // be done here.
          this._currentSelection = Number(rowId);
          this._setFocusedRow(this.focusedRow, rowId);
//            }
//            else {
//              this._currentSelection = -1;
//            }
        }
//          else if (!this._config.maintainSelection && this._deselectionPending >= 0) {
//            // deselect all otheres when the deselectionPending is set
//            for ( var i = 0; i < this._modelRepresentation.length; i++) {
//              if (i != this._deselectionPending) {
//                this._checkRow(i, false);
//              }
//            }
//          }
//          }
        this._selectionPending = -1;
        this._deselectionPending = -1;
//          // check, if the select all checkbox needs to change
//  //        this._checkSelectAllStatus();
//          // send the new selection state
//          //defect 704588, focus changed at mouse down not click
        this.onFocusChanged(this._model[rowId]);
////          this._onSelectionChanged(this.getCheckedData());
//
//          // if we are in selection maintaining mode, where a checkbox may be 
//          // selected from model, we must highlight the one the user clicked on and 
//          // lolight the previous
        if (this._config.maintainSelection) {
          this._lowLightAll(rowId);
          this._setRowSelected(rowId);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Helper to set a row to selected
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _setRowSelected: function _setRowSelected_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var row = this._modelRepresentation[rowId];
      row.checked = true;
      dojo.removeClass(row.dataRow, 'bSpaceTableOver');
      dojo.removeClass(row.actionRow, 'bSpaceTableOver');
      dojo.addClass(row.dataRow, 'bSpaceTableSelected');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.addClass(nodes[i], 'bSpaceTableSelected');
            dojo.removeClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      
      dojo.addClass(row.actionRow, 'bSpaceTableSelected');
      this._lastChecked = rowId;

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },
    /**
     * Helper to set a row to unselected
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _setRowUnselected: function _setRowUnselected_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var row = this._modelRepresentation[rowId];
      row.checked = false;
      dojo.addClass(row.dataRow, 'bSpaceTableOver');
      dojo.addClass(row.actionRow, 'bSpaceTableOver');

      dojo.removeClass(row.dataRow, 'bSpaceTableSelected');
      dojo.removeClass(row.actionRow, 'bSpaceTableSelected');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.removeClass(nodes[i], 'bSpaceTableSelected');
            dojo.addClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      this._lastChecked = null;

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },

    /*
      * Handle all mouse over event for the table area.
      */
    _onTableMouseOver: function _onTableMouseOver_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(4);
      var node = event.target;
      if (!node || this.animationInAction) {
        return;
      }
      var rowId = node["_rowId"];
      // first check if the node has a row id
      if (rowId !== undefined && rowId >= 0) {
        //if it is a real event
        if (event.screenX) {
          dojo.stopEvent(event);
        }
        // the row id points to an entry in modelRepresentation array, which defines a row in the table
        var row = this._modelRepresentation[rowId];
        // if the current node is an action node, then the hover class for the action needs to be set.
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          var actionNodes = row.actions;
          var actions = this._model[rowId].inlineActions;
          if (actions[actionId].enabled) {
            dojo.addClass(actionNodes[actionId], actions[actionId].hoverClass);
          }
        }
        // the row is not already marked with hover, then set the necessary classes
        if (this._lastOver !== rowId) {
          // if there was another row marked before, then the old one has to be reset
          this._setHoverForRow(row, rowId);
          if (this._lastOver !== null) {
            var oldRow = this._modelRepresentation[this._lastOver];
            this._unsetHoverForRow(oldRow, this._lastOver);
          }
          this._lastOver = rowId;
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(4);
    },

    _onTableMouseOut: function _onTableMouseOut_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(4);
      var node = event.target;
      if (!node || this.animationInAction) {
        return;
      }
      // when the mouse is outside the table the mouse over marking has to be removed
      var outside = !this._insideTable(event);
      if (this._lastChecked && outside) {
        this._setRowUnselected(this._lastChecked);
      }
      var rowId = node["_rowId"];
      if (rowId !== undefined && rowId >= 0) {
        dojo.stopEvent(event);
        var row = this._modelRepresentation[rowId];
        // if the target was an action node, then disable the action again
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          // remove the hover marking from the current action
          var actions = this._model[rowId].inlineActions;
          var actionNodes = row.actions;
          dojo.removeClass(actionNodes[actionId], actions[actionId].hoverClass);
        }
        if (this._lastOver !== null && outside) {
          var oldRow = this._modelRepresentation[this._lastOver];
          this._unsetHoverForRow(oldRow, this._lastOver);
          this._lastOver = null;
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(4);
    },


//      /**
//       * Handle the shift key pressed and mouse click
//       * 
//       * @param {Number} rowId
//       * @type {void}
//       */
//      _handleShiftKeyList: function(rowId){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        var allItems = this._modelRepresentation;
//        // only if we have an selection that indicates where to start
//        // marking
//        if (this._currentSelection >= 0) {
//          var startIndex = this._currentSelection;
//          var stopIndex = rowId;
//          if (startIndex > stopIndex) {
//            startIndex = rowId;
//            stopIndex = this._currentSelection;
//          }
//          for ( var i = 0; i < allItems.length; i++) {
//            var aItem = allItems[i];
//            if (i < startIndex || i > stopIndex) {
//              this._checkRow(i, false);
//            }
//            else if (!aItem.checkbox.checked) {
//              this._checkRow(i, true);
//            }
//          }
//        }
//        else {
//          this._checkRow(rowId, true);
//          this._currentSelection = Number(rowId);
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },


    _setHoverForRow: function(row, rowId){
      // when it is a group header, only the checkbox gets displayed
//        if (this._config.isSelectable !== false) {
//          dojo.addClass(row.checkboxRow, 'bSpaceTableOver');
//        }
//        if (!row.isGroupHeader) {
      dojo.addClass(row.dataRow, 'bSpaceTableOver');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.addClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      dojo.addClass(row.actionRow, 'bSpaceTableOver');
//        }
    },

    _unsetHoverForRow: function(row, rowId){
//        if (this._config.isSelectable !== false) {
//          dojo.removeClass(row.checkboxRow, 'bSpaceTableOver');
//        }
      // for group headers only the checkbox classes have to be set
//        if (!row.isGroupHeader) {
      dojo.removeClass(row.actionRow, 'bSpaceTableOver');
      dojo.removeClass(row.dataRow, 'bSpaceTableOver');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.removeClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
//        if (this._config.isSelectable !== false) {
//          dojo.removeClass(row.checkboxRow, 'bSpaceTableOver');
//          // Only if not highlighted, otherwise, it won't be display=none
//          if (!row.checkbox.checked) {
//            com.ibm.bspace.common.util.ui.CheckBox.setChecked(row.checkbox, false);
//          }
//        }
//        }
    },

    /*
     * Check for an event if it happened inside the table content or outside.
     */
    _insideTable: function(event){
      // for IE it is toTarget, for FF relatedTarget
      var related = event.relatedTarget ? event.relatedTarget : event.toTarget;
      var node = event.target;
      if (related) {
        try {
          if (related["_parentId"] !== undefined && related["_rowId"] !== undefined &&
            related["_parentId"] === node["_parentId"]) {
            return true;
          }
          else {
            return false;
          }
        }
        catch (err) {
          // FF throws an exception if the event is outside the browser.
          // Permission denied to access property 'tagName' from a non-chrome context
          return false;
        }
      }
      return false;
    },

    /*
     * If onmousedown occurs, dnd will have higher priority than the selection handling.
     * dnd needs the item to be checked, because we want the behaviour that all checked items
     * + the current item will be dragged. After onmousedown happened, the selection handling happens
     * in the _onRowClick method. For ctrlKey, we remember the previous checked state.
     */
    _onTableMouseDown: function _onTableMouseDown_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no active table data");
        return;
      }

      var node = event.target;
      if (!node) {
        return;
      }
      var rowId = node["_rowId"];

//        if (rowId !== undefined && rowId >= 0 && this._config.isSelectable !== false) {
//          var row = this._modelRepresentation[rowId];
//          // shiftKey implementation: select all items in between first selected item
//          // (item with oldest timestamp) and current selected item.
//          // not supported in maintainSelection mode
//          if (event.shiftKey && !this._config.maintainSelection) {
//            // Find most recent checked item
//            this._handleShiftKeyList(rowId);
//
//            // stopping the event to prevent the user selecting text in the table when doing multiselection
//            dojo.stopEvent(event);
//          }
//          else {
      // no check box and not ctrl
      if (!(event.ctrlKey) && this._config.isSelectable !== false) {
//        if (!(event.ctrlKey || (node.type && node.type == "checkbox"))) {

        // Select this item if not already checked
        // if already checked, the selection does not change
        // and the deselection is done in the onclick event
//              if (!row.checkbox.checked) {
//                if (!this._config.maintainSelection) {
//                  this._checkRow(rowId, true);
//                  // deselect all otheres when row clicked
//                  for ( var i = 0; i < this._modelRepresentation.length; i++) {
//                    if (i != rowId) {
//                      this._checkRow(i, false);
//                    }
//                  }
//                  this._onSelectionChanged(this.getCheckedData());
//                }
//                else {
//                  // deselect all otheres
        this._lowLightAll(rowId);
//                }
//                // set timestamp of selection for shift key handling
//                this._currentSelection = Number(rowId);
//              }
//              else {
//                // deselect all besides this item in the onclick event.
//                // The deselection can only be done here in case
//                // an unselected item was selected.
//                this._deselectionPending = rowId;
//              }

        // Fire onFocusChanged for teamlist integration.
        this._setFocusedRow(this.focusedRow, rowId);
      }
      // if we are in selection maintaining mode, where a checkbox may be 
      // selected from model, we must highlight the one the user clicked on and 
      // lolight the previous
      if (this._config.maintainSelection) {
        this._lowLightAll(rowId);
        this._setRowSelected(rowId);
      }
//          }
//        }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * Helper function to remove the orange highlight from all rows, except the one given in rowId
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _lowLightAll: function _lowLightAll_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      for ( var i = 0; i < this._modelRepresentation.length; i++) {
        //if (i != rowId) {
        dojo.removeClass(this._modelRepresentation[i].dataRow, 'bSpaceTableSelected');
//          dojo.removeClass(this._modelRepresentation[i].checkboxRow, 'bSpaceTableSelected');
        dojo.removeClass(this._modelRepresentation[i].actionRow, 'bSpaceTableSelected');
        //  }
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },


    /**
     * create a sort indicator td
     * 
     * @type {TD} a td containing the soirt indicator
     */
    _createSortIndicator: function _createSortIndicator_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(3);

      var td_sort = document.createElement('td'); // a11y: no role
      td_sort.style.cursor = "pointer";
      td_sort.title = this._nlsResources["KEY_SortDirectionAsc"]; // default is ASC
      dijit.setWaiRole(td_sort, "columnheader"); // a11y

      // TODO : a11y
      var sortDiv = document.createElement('div');
      sortDiv.className = "sortHidden";
      td_sort.appendChild(sortDiv);

      // a11y stuff (high contrast) for sortDesc
      var sortAscText = document.createElement("span");
      sortAscText.innerHTML = "&#9650;";
      dojo.addClass(sortAscText, "a11yButtonText");
      sortDiv.appendChild(sortAscText);

      com.ibm.bpm.social.util.common.Log.logExit(3);
      return td_sort;
    },

    /**
     * Render the top cell of a specific column
     * 
     * @param {Object} column object describing the column
     */
    _renderHeaderForColumn: function _renderHeaderForColumn_Table(column){

      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

//        // no row for activeGroupByProperty
//        if (this._config.activeGroupByProperty == column.id) {
//          com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//          return;
//        }

      var mainHeaderCell = document.createElement('td');
      // a11y
      // TODO: shouldn't this be on the sub-table in the label TD?
      dijit.setWaiRole(mainHeaderCell, "presentation");

      mainHeaderCell.displayedProperty = column;
      mainHeaderCell.className = 'bSpaceTableHeader tableCell';
      mainHeaderCell.id = this.id + "_" + column.id;

      if (column.width) {
        mainHeaderCell.style.width = column.width;
      }

      this._dnTableHeader.appendChild(mainHeaderCell);

      // add a column to the table header row (with border) and trailing separator
      // this._appendVerticalSeparator(this._dnTableHeader);

      // to conform to the UI guidelines/style proposals, render table in the table
      // to get a frame around the cell ?
      var td_table = document.createElement('table');
      dijit.setWaiRole(td_table, "presentation"); // a11y
      td_table.cellSpacing = 0;
      td_table.cellPadding = 0;
      mainHeaderCell.appendChild(td_table);

      var td_tbody = document.createElement('tbody'); // a11y: no role
      td_table.appendChild(td_tbody);
      var td_tr = document.createElement('tr'); // a11y: no role
      td_tbody.appendChild(td_tr);

//        //
//        // append image for the search indicator
//        //
//        if (this._config.isFilterable !== false && column.isFilterable !== false) {
//          var td_search = document.createElement('td'); // a11y: no role
//          td_search.title = this._nlsResources["KEY_ColumnFilter"];
//          td_search.style.cursor = "pointer";
//          td_tr.appendChild(td_search);
//          var div = document.createElement('div');
//          dijit.setWaiRole(div, "button"); // a11y
//
//          // Create title for the search button.
//          if (dojo.isIE) {
//            // a11y for IE only
//            div.title = this._nlsResources["KEY_ColumnFilter"];
//          }
//          else {
//            // a11y for FF only
//            var genId = "id_" + new Date().getTime();
//            div.setAttribute("aria-labelledby", genId);
//            var hiddenTitle = document.createElement("span");
//            hiddenTitle.innerHTML = this._nlsResources["KEY_ColumnFilter"];
//            hiddenTitle.id = genId;
//            hiddenTitle.style.display = "none";
//            td_search.appendChild(hiddenTitle);
//          }
//
//          // For keyboard handling: search button is accessible via tab.
//          div.tabIndex = 0;
//
//          // Make it visible, if it is in focus.
//          this.connect(div, "onfocus", function(){
//            dojo.addClass(div, 'magnifier_inactive');
//          });
//          this.connect(div, "onblur", function(){
//            dojo.removeClass(div, 'magnifier_inactive');
//          });
//
//          div.className = 'magnifier_hidden';
//          td_search.appendChild(div);
//
//          // a11y stuff (high contrast) for sortDesc
//          var serachText = document.createElement("span");
//          serachText.innerHTML = "&#9660;";
//          dojo.addClass(serachText, "a11yButtonText");
//          div.appendChild(serachText);
//
//          this.connect(td_search, "onclick", '_doSearch');
//          this.connect(td_search, "onkeypress", function(e){
//            if (e.keyCode == dojo.keys.ENTER) {
//              this._doSearch(e);
//            }
//          });
//
//          // highlite indicators on mouse over
//          this.connect(td_search, "mouseover", function(){
//            if (this._modelRepresentation && this._modelRepresentation.length > 0) {
//              dojo.addClass(td_search, 'bSpaceTableSelected');
//            }
//          });
//          this.connect(td_search, "mouseout", function(){
//            dojo.removeClass(td_search, 'bSpaceTableSelected');
//          });
//        }
//        // if filtering is not enabled for this property
//        else {
      var td_search = document.createElement('td'); // a11y: no role          
      td_tr.appendChild(td_search);
      var div = document.createElement('div');
      div.className = 'filterhidden';
      td_search.appendChild(div);
//        }

      //
      // append text cell / header title
      //
      var td_text = document.createElement('td'); // a11y: no role
      dijit.setWaiRole(td_text, "presentation");
      td_tr.appendChild(td_text);
      // td_text.noWrap = true;
      td_text.className = 'bspace-style bspace-heading5 headerCell';

      var innerDiv = document.createElement('div');
      innerDiv.className = "headerInnerText";
      td_text.appendChild(innerDiv);
      innerDiv.innerHTML = column.label;
      dijit.setWaiRole(innerDiv, "presentation");
      innerDiv.id = this.id + "_label_" + column.id;


//        var th = document.createElement('th');
//        th.innerHTML = column.label;
//        this._dataTableHeaderRow.appendChild(th);

      //
      // containing div for the sort indicators
      //
      if (this._config.isSortable !== false && column.isSortable !== false) {
        td_text.tabIndex = -1; // enable focus setting by code
        mainHeaderCell.focusNode = td_text;
        var td_sort = this._createSortIndicator();
        td_tr.appendChild(td_sort);
        dijit.setWaiRole(innerDiv, "presentation");
        dijit.setWaiState(td_sort, 'labelledby', innerDiv.id);
        // innerDiv.title = td_sort.title;
        innerDiv.style.cursor = "pointer";
      }
      else {
        // if sortin is not enabled for this property
        td_sort = document.createElement('td'); // a11y: no role          
        td_tr.appendChild(td_sort);
        div = document.createElement('div');
        div.className = 'sortHidden';
        td_sort.appendChild(div);
        dijit.setWaiState(td_sort, 'hidden', 'true');
        dijit.setWaiRole(innerDiv, "columnheader");
      }

      // cell callback for mouse events
      // KJ: There is no need for a hover or click feedback, if there are no actions placed
      // in the header.
      if ((this._config.isSortable !== false && column.isSortable !== false) ||
        (this._config.isFilterable !== false && column.isFilterable !== false)) {
        this.connect(mainHeaderCell, "onmouseover", '_onHeaderCellMouseOver');
        this.connect(mainHeaderCell, "onmouseout", '_onHeaderCellMouseOut');
        this.connect(mainHeaderCell, "onclick", '_doSort');
        this.connect(mainHeaderCell, "onkeypress", function(e){
          if (e.keyCode == dojo.keys.ENTER) {
            this._doSort(e);
          }
        });
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },


//    /**
//     * handle a click on the "magnifier" icon
//     * 
//     * @param {Object} e onClick event object
//     */
//    _doSearch: function _doSearch_Table(e){
//      var column = this._getParentWithClass(e.target, 'bSpaceTableHeader');
//      var property = column.displayedProperty.id;
//
//      // fire the event
//      this.onFilter(property, column);
//
//    },


    /**
     * handle a click on a column header = do sort
     * 
     * @param {Object} e onClick event object
     */
    _doSort: function _doSort_Table(e){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var column = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var property = column.displayedProperty;
      var isSort;
      if (property.isSort === true) {
        // switch direction
        property.sortDir = (property.sortDir == "ASC" ? "DESC" : "ASC");
      }

      // remember focused header
      var headerNode = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      this.focusedColumn = headerNode._focusColumnId;
      this._refreshedBySort = true;

      this.onSort(property.id, property.sortDir);

      com.ibm.bpm.social.util.common.Log.logExit(2);
    },


    _createActionTd: function _createActionTd_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

      var td = null;
      td = document.createElement('td'); // a11y : no role
      td.className = 'tableData actions';

      // ensure image is added left
      td.align = 'right';
      td.noWrap = true;
      if (document.body.dir == 'rtl') {
        td.align = 'left';
      }

      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
      return td;
    },

    /**
     * After the available actions call, we get notified here to update the state of all inline actions
     * 
     * @type void
     */
    updateActions: function updateActions_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(1);

      for ( var j = 0; j < this._model.length; j++) {
        var item = this._model[j];

        if (item.inlineActions && item.inlineActions.length > 0) {
          var actionRepresentation = this._modelRepresentation[j].actions;

          for ( var i = 0; i < actionRepresentation.length; i++) {
            var actionDiv = actionRepresentation[i];

            var action = item.inlineActions[actionDiv._actionId]; //  actionDiv._actionId == index of the action

            // we can't ensure that each action has only one link, User might have it more times in his saved search
            var linkedCells = dojo.filter(this._modelRepresentation[j].cellsWithLinks, function(cell){
              return cell._actionName == action.id;
            });

            if (action.enabled === false) {
              dojo.addClass(actionDiv, action.disabledClass);
              for ( var c = 0; c < linkedCells.length; c++) {
                dojo.removeClass(linkedCells[c], "bpm-task-clickable-link");
                dijit.removeWaiRole(linkedCells[c], "link");
              }
            }
            else {
              dojo.removeClass(actionDiv, action.disabledClass);
              for (c = 0; c < linkedCells.length; c++) {
                dojo.addClass(linkedCells[c], "bpm-task-clickable-link");
                dijit.setWaiRole(linkedCells[c], "link");
              }
              actionDiv.tabIndex = -1; // enable focus setting by code
            }
          }

        }
      }


      com.ibm.bpm.social.util.common.Log.logExit(1);
    },

    /**
     * set the action icons (at the end of the row)
     * 
     * @param {Object} tr tr to append the actions
     * @param {Object} td td to append the actions
     * @param {Number} index index of the item in the model
     */
    _setActions: function _setActions_Table(tr, td, index){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      var actions = this._model[index].inlineActions;

      if (actions) {
        this._modelRepresentation[index].actions = [];
        var actionRepresentation = this._modelRepresentation[index].actions;
        for ( var i = 0; i < actions.length; i++) {
          var action = actions[i];
          var actionDiv = document.createElement("span");
          dojo.addClass(actionDiv, "itemAction");

          if (action.description) {
            actionDiv.title = action.description; // =tooltip;
          }
          else if (action.tooltip) {
            actionDiv.title = action.tooltip; // =tooltip;
          }

          dojo.addClass(actionDiv, action.iconClass);
          if (action.enabled === false) {
            dojo.addClass(actionDiv, action.disabledClass);
          }
          // AB: enable keyboard navigation that way. Only problem actions and 
          // data rows are scrolling differently.
          //actionDiv.tabIndex = 0;

          dojo.addClass(actionDiv, "a11yFilledBox");
          if (dojo.hasClass(dojo.body(), "dijit_a11y")) {
            actionDiv.innerHTML = "&#x25EF;";
          }

          td.appendChild(actionDiv);
          // store the actions id in the node and the node on the representation array
          actionRepresentation[i] = actionDiv;
          // index of the action in the specific row
          actionDiv._actionId = i;
          actionDiv._focusColumnId = this._maxFocusColumns;
          this._maxFocusColumns++;
        }
        // We need the reference to the actions for the key handling.
        tr.actions = actions;
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Render a specific row identified by <index>
     * 
     * @param {Object} index
     */
    _renderRow: function _renderRow_Table(index){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      var tr = this._createRow(index);
      this._dnTableBody.appendChild(tr);
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },


    /**
     * Refresh a row
     * 
     * @param {Number} rowId
     * 
     * @type {void}
     */
    _refreshRow: function _refreshRow_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var trData = this._getParentWithType(this._modelRepresentation[rowId].dataRow, "tr");
//        var trCb = this._getParentWithType(this._modelRepresentation[rowId].checkboxRow, "tr");
      var trIA = this._getParentWithType(this._modelRepresentation[rowId].actionRow, "tr");
      var trNew = this._createRow(rowId);
      trNew.className = trData.className;
      this._dnTableBody.replaceChild(trNew, trData);
//        this._modelRepresentation[rowId].checkboxRow.className = trCb.className;
      this._modelRepresentation[rowId].actionRow.className = trIA.className;
//        this.CheckboxTableBody.replaceChild(this._modelRepresentation[rowId].checkboxRow, trCb);
      this.ActionTableBody.replaceChild(this._modelRepresentation[rowId].actionRow, trIA);

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },

    /**
     * Set the row id for all elements under the current node.
     * 
     * @param {DOMElement} node
     * @param {Number} rowId
     * @param {DOMElement} parentId
     * @type {void}
     */
    _setRowId: function(node, rowId, parentId){
      if (!node) {
        return;
      }
      node._rowId = rowId;
      node._parentId = parentId;
      var children = node.getElementsByTagName("*");
      if (children) {
        for ( var k = 0; k < children.length; k++) {
          children[k]._rowId = rowId;
          children[k]._parentId = parentId;
        }
      }
    },


    /**
     * Row creator for dojo DND
     * 
     * @param {Number} index
     * @param {String} hint If the method is used for avatar creation, hint is "avatar", otherwise it is null.
     */
    _createRow: function _createRow_Table(index, hint){
      var tr;
      var data = this._model[index];

//        if (this._groupHeaderOffset < 0) {
//          // we need to now the offset per model entry for group headers
//          // they are inserted in the modelRepresentation to calculate
//          // the position correct for calls from outside the table with indices
//          this._groupHeaderOffset = 0;
//        }

      // insert the representation for this row. Add the specific DOM nodes later.
      if (hint !== "avatar") {
        this._modelRepresentation[index] = {
          dataRow: null,
          focusNode: null,
//            checkboxRow: null,
//            checkbox: null,
          actionRow: null,
          actions: null,
          cellsWithLinks: [],
//            isGroupHeader: false,
//            offset: this._groupHeaderOffset
          offset: 0
        };
      }
//        if (data instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) {
//          this._groupHeaderOffset++;
//          this._modelRepresentation[index].offset = this._groupHeaderOffset;
//          tr = this._createGroupBanner(index, data);
//        }
//        else {
      tr = this._createDataRow(index, data, hint);
//        }

//        // IE7 DnD source won't work with tr, will need a table instead.
//        if (hint == "avatar" && dojo.isIE < 8) {
//          var ta = document.createElement("table");
//          var tb = document.createElement("tbody");
//          ta.appendChild(tb);
//          tb.appendChild(tr);
//          return ta;
//        }
      return tr;
    },


    /**
     * Create a regular data row.
     * 
     * @param {Number} index
     * @param {Object} data a single row from the model
     * @param {String} hint If the method is used for avatar creation, hint is "avatar", otherwise it is null.
     *        If the creation is for the avatar, there is no need for the checkbox and the inline action.
     */
    _createDataRow: function _createDataRow_Table(index, data, hint){
      com.ibm.bpm.social.util.common.Log.logEntry(2);

      this._maxFocusColumns = 0;

      var td;
      var img;
      var tr = document.createElement('tr');
      dijit.setWaiRole(tr, "row"); // a11y
      if (hint !== "avatar") {
        this._modelRepresentation[index].dataRow = tr;
      }

      // The first data row is in the tab order. From there you have to navigate with arrow
      // down/up through the table.
      if (index === 0) {
//          if (index == 0 && this._config.isSelectable !== false) {
        // tr.tabIndex = 0;
//          // set an event handler for focus in complete table
//          this._tableContentConnects.push(dojo.connect(tr, "onfocus", this, "_onTableMouseOver"));
      }
      else {
//            // Need this, otherwise we cannot focus on row.
        tr.tabIndex = -1;
      }

      var tr_class = 'taskRow bSpaceTableRow';
      if (index === 0) {
        tr_class += ' firstRow';
      }
      if (this._isPrototypTable) {
        tr_class += ' tablePrototype';
      }
      tr.className = tr_class;
//        // create the checkbox
//        if (this._config.isSelectable !== false && hint !== "avatar") {
//          var cbtr = document.createElement('tr');
//          dijit.setWaiRole(cbtr, "row"); // a11y
//
//          // Need this, otherwise we cannot focus on row.
//          cbtr.tabIndex = -1;
//
//          if (this._config.maintainSelection) {
//            cbtr.className = "taskRow";
//          }
//          else {
//            cbtr.className = tr_class;
//          }
//
//          td = document.createElement('td');
//          dijit.setWaiRole(td, "gridcell"); // a11y
//          dojo.addClass(td, 'tableData bSpaceTableCheckBox');
//          var checkbox = document.createElement('input');
//          dijit.setWaiRole(checkbox, "checkbox"); // a11y
//          checkbox.type = 'checkbox';
//          // Checkbboxes in list must not be accessible via tab key, otherwise
//          // you would have to tab a lot to pass the list (in case that there
//          // are a lot of tasks).
//          checkbox.tabIndex = -1;
//          td.appendChild(checkbox);
//          // convert to common UI checkbox
//          com.ibm.bspace.common.util.ui.CheckBox.wrapElement(checkbox);
//          // disable checkbox, only enabled when mouse over or highlight
//          // UI Checkbox renders with a overlay button
//          var lineNo = Number(index) + 1;
//          checkbox.nextSibling.title = this._nlsResources["KEY_selectItemInRow"].replace("${1}", lineNo);
//
//          // no tabStop on line checkboxes
//          checkbox.tabIndex = -1;
//          cbtr.appendChild(td);
//
//          // if selected in model, also set the checkbox on
//          if (data.enabled) {
//            com.ibm.bspace.common.util.ui.CheckBox.setChecked(checkbox, data.enabled);
//          }
//          this.CheckboxTableBody.appendChild(cbtr);
//
//          // store the node in model representation and the index in the node
//          this._modelRepresentation[index].checkbox = checkbox;
//          this._modelRepresentation[index].checkboxRow = cbtr;
//          // set row id for all elements of the value
//          this._setRowId(cbtr, index, this.id);
      //}
      // iterate through the cells of the row
      var dnColumn = this._dnTableHeader.firstChild;

      while (dnColumn) {

        td = document.createElement('td');
        if (dojo.hasClass(dnColumn, 'bSpaceTableHeader')) { // check if the current column is a real col (not only separator)

          td.className = 'tableData';
          dijit.setWaiRole(td, "columnheader"); // a11y
          dijit.setWaiState(td, "describedby", dnColumn.id); // a11y

          var value = null;
          var formatter = null;
          var valueDiv = null;
          var linkToAction = null;

          // find corresponding column definition
          for ( var i = 0; i < this._columns.length; i++) {
            if (dnColumn.displayedProperty.id == this._columns[i].id) {
              formatter = this._columns[i].formatter;
              linkToAction = this._columns[i].linkToAction;
              break;
            }
          }

          value = data[dnColumn.displayedProperty.id];
          if (value != undefined) {
            if (dojo.isFunction(formatter)) {
              // the formatter returns either a completely formatted div or null
              valueDiv = formatter(value);
            }
            if (!valueDiv) {
              valueDiv = document.createElement('div'); //a11y: none
              valueDiv.innerHTML = value;
              valueDiv.title = value; // =tooltip;
              valueDiv.tabIndex = -1;
            }

            // some fields can be links which do the same as inline actions
            if (linkToAction) {
              dojo.addClass(valueDiv, "bpm-task-clickable-link");
              // store the id for later use
              for (i = 0; i < data.inlineActions.length; i++) {
                if (data.inlineActions[i].id == linkToAction) {
                  valueDiv._actionId = i;
                  valueDiv._actionName = linkToAction;
                  // AB: enable keyboard navigation that way. Only problem actions and 
                  // data rows are scrolling differently.
                  //valueDiv.tabIndex = 0;
                }
              }
              this._modelRepresentation[index].cellsWithLinks.push(valueDiv);
            }

            // do not allow to wrap lines
            dojo.addClass(valueDiv, "tableCell");
            // KJ: If display is block, the width of the div will adapt to the cell
            // width. But we need the real size to do the Firefox maximize handling in
            // _setMaxWidth().
            if (dojo.isFF || (dojo.isIE && dojo.isIE > 7)) {
              valueDiv.style.display = "inline";
            }
            td.appendChild(valueDiv);
            td.focusNode = valueDiv;
            valueDiv._focusColumnId = this._maxFocusColumns;
            this._maxFocusColumns++;
          }
        } // end of if (dojo.hasClass(dnColumn, 'bSpaceTableHeader'))

        // add empty gif if we have no values to display
        if (!td.firstChild) {
          dojo.addClass(td, 'bSpaceBlankImageHolder');
          img = document.createElement('img'); //a11y: none
          img.src = dojo.moduleUrl("com.ibm.bspace.common.util.ui", "img/blank.gif");
          img.alt = this._nlsResources["task_saved_searches_empty_cell"];
          td.appendChild(img);
          td.focusNode = img;
          img._focusColumnId = this._maxFocusColumns;
          this._maxFocusColumns++;
        }

        if (dnColumn.style.width) {
          td.style.width = dnColumn.style.width;
        }

        tr.appendChild(td);
        // set row id for all elements of row
        this._setRowId(tr, index, this.id);

        dnColumn = dnColumn.nextSibling;

      } // end of while

      if (hint !== "avatar") {
        // append buttons at the end
        var atr = document.createElement('tr');
        dijit.setWaiRole(atr, "presentation"); // a11y
        // Need this, otherwise we cannot focus on row.
        //AB: we can not tab throguth actions
        //atr.tabIndex = -1;

        this._modelRepresentation[index].actionRow = atr;
        // keep links to the other parts of the row
        atr.className = tr_class;

        var actionsTd = this._createActionTd();
        actionsTd._rowId = index;
        atr.appendChild(actionsTd);
        this._setActions(atr, actionsTd, index);
        this.ActionTableBody.appendChild(atr);
        // set row id for all elements of the row
        this._setRowId(atr, index, this.id);
      }

      com.ibm.bpm.social.util.common.Log.logExit(2);
      return tr;
    },


//      /**
//       * 
//       * Create group banner ( a row visually emphasized )
//       * 
//       * @param {Object} index
//       * @param {Object} data
//       */
//      _createGroupBanner: function _createGroupBanner_Table(index, data){
//        var tr = document.createElement('tr');
//
//        this._modelRepresentation[index].isGroupHeader = true;
//        var tr_class = 'groupBanner bSpaceTableCheckBox bSpaceTableRow';
//
//        // The first row is in the tab order. From there you have to navigate with arrow
//        // down/up through the table.
//        tr.tabIndex = -1;
//        if (index === 0) {
//          tr_class += ' firstRow';
//          if (this._config.isSelectable !== false) {
//            tr.tabIndex = 0;
//            // set an event handler for focus in complete table
//            this._tableContentConnects.push(dojo.connect(tr, "onfocus", this, "_onTableMouseOver"));
//          }
//        }
//        if (this._isPrototypTable) {
//          tr_class += ' tablePrototype';
//        }
//        tr.className = tr_class;
//
//        this._modelRepresentation[index].dataRow = tr;
//
//        // create checkbox
//        var cbtr = document.createElement('tr');
//        cbtr.className = tr_class;
//        dijit.setWaiRole(cbtr, "row"); // a11y
//        // Need this, otherwise we cannot focus on row.
//        cbtr.tabIndex = -1;
//
//        var td = document.createElement('td');
//        cbtr.appendChild(td);
//        this.CheckboxTableBody.appendChild(cbtr);
//        td.align = 'center';
//        dojo.addClass(td, 'bSpaceTableCheckBox');
//
//        var selectGroupCheckbox = document.createElement('input');
//        selectGroupCheckbox._rowId = index;
//        dijit.setWaiRole(selectGroupCheckbox, "checkbox"); // a11y
//        selectGroupCheckbox.type = 'checkbox';
//        selectGroupCheckbox.tabIndex = -1;
//        td.appendChild(selectGroupCheckbox);
//        this._modelRepresentation[index].checkbox = selectGroupCheckbox;
//        this._modelRepresentation[index].checkboxRow = cbtr;
//
//        com.ibm.bspace.common.util.ui.CheckBox.wrapElement(selectGroupCheckbox);
//        selectGroupCheckbox.nextSibling.title =
//          data.groupName ? dojo.string.substitute(this._nlsResources["KEY_selectGroupItems"], {
//            1: data.groupName
//          }) : this._nlsResources["KEY_selectGroupItemsUncategorized"];
//        // set row id for all elements of the value
//        this._setRowId(cbtr, index, this.id);
//
//        // action row
//        var atr = document.createElement('tr');
//        this._modelRepresentation[index].actionRow = atr;
//        dijit.setWaiRole(atr, "row"); // a11y
//
//        // Need this, otherwise we cannot focus on row.
//        atr.tabIndex = -1;
//
//        atr.className = tr_class;
//
//        var actionsTd = this._createActionTd();
//        atr.appendChild(actionsTd);
//        this.ActionTableBody.appendChild(atr);
//        // set row id for all elements of the row
//        this._setRowId(atr, index, this.id);
//
//
//        // data row
//        td = document.createElement('td');
//        td.style.padding = "0";
//        td.colSpan = this._dnTableHeader.cells.length;
//        td.appendChild(this._config.groupFormatter.renderGroup(data.groupName));
//        // set the index for all children, so that a mouse event works properly
//        this._setRowId(tr, index, this.id);
//
//        tr.appendChild(td);
//
//        this._groupHeaders.push(tr);
//
//        return tr;
//      },

    /**
     * handle mouse over row event
     * 
     * @param {Object} e
     */
    _onHeaderCellMouseOver: function _onHeaderCellMouseOver_Table(e){
      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: no action since message is shown");
        return;
      }

      var td = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var gif = null;
      if (td) {
        dojo.addClass(td, 'bSpaceTableOver');

        // show magnifier icon
        gif = dojo.query(".magnifier_hidden", td)[0];
        if (gif) {
          dojo.addClass(gif, 'magnifier_inactive');
        }

        // show sort icon if sortable and not already sorted
        if (td.displayedProperty.isSortable && !td.displayedProperty.isSort) {
          gif = dojo.query(".sortHidden", td)[0];
          if (gif) {
            dojo.addClass(gif, 'sortAscPrimary');
          }
        }
      }
    },

    _onHeaderCellMouseOut: function _onHeaderCellMouseOut_Table(e){


      var td = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var gif = null;
      if (td) {
        dojo.removeClass(td, 'bSpaceTableOver');

        gif = dojo.query(".magnifier_inactive", td)[0];
        // hide the magnifier icon
        if (gif) {
          dojo.removeClass(gif, 'magnifier_inactive');
        }

        // hide sort icons if sortable and not already sorted
        if (td.displayedProperty.isSortable && !td.displayedProperty.isSort) {
          gif = dojo.query(".sortHidden", td)[0];
          if (gif) {
            dojo.removeClass(gif, 'sortAscPrimary');
          }
        }
      }
    },


    /**
     * Check a checkbox of a row of or off
     * 
     * @param {Number} rowId index of the row to be checked on/off
     * @param {Boolean} check check the checkbox on / off
     * @type {void}
     */
    checkRow: function checkRow_Table(rowId, check){
      this._checkRow(rowId, check);
    },


    /**
     * Check the checkbox in the specified mode
     * 
     * @param {Number} rowId index of the row to be checked on/off
     * @param {Boolean} check check the checkbox on / off
     * @type {void}
     */
    _checkRow: function _checkRow_Table(rowId, check){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      // highlight row and remember selection
      if (check) {
        this._setRowSelected(rowId);
        this._enableNextAction(rowId, 0);
      }
      else {
        this._setRowUnselected(rowId);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    /**
     * create a simple table to do some tests
     */
    _prototypeTable: function _prototypeTable_Table(sampleModel){
      var lconfig = this._config; // save old config
      this._isPrototypTable = true;

      //   ToDo: if we have a config but no sample model, try to generate a model from config

      // if we also got a model, we can test under real circumstances
      if (this._config && sampleModel && sampleModel.length > 0) {

        // no grouping here
//          this._config.activeGroupByProperty = null;
        this.setModel([sampleModel[0]]);
//          this._config.activeGroupByProperty = lconfig.activeGroupByProperty;
        // ok, all done.
        this._isPrototypTable = false;
      }

      else {

        var simpleProperty = [{
          id: "LOADING",
          label: "",
          formatter: null,
          description: null,
          inline: null
        }];

        var simpleConfig = {
          actions: [],
          displayedProperties: simpleProperty
        };

        var simpleModel = [{
          LOADING: "Loading ..."
        }];

        this.setConfig(simpleConfig);
        this.setModel(simpleModel);

        // restore old config, due to lazy rendering table will remain until next setModel
        // setConfig respects _isPrototypTable, thus we must unset it first
        this._isPrototypTable = false;

        if (lconfig) {
          this.setConfig(lconfig);
        }
      }
    },


    /**
     * Step up the DOM tree from domNode unless we find a node with the named class
     * 
     * @param {Object} node
     * @param {Object} className
     */
    _getParentWithClass: function _getParentWithClass_Table(node, className){
      var newNode = node;
      while (newNode && !dojo.hasClass(newNode, className)) {
        newNode = newNode.parentNode;
      }
      return newNode;
    },

    /**
     * Step up the DOM tree from domNode unless we find a node with the named class
     * 
     * @param {Object} node
     * @param {Object} type
     */
    _getParentWithType: function _getParentWithType_Table(node, type){
      var newNode = node;
      while (newNode && newNode.nodeName != type.toUpperCase()) {
        newNode = newNode.parentNode;
      }
      return newNode;
    },

    /**
     * Find the header cell for the given property
     * 
     * @param {String} property
     */

    _getColumnForProperty: function _getColumnForProperty_Table(property){
      var td = null;
      var columns = dojo.query("td.bSpaceTableHeader", this._dnTableHeader);
      property = property.toUpperCase();
      for ( var i = 0; i < columns.length; i = i + 1) {
        if (columns[i].displayedProperty.id == property) {
          td = columns[i];
          break;
        }
      }
      return td;
    },

    /**
     * Keyboard navigation on table works as follows
     * <ul>
     * <li>arrow down: focus next visible row</li>
     * <li>arrow up: focus previous visible row or header</li>
     * <li>left arrow: focus previous column (in header or table row)</li>
     * <li>right arrow: focus next column (in header or table row)</li>
     * <li>enter: execute action if defined for current column</li>
     * <li>tab: unset hover before table is left</li>
     * </ul>
     */
    _onKeyEvent: function _onKeyEvent_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "  key " + event.keyCode + "  row " + this.focusedRow +
        "  focusedColumn " + this.focusedColumn, "Table");
      // var tr = event.currentTarget;
      var node = event.target;

      var rowId = this.focusedRow;

      var k = dojo.keys;
      switch (event.keyCode) {
        case k.DOWN_ARROW :
//            if (this._lastChecked !== null) {
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus onheader, focus next visible row
            this.focusedRow = 0;
            this._focusNextVisibleRow();
          }
          else if (rowId + 1 < this._modelRepresentation.length) {
            // Set focus on next row
            this._setFocusedRow(rowId, rowId + 1);
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.UP_ARROW :
//            if (this._lastChecked !== null) {
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId > 0) {
            // focus previous row
            this._setFocusedRow(rowId, rowId - 1);
          }
          else {
            // focus header
            this._setFocusInHeader(true);
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.LEFT_ARROW :
//            if (this._lastChecked !== null) {
//              this._enableNextAction(rowId, 0);
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus previous or first header item
            if (this.focusedColumn > 0) {
              var columns = this._getChildElementCount(this.HeaderTable.rows[0]);
              if (this.focusedColumn >= columns) {
                // focus on Actions, move to last table header
                this.focusedColumn = columns - 1;
              }
              else {
                this.focusedColumn--;
              }
              this._setFocusInHeader(true);
            }
          }
          else {
            // focus previous or first row item
            if (this.focusedColumn > 0) {
              this.focusedColumn--;
              this._setFocusInRow();
            }
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.RIGHT_ARROW :
//            if (this._lastChecked !== null) {
//              this._enableNextAction(rowId, 0);
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus next or last header item
            if (this.focusedColumn < this._getChildElementCount(this.HeaderTable.rows[0])) {
              this.focusedColumn++;
              this._setFocusInHeader(true);
            }
          }
          else {
            // focus next or last row item
            if (this.focusedColumn < this._maxFocusColumns - 1) {
              this.focusedColumn++;
              this._setFocusInRow();
            }
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
//          case k.SPACE :
//            if (rowId !== -1) {
//              row = this._modelRepresentation[rowId];
//              this._checkRow(rowId, !row.checked);
//            }
//        break;
        case k.TAB :
          // Remove hover indicator in this case, because table area will be left.
          if (this._lastOver !== null) {
            var row = this._modelRepresentation[this._lastOver];
            this._unsetHoverForRow(row, this._lastOver);
            this._lastOver = null;
          }
//          if (this._lastChecked !== null) {
//            this._setRowUnselected(this._lastChecked);
//          }
          break;

        case k.ENTER :
          // enter was on an action?
          var actionId = node["_actionId"];
          if (actionId !== undefined && actionId >= 0) {
            var actions = this._model[rowId].inlineActions;
            var action = actions[actionId];
            // fire only if enabled
            if (action.enabled) {
              this.onActionRequested(action.id, this._model[rowId]);
            }
          }
          break;
        default:
          break;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "  key " + event.keyCode + "  row " + this.focusedRow +
        "  focusedColumn " + this.focusedColumn, "Table");
    },

    /**
     * Sets the focus to a new row. Sets the focus to the focusNode of the current focus column and highlights
     * the focused row. Scrolls the view if the new focused row is not (completely) visible.
     * 
     * @param currentIndex
     * @param newIndex
     * @returns {void} nothing
     */
    _setFocusedRow: function _setFocusedRow_Table(currentIndex, newIndex){
      this.focusedRow = newIndex;
      var focusNode = null;
      var newDataRow = this._modelRepresentation[newIndex].dataRow;
      var y;
      if (currentIndex) {
        if (newIndex > currentIndex) {
          // scroll down ?
          var newDataRowBottom = newDataRow.offsetTop + newDataRow.offsetHeight;
          // bottom of row is outside visible view?
          if ((this.DataTable.offsetTop + newDataRowBottom) > this.VScrollerContainer.offsetHeight) {
            y = this.DataTable.offsetTop - (newDataRow.offsetHeight * (newIndex - currentIndex));
            if (com.ibm.bpm.social.util.common.Util.isIPad) {
              this.BodyCenter.scrollTop = -y;
            }
            else {
              this.VScrollerContainer.scrollTop = -y;
            }
          }
        }
        else if (newIndex < currentIndex) {
          // scroll up ?
          // top of row is outside visible view?
          if (newDataRow.offsetTop < this.VScrollerContainer.scrollTop) {
            y = newDataRow.offsetTop;
            if (com.ibm.bpm.social.util.common.Util.isIPad) {
              this.BodyCenter.scrollTop = y;
            }
            else {
              this.VScrollerContainer.scrollTop = y;
            }
          }
        }
      }
      // set focus
      this._setFocusInRow();
    },

    /**
     * Sets the focus to the focusNode of the current focus column in the focused row.
     */
    _setFocusInRow: function _setFocusInRow_Table(){
      if (this.focusedRow != -1) {
        var focusNode = null;
        var dataRow = this._modelRepresentation[this.focusedRow].dataRow;
        var columns = this._getChildElementCount(dataRow);
        if (this.focusedColumn < columns) {
          focusNode = dataRow.childNodes[this.focusedColumn].focusNode;
        }
        else {
          var actionsIndex = this.focusedColumn - columns;
          var actions = this._modelRepresentation[this.focusedRow].actions;
          if (actionsIndex < actions.length) {
            focusNode = actions[actionsIndex];
          }
        }
        if (focusNode) {
          this._setFocusToNode(focusNode, true);
          this.onFocusChanged(this._model[this.focusedRow]);
          this._onTableMouseOver({
            target: dataRow
          });
        }
      }
    },

    /**
     * Sets the focus to the focusNode of the current focus column in the header row.
     */
    _setFocusInHeader: function _setFocusInHeader_Table(setFocus){
      this.focusedRow = -1;
      var focusNode = null;
      var headerRow = this.HeaderTable.rows[0];
      if (this.focusedColumn < this._getChildElementCount(headerRow)) {
        focusNode = headerRow.childNodes[this.focusedColumn].focusNode;
      }
      else {
        focusNode = this.HeaderRightFocusNode;
      }
      if (focusNode) {
        if (setFocus) {
          this._setFocusToNode(focusNode, true);
        }
        else {
          this._setFocusNode(focusNode);
        }
      }
    },

    _enableNextAction: function _enableNextAction_Table(rowId, index, newIndex){
//        var actions = this._modelRepresentation[rowId].actions;
//        if (actions && actions.length > 0) {
      // If the first action has no key or ENTER defined, trigger it on enter.
      var inlineActions = this._model[rowId].inlineActions;
      if (inlineActions && inlineActions.length > 0) {
        var actionId = index;
        dojo.removeClass(inlineActions[actionId], inlineActions[actionId].hoverClass);
        if (newIndex > index) {
          while (!inlineActions[newIndex].enabled && newIndex !== index) {
            newIndex++;
            if (newIndex >= inlineActions.length) {
              newIndex = 0;
            }
          }
        }
        else {
          while (!inlineActions[newIndex].enabled && newIndex !== index) {
            newIndex--;
            if (newIndex < 0) {
              newIndex = inlineActions.length - 1;
            }

          }
        }
        if (inlineActions[newIndex].enabled) {
          dojo.addClass(inlineActions[newIndex], inlineActions[newIndex].hoverClass);
          this._currentActionIndex = newIndex;
        }
        else {
          this._currentActionIndex = null;
        }
      }
      else {
        this._currentActionIndex = null;
      }
    },

//      /**
//       * create a target for droppable tasks
//       */
//      _createDndTarget: function _createDndTarget_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // create a target for droppable tasks
//        if (this.dndAccept != this._config.supportedDndTypes) {
//          this.dndAccept = this._config.supportedDndTypes || [];
//          // Create a target for drop actions in the empty area below the rows.
//          if (!this.dndTarget) {
//            this.dndTarget = new com.ibm.bpc.widget.util.DnDSource(this.Center, {
//              accept: this.dndAccept,
//              horizontal: false,
//              copyOnly: false
//            });
//            // this is just for completeness.
//            // typically dndSource is created after dndTarget, so this code
//            // normally shouldn't be necessary. Nevertheless, be defensive.
//            if (this.dndSource) {
//              this.dndSource.accept = dojo.clone(this.dndTarget.accept);
//            }
//          }
//          else {
//            // To update just the accepted types, recreate the dndTarget.accept object
//            // dndTarget.accept is an object mapping each accept to 1:
//            // Object { bspace_htm_TaskList=1}
//            var accept = {};
//            for ( var i = 0; i < this.dndAccept.length; i++) {
//              accept[this.dndAccept[i]] = 1;
//            }
//            this.dndTarget.accept = accept;
//            // if we have a DnDSource, update the accepted types
//            if (this.dndSource) {
//              this.dndSource.accept = accept;
//            }
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//      /**
//       * create a DnD source on the table body
//       */
//      _createDndSource: function _createDndSource_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // create a source for the rows
//        if (this.dndType != this._config.publishedDndTypes) {
//          this.dndType = this._config.publishedDndTypes || [];
//          if (!this.dndSource && this.dndType.length > 0) {
//            var args = {
//              creator: dojo.hitch(this, this._dndCreator),
//              accept: this.dndAccept,
//              horizontal: false,
//              copyOnly: false
//            };
//            this.dndSource = new com.ibm.bpc.widget.util.DnDSource(this._dnTableBody, args);
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//
//      /*
//       * Creator callback for DnD
//       */
//      _dndCreator: function _dndCreator_Table(data, hint){
//        // Add extra row as separator below
//        var types = [];
//        // check for a specific dnd type on the item
//        var item = data["dnd.ref"]._model[data["dnd.index"]];
//        if (item.dndType) {
//          types = item.dndType;
//        }
//        else {
//          // otherwise use the class dnd type
////          types =
////            (data["dnd.ref"]._model[data["dnd.index"]] instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) ? ""
////              : data["dnd.ref"].dndType;
//          types = data["dnd.ref"].dndType;
//        }
//        var node = data["dnd.ref"]._createRow(data["dnd.index"], hint);
//        return {
//          node: node,
//          data: data,
//          type: types
//        };
//      },
//
//
//      /**
//       * Check if row is selected. Insert the node id with the marker1. This function is called from the
//       * DnDSource. The table is connected to the DnDSource via the synchronizeSource function.
//       */
//      getDnDSelection: function(){
//        var selection = {};
//        if (this._modelRepresentation) {
//          for ( var i = 0; i < this._modelRepresentation.length; i++) {
//            var row = this._modelRepresentation[i];
////            if (!row.isGroupHeader) {
//            //if (row.dataRow["_rowId"] == this._lastOver || row.checkbox.checked) {
//            if (row.dataRow["_rowId"] == this._lastOver) {
//              selection[row.dataRow.id] = 1;
//            }
////            }
//          }
//        }
//        return selection;
//      },
//
//      /*******************************************************************************************************
//       * Callback for the DnD Subscription
//       * 
//       * @param {Object} source
//       * @param {Object} nodes
//       * @param {Object} iscopy
//       * @param {Object} target
//       */
//
//      _onDnDNotification: function _onDnDNotification_Table(source, nodes, iscopy, target){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // accept only events if we are the target
//        if ((this.dndTarget && (this.dndTarget.node == target.node)) ||
//          (this.dndSource && this.dndSource.node == target.node)) {
//          var typeToObjectsMap = {};
//          dojo.forEach(nodes, function(aNode){
//            var data = source.getItem(aNode.id);
//            dojo.forEach(data.type, function(type){
//              if (!typeToObjectsMap[type]) {
//                typeToObjectsMap[type] = [];
//              }
//              typeToObjectsMap[type].push(data.data);
//            });
//          });
//          for ( var k in typeToObjectsMap) {
//            this.onDrop(typeToObjectsMap[k], k);
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//      /*
//       *  When a drag starts, the current node should be selected. The ctrl key
//       *  selection is triggered by the onclick not the on mouse down, but for
//       *  the DnD it has to be selected before the onclick event.
//       *  Thus select the current node, if not already done.
//       */
//      _onDnDStart: function _onDnDStart_Table(source, nodes, copy){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        if (source && source.anchor) {
//          var rowId = source.anchor["_rowId"];
//          this._selectionPending = -1;
//          if (rowId !== undefined && rowId >= 0) {
//            var row = this._modelRepresentation[rowId];
//            // if the current node is the lastOver
//            if (this._lastOver == rowId) {
//              // set the checkbox, because for ctrl it is not set already
////              if (!row.checkbox.checked) {
//              this._currentSelection = Number(rowId);
//              this._checkRow(rowId, true);
//              this.focusedRow = rowId;
//              this.onFocusChanged(this._model[rowId]);
//              this._onSelectionChanged(this.getCheckedData());
//              this._selectionPending = rowId;
////              }
//            }
//          }
//          // flag that a DnD is started, that is necessary to finish
//          // the correct selection in the onclick event. As a mousedown
//          // on an unselected row deselects all others and selects the current.
//          // But if the current node is already selected and a DnD is started
//          // the deselection should not happen. Thus the deselection happens in
//          // the onclick in that case.
//          this._deselectionPending = -1;
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },

//      /**
//       * _prepareModelForGroupBy - insert the group headers into the model - determine the groups and build a
//       * list of them
//       * 
//       * @param {Array} model the model we got from outside
//       */
//      _prepareModelForGroupBy: function _prepareModelForGroupBy_Table(model){
//        var preparedItems = [];
//        var item;
//        var groupByProperty = this._config.activeGroupByProperty;
//
//        if (groupByProperty) {
//          var lastGroup = {}; // Set value to something that won't match a group property value
//
//          // determine the groups, extract them into new list
//
//          for ( var i = 0; i < model.length; i++) {
//            item = model[i];
//            var groupName = item[groupByProperty];
//
//            if (groupName != lastGroup) {
//              lastGroup = groupName;
//              // insert the group header as new row.
//              preparedItems.push(new com.ibm.bpm.social.widgets.task.list._GroupHeader(groupName));
//
//
//            }
//            preparedItems.push(item);
//          }
//        }
//        else {
//          // no grouping
//          preparedItems = model;
//        }
//
//        return preparedItems;
//      },
//
//      _initializeGroupFormatter: function _initializeGroupFormatter_Table(){
//        if (this._config.groupFormatter) {
//          this._cleanupGroupFormatter();
//          if (this._config.activeGroupByProperty && this._model.length > 1) {
//            var groupsList = this._getListOfGroups(this._model);
//            this._config.groupFormatter.initialize(groupsList);
//          }
//        }
//        this._groupHeaders = [];
//
//      },
//
//      _cleanupGroupFormatter: function _cleanupGroupFormatter_Table(){
//        if (this._config.groupFormatter) {
//          this._config.groupFormatter.cleanup();
//        }
//      },
//
//
//      /**
//       * get a list of groups and the count of members in the groups
//       * 
//       * @param {Array} items (the model)
//       * @return com.ibm.widget.util.Group[]
//       */
//
//      _getListOfGroups: function _getListOfGroups_Table(items){
//        var groupsList = [];
//        var groupEntry = null;
//        var item = null;
//
//        for ( var i = 0; i < items.length; i++) {
//          item = items[i];
//          if (item instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) {
//            if (groupEntry) {
//              groupsList.push(groupEntry);
//            }
//
//            groupEntry = new com.ibm.widget.util.Group(item.groupName);
//
//          }
//          else {
//            if (groupEntry) {
//              groupEntry.countVisible++;
//            }
//          }
//        }
//
//        if (groupEntry) {
//          groupsList.push(groupEntry);
//        }
//
//        return groupsList;
//      },


//      /**
//       * Checks selectAll checkbox, if all items are selected. Unchecks selectAll checkbox, if at least one
//       * item is not selected.
//       */
//      _checkSelectAllStatus: function _checkSelectAllStatus_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//
//        // global check all checkbox
//        var rows = this._modelRepresentation;
//        var allChecked = true;
////        var allInGroupChecked = true;
//        for ( var i = (rows.length) - 1; i >= 0; i--) {
//          var checkbox = rows[i].checkbox;
////          if (rows[i].isGroupHeader === true && i !== ((rows.length) - 1)) {
////            com.ibm.bspace.common.util.ui.CheckBox.setChecked(checkbox, allInGroupChecked);
////            allInGroupChecked = true;
////          }
////          else {
//          if (!checkbox.checked) {
//            //            allInGroupChecked = false;
//            allChecked = false;
//          }
////          }
//        }
//        if (allChecked) {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, true);
//        }
//        else {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//        }
//
//      },

    _appendOrResizeVScrollerImage: function _appendOrResizeVScrollerImage_Table(imageHeight, scrollerWidth){

      if (dojo.isIE) {
        // IE8 bug: if the scrollbar container has the same width as it's content, the scrollbar cannot scrolled
        // with the arrow keys
        if (!this.VScrollImage) {
          dojo.addClass(this.VScroller, 'bSpaceBlankImageHolder');
          var scrollImage = document.createElement('img'); //a11y: none
          scrollImage.src = dojo.moduleUrl("com.ibm.bspace.common.util.ui", "img/blank.gif");
          this.VScrollImage = this.VScroller.appendChild(scrollImage);
        }

        this.VScrollImage.height = imageHeight;
        // if the width of the  scrollContainer is not 3 pixel wider, the scrollbar cannot scrolled with the arrow keys
        if (dojo.isIE == 8) {
          this.VScrollerContainer.style.width = (scrollerWidth + 3) + 'px';
        }
        else {
          this.VScrollerContainer.style.width = (scrollerWidth + 1) + 'px';
        }
      }
    },

    /**
     * wrapper around onSelectionChanged to prevent double events
     * 
     * @param {Array} items an array of 0...n items
     */
    _onSelectionChanged: function(/* Array */items){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var changed = false;
      items.sort();

      if (items.length == this._lastPublishedSelection.length) {
        for ( var i = 0; i < items.length; i++) {
          if (items[i] != this._lastPublishedSelection[i]) {
            changed = true;
            break;
          }
        }
      }
      else {
        // selection changed definitely as size changed
        changed = true;
      }

      if (changed) {
        this._lastPublishedSelection = items.slice(0);
        this.onSelectionChanged(items);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "items changed :" + (changed ? "true" : " false"));
    },

    /**
     * Set the page size which is later used to decide whether a trailing section should be displayed
     * 
     * @param {int} actual page size requested by user, initially 25, doubled by "Show more"
     * @type {void}
     */
    setPageSize: function setPageSize_Table(pageSize){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      this.pageSize = pageSize;
      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    _getChildElementCount: function _getChildElementCount_Table(node){
      if (node.childElementCount) {
        return node.childElementCount;
      }
      if (node.children) {
        return node.children.length;
      }
      return 0;
    }

  });


//dojo.declare("com.ibm.bpm.social.widgets.task.list._GroupHeader", null, {
//  constructor: function(groupName){
//    this.groupName = groupName;
//  }
//});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.SearchBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.SearchBar"] = true;
// SearchBar.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.SearchBar");





dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.list.SearchBar",
    [dijit._Widget,
     dijit._Templated],
    {
      widgetsInTemplate: true,
      templateString: '<div class="bpm-task-list-content-hide" dojoAttachPoint="searchBar"  tabindex="-1">'
        + '<div class="bpm-task-list-header-search" dojoAttachPoint="searchBarTable">'
        + '<span class="bpm-social-search-tf-left">'
        + '<div dojoAttachPoint="submitButton" class="bpm-social-magnify" role="button" tabindex="-1">'
        + '<span class="bpm-social-a11y">&#8629;</span></div>'
        + '</span>'
        + '<span class="bpm-social-search-tf-center">'
        + '<label for="${id}_input" style="display: none" dojoAttachPoint="textBoxLabel"></label>'
        + '<input id="${id}_input" name="search" dojoAttachPoint="textBox" class="bpm-social-search-tf-center" type="text" />'
        + '</span>'
        + '<span class="bpm-social-search-tf-right">'
        + '<div dojoAttachPoint="clearButton" class="bpm-social-close-x bpm-task-list-content-hide"  role="button" tabindex="0">'
        + '<span class="bpm-social-a11y">x</span></div>' + '</span>' + '</div>' + '<div class="nofloat">' + '</div>'
        + '</div>',

      templatePath: null,

      /**
       * postCreate lifecycle function
       * 
       */
      postCreate: function postCreate_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this.clearButton.disabled = true;
        dijit.setWaiState(this.clearButton, 'disabled', 'true');
        this._enableClear = false;
        this._visible = false;
        this.textBoxLabel.innerHTML = this._nlsResources["task-list-search-bar-input-label"];
        dijit.setWaiState(this.textBox, "label", this._nlsResources["task-list-search-bar-input-label"]);
        this.submitButton.title = this._nlsResources["task-list-search-bar-submit-tooltip"];
        //com.ibm.bspace.common.util.ui.TextField.wrapElement(this.textBox);
        // TODO: send search request
        this._connectHandlers = [];

        //configure BTD here
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.textBox);

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * show the search bar
       * 
       */
      show: function show_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this._visible) {
          return;
        }
        this._visible = true;

        this._connectHandlers = [];

        this._connectHandlers.push(dojo.connect(this.submitButton, "onclick", this, function(){
          this._submit();
        }));
        this._connectHandlers.push(dojo.connect(this.submitButton, "onkeypress", this, function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this._submit();
          }
        }));

        this._connectHandlers.push(dojo.connect(this.clearButton, "onclick", this, function(){
          this._clear();
        }));
        this._connectHandlers.push(dojo.connect(this.clearButton, "onkeypress", this, function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this._clear();
          }
        }));

        this._connectHandlers.push(dojo.connect(this.textBox, "onkeydown", this, function(evt){
          this._handleKeyEvent(evt);
        }));

        dojo.removeClass(this.searchBar, "bpm-task-list-content-hide");
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Do not show the search bar, if not supported
       * 
       * @type (void)
       */
      hide: function hide_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        // TODO : how to remove ? Only hide
//        if (this.domNode) {
//          this.domNode.parentNode.removeChild(this.domNode);
//        }
        this._visible = false;
        dojo.addClass(this.searchBar, "bpm-task-list-content-hide");
        if (this._connectHandlers) {
          for ( var i = 0; i < this._connectHandlers.length; i++) {
            dojo.disconnect(this._connectHandlers[i]);
          }
        }
        this._connectHandlers = null;
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Callback to send the submit
       * 
       * @type (void)
       */
      onSubmit: function onSubmit_SearchBar(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.log(2, "search text: " + text);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback to send the clear
       * 
       * @type (void)
       */
      onClear: function onClear_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * destroy the widgets {void} nothing
       */
      destroy: function destroy_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this._connectHandlers) {
          for ( var i = 0; i < this._connectHandlers.length; i++) {
            try {
              dojo.disconnect(this._connectHandlers[i]);
            }
            catch (e) {
              com.ibm.bpm.social.util.common.Log.logError(2, "destroy_SearchBar", e);
            }
          }
          this._connectHandlers = null;
        }
        //com.ibm.bspace.common.util.ui.TextField.unwrapElement(this.textBox);
        this.inherited('destroy', arguments);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Handle the key event for the input field. For key pressed enter send the submit event.
       * 
       * @param {Object} event
       */
      _handleKeyEvent: function _handleKeyEvent_SearchBar(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
          dojo.stopEvent(evt);
          this._submit();
        }
      },

      /**
       * clear the entry in the input field and send a refresh event
       */
      _clear: function _clear_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.textBox.value = "";
        this.textBox.focus();
        this._lastSearchText = null;
        this.onClear();

        this.clearButton.title = "";
        this.clearButton.disabled = true;
        dijit.setWaiState(this.clearButton, "disabled", "true");
        dojo.addClass(this.clearButton, "bpm-task-list-content-hide");
        this._enableClear = false;
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Show the clear button if we have a text in the text field
       * 
       * @type {void}
       */
      _enableClearButton: function _enableClearButton_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.clearButton.title = this._nlsResources["task-list-search-bar-clear-tooltip"];
        this.clearButton.disabled = false;
        dijit.setWaiState(this.clearButton, 'disabled', 'false');
        dojo.removeClass(this.clearButton, "bpm-task-list-content-hide");
        this._enableClear = true;
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Get the data from the text input field and send the search event
       * 
       */
      _submit: function _submit_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._isValid()) {
          this._lastSearchText = this._reworkText(this.textBox.value);
          this.onSubmit(this._lastSearchText);
          if (!this._enableClear) {
            this._enableClearButton();
          }
        }
        else if (this._lastSearchText) {
          // search is not valid if the input field is empty, clear the search if there was one performed
          this._clear();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Set the text from outside
       * 
       * @param {String} text the sreach text to display
       * @type {void}
       */
      setSearchText: function setSearchText_SearchBar(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "text to display: " + text);
        this.textBox.value = text;
        if (!this._enableClear) {
          this._enableClearButton();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Check for the text field to be valid
       */
      _isValid: function _isValid_SearchBar(){
        if (this.textBox && this.textBox.value !== "") {
          return true;
        }
        return false;
      },

      /**
       * Make sure no single quotes are in the search string, then the last one is escaped
       * 
       * @param text
       * @returns {string} text
       */
      _reworkText: function _reworkText_SearchBar(text){
        // do not fix the search string but provide an error message now
        var newText = text;
        if (text) {
          newText = dojo.string.trim(text);
        }

        return newText;
      }

    });

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.PaginationBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.PaginationBar"] = true;
// PaginationBar.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.list.PaginationBar");







dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.list.PaginationBar",
    [dijit._Widget,
     dijit._Templated],
    {


      // ASZ: Please leave the &nbsp; in the paginationText as placeholder for the message. This is required for the height measuring
      templateString: '<div>\
        <div dojoAttachPoint="paginationText" class="bpm-task-pagination">&nbsp;</div>\
        <button dojoAttachPoint="paginationButton" id="taskListPaginationButton_${id}" type="button"></button>\
        </div>',

      count: null,

      globalCount: null,

      postCreate: function(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this.paginationButton.innerHTML = this._nlsResources["task_show_more_button"];
        this.paginationButton.title = this._nlsResources["task_show_more_button_tooltip"];
        com.ibm.bspace.common.util.ui.Button.wrapElement(this.paginationButton);
        this.connect(this.paginationButton, "onclick", "_showMore");
      },

      destroy: function(){
        com.ibm.bspace.common.util.ui.Button.unwrapElement(this.paginationButton);
      },

      getPageSize: function getPageSize_PaginationBar(){
        if (this.count === null) {
          this.count = com.ibm.bpm.social.widgets.task.list.PaginationBar.MORE_COUNT;
        }
        return this.count;
      },

      _showMore: function _showMore_PaginationBar(){
        com.ibm.bspace.common.util.ui.Button.setDisabled(this.paginationButton, true);
        this.count = this.count * 2;
        this.onShowMore();
      },

      onShowMore: function onShowMore_PaginationBar(){},

      setLocalCount: function setLocalCount_PaginationBar(count, countLimitExceeded){
        this.localCount = count;
        this._updatePaginationText(countLimitExceeded);
      },

      getGlobalCount: function getGlobalCount_PaginationBar(){
        return this.globalCount;
      },

      setGlobalCount: function setGlobalCount_PaginationBar(count, countLimitExceeded){
        this.globalCount = count;
        this._updatePaginationText(countLimitExceeded);
      },

      showPaginationButton: function showPaginationButton_PaginationBar(show){
        if (show === false) {
          com.ibm.bpm.social.util.common.Log.log(2, "Hiding pagination button.");
          dojo.style(this.paginationButton, "display", "none");
        }
        else {
          // all other values, even the undefined case: in doubt, do not make user stuck
          com.ibm.bpm.social.util.common.Log.log(2, "Showing pagination button.");
          com.ibm.bspace.common.util.ui.Button.setDisabled(this.paginationButton, false);
          dojo.style(this.paginationButton, "display", "block");
        }
      },

      _updatePaginationText: function _updatePaginationText_PaginationBar(countLimitExceeded){
        // if one is not yet set do nothing
        if (typeof this.localCount == "number" && typeof this.globalCount == "number") {
          var nlsKey = "task_showing_x_of_y_results";
          if (countLimitExceeded && countLimitExceeded === true) {
            nlsKey = "task_showing_x_of_y_results_truncated";
          }
          var msg = dojo.string.substitute(this._nlsResources[nlsKey], {
            1: this.localCount,
            2: this.globalCount
          });

          this.paginationText.innerHTML = msg;
        }
      }
    });

com.ibm.bpm.social.widgets.task.list.PaginationBar.MORE_COUNT = 25;

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler"] = true;
// TasksListRefreshHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler");




/**
 * Handles refresh processing if "TASK_RESOURCE_ASSIGNED" or "TASK_FIELD_CHANGED" task list push events are
 * received.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler", null, {

  _disabled: false,

  _refreshCallback: null,

  _setModelCallback: null,

  _updateModelCallback: null,

  _parentId: null,

  _deltaRefresh: false,

  _subscribedTaskListUpdated: null,

  _refreshing: false,

  _autoRefresh: false,

  _waitingForQueryResult: false,

  _suspendedFull: false,

  _suspendedFullIds: null,

  _queryResultQueued: false,

  _refreshRequestQueued: false,

  _fullRefreshRequestQueued: false,

  _moreRefreshRequestsQueued: false,

  _refreshWaitLevel: 0,

  _refreshTimer: null,

  _delayedTasks: null,

  _delayedTaskIds: null,

  _handledMessageTimes: null, // serialize messages - {tkiid: timeOfOccurrence}

  _fullScreenDialogWithoutRefreshOpened: false,

  _fullRefreshOnResume: false,

  /**
   * @param refreshCallback call back method for executing a full refresh
   * @param setModelCallback call back method to display query result
   * @param updateModelCallback call back method for delta updates of the list
   * @param parentId the id of the caller for logging
   * @param deltaRefresh list supports delta updates
   */
  constructor: function constructor_TasksListRefreshHandler(refreshCallback, setModelCallback, updateModelCallback,
    parentId, deltaRefresh){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListRefreshHandler");

    this._refreshCallback = refreshCallback;
    this._setModelCallback = setModelCallback;
    this._updateModelCallback = updateModelCallback;
    this._parentId = parentId;
    this._deltaRefresh = deltaRefresh;
    this._suspendedFullIds = new Array();
    this._delayedTasks = new Array();
    this._delayedTaskIds = new Array();
    this._handledMessageTimes = {
      latest: null,
      ids: new Array()
    };

    com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListRefreshHandler");
    return;
  },

  /**
   * 
   * @type {void}
   */
  destroy: function destroy_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._unsubscribeTaskListUpdated();
    this._clearRefreshTimer();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Disables the refresh handling while a task list is not visible. This is resumed on the next list refresh.
   */
  disable: function disable_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(2, this._parentId);
    this._disabled = true;
    this._unsubscribeTaskListUpdated();
    this._clearRefreshTimer();
  },

  /**
   * Refresh of the list is started.
   * <p>
   * Clears the timer and resets all refresh properties.
   */
  refreshStarted: function refreshStarted_TasksListRefreshHandler(autoRefresh){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh started, autoRefresh=" + autoRefresh);
    // initialize refresh handling
    this._disabled = false;
    this._refreshing = true;
    this._autoRefresh = autoRefresh;
    if (!this._deltaRefresh) {
      // do not listen any longer
      this._unsubscribeTaskListUpdated();
    }
    // clear running timers and parameters
    this._clearRefreshTimer();
    this._waitingForQueryResult = false;
    this._suspendedFull = false;
    this._suspendedFullIds = new Array();
    this._queryResultQueued = false;
    this._refreshRequestQueued = false;
    this._fullRefreshRequestQueued = false;
    this._moreRefreshRequestsQueued = false;
    this._delayedTasks = new Array();
    this._delayedTaskIds = new Array();
    this._fullScreenDialogWithoutRefreshOpened = false;
    this._fullRefreshOnResume = false;
    this._refreshWaitLevel = 0;
  },

  /**
   * Listen for notifications while waiting for the query result.
   */
  waitingForQueryResult: function waitingForQueryResult_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Query submitted");
    // listen while waiting for query result
    this._subscribeTaskListUpdated();
    this._waitingForQueryResult = true;
  },

  /**
   * The query is finished. Result is not displayed if refresh is suspended.
   * 
   * @param success {Boolean} the query finished successful
   */
  queryFinished: function queryFinished_TasksListRefreshHandler(success){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Query finished");
    if (!this._disabled) {
      this._waitingForQueryResult = false;
      if (this._suspendedFull) {
        // delay display of query result
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Delay display of query result");
        this._queryResultQueued = true;
      }
      else if ((this._refreshRequestQueued || this._fullRefreshRequestQueued) && this._autoRefresh) {
        // another auto refresh request is queued, ignore this query result
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Ignore query result");
        this.refreshFinished(this._autoRefresh);
      }
      else if (success) {
        // no refresh request queued or refresh was not an auto refresh, prepare list from query result (remove "Loading" message)
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Display query result");
        this._setModelCallback(this._autoRefresh);
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Refresh of the list is finished.
   * <p>
   * Starts listening for notifications and sets the timer if a request has been queued while waiting for the
   * query result.
   */
  refreshFinished: function refreshFinished_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh finished");
    if (!this._disabled) {
      this._refreshing = false;
      // make sure we are listening after refresh
      this._subscribeTaskListUpdated();
      // TODO do we really want to refresh one second after last refresh completed???
      if (this._refreshRequestQueued) {
        // start delayed refresh processing
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start delayed refresh processing after refresh");
        this._prepareRefreshAfterNotification();
      }
      if (this._fullRefreshRequestQueued) {
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start full refresh processing after refresh");
        this.doFullRefresh();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Suspends full refresh handling for lists supporting delta refresh.
   * <p>
   * Should be called if the view should not be disturbed by full refresh processing, e.g. if a dialog is
   * opened or tasks are expanded. Refresh is delayed until resumeFullRefresh is called.
   * <p>
   * Suspend requests are stacked and automatic refresh is only resumed if all suspend requests are resumed or
   * if a refresh is triggered.
   * <p>
   * If the dialog is "Process Diagram" or "Audit History" a full refresh will be triggered on resume if any
   * notifications are received in the meantime
   * 
   * @param dialogId a unique id of the dialog which has been opened
   */
  suspendFullRefresh: function suspendFullRefresh_TasksListRefreshHandler(dialogId){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspend full refresh for " + dialogId);
    if (!this._disabled) {
      if (this._isFullScreenDialogWithoutRefresh(dialogId)) {
        // do a full refresh on return from the dialog
        this._fullScreenDialogWithoutRefreshOpened = true;
      }
      this._suspendedFullIds.push(dialogId);
      if (!this._suspendedFull && (!this._refreshing || this._waitingForQueryResult)) {
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspend full refresh");
        this._suspendedFull = true;
        // delay refresh until refresh handling is resumed
        this._clearRefreshTimer();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
  },

  /**
   * Resumes full refresh handling, triggers an immediate refresh if a request was queued or shows the result
   * of a refresh which was received in the meantime.
   * 
   * @param dialogId a unique id of the dialog which has been opened
   */
  resumeFullRefresh: function resumeFullRefresh_TasksListRefreshHandler(dialogId){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Resume full refresh for " + dialogId);
    if (!this._disabled) {
      if (this._suspendedFull) {
        var index = dojo.indexOf(this._suspendedFullIds, dialogId);
        if (index >= 0) {
          this._suspendedFullIds.splice(index, 1);
        }
        if (this._suspendedFullIds.length === 0) {
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Resume full refresh");
          this._suspendedFull = false;
          if (this._queryResultQueued &&
            ((!this._refreshRequestQueued && !this._fullRefreshRequestQueued) || !this._autoRefresh)) {
            // no additional request or manual refresh - update list with queued query result
            com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do setModel after suspend");
            this._setModelCallback(this._autoRefresh);
          }
          else if (!this._refreshing) {
            if ((this._refreshRequestQueued && this._refreshTimer === null) || this._fullRefreshRequestQueued ||
              this._fullRefreshOnResume) {
              // do immediate refresh after suspend if request was queued and no refresh is in progress
              com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do refresh after suspend");
              this._updateTaskList();
            }
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Suspended dialogIds: " + this._suspendedFullIds);
        }
      }
      else {
        com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
      }
    }
  },

  /**
   * Checks if the opened dialog is "Process Diagram" or "Audit History"
   * 
   * @param dialogId
   * @returns {boolean} true if the dialog is a full screen dialog which does not trigger a refresh on return
   *          to the list
   */
  _isFullScreenDialogWithoutRefresh: function __isFullScreenDialogWithoutRefresh_TasksListRefreshHandler(dialogId){
    if (dialogId.indexOf("ProcessDiagramDialog") !== -1 || dialogId.indexOf("AuditHistoryDialog") !== -1) {
      com.ibm.bpm.social.util.common.Log.log(2, "Dialog does not trigger refresh on return: " + dialogId);
      return true;
    }
    return false;
  },

  /**
   * Trigger an immediate full refresh.
   * <p>
   * If refresh or full refresh is suspended delay until refresh is resumed
   * 
   */
  doFullRefresh: function doFullRefresh_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (!this._disabled) {
      this._fullRefreshRequestQueued = true;
      if (!this._refreshing && !this._suspendedFull) {
        // clear timer
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Clear timer");
        this._clearRefreshTimer();
        this._updateTaskList();
      }
      else if (!this._deltaRefresh) {
        // refresh request queued while refreshing or suspended, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Starts listening for task list update events
   */
  _subscribeTaskListUpdated: function _subscribeTaskListUpdated_TasksListRefreshHandler(){
    // task list refresh if task list push event received for task assigned to me or my groups 
    if (this._subscribedTaskListUpdated === null) {
      this._subscribedTaskListUpdated =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED, dojo.hitch(this, this._handleTaskListMessage));
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Subscribed");
    }
  },

  /**
   * Stops listening for task list update events
   */
  _unsubscribeTaskListUpdated: function _unsubscribeTaskListUpdated_TasksListRefreshHandler(){
    if (this._subscribedTaskListUpdated !== null) {
      dojo.unsubscribe(this._subscribedTaskListUpdated);
      this._subscribedTaskListUpdated = null;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Unsubscribed");
    }
  },

  /**
   * Triggers a delayed refresh of the list.<br>
   * The initial wait time is 1 second (wait level 1).<br>
   * If within this second another notification is received the refresh is delayed for another 2 seconds (wait
   * level 2).<br>
   * If within these 2 seconds another notification is received the refresh is delayed for another 4 seconds
   * (wait level 3).
   * <p>
   * The maximum delay is 7 seconds.
   * <p>
   * If the list is actually refreshing, setting the timer is delayed until the actual refresh is finished.
   * 
   * @param data notification data
   * @returns {void} nothing
   */
  _prepareRefreshAfterNotification: function _prepareRefreshAfterNotification_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Prepare, waitLevel=" + this._refreshWaitLevel +
      ", refreshing=" + this._refreshing);
    if (this._refreshTimer === null) {
      this._refreshRequestQueued = true;
      this._refreshWaitLevel = 1;
      if (!this._refreshing) {
        // start timer
        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Start timer");
        this._setRefreshTimeout();
      }
      else if (!this._deltaRefresh) {
        // refresh request queued while refreshing or suspended, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Additional request");
      this._moreRefreshRequestsQueued = true;
      if (!this._deltaRefresh) {
        // additional refresh request queued, do not listen any longer
        this._unsubscribeTaskListUpdated();
      }
    }
  },

  /**
   * Sets the timer for refresh
   */
  _setRefreshTimeout: function _setRefreshTimeout_TasksListRefreshHandler(){
    if (this._refreshTimer === null) {
      // wait level 1: 1 second, 2: 2 seconds, 3: 4 seconds -> max of 7 seconds
      var waitTime = Math.pow(2, (this._refreshWaitLevel - 1)) * 1000;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Prepare " + waitTime);
      this._refreshTimer = setTimeout(dojo.hitch(this, this._refreshAfterNotification), waitTime);
    }
    else {
      // Code logic problem!
      com.ibm.bpm.social.util.common.Log.log(1, this._parentId + " - Refresh timer is already set!!!");
    }
  },

  /**
   * This method is called when the refresh timer has finished. If additional notifications have been received
   * another timer is started for delayed refresh, if the maximum waits are not exceeded.
   */
  _refreshAfterNotification: function _refreshAfterNotification_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._refreshTimer = null;
    com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Refresh?");
    if (this._moreRefreshRequestsQueued && this._refreshWaitLevel < 3) {
      // extend wait
      this._refreshWaitLevel++;
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Extend wait, waitLevel=" + this._refreshWaitLevel);
      this._setRefreshTimeout();
      if (this._refreshWaitLevel < 3) {
        // listen for more requests
        this._moreRefreshRequestsQueued = false;
        this._subscribeTaskListUpdated();
      }
    }
    else {
      // no other requests or wait limit is reached
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do refresh");
      this._updateTaskList();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Handles the task list update push messages
   * 
   * @param message the push message for task list updates
   * @returns {_handleTaskListMessage_TasksListRefreshHandler} {void} nothing
   */
  _handleTaskListMessage: function _handleTaskListMessage_TasksListRefreshHandler(message){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message: channel=" + message.channel);
    var data = message.data;
    var handleTask = false;
    var newTaskAssignment = false;
    if (data && data.task) {
      com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message data: kind=" + data.kind +
        ", timeOfOccurrence=" + data.timeOfOccurrence);
      com.ibm.bpm.social.util.common.Log.log(2, "Incoming task list message task data: tkiid=" + data.task.tkiid +
        ", assignedTo=" + data.task.assignedTo + ", state=" + data.task.state + ", priority=" + data.task.priority +
        ", dueTime=" + data.task.dueTime + ", atRiskTime=" + data.task.atRiskTime + ", completionTime=" +
        data.task.completionTime);
      switch (data.kind) {
        case "TASK_RESOURCE_ASSIGNED" :
          var groupMessage = message.channel && message.channel.indexOf("/bpm/process_portal/task_list/group/") === 0;
          // check for delayed processing
          if (data.task.assignedToType) {
            if (data.task.assignedToType === "group") {
              if (groupMessage) {
                // ignore additional user message
                newTaskAssignment = true;
                handleTask = true;
              }
              else {
                com.ibm.bpm.social.util.common.Log.log(3, "Additional user message is ignored for task " + data.task.tkiid);
              }

            }
            else if (data.task.assignedToType === "user") {
              var userData = com.ibm.bpm.social.util.common.Util.getCurrentUser();
              var userId = userData.userName;
              if (data.task.assignedTo === userId) {
                if (!groupMessage) {
                  // ignore additional group message
                  newTaskAssignment = true;
                  handleTask = true;
                }
                else {
                  com.ibm.bpm.social.util.common.Log.log(3, "Additional group message is ignored for task " +
                    data.task.tkiid);
                }
              }
              else {
                // assigned to another user, needs to be removed from list
                handleTask = true;
              }
            }
          }
          break;
        case "TASK_FIELD_CHANGED" :
          handleTask = true;
          break;
        default:
          com.ibm.bpm.social.util.common.Log.log(3, "Invalid message kind: " * data.kind);
          break;
      }
      if (this._fullScreenDialogWithoutRefreshOpened) {
        // do not handle update, trigger a full refresh on dialog close
        com.ibm.bpm.social.util.common.Log.log(2, "Full refresh will be triggered after resume!");
        this._fullRefreshOnResume = true;
      }
      else if (this._deltaRefresh) {
        if (handleTask) {
          if (this._serializeMessage(data)) {
            if (newTaskAssignment || dojo.indexOf(this._delayedTaskIds, data.task.tkiid) !== -1 || this._refreshing) {
              // delayed delta refresh for 
              // - tasks which are assigned and may need to be added to the list 
              // - tasks which are already part of delayed delta refresh
              //    ensures that state change for inline completion is not handled before task assignment (which adds task to list)
              // - all tasks while the task list is refreshing
              com.ibm.bpm.social.util.common.Log.log(2, "Delayed delta refresh for task " + data.task.tkiid);
              this._addDelayedTask(data.task);
              this._prepareRefreshAfterNotification();
            }
            else {
              // immediate delta refresh
              com.ibm.bpm.social.util.common.Log.log(2, "Immediate delta refresh for task " + data.task.tkiid);
              this._updateModelCallback([data.task], true);
            }
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(2, "Redundant message is ignored for task " + data.task.tkiid);
        }
      }
      else {
        // delayed full refresh
        com.ibm.bpm.social.util.common.Log.log(2, "Delayed full refresh");
        this._prepareRefreshAfterNotification();
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "No task data!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Messages need to be serialized because they may not come in the order they have been submitted. Older
   * messages are ignored.
   * 
   * @param data message data
   * @returns {Boolean} true if the message needs to be handled, false if the message must be ignored
   */
  _serializeMessage: function _serializeMessage_TasksListRefreshHandler(data){
    var handle = true;
    var timeOfOccurrence = this._handledMessageTimes[data.task.tkiid];
    var handledTime = dojo.date.stamp.fromISOString(timeOfOccurrence);
    var newTime = dojo.date.stamp.fromISOString(data.timeOfOccurrence);
    com.ibm.bpm.social.util.common.Log.log(2, data.task.tkiid + " - timeOfOccurrence " + data.timeOfOccurrence +
      ", already handled " + timeOfOccurrence);
    if (timeOfOccurrence !== undefined) {
      handle = dojo.date.compare(newTime, handledTime) >= 0;
    }
    else {
      this._handledMessageTimes.ids.push(data.task.tkiid);
    }
    if (handle) {
      // save/replace time for handled message
      this._handledMessageTimes[data.task.tkiid] = data.timeOfOccurrence;
      if (this._handledMessageTimes.latest !== null) {
        var lastHandledTime = dojo.date.stamp.fromISOString(this._handledMessageTimes.latest);
        if (dojo.date.compare(newTime, lastHandledTime) > 0) {
          this._handledMessageTimes.latest = data.timeOfOccurrence;
        }
      }
      else {
        this._handledMessageTimes.latest = data.timeOfOccurrence;
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Newer message already handled for task " + data.task.tkiid);
    }
    return handle;
  },

  /**
   * Add or replace the task data for a task message which is handled delayed
   * 
   * @param task
   * @returns {void} nothing
   */
  _addDelayedTask: function _addDelayedTask_TasksListRefreshHandler(task){
    if (this._delayedTasks === null) {
      this._delayedTasks = new Array();
      this._delayedTaskIds = new Array();
    }
    var index = dojo.indexOf(this._delayedTaskIds, task.tkiid);
    if (index !== -1) {
      // replace task with latest version
      this._delayedTasks[index] = task;
      com.ibm.bpm.social.util.common.Log.log(2, "Replaced delayed delta refresh data for task " + task.tkiid);
    }
    else {
      // add task
      this._delayedTasks.push(task);
      this._delayedTaskIds.push(task.tkiid);
      com.ibm.bpm.social.util.common.Log.log(2, "Added delayed delta refresh data for task " + task.tkiid);
    }
  },

  /**
   * Update the task list with full refresh or delta refresh with task messages which have been delayed.
   * 
   * @returns {void} nothing
   */
  _updateTaskList: function _updateTaskList_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (!this._disabled) {
      if (!this._deltaRefresh || this._fullRefreshRequestQueued || this._fullRefreshOnResume) {
        if (this._suspendedFull) {
          // delay full refresh until it is resumed
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Full refresh is suspended by " +
            this._suspendedFullIds);
        }
        else {
          // do full refresh
          com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Do full refresh");
          this._refreshCallback(true);
        }
      }
      else if (this._deltaRefresh && this._delayedTasks.length > 0) {
        // delayed adding of new tasks

        // TODO if search string specified delayed query to check if tasks need to be added!

        com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Update model for task(s): " + this._delayedTaskIds);
        this._updateModelCallback(this._delayedTasks);
        this._delayedTasks = new Array();
        this._delayedTaskIds = new Array();
        this._cleanupHandledMessageTimes();
        this._refreshRequestQueued = false;
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "Disabled!");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Remove all remembered message times which are older than 1 minute
   */
  _cleanupHandledMessageTimes: function _cleanupHandledMessageTimes_TasksListRefreshHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "Number of last handled message times: " +
      this._handledMessageTimes.ids.length);
    var latest = dojo.date.stamp.fromISOString(this._handledMessageTimes.latest);
    var toOld = dojo.date.add(latest, "minute", -1);
    com.ibm.bpm.social.util.common.Log.log(3, "Removing messages older than " + toOld);
    for ( var i = 0; i < this._handledMessageTimes.ids.length; i++) {
      var tkiid = this._handledMessageTimes.ids[i];
      var lastOccurence = dojo.date.stamp.fromISOString(this._handledMessageTimes[tkiid]);
      if (dojo.date.compare(toOld, lastOccurence) >= 0) {
        this._handledMessageTimes[tkiid] = undefined;
        this._handledMessageTimes.ids.splice(i, 1);
        com.ibm.bpm.social.util.common.Log.log(3, "Removed last handled message time for " + tkiid);
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "Number of last handled message times: " +
      this._handledMessageTimes.ids.length);
  },

  /**
   * Clears the timer if it is running
   */
  _clearRefreshTimer: function _clearRefreshTimer_TasksListRefreshHandler(){
    if (this._refreshTimer !== null) {
      com.ibm.bpm.social.util.common.Log.log(3, this._parentId + " - Clear timer");
      clearTimeout(this._refreshTimer);
      this._refreshTimer = null;
    }
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.EntityQuery"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.EntityQuery"] = true;
// EntityQuery.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

// **************************************
//
// File Version: 1.5 11/10/03 16:29:49
//
// **************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.EntityQuery");



/*
 * Creates and executes a list query based on various parameters.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.util.EntityQuery", null, {
  _customFilter: null,
  _userRoleFilter: null,
  _queue: null,
  _prioritiesConfig: null,
  _idFilter: null,
  _groupingProperties: null,
  _sortProperties: null,
  _async: true,
  _offset: 0,
  _pageSize: null,
  _selectedAttributes: null,
  _type: null,

  constructor: function constructor_EntityQuery(parameters){
    this.restHandler = parameters.restHandler;
    this._D2DFilter = parameters.D2DFilter;
    this._userId = parameters.userId;
    this._widgetId = parameters.widgetId;
    this._systems = parameters.systems;
  },

  /**
   * returns the type of the query, can be com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE or
   * com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE.
   * 
   * @return the type of the query
   */
  getType: function getType_EntityQuery(){
    return this._type;
  },

  setType: function setType_EntityQuery(queue){
    if (queue.entityTypeName == "TASK") {
      this._type = com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE;
    }
    else if (queue.entityTypeName == "PROCESS_INSTANCE") {
      this._type = com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE;
    }
    else {
      var e = new Error("Unexpected entityTypeName '" + queue.entityTypeName + "'.");
      com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected entityTypeName '" + queue.entityTypeName + "'.");
      throw e;
    }
  },
  setAsync: function setAsync_EntityQuery(async){
    this._async = async;
  },

  setOffset: function setOffset_EntityQuery(offset){
    this._offset = offset;
  },

  setPageSize: function setPageSize_EntityQuery(pageSize){
    this._pageSize = pageSize;
  },

  setSelectedAttributes: function setSelectedAttributes_EntityQuery(selectedAttributes){
    this._selectedAttributes = selectedAttributes;
  },

  setCustomFilter: function setCustomFilter_EntityQuery(customFilter){
    this._customFilter = customFilter;
  },

  clear: function clear_EntityQuery(){
    this._type = null;
    this._customFilter = null;
    this._selectedAttributes = null;
    this._offset = null;
    this._pageSize = null;
    this._async = null;
    this._userRoleFilter = null;
    this._queue = null;
    this._prioritiesConfig = null;
    this._idFilter = null;
    this._sortProperties = null;
  },

  /*
   * Checks if the filter syntax is valid by executing a query.
   */
  testCustomFilter: function testCustomFilter_EntityQuery(customFilter, error){
    // Store current query state in local variables
    var oldFilter = this._customFilter;
    this.setCustomFilter(customFilter);

    var oldQueue = this._queue;
    this.setQueue(customFilter.queue);

    var oldAsync = this._async;
    this.setAsync(false);

    var oldSelectedAttributes = this._selectedAttributes;
    this.setSelectedAttributes(null);

    var oldOffset = this._offset;
    this.setOffset(0);

    var oldPageSize = this._pageSize;
    this.setPageSize(1);

    var ok = false;
    this._loadItems(function(responseObject, ioArgs){
      ok = true;
    }, error);

    // Reset the query state
    this.setCustomFilter(oldFilter);
    this.setQueue(oldQueue);
    this.setAsync(oldAsync);
    this.setSelectedAttributes(oldSelectedAttributes);
    this.setOffset(oldOffset);
    this.setPageSize(oldPageSize);

    return ok;
  },

  setQueue: function setQueue_EntityQuery(queue){
    this._queue = queue;
  },

  setUserRoleFilter: function setUserRoleFilter_EntityQueue(userRoleFilter){
    this._userRoleFilter = userRoleFilter;
  },

  setIdFilter: function setIdFilter_EntityQueue(idFilter){
    this._idFilter = idFilter;
  },

  setPrioritiesConfig: function setPrioritiesConfig_EntityQueue(prioritiesConfig){
    this._prioritiesConfig = prioritiesConfig;
  },

  setGroupingProperties: function setGroupingProperties_EntityQueue(groupingProperties){
    this._groupingProperties = groupingProperties;
  },

  setSearchFilter: function setSearchFilter_EntityQueue(text){
    this._searchFilter = text;
  },

  /**
   * @param load
   * @param error {void} nothing
   * @returns {dojo.Deferred} an object which allows to cancel the dojo.xhr request
   */
  performContentRequest: function performContentRequest_EntityQuery(load, error){
    return this._loadItems(load, error);
  },

  performCountRequest: function performCountRequest_EntityQuery(load, error){
    this._countItems(load, error);
  },

  /*
   * Returns whereClause and/or interactionFilter.
   */
  toObject: function toObject_EntityQuery(){
    return this._getWhereClause();
  },

  _loadItems: function _loadItems_EntityQuery(load, error){

    var params = {};
    // set the order by statement
    var sortBy = null;
    var sortProperties = (this._userRoleFilter ? this._userRoleFilter.sortProperties : null);
    // sort order 
    if (sortProperties && sortProperties.length > 0) {
      for ( var i = 0; i < sortProperties.length; i++) {
        if (!sortBy) {
          sortBy = sortProperties[i].property + " " + sortProperties[i].sortDir;
        }
        else {
          sortBy += "," + sortProperties[i].property + " " + sortProperties[i].sortDir;
        }
      }
    }

    // the following is old mimic and only there for compat reasons
    // if we have no sortProperties from the _userRoleFilter, get the sort from the items

    // check for common sort settings 
    if (!sortProperties && this._queue.item && this._queue.item.properties) { // TODO ?
      var properties = this._queue.item.properties;
      for (i = 0; i < properties.length; i++) {
        if (properties[i].isSort === true) {
          if (!sortBy) {
            sortBy =
              properties[i].property + " " + this._replaceOrderParameter(properties[i].property, properties[i].sortDir);
          }
          else {
            // don't set the same property twice
            if (!this._groupingProperties || this._groupingProperties.property != properties[i].property) {
              sortBy =
                sortBy + "," + properties[i].property + " " +
                  this._replaceOrderParameter(properties[i].property, properties[i].sortDir);
            }
          }
          // TODO what is the API limit for sortBy ? 
          // break;
        }
      }
    }

    params.sortAttributes = sortBy;

    var currentQuery = this._getWhereClause();
    var whereClause = "";
    if (currentQuery.whereClause) {
      whereClause = currentQuery.whereClause;
    }

    // for process based queries include the task data
    if (this._queue.entityTypeName == "PROCESS_INSTANCE") {
      params.includeTaskData = true;
    }

    params.queryFilter = whereClause;
    params.interactionFilter = currentQuery.interactionFilter;
    params.queryTable = this._queue.getPlainQueueName();
    params.domain = this._queue.getDomain();
    params.processAppName = this._queue.processAppName; // may be null, or undefined, or a valid String
    params.selectedAttributes = this._selectedAttributes;
    params.type = this._type;
    params.offset = this._offset;
    params.size = this._pageSize;
    params.searchFilter = this._searchFilter;
    params.async = this._async;
    params.load = function(responseObject, ioArgs){
      load(responseObject);
    };
    params.error = function(data, ioArgs){
      error(data, ioArgs);
    };
    // Fetch data from server    
    return this.restHandler.getQueryTableContent(params);
  },

  _countItems: function _countItems_EntityQuery(load, error){
    var params = {};

    var currentQuery = this._getWhereClause();

    params.whereClause = currentQuery.whereClause;
    params.interactionFilter = currentQuery.interactionFilter;
    params.queryTable = this._queue.getPlainQueueName();
    params.domain = this._queue.getDomain();
    params.processAppName = this._queue.processAppName; // may be null, or undefined, or a valid String
    params.selectedAttributes = this._selectedAttributes;
    params.type = this._type;
    params.offset = this._offset;
    params.size = this._pageSize;
    params.searchFilter = this._searchFilter;
    params.async = this._async;
    params.load = function(responseObject, ioArgs){
      load(responseObject);
    };
    params.error = function(data, ioArgs){
      error(data, ioArgs);
    };

    this.restHandler.getQueryTableCount(params);
  },

  _getWhereClause: function _getWhereClause_EntityQuery(){
    // get the properties map for the current queue
    var propertiesMap = {};
    if (this._queue[com.ibm.bpm.social.widgets.task.util.Constants.QUEUE_PROPERTIES_MAP]) {
      propertiesMap = this._queue[com.ibm.bpm.social.widgets.task.util.Constants.QUEUE_PROPERTIES_MAP];
    }
    var whereClause = null;
    var interactionFilter = null;
    var whereClauseOptionalProperties = null;
    if (this._userRoleFilter) {
      whereClause = this._userRoleFilter.templateWhereClause;
      interactionFilter = this._userRoleFilter.interactionFilter;
      whereClauseOptionalProperties = this._userRoleFilter.templateWhereClauseOptional;
    }
    var whereClauseOptional = "";
    if (!interactionFilter || whereClause) {
      // ensure empty or undefined interactionFilter to be mapped to null
      // or, if there is a whereclause, then we do not use the interaction filter but the where clause
      interactionFilter = null;
    }

    // optional whereClauses are added only if the optional property is available
    if (whereClauseOptionalProperties && this.restHandler.isApiTypeWps()) {
      dojo.forEach(whereClauseOptionalProperties, function(optionalItem, index){
        if (propertiesMap[optionalItem.property] !== null) {
          var clause = optionalItem.clause;
          if (optionalItem.clauses) {
            // more than one possible clause
            // last clause with all properties available is used
            for ( var i = 0; i < optionalItem.clauses.length; i++) {
              var validClause = true;
              var item = optionalItem.clauses[i];
              for ( var p = 0; p < item.additionalProperties.length; p++) {
                if (propertiesMap[item.additionalProperties[p]] === null) {
                  validClause = false;
                  break;
                }
              }
              if (validClause) {
                clause = item.clause;
              }
            }
          }
          if (clause) {
            if (index > 0) {
              whereClauseOptional += " AND ";
            }
            whereClauseOptional += clause;
          }
        }
      });

      if (whereClauseOptional !== "") {
        if (whereClause === null || whereClause === undefined || whereClause === "") {
          whereClause = whereClauseOptional;
        }
        else {
          whereClause += " AND " + whereClauseOptional;
        }
      }
    }

    if (this._customFilter) {
      if (!whereClause) {
        whereClause = this._createFilterWhereClause();
      }
      else {
        whereClause = whereClause + " AND (" + this._createFilterWhereClause() + ")";
      }
    }
    // Add D2D filter, if there is one.
    if (this._D2DFilter) {
      var applicNames = this._D2DFilter.split(",");
      if (applicNames) {
        // use the replacement variable for the application
        var D2DWhereClause = "${TASKAPPLICNAME}" + " IN (";
        for ( var k in applicNames) {
          if (k > 0) {
            D2DWhereClause += ",";
          }
          D2DWhereClause +=
            "'" + com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(applicNames[k]) + "'";
        }
        D2DWhereClause += ")";

        var applicationAttribute = propertiesMap["TASKAPPLICNAME"];
        // but only if the task applic name column is in the table
        if (applicationAttribute) {
          whereClause += " AND (" + D2DWhereClause + ")";
        }
      }
    }
    whereClause = this._replaceWhereParameters(whereClause, propertiesMap);

    if (this._idFilter && this._idFilter.property) {
      // Get attribute name for the parent property.
      var parentPropertyAttribute = propertiesMap[this._idFilter.property];

      if (this._idFilter.ids && this._idFilter.ids.length > 0) {
        var idWhereClause = null;
        if (this._idFilter.ids.length == 1) {
          idWhereClause = parentPropertyAttribute + " = ID('" + this._idFilter.ids[0] + "')";
        }
        else if (this._idFilter.ids.length > 1) {
          idWhereClause = parentPropertyAttribute + " IN (";
          dojo.forEach(this._idFilter.ids, function(id, index){
            if (index > 0) {
              idWhereClause += ", ";
            }
            idWhereClause += "ID('" + id + "')";
          });
          idWhereClause += ")";
        }

        if (whereClause === "") {
          whereClause = idWhereClause;
        }
        else {
          whereClause += " AND " + idWhereClause;
        }
      }
      else {
        if (whereClause === "") {
          whereClause = "(" + parentPropertyAttribute + " IS NULL";
        }
        else {
          whereClause += " AND (" + parentPropertyAttribute + " IS NULL";
        }

        // Get expanded tree nodes from cookie.
        var cookieForExpandedIds = dojo.cookie(this._widgetId + "_lazyLoadingTreeSaveStateCookie");
        var expandedIds = [];
        if (cookieForExpandedIds) {
          expandedIds = cookieForExpandedIds.split(",");
        }

        // Remove "null" elements from the array. I'm not quite sure where they come from, I
        // suppose from the DOJO tree implementation.
        expandedIds = dojo.filter(expandedIds, function(id){
          return (id != "null" && id != "undefined" && id !== "");
        });

        // Now create the where clause by using the expanded id's.
        if (expandedIds.length == 1) {
          whereClause += " OR " + parentPropertyAttribute + " = ID('" + expandedIds[0] + "')";
        }
        else if (expandedIds.length > 1) {
          whereClause += " OR " + parentPropertyAttribute + " IN (";
          dojo.forEach(expandedIds, function(id, index){
            if (index > 0) {
              whereClause += ", ";
            }
            whereClause += "ID('" + id + "')";
          });
          whereClause += ")";
        }

        whereClause += ")";
      }
    }

    return {
      whereClause: whereClause,
      interactionFilter: interactionFilter
    };
  },

  _replaceOrderParameter: function _replaceOrderParameter_EntityQuery(propertyName, order){
    var properties = this._queue.item.properties;
    var property = dojo.filter(properties, function(elem){
      return (elem.property == propertyName && elem.type == "priority");
    });
    var result = order;
    if (property.length > 0) {
      var prios = this._prioritiesConfig;
      if (prios) {
        // get the settings for high priority
        var high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
        var low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
        // specify the sort order depending on the high and low range values
        if (typeof high == "number" && typeof low == "number") {
          if (high < low) {
            if (order == "ASC") {
              // ascending means high values last
              result = "DESC";
            }
            else {
              result = "ASC";
            }
          }
        }
      }
    }
    return result;
  },

  _createFilterWhereClause: function _createFilterWhereClause_EntityQuery(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var whereClause = "";
    if (this._customFilter) {
      // if the filter has a user defined where clause
      if (this._customFilter.whereClause) {
        whereClause = "(" + this._customFilter.whereClause + ")";
      }
      // go over all filter criterias
      if (this._customFilter.criteria && this._customFilter.criteria.length > 0) {
        var criterias = this._customFilter.criteria;
        var lastPropertyName = null;
        for ( var k in criterias) {
          // operator only if there are multiple filter criteria
          var f = criterias[k];
          if (k > 0) {
            if (lastPropertyName == f.property) {
              whereClause = whereClause + " OR ";
            }
            else {
              whereClause = whereClause + ") AND (";
            }
          }
          else {
            // if there was a fixed whereclause already
            if (whereClause !== "") {
              whereClause += " AND ";
            }
            // otherwise just start the whereclause here
            whereClause = whereClause + "(";
          }
          lastPropertyName = f.property;
          var type = f.type;
          var dateUntil;
          var date;
          var date24;
          switch (type) {
            case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_ID_CONFIG :
              //case "id" :
              var v, domain;

              if (f.properties) {
                // ID filtering with multiple properties
                if (this._systems.isApiTypeFederated()) {
                  // we need to prefix the ID in Federation
                  domain = f.domain || this._queue.domain;
                  if (f.systemID) {
                    // there is already a systemID (usually coming from a Filter event)
                    var prefixedID = this.restHandler.prefixID(f.value, domain, f.systemID);
                    whereClause = whereClause + " " + f.property + "=ID('" + prefixedID + "')";
                    for (v = 0; v < f.properties.length; v++) {
                      whereClause = whereClause + " OR " + f.properties[v] + "=ID('" + prefixedID + "')";
                    }
                  }
                  else {
                    // there is no systemID (usually coming from an adhoc filter), we need to search in all systems of the matching type
                    var systemIDs = this._systems.getSystems(domain);

                    // ID can only be WLE or WPS because it is an equal compare no wild card search
                    var isWleID =
                      com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE.test(f.value) ||
                        com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE.test(f.value);
                    var isWpsID = com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS.test(f.value);

                    var prefixedIDs = [];

                    for (v = 0; v < systemIDs.length; v++) {
                      if ((isWleID && this._systems.isSystemWle(systemIDs[v])) ||
                        (isWpsID && this._systems.isSystemWps(systemIDs[v]))) {
                        prefixedIDs.push(this.restHandler.prefixID(f.value, domain, systemIDs[v]));
                      }
                    }
                    // if a filter event includes a wrong ID then we can not set the prefix 
                    if (prefixedIDs.length === 0) {
                      throw new Error("prefixedIDs.length==0");
                    }

                    whereClause = whereClause + " " + f.property + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                    for (v = 0; v < f.properties.length; v++) {
                      whereClause =
                        whereClause + " OR " + f.properties[v] + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                    }
                  }
                }
                else {
                  // in non-federation we do not need to prefix the ID
                  whereClause = whereClause + " " + f.property + "=ID('" + f.value + "')";
                  for (v = 0; v < f.properties.length; v++) {
                    whereClause = whereClause + " OR " + f.properties[v] + "=ID('" + f.value + "')";
                  }
                }
              }
              else {
                // ID filtering with one property
                if (this._systems.isApiTypeFederated()) {
                  // we need to prefix the ID in Federation
                  domain = f.domain || this._queue.domain;
                  if (f.systemID) {
                    // there is already a systemID (usually coming from a Filter event)
                    prefixedID = this.restHandler.prefixID(f.value, domain, f.systemID);
                    whereClause = whereClause + " " + f.property + "=ID('" + prefixedID + "')";
                  }
                  else {
                    // there is no systemID (usually coming from an adhoc filter), we need to search in all systems
                    systemIDs = this._systems.getSystems(domain);

                    isWleID =
                      com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE.test(f.value) ||
                        com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE.test(f.value);
                    isWpsID = com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS.test(f.value);

                    prefixedIDs = [];

                    for (v = 0; v < systemIDs.length; v++) {
                      if ((isWleID && this._systems.isSystemWle(systemIDs[v])) ||
                        (isWpsID && this._systems.isSystemWps(systemIDs[v]))) {
                        prefixedIDs.push(this.restHandler.prefixID(f.value, domain, systemIDs[v]));
                      }
                    }
                    // if a filter event includes a wrong ID then we can not set the prefix 
                    if (prefixedIDs.length === 0) {
                      throw new Error("prefixedIDs.length==0");
                    }

                    whereClause = whereClause + " " + f.property + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                  }
                }
                else {
                  // in non-federation we do not need to prefix the ID
                  whereClause = whereClause + " " + f.property + "=ID('" + f.value + "')";
                }
              }

              break;
            case "text" :
            case "user" :
              whereClause =
                whereClause + " " + f.property + " LIKE '%" +
                  com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(f.value) + "%'";
              break;
            case "date" :
              date = new Date(f.value);
              date.setHours(0, 0, 0);
              date24 = dojo.date.add(date, "day", 1);
              date24.setHours(0, 0, 0);
              whereClause =
                whereClause + " " + f.property + " >= TS('" +
                  com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "') AND " + f.property + " < TS('" +
                  com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              break;
            case "dateRange" :
              date = null;
              if (f.fromValue && !f.untilValue) {
                date = new Date(f.fromValue);
                date.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (!f.fromValue && f.untilValue) {
                date = new Date(f.untilValue);
                date = dojo.date.add(date, "day", 1);
                date.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " < TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (f.fromValue && f.untilValue) {
                var dateFrom = new Date(f.fromValue);
                dateUntil = new Date(f.untilValue);

                if (dojo.date.compare(dateFrom, dateUntil) > 0) {
                  var temp = f.fromValue;
                  f.fromValue = f.untilValue;
                  f.untilValue = temp;
                  dateFrom = new Date(f.fromValue);
                  dateUntil = new Date(f.untilValue);
                }

                dateFrom.setHours(0, 0, 0);
                dateUntil = dojo.date.add(dateUntil, "day", 1);
                dateUntil.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(dateFrom) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(dateUntil) + "')";
              }
              break;
            case "simpleDate" :
              var value = f.value;
              date = new Date();
              if (value == "UntilNow") {
                whereClause =
                  whereClause + " " + f.property + " <= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (value == "Yesterday") {
                var date0 = dojo.date.add(date, "day", -1);
                date0.setHours(0, 0, 0);
                date24 = date;
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date0) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "Today") {
                date.setHours(0, 0, 0);
                date24 = dojo.date.add(date, "day", 1);
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "Tomorrow") {
                date0 = dojo.date.add(date, "day", +1);
                date0.setHours(0, 0, 0);
                date24 = dojo.date.add(date0, "day", +1);
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date0) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "FromNow") {
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              break;
            case "boolean" :
              whereClause = whereClause + " " + f.property + " = " + f.value;
              break;
            case "number" :
            case "float" :
            case "numericPriority" :
              var operatorMap = {
                equal: "=",
                less: "<",
                lessEqual: "<=",
                greater: ">",
                greaterEqual: ">="
              };
              whereClause = whereClause + " " + f.property + " " + operatorMap[f.operator] + " " + f.value;
              break;
            case "priority" :
              var prios = this._prioritiesConfig;
              var high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
              var low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
              if (f.value == "High") {
                if (high < low) {
                  whereClause = whereClause + " " + f.property + " <= " + high;
                }
                else {
                  whereClause = whereClause + " " + f.property + " >= " + high;
                }
              }
              else if (f.value == "Medium") {
                if (high < low) {
                  whereClause = whereClause + " (" + f.property + " > " + high + " AND " + f.property + " < " + low + ") ";
                }
                else {
                  whereClause = whereClause + " (" + f.property + " > " + low + " AND " + f.property + " < " + high + ") ";
                }
              }
              else {
                if (high < low) {
                  whereClause = whereClause + " " + f.property + " >= " + low;
                }
                else {
                  whereClause = whereClause + " " + f.property + " <= " + low;
                }
              }
              break;
            case "defaultPriority" :
              var range = com.ibm.bpm.social.widgets.task.util.Constants.propertyValueMapper(f.value);
              if (range) {
                if (!range.low && range.high) {
                  whereClause = whereClause + " " + f.property + " < " + range.high;
                }
                else if (range.low && range.high) {
                  whereClause =
                    whereClause + " (" + f.property + " < " + range.high + " AND " + f.property + " >= " + range.low + ") ";
                }
                else if (range.low && !range.high) {
                  whereClause = whereClause + " " + f.property + " >= " + range.low;
                }
              }
              break;
            case "TASK.STATE" :
              if (f.value == "STATE_RUNNING") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "STATE_RUNNING" + " OR " + f.property + "=" + "STATE_CLAIMED" +
                    ") ";
              }
              else if (f.value == "STATE_FAILED") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "STATE_FAILED" + " OR " + f.property + "=" + "STATE_EXPIRED" +
                    ") ";
              }
              else {
                whereClause = whereClause + " " + f.property + "=" + f.value;
              }
              break;
            case "PROCESS_INSTANCE.STATE" :
              // Unsupported values in WLE and mixed ["STATE_TERMINATING", "STATE_COMPENSATED", "STATE_COMPENSATING", "STATE_FAILING", "STATE_COMPENSATION_FAILED"]
              var isWpsOnly = false;
              if (this._systems) {
                isWpsOnly = this._systems.isWpsOnly(this._queue.domain);
              }
              if (f.value == "STATE_FAILED") {
                whereClause += " (";
                whereClause += f.property + "=" + "STATE_FAILED";
                if (isWpsOnly) {
                  whereClause += " OR " + f.property + "=" + "STATE_FAILING";
                }
                whereClause += " OR " + f.property + "=" + "STATE_TERMINATED";
                if (isWpsOnly) {
                  whereClause += " OR " + f.property + "=" + "STATE_TERMINATING";
                }
                whereClause += ") ";
              }
              else if (f.value == "STATE_COMPENSATING") {
                if (isWpsOnly) {
                  whereClause =
                    whereClause + " (" + f.property + "=" + "STATE_COMPENSATING" + " OR " + f.property + "=" +
                      "STATE_COMPENSATED" + ") ";
                }
              }
              else {
                if (isWpsOnly || f.value != "STATE_COMPENSATION_FAILED") {
                  whereClause = whereClause + " " + f.property + "=" + f.value;
                }
              }
              break;
            case "ESCALATION.ACTIVATION_STATE" :
              if (f.value == "ACTIVATION_STATE_RUNNING") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "ACTIVATION_STATE_RUNNING" + " OR " + f.property + "=" +
                    "ACTIVATION_STATE_CLAIMED" + ") ";
              }
              else {
                whereClause = whereClause + " " + f.property + "=" + f.value;
              }
              break;
            case "usergroup" :
              var input = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(f.value);
              whereClause =
                whereClause + " (" + f.property + " LIKE 'cn=%" + input + "%,o=%') OR (" + f.property +
                  " NOT LIKE 'cn=%' AND " + f.property + " LIKE '%" + input + "%')";
              break;
            default:
              whereClause = whereClause + " " + f.property + "=" + f.value;
              break;
          }
        }
        whereClause = whereClause + ")";
      }
    }
    com.ibm.bpm.social.util.common.Log.log(2,
      "com.ibm.bpm.social.widgets.task.util.EntityQuery._createFilterWhereClause whereclause :" + whereClause);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return whereClause;
  },

  _replaceWhereParameters: function _replaceWhereParameters_EntityQuery(whereClause, propertiesMap){
    // replace date values if necessary
    if (!whereClause) {
      return "";
    }
    var todayDate = new Date();
    var date;

    var today = null;
    if (whereClause.indexOf("{today}") >= 0) {
      today = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(todayDate);
      propertiesMap["today"] = today;
    }
    // create the iso date string for yesterday date
    // yyyy-mm-ddT23:59:59+GMThh:mm
    var yesterday = null;
    if (whereClause.indexOf("{yesterday}") >= 0) {
      date = dojo.date.add(todayDate, "day", -1);
      date.setHours(23, 59, 59);
      yesterday = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date);
      propertiesMap["yesterday"] = yesterday;
    }
    // create the iso date string for tomorrows date
    // yyyy-mm-ddT00:00:00+GMThh:mm
    var tomorrow = null;
    if (whereClause.indexOf("{tomorrow}") >= 0) {
      date = dojo.date.add(todayDate, "day", 1);
      date.setHours(0, 0, 0);
      tomorrow = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date);
      propertiesMap["tomorrow"] = tomorrow;
    }
    // get the high prio value
    var high = null;
    var highCompare = null;
    var low = null;
    if (whereClause.indexOf("{high}") >= 0) {
      var prios = this._prioritiesConfig;
      highCompare = "<=";
      high = 0;
      if (prios) {
        // get the settings for high priority
        high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
        low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
        // specify the compare sign depending on the high and low range
        if (typeof high == "number" && typeof low == "number") {
          if (high > low) {
            // all values greate equal high have high priority
            highCompare = ">=";
          }
        }
        propertiesMap["high"] = high;
        propertiesMap["highCompare"] = highCompare;
      }
    }
    var user = null;
    if (whereClause.indexOf("{user}") >= 0) {
      // Escape the userId to support user id's with special characters
      user = window.encodeURIComponent(this._userId);
      propertiesMap["user"] = user;
    }
    // insert the variables in the where clause
    //        whereClause = dojo.string.substitute(whereClause, {
    //            today: today,
    //            tomorrow: tomorrow,
    //            yesterday: yesterday,
    //            user: user,
    //            high: high,
    //            highCompare: highCompare
    //        });

    whereClause = dojo.string.substitute(whereClause, propertiesMap);
    return whereClause;
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Action"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Action"] = true;
// Action.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.util.Action");


dojo.declare("com.ibm.bpm.social.widgets.task.util.Action", null, {
  /**
   * Id of the action to be referred to
   */
  id: "",

  /**
   * message key for action label
   */
  titleKey: "",

  /**
   * Action label for current language, set by the user or defined through titleKey
   */
  displayName: "",

  /**
   * message key for action tooltip
   */
  tooltip: "",

  /**
   * Action tooltip. Set by the user or specified through the tooltip message key.
   */
  description: "",

  /**
   * Specifies, if the actions is enabled at all. It is not displayed otherwise.
   */
  enabled: true,

  /**
   * CSS class to use for the enabled action
   */
  iconClass: "",

  /**
   * CSS class to use for the enabled action
   */
  hoverClass: "",

  /**
   * CSS class to use for the enabled action
   */
  disabledClass: "",


// currently not used:   
//  /**
//   * Array describing if the action is not supported on all back ends <br>
//   * may contain strings of com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_*
//   */
//  unsupported_api: [],
//
//  /**
//   * Selection mode for an action. When is an action activated. <br>
//   * One of : multiple, single, singleNone, always, none
//   */
//  selectionMode: "",
//
//  /**
//   * Specifies, if the action is custom or predefined.
//   */
//  custom: false

  constructor: function constructor_Action(obj){
    if (obj) {
      dojo.safeMixin(this, obj);
    }
  }
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TasksListController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.list.TasksListController"] = true;
// TasksListController.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TasksListController");



























dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.list.TasksListController",
    [dijit._Widget,
     dijit._Templated],
    {

      /**
       * Need 2 containers for calculation of list height. Outer container (class="bpm-task-list-view")
       * contains the paddings in css. Inner container ("listView") is needed to calculate the bottom
       * margin/padding
       */
      templateString: '\
        <div class="bpm-task-list-view">\
          <div dojoAttachPoint="listView">\
              <div class="bpm-task-list-header" dojoAttachPoint="listHeader">\
                <div dojoAttachPoint="listTitle" id="${id}_heading" class="bpm-task-list-header-title" ></div>\
                <div dojoAttachPoint="listRefresh" class="bpm-task-list-header-refresh" style="display:none;" onclick="com.ibm.bpm.social.portal.PageHandler.getInstance().selectWorkTab();" href="javascript:void(0);"></div>\
                <div dojoAttachPoint="listSearch" class="bpm-task-list-header-search"></div>\
				<div class="bpm-task-list-header-break"></div>\
                  <div dojoAttachPoint="collapseExpandAllButton" class="bpm-social-show-hide-button bpm-social-show-all" tabindex="0">\
                    <div dojoAttachPoint="collapseExpandAllButtonText" class="bpm-social-show-hide-button-text">&#9660;</div>\
                  </div>\
              </div>\
              <div dojoAttachPoint="listAction" class="bpm-task-list-actions">\
                <div class="bpm-task-list-title" dojoAttachPoint="listActionBarTitle" style="display:none;"></div>\
                <div class="bpm-task-list-action-right" dojoAttachPoint="listActionRight" role="tablist">\
                  <div dojoAttachPoint="collapseExpandAllButtonOldLocation"></div>\
                </div>\
              </div>\
              <div dojoAttachPoint="list" role="presentation">\
                  <div dojoAttachPoint="listMessage" class="bpm-task-list-content bpm-task-list-message-hide" ></div>\
                  <div dojoAttachPoint="listContent" class="bpm-task-list-content" tabindex="-1" >\
                    <div dojoAttachPoint="itemList"></div>\
                  </div>\
              </div>\
              <div dojoAttachPoint="pagination"></div>\
           </div>\
        </div>',

      widgetsInTemplate: true,

      listWidget: null,

      selectedFilterConfig: null,

      _isCEButtonInCollapsedState: true,

      _isLastCategoryInCollapsedState: false,

      _numberOfCollapsedCategories: 0,

      _moreItemsAvailable: true,

      _deferredForActions: null,

      _deferredForQuery: null,

      _totalCountCache: null,

      _totalCountLimitExceeded: false,

      _restHandler: null,

      _errorHandler: null,

      _refreshHandler: null,

      _subscribedTaskCompletion: null,

      _subscribedInstanceView: null,

      _subscribedShowDialog: null,

      _subscribedHideDialog: null,

      _scrollTop: 0,

      _startPageSettings: null,

      _searchFilterApplied: false,

      _searchText: null,

      /**
       * 
       * @type {void}
       */
      constructor: function constructor_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController");

        // keep track of created widgets
        this._childWidgets = [];

        // flag whether last category is collapsed, which requires special behavior
        this._isLastCategoryInCollapsedState = false;

        // flag whether any visible category is collapsed, which requires special behavior
        this._numberOfCollapsedCategories = 0;
        // make sure the count is set to null initially, as non-null will use the cached version  
        this._totalCountCache = null;
        this._totalCountLimitExceeded = false;

        // flag whether more items than requested are available (we do fetch one more)
        this._moreItemsAvailable = true;

        // object containing to dojo.subscribe handle if we are waiting for claim dialog response
        this._waitingForClaimDialogChannel = null;

        // track settings for the start page 
        this._startPageSettings = new com.ibm.bpm.social.util.common.PageSettings();

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      destroyChildren: function destroyChildren_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // destroy all children
        while (this._childWidgets.length > 0) {
          var widget = this._childWidgets.pop();
          if (widget) {
            widget.destroy();
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      destroy: function destroy_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        for ( var i = 0; i < this._connectHandlers.length; i++) {
          try {
            dojo.disconnect(this._connectHandlers[i]);
          }
          catch (e) {
            com.ibm.bpm.social.util.common.Log.logError(2, "destroy_TasksListController", e);
          }

        }
        this._connectHandlers = null;

        if (this._confirmClaimDialog) {
          this._confirmClaimDialog.destroy();
        }

        if (this._refreshHandler) {
          this._refreshHandler.destroy();
        }

        this._unSubscribeViewChanges();

        // destroy child widgets
        this.destroyChildren();


        com.ibm.bpm.social.util.common.Log.logExit(2);
        this.inherited(arguments);
      },

      /**
       * postCreate
       */
      postCreate: function postCreate_TasksListController(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        //com.ibm.bpm.social.util.common.Util.loadStylesheet("css/css_sprite.css");
        // Including the entire Portal.css stylesheet causes problems for the page layout. (Assertion by Portal team)
        // Styles should be copied to other stylesheets on an as needed basis.
        // com.ibm.bpm.social.util.common.Util.loadStylesheet("css/Portal.css"); 

        this.containerNode = this.domNode;

        // TODO find a place for creation with params, e.g. endpoint
        this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
        this._processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
        this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
        this._connectHandlers = new Array();

        // connect onresize    
        this._connectHandlers.push(this.connect(window, "onresize", "_onResize"));

        //
        // collapse/expand all button
        this.collapseExpandAllButton.title = this._nlsResources["task-list-expand-all"];

        var self = this;
        // onclick 
        this.connect(this.collapseExpandAllButton, "ondijitclick", function(){
          if (self._isCEButtonInCollapsedState) {
            self._showCollapseAllButton();
            self.listWidget.expandAll();
          }
          else {
            self._showExpandAllButton();
            self._collapsingAll = true;
            self.listWidget.collapseAll();
            self._collapsingAll = false;
            self._onResize();
          }
        });
        // hover
        this.connect(this.collapseExpandAllButton, "onmouseover", function(){
          if (self._isCEButtonInCollapsedState) {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-show-all");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-show-all-hover");
          }
          else {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-hide-all");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-hide-all-hover");
          }
        });
        // mouse out
        this.connect(this.collapseExpandAllButton, "onmouseout", function(){
          if (self._isCEButtonInCollapsedState) {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-show-all-hover");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-show-all");
          }
          else {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-hide-all-hover");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-hide-all");
          }
        });
		setTimeout(function() {
			if(self !== undefined)
				self._onResize();
		}, 300);
      },

      _createTableConfigObject: function _createTableConfigObject_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var activeQueue = this.config.queue;
        this.config.priorities = {
          type: "defaultPriority",
          showImage: false
        };
        var formatterFactory = new com.ibm.bpm.social.widgets.task.util.PropertyFormatter({
          config: this.config,
          queueConfig: activeQueue,
          widgetId: this.id
        });

        var displayedProperties = [];
        var property = null;


        // we need to map the properties received from the backend to map to the table config object
        for ( var i = 0; i < activeQueue.item.properties.length; i++) {
          property = activeQueue.item.properties[i];
          if (property.displayed) {
            var linkToAction = null;
            if (property.property == "TAD_DISPLAY_NAME") {
              linkToAction = "runTask";
            }
            else if (property.property == "PI_NAME") { // "PI_DISPLAY_NAME" is filtered out in queue._setAtrributes 
              linkToAction = "viewInstance";
            }

            // choose formatter
            var formatterType = null;
            if (property.property == "ASSIGNED_TO_ROLE") {
              formatterType = "ASSIGNED_TO_ROLE";
            }
            else if (property.property == "ASSIGNED_TO_PARTICIPANT") {
              formatterType = "ASSIGNED_TO_ROLE";
            }
            else if (property.property == "PRIORITY") {
              formatterType = "defaultPriority";
            }
            else {
              formatterType = property.type;
            }
            var formatter = formatterFactory.getFormatter(formatterType, false, property.title);

            displayedProperties.push({
              id: property.property,
              label: property.title,
              formatter: formatter,
              description: property.description,
              inline: property.inline,
              labelDisplayed: property.labelDisplayed,
              isSort: property.isSort,
              isSortable: property.isSortable,
              isFilterable: property.isFilterable,
              sortDir: property.sortDir,
              linkToAction: linkToAction,
              type: property.type
            });
          }
        }

        var config = { // table config
          displayedProperties: displayedProperties,
          labelDisplayed: true,
          supportedDndTypes: null,
          publishedDndTypes: null,
          parentProperty: null, // TODO ASZ this.config.parentProperty
          noItemsFoundMessage: this._nlsResources["task-list-no-tasks"],
          activeGroupByProperty: null, // TODO ASZ  this.config.activeGroupByProperty,
          groupFormatter: null,
          isSelectable: false,
          fitToContent: false,
          isFilterable: false,
          sortProperties: this.selectedFilterConfig.sortProperties
        };

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return config;
      },

      /**
       * 
       * @type {void}
       */
      _createInlineActions: function _createInlineActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var actions = [];

        // Note: Order matters (defines the order in the actions table)

        var runTask = new com.ibm.bpm.social.widgets.task.util.Action({
          id: "runTask",
          titleKey: this._nlsResources["task_action_run_task"],
          tooltip: this._nlsResources["task_action_run_task_description"],
          enabled: false,
          hoverClass: "bpm-social-run-hover",
          iconClass: "bpm-social-run",
          disabledClass: "bpm-social-run-disabled action_disabled"
        });
        actions.push(runTask);

        var viewInstance = new com.ibm.bpm.social.widgets.task.util.Action({
          id: "viewInstance",
          titleKey: this._nlsResources["task_action_view_instance_details"],
          tooltip: this._nlsResources["task_action_view_instance_details_description"],
          enabled: false,
          hoverClass: "bpm-social-details-hover",
          iconClass: "bpm-social-details",
          disabledClass: "bpm-social-details-disabled action_disabled"
        });

        actions.push(viewInstance);

        if (this.config.queue.entityTypeName != "PROCESS_INSTANCE") {
          var modifyTask = new com.ibm.bpm.social.widgets.task.util.Action({
            // this is a meta-action that includes the actions Assign, Reassign back to group, change due date, change priority 
            id: "modifyTask",
            titleKey: this._nlsResources["task_menu_modify_task"],
            tooltip: this._nlsResources["task_dialog_modifyTask_description"],
            enabled: false,
            hoverClass: "bpm-social-edit-hover",
            iconClass: "bpm-social-edit",
            disabledClass: "bpm-social-edit-disabled action_disabled"
          });
          actions.push(modifyTask);
        }
        else {
          var modifyInstance = new com.ibm.bpm.social.widgets.task.util.Action({
            id: "modifyInstance",
            titleKey: this._nlsResources["process_menu_modify_instance"],
            tooltip: this._nlsResources["process_menu_modify_instance_description"],
            enabled: false,
            hoverClass: "bpm-social-edit-hover",
            iconClass: "bpm-social-edit",
            disabledClass: "bpm-social-edit-disabled action_disabled"
          });
          actions.push(modifyInstance);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return actions;
      },

      /**
       * Set the Controller config
       * 
       * @param {com.ibm.bpm.social.widgets.task.config.ControllerConfig} config
       * 
       */
      setConfig: function setConfig_TasksListController(config){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.config = config;
        this._isDirty = true;

        // initialize refresh handler
        if (this._refreshHandler === null) {
          var deltaRefresh = true;
          if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "table") {
            deltaRefresh = false;
          }
          this._refreshHandler =
            new com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler(dojo.hitch(this,
              this._refreshHandlerFullRefresh),
              dojo.hitch(this, this._setModel),
              dojo.hitch(this, this._updateModel),
              this.id,
              deltaRefresh);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * calculate the height for the list content. return list content height in px
       */
      _getListContentHeight: function _getListContentHeight_TasksListController(){
        var viewport = dojo.window.getBox();
        var posNode = dojo.position(this.domNode);
        var posListView = dojo.position(this.listView);
        var bottom = posNode.y + posNode.h - posListView.y - posListView.h;
        var posList = dojo.position(this.list);
        var posPagination = dojo.position(this.pagination);
        var result = viewport.h - posList.y - posPagination.h - bottom;
        // no min height
        if (result < 0) {
          result = 0;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "height: " + result + " px");
        return result;
      },

      /**
       * return list content width in px
       */
      _getListContentWidth: function _getListContentWidth_TasksListController(){
        // TODO clientWidth is too small if we are actually scrolling, 
        // however using offsetWidth results in the table having outer scroll bars
        var result = this.listContent.clientWidth;

        // container size will be limited to 900, so this should not happen
        if (result < 400) {
          result = 400;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "width:" + result + " px");
        return result;
      },

      /**
       * create the visible parts
       */

      createContent: function createContent_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.config) {

          var self = this;

          // delete old content

          if (this._childWidgets.length > 0) {
            this.destroyChildren();
          }

          // create new content

          // create filter area
          if (this.config && this.config.filterConfigs && this.config.filterConfigs.length > 1) {
            dojo.forEach(this.config.filterConfigs, function(filter, index){
              var filterSpan = document.createElement("div");
              dijit.setWaiRole(filterSpan, "tab");
              filterSpan.tabIndex = "0";
              dojo.style(filterSpan, "display", "inline-block");
              dojo.addClass(filterSpan, "bpm-task-list-filter");
              filterSpan.innerHTML = self._nlsResources[filter.titleKey];
              self.listActionRight.insertBefore(filterSpan, self.collapseExpandAllButtonOldLocation);
              self[filter.id + "_filter"] = filterSpan;
              self.connect(filterSpan, "ondijitclick", function(e){
                if (filter.id !== self.config.selectedFilter) {
                  self._doFilter(filter);
                }
              });

              if (index > 0) {
                var sepSpan = document.createElement("div");
                dojo.style(sepSpan, "display", "inline-block");
                sepSpan.innerHTML = "|";
                dijit.setWaiState(sepSpan, "hidden", "true");
                self.listActionRight.insertBefore(sepSpan, filterSpan);
              }
            });
          }

          var selectedFilter = this.config.selectedFilter;
          if (selectedFilter) {
            var selectedFilterFindings = dojo.filter(this.config.filterConfigs, function(filter){
              return filter.id === selectedFilter;
            });
            if (selectedFilterFindings.length > 0) {
              this._setFilterConfig(selectedFilterFindings[0]);
              if (this.config.filterConfigs.length > 1) {
                this._decorateSelectedFilter(selectedFilterFindings[0]);
              }
            }

            // set title
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
              this.listTitle.innerHTML = "";
              this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
            }
            else {
              if (this.config.viewConfig.titleKey) {
                this.listTitle.innerHTML = this._nlsResources[this.config.viewConfig.titleKey];
              }
              else {
	              this.listTitle.innerHTML = "";
    	          this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
              }
            }

            // If user is using Safari on iPad, display refresh icon for work view
            if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) {
              dojo.style(this.listRefresh, "display", "block");
            }

            // determine categories
            var categories = null;
            if (this.selectedFilterConfig && this.selectedFilterConfig.categories) {
              if (typeof this.selectedFilterConfig.categories == "function") {
                this.selectedFilterConfig.categories();
              }
              categories = this.selectedFilterConfig.categories;
            }

            // create query
            if (!this._query) {
              var restHandler = com.ibm.bpm.social.util.common.QueryRestHandler.getInstance();

              this._query = new com.ibm.bpm.social.widgets.task.util.EntityQuery({
                restHandler: restHandler,
                //          D2DFilter: parameters.D2DFilter,             
                widgetId: this.config.id,
                systems: this._restHandler.getSystems(),
                userId: com.ibm.bpm.social.util.common.Util.getCurrentUser().userName
              });
            }

            // check if collapse/expand is possible
            if (!this.config.viewConfig.allowCollapseExpand) {
              this.listActionRight.style.display = "none"; // no expand/collapse action on table views
            }

            // create the view widget
            if (this.config.viewConfig.viewType == "table") {
              // table has its own scroll bars
              this.listContent.style.overflow = "hidden";

              dijit.setWaiRole(this.listActionRight, "presentation");
              this.listWidget = new com.ibm.bpm.social.widgets.task.list.Table({}, this.itemList);
              this._isDirty = true;

              this._connectHandlers.push(this.connect(this.listWidget, "onActionRequested", "_onActionRequested"));
              this._connectHandlers.push(this.connect(this.listWidget, "onSort", "_onSort"));

            }
            else { //  default is List view 
              this.listWidget = new com.ibm.bpm.social.widgets.task.list.List({
                config: {
                  controller: this,
                  filter: this.selectedFilterConfig,
                  type: this.config.type
                }
              }, this.itemList);

              this._connectHandlers.push(this.connect(this.listWidget, "onCategoryCollapsed", "_onCategoryCollapsed"));
              this._connectHandlers.push(this.connect(this.listWidget, "onCategoryExpanded", "_onCategoryExpanded"));
              this._connectHandlers.push(this.connect(this.listWidget, "onWorkOn", "_workOn"));
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskDetailsNeeded", "_getBulkTaskDetails"));
              this._connectHandlers.push(this.connect(this.listWidget, "onRefreshNeeded", "_refresh"));
              this._connectHandlers.push(this.connect(this.listWidget, "onDecreaseLocalCount",
                function(){
                  if (this._paginationWidget.localCount > 1) {
                    this._paginationWidget.setLocalCount(this._paginationWidget.localCount - 1,
                      this._totalCountLimitExceeded);
                  }
                  else {
                    this._refresh(true);
                  }
                }));
              this._connectHandlers.push(this.connect(this.listWidget, "onDecreaseGlobalCount",
                function(){
                  this._totalCountCache--;
                  this._paginationWidget.setGlobalCount(this._paginationWidget.globalCount - 1,
                    this._totalCountLimitExceeded);
                }));
              this._connectHandlers.push(this.connect(this.listWidget, "onIncreaseLocalCount", function(){
                if (this._paginationWidget.localCount === 0) {
                  this._removeMessage();
                  dojo.style(this.collapseExpandAllButton, "display", "block");
                }
                this._paginationWidget.setLocalCount(this._paginationWidget.localCount + 1, this._totalCountLimitExceeded);
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onIncreaseGlobalCount",
                function(){
                  this._totalCountCache++;
                  this._paginationWidget.setGlobalCount(this._paginationWidget.globalCount + 1,
                    this._totalCountLimitExceeded);
                }));

              // no refresh on notifications while a dialog is open or any task row is expanded
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskRowExpanded", function(item){
                if (this.selectedFilterConfig.id === "openTasks") {
                  var idString = item.systemID ? item.systemID + "_" + item["TASK.TKIID"] : item["TASK.TKIID"];
                  this._refreshHandler.suspendFullRefresh("taskRowExpanded_" + idString);
                }
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskRowCollapsed", function(item){
                if (!this._collapsingAll) {
                  this._onResize();
                }
                if (this.selectedFilterConfig.id === "openTasks") {
                  var idString = item.systemID ? item.systemID + "_" + item["TASK.TKIID"] : item["TASK.TKIID"];
                  this._refreshHandler.resumeFullRefresh("taskRowExpanded_" + idString);
                }
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onFullRefreshNeeded", function(){
                this._refreshHandler.doFullRefresh();
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onFocusNeeded", function(){
                com.ibm.bpm.social.util.common.Log.log(3, "Focus active task list tab header");
                this.onFocusNeeded();
              }));
            }

            this._connectHandlers.push(dojo.connect(this.listWidget, "onDetailsRequested", this, function(item){
              var taskId = item["TASK.TKIID"];
              self.listWidget.showMessage(self._nlsResources["task-row-details-loading"], taskId);

              // Fetch data from the server
              self._restHandler.getTask(taskId, function(task){
                self._setTaskDetails(task);
                self._onResize();
              }, function(data, ioArgs){
                // task is assigned to somebody else or deleted
                if (ioArgs.xhr.status == 401 || ioArgs.xhr.status == 404) {
                  self.listWidget.showMessage(self._nlsResources["task-row-details-error-not-available"], taskId);
                }
                else {
                  self.listWidget.showMessage(data.message, taskId);
                }
              }, true, null, item.systemID);
            }));

            this._connectHandlers.push(this.connect(this.listWidget, "onShowCollapseAllButton", function(){
              if (self._isCEButtonInCollapsedState) {
                self._showCollapseAllButton();
              }
            }));

            // pagination bar
            this._paginationWidget = new com.ibm.bpm.social.widgets.task.list.PaginationBar({}, this.pagination);
            this.pagination = this._paginationWidget.domNode;
            this._connectHandlers.push(this.connect(this._paginationWidget, "onShowMore", "_refresh"));

            // Search bar          
            this._searchWidget = new com.ibm.bpm.social.widgets.task.list.SearchBar({}, this.listSearch);
            if (this._searchText) {
              this._searchWidget.setSearchText(this._searchText);
            }
            //this.listSearch = this._searchWidget.domNode;
            this.connect(this._searchWidget, "onSubmit", "_search");
            this.connect(this._searchWidget, "onClear", "_clearSearch");
            this._setSearch();
            // TODO filter

          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Handler for sort requests (currently only from table)
       * 
       * @param {String} property column / property used as sort criteria
       * @param {String} direction one of {ASC|DESC}
       * @type {void}
       */
      _onSort: function _onSort_TasksListController(property, direction){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var newSortProp = {
          isSort: true,
          property: property,
          sortDir: direction
        };

        // ensure no doubble sort
        this.selectedFilterConfig.sortProperties = dojo.filter(this.selectedFilterConfig.sortProperties, function(sortProp){
          return sortProp.property != newSortProp.property;
        });

        this.selectedFilterConfig.sortProperties.unshift(newSortProp);


        // remove older sort properties ( 2 are allowed )
        while (this.selectedFilterConfig.sortProperties.length > 2) {
          this.selectedFilterConfig.sortProperties.pop();
        }


        this._setFilterConfig(this.selectedFilterConfig);
        this.listWidget.setSortProperties(this.selectedFilterConfig.sortProperties);

        this.refresh();


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Generic handler on all actions send out from the listWidget
       * 
       * Note: this is only used for the table
       * 
       * @param {String} actionId
       * @param {Object} data a object specific to the action, e.g. a task or other item
       * @type {void}
       */
      _onActionRequested: function _onActionRequested_TasksListController(actionId, data){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        if (actionId) {
          if (actionId == "runTask") {
            this._workOn(data);
          }
          else if (actionId == "modifyTask") {

            // Fetch complete task from the server since the saved search may not contain all necessary fields
            var task = null;
            this._restHandler.getTask(data["TASK.TKIID"], function(result){
              task = result;
            }, function(response, ioArgs){
              var name = data.TAD_DISPLAY_NAME || data.NAME || data["TASK.TKIID"];
              var statusCodeMap = {
                401: {
                  key: "KEY_TaskModifyNotAuthorized",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                },
                404: {
                  key: "KEY_UpdateNotExistingTask",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                },
                409: {
                  key: "KEY_TaskModifyNotAvailable",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                }
              };
              myself._errorHandler.showError(response, ioArgs, statusCodeMap, null, null);
            }, false, null, data.systemID);
            if (task) {
              task.availableActions = data.availableActions;
              this._modifyTask(task);
            }
            else {
              this.refresh();
            }
          }
          else if (actionId == "modifyInstance") {
            this._modifyInstance(data);
          }
          else if (actionId == "viewInstance") {
            this._viewInstance(data);
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      _modifyTask: function _modifyTask_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!this._modifyTaskHandler) {
          this._modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler({
            task: item,
            availableActions: item.availableActions
          });
          this._childWidgets.push(this._modifyTaskHandler);
          this._connectHandlers.push(this.connect(this._modifyTaskHandler, "onTaskModified", this.refresh));
        }
        else {
          this._modifyTaskHandler.init({
            task: item,
            availableActions: item.availableActions
          });
        }

        this._modifyTaskHandler.showDialog();

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
      /**
       * 
       * @type {void}
       */
      _modifyInstance: function _modifyInstance_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        item.piid = item.piid || item["PROCESS_INSTANCE.PIID"];

        var process = null;
        var myself = this;
        this._processRestHandler.getProcess(item.piid, function(responseObject, ioArgs){
          process = responseObject;
        }, function(data, ioArgs){
          com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_modifyInstance_error");
          com.ibm.bpm.social.util.common.Log.logError(2, data, ioArgs);
          myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListData" +
            com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(myself.config.type), null);
          com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_modifyInstance_error");

        }, false, null, item.systemID);

        if (process) {

          this._modifyInstanceDialog = new com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog({
            "instanceId": process.piid,
            "dueDate": process.dueDate,
            "callback": dojo.hitch(this, this._onInstanceModified)
          }, dojo.create("div"));
          this._modifyInstanceDialog.show();

          // Note: ChangeDueDateDialog destroys itself on hide. Just ensure that it is cleaned otherwise 
          this._childWidgets.push(this._modifyInstanceDialog);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * 
       * @param {Object} item an object having a piid set
       * @type {void}
       */
      _viewInstance: function _viewInstance_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (item) {
          item.piid = item.piid || item["PROCESS_INSTANCE.PIID"];
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
            'instanceId': item.piid
          }]);
        }
        else {
          com.ibm.bpm.social.util.common.Log.logError(1, "Error: Received item without piid set.");
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Callback for the ChangeDueDateDialog (_modifyInstanceDialog)
       */
      _onInstanceModified: function _onInstanceModified_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      _onCategoryCollapsed: function _onCategoryCollapsed_TasksListController(categoryId, count){
        // TODO streamline strategy how many more items we fetch
        com.ibm.bpm.social.util.common.Log.log(1, "Category '" + categoryId + "' collapsed, possibly needing " + count +
          " more items.");
        this._numberOfCollapsedCategories++;
        if (this._numberOfCollapsedCategories > 6) {
          com.ibm.bpm.social.util.common.Log.log(1, "Warning: count for collapsed items greater than 6");
          this._numberOfCollapsedCategories = 6;
        }
        this._refresh(true);
      },

      _onCategoryExpanded: function _onCategoryExpanded_TasksListController(categoryId, count){
        com.ibm.bpm.social.util.common.Log.log(1, "Category '" + categoryId + "' expanded, showing " + count +
          " more items.");
        this._numberOfCollapsedCategories--;
        if (this._numberOfCollapsedCategories < 0) {
          com.ibm.bpm.social.util.common.Log.log(1, "Warning: count for collapsed items less than 0");
          this._numberOfCollapsedCategories = 0;
        }
        this._refresh(true);
      },

      refresh: function refresh_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        try {

          // refresh from outside, show loading message instead of previous content
          this._showMessage("task-list-loading");

          this._refresh(false);
        }
        catch (xcp) {
          com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
          throw (xcp);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _refreshHandlerFullRefresh: function _refreshHandlerFullRefresh_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._saveListFocus();
        this._refresh(true);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _refresh: function _refresh_TasksListController(autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "autoRefresh=" + autoRefresh);

        // TODO: set collapse-all/expand-all button to default
        // For this, encapsulate the button stuff as reusable method (currently all done in postCreate)

        // save scroll position
        this._scrollTop = 0;
        if (this.listContent) {
          com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
            this.listContent.scrollHeight + ", scrollTop: " + this.listContent.scrollTop);
          this._scrollTop = this.listContent.scrollTop;
        }

        // load items 
        var size = this._paginationWidget.getPageSize();
        this._refreshItems(true, autoRefresh, size);

        // Reset expandCollapseAll button
        this._showExpandAllButton();

        // indicate that we had a refresh
        this.onRefreshDone();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Send out event to TasksContainer that we did a refresh
       * 
       * @type {void}
       */
      onRefreshDone: function onRefreshDone_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        com.ibm.bpm.social.util.common.Log.logExit(3);
      },

      /*
       * Callback method for API call to fetch items from server.
       * @param {Object} autoRefresh indicates whether this is an auto-refresh-run or not
       */
      _setModel: function _setModel_TasksListController(autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // this method might be called by an async XHR response
        if (this._destroyed) {
          com.ibm.bpm.social.util.common.Log.log(1, "List is already destroyed.");
          return;
        }


        // TODO autorefresh?
        var count = 0;
        var model = this.items;

        // compute whether to show the pagination button or not before calling this._onResize() later on
        if (this._paginationWidget) {
          if (this._moreItemsAvailable === false) {
            // hide Show more button 
            this._paginationWidget.showPaginationButton(false);
          }
          else {
            // show Show more button 
            this._paginationWidget.showPaginationButton(true);
          }
        }

        // special behaviour: when there are categories collapsed, then we will show the collapsed categories
        // with no items on the screen (rather than the "no items found" message)
        if (!model || (model.length === 0 && this._numberOfCollapsedCategories === 0)) {
          com.ibm.bpm.social.util.common.Log.log(1, "No Tasks.");
          var noItemsMessage = null;
          if (this.config.queue.entityTypeName == "TASK") {
            noItemsMessage = "task-list-no-tasks";
          }
          else if (this.config.queue.entityTypeName == "PROCESS_INSTANCE") {
            noItemsMessage = "task-list-no-instances";
          }
          else {
            var e = new Error("Unexpected entityTypeName '" + this.config.queue.entityTypeName + "'");
            com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected entityTypeName '" + this.config.queue.entityTypeName +
              "'", e);
            throw e;
          }

          this._showMessage(noItemsMessage);
          dojo.style(this.collapseExpandAllButton, "display", "none");

          // set list widget to correct state - no items
          this.listWidget.setModel(model);

          // Update pagination bar
          this._paginationWidget.setLocalCount(0);
          this._paginationWidget.setGlobalCount(0);

        }
        else {
          // retrieve available actions async
          this._retrieveAvailableActionsForQueryResult(this.items);

          // count items; thereby ignore the last one if it is the "extra item"
          var effectiveLength = 0;
          var i;
          var removeExtra = false;
          for (i = 0; i < model.length; i++) {
            if (model[i].isExtra !== true) {
              effectiveLength++;
            }
            else {
              removeExtra = true;
            }
          }
          count = effectiveLength;

          // categorize into groups (do this here, the we can reuse maybe for table)

          // update List widget


          if (this.config.viewConfig.viewType == "table") {
            if (this._isDirty) {
              var tableConfig = this._createTableConfigObject();
              this.listWidget.setConfig(tableConfig);
              this._isDirty = false;
            }
            if (model.length > 0 && removeExtra) {
              // table doesn't handle the extra item we fetched for pagination 
              model.pop();
            }
          }


          // set page size first, because this helps determine which sections will be hidden 
          this.listWidget.setPageSize(this._paginationWidget.getPageSize());
          // this is a more expensive operation

          // PERFORMANCE TEST var ts = new Date();
          this.listWidget.setModel(model);
          // PERFORMANCE TEST var te = new Date();
          // PERFORMANCE TEST console.debug("Tasks: " + count + " time: " + dojo.date.difference(te, ts, "millisecond"));

          // Setting counters

          // Note that we do caching for the total count, as this will in many cases not change over time, 
          // thus we avoid to go to the backend for the total count whenever we can.
          // However there may be situations where the total count changes over time (e.g.  by the fact that we operate
          // in a multi-user environment). 
          // As this could lead to an error message like "Showing 43 of approximately 41 results", we will at least
          // avoid this case: thus, the minimal heuristic is
          //   "if the local count is found greater that the global count, increase the global count without asking the backend".
          if (this._totalCountCache && this._totalCountCache < count) {
            // update cache value 
            this._totalCountCache = count;
            // We always request one extra item to decide whether we should render the Show more button.
            // Thus, we could possibly have received more items than our cached totalCount value.
            // In this case, we know that the cached value is too small;
            // to avoid the extra call out to the backend, we show the (known to be wrong) cached value for the total count anyway
            // (at least we have corrected it to be not smaller than what we show)
            // In addition, in this case we switch to the alternate message "showing x of more than y".
            // Flag will be cached as well, and will be reset later when once we should get an accurate count again.
            // Note: the count will be accurate only when all categories are expanded.
            if (this._moreItemsAvailable === true) {
              this._totalCountLimitExceeded = true;
            }
          }
          // Update pagination bar
          this._paginationWidget.setLocalCount(count, this._totalCountLimitExceeded);

          if (this._totalCountCache != this._paginationWidget.getGlobalCount()) {
            // update value rendered in the pagination widget
            this._paginationWidget.setGlobalCount(this._totalCountCache, this._totalCountLimitExceeded);
          }

          dojo.style(this.collapseExpandAllButton, "display", "block");

          this._onResize();

          this._removeMessage();
        }

        var focusRestored = false;
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          focusRestored = this.listWidget.restoreFocus(autoRefresh);
        }

        if (!focusRestored) {
          // restore scroll position approximately
          if (this.listContent && this._scrollTop) {
            com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
              this.listContent.scrollHeight + ", scrollTop: " + this.listContent.scrollTop);
            this.listContent.scrollTop =
              Math.min(this._scrollTop, this.listContent.scrollHeight - this.listContent.clientHeight);
          }
        }

        this._refreshHandler.refreshFinished();

        // listen for view changes to suspend/resume automatic refresh
        this._subscribeViewChanges();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _updateModel: function _updateModel_TasksListController(tasks, notNewAssignment){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this.listWidget && this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          // map event properties to query properties
          var items = com.ibm.bpm.social.widgets.task.config.FilterConfig.mapTasks(tasks);
          this.listWidget.updateModel(items, false, this._searchFilterApplied, notNewAssignment);
        }
        else {
          throw new Error("Programming error: No delta refresh (updateModel) for com.ibm.bpm.social.widgets.task.list.Table!");
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /*
       * Refresh the list content.
       * @param {boolean} async indicates whether the list shall be loaded asynchronous or synchronous
       * @param {Object} autoRefresh indicates whether this is an auto-refresh-run or not
       */
      _refreshItems: function _refreshItems_TasksListController(async, autoRefresh, size){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        if (this._deferredForQuery) {
          // another query is still in progress, cancel it
          this._deferredForQuery.cancel();
        }

        this._refreshHandler.refreshStarted(autoRefresh);

        // Set the default if parameters have not be specified
        if (this._flagInitConfiguration) {
          // ASZ: There is no _initConfiguration, thus removing
          // this._initConfiguration(this.config);
          this._flagInitConfiguration = false;
        }

        try {
          // If async is not defined, the list is loaded asynchronously.
          if (!async) {
            // Default is asyn because otherwise the GUI would stall.
            // If pagination triggers the refresh, no arguments are provided and the default apply.
            async = true;
          }
          // If autoRefresh is not defined, it is assumed that this is not an auto-refresh-run.
          if (autoRefresh === null || autoRefresh !== true) {
            autoRefresh = false;
          }
          // Reset selectedItems, because the new set of items will most likely differ from the old one.
          // Basically this also enforces that only visible items can be selected.
          // aschoen: Are there situation when the refresh is triggered as callback from the pagination and the selection
          // aschoen: need to be kept?
          // aschoen: For example because the user enters a item number which has been shown before.
          // aschoen: But I doubt this because even if the same window (:= (first,last)) is shown, the data may have changed.
          // aschoen: If more logic is required, we need to move this to the loadCallback method. There we could check whether
          // aschoen: the already selected items are visible.
          this.selectedItems = [];

          // Reset the toolbar actions.
          // TODO
//          var allowedActions = this._getAllowedActions();
//          this.toolbarWidget.setAvailableActions(allowedActions);

          this._loadItems(async, autoRefresh, 0, size);
        }
        catch (e) {
          if (e && e.type == "list") {
            this._showMessage(e.messageKey, e.severity, e.parameters);
          }
          else {
            throw e;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _loadItems: function _loadItems_TasksListController(async, autoRefresh, offset, pageSize){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // clear the query object 
        this._query.clear();

        // Set defaults
        if (offset === null || offset === undefined || (offset < 0)) {
          offset = 0;
        }
        if (pageSize === null || pageSize === undefined || (pageSize <= 0)) {
          pageSize = this.config["pageSize"];
        }

        com.ibm.bpm.social.util.common.Log.log(1, " offset: " + offset);
        com.ibm.bpm.social.util.common.Log.log(1, " pageSize: " + pageSize);

        // if there was a problem detected with the current queue
        // throw the exception and don't process the data
        if (this.config.queue.exception) {
          this._flagErrorCaseRefreshLoadItems = false;
          throw this.config.queue.exception;
        }
        else {
          // update the queue title
          if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
              this.listTitle.innerHTML = "";
              this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
              var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=savedsearch&tw.local.id='+encodeURIComponent(this.config.queue.name);
              if (this._searchText) {
              	newHash = newHash + "&tw.local.searchFilter=" + encodeURIComponent(this._searchText);
              }
              com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
          } else {
	          // update URL hash
	          var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=tasks';
	          if (this.config.selectedFilter == 'openTasks') {
	          	newHash = newHash + '&tw.local.state=open';
	          }
	          if (this.config.selectedFilter == 'completedTasks') {
	          	newHash = newHash + '&tw.local.state=close';
	          }
	          if (this._searchText) {
	          	newHash = newHash + '&tw.local.searchFilter=' + encodeURIComponent(this._searchText);
	          }
	       	  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
          }
          
          // choose type
          this._query.setType(this.config.queue);
          // set search capability
          this._checkSearchCapability();
          this._setSearch();

          if (offset === 0) {
            // reinitialize list of dojo.Deferred for retrieved tasks
            this._deferredForActions = null;
          }
          var self = this;
          // We fetch one object more from the backend, to decide whether we should render a "more available" indicator 
          this._updateQuery(async, pageSize + 1, offset);
          // only for list view type we have categories
          if (this.config.selectedFilter != com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS &&
            this.listWidget) {
            if (this.selectedFilterConfig && this.selectedFilterConfig.selectedProperties) {
              this._query.setSelectedAttributes(this.selectedFilterConfig.selectedProperties);
            }
            if (this.selectedFilterConfig && this.selectedFilterConfig.categories && this.listWidget._categoryWidgets) {
              if (this.listWidget._categoryWidgets.length > 0 && this.listWidget.getExpandedCategoriesCount() === 0) {
                this._paginationWidget.setLocalCount(0, this._totalCountLimitExceeded);
                // when we come here, then all are collapsed.
                // Make sure that all categories are shown on the screen: 
                this.listWidget.setModel([]);
                com.ibm.bpm.social.util.common.Log.logExit(1,
                  "Exiting because no category is expanded -> no need to load something");
                return;
              }

              var tmpWhereClause = null;
              tmpWhereClause = this._computeWhereClauseForExpandedSections(this.listWidget._categoryWidgets);
              if (tmpWhereClause) {
                var tmpFilter = {
                  whereClause: tmpWhereClause
                };
                this._query.setCustomFilter(tmpFilter);
              }
            }
          }
          // set the search text
          this._query.setSearchFilter(this._searchText);

          // create the inline actions
          this._inlineActions = this._createInlineActions();

          this._refreshHandler.waitingForQueryResult();

          this._deferredForQuery =
            this._query.performContentRequest(function _loadItems_success_TasksListController(responseObject){
              com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_loadItems_success");
              self._deferredForQuery = null;
              self._flagErrorCaseRefreshLoadItems = false;
              self.itemIdentifier = responseObject.identifier;
              //self.items = responseObject.items;
              // we have fetched more items than requested;
              // if we really got more than requested, then mark the extra items
              var objectsToBeReturned = pageSize;
              var objectsReceived = responseObject.items.length;
              if (objectsReceived > objectsToBeReturned) {
                self._moreItemsAvailable = true;
              }
              else {
                self._moreItemsAvailable = false;
                objectsToBeReturned = objectsReceived; // may be pageSize or less than pageSize
              }
              var i, item;
              self.items = [];

              for (i = 0; i < objectsReceived; i++) {
                item = responseObject.items[i];
                self.items.push(item);
                if (i == objectsToBeReturned && self._moreItemsAvailable === true) {
                  // extra item, mark it
                  item.isExtra = true;
                }
                else {
                  // mark the others as well
                  item.isExtra = false;
                }

                // add inlineActions for table view 
                if (self.config.viewConfig.viewType == "table" && self._inlineActions) {
                  item.inlineActions = [];
                  for ( var j = 0; j < self._inlineActions.length; j++) {
                    // each item gets a own copy of the inlineActions
                    item.inlineActions.push(dojo.clone(self._inlineActions[j]));
                  }
                }

                // map task status to translated strings
                if (item["STATUS"]) {
                  item.STATUS = self._nlsResources["task_status_" + item.STATUS];
                }
                if (item["PI_STATUS"]) {
                  item.PI_STATUS = self._nlsResources["instance_status_" + item.PI_STATUS];
                }


              }
              if (objectsReceived < pageSize + 1 && self._numberOfCollapsedCategories === 0) {
                // update the total count
                self._totalCountCache = objectsReceived;
              }

              // if we already have fetched the total count from the server, then we do not do it again;
              // this introduces the risk that the total count becomes inaccurate over time,
              // but we do intentionally take this risk (which is why we say "... of approximately yyy").

              // However: due to an enhancement in the API, we get the totalCount for free, even when we only ask for e.g. 25 items.
              // Note though that this totalCount value is not always what we need, but only when no DUE date filtering is done.
              // Otherwise (when we filter away collapsed categories by due date expressions), the totalCount we get for free is as well reduced,
              // because the totalCount delivered by the API uses those filters as well - thus we have to check this first.
              // Note that this is not a bug by the API but rather a limitation: if the totalCount would ignore the dueDate, then
              // the totalCount attribute would no longer come for free, but rather would cause a performance penalty
              // (due to the fact that a second query would be needed internally in the backend).
              var dueDateUsed = false;
              if (self._query._getWhereClause) {
                var whereClauseObject = self._query._getWhereClause();
                if (whereClauseObject && whereClauseObject.whereClause && whereClauseObject.whereClause !== "") {
                  dueDateUsed = true;
                }
              }
              if (dueDateUsed === false && responseObject.totalCount) {
                // we can use the count data that we got for free
                var count = responseObject.totalCount;
                var countLimitExceeded = false;
                if (responseObject.countLimitExceeded && responseObject.countLimitExceeded === true) {
                  countLimitExceeded = true;
                }
                // Set the count on the paginationWidget
                if (self._paginationWidget) {
                  self._paginationWidget.setGlobalCount(count, countLimitExceeded);
                }
                // Set the count cache and the flag whether this did hit the truncation limit
                self._totalCountCache = count;
                self._totalCountLimitExceeded = countLimitExceeded;
                // TODO: exploit the countLimitExceeded attribute in the result object; if true, switch to a slightly different message.
                // However, for this first find out whether the regular count call as triggered by self._countItems() may be truncated as well.
                // This is relevant for switching back to the regular message.              
              }
              else {
                // we can NOT use the count data that we got for free, as its accuracy may be affected by the DUE date filter.
                // If possible, use the cache; if cache not set yet, use an extra count call
                if (self._totalCountCache === null) {
                  // count items
                  self._countItems(true, false, true);
                }
              }
              self._searchFilterApplied = (self._searchText !== null);
              self._refreshHandler.queryFinished(true);

              // create a new start page config object
              self._startPageSettings = self.createPageSettings();
              self.onLoadItemsSuccess(self._startPageSettings);

              com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_loadItems_success");

            }, function _loadItems_error_TasksListController(data, ioArgs){
              com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_loadItems_error");
              com.ibm.bpm.social.util.common.Log.logObject(2, data, "request data");
              com.ibm.bpm.social.util.common.Log.logObject(2, ioArgs, "request ioArgs");
              self._deferredForQuery = null;
              if (data.dojoType == 'cancel') {
                // request has been cancelled
                com.ibm.bpm.social.util.common.Log.logError(2, "Request has been cancelled.");
                self._refreshHandler.queryFinished(false);
              }
              else {
                if (data.dojoType == 'timeout') {
                  // request timed out
                  com.ibm.bpm.social.util.common.Log.logError(2, "Request timed out.");
                  // data, ioArgs, statusCodeMap, errorCodeMap, callback
                  self._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListDataTimeout", null, false, null, null);
                }
                else {
                  // handle error
                  var statusCodeMap =
                    {
                      400: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      },
                      404: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      },
                      0: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      }
                    };
                  var errorCodeMap = {
                    "CWTBG0564E": {
                      key: "KEY_Search_Error"
                    },
                    "CWTBG0567E": {
                      key: "KEY_Search_To_Large_Error"
                    },
                    hideDetails: false
                  };
                  // data, ioArgs, statusCodeMap, errorCodeMap, callback
                  self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, null);
                }
                self.items = [];
                self._refreshHandler.queryFinished(false);
                // show empty message in the list 
                var noItemsMessage = null;
                if (self.config.queue.entityTypeName != "PROCESS_INSTANCE") {
                  noItemsMessage = "task-list-no-tasks";
                }
                else {
                  noItemsMessage = "task-list-no-instances";
                }
                self._showMessage(noItemsMessage);
              }
              com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_loadItems_error");
            });
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },


      /**
       * Post an event that settings relevant to the page settings have changed
       * 
       * @type {void}
       */
      onLoadItemsSuccess: function onLoadItemsSuccess_TasksListController(PageSettings){
        com.ibm.bpm.social.util.common.Log.logExit(2, "Event published ");
      },

      _countItems: function _countItems_TasksListController(async, autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // if there was a problem detected with the current queue
        // throw the expection and don't process the data
        if (this.config.queue.exception) {
          this._flagErrorCaseRefreshCountItems = false;
          throw this.config.queue.exception;
        }
        else {
          var self = this;
          this._query.clear();
          // TODO set the type correct
          this._query.setType(this.config.queue);
          this._updateQuery(async);
          // we used the filter for excluding the collapsed categories - reset as for the count we want them all 
          var tmpFilter = null;
          this._query.setCustomFilter(tmpFilter);
          this._query.performCountRequest(function(responseObject){
            self._flagErrorCaseRefreshCountItems = false;
            self._countCallback(async, autoRefresh, responseObject.count);
          }, function(data, ioArgs){
            // TODO remove
//                // Not every user is a business category or work basket administrator:
//                if (ioArgs.xhr.status == 401 &&
//                  (self.config.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE || self.config.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE)) {
//                  // Disable all actions in this case.
//                  self.toolbarWidget.setAvailableActions();
//                  // Display message in view widget.
//                  var messageKey =
//                    "KEY_LoadListDataNotAuthorized" +
//                      com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type);
//                  self._showMessage(messageKey, "WARNING");
//                }
//                else
            // TODO Do wee need this?
            if (ioArgs.xhr.status == 400 && !self._flagErrorCaseRefreshCountItems) {
              self._flagInitConfiguration = true;
              self._flagErrorCaseRefreshCountItems = true;
              self._refreshItems(true);
            }
//            else {
            // TODO do not show error dialog, because performContentRequest may also show one
            // TODO ? log error somewhere?
//              self._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListData" +
//                com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type), null);
//            }
          });
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _updateQuery: function _updateQuery_TasksListController(async, pageSize, offset, ids){

        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._query.setPageSize(pageSize);
        this._query.setOffset(offset);
        this._query.setAsync(async);
        this._query.setQueue(this.config.queue);

        // set the right filter config
        var selectedFilter = this.config.selectedFilter;
        if (selectedFilter) {
          var selectedFilterFindings = dojo.filter(this.config.filterConfigs, function(filter){
            return filter.id === selectedFilter;
          });
          if (selectedFilterFindings.length > 0) {
            this._setFilterConfig(selectedFilterFindings[0]);
//            if (this.config.viewConfig.viewType == "list") {
//              this._decorateSelectedFilter(selectedFilterFindings[0]);
//            }

          }
        }
        this._query.setUserRoleFilter(this.selectedFilterConfig);


        // TODO
//        this._query.setIdFilter({
//          "property": this.config.parentProperty,
//          "ids": ids
//        });
        // TODO ???
//        this._query.setPrioritiesConfig(this.config[com.ibm.bpm.social.widgets.task.util.Constants.PRIORITIES_CONFIG]);
//        this._query.setGroupingProperties({
//          "property": this.config.activeGroupByProperty,
//          "order": this.config.activeGroupBySortOrder
//        });
        // TODO for now
//        if (this.selectedFilterConfig.groupingProperties) {
//          this._query.setGroupingProperties(this.selectedFilterConfig.groupingProperties);
//        }

        // TODO
//        var selectedAttributes = this._getSelectedAttributes(this.config.queue, this.config.activeGroupByProperty);
//        this._query.setSelectedAttributes(selectedAttributes);
        // TODO modify ???
        if (this.config.viewConfig.selectedAttributes) {
          var selectedAttributes = this.config.viewConfig.selectedAttributes;
          this._query.setSelectedAttributes(selectedAttributes);
        }

        // TODO
//        var customFilter = this.config.activeFilter;
//        this._sortFilterCriteria(customFilter);
//        this._query.setCustomFilter(customFilter);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      getQuery: function getQuery_TasksListController(){
        return this._query;
      },

      _countCallback: function _countCallback_TasksListController(async, autoRefresh, count){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // this method might be called by an async XHR response
        if (this._destroyed) {
          com.ibm.bpm.social.util.common.Log.log(1, "List is already destroyed.");
          return;
        }

        //
        // Set the count cache and the flag whether this did hit the truncation limit
        //
        this._totalCountCache = count;
        // assumption: this count call does not truncate the count.
        this._totalCountLimitExceeded = false;

        //
        // Set the count on the paginationWidget
        //
        if (this._paginationWidget) {
          this._paginationWidget.setGlobalCount(count, this._totalCountLimitExceeded);
        }

        // TODO remove
//        //
//        // Check if navigation state information needs to be applied.
//        //
//        if (this.pendingOffset != null) {
//          this.paginationWidget.setFirstVisibleItem(this.pendingOffset);
//          this.pendingOffset = null;
//        }
//
//        var offset = this.paginationWidget.getFirstVisibleItem() - 1;
//        var pageSize = this.paginationWidget.getItemsPerPage();
//        var lastItem = this.paginationWidget.getLastVisibleItem();
//
//        // Ge: We still need recalculate the allowed actions without selection
//        //     no matter what the count and selection is
//        var allowedActions = this._getAllowedActions();
//        this.toolbarWidget.setAvailableActions(allowedActions);
//
//        if (count > 0) {
//          this._loadItems(async, autoRefresh, offset, pageSize);
//        }
//        else {
//          // no items in the list, therefore reset the item array
//          this.items = [];
//          this._showNoItemMessage();
//
//          if ((dojo.isIE && (dojo.isIE < 8))) {
//            // for some reasons showing a message extends the container to app. 1000px
//            // readjust here
//            this.viewContainer.style.height = this.viewContainerHeight + "px";
//          }
//
//          // count == 0
//          // Do not refresh, if the dom node does not exist anymore, e.g. if the widget
//          // was already destroyed and an async list refresh callback happens.
//          // + Do not refresh the list, if it is an auto-refresh and tasks are selected.
//          if (this.domNode && (!autoRefresh || (autoRefresh && this.getSelectedItems().length === 0))) {
//            // Start a new refresh cycle, if wanted
//            this._createRefreshTimeout();
//          }
//
//          this._updateFilterDescription();
//
//        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /*
       * Shows a message in the list body.
       */
      _showMessage: function _showMessage_TasksListController(messageKey, severity, parameters){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // Check, if there is a message key defined.
        if (messageKey) {
          var message = this._nlsResources[messageKey];
          if (message) {
            // if there are parameters defined for the message
            if (parameters) {
              message = dojo.string.substitute(message, parameters);
            }
            var viewMessage = document.createElement("div");
            viewMessage.style.padding = "4px";
            viewMessage.style.position = "relative";

            // Handle message
            var text = document.createElement("div");
            text.innerHTML = message;

            // Handle severity
            if (severity == "WARNING") {
              var warning = document.createElement("div");
              dojo.addClass(warning, "smallWarning");
              warning.style.position = "absolute";
              if (this.isLeftToRight()) {
                warning.style.left = "12px";
                warning.style.right = "auto";
                text.style.marginLeft = "25px";
                text.style.marginRight = "0";
              }
              else {
                warning.style.right = "12px";
                warning.style.left = "auto";
                text.style.marginRight = "25px";
                text.style.marginLeft = "0";
              }
              viewMessage.appendChild(warning);
            }

            viewMessage.appendChild(text);

            // Remove a previous message, e.g. if the query table does not exist and
            // the user refreshes the list.
            if (this._viewMessage) {
              this.listMessage.removeChild(this._viewMessage);
            }
            this._viewMessage = viewMessage;
            this.listMessage.appendChild(viewMessage);
            // switch the css classes

            dojo.removeClass(this.listMessage, "bpm-task-list-message-hide");
            dojo.addClass(this.listContent, "bpm-task-list-content-hide");
            dojo.addClass(this.pagination, "bpm-task-list-content-hide");
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(1, "Warning: TasksListController._showMessage received an empty message");
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1,
            "Warning: TasksListController._showMessage received an empty message key");
        }

        com.ibm.bpm.social.util.common.Log.log(1, "TasksListController._showMessage exit");
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _removeMessage: function _removeMessage_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._viewMessage) {
          this.listMessage.removeChild(this._viewMessage);
          // switch the css classes
          dojo.removeClass(this.listContent, "bpm-task-list-content-hide");
          dojo.removeClass(this.pagination, "bpm-task-list-content-hide");
          dojo.addClass(this.listMessage, "bpm-task-list-message-hide");
          this._viewMessage = null;
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Return the due date where clause, one pair for each expanded section
       * 
       * @param id object[] the array of categories
       */
      _computeWhereClauseForExpandedSections: function _computeWhereClauseForExpandedSections_TasksListController(categories){
        var whereClause = null;
        var partialClause = null;
        if (categories) {
          // Optimization for the frequent special case that all categories are expanded: no where clause at all
          // Thus, do the quick check whether no categories expanded. This would allow a quick return
          // and, more important, the backend would not have to look at the due date for filtering. 
          var allCategoriesAreExpanded = true;
          dojo.forEach(categories, function(category){
            if (category && category.id && (category.isExpanded() === false)) {
              allCategoriesAreExpanded = false;
            }
          });
          if (allCategoriesAreExpanded === true) {
            // quick return
            return null;
          }

          // assumption: categories are shown in ascending order
          // Loop over the categories and add the start due date and the end due date to the where clause.
          // Expanded sections only.
          // TODO  Optimization: Simplify WHERE clause by combining adjacent expanded sections
          dojo.forEach(categories, function(category){
            if (category && category.id && category.isExpanded()) {
              partialClause = null;
              // there may be invisible categories - allow those as well, there could have items shown up
              var startDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate(category.id);
              var endDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate(category.id);
              // special case first category: no start due date
              if (startDueDate === null && endDueDate !== null) {
                // only end due date, e.g for OVERDUE
                partialClause = "(DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(endDueDate) + "'))";
              }
              else if (startDueDate !== null && endDueDate === null) {
                // only start due date, e.g. for DUE LATER;
                // AT RISK has as well only a start date, but that one is handled separately, combined with IS_AT_RISK flag.
                partialClause = "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startDueDate) + "'))";
              }
              else if (startDueDate !== null && endDueDate !== null) {
                partialClause =
                  "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startDueDate) + "')" + " AND " +
                    " DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(endDueDate) + "'))";
              }
              // defensive: there may be situations where neither start nor end is set - do only append anything in that case
              if (partialClause !== null) {
                if (whereClause === null) {
                  whereClause = partialClause;
                }
                else {
                  whereClause = whereClause + " OR " + partialClause;
                }
              }
            }
          });
          // Special case for atRisk:
          // This category is not determined by the due date;
          // thus, both the expanded case and the collapsed case have to be reflected in the where clause 
          dojo.forEach(categories, function(category){
            if (category && category.id && category.id === com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
              // Compute the atRisk condition; note that overdue tasks are always atRisk, this overdue have to be excluded
              partialClause = null;
              var startAtRiskDate =
                com.ibm.bpm.social.widgets.task.list.Category
                  .getCategoryEndDueDate(com.ibm.bpm.social.widgets.task.list.Category.OVERDUE);
              if (category && category.id && category.isExpanded()) {
                // if the atRisk category is EXPANDED, then this is the contribution to the WHERE clause:
                partialClause =
                  "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startAtRiskDate) + "')" + " AND " +
                    " IS_AT_RISK = true) ";
                if (partialClause !== null) {
                  if (whereClause === null) {
                    whereClause = partialClause;
                  }
                  else {
                    whereClause = whereClause + " OR " + partialClause;
                  }
                }
              }
              else {
                // if the atRisk category is COLLAPSED, then this is the contribution to the WHERE clause:
                // limitation: "NOT (...)" not accepted by condition parser; thus use De Morgan rules
                partialClause =
                  "(DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startAtRiskDate) + "')" + " OR " +
                    " IS_AT_RISK <> true) ";
                if (partialClause !== null) {
                  if (whereClause === null) {
                    // no tasks outside AT_RISK do qualify, no tasks inside AT_RISK category do qualify either -> enforce the empty list:
                    whereClause = "(IS_AT_RISK=true AND IS_AT_RISK=false)";
                  }
                  else {
                    whereClause = "(" + whereClause + ") AND " + partialClause;
                  }
                }
              }
              // there may be invisible categories - allow those as well, there could have items shown up
              var startDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate(category.id);
              var endDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate(category.id);
              // special case first category: no start due date
              var partialClause = null;

            }
          });
        }
        return whereClause;
      },

      /**
       * Set the Filter config and copy the settings to the current queue
       * 
       * @param {com.ibm.bpm.social.widgets.task.config.FilterConfig} filterConfig
       */
      _setFilterConfig: function _setFilterConfig_TasksListController(filterConfig){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.selectedFilterConfig = filterConfig;
        var property, i, j, k, index, filterProperty, queueProperty;
        var item = this.config.queue.getAttributes();
        if (item && item.properties) {
          // if we have a filter config, it overrules the sort order from the queue
          // ignore sortProperties received with the attributes call
          if (filterConfig.sortProperties && filterConfig.sortProperties.length > 0) {
            var queueProperties = item.properties;
            var filterProperties = filterConfig.sortProperties;
            // reset old filter settings
            for (k = 0; k < queueProperties.length; k++) {
              queueProperties[k].isSort = false;
            }

            for (i = 0; i < filterConfig.sortProperties.length; i++) {
              filterProperty = filterProperties[i];
              for (j = 0; j < queueProperties.length; j++) {
                // mix in the filter config for this property
                queueProperty = queueProperties[j];

                if (filterProperty.property == queueProperty.property) {
                  // mixin the filter sort attributes 
                  dojo.mixin(queueProperty, filterProperty);
                }
              }
            }
          }
          else if (item && item.sortProperties) {
            filterConfig.sortProperties = item.sortProperties;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      retrieveAvailableActionsForItems: function retrieveAvailableActionsForItems_TasksListController(items){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        // retrieve available actions async
        this._retrieveAvailableActionsForQueryResult(items);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Retrieve the available actions for the actual query result.
       * <p>
       * Maintains the map of dojo.Deferred objects for each task.
       * <p>
       * The list of available actions is added as property availableActions to the task item.
       * 
       * @param {Array} items a list of task items as returned by the query
       * @returns {void} nothing
       */
      _retrieveAvailableActionsForQueryResult: function _retrieveAvailableActionsForQueryResult_TasksListController(items){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var actions = null;
        var deferredTaskActions =
          this._retrieveAvailableTaskActions(items, com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS,
            true);
        if (this.config.viewConfig.viewType == "table") {
          // only necessary for saved searches not for task list
          var deferredProcessActions = this._retrieveAvailableProcessActions(items, actions, true);
          if (deferredTaskActions) {
            deferredTaskActions.then(dojo.hitch(this, "_updateTaskActions"), null);
          }
          if (deferredProcessActions) {
            deferredProcessActions.then(dojo.hitch(this, "_updateProcessActions"), null);
          }
        }
        else if (deferredTaskActions) {
          this._addDeferredForActions(deferredTaskActions, items);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback invoked after the available actions are retrieved and appended to the model (this.items)
       * (see _retrieveAvailableTaskActions). Updates the inline actions in this.items.inlineActions from
       * this.items.availableActions, then notifies the table to update the actions <br>
       * Note: only called if using table view (custom searches)
       * 
       * @returns {void} nothing
       */
      _updateTaskActions: function _updateTaskActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var item, action, inlineActions, availableActions, piid, canModifyTask, canRunTask, canViewInstanceDetails, canComplete, canClaim;

        // iterate over the model and determine available actions
        for ( var i = 0; i < this.items.length; i++) {
          item = this.items[i];
          availableActions = dojo.isArray(item.availableActions) ? item.availableActions : [];
          inlineActions = dojo.isArray(item.inlineActions) ? item.inlineActions : [];

          if (this.config.queue.entityTypeName != "PROCESS_INSTANCE") {
            // Which task actions qualify?
            //
            var ifReassignBackToGroupOK = dojo.indexOf(availableActions, "ACTION_CANCELCLAIM") != -1;
            var ifTransferToUserOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOUSER") != -1;
            var ifTransferToGroupOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOGROUP") != -1;
            var ifModifyPriorityOK = dojo.indexOf(availableActions, "ACTION_UPDATEPRIORITY") != -1;
            var ifModifyDueDateOK = dojo.indexOf(availableActions, "ACTION_UPDATEDUEDATE") != -1;

            piid = null;
            if (typeof item["PROCESS_INSTANCE.PIID"] == "string") {
              piid = item["PROCESS_INSTANCE.PIID"];
              item.piid = piid; // shortcut for our _viewInstance method 
            }
            else if (typeof item.piid == "string") {
              piid = item.piid;
            }

            canModifyTask =
              (ifReassignBackToGroupOK || ifTransferToUserOK || ifTransferToGroupOK || ifModifyPriorityOK || ifModifyDueDateOK);

            canViewInstanceDetails = (piid !== null);
          }
          else {
            canModifyTask = false; // modifyTask is not shown in processes list
            canViewInstanceDetails = true; // if we got the process instance, we are allowed to see the details
          }

          canClaim = dojo.indexOf(availableActions, "ACTION_CLAIM") != -1;
          canComplete = dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1;
          canRunTask = (canComplete || canClaim);

          for ( var j = 0; j < inlineActions.length; j++) {
            action = inlineActions[j];
            if (action.id == "modifyTask") {
              action.enabled = canModifyTask;
            }
            else if (action.id == "runTask") {
              action.enabled = canRunTask;
            }
            else if (action.id == "viewInstance") {
              action.enabled = canViewInstanceDetails;
            }
          }
        }

        // notify the table that the actions may have changed
        this.listWidget.updateActions();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback invoked after the available actions are retrieved and appended to the model (this.items)
       * (see _retrieveAvailableProcessActions). Updates the inline actions in this.items.inlineActions from
       * this.items.availableActions, then notifies the table to update the actions <br>
       * Note: only called if using table view (custom searches)
       * 
       * @returns {void} nothing
       */
      _updateProcessActions: function _updateProcessActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var item, action, inlineActions, availableActions, piid, canViewInstanceDetails, canModifyInstance;

        // iterate over the model and determine available actions
        for ( var i = 0; i < this.items.length; i++) {
          item = this.items[i];
          availableActions = dojo.isArray(item.availableProcessActions) ? item.availableProcessActions : [];
          inlineActions = dojo.isArray(item.inlineActions) ? item.inlineActions : [];

          canViewInstanceDetails = true; // if we got the process instance, we are allowed to see the details
          canModifyInstance = dojo.indexOf(availableActions, "ACTION_CHANGE_INSTANCE_DUEDATE") != -1;

          for ( var j = 0; j < inlineActions.length; j++) {
            action = inlineActions[j];
            if (action.id == "modifyInstance") {
              action.enabled = canModifyInstance;
            }
            else if (action.id == "viewInstance") {
              action.enabled = canViewInstanceDetails;
            }
          }
        }

        // notify the table that the actions may have changed
        this.listWidget.updateActions();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Retrieve the available actions for a list of processes.
       * <p>
       * The list of available actions is added as property availableActions to the processes item.
       * 
       * @param {Array} items a list of process items as returned by the query
       * @param {Array} actions a list of actions to retrieve (currently not supported)
       * @param {Boolean} async an asynchronous request
       * @returns {dojo.Deferred} an object which allows to add callback handlers for dojo.xhr request
       */
      _retrieveAvailableProcessActions: function _retrieveAvailableProcessActions_TasksListController(items, actions, async){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var deferred = null;

        if (items && items.length > 0) {
          var domain = items && items.length > 0 && items[0].domain ? items[0].domain : null;
          var self = this;
          deferred =
            this._processRestHandler.getActions(items, "PROCESS_INSTANCE.PIID", actions,
              function _retrieveAvailableProcessActions_success__TasksListController(responseObject, ioArgs){

                var result = responseObject;
                // dispatch getActions result to the individual item
                if (result && result.process) {
                  dojo.forEach(result.process, function(processResult){
                    var filteredArr = dojo.filter(items, function(item){
                      return item["PROCESS_INSTANCE.PIID"] === processResult.piid;
                    });
                    if (filteredArr.length === 1) {
                      filteredArr[0].availableProcessActions = processResult.actions || [];
                      com.ibm.bpm.social.util.common.Log.log(2, "Actions for " + processResult.piid + ": " +
                        (processResult.actions !== null ? processResult.actions.toString() : ""), "TasksListController",
                        "_retrieveAvailableProcessActions");
                    }
                  });
                }
              }, function _retrieveAvailableProcessActions_error__TasksListController(data, ioArgs){
                // TODO ??? notification?
                self._errorHandler.showGenericError(data, ioArgs);
              }, async ? async : false, domain);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return deferred;
      },

      /**
       * Retrieve the available actions for a list of tasks.
       * <p>
       * The list of available actions is added as property availableActions to the task item.
       * 
       * @param {Array} items a list of task items as returned by the query
       * @param {Array} actions a list of actions to retrieve
       * @param {Boolean} async an asynchronous request
       * @returns {dojo.Deferred} an object which allows to add callback handlers for dojo.xhr request
       */
      _retrieveAvailableTaskActions: function _retrieveAvailableTaskActions_TasksListController(items, actions, async){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        var deferred = null;

        if (items && items.length > 0) {
          var domain = items && items.length > 0 && items[0].domain ? items[0].domain : null;
          var self = this;
          deferred =
            this._restHandler.getActions(items, "TASK.TKIID", actions, function(responseObject, ioArgs){
              var result = responseObject;
              // dispatch getActions result to the individual item
              if (result && result.tasks) {
                dojo.forEach(result.tasks, function(taskResult){
                  var filteredArr = dojo.filter(items, function(item){
                    return item["TASK.TKIID"] === taskResult.tkiid;
                  });
                  if (filteredArr.length === 1) {
                    filteredArr[0].availableActions = taskResult.actions || [];
                    com.ibm.bpm.social.util.common.Log.log(3, "Actions for " + taskResult.tkiid + ": " +
                      (taskResult.actions !== null ? taskResult.actions.toString() : ""), "TasksListController",
                      "_retrieveAvailableTaskActions");
                  }
                });
              }
            }, function(data, ioArgs){
              // TODO ??? notification?
              self._errorHandler.showGenericError(data, ioArgs);
            }, async ? async : false, domain);
        }
        com.ibm.bpm.social.util.common.Log.logExit(3);
        return deferred;
      },

      /**
       * Handles a map of task ids and the dojo.Deferred object corresponding to the available actions call in
       * order to register a callback for processing when the actions are available.
       * 
       * @param {dojo.Deferred} deferred an object which allows to add callback handlers for dojo.xhr request
       * @param {Array} items a list of task items as returned by the query
       * @returns {void} nothing
       */
      _addDeferredForActions: function _addDeferredForActions_TasksListController(deferred, items){
        var self = this;
        dojo.forEach(items, function(item){
          var entry = self._getDeferredForActions(item);
          if (entry !== null) {
            // replace entry
            entry.deferred = deferred;
          }
          else {
            // add new entry
            if (self._deferredForActions === null) {
              self._deferredForActions = new Array();
            }
            self._deferredForActions.push({
              id: self._restHandler.prefixID(item["TASK.TKIID"], item.domain, item.systemID),
              deferred: deferred
            });
          }
        });
      },

      /**
       * @param {Object} item a task item as returned by the Query
       * @returns {dojo.Deferred} an object which allows to add callback handlers for the dojo.xhr request
       *          which retrieves the actions for the task
       */
      _getDeferredForActions: function _getDeferredForActions_TasksListController(item){
        var entry = null;
        if (this._deferredForActions !== null) {
          var id = this._restHandler.prefixID(item["TASK.TKIID"], item.domain, item.systemID);
          var filteredArr = dojo.filter(this._deferredForActions, function(entry){
            return entry.id == id;
          });
          if (filteredArr.length > 0) {
            entry = filteredArr[0];
          }
        }
        return entry;
      },

      /**
       * @param item a task item
       * @param {Function(data)} callback a callback function to be executed when the retrieval of available
       *        actions for the item is finished successfully
       * @returns {Boolean} true if a dojo.Deferred object is found for the task
       */
      setActionsCallbackSuccess: function setActionsCallbackSuccess_TasksListController(item, callback){
        var result = false;
        var entry = this._getDeferredForActions(item);
        if (entry !== null) {
          entry.deferred.addCallback(callback);
          result = true;
        }
        else {
          // programming error
          throw new Error(item.TKIID + " - No deferred found to add callback!");
        }
        return result;
      },

      /**
       * @param item a task item containing TKIID and optional domain, systemID
       * @param {Function(data)} callback a callback function to be executed when the retrieval of available
       *        actions for the item is finished and failed
       * @returns {Boolean} true if a dojo.Deferred object is found for the task
       */
      setActionsCallbackError: function setActionsCallbackError_TasksListController(item, callback){
        var result = false;
        var entry = this._getDeferredForActions(item);
        if (entry !== null) {
          entry.deferred.addErrback(callback);
          result = true;
        }
        return result;
      },

      _getBulkTaskDetails: function _getBulkTaskDetails_TasksListController(tkiidList){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var myself = this;

        // Fetch data from the server
        myself._restHandler.getTasksInBulkMode(tkiidList, function(bulkResult){
          // Note: In bulk mode, a successful status (HTTP 200) can still mean errors.
          // Thus, error handling will be done in _setTaskDetailsBulk as well.
          myself._setTaskDetailsBulk(bulkResult);
        }, function(data, ioArgs){
          for ( var k = 0; k < tkiidList.length; k++) {
            var taskId = tkiidList[k];
            // task is assigned to somebody else or deleted
            if (ioArgs.xhr.status == 401 || ioArgs.xhr.status == 404) {
              myself.listWidget.showMessage(myself._nlsResources["task-row-details-error-not-available"], taskId);
            }
            else {
              myself.listWidget.showMessage(data.message, taskId);
            }
          }
        }, false, null);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _workOn: function _workOn_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // save focus here because claim dialog may get focus before TCUI is shown
        this._saveListFocus();

        var workOn = true;
        if (item.availableActions) {
          // try to claim Task if it can be claimed
          var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
          var confirm =
            (userProfile && userProfile.userPreferences && userProfile.userPreferences['Alert On Assign And Run'] === "true");

          if (confirm) {
            // ensure up-to-date actions for this task
            this
              ._retrieveAvailableTaskActions([item], com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS);
          }

          if (dojo.indexOf(item.availableActions, "ACTION_CLAIM") != -1) {
            // we can claim, check if we need a dialog
            if (confirm) {
              // show confirmation dialog, if we can claim the task
              com.ibm.bpm.social.util.common.Log.log(2, "show confirm dialog for claim");
              if (!this._confirmClaimDialog) {
                this._createClaimDialog();
              }
              this._confirmClaimDialog.showConfirmationDialog(item);
              workOn = false;
            }
            else {
              workOn = this._claim(item);
            }
          }
        }
        else {
          // item may be a process instance
          if (item.TKIID || item["TASK.TKIID"]) {
            item["TASK.TKIID"] = item.TKIID || item["TASK.TKIID"];
            workOn = true;
            this._waitingForClaimDialogChannel =
              dojo.subscribe(com.ibm.bpm.social.portal.Topics.COACH_CANCELED, dojo.hitch(this, this._claimDialogCancelled));
          }
        }

        if (workOn) {
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
            "taskId": item["TASK.TKIID"]
          }]);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _claim: function _claim_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var claimed = false;
        var self = this;
        this._restHandler.claimTask(item["TASK.TKIID"], function(responseObject, ioArgs){
          claimed = true;

        }, function(data, ioArgs){
          if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status == 409 && ioArgs.xhr.responseText) {
            var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
            if (response.errorNumber == "CWTBG0014E") {
              // task has been claimed since last list refresh
              // check if user is allowed to complete
              // refresh available actions for this item synchronously
              self._retrieveAvailableTaskActions([item],
                com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS);
              if (item.availableActions) {
                if (dojo.indexOf(item.availableActions, "ACTION_COMPLETE") != -1) {
                  // continue normal processing
                  claimed = true;
                }
              }
            }
          }

          if (!claimed) {
            // handle errors
            var name = item.TAD_DISPLAY_NAME;
            var statusCodeMap = {
              409: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              401: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              403: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              404: {
                key: "KEY_ClaimNotExistingTask",
                parameters: {
                  1: name
                }
              }
            };
            var errorCodeMap = {
              "CWTKA0072E": {
                key: "KEY_ClaimSuspendedTask",
                parameters: {
                  1: name
                }
              }
            };

            self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
            // trigger list refresh
            self._refresh(true);
          }
        }, item.system ? item.system : null);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return claimed;
      },

      /**
       * 
       * @type {void}
       */
      _createClaimDialog: function _createClaimDialog_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._confirmClaimDialog = new com.ibm.bpm.social.widgets.task.util.ClaimDialog({
          "restHandler": this._restHandler
        });

        this._childWidgets.push(this._confirmClaimDialog);

        this.connect(this._confirmClaimDialog, "onClaimed", "_claimDialogSuccessful");
        this.connect(this._confirmClaimDialog, "onFailed", "_claimDialogFailed");
        this.connect(this._confirmClaimDialog, "onCancelled", "_claimDialogCancelled");

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _claimDialogSuccessful: function _claimDialogSuccessful_TasksListController(taskId){
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
          "taskId": taskId
        }]);
      },

      _claimDialogFailed: function _claimDialogFailed_TasksListController(taskId){
        this._refresh(true);
      },

      _claimDialogCancelled: function _claimDialogCancelled_TasksListController(taskId){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (this._waitingForClaimDialogChannel) {
          // currently the TaskCompletionWidget might be shown, show myself again.
          dojo.unsubscribe(this._waitingForClaimDialogChannel);
          this._waitingForClaimDialogChannel = null;
          // triggers refresh
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * set the search text for the next refresh
       * 
       * @type {void}
       */
      setSearchText: function setSearchText_TasksListController(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (text === "") {
          // clear on empty string 
          text = null;
        }
        this._searchText = text;
        if (this._searchWidget) {
          this._searchWidget.setSearchText(text);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _search: function _search_TasksListController(text){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._searchText = text;

        this._showMessage("task-list-loading");
        var size = this._paginationWidget.getPageSize();
        // We maintain now collapse/expand state of categories, so this should be respected here also, 
        // even if it is ugly that a category disappears on expand.
//        if (this.selectedFilterConfig && this.selectedFilterConfig.categories) {
//          // expand all categories prior to search. This avoids the ugly case:
//          //  - user collapses "Overdue" category
//          //  - user searches for "blabla", nothing found, "Overdue" category stays visible but collapsed 
//          //  - user expands "Overdue"
//          // ==> "Overdue" category disappears ... which would be confusing
//          var myself = this;
//          dojo.forEach(this.listWidget._categoryWidgets, function(category){
//            if (category && category.isExpanded && (category.isExpanded() === false)) {
//              category.toggle(true); // set silent flag to true, to avoid reload on each category expansion
//              myself._numberOfCollapsedCategories--; // used to decide what message we show 
//            }
//          });
//        }
        this._refreshItems(true, false, size);
        //this.refresh();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _clearSearch: function _clearSearch_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._searchText = null;
        this.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _setSearch: function _setSearch_TasksListController(){
        // TODO : check if system has text search index , check if organized by tasks 
        if (this._searchEnabled) {
          this._searchWidget.show();
        }
        else {
          this._searchWidget.hide();
        }
      },

      _checkSearchCapability: function _checkSearchCapability_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var systems = this._restHandler.getSystems();
        this._searchEnabled = false;
        if (this._query._type === com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
          this._searchEnabled = true;
        }
        // as we do only have one system right now, it is the first one
        if (this._searchEnabled && systems && systems._systemIDs) {
          var systemId = systems._systemIDs[0];
          if (systemId) {
            this._searchEnabled = systems.isSystemSearchable(systemId);
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(1, "searchEnabled : this._searchEnabled");
      },

      _doFilter: function(filter){
        this.config.selectedFilter = filter.id;

        this._setFilterConfig(filter);

        this.listWidget.setConfig({
          controller: this,
          filter: filter,
          type: this.config.type
        });

        // remove previous filter settings
        this._query.setCustomFilter(null);

        // clear total count cache
        this._totalCountCache = null;

        this.refresh();

        this._decorateSelectedFilter(filter);
      },

      _decorateSelectedFilter: function _decorateSelectedFilter_TasksListController(filter){
        var self = this;
        dojo.forEach(this.config.filterConfigs, function(currentFilter){
          var currentFilterSpan = self[currentFilter.id + "_filter"];
          dojo.removeClass(currentFilterSpan, "bpm-task-list-filter-active");
          dojo.addClass(currentFilterSpan, "bpm-task-list-filter-inactive");
          dijit.setWaiState(currentFilterSpan, "selected", "false");
        });
        var filterSpan = this[filter.id + "_filter"];
        dojo.removeClass(filterSpan, "bpm-task-list-filter-inactive");
        dojo.addClass(filterSpan, "bpm-task-list-filter-active");
        dijit.setWaiState(filterSpan, "selected", "true");
      },

      _setTaskDetailsBulk: function _setTaskDetailsBulk_TasksListController(bulkResult){
        if (bulkResult) {
          var k;
          // handle those tasks for which we received a successful response  
          if (bulkResult.tasks) {
            for (k = 0; k < bulkResult.tasks.length; k++) {
              this._setTaskDetails(bulkResult.tasks[k]);
            }
          }
          // as well, handle those tasks for which we received an errorneous response  
          if (bulkResult.failedOperations) {
            for (k = 0; k < bulkResult.failedOperations.length; k++) {
              var currentError = bulkResult.failedOperations[k];
              var currentErrMsg = currentError.errorMessage;
              // Special case: For tasks that were deleted in the meantime,
              // show the easy to understand error message (same as non-bulk mode)
              if (currentErrMsg && currentErrMsg.substr(0, 11) == "CWTBG0048E:") {
                currentErrMsg = this._nlsResources["task-row-details-error-not-available"];
              }
              this.listWidget.showMessage(currentErrMsg, currentError.tkiid);
            }
          }
          this._onResize();
        }
      },

      _setTaskDetails: function _setTaskDetails_TasksListController(task){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var taskId = task.tkiid;

        // Process business data
        var businessDataForTR = [];
        if (task.processData && task.processData.businessData) {
          for ( var k in task.processData.businessData) {
            var label = null;
            if (task.processData.businessData[k].label) {
              label = task.processData.businessData[k].label;
            }
            else {
              label = task.processData.businessData[k].name;
            }
            var newBusinessDataEntryForTR = {
              label: label,
              value: task.processData.businessData[k].value,
              type: task.processData.businessData[k].type,
              name: task.processData.businessData[k].name,
              alias: task.processData.businessData[k].alias
            };
            businessDataForTR.push(newBusinessDataEntryForTR);
          }
        }

        var details = {
          narrative: task.description,
          businessData: businessDataForTR
        };
        //
        // Add variables to details if available in order to show choices for SimpleChoice Human Service.
        //
        if (task.data && task.data.variables) {
          details.variables = task.data.variables;
        }

        //
        // Check for know Human Service that require special handling.
        // These services are part of the system toolkit.
        var serviceID = task.serviceID;
        var isFinished = task.state == "STATE_FINISHED";

        if (serviceID == "1.71d16b41-1693-4f57-9227-a149d8bf3c1b") {
          // Simple Approval Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleApproval(taskId, details);
          }
        }
        else if (serviceID == "1.46a5689a-0864-4a33-a4c6-040ab815d75a") {
          // Simple Completion Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleCompletion(taskId, details);
          }
        }
        else if (serviceID == "1.63e23d5c-9a1e-4104-b4a7-cef70df65289") {
          // Simple Choice Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleChoice(taskId, details);
          }
        }
        else {
          // Default case => no special handling, just show the details
          this.listWidget.setDetails(taskId, details);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Show a warning if we are opening a inline task in the open tasks view that is already finished in the
       * backend
       * 
       * @type {void}
       */
      _showInlineTaskFinishedWarning: function _showInlineTaskFinishedWarning_TasksListController(displayName){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._errorHandler.showWarningDialog("KEY_TaskInlineAlreadyCompleted", {
          1: displayName
        }, function(){
          this._refresh(true);
        });

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _showExpandAllButton: function _showExpandAllButton_TasksListController(){
        this._isCEButtonInCollapsedState = true;
        this.collapseExpandAllButton.title = this._nlsResources["task-list-expand-all"];
        if (dojo.hasClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover")) {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-show-all-hover");
        }
        else {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-hide-all");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-show-all");
        }
        this.collapseExpandAllButtonText.innerHTML = "&#9660;";
      },

      _showCollapseAllButton: function _showCollapseAllButton_TasksListController(){
        this._isCEButtonInCollapsedState = false;
        this.collapseExpandAllButton.title = this._nlsResources["task-list-collapse-all"];
        if (dojo.hasClass(this.collapseExpandAllButton, "bpm-social-show-all-hover")) {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-show-all-hover");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover");
        }
        else {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-show-all");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-hide-all");
        }
        this.collapseExpandAllButtonText.innerHTML = "&#9650;";
      },

      _onResize: function _onResize_TasksListController(){
        var contentHeight = this._getListContentHeight();
        dojo.style(this.listContent, "height", contentHeight + "px");

        if (this.listWidget) {
          if (this.config && this.config.viewConfig.viewType == "table") {
            // table must always be limited
            this.listWidget.resize({
              h: contentHeight,
              w: this._getListContentWidth()
            });
            this.listWidget.renderTable(false);
          }
          else {
            var padding = "";
            com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
              this.listContent.scrollHeight);
            if (this.listContent.clientHeight < this.listContent.scrollHeight) {
              // we have a scrollbar, add padding between list and scrollbar
              padding = "10px";
            }
            if (document.body.dir == 'rtl') {
              dojo.style(this.listWidget.domNode, "paddingLeft", padding);
            }
            else {
              dojo.style(this.listWidget.domNode, "paddingRight", padding);
            }
          }
        }
      },

      /**
       * listen for view changes to suspend/resume automatic refresh
       */
      _subscribeViewChanges: function _subscribeViewChanges_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);

        if (this._subscribedTaskCompletion === null) {
          // disable automatic refresh if the task completion view is shown, list is always refreshed on return
          this._subscribedTaskCompletion =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, dojo.hitch(this, this._disableRefresh));
        }

        if (this._subscribedInstanceView === null) {
          // disable automatic refresh if the instance view is shown, list is always refreshed on return
          this._subscribedInstanceView =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, dojo.hitch(this, this._disableRefresh));
        }

        if (this._subscribedShowDialog === null) {
          // suspend automatic full refresh if a dialog is shown, list may not be refreshed on return
          this._subscribedShowDialog =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, dojo.hitch(this, this._onShowDialog));
        }

        if (this._subscribedHideDialog === null) {
          // resume automatic full refresh if a dialog is hidden
          this._subscribedHideDialog =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, dojo.hitch(this, this._onHideDialog));
        }

      },

      _unSubscribeViewChanges: function _unSubscribeViewChanges_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);

        if (this._subscribedTaskCompletion !== null) {
          dojo.unsubscribe(this._subscribedTaskCompletion);
          this._subscribedTaskCompletion = null;
        }

        if (this._subscribedInstanceView !== null) {
          dojo.unsubscribe(this._subscribedInstanceView);
          this._subscribedInstanceView = null;
        }

        if (this._subscribedShowDialog !== null) {
          dojo.unsubscribe(this._subscribedShowDialog);
          this._subscribedShowDialog = null;
        }

        if (this._subscribedHideDialog !== null) {
          dojo.unsubscribe(this._subscribedHideDialog);
          this._subscribedHideDialog = null;
        }
      },

      onHide: function onHide_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        this._refreshHandler.disable();
        this._unSubscribeViewChanges();
      },


      _disableRefresh: function _disableRefresh_TasksListController(data){
        var details = "";
        if (data && data.taskId) {
          details = " taskId=" + data.taskId;
        }
        else if (data && data.instanceId) {
          details = " instanceId=" + data.instanceId;
          this._saveListFocus();
        }
        com.ibm.bpm.social.util.common.Log.log(3, this.id + details);

        this._refreshHandler.disable();
        this._unSubscribeViewChanges();
      },

      _onShowDialog: function _onShowDialog_TasksListController(dialogId){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        this._saveListFocus();
        this._refreshHandler.suspendFullRefresh(dialogId);
      },

      _onHideDialog: function _onHideDialog_TasksListController(dialogId){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          this.listWidget.setKeepSavedFocus(false);
        }
        this._refreshHandler.resumeFullRefresh(dialogId);
      },

      _saveListFocus: function _saveListFocus_TasksListController(){
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          this.listWidget.saveFocus();
          this.listWidget.setKeepSavedFocus(true);
        }
      },

      /**
       * Return settings that describe the current content
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings} object with query information
       */
      getPageSettings: function getPageSettings_TasksListController(){
        return this._startPageSettings;
      },

      /**
       * collect the controller settings for the start page.
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings} object with query information
       */
      createPageSettings: function createPageSettings_TasksListController(){
        var result = new com.ibm.bpm.social.util.common.PageSettings({
          savedSearchName: this._query._queue.name,
          searchFilterString: this._query._searchFilter,
          sortAttributes: this.selectedFilterConfig.sortProperties,
          filterConfigId: this.selectedFilterConfig.id
        });
        return result;
      },

      onFocusNeeded: function onFocusNeeded_TasksListController(){}

    });

com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS = ["ACTION_CLAIM",
                                                                         "ACTION_COMPLETE",
                                                                         "ACTION_CANCELCLAIM",
                                                                         "ACTION_REASSIGNTOUSER",
                                                                         "ACTION_REASSIGNTOGROUP",
                                                                         "ACTION_UPDATEPRIORITY",
                                                                         "ACTION_UPDATEDUEDATE",
                                                                         "ACTION_ADDCOMMENT"];

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.config.ControllerConfig"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.config.ControllerConfig"] = true;
// ControllerConfig.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.config.ControllerConfig");






/**
 * ControllerConfig: Controller Settings for a Worktab sub-tab
 */
dojo.declare("com.ibm.bpm.social.widgets.task.config.ControllerConfig", null, {
  controllerId: null,
  viewConfig: null,
  filterConfigs: null,
  type: null,

  constructor: function constructor_ControllerConfig(controllerId, type, viewConfig, filterConfigs, selectedFilter){
    this.controllerId = controllerId;
    this.type = type;
    if (viewConfig) {
      this.viewConfig = viewConfig;
    }
    if (filterConfigs) {
      this.filterConfigs = filterConfigs;
    }
    if (selectedFilter) {
      this.selectedFilter = selectedFilter;
    }
    this.pageSize = 20;
  }
});

com.ibm.bpm.social.widgets.task.config.ControllerConfig.getMyTasksConfig =
  function(){
    var viewConfig = com.ibm.bpm.social.widgets.task.config.ViewConfig.getMyTasksConfig();

    return new com.ibm.bpm.social.widgets.task.config.ControllerConfig(viewConfig.id,
      com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE,
      viewConfig,
      [com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigOpenTasks(),
       com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCompletedTasks()],
      "openTasks");
  };

com.ibm.bpm.social.widgets.task.config.ControllerConfig.getCustomViewsConfig =
  function(){
    var viewConfig = com.ibm.bpm.social.widgets.task.config.ViewConfig.getCustomViewsConfig();

    return new com.ibm.bpm.social.widgets.task.config.ControllerConfig(viewConfig.id,
      com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE,
      viewConfig,
      [com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews()],
      "customViews");
  };

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.QueryRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.QueryRestHandler"] = true;
// QueryRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.QueryRestHandler");






dojo.declare("com.ibm.bpm.social.util.common.QueryRestHandler", com.ibm.bpm.social.util.common.RestHandler, {

  constructor: function constructor_QueryRestHandler(params){},

  /**
   * Returns all query tables for the given entity for the given domain.
   * 
   * @param {String} type
   * @param {String} requiredContent
   * @param {function} load
   * @param {function} error
   * @param {String} domain federation only
   * @param {String} process application name
   * @returns {void} nothing
   * @throws error if no domain name is specified for a federated backend
   */
  getAllQueryTables: function getAllQueryTables_QueryRestHandler(type, requiredContent, load, error, domain, appName){
    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(type);

    // Call API
    if (apiType) {
      var url = apiType;
      var parameters = "";
      // AB: For Escalations , Workbasktest and BusinessCategories still the old API style is necessary 
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        url += "/queryTables";
      }
      else {
        url += "/queries";
      }

      if (this.isApiTypeFederated()) {
        if (domain) {
          parameters += "&domain=" + domain;
        }
        else {
          this.errorNoDomain();
        }
        parameters += "&all=true";
      }

      if (requiredContent) {
        parameters += "&content=" + requiredContent;
      }
      // TODO AB: for federated and WLE API we can not use these parameters right now. 
      // they would not return query tables as these values are not correct for WLE
      if (this.isApiTypeWps()) {
        if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE ||
          type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE) {
          // We only support templates with authorizationType=NONE, otherwise
          // only the BPC admin would see the templates.
          parameters += "&authorizationType=NONE";
        }
        else {
          // For instance QT'S only query tables requiring instance-based authorization
          // are returned, because we heavily make use of WORKITEM_REASON.
          parameters += "&authorizationType=INSTANCE_BASED";
        }
        // Get composite query tables only.
        parameters += "&kind=COMPOSITE";
      }
      // User parameters are not supported.
      parameters += "&userParametersAllowed=false";

      if (appName) {
        parameters += "&processAppName=" + encodeURIComponent(appName);
      }

      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      url = this.getRequestUrl(url);
      var requestLoad = load;
      var expirationPeriod = 3600000;
      var data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
      var requestError = error;
      var myself = this;
      // if we have data in the cache for this url
      if (data) {
        // Handling 200 requests
        if (data.data && typeof load == "function") {
          // emulate 2.argument
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else if (data.ioArgs && typeof error == "function") {
          // handle error requests 401 unauthorized
          // emulate 2.argument
          error.apply(null, [{},
                             data.ioArgs]);
          return;
        }
      }
      else {
        if (typeof load == "function") {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
        if (typeof error == "function") {
          requestError = function(data, ioArgs){
            // do not cache other errors as they might stay for a server error.
            // But unauthorized is an error related to user rights and not server
            // problems.
            if (ioArgs.xhr.status == 401) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                ioArgs: ioArgs
              }, expirationPeriod);
            }
            return error.apply(this, [data,
                                      ioArgs]);
          };
        }
      }
      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: true,
        load: requestLoad,
        error: requestError
      }, true);
    }
    else {
      // TODO: trace
      /*@pass@*/
    }
  },

  /**
   * Returns all attributes for a query table for the given entity.
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {boolean} includeTaskData flag whether to include task data into a processes query
   * @param {function} load
   * @param {function} error
   * @param {boolean} preventCache
   * @param {String} domain federation only
   * @param {String} systemID federation only
   * @returns {void} nothing
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getAttributesForQueryTable: function getAttributesForQueryTable_QueryRestHandler(type, queryTable, processAppName,
    includeTaskData, load, error, preventCache, domain, systemID){

    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(type);

    // Call API
    if (apiType) {
      var url = apiType;
      // AB: For Escalations , Workbasktest and BusinessCategories still the old API style is necessary
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        url += "/queryTable/";
      }
      else {
        url += "/query/";
      }
      url += encodeURIComponent(queryTable) + "/attributes";

      var parmString = "";

      if (processAppName) {
        parmString += "&processAppName=" + encodeURIComponent(processAppName);
      }

      if (typeof includeTaskData == "boolean") {
        parmString += "&includeTaskData=" + includeTaskData;
      }

      if (this.isApiTypeFederated()) {
        if (systemID) {
          parmString += "&systemID=" + systemID;
        }
        else if (domain) {
          parmString += "&domain=" + domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parmString += "&all=true";
      }

      if (parmString.length > 0) {
        url = url + "?" + parmString.substr(1);
      }

      url = this.getRequestUrl(url);

      var requestLoad = load;
      if (typeof load == "function") {
        // Handling for 200 (ok) is implemented, do not cache an 404 result,
        // because a 404 indicates a real error. One of the next calls for
        // this template should bring the real template.
        var expirationPeriod = 3600000;
        var data = null;
        // If the preventCache is not provided, do try cache
        // otherwise take  preventCache parameter
        if (typeof preventCache == "undefined" || !preventCache) {
          data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
        }
        if (data) {
          // emulate 2.argument ??
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
      }

      if (preventCache === undefined || preventCache === null) {
        // ASZ@696877: allow browser to cache attributes.
        preventCache = false;
      }

      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: true,
        load: requestLoad,
        error: error,
        preventCache: preventCache

      }, true);
    }
    else {
      // TODO: trace
    }
  },

  /**
   * Returns all entities in a query table.
   * 
   * Use the object format for the parameters
   * 
   * @param {object}
   * 
   * 
   * deprecated way of passing the parameters please use the object style
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {String} selectedAttributes
   * @param {String} interactionFilter
   * @param {String} queryFilter
   * @param {String} sortAttributes
   * @param {String} offset
   * @param {String} size
   * @param {Function} load
   * @param {Function} error
   * @param {boolean} async
   * @param {String} domain federation only
   * @param {String} systemID federation only, mutually exclusive with domain contain the systemID property!
   * @returns {dojo.Deferred} an object which allows to cancel the dojo.xhr request
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getQueryTableContent: function getQueryTableContent_QueryRestHandler(type, queryTable, processAppName, selectedAttributes,
    interactionFilter, queryFilter, sortAttributes, offset, size, load, error, async, domain, systemID){
    var param = {};
    if (type.queryTable) {
      param = type;
    }
    else {
      param.type = type;
      param.queryTable = queryTable;
      param.processAppName = processAppName;
      param.selectedAttributes = selectedAttributes;
      param.interactionFilter = interactionFilter;
      param.queryFilter = queryFilter;
      param.sortAttributes = sortAttributes;
      param.offset = offset;
      param.size = size;
      param.load = load;
      param.error = error;
      param.async = async;
      param.domain = domain;
      param.systemID = systemID;
    }
    return this._getQueryTableContent(param);
  },


  _getQueryTableContent: function getQueryTableContent_QueryRestHandler(params){

    var deferred = null;

    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(params.type);

    // Call API
    if (apiType) {
      var url = apiType + "/query";
      var parameters = "";
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        parameters += "&queryTableName=" + params.queryTable;
      }
      else {
        url += "/" + encodeURIComponent(params.queryTable);
      }

      // include the task data into process queries.
      if (params.includeTaskData) {
        parameters += "&includeTaskData=true";
      }
      if (params.processAppName) {
        parameters += "&processAppName=" + encodeURIComponent(params.processAppName);
      }

      if (this.isApiTypeFederated()) {
        if (params.systemID) {
          parameters += "&systemID=" + params.systemID;
        }
        else if (params.domain) {
          parameters += "&domain=" + params.domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parameters += "&all=true";
      }

      if (params.searchFilter) {
        parameters += "&searchFilter=" + window.encodeURIComponent(params.searchFilter);
      }
      if (params.selectedAttributes) {
        parameters += "&selectedAttributes=" + encodeURIComponent(params.selectedAttributes);
      }
      if (params.interactionFilter) {
        parameters += "&interactionFilter=" + window.encodeURIComponent(params.interactionFilter);
      }
      if (params.queryFilter) {
        parameters += "&queryFilter=" + window.encodeURIComponent(params.queryFilter);
      }
      if (params.offset) {
        parameters += "&offset=" + params.offset;
      }
      if (params.size) {
        parameters += "&size=" + params.size;
      }
      else {
        // add threshold
        parameters += "&size=" + this.threshold;
      }
      if (params.sortAttributes) {
        parameters += "&sortAttributes=" + encodeURIComponent(params.sortAttributes);
      }
      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      if (params.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE ||
        params.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
        url = url + "&admin=true";
      }

      var myself = this;
      deferred = this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: !params.async,
        load: function(responseObject, ioArgs){
          if (myself.isApiTypeFederated() && params.systemID && responseObject.items && responseObject.items.length > 0) {
            // add systemID to items
            // TODO SA remove this warning - just to see if this happens
            com.ibm.bpm.social.util.common.Log.consoleWarn("Adding systemID to query result!!!");
            for ( var i = 0; i < responseObject.items.length; i++) {
              responseObject.items[i].systemID = params.systemID;
            }
          }
          params.load(responseObject, ioArgs);
        },
        error: params.error
      });
    }
    else {
      // TODO: trace
    }
    return deferred;
  },

  /**
   * 
   * Returns the number of all entities in a query table.
   * 
   * @param {Object} param
   * 
   * the simple params are deprecated use the object structure
   * 
   * @param {String} type
   * @param {String} queryTable
   * @param {String} processAppName
   * @param {String} interactionFilter
   * @param {String} queryFilter
   * @param {Function} load
   * @param {Function} error
   * @param {boolean} async
   * @param {String} domain federation only
   * @param {String} systemID federation only
   * @returns {void} nothing
   * @throws error if no domain name or system id is specified for a federated backend
   */
  getQueryTableCount: function getQueryTableCount_QueryRestHandler(type, queryTable, processAppName, interactionFilter,
    queryFilter, load, error, async, domain, systemID){
    var params = {};
    // new format
    if (type.queryTable) {
      params = type;
    }
    else {
      // old style
      params.type = type;
      params.queryTable = queryTable;
      params.processAppName = processAppName;
      params.interactionFilter = interactionFilter;
      params.queryFilter = queryFilter;
      params.load = load;
      params.error = error;
      params.async = async;
      params.domain = domain;
      params.systemID = systemID;
    }
    this._getQueryTableCount(params);
  },

  _getQueryTableCount: function _getQueryTableCount_QueryRestHandler(params){
    // Map query table type to REST API
    var apiType = this._getQueryTableAPI(params.type);

    // Call API
    if (apiType) {
      var url = apiType;
      var parameters = "";
      if (apiType.indexOf("/businessCategories") === 0 || apiType.indexOf("/workBaskets") === 0) {
        // new query API is not available for processTemplates calls
        url += "/queryCount";
        parameters += "&queryTableName=" + params.queryTable;
      }
      else {
        url += "/query/" + params.queryTable + "/count";
      }

      if (params.searchFilter) {
        parameters += "&searchFilter=" + window.encodeURIComponent(params.searchFilter);
      }
      if (params.processAppName) {
        parameters += "&processAppName=" + encodeURIComponent(params.processAppName);
      }

      if (this.isApiTypeFederated()) {
        if (params.systemID) {
          parameters += "&systemID=" + params.systemID;
        }
        else if (params.domain) {
          parameters += "&domain=" + params.domain;
        }
        else {
          this.errorNoDomainOrSystemID();
        }
        parameters += "&all=true";
      }

      if (params.interactionFilter) {
        parameters += "&interactionFilter=" + window.encodeURIComponent(params.interactionFilter);
      }
      if (params.queryFilter) {
        parameters += "&queryFilter=" + window.encodeURIComponent(params.queryFilter);
      }
      if (parameters.length > 0) {
        url = url + "?" + parameters.substr(1);
      }

      // Add parameter "admin=true" if we have business categories.
      if (params.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE ||
        params.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
        url = url + "&admin=true";
      }

      this.performRequest("get", {
        url: url,
        handleAs: "json",
        sync: !params.async,
        load: params.load,
        error: params.error
      });
    }
    else {
      // TODO: trace
    }
  },

  _getQueryTableAPI: function _getQueryTableAPI_QueryRestHandler(type){
    var apiType = null;
    if (type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
      apiType = "/tasks";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TEMPLATE_TYPE) {
      apiType = "/taskTemplates";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE) {
      apiType = "/processes";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TEMPLATE_TYPE) {
      apiType = "/processTemplates";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.ESCALATION_TYPE) {
      apiType = "/escalations";
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE) {
      apiType = "/workBaskets"; //FIXME: change to correct base URI of work basket REST service
    }
    else if (type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE) {
      apiType = "/businessCategories";
    }

    return apiType;
  },

  statics: {
    _instance: null,

    _createInstance: function(params){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.util.common.QueryRestHandler();
        // TODO handle params, e.g. 
//        this._instance.setIWidget(iWidget);
//        this._instance.setUrl("prefix", this.getServiceURLRoot());
//        this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//        this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
      }
      else {
        // programming error
        var error = new Error("RestHandler instance has already been initialized!");
        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
        throw error;
      }
      return this._instance;
    },

    _getInstance: function(){
      if (this._instance === null) {
        // TODO for now return default handler
        this._instance = new com.ibm.bpm.social.util.common.QueryRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.util.common.QueryRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.QueryRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.QueryRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.QueryRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Queue"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Queue"] = true;
// Queue.js
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
// END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.Queue");





/**
 * Wraps the config object queue as described in the Config document and provides some helper functions
 * 
 * @author ASZ
 * 
 */

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.Queue",
    null,
    {
      /**
       * Identifier of the task queue as provided by the REST API. In domain setups, the name will get
       * suffixed with "-<i>domainName</i>", e.g. IBM.ALLTASKS-defaultDomain
       * 
       * @type {String}
       */
      name: "",

      /**
       * Display name of task queue provided by the REST API.
       * 
       * @type {String}
       */
      displayName: "",

      /**
       * Description as provided by the REST API.
       * 
       * @type {String}
       */
      description: "",

      /**
       * Display name of task queue provided by the REST API deprecated, use displayName.
       * 
       * @type {String}
       */
      title: "",

      /**
       * An Object, if the Queue has an error, Null otherwise
       * 
       * @type {Object}
       */
      exception: null,


      /**
       * JSON object which defines the item representation, e.g. which properties are shown.
       * 
       * <code>
       *   item = [ 
       *     titleKey: "Property key used for title (configurable,persisted)" 
       *     properties: "Array of properties for this query table"
       *   ]
       * </code>
       * 
       * @type {Object}
       */
      item: null,

      /**
       * Maps the different variables in a whereClause to the right values. That can be place holders for
       * query table attribute names or date values like today.
       * 
       * Is an Array of strings, with name-value pair where
       * <li>Name is the replacement variable of the whereClause
       * <li> value is the attribute name in the query table
       * 
       * @type {String[]}
       */
      queuePropertiesMap: null,

      /**
       * ID/name of the domain the query table belongs to, contains the value as retrieved from the REST API.
       * 
       * @type {String}
       */
      domain: null,

      /**
       * Process application name the query table belongs to, contains the value as retrieved from the REST
       * API.
       * 
       * @type {String}
       */
      processAppName: null,

      /**
       * Display name of the domain the query table belongs to.
       * 
       * @type {String}
       */
      domainDisplayName: null,


      /**
       * Type of domain the query table belongs to.
       * 
       * One of
       * <ol>
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE
       * <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED
       * </ol>
       * 
       * 
       * @type {String}
       */
      domainType: null,

      /**
       * deprecated, use <link>domainType</link>
       * 
       * @type {String}
       */
      type: null,


      /**
       * The used navigation type of the list as selected by the user. One of
       * <ul>
       * <li>pagings
       * <li>scrolling
       * </ul>
       * In single system setups, this just contains the type as set for Widget. Depending on the domain
       * setup, if the domain is a multi system domain, this is always set to <i>scrolling</i>.
       * 
       * @type {String}
       */
      navigationType: null,


      /**
       * An Domain object as returned by Systems.getDomain
       * 
       * @type {Object}
       */
      _domainInstance: null,

      /**
       * Flag to remember whether there are multiple domains available in federation
       * 
       * @type {boolean}
       */
      _federationHasMultipleDomains: false,

      /**
       * Flag if Attributes for this queue have been already loaded/set via setAttributes()
       * 
       * @type {boolean}
       */
      attributesLoaded: false,

      /**
       * create a Queue object from scratch or an unnamed object
       * 
       * @constructor
       * @param {Object} aQueue an object to copy/init attributes from
       * @param {Object} systems a reference to a util.Systems instance (optional)
       * @type {void}
       */
      constructor: function constructor_Queue(aQueue, systems){
        com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Queue", "constructor");

        if (aQueue) {
          dojo.safeMixin(this, aQueue);

          //
          // mappings for deprecated attributes:
          //
          if (!aQueue.displayName) {
            this.displayName = aQueue.title;
          }
          else {
            this.title = aQueue.displayName;
          }
          if (!aQueue.domainType) {
            this.domainType = aQueue.type;
          }
          else {
            this.type = aQueue.domainType;
          }

          //
          // ensure unique name:
          // 
          if (aQueue.name) {
            this.setName(aQueue.name);
          }

          if (aQueue.displayName) {
            this.setDisplayName(aQueue.displayName);
          }

          if (aQueue.domainDisplayName) {
            this.setDomainDisplayName(aQueue.domainDisplayName);
          }

          if (aQueue.description) {
            this.setDescription(aQueue.description);
          }

          if (aQueue.processAppName) {
            this.setProcessAppName(aQueue.processAppName);
          }

          if (aQueue.icmSystems) {
            this._icmSystems = aQueue.icmSystems;
          }

          if (systems) {
            this._systems = systems;

            // retrieve the domain object from the Systems class.
            if (aQueue.domain) {
              this.setDomain(this._systems.getDomain(this.domain));
            }

            if (systems.isApiTypeFederated() && systems.getDomainNames().length > 1) {
              this._federationHasMultipleDomains = true;
            }
          }

          com.ibm.bpm.social.util.common.Log.logExit(3, "", "Queue", "constructor");
          return;
        }
      },


      /**
       * Set Description for this queue
       * 
       * @type {void}
       */
      setDescription: function setDescription_Queue(desc){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // need to prevent code injection
        this.description = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(desc);
        com.ibm.bpm.social.util.common.Log.logExit(2, "set to " + this.description);
        return;
      },

      /**
       * Return the description for this queue
       * 
       * @type {String}
       */
      getDescription: function getDescription_Queue(){
        return this.description;
      },

      /**
       * Set ProcessAppName for this queue
       * 
       * @type {void}
       */
      setProcessAppName: function setProcessAppName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.processAppName = name;

        // process app name has changed, we need to update the queue name:
        this.setName(this.name);

        com.ibm.bpm.social.util.common.Log.logExit(2, "set to " + this.processAppName);
        return;
      },

      /**
       * Return the ProcessAppName for this queue
       * 
       * @type {String}
       */
      getProcessAppName: function getProcessAppName_Queue(){
        return this.processAppName;
      },

      /**
       * Return the Reference to the systems object
       * 
       * @type {Object}
       */
      getSystems: function getSystems_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning " + (this._systems ? this._systems : "null"));
        return this._systems;
      },
      /**
       * Set the Reference to the systems object
       * 
       * @type {Object}
       */
      setSystems: function setSystems_Queue(systems){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._systems = systems;
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Set the navigation type this queue shall be used with. Can be one of
       * <ol>
       * <li>com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_PAGING = "paging"
       * <li>com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_SCROLLING = "scrolling"
       * </ol>
       * 
       * @param {String} navType the type
       * @type {void}
       */
      setNavigationType: function setNavigationType_Queue(navType){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!navType ||
          (navType != com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_SCROLLING && navType != com.ibm.bpm.social.widgets.task.util.Constants.NAVIGATION_TYPE_PAGING)) {
          var error = new Error("Queue.setNavigationType: got invalid type " + navType);
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }
        this.navigationType = navType;
        com.ibm.bpm.social.util.common.Log.logExit(2, "set nav type to" + navType);
        return;
      },

      /**
       * Set the type of the domain this queue relates to.
       * 
       * @param {String} type
       * @type {void}
       */
      setDomainType: function setDomainType_Queue(type){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.domainType = type;
        this.type = type;
        com.ibm.bpm.social.util.common.Log.logExit(2, "type set to " + type);
        return;
      },

      /**
       * Get the domain type which this queue relates to.
       * 
       * @returns {String} the domain in the requested format String or Object
       */
      getDomainType: function getDomainType_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var result = this.type ? this.type : this.domainType;
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning domain type " + result);
        return result;
      },


      /**
       * Return the domain which this queue relates to.
       * 
       * @param {Boolean} asObject If true, return the domain object as returned by Systems.getDomain. Else
       *        return the domain name.
       * @returns {String} the domain in the requested format String or Object
       */
      getDomain: function getDomain_Queue(asObject){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var result = (asObject ? this._domain : this.domain);
        com.ibm.bpm.social.util.common.Log.logExit(2, "returning domain as " + (typeof result));
        return result;
      },

      /**
       * Get the domain name as string
       * 
       * @type {String}
       */
      getDomainName: function getDomainName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var rc = this.getDomain(false);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return rc;
      },


      /**
       * Set the the related domain of the queue.
       * 
       * @param {Object} domain Either a domain object as returned by Systems.getDomain or the name of the
       *        domain
       * @param {Boolean} noCheck do
       * @type {void}
       */
      setDomain: function setDomain_Queue(domain){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (typeof domain == "string") {
          this.domain = domain;
          this.setDomainDisplayName(domain); // might be overwritten later
          // ensure that we recurse only once
          if (this._systems && !this._recursion) {
            this._recursion = true;
            var domainNames = this._systems.getDomainNames();
            if (dojo.indexOf(domainNames, domain) > -1) {
              this.setDomain(this._systems.getDomain(domain));
            }
          }
          this._recursion = false;
        }
        else {
          this.setDomainDisplayName(domain.displayName);
          this._domain = domain;
          this.domain = domain.name;
          this.setDomainType(this._systems.getDomainType(domain.displayName));
        }

        // domain has changed, we need to update the queue name:
        this.setName(this.name);


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Set the display name for the domain Attention, is owerwritten when using setDomains
       * 
       * @param {String} name name of the queue
       * @type {void}
       */
      setDomainDisplayName: function setDomainDisplayName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!name) {
          var error = new Error("Queue.setDomainDisplayName: got null or empty string as name!");
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }

        this.domainDisplayName = name;

        com.ibm.bpm.social.util.common.Log.logExit(2, "set name to " + this.domainDisplayName);
        return;
      },


      /**
       * Get the display name of the domain.
       * 
       * @return {String} the name
       */
      getDomainDisplayName: function getDomainDisplayName_Queue(){
        return this.domainDisplayName;
      },
      /**
       * Set the display Name of the Queue
       * 
       * @param {String} name name of the queue
       * @type {void}
       */
      setDisplayName: function setDisplayName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        this.displayName = name;

        // deprecated
        this.title = this.displayName;

        com.ibm.bpm.social.util.common.Log.logExit(3, "set name to " + this.displayName);
        return;
      },


      /**
       * Get the display name of the queue.
       * 
       * @return {String} the name
       */
      getDisplayName: function getDisplayName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        var name;
        if (this.displayName) {
          name = this.displayName;
        }
        else if (this.title) {
          name = this.title;
          this.displayName = name;
        }
        else {
          // this.name may contain domain and/or process app name
          name = this.name.split("$#")[0];
          this.displayName = name;
        }

        com.ibm.bpm.social.util.common.Log.logExit(3, "returning: " + name);
        return name;
      },


      /**
       * Set the name of the queue. In a domain setup, the domain is appended automatically to get a unique
       * name.
       * 
       * @param {String} name name of the queue
       * @return {String} the unique name
       */
      setName: function setName_Queue(name){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        if (!name) {
          var error = new Error("Queue.setName: got null or empty string as name!");
          com.ibm.bpm.social.util.common.Log.logError(error.description);
          throw error;
        }

        this.name = name.split("$#")[0]; // this string is in both, domain and appname separator

        if (this.domain) {
          this.name = this.name + com.ibm.bpm.social.widgets.task.util.Queue.DOMAIN_SEPARATOR + this.domain;
        }

        if (this.processAppName) {
          this.name = this.name + com.ibm.bpm.social.widgets.task.util.Queue.APPNAME_SEPARATOR + this.processAppName;
        }

        com.ibm.bpm.social.util.common.Log.logExit(3, "set name to " + name);
        return this.name;
      },

      /**
       * Get the name(id) of the queue.
       * 
       * @return {String} the name
       */
      getName: function getName_Queue(){
        return this.name;
      },

      /**
       * Get the plain name of the queue.
       * 
       * This is either just the "name" attribute, or in a domain setup, the name of the queue without the
       * domain added.
       * 
       * 
       * @type {void}
       */
      getPlainQueueName: function getPlainQueueName_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var pName = this.name.split("$#")[0];
        com.ibm.bpm.social.util.common.Log.logExit(2, "plain name is " + pName);
        return pName;
      },

      /**
       * Get the fully qualified display Name of a queue<br>
       * If domain_prefix is null or there is only one domain used in federation, just return the queue title
       * <br>
       * Returned format: <b>&lt;prefix&gt; - &lt;suffix&gt;</b>
       * 
       * @returns {String} the domain prefixed name, e.g. "Domain X - QT1" or just the Queue displayName
       */

      getQualifiedDisplayName: function getQualifiedDisplayName_Queue(noAppName){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        var domain = this.domainTitle ? this.domainTitle : this.domain;
        var displayName = this.getDisplayName();
        if (domain && this._federationHasMultipleDomains) {
          displayName = domain + " - " + displayName;
        }
        if (this.processAppName && !noAppName) {
          displayName = displayName + " (" + this.processAppName + ")";
        }
        com.ibm.bpm.social.util.common.Log.logExit(3, "returning: " + displayName);
        return displayName;
      },

      /**
       * Create a div with an exclamation mark indication that this queue has an error
       * 
       * @param {String} message the message to be included
       * 
       * @type {String}
       */
      getExceptionHtml: function getExceptionHtml_Queue(message){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var htmlName =
          '\
          <table id="hugo" style="white-space: nowrap;border-spacing: 0;border-collapse: collapsed" title="_title_">\
            <tbody>\
              <tr>\
                <td>\
                  <div class="smallWarning"></div>\
                </td>\
                <td>&nbsp;_queueName_</td>\
              </tr>\
            </tbody>\
          </table>';

        htmlName =
          htmlName.replace("_queueName_", com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this
            .getQualifiedDisplayName()));
        htmlName = htmlName.replace("_title_", message);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return htmlName;
      },


      /**
       * Helper function to determine base on type or content if the given attribute property is associated
       * with a label
       * 
       * @param {Object} property
       * @param {String} type
       * @returns {Boolean} true if also a label exists
       */
      _hasLabelAssociated: function _hasLabelAssociated_Queue(property, type){
        if (property.content == "TASK_DESC.DESCRIPTION" ||
          type == com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG) {
          return false;
        }
        else {
          return true;
        }
      },

      /**
       * Helper function to determine base on type or content if the given attribute is inline
       * 
       * @param {Object} property
       * @param {String} type
       * @returns {Boolean} true if also a label exists
       */
      _isInlineValue: function _isInlineValue_Queue(property, type){
        if (type == com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_PRIORITY_CONFIG ||
          property.content == "TASK.IS_WAIT_FOR_SUB_TK" || property.content == "TASK.IS_ESCALATED") {
          return true;
        }
        else {
          return false;
        }
      },

      /**
       * get the attributes for the queue
       * 
       * @returns {object} return the list of attributes
       */
      getAttributes: function getAttributes_Queue(){
        if (this.attributesLoaded !== true) {
          this._setAttributes();
        }
        return this.item;
      },

      /**
       * Set the attributes describing the queue
       * 
       * @param {Object} attributes JSON object as return from REST call /tasks/query/{queryName}/attributes
       * @type {void}
       */
      _setAttributes: function _setAttributes_Queue(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var priority = null; // TODO: ASZ@MoveToSocial : was EntityList.config.priorities;
        // var pushFilterTypeMapping = null; // TODO: ASZ@MoveToSocial : was EntityList.config.pushFilterTypeMapping;
        var property;
        var attributes = null;
        if (this.attributesLoaded !== true) {
          if (this.queueHandler) {
            attributes = this.queueHandler.getAttributes(this);
          }
        }
        else {
          return;
        }

        this.setDisplayName(attributes.displayName);

        // Reset the changes from previous runs.
        this.item = {
          properties: [],
          sortProperties: []
        };

        var sortHash = {};

        // store sort order
        if (attributes.sortAttributes) {
          for ( var i = 0; i < attributes.sortAttributes.length; i++) {
            var sort = {
              name: attributes.sortAttributes[i].name,
              sortDir: attributes.sortAttributes[i].sortOrder,
              sourceAttribute: attributes.sortAttributes[i].sourceAttribute,
              isSort: true
            };
            this.item.sortProperties.push(sort);
            // keep track for later mapping 
            sortHash[sort.sourceAttribute] = sort;
          }
        }


        for (i = 0; i < attributes.items.length; i++) {
          var propertyMetaData = attributes.items[i];

          var displayed = true;
          if (propertyMetaData.name == "KIND" || propertyMetaData.name == "STATE" ||
            propertyMetaData.name == "CONTAINMENT_CTX_ID" || propertyMetaData.name == "PI_DISPLAY_NAME" ||
            propertyMetaData.name == "PI_STATE" || propertyMetaData.name == "PT_NAME") {
            displayed = false;
          }

          var type = com.ibm.bpm.social.widgets.task.util.Constants.mapPropertyType(propertyMetaData, priority);
          property = {
            description: propertyMetaData.description,
            property: propertyMetaData.name,
            title: com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(propertyMetaData.displayName),
            type: type,
            labelDisplayed: this._hasLabelAssociated(propertyMetaData, type),
            inline: this._isInlineValue(propertyMetaData, type),
            content: propertyMetaData.content,
            displayed: displayed,
            isSortable: false,
            isFilterable: false,
            sortDir: "ASC",
            sourceAttribute: propertyMetaData.sourceAttribute,
            isSort: false
          };

          this.item.properties.push(property);

          // Set the sortable flag for the current property.
          if (typeof propertyMetaData.isSortable == "boolean" && !propertyMetaData.isSortable) {
            property.isSortable = false;
          }
          else {
            property.isSortable = true;
          }

          // do not map PI_DISPLAY_NAME because PI_NAME has the same sourceAttribute -> displayed has to be true
          if (displayed && sortHash[property.sourceAttribute]) {
            // if in the sortHash, saved search defines sort for this property
            // map sortname to property name  
            sortHash[property.sourceAttribute].property = property.property;
          }

          // Set the filterable flag for the current property. 
          if (typeof propertyMetaData.isFilterable == "boolean" && !propertyMetaData.isFilterable) {
            property.isFilterable = false;
          }
          else {
            property.isFilterable = true;
          }
        }


        // TODO ASZ@Move2Social : Do we need this? 
        // set the mapping of the required source attribute to the respective queue attribute name
        //          this.queuePropertiesMap =
        //            myself._createReqiredPropertiesMap(attributes.items,
        //              myself.config[com.ibm.bpm.social.widgets.task.util.Constants.LIST_PROPERTIES]);
        //          };

        // flag that we have retrieved attributes, may be used to prevent a reload via REST API
        this.attributesLoaded = true;

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      }


    });

/**
 * Character used to separate QT name and domain. ("$#$" in not allowed in QT names and saved searches.)
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Queue.DOMAIN_SEPARATOR = "$#$";
com.ibm.bpm.social.widgets.task.util.Queue.APPNAME_SEPARATOR = "$##$";

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.QueueLister"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.QueueLister"] = true;
// QueueLister.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.QueueLister");







/**
 * Virtual base class for listing queues <br>
 * Children must at least set the type to e.g. com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE
 */

dojo.declare("com.ibm.bpm.social.widgets.task.util.QueueLister", null, {

  _queues: null,

  _restHandler: null,

  type: null,

  queuesToIgnore: {
    "portal.savedsearch.inbox": true,
    "portal.savedsearch.help_reqs": true,
    "portal.savedsearch.history": true,
    "IBM.PI_PROCESSLIST_ALL": true,
    "IBM.DEFAULTALLPROCESSLIST_75": true,
    "portal.savedsearch.alerts": true,
    "IBM.DEFAULTALLTASKSLIST_75": true,
    "IBM.PI_TASKLIST_ALL": true
  },


  constructor: function constructor_QueueLister(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);

    if (!this.type) {
      com.ibm.bpm.social.util.common.Log.logError(1, "Error: QueueLister requires type set!");
    }

    // TODO find a place for creation with params, e.g. endpoint
    this._restHandler = com.ibm.bpm.social.util.common.QueryRestHandler.getInstance();

    this._queues = this._getAllQueues();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * 
   * @param name name of the queue
   * @returns {object} queue object with queue meta data from all query table call
   */

  getQueue: function getQueue_QueueLister(name){
    var queue = null;
    if (!this._queues) {
      this._queues = this._getAllQueues();
    }
    var filtered = dojo.filter(this._queues, function(item){
      return item.name === name;
    });
    if (filtered.length === 1) {
      queue = filtered[0];
    }
    return queue;
  },

//  /**
//   * 
//   * @param queue queue object
//   * @returns {object} queue object where the attributes are set
//   */
//  setQueueAttributes: function setQueueAttributes(queue){
//    if (queue) {
//      // If not already done, load attributes for Queue, 
//      // and pass them into the queue (invoking setAttributes as callback)
//      if (queue.attributesLoaded !== true) {
//        this._restHandler.getAttributesForQueryTable(this.type, queue.getPlainQueueName(), queue.processAppName, function(data){
//          queue.setAttributes(data);
//        }, dojo.hitch(this, this._getAttributesForQueueErrorCallback), true, queue.domain);
//      }
//    }
//    return queue;
//  },

  /**
   * 
   * @param queue queue object
   * @returns {object} queue attributes where the attributes are set
   */
  getAttributes: function getAttributes_QueueLister(queue){
    var attributes = null;
    if (queue) {
      // If not already done, load attributes for Queue, 
      // and pass them into the queue (invoking setAttributes as callback)

      if (queue.attributesLoaded !== true) {
        var load = function(data){
          attributes = data;
        };
        this._restHandler.getAttributesForQueryTable(this.type, queue.getPlainQueueName(), queue.processAppName,
          this.type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE, load, dojo.hitch(this,
            this._getAttributesForQueueErrorCallback), true, queue.domain);
      }
    }
    return attributes;
  },


  /**
   * Load all task queues and return only the custom queues. Thus filter out the list of bSpace and WLE
   * internal queues.
   * 
   * {object} List of queues
   */
  getCustomQueues: function getCustomQueues_QueueLister(){
    var allQueues = this._queues;
    if (!allQueues) {
      allQueues = this._getAllTaskQueues();
    }
    var queues = [];
    if (allQueues) {
      for ( var k in allQueues) {
        var queue = allQueues[k];
        if (queue && queue.kind == "SAVED_SEARCH" && this.queuesToIgnore[queue.name] !== true) {
          queues.push(queue);
        }
      }
    }
    return queues;
  },

  /**
   * Return all QTs available
   * 
   * @returns {Queues[]} if no qt is found, returns null otherwise. If null is returned, details may be found
   *          in
   *          <li> this._errorObject.data </li>
   *          <li> this._errorObject.ioArgs </li>
   */

  _getAllQueues: function _getAllQueues_QueueLister(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var i, j, qtTitle;
    var allQueryTables = null;
    this._retrievedQTs = null;
    var systems = this._restHandler.getSystems();
    var domainNames = systems.getDomainNames();

    // cases:
    // 1) domains.length=0 || isApiTypeFederated==false 
    //   - Single server setup (one WLE/WPS) : query w/o domain name
    //   - No domain to be displayed 
    // 
    // 2) domains.length=1:
    //   - domain setup with only one system: query with domain name but 
    //   - No domain to be displayed
    // 
    // 3) domains.length=n
    //  - domain setup with multiple systems: query with domain name and
    //  - domain to be displayed
    // standard handling
    // retrieve all QTs 
    if (!systems.isApiTypeFederated()) {
      this._restHandler.getAllQueryTables(this.type, null, dojo.hitch(this, this._getAllQTLoadCallback), dojo.hitch(this,
        this._getAllQTErrorCallback));

      // this._retrievedQTs is null in case an error occurred at the REST API call
      if (this._retrievedQTs) {
        allQueryTables = this._retrievedQTs;
      }
    }
    else {
      // federated TODO ???
      allQueryTables = [];

      for (i = 0; i < domainNames.length; i++) {
        var domainName = domainNames[i];

        // only retrieve queries if
        // (
        // no process application is set and the domain does not include any icm systems
        // OR
        // a process application is set and the domain contains at least one system with that process application
        // ) AND
        // widget type is com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE or domain does not contain ICM
        if (systems.isDomainWleOnly(domainName)) {
          // federation only for WLE only domains
          // do not send the attributes for WLE or mixed domains as they are not necessary in that case
          this._restHandler.getAllQueryTables(this.type, null, dojo.hitch(this, this._getAllQTLoadCallback), dojo.hitch(
            this, this._getAllQTErrorCallback), domainName);
        }

        if (this._errorObject) {
          // TODO SaS: as long as all=true is not yet fully available and not used by us we need to see 404 as a valid response
          // (happens if an application name is not known in all systems of the domain)
          if (this._errorObject.ioArgs.xhr.status == 404) {
            this._errorObject = null;
          }
          else {
            // an unexected error occurred
            allQueryTables = null;
            break;
          }
        }
        else {
          // we must update the queue be identified with the domain and process application
          for (j = 0; j < this._retrievedQTs.length; j++) {
            this._retrievedQTs[j].domain = domainName;
          }
          this._retrievedQTs = this._checkEligibleQTs(this._retrievedQTs, domainName);

          allQueryTables = allQueryTables.concat(this._retrievedQTs);
        }
      }
    }

    if (allQueryTables) {
      for (j = 0; j < allQueryTables.length; j++) {
        // convert raw objects to Queues 
        allQueryTables[j] = new com.ibm.bpm.social.widgets.task.util.Queue(allQueryTables[j], systems);
      }

      // Sort on name
      if (allQueryTables.length > 1) {
        // sort on name
        allQueryTables.sort(function(qt1, qt2){
          var qt1Name = qt1.getQualifiedDisplayName();
          var qt2Name = qt2.getQualifiedDisplayName();
          if (qt1Name < qt2Name) //sort string ascending
          {
            return -1;
          }
          if (qt1Name > qt2Name) {
            return 1;
          }
          return 0;
        });

      }
    }

    var found_queues = [];
    for (j = 0; j < allQueryTables.length; j++) {
      found_queues.push(allQueryTables[j].getQualifiedDisplayName());
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, "found : " + found_queues.join(", "));
    return allQueryTables;
  },

  /**
   * XHR helper for _getAllQueues<br>
   * Returns a list of QTs in _allQueryTables
   * 
   * @type {void}
   */
  _getAllQTLoadCallback: function _getAllQTLoadCallback_QueueLister(responseObject, ioArgs){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (responseObject) {
      this._retrievedQTs = responseObject.items;
      this._errorObject = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * XHR helper for _getAllQueues
   * 
   * @type {void}
   */
  _getAllQTErrorCallback: function _getAllQTErrorCallback_QueueLister(data, ioArgs){
    this._errorObject = {};
    this._errorObject.data = data;
    this._errorObject.ioArgs = ioArgs;
    this._retrievedQTs = null;
    com.ibm.bpm.social.util.common.Log.logError(1, "", data);
  },

  /**
   * XHR helper for _getAttributesForQueue
   * 
   * @type {void}
   */
  _getAttributesForQueueErrorCallback: function _getAttributesForQueueErrorCallback_QueueLister(data, ioArgs){
    this._errorObject = {};
    this._errorObject.data = data;
    this._errorObject.ioArgs = ioArgs;
    com.ibm.bpm.social.util.common.Log.logError(1, "", data);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ProcessQueues"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ProcessQueues"] = true;
// ProcessQueues.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.ProcessQueues");







dojo.declare("com.ibm.bpm.social.widgets.task.util.ProcessQueues", [com.ibm.bpm.social.widgets.task.util.QueueLister], {

  type: com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE,

  statics: {
    _instance: null,
    _getInstance: function(){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.widgets.task.util.ProcessQueues();
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.util.ProcessQueues.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TaskQueues"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TaskQueues"] = true;
// TaskQueues.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011,2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.TaskQueues");







dojo.declare("com.ibm.bpm.social.widgets.task.util.TaskQueues", [com.ibm.bpm.social.widgets.task.util.QueueLister], {

  type: com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE,

  statics: {
    _instance: null,
    _getInstance: function(){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.widgets.task.util.TaskQueues();
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.util.TaskQueues.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TabConfiguration"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TabConfiguration"] = true;
// TabConfiguration.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.util.TabConfiguration");



/**
 * based on LightWeightTabConfiguration.js Version: 1.22 11/10/05 16:22:49
 * 
 * This class contains information for one tab of the com.ibm.bpm.social.widgets.task.util.TabContainer
 * 
 * This informations are used for generating a tab inside the lightweight tab container
 */
dojo.declare("com.ibm.bpm.social.widgets.task.util.TabConfiguration", null,
  {
    //name of the tab - display in the lightweight tab container
    tabName: "",
    //index of the tab - is used for creating a unique name inside the tab container - should be
    //unique for every tab of a tab container
    id: 0,
    //is the tab active - only active tabs will be created
    isActive: false,
    //position of the tab inside the tab container - is set during the creation of the tabs
    position: 0,
    //context for doing call back methods
    context: null,
    //name of the function inside the passed context that has to be called when the tab is loaded
    //has to be unique for the different tabs, because it is used as identifier for the content of the tab
    loadFunction: "",
    //text of the tool tip that is shown when the mouse over event for the tab is send
    tooltip: "",
    //id of the widget that contains the tab - this information is necessary for generating a unique id
    widgetID: "",
    //mark if the content of the current tab is shown
    //if no tab is marked for showing the content after open it - the content of the first tab is shown
    //else only one tab should be marked as open - if more than one tab is marked as the content of the last one will be shown
    contentIsShownAfterInit: false,

    // user defined properties for the tab
    custom: null,


    //the created tab (dom tree)
    tab: null,

    constructor: function constructor_TabConfiguration(tabName, id, isActive, context, loadFunction, tooltip, widgetID,
      custom){
      this.tabName = tabName;
      this.id = id;
      this.isActive = isActive;
      this.context = context;
      this.loadFunction = loadFunction;
      this.tooltip = tooltip;
      this.widgetID = widgetID;
      if (custom) {
        this.custom = custom;
      }

      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        this.tooltip = null;
      }
    },

    setContentIsShownAfterInit: function setContentIsShownAfterInit_TabConfiguration(resetFlag){
      if (resetFlag === false) {
        this.contentIsShownAfterInit = false;
      }
      else {
        this.contentIsShownAfterInit = true;
      }
    },

    setPosition: function setPosition_TabConfiguration(position){
      this.position = position;
    }
  });

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.TabContainer"] = true;
// TabContainer.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.util.TabContainer");






/**
 * based on LightWeightTabContainer.js Version: 1.22 11/10/05 16:22:49
 * 
 * this class implements an lightweight tab container
 * 
 * The content of the tabs is created only when the tab is selected. After creating the content of a tab and
 * selecting a new one the old content is hidden, but not deleted. After selecting the tab again the content
 * has not to be created again.
 * 
 * For the creation of the content of the tab a context a function (loadFunction) has to be set inside the
 * TabConfiguration-object.
 * 
 * Furthermore a function has to be provided inside the class passed as context that checks if the tab has to
 * be reloaded or not. If the tab has to be reloaded - all other tabs will also be reloaded. This function has
 * to be called as reloadAllTabs and returns a boolean value.
 * 
 * The lightweight tab container has also a function for resizing the tab content.
 * 
 * For getting know if a tab has been changed the function showTab is provided. This function is only provided
 * as point for connecting.
 * 
 * Furthermore the functionality for showing an error on a tab is provided. Therefore the function markError
 * has to be called. The error-symbol is only shown if the content of the tab is currently not shown. For
 * removing the error-symbol the function removeError has to be called.
 * 
 * using this class - you have first to call the function configuration and pass the parent node where the
 * lightweight tab container has to be added and an array of TabConfiguration-objects for all tabs - than you
 * have to call the function createTabs
 */
dojo.declare("com.ibm.bpm.social.widgets.task.util.TabContainer", [dijit._Widget,
                                                                   dijit._Templated], {

  //parent node where the nodes of the lightweight tab container will be added
  parentNode: null,
  //array containing the TabConfiguration-objects for all tabs
  tabInformations: null,
  //list of all tab nodes
  tabList: null,

  //needed, when an new tab is selected
  lastTab: null,

  rtl: false,

  tabHeaderDiv: null,
  paramTabHeaderDiv: false,

  tabContent: null,
  paramTabContentDiv: false,

  // template of the lightweight tab container    
  // To ensure that the css definitions from LightWeightTab.css have precedence (e.g. TaskEditor.css)
  // the class "lightWeightTabContainer" has been added to tabContainer for more specific css rules.
  templateString: //'<div class="lightWeightTabContainer bspace-style bspace-text5" dojoAttachPoint="tabContainer">\
  '<div class="lightWeightTabContainer" dojoAttachPoint="tabContainer">\
      </div>',

  constructor: function constructor_TabContainer(params){
    var loc = dojo.locale;
    if ((loc.indexOf("iw") === 0) || (loc.indexOf("he") === 0) || (loc.indexOf("ar") === 0)) {
      this.rtl = true;
    }

    if (params) {
      if (params.tabHeaderDiv) {
        this.tabHeaderDiv = params.tabHeaderDiv;
        this.paramTabHeaderDiv = true;
      }

      if (params.tabContent) {
        this.tabContent = params.tabContentDiv;
        this.paramTabContentDiv = true;
      }
    }

  },

  postCreate: function(){
    /* create tab header */
    if (!this.paramTabHeaderDiv) {
      this.tabHeaderDiv = document.createElement("div");
      dojo.addClass(this.tabHeaderDiv, "lightWeightTabContainer");
      this.tabContainer.appendChild(this.tabHeaderDiv);
    }
    this.tabHeader = document.createElement("div");
    dojo.addClass(this.tabHeader, "tabHeader");
    this.tabHeaderDiv.appendChild(this.tabHeader);

    /* create tab content */
    if (!this.paramTabContentDiv) {
      this.tabContent = document.createElement("div");
      dojo.addClass(this.tabContent, "tabContent");
    }
    this.tabContainer.appendChild(this.tabContent);
  },

  /**
   * this function is needed to set the configuration for the lightweight tab container
   */
  setTabConfiguration: function setTabConfiguration_TabContainer(tabInformations){
    this.parentNode = this.tabContainer.parentNode;
    this.tabInformations = tabInformations;

    //array containing all connects to disconnect them
    this.allCreatedConnects = new Array();

    //array containing all widgets which are not part of the dom tree
    this.allWidgets = new Array();
  },

  /**
   * This function displays the active tab and hides all others. It also set the style for the active tab and
   * remove it from the former active tab. It is called after selecting a tab.
   */
  selectTab: function selectTab_TabContainer(tabInformation, tab){

    //check if same tab is selected
    if (tabInformation.tabName === this.lastTab.tabName) {
      return;
    }

    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0) {
      //remove class from last active tab
      dojo.removeClass(activeTab[0], "activeLightWeightTab");
      dijit.setWaiState(activeTab[0], "selected", "false");
      activeTab[0].firstChild.lastChild.tabIndex = -1;
    }

    //mark current tab as active
    dojo.addClass(tab, "activeLightWeightTab");
    dijit.setWaiState(tab, "selected", "true");
    tab.firstChild.lastChild.tabIndex = 0;

    //hide the content of the former active tab
    this.hideTabContent();
    //show the content of the new active tab (this could include the creation of the content)
    this.showTabContent(tabInformation);

    //set new tab to last tab
    this.lastTab = tabInformation;
  },

  /**
   * This function is called if a key is pressed. if the key is the right or left arrow the tab is changed.
   */
  _changeTabByKey: function _changeTabByKey_TabContainer(event, tab, sortedTabInformations, tabInformation){
    var keys = dojo.keys;

    switch (event.keyCode) {
      case (!this.rtl && keys.RIGHT_ARROW) || (this.rtl && keys.LEFT_ARROW) :
        var nextSibling = tab.nextSibling;

        //check if a next tab exists
        if (nextSibling) {
          // There is a separator "|" between tabs!
          var nextTab = nextSibling.nextSibling;
          nextTab.firstChild.lastChild.focus();

          //call the function for handling a change of the tab
          this.selectTab(sortedTabInformations[tabInformation.position + 1], nextTab);
        }
        break;
      case (!this.rtl && keys.LEFT_ARROW) || (this.rtl && keys.RIGHT_ARROW) :
        var previousSibling = tab.previousSibling;

        //check if previous tab exists
        if (previousSibling) {
          // There is a separator "|" between tabs!
          var previousTab = previousSibling.previousSibling;
          previousTab.firstChild.lastChild.focus();

          //call the function for handling a change of the tab
          this.selectTab(sortedTabInformations[tabInformation.position - 1], previousTab);
        }
        break;
      default:
        break;
    }
  },

  /**
   * this function creates the tab container and the content of the first tab
   */
  createTabs: function createTabs_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.createTabs entry");

    var myself = this;

    //creating a node for surrounding the tabs
    myself.tabList = document.createElement("div");
    dojo.addClass(myself.tabList, "lightWeightTabList");
    dojo.addClass(myself.tabList, "lightWeightTabLink");

    dijit.setWaiRole(myself.tabList, "tablist");

    //used for set the position of the tab inside the TabConfiguration-object
    var position = 1;

    //this variable is used for marking the tab for which the content is shown as the active one
    var activeTab = null;

    //used for navigating between tabs with arrows
    var sortedTabInformations = {};

    //iterate through all tabs and create it
    dojo.forEach(myself.tabInformations, function(tabInformation, index){

      //check if the tab should be active - if not it will not be created
      if (tabInformation.isActive) {
        //id of the node - consisting of the widget id and the index of the tab
        var id = tabInformation.widgetID + "_" + tabInformation.id;

        //set position in tabInformation - is used for navigate with arrows through tabs
        tabInformation.setPosition(position);
        sortedTabInformations[position] = tabInformation;
        position = position + 1;

        // create new tab
        //first create a div element
        tabInformation.tab = document.createElement("div");
        tabInformation.tab.id = id;
        tabInformation.tab.title = tabInformation.tooltip;
        tabInformation.tab.style.display = "inline-block";
        dojo.addClass(tabInformation.tab, "lightWeightTab");
        dijit.setWaiRole(tabInformation.tab, "tab");

        //create a link containing the name of the tab
        var link = document.createElement("a");
        link.href = "javascript:{}";
        link.innerHTML = tabInformation.tabName;
        //wrap the created link
        var linkWrapper = document.createElement("div");
        linkWrapper.style.display = "inline-block";
        dojo.addClass(tabInformation.tab, "lightWeightTabLinkDiv");
        linkWrapper.appendChild(link);
        tabInformation.tab.appendChild(linkWrapper);
        tabInformation.tab.focusNode = link;

        // Add a11y stuff (high contrast) for the error symbol				
        var errorText = document.createElement("span");
        errorText.innerHTML = "!";
        errorText.style.display = "none";
        dojo.addClass(errorText, "a11yErrorText");

        tabInformation.tab.appendChild(errorText);

        //add event to tab - containing the function that should process in case
        //of selecting a tab
        myself.allCreatedConnects.push(dojo.connect(link, "onclick", dojo.hitch(myself, "selectTab", tabInformation,
          tabInformation.tab)));

        //set tab index for changing the order when using tab key
        link.tabIndex = -1;

        // Mark the first tab as open
        //content for the tab is created later
        if (index === 0 || tabInformation.contentIsShownAfterInit) {
          myself.lastTab = tabInformation;
          activeTab = tabInformation.tab;
        }

        // Create separator between tabs
        if (index > 0) {
          var sep = document.createElement("span");
          dojo.addClass(sep, "lightWeightTabSeparator");
          sep.innerHTML = "|";
          myself.tabList.appendChild(sep);
        }

        // Navigate through tabs by using LEFT and RIGHT arrow keys.
        myself.allCreatedConnects.push(dojo.connect(tabInformation.tab, "onkeydown", function(event){
          myself._changeTabByKey(event, tabInformation.tab, sortedTabInformations, tabInformation);
        }));

        //add the tab to the div containing all tabs
        myself.tabList.appendChild(tabInformation.tab);

      }
    });

    //adds all tabs to the node containing the tabs
    myself.tabHeader.appendChild(myself.tabList);

    //adds the tab container to the defined parent node
    myself.parentNode.appendChild(myself.tabContainer);

    //mark the tab as active
    dojo.addClass(activeTab, "activeLightWeightTab");
    activeTab.firstChild.lastChild.tabIndex = 0;
    dijit.setWaiState(activeTab, "selected", "true");

    //creates the content of the first tab - that is shown after creating the tab container
    myself.showTabContent(myself.lastTab);

    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.createTabs exit");
  },

  /**
   * this function is responsible for showing the content of a tab - for example after changing a tab
   */
  showTabContent: function showTabContent_TabContainer(tabInformation){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var context = tabInformation.context;
    var loadFunction = tabInformation.loadFunction;
    var id = "tab_" + tabInformation.id + "_" + loadFunction;
    var contentNode = null;

    //check if all tabs has to be reloaded
    var reloadTabs = dojo.hitch(context, "reloadAllTabs")();
    var allContentNodes = this.tabContent.childNodes;

    //if all tabs has to be reloaded first the old content of the tabs has to be destroyed
    if (reloadTabs) {
      //implementation this way because for each does not work correct here
      //for destroying the content iterate through all nodes of the tabContent area and destroy them
      if (allContentNodes.length > 0) {
        var node = allContentNodes[0];
        var nextNode = null;

        while (node) {
          nextNode = node.nextSibling;

          //destroy all widgets and elements of the content of the tab
          this.destroyTabContent(node);

          //destroy the div that has not been destroyed yet
          dojo._destroyElement(node);

          //read the next node
          node = nextNode;
        }
      }

    }
    else {
      //read the content of the selected tab if it has been already generated
      //therefore read all child-nodes of the content area, iterate through the nodes 
      //and check if the set class name is equivalent to the id
      dojo.forEach(allContentNodes, function(node){
        if (node.id === id) {
          contentNode = node;
        }
      });
    }

    //if the content of the selected tab has not been generated
    if (!contentNode) {

      //create the div element containing the content
      var divTab = document.createElement("div");

      //add the id to the element - so it can be identified later
      divTab.id = id;
      dojo.addClass(divTab, "tabContent");
      dijit.setWaiRole(divTab, "tabpanel");

      //append the div to the content area
      this.tabContent.appendChild(divTab);

      //if the function that has to be called when creating the content of the tab is set 
      //call it
      if (loadFunction) {
        //the div is passed for adding the content of the tab inside it
        dojo.hitch(context, loadFunction)(divTab, tabInformation);
      }
      else {
        //log an message, because the content for the tab could not be generated
        com.ibm.bpm.social.util.common.Log.log(2, "For the current tab no load function is passed");
      }
      contentNode = divTab;
    }

    //display the content of the selected tab
    contentNode.style.display = "block";
    //call the function so that inside another context it can be recognized, that the tab has been changed
    this.showTab(tabInformation);
    //this field is used to check if in the content area - the content of a tab is shown or not
    this._isVisible = true;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  //function for connecting to recognize if a tab is changed - this is necessary, because in case
  //of that the content has already been loaded only the content is shown and the load function has
  //not been called
  //in some cases after a tab change some functions has to be called for example to change the size
  //of the content area
  showTab: function showTab_TabContainer(tabInformation){}, // TODO tabInformation???

  /**
   * this function is used to hide the content area
   */
  hideTabContent: function hideTabContent_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.hideTabContent entry");

    //check if the content of an area is visible
    //if nothing is visible - no area has to be hidden
    if (this._isVisible) {

      //iterate through all child nodes of the content area and hide them
      var allContentNodes = this.tabContent.childNodes;

      dojo.forEach(allContentNodes, function(node){
        node.style.display = "none";
      });
      this._isVisible = false;
    }
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.hideTabContent exit");
  },

  /**
   * this function is used to show the content of the first tab of the lightweight tab container
   */
  showFirstTab: function showFirstTab_TabContainer(){
    var tabInformation = this.tabInformations[0];
    var tab = tabInformation.tab;

    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0) {
      //remove class from last active tab
      dojo.removeClass(activeTab[0], "activeLightWeightTab");
      activeTab[0].firstChild.lastChild.tabIndex = -1;
    }

    //mark current tab as active
    dojo.addClass(tab, "activeLightWeightTab");
    tab.firstChild.lastChild.tabIndex = 0;

    //hide the content that is shown at the moment
    this.hideTabContent();
    //show the content of the first tab
    this.showTabContent(tabInformation);

    //set new tab to last tab
    this.lastTab = tabInformation;
  },

  /**
   * this function is used to trigger a reload of the content of the tabs even no other tab has been selected
   * 
   * this function could be used after an update if the data of the content should contain the latest data
   */
  reloadTabs: function reloadTabs_TabContainer(){
    //the content could only be updated, if the content of a tab is shown
    if (this.lastTab) {
      this.showTabContent(this.lastTab);
    }
  },

  /**
   * this function is used to destroy the content of a tab - for example for reloading the content
   */
  destroyTabContent: function destroyTabContent_TabContainer(div){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroyTabContent entry");

    //read all widgets of the passed node
    var allWidgets = div.getElementsByTagName('*');
    var length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    if (div.hasChildNodes()) {
      //iterate through all child nodes of the passed div to remove them
      while (div.childNodes.length > 0) {
        div.removeChild(div.firstChild);
      }
    }
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroyTabContent exit");
  },

  /**
   * this function is used to mark an error at a tab - if the content of the tab is not shown a error-symbol
   * is shown behind the tab
   */
  markError: function markError_TabContainer(tabName){
    var i;
    var tabInformation;
    var id;

    //iterate through the tabConfiguration-objects
    for (i = 0; i < this.tabInformations.length; i++) {
      tabInformation = this.tabInformations[i];

      //check if the current tab should be marked as error
      if (tabInformation.tabName === tabName) {

        //read the node that should to marked as error
        id = "#" + tabInformation.widgetID + "_" + tabInformation.id;
        var nodes = dojo.query(id, this.tabHeader);

        //add the class for showing the error-symbol to the node
        if (nodes.length > 0) {
          dojo.addClass(nodes[0], "lightWeigthTabShowError");
        }
        return;
      }
    }
  },

  /**
   * this function is used to remove a error-symbol from a tab
   */
  removeError: function removeError_TabContainer(tabName){
    var i;
    var tabInformation;
    var id;

    //iterate through the tabConfiguration-objects
    for (i = 0; i < this.tabInformations.length; i++) {
      tabInformation = this.tabInformations[i];

      //check if from the current tab error-symbol should be removed
      if (tabInformation.tabName === tabName) {

        //read the node where the error-symbol should remove
        id = "#" + tabInformation.widgetID + "_" + tabInformation.id;
        var nodes = dojo.query(id, this.tabHeader);
        if (nodes.length > 0) {
          //remove the class for showing the error-symbol to the node
          dojo.removeClass(nodes[0], "lightWeigthTabShowError");
        }
        return;
      }
    }
  },

  /**
   * this function resize the lightweight tab container and returns an object containing the height and width
   * of the content pane
   */
  resizeContentPaneDimension: function resizeContentPaneDimension_TabContainer(height){
    com.ibm.bpm.social.util.common.Log.log(1,
      "com.ibm.bpm.social.widgets.task.util.TabContainer.resizeContentPaneDimension entry");

    //change the height of the lightweight tab container
    this.domNode.style.height = height + "px";

    var width;

    //calculate the height of the content area of the lightweight tab container
    height = height - 25;
    if (!this.paramTabHeaderDiv) {
      if (dojo.isIE < 8) {
        height = height - this.tabHeaderDiv.offsetHeight;
      }
      else {
        height = height - this.tabHeaderDiv.offsetHeight - this.tabHeader.offsetHeight;
      }
    }
    if (height > 0) {
      //set the height of the content area
      this.tabContent.style.height = (height) + "px";

      //subtract the padding of the content area for resizing the content inside
      height = this.tabContent.offsetHeight;
      height = height - 24;
      width = this.tabContent.offsetWidth;
      width = width - 24;
    }
    else {

      //calculate the result even if the content area is not resized
      height = this.tabContent.offsetHeight;
      height = height - 24;
      width = this.tabContent.offsetWidth;
      width = width - 24;
    }
    var dimension = {
      h: height,
      w: width
    };
    com.ibm.bpm.social.util.common.Log.log(1,
      "com.ibm.bpm.social.widgets.task.util.TabContainer.resizeContentPaneDimension exit");
    return dimension;
  },

  /**
   * this function is used to destroy all widgets of the lightweight tab container
   */
  destroy: function destroy_TabContainer(){
    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroy entry");

    //disconnect the saved connects
    var i;
    for (i in this.allCreatedConnects) {
      dojo.disconnect(this.allCreatedConnects[i]);
    }
    this.allCreatedConnects = new Array();

    //destroy all widgets that are not part of the dom node
    for (i in this.allWidgets) {
      if (this.allWidgets[i].destroyRecursive) {
        this.allWidgets[i].destroyRecursive();
      }
      else {
        this.allWidgets[i].destroy();
      }
      delete this.allWidgets[i];
    }
    this.allWidgets = new Array();

    //read all elements of the lightweight tab content
    var allWidgets = this.tabContent.getElementsByTagName('*');
    var length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    if (this.tabContainer.hasChildNodes()) {
      // remove tab header div
      if (!this.paramTabHeaderDiv) {
        this.tabContainer.removeChild(this.tabHeaderDiv);
      }
      // remove tab content div
      if (!this.paramTabContentDiv) {
        this.tabContainer.removeChild(this.tabContent);
      }
    }

    //read all elements of the lightweight tab header
    allWidgets = this.tabHeaderDiv.getElementsByTagName('*');
    length = allWidgets.length - 1;

    //iterate through all widgets and destroy them
    while (length >= 0) {
      if (allWidgets[length] && dijit.byId(allWidgets[length].id)) {
        if (dijit.byId(allWidgets[length].id).destroyRecursive) {
          dijit.byId(allWidgets[length].id).destroyRecursive();
        }
        else {
          dijit.byId(allWidgets[length].id).destroy();
        }
      }
      length = length - 1;
    }

    this.inherited("destroy", arguments);

    com.ibm.bpm.social.util.common.Log.log(1, "com.ibm.bpm.social.widgets.task.util.TabContainer.destroy exit");
  },

  focusActiveTab: function focusActiveTab_TabContainer(){
    //read active tab
    var activeTab = dojo.query(".activeLightWeightTab", this.tabList);
    if (activeTab.length > 0 && activeTab[0].focusNode) {
      // focus header node
      activeTab[0].focusNode.focus();
    }

  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.TasksContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.TasksContainer"] = true;
// TasksContainer.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.TasksContainer");
























dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.TasksContainer",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: '<div class="bpm-task-list-container" dojoType="dijit.layout.ContentPane">\
    	  <div role="button" dojoAttachPoint="startPageButton"></div>\
        <div dojoAttachPoint="tabContainer"></div>\
        </div>',

      widgetsInTemplate: true,

      _pageHandler: null,

      _listControllers: null,

      constructor: function constructor_TasksContainer(params){
        if (params) {
          if (params.tabHeaderDiv) {
            this._tabHeaderDiv = params.tabHeaderDiv;
          }
        }
        this._subscribedChannels = [];
      },

      postCreate: function postCreate_TasksContainer(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //com.ibm.bpm.social.util.common.Util.loadStylesheet("widgets/task/themes/TasksContainer.css");

        this._pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();

        var newStartPageButton = this._pageHandler.createStartPageIcon();
        this.startPageButton.parentNode.replaceChild(newStartPageButton, this.startPageButton);
        this.startPageButton = newStartPageButton;

        this.connect(this.startPageButton, "onclick", this._onStartPageButtonClick);
        this.connect(this.startPageButton, "onkeypress", function(e){
          if (e.keyCode == dojo.keys.ENTER) {
            this._onStartPageButtonClick(e);
          }
        });

        this.containerNode = this.domNode;
        // array for the tabs
        this.tabs = [];
        this._connectsMenu = [];

        // an array for the created listController widgets
        this._listControllers = [];

        // when the Critical Path Management updates something, we need to refresh
        this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED, dojo.hitch(
          this, this.reloadActiveTab)));

        // suspend automatic refresh if dashboards are shown
        this._subscribedChannels.push(dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", dojo.hitch(this,
          function(args){
            if (args !== null && args.showing !== "workPage") {
              // notify actual tab that it will be hidden
              this._onHideTab(this.tabContainerWidget.lastTab.id);
            }
            // no notification needed if workPage is shown again because the tab will be refreshed
          })));


        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Refresh/Reload all tabs of the TasksContainer
       */
      refresh: function refresh_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        try {
          if (!this.tabs || this.tabs.length === 0) {
            this._createContent();
          }
          this.tabContainerWidget.reloadTabs();
        }
        catch (e) {
          com.ibm.bpm.social.util.common.Log.logError(0, "refresh_TasksContainer", e);
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(e.message, /* details */e.stack, /* callback */null);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },


      /**
       * Create the lightweight tab container and it's tabs using Tab Configuration objects
       * 
       * @type void
       */
      _createContent: function _createContent_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // start page handling
        var storedStartPageConfig = this._pageHandler.getStoredPageSettings();

        var startPageId = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS;
        if (storedStartPageConfig.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
          startPageId = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        }


        // Create My Tasks and Saved Searches tab  (lightweight tabs)
        // first prepare for a TabConfiguration-object for every shown tab - the information
        // of this objects are used for creating the tabs
        // new com.ibm.bpm.social.widgets.task.util.TabConfiguration(tabName, id, isActive, context, loadFunction, tooltip, widgetID, custom)

        // My Tasks Tab 
        var myTasksControllerConfig = com.ibm.bpm.social.widgets.task.config.ControllerConfig.getMyTasksConfig();
        var queue = this._getQueue(myTasksControllerConfig.viewConfig.queueName);
        if (queue !== null) {
          myTasksControllerConfig.queue = queue;
          this.tabs.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS,
            title: this._nlsResources[myTasksControllerConfig.viewConfig.titleKey],
            tooltip: this._nlsResources[myTasksControllerConfig.viewConfig.tooltipKey],
            controllerConfig: myTasksControllerConfig
          });
          var listController = new com.ibm.bpm.social.widgets.task.list.TasksListController({
            typeId: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS
          });
          if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS) {
            //listController.setSearchText(storedStartPageConfig.searchFilterString);
            myTasksControllerConfig.selectedFilter = storedStartPageConfig.filterConfigId;
          }
          // track the state of the controller's default page config
          this.connect(listController, "onLoadItemsSuccess", this._checkAndUpdateStartPageIcon);
          this.connect(listController, "onFocusNeeded", this._setFocusToActiveTab);
          this._listControllers.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS,
            controller: listController
          });
        }
        else {
          var message = this._nlsResources["task_error_saved_search_does_not_exists"];
          message = dojo.string.substitute(message, [myTasksControllerConfig.viewConfig.queueName]);
          var details = this._nlsResources["task_error_saved_search_does_not_exists_myTasks_details"];
          details = dojo.string.substitute(details, [myTasksControllerConfig.viewConfig.queueName]);
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, /* details */details, /* callback */null);
          com.ibm.bpm.social.util.common.Log.consoleError(message);
        }


        // Saved Searches Tab Configuration

        this._dropDown = document.createElement("span");
        var queueNotFound = false;

        // check first, if we need custom views
        var showSavedSearchesTab = this._createQueueDropDown(this._dropDown);
        if (showSavedSearchesTab) {
          var customViewControllerConfig = com.ibm.bpm.social.widgets.task.config.ControllerConfig.getCustomViewsConfig();
          // set the default queue
          queue = null;
          if (storedStartPageConfig.savedSearchName) {
            queue = this._getQueue(storedStartPageConfig.savedSearchName);
          }
          if (queue) {
            customViewControllerConfig.queue = queue;
            // TODO ASZ: Map  SortAttribs from user profile to the queue
            // DOES NOT WORK currently
//            if (defaultPage.sortAttributes.length > 0) {
//              queue.sortAttributes = [];
//              for ( var i = 0; i < defaultPage.sortAttributes.length; i++) {
//                queue.sortAttributes.push({
//                  name: defaultPage.sortAttributes[i].property,
//                  sortOrder: defaultPage.sortAttributes[i].sortDir
//                });
//              }
//            }

          }
          else {
            queueNotFound = true;
            customViewControllerConfig.queue = this._queues[0];
          }


          this.tabs.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES,
            title: this._nlsResources[customViewControllerConfig.viewConfig.titleKey],
            tooltip: this._nlsResources[customViewControllerConfig.viewConfig.tooltipKey],
            controllerConfig: customViewControllerConfig
          });
          var tableController = new com.ibm.bpm.social.widgets.task.list.TasksListController({
            typeId: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES
          });
          if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES) {
            // tableController.setSearchText(storedStartPageConfig.searchFilterString);
          }
          // track the state of the controller's default page config
          this.connect(tableController, "onLoadItemsSuccess", this._checkAndUpdateStartPageIcon);
          this.connect(listController, "onFocusNeeded", this._setFocusToActiveTab);
          this._listControllers.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES,
            controller: tableController
          });
        }
        this._tabHeaderDiv.style.display = "none";

        //create an array containing the TabConfiguration-objects
        var tabs = new Array();

        for ( var i = 0; i < this.tabs.length; i++) {
          var tab = this.tabs[i];
          var tabConfig =
            new com.ibm.bpm.social.widgets.task.util.TabConfiguration(tab.title /* tabName */,
              tab.id /* tab id */,
              true /* isActive */,
              this /* context */,
              "_initTab" /* loadFunction */,
              tab.tooltip /* tab tooltip */,
              this.id /* widget id */,
              tab.controllerConfig /* our user defined properties stored in TabConfiguration.custom */);

          tabs.push(tabConfig);
          if (tab.id == startPageId) {
            tabConfig.setContentIsShownAfterInit();
          }
        }

        //create the lightweight tab container
        this.tabContainerWidget = new com.ibm.bpm.social.widgets.task.util.TabContainer({
          tabHeaderDiv: this._tabHeaderDiv
        }, this.tabContainer);
        this.tabContainerWidget.setTabConfiguration(tabs);


        // due to errors we can't emsure that there is a tab at all ( both myTasks and SavedSearches might not be there)
        var savedSearchesTab = this._getTabConfig(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES);
        var myTasksTab = this._getTabConfig(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS);

        if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES) {
          // cases:
          // 1. showSavedSearchesTab && !queueNotFound    -> all ok, show savedSearchesTab, no message
          // 2. showSavedSearchesTab && queueNotFound     -> show queueNotFound message,  show myTasksTab
          // 3. !showSavedSearchesTab                     -> show queueNotFound message,  show myTasksTab
          var showQueueNotFoundMessage = !showSavedSearchesTab || queueNotFound;

          if (showQueueNotFoundMessage) {
            message = this._nlsResources["task_info_saved_search_not_available"];
            message = dojo.string.substitute(message, [storedStartPageConfig.savedSearchName]);
            details = this._nlsResources["task_info_saved_search_not_available_details"];
            details = dojo.string.substitute(details, [storedStartPageConfig.savedSearchName]);
            com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
            com.ibm.bpm.social.util.common.Log.consoleError(message);
          }

          if (showSavedSearchesTab && !queueNotFound) {
            if (myTasksTab) {
              myTasksTab.setContentIsShownAfterInit(false);
            }
            if (savedSearchesTab) {
              savedSearchesTab.setContentIsShownAfterInit();
            }
          }
          else {
            // show myTasksTab
            if (myTasksTab) {
              myTasksTab.setContentIsShownAfterInit();
            }
            if (savedSearchesTab) {
              savedSearchesTab.setContentIsShownAfterInit(false);
            }
          }

        }


        //after creating the lightweight tab container the content for the first tab is also created and shown
        this.tabContainerWidget.createTabs();
        this.connect(this.tabContainerWidget, "showTab", this._onShowTab);

        // insert the new drop down if there are saved searches
        if (showSavedSearchesTab) {
          var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
          var ssTabConfig = this._getTabConfig(id);
          if (ssTabConfig) {
            ssTabConfig.tab.appendChild(this._dropDown);
          }
          dojo.addClass(this._dropDown, "bpm-social-drop-down");
          dojo.addClass(this._dropDown, "bpm-social-drop-down-right");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Update the default page icon if the page content changed
       * 
       * @type {void}
       */
      _checkAndUpdateStartPageIcon: function _checkAndUpdateStartPageIcon_TasksContainer(ccPageConfig){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksContainer");
        // we can't use the ccPageConfig delivered by controller, because we can't identify the controller from it 
        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        var newPageConfig = listController.getPageSettings();
        this._pageHandler.checkAndUpdateStartPageIcon(newPageConfig, this.startPageButton);

        com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksContainer");
        return;
      },

      _setFocusToActiveTab: function _setFocusToActiveTab_TasksContainer(){
        if (this._tabHeaderDiv.style.display !== "none") {
          this.tabContainerWidget.focusActiveTab();
        }
        else {
          // no saved searches, so tab header is hidden, set focus to Work link
          var workLink = dojo.byId("processPortalWorkLink");
          if (workLink) {
            workLink.focus();
          }
        }
      },

      /**
       * Function getting invoked by TabContainer <br>
       * Returning false means that we don't reload all tabs on showTab
       * 
       * @returns {Boolean}
       */
      reloadAllTabs: function reloadAllTabs_TasksContainer(){
        return false;
      },


      /**
       * Reload the currently shown tab
       */
      reloadActiveTab: function reloadActiveTab_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        listController.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _initTab: function _initTab_TasksContainer(div, tabConfiguration){

        var listController = this._getListController(tabConfiguration.id);
        if (listController) {
          listController.setConfig(tabConfiguration.custom);
          listController.createContent();
          div.appendChild(listController.domNode);
        }
      },

      _onShowTab: function _onShowTab_TasksContainer(tabConfiguration){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "New tab selected: " + tabConfiguration.tabName);
        if (this.tabContainerWidget.lastTab.id !== tabConfiguration.id) {
          // notify actual tab that it will be hidden
          this._onHideTab(this.tabContainerWidget.lastTab.id);
        }
        // refresh new tab
        var listController = this._getListController(tabConfiguration.id);
        listController.refresh();
        var newContentDiv = dojo.byId("tab_" + tabConfiguration.id + "__initTab"); // tab_SavedSearches__initTab
        if (newContentDiv) {
          dijit.setWaiState(newContentDiv, "labelledby", listController.listTitle.id);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _onHideTab: function _onHideTab_TasksContainer(tabId){
        com.ibm.bpm.social.util.common.Log.log(3, "tabId=" + tabId);
        if (tabId) {
          var listController = this._getListController(tabId);
          if (listController !== null) {
            listController.onHide();
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _getListController: function _getListController_TasksContainer(id){
        var listController = null;
        var filtered = dojo.filter(this._listControllers, function(item){
          return item.id === id;
        });
        if (filtered.length === 1) {
          listController = filtered[0].controller;
        }
        return listController;
      },

      /**
       * Check for available queues/saved searches and put them into a drop down
       * 
       * @param {domNode} anker Where to place the drop down
       * @returns {Boolean} false if no queues found, true otherwise
       */
      _createQueueDropDown: function _createQueueDropDown_TasksContainer(anker){
        this._queueDropDown = new com.ibm.bpm.social.widgets.ActionDropDownMenu({//UX: allow positioning of popup below the anchor in same place each time.
          leftClickToOpen: true
        });
        dojo.addClass(this._queueDropDown.domNode, "processPortalDropDownMenu");//UX: override icon height & width on .dijitMenuItemIcon
        dojo.addClass(this._queueDropDown.domNode, "tundra");//so the menu separator shows up
        dojo.style(this._queueDropDown.domNode, {
          "paddingLeft": "15px",
          "paddingRight": "15px"
        });//UX
        var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var taskQueuesList = taskQueues.getCustomQueues();
        var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
        var processQueuesList = processQueues.getCustomQueues();
        this._queues = [];
        var elementsExist = false;
        if (taskQueuesList && taskQueuesList.length > 0) {
          for ( var k = 0; k < taskQueuesList.length; k++) {
            var queue = taskQueuesList[k];
            queue.queueHandler = taskQueues;
            var menuItem = new dijit.MenuItem({
              label: queue.getDisplayName()
            });
            this._queueDropDown.addChild(menuItem);
            menuItem.containerNode._queueId = k;
            this._connectsMenu.push(this.connect(menuItem, "onClick", "_onQueueDropdownClick"));
            this._queues.push(queue);
          }
          elementsExist = true;
        }
        // insert a separator, of there tasks and instances saved searches
        var numOfTaskQueues = 0;
        if (taskQueuesList && taskQueuesList.length > 0 && processQueuesList && processQueuesList.length > 0) {
          this._queueDropDown.addChild(new dijit.MenuSeparator());
          numOfTaskQueues = taskQueuesList.length;
        }
        if (processQueuesList && processQueuesList.length > 0) {
          for (k = 0; k < processQueuesList.length; k++) {
            queue = processQueuesList[k];
            queue.queueHandler = processQueues;
            menuItem = new dijit.MenuItem({
              label: queue.getDisplayName()
            });
            this._queueDropDown.addChild(menuItem);
            menuItem.containerNode._queueId = k + numOfTaskQueues;
            this.connect(menuItem, "onClick", "_onQueueDropdownClick");
            this._queues.push(queue);
          }
          elementsExist = true;
        }

        // insert the drop down only, if it exists
        if (elementsExist) {
          // <div dojoAttachPoint="menuButton" class="bpm-social-drop-down" tabIndex="0" style="display:none" waiRole="button"><div class="bpm-social-drop-down-text">&#9660;</div></div>
          anker.setAttribute("role", "button");
          anker.setAttribute("tabindex", 0);
          var span = document.createElement("span");
          dojo.addClass(span, "bpm-social-drop-down-text");
          span.innerHTML = "&#9660;";
          anker.appendChild(span);
          this.connect(anker, "onkeypress", function(e){
            if (e.keyCode == dojo.keys.ENTER) {
              this._queueDropDown._scheduleOpen(e.target);
            }
          });
          this._queueDropDown.bindDomNode(anker);
        }
        return elementsExist;
      },


      /**
       * Store the Work tab as default start page
       */
      _onStartPageButtonClick: function _onStartPageButtonClick_TasksContainer(evt){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        var newPageConfig = listController.getPageSettings();
        this._pageHandler.setStartPage(newPageConfig, this.startPageButton);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Get the TabConfig with id
       * 
       * @return {TabConfiguration}
       */
      _getTabConfig: function _getTabConfig_TasksContainer(id){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var tabInfo = null;
        if (this.tabContainerWidget && this.tabContainerWidget.tabInformations) {
          var tabInfos = dojo.filter(this.tabContainerWidget.tabInformations, function(item){
            return item.id === id;
          });
          if (tabInfos && tabInfos[0]) {
            tabInfo = tabInfos[0];
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return tabInfo;
      },


      /**
       * Callback when user selected a saved search from the drop down
       * 
       * @type {void}
       */
      _onQueueDropdownClick: function _onQueueDropdownClick_TasksContainer(domEvent){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        var tabConfig = this._getTabConfig(id);
        if (tabConfig) {

          var target = null;
          if (dojo.isIE == 8) {
            target = domEvent.srcElement;
          }
          else {
            target = domEvent.target;
          }

          // for keyboard access, we get the the row of the html table instead of the menu item label
          if (dojo.hasClass(target, "dijitMenuItem")) {
            target = dojo.query(".dijitMenuItemLabel", target)[0];
          }

          var queue = this._queues[target._queueId]; // == menuItem.containerNode._queueId
          this._setCustomViewQueue(queue);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

	_setCustomViewQueue: function _setCustomViewQueue(queue) {
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        var tabConfig = this._getTabConfig(id);
		var controllerConfig = tabConfig.custom;
		controllerConfig.queue = queue;
		// reinit filter
		controllerConfig.filterConfigs =
			[com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews()];
		var controller = this._getListController(id);
		controller.setConfig(controllerConfig);
		//check if same tab is selected, if yes , refresh is done in _onShowTab via showTab callback
		if (tabConfig.id != this.tabContainerWidget.lastTab.id) {
			this.tabContainerWidget.selectTab(tabConfig, tabConfig.tab);
		} else {
			controller.refresh();
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _getQueue: function _getQueue_TasksContainer(queueName, includeProcessQueues){
        // retrieve all (Saved Searches)
        var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var q = taskQueues.getQueue(queueName); // returns null on queue not found
        var queue = q;

        if (q) {
          // create a new queue based on the queue returned from taskQueues
          queue = new com.ibm.bpm.social.widgets.task.util.Queue(q);
          queue.queueHandler = taskQueues;
        }
        else {
          var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
          q = processQueues.getQueue(queueName); // returns null on queue not found
          if (q) {
            // create a new queue based on the queue returned from processQueues
            queue = new com.ibm.bpm.social.widgets.task.util.Queue(q);
            queue.queueHandler = processQueues;
          }
        }
        return queue;
      },

      _cleanupMenu: function _cleanupMenu_TasksContainer(){
        if (this._connectsMenu) {
          for ( var k = 0; k < this._connectsMenu.length; k++) {
            dojo.disconnect(this._connectsMenu[k]);
          }
          this._connectsMenu = [];
        }
        if (this._queueDropDown) {
          this._queueDropDown.destroyRecursive();
          this._queueDropDown.destroy();
        }
      },

      destroy: function destroy_TasksController(){

        for ( var i = 0; i < this._subscribedChannels.length; i++) {
          if (this._subscribedChannels[i]) {
            dojo.unsubscribe(this._subscribedChannels[i]);
          }
        }

        this._cleanupMenu();
        if (this._listControllers) {
          for ( var k = 0; k < this._listControllers.length; k++) {
            try {
              this._listControllers[k].controller.destroy();
            }
            catch (e) {
              com.ibm.bpm.social.util.common.Log.logError(2, "destroy_TasksController", e);
            }
          }
          this._listControllers = null;
        }
        this.inherited(arguments);
      }


    });

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.views.TaskListView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.views.TaskListView"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.TaskListView");





















dojo.declare("com.ibm.bpm.social.portal.views.TaskListView", [dijit._Widget,
                                                              dijit._Templated], {
  templateString: "",
  CLASS_NAME: "TaskListView",
  LOG_LEVEL: 1,

  templateString:"<div dojoAttachPoint=\"container\" class=\"workView workViewTask\">\n\t<div class=\"bpm-task-list-nav\" dojoAttachPoint=\"taskListNav\"></div>\n\t<div style=\"height: 100%;\">\n\t\t<div class=\"bpm-task-list-pseudo-dashboard-banner\"><h2 dojoAttachpoint=\"dashboardBannerHeader\"></h2></div>\n\t\t<div class=\"sidebarContent\" dojoAttachPoint=\"sidebarContent\"></div>\n\t\t<div class=\"mainContent taskListContent\" dojoAttachPoint=\"taskListContent\"></div>\n\t</div>\n</div>\n",

  postCreate: function(arguments){
	var FUNC_NAME = "postCreate";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    //Create placeholder divs to be clobbered
    dojo.create("div", {
      'id': 'tasklist_content_widget_target'
    }, this.taskListContent);

    dojo.create("div", {
      'id': 'tasklist_sidebar_widget_target'
    }, this.sidebarContent);

	this.dashboardBannerHeader.innerHTML = this.messages["workPage-dashboardBannerTitle"];
    this.initializeContentWidgets();
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },
  
  initializeContentWidgets: function(){
	var FUNC_NAME = "initializeContentWidgets";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Content teams place widget initialization here ***//
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TasksContainer...", this.CLASS_NAME, FUNC_NAME);
    this.listContainer = new com.ibm.bpm.social.widgets.task.TasksContainer({
      tabHeaderDiv: this.taskListNav
    }, 'tasklist_content_widget_target');
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.sidebar = new com.ibm.bpm.social.portal.PortalSidebar({
		showWidgetTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST
	}, 'tasklist_sidebar_widget_target');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the LaunchableProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.launch = new com.ibm.bpm.social.widgets.LaunchableProcessList({
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		showTaskTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION
	});
		
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the FollowedProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.followed = new com.ibm.bpm.social.widgets.FollowedProcessList({
		refreshRate:60,
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		refreshTopic:com.ibm.bpm.social.portal.Topics.REFRESH_FOLLOW_LIST,
		profileChangedTopic:com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED
	}); 
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TaggedProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.tagged = new com.ibm.bpm.social.widgets.TaggedProcessList({
		refreshRate: 60,
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		showTaskTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION,
		refreshTopic:com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST
	});
	var taggedTabIndex;
	var selectedTabIndex=0;
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Adding tabs to the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.sidebar.addTab(this.messages['launchableProcessList-tabName'], this.launch, 'bpm-social-launch-alt-icon', 'O');
	this.sidebar.addTab(this.messages['followedProcessList-tabName'], this.followed, 'bpm-social-followed-alt-icon', '&#9733;');
	taggedTabIndex = this.sidebar.addTab(this.messages['taggedProcessList-tabName'], this.tagged, 'bpm-social-mentions-alt-icon', '&#64;', true);
	this.tagged.claimControlOfTab(this.sidebar, taggedTabIndex);
	
    this.sidebar.startup(selectedTabIndex);
	this.mentionsVisible = false;
    this.connect(this.sidebar, "selectTab", "_sidebarTabSelected");
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _sidebarTabSelected: function(index){
    if (index == 2) {
      // @Mentions tab selected
      this.tagged.onSelected();
      this.mentionsVisible = true;
    }
    else {
      this.tagged.onDeSelected();
      this.mentionsVisible = false;
    }
  },

  load: function(arguments){
	var FUNC_NAME = "load";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Called from the WorkViewContainer every time immediately before the view is displayed ***//
    
    //Add individual widget refresh calls
    
	var queue = null;
	if (arguments.savedSearchName) {
		queue = this.listContainer._getQueue(arguments.savedSearchName,true);
	}
	if (queue) {
		if (arguments.searchFilter) {
			this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES).setSearchText(arguments.searchFilter);
		}
		this.listContainer._setCustomViewQueue(queue);
	} else {
		if (arguments.state) {
			var filter = null;
			if (arguments.state == 'open') {
				filter = com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById(com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS);
			}
			if (arguments.state == 'close') {
				filter = com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById(com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS);
			}
			if (filter) {
				this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS)._doFilter(filter);
			}			
		}
		if (arguments.searchFilter) {
			this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS).setSearchText(arguments.searchFilter);
		}
	    this.listContainer.refresh();
		var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS;
	    var tabConfig = this.listContainer._getTabConfig(id);
		this.listContainer.tabContainerWidget.selectTab(tabConfig, tabConfig.tab);
	}

	dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{
      "viewType": "TaskListView"
    }]);
	if (this.mentionsVisible) {		
      this.tagged.onShow();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  unload: function(arguments){
	var FUNC_NAME = "unload";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	if (this.mentionsVisible) {
      this.tagged.onHide();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  destroy: function(){
	var FUNC_NAME = "destroy";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

    this.inherited(arguments);

	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.WorkViewContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.WorkViewContainer"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.WorkViewContainer");





















dojo.declare(
	"com.ibm.bpm.social.portal.WorkViewContainer",
	[dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "WorkViewContainer",
	
	LOG_LEVEL: 1,

	templateString:"<div dojoAttachPoint=\"container\" class=\"workViewContainer\"></div>\n",

    view_List: null,

    view_Completion: null,
    
    view_Instance: null,

    currentView: null,

    queuedViews: [],

    selectViewActive: false,

    _cometdConnectCounter: 0,

    _cometdConnectMax: 10,

    _userProfileChangeSubscribeHandle: null,

	_ignorePageTab: false,
	
    postCreate: function() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Loading stylesheet...", this.CLASS_NAME, FUNC_NAME);
		this._loadStylesheet();

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Registering listeners...", this.CLASS_NAME, FUNC_NAME);
		this._registerListeners();

		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Registering work page listeners...", this.CLASS_NAME, FUNC_NAME);
		this.__registerWorkPageListener();

		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	__registerWorkPageListener: function(){
		var FUNC_NAME = "_registerWorkpageListener";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

		dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", dojo.hitch( this, function _onPageTabSwitchEvent(args){
			if(args!=null && args.showing=="dashboardPage"){
				dojo.addClass("processPortalContent", "dashboards");
				dojo.removeClass(dojo.body(), "processPortalWork");

				if(this.currentView){
					com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL,"Calling currentView.unload due to PageTabSwitchEvent", this.CLASS_NAME, FUNC_NAME);
					this.currentView.unload();
				}

			} else {
				dojo.removeClass("processPortalContent", "dashboards");
				dojo.addClass(dojo.body(), "processPortalWork");

				if(this.currentView){
					if ((this.currentView instanceof com.ibm.bpm.social.portal.views.TaskCompletionView)
						&& !this.currentView.taskId && !this.taskTemplateId) {
						// if there is no task, it is because the last view was for
						// a task that was completed.  switch to the task list
						this._loadTaskListView({});
					} else {
					    if (!this._ignorePageTab) {
							var obj = {
								taskId: this.currentView.taskId,
								taskTemplateId: this.currentView.taskTemplateId,
								instanceId: this.currentView.instanceId
							};
							com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Calling currentView.load due to PageTabSwitchEvent", this.CLASS_NAME, FUNC_NAME);
							this.currentView.load(obj);
						} 
					}
				}
			}
			
			this._ignorePageTab = false; //reset ignorePageTab no matter what happens
		}));

		if(dojo.hasClass(dojo.byId("workPageDiv"), "processPortalSelectedPage")){
			dojo.addClass(dojo.body(), "processPortalWork");
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_registerListeners: function(){

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, dojo.hitch(this, this._loadTaskCompletionView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, dojo.hitch(this, this._loadProcessInstanceView));
		
		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE, dojo.hitch(this, this._loadLegacyProcessInstanceView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, dojo.hitch(this, this._loadTaskListView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM, dojo.hitch(this, this._openProcessDiagram));

		if (!(dojo.isSafari && navigator.userAgent.match(/iPad/i))) { // If not Safari on iPad allow CometD
			this._registerCometdListeners();
		}
	},

	_registerCometdListeners: function(){
		this._cometdConnectCounter++;
		if(this._cometdConnectCounter >= this._cometdConnectMax){
			return;
		}

		if(! (window.connectedToBayeuxServer==true && window.connectionEstablished==true) ){
			setTimeout(dojo.hitch(this, this._registerCometdListeners), 500);
		}

		if(this._userProfileChangeSubscribeHandle == null){
			var channel = "/" + com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED.replace(/\./g, "/");
			this._userProfileChangeSubscribeHandle = dojox.cometd.subscribe( channel, dojo.hitch(this, this._handleUserProfileChange) );
		}
	},
	
	_loadProcessInstanceView: function(obj){
		var FUNC_NAME = "_loadProcessInstanceView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.instanceId)=="undefined"){
			return;
		}
		
		com.ibm.bpm.social.portal.PageHandler.getInstance().showProcessDashboard(null, { "tw.local.selectedInstanceId": "" + obj.instanceId });
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_loadLegacyProcessInstanceView: function(obj){
		var FUNC_NAME = "_loadLegacyProcessInstanceView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.instanceId)=="undefined"){
			return;
		}
		if (!this.view_Instance) {
          this.view_Instance = new com.ibm.bpm.social.portal.views.InstanceDetailsView({
		    'top': this.domNode.offsetTop,
		    'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		this.selectView(this.view_Instance, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	

	_loadTaskCompletionView: function(obj){
		this._ignorePageTab = true;
		var FUNC_NAME = "_loadTaskCompletionView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.taskId)=="undefined" && typeof(obj.taskTemplateId)=="undefined"){
			return;
		}

		//Close any open "Process Diagram" dialogs.  This is important for ad-hoc events that do not use dialogs
		dojo.query(".bpm-social-popup-participants").forEach(function(dialog){
			try{
				dijit.byNode(dialog).delayedDestroy();
			} catch (e){

			}
		});

 		if (!this.view_Completion) {
          this.view_Completion = new com.ibm.bpm.social.portal.views.TaskCompletionView({
            'top': this.domNode.offsetTop,
            'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		
		this.selectView(this.view_Completion, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_loadTaskListView: function(obj){
		var FUNC_NAME = "loadTaskListView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		if (!this.view_List) {
          this.view_List = new com.ibm.bpm.social.portal.views.TaskListView({
		    'top': this.domNode.offsetTop,
		    'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		
		this.selectView(this.view_List, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_handleUserProfileChange: function(obj){
		if(obj.data && obj.data.username){
			com.ibm.bpm.social.util.common.Util.getUserAvatarURL(obj.data.username);
		}
	},

	_openProcessDiagram: function(obj){
		//console.warn("WorkViewContainer::_openProcessDiagram entering with obj:",obj);
		if(obj.instanceId && obj.criticalPathFlowIds){
			var instData = com.ibm.bpm.social.util.common.Util.getInstanceData({instanceId:obj.instanceId, header: true});

			var flows = obj.criticalPathFlowIds;
			var parsedFlows = [];
			var bit = null;

			for(var x = 0; x < flows.length; x++){
				if(flows[x]!=null){
					bit = flows[x].replace(/_colon_/g, ":").replace(/_hyphen_/g, "-");
					parsedFlows.push(bit);
				}
			}

			
			var pdd = new com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog({
				"instanceId": obj.instanceId,
				"title": instData.processTemplateName,
				"subTitle": instData.name,
				"criticalPathFlowIds": parsedFlows
			}, dojo.create("div"));
			pdd.show();
		}
	},

	_loadStylesheet: function(){

	},

	_loadSpecifiedStylesheet: function(path){
		if(path.indexOf("/")==0){
			if(dojo.query('link[href="'+path+'"]').length == 0){
	            var linkElement = document.createElement("link");
	            linkElement.rel = "stylesheet";
	            linkElement.type = "text/css";
            	linkElement.href = path;
	            var headElement= document.getElementsByTagName("head")[0];
	            headElement.appendChild(linkElement);
			}
		} else {
			if(dojo.query('link[href$="'+path+'"]').length == 0){
	            var linkElement = document.createElement("link");
	            linkElement.rel = "stylesheet";
	            linkElement.type = "text/css";
	            linkElement.href = dojo.moduleUrl("com.ibm.bpm.social", path);
	            var headElement= document.getElementsByTagName("head")[0];
	            headElement.appendChild(linkElement);
			}
		}
	},

	selectView: function(view, obj){
		var FUNC_NAME = "selectView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Switching to view: "+view, this.CLASS_NAME, FUNC_NAME);
		if(this.selectViewActive){
			this.queuedViews[0] = {'view': view, 'obj': obj}; // change to .push for infinite depth
			return;
		}

		this.selectViewActive = true;

		//Register callback for displaying view when ready
		var handle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY, dojo.hitch(this, function(){

			if(this.currentView!=null){
				if(this.currentView.unload){
					this.currentView.unload();
				}
			/*
				dojo.addClass(this.currentView.domNode, 'workViewFadeOut');

				dojo.removeClass(this.currentView.domNode, 'workViewSelected');

				dojo.addClass(view.domNode, 'workViewFadeIn');

				var fInNode = view.domNode;
				var fOutNode = this.currentView.domNode;

				dojo.fadeOut({
					node: fOutNode,
					onEnd: dojo.hitch(this, function(){

						dojo.fadeIn({
                                        		node: fInNode,
                                        		onEnd: dojo.hitch(this, function(){

                                                		dojo.removeClass(fOutNode, 'workViewFadeOut');

                                                		dojo.addClass(fInNode, 'workViewSelected');

                                                		dojo.removeClass(fInNode, 'workViewFadeIn');
                                        		})
						}).play();
					})
				}).play();
			*/
				dojo.removeClass(this.currentView.domNode, 'workViewSelected');

				dojo.addClass(view.domNode, 'workViewSelected');

			} else {
				dojo.addClass(view.domNode, 'workViewSelected');
			}

			this.currentView = view;

			dojo.unsubscribe(handle);

			this.selectViewActive = false;

			if(this.queuedViews.length > 0){
				var local = this.queuedViews.pop();
				this.selectView(local.view, local.obj);
			}

		}));

		//Call view initialization
		try {
		  view.load(obj);
		} catch(e){
		  // in case the load has an exception, should not happen but if it does one is stuck in a view.
		  // TODO: one could show an error message. So far it only makes sure the user can still work.
		  com.ibm.bpm.social.util.common.Log.log(2, e,this.CLASS_NAME, FUNC_NAME);
		  this.currentView = view;
		  dojo.unsubscribe(handle);
		  this.selectViewActive = false;
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	destroy: function() {
		/**
		 * This is also called from theme.html::customLogoutFunction
		 */
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this._destroyView(this.view_List);
		this._destroyView(this.view_Completion);
		this._destroyView(this.view_Instance);
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_destroyView: function(view){
		var FUNC_NAME = "destroyView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(view && view.destroy){
			view.destroy();
			view = null;
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.sametime.SametimeBridge"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.sametime.SametimeBridge"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/   

dojo.provide("com.ibm.bpm.social.util.sametime.SametimeBridge");



dojo.declare("com.ibm.bpm.social.util.sametime.SametimeBridge", null, {
	
	bootstrap: false,
	liveNameQueue: [],
	loadcount: 0,
	loadtotal: 2,
	loginAttempts: 0,
	loginAttemptsMax: 5,
	deferredLogin: false,
	
	onLoad: function() {
		if(typeof(window.stproxyConfig)!=="undefined"){
			this.loadSTProxy();
		}
	},

	loadSTProxy: function() {
		console.debug("loadSTProxy::entering...");
		
		this._embedCSS(window.stproxyConfig.server+"/stwebclient/dojo.blue/sametime/themes/WebClientAll.css");

		this._embedJavascript(window.stproxyConfig.server+"/stbaseapi/baseComps.js", this.checkLoadCompletion);
		
		this._embedJavascript(window.stproxyConfig.server+"/stwebclient/widgetsLight.js", this.checkLoadCompletion);

		console.debug("loadSTProxy::exiting...");
	},
	
	_embedCSS: function(url){
		var linkElement=document.createElement("link");
		linkElement.rel="stylesheet";
		linkElement.type="text/css";
		linkElement.href=url;
		document.getElementsByTagName("head")[0].appendChild(linkElement);
	},

	_embedJavascript: function(url, callback){
		console.debug("_embedJavascript::entering... with url:",url);
		var script=document.createElement("script");
		script.type="text/javascript";
		if (script.readyState){  //Internet Explorer
			script.onreadystatechange = dojo.hitch(this, function(){
				if (script.readyState == "loaded" || script.readyState == "complete"){
					 script.onreadystatechange = null;
					 dojo.hitch(this, callback)();
				}
			});
		} else {  //Everyone else
			script.onload = dojo.hitch(this, callback);
		}
		script.src=url;
		document.getElementsByTagName("head")[0].appendChild(script);
		console.debug("_embedJavascript::exiting...");
	},
	
	processQueuedLiveNames: function() {
		console.debug("processQueuedLiveNames::entering...");
		var x = 0;
		while(x = this.liveNameQueue.pop())
		{
			this.createLiveName(x.shortName,x.domNode);
		}
		console.debug("processQueuedLiveNames::exiting...");
	},
	
	checkLoadCompletion: function() {
		console.debug("checkLoadCompletion::entering...");
		this.loadcount++;
		if(this.loadcount==this.loadtotal){
			this.bootstrap = true;

			if(typeof(window.stproxy)!=="undefined" && this.deferredLogin!==false){
				this.loginByShortname(this.deferredLogin);
			}
		}
		console.debug("checkLoadCompletion::exiting...");
	},
	
	loginCurrentUser: function(){
		
		var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
		this.loginByShortname(user.userName);
		/*
		dojo.xhrGet({
			url: "/rest/bpm/wle/v1/user/current?parts=none",
			handleAs: "json",
			sync: true,
			headers: {
				"Content-Type": "application/json", 
				"Accept":"application/json"
			},
			load: dojo.hitch(this, function(data, xhrArgs){
				this.loginByShortname(data.data.userName);
			})
		});
		*/
	},
	
	loginByShortname : function (shortName) {
		console.debug("loginByShortname::entering...", ["shortName", shortName]);
		if(this.bootstrap==false){
			console.debug("loginByShortname deferring login");
			this.deferredLogin = shortName;
			return;
		}
		
		this.userName = shortName;
		this.loginAttempts++;
		if(!window.stproxy && this.loginAttempts <= this.loginAttemptsMax)
		{
			console.debug("ST not ready, returning early...");
			setTimeout(dojo.hitch(this, function(){
				this.loginByShortname(shortName);
			}), 3000);
			return;
		}
				
		/* 
		window.stproxy.login.loginByPassword(
			user,
			password,
			window.stproxy.awareness.AVAILABLE,
			'I am available',
			dojo.hitch(this, this.loginOK), 
			dojo.hitch(this.loginFailed)
		);
		*/
		
		/* */
		window.stproxy.login.loginByToken(
			shortName,
			window.stproxy.awareness.AVAILABLE,
			"",
			dojo.hitch(this, this.loginOK), 
			dojo.hitch(this, this.loginFailed)
		);
		
		console.debug("loginByShortname::exiting...");
	},
		
	loginOK: function(){
		console.debug("Login succeeded...");
		this.processQueuedLiveNames();
	},
		
	loginFailed: function(){
		console.error("Login failed...");
	},

	createLiveName : function (shortName,domNode) {
		if(this.bootstrap == false) {
			this.liveNameQueue.push({'shortName' : shortName, 'domNode': domNode});
			return;
		}

		//Production code would perform shortName to fullName User Cache lookup
		var userData = com.ibm.bpm.social.util.common.Util.getUser(shortName);
		
		if(typeof(domNode) == 'string') {
			//var ln = new sametime.LiveName({'userId': this.userHash[shortName] },dojo.byId(domNode));
			var ln = new sametime.LiveName({'userId': userData.emailAddress },dojo.byId(domNode));
			dojo.style(ln.domNode , {display : "inline",color : "#B9C1ED", cursor : "pointer"});
			ln.activate();
		} else {
			//var ln = new sametime.LiveName({'userId': this.userHash[shortName] },domNode);
			var ln = new sametime.LiveName({'userId':userData.emailAddress },domNode);
			dojo.style(ln.domNode , {display : "inline", color: "#B9C1ED", cursor : "pointer"});
			ln.activate();
		}
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.connections.ConnectionsBridge"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.connections.ConnectionsBridge"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/   

dojo.provide("com.ibm.bpm.social.util.connections.ConnectionsBridge");





dojo.declare("com.ibm.bpm.social.util.connections.ConnectionsBridge", null, {
	
	connectionsServer: null,
	bootstrap: false,
	bizCardQueue: [],
	loadcount: 0,
	loadtotal: 1,
	objectCheckCounter: 0,
	objectCheckMax: 25,
	objectCheckIntervalId: null,
	avatars: {},
	avatarServiceAvailable: null,
	messages: null,
	
	onLoad: function _connectionsBridge_onLoad(connUrl) {
	    // We only need the scheme/host/port for Business Card support.
	    var qIdx = connUrl.indexOf("?");
	    if (qIdx != -1) {
	        connUrl = connUrl.substring(0, qIdx);
	    }	
		this.connectionsServer = connUrl;
		
		// Load Localization Bundles
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		var cookie = dojo.cookie("card.popup.slim");
		if(typeof(cookie)=="undefined"){
			dojo.cookie("card.popup.slim", "1");
		}
		
		this.loadConnectionsAPI();
		
		this.isAvatarServiceAvailable();	
		
		/**
		 * Link click override, to prevent unwanted navigation away from Process Portal
		 **/
		dojo.connect(dojo.body(), "onclick", function(e){
			try{
				var _connUrl = new dojo._Url(connUrl);
				var _tgtUrl = new dojo._Url(e.target.href);
	
				if(_connUrl.host==_tgtUrl.host && _connUrl.port==_tgtUrl.port){
					dojo.stopEvent(e);
					window.open(e.target.href);
				}	
			} catch (err) { }
		});
	},

	loadConnectionsAPI: function _connectionsBridge_loadConnectionsAPI() {
		/** As a result of this next line, a window level object named "SemTagSvc" is created for us to interact with the Connections API **/
		this._embedJavascript(this.connectionsServer+"/profiles/ibm_semanticTagServlet/javascript/semanticTagService.js", this.checkLoadCompletion);
	},

	/*
	_embedCSS: function(url){
		var linkElement=document.createElement("link");
		linkElement.rel="stylesheet";
		linkElement.type="text/css";
		linkElement.href=url;
		document.getElementsByTagName("head")[0].appendChild(linkElement);
	},
	*/
	
	_embedJavascript: function _connectionsBridge__embedJavascript(url, callback){
		//console.debug("_embedJavascript::entering with url:",url);
		var script=document.createElement("script");
		script.type="text/javascript";
		if (script.readyState){  //Internet Explorer
			script.onreadystatechange = dojo.hitch(this, function(){
				if (script.readyState == "loaded" || script.readyState == "complete"){
					 script.onreadystatechange = null;
					 dojo.hitch(this, callback)();
				}
			});
		} else {  //Everyone else
			script.onload = dojo.hitch(this, callback);
		}
		script.src=url;
		document.getElementsByTagName("head")[0].appendChild(script);
		//console.debug("_embedJavascript::exiting");
	},
	
	checkLoadCompletion: function _connectionsBridge_checkLoadCompletion() {
		this.loadcount++;
		if(this.loadcount==this.loadtotal){
			this.bootstrap = true;
			
			/** Wait for the Javascript object to be created **/
			this.objectCheckIntervalId = setInterval(dojo.hitch(this, function(){
				this.objectCheckCounter++;
				if(typeof(SemTagSvc)!="undefined"  && SemTagSvc!=null){
					/** Once defined, process all queued wrapper requests **/
					clearInterval(this.objectCheckIntervalId);
					this.objectCheckIntervalId = null;
					this.processQueuedBusinessCards();
					
					dojo.connect(LCSemTagMenu,"mouseover", com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
					
					dojo.publish(com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED, [{}]);
				}
				
				/** Only wait and check for a set period of time **/
				if(this.objectCheckCounter >= this.objectCheckMax && this.objectCheckIntervalId!=null){
					clearInterval(this.objectCheckIntervalId);
				}
			}), 500);
			
		}
	},
	
	/**
	 * Convert an element containing a standard user's display name into a Connections-enabled
	 * Business Card link.
	 */
	wrapElement: function _connectionsBridge_wrapElement(/*Node or ID*/ node, /*String*/ userName, /*String*/ userEmail) {
		//console.debug("ConnectionsBridge::wrapElement entering with userName:{"+userName+"} and userEmail:{"+userEmail+"}");
		
		if(typeof(node)=="undefined" || 
				(typeof(userName)=="undefined" || userName.length==0) ||
					(typeof(userEmail)=="undefined" || userEmail.length==0)){
			return false;
		}
		
		var success = false;
		
		if(this.bootstrap==true && typeof(SemTagSvc)!="undefined" && SemTagSvc!=null){
			
			if(typeof(node)=="string"){ /** node is a String containing the ID of the HTMLElement **/
				node = dojo.byId(node);
			}

			if(node.getAttribute("connectId")!=null){
				var connectId = node.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			
			node.innerHTML = "";
			
			var nameNode = dojo.create("a", null, node);
			dojo.addClass(nameNode, "fn");
			dojo.addClass(nameNode, "url");
			nameNode.href = "javascript:void(0);";
			nameNode.innerHTML = userName;
			
			var emailNode = dojo.create("span", null, node);
			dojo.addClass(emailNode, "email");
			dojo.style(emailNode, "display", "none");
			emailNode.innerHTML = userEmail;

			dojo.addClass(node, "vcard");
			
			setTimeout(dojo.hitch(this, function(){
				SemTagSvc.parseDom(null, node); //.parentNode
			}), 500);
			
			success = true;
		} else {
			this.bizCardQueue.push([node, userName, userEmail]);
			success = false;
		}
		
		return success;
	},
	
	/**
	 * Convert an element containing a standard user's avatar into a Connections-enabled
	 * Business Card link.
	 */
	wrapImage: function _connectionsBridge_wrapImage(/*Image Node or ID*/ node, /*String*/ userEmail) {
		//console.debug("ConnectionsBridge::wrapImage entering with userEmail:{"+userEmail+"}");
		
		if(typeof(node)=="undefined" ||
			(typeof(userEmail)=="undefined" || userEmail.length==0)){
			return false;
		}
		
		var success = false;
		
		if(this.bootstrap==true && typeof(SemTagSvc)!="undefined" && SemTagSvc!=null){

			if(typeof(node)=="string"){ /** node is a String containing the ID of the IMGElement **/
				node = dojo.byId(node);
			}
			
			if(node.getAttribute("connectId")!=null){
				var connectId = node.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			
			//Check to see if the hierarchy necessary is already in place
			var container = null;
			var nameNode = null;
			
			var bca = false;
			try{
				bca = dojo.hasClass(node.parentNode, "bpm-bizcard-anchor");
				nameNode = node.parentNode;
			} catch (e){
				bca = false;
				nameNode = null;
			}
			
			/*
			if(nameNode.getAttribute("connectId")!=null){
				var connectId = nameNode.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			*/
			
			var bcc = false;
			try{
				bcc = dojo.hasClass(node.parentNode.parentNode, "bpm-bizcard-container");
				container = node.parentNode.parentNode;
			} catch (e){
				bcc = false;
				container = null;
			}
			
			if(! (bca && bcc) ){
				container = dojo.create("span", {"class": "bpm-bizcard-container"}, node, "before");

				nameNode = dojo.create("a", {"class": "bpm-bizcard-anchor"}, container);
				nameNode.href = "javascript:void(0);";
			
				dojo.place(nameNode, container, "only");
			}
			
			dojo.place(node, nameNode, "only");
			
			dojo.addClass(nameNode, "fn");
			dojo.addClass(nameNode, "url");
			
			var emailNode = dojo.create("span", null, container);
			dojo.addClass(emailNode, "email");
			dojo.style(emailNode, "display", "none");
			emailNode.innerHTML = userEmail;

			dojo.addClass(container, "vcard");
			
			setTimeout(dojo.hitch(this, function(){
				SemTagSvc.parseDom(null, container); //node.parentNode
			}), 500);
			
			success = true;
			
		} else {
			this.bizCardQueue.push([node, userEmail]);
			success = false;
		}
		
		return success;
	},
	
	processQueuedBusinessCards: function _connectionsBridge_processQueuedBusinessCards(){
		var x = 0;
		while(x = this.bizCardQueue.pop())
		{
			if(x.length==2){
				this.wrapImage(x[0], x[1]); /** node and userEmail **/
			} else if(x.length==3){
				this.wrapElement(x[0], x[1], x[2]); /** node, userName, and userEmail **/
			}
			
		}
	},
	
	isAvatarCached: function _connectionsBridge_isAvatarCached(email){
		return (typeof(this.avatars[email])!="undefined" && this.avatars[email]!=null);
	},
	
	clearAvatarCache: function _connectionsBridge_clearAvatarCache(email){
		if(typeof(email)=="undefined"){
			this.avatars = {};
		}
		
		this.avatars[email] = null;
	},
	
	getAvatarUrl: function _connectionsBridge_getAvatarUrl(email){
		if(!this.avatars[email] || this.avatars[email]==null){
		
			var url = (typeof(window.MASHUPS_PROXY_URL)!="undefined") ? window.MASHUPS_PROXY_URL : window.endpointURLObject.mumJS +"/proxy";
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += this.connectionsServer.replace("://", "/");
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += "profiles/json/profile.do?email=";		// ...profiles/atom/profile.do...???
			url += encodeURIComponent(email);
			
			//	{url} == {https://localhost:9080/mum/proxy/https/w3-connections.ibm.com/profiles/atom/profile.do?email=user@us.ibm.com}
			dojo.xhrGet({
				"url": url,
				"handleAs":"json",
				"sync": true,
				"load": dojo.hitch(this, function(data, xhrArgs){
					if(data["X_inDirectory"]=="true"){
						this.avatars[email] = data["photo"];
					} else {
						this.avatars[email] = null;
					}
				}),
				"error":dojo.hitch(this, function(data, xhrArgs){
					this.avatars[email] = null;
				}),
				"failOk": true,
				"preventCache": true
			});
		}	
		return this.avatars[email];
	},
	
	isAvatarServiceAvailable: function _connectionsBridge_isAvatarServiceAvailable(){
		if(this.avatarServiceAvailable==null){
			this.avatarServiceAvailable = false;
			
			var url = (typeof(window.MASHUPS_PROXY_URL)!="undefined") ? window.MASHUPS_PROXY_URL : "/mum/proxy";
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += this.connectionsServer.replace("://", "/");
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += "profiles/html/searchProfiles.do";
			
			var self = this;
			dojo.xhrGet({
				"url": url,
				"handleAs":"text",
				"sync": true,
				"handle": function(data, xhrArgs){
					self.avatarServiceAvailable = (xhrArgs.xhr.status==200)
					if(!self.avatarServiceAvailable){
						console.error(self.messages.connectionsProxyError.replace("{0}", self.connectionsServer));
					}
				},
				"failOk": true,
				"preventCache": true
			});
		}
		//console.warn("returning this.avatarServiceAvailable:{"+this.avatarServiceAvailable+"}");
		return this.avatarServiceAvailable;
	}
	

});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.OCFakeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.OCFakeModel"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.OCFakeModel");



//
// Simple model that "invents" matches in the desired window.
//
dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.OCFakeModel", com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel,
{
  _MAX_ITEMS: 10000,
  
  
  /**
   * @param searchText A not nullable and possible empty string.
   * @param offset A not nullable integer >=0.
   * @param size A not nullable integer >=0.
   * @return The items that verify the query (filter in this case) and that fall in the specified pagination window.
   */
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size ) /*{cn,userID}[]*/
  {
    // Remove wildcards
    searchText = this._removeWildcards(searchText).toUpperCase();

    // Filter.
    var matcher = function(x) 
    {
      return (x && x.cn) ? (x.cn.toUpperCase().indexOf(searchText) != -1) : false;
    }
    
    // Invent data.
    var filtered = this._getItems( offset, size, matcher );

    return filtered;
  },
  
  
  _getItems: function( /*int*/ offset, /*int*/ size, matcher ) /*{cn,userID}[]*/
  {
    // Define default values.
    if (!size)
    {
      size = 30;
    }
    if (!offset)
    {
      offset = 0;
    }
    
    var options = [];
    var matchedIndex = 0;
    for (i=0; options.length<size && i<this._MAX_ITEMS; i++)
    {
      var item = {cn:i + "_FakeUser", userID:i};
      //this._log("Testing " + dojo.toJson(item));
      if( matcher(item) )
      {
        //this._log("  Matched");
        if ( matchedIndex >= offset )
        {
          //this._log("  In window");
          options.push(item);
        }
        matchedIndex++;
      }
    }
    
    return options;
  },

  
  _log: function( message )
  {
    console.log( "OCFakeModel: " + message );
  }  
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.OCStaticModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.OCStaticModel"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.OCStaticModel");




dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.OCStaticModel", com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel,
{
  options: null,
  
  constructor: function( /*{cn,userID}[]*/ options )
  {
    //this._log("constructor() - options:" + dojo.toJson(options));
    this.options = options;
  },
  
  
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size ) /*{cn,userID}[]*/
  {
    // Remove wildcards
    searchText = this._removeWildcards(searchText).toUpperCase();

    // Filter.
    var matches = function(x) 
    {
      return (x && x.cn) ? (x.cn.toUpperCase().indexOf(searchText) != -1) : false;
    }
    
    // Because of IE incompatibility, cannot use "this.options.filter( matches );"
    var filtered = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.filterArray(this.options, matches);

    // Truncate to the desired paging window.
    return filtered.splice( offset, size );
  },

  
  _log: function( message )
  {
    console.log( "OCStaticModel: " + message );
  }  
}
);


}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip");
















dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class=\"bpm-social-participants-activity-tooltip-content\" role=\"group\" aria-labelledby=\"activity_details_tooltip_title\" dojoAttachPoint=\"container\">\n<div class=\"mad-header\" role=\"presentation\">\n\t<div id=\"activity_details_tooltip_title\" class='title' dojoAttachPoint=\"header\" role=\"listitem\" tabindex=\"0\">{title}</div>\n\t<div class='counter-wrapper' dojoAttachPoint='wrapper' role=\"listitem\" tabindex=\"-1\">\n\t\t<span class=\"previous\" dojoAttachPoint=\"previous\">\n\t\t\t<img src='' dojoAttachPoint='previousImg' dojoAttachEvent=\"ondijitclick:_clickPrevious\" tabindex=\"-1\">\n\t\t</span>\n\t\t<span class='counter-scroll'>\n\t\t\t<span class='counter' dojoAttachPoint='counter'></span>\n\t\t</span>\n\t\t<span class=\"next\" dojoAttachPoint=\"next\">\n\t\t\t<img src='' dojoAttachPoint='nextImg' dojoAttachEvent=\"ondijitclick:_clickNext\" tabindex=\"-1\">\n\t\t</span>\n\t</div>\n\t<div style=\"clear: both;\"></div>\n\t<div class=\"content\" dojoAttachPoint='content' role=\"listitem\" aria-describedby=\"owner-text-wrapper start-date due-date\" tabindex=\"0\">\n\t\t<div class=\"owner-data\">\n\t\t\t<!-- <img class=\"ownerImg bpm-social-default-avatar-small\" dojoAttachPoint=\"ownerImg\" alt=\"\"/>  -->\n\t\t\t<div dojoAttachPoint=\"avatarContainer\" ></div>\t\n\t\t\t<div id=\"owner-text-wrapper\" class=\"owner-text-wrapper\" role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\">\n\t\t\t\t<span class=\"ownerField\" dojoAttachPoint=\"ownerField\">{owner}</span>\n\t\t\t\t<span class=\"ownerLabel\" dojoAttachPoint=\"ownerLabel\">{ownerLabel}</span>\n\t\t\t</div>\n\t\t\t</div>\n        <div>\n    \t\t<div role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\" id=\"start-date\"><label id=\"start_date_label\" dojoAttachPoint=\"startDateLabel\">{startDateLabel}</label><span id=\"start_date_field\" class='startDate' dojoAttachPoint=\"startDateField\">{startDate}</span></div>\n    \t\t<div role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\" id=\"due-date\"><label id=\"due_date_label\" dojoAttachPoint=\"dueEndDateLabel\">{dueEndDateLabel}</label><span id=\"due_date_field\" class='dueEndDate' dojoAttachPoint=\"dueEndDateField\">{dueEndDate}</span></div>\n\t\t</div>\n\t</div>\n</div>\n<hr>\n<div class=\"mad-body\">\n\t<span class='title' dojoAttachPoint=\"expertsTitle\" role=\"listitem\" tabindex=\"0\">{expertsTitle}</span>\n\t<div style=\"clear: both;\"></div>\n\t<div class='activityExperts'>\n\t\t<div class='grid' dojoAttachPoint=\"grid\"></div>\n\t\t<div class='bpm-social-experts-layout' dojoAttachPoint='autoExperts'>\n\t\t\t<div class='bpm-social-experts-layout-auto-none' dojoAttachPoint='noExperts' style='display:none;'>\n\t\t\t\t<div class='header' dojoAttachPoint='noExperts_header'></div>\n\t\t\t\t<div class='subscript' dojoAttachPoint='noExperts_subscript'></div>\n\t\t\t\t<div class='content' dojoAttachPoint='noExperts_content'></div>\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n</div>\n\n",
	
	tasks: [],
	taskInstances: [],
	currentTaskIndex: 0,
	
	currentMaxIndex: 3,
	currentMinIndex: 0,
	
	livenames: null,	
	expertTiles: [],
	
    postCreate: function() {
    	this.inherited(arguments);
		this.taskInstances = [];
    	
		
		this.messages = dojo.i18n.getLocalization('com.ibm.bpm.social', 'bpmSocial');
		
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.expertsTitle.innerHTML = this.messages.activityExpertsTitle;
		
		this.startDateLabel.innerHTML = this.messages.startDate+":";
		this.ownerLabel.innerHTML = this.messages.owner;
		
		this.noExperts_header.innerHTML = this.messages.noExperts_AutoHeader;
		
        dojo.attr(this.wrapper,"aria-label",this.messages.activity_details_counter_label);

    		this.previousImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/blank.gif");
    		this.nextImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/blank.gif");
    	  this._avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContainer);

//        if (dojo.hasClass(document.body,"dijit_a11y")) {
//            this.ownerImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
//        }

        this.nextImg.alt = this.messages.next_button_label;
        dijit.setWaiRole(this.nextImg,"button");
        this.previousImg.alt = this.messages.previous_button_label;
        dijit.setWaiRole(this.previousImg,"button");

		if(this.tasks.length==1){
			dojo.addClass(this.container, 'singleUser');
			dojo.empty(this.wrapper);
		} else {
            dojo.attr(this.wrapper,"tabindex","0");
            if (dojo.hasClass(document.body,"dijit_a11y")) {
                this.previousImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Chevron_gray_left.png");
                this.nextImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Chevron_gray_right.png");
                //this.ownerImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
            }
			for(var idx = 0; idx < this.tasks.length; idx++){
				var t = dojo.create('a', {
					'class': 'task-instance',
					'onclick': dojo.hitch(this, this._taskInstanceSelected),
					'index': idx+1,
					'href': 'javascript:void(0)',
					'innerHTML': idx+1
				}, this.counter);
				if(idx==0){
					dojo.addClass(t, 'current');
                    dijit.setWaiState(t,"disabled","true");
				}
				this.taskInstances.push(t);
			}
            this._disablePreviousOrNext(this.previous,this.previousImg,true);            
			
			/*
			if(this.tasks.length < this.currentMaxIndex){
				dojo.addClass(this.next, 'disabled');
			}
			*/
		}
		
		this._setTaskDetails(this.tasks[this.currentTaskIndex]);
		
		setTimeout(dojo.hitch(this, function(){
			this._checkForCounterScroll(0);
		}), 100)
	},
	
	_updateTaskDetails: function(){
		dojo.query('a.task-instance.current', this.domNode).forEach(function(node){
            dojo.removeClass(node,'current');
            dijit.setWaiState(node,"disabled","false");
        });	
		dojo.query('a.task-instance[index='+(this.currentTaskIndex+1)+']', this.domNode).forEach(function(node){
            dojo.addClass(node,'current');
            dijit.setWaiState(node,"disabled","true");
        });	

		
		dojo.fx.chain([
		    dojo.animateProperty({
		    	node: this.content,
		        duration: 300,
		        properties: {
		        	color: {
		        		start: "black",
		            	end: "white"
		            }
		        },
		        onBegin: dojo.hitch(this, function(){
		        	com.ibm.bpm.social.widgets.common.BizCard._hideTooltip();
		        }),
		        onEnd: dojo.hitch(this, function(){
		        	this._setTaskDetails(this.tasks[this.currentTaskIndex]);
		        })
		    }),
		    dojo.animateProperty({
		        node: this.content,
		        duration: 300,
		        properties: {
		            color: {
		            	start: "white",
		            	end: "black"
		            }
		        }
		    })
		]).play();
	},
	
	_setTaskDetails: function(t){
		
		this.dueEndDateLabel.innerHTML = (t.completionTime==null ? this.messages.dueDate : this.messages.endDate)+":";
 	   
		this.header.innerHTML = t.name;
		this.header.title = t.name;

        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.header);
        
        this._applyStylingForBTD(this.header);
		
        this.startDateField.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(t.startTime);
        
		this.dueEndDateField.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(t.completionTime==null ? t.dueTime : t.completionTime);
        
		var user = null;
		if(t.owner){
			user = com.ibm.bpm.social.util.common.Util.getUser(t.owner);
			this.ownerField.innerHTML = user.fullName;
      this._avatar.setUser(user, " bpm-social-default-avatar-small", true);
      this._avatar.setCSSClass("owner-visible");
			dojo.style(this.ownerLabel, "visibility", "visible");
			dojo.style(this.ownerField, "visibility", "visible");
			dojo.query(".owner-text-wrapper", this.domNode).removeClass("no-owner");
		} else if(t.assignedToDisplayName){
			this.ownerField.innerHTML = t.assignedToDisplayName;
			this._avatar.setCSSClass("owner-hidden");
			dojo.style(this.ownerLabel, "visibility", "hidden");
			dojo.style(this.ownerField, "visibility", "visible");
			dojo.query(".owner-text-wrapper", this.domNode).addClass("no-owner");
		} else {
      this._avatar.setCSSClass("owner-hidden");
			dojo.style(this.ownerLabel, "visibility", "hidden");
			dojo.style(this.ownerField, "visibility", "hidden");
			dojo.query(".owner-text-wrapper", this.domNode).addClass("no-owner");
		}

		// Set the BTD off the ownerField before adding the business card integration
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.ownerField);
		this._applyStylingForBTD(this.ownerField);
		if(user!=null){
			/** Business Card integration **/
			//com.ibm.bpm.social.widgets.common.BizCard.unwrapImage(this.ownerImg); //fixes defect 93774
			com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.ownerField, user);
			//com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.ownerImg, user);
			/** End Business Card integration **/
		}
        
    },

	/*
	 * Called indirectly from the surrounding TooltipDialog onShow Event
	 */
	getExperts: function(){
		this.socialRestHandler.getExpertsForTask(this.tasks[this.currentTaskIndex].tkiid, 
			dojo.hitch(this, "_appendExpertsToTooltip"),
			dojo.hitch(this, "_onXhrErrors"), false);
	},
	
	_appendExpertsToTooltip: function(data, xhrArgs){
        var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();        
        var currentUserId = userProfileObj.userID;
		var ee = null;
		if(data.systemIdentifiedUsers.length==0){
			dojo.style(this.noExperts, "display", "block");
			return;
		}
        var expertsCt = 0;
		for(var idx = 0; idx < data.systemIdentifiedUsers.length; idx++){
            //Do not show if the expert matches the current user.
            //We only show 6, but REST might return 7.
            if (currentUserId != data.systemIdentifiedUsers[idx]["userID"]) {
			data.systemIdentifiedUsers[idx]['taskId'] = this.tasks[this.currentTaskIndex].tkiid;
			data.systemIdentifiedUsers[idx]['includeTaskDetails'] = true;
			ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(data.systemIdentifiedUsers[idx]);
			this.grid.appendChild(ee.domNode);
			this.expertTiles.push(ee);
                expertsCt++;
                if (expertsCt == 6) {
                    break;
                }
            }
		}
		
		var position=null;
		if (!!ee) {
			position = dojo.position(ee.domNode);
		}
		if (position == null || position.w <= 206) { // Check to see if the size of the experts node is larger than expected size
			if(expertsCt <= 2){
				dojo.addClass(this.grid, "singleRow");
			} else if(expertsCt <= 4){
				dojo.addClass(this.grid, "doubleRow");
			} else if(expertsCt <= 6){
				dojo.addClass(this.grid, "tripleRow");
			} else {
				dojo.addClass(this.grid, "tripleRow");
				dojo.addClass(this.grid, "scrollExperts");
			}
		} else { // width of the experts element > than specified width so only have 1 entry per row
			if(expertsCt <= 1){
				dojo.addClass(this.grid, "singleRow");
			} else if(expertsCt <= 2){
				dojo.addClass(this.grid, "doubleRow");
			} else if(expertsCt <= 3){
				dojo.addClass(this.grid, "tripleRow");
			} else {
				dojo.addClass(this.grid, "tripleRow");
				dojo.addClass(this.grid, "scrollExperts");
			}
		}
	},
	
	_onXhrErrors: function(data,xhrArgs) {
		// TODO: how do we show errors?
		console.error(data, xhrArgs);
	},
	
	_taskInstanceSelected: function(e){
		
		var oldIdx = this.currentTaskIndex;
		
		this.currentTaskIndex = e.target.getAttribute('index')-1;
		
		var scrollDir = (this.tasks.length < this.currentMaxIndex ? 0 : this.currentTaskIndex - oldIdx);
		
		this._checkForCounterScroll(scrollDir);
		
		this._updateTaskDetails();
	},
	
	_clickPrevious: function(e){
        dojo.stopEvent(e);
		
		if(dojo.hasClass(this.previous,'disabled')){
			return;
		}
        this._disablePreviousOrNext(this.next,this.nextImg,false);
		
		this.currentTaskIndex--;
		if(this.currentTaskIndex < 0){
			this.currentTaskIndex = 0;
		}
		
		if(this.currentTaskIndex==0){
            this._disablePreviousOrNext(this.previous,this.previousImg,true);
		}
		this._checkForCounterScroll(-1);
		this._updateTaskDetails();

	},
	
	_clickNext: function(e){
        dojo.stopEvent(e);
		
		if(dojo.hasClass(this.next, 'disabled')){
			return;
		}
        this._disablePreviousOrNext(this.previous,this.previousImg,false);
		
		this.currentTaskIndex++;
		if(this.currentTaskIndex > this.tasks.length-1){
			this.currentTaskIndex = this.tasks.length - 1;
		}
		
		if(this.currentTaskIndex==this.tasks.length - 1){
            this._disablePreviousOrNext(this.next,this.nextImg,true);
		}
		this._checkForCounterScroll(1);
		this._updateTaskDetails();
	},
	
	_checkForCounterScroll: function(dir) {
		/*if(dir==0){
			this._checkPreviousNextState();
			return;
		}*/
		
		//scroll to the correct page number
		var scrollToIndex = this.currentTaskIndex;
		if(this.currentTaskIndex == this.taskInstances.length-1) {
			scrollToIndex--;
		}
		else if(this.currentTaskIndex == 0) {
			scrollToIndex++;
		}
		var pageLink = this.taskInstances[scrollToIndex];
        if (!pageLink) {
            return;
        }
        
		var scrollAmt = 25 - pageLink.offsetLeft;
		if(this.currentTaskIndex == 0) {
			scrollAmt--; //forces the ipad to refresh, fixes defact 91836
		}
		dojo.query(this.counter).style('left', ""+scrollAmt+"px");
		
		//get the adjacent numbers
		var leftmost = (scrollToIndex-2 >= 0) ? this.taskInstances[scrollToIndex-2] : null;
		var leftish = (scrollToIndex-1 >= 0) ? this.taskInstances[scrollToIndex-1] : null;
		var center = this.taskInstances[scrollToIndex];
		var rightish = (scrollToIndex+1 < this.taskInstances.length) ? this.taskInstances[scrollToIndex+1] : null;
		var rightmost = (scrollToIndex+2 < this.taskInstances.length) ? this.taskInstances[scrollToIndex+2] : null;
		
		//hide anything that would get cut-off by the arrow buttons
		if(leftmost != null) {
			dojo.style(leftmost, "visibility", "hidden");
		}
		if(leftish != null) {
			dojo.style(leftish, "visibility", "");
		}
		dojo.style(center, "visibility", "");
		if(rightish != null) {
			dojo.style(rightish, "visibility", "");
		}
		if(rightmost != null) {
			dojo.style(rightmost, "visibility", "hidden");
		}
		/*
		var lclCounter = dojo.query(this.counter);
		
		//For some reason the pixels here are just a little off (by 1/2 a pixel)
		//so adjusting the pixel movements by one does not fix the issue of the
		//numbers scrolling out of visibility.  What is even stranger, the ipads
		//numbers scroll into invisibility the opposite direction.  Here I adjust
		//the scrolling every other time to adjust for this movement, and the 
		//numbers stay where they should.
		//Defect 93772
		var scrollOffset;
		if(this.currentTaskIndex < 10) {
			scrollOffset = 12;
		} else {
			scrollOffset = 18;
			if(com.ibm.bpm.social.util.common.Util.isIPad || dojo.isIE) {
				scrollOffset--;
				//every 10 times or so
				if((this.currentTaskIndex % 10) == 0) {
					scrollOffset++;
				}
			} else if((this.currentTaskIndex % 2) > 0) {
				scrollOffset++;
			}
		}
		
		//var scrollOffset = (this.currentTaskIndex < 10 ? 12 : 18); //Used to determine if we need a big scroll or smaller
		
		var curLeft = parseInt(lclCounter.style('left'));
		
		var offset;
		if(dir > 0) {
			if(this.currentTaskIndex >= this.currentMaxIndex){
				offset = (dir * -1 * scrollOffset) + curLeft;
				lclCounter.style('left', offset+'px');
				if(this.currentMaxIndex < this.tasks.length){
					this.currentMaxIndex += dir;
					this.currentMinIndex += dir;
				}
			}
		} else if(dir < 0){
			if(this.currentTaskIndex <= this.currentMaxIndex){
				offset = ((dir * -1 * scrollOffset) + curLeft);
				if(offset > 0) { 
					offset = 0;
				}
				lclCounter.style('left', offset+'px');
				
				if(this.currentMinIndex > 0){
					this.currentMaxIndex += dir;
					this.currentMinIndex += dir;
				}
			}
		}*/
		this._checkPreviousNextState();
	},
	
	_checkPreviousNextState: function(){
		if(this.currentTaskIndex == this.tasks.length-1){
			if(!dojo.hasClass(this.next, 'disabled')){
                this._disablePreviousOrNext(this.next,this.nextImg,true);
			}
		} else {
            this._disablePreviousOrNext(this.next,this.nextImg,false);
		}
			
		if(this.currentTaskIndex == 0){
			if(!dojo.hasClass(this.previous, 'disabled')){
                this._disablePreviousOrNext(this.previous,this.previousImg,true);
			}
		} else {
            this._disablePreviousOrNext(this.previous,this.previousImg,false);
		}
	},

    _disablePreviousOrNext: function(span,img,disable) {
         if (disable) {
             dojo.addClass(span, 'disabled');
             dijit.setWaiState(img,"disabled","true");
             dojo.attr(img,"tabindex","-1");
		} else {
             dojo.removeClass(span, 'disabled');
             dijit.setWaiState(img,"disabled","false");
             dojo.attr(img,"tabindex","0");
         }
	},
	
	_applyStylingForBTD: function(node) {
		var isDocLanguageRTL = ("rtl" == document.body.getAttribute('dir'));
		
		if (isDocLanguageRTL)
			dojo.style(node, "textAlign", "right");
		else
			dojo.style(node, "textAlign", "left");
	},
				
	destroy: function() {
		if(this.expertTiles && this.expertTiles.length>0){
			for(var idx = 0; idx < this.expertTiles; idx++){
				if(this.expertTiles[idx].destroy){
					this.expertTiles[idx].destroy();
				}				
			}
			this.expertTiles = {};
		}			
    if (this._avatar) {
      this._avatar.destroy();
      this._avatar = null;
    }
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton");









dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton",
	[dijit._Widget, dijit._Templated, dijit.form.DropDownButton],
	{

	label: '',
	tasks: [],
	displayDropDown: true,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.activityDetails = new com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip({
			'tasks': this.tasks
		});
		
		this.dropDown = new dijit.TooltipDialog({
            'content': this.activityDetails.domNode,
            'class': 'bpm-social-participants-activity-tooltip'
        });
		
    	this.label = '';
 
    	this.inherited(arguments);
    	
    	dojo.addClass(this.domNode, 'bpm-social-participants-activity');

        dijit.setWaiRole(this.dropDown.containerNode,"role","presentation");//fix RPT violation
    	
    	if(this.displayDropDown){
			this.openDropDown();
		}
	},
	
	openDropDown: function(){
		this.inherited(arguments);        
		
		this.activityDetails.getExperts();
        this.activityDetails.header.focus();


	},
	
	closeDropDown: function(foo, force){
		if(force==true){
			this.inherited(arguments);
		}
	},
	
	destroy: function() {
		if(this.activityDetails && this.activityDetails.destroy){
			this.activityDetails.destroy();
		}
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton");













dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton",
	[dijit._Widget, dijit._Templated, dijit.form.DropDownButton],
	{

	label: '',
	activityOwner: null,	
	livenames: null,	
	currentUser: null,
	avgHours: 0,
	completed: 0,
	userImageSize: 24,
	popupUserImageSize: 48,
	container: null,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
    	var data = {
    			'fullName': this.currentUser.fullName,
    			'userName': this.currentUser.userName,
    			'displayName': this.currentUser.fullName,
    			'title': this.currentUser.userPreferences['Title'],
    			'taskEmailAddress': this.currentUser.userPreferences['Task Email Address'],
    			'phoneNumber': this.currentUser.userPreferences['Phone Number'],
    			'avgHours': this.avgHours,
    			'completed': this.completed,
    			'chatString': ''
       	};
    	    	
		if(com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
			this.activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
			this.dropDown = new dijit.TooltipDialog({
				'content': this.activityOwner.domNode,
				'class': 'bpm-social-participants-image-tooltip'
			});
			dojo.connect(this, 'onMouseEnter', this, 'openDropDown');
		}

    	this.label = '';
 
    	this.inherited(arguments);
    	
    	dojo.addClass(this.domNode, 'bpm-social-participants-image');

	    imgSrc = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this.currentUser.userName);

	    var props = {
       		'src': imgSrc,
       		'height': this.popupUserImageSize,
    		'width': this.popupUserImageSize,
    		'alt': this.currentUser.fullName
       	};

    	var textNode = dojo.query(".dijitButtonText", this.domNode)[0];
    	this.currentImage = dojo.create("img", props, textNode);

    	//if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
    	//	dojo.connect(this, 'onClick', this, function(e){
    	//		if(this.container!=null && typeof(this.container.destroy)!='undefined'){
    	//			this.container.destroy();
    	//		}
    	//	});
    	//}

		/** Business Card integration **/
    	if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
    		var user = com.ibm.bpm.social.util.common.Util.getUser(this.currentUser.userName);
    		com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.currentImage, user);
    	}
		/** End Business Card integration **/
    },
	
	openDropDown: function(){
		if(this.dropDown){
			if(this.container){
				//callback to parent container to close other dropdowns
				this.container.closeDropdowns();
			}
			this.inherited(arguments);
		}
	},
	
	isDropDownOpen: function(){
		return dojo.query('.dijitHasDropDownOpen', this.domNode).length > 0;
	},
	
	destroy: function() {
		if(this.activityOwner && this.activityOwner.destroy){
			this.activityOwner.destroy();
			this.activityOwner = null;
		}
		
		this.inherited('destroy',arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer");











dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{

	templateString: "",
	templateString:"<div class=\"bpm-social-participants-image-container hidden\" dojoAttachPoint=\"container\">\n<div dojoAttachPoint=\"scroll\" class=\"scroll\">\n<div dojoAttachPoint=\"innerScroll\" class=\"innerScroll\"></div>\n</div>\n</div>\n",
	tasks: [],
	dropdowns: [],
	displayedUsers: {},
	displayedUsersCount: 0,
	displayDropDown: false,
	displayDropDownIndex: null,
	currentWidth: 0,
	cssSingleIconWidth: 57,
	//userDataUrl: "/ProcessPortal/api/rest/task/{tid}/userdata?username={username}",
	userDataUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/userdata?username={username}",
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.displayedUsers = {};
		this.dropdowns = [];
		
		var currentUser, avgHours, completed;
		
		if(this.tasks.length==1){
			dojo.addClass(this.container, "singleUser");
			this.displayDropDown = true;
			this.displayDropDownIndex = 0;
		}
		
		for(var idx = 0; idx < this.tasks.length; idx++){
			
	    	currentUser = com.ibm.bpm.social.util.common.Util.getUser(this.tasks[idx].assignedTo,true);

	    	if(this.displayedUsers[currentUser.userName]){
	    		continue;
	    	} else {
	    		this.displayedUsers[currentUser.userName] = true;
	    		this.displayedUsersCount++;
	    	}
	    	
	    	avgHours = 0;
	    	completed = 0;
	    	
			this.socialRestHandler.getExpertUserData(this.tasks[idx].tkiid, currentUser.userName,
				function(data, xhrArgs) {
					avgHours = data.avgHours;
					completed = data.completed;
				}, "", false);
			
	   		//03. Create Dropdown Button (Image)
	   		 var button = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton({
	   			'currentUser': currentUser,
	   			'avgHours': avgHours,
	   			'completed': completed,
	   			'container': this
	   		}, dojo.create("div", {}, this.innerScroll));
	   		 
	   		this.dropdowns.push(button);

	   		dojo.style(this.innerScroll, 'width', (this.displayedUsersCount * this.cssSingleIconWidth)+'px');
		}
		dojo.removeClass(this.container, "hidden");
		
		this.inherited('postCreate',arguments);
		
		if(this.displayDropDown){
			if(this.displayDropDownIndex && this.displayDropDownIndex < this.dropdowns.length){
				this.openDropDown(this.displayDropDownIndex);
			} else {
				this.openFirstDropDown();
			}
		}
	},
	
	openFirstDropDown: function(){
		this.openDropDown(0);
		
	},
	
	openDropDown: function(idx){
		if(idx < this.dropdowns.length){
			this.dropdowns[idx].openDropDown();
		}
	},

	getDropDowns: function(){
		return this.dropdowns;
	},
	
	closeDropdowns: function(){
		for(var idx = 0; idx < this.dropdowns.length; idx++){
			this.dropdowns[idx].closeDropDown();
		}
	},
	
	destroy: function() {
		if(this.dropdowns.length > 0){
			for(var idx = 0; idx < this.dropdowns.length; idx++){
				if(this.dropdowns[idx] && this.dropdowns[idx].destroy){
					this.dropdowns[idx].closeDropDown();
					this.dropdowns[idx].destroy();
				}
			}
		}
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bspace.common.util.ui.FilterField"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bspace.common.util.ui.FilterField"] = true;
dojo.provide("com.ibm.bspace.common.util.ui.FilterField");

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog"] = true;
/*
Licensed Materials - Property of IBM
5724-M23, 5724-M24, 5724-L01, 5724-R31
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog");



dojo.registerModulePath("com.ibm.socialPortal.templates", window.endpointURLObject.processPortalJS +"/com/ibm/bpm/social/widgets/dialogs/templates");
dojo.declare("com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog", com.ibm.bspace.common.util.ui.Dialog, {
	templateString: dojo.cache("com.ibm.socialPortal.templates", "GoToSpacesDialog.html", "<div class=\"bspace-style\">\n\t<div class=\"spaceSelectorDropDown\">\n\t\t<div class=\"spaceSelectorDropDownFilter\">\n\t\t\t<label id=\"spaceSelectorDropDownFilterText\" dojoAttachPoint=\"filterText\"></label>\n\t\t\t<div class=\"spaceSelectorDropDownFilterField\" dojoAttachPoint=\"filter\"></div>\n\t\t</div>\n\t\t<div class=\"spaceSelectorDropDownList\" dojoAttachPoint=\"spacesList\" role=\"list\">\n\t\t</div>\n\t</div>\n\t<div class=\"spaceSelectorCancelButtonPanel lotusDialogFooter\" dojoAttachPoint=\"cancelButtonPanel\">\n\t   <div class=\"spaceSelectorCancelButtonPanelDiv\">\n\t      <input type=\"button\" dojoAttachPoint=\"cancelButton\" class=\"lotusFormButton\" value=\"Cancel\"/>\n\t   </div>\t\t\t\n\t</div>\t \n</div>\n"),

     postCreate: function() {
         this.connections = new Array();

		
		this.goToSpacesMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");		
	
		// Update Bspace Go to Spaces styling elements for social portal
		this.cancelButton.value = this.goToSpacesMessages.cancel;
		this.cancelButton.title = this.goToSpacesMessages.cancel;					
		this.filterText.innerHTML = this.goToSpacesMessages.goToSpaces_navigateMessage;

 		
 		

 		com.ibm.bspace.common.util.ui.Helper.loadStyleSheet();
		
 		this.divFilterHolder = document.createElement("div");
 		if(document.body.dir=='rtl'){
 			this.divFilterHolder.className = "goToSpacesFilterField_rtl";
 		}
 		this.goToSpacesFilterField = new com.ibm.bspace.common.util.ui.FilterField({hint: this.goToSpacesMessages.goToSpaces_filter, title: this.goToSpacesMessages.goToSpaces_filterTitle, width: 200}, this.divFilterHolder);
 		
 		this.goToSpacesFilterField.setOnFilterCallback(this, this.applyFilter);
 		this.filter.appendChild(this.goToSpacesFilterField.domNode);

         this.inherited(arguments);
     },
 	applyFilter: function(filterValue) {
 		var f = filterValue.toLowerCase();
 		if (this.spaceList) {
 			for (var i=0; i<this.spaceList.length; i++) {
 				var show = 'block';
 				if (f) {
 					if (this.spaceList[i].title.toLowerCase().indexOf(f) == -1) {
 						show = 'none';
 					}
 				}
 				this.spaceList[i].div.style.display = show;
 			}
 		}
 	},
	_handleKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			var focus = dijit._curFocus;
			var next = null;
			if (this.isPopup) {
				next = this.goToSpacesFilterField.filterBox;
			}
			if (focus == this.goToSpacesFilterField.filterBox) {
				var div = this.spacesList.firstChild;
				while (div && div.style.display == 'none') {
					div = div.nextSibling;
				}
				if (e.shiftKey) {
					if (this.isPopup) {
						div = this.spacesList.lastChild;
						while (div && div.style.display == 'none') {
							div = div.previousSibling;
						}
					} else {
						div = null;
					}
				}
				if (div) {
					var id = div.id.substring(14);
					next = document.getElementById(id);
				}
			} else {
				var focus = document.getElementById('spaceEntryDiv_'+focus.id);
				if (focus) {
					var div = focus.nextSibling;
					while (div && div.style.display == 'none') {
						div = div.nextSibling;
					}
					if (e.shiftKey) {
						div = focus.previousSibling;
						while (div && div.style.display == 'none') {
							div = div.previousSibling;
						}
					}
					if (div) {
						var id = div.id.substring(14);
						next = document.getElementById(id);
					}
				}
			}
			if (next) {
				next.focus();
				dojo.stopEvent(e);
			}
		}
	},
	
	refresh: function() {
		while (this.connections.length > 0) {
			dojo.disconnect(this.connections.pop());
		}
		this.connections.push(dojo.connect(this.domNode,'onkeypress',this,'_handleKeyPress'));
		this.connections.push(dojo.connect(this.cancelButton,'onclick',this,'hide'));

		this.spacesList.innerHTML = "";
		this.getSpaces();

		this.dirty = false;
	},
	
	_compareSpaces: function(space1, space2) {
		var rc = 0;
		if (space1.title < space2.title) {
			rc = -1;
		}
		if (space1.title > space2.title) {
			rc = 1;
		}
		return rc;
	},
	
	_addSpaceDiv: function(data, messages) {
		
		var isLTR = document.body.dir != 'rtl';
		var spaceEntryDiv = document.createElement("div");
		spaceEntryDiv.id = 'spaceEntryDiv_'+data.id;
		spaceEntryDiv.style.clear = "both";
		spaceEntryDiv.style.padding = "4px 0px 4px 0px";
		spaceEntryDiv.style.width = "100%";					
		
		var spaceIconDiv = document.createElement("div");
		if (document.body.dir == 'rtl') {
			spaceIconDiv.innerHTML="<img src=\""+data.icon+"\" alt=\"\" style=\"padding-right:3px; height:16px; width:16px;\"></img>&nbsp;";
		}
		else {
			spaceIconDiv.innerHTML="<img src=\""+data.icon+"\" alt=\"\" style=\"padding-left:3px; height:16px; width:16px;\"></img>&nbsp;";
		}
		dojo.style(spaceIconDiv, "cssFloat", isLTR ? "left" : "right");	
		spaceEntryDiv.appendChild(spaceIconDiv);
							
		var spaceLinkDiv = document.createElement("div");
		dojo.style(spaceLinkDiv, "cssFloat", isLTR ? "left" : "right");
		dojo.attr(spaceLinkDiv,"role","listitem");
		dojo.attr(spaceLinkDiv,"aria-labelledby", data.id+"_bsGoToSpacesSpaceOwnerDiv");
							
		var a = document.createElement('a');
		a.id = data.id;
		a.href = "#";
		a.innerHTML = data.title;
		dojo.attr(a,"role","link");
		
		///// Defect # 100590 and 100846
		var titleStr = data.description + " - " + dojo.string.substitute(messages.goToSpaces_ownerLabel, [data.owner]);
		a.title = titleStr;
		
		spaceLinkDiv.appendChild(a);
		this.connections.push(dojo.connect(a,'onclick',this,'_changeSpace'));
		spaceEntryDiv.appendChild(spaceLinkDiv);
		
		//accessibility for JAWS									
		var spaceOwnerLabelledByDiv = document.createElement("span");
		var str = "";
		if (data.description) str = data.description + ".  ";
		spaceOwnerLabelledByDiv.innerHTML = str + dojo.string.substitute(messages.goToSpaces_ownedBy, [data.owner]);
		spaceOwnerLabelledByDiv.id = data.id+"_bsGoToSpacesSpaceOwnerDiv";
		dojo.attr(spaceOwnerLabelledByDiv,"style",{"display":"none"});
		spaceEntryDiv.appendChild(spaceOwnerLabelledByDiv);																
							
		var spaceOwnerDiv = document.createElement("div");
		spaceOwnerDiv.className = "goToSpacesListOwner";
		spaceOwnerDiv.innerHTML = data.owner;
		spaceOwnerDiv.style.color = "#999999";
		dojo.style(spaceOwnerDiv, "cssFloat", isLTR ? "right" : "left");
		if (document.body.dir == 'rtl') {
			spaceOwnerDiv.style.textAlign = "left";
			spaceOwnerDiv.style.paddingLeft = "10px";
		}
		else {
			spaceOwnerDiv.style.textAlign = "right";
			spaceOwnerDiv.style.paddingRight = "10px";
		}					
		spaceEntryDiv.appendChild(spaceOwnerDiv);
		data.div = spaceEntryDiv;
		
		this.spacesList.appendChild(spaceEntryDiv);
	},
	
	_changeSpace: function(e) {
		var pageId = e.target.id;
		location.href = window.endpointURLObject.mumJS +"/enabler#pid=" + pageId;
    },
    
    getSpaces: function() {
		var self = this;
		var xhrArgs = {
			url: window.endpointURLObject.mumJS +"/mycontenthandler?uri=space%3Acollection&num=20&start=0&aspect=ac&nocache=true",
			handleAs: "xml",
			preventCache: true,
			load: function(data){
				var list = new Array();
				var docNode = data.documentElement;

			   // Write text content into the display.
			   for(var i = 0; i < docNode.childNodes.length; i++){
				 var feedElement = docNode.childNodes[i];
			     if (feedElement.nodeName == "atom:entry") {

			    	 var spaceName = "Space";
			    	 var spaceId = "";
			    	 var spaceIcon = "";
			    	 var spaceOwner = "";
			    	 var isHidden = false;
			    	 var canEdit = false;
					 var isLegacyProcessPortalSpace = false;
			    	 var spaceElements = feedElement.childNodes;
			    	 for (var j=0; j<spaceElements.length; j++) {
			    		 
			    		 // Ignore the space ID (found in the atom:id element), and instead get the navigation
			    		 // model ID which is the ID of the first page in the space.  This will be used to
			    		 // form the URL which will take us directly to the space.
			    		 if (spaceElements[j].nodeName == "atom:link" && spaceElements[j].getAttribute("ca:rel") && spaceElements[j].getAttribute("ca:rel") == "navigation") {
			    			 // The ID is stored in the format "nm:id:PAGEID"
			    			 var navigationId = spaceElements[j].getAttribute("ext:uri");
			    			 if (navigationId) {
			    				 var idTokens = navigationId.split(":");
				    			 spaceId = idTokens[idTokens.length-1];
			    			 }
			    		 }
			    		 
			    		 // Look for space title to display. This is a fallback in case the
			    		 // localized title in the atom:content doesn't exist.
			    		 if (spaceElements[j].nodeName == "atom:title") {
			    			 spaceName = spaceElements[j].firstChild.nodeValue;
			    		 }
			    		 
			    		 // Look for space content which contains the metadata for the space.
			    		 if (spaceElements[j].nodeName == "atom:content") {
			    			 
			    			 var metadataElements = spaceElements[j].firstChild.childNodes;
			    			 for (var k=0; k<metadataElements.length; k++) {
			    				 // Look for space icon metadata.
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}iconUrl") {
			    					 spaceIcon = metadataElements[k].firstChild.nodeValue;
			    				 }
			    				 
			    				 // Look for space owner display name.
			    				 if (metadataElements[k].nodeName == "ca:owner") {
			    					 spaceOwner = metadataElements[k].getAttribute("ca:display-name");

			    				 }
			    				 
			    				 // If found, overwrite the default title with the localized one.
			    				 if (metadataElements[k].nodeName == "ca:title") {
			    					 var locale = metadataElements[k].getAttribute("xml:lang");
			    					 if (locale == dojo.locale) {
			    						spaceName =  metadataElements[k].firstChild.nodeValue;
			    					 }
			    				 }
								 
								 // Look for flag indicating this is a legacy BPM 8.0 Process Portal space
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && 
			    						 metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.bspace.bpm.socialSpace") {
			    					 isLegacyProcessPortalSpace = (metadataElements[k].firstChild.nodeValue === 'true');
			    				 }
			    				 
			    				 // Look for the hidden flag (absense of flag means space is not hidden).
			    				 //<ca:meta-data ca:name="{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.mashups.space.hidden">true</ca:meta-data>
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && 
			    						 metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.mashups.space.hidden") {
			    					 isHidden = (metadataElements[k].firstChild.nodeValue === 'true');
			    				 }
			    				 
			    				 // If the user has either the EDITOR or MANAGER role, then they are allowed to edit the space.
			    				 if (metadataElements[k].nodeName == "ac:allowed-access" && metadataElements[k].getAttribute("xmlns:ac") && 
			    						 metadataElements[k].getAttribute("xmlns:ac") == "http://www.ibm.com/xmlns/prod/lotus/access-control/v1.0") {
					    			 var accessLevelElements = metadataElements[k].childNodes;
					    			 for (var m=0; m<accessLevelElements.length; m++) {
					    				 var accessType = accessLevelElements[m].getAttribute("ac:type");
					    				 if (accessType && (accessType == "EDITOR" || accessType == "MANAGER")) {
					    					 canEdit = true;
											 break;
					    				 }
					    			 }
			    				 }
			    			 }
			    		 }
			    	 }
			    	 
			    	 
					// Only include the space if it is not hidden or if the user is an owner/editor of the space.
					// If the space is hidden and the user is only a viwer then it should not display.
					if (!isLegacyProcessPortalSpace) {
						if (!isHidden || canEdit) {
							list.push({
									id: spaceId,
									title: spaceName,
									owner: spaceOwner,
									description: spaceName,
									icon: spaceIcon
							});
						}
					}
			     }
			   }
			   
				list.sort(function(space1, space2) {
					var rc = 0;
					if (space1.title < space2.title) {
						rc = -1;
					}
					if (space1.title > space2.title) {
						rc = 1;
					}
					return rc;
				});

				for (var i=0; i<list.length; i++) {
					self._addSpaceDiv(list[i], self.goToSpacesMessages);
				}
				self.spaceList = list;
				if (self.goToSpacesFilterField) {
					self.goToSpacesFilterField.reset();
				}
				
			},
			 error: function(error, ioargs){
				var errorDialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({message: "" + error });
				errorDialog.show();
			}
		};

		// Call the asynchronous xhrGet
		var deferred = dojo.xhrGet(xhrArgs);    	
    },
	
	show: function() {
		this.inherited(arguments);
		this.refresh();
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.debug.LoggingWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.debug.LoggingWidget"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.debug.LoggingWidget");





dojo.declare(
	"com.ibm.bpm.social.widgets.debug.LoggingWidget",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class=\"bpm-process-portal-logging-widget\">\n\t<p class=\"bpm-process-portal-logging-widget-description\" dojoAttachPoint=\"descriptionNode\"></p>\n\t<input type=\"button\" class=\"bpm-process-portal-logging-widget-change-size-button\" dojoAttachPoint=\"changeSizeButton\" />\n\t<input type=\"button\" class=\"bpm-process-portal-logging-widget-select-all-button\" dojoAttachPoint=\"selectAllButton\" />\n\t<div class=\"log-list\" dojoAttachPoint=\"logList\"></div>\n</div>\n",
	constructor: function() {
		this.loggingWidgetConnectors = [];
		if(com && com.ibm && com.ibm.bpm && com.ibm.bpm.social && com.ibm.bpm.social.debug) {
			this.loggingTopic = com.ibm.bpm.social.debug.MESSAGE_LOGGED_TOPIC;
		}
		else {
			this.loggingTopic = "defaltLoggingTopic";
		}
		this.list = null;
		this.loggingSubscription = null;
		
		this.descriptionText = "Logging...";
		this.selectAllText = "Select All";
		this.minimizeText = "Minimize";
		this.maximizeText = "Maximize";
	},
	
    postCreate: function() {
		this.inherited(arguments);
		
		//load all existing logs
		if(com && com.ibm && com.ibm.bpm && com.ibm.bpm.social && com.ibm.bpm.social.debug) {
			this.loggingTopic = com.ibm.bpm.social.debug.MESSAGE_LOGGED_TOPIC;
			var existingLogs = com.ibm.bpm.social.debug.getLogs();
			if(existingLogs != null) {
				for(var i=0; i<existingLogs.length; i++) {
					this.addLog(existingLogs[i].type, existingLogs[i].message);
				}
			}
		}
		
		//subscribe to updates (whenever a message is logged)
		this.loggingSubscription = dojo.subscribe(this.loggingTopic, this, function(msgObj) {
			if(msgObj != null) {
				this.addLog(msgObj.type, msgObj.message);
			}
		});
		
		//add text and listeners
		if(this.selectAllButton != null) {
			dojo.attr(this.selectAllButton, "value", this.selectAllText);
			this.loggingWidgetConnectors.push(dojo.connect(this.selectAllButton, "click", dojo.hitch(this, this.highlightAllText)));
		}
		if(this.changeSizeButton != null) {
			dojo.attr(this.changeSizeButton, "value", this.minimizeText);
			this.loggingWidgetConnectors.push(dojo.connect(this.changeSizeButton, "click", dojo.hitch(this, this.togglePanel)));
		}
		if(this.descriptionNode != null) {
			dojo.attr(this.descriptionNode, "innerHTML", this.descriptionText);
		}
		
		//hide the select all button on ipad
		if(com.ibm.bpm.social.util.common.Util.isIPad) {
			dojo.style(this.selectAllButton, "display", "none");
		}
	},
	
	highlightAllText: function() {
		if(this.list != null && !com.ibm.bpm.social.util.common.Util.isIPad) {
			//remove old highlighting
			if(document.selection) {
				document.selection.empty();
			}
			else if(window.getSelection) {
				window.getSelection().removeAllRanges();
			}
			
			//highlight text
			if(document.selection) {
				var range = document.body.createTextRange();
				range.moveToElementText(this.list);
				range.select();
			}
			else if(window.getSelection) {
				var range = document.createRange();
				range.selectNode(this.list);
				window.getSelection().addRange(range);
			}
		}
	},
	
	isScrolledToBottom: function() {
		if(this.logList == null) {
			return false;
		}
		return this.logList.scrollTop + this.logList.offsetHeight + 5 >= this.logList.scrollHeight;
	},
	
	scrollToBottom: function() {
		if(this.logList != null) {
			this.logList.scrollTop = this.logList.scrollHeight;
		}
	},
	
	isMinimized: function() {
		return this.domNode != null && dojo.hasClass(this.domNode, "minimized");
	},
	
	togglePanel: function() {
		if(this.domNode != null) {
			if(this.isMinimized()) {
				this.maxmimizePanel();
			}
			else {
				this.minimizePanel();
			}
		}
	},
	
	minimizePanel: function() {
		if(this.domNode != null) {
			dojo.addClass(this.domNode, "minimized");
			if(this.changeSizeButton != null) {
				dojo.attr(this.changeSizeButton, "value", this.maximizeText);
			}
		}
	},
	
	maxmimizePanel: function() {
		if(this.domNode != null) {
			dojo.removeClass(this.domNode, "minimized");
			if(this.changeSizeButton != null) {
				dojo.attr(this.changeSizeButton, "value", this.minimizeText);
			}
			this.scrollToBottom();
		}
	},
	
	addLog: function(type, msg) {
		if(this.list == null) {
			this.list = dojo.create("ol", null, this.logList);
		}
		var wasScrolledToBottom = this.isScrolledToBottom();
		var li = dojo.create("li", {'class': "bpm-process-portal-logging-widget-"+type+"-message"}, this.list);
		var msgSpace = dojo.create("span", {'class': "bpm-process-portal-logging-widget-message", 'innerHTML': ""+msg}, li);
		
		//maintain scroll to bottom
		if(wasScrolledToBottom) {
			this.scrollToBottom();
		}
	},
    
	destroy: function(finalize) {
		dojo.forEach(this.loggingWidgetConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		if(this.loggingSubscription != null) {
			dojo.unsubscribe(this.loggingSubscription);
		}
		this.inherited(arguments);
	}
});

}


dojo.i18n._preloadLocalizations("com.ibm.bpm.nls.social_base", ["ROOT","ar","cs","da","de","el","en","es","fi","fr","he","he-il","hu","it","iw","iw-il","ja","ko","nb","nl","no","pl","pt","pt-br","ro","ru","sk","sv","tr","xx","zh","zh-tw"]);
